package basic

import (
	"bytes"
	"errors"
	"fmt"
	"strconv"
	"strings"
	"time"
)

// this file contains bright 9100 stars
// 9100颗亮星列表

type InnerStarData struct {
	HR     uint16  //Bright Star Number;[1/9110]+ Harvard Revised Number;亮星编号
	Name   string  //Name, generally Bayer(如天狼星：Alpha CMA) and/or Flamsteed(如天狼星：9 CMA) name
	HD     uint32  //Henry Draper Catalog Number;HD星表编号
	Ra     float64 //Ra J2000;J2000历元赤经
	Dec    float64 //De J2000;J2000历元赤纬
	Mag    float64 //视星等
	PmRA   float64 //赤经年自行
	PmDec  float64 //赤纬年自行
	RadVel float64 //径向速度 km/s
	RotVel float64 //自行速度 km/s
	Pc     float64 //秒差距
	HIP    uint32  //HIP星表编号
}
type StarData struct {
	InnerStarData
	ChineseName      string
	ChineseAlias     string
	ChineseBayerName string
	CommonName       string
	ComminAliasName  string
	Cst              string
	CstChinese       string
}

func parseStarData(star []byte) (InnerStarData, error) {
	var err error
	var stardata InnerStarData
	if len(star) < 160 {
		return stardata, errors.New("invalid stardat")
	}
	for i := 0; i < 4; i++ {
		if star[i] == ' ' {
			continue
		}
		stardata.HR = stardata.HR*10 + uint16(star[i]-48)
	}
	stardata.Name = string(bytes.TrimSpace(star[4:14]))
	for i := 25; i < 31; i++ {
		if star[i] == ' ' {
			continue
		}
		stardata.HD = stardata.HD*10 + uint32(star[i]-48)
	}
	stardata.Ra, err = parseRa(star)
	if err != nil {
		return stardata, fmt.Errorf("parse ra failed:%v", err)
	}
	stardata.Dec, err = parseDec(star)
	if err != nil {
		return stardata, fmt.Errorf("parse dec failed:%v", err)
	}
	magOri := string(bytes.TrimSpace(star[102:107]))
	if magOri != "" {
		stardata.Mag, err = strconv.ParseFloat(magOri, 64)
		if err != nil {
			return stardata, fmt.Errorf("parse mag failed:%v", err)
		}
	} else {
		stardata.Mag = 9999.9999
	}
	stardata.PmRA, _ = strconv.ParseFloat(string(bytes.TrimSpace(star[148:154])), 64)
	stardata.PmDec, _ = strconv.ParseFloat(string(bytes.TrimSpace(star[154:160])), 64)
	if len(star) >= 170 {
		stardata.RadVel, _ = strconv.ParseFloat(string(bytes.TrimSpace(star[166:170])), 64)
	}
	if len(star) >= 179 {
		stardata.RotVel, _ = strconv.ParseFloat(string(bytes.TrimSpace(star[176:179])), 64)
	}
	if len(star) > 161 {
		rc, _ := strconv.ParseFloat(string(bytes.TrimSpace(star[162:166])), 64)
		if rc != 0 {
			stardata.Pc = 1 / rc
		}
	}
	return stardata, nil
}

func parseRa(star []byte) (float64, error) {
	var sec float64
	var err error
	ra := float64(0)
	for i := 75; i < 77; i++ {
		if star[i] == ' ' {
			continue
		}
		ra = ra*10 + float64(star[i]-48)
	}
	minute := uint8(0)
	for i := 77; i < 79; i++ {
		if star[i] == ' ' {
			continue
		}
		minute = minute*10 + (star[i] - 48)
	}
	ori := string(bytes.TrimSpace(star[79:83]))
	if ori != "" {
		sec, err = strconv.ParseFloat(ori, 64)
		if err != nil {
			return ra, err
		}
	}
	ra += float64(minute)/60 + sec/3600
	return ra * 15, nil
}

func parseDec(star []byte) (float64, error) {
	var sec float64
	var err error
	underZero := false
	if star[83] == '-' {
		underZero = true
	}
	dec := float64(0)
	for i := 84; i < 86; i++ {
		if star[i] == ' ' {
			continue
		}
		dec = dec*10 + float64(star[i]-48)
	}
	minute := uint8(0)
	for i := 86; i < 88; i++ {
		if star[i] == ' ' {
			continue
		}
		minute = minute*10 + (star[i] - 48)
	}
	ori := string(bytes.TrimSpace(star[88:90]))
	if ori != "" {
		sec, err = strconv.ParseFloat(ori, 64)
		if err != nil {
			return dec, err
		}
	}
	dec += float64(minute)/60 + sec/3600
	if underZero {
		dec = -dec
	}
	return dec, nil
}

var stardat [][]byte
var starhdindex map[string]int
var hr2detail map[uint16][]string
var hr2hip map[uint16]uint32
var chnidx map[string]uint16

func LoadStarData() error {
	hr2detail = map[uint16][]string{9089: []string{"垒壁阵十二", "", "双鱼座30", "", "双鱼座", "Pisces"}, 9098: []string{"八魁二", "", "鲸鱼座2", "", "鲸鱼座", "Cetus"}, 3: []string{"垒壁阵十一", "", "双鱼座33", "", "双鱼座", "Pisces"}, 15: []string{"壁宿二", "", "仙女座α", "Alpheratz", "仙女座", "Andromeda"}, 25: []string{"火鸟四", "", "凤凰座ε", "", "凤凰座", "Phoenix"}, 33: []string{"八魁一", "", "鲸鱼座6", "", "鲸鱼座", "Cetus"}, 48: []string{"八魁六", "", "鲸鱼座7", "", "鲸鱼座", "Cetus"}, 63: []string{"天厩一", "", "仙女座θ", "", "仙女座", "Andromeda"}, 68: []string{"天厩三", "", "仙女座σ", "", "仙女座", "Andromeda"}, 74: []string{"天仓一", "", "鲸鱼座ι", "", "鲸鱼座", "Cetus"}, 77: []string{"鸟喙六", "", "杜鹃座ζ", "", "杜鹃座", "Tucana"}, 98: []string{"蛇尾一", "", "水蛇座β", "", "水蛇座", "Hydrus"}, 100: []string{"火鸟五", "", "凤凰座κ", "", "凤凰座", "Phoenix"}, 21: []string{"王良一", "", "仙后座β", "Caph", "仙后座", "Cassiopeia"}, 125: []string{"火鸟八", "", "凤凰座λ1", "", "凤凰座", "Phoenix"}, 126: []string{"鸟喙四", "", "杜鹃座β1", "", "杜鹃座", "Tucana"}, 123: []string{"王良五", "", "仙后座λ", "", "仙后座", "Cassiopeia"}, 130: []string{"王良二", "", "仙后座κ", "", "仙后座", "Cassiopeia"}, 154: []string{"奎宿六", "", "仙女座π", "", "仙女座", "Andromeda"}, 153: []string{"附路", "", "仙后座ζ", "", "仙后座", "Cassiopeia"}, 163: []string{"奎宿四", "", "仙女座ε", "", "仙女座", "Andromeda"}, 165: []string{"奎宿五", "", "仙女座δ", "", "仙女座", "Andromeda"}, 180: []string{"火鸟七", "", "凤凰座μ", "", "凤凰座", "Phoenix"}, 191: []string{"水委三", "", "凤凰座η", "", "凤凰座", "Phoenix"}, 39: []string{"壁宿一", "", "飞马座γ", "Algenib", "飞马座", "Pegasus"}, 194: []string{"天溷四", "", "鲸鱼座φ1", "", "鲸鱼座", "Cetus"}, 193: []string{"阁道六", "", "仙后座ο", "", "仙后座", "Cassiopeia"}, 215: []string{"奎宿二", "", "仙女座ζ", "", "仙女座", "Andromeda"}, 224: []string{"外屏一", "", "双鱼座δ", "", "双鱼座", "Pisces"}, 223: []string{"阁道五", "", "仙后座ν", "", "仙后座", "Cassiopeia"}, 219: []string{"王良三", "", "仙后座η", "", "仙后座", "Cassiopeia"}, 226: []string{"奎宿七", "", "仙女座ν", "", "仙女座", "Andromeda"}, 264: []string{"策", "", "仙后座γ", "", "仙后座", "Cassiopeia"}, 269: []string{"奎宿八", "", "仙女座μ", "", "仙女座", "Andromeda"}, 271: []string{"奎宿一", "", "仙女座η", "", "仙女座", "Andromeda"}, 294: []string{"外屏二", "", "双鱼座ε", "", "双鱼座", "Pisces"}, 322: []string{"火鸟九", "", "凤凰座β", "", "凤凰座", "Phoenix"}, 338: []string{"水委二", "", "凤凰座ζ", "", "凤凰座", "Phoenix"}, 334: []string{"天仓二", "", "鲸鱼座η", "", "鲸鱼座", "Cetus"}, 335: []string{"军南门", "", "仙女座φ", "", "仙女座", "Andromeda"}, 343: []string{"阁道四", "", "仙后座θ", "", "仙后座", "Cassiopeia"}, 351: []string{"奎宿十五", "", "双鱼座χ", "", "双鱼座", "Pisces"}, 352: []string{"奎宿十一", "", "双鱼座τ", "", "双鱼座", "Pisces"}, 361: []string{"外屏三", "", "双鱼座ζA", "", "双鱼座", "Pisces"}, 360: []string{"奎宿十四", "", "双鱼座φ", "", "双鱼座", "Pisces"}, 383: []string{"奎宿十三", "", "双鱼座υ", "", "双鱼座", "Pisces"}, 402: []string{"天仓三", "", "鲸鱼座θ", "", "鲸鱼座", "Cetus"}, 99: []string{"火鸟六", "", "凤凰座α", "Ankaa", "凤凰座", "Phoenix"}, 399: []string{"华盖五", "", "仙后座ψ", "", "仙后座", "Cassiopeia"}, 429: []string{"火鸟十", "", "凤凰座γ", "", "凤凰座", "Phoenix"}, 434: []string{"外屏四", "", "双鱼座μ", "", "双鱼座", "Pisces"}, 440: []string{"天园一", "", "凤凰座δ", "", "凤凰座", "Phoenix"}, 437: []string{"右更二", "", "双鱼座η", "", "双鱼座", "Pisces"}, 458: []string{"天大将军六", "", "仙女座υ", "", "仙女座", "Andromeda"}, 464: []string{"天大将军三", "", "仙女座51", "", "仙女座", "Andromeda"}, 477: []string{"天大将军七", "", "仙女座τ", "", "仙女座", "Andromeda"}, 489: []string{"外屏五", "", "双鱼座ν", "", "双鱼座", "Pisces"}, 496: []string{"天大将军二", "", "英仙座φ", "", "英仙座", "Perseus"}, 509: []string{"天仓五", "", "鲸鱼座τ", "", "鲸鱼座", "Cetus"}, 510: []string{"右更四", "", "双鱼座ο", "", "双鱼座", "Pisces"}, 168: []string{"王良四", "", "仙后座α", "Shedir", "仙后座", "Cassiopeia"}, 188: []string{"土司空", "", "鲸鱼座β", "Diphda", "鲸鱼座", "Cetus"}, 549: []string{"外屏六", "", "双鱼座ξ", "", "双鱼座", "Pisces"}, 542: []string{"阁道二", "", "仙后座ε", "", "仙后座", "Cassiopeia"}, 570: []string{"蛇腹四", "", "水蛇座η2", "", "水蛇座", "Hydrus"}, 566: []string{"天园二", "", "波江座χ", "", "波江座", "Eridanus"}, 548: []string{"华盖七", "", "仙后座ω", "", "仙后座", "Cassiopeia"}, 565: []string{"鈇锧五", "", "鲸鱼座56", "", "鲸鱼座", "Cetus"}, 591: []string{"蛇首一", "", "水蛇座α", "", "水蛇座", "Hydrus"}, 585: []string{"鈇锧四", "", "鲸鱼座υ", "", "鲸鱼座", "Cetus"}, 575: []string{"杠七", "", "仙后座48", "", "仙后座", "Cassiopeia"}, 337: []string{"奎宿九", "", "仙女座β", "Mirach", "仙女座", "Andromeda"}, 580: []string{"杠五", "", "仙后座50", "", "仙后座", "Cassiopeia"}, 390: []string{"奎宿十三", "", "双鱼座υ", "Adhil", "双鱼座", "Pisces"}, 622: []string{"天大将军九", "", "三角座β", "", "三角座", "Triangulum"}, 649: []string{"天囷五", "", "鲸鱼座ξ1", "", "鲸鱼座", "Cetus"}, 674: []string{"天园三", "", "波江座φ", "", "波江座", "Eridanus"}, 660: []string{"天大将军十一", "", "三角座δ", "", "三角座", "Triangulum"}, 664: []string{"天大将军十", "", "三角座γ", "", "三角座", "Triangulum"}, 705: []string{"蛇腹三", "", "水蛇座δ", "", "水蛇座", "Hydrus"}, 708: []string{"刍藁一", "", "鲸鱼座ρ", "", "鲸鱼座", "Cetus"}, 721: []string{"天园四", "", "波江座κ", "", "波江座", "Eridanus"}, 718: []string{"天囷六", "", "鲸鱼座ξ2", "", "鲸鱼座", "Cetus"}, 707: []string{"阁道一", "", "仙后座ι", "", "仙后座", "Cassiopeia"}, 754: []string{"天囷七", "", "鲸鱼座ν", "", "鲸鱼座", "Cetus"}, 779: []string{"天囷九", "", "鲸鱼座δ", "", "鲸鱼座", "Cetus"}, 781: []string{"刍藁六", "", "鲸鱼座ε", "", "鲸鱼座", "Cetus"}, 806: []string{"蛇腹二", "", "水蛇座ε", "", "水蛇座", "Hydrus"}, 789: []string{"天园五", "", "波江座s", "", "波江座", "Eridanus"}, 788: []string{"大陵八", "", "英仙座12", "", "英仙座", "Perseus"}, 804: []string{"天囷八", "", "鲸鱼座γ", "", "鲸鱼座", "Cetus"}, 801: []string{"胃宿一", "", "白羊座35", "", "白羊座", "Aries"}, 811: []string{"天苑七", "", "鲸鱼座π", "", "鲸鱼座", "Cetus"}, 813: []string{"天囷四", "", "鲸鱼座μ", "", "鲸鱼座", "Cetus"}, 818: []string{"天苑八", "", "波江座τ1", "", "波江座", "Eridanus"}, 837: []string{"蛇腹一", "", "水蛇座ζ", "", "水蛇座", "Hydrus"}, 824: []string{"胃宿二", "", "白羊座39", "", "白羊座", "Aries"}, 838: []string{"胃宿三", "", "白羊座41", "", "白羊座", "Aries"}, 872: []string{"附白二", "", "水蛇座ν", "", "水蛇座", "Hydrus"}, 840: []string{"大陵七", "", "英仙座16", "", "英仙座", "Perseus"}, 834: []string{"天船一", "", "英仙座η", "", "英仙座", "Perseus"}, 850: []string{"天苑九", "", "波江座τ2", "", "波江座", "Eridanus"}, 854: []string{"大陵二", "", "英仙座τ", "", "英仙座", "Perseus"}, 403: []string{"阁道三", "", "仙后座δ", "Ruchbah", "仙后座", "Cassiopeia"}, 879: []string{"积尸", "", "英仙座π", "", "英仙座", "Perseus"}, 896: []string{"天囷三", "", "鲸鱼座λ", "", "鲸鱼座", "Cetus"}, 919: []string{"天苑十", "", "波江座τ3", "", "波江座", "Eridanus"}, 915: []string{"天船二", "", "英仙座γ", "", "英仙座", "Perseus"}, 921: []string{"大陵六", "", "英仙座ρ", "", "英仙座", "Perseus"}, 472: []string{"水委一", "", "波江座α", "Achernar", "波江座", "Eridanus"}, 937: []string{"大陵三", "", "英仙座ι", "", "英仙座", "Perseus"}, 941: []string{"大陵四", "", "英仙座κ", "", "英仙座", "Perseus"}, 947: []string{"大陵增十八", "", "英仙座ω", "", "英仙座", "Perseus"}, 963: []string{"天苑增三", "", "天炉座α", "", "天炉座", "Fornax"}, 972: []string{"天阴二", "", "白羊座ζ", "", "白羊座", "Aries"}, 539: []string{"天仓四", "", "鲸鱼座ζ", "BatenKaitos", "鲸鱼座", "Cetus"}, 1003: []string{"天苑十一", "", "波江座τ4", "", "波江座", "Eridanus"}, 1030: []string{"天廪四", "", "金牛座ο", "", "金牛座", "Taurus"}, 1038: []string{"天廪三", "", "金牛座ξ", "", "金牛座", "Taurus"}, 1066: []string{"天廪一", "", "金牛座f", "", "金牛座", "Taurus"}, 1084: []string{"天苑四", "", "波江座ε", "", "波江座", "Eridanus"}, 1088: []string{"天苑十二", "", "波江座τ5", "", "波江座", "Eridanus"}, 1087: []string{"天船四", "", "英仙座ψ", "", "英仙座", "Perseus"}, 1123: []string{"卷舌六", "", "英仙座o", "", "英仙座", "Perseus"}, 1143: []string{"天园七", "", "波江座h", "", "波江座", "Eridanus"}, 1122: []string{"天船五", "", "英仙座δ", "", "英仙座", "Perseus"}, 1136: []string{"天苑三", "", "波江座δ", "", "波江座", "Eridanus"}, 1175: []string{"蛇首二", "", "网罟座β", "", "网罟座", "Reticulum"}, 544: []string{"娄宿增六", "", "三角座α", "Mothallah", "三角座", "Triangulum"}, 1135: []string{"卷舌一", "", "英仙座ν", "", "英仙座", "Perseus"}, 546: []string{"娄宿二", "", "白羊座γ2", "Mesarthim", "白羊座", "Aries"}, 1162: []string{"天苑二", "", "波江座π", "", "波江座", "Eridanus"}, 553: []string{"娄宿一", "", "白羊座β", "Sheratan", "白羊座", "Aries"}, 1173: []string{"天苑十三", "", "波江座τ6", "", "波江座", "Eridanus"}, 1208: []string{"附白一", "", "水蛇座γ", "", "水蛇座", "Hydrus"}, 595: []string{"外屏七", "", "双鱼座α", "Alrescha", "双鱼座", "Pisces"}, 1195: []string{"天园九", "", "波江座g", "", "波江座", "Eridanus"}, 1148: []string{"杠一", "", "鹿豹座γ", "", "鹿豹座", "Camelopardalis"}, 1213: []string{"天苑十五", "", "波江座τ8", "", "波江座", "Eridanus"}, 1203: []string{"卷舌四", "", "英仙座ζ", "", "英仙座", "Perseus"}, 1220: []string{"卷舌二", "", "英仙座ε", "", "英仙座", "Perseus"}, 603: []string{"天大将军一", "", "仙女座γ2", "Almaak", "仙女座", "Andromeda"}, 1240: []string{"天苑十六", "", "波江座τ9", "", "波江座", "Eridanus"}, 1239: []string{"毕宿八", "", "金牛座λ", "", "金牛座", "Taurus"}, 1251: []string{"毕宿增三", "", "金牛座ν", "", "金牛座", "Taurus"}, 1256: []string{"月", "", "金牛座37", "", "金牛座", "Taurus"}, 1261: []string{"积水", "", "英仙座λ", "", "英仙座", "Perseus"}, 1273: []string{"天船六", "", "英仙座c", "", "英仙座", "Perseus"}, 1318: []string{"九州殊口一", "", "波江座39", "", "波江座", "Eridanus"}, 1336: []string{"夹白二", "", "网罟座α", "", "网罟座", "Reticulum"}, 1303: []string{"天船七", "", "英仙座μ", "", "英仙座", "Perseus"}, 617: []string{"娄宿三", "", "白羊座α", "Hamal", "白羊座", "Aries"}, 1320: []string{"毕宿增七", "", "金牛座μ", "", "金牛座", "Taurus"}, 1338: []string{"金鱼一", "", "剑鱼座γ", "", "剑鱼座", "Dorado"}, 1329: []string{"天街二", "", "金牛座ω2", "", "金牛座", "Taurus"}, 1347: []string{"天园十", "", "波江座υ4", "", "波江座", "Eridanus"}, 1324: []string{"天船八", "", "英仙座b", "", "英仙座", "Perseus"}, 1346: []string{"毕宿四", "", "金牛座γ", "", "金牛座", "Taurus"}, 1348: []string{"砺石四", "", "金牛座φ", "", "金牛座", "Taurus"}, 1373: []string{"毕宿三", "", "金牛座δ1", "", "金牛座", "Taurus"}, 1393: []string{"天园十一", "", "波江座υ3", "", "波江座", "Eridanus"}, 1387: []string{"天街一", "", "金牛座κ1", "", "金牛座", "Taurus"}, 1389: []string{"毕宿二", "", "金牛座δ3", "", "金牛座", "Taurus"}, 1394: []string{"毕宿七", "", "金牛座71", "", "金牛座", "Taurus"}, 1396: []string{"天节一", "", "金牛座π", "", "金牛座", "Taurus"}, 1412: []string{"毕宿六", "", "金牛座θ2", "", "金牛座", "Taurus"}, 1453: []string{"天园十三", "", "波江座υ1", "", "波江座", "Eridanus"}, 1444: []string{"天节二", "", "金牛座ρ", "", "金牛座", "Taurus"}, 1465: []string{"金鱼二", "", "剑鱼座α", "", "剑鱼座", "Dorado"}, 1464: []string{"天园十二", "", "波江座υ2", "", "波江座", "Eridanus"}, 1458: []string{"天节七", "", "金牛座d", "", "金牛座", "Taurus"}, 424: []string{"勾陈一", "北极星", "小熊座α", "Polaris", "小熊座", "UrsaMinor"}, 1463: []string{"九州殊口四", "", "波江座ν", "", "波江座", "Eridanus"}, 1473: []string{"天节五", "", "金牛座c", "", "金牛座", "Taurus"}, 1481: []string{"九斿增四", "", "波江座l", "", "波江座", "Eridanus"}, 1479: []string{"附耳", "", "金牛座σ2", "", "金牛座", "Taurus"}, 1496: []string{"九斿八", "", "波江座54", "", "波江座", "Eridanus"}, 1497: []string{"诸王六", "", "金牛座τ", "", "金牛座", "Taurus"}, 1520: []string{"九斿二", "", "波江座μ", "", "波江座", "Eridanus"}, 1543: []string{"参旗六", "", "猎户座π3", "", "猎户座", "Orion"}, 1544: []string{"参旗五", "", "猎户座π2", "", "猎户座", "Orion"}, 1552: []string{"参旗七", "", "猎户座π4", "", "猎户座", "Orion"}, 1556: []string{"参旗一", "", "猎户座ο1", "", "猎户座", "Orion"}, 1560: []string{"九斿三", "", "波江座ω", "", "波江座", "Eridanus"}, 874: []string{"天苑六", "", "波江座η", "Azha", "波江座", "Eridanus"}, 1567: []string{"参旗八", "", "猎户座π5", "", "猎户座", "Orion"}, 1570: []string{"参旗四", "", "猎户座π1", "", "猎户座", "Orion"}, 1580: []string{"参旗二", "", "猎户座ο2", "", "猎户座", "Orion"}, 1577: []string{"五车一", "", "御夫座ι", "", "御夫座", "Auriga"}, 1568: []string{"八谷五", "", "鹿豹座7", "", "鹿豹座", "Camelopardalis"}, 1601: []string{"参旗九", "", "猎户座π6", "", "猎户座", "Orion"}, 1611: []string{"九斿五", "", "波江座64", "", "波江座", "Eridanus"}, 1617: []string{"玉井二", "", "波江座ψ", "", "波江座", "Eridanus"}, 1605: []string{"柱一", "", "御夫座ε", "", "御夫座", "Auriga"}, 1612: []string{"柱二", "", "御夫座ζ", "", "御夫座", "Auriga"}, 1620: []string{"天高一", "", "金牛座ι", "", "金牛座", "Taurus"}, 1654: []string{"屏二", "", "天兔座ε", "", "天兔座", "Lepus"}, 1641: []string{"柱三", "", "御夫座η", "", "御夫座", "Auriga"}, 1637: []string{"八谷六", "", "御夫座9", "", "御夫座", "Auriga"}, 1656: []string{"天高三", "", "金牛座m", "", "金牛座", "Taurus"}, 1679: []string{"玉井一", "", "波江座λ", "", "波江座", "Eridanus"}, 1696: []string{"军井一", "", "天兔座ι", "", "天兔座", "Lepus"}, 1702: []string{"屏一", "", "天兔座μ", "", "天兔座", "Lepus"}, 1705: []string{"军井二", "", "天兔座κ", "", "天兔座", "Lepus"}, 1689: []string{"天潢五", "", "御夫座μ", "", "御夫座", "Auriga"}, 1744: []string{"夹白一", "", "剑鱼座θ", "", "剑鱼座", "Dorado"}, 897: []string{"天园六", "", "波江座θ2", "Acamar", "波江座", "Eridanus"}, 1735: []string{"玉井四", "", "猎户座τ", "", "猎户座", "Orion"}, 1729: []string{"咸池三", "", "御夫座λ", "", "御夫座", "Auriga"}, 1739: []string{"天高四", "", "金牛座n", "", "金牛座", "Taurus"}, 1756: []string{"军井三", "", "天兔座λ", "", "天兔座", "Lepus"}, 1788: []string{"参宿增三", "", "猎户座η", "", "猎户座", "Orion"}, 911: []string{"天囷一", "", "鲸鱼座α", "Menkar", "鲸鱼座", "Cetus"}, 936: []string{"大陵五", "", "英仙座β", "Algol", "英仙座", "Perseus"}, 1862: []string{"丈人二", "", "天鸽座ε", "", "天鸽座", "Columba"}, 1855: []string{"参宿增三十六", "", "猎户座υ", "", "猎户座", "Orion"}, 1843: []string{"柱七", "", "御夫座χ", "", "御夫座", "Auriga"}, 951: []string{"天阴四", "", "白羊座δ", "Botein", "白羊座", "Aries"}, 1922: []string{"金鱼三", "", "剑鱼座β", "", "剑鱼座", "Dorado"}, 1876: []string{"觜宿二", "", "猎户座φ1", "", "猎户座", "Orion"}, 1892: []string{"伐一", "", "猎户座c", "", "猎户座", "Orion"}, 1899: []string{"伐三", "", "猎户座ι", "", "猎户座", "Orion"}, 984: []string{"天苑五", "", "波江座ζ", "Zibal", "波江座", "Eridanus"}, 1907: []string{"觜宿三", "", "猎户座φ2", "", "猎户座", "Orion"}, 1910: []string{"天关", "", "金牛座ζ", "", "金牛座", "Taurus"}, 1017: []string{"天船三", "", "英仙座α", "Mirphak", "英仙座", "Perseus"}, 1983: []string{"厕三", "", "天兔座γ", "", "天兔座", "Lepus"}, 2015: []string{"金鱼四", "", "剑鱼座δ", "", "剑鱼座", "Dorado"}, 1995: []string{"柱六", "", "御夫座τ", "", "御夫座", "Auriga"}, 1131: []string{"卷舌五", "", "英仙座ο", "Atik", "英仙座", "Perseus"}, 2011: []string{"柱四", "", "御夫座υ", "", "御夫座", "Auriga"}, 2035: []string{"厕四", "", "天兔座δ", "", "天兔座", "Lepus"}, 2012: []string{"柱五", "", "御夫座ν", "", "御夫座", "Auriga"}, 2056: []string{"子一", "", "天鸽座λ", "", "天鸽座", "Columba"}, 2034: []string{"诸王一", "", "金牛座136", "", "金牛座", "Taurus"}, 2047: []string{"司怪四", "", "猎户座χ1", "", "猎户座", "Orion"}, 2029: []string{"八谷二", "", "御夫座ξ", "", "御夫座", "Auriga"}, 2084: []string{"司怪一", "", "金牛座139", "", "金牛座", "Taurus"}, 2077: []string{"八谷一", "", "御夫座δ", "", "御夫座", "Auriga"}, 1142: []string{"昴宿一", "", "金牛座17", "Electra", "金牛座", "Taurus"}, 2095: []string{"五车四", "", "御夫座θ", "", "御夫座", "Auriga"}, 2135: []string{"司怪三", "", "猎户座χ2", "", "猎户座", "Orion"}, 2134: []string{"司怪二", "", "双子座1", "", "双子座", "Gemini"}, 2159: []string{"水府一", "", "猎户座ν", "", "猎户座", "Orion"}, 2199: []string{"水府二", "", "猎户座ξ", "", "猎户座", "Orion"}, 2198: []string{"水府四", "", "猎户座f1", "", "猎户座", "Orion"}, 2256: []string{"孙一", "", "天鸽座κ", "", "天鸽座", "Columba"}, 1145: []string{"昴宿二", "", "金牛座q", "Taygeta", "金牛座", "Taurus"}, 1149: []string{"昴宿四", "", "金牛座20", "Maia", "金牛座", "Taurus"}, 2298: []string{"四渎四", "", "麒麟座εA", "", "麒麟座", "Monoceros"}, 2361: []string{"孙增二", "", "大犬座λ", "", "大犬座", "CanisMajor"}, 2343: []string{"井宿二", "", "双子座ν", "", "双子座", "Gemini"}, 2387: []string{"军市六", "", "大犬座ξ1", "", "大犬座", "CanisMajor"}, 2385: []string{"四渎三", "", "麒麟座13", "", "麒麟座", "Monoceros"}, 2429: []string{"野鸡", "", "大犬座ν2", "", "大犬座", "CanisMajor"}, 1156: []string{"昴宿五", "", "金牛座23", "Merope", "金牛座", "Taurus"}, 2451: []string{"老人增二", "", "船尾座ν", "", "船尾座", "Puppis"}, 2443: []string{"军市二", "", "大犬座ν3", "", "大犬座", "CanisMajor"}, 2427: []string{"座旗五", "", "御夫座ψ2", "", "御夫座", "Auriga"}, 1165: []string{"昴宿六", "", "金牛座η", "Alcyone", "金牛座", "Taurus"}, 2484: []string{"井宿四", "", "双子座ξ", "", "双子座", "Gemini"}, 2503: []string{"四渎二", "", "麒麟座17", "", "麒麟座", "Monoceros"}, 2506: []string{"阙丘一", "", "麒麟座18", "", "麒麟座", "Monoceros"}, 2538: []string{"弧矢八", "", "大犬座κ", "", "大犬座", "CanisMajor"}, 2553: []string{"老人增一", "", "船尾座τ", "", "船尾座", "Puppis"}, 2540: []string{"五诸侯一", "", "双子座θ", "", "双子座", "Gemini"}, 2580: []string{"军市五", "", "大犬座ο1", "", "大犬座", "CanisMajor"}, 2574: []string{"天狼增二", "", "大犬座θ", "", "大犬座", "CanisMajor"}, 2590: []string{"军市三", "", "大犬座π", "", "大犬座", "CanisMajor"}, 2585: []string{"座旗二", "", "御夫座ψ10", "", "御夫座", "Auriga"}, 2646: []string{"弧矢增二", "", "大犬座σ", "", "大犬座", "CanisMajor"}, 2653: []string{"军市增五", "", "大犬座ο2", "", "大犬座", "CanisMajor"}, 1178: []string{"昴宿七", "", "金牛座27", "Atlas", "金牛座", "Taurus"}, 1231: []string{"天苑一", "", "波江座γ", "Zaurak", "波江座", "Eridanus"}, 2736: []string{"飞鱼二", "", "飞鱼座γ2", "", "飞鱼座", "Volans"}, 2697: []string{"五诸侯二", "", "双子座τ", "", "双子座", "Gemini"}, 2714: []string{"阙丘二", "", "麒麟座δ", "", "麒麟座", "Monoceros"}, 2749: []string{"弧矢增三", "", "大犬座ω", "", "大犬座", "CanisMajor"}, 2803: []string{"飞鱼五", "", "飞鱼座δ", "", "飞鱼座", "Volans"}, 2773: []string{"弧矢九", "", "船尾座π", "", "船尾座", "Puppis"}, 2763: []string{"井宿八", "", "双子座λ", "", "双子座", "Gemini"}, 1228: []string{"卷舌三", "", "英仙座ξ", "Menkib", "英仙座", "Perseus"}, 2828: []string{"南河一", "", "小犬座ε", "", "小犬座", "CanisMinor"}, 2821: []string{"五诸侯三", "", "双子座ι", "", "双子座", "Gemini"}, 2852: []string{"北河一", "", "双子座ρ", "", "双子座", "Gemini"}, 2878: []string{"弧矢二十四", "", "船尾座σ", "", "船尾座", "Puppis"}, 2864: []string{"水位一", "", "小犬座6", "", "小犬座", "CanisMinor"}, 1298: []string{"九州殊口二", "", "波江座ο1", "Beid", "波江座", "Eridanus"}, 2905: []string{"五诸侯四", "", "双子座υ", "", "双子座", "Gemini"}, 3024: []string{"飞鱼六", "", "飞鱼座ζ", "", "飞鱼座", "Volans"}, 2985: []string{"积薪", "", "双子座κ", "", "双子座", "Gemini"}, 3017: []string{"弧矢三", "", "船尾座C", "", "船尾座", "Puppis"}, 1325: []string{"九州殊口增十一", "", "波江座ο2", "Keid", "波江座", "Eridanus"}, 3034: []string{"弧矢五", "", "船尾座ο", "", "船尾座", "Puppis"}, 3045: []string{"弧矢增十七", "", "船尾座ξ", "", "船尾座", "Puppis"}, 3067: []string{"五诸侯五", "", "双子座φ", "", "双子座", "Gemini"}, 3185: []string{"弧矢增三十二", "", "船尾座ρ", "", "船尾座", "Puppis"}, 1409: []string{"毕宿一", "", "金牛座ε", "Ain", "金牛座", "Taurus"}, 3249: []string{"柳宿增十", "", "巨蟹座β", "", "巨蟹座", "Cancer"}, 3340: []string{"小斗八", "", "蝘蜓座θ", "", "蝘蜓座", "Chamaeleon"}, 1457: []string{"毕宿五", "", "金牛座α", "Aldebaran", "金牛座", "Taurus"}, 3275: []string{"上台增四", "", "天猫座31", "", "天猫座", "Lynx"}, 3314: []string{"外厨一", "", "长蛇座C", "", "长蛇座", "Hydra"}, 3347: []string{"飞鱼三", "", "飞鱼座β", "", "飞鱼座", "Volans"}, 3426: []string{"天狗一", "", "船帆座e", "", "船帆座", "Vela"}, 3410: []string{"柳宿一", "", "长蛇座δ", "", "长蛇座", "Hydra"}, 3418: []string{"柳宿二", "", "长蛇座σ", "", "长蛇座", "Hydra"}, 3438: []string{"天狗四", "", "罗盘座β", "", "罗盘座", "Pyxis"}, 3403: []string{"内阶增七", "", "大熊座π2", "", "大熊座", "UrsaMajor"}, 3445: []string{"天社二", "", "船帆座B", "", "船帆座", "Vela"}, 3454: []string{"柳宿三", "", "长蛇座η", "", "长蛇座", "Hydra"}, 1542: []string{"紫微右垣六", "少卫", "鹿豹座α", "", "鹿豹座", "Camelopardalis"}, 3468: []string{"天狗五", "", "罗盘座α", "", "罗盘座", "Pyxis"}, 3459: []string{"外厨二", "", "长蛇座F", "", "长蛇座", "Hydra"}, 3477: []string{"天狗二", "", "船帆座D", "", "船帆座", "Vela"}, 3485: []string{"天社三", "", "船帆座δ", "", "船帆座", "Vela"}, 3482: []string{"柳宿五", "", "长蛇座ε", "", "长蛇座", "Hydra"}, 3492: []string{"柳宿四", "", "长蛇座ρ", "", "长蛇座", "Hydra"}, 3518: []string{"天狗六", "", "罗盘座γ", "", "罗盘座", "Pyxis"}, 3547: []string{"柳宿六", "", "长蛇座ζ", "", "长蛇座", "Hydra"}, 3556: []string{"天狗七", "", "罗盘座δ", "", "罗盘座", "Pyxis"}, 1666: []string{"玉井三", "", "波江座β", "Cursa", "波江座", "Eridanus"}, 3579: []string{"轩辕一", "", "大熊座10", "", "大熊座", "UrsaMajor"}, 3615: []string{"飞鱼一", "", "飞鱼座α", "", "飞鱼座", "Volans"}, 3576: []string{"三师一", "", "大熊座ρ", "", "大熊座", "UrsaMajor"}, 3594: []string{"三台二", "", "大熊座κ", "", "大熊座", "UrsaMajor"}, 3613: []string{"柳宿七", "", "长蛇座ω", "", "长蛇座", "Hydra"}, 1713: []string{"参宿七", "", "猎户座β", "Rigel", "猎户座", "Orion"}, 3619: []string{"文昌五", "", "大熊座f", "", "大熊座", "UrsaMajor"}, 3665: []string{"柳宿八", "", "长蛇座θ", "", "长蛇座", "Hydra"}, 3662: []string{"文昌六", "", "大熊座e", "", "大熊座", "UrsaMajor"}, 1708: []string{"五车二", "", "御夫座α", "Capella", "御夫座", "Auriga"}, 3690: []string{"轩辕三", "", "天猫座38", "", "天猫座", "Lynx"}, 3706: []string{"星宿六", "", "长蛇座26", "", "长蛇座", "Hydra"}, 3709: []string{"星宿五", "", "长蛇座27", "", "长蛇座", "Hydra"}, 3705: []string{"轩辕四", "", "天猫座α", "", "天猫座", "Lynx"}, 3734: []string{"天社五", "", "船帆座κ", "", "船帆座", "Vela"}, 3731: []string{"轩辕七", "", "狮子座κ", "", "狮子座", "Leo"}, 3759: []string{"星宿二", "", "长蛇座τ1", "", "长蛇座", "Hydra"}, 3803: []string{"天社六", "", "船帆座n", "", "船帆座", "Vela"}, 3757: []string{"内阶四", "", "大熊座h", "", "大熊座", "UrsaMajor"}, 1790: []string{"参宿五", "", "猎户座γ", "Bellatrix", "猎户座", "Orion"}, 3782: []string{"酒旗二", "", "狮子座ξ", "", "狮子座", "Leo"}, 3787: []string{"星宿三", "", "长蛇座τ2", "", "长蛇座", "Hydra"}, 3775: []string{"文昌四", "", "大熊座θ", "", "大熊座", "UrsaMajor"}, 3825: []string{"海石三", "", "船底座h", "", "船底座", "Carina"}, 3771: []string{"紫微右垣四", "", "大熊座d", "", "大熊座", "UrsaMajor"}, 3845: []string{"星宿四", "", "长蛇座ι", "", "长蛇座", "Hydra"}, 3884: []string{"海石四", "", "船底座l", "", "船底座", "Carina"}, 3873: []string{"轩辕九", "", "狮子座ε", "", "狮子座", "Leo"}, 3890: []string{"海石五", "", "船底座υ", "", "船底座", "Carina"}, 3888: []string{"文昌二", "", "大熊座υ", "", "大熊座", "UrsaMajor"}, 3903: []string{"张宿一", "", "长蛇座υ1", "", "长蛇座", "Hydra"}, 3894: []string{"文昌三", "", "大熊座φ", "", "大熊座", "UrsaMajor"}, 1791: []string{"五车五", "", "金牛座β", "Alnath", "金牛座", "Taurus"}, 3975: []string{"轩辕十三", "", "狮子座η", "", "狮子座", "Leo"}, 3974: []string{"内平二", "", "小狮座21", "", "小狮座", "LeoMinor"}, 3980: []string{"轩辕十七", "", "狮子座31", "", "狮子座", "Leo"}, 3994: []string{"张宿二", "", "长蛇座λ", "", "长蛇座", "Hydra"}, 4037: []string{"南船四", "", "船底座ω", "", "船底座", "Carina"}, 1829: []string{"厕二", "", "天兔座β", "Nihal", "天兔座", "Lepus"}, 4050: []string{"南船一", "", "船底座q", "", "船底座", "Carina"}, 1852: []string{"参宿三", "", "猎户座δ", "Mintaka", "猎户座", "Orion"}, 4094: []string{"张宿三", "", "长蛇座μ", "", "长蛇座", "Hydra"}, 4114: []string{"海山一", "", "船底座s", "", "船底座", "Carina"}, 4140: []string{"南船二", "", "船底座p", "", "船底座", "Carina"}, 4133: []string{"轩辕十六", "", "狮子座ρ", "", "狮子座", "Leo"}, 4174: []string{"小斗三", "", "蝘蜓座γ", "", "蝘蜓座", "Chamaeleon"}, 4171: []string{"张宿六", "", "长蛇座φ3", "", "长蛇座", "Hydra"}, 4199: []string{"南船三", "", "船底座θ", "", "船底座", "Carina"}, 4234: []string{"小斗四", "", "蝘蜓座δ2", "", "蝘蜓座", "Chamaeleon"}, 4216: []string{"海山增二", "", "船帆座μ", "", "船帆座", "Vela"}, 4232: []string{"翼宿五", "", "长蛇座ν", "", "长蛇座", "Hydra"}, 4247: []string{"势四", "", "小狮座46", "", "小狮座", "LeoMinor"}, 4248: []string{"天牢一", "", "大熊座ω", "", "大熊座", "UrsaMajor"}, 4259: []string{"少微二", "", "狮子座54", "", "狮子座", "Leo"}, 1865: []string{"厕一", "", "天兔座α", "Arneb", "天兔座", "Lepus"}, 4291: []string{"灵台三", "", "狮子座58", "", "狮子座", "Leo"}, 4294: []string{"灵台二", "", "狮子座c", "", "狮子座", "Leo"}, 1879: []string{"觜宿一", "", "猎户座λ", "Meissa", "猎户座", "Orion"}, 1903: []string{"参宿二", "", "猎户座ε", "Alnilam", "猎户座", "Orion"}, 4310: []string{"灵台一", "", "狮子座χ", "", "狮子座", "Leo"}, 4314: []string{"翼宿二十", "", "长蛇座χ1", "", "长蛇座", "Hydra"}, 4335: []string{"太尊", "", "大熊座ψ", "", "大熊座", "UrsaMajor"}, 4343: []string{"翼宿十六", "", "巨爵座β", "", "巨爵座", "Crater"}, 1956: []string{"丈人一", "", "天鸽座α", "Phact", "天鸽座", "Columba"}, 4362: []string{"虎贲", "", "狮子座72", "", "狮子座", "Leo"}, 4382: []string{"翼宿七", "", "巨爵座δ", "", "巨爵座", "Crater"}, 4386: []string{"太微右垣二", "西上将", "狮子座σ", "", "狮子座", "Leo"}, 4392: []string{"天牢六", "", "大熊座56", "", "大熊座", "UrsaMajor"}, 4399: []string{"太微右垣三", "西次将", "狮子座ι", "", "狮子座", "Leo"}, 4402: []string{"翼宿十", "", "巨爵座ε", "", "巨爵座", "Crater"}, 4405: []string{"翼宿二", "", "巨爵座γ", "", "巨爵座", "Crater"}, 4418: []string{"明堂一", "", "狮子座τ", "", "狮子座", "Leo"}, 4432: []string{"明堂三", "", "狮子座e", "", "狮子座", "Leo"}, 1948: []string{"参宿一", "", "猎户座ζ", "Alnitak", "猎户座", "Orion"}, 4450: []string{"青丘五", "", "长蛇座ξ", "", "长蛇座", "Hydra"}, 4467: []string{"海山五", "", "半人马座λ", "", "半人马座", "Centaurus"}, 4468: []string{"翼宿十三", "", "巨爵座θ", "", "巨爵座", "Crater"}, 4471: []string{"明堂二", "", "狮子座υ", "", "狮子座", "Leo"}, 4494: []string{"青丘七", "", "长蛇座ο", "", "长蛇座", "Hydra"}, 4514: []string{"翼宿三", "", "巨爵座ζ", "", "巨爵座", "Crater"}, 4515: []string{"内屏一", "", "室女座ξ", "", "室女座", "Virgo"}, 4520: []string{"海山六", "", "苍蝇座λ", "", "苍蝇座", "Musca"}, 4517: []string{"内屏二", "", "室女座ν", "", "室女座", "Virgo"}, 4518: []string{"太阳守", "", "大熊座χ", "", "大熊座", "UrsaMajor"}, 4527: []string{"太子", "", "狮子座93", "", "狮子座", "Leo"}, 2004: []string{"参宿六", "", "猎户座κ", "Saiph", "猎户座", "Orion"}, 4552: []string{"青丘一", "", "长蛇座β", "", "长蛇座", "Hydra"}, 4583: []string{"小斗二", "", "蝘蜓座ε", "", "蝘蜓座", "Chamaeleon"}, 4589: []string{"内屏三", "", "室女座π", "", "室女座", "Virgo"}, 4608: []string{"内屏四", "", "室女座ο", "", "室女座", "Virgo"}, 4621: []string{"马尾三", "", "半人马座δ", "", "半人马座", "Centaurus"}, 2040: []string{"子二", "", "天鸽座β", "Wazn", "天鸽座", "Columba"}, 4630: []string{"轸宿二", "", "乌鸦座ε", "", "乌鸦座", "Corvus"}, 4638: []string{"马尾二", "", "半人马座ρ", "", "半人马座", "Centaurus"}, 4656: []string{"十字架四", "", "南十字座δ", "", "南十字座", "Crux"}, 2061: []string{"参宿四", "", "猎户座α", "Betelgeuse", "猎户座", "Orion"}, 4667: []string{"郎位十", "", "后发座7", "", "后发座", "ComaBerenices"}, 4674: []string{"小斗一", "", "蝘蜓座β", "", "蝘蜓座", "Chamaeleon"}, 4695: []string{"谒者", "", "室女座c", "", "室女座", "Virgo"}, 4707: []string{"郎位七", "", "后发座12", "", "后发座", "ComaBerenices"}, 4716: []string{"相", "", "猎犬座5", "", "猎犬座", "CanesVenatici"}, 4733: []string{"郎位三", "", "后发座14", "", "后发座", "ComaBerenices"}, 4732: []string{"马尾一", "", "半人马座G", "", "半人马座", "Centaurus"}, 2088: []string{"五车三", "", "御夫座β", "Menkalinan", "御夫座", "Auriga"}, 4737: []string{"郎位一", "", "后发座γ", "", "后发座", "ComaBerenices"}, 4738: []string{"郎位四", "", "后发座16", "", "后发座", "ComaBerenices"}, 4743: []string{"库楼十", "", "半人马座σ", "", "半人马座", "Centaurus"}, 2216: []string{"钺", "", "双子座η", "Propus", "双子座", "Gemini"}, 4775: []string{"左辖", "", "乌鸦座η", "", "乌鸦座", "Corvus"}, 4787: []string{"紫微右垣二", "少尉", "天龙座κ", "", "天龙座", "Draco"}, 4786: []string{"轸宿四", "", "乌鸦座β", "", "乌鸦座", "Corvus"}, 4789: []string{"郎位十一", "", "后发座23", "", "后发座", "ComaBerenices"}, 4798: []string{"蜜蜂三", "", "苍蝇座α", "", "苍蝇座", "Musca"}, 4802: []string{"库楼八", "", "半人马座τ", "", "半人马座", "Centaurus"}, 4819: []string{"库楼七", "", "半人马座γ", "", "半人马座", "Centaurus"}, 2282: []string{"孙增一", "", "大犬座ζ", "Furud", "大犬座", "CanisMajor"}, 4828: []string{"九卿一", "", "室女座ρ", "", "室女座", "Virgo"}, 4844: []string{"蜜蜂一", "", "苍蝇座β", "", "苍蝇座", "Musca"}, 4883: []string{"郎将", "", "后发座31", "", "后发座", "ComaBerenices"}, 2294: []string{"军市一", "", "大犬座β", "Mirzam", "大犬座", "CanisMajor"}, 4910: []string{"太微左垣三", "", "室女座δ", "", "室女座", "Virgo"}, 4920: []string{"五诸侯二", "", "后发座36", "", "后发座", "ComaBerenices"}, 4924: []string{"周鼎二", "", "后发座37", "", "后发座", "ComaBerenices"}, 2286: []string{"井宿一", "", "双子座μ", "Tejat", "双子座", "Gemini"}, 4940: []string{"库楼六", "", "半人马座F", "", "半人马座", "Centaurus"}, 4954: []string{"周鼎三", "", "后发座41", "", "后发座", "ComaBerenices"}, 4963: []string{"平道一", "", "室女座θ", "", "室女座", "Virgo"}, 4968: []string{"太微左垣五", "东上将", "后发座α", "", "后发座", "ComaBerenices"}, 4983: []string{"周鼎一", "", "后发座β", "", "后发座", "ComaBerenices"}, 5020: []string{"平一", "", "长蛇座γ", "", "长蛇座", "Hydra"}, 5028: []string{"柱十一", "", "半人马座ι", "", "半人马座", "Centaurus"}, 2326: []string{"老人", "", "船底座α", "Canopus", "船底座", "Carina"}, 5068: []string{"天门二", "", "室女座69", "", "室女座", "Virgo"}, 5089: []string{"库楼五", "", "半人马座D", "", "半人马座", "Centaurus"}, 5105: []string{"天田一", "", "室女座o", "", "室女座", "Virgo"}, 5112: []string{"三公二", "", "猎犬座24", "", "猎犬座", "CanesVenatici"}, 5107: []string{"角宿二", "", "室女座ζ", "", "室女座", "Virgo"}, 5132: []string{"南门一", "", "半人马座ε", "", "半人马座", "Centaurus"}, 5168: []string{"柱九", "", "半人马座i", "", "半人马座", "Centaurus"}, 5185: []string{"右摄提二", "", "牧夫座τ", "", "牧夫座", "Bootes"}, 2421: []string{"井宿三", "", "双子座γ", "Alhena", "双子座", "Gemini"}, 5192: []string{"库楼四", "", "半人马座2", "", "半人马座", "Centaurus"}, 5200: []string{"右摄提三", "", "牧夫座υ", "", "牧夫座", "Bootes"}, 5190: []string{"衡一", "", "半人马座ν", "", "半人马座", "Centaurus"}, 5193: []string{"衡二", "", "半人马座μ", "", "半人马座", "Centaurus"}, 5226: []string{"天一", "", "天龙座i", "", "天龙座", "Draco"}, 5210: []string{"柱八", "", "半人马座3", "", "半人马座", "Centaurus"}, 5221: []string{"柱七", "", "半人马座H", "", "半人马座", "Centaurus"}, 5231: []string{"库楼一", "", "半人马座ζ", "", "半人马座", "Centaurus"}, 5248: []string{"衡三", "", "半人马座φ", "", "半人马座", "Centaurus"}, 5249: []string{"柱二", "", "半人马座υ1", "", "半人马座", "Centaurus"}, 5264: []string{"天田二", "", "室女座τ", "", "室女座", "Virgo"}, 5260: []string{"柱一", "", "半人马座υ2", "", "半人马座", "Centaurus"}, 2473: []string{"井宿五", "", "双子座ε", "Mebsuta", "双子座", "Gemini"}, 5285: []string{"衡四", "", "半人马座χ", "", "半人马座", "Centaurus"}, 5287: []string{"平二", "", "长蛇座π", "", "长蛇座", "Hydra"}, 2491: []string{"天狼", "", "大犬座α", "Sirius", "大犬座", "CanisMajor"}, 5321: []string{"北极四", "后宫", "小熊座4", "", "小熊座", "UrsaMinor"}, 5304: []string{"帝席一", "", "牧夫座d", "", "牧夫座", "Bootes"}, 5315: []string{"亢宿一", "", "室女座κ", "", "室女座", "Virgo"}, 2618: []string{"弧矢七", "", "大犬座ε", "Adhara", "大犬座", "CanisMajor"}, 5350: []string{"天枪二", "", "牧夫座ι", "", "牧夫座", "Bootes"}, 5351: []string{"玄戈", "", "牧夫座λ", "", "牧夫座", "Bootes"}, 5303: []string{"异雀七", "", "天燕座η", "", "天燕座", "Apus"}, 5359: []string{"亢宿四", "", "室女座λ", "", "室女座", "Virgo"}, 5354: []string{"柱三", "", "豺狼座ι", "", "豺狼座", "Lupus"}, 5370: []string{"亢池一", "", "牧夫座20", "", "牧夫座", "Bootes"}, 5367: []string{"柱六", "", "半人马座ψ", "", "半人马座", "Centaurus"}, 5378: []string{"柱五", "", "半人马座A", "", "半人马座", "Centaurus"}, 5404: []string{"天枪三", "", "牧夫座θ", "", "牧夫座", "Bootes"}, 5396: []string{"柱四", "", "豺狼座τ2", "", "豺狼座", "Lupus"}, 2657: []string{"天狼增四", "", "大犬座γ", "Muliphein", "大犬座", "CanisMajor"}, 5409: []string{"亢宿三", "", "室女座φ", "", "室女座", "Virgo"}, 5429: []string{"梗河三", "", "牧夫座ρ", "", "牧夫座", "Bootes"}, 5425: []string{"车骑三", "", "豺狼座σ", "", "豺狼座", "Lupus"}, 5447: []string{"梗河二", "", "牧夫座σ", "", "牧夫座", "Bootes"}, 5440: []string{"库楼二", "", "半人马座η", "", "半人马座", "Centaurus"}, 5453: []string{"车骑二", "", "豺狼座ρ", "", "豺狼座", "Lupus"}, 2650: []string{"井宿七", "", "双子座ζ", "Mekbuda", "双子座", "Gemini"}, 5475: []string{"左摄提二", "", "牧夫座π2", "", "牧夫座", "Bootes"}, 5477: []string{"左摄提三", "", "牧夫座ζ", "", "牧夫座", "Bootes"}, 5469: []string{"骑官十", "", "豺狼座α", "", "豺狼座", "Lupus"}, 5471: []string{"阳门一", "", "半人马座B", "", "半人马座", "Centaurus"}, 5487: []string{"亢宿增七", "", "室女座μ", "", "室女座", "Virgo"}, 5485: []string{"阳门二", "", "半人马座C1", "", "半人马座", "Centaurus"}, 5502: []string{"左摄提一", "", "牧夫座ο", "", "牧夫座", "Bootes"}, 5470: []string{"异雀八", "", "天燕座α", "", "天燕座", "Apus"}, 5526: []string{"阵车一", "", "长蛇座E", "", "长蛇座", "Hydra"}, 2693: []string{"弧矢一", "", "大犬座δ", "Wezen", "大犬座", "CanisMajor"}, 5528: []string{"骑官九", "", "豺狼座ο", "", "豺狼座", "Lupus"}, 5589: []string{"天床五", "", "小熊座Rr", "", "小熊座", "UrsaMinor"}, 5571: []string{"骑官四", "", "豺狼座β", "", "豺狼座", "Lupus"}, 5576: []string{"骑官三", "", "半人马座κ", "", "半人马座", "Centaurus"}, 2777: []string{"天樽二", "", "双子座δ", "Wasat", "双子座", "Gemini"}, 5603: []string{"折威七", "", "天秤座σ", "", "天秤座", "Libra"}, 5605: []string{"骑官八", "", "豺狼座π", "", "豺狼座", "Lupus"}, 5626: []string{"骑官五", "", "豺狼座λ", "", "豺狼座", "Lupus"}, 5646: []string{"骑阵将军", "", "豺狼座κ1", "", "豺狼座", "Lupus"}, 5652: []string{"氐宿二", "", "天秤座ι1", "", "天秤座", "Libra"}, 5649: []string{"车骑一", "", "豺狼座ζ", "", "豺狼座", "Lupus"}, 5660: []string{"顿顽二", "", "豺狼座i", "", "豺狼座", "Lupus"}, 5681: []string{"七公七", "", "牧夫座δ", "", "牧夫座", "Bootes"}, 5686: []string{"阵车三", "", "豺狼座f", "", "豺狼座", "Lupus"}, 5683: []string{"骑官七", "", "豺狼座μ", "", "豺狼座", "Lupus"}, 5671: []string{"三角形一", "", "南三角座γ", "", "南三角座", "TriangulumAustrale"}, 2827: []string{"弧矢二", "", "大犬座η", "Aludra", "大犬座", "CanisMajor"}, 5695: []string{"骑官二", "", "豺狼座δ", "", "豺狼座", "Lupus"}, 5705: []string{"顿顽一", "", "豺狼座φ1", "", "豺狼座", "Lupus"}, 5708: []string{"骑官六", "", "豺狼座ε", "", "豺狼座", "Lupus"}, 2845: []string{"南河二", "", "小犬座β", "Gomeisa", "小犬座", "CanisMinor"}, 5778: []string{"贯索二", "", "北冕座θ", "", "北冕座", "CoronaBorealis"}, 2891: []string{"北河二", "", "双子座α", "Castor", "双子座", "Gemini"}, 5788: []string{"天市右垣六", "", "巨蛇座δ", "", "巨蛇座", "Serpens"}, 5776: []string{"骑官一", "", "豺狼座γ", "", "豺狼座", "Lupus"}, 5787: []string{"氐宿三", "", "天秤座γ", "", "天秤座", "Libra"}, 5794: []string{"天辐一", "", "天秤座υ", "", "天秤座", "Libra"}, 5812: []string{"天辐二", "", "天秤座τ", "", "天秤座", "Libra"}, 5838: []string{"日", "", "天秤座κ", "", "天秤座", "Libra"}, 5839: []string{"从官一", "", "豺狼座ψ2", "", "豺狼座", "Lupus"}, 5849: []string{"贯索五", "", "北冕座γ", "", "北冕座", "CoronaBorealis"}, 5903: []string{"勾陈四", "", "小熊座ζ", "", "小熊座", "UrsaMinor"}, 5867: []string{"天市右垣五", "", "巨蛇座β", "", "巨蛇座", "Serpens"}, 5889: []string{"贯索六", "", "北冕座δ", "", "北冕座", "CoronaBorealis"}, 5881: []string{"天乳", "", "巨蛇座μ", "", "巨蛇座", "Serpens"}, 5892: []string{"天市右垣八", "", "巨蛇座ε", "", "巨蛇座", "Serpens"}, 5883: []string{"从官二", "", "豺狼座χ", "", "豺狼座", "Lupus"}, 5914: []string{"七公四", "", "武仙座χ", "", "武仙座", "Hercules"}, 5908: []string{"西咸三", "", "天秤座θ", "", "天秤座", "Libra"}, 5897: []string{"三角形二", "", "南三角座β", "", "南三角座", "TriangulumAustrale"}, 5933: []string{"天市右垣四", "", "巨蛇座γ", "", "巨蛇座", "Serpens"}, 5928: []string{"房宿二", "", "天蝎座ρ", "", "天蝎座", "Scorpius"}, 5947: []string{"贯索七", "", "北冕座ε", "", "北冕座", "CoronaBorealis"}, 5941: []string{"西咸二", "", "天秤座48", "", "天秤座", "Libra"}, 5944: []string{"房宿一", "", "天蝎座π", "", "天蝎座", "Scorpius"}, 5948: []string{"积卒二", "", "豺狼座η", "", "豺狼座", "Lupus"}, 2943: []string{"南河三", "", "小犬座α", "Procyon", "小犬座", "CanisMinor"}, 5971: []string{"贯索八", "", "北冕座ι", "", "北冕座", "CoronaBorealis"}, 5986: []string{"紫微左垣二", "上宰", "天龙座θ", "", "天龙座", "Draco"}, 5987: []string{"积卒一", "", "豺狼座θ", "", "豺狼座", "Lupus"}, 5993: []string{"钩钤一", "", "天蝎座ω1", "", "天蝎座", "Scorpius"}, 5997: []string{"钩钤二", "", "天蝎座ω2", "", "天蝎座", "Scorpius"}, 6023: []string{"七公三", "", "武仙座φ", "", "武仙座", "Hercules"}, 6027: []string{"键闭", "", "天蝎座ν", "", "天蝎座", "Scorpius"}, 2990: []string{"北河三", "", "双子座β", "Pollux", "双子座", "Gemini"}, 6116: []string{"勾陈增九", "", "小熊座η", "", "小熊座", "UrsaMinor"}, 6092: []string{"七公二", "", "武仙座τ", "", "武仙座", "Hercules"}, 6020: []string{"异雀六", "", "天燕座δ1", "", "天燕座", "Apus"}, 3165: []string{"弧矢增二十二", "", "船尾座ζ", "Naos", "船尾座", "Puppis"}, 6095: []string{"天市右垣二", "河间", "武仙座γ", "", "武仙座", "Hercules"}, 6093: []string{"列肆一", "", "巨蛇座σ", "", "巨蛇座", "Serpens"}, 6103: []string{"天纪一", "", "北冕座ξ", "", "北冕座", "CoronaBorealis"}, 6132: []string{"紫微左垣三", "少宰", "天龙座η", "", "天龙座", "Draco"}, 6104: []string{"东咸三", "", "蛇夫座ψ", "", "蛇夫座", "Ophiuchus"}, 6118: []string{"东咸二", "", "蛇夫座χ", "", "蛇夫座", "Ophiuchus"}, 6129: []string{"车肆一", "", "蛇夫座υ", "", "蛇夫座", "Ophiuchus"}, 6161: []string{"尚书二", "", "天龙座15", "", "天龙座", "Draco"}, 3207: []string{"天社一", "", "船帆座γ2", "Regor", "船帆座", "Vela"}, 3210: []string{"水位四", "", "巨蟹座ζ2", "Tegmine", "巨蟹座", "Cancer"}, 6147: []string{"东咸一", "", "蛇夫座φ", "", "蛇夫座", "Ophiuchus"}, 6153: []string{"东咸四", "", "蛇夫座ω", "", "蛇夫座", "Ophiuchus"}, 6159: []string{"斗四", "", "武仙座h", "", "武仙座", "Hercules"}, 6102: []string{"异雀四", "", "天燕座γ", "", "天燕座", "Apus"}, 6165: []string{"心宿三", "", "天蝎座τ", "", "天蝎座", "Scorpius"}, 6175: []string{"天市右垣十一", "", "蛇夫座ζ", "", "蛇夫座", "Ophiuchus"}, 6200: []string{"七公一", "", "武仙座42", "", "武仙座", "Hercules"}, 6223: []string{"尚书三", "", "天龙座g", "", "天龙座", "Draco"}, 6212: []string{"天纪二", "", "武仙座ζ", "", "武仙座", "Hercules"}, 6163: []string{"异雀三", "", "天燕座β", "", "天燕座", "Apus"}, 6322: []string{"勾陈三", "", "小熊座ε", "", "小熊座", "UrsaMinor"}, 6229: []string{"龟四", "", "天坛座η", "", "天坛座", "Ara"}, 6243: []string{"车肆二", "", "蛇夫座20", "", "蛇夫座", "Ophiuchus"}, 6241: []string{"尾宿二", "", "天蝎座ε", "", "天蝎座", "Scorpius"}, 6247: []string{"尾宿一", "", "天蝎座μ1", "", "天蝎座", "Scorpius"}, 6262: []string{"尾宿三", "", "天蝎座ζ1", "", "天蝎座", "Scorpius"}, 6281: []string{"斛一", "", "蛇夫座ι", "", "蛇夫座", "Ophiuchus"}, 6315: []string{"尚书五", "", "天龙座h", "", "天龙座", "Draco"}, 6299: []string{"斛二", "", "蛇夫座κ", "", "蛇夫座", "Ophiuchus"}, 6285: []string{"龟五", "", "天坛座ζ", "", "天坛座", "Ara"}, 6295: []string{"龟一", "", "天坛座ε1", "", "天坛座", "Ara"}, 6324: []string{"天纪三", "", "武仙座ε", "", "武仙座", "Hercules"}, 3307: []string{"海石一", "", "船底座ε", "Avior", "船底座", "Carina"}, 6355: []string{"宦者三", "", "武仙座60", "", "武仙座", "Hercules"}, 6396: []string{"紫微左垣四", "", "天龙座ζ", "", "天龙座", "Draco"}, 6380: []string{"尾宿四", "", "天蝎座η", "", "天蝎座", "Scorpius"}, 3323: []string{"内阶一", "", "大熊座ο", "Muscida", "大熊座", "UrsaMajor"}, 6410: []string{"天市左垣一", "", "武仙座δ", "", "武仙座", "Hercules"}, 6418: []string{"女床一", "", "武仙座π", "", "武仙座", "Hercules"}, 6401: []string{"天江二", "", "蛇夫座36", "", "蛇夫座", "Ophiuchus"}, 6431: []string{"天纪六", "", "武仙座u", "", "武仙座", "Hercules"}, 6436: []string{"女床二", "", "武仙座e", "", "武仙座", "Hercules"}, 6446: []string{"市楼四", "", "巨蛇座ν", "", "巨蛇座", "Serpens"}, 6417: []string{"异雀一", "", "天燕座ζ", "", "天燕座", "Apus"}, 6453: []string{"天江三", "", "蛇夫座θ", "", "蛇夫座", "Ophiuchus"}, 6484: []string{"女床三", "", "武仙座ρ", "", "武仙座", "Hercules"}, 6461: []string{"杵三", "", "天坛座β", "", "天坛座", "Ara"}, 6462: []string{"龟二", "", "天坛座γ", "", "天坛座", "Ara"}, 6486: []string{"天江四", "", "蛇夫座b", "", "蛇夫座", "Ophiuchus"}, 6492: []string{"糠", "", "蛇夫座d", "", "蛇夫座", "Ophiuchus"}, 3449: []string{"鬼宿三", "", "巨蟹座γ", "AsellusBorealis", "巨蟹座", "Cancer"}, 6500: []string{"龟三", "", "天坛座δ", "", "天坛座", "Ara"}, 6519: []string{"天龠六", "", "蛇夫座c", "", "蛇夫座", "Ophiuchus"}, 6510: []string{"杵二", "", "天坛座α", "", "天坛座", "Ara"}, 6554: []string{"天棓二", "", "天龙座ν1", "", "天龙座", "Draco"}, 3461: []string{"鬼宿四", "", "巨蟹座δ", "AsellusAustralis", "巨蟹座", "Cancer"}, 3572: []string{"柳宿增三", "", "巨蟹座α", "Acubens", "巨蟹座", "Cancer"}, 6537: []string{"杵一", "", "天坛座σ", "", "天坛座", "Ara"}, 6561: []string{"天市左垣十", "", "巨蛇座ξ", "", "巨蛇座", "Serpens"}, 6567: []string{"市楼一", "", "蛇夫座μ", "", "蛇夫座", "Ophiuchus"}, 6588: []string{"天棓五", "", "武仙座ι", "", "武仙座", "Hercules"}, 6581: []string{"市楼二", "", "巨蛇座ο", "", "巨蛇座", "Serpens"}, 6636: []string{"女史", "", "天龙座ψ1A", "", "天龙座", "Draco"}, 6580: []string{"尾宿七", "", "天蝎座κ", "", "天蝎座", "Scorpius"}, 6595: []string{"天龠三", "", "蛇夫座58", "", "蛇夫座", "Ophiuchus"}, 3569: []string{"三台一", "", "大熊座ι", "Talitha", "大熊座", "UrsaMajor"}, 6582: []string{"孔雀一", "", "孔雀座η", "", "孔雀座", "Pavo"}, 6623: []string{"天市左垣三", "", "武仙座μ", "", "武仙座", "Hercules"}, 6616: []string{"天龠八", "", "人马座3", "", "人马座", "Sagittarius"}, 6615: []string{"尾宿六", "", "天蝎座ι1", "", "天蝎座", "Scorpius"}, 6629: []string{"宗正二", "", "蛇夫座γ", "", "蛇夫座", "Ophiuchus"}, 6630: []string{"傅说", "", "天蝎座G", "", "天蝎座", "Scorpius"}, 6695: []string{"天纪九", "", "武仙座θ", "", "武仙座", "Hercules"}, 3634: []string{"天记", "", "船帆座λ", "Suhail", "船帆座", "Vela"}, 6698: []string{"天市左垣九", "", "蛇夫座ν", "", "蛇夫座", "Ophiuchus"}, 6712: []string{"宗人一", "", "蛇夫座66", "", "蛇夫座", "Ophiuchus"}, 6714: []string{"宗人二", "", "蛇夫座67", "", "蛇夫座", "Ophiuchus"}, 6729: []string{"帛度一", "", "武仙座95", "", "武仙座", "Hercules"}, 6723: []string{"宗人三", "", "蛇夫座68", "", "蛇夫座", "Ophiuchus"}, 6733: []string{"市楼三", "", "蛇夫座τ", "", "蛇夫座", "Ophiuchus"}, 6752: []string{"宗人四", "", "蛇夫座70", "", "蛇夫座", "Ophiuchus"}, 6779: []string{"天市左垣四", "中山", "武仙座ο", "", "武仙座", "Hercules"}, 6745: []string{"孔雀二", "", "孔雀座π", "", "孔雀座", "Pavo"}, 6787: []string{"帛度二", "", "武仙座102", "", "武仙座", "Hercules"}, 6812: []string{"斗宿三", "", "人马座μ", "", "人马座", "Sagittarius"}, 6832: []string{"箕宿四", "", "人马座η", "", "人马座", "Sagittarius"}, 6920: []string{"柱史", "", "天龙座φ", "", "天龙座", "Draco"}, 3685: []string{"南船五", "", "船底座β", "Miaplacidus", "船底座", "Carina"}, 6927: []string{"御女四", "", "天龙座χ", "", "天龙座", "Draco"}, 6869: []string{"天市左垣八", "东海", "巨蛇座η", "", "巨蛇座", "Serpens"}, 6895: []string{"屠肆一", "", "武仙座109", "", "武仙座", "Hercules"}, 6923: []string{"扶筐三", "", "天龙座b", "", "天龙座", "Draco"}, 6897: []string{"鳖一", "", "望远镜座α", "", "望远镜座", "Telescopium"}, 3699: []string{"海石二", "", "船底座ι", "Aspidiske", "船底座", "Carina"}, 6916: []string{"孔雀三", "", "孔雀座ν", "", "孔雀座", "Pavo"}, 6978: []string{"扶筐二", "", "天龙座d", "", "天龙座", "Draco"}, 6951: []string{"鳖十一", "", "南冕座θ", "", "南冕座", "CoronaAustrilis"}, 6973: []string{"天弁一", "", "盾牌座α", "", "盾牌座", "Scutum"}, 7020: []string{"天弁二", "", "盾牌座δ", "", "盾牌座", "Scutum"}, 6982: []string{"孔雀八", "", "孔雀座ζ", "", "孔雀座", "Pavo"}, 7032: []string{"天弁三", "", "盾牌座ε", "", "盾牌座", "Scutum"}, 7051: []string{"织女二", "", "天琴座ε1", "", "天琴座", "Lyra"}, 7056: []string{"织女三", "", "天琴座ζ1", "", "天琴座", "Lyra"}, 7039: []string{"斗宿一", "", "人马座φ", "", "人马座", "Sagittarius"}, 7061: []string{"宗一", "", "武仙座110", "", "武仙座", "Hercules"}, 7069: []string{"宗二", "", "武仙座111", "", "武仙座", "Hercules"}, 7063: []string{"天弁四", "", "盾牌座β", "", "盾牌座", "Scutum"}, 3748: []string{"星宿一", "", "长蛇座α", "Alphard", "长蛇座", "Hydra"}, 7125: []string{"扶筐四", "", "天龙座ο", "", "天龙座", "Draco"}, 7074: []string{"孔雀四", "", "孔雀座λ", "", "孔雀座", "Pavo"}, 7180: []string{"紫微左垣五", "少弼", "天龙座υ", "", "天龙座", "Draco"}, 7139: []string{"渐台一", "", "天琴座δ2", "", "天琴座", "Lyra"}, 7157: []string{"辇道一", "", "天琴座R", "", "天琴座", "Lyra"}, 3773: []string{"轩辕八", "", "狮子座λ", "Alterf", "狮子座", "Leo"}, 7107: []string{"孔雀五", "", "孔雀座κ", "", "孔雀座", "Pavo"}, 7149: []string{"天弁五", "", "盾牌座η", "", "盾牌座", "Scutum"}, 7150: []string{"建一", "", "人马座ξ2", "", "人马座", "Sagittarius"}, 7152: []string{"鳖八", "", "南冕座ε", "", "南冕座", "CoronaAustrilis"}, 7176: []string{"吴越增一", "", "天鹰座ε", "", "天鹰座", "Aquila"}, 7193: []string{"天弁六", "", "天鹰座i", "", "天鹰座", "Aquila"}, 3852: []string{"轩辕十五", "", "狮子座ο", "Subra", "狮子座", "Leo"}, 7188: []string{"鳖三", "", "南冕座ζ", "", "南冕座", "CoronaAustrilis"}, 7217: []string{"建二", "", "人马座ο", "", "人马座", "Sagittarius"}, 7235: []string{"天市左垣六", "", "天鹰座ζ", "", "天鹰座", "Aquila"}, 7236: []string{"天弁七", "", "天鹰座λ", "", "天鹰座", "Aquila"}, 7226: []string{"鳖七", "", "南冕座γ", "", "南冕座", "CoronaAustrilis"}, 7234: []string{"斗宿五", "", "人马座τ", "", "人马座", "Sagittarius"}, 7242: []string{"鳖四", "", "南冕座δ", "", "南冕座", "CoronaAustrilis"}, 7254: []string{"鳖六", "", "南冕座α", "", "南冕座", "CoronaAustrilis"}, 7264: []string{"建三", "", "人马座π", "", "人马座", "Sagittarius"}, 7259: []string{"鳖五", "", "南冕座β", "", "南冕座", "CoronaAustrilis"}, 7298: []string{"辇道二", "", "天琴座η", "", "天琴座", "Lyra"}, 7352: []string{"御女一", "", "天龙座τ", "", "天龙座", "Draco"}, 7314: []string{"辇道三", "", "天琴座θ", "", "天琴座", "Lyra"}, 7328: []string{"奚仲一", "", "天鹅座κ", "", "天鹅座", "Cygnus"}, 7304: []string{"建四", "", "人马座d", "", "人马座", "Sagittarius"}, 7371: []string{"天厨六", "", "天龙座π", "", "天龙座", "Draco"}, 7340: []string{"建五", "", "人马座ρ1", "", "人马座", "Sagittarius"}, 7342: []string{"建六", "", "人马座υ", "", "人马座", "Sagittarius"}, 3905: []string{"轩辕十", "", "狮子座μ", "Rasalas", "狮子座", "Leo"}, 7343: []string{"天渊一", "", "人马座β2", "", "人马座", "Sagittarius"}, 7377: []string{"右旗三", "", "天鹰座δ", "", "天鹰座", "Aquila"}, 7387: []string{"右旗四", "", "天鹰座ν", "", "天鹰座", "Aquila"}, 7405: []string{"齐增五", "", "狐狸座α", "", "狐狸座", "Vulpecula"}, 7420: []string{"奚仲二", "", "天鹅座ι2", "", "天鹅座", "Cygnus"}, 3982: []string{"轩辕十四", "", "狮子座α", "Regulus", "狮子座", "Leo"}, 7462: []string{"天厨二", "", "天龙座σ", "", "天龙座", "Draco"}, 7429: []string{"右旗一", "", "天鹰座μ", "", "天鹰座", "Aquila"}, 7469: []string{"奚仲三", "", "天鹅座θ", "", "天鹅座", "Cygnus"}, 7440: []string{"狗一", "", "人马座h2", "", "人马座", "Sagittarius"}, 7447: []string{"右旗五", "", "天鹰座ι", "", "天鹰座", "Aquila"}, 7446: []string{"右旗八", "", "天鹰座κ", "", "天鹰座", "Aquila"}, 7478: []string{"辇道增六", "", "天鹅座φ", "", "天鹅座", "Cygnus"}, 7488: []string{"左旗二", "", "天箭座β", "", "天箭座", "Sagitta"}, 7528: []string{"天津二", "", "天鹅座δ", "", "天鹅座", "Cygnus"}, 4031: []string{"轩辕十一", "", "狮子座ζ", "Adhafera", "狮子座", "Leo"}, 7515: []string{"天鸡二", "", "人马座f", "", "人马座", "Sagittarius"}, 7536: []string{"左旗三", "", "天箭座δ", "", "天箭座", "Sagitta"}, 7582: []string{"天厨三", "", "天龙座ε", "", "天龙座", "Draco"}, 7570: []string{"天桴四", "", "天鹰座η", "", "天鹰座", "Aquila"}, 4033: []string{"三台三", "", "大熊座λ", "TaniaBorealis", "大熊座", "UrsaMajor"}, 7597: []string{"狗国一", "", "人马座ω", "", "人马座", "Sagittarius"}, 7615: []string{"辇道增五", "", "天鹅座η", "", "天鹅座", "Cygnus"}, 7604: []string{"狗国四", "", "人马座b", "", "人马座", "Sagittarius"}, 7635: []string{"左旗五", "", "天箭座γ", "", "天箭座", "Sagitta"}, 7618: []string{"狗国二", "", "人马座60", "", "人马座", "Sagittarius"}, 7590: []string{"孔雀九", "", "孔雀座ε", "", "孔雀座", "Pavo"}, 7650: []string{"狗国三", "", "人马座c", "", "人马座", "Sagittarius"}, 7685: []string{"天厨四", "", "天龙座ρ", "", "天龙座", "Draco"}, 7665: []string{"孔雀六", "", "孔雀座δ", "", "孔雀座", "Pavo"}, 7710: []string{"天桴一", "", "天鹰座θ", "", "天鹰座", "Aquila"}, 7735: []string{"天津三", "", "天鹅座ο1", "", "天鹅座", "Cygnus"}, 7724: []string{"左旗九", "", "天鹰座ρ", "", "天鹰座", "Aquila"}, 7751: []string{"天津增三十七", "", "天鹅座ο2", "", "天鹅座", "Cygnus"}, 7754: []string{"牛宿二", "", "摩羯座α2", "", "摩羯座", "Capricornus"}, 7773: []string{"牛宿增七", "", "摩羯座ν", "", "摩羯座", "Capricornus"}, 4057: []string{"轩辕十二", "", "狮子座γ2", "Algieba", "狮子座", "Leo"}, 4069: []string{"三台四", "", "大熊座μ", "TaniaAustralis", "大熊座", "UrsaMajor"}, 7822: []string{"牛宿六", "", "摩羯座ρ", "", "摩羯座", "Capricornus"}, 7850: []string{"天钩三", "", "仙王座θ", "", "仙王座", "Cepheus"}, 7831: []string{"离珠四", "", "天鹰座69", "", "天鹰座", "Aquila"}, 7852: []string{"败瓜一", "", "海豚座ε", "", "海豚座", "Delphinus"}, 7871: []string{"瓠瓜五", "", "海豚座ζ", "", "海豚座", "Delphinus"}, 7873: []string{"离珠一", "", "天鹰座70", "", "天鹰座", "Aquila"}, 7869: []string{"波斯二", "", "印地安座α", "", "印地安座", "Indus"}, 7884: []string{"离珠二", "", "天鹰座l", "", "天鹰座", "Aquila"}, 4287: []string{"翼宿一", "", "巨爵座α", "Alkes", "巨爵座", "Crater"}, 7928: []string{"瓠瓜三", "", "海豚座δ", "", "海豚座", "Delphinus"}, 7913: []string{"孔雀七", "", "孔雀座β", "", "孔雀座", "Pavo"}, 7957: []string{"天钩四", "", "仙王座η", "", "仙王座", "Cepheus"}, 7936: []string{"天田四", "", "摩羯座ψ", "", "摩羯座", "Capricornus"}, 7948: []string{"瓠瓜二", "", "海豚座γ2", "", "海豚座", "Delphinus"}, 4295: []string{"北斗二", "天璇", "大熊座β", "Merak", "大熊座", "UrsaMajor"}, 7951: []string{"女宿四", "", "宝瓶座k", "", "宝瓶座", "Aquarius"}, 7980: []string{"天田二", "", "摩羯座ω", "", "摩羯座", "Capricornus"}, 7990: []string{"女宿二", "", "宝瓶座μ", "", "宝瓶座", "Aquarius"}, 8028: []string{"天津五", "", "天鹅座ν", "", "天鹅座", "Cygnus"}, 8047: []string{"车府五", "", "天鹅座V832", "", "天鹅座", "Cygnus"}, 8060: []string{"周一", "", "摩羯座η", "", "摩羯座", "Capricornus"}, 8079: []string{"车府六", "", "天鹅座ξ", "", "天鹅座", "Cygnus"}, 8075: []string{"秦一", "", "摩羯座θ", "", "摩羯座", "Capricornus"}, 8080: []string{"天田三", "", "摩羯座24", "", "摩羯座", "Capricornus"}, 8093: []string{"天垒城十", "", "宝瓶座ν", "", "宝瓶座", "Aquarius"}, 8097: []string{"司非一", "", "小马座γ", "", "小马座", "Equuleus"}, 8115: []string{"天津八", "", "天鹅座ζ", "", "天鹅座", "Cygnus"}, 8123: []string{"司非二", "", "小马座δ", "", "小马座", "Equuleus"}, 8130: []string{"天津六", "", "天鹅座τ", "", "天鹅座", "Cygnus"}, 8146: []string{"天津七", "", "天鹅座υ", "", "天鹅座", "Cygnus"}, 8135: []string{"离瑜二", "", "显微镜座ε", "", "显微镜座", "Microseopium"}, 8173: []string{"人二", "", "飞马座1", "", "飞马座", "Pegasus"}, 8167: []string{"代一", "", "摩羯座ι", "", "摩羯座", "Capricornus"}, 8181: []string{"孔雀十", "", "孔雀座γ", "", "孔雀座", "Pavo"}, 8204: []string{"燕", "", "摩羯座ζ", "", "摩羯座", "Capricornus"}, 4301: []string{"北斗一", "天枢", "大熊座α", "Dubhe", "大熊座", "UrsaMajor"}, 8213: []string{"晋", "", "摩羯座b", "", "摩羯座", "Capricornus"}, 8225: []string{"人一", "", "飞马座2", "", "飞马座", "Pegasus"}, 8252: []string{"车府四", "", "天鹅座ρ", "", "天鹅座", "Cygnus"}, 8260: []string{"垒壁阵二", "", "摩羯座ε", "", "摩羯座", "Capricornus"}, 8264: []string{"天垒城一", "", "宝瓶座ξ", "", "宝瓶座", "Aquarius"}, 4357: []string{"太微右垣五", "西上相", "狮子座δ", "Zosma", "狮子座", "Leo"}, 8254: []string{"蛇尾三", "", "南极座ν", "", "南极座", "Octans"}, 8288: []string{"垒壁阵一", "", "摩羯座κ", "", "摩羯座", "Capricornus"}, 4359: []string{"太微右垣四", "西次相", "狮子座θ", "Chertan", "狮子座", "Leo"}, 8309: []string{"臼一", "", "天鹅座μ2", "", "天鹅座", "Cygnus"}, 8313: []string{"人三", "", "飞马座9", "", "飞马座", "Pegasus"}, 8315: []string{"臼二", "", "飞马座κ", "", "飞马座", "Pegasus"}, 8305: []string{"天钱三", "", "南鱼座ι", "", "南鱼座", "PiscisAustrinus"}, 8334: []string{"造父五", "", "仙王座ν", "", "仙王座", "Cepheus"}, 8335: []string{"螣蛇三", "", "天鹅座π2", "", "天鹅座", "Cygnus"}, 4377: []string{"三台五", "下台一", "大熊座ν", "AlulaBorealis", "大熊座", "UrsaMajor"}, 8353: []string{"败臼一", "", "天鹤座γ", "", "天鹤座", "Grus"}, 8402: []string{"盖屋一", "", "宝瓶座ο", "", "宝瓶座", "Aquarius"}, 4434: []string{"紫微右垣三", "上辅", "天龙座λ", "Giausar", "天龙座", "Draco"}, 8411: []string{"败臼二", "", "天鹤座λ", "", "天鹤座", "Grus"}, 8418: []string{"垒壁阵五", "", "宝瓶座ι", "", "宝瓶座", "Aquarius"}, 8430: []string{"臼三", "", "飞马座ι", "", "飞马座", "Pegasus"}, 8431: []string{"天钱四", "", "南鱼座μ", "", "南鱼座", "PiscisAustrinus"}, 8454: []string{"杵二", "", "飞马座π2", "", "飞马座", "Pegasus"}, 8447: []string{"天钱五", "", "南鱼座τ", "", "南鱼座", "PiscisAustrinus"}, 4534: []string{"五帝座一", "", "狮子座β", "Denebola", "狮子座", "Leo"}, 8465: []string{"造父二", "", "仙王座ζ", "", "仙王座", "Cepheus"}, 8494: []string{"螣蛇九", "", "仙王座ε", "", "仙王座", "Cepheus"}, 8486: []string{"鹤十二", "", "天鹤座μ1", "", "天鹤座", "Grus"}, 8498: []string{"杵一", "", "蝎虎座1", "", "蝎虎座", "Lacerta"}, 8502: []string{"鸟喙一", "", "杜鹃座α", "", "杜鹃座", "Tucana"}, 8523: []string{"车府三", "", "蝎虎座2", "", "蝎虎座", "Lacerta"}, 8522: []string{"臼四", "", "飞马座32", "", "飞马座", "Pegasus"}, 8520: []string{"土公吏一", "", "飞马座31", "", "飞马座", "Pegasus"}, 4540: []string{"太微右垣一", "右执法", "室女座β", "Zavijava", "室女座", "Virgo"}, 8538: []string{"螣蛇十", "", "蝎虎座β", "", "蝎虎座", "Lacerta"}, 8541: []string{"螣蛇二", "", "蝎虎座4", "", "蝎虎座", "Lacerta"}, 8539: []string{"坟墓四", "", "宝瓶座π", "", "宝瓶座", "Aquarius"}, 8540: []string{"鸟喙二", "", "杜鹃座δ", "", "杜鹃座", "Tucana"}, 8558: []string{"坟墓一", "", "宝瓶座ζ2", "", "宝瓶座", "Aquarius"}, 8571: []string{"造父一", "", "仙王座δ", "", "仙王座", "Cepheus"}, 8556: []string{"鹤十一", "", "天鹤座δ1", "", "天鹤座", "Grus"}, 8573: []string{"垒壁阵六", "", "宝瓶座σ", "", "宝瓶座", "Aquarius"}, 8585: []string{"螣蛇一", "", "蝎虎座α", "", "蝎虎座", "Lacerta"}, 8597: []string{"坟墓三", "", "宝瓶座η", "", "宝瓶座", "Aquarius"}, 8613: []string{"螣蛇十五", "", "蝎虎座9", "", "蝎虎座", "Lacerta"}, 8632: []string{"车府二", "", "蝎虎座11", "", "蝎虎座", "Lacerta"}, 8628: []string{"羽林军八", "", "南鱼座ε", "", "南鱼座", "PiscisAustrinus"}, 8641: []string{"离宫三", "", "飞马座ο", "", "飞马座", "Pegasus"}, 8636: []string{"鹤二", "", "天鹤座β", "", "天鹤座", "Grus"}, 4554: []string{"北斗三", "天玑", "大熊座γ", "Phad", "大熊座", "UrsaMajor"}, 8649: []string{"羽林军十三", "", "宝瓶座g", "", "宝瓶座", "Aquarius"}, 8655: []string{"鹤四", "", "天鹤座η", "", "天鹤座", "Grus"}, 8667: []string{"离宫一", "", "飞马座λ", "", "飞马座", "Pegasus"}, 8665: []string{"雷电二", "", "飞马座ξ", "", "飞马座", "Pegasus"}, 8675: []string{"鹤三", "", "天鹤座ε", "", "天鹤座", "Grus"}, 8679: []string{"羽林军二十四", "", "宝瓶座τ2", "", "宝瓶座", "Aquarius"}, 8694: []string{"天钩八", "", "仙王座ι", "", "仙王座", "Cepheus"}, 8699: []string{"车府一", "", "蝎虎座15", "", "蝎虎座", "Lacerta"}, 8695: []string{"败臼三", "", "南鱼座γ", "", "南鱼座", "PiscisAustrinus"}, 8698: []string{"垒壁阵七", "", "宝瓶座λ", "", "宝瓶座", "Aquarius"}, 4623: []string{"右辖", "", "乌鸦座α", "Alchiba", "乌鸦座", "Corvus"}, 8720: []string{"天纲", "", "南鱼座δ", "", "南鱼座", "PiscisAustrinus"}, 8747: []string{"鹤六", "", "天鹤座ζ", "", "天鹤座", "Grus"}, 4660: []string{"北斗四", "天权", "大熊座δ", "Megrez", "大熊座", "UrsaMajor"}, 8773: []string{"霹雳一", "", "双鱼座β", "", "双鱼座", "Pisces"}, 8780: []string{"螣蛇十六", "", "仙女座3", "", "仙女座", "Andromeda"}, 8789: []string{"羽林军三十", "", "宝瓶座c1", "", "宝瓶座", "Aquarius"}, 8787: []string{"鹤八", "", "天鹤座θ", "", "天鹤座", "Grus"}, 8795: []string{"雷电四", "", "飞马座55", "", "飞马座", "Pegasus"}, 8819: []string{"紫微左垣七", "", "仙王座π", "", "仙王座", "Cepheus"}, 8812: []string{"羽林军二十八", "", "宝瓶座c2", "", "宝瓶座", "Aquarius"}, 8817: []string{"羽林军二十九", "", "宝瓶座89", "", "宝瓶座", "Aquarius"}, 8820: []string{"鹤七", "", "天鹤座ι", "", "天鹤座", "Grus"}, 8830: []string{"螣蛇十七", "", "仙女座7", "", "仙女座", "Andromeda"}, 8834: []string{"垒壁阵八", "", "宝瓶座φ", "", "宝瓶座", "Aquarius"}, 8841: []string{"羽林军三十九", "", "宝瓶座ψ1", "", "宝瓶座", "Aquarius"}, 8850: []string{"羽林军四十三", "", "宝瓶座χ", "", "宝瓶座", "Aquarius"}, 8852: []string{"霹雳二", "", "双鱼座γ", "", "双鱼座", "Pisces"}, 8848: []string{"鹤五", "", "杜鹃座γ", "", "杜鹃座", "Tucana"}, 8860: []string{"螣蛇十八", "", "仙女座8", "", "仙女座", "Andromeda"}, 8858: []string{"羽林军三十八", "", "宝瓶座ψ2", "", "宝瓶座", "Aquarius"}, 8872: []string{"天钩九", "", "仙王座ο", "", "仙王座", "Cepheus"}, 8865: []string{"羽林军三十七", "", "宝瓶座ψ3", "", "宝瓶座", "Aquarius"}, 8880: []string{"离宫五", "", "飞马座τ", "", "飞马座", "Pegasus"}, 8892: []string{"羽林军三十四", "", "宝瓶座b1", "", "宝瓶座", "Aquarius"}, 8905: []string{"离宫六", "", "飞马座υ", "", "飞马座", "Pegasus"}, 8906: []string{"羽林军三十三", "", "宝瓶座b2", "", "宝瓶座", "Aquarius"}, 8911: []string{"云雨一", "", "双鱼座κ", "", "双鱼座", "Pisces"}, 8916: []string{"霹雳三", "", "双鱼座θ", "", "双鱼座", "Pisces"}, 8923: []string{"雷电六", "", "飞马座70", "", "飞马座", "Pegasus"}, 8937: []string{"火鸟一", "", "玉夫座β", "", "玉夫座", "Sculptor"}, 8939: []string{"羽林军三十一", "", "宝瓶座b3", "", "宝瓶座", "Aquarius"}, 8949: []string{"火鸟二", "", "凤凰座ι", "", "凤凰座", "Phoenix"}, 8961: []string{"螣蛇十九", "", "仙女座λ", "", "仙女座", "Andromeda"}, 8965: []string{"螣蛇二十二", "", "仙女座ι", "", "仙女座", "Andromeda"}, 4662: []string{"轸宿一", "", "乌鸦座γ", "Gienah", "乌鸦座", "Corvus"}, 8968: []string{"羽林军四十四", "", "宝瓶座ω1", "", "宝瓶座", "Aquarius"}, 8969: []string{"霹雳四", "", "双鱼座ι", "", "双鱼座", "Pisces"}, 8976: []string{"螣蛇二十一", "", "仙女座κ", "", "仙女座", "Andromeda"}, 8982: []string{"鈇钺一", "", "宝瓶座104", "", "宝瓶座", "Aquarius"}, 8984: []string{"云雨四", "", "双鱼座λ", "", "双鱼座", "Pisces"}, 8988: []string{"羽林军四十五", "", "宝瓶座ω2", "", "宝瓶座", "Aquarius"}, 9003: []string{"螣蛇二十", "", "仙女座ψ", "", "仙女座", "Andromeda"}, 9008: []string{"螣蛇十三", "", "仙后座τ", "", "仙后座", "Cassiopeia"}, 9067: []string{"垒壁阵九", "", "双鱼座27", "", "双鱼座", "Pisces"}, 9071: []string{"螣蛇十一", "", "仙后座σ", "", "仙后座", "Cassiopeia"}, 9072: []string{"霹雳五", "", "双鱼座ω", "", "双鱼座", "Pisces"}, 9084: []string{"", "", "南极座θ", "", "南极座", "Octans"}, 45: []string{"", "", "飞马座χ", "", "飞马座", "Pegasus"}, 105: []string{"", "", "玉夫座η", "", "玉夫座", "Sculptor"}, 127: []string{"", "", "杜鹃座β2", "", "杜鹃座", "Tucana"}, 179: []string{"", "", "仙后座ξ", "", "仙后座", "Cassiopeia"}, 184: []string{"", "", "仙后座π", "", "仙后座", "Cassiopeia"}, 248: []string{"", "", "鲸鱼座20", "", "鲸鱼座", "Cetus"}, 253: []string{"", "", "仙后座υ1", "", "仙后座", "Cassiopeia"}, 265: []string{"", "", "仙后座υ2", "", "仙后座", "Cassiopeia"}, 280: []string{"", "", "玉夫座α", "", "玉夫座", "Sculptor"}, 370: []string{"", "", "凤凰座ν", "", "凤凰座", "Phoenix"}, 377: []string{"", "", "杜鹃座κ", "", "杜鹃座", "Tucana"}, 382: []string{"", "", "仙后座φ", "", "仙后座", "Cassiopeia"}, 412: []string{"", "", "鲸鱼座46", "", "鲸鱼座", "Cetus"}, 417: []string{"", "", "仙女座ω", "", "仙女座", "Andromeda"}, 442: []string{"", "", "仙后座χ", "", "仙后座", "Cassiopeia"}, 531: []string{"", "", "鲸鱼座χ", "", "鲸鱼座", "Cetus"}, 555: []string{"", "", "凤凰座ψ", "", "凤凰座", "Phoenix"}, 569: []string{"", "", "白羊座λ", "", "白羊座", "Aries"}, 612: []string{"", "", "天炉座ν", "", "天炉座", "Fornax"}, 620: []string{"", "", "仙女座58", "", "仙女座", "Andromeda"}, 623: []string{"", "", "白羊座14", "", "白羊座", "Aries"}, 642: []string{"", "", "三角座6", "", "三角座", "Triangulum"}, 643: []string{"", "", "仙女座b", "", "仙女座", "Andromeda"}, 699: []string{"", "", "仙女座65", "", "仙女座", "Andromeda"}, 740: []string{"", "", "鲸鱼座σ", "", "鲸鱼座", "Cetus"}, 749: []string{"", "", "天炉座ω", "", "天炉座", "Fornax"}, 794: []string{"", "", "波江座ι", "", "波江座", "Eridanus"}, 799: []string{"", "", "英仙座θ", "", "英仙座", "Perseus"}, 841: []string{"", "", "天炉座β", "", "天炉座", "Fornax"}, 843: []string{"", "", "英仙座17", "", "英仙座", "Perseus"}, 909: []string{"", "", "时钟座β", "", "时钟座", "Horologium"}, 882: []string{"", "", "英仙座24", "", "英仙座", "Perseus"}, 887: []string{"", "", "白羊座ε", "", "白羊座", "Aries"}, 918: []string{"", "", "英仙座k", "", "英仙座", "Perseus"}, 994: []string{"", "", "波江座15", "", "波江座", "Eridanus"}, 996: []string{"", "", "鲸鱼座κ1", "", "鲸鱼座", "Cetus"}, 1008: []string{"", "", "波江座e", "", "波江座", "Eridanus"}, 1002: []string{"", "", "英仙座l", "", "英仙座", "Perseus"}, 1044: []string{"", "", "英仙座34", "", "英仙座", "Perseus"}, 1083: []string{"", "", "网罟座κ", "", "网罟座", "Reticulum"}, 1052: []string{"", "", "英仙座σ", "", "英仙座", "Perseus"}, 1070: []string{"", "", "波江座v", "", "波江座", "Eridanus"}, 1101: []string{"", "", "金牛座10", "", "金牛座", "Taurus"}, 1106: []string{"", "", "波江座y", "", "波江座", "Eridanus"}, 1134: []string{"", "", "天炉座δ", "", "天炉座", "Fornax"}, 1211: []string{"", "", "波江座32", "", "波江座", "Eridanus"}, 1247: []string{"", "", "网罟座δ", "", "网罟座", "Reticulum"}, 1264: []string{"", "", "网罟座γ", "", "网罟座", "Reticulum"}, 1266: []string{"", "", "网罟座ι", "", "网罟座", "Reticulum"}, 1302: []string{"", "", "时钟座δ", "", "时钟座", "Horologium"}, 1311: []string{"", "", "金牛座47", "", "金牛座", "Taurus"}, 1326: []string{"", "", "时钟座α", "", "时钟座", "Horologium"}, 1306: []string{"", "", "英仙座f", "", "英仙座", "Perseus"}, 1355: []string{"", "", "网罟座ε", "", "网罟座", "Reticulum"}, 1343: []string{"", "", "英仙座54", "", "英仙座", "Perseus"}, 1350: []string{"", "", "英仙座d", "", "英仙座", "Perseus"}, 1380: []string{"", "", "金牛座δ2", "", "金牛座", "Taurus"}, 1392: []string{"", "", "金牛座υ", "", "金牛座", "Taurus"}, 1407: []string{"", "", "金牛座75", "", "金牛座", "Taurus"}, 1411: []string{"", "", "金牛座θ1", "", "金牛座", "Taurus"}, 1437: []string{"", "", "波江座45", "", "波江座", "Eridanus"}, 1454: []string{"", "", "英仙座58", "", "英仙座", "Perseus"}, 1502: []string{"", "", "雕具座α", "", "雕具座", "Caelum"}, 1533: []string{"", "", "御夫座1", "", "御夫座", "Auriga"}, 1551: []string{"", "", "御夫座2", "", "御夫座", "Auriga"}, 1592: []string{"", "", "御夫座4", "", "御夫座", "Auriga"}, 1603: []string{"", "", "鹿豹座β", "", "鹿豹座", "Camelopardalis"}, 1652: []string{"", "", "雕具座γ1", "", "雕具座", "Caelum"}, 1638: []string{"", "", "猎户座11", "", "猎户座", "Orion"}, 1674: []string{"", "", "剑鱼座ζ", "", "剑鱼座", "Dorado"}, 1676: []string{"", "", "猎户座15", "", "猎户座", "Orion"}, 1698: []string{"", "", "猎户座ρ", "", "猎户座", "Orion"}, 1743: []string{"", "", "天鸽座ο", "", "天鸽座", "Columba"}, 1726: []string{"", "", "御夫座16", "", "御夫座", "Auriga"}, 1765: []string{"", "", "猎户座o", "", "猎户座", "Orion"}, 1770: []string{"", "", "猎户座23", "", "猎户座", "Orion"}, 1784: []string{"", "", "猎户座e", "", "猎户座", "Orion"}, 1780: []string{"", "", "金牛座111", "", "金牛座", "Taurus"}, 1789: []string{"", "", "猎户座ψ1", "", "猎户座", "Orion"}, 1811: []string{"", "", "猎户座ψ2", "", "猎户座", "Orion"}, 1810: []string{"", "", "金牛座o", "", "金牛座", "Taurus"}, 1834: []string{"", "", "猎户座31", "", "猎户座", "Orion"}, 1839: []string{"", "", "猎户座32", "", "猎户座", "Orion"}, 1845: []string{"", "", "金牛座119", "", "金牛座", "Taurus"}, 1931: []string{"", "", "猎户座σ", "", "猎户座", "Orion"}, 1937: []string{"", "", "猎户座d", "", "猎户座", "Orion"}, 1934: []string{"", "", "猎户座ω", "", "猎户座", "Orion"}, 1946: []string{"", "", "金牛座126", "", "金牛座", "Taurus"}, 1963: []string{"", "", "猎户座b", "", "猎户座", "Orion"}, 1998: []string{"", "", "天兔座ζ", "", "天兔座", "Lepus"}, 2020: []string{"", "", "绘架座β", "", "绘架座", "Pictor"}, 2002: []string{"", "", "金牛座132", "", "金牛座", "Taurus"}, 2010: []string{"", "", "金牛座134", "", "金牛座", "Taurus"}, 2042: []string{"", "", "绘架座γ", "", "绘架座", "Pictor"}, 2037: []string{"", "", "猎户座56", "", "猎户座", "Orion"}, 2087: []string{"", "", "天鸽座ξ", "", "天鸽座", "Columba"}, 2085: []string{"", "", "天兔座η", "", "天兔座", "Lepus"}, 2106: []string{"", "", "天鸽座γ", "", "天鸽座", "Columba"}, 2120: []string{"", "", "天鸽座η", "", "天鸽座", "Columba"}, 2091: []string{"", "", "御夫座π", "", "御夫座", "Auriga"}, 2128: []string{"", "", "麒麟座3", "", "麒麟座", "Monoceros"}, 2124: []string{"", "", "猎户座μ", "", "猎户座", "Orion"}, 2148: []string{"", "", "天兔座17", "", "天兔座", "Lepus"}, 2155: []string{"", "", "天兔座θ", "", "天兔座", "Lepus"}, 2212: []string{"", "", "绘架座δ", "", "绘架座", "Pictor"}, 2227: []string{"", "", "麒麟座γ", "", "麒麟座", "Monoceros"}, 2219: []string{"", "", "御夫座κ", "", "御夫座", "Auriga"}, 2215: []string{"", "", "天猫座1", "", "天猫座", "Lynx"}, 2238: []string{"", "", "天猫座2", "", "天猫座", "Lynx"}, 2296: []string{"", "", "天鸽座δ", "", "天鸽座", "Columba"}, 2356: []string{"", "", "麒麟座β", "", "麒麟座", "Monoceros"}, 2414: []string{"", "", "大犬座ξ2", "", "大犬座", "CanisMajor"}, 2456: []string{"", "", "麒麟座15", "", "麒麟座", "Monoceros"}, 2478: []string{"", "", "双子座30", "", "双子座", "Gemini"}, 2470: []string{"", "", "天猫座12", "", "天猫座", "Lynx"}, 2550: []string{"", "", "绘架座α", "", "绘架座", "Pictor"}, 2571: []string{"", "", "大犬座15", "", "大犬座", "CanisMajor"}, 2564: []string{"", "", "双子座e", "", "双子座", "Gemini"}, 2593: []string{"", "", "大犬座μ", "", "大犬座", "CanisMajor"}, 2596: []string{"", "", "大犬座ι", "", "大犬座", "CanisMajor"}, 2560: []string{"", "", "天猫座15", "", "天猫座", "Lynx"}, 2648: []string{"", "", "麒麟座19", "", "麒麟座", "Monoceros"}, 2672: []string{"", "", "船尾座H", "", "船尾座", "Puppis"}, 2702: []string{"", "", "船尾座A", "", "船尾座", "Puppis"}, 2701: []string{"", "", "麒麟座20", "", "麒麟座", "Monoceros"}, 2696: []string{"", "", "御夫座63", "", "御夫座", "Auriga"}, 2740: []string{"", "", "船尾座I", "", "船尾座", "Puppis"}, 2746: []string{"", "", "船尾座L1", "", "船尾座", "Puppis"}, 2745: []string{"", "", "大犬座27", "", "大犬座", "CanisMajor"}, 2764: []string{"", "", "大犬座145", "", "大犬座", "CanisMajor"}, 2782: []string{"", "", "大犬座τ", "", "大犬座", "CanisMajor"}, 2818: []string{"", "", "天猫座21", "", "天猫座", "Lynx"}, 2854: []string{"", "", "小犬座γ", "", "小犬座", "CanisMinor"}, 2937: []string{"", "", "船尾座F", "", "船尾座", "Puppis"}, 2944: []string{"", "", "船尾座m", "", "船尾座", "Puppis"}, 2930: []string{"", "", "双子座ο", "", "双子座", "Gemini"}, 2961: []string{"", "", "船尾座n1", "", "船尾座", "Puppis"}, 2970: []string{"", "", "麒麟座α", "", "麒麟座", "Monoceros"}, 2973: []string{"", "", "双子座σ", "", "双子座", "Gemini"}, 2993: []string{"", "", "船尾座1", "", "船尾座", "Puppis"}, 2996: []string{"", "", "船尾座l", "", "船尾座", "Puppis"}, 3003: []string{"", "", "双子座g", "", "双子座", "Gemini"}, 3046: []string{"", "", "船尾座Q", "", "船尾座", "Puppis"}, 3055: []string{"", "", "船尾座P", "", "船尾座", "Puppis"}, 3080: []string{"", "", "船尾座A", "", "船尾座", "Puppis"}, 3084: []string{"", "", "船尾座b", "", "船尾座", "Puppis"}, 3090: []string{"", "", "船尾座J", "", "船尾座", "Puppis"}, 3117: []string{"", "", "船底座χ", "", "船底座", "Carina"}, 3102: []string{"", "", "船尾座J", "", "船尾座", "Puppis"}, 3122: []string{"", "", "麒麟座27", "", "麒麟座", "Monoceros"}, 3141: []string{"", "", "麒麟座28", "", "麒麟座", "Monoceros"}, 3149: []string{"", "", "双子座χ", "", "双子座", "Gemini"}, 3223: []string{"", "", "飞鱼座ε", "", "飞鱼座", "Volans"}, 3173: []string{"", "", "天猫座27", "", "天猫座", "Lynx"}, 3188: []string{"", "", "麒麟座ζ", "", "麒麟座", "Monoceros"}, 3192: []string{"", "", "船尾座16", "", "船尾座", "Puppis"}, 3211: []string{"", "", "船尾座19", "", "船尾座", "Puppis"}, 3225: []string{"", "", "船尾座h1", "", "船尾座", "Puppis"}, 3229: []string{"", "", "船尾座20", "", "船尾座", "Puppis"}, 3237: []string{"", "", "船尾座r", "", "船尾座", "Puppis"}, 3243: []string{"", "", "船尾座h2", "", "船尾座", "Puppis"}, 3318: []string{"", "", "蝘蜓座α", "", "蝘蜓座", "Chamaeleon"}, 3270: []string{"", "", "船尾座Q", "", "船尾座", "Puppis"}, 3282: []string{"", "", "船尾座w", "", "船尾座", "Puppis"}, 3294: []string{"", "", "船帆座B", "", "船帆座", "Vela"}, 3414: []string{"", "", "船底座e2", "", "船底座", "Carina"}, 3433: []string{"", "", "罗盘座ζ", "", "罗盘座", "Pyxis"}, 3431: []string{"", "", "长蛇座a", "", "长蛇座", "Hydra"}, 3447: []string{"", "", "船帆座ο", "", "船帆座", "Vela"}, 3457: []string{"", "", "船底座d", "", "船底座", "Carina"}, 3452: []string{"", "", "船帆座n", "", "船帆座", "Vela"}, 3441: []string{"", "", "长蛇座9", "", "长蛇座", "Hydra"}, 3487: []string{"", "", "船帆座a", "", "船帆座", "Vela"}, 3484: []string{"", "", "长蛇座D", "", "长蛇座", "Hydra"}, 3475: []string{"", "", "巨蟹座ιA", "", "巨蟹座", "Cancer"}, 3498: []string{"", "", "船底座f", "", "船底座", "Carina"}, 3520: []string{"", "", "船帆座g", "", "船帆座", "Vela"}, 3571: []string{"", "", "船底座c", "", "船底座", "Carina"}, 3574: []string{"", "", "船帆座H", "", "船帆座", "Vela"}, 3582: []string{"", "", "船底座b1", "", "船底座", "Carina"}, 3591: []string{"", "", "船帆座w", "", "船帆座", "Vela"}, 3614: []string{"", "", "船帆座C", "", "船帆座", "Vela"}, 3628: []string{"", "", "罗盘座κ", "", "罗盘座", "Pyxis"}, 3616: []string{"", "", "大熊座σ2", "", "大熊座", "UrsaMajor"}, 3624: []string{"", "", "大熊座τ", "", "大熊座", "UrsaMajor"}, 3659: []string{"", "", "船底座a", "", "船底座", "Carina"}, 3663: []string{"", "", "船底座i", "", "船底座", "Carina"}, 3682: []string{"", "", "船帆座l", "", "船帆座", "Vela"}, 3684: []string{"", "", "船帆座k", "", "船帆座", "Vela"}, 3696: []string{"", "", "船底座g", "", "船底座", "Carina"}, 3728: []string{"", "", "船底座k", "", "船底座", "Carina"}, 3718: []string{"", "", "罗盘座θ", "", "罗盘座", "Pyxis"}, 3733: []string{"", "", "罗盘座λ", "", "罗盘座", "Pyxis"}, 3749: []string{"", "", "长蛇座G", "", "长蛇座", "Hydra"}, 3765: []string{"", "", "唧筒座ε", "", "唧筒座", "Antlia"}, 3786: []string{"", "", "船帆座ψ", "", "船帆座", "Vela"}, 3800: []string{"", "", "小狮座10", "", "小狮座", "LeoMinor"}, 3799: []string{"", "", "大熊座26", "", "大熊座", "UrsaMajor"}, 3836: []string{"", "", "船帆座M", "", "船帆座", "Vela"}, 3834: []string{"", "", "六分仪座2", "", "六分仪座", "Sextans"}, 3856: []string{"", "", "船底座m", "", "船底座", "Carina"}, 3858: []string{"", "", "长蛇座I", "", "长蛇座", "Hydra"}, 3871: []string{"", "", "唧筒座θ", "", "唧筒座", "Antlia"}, 3912: []string{"", "", "船帆座M", "", "船帆座", "Vela"}, 3940: []string{"", "", "船帆座φ", "", "船帆座", "Vela"}, 3950: []string{"", "", "狮子座π", "", "狮子座", "Leo"}, 3970: []string{"", "", "长蛇座υ2", "", "长蛇座", "Hydra"}, 3981: []string{"", "", "六分仪座α", "", "六分仪座", "Sextans"}, 3990: []string{"", "", "船帆座Q", "", "船帆座", "Vela"}, 4023: []string{"", "", "船帆座Q", "", "船帆座", "Vela"}, 4054: []string{"", "", "狮子座40", "", "狮子座", "Leo"}, 4074: []string{"", "", "船帆座J", "", "船帆座", "Vela"}, 4080: []string{"", "", "船帆座r", "", "船帆座", "Vela"}, 4090: []string{"", "", "小狮座30", "", "小狮座", "LeoMinor"}, 4104: []string{"", "", "唧筒座α", "", "唧筒座", "Antlia"}, 4100: []string{"", "", "小狮座β", "", "小狮座", "LeoMinor"}, 4112: []string{"", "", "大熊座36", "", "大熊座", "UrsaMajor"}, 4159: []string{"", "", "船底座r", "", "船底座", "Carina"}, 4167: []string{"", "", "船帆座p", "", "船帆座", "Vela"}, 4166: []string{"", "", "小狮座37", "", "小狮座", "LeoMinor"}, 4180: []string{"", "", "船帆座x", "", "船帆座", "Vela"}, 4200: []string{"", "", "船底座w", "", "船底座", "Carina"}, 4257: []string{"", "", "船底座u", "", "船底座", "Carina"}, 4273: []string{"", "", "唧筒座ι", "", "唧筒座", "Antlia"}, 4293: []string{"", "", "船帆座I", "", "船帆座", "Vela"}, 4299: []string{"", "", "狮子座p2", "", "狮子座", "Leo"}, 4300: []string{"", "", "狮子座b", "", "狮子座", "Leo"}, 4325: []string{"", "", "船底座z", "", "船底座", "Carina"}, 4337: []string{"", "", "船底座x", "", "船底座", "Carina"}, 4368: []string{"", "", "狮子座φ", "", "狮子座", "Leo"}, 4380: []string{"", "", "大熊座55", "", "大熊座", "UrsaMajor"}, 4390: []string{"", "", "半人马座π", "", "半人马座", "Centaurus"}, 4460: []string{"", "", "半人马座A", "", "半人马座", "Centaurus"}, 4530: []string{"", "", "苍蝇座μ", "", "苍蝇座", "Musca"}, 4537: []string{"", "", "半人马座j", "", "半人马座", "Centaurus"}, 4546: []string{"", "", "半人马座B", "", "半人马座", "Centaurus"}, 4599: []string{"", "", "南十字座θ1", "", "南十字座", "Crux"}, 4603: []string{"", "", "南十字座θ2", "", "南十字座", "Crux"}, 4616: []string{"", "", "南十字座η", "", "南十字座", "Crux"}, 4671: []string{"", "", "苍蝇座ε", "", "苍蝇座", "Musca"}, 4679: []string{"", "", "南十字座ζ", "", "南十字座", "Crux"}, 4682: []string{"", "", "半人马座F", "", "半人马座", "Centaurus"}, 4697: []string{"", "", "后发座11", "", "后发座", "ComaBerenices"}, 4700: []string{"", "", "南十字座ε", "", "南十字座", "Crux"}, 4765: []string{"", "", "天龙座4", "", "天龙座", "Draco"}, 4773: []string{"", "", "苍蝇座γ", "", "苍蝇座", "Musca"}, 4795: []string{"", "", "天龙座6", "", "天龙座", "Draco"}, 4813: []string{"", "", "室女座χ", "", "室女座", "Virgo"}, 4817: []string{"", "", "半人马座l", "", "半人马座", "Centaurus"}, 4831: []string{"", "", "半人马座w", "", "半人马座", "Centaurus"}, 4842: []string{"", "", "南十字座ι", "", "南十字座", "Crux"}, 4874: []string{"", "", "半人马座p", "", "半人马座", "Centaurus"}, 4888: []string{"", "", "半人马座E", "", "半人马座", "Centaurus"}, 4894: []string{"", "", "后发座35", "", "后发座", "ComaBerenices"}, 4889: []string{"", "", "半人马座n", "", "半人马座", "Centaurus"}, 4902: []string{"", "", "室女座ψ", "", "室女座", "Virgo"}, 4898: []string{"", "", "南十字座μ1", "", "南十字座", "Crux"}, 4897: []string{"", "", "南十字座λ", "", "南十字座", "Crux"}, 4931: []string{"", "", "大熊座78", "", "大熊座", "UrsaMajor"}, 4923: []string{"", "", "苍蝇座δ", "", "苍蝇座", "Musca"}, 4933: []string{"", "", "半人马座ξ1", "", "半人马座", "Centaurus"}, 4942: []string{"", "", "半人马座ξ2", "", "半人马座", "Centaurus"}, 4958: []string{"", "", "长蛇座ψ", "", "长蛇座", "Hydra"}, 4993: []string{"", "", "苍蝇座η", "", "苍蝇座", "Musca"}, 5017: []string{"", "", "猎犬座20", "", "猎犬座", "CanesVenatici"}, 5015: []string{"", "", "室女座σ", "", "室女座", "Virgo"}, 5019: []string{"", "", "室女座61", "", "室女座", "Virgo"}, 5035: []string{"", "", "半人马座j", "", "半人马座", "Centaurus"}, 5041: []string{"", "", "半人马座m", "", "半人马座", "Centaurus"}, 5072: []string{"", "", "室女座70", "", "室女座", "Virgo"}, 5095: []string{"", "", "室女座l", "", "室女座", "Virgo"}, 5127: []string{"", "", "猎犬座25", "", "猎犬座", "CanesVenatici"}, 5154: []string{"", "", "大熊座83", "", "大熊座", "UrsaMajor"}, 5141: []string{"", "", "半人马座Q", "", "半人马座", "Centaurus"}, 5172: []string{"", "", "半人马座m", "", "半人马座", "Centaurus"}, 5201: []string{"", "", "牧夫座e", "", "牧夫座", "Bootes"}, 5196: []string{"", "", "室女座89", "", "室女座", "Virgo"}, 5329: []string{"", "", "牧夫座κ2", "", "牧夫座", "Bootes"}, 5361: []string{"", "", "牧夫座A", "", "牧夫座", "Bootes"}, 5358: []string{"", "", "半人马座v", "", "半人马座", "Centaurus"}, 5381: []string{"", "", "长蛇座k", "", "长蛇座", "Hydra"}, 5395: []string{"", "", "豺狼座τ1", "", "豺狼座", "Lupus"}, 5339: []string{"", "", "南极座δ", "", "南极座", "Octans"}, 5407: []string{"", "", "长蛇座l", "", "长蛇座", "Hydra"}, 5460: []string{"", "", "半人马座α2", "", "半人马座", "Centaurus"}, 5480: []string{"", "", "牧夫座31", "", "牧夫座", "Bootes"}, 5463: []string{"", "", "圆规座α", "", "圆规座", "Circinus"}, 5489: []string{"", "", "半人马座C2", "", "半人马座", "Centaurus"}, 5511: []string{"", "", "室女座19", "", "室女座", "Virgo"}, 4689: []string{"太微左垣一", "左执法", "室女座η", "Zaniah", "室女座", "Virgo"}, 5535: []string{"", "", "天秤座11", "", "天秤座", "Libra"}, 5544: []string{"", "", "牧夫座ξ", "", "牧夫座", "Bootes"}, 5570: []string{"", "", "天秤座16", "", "天秤座", "Libra"}, 5600: []string{"", "", "牧夫座ω", "", "牧夫座", "Bootes"}, 5601: []string{"", "", "室女座110", "", "室女座", "Virgo"}, 5618: []string{"", "", "牧夫座i", "", "牧夫座", "Bootes"}, 5616: []string{"", "", "牧夫座ψ", "", "牧夫座", "Bootes"}, 5634: []string{"", "", "牧夫座c", "", "牧夫座", "Bootes"}, 5651: []string{"", "", "豺狼座e", "", "豺狼座", "Lupus"}, 5670: []string{"", "", "圆规座β", "", "圆规座", "Circinus"}, 5666: []string{"", "", "圆规座ε", "", "圆规座", "Circinus"}, 5698: []string{"", "", "豺狼座ν1", "", "豺狼座", "Lupus"}, 5712: []string{"", "", "豺狼座φ2", "", "豺狼座", "Lupus"}, 5727: []string{"", "", "北冕座η", "", "北冕座", "CoronaBorealis"}, 5704: []string{"", "", "圆规座γ", "", "圆规座", "Circinus"}, 5723: []string{"", "", "天秤座ε", "", "天秤座", "Libra"}, 5724: []string{"", "", "豺狼座k", "", "豺狼座", "Lupus"}, 5826: []string{"", "", "小熊座θ", "", "小熊座", "UrsaMinor"}, 5777: []string{"", "", "天秤座37", "", "天秤座", "Libra"}, 5781: []string{"", "", "豺狼座d", "", "豺狼座", "Lupus"}, 5771: []string{"", "", "南三角座ε", "", "南三角座", "TriangulumAustrale"}, 5797: []string{"", "", "豺狼座ω", "", "豺狼座", "Lupus"}, 5833: []string{"", "", "北冕座ζ2", "", "北冕座", "CoronaBorealis"}, 5820: []string{"", "", "豺狼座ψ1", "", "豺狼座", "Lupus"}, 5824: []string{"", "", "天秤座42", "", "天秤座", "Libra"}, 5825: []string{"", "", "豺狼座g", "", "豺狼座", "Lupus"}, 5842: []string{"", "", "巨蛇座ι", "", "巨蛇座", "Serpens"}, 5868: []string{"", "", "巨蛇座λ", "", "巨蛇座", "Serpens"}, 5879: []string{"", "", "巨蛇座κ", "", "巨蛇座", "Serpens"}, 5885: []string{"", "", "天蝎座b", "", "天蝎座", "Scorpius"}, 5901: []string{"", "", "北冕座κ", "", "北冕座", "CoronaBorealis"}, 5899: []string{"", "", "巨蛇座ρ", "", "巨蛇座", "Serpens"}, 5904: []string{"", "", "天蝎座2", "", "天蝎座", "Scorpius"}, 5972: []string{"", "", "巨蛇座π", "", "巨蛇座", "Serpens"}, 5982: []string{"", "", "武仙座υ", "", "武仙座", "Hercules"}, 5962: []string{"", "", "矩尺座η", "", "矩尺座", "Norma"}, 5961: []string{"", "", "矩尺座ι1", "", "矩尺座", "Norma"}, 5985: []string{"", "", "天蝎座β2", "", "天蝎座", "Scorpius"}, 5980: []string{"", "", "矩尺座δ", "", "矩尺座", "Norma"}, 6018: []string{"", "", "北冕座τ", "", "北冕座", "CoronaBorealis"}, 6031: []string{"", "", "天蝎座ψ", "", "天蝎座", "Scorpius"}, 6028: []string{"", "", "天蝎座c2", "", "天蝎座", "Scorpius"}, 6024: []string{"", "", "矩尺座κ", "", "矩尺座", "Norma"}, 6030: []string{"", "", "南三角座δ", "", "南三角座", "TriangulumAustrale"}, 6058: []string{"", "", "矩尺座γ1", "", "矩尺座", "Norma"}, 6070: []string{"", "", "天蝎座d", "", "天蝎座", "Scorpius"}, 6072: []string{"", "", "矩尺座γ2", "", "矩尺座", "Norma"}, 6081: []string{"", "", "天蝎座ο", "", "天蝎座", "Scorpius"}, 6112: []string{"", "", "蛇夫座ρ", "", "蛇夫座", "Ophiuchus"}, 6115: []string{"", "", "矩尺座ε", "", "矩尺座", "Norma"}, 6098: []string{"", "", "南三角座ζ", "", "南三角座", "TriangulumAustrale"}, 6141: []string{"", "", "天蝎座i", "", "天蝎座", "Scorpius"}, 6155: []string{"", "", "矩尺座μ", "", "矩尺座", "Norma"}, 6168: []string{"", "", "武仙座σ", "", "武仙座", "Hercules"}, 6220: []string{"", "", "武仙座η", "", "武仙座", "Hercules"}, 6254: []string{"", "", "武仙座52", "", "武仙座", "Hercules"}, 6252: []string{"", "", "天蝎座μ2", "", "天蝎座", "Scorpius"}, 6271: []string{"", "", "天蝎座ζ2", "", "天蝎座", "Scorpius"}, 6318: []string{"", "", "蛇夫座30", "", "蛇夫座", "Ophiuchus"}, 6334: []string{"", "", "天蝎座k", "", "天蝎座", "Scorpius"}, 6415: []string{"", "", "蛇夫座41", "", "蛇夫座", "Ophiuchus"}, 6445: []string{"", "", "蛇夫座ξ", "", "蛇夫座", "Ophiuchus"}, 6498: []string{"", "", "蛇夫座σ", "", "蛇夫座", "Ophiuchus"}, 6555: []string{"", "", "天龙座ν2", "", "天龙座", "Draco"}, 6596: []string{"", "", "天龙座ω", "", "天龙座", "Draco"}, 6569: []string{"", "", "天坛座λ", "", "天坛座", "Ara"}, 6631: []string{"", "", "天蝎座ι2", "", "天蝎座", "Scorpius"}, 6703: []string{"", "", "武仙座ξ", "", "武仙座", "Hercules"}, 6707: []string{"", "", "武仙座ν", "", "武仙座", "Hercules"}, 6700: []string{"", "", "人马座4", "", "人马座", "Sagittarius"}, 6713: []string{"", "", "武仙座93", "", "武仙座", "Hercules"}, 6710: []string{"", "", "巨蛇座ζ", "", "巨蛇座", "Serpens"}, 6743: []string{"", "", "天坛座θ", "", "天坛座", "Ara"}, 6770: []string{"", "", "蛇夫座71", "", "蛇夫座", "Ophiuchus"}, 6771: []string{"", "", "蛇夫座72", "", "蛇夫座", "Ophiuchus"}, 6783: []string{"", "", "望远镜座ε", "", "望远镜座", "Telescopium"}, 6801: []string{"", "", "人马座11", "", "人马座", "Sagittarius"}, 6872: []string{"", "", "天琴座κ", "", "天琴座", "Lyra"}, 6868: []string{"", "", "武仙座106", "", "武仙座", "Hercules"}, 6866: []string{"", "", "蛇夫座74", "", "蛇夫座", "Ophiuchus"}, 6855: []string{"", "", "孔雀座ξ", "", "孔雀座", "Pavo"}, 6884: []string{"", "", "盾牌座ζ", "", "盾牌座", "Scutum"}, 6896: []string{"", "", "人马座21", "", "人马座", "Sagittarius"}, 6945: []string{"", "", "天龙座42", "", "天龙座", "Draco"}, 6905: []string{"", "", "望远镜座ζ", "", "望远镜座", "Telescopium"}, 6930: []string{"", "", "盾牌座γ", "", "盾牌座", "Scutum"}, 6934: []string{"", "", "望远镜座δ1", "", "望远镜座", "Telescopium"}, 7053: []string{"", "", "天琴座ε2", "", "天琴座", "Lyra"}, 7116: []string{"", "", "人马座ν1", "", "人马座", "Sagittarius"}, 7133: []string{"", "", "武仙座113", "", "武仙座", "Hercules"}, 7120: []string{"", "", "人马座ν2", "", "人马座", "Sagittarius"}, 7142: []string{"", "", "巨蛇座θ2", "", "巨蛇座", "Serpens"}, 7134: []string{"", "", "望远镜座λ", "", "望远镜座", "Telescopium"}, 7192: []string{"", "", "天琴座λ", "", "天琴座", "Lyra"}, 7309: []string{"", "", "天龙座54", "", "天龙座", "Draco"}, 7292: []string{"", "", "人马座ψ", "", "人马座", "Sagittarius"}, 7306: []string{"", "", "狐狸座1", "", "狐狸座", "Vulpecula"}, 7372: []string{"", "", "天鹅座2", "", "天鹅座", "Cygnus"}, 7426: []string{"", "", "天鹅座8", "", "天鹅座", "Cygnus"}, 7437: []string{"", "", "狐狸座9", "", "狐狸座", "Vulpecula"}, 7424: []string{"", "", "望远镜座ι", "", "望远镜座", "Telescopium"}, 7517: []string{"", "", "天鹅座15", "", "天鹅座", "Cygnus"}, 7565: []string{"", "", "狐狸座12", "", "狐狸座", "Vulpecula"}, 7592: []string{"", "", "狐狸座13", "", "狐狸座", "Vulpecula"}, 7595: []string{"", "", "天鹰座ξ", "", "天鹰座", "Aquila"}, 7581: []string{"", "", "人马座ι", "", "人马座", "Sagittarius"}, 7619: []string{"", "", "天鹅座ψ", "", "天鹅座", "Cygnus"}, 7613: []string{"", "", "天鹅座22", "", "天鹅座", "Cygnus"}, 7623: []string{"", "", "人马座θ1", "", "人马座", "Sagittarius"}, 7653: []string{"", "", "狐狸座15", "", "狐狸座", "Vulpecula"}, 7673: []string{"", "", "望远镜座ξ", "", "望远镜座", "Telescopium"}, 7750: []string{"", "", "仙王座κ", "", "仙王座", "Cepheus"}, 7708: []string{"", "", "天鹅座b2", "", "天鹅座", "Cygnus"}, 7730: []string{"", "", "天鹅座30", "", "天鹅座", "Cygnus"}, 7740: []string{"", "", "天鹅座33", "", "天鹅座", "Cygnus"}, 7736: []string{"", "", "天鹅座b3", "", "天鹅座", "Cygnus"}, 7744: []string{"", "", "狐狸座23", "", "狐狸座", "Vulpecula"}, 7806: []string{"", "", "天鹅座39", "", "天鹅座", "Cygnus"}, 7834: []string{"", "", "天鹅座41", "", "天鹅座", "Cygnus"}, 7844: []string{"", "", "天鹅座ω1", "", "天鹅座", "Cygnus"}, 7866: []string{"", "", "天鹅座47", "", "天鹅座", "Cygnus"}, 7848: []string{"", "", "孔雀座φ1", "", "孔雀座", "Pavo"}, 7859: []string{"", "", "孔雀座ρ", "", "孔雀座", "Pavo"}, 7891: []string{"", "", "狐狸座29", "", "狐狸座", "Vulpecula"}, 7920: []string{"", "", "印地安座η", "", "印地安座", "Indus"}, 7939: []string{"", "", "狐狸座30", "", "狐狸座", "Vulpecula"}, 7942: []string{"", "", "天鹅座52", "", "天鹅座", "Cygnus"}, 7963: []string{"", "", "天鹅座λ", "", "天鹅座", "Cygnus"}, 7977: []string{"", "", "天鹅座55", "", "天鹅座", "Cygnus"}, 7952: []string{"", "", "印地安座ζ", "", "印地安座", "Indus"}, 7965: []string{"", "", "显微镜座α", "", "显微镜座", "Microseopium"}, 7995: []string{"", "", "狐狸座31", "", "狐狸座", "Vulpecula"}, 8001: []string{"", "", "天鹅座57", "", "天鹅座", "Cygnus"}, 7986: []string{"", "", "印地安座β", "", "印地安座", "Indus"}, 8039: []string{"", "", "显微镜座γ", "", "显微镜座", "Microseopium"}, 8089: []string{"", "", "天鹅座f2", "", "天鹅座", "Cygnus"}, 8143: []string{"", "", "天鹅座σ", "", "天鹅座", "Cygnus"}, 8140: []string{"", "", "印地安座θ", "", "印地安座", "Indus"}, 8151: []string{"", "", "显微镜座θ1", "", "显微镜座", "Microseopium"}, 8255: []string{"", "", "天鹅座72", "", "天鹅座", "Cygnus"}, 8279: []string{"", "", "仙王座9", "", "仙王座", "Cepheus"}, 8317: []string{"", "", "仙王座11", "", "仙王座", "Cepheus"}, 8383: []string{"", "", "仙王座VV", "", "仙王座", "Cepheus"}, 8368: []string{"", "", "印地安座δ", "", "印地安座", "Indus"}, 8387: []string{"", "", "印地安座ε", "", "印地安座", "Indus"}, 8413: []string{"", "", "飞马座ν", "", "飞马座", "Pegasus"}, 8433: []string{"", "", "南鱼座υ", "", "南鱼座", "PiscisAustrinus"}, 8468: []string{"", "", "仙王座24", "", "仙王座", "Cepheus"}, 8551: []string{"", "", "飞马座35", "", "飞马座", "Pegasus"}, 8572: []string{"", "", "蝎虎座5", "", "蝎虎座", "Lacerta"}, 8560: []string{"", "", "天鹤座δ2", "", "天鹤座", "Grus"}, 8579: []string{"", "", "蝎虎座6", "", "蝎虎座", "Lacerta"}, 8576: []string{"", "", "南鱼座β", "", "南鱼座", "PiscisAustrinus"}, 8582: []string{"", "", "杜鹃座ν", "", "杜鹃座", "Tucana"}, 8622: []string{"", "", "蝎虎座10", "", "蝎虎座", "Lacerta"}, 8644: []string{"", "", "天鹤座ρ", "", "天鹤座", "Grus"}, 8630: []string{"", "", "南极座β", "", "南极座", "Octans"}, 8717: []string{"", "", "飞马座ρ", "", "飞马座", "Pegasus"}, 8762: []string{"", "", "仙女座ο", "", "仙女座", "Andromeda"}, 8797: []string{"", "", "仙后座1", "", "仙后座", "Cassiopeia"}, 8796: []string{"", "", "飞马座56", "", "飞马座", "Pegasus"}, 8863: []string{"", "", "玉夫座γ", "", "玉夫座", "Sculptor"}, 8904: []string{"", "", "仙后座4", "", "仙后座", "Cassiopeia"}, 8943: []string{"", "", "飞马座72", "", "飞马座", "Pegasus"}, 8997: []string{"", "", "飞马座78", "", "飞马座", "Pegasus"}, 9004: []string{"", "", "双鱼座19", "", "双鱼座", "Pisces"}, 9016: []string{"", "", "玉夫座δ", "", "玉夫座", "Sculptor"}, 9062: []string{"", "", "杜鹃座η", "", "杜鹃座", "Tucana"}, 9064: []string{"", "", "飞马座ψ", "", "飞马座", "Pegasus"}, 4730: []string{"十字架二", "", "南十字座α2", "Acrux", "南十字座", "Crux"}, 4757: []string{"轸宿三", "", "乌鸦座δ", "Algorab", "乌鸦座", "Corvus"}, 4763: []string{"十字架一", "", "南十字座γ", "Gacrux", "南十字座", "Crux"}, 4785: []string{"常陈四", "", "猎犬座β", "Chara", "猎犬座", "CanesVenatici"}, 4825: []string{"太微左垣二", "东上相", "室女座γ", "Porrima", "室女座", "Virgo"}, 4853: []string{"十字架三", "", "南十字座β", "Mimosa", "南十字座", "Crux"}, 4905: []string{"北斗五", "玉衡", "大熊座ε", "Alioth", "大熊座", "UrsaMajor"}, 4915: []string{"常陈一", "", "猎犬座α2", "CorCaroli", "猎犬座", "CanesVenatici"}, 4932: []string{"太微左垣四", "东次将", "室女座ε", "Vindemiatrix", "室女座", "Virgo"}, 5054: []string{"北斗六", "开阳", "大熊座ζ", "Mizar", "大熊座", "UrsaMajor"}, 5056: []string{"角宿一", "", "室女座α", "Spica", "室女座", "Virgo"}, 5062: []string{"开阳增一", "辅星", "大熊座80", "Alcor", "大熊座", "UrsaMajor"}, 5191: []string{"北斗七", "摇光", "大熊座η", "Alkaid", "大熊座", "UrsaMajor"}, 1140: []string{"卷舌五", "", "英仙座ο", "Celaeno", "英仙座", "Perseus"}, 5235: []string{"右摄提一", "", "牧夫座η", "Muphrid", "牧夫座", "Bootes"}, 5267: []string{"马腹一", "", "半人马座β", "Hadar", "半人马座", "Centaurus"}, 1151: []string{"昴宿四", "", "金牛座20", "Asterope", "金牛座", "Taurus"}, 5291: []string{"紫微右垣一", "右枢", "天龙座α", "Thuban", "天龙座", "Draco"}, 1180: []string{"昴宿七", "", "金牛座27", "Pleione", "金牛座", "Taurus"}, 5288: []string{"库楼三", "", "半人马座θ", "Menkent", "半人马座", "Centaurus"}, 5340: []string{"大角", "", "牧夫座α", "Arcturus", "牧夫座", "Bootes"}, 5338: []string{"亢宿二", "", "室女座ι", "Syrma", "室女座", "Virgo"}, 5430: []string{"北极三", "庶子", "小熊座5", "", "小熊座", "UrsaMinor"}, 5435: []string{"招摇", "", "牧夫座γ", "Seginus", "牧夫座", "Bootes"}, 5459: []string{"南门二", "", "半人马座α1", "RigilKent", "半人马座", "Centaurus"}, 5506: []string{"梗河一", "", "牧夫座ε", "Izar", "牧夫座", "Bootes"}, 5563: []string{"北极二", "帝", "小熊座β", "Kocab", "小熊座", "UrsaMinor"}, 5531: []string{"氐宿一", "", "天秤座α", "Zubenelgenubi", "天秤座", "Libra"}, 5602: []string{"七公增五", "", "牧夫座β", "Nekkar", "牧夫座", "Bootes"}, 5685: []string{"氐宿四", "", "天秤座β", "Zubeneschamali", "天秤座", "Libra"}, 5735: []string{"北极一", "太子", "小熊座γ", "Pherkad", "小熊座", "UrsaMinor"}, 5733: []string{"七公六", "", "牧夫座μ1", "Alkalurops", "牧夫座", "Bootes"}, 5744: []string{"紫微左垣一", "左枢", "天龙座ι", "Edasich", "天龙座", "Draco"}, 5747: []string{"贯索三", "", "北冕座β", "Nusakan", "北冕座", "CoronaBorealis"}, 5793: []string{"贯索四", "", "北冕座α", "Alphekka", "北冕座", "CoronaBorealis"}, 5854: []string{"天市右垣七", "蜀", "巨蛇座α", "Unukalhai", "巨蛇座", "Serpens"}, 5953: []string{"房宿三", "", "天蝎座δ", "Dschubba", "天蝎座", "Scorpius"}, 5984: []string{"房宿四", "", "天蝎座β1", "Acrab", "天蝎座", "Scorpius"}, 6056: []string{"天市右垣九", "梁", "蛇夫座δ", "YedPrior", "蛇夫座", "Ophiuchus"}, 6075: []string{"天市右垣十", "楚", "蛇夫座ε", "YedPosterior", "蛇夫座", "Ophiuchus"}, 6084: []string{"心宿一", "", "天蝎座σ", "AlNiyat", "天蝎座", "Scorpius"}, 6117: []string{"斗一", "", "武仙座ω", "Cujam", "武仙座", "Hercules"}, 6134: []string{"心宿二", "大火", "天蝎座α", "Antares", "天蝎座", "Scorpius"}, 6148: []string{"天市右垣一", "河中", "武仙座β", "Kornephoros", "武仙座", "Hercules"}, 6149: []string{"列肆二", "", "蛇夫座λ", "Marfik", "蛇夫座", "Ophiuchus"}, 6217: []string{"三角形三", "", "南三角座α", "Atria", "南三角座", "TriangulumAustrale"}, 6369: []string{"天棓增九", "", "天龙座μ", "Alrakis", "天龙座", "Draco"}, 6378: []string{"天市左垣十一", "", "蛇夫座η", "Sabik", "蛇夫座", "Ophiuchus"}, 6406: []string{"帝座", "", "武仙座α1", "Rasalgethi", "武仙座", "Hercules"}, 6536: []string{"天棓三", "", "天龙座β", "Rastaban", "天龙座", "Draco"}, 6526: []string{"天市左垣二", "", "武仙座λ", "Maasym", "武仙座", "Hercules"}, 6508: []string{"尾宿九", "", "天蝎座υ", "Lesath", "天蝎座", "Scorpius"}, 6789: []string{"勾陈二", "", "小熊座δ", "Yildun", "小熊座", "UrsaMinor"}, 6527: []string{"尾宿八", "", "天蝎座λ", "Shaula", "天蝎座", "Scorpius"}, 6556: []string{"候", "", "蛇夫座α", "Rasalhague", "蛇夫座", "Ophiuchus"}, 6553: []string{"尾宿五", "", "天蝎座θ", "Girtab", "天蝎座", "Scorpius"}, 6603: []string{"宗正一", "", "蛇夫座β", "Cebalrai", "蛇夫座", "Ophiuchus"}, 6688: []string{"天棓一", "", "天龙座ξ", "Grumium", "天龙座", "Draco"}, 6705: []string{"天棓四", "", "天龙座γ", "Etamin", "天龙座", "Draco"}, 6746: []string{"箕宿一", "", "人马座γ2", "Alnasl", "人马座", "Sagittarius"}, 6859: []string{"箕宿二", "", "人马座δ", "KausMedia", "人马座", "Sagittarius"}, 6879: []string{"箕宿三", "", "人马座ε", "KausAustralis", "人马座", "Sagittarius"}, 6913: []string{"斗宿二", "", "人马座λ", "KausBorealis", "人马座", "Sagittarius"}, 7001: []string{"织女一", "织女", "天琴座α", "Vega", "天琴座", "Lyra"}, 7106: []string{"渐台二", "", "天琴座β", "Sheliak", "天琴座", "Lyra"}, 7121: []string{"斗宿四", "", "人马座σ", "Nunki", "人马座", "Sagittarius"}, 7141: []string{"天市左垣七", "", "巨蛇座θ1", "Alya", "巨蛇座", "Serpens"}, 5533: []string{"异雀八", "", "天燕座α", "Merga", "天燕座", "Apus"}, 7178: []string{"渐台三", "", "天琴座γ", "Sulafat", "天琴座", "Lyra"}, 7194: []string{"斗宿六", "", "人马座ζ", "Ascella", "人马座", "Sagittarius"}, 7310: []string{"天厨一", "", "天龙座δ", "Altais", "天龙座", "Draco"}, 7337: []string{"天渊二", "", "人马座β1", "Arkab", "人马座", "Sagittarius"}, 7348: []string{"天渊三", "", "人马座α", "Rukbat", "人马座", "Sagittarius"}, 7417: []string{"辇道增七", "", "天鹅座β1", "Albireo", "天鹅座", "Cygnus"}, 6008: []string{"钩钤二", "", "天蝎座ω2", "Marsic", "天蝎座", "Scorpius"}, 7479: []string{"左旗一", "", "天箭座α", "Sham", "天箭座", "Sagitta"}, 7525: []string{"河鼓三", "", "天鹰座γ", "Tarazed", "天鹰座", "Aquila"}, 7557: []string{"河鼓二", "牛郎", "天鹰座α", "Altair", "天鹰座", "Aquila"}, 7602: []string{"河鼓一", "", "天鹰座β", "Alshain", "天鹰座", "Aquila"}, 7747: []string{"牛宿增六", "", "摩羯座α1", "Algedi", "摩羯座", "Capricornus"}, 7776: []string{"牛宿一", "", "摩羯座β1", "Dabih", "摩羯座", "Capricornus"}, 7796: []string{"天津一", "", "天鹅座γ", "Sadr", "天鹅座", "Cygnus"}, 7790: []string{"孔雀十一", "", "孔雀座α", "Peacock", "孔雀座", "Pavo"}, 7882: []string{"瓠瓜四", "", "海豚座β", "Rotanev", "海豚座", "Delphinus"}, 7906: []string{"瓠瓜一", "", "海豚座α", "Sualocin", "海豚座", "Delphinus"}, 7924: []string{"天津四", "", "天鹅座α", "Deneb", "天鹅座", "Cygnus"}, 7949: []string{"天津九", "", "天鹅座ε", "Gienah", "天鹅座", "Cygnus"}, 7950: []string{"女宿一", "", "宝瓶座ε", "Albali", "宝瓶座", "Aquarius"}, 8131: []string{"虚宿二", "", "小马座α", "Kitalpha", "小马座", "Equuleus"}, 8162: []string{"天钩五", "", "仙王座α", "Alderamin", "仙王座", "Cepheus"}, 8238: []string{"上卫增一", "", "仙王座β", "Alfirk", "仙王座", "Cepheus"}, 8232: []string{"虚宿一", "", "宝瓶座β", "Sadalsuud", "宝瓶座", "Aquarius"}, 8278: []string{"垒壁阵三", "", "摩羯座γ", "Nashira", "摩羯座", "Capricornus"}, 8301: []string{"螣蛇四", "", "天鹅座π1", "Azelfafage", "天鹅座", "Cygnus"}, 8316: []string{"造父四", "", "仙王座μ", "TheGarnetStar", "仙王座", "Cepheus"}, 8308: []string{"危宿三", "", "飞马座ε", "Enif", "飞马座", "Pegasus"}, 8322: []string{"垒壁阵四", "", "摩羯座δ", "DenebAlgedi", "摩羯座", "Capricornus"}, 8417: []string{"天钩六", "", "仙王座ξ", "Kurhah", "仙王座", "Cepheus"}, 8414: []string{"危宿一", "", "宝瓶座α", "Sadalmelik", "宝瓶座", "Aquarius"}, 8425: []string{"鹤一", "", "天鹤座α", "Alnair", "天鹤座", "Grus"}, 8450: []string{"危宿二", "", "飞马座θ", "Biham", "飞马座", "Pegasus"}, 8499: []string{"泣二", "", "宝瓶座θ", "Ancha", "宝瓶座", "Aquarius"}, 8518: []string{"坟墓二", "", "宝瓶座γ", "Sadachbia", "宝瓶座", "Aquarius"}, 8634: []string{"雷电一", "", "飞马座ζ", "Homan", "飞马座", "Pegasus"}, 8650: []string{"离宫四", "", "飞马座η", "Matar", "飞马座", "Pegasus"}, 8684: []string{"离宫二", "", "飞马座μ", "Sadalbari", "飞马座", "Pegasus"}, 8709: []string{"羽林军二十六", "", "宝瓶座δ", "Skat", "宝瓶座", "Aquarius"}, 8610: []string{"螣蛇十五", "", "蝎虎座9", "Situla", "蝎虎座", "Lacerta"}, 8728: []string{"北落师门", "", "南鱼座α", "Fomalhaut", "南鱼座", "PiscisAustrinus"}, 8775: []string{"室宿二", "", "飞马座β", "Sheat", "飞马座", "Pegasus"}, 8781: []string{"室宿一", "", "飞马座α", "Markab", "飞马座", "Pegasus"}, 8974: []string{"少卫增八", "", "仙王座γ", "Errai", "仙王座", "Cepheus"}}
	hr2hip = map[uint16]uint32{1: 424, 2: 417, 3: 443, 4: 476, 5: 518, 6: 522, 7: 531, 8: 544, 9: 560, 10: 602, 11: 635, 12: 636, 13: 655, 14: 664, 15: 677, 16: 671, 17: 699, 18: 696, 19: 716, 20: 760, 21: 746, 22: 729, 23: 730, 24: 761, 25: 765, 26: 813, 27: 841, 28: 857, 29: 840, 30: 814, 31: 873, 32: 865, 33: 910, 34: 930, 35: 950, 36: 967, 37: 983, 38: 1030, 39: 1067, 40: 1076, 41: 1086, 42: 1096, 43: 1099, 44: 1123, 45: 1168, 46: 1158, 47: 1074, 48: 1170, 49: 1193, 50: 1196, 51: 1191, 52: 1208, 53: 1215, 54: 1195, 55: 1296, 56: 1302, 57: 1288, 58: 1268, 59: 1319, 60: 1354, 61: 1331, 62: 1372, 63: 1366, 64: 1342, 65: 1415, 66: 1402, 67: 1421, 68: 1473, 69: 1465, 70: 1501, 71: 1493, 72: 1499, 73: 1502, 74: 1562, 75: 1575, 76: 1603, 77: 1599, 78: 1630, 79: 1657, 80: 1645, 81: 1670, 82: 1686, 83: 1647, 84: 1708, 85: 1728, 86: 1772, 87: 1706, 88: 1803, 89: 1830, 90: 1901, 91: 1921, 93: 1960, 94: 1939, 96: 1982, 97: 2006, 98: 2021, 99: 2081, 100: 2072, 101: 2100, 102: 2159, 103: 2219, 104: 2225, 105: 2210, 106: 2224, 107: 2235, 108: 2237, 109: 2240, 110: 2270, 111: 2256, 112: 2422, 113: 2377, 114: 2355, 115: 2337, 116: 2330, 117: 2353, 118: 2381, 119: 2388, 120: 2383, 121: 2474, 122: 2475, 123: 2505, 124: 2497, 125: 2472, 126: 2484, 127: 2487, 128: 2553, 129: 2628, 130: 2599, 131: 2568, 132: 2548, 133: 2565, 134: 2583, 135: 2611, 136: 2578, 137: 2707, 138: 2661, 139: 2629, 140: 2711, 141: 2734, 142: 2762, 143: 2787, 144: 2854, 145: 2832, 146: 2876, 147: 2802, 148: 2789, 149: 2865, 150: 2846, 151: 2852, 152: 2900, 153: 2920, 154: 2912, 155: 2903, 156: 2926, 157: 2942, 158: 3132, 159: 2941, 160: 2889, 161: 2954, 162: 2937, 163: 3031, 164: 3083, 165: 3092, 166: 3093, 167: 3138, 168: 3179, 169: 3042, 170: 3142, 171: 3137, 172: 3175, 173: 3185, 174: 3193, 175: 3231, 176: 3170, 177: 3299, 178: 3269, 179: 3300, 180: 3245, 181: 3334, 183: 3277, 184: 3414, 185: 3356, 186: 3352, 187: 3330, 188: 3419, 189: 3478, 190: 3436, 191: 3405, 192: 3572, 193: 3504, 194: 3455, 195: 3456, 196: 3544, 197: 3505, 198: 3521, 199: 3489, 200: 3641, 201: 3552, 202: 3527, 203: 3559, 204: 3611, 205: 3604, 206: 3576, 207: 3649, 208: 3721, 209: 3583, 210: 3607, 211: 3632, 212: 3750, 213: 3675, 214: 3685, 215: 3693, 216: 3697, 217: 3730, 218: 3717, 219: 3821, 220: 3741, 221: 3760, 222: 3765, 223: 3801, 224: 3786, 225: 3810, 226: 3881, 227: 3849, 228: 3834, 229: 3807, 230: 3885, 232: 3858, 233: 3951, 234: 3919, 235: 3909, 236: 3781, 237: 3988, 238: 3965, 239: 3903, 240: 4283, 241: 4023, 242: 3949, 243: 3992, 244: 4151, 245: 4049, 246: 4129, 247: 4104, 248: 4147, 249: 4185, 250: 4212, 251: 4164, 252: 4084, 253: 4292, 254: 4267, 255: 4257, 256: 4298, 257: 4200, 258: 4288, 259: 4317, 260: 4383, 261: 4446, 262: 4366, 263: 4346, 264: 4427, 265: 4422, 266: 4440, 267: 4371, 268: 4363, 269: 4436, 270: 4293, 271: 4463, 272: 4501, 273: 4572, 274: 4510, 275: 4552, 276: 4520, 277: 4558, 278: 4709, 279: 4587, 280: 4577, 281: 4563, 282: 4675, 284: 4655, 285: 5372, 286: 7283, 287: 4844, 288: 4770, 289: 4911, 290: 4903, 291: 4889, 292: 4962, 293: 4852, 294: 4906, 295: 4829, 296: 4914, 297: 5021, 298: 4998, 299: 4890, 300: 4932, 301: 4979, 302: 5062, 303: 5034, 304: 4879, 305: 5045, 306: 5928, 307: 5074, 308: 5081, 309: 5186, 310: 5131, 311: 5132, 312: 5412, 313: 5141, 314: 5144, 315: 5121, 316: 5240, 317: 5164, 318: 5251, 319: 5204, 320: 5170, 321: 5336, 322: 5165, 323: 5193, 324: 5317, 325: 5259, 326: 5361, 327: 5319, 328: 5310, 329: 5296, 330: 5346, 331: 5300, 332: 5268, 333: 5626, 334: 5364, 335: 5434, 336: 5518, 337: 5447, 338: 5348, 339: 5454, 340: 5493, 341: 5494, 342: 5566, 343: 5542, 344: 5483, 345: 5589, 346: 5485, 347: 5510, 348: 5550, 349: 5544, 350: 5477, 351: 5571, 352: 5586, 353: 5594, 354: 5688, 355: 5650, 356: 5679, 357: 5947, 358: 5636, 359: 5661, 360: 5742, 361: 5737, 362: 5743, 363: 5772, 364: 5778, 365: 5926, 366: 5799, 367: 5824, 368: 5833, 369: 5939, 370: 5862, 371: 5936, 372: 5993, 373: 5951, 374: 6025, 375: 6261, 376: 6087, 377: 5896, 378: 6061, 379: 6140, 380: 5992, 381: 6378, 382: 6242, 383: 6193, 384: 6312, 385: 6226, 386: 6522, 387: 6283, 388: 6272, 389: 6315, 390: 6411, 391: 6486, 392: 6432, 393: 6429, 394: 6427, 395: 6514, 396: 6512, 397: 6492, 398: 6685, 399: 6692, 400: 6502, 401: 6539, 402: 6537, 403: 6686, 404: 6564, 405: 6589, 406: 6605, 407: 6669, 408: 6592, 409: 6711, 410: 6702, 411: 6595, 412: 6670, 413: 6706, 414: 6732, 415: 6776, 416: 6717, 417: 6813, 418: 6794, 419: 6751, 420: 6631, 421: 6748, 422: 6823, 423: 6759, 424: 11767, 425: 6822, 426: 6868, 427: 7078, 428: 7050, 429: 6867, 430: 6999, 431: 6888, 432: 6981, 433: 6960, 434: 7007, 435: 6952, 436: 7016, 437: 7097, 438: 7147, 439: 7251, 440: 7083, 441: 7118, 442: 7294, 443: 7115, 444: 7222, 445: 7213, 446: 7321, 447: 7189, 448: 7276, 449: 7557, 450: 7359, 451: 7345, 452: 7444, 453: 7361, 454: 7493, 455: 7436, 456: 7650, 457: 7447, 458: 7513, 459: 7450, 460: 7387, 461: 7617, 462: 7463, 463: 7535, 464: 7607, 465: 7651, 466: 7580, 467: 7271, 468: 7506, 469: 7719, 470: 7786, 471: 7643, 472: 7588, 473: 7679, 474: 7676, 475: 7740, 476: 7825, 477: 7818, 478: 7965, 479: 7673, 480: 8016, 481: 7939, 482: 7963, 483: 7918, 484: 7874, 485: 7906, 486: 7751, 487: 0, 488: 8020, 489: 7884, 490: 7943, 491: 8046, 492: 7916, 493: 7981, 494: 7886, 495: 8044, 496: 8068, 497: 7955, 498: 7941, 499: 8115, 500: 7999, 501: 7911, 502: 8148, 503: 8086, 504: 8127, 505: 7921, 506: 7978, 507: 8094, 508: 8159, 509: 8102, 510: 8198, 511: 8362, 512: 7601, 513: 8230, 514: 8209, 515: 8271, 516: 7879, 517: 8233, 518: 8370, 519: 8240, 520: 8241, 521: 8423, 522: 8387, 523: 8433, 524: 8320, 525: 7568, 526: 8475, 527: 8404, 528: 8369, 529: 8598, 530: 8544, 531: 8497, 532: 8480, 533: 8704, 534: 8588, 535: 8514, 536: 8714, 537: 8552, 538: 8771, 539: 8645, 540: 8847, 541: 8593, 542: 8886, 543: 8814, 544: 8796, 546: 8832, 547: 8778, 548: 9009, 549: 8833, 550: 8366, 551: 8922, 552: 8930, 553: 8903, 554: 8820, 555: 8837, 556: 9001, 557: 9021, 558: 8882, 559: 8993, 560: 8998, 561: 9220, 562: 9158, 563: 9110, 564: 9132, 565: 9061, 566: 9007, 567: 9312, 568: 9222, 569: 9153, 570: 8928, 571: 8983, 572: 9568, 573: 9085, 574: 9095, 575: 9480, 576: 9207, 577: 9307, 578: 9295, 579: 9586, 580: 9598, 581: 9727, 582: 9353, 583: 9326, 584: 9149, 585: 9347, 586: 9564, 587: 9372, 588: 9313, 589: 9573, 590: 9505, 591: 9236, 592: 9763, 593: 8991, 594: 9440, 595: 9487, 597: 10054, 598: 9690, 599: 9570, 600: 9318, 601: 9533, 602: 9459, 603: 9640, 605: 9621, 606: 9521, 607: 9589, 608: 9572, 609: 9627, 610: 9631, 611: 9622, 612: 9677, 613: 9836, 614: 9809, 615: 9859, 616: 9827, 617: 9884, 618: 9990, 619: 9983, 620: 9977, 621: 10115, 622: 10064, 623: 10053, 624: 10050, 625: 9999, 626: 10350, 627: 10227, 628: 10176, 629: 10180, 630: 10069, 631: 10155, 632: 10035, 633: 10203, 634: 10220, 635: 10212, 636: 10111, 637: 10138, 638: 10215, 639: 10234, 640: 10438, 641: 10379, 642: 10280, 643: 10340, 644: 10296, 645: 10366, 646: 10306, 647: 10403, 648: 10328, 649: 10324, 650: 10305, 651: 10326, 652: 10320, 653: 10562, 654: 10633, 655: 10559, 656: 10540, 657: 10535, 658: 10512, 659: 10440, 660: 10644, 661: 10718, 662: 10729, 663: 10687, 664: 10670, 665: 10657, 666: 10642, 667: 10418, 668: 10893, 669: 10732, 670: 10819, 671: 10814, 672: 10723, 673: 10830, 674: 10602, 675: 10793, 676: 10795, 677: 10848, 678: 10513, 679: 10924, 680: 10884, 681: 10826, 682: 10944, 683: 10798, 684: 10854, 685: 11060, 686: 10832, 687: 11090, 688: 10871, 689: 11021, 690: 11174, 691: 11046, 692: 11029, 693: 11033, 694: 11220, 695: 11072, 696: 11279, 697: 11108, 698: 11043, 699: 11313, 700: 11121, 701: 11102, 702: 11249, 703: 11212, 704: 11261, 705: 11001, 706: 11231, 707: 11569, 708: 11345, 709: 11465, 710: 11348, 711: 11415, 712: 11432, 713: 11381, 714: 11258, 715: 11095, 716: 11593, 717: 11486, 718: 11484, 719: 11474, 720: 11548, 721: 11407, 722: 11293, 723: 11578, 724: 11477, 725: 11603, 726: 11651, 727: 11524, 728: 11670, 729: 11678, 730: 11622, 731: 11698, 732: 11687, 733: 11644, 734: 11479, 735: 11671, 736: 11784, 737: 11738, 738: 11840, 739: 11791, 740: 11783, 741: 11843, 742: 11802, 743: 12273, 744: 11867, 745: 11907, 746: 12057, 747: 12239, 748: 12072, 749: 11918, 750: 12086, 751: 12022, 752: 12002, 753: 12114, 754: 12093, 755: 11925, 756: 12181, 757: 12160, 758: 12193, 759: 12107, 760: 12218, 761: 12200, 762: 11900, 763: 12153, 764: 12184, 765: 12189, 766: 12148, 767: 12122, 768: 12287, 769: 12305, 770: 12272, 771: 12247, 772: 12186, 773: 12332, 774: 13055, 775: 12318, 776: 11757, 777: 12288, 778: 12225, 779: 12387, 780: 12300, 781: 12390, 782: 12489, 783: 12447, 784: 12444, 785: 12692, 786: 12435, 787: 12686, 788: 12623, 789: 12413, 790: 12530, 791: 12821, 792: 12690, 793: 12640, 794: 12486, 795: 12584, 796: 12562, 797: 12647, 798: 12389, 799: 12777, 800: 12768, 801: 12719, 802: 12484, 803: 12744, 804: 12706, 805: 12608, 806: 12394, 807: 12611, 808: 12784, 809: 12803, 810: 12653, 811: 12770, 812: 12832, 813: 12828, 814: 12708, 815: 13133, 816: 12862, 817: 12786, 818: 12843, 819: 12982, 820: 12990, 821: 12775, 822: 12855, 823: 12717, 824: 13061, 825: 13178, 826: 12964, 827: 13008, 828: 13108, 829: 13367, 830: 13121, 831: 13176, 832: 13064, 833: 12871, 834: 13268, 835: 13040, 836: 13165, 837: 12876, 838: 13209, 839: 13347, 840: 13254, 841: 13147, 842: 13339, 843: 13328, 844: 13197, 845: 13202, 846: 13424, 847: 13327, 848: 13225, 849: 13462, 850: 13288, 851: 13265, 852: 13141, 853: 13271, 854: 13531, 855: 13490, 856: 13448, 857: 13402, 858: 13363, 859: 13421, 860: 13665, 861: 13700, 862: 13479, 863: 13473, 864: 13732, 865: 13713, 866: 13387, 867: 13654, 868: 13502, 869: 13702, 870: 13679, 871: 13518, 872: 13244, 873: 13775, 874: 13701, 875: 13717, 876: 13832, 877: 13756, 878: 13834, 879: 13879, 880: 13537, 881: 14417, 882: 13905, 883: 13782, 884: 13768, 885: 13965, 886: 13949, 887: 13914, 889: 13835, 890: 14043, 891: 14047, 892: 13874, 893: 13789, 894: 13997, 895: 13883, 896: 13954, 897: 13847, 899: 13951, 900: 13907, 901: 13942, 902: 14036, 903: 13947, 904: 14040, 905: 14109, 906: 14844, 907: 14060, 908: 14104, 909: 13884, 910: 14143, 911: 14135, 912: 14110, 913: 14124, 914: 14086, 915: 14328, 916: 14232, 917: 14168, 918: 14382, 919: 14146, 920: 14392, 921: 14354, 922: 14502, 923: 14365, 924: 14318, 925: 14293, 926: 14315, 927: 14376, 929: 14187, 930: 14566, 931: 14439, 932: 14862, 933: 14544, 934: 14240, 935: 14456, 936: 14576, 937: 14632, 938: 14514, 939: 14131, 940: 14586, 941: 14668, 942: 14607, 943: 14551, 944: 14677, 945: 14719, 946: 14748, 947: 14817, 948: 14764, 949: 14914, 950: 14887, 951: 14838, 952: 14821, 953: 14757, 954: 14893, 955: 14819, 956: 15004, 957: 14814, 958: 14915, 959: 14547, 960: 14749, 961: 15547, 962: 14954, 963: 14879, 964: 15192, 965: 16489, 966: 15118, 967: 15309, 968: 14913, 969: 15219, 970: 14972, 971: 15154, 972: 15110, 973: 15209, 974: 15024, 975: 15191, 976: 15204, 977: 14930, 978: 15241, 979: 15282, 980: 15125, 981: 14521, 982: 15338, 983: 15203, 984: 15197, 985: 15520, 986: 15334, 987: 15404, 988: 15244, 989: 15444, 990: 15218, 991: 15416, 992: 15383, 993: 15357, 994: 15382, 995: 15514, 996: 15457, 997: 15411, 998: 15305, 999: 15549, 1000: 15557, 1001: 15669, 1002: 15648, 1003: 15474, 1004: 15479, 1005: 15627, 1006: 15330, 1007: 15619, 1008: 15510, 1009: 15890, 1010: 15371, 1011: 15770, 1012: 15696, 1013: 15585, 1014: 15353, 1015: 15737, 1016: 15643, 1017: 15863, 1018: 15700, 1019: 15876, 1020: 15979, 1021: 15655, 1022: 15861, 1023: 15807, 1024: 15776, 1025: 15201, 1026: 15925, 1027: 15870, 1028: 15850, 1029: 15988, 1030: 15900, 1031: 15816, 1032: 16319, 1033: 16177, 1034: 16147, 1035: 16228, 1036: 16077, 1037: 16210, 1038: 16083, 1039: 16095, 1040: 16281, 1041: 16168, 1042: 15987, 1043: 16303, 1044: 16244, 1045: 16029, 1046: 16292, 1047: 16252, 1048: 16181, 1049: 16015, 1050: 16142, 1051: 16340, 1052: 16335, 1053: 15840, 1054: 16112, 1055: 16725, 1056: 16424, 1058: 16156, 1059: 16447, 1060: 16266, 1061: 16322, 1062: 16263, 1063: 16470, 1064: 15968, 1065: 16411, 1066: 16369, 1067: 16358, 1068: 16587, 1069: 16499, 1070: 16341, 1071: 16602, 1072: 16516, 1073: 16599, 1074: 16518, 1075: 16285, 1076: 16310, 1077: 16712, 1078: 16591, 1079: 16511, 1080: 17056, 1081: 16339, 1082: 16449, 1083: 16245, 1084: 16537, 1085: 16641, 1086: 16664, 1087: 16826, 1088: 16611, 1089: 16695, 1090: 16509, 1091: 16677, 1092: 16368, 1093: 16628, 1094: 16974, 1095: 16672, 1096: 16531, 1097: 16938, 1098: 16780, 1099: 16846, 1100: 16803, 1101: 16852, 1102: 16924, 1103: 17026, 1104: 16661, 1105: 17296, 1106: 16870, 1107: 19454, 1108: 16989, 1109: 16290, 1110: 17058, 1111: 17027, 1112: 17342, 1113: 17203, 1114: 17007, 1115: 17103, 1116: 17086, 1117: 17057, 1118: 17181, 1119: 17120, 1120: 17136, 1121: 17167, 1122: 17358, 1123: 17313, 1124: 17585, 1125: 17214, 1126: 17309, 1127: 17437, 1128: 17223, 1129: 17587, 1130: 17475, 1131: 17448, 1132: 17408, 1133: 17460, 1134: 17304, 1135: 17529, 1136: 17378, 1137: 17453, 1138: 17854, 1139: 17395, 1140: 17489, 1141: 17584, 1142: 17499, 1143: 17351, 1144: 17527, 1145: 17531, 1146: 17457, 1147: 17707, 1148: 17959, 1149: 17573, 1150: 17506, 1151: 17579, 1152: 17588, 1153: 17563, 1154: 16827, 1155: 17884, 1156: 17608, 1157: 17436, 1158: 17891, 1159: 17595, 1160: 17772, 1161: 17858, 1162: 17593, 1163: 17735, 1164: 17732, 1165: 17702, 1166: 18067, 1167: 17493, 1168: 17464, 1169: 17534, 1170: 17846, 1171: 17618, 1172: 17776, 1173: 17651, 1174: 17771, 1175: 17440, 1176: 17932, 1177: 17886, 1178: 17847, 1179: 17689, 1180: 17851, 1181: 17717, 1182: 17805, 1183: 17900, 1184: 17738, 1185: 17921, 1186: 17734, 1187: 17798, 1188: 17954, 1189: 17797, 1191: 18081, 1192: 18217, 1193: 18059, 1194: 18033, 1195: 17874, 1196: 18438, 1197: 18094, 1198: 18212, 1199: 18089, 1200: 17982, 1201: 18170, 1202: 18141, 1203: 18246, 1204: 18505, 1205: 18488, 1206: 18173, 1207: 18396, 1208: 17678, 1209: 18350, 1210: 18453, 1211: 18255, 1213: 18216, 1214: 18213, 1215: 18434, 1216: 18199, 1217: 18339, 1218: 18471, 1219: 18262, 1220: 18532, 1221: 18485, 1222: 18508, 1223: 18547, 1224: 18481, 1225: 18455, 1226: 18565, 1227: 18275, 1228: 18614, 1229: 18657, 1230: 19461, 1231: 18543, 1232: 18606, 1233: 18658, 1234: 18769, 1235: 18647, 1236: 18401, 1237: 18717, 1238: 18735, 1239: 18724, 1240: 18673, 1241: 19129, 1242: 19018, 1243: 18805, 1244: 18788, 1245: 18592, 1246: 18723, 1247: 18597, 1248: 19177, 1249: 18859, 1250: 18691, 1251: 18907, 1252: 19009, 1253: 18957, 1254: 18975, 1255: 19172, 1256: 19038, 1257: 18993, 1258: 18926, 1259: 18942, 1260: 19272, 1261: 19167, 1262: 19076, 1263: 18990, 1264: 18744, 1265: 19011, 1266: 18772, 1267: 19037, 1268: 19171, 1269: 19205, 1270: 19412, 1271: 17328, 1272: 19121, 1273: 19343, 1274: 19111, 1275: 19095, 1276: 19400, 1277: 19302, 1278: 19335, 1279: 19261, 1280: 19284, 1281: 19730, 1282: 19673, 1283: 19388, 1284: 19376, 1285: 19242, 1286: 19525, 1287: 19513, 1288: 19398, 1289: 20860, 1290: 19483, 1291: 19331, 1292: 19554, 1293: 19511, 1294: 19233, 1295: 19641, 1296: 19823, 1297: 19672, 1298: 19587, 1299: 19509, 1300: 19571, 1301: 19746, 1302: 19515, 1303: 19812, 1304: 20982, 1305: 19968, 1306: 19811, 1307: 19720, 1308: 19718, 1309: 19719, 1310: 19736, 1311: 19740, 1312: 19725, 1313: 19983, 1314: 19949, 1315: 19799, 1316: 19642, 1317: 20776, 1318: 19777, 1319: 19877, 1320: 19860, 1321: 19855, 1322: 19859, 1323: 19721, 1324: 20070, 1325: 19849, 1326: 19747, 1327: 20266, 1328: 20063, 1329: 19990, 1330: 20156, 1331: 20087, 1332: 19996, 1333: 20234, 1334: 20089, 1335: 20376, 1336: 19780, 1337: 20241, 1338: 19893, 1339: 20171, 1340: 19805, 1341: 20186, 1342: 20380, 1343: 20252, 1344: 20231, 1345: 20075, 1346: 20205, 1347: 20042, 1348: 20250, 1349: 20188, 1350: 20354, 1351: 20219, 1352: 20456, 1353: 20106, 1354: 20255, 1355: 19921, 1356: 20261, 1357: 19917, 1358: 20284, 1359: 20138, 1360: 20268, 1361: 20281, 1362: 20263, 1363: 20271, 1364: 20161, 1365: 20109, 1366: 20341, 1367: 20264, 1368: 20400, 1369: 20430, 1370: 20417, 1371: 20498, 1372: 20020, 1373: 20455, 1374: 20347, 1375: 20493, 1376: 20484, 1377: 20579, 1378: 20533, 1379: 20591, 1380: 20542, 1381: 20522, 1382: 20772, 1383: 20507, 1384: 20465, 1385: 20614, 1386: 20467, 1387: 20635, 1388: 20641, 1389: 20648, 1390: 20704, 1391: 20661, 1392: 20711, 1393: 20535, 1394: 20713, 1395: 20384, 1396: 20732, 1397: 20715, 1398: 20613, 1399: 20789, 1400: 20771, 1401: 21247, 1402: 20804, 1403: 20842, 1404: 20630, 1405: 20552, 1406: 20904, 1407: 20877, 1408: 20873, 1409: 20889, 1410: 20765, 1411: 20885, 1412: 20894, 1413: 20848, 1414: 20901, 1415: 20884, 1416: 20619, 1417: 21148, 1418: 20781, 1419: 21038, 1420: 20963, 1421: 20892, 1422: 20995, 1423: 20922, 1424: 21108, 1425: 20985, 1426: 20049, 1427: 21029, 1428: 21039, 1429: 20856, 1430: 21036, 1431: 20997, 1432: 21137, 1433: 20934, 1434: 21242, 1435: 20825, 1436: 21152, 1437: 21139, 1438: 21110, 1439: 21042, 1440: 21452, 1441: 21192, 1442: 21251, 1443: 21060, 1444: 21273, 1445: 21323, 1446: 21269, 1447: 21239, 1448: 21295, 1449: 21278, 1450: 21298, 1451: 21296, 1452: 21297, 1453: 21248, 1454: 21476, 1455: 21408, 1456: 20297, 1457: 21421, 1458: 21402, 1459: 21459, 1460: 21377, 1461: 21363, 1462: 21428, 1463: 21444, 1464: 21393, 1465: 21281, 1466: 21730, 1467: 21727, 1468: 22152, 1469: 21515, 1470: 21619, 1471: 21604, 1472: 21588, 1473: 21589, 1474: 21547, 1475: 21253, 1476: 21486, 1477: 21689, 1478: 21673, 1479: 21683, 1480: 21670, 1481: 21594, 1482: 21823, 1483: 21644, 1484: 21735, 1485: 20446, 1486: 21966, 1487: 21685, 1488: 21718, 1489: 21847, 1490: 21819, 1491: 22361, 1492: 21479, 1493: 21972, 1494: 21928, 1495: 21743, 1496: 21763, 1497: 21881, 1498: 21666, 1499: 21961, 1500: 22034, 1501: 22003, 1502: 21770, 1503: 21861, 1504: 21756, 1505: 21986, 1507: 22044, 1508: 22024, 1509: 21958, 1510: 22508, 1511: 22287, 1512: 22128, 1513: 22028, 1514: 22220, 1515: 22300, 1516: 21914, 1517: 22176, 1518: 21998, 1519: 22157, 1520: 22109, 1521: 22086, 1522: 22189, 1523: 23265, 1524: 22136, 1525: 22192, 1526: 22144, 1527: 22626, 1528: 22407, 1529: 22393, 1530: 22040, 1531: 21611, 1532: 22263, 1533: 22453, 1534: 22354, 1535: 22545, 1536: 22336, 1537: 22441, 1538: 22325, 1539: 22280, 1540: 22081, 1541: 21949, 1542: 22783, 1543: 22449, 1544: 22509, 1545: 22439, 1546: 22720, 1547: 22565, 1548: 22329, 1549: 22479, 1550: 22699, 1551: 22678, 1552: 22549, 1553: 22597, 1554: 22697, 1555: 22854, 1556: 22667, 1557: 22488, 1558: 22842, 1559: 22573, 1560: 22701, 1561: 22936, 1562: 22730, 1563: 22531, 1564: 22534, 1565: 22774, 1566: 22850, 1567: 22797, 1568: 23040, 1569: 22833, 1570: 22845, 1571: 22834, 1572: 23442, 1573: 22955, 1574: 22840, 1575: 22949, 1576: 22913, 1577: 23015, 1578: 22923, 1579: 22860, 1580: 22957, 1581: 22881, 1582: 22958, 1583: 22897, 1584: 22847, 1585: 23043, 1586: 23068, 1587: 23582, 1588: 23216, 1589: 23617, 1590: 23088, 1591: 23041, 1592: 23179, 1593: 23380, 1594: 23484, 1595: 23060, 1596: 23092, 1597: 22844, 1598: 22737, 1599: 23261, 1600: 23161, 1601: 23123, 1602: 23268, 1603: 23522, 1604: 23166, 1605: 23416, 1606: 22717, 1607: 23203, 1608: 23221, 1609: 23279, 1610: 23287, 1611: 23231, 1612: 23453, 1613: 23296, 1614: 23311, 1615: 23511, 1616: 25911, 1617: 23364, 1618: 23408, 1619: 23419, 1620: 23497, 1621: 23362, 1622: 23734, 1623: 23743, 1624: 23766, 1625: 23475, 1626: 23583, 1627: 23611, 1628: 23430, 1629: 22871, 1630: 23753, 1631: 23377, 1632: 23614, 1633: 23589, 1634: 23474, 1635: 23446, 1636: 24003, 1637: 23783, 1638: 23607, 1639: 23724, 1640: 23551, 1641: 23767, 1642: 23695, 1643: 24254, 1644: 23799, 1645: 23554, 1646: 23643, 1647: 24017, 1648: 23680, 1649: 23482, 1650: 24440, 1651: 23555, 1652: 23595, 1653: 23596, 1654: 23685, 1655: 23668, 1656: 23835, 1657: 23794, 1658: 23871, 1659: 23900, 1660: 23883, 1661: 23777, 1662: 23852, 1663: 23649, 1664: 23879, 1665: 23831, 1666: 23875, 1667: 23653, 1668: 24109, 1669: 24072, 1670: 24019, 1671: 23916, 1672: 23983, 1673: 23941, 1674: 23693, 1675: 24313, 1676: 24010, 1677: 23467, 1678: 24348, 1679: 23972, 1680: 23910, 1681: 24041, 1682: 23251, 1683: 24732, 1684: 24197, 1685: 24130, 1686: 25110, 1687: 24162, 1688: 24479, 1689: 24340, 1690: 24196, 1691: 24203, 1692: 24451, 1693: 24169, 1694: 24114, 1695: 23840, 1696: 24244, 1697: 24294, 1698: 24331, 1699: 24189, 1700: 23737, 1701: 24349, 1702: 24305, 1703: 24377, 1705: 24327, 1706: 24504, 1707: 24645, 1708: 24608, 1709: 24450, 1710: 24394, 1711: 24512, 1712: 24575, 1713: 24436, 1714: 27015, 1715: 24373, 1716: 23148, 1717: 24493, 1718: 24555, 1719: 24836, 1720: 24914, 1721: 24426, 1722: 24738, 1723: 24505, 1724: 24607, 1725: 24771, 1726: 24727, 1727: 24384, 1728: 24740, 1729: 24813, 1730: 24532, 1731: 24618, 1732: 24799, 1733: 24881, 1734: 24832, 1735: 24674, 1736: 24951, 1737: 24679, 1738: 24902, 1739: 24822, 1740: 24879, 1741: 24820, 1742: 24520, 1743: 24659, 1744: 24372, 1745: 25714, 1746: 24817, 1747: 24786, 1748: 24847, 1749: 25048, 1750: 24984, 1751: 25197, 1752: 25001, 1753: 24825, 1754: 24827, 1755: 24977, 1756: 24845, 1757: 24873, 1758: 24831, 1759: 24925, 1760: 25143, 1761: 25011, 1762: 24927, 1763: 25041, 1764: 25028, 1765: 25044, 1766: 24909, 1767: 24829, 1768: 25192, 1769: 25053, 1770: 25142, 1771: 25045, 1772: 25007, 1773: 25292, 1774: 25216, 1775: 25290, 1776: 25291, 1777: 25205, 1778: 25187, 1779: 25343, 1780: 25278, 1781: 25223, 1782: 25240, 1783: 25202, 1784: 25247, 1785: 25180, 1786: 25288, 1787: 25282, 1788: 25281, 1789: 25302, 1790: 25336, 1791: 25428, 1792: 25280, 1793: 25194, 1794: 25476, 1795: 25471, 1796: 25475, 1797: 25221, 1798: 25410, 1799: 25329, 1800: 25365, 1801: 25098, 1802: 25769, 1803: 25378, 1804: 25492, 1805: 25541, 1806: 25401, 1807: 25453, 1808: 25499, 1809: 25502, 1810: 25539, 1811: 25473, 1812: 25397, 1813: 25317, 1814: 25555, 1815: 24256, 1816: 25583, 1817: 25486, 1818: 25303, 1819: 25638, 1820: 25582, 1821: 25695, 1822: 25730, 1823: 25532, 1824: 25816, 1825: 25810, 1826: 25667, 1827: 25488, 1828: 25973, 1829: 25606, 1830: 25708, 1831: 25806, 1832: 25790, 1833: 25751, 1834: 25737, 1835: 25608, 1836: 25429, 1837: 25785, 1838: 25685, 1839: 25813, 1840: 25786, 1842: 25861, 1843: 25984, 1844: 26638, 1845: 25945, 1846: 26057, 1847: 25950, 1848: 25869, 1849: 25853, 1850: 26054, 1852: 25930, 1853: 26410, 1854: 26071, 1855: 25923, 1856: 25768, 1857: 26408, 1858: 26064, 1859: 25482, 1860: 26072, 1861: 25980, 1862: 25859, 1863: 26020, 1864: 26093, 1865: 25985, 1866: 26344, 1867: 25700, 1868: 26063, 1869: 26315, 1870: 25887, 1871: 26098, 1872: 26126, 1873: 26106, 1874: 26108, 1875: 26248, 1876: 26176, 1877: 25993, 1878: 26291, 1879: 26207, 1881: 26019, 1882: 25781, 1883: 26215, 1884: 26363, 1885: 28532, 1886: 26197, 1887: 26199, 1888: 26091, 1889: 26332, 1890: 26233, 1891: 26234, 1892: 26237, 1893: 26220, 1895: 26221, 1896: 26224, 1897: 26235, 1898: 26257, 1899: 26241, 1900: 26263, 1901: 26268, 1902: 26396, 1903: 26311, 1904: 26463, 1905: 26382, 1906: 26314, 1907: 26366, 1908: 26386, 1909: 26219, 1910: 26451, 1911: 26345, 1912: 26079, 1913: 26414, 1914: 26536, 1915: 26309, 1916: 26882, 1917: 26001, 1918: 26427, 1919: 26395, 1920: 26487, 1921: 26571, 1922: 26069, 1923: 26477, 1924: 26606, 1925: 26779, 1926: 26412, 1927: 26300, 1928: 26640, 1929: 26616, 1930: 26200, 1931: 26549, 1933: 26535, 1934: 26594, 1935: 26460, 1936: 26190, 1937: 26563, 1938: 26712, 1939: 26718, 1940: 26624, 1941: 26942, 1942: 26623, 1943: 27046, 1944: 26602, 1945: 26781, 1946: 26777, 1947: 26545, 1948: 26727, 1950: 26713, 1951: 26836, 1952: 26736, 1953: 25918, 1954: 26853, 1955: 26762, 1956: 26634, 1957: 26728, 1958: 26649, 1959: 26820, 1960: 26368, 1961: 26964, 1962: 26821, 1963: 26885, 1964: 26169, 1965: 26866, 1966: 26796, 1967: 26926, 1968: 26865, 1969: 27249, 1970: 26953, 1971: 27196, 1972: 26862, 1973: 26868, 1974: 27192, 1975: 26966, 1976: 27472, 1977: 27181, 1978: 27118, 1979: 27319, 1980: 27075, 1981: 26981, 1983: 27072, 1984: 27001, 1985: 27265, 1986: 27212, 1987: 27280, 1988: 27253, 1989: 27316, 1990: 27338, 1991: 26264, 1992: 27592, 1993: 27364, 1994: 27795, 1995: 27483, 1996: 27204, 1997: 27421, 1998: 27288, 1999: 27386, 2000: 27308, 2001: 27341, 2002: 27468, 2003: 27625, 2004: 27366, 2005: 27303, 2006: 27731, 2007: 27435, 2008: 27243, 2009: 27325, 2010: 27511, 2011: 27639, 2012: 27673, 2013: 27629, 2014: 27549, 2015: 27100, 2016: 27581, 2017: 27383, 2018: 27661, 2019: 27560, 2020: 27321, 2021: 27517, 2022: 26394, 2023: 27317, 2024: 27588, 2025: 27777, 2026: 27533, 2027: 27971, 2028: 27778, 2029: 27949, 2030: 27747, 2031: 27658, 2032: 27512, 2033: 27743, 2034: 27830, 2035: 27654, 2036: 27670, 2037: 27750, 2038: 27829, 2039: 27713, 2040: 27628, 2041: 28205, 2042: 27530, 2043: 27705, 2044: 27583, 2045: 28086, 2046: 27973, 2047: 27913, 2048: 27900, 2049: 27621, 2050: 27924, 2051: 27902, 2052: 27965, 2053: 27766, 2054: 28162, 2055: 27787, 2056: 27810, 2057: 27939, 2058: 27929, 2059: 25776, 2060: 27876, 2061: 27989, 2062: 27369, 2063: 28041, 2064: 27534, 2065: 27938, 2066: 28117, 2067: 28066, 2068: 27901, 2069: 27835, 2070: 28011, 2071: 28019, 2072: 27737, 2073: 27660, 2074: 28154, 2075: 28110, 2076: 28139, 2077: 28358, 2078: 28831, 2079: 28385, 2080: 28390, 2081: 28343, 2082: 27955, 2083: 27896, 2084: 28237, 2085: 28103, 2086: 28085, 2087: 28010, 2088: 28360, 2089: 27937, 2090: 28118, 2091: 28404, 2092: 28098, 2093: 28232, 2094: 27947, 2095: 28380, 2096: 28438, 2097: 28252, 2098: 28138, 2099: 28302, 2100: 28271, 2101: 28499, 2102: 27890, 2103: 28296, 2104: 27897, 2105: 28562, 2106: 28199, 2107: 28321, 2108: 28325, 2109: 28370, 2110: 28517, 2111: 28500, 2112: 28637, 2113: 28413, 2114: 28180, 2115: 28642, 2116: 28561, 2117: 28287, 2118: 28434, 2119: 28677, 2120: 28328, 2121: 28829, 2122: 28644, 2123: 28765, 2124: 28614, 2125: 27566, 2126: 28951, 2127: 28609, 2128: 28574, 2129: 28520, 2130: 28691, 2131: 28524, 2132: 28823, 2133: 28686, 2134: 28734, 2135: 28716, 2136: 28622, 2137: 28820, 2138: 28484, 2139: 28861, 2140: 28675, 2141: 28908, 2142: 28744, 2143: 28946, 2144: 28812, 2145: 28814, 2146: 28930, 2147: 29025, 2148: 28816, 2149: 28756, 2150: 28854, 2151: 28596, 2152: 29246, 2153: 29114, 2154: 28949, 2155: 28910, 2156: 28874, 2157: 28764, 2158: 28790, 2159: 29038, 2160: 28855, 2161: 28973, 2162: 28796, 2163: 28943, 2164: 28899, 2165: 29490, 2166: 28984, 2167: 29138, 2168: 29048, 2169: 29196, 2170: 28992, 2171: 28957, 2172: 29404, 2173: 29225, 2174: 29151, 2175: 29388, 2177: 29034, 2178: 28988, 2179: 29210, 2180: 29150, 2181: 29064, 2182: 29178, 2183: 29205, 2184: 29326, 2185: 29379, 2186: 29234, 2187: 29129, 2188: 29569, 2189: 29451, 2190: 29416, 2191: 29371, 2192: 29233, 2193: 29433, 2194: 28909, 2195: 29323, 2196: 28991, 2197: 29450, 2198: 29434, 2199: 29426, 2200: 29294, 2201: 29730, 2202: 29401, 2203: 29263, 2204: 29185, 2205: 29417, 2206: 29347, 2207: 29545, 2208: 29525, 2209: 29997, 2210: 29487, 2211: 29304, 2212: 29276, 2213: 29488, 2214: 29616, 2215: 29919, 2216: 29655, 2217: 29725, 2218: 29575, 2219: 29696, 2220: 29650, 2221: 29134, 2222: 29678, 2223: 29704, 2224: 29629, 2225: 29568, 2226: 29546, 2227: 29651, 2228: 29884, 2229: 29736, 2230: 29789, 2231: 29728, 2232: 29739, 2233: 29716, 2234: 29711, 2235: 29798, 2236: 29746, 2237: 29705, 2238: 30060, 2239: 29949, 2240: 29840, 2241: 29800, 2242: 29679, 2243: 29692, 2244: 29735, 2245: 29353, 2246: 29791, 2247: 29850, 2248: 29839, 2249: 29771, 2250: 29881, 2251: 29860, 2252: 29756, 2253: 29931, 2254: 29843, 2255: 29885, 2256: 29807, 2257: 30272, 2258: 30019, 2259: 29982, 2260: 29895, 2261: 29271, 2262: 29808, 2263: 29842, 2264: 30247, 2265: 29852, 2266: 29941, 2267: 29996, 2268: 29993, 2269: 30099, 2270: 30028, 2271: 30011, 2272: 30200, 2273: 30073, 2274: 29788, 2275: 30093, 2276: 30169, 2277: 30218, 2278: 29913, 2279: 30069, 2280: 30217, 2281: 29990, 2282: 30122, 2283: 29673, 2284: 30214, 2285: 30794, 2286: 30343, 2287: 30318, 2288: 30143, 2289: 30520, 2290: 30104, 2291: 30651, 2292: 30382, 2293: 30679, 2294: 30324, 2295: 30387, 2296: 30277, 2297: 30518, 2298: 30419, 2299: 30422, 2300: 30448, 2301: 30407, 2302: 30517, 2303: 30420, 2304: 30570, 2305: 30457, 2306: 30426, 2307: 30376, 2308: 30564, 2309: 30468, 2310: 30541, 2311: 30436, 2312: 30547, 2313: 30545, 2314: 30762, 2315: 30594, 2316: 30444, 2317: 30595, 2318: 30503, 2319: 30741, 2320: 30342, 2321: 30608, 2323: 30505, 2324: 30666, 2325: 30660, 2326: 30438, 2327: 30692, 2328: 30675, 2329: 30566, 2330: 30769, 2331: 31039, 2332: 30827, 2333: 30728, 2334: 30717, 2335: 30720, 2336: 30423, 2337: 30352, 2338: 30972, 2339: 30880, 2340: 30815, 2342: 30804, 2343: 30883, 2344: 30772, 2345: 30463, 2346: 31922, 2347: 30798, 2348: 30591, 2349: 30711, 2350: 32296, 2351: 30886, 2352: 30321, 2353: 30706, 2354: 30480, 2355: 30906, 2356: 30867, 2359: 30836, 2360: 30524, 2361: 30788, 2362: 30977, 2363: 31940, 2364: 30840, 2365: 31703, 2367: 30986, 2368: 30849, 2369: 30703, 2370: 31066, 2371: 31105, 2372: 31173, 2373: 31024, 2374: 31107, 2375: 31119, 2376: 31359, 2377: 30565, 2378: 31168, 2379: 31084, 2380: 31037, 2381: 31121, 2382: 31159, 2383: 31290, 2384: 30953, 2385: 31216, 2386: 31167, 2387: 31125, 2388: 31072, 2389: 30932, 2390: 31056, 2391: 31277, 2392: 31205, 2393: 31099, 2394: 31676, 2395: 31278, 2396: 31946, 2397: 31190, 2398: 31434, 2399: 31165, 2400: 31079, 2401: 32439, 2402: 31665, 2403: 31266, 2404: 31385, 2405: 31579, 2406: 31448, 2407: 31241, 2408: 30973, 2409: 31446, 2410: 31068, 2411: 31299, 2412: 31137, 2413: 31523, 2414: 31416, 2415: 31362, 2416: 31265, 2417: 31650, 2418: 31583, 2419: 31771, 2420: 31789, 2421: 31681, 2422: 31646, 2423: 31564, 2424: 31457, 2425: 31737, 2426: 31662, 2427: 31832, 2428: 31599, 2429: 31592, 2430: 31672, 2431: 31509, 2432: 31697, 2433: 31593, 2434: 31889, 2435: 31407, 2436: 31813, 2437: 31674, 2438: 31852, 2439: 31850, 2440: 31733, 2441: 31790, 2442: 31766, 2443: 31700, 2444: 31617, 2445: 31603, 2446: 31637, 2447: 31688, 2448: 31758, 2449: 31876, 2450: 31827, 2451: 31685, 2452: 32040, 2453: 32019, 2454: 31945, 2455: 31859, 2456: 31978, 2457: 32020, 2458: 32012, 2459: 32173, 2460: 31870, 2461: 31992, 2462: 31765, 2463: 32261, 2464: 32190, 2465: 31886, 2466: 32104, 2467: 32067, 2468: 31711, 2469: 32064, 2470: 32438, 2471: 32268, 2473: 32246, 2474: 32177, 2475: 32007, 2476: 31966, 2477: 32489, 2478: 32249, 2479: 32226, 2480: 32311, 2481: 32144, 2482: 32091, 2483: 32480, 2484: 32362, 2486: 32609, 2487: 32562, 2488: 32208, 2489: 32404, 2490: 32864, 2491: 32349, 2492: 32292, 2493: 32322, 2494: 32463, 2495: 32368, 2497: 32339, 2498: 32411, 2499: 32539, 2500: 32366, 2501: 32385, 2502: 32474, 2503: 32533, 2504: 32492, 2505: 31977, 2506: 32578, 2507: 32418, 2508: 32558, 2509: 32504, 2510: 32434, 2511: 33104, 2512: 32740, 2513: 32375, 2514: 32617, 2515: 32402, 2516: 32844, 2517: 32682, 2518: 32537, 2519: 32753, 2520: 33048, 2521: 32698, 2522: 32677, 2523: 32494, 2524: 32477, 2525: 32814, 2526: 32531, 2527: 33694, 2528: 32744, 2529: 32921, 2530: 32851, 2531: 32222, 2532: 33056, 2533: 32968, 2534: 32838, 2535: 32809, 2536: 32332, 2537: 32782, 2538: 32759, 2539: 33041, 2540: 33018, 2541: 33064, 2542: 33044, 2543: 32931, 2544: 32827, 2545: 32810, 2546: 32771, 2547: 33133, 2548: 32765, 2549: 32855, 2550: 32607, 2551: 33024, 2552: 32990, 2553: 32768, 2554: 32761, 2555: 33082, 2556: 33271, 2557: 33269, 2558: 32938, 2559: 31897, 2560: 33449, 2561: 33444, 2562: 32775, 2563: 32918, 2564: 33202, 2565: 33077, 2566: 33079, 2567: 33040, 2568: 33377, 2569: 33277, 2570: 33154, 2571: 33092, 2572: 33184, 2573: 33415, 2574: 33160, 2575: 33006, 2576: 33094, 2577: 33210, 2578: 33126, 2579: 33015, 2580: 33152, 2581: 33827, 2582: 33243, 2583: 33165, 2584: 33297, 2585: 33485, 2586: 33421, 2587: 33016, 2588: 33248, 2589: 33372, 2590: 33302, 2591: 33189, 2592: 33012, 2593: 33345, 2594: 33139, 2595: 33316, 2596: 33347, 2597: 33465, 2598: 33330, 2599: 33420, 2600: 33614, 2601: 33595, 2602: 32912, 2603: 33447, 2604: 33394, 2605: 33650, 2606: 33584, 2607: 33478, 2608: 33357, 2609: 37391, 2610: 33603, 2611: 33492, 2612: 33451, 2613: 33644, 2614: 33532, 2615: 33715, 2616: 33575, 2617: 34238, 2618: 33579, 2619: 33558, 2620: 33823, 2621: 33591, 2622: 33719, 2623: 33666, 2624: 33729, 2625: 33703, 2626: 33590, 2627: 33754, 2628: 33721, 2629: 33854, 2630: 33927, 2631: 33929, 2632: 33914, 2633: 33875, 2634: 33588, 2635: 33937, 2636: 33873, 2637: 33760, 2638: 33577, 2639: 33878, 2640: 33804, 2641: 33774, 2642: 34250, 2643: 34017, 2644: 34217, 2645: 34168, 2646: 33856, 2647: 34002, 2648: 33971, 2649: 34033, 2650: 34088, 2651: 34055, 2652: 33779, 2653: 33977, 2654: 34107, 2655: 34086, 2656: 34066, 2657: 34045, 2658: 33909, 2659: 34182, 2660: 34267, 2661: 33800, 2662: 33682, 2663: 34215, 2664: 34142, 2665: 34358, 2666: 34081, 2667: 34065, 2668: 34069, 2669: 34367, 2670: 34234, 2671: 34356, 2672: 34059, 2673: 34428, 2674: 34000, 2675: 34462, 2676: 34338, 2677: 34200, 2678: 34301, 2679: 34297, 2680: 34248, 2681: 34956, 2682: 34387, 2683: 34105, 2684: 34440, 2685: 34253, 2686: 34318, 2687: 34177, 2688: 34331, 2689: 33384, 2690: 34360, 2691: 34339, 2692: 34608, 2693: 34444, 2694: 34536, 2695: 34489, 2696: 34752, 2697: 34693, 2698: 34349, 2699: 34561, 2700: 34722, 2701: 34622, 2702: 34495, 2703: 34912, 2704: 34579, 2705: 34617, 2706: 34819, 2707: 34724, 2708: 34624, 2709: 36019, 2710: 34768, 2711: 34860, 2712: 34270, 2713: 34789, 2714: 34769, 2715: 35146, 2716: 34758, 2717: 34909, 2718: 34798, 2719: 34670, 2720: 34782, 2721: 35136, 2722: 34995, 2723: 34888, 2724: 34814, 2725: 35025, 2726: 34817, 2727: 34802, 2728: 35005, 2729: 34987, 2730: 34914, 2731: 34975, 2732: 34982, 2733: 34940, 2734: 34924, 2735: 34473, 2736: 34481, 2737: 35304, 2738: 35152, 2739: 34999, 2740: 34834, 2741: 34937, 2742: 36547, 2743: 34954, 2744: 35080, 2745: 34981, 2746: 34899, 2747: 35120, 2748: 34922, 2749: 35037, 2750: 35044, 2751: 35384, 2752: 35127, 2753: 35341, 2754: 34780, 2755: 35132, 2756: 35083, 2757: 35345, 2758: 35180, 2759: 35054, 2760: 35278, 2761: 35029, 2762: 35020, 2763: 35350, 2764: 35210, 2765: 35301, 2766: 35205, 2767: 35084, 2768: 35241, 2769: 35202, 2770: 35226, 2771: 35181, 2772: 35735, 2773: 35264, 2774: 35326, 2775: 35623, 2776: 35643, 2777: 35550, 2778: 35476, 2779: 35509, 2780: 35548, 2781: 35412, 2782: 35415, 2783: 35783, 2784: 35785, 2785: 35442, 2786: 35427, 2787: 35363, 2788: 35487, 2789: 35347, 2790: 35406, 2791: 35393, 2792: 35725, 2793: 35710, 2794: 35463, 2795: 35699, 2796: 35615, 2797: 36858, 2798: 35642, 2799: 35609, 2800: 35611, 2801: 35712, 2802: 35626, 2803: 35228, 2804: 35984, 2805: 35907, 2806: 35707, 2807: 35737, 2808: 35846, 2809: 36211, 2810: 35842, 2811: 35749, 2812: 35727, 2813: 35564, 2815: 35589, 2816: 35941, 2817: 35933, 2818: 36145, 2819: 35795, 2820: 35987, 2821: 36046, 2822: 35848, 2823: 35855, 2824: 35887, 2825: 35951, 2826: 35920, 2827: 35904, 2828: 36041, 2829: 35893, 2830: 36528, 2831: 35975, 2832: 35998, 2833: 36055, 2834: 35957, 2835: 36152, 2836: 36123, 2837: 36156, 2838: 36080, 2839: 36017, 2840: 36141, 2841: 36024, 2842: 35960, 2844: 36348, 2845: 36188, 2846: 36238, 2847: 36045, 2848: 32500, 2849: 36439, 2850: 36140, 2851: 36265, 2852: 36366, 2853: 36186, 2854: 36284, 2855: 36168, 2856: 36143, 2857: 36393, 2858: 36340, 2859: 36251, 2860: 36236, 2861: 36429, 2862: 36114, 2863: 36258, 2864: 36425, 2865: 36388, 2866: 36399, 2867: 36396, 2868: 36395, 2869: 36304, 2870: 36345, 2872: 36624, 2873: 36362, 2874: 36431, 2875: 36363, 2876: 36526, 2877: 36616, 2878: 36377, 2879: 36691, 2880: 36641, 2881: 36514, 2882: 36515, 2883: 36640, 2884: 36444, 2885: 36582, 2886: 36760, 2887: 36723, 2888: 36346, 2889: 36608, 2891: 36850, 2892: 36496, 2893: 36796, 2894: 37046, 2895: 36657, 2896: 36896, 2897: 36736, 2898: 36965, 2899: 36732, 2900: 36721, 2901: 36812, 2902: 36773, 2903: 37023, 2904: 36869, 2905: 36962, 2906: 36795, 2907: 36728, 2908: 36729, 2909: 36817, 2911: 36778, 2912: 36837, 2913: 36807, 2914: 37140, 2915: 37091, 2916: 36848, 2917: 36787, 2918: 37031, 2919: 36039, 2920: 37000, 2921: 36981, 2922: 36917, 2923: 36986, 2924: 37204, 2925: 36856, 2926: 37165, 2927: 37088, 2928: 37036, 2929: 37406, 2930: 37265, 2931: 37269, 2932: 37119, 2933: 37089, 2934: 36942, 2935: 37369, 2936: 37339, 2937: 37096, 2938: 37300, 2939: 37441, 2940: 37043, 2941: 36977, 2942: 37127, 2943: 37279, 2944: 37173, 2945: 37128, 2946: 37609, 2947: 37261, 2948: 37229, 2951: 37428, 2952: 37184, 2953: 37404, 2954: 37223, 2955: 37202, 2956: 37293, 2957: 37174, 2958: 37394, 2959: 37379, 2960: 37364, 2961: 37297, 2962: 37580, 2963: 37322, 2964: 37329, 2965: 37508, 2966: 37478, 2967: 37521, 2968: 37345, 2969: 37701, 2970: 37447, 2971: 37248, 2972: 37399, 2973: 37629, 2974: 37415, 2975: 37949, 2976: 37465, 2977: 37934, 2978: 37636, 2979: 36914, 2981: 37450, 2982: 37614, 2983: 37704, 2984: 37461, 2985: 37740, 2986: 37514, 2987: 37723, 2988: 37590, 2989: 37705, 2990: 37826, 2991: 37811, 2992: 37660, 2993: 37648, 2994: 37623, 2995: 37616, 2996: 37677, 2997: 38784, 2998: 37606, 2999: 37946, 3000: 36982, 3001: 37666, 3002: 37664, 3003: 37908, 3004: 37751, 3005: 37617, 3006: 37530, 3007: 37710, 3008: 37921, 3009: 37842, 3010: 37843, 3011: 37752, 3012: 37599, 3013: 38016, 3014: 37901, 3015: 37891, 3016: 37803, 3017: 37819, 3018: 37853, 3019: 37951, 3020: 37823, 3021: 38106, 3022: 37915, 3023: 37995, 3024: 37504, 3025: 37938, 3026: 38031, 3027: 38037, 3028: 38325, 3029: 38048, 3030: 38147, 3031: 37854, 3032: 37982, 3033: 38140, 3034: 38070, 3035: 38010, 3036: 37773, 3037: 38020, 3038: 37720, 3039: 38449, 3040: 38319, 3041: 38074, 3042: 38184, 3043: 38146, 3044: 38211, 3045: 38170, 3046: 38089, 3047: 38253, 3048: 38216, 3049: 38167, 3050: 38300, 3051: 38246, 3052: 38200, 3053: 38394, 3054: 38307, 3055: 38164, 3056: 38087, 3057: 38183, 3058: 38159, 3059: 38373, 3060: 38316, 3061: 38406, 3062: 38152, 3063: 38372, 3064: 38382, 3065: 38623, 3066: 38639, 3067: 38538, 3068: 38375, 3069: 38292, 3070: 38160, 3071: 38267, 3072: 38474, 3073: 38427, 3074: 38355, 3075: 39117, 3076: 38226, 3077: 38800, 3078: 38370, 3079: 38423, 3080: 38414, 3081: 38210, 3082: 39538, 3083: 38723, 3084: 38455, 3085: 38497, 3086: 38722, 3087: 38712, 3088: 38438, 3089: 38500, 3090: 38518, 3091: 38593, 3092: 38635, 3093: 38794, 3094: 38959, 3095: 38848, 3096: 38686, 3097: 38868, 3098: 38870, 3099: 38792, 3100: 38667, 3101: 38732, 3102: 38835, 3103: 38945, 3104: 38975, 3105: 38656, 3106: 39221, 3107: 38795, 3108: 40559, 3109: 39261, 3110: 38962, 3111: 38845, 3112: 39340, 3113: 38901, 3114: 38846, 3115: 39067, 3116: 38872, 3117: 38827, 3118: 38879, 3119: 39348, 3120: 38783, 3121: 38917, 3122: 39079, 3123: 39023, 3124: 39191, 3125: 39180, 3126: 38834, 3127: 39194, 3128: 39177, 3129: 38957, 3130: 39279, 3131: 39095, 3132: 39263, 3133: 38963, 3134: 39236, 3135: 39172, 3136: 39213, 3137: 39014, 3138: 38908, 3139: 38863, 3140: 39061, 3141: 39211, 3142: 39035, 3143: 39038, 3144: 39271, 3145: 39311, 3146: 39137, 3147: 38994, 3148: 39122, 3149: 39424, 3150: 39326, 3151: 39153, 3152: 39073, 3153: 39070, 3154: 39251, 3155: 39299, 3156: 39184, 3157: 39225, 3158: 39535, 3159: 39138, 3160: 39380, 3161: 39238, 3162: 39360, 3163: 39567, 3164: 39607, 3165: 39429, 3166: 39420, 3167: 39722, 3168: 39524, 3169: 39659, 3170: 39487, 3171: 39041, 3172: 39625, 3173: 39847, 3174: 39675, 3175: 39995, 3176: 39780, 3177: 39617, 3178: 39527, 3179: 39584, 3180: 39566, 3181: 39938, 3182: 40215, 3183: 39734, 3184: 39874, 3185: 39757, 3186: 39530, 3187: 39690, 3188: 39863, 3189: 39898, 3190: 39876, 3191: 40023, 3192: 39906, 3193: 40093, 3194: 39943, 3195: 39866, 3196: 39914, 3197: 41208, 3198: 40058, 3199: 39920, 3200: 40305, 3201: 40085, 3202: 40035, 3203: 39919, 3204: 39961, 3205: 39974, 3207: 39953, 3210: 40167, 3211: 40084, 3212: 40107, 3213: 39970, 3214: 40177, 3215: 40240, 3216: 40793, 3217: 39843, 3218: 39957, 3219: 40063, 3220: 39903, 3221: 40474, 3222: 40231, 3223: 39794, 3224: 40293, 3225: 40091, 3226: 40096, 3227: 40077, 3228: 40342, 3229: 40259, 3230: 40211, 3231: 40354, 3232: 40155, 3233: 40220, 3234: 40183, 3235: 40646, 3236: 40889, 3237: 40274, 3238: 40291, 3239: 40341, 3240: 40321, 3241: 40324, 3242: 40344, 3243: 40326, 3244: 40285, 3245: 40772, 3246: 40677, 3247: 40282, 3248: 40534, 3249: 40526, 3250: 40357, 3251: 40476, 3252: 40617, 3253: 40485, 3254: 40875, 3255: 40561, 3256: 40433, 3257: 40604, 3258: 40878, 3259: 40693, 3260: 40429, 3261: 40656, 3262: 40843, 3263: 41060, 3264: 40866, 3265: 40766, 3266: 40678, 3267: 40673, 3268: 40881, 3269: 40818, 3270: 40706, 3271: 40858, 3272: 40859, 3273: 40787, 3274: 40655, 3275: 41075, 3276: 40876, 3277: 41152, 3278: 40941, 3279: 40944, 3280: 40680, 3281: 40990, 3282: 40945, 3283: 40943, 3284: 41117, 3285: 41036, 3286: 40947, 3287: 41224, 3288: 41080, 3289: 41067, 3290: 41163, 3291: 41074, 3292: 41262, 3293: 40932, 3294: 41039, 3295: 41214, 3296: 41107, 3297: 41211, 3298: 40926, 3299: 41319, 3300: 41081, 3301: 40817, 3302: 40834, 3303: 41676, 3304: 41377, 3305: 41299, 3306: 41325, 3307: 41037, 3308: 41242, 3309: 41484, 3310: 41404, 3312: 41389, 3314: 41307, 3315: 41260, 3316: 41282, 3317: 41306, 3318: 40702, 3319: 41400, 3320: 41328, 3321: 41375, 3322: 41250, 3323: 41704, 3324: 41395, 3325: 41452, 3326: 41323, 3327: 41361, 3329: 41574, 3330: 41296, 3331: 41413, 3332: 41927, 3333: 41578, 3334: 41003, 3335: 41475, 3336: 41449, 3337: 41564, 3338: 41547, 3339: 41504, 3340: 40888, 3341: 41373, 3342: 41597, 3343: 41515, 3344: 41572, 3345: 41603, 3346: 41321, 3347: 41312, 3348: 41798, 3349: 41464, 3350: 41483, 3351: 41894, 3352: 42249, 3353: 41640, 3354: 42080, 3355: 41816, 3356: 41621, 3357: 41822, 3358: 41616, 3359: 41639, 3360: 41935, 3361: 41843, 3362: 41723, 3363: 41674, 3364: 41726, 3365: 41975, 3366: 41909, 3367: 41817, 3368: 41653, 3369: 41940, 3370: 41451, 3371: 41737, 3372: 41904, 3373: 41806, 3374: 41893, 3375: 41781, 3376: 42010, 3377: 42090, 3378: 42008, 3379: 42484, 3380: 42049, 3381: 41949, 3382: 41813, 3383: 42028, 3384: 41926, 3385: 41939, 3386: 41861, 3387: 42133, 3388: 41970, 3389: 42001, 3390: 41986, 3391: 42438, 3392: 42142, 3393: 41191, 3394: 42187, 3395: 42172, 3396: 42173, 3397: 42075, 3398: 42146, 3399: 42070, 3400: 42372, 3401: 42434, 3402: 42147, 3403: 42527, 3404: 42123, 3405: 42415, 3406: 42265, 3407: 42088, 3408: 42452, 3409: 42365, 3410: 42313, 3411: 42299, 3412: 42353, 3413: 42177, 3414: 42134, 3415: 42129, 3416: 42368, 3417: 41907, 3418: 42402, 3419: 42302, 3420: 42334, 3421: 42291, 3422: 42604, 3423: 42538, 3424: 42462, 3425: 42394, 3426: 42312, 3427: 42516, 3428: 42549, 3429: 42556, 3430: 42430, 3431: 42509, 3432: 42286, 3433: 42483, 3434: 42455, 3435: 42400, 3436: 42777, 3437: 42606, 3438: 42515, 3439: 42540, 3440: 42459, 3441: 42662, 3442: 42504, 3443: 42440, 3444: 42564, 3445: 42570, 3446: 42701, 3447: 42536, 3448: 42535, 3449: 42806, 3450: 42795, 3451: 42871, 3452: 42624, 3453: 42614, 3454: 42799, 3455: 42579, 3456: 42679, 3457: 42568, 3458: 42854, 3459: 42835, 3460: 42425, 3461: 42911, 3462: 42712, 3463: 42775, 3464: 42954, 3465: 42917, 3466: 42715, 3467: 42726, 3468: 42828, 3469: 42931, 3470: 43266, 3471: 42717, 3472: 42951, 3473: 42928, 3474: 43100, 3475: 43103, 3476: 42834, 3477: 42884, 3478: 43026, 3479: 42923, 3480: 43035, 3481: 43121, 3482: 43109, 3483: 43002, 3484: 43067, 3485: 42913, 3486: 43142, 3487: 43023, 3488: 43073, 3489: 42936, 3490: 43114, 3491: 42850, 3492: 43234, 3493: 43204, 3494: 43082, 3495: 42895, 3496: 43148, 3497: 43177, 3498: 43105, 3499: 43410, 3500: 43305, 3501: 43209, 3502: 42637, 3503: 43195, 3504: 43427, 3505: 43644, 3506: 43624, 3507: 43338, 3508: 43531, 3509: 43553, 3510: 43454, 3511: 43550, 3512: 43352, 3513: 43370, 3514: 43325, 3516: 43394, 3517: 43354, 3518: 43409, 3519: 43584, 3520: 43347, 3521: 43575, 3522: 43587, 3523: 43496, 3524: 42794, 3525: 43392, 3526: 43570, 3527: 43413, 3528: 43685, 3529: 43580, 3530: 43486, 3531: 43903, 3532: 43721, 3533: 43573, 3534: 43590, 3535: 43603, 3536: 43499, 3537: 43414, 3538: 43726, 3539: 43589, 3540: 43834, 3541: 43811, 3542: 43593, 3543: 43012, 3544: 43351, 3545: 43923, 3546: 43894, 3547: 43813, 3548: 43673, 3549: 43620, 3550: 43851, 3551: 43671, 3552: 43822, 3554: 43798, 3555: 43932, 3556: 43825, 3557: 43902, 3558: 43957, 3559: 43853, 3560: 43669, 3561: 43970, 3562: 43807, 3563: 44031, 3564: 43899, 3565: 44001, 3566: 44064, 3567: 44010, 3568: 43763, 3569: 44127, 3570: 43797, 3571: 43783, 3572: 44066, 3573: 44042, 3574: 43878, 3575: 44154, 3576: 44390, 3577: 44126, 3578: 44075, 3579: 44248, 3580: 44231, 3581: 45421, 3582: 43937, 3583: 44024, 3584: 44162, 3585: 44130, 3586: 44331, 3587: 44307, 3588: 44093, 3589: 44342, 3590: 44315, 3591: 44191, 3592: 44504, 3593: 44213, 3594: 44471, 3595: 44405, 3596: 44356, 3597: 44290, 3598: 44143, 3599: 44406, 3600: 44299, 3601: 44512, 3602: 44367, 3603: 44613, 3604: 44256, 3605: 44337, 3606: 44574, 3607: 44431, 3608: 44717, 3609: 44857, 3610: 44283, 3611: 44423, 3612: 44700, 3613: 44659, 3614: 44511, 3615: 44382, 3616: 45038, 3617: 44768, 3618: 44738, 3619: 44901, 3620: 44825, 3621: 44818, 3622: 44565, 3623: 44798, 3624: 45075, 3625: 44897, 3626: 44892, 3627: 44946, 3628: 44824, 3629: 44708, 3630: 44883, 3631: 44753, 3632: 44665, 3633: 45306, 3634: 44816, 3635: 44984, 3636: 44936, 3637: 44887, 3638: 44923, 3639: 45058, 3640: 45033, 3641: 44961, 3642: 44626, 3643: 44599, 3644: 45001, 3645: 45461, 3646: 45037, 3647: 44979, 3648: 45333, 3649: 45150, 3650: 45170, 3651: 45167, 3652: 45290, 3653: 45158, 3654: 45085, 3655: 45184, 3656: 45127, 3657: 45272, 3658: 45122, 3659: 45080, 3660: 45455, 3661: 45189, 3662: 45493, 3663: 45101, 3664: 45412, 3665: 45336, 3666: 45764, 3667: 45259, 3668: 45242, 3669: 45410, 3670: 45270, 3672: 45314, 3673: 45219, 3674: 45344, 3675: 45424, 3676: 45590, 3677: 45386, 3678: 43908, 3679: 45328, 3680: 45413, 3681: 45527, 3682: 45439, 3683: 45526, 3684: 45448, 3685: 45238, 3686: 45661, 3687: 45559, 3688: 45481, 3689: 45614, 3690: 45688, 3691: 45418, 3692: 45505, 3693: 45437, 3694: 45544, 3695: 45166, 3696: 45496, 3697: 45836, 3698: 45915, 3699: 45556, 3700: 45603, 3701: 45858, 3702: 45744, 3703: 45631, 3704: 45743, 3705: 45860, 3706: 45751, 3707: 45896, 3708: 45675, 3709: 45811, 3710: 45754, 3711: 45874, 3712: 45571, 3713: 45615, 3714: 45854, 3715: 45837, 3716: 45814, 3717: 45742, 3718: 45902, 3719: 46410, 3720: 45581, 3721: 45585, 3722: 46247, 3723: 46058, 3724: 45999, 3725: 46168, 3726: 45924, 3727: 46125, 3728: 45856, 3729: 45980, 3730: 45962, 3731: 46146, 3732: 45920, 3733: 46026, 3734: 45941, 3735: 46075, 3736: 46232, 3737: 46114, 3738: 46221, 3739: 46093, 3740: 46049, 3741: 46288, 3742: 46101, 3743: 46471, 3744: 46365, 3745: 46329, 3746: 46297, 3747: 46566, 3748: 46390, 3749: 46371, 3750: 46404, 3751: 47193, 3752: 46225, 3753: 46283, 3754: 46454, 3755: 46457, 3756: 46396, 3757: 46733, 3758: 46504, 3759: 46509, 3760: 46529, 3761: 46328, 3762: 46543, 3763: 46511, 3764: 46652, 3765: 46515, 3766: 46517, 3767: 46569, 3768: 47013, 3769: 46735, 3770: 46578, 3771: 46977, 3772: 46618, 3773: 46750, 3774: 47119, 3775: 46853, 3776: 46482, 3777: 46358, 3778: 46873, 3779: 46774, 3780: 46657, 3782: 46771, 3783: 46460, 3784: 46594, 3785: 46744, 3786: 46651, 3787: 46776, 3788: 46768, 3789: 46734, 3790: 46736, 3791: 46904, 3792: 46891, 3793: 46620, 3794: 46840, 3795: 46107, 3796: 46813, 3797: 46963, 3798: 46810, 3799: 47006, 3800: 46952, 3801: 46869, 3802: 46859, 3803: 46701, 3804: 46938, 3805: 46893, 3806: 47260, 3807: 46811, 3808: 46880, 3809: 47029, 3810: 46897, 3811: 47053, 3812: 46874, 3813: 46737, 3814: 46982, 3815: 47080, 3816: 46806, 3817: 46914, 3818: 47096, 3819: 46950, 3820: 47168, 3821: 46741, 3822: 47070, 3823: 47039, 3824: 47401, 3825: 46974, 3826: 47189, 3827: 47205, 3828: 47159, 3829: 47300, 3830: 47187, 3831: 47145, 3832: 47249, 3833: 47199, 3834: 47310, 3835: 47224, 3836: 47175, 3837: 47172, 3838: 47594, 3839: 47654, 3840: 47204, 3841: 47115, 3842: 47267, 3843: 47884, 3844: 47328, 3845: 47431, 3846: 47427, 3847: 48017, 3848: 47454, 3849: 47452, 3850: 47544, 3851: 47570, 3852: 47508, 3853: 47550, 3854: 47633, 3855: 47664, 3856: 47391, 3857: 47631, 3858: 47522, 3859: 47791, 3860: 46928, 3861: 47701, 3862: 47592, 3863: 47479, 3864: 47498, 3865: 47911, 3866: 47723, 3867: 47627, 3868: 47559, 3869: 47779, 3870: 47965, 3871: 47758, 3872: 47694, 3873: 47908, 3874: 47762, 3875: 47717, 3876: 47943, 3877: 47959, 3878: 47868, 3879: 47960, 3880: 48029, 3881: 48113, 3882: 48036, 3883: 47893, 3884: 47854, 3885: 48266, 3886: 47963, 3887: 47913, 3888: 48319, 3889: 48218, 3890: 48002, 3892: 48191, 3893: 48273, 3894: 48402, 3895: 48119, 3896: 48324, 3897: 48219, 3898: 48224, 3899: 48341, 3900: 48390, 3901: 48351, 3902: 47956, 3903: 48356, 3904: 48287, 3905: 48455, 3906: 48414, 3907: 48413, 3908: 48396, 3909: 48437, 3910: 48348, 3911: 48638, 3912: 48374, 3913: 48339, 3914: 48310, 3915: 48519, 3916: 48468, 3917: 48682, 3918: 48893, 3919: 48559, 3920: 48469, 3921: 48584, 3922: 48802, 3923: 48615, 3924: 48527, 3925: 48561, 3926: 48734, 3927: 48613, 3928: 48833, 3929: 48861, 3930: 48706, 3931: 48763, 3932: 48748, 3933: 48776, 3934: 49688, 3935: 48730, 3936: 48895, 3937: 48883, 3938: 48876, 3939: 49005, 3940: 48774, 3941: 48799, 3942: 48982, 3943: 48835, 3944: 48715, 3945: 48990, 3946: 48943, 3947: 48926, 3948: 48806, 3949: 48782, 3950: 49029, 3951: 49081, 3952: 49220, 3953: 49052, 3954: 49363, 3955: 49137, 3956: 49222, 3957: 49160, 3958: 49408, 3959: 49293, 3960: 49164, 3961: 49329, 3962: 49294, 3963: 49321, 3964: 49259, 3965: 49339, 3966: 49233, 3967: 49223, 3968: 49343, 3969: 49445, 3970: 49402, 3971: 49281, 3972: 49418, 3973: 49530, 3974: 49593, 3975: 49583, 3976: 49485, 3977: 49569, 3978: 49477, 3979: 49658, 3980: 49637, 3981: 49641, 3982: 49669, 3983: 49065, 3984: 49645, 3985: 49701, 3986: 49689, 3987: 49870, 3988: 49802, 3989: 49812, 3990: 49712, 3991: 49809, 3992: 49769, 3993: 49893, 3994: 49841, 3995: 49698, 3996: 49865, 3997: 49326, 3998: 49929, 3999: 49751, 4000: 49900, 4001: 49844, 4002: 49764, 4003: 49967, 4004: 50027, 4005: 50013, 4006: 50109, 4007: 49926, 4008: 50222, 4009: 49934, 4010: 49994, 4011: 50066, 4012: 50174, 4013: 50075, 4014: 50218, 4015: 50103, 4016: 50447, 4017: 50070, 4018: 50044, 4019: 50122, 4020: 50078, 4021: 50433, 4022: 50067, 4023: 50191, 4024: 50303, 4025: 50083, 4026: 50448, 4027: 50316, 4028: 50305, 4029: 50234, 4030: 50319, 4031: 50335, 4032: 50336, 4033: 50372, 4034: 50292, 4035: 50333, 4036: 50241, 4037: 50099, 4038: 50232, 4039: 50384, 4040: 50340, 4041: 50459, 4042: 50414, 4043: 50287, 4044: 50509, 4045: 50332, 4046: 50546, 4047: 50685, 4048: 50516, 4049: 50456, 4050: 50371, 4051: 50606, 4052: 50635, 4053: 50492, 4054: 50564, 4055: 50536, 4056: 50480, 4057: 50583, 4059: 50552, 4060: 50584, 4061: 50493, 4062: 51384, 4063: 50555, 4064: 50684, 4065: 50520, 4066: 50609, 4067: 50786, 4068: 50693, 4069: 50801, 4070: 50755, 4071: 50728, 4072: 50933, 4073: 50739, 4074: 50676, 4075: 50860, 4076: 50790, 4077: 50851, 4078: 50904, 4079: 50870, 4080: 50799, 4081: 50935, 4082: 50885, 4083: 50868, 4084: 51502, 4085: 50939, 4086: 50888, 4087: 50903, 4088: 51008, 4089: 50847, 4090: 51056, 4091: 50916, 4092: 51046, 4093: 51014, 4094: 51069, 4095: 50993, 4096: 51200, 4097: 51161, 4098: 51248, 4099: 51077, 4100: 51233, 4101: 51213, 4102: 50954, 4103: 51290, 4104: 51172, 4105: 50976, 4106: 51401, 4107: 51140, 4108: 51448, 4109: 51302, 4110: 51192, 4111: 51245, 4112: 51459, 4113: 51420, 4114: 51232, 4115: 51194, 4116: 51362, 4117: 51364, 4118: 51376, 4119: 51437, 4120: 51313, 4121: 51883, 4122: 51490, 4123: 51491, 4124: 51556, 4125: 51475, 4126: 51808, 4127: 51585, 4128: 51461, 4129: 51425, 4130: 51551, 4131: 51697, 4132: 51658, 4133: 51624, 4134: 51523, 4135: 51561, 4136: 51560, 4137: 51685, 4138: 51438, 4139: 51610, 4140: 51576, 4141: 51814, 4142: 51495, 4143: 51635, 4144: 51623, 4145: 51718, 4146: 51775, 4147: 51676, 4148: 51802, 4149: 51795, 4150: 51914, 4151: 51732, 4152: 51852, 4153: 51821, 4154: 51816, 4155: 51907, 4156: 51905, 4157: 51885, 4158: 51933, 4159: 51849, 4160: 51974, 4161: 51557, 4162: 51979, 4163: 52009, 4164: 51912, 4165: 52136, 4166: 52098, 4167: 51986, 4168: 52139, 4169: 52004, 4170: 51835, 4171: 52085, 4172: 52113, 4173: 52043, 4174: 51839, 4175: 52112, 4176: 52338, 4177: 52102, 4178: 52353, 4179: 52127, 4180: 52154, 4181: 52425, 4182: 52316, 4183: 52273, 4184: 52366, 4185: 52221, 4186: 52150, 4187: 52478, 4188: 52308, 4189: 52422, 4190: 52391, 4191: 52469, 4192: 52457, 4193: 52452, 4194: 52407, 4195: 52577, 4196: 52370, 4197: 52513, 4198: 52405, 4199: 52419, 4200: 52468, 4201: 52584, 4202: 52685, 4203: 52638, 4204: 52487, 4205: 52502, 4206: 52340, 4207: 52660, 4208: 52686, 4209: 52689, 4211: 52520, 4212: 52537, 4213: 52535, 4214: 52737, 4215: 52877, 4216: 52727, 4217: 52679, 4218: 52795, 4219: 52678, 4220: 52701, 4221: 52742, 4222: 52736, 4223: 52882, 4224: 52863, 4225: 52841, 4226: 52797, 4227: 52911, 4228: 52827, 4229: 52913, 4230: 52959, 4231: 52595, 4232: 52943, 4233: 52948, 4234: 52633, 4235: 53043, 4236: 53064, 4237: 52980, 4238: 52965, 4239: 52922, 4240: 53035, 4241: 53155, 4242: 53157, 4243: 53257, 4244: 53141, 4245: 53167, 4246: 53261, 4247: 53229, 4248: 53295, 4249: 53240, 4250: 53154, 4251: 53252, 4252: 53259, 4253: 53273, 4254: 53355, 4255: 53316, 4256: 53377, 4257: 53253, 4258: 53426, 4259: 53417, 4261: 53387, 4262: 53272, 4263: 53379, 4264: 53465, 4265: 53423, 4266: 53334, 4267: 53449, 4268: 53151, 4269: 53472, 4270: 53492, 4271: 53394, 4272: 53761, 4273: 53502, 4274: 53530, 4275: 53706, 4276: 53589, 4277: 53721, 4278: 53726, 4279: 53546, 4280: 53781, 4281: 53737, 4282: 53699, 4283: 53798, 4284: 53723, 4285: 53791, 4286: 53860, 4287: 53740, 4288: 53838, 4289: 53778, 4290: 53701, 4291: 53807, 4292: 53762, 4293: 53773, 4294: 53824, 4295: 53910, 4296: 53771, 4297: 53849, 4298: 53818, 4299: 53907, 4300: 53954, 4301: 54061, 4302: 53963, 4303: 54027, 4304: 53700, 4305: 54029, 4306: 54049, 4307: 54030, 4308: 54048, 4309: 54136, 4310: 54182, 4311: 54137, 4312: 53702, 4313: 54173, 4314: 54204, 4315: 54214, 4316: 54185, 4317: 54255, 4318: 54264, 4319: 54336, 4320: 54311, 4321: 54291, 4322: 54388, 4323: 54294, 4324: 54349, 4325: 54301, 4326: 54289, 4327: 54360, 4328: 54400, 4329: 54327, 4330: 54540, 4331: 54430, 4332: 54487, 4333: 54522, 4334: 54477, 4335: 54539, 4336: 54537, 4337: 54463, 4338: 54461, 4339: 54561, 4340: 54721, 4341: 54688, 4342: 54630, 4343: 54682, 4344: 54765, 4345: 54745, 4346: 54725, 4347: 54742, 4348: 54749, 4349: 54673, 4350: 54746, 4351: 54842, 4352: 54751, 4353: 54783, 4354: 54811, 4355: 54767, 4356: 54849, 4357: 54872, 4358: 54863, 4359: 54879, 4360: 54840, 4361: 54829, 4362: 54951, 4363: 55044, 4364: 54929, 4365: 55016, 4366: 55033, 4367: 55086, 4368: 55084, 4369: 55106, 4370: 55068, 4371: 55137, 4372: 55130, 4373: 55164, 4374: 0, 4375: 0, 4376: 55168, 4377: 55219, 4378: 55209, 4379: 55140, 4380: 55266, 4381: 55249, 4382: 55282, 4383: 55412, 4384: 55280, 4385: 55225, 4386: 55434, 4387: 55308, 4388: 55485, 4389: 55350, 4390: 55425, 4391: 55564, 4392: 55560, 4393: 55528, 4394: 55595, 4395: 55598, 4396: 55588, 4397: 55497, 4398: 55581, 4399: 55642, 4400: 55650, 4401: 55597, 4402: 55687, 4403: 55667, 4404: 55716, 4405: 55705, 4406: 55657, 4407: 55797, 4408: 55765, 4409: 55756, 4410: 55791, 4411: 55763, 4412: 55821, 4413: 55779, 4414: 55846, 4415: 55831, 4416: 55874, 4417: 55849, 4418: 55945, 4419: 55941, 4420: 55953, 4421: 56035, 4422: 56034, 4423: 56000, 4424: 56083, 4425: 55979, 4426: 56080, 4427: 56120, 4428: 56078, 4429: 56253, 4430: 56135, 4431: 56148, 4432: 56127, 4433: 56146, 4434: 56211, 4435: 56170, 4436: 56194, 4437: 56242, 4438: 56201, 4439: 56290, 4440: 56245, 4441: 56243, 4442: 56250, 4443: 56280, 4445: 56293, 4446: 56318, 4447: 56319, 4448: 56287, 4449: 56332, 4450: 56343, 4451: 56364, 4452: 56410, 4453: 56391, 4454: 56429, 4455: 56445, 4456: 56473, 4457: 56510, 4458: 56452, 4459: 56508, 4460: 56480, 4461: 56583, 4462: 56497, 4463: 56518, 4464: 56553, 4465: 56601, 4466: 56573, 4467: 56561, 4468: 56633, 4469: 56620, 4470: 56632, 4471: 56647, 4472: 56606, 4473: 56657, 4474: 56731, 4475: 56656, 4476: 56700, 4477: 56770, 4478: 56756, 4479: 56675, 4480: 56789, 4481: 56816, 4482: 56784, 4483: 56779, 4484: 56775, 4485: 56727, 4486: 56809, 4487: 56754, 4488: 56802, 4489: 56830, 4490: 56901, 4491: 56899, 4492: 56862, 4493: 56944, 4494: 56922, 4495: 56975, 4496: 56997, 4497: 56970, 4498: 57001, 4499: 56986, 4500: 57045, 4501: 57029, 4502: 57013, 4503: 57047, 4504: 57111, 4505: 57082, 4506: 57079, 4507: 56996, 4508: 57165, 4509: 57137, 4510: 57214, 4511: 57175, 4512: 57240, 4513: 57211, 4514: 57283, 4515: 57328, 4516: 57322, 4517: 57380, 4518: 57399, 4519: 57371, 4520: 57363, 4521: 57477, 4522: 57439, 4523: 57443, 4524: 57497, 4525: 57507, 4526: 57512, 4527: 57565, 4528: 57562, 4529: 57587, 4530: 57581, 4531: 57606, 4532: 57613, 4533: 57629, 4534: 57632, 4535: 57646, 4536: 57670, 4537: 57669, 4538: 57696, 4539: 57732, 4540: 57757, 4541: 57741, 4542: 57749, 4543: 57779, 4544: 57791, 4545: 57805, 4546: 57803, 4547: 57819, 4548: 57841, 4549: 57851, 4550: 57939, 4551: 57870, 4552: 57936, 4553: 57971, 4554: 58001, 4555: 58002, 4556: 58041, 4557: 58057, 4558: 58082, 4559: 58110, 4560: 58112, 4561: 58117, 4562: 58119, 4563: 58103, 4564: 58159, 4565: 58158, 4566: 58181, 4567: 58188, 4568: 58177, 4569: 58259, 4570: 58242, 4571: 58272, 4572: 58287, 4573: 58326, 4574: 58369, 4575: 58389, 4576: 58379, 4577: 58388, 4578: 58427, 4579: 58436, 4580: 58445, 4581: 58460, 4582: 58453, 4583: 58484, 4584: 58512, 4585: 58510, 4586: 58545, 4587: 58576, 4588: 58574, 4589: 58590, 4590: 58587, 4591: 58603, 4592: 58642, 4593: 58654, 4594: 58684, 4595: 58697, 4596: 58706, 4597: 58720, 4598: 58741, 4599: 58758, 4600: 58803, 4601: 58810, 4602: 58858, 4603: 58867, 4604: 58884, 4605: 58905, 4606: 58874, 4607: 58921, 4608: 58948, 4609: 58952, 4610: 58989, 4611: 59003, 4612: 59008, 4613: 59010, 4614: 59046, 4615: 59050, 4616: 59072, 4617: 59151, 4618: 59173, 4620: 59184, 4621: 59196, 4622: 59200, 4623: 59199, 4624: 59229, 4625: 59232, 4626: 59285, 4627: 59291, 4628: 59307, 4629: 59309, 4630: 59316, 4631: 59353, 4632: 59352, 4633: 59364, 4634: 59396, 4635: 59394, 4636: 59392, 4637: 59439, 4638: 59449, 4639: 59384, 4640: 59468, 4641: 59458, 4642: 59489, 4643: 59501, 4644: 59517, 4645: 59551, 4646: 59504, 4647: 59588, 4648: 59607, 4649: 59647, 4650: 59608, 4651: 59622, 4652: 59654, 4653: 59678, 4654: 59708, 4655: 59728, 4656: 59747, 4657: 59750, 4658: 59785, 4659: 59746, 4660: 59774, 4661: 59801, 4662: 59803, 4663: 59819, 4664: 59851, 4665: 59796, 4666: 59831, 4667: 59847, 4668: 59856, 4669: 59898, 4670: 59895, 4671: 59929, 4672: 59920, 4673: 59923, 4674: 60000, 4675: 59950, 4676: 59941, 4677: 59983, 4678: 59984, 4679: 60009, 4680: 60018, 4681: 60030, 4682: 60059, 4683: 59879, 4684: 60066, 4685: 60087, 4686: 59767, 4687: 60044, 4688: 60098, 4689: 60129, 4690: 60122, 4691: 60157, 4692: 60183, 4693: 60170, 4694: 60168, 4695: 60172, 4696: 60189, 4697: 60202, 4698: 60197, 4699: 60221, 4700: 60260, 4701: 60212, 4702: 60308, 4703: 60320, 4704: 60329, 4705: 60327, 4706: 60379, 4707: 60351, 4708: 60353, 4709: 60638, 4710: 60417, 4711: 60425, 4712: 60449, 4713: 60454, 4714: 60463, 4715: 60467, 4716: 60485, 4717: 60514, 4718: 60549, 4719: 60525, 4720: 60601, 4721: 60591, 4722: 60595, 4723: 60603, 4724: 60610, 4725: 60599, 4726: 60584, 4727: 60589, 4728: 60646, 4730: 60718, 4732: 60710, 4733: 60697, 4734: 60729, 4735: 60735, 4736: 60771, 4737: 60742, 4738: 60746, 4739: 60781, 4740: 60699, 4741: 60804, 4742: 60809, 4743: 60823, 4744: 60851, 4745: 60795, 4746: 60813, 4747: 60861, 4748: 60855, 4749: 60870, 4750: 60880, 4751: 60891, 4752: 60904, 4753: 60941, 4754: 60969, 4755: 60979, 4756: 60957, 4757: 60965, 4758: 60994, 4759: 61015, 4760: 60978, 4761: 60988, 4762: 60992, 4763: 61084, 4765: 60998, 4766: 61071, 4767: 61053, 4768: 61136, 4769: 61181, 4770: 61103, 4771: 61158, 4772: 61134, 4773: 61199, 4774: 61172, 4775: 61174, 4776: 61212, 4777: 61246, 4778: 61270, 4779: 61296, 4780: 61295, 4781: 61318, 4782: 61328, 4783: 61309, 4784: 61320, 4785: 61317, 4786: 61359, 4787: 61281, 4788: 61379, 4789: 61394, 4790: 61443, 4791: 61415, 4792: 61418, 4793: 61420, 4794: 61468, 4795: 61384, 4796: 61498, 4797: 61496, 4798: 61585, 4799: 61558, 4800: 61532, 4801: 61571, 4802: 61622, 4803: 61621, 4804: 61738, 4805: 61637, 4806: 61703, 4807: 61658, 4808: 61667, 4809: 61688, 4810: 61720, 4811: 61692, 4812: 61719, 4813: 61740, 4814: 61796, 4815: 61724, 4816: 61748, 4817: 61789, 4818: 61916, 4819: 61932, 4820: 61981, 4821: 61910, 4823: 61966, 4824: 61937, 4825: 61941, 4827: 61951, 4828: 61960, 4829: 61968, 4830: 62027, 4831: 62012, 4832: 62026, 4833: 61936, 4834: 62058, 4835: 62084, 4836: 62081, 4837: 62103, 4838: 62128, 4839: 62131, 4840: 62046, 4841: 62212, 4842: 62268, 4843: 62172, 4844: 62322, 4845: 62207, 4846: 62223, 4847: 62267, 4848: 62327, 4849: 62325, 4850: 62360, 4851: 62356, 4852: 62170, 4853: 62434, 4854: 62376, 4855: 62394, 4856: 62421, 4857: 62448, 4858: 62443, 4859: 62402, 4860: 62500, 4861: 62478, 4862: 62608, 4863: 62423, 4864: 62523, 4865: 62541, 4866: 62516, 4867: 62512, 4868: 62646, 4869: 62576, 4870: 63031, 4871: 62655, 4872: 62703, 4873: 62653, 4874: 62683, 4875: 62641, 4876: 62732, 4877: 62743, 4878: 62757, 4879: 62786, 4880: 62799, 4881: 62788, 4882: 62821, 4883: 62763, 4884: 62807, 4885: 62861, 4886: 62825, 4887: 62894, 4888: 62867, 4889: 62896, 4890: 62931, 4891: 62875, 4892: 62561, 4893: 62572, 4894: 62886, 4895: 62986, 4896: 62915, 4897: 63007, 4898: 63003, 4899: 63005, 4900: 62933, 4901: 62983, 4902: 62985, 4903: 63033, 4904: 62972, 4905: 62956, 4906: 63066, 4907: 63165, 4908: 63117, 4909: 63024, 4910: 63090, 4911: 63109, 4912: 63159, 4913: 63210, 4914: 63121, 4915: 63125, 4916: 63076, 4917: 63143, 4918: 63243, 4919: 63211, 4920: 63355, 4921: 63414, 4922: 63490, 4923: 63613, 4924: 63462, 4925: 63494, 4926: 63497, 4927: 63340, 4928: 63432, 4929: 63533, 4930: 63688, 4931: 63503, 4932: 63608, 4933: 63724, 4934: 63584, 4935: 63738, 4936: 63647, 4937: 63750, 4938: 63820, 4939: 63883, 4940: 63945, 4941: 63972, 4942: 64004, 4943: 63901, 4944: 64033, 4945: 63916, 4946: 63948, 4947: 64003, 4948: 63933, 4949: 63950, 4950: 63822, 4951: 64053, 4952: 64094, 4953: 63952, 4954: 64022, 4955: 64078, 4956: 64077, 4957: 64122, 4958: 64166, 4959: 64181, 4960: 64179, 4961: 64224, 4962: 64226, 4963: 64238, 4964: 64212, 4965: 64320, 4966: 64390, 4967: 64217, 4968: 64241, 4970: 64332, 4971: 64246, 4972: 64395, 4973: 64348, 4974: 64231, 4975: 64425, 4976: 64587, 4977: 64466, 4978: 64375, 4979: 64408, 4980: 64478, 4981: 64407, 4982: 64472, 4983: 64394, 4984: 64417, 4985: 64515, 4986: 64445, 4987: 64449, 4988: 64580, 4989: 64583, 4990: 64520, 4991: 64557, 4992: 64496, 4993: 64661, 4994: 64682, 4995: 64577, 4996: 64623, 4997: 64540, 4998: 64607, 4999: 64651, 5000: 64790, 5001: 64725, 5002: 64820, 5003: 64527, 5004: 64692, 5005: 64769, 5006: 64803, 5007: 64751, 5008: 64822, 5009: 64437, 5010: 64779, 5011: 64792, 5012: 64994, 5013: 64823, 5014: 64838, 5015: 64852, 5016: 64933, 5017: 64844, 5018: 64774, 5019: 64924, 5020: 64962, 5021: 64954, 5022: 64927, 5023: 64906, 5024: 65108, 5025: 64979, 5026: 65112, 5027: 65129, 5028: 65109, 5029: 65144, 5030: 65289, 5031: 65119, 5032: 65072, 5033: 65183, 5035: 65271, 5036: 65247, 5037: 65198, 5038: 65288, 5039: 65303, 5040: 65241, 5041: 65387, 5042: 65468, 5043: 65311, 5044: 65301, 5045: 65230, 5046: 65372, 5047: 65323, 5048: 65479, 5049: 65522, 5050: 65420, 5051: 65628, 5052: 65376, 5053: 65417, 5054: 65378, 5056: 65474, 5057: 65466, 5058: 65535, 5059: 65545, 5060: 65593, 5061: 65606, 5062: 65477, 5063: 65630, 5064: 65581, 5065: 65621, 5066: 65755, 5067: 65550, 5068: 65639, 5069: 65783, 5070: 65530, 5071: 65810, 5072: 65721, 5073: 65536, 5074: 65603, 5075: 65613, 5076: 65678, 5077: 65723, 5078: 65796, 5079: 65698, 5080: 65835, 5081: 65790, 5082: 66121, 5083: 65754, 5084: 66753, 5085: 65728, 5086: 65862, 5087: 65855, 5088: 65892, 5089: 65936, 5090: 65969, 5091: 65595, 5092: 66019, 5093: 66152, 5094: 66015, 5095: 66006, 5096: 65951, 5097: 66065, 5098: 66060, 5099: 66091, 5100: 66098, 5101: 66100, 5102: 66086, 5103: 66236, 5104: 66254, 5105: 66200, 5106: 66247, 5107: 66249, 5108: 66223, 5109: 66198, 5110: 66257, 5111: 66320, 5112: 66234, 5113: 66438, 5114: 66326, 5115: 66607, 5116: 66294, 5117: 66405, 5118: 66427, 5119: 66574, 5120: 66400, 5121: 66454, 5122: 66521, 5123: 66417, 5124: 66575, 5125: 66668, 5126: 66385, 5127: 66458, 5128: 66563, 5129: 66522, 5130: 66681, 5131: 66251, 5132: 66657, 5133: 66475, 5134: 66666, 5135: 66645, 5136: 66656, 5137: 66594, 5138: 66640, 5139: 66435, 5140: 66849, 5141: 66821, 5142: 66634, 5143: 66690, 5144: 66727, 5145: 66725, 5146: 66800, 5147: 66825, 5148: 66704, 5149: 66763, 5150: 66803, 5151: 66925, 5152: 66923, 5153: 66700, 5154: 66738, 5155: 66924, 5156: 66860, 5157: 66984, 5158: 67036, 5159: 66936, 5160: 66892, 5161: 66907, 5162: 66798, 5163: 67004, 5164: 66992, 5165: 67057, 5166: 67071, 5167: 67143, 5168: 67153, 5169: 67005, 5170: 67139, 5171: 67261, 5172: 67234, 5173: 67172, 5174: 67244, 5175: 67292, 5176: 67304, 5177: 67103, 5178: 67271, 5179: 67194, 5180: 67210, 5181: 67288, 5182: 67239, 5183: 67246, 5184: 66903, 5185: 67275, 5186: 67250, 5187: 67231, 5188: 68009, 5189: 67408, 5190: 67464, 5191: 67301, 5192: 67457, 5193: 67472, 5194: 67664, 5195: 67384, 5196: 67494, 5197: 67523, 5198: 67537, 5199: 67410, 5200: 67459, 5201: 67480, 5202: 67555, 5203: 66878, 5204: 67483, 5205: 67545, 5206: 67663, 5207: 67703, 5208: 67648, 5209: 67620, 5210: 67669, 5212: 67696, 5213: 67485, 5214: 67596, 5215: 67605, 5216: 67548, 5217: 67836, 5218: 67942, 5219: 67665, 5220: 67714, 5221: 67786, 5222: 67819, 5223: 67861, 5224: 67851, 5225: 67787, 5226: 67627, 5227: 67589, 5228: 67890, 5229: 67782, 5230: 67973, 5231: 68002, 5232: 67929, 5233: 67953, 5234: 68080, 5235: 67927, 5236: 68101, 5237: 68021, 5238: 67848, 5239: 68079, 5240: 68431, 5241: 68191, 5242: 68270, 5243: 68030, 5244: 68092, 5245: 68065, 5246: 68177, 5247: 68103, 5248: 68245, 5249: 68282, 5250: 68269, 5251: 68333, 5252: 68413, 5253: 68455, 5254: 68279, 5255: 68276, 5256: 68184, 5257: 68390, 5258: 68380, 5259: 68493, 5260: 68523, 5261: 68815, 5262: 68498, 5263: 68478, 5264: 68520, 5265: 68581, 5266: 68670, 5267: 68702, 5268: 68641, 5269: 68673, 5270: 68594, 5271: 68567, 5272: 68705, 5273: 68682, 5274: 68687, 5275: 68707, 5276: 68739, 5277: 68763, 5278: 68842, 5279: 69090, 5280: 68637, 5281: 68902, 5282: 68537, 5283: 68776, 5284: 68808, 5285: 68862, 5286: 68872, 5287: 68895, 5288: 68933, 5289: 69065, 5290: 68940, 5291: 68756, 5292: 69122, 5293: 69241, 5294: 69113, 5295: 69298, 5296: 69174, 5297: 69191, 5298: 69127, 5299: 69038, 5300: 69068, 5301: 69269, 5302: 69107, 5303: 69896, 5304: 69226, 5305: 68956, 5306: 69778, 5307: 69340, 5308: 69462, 5309: 69398, 5310: 69316, 5311: 69491, 5312: 69415, 5313: 69389, 5314: 69458, 5315: 69427, 5316: 69618, 5317: 69493, 5318: 69598, 5319: 69655, 5320: 69763, 5321: 69112, 5322: 69564, 5323: 69536, 5324: 69623, 5325: 69671, 5326: 69754, 5327: 70418, 5328: 69481, 5329: 69483, 5330: 69612, 5331: 69614, 5332: 69658, 5333: 69592, 5334: 69373, 5335: 69569, 5336: 70248, 5337: 69722, 5338: 69701, 5339: 70638, 5340: 69673, 5341: 69727, 5342: 69747, 5343: 0, 5344: 69792, 5345: 69650, 5346: 69751, 5347: 69735, 5348: 69904, 5349: 70035, 5350: 69713, 5351: 69732, 5352: 69829, 5353: 69881, 5354: 69996, 5355: 69929, 5356: 69965, 5357: 69995, 5358: 70069, 5359: 69974, 5360: 69818, 5361: 69879, 5362: 70054, 5363: 69862, 5364: 70104, 5365: 69989, 5366: 70012, 5367: 70090, 5368: 70022, 5369: 69955, 5370: 70027, 5371: 70264, 5372: 69958, 5373: 70029, 5374: 70051, 5375: 70270, 5376: 70243, 5377: 70325, 5378: 70300, 5379: 70492, 5380: 70363, 5381: 70306, 5382: 70528, 5383: 70336, 5384: 70319, 5385: 70327, 5387: 70310, 5388: 70384, 5389: 70874, 5390: 70469, 5391: 70657, 5392: 70400, 5393: 70452, 5394: 70414, 5395: 70574, 5396: 70576, 5397: 70518, 5398: 70580, 5399: 70538, 5400: 70626, 5401: 70663, 5402: 70517, 5403: 70798, 5404: 70497, 5405: 70602, 5406: 70680, 5407: 70753, 5408: 71002, 5409: 70755, 5410: 70794, 5411: 70685, 5412: 70915, 5413: 70931, 5414: 70781, 5415: 70786, 5416: 70762, 5417: 70966, 5418: 70894, 5419: 70987, 5420: 70791, 5421: 71116, 5422: 70892, 5423: 70873, 5424: 70949, 5425: 71121, 5426: 71184, 5427: 71182, 5428: 71154, 5429: 71053, 5430: 70692, 5431: 71239, 5432: 71333, 5433: 71094, 5434: 71115, 5435: 71075, 5436: 70952, 5437: 71040, 5438: 71295, 5439: 71353, 5440: 71352, 5441: 71168, 5442: 71111, 5443: 71530, 5444: 71419, 5445: 71243, 5446: 71429, 5447: 71284, 5448: 71277, 5449: 71453, 5450: 71500, 5451: 71251, 5452: 71280, 5453: 71536, 5454: 71406, 5455: 71469, 5456: 71581, 5457: 71639, 5458: 71658, 5459: 71683, 5460: 71681, 5461: 71746, 5462: 71571, 5463: 71908, 5464: 71568, 5465: 71861, 5466: 71783, 5467: 71573, 5468: 71618, 5469: 71860, 5470: 72370, 5471: 71865, 5472: 71729, 5473: 71759, 5474: 71853, 5475: 71762, 5477: 71795, 5479: 71196, 5480: 71832, 5481: 71837, 5482: 72131, 5483: 71857, 5484: 71974, 5485: 72010, 5486: 72097, 5487: 71957, 5488: 72121, 5489: 72104, 5490: 71995, 5491: 75736, 5492: 71876, 5493: 72012, 5494: 72250, 5495: 72290, 5496: 72122, 5497: 72197, 5498: 72308, 5499: 72210, 5500: 72438, 5501: 72154, 5502: 72125, 5503: 72194, 5504: 72217, 5505: 72105, 5507: 72139, 5508: 72296, 5509: 72338, 5510: 72124, 5511: 72220, 5512: 72208, 5513: 72310, 5514: 72323, 5515: 72471, 5516: 72357, 5517: 72378, 5518: 72373, 5519: 72432, 5520: 72833, 5521: 72488, 5522: 72449, 5523: 72489, 5524: 72412, 5525: 73540, 5526: 72571, 5527: 72773, 5528: 72683, 5529: 72469, 5530: 72603, 5531: 72622, 5532: 72552, 5533: 72487, 5534: 72567, 5535: 72631, 5536: 72629, 5537: 72508, 5538: 72524, 5539: 72965, 5540: 73223, 5541: 72582, 5542: 72772, 5543: 72800, 5544: 72659, 5545: 73771, 5546: 73036, 5547: 73415, 5548: 72929, 5549: 72959, 5550: 72846, 5551: 73129, 5552: 72664, 5553: 72848, 5554: 72934, 5555: 73394, 5556: 73095, 5557: 74296, 5558: 73049, 5559: 73111, 5560: 73154, 5561: 73116, 5562: 73107, 5563: 72607, 5564: 73133, 5565: 73171, 5566: 73241, 5567: 73087, 5568: 73184, 5569: 73068, 5570: 73165, 5571: 73273, 5572: 73279, 5573: 73193, 5574: 73156, 5575: 73166, 5576: 73334, 5577: 73284, 5578: 73249, 5579: 73341, 5580: 73354, 5581: 73100, 5582: 73310, 5583: 73309, 5584: 73350, 5585: 73493, 5586: 73473, 5587: 73559, 5588: 73369, 5589: 73199, 5590: 73497, 5591: 73566, 5592: 73464, 5593: 73776, 5594: 73536, 5595: 73624, 5596: 72573, 5597: 73454, 5598: 73980, 5599: 73608, 5600: 73568, 5601: 73620, 5602: 73555, 5603: 73714, 5604: 73764, 5605: 73807, 5607: 73826, 5608: 73507, 5609: 73634, 5610: 73716, 5611: 74047, 5612: 73637, 5613: 73675, 5614: 73865, 5615: 73901, 5616: 73745, 5617: 74006, 5618: 73695, 5619: 73937, 5620: 73927, 5621: 74184, 5622: 73945, 5623: 74181, 5624: 74066, 5625: 74100, 5626: 74117, 5627: 73841, 5628: 74421, 5629: 73706, 5630: 73941, 5631: 74026, 5632: 74273, 5633: 74000, 5634: 73996, 5635: 73909, 5636: 74224, 5637: 74305, 5638: 74087, 5639: 74121, 5640: 74096, 5641: 74239, 5642: 74336, 5643: 74329, 5644: 74582, 5645: 74470, 5646: 74376, 5647: 74380, 5648: 74083, 5649: 74395, 5650: 74418, 5651: 74449, 5652: 74392, 5653: 74479, 5654: 74386, 5655: 74490, 5656: 74493, 5657: 74500, 5658: 74539, 5659: 74432, 5660: 74604, 5661: 74750, 5662: 74600, 5663: 74696, 5664: 74778, 5665: 74505, 5666: 74837, 5667: 74707, 5668: 74716, 5669: 74623, 5670: 74824, 5671: 74946, 5672: 74272, 5673: 74514, 5674: 74561, 5675: 74649, 5676: 74596, 5677: 74571, 5678: 74732, 5679: 74689, 5680: 74941, 5681: 74666, 5682: 74875, 5683: 74911, 5684: 75091, 5685: 74785, 5686: 74857, 5687: 74950, 5688: 74929, 5689: 75003, 5690: 74901, 5691: 74605, 5692: 74896, 5693: 74625, 5694: 74975, 5695: 75141, 5696: 75164, 5697: 75151, 5698: 75206, 5699: 75181, 5700: 75308, 5701: 75110, 5702: 75000, 5703: 75118, 5704: 75323, 5705: 75177, 5706: 75101, 5707: 75127, 5708: 75264, 5709: 75049, 5710: 75119, 5711: 75125, 5712: 75304, 5713: 75565, 5714: 74793, 5715: 75043, 5716: 75093, 5717: 75230, 5718: 75178, 5719: 75439, 5720: 75352, 5721: 75342, 5722: 75476, 5723: 75379, 5724: 75501, 5725: 75665, 5726: 75257, 5727: 75312, 5729: 76996, 5730: 76013, 5731: 75256, 5732: 75369, 5733: 75411, 5734: 75415, 5735: 75097, 5736: 75647, 5737: 75260, 5738: 75742, 5739: 75530, 5740: 75535, 5741: 75572, 5742: 75828, 5743: 75730, 5744: 75458, 5745: 75674, 5746: 75761, 5747: 75695, 5748: 75587, 5749: 75939, 5750: 75944, 5751: 76063, 5752: 75770, 5753: 76048, 5754: 75678, 5755: 75696, 5756: 76028, 5757: 76664, 5758: 75971, 5759: 75788, 5760: 75919, 5761: 75928, 5762: 76106, 5763: 75973, 5764: 76126, 5765: 76143, 5766: 76407, 5767: 76207, 5768: 75822, 5769: 76006, 5770: 76069, 5771: 76440, 5772: 76133, 5773: 76234, 5774: 76041, 5775: 76259, 5776: 76297, 5777: 76219, 5778: 76127, 5779: 76233, 5780: 76243, 5781: 76371, 5782: 76750, 5783: 76198, 5784: 76397, 5785: 75974, 5786: 76877, 5787: 76333, 5788: 76276, 5790: 76395, 5791: 76291, 5792: 76736, 5793: 76267, 5794: 76470, 5795: 76337, 5796: 76372, 5797: 76552, 5798: 76618, 5799: 76427, 5800: 76307, 5801: 76503, 5802: 76425, 5803: 76716, 5804: 76424, 5805: 76591, 5806: 76532, 5807: 76608, 5808: 76366, 5809: 76567, 5810: 76569, 5811: 76311, 5812: 76600, 5813: 76456, 5814: 76628, 5815: 76602, 5816: 76603, 5817: 76384, 5818: 76376, 5819: 76666, 5820: 76705, 5821: 76809, 5822: 76739, 5823: 76534, 5824: 76742, 5825: 76829, 5826: 76008, 5827: 76617, 5828: 76509, 5829: 75809, 5830: 76568, 5831: 76733, 5832: 76935, 5833: 76669, 5835: 76594, 5836: 77042, 5837: 76939, 5838: 76880, 5839: 76945, 5840: 76810, 5841: 76651, 5842: 76852, 5843: 76866, 5844: 76519, 5845: 76878, 5846: 77086, 5847: 77007, 5848: 77060, 5849: 76952, 5850: 76985, 5851: 77390, 5853: 77052, 5854: 77070, 5855: 77048, 5856: 77235, 5857: 76957, 5858: 77111, 5859: 77163, 5860: 77286, 5861: 77186, 5862: 77350, 5863: 77227, 5864: 77358, 5865: 77454, 5866: 77287, 5867: 77233, 5868: 77257, 5869: 77562, 5870: 77336, 5871: 77541, 5872: 77574, 5873: 77645, 5874: 77412, 5875: 77464, 5876: 77785, 5877: 77397, 5878: 77272, 5879: 77450, 5880: 77442, 5881: 77516, 5882: 77678, 5883: 77634, 5884: 77817, 5885: 77635, 5886: 77277, 5887: 77370, 5888: 77578, 5889: 77512, 5890: 77771, 5891: 77982, 5892: 77622, 5893: 77729, 5894: 77615, 5895: 77660, 5896: 77689, 5897: 77952, 5898: 77927, 5899: 77661, 5900: 77990, 5901: 77655, 5902: 77811, 5903: 77055, 5904: 77840, 5905: 78045, 5906: 77858, 5907: 77859, 5908: 77853, 5909: 77782, 5910: 77900, 5911: 77801, 5912: 77909, 5913: 77835, 5914: 77760, 5915: 77939, 5916: 77985, 5917: 77984, 5918: 78046, 5919: 77910, 5920: 78279, 5921: 78117, 5922: 77738, 5923: 78058, 5924: 77902, 5925: 78105, 5926: 78106, 5927: 78059, 5928: 78104, 5929: 78142, 5930: 78078, 5931: 77995, 5932: 77907, 5933: 78072, 5934: 78168, 5935: 78236, 5936: 78012, 5937: 78355, 5938: 77986, 5939: 78476, 5940: 78132, 5941: 78207, 5942: 78246, 5943: 78323, 5944: 78265, 5945: 78367, 5946: 78469, 5947: 78159, 5948: 78384, 5949: 78017, 5950: 78153, 5951: 78603, 5952: 78445, 5953: 78401, 5954: 78400, 5955: 78868, 5956: 78486, 5957: 78276, 5958: 78322, 5959: 78436, 5960: 78180, 5961: 78662, 5962: 78639, 5963: 78442, 5964: 78286, 5965: 78575, 5966: 78481, 5967: 78655, 5968: 78459, 5969: 78650, 5970: 78665, 5971: 78493, 5972: 78554, 5973: 78699, 5974: 78726, 5975: 78747, 5976: 78685, 5977: 0, 5979: 78989, 5980: 78914, 5981: 78542, 5982: 78592, 5983: 78649, 5984: 78820, 5985: 78821, 5986: 78527, 5987: 78918, 5988: 78877, 5989: 78849, 5990: 78870, 5991: 78970, 5992: 78840, 5993: 78933, 5994: 79153, 5995: 78632, 5996: 78955, 5997: 78990, 5998: 79031, 5999: 79080, 6000: 79081, 6001: 79050, 6002: 79005, 6003: 79098, 6004: 79007, 6005: 78985, 6006: 79173, 6007: 79199, 6008: 79043, 6009: 79045, 6010: 79072, 6011: 79120, 6012: 79203, 6013: 79102, 6014: 79137, 6015: 79320, 6016: 79195, 6017: 79302, 6018: 79119, 6019: 79497, 6020: 80047, 6021: 80057, 6022: 79490, 6023: 79101, 6024: 79509, 6025: 78893, 6027: 79374, 6028: 79404, 6029: 79399, 6030: 79664, 6031: 79375, 6032: 79337, 6033: 79387, 6034: 78661, 6035: 79332, 6036: 79125, 6037: 79797, 6038: 79164, 6039: 79349, 6040: 79689, 6041: 79463, 6042: 79530, 6043: 79350, 6044: 79596, 6045: 79653, 6046: 79358, 6047: 79488, 6048: 79540, 6049: 79661, 6050: 79357, 6051: 79599, 6052: 79441, 6053: 79605, 6054: 79622, 6055: 79754, 6056: 79593, 6057: 79581, 6058: 79790, 6059: 79812, 6060: 79672, 6061: 79692, 6062: 79932, 6063: 79607, 6065: 79666, 6066: 79785, 6067: 79781, 6068: 79686, 6069: 79414, 6070: 79881, 6071: 79963, 6072: 80000, 6073: 80054, 6074: 79757, 6075: 79882, 6076: 79945, 6077: 79980, 6078: 79938, 6079: 79280, 6080: 80066, 6081: 80079, 6082: 79420, 6083: 80208, 6084: 80112, 6085: 80212, 6086: 79804, 6087: 80021, 6088: 79601, 6089: 80455, 6090: 79953, 6091: 80008, 6092: 79992, 6093: 80179, 6094: 80337, 6095: 80170, 6096: 80227, 6097: 80324, 6098: 80686, 6099: 80421, 6100: 80390, 6101: 79867, 6102: 81065, 6103: 80181, 6104: 80343, 6105: 80399, 6107: 80197, 6108: 80214, 6109: 80645, 6110: 80247, 6111: 80351, 6112: 80473, 6114: 80675, 6115: 80582, 6116: 79822, 6117: 80463, 6118: 80569, 6119: 80488, 6120: 80788, 6121: 80514, 6122: 80672, 6123: 80460, 6124: 80558, 6125: 80874, 6126: 80161, 6127: 80375, 6128: 80620, 6129: 80628, 6130: 80309, 6131: 80782, 6132: 80331, 6133: 84461, 6134: 80763, 6135: 81141, 6136: 80693, 6137: 80719, 6138: 82076, 6139: 83255, 6140: 80793, 6141: 80815, 6142: 80945, 6143: 80911, 6144: 80840, 6145: 80910, 6146: 80704, 6147: 80894, 6148: 80816, 6149: 80883, 6150: 80710, 6151: 81252, 6152: 80843, 6153: 80975, 6154: 80898, 6155: 81122, 6156: 80809, 6157: 80888, 6158: 81007, 6159: 81008, 6160: 81198, 6161: 80650, 6162: 80953, 6163: 81852, 6164: 81305, 6165: 81266, 6166: 81304, 6167: 81507, 6168: 81126, 6169: 81231, 6170: 80991, 6171: 81300, 6172: 81710, 6173: 80480, 6174: 81472, 6175: 81377, 6176: 81337, 6177: 81657, 6178: 81523, 6179: 81440, 6180: 80920, 6181: 81425, 6182: 81873, 6183: 81289, 6184: 81290, 6185: 81292, 6187: 81702, 6188: 81733, 6189: 81580, 6190: 81632, 6191: 80850, 6192: 81741, 6193: 81728, 6194: 81634, 6195: 81641, 6196: 81724, 6197: 81847, 6198: 81358, 6199: 81437, 6200: 81497, 6201: 81687, 6202: 81754, 6203: 81659, 6204: 82129, 6205: 81734, 6206: 81903, 6207: 81966, 6208: 81670, 6209: 81914, 6210: 81904, 6211: 81891, 6212: 81693, 6213: 81729, 6214: 81972, 6215: 82110, 6216: 81939, 6217: 82273, 6218: 81992, 6219: 82171, 6220: 81833, 6221: 82135, 6222: 81911, 6223: 81660, 6224: 82037, 6225: 82140, 6226: 81840, 6227: 82028, 6228: 82073, 6229: 82363, 6230: 82012, 6231: 82539, 6232: 82162, 6233: 82517, 6234: 82216, 6235: 82259, 6236: 82418, 6237: 82020, 6238: 81428, 6239: 82241, 6240: 82339, 6241: 82396, 6242: 82172, 6243: 82369, 6244: 82453, 6245: 82493, 6246: 82350, 6247: 82514, 6248: 82405, 6249: 82543, 6250: 82402, 6251: 82672, 6252: 82545, 6253: 82806, 6254: 82321, 6255: 82480, 6256: 82355, 6257: 82650, 6258: 82422, 6259: 82426, 6260: 82676, 6261: 82669, 6262: 82671, 6263: 0, 6264: 82419, 6265: 82706, 6266: 82716, 6267: 81854, 6268: 82526, 6269: 82621, 6270: 82504, 6271: 82729, 6272: 82775, 6273: 82731, 6274: 82868, 6275: 82902, 6276: 83150, 6277: 82693, 6278: 82737, 6279: 82587, 6280: 82730, 6281: 82673, 6282: 82855, 6283: 82911, 6284: 82861, 6285: 83081, 6286: 82611, 6287: 82764, 6288: 82960, 6289: 83057, 6290: 82798, 6291: 82925, 6292: 82780, 6293: 82802, 6294: 82951, 6295: 83153, 6296: 82979, 6297: 83187, 6298: 83100, 6299: 83000, 6300: 83216, 6301: 82989, 6302: 83090, 6303: 83230, 6304: 83323, 6305: 82987, 6306: 82867, 6307: 83007, 6308: 83176, 6310: 83196, 6311: 83235, 6312: 83321, 6313: 83013, 6314: 83431, 6315: 82860, 6316: 83336, 6317: 83223, 6318: 83262, 6319: 82898, 6320: 83535, 6321: 83331, 6322: 82080, 6323: 83481, 6324: 83207, 6325: 83254, 6326: 83308, 6327: 83491, 6328: 83274, 6329: 83342, 6330: 83138, 6331: 83594, 6332: 83313, 6333: 83367, 6334: 83574, 6335: 82880, 6336: 83365, 6337: 83430, 6338: 83650, 6339: 83435, 6340: 83567, 6341: 83478, 6342: 83504, 6343: 83493, 6344: 83693, 6345: 83114, 6346: 83462, 6347: 83706, 6348: 83289, 6349: 83601, 6350: 83684, 6351: 83494, 6352: 83565, 6353: 83635, 6354: 83740, 6355: 83613, 6356: 83984, 6357: 84158, 6358: 83677, 6359: 83688, 6360: 83359, 6361: 83738, 6362: 83593, 6363: 83575, 6364: 83692, 6365: 83854, 6366: 83896, 6367: 83853, 6368: 84248, 6369: 83608, 6371: 84033, 6372: 83906, 6373: 84510, 6374: 84105, 6375: 83962, 6376: 83816, 6377: 83838, 6378: 84012, 6379: 83317, 6380: 84143, 6381: 84150, 6382: 84151, 6383: 83857, 6384: 84311, 6385: 84036, 6386: 84147, 6387: 84175, 6388: 83947, 6389: 84226, 6390: 84113, 6391: 84054, 6392: 84332, 6393: 84177, 6394: 84217, 6395: 84021, 6396: 83895, 6397: 84401, 6398: 84425, 6399: 84108, 6400: 84827, 6401: 84405, 6403: 84445, 6404: 84402, 6405: 84489, 6406: 84345, 6408: 84731, 6409: 84551, 6410: 84379, 6411: 84979, 6412: 84479, 6413: 84524, 6414: 84500, 6415: 84514, 6416: 84720, 6417: 84969, 6418: 84380, 6419: 84431, 6420: 84690, 6421: 84183, 6422: 84650, 6423: 84759, 6424: 84626, 6425: 84625, 6426: 84709, 6427: 84748, 6428: 84649, 6429: 85760, 6430: 84599, 6431: 84573, 6432: 84631, 6433: 84671, 6434: 84704, 6435: 84792, 6436: 84606, 6437: 84530, 6438: 85049, 6439: 84801, 6440: 85147, 6441: 84856, 6442: 85068, 6443: 84691, 6444: 84656, 6445: 84893, 6446: 84880, 6447: 85169, 6448: 84496, 6449: 84883, 6450: 85020, 6451: 85079, 6452: 84833, 6453: 84970, 6454: 85019, 6455: 84821, 6456: 85048, 6457: 84887, 6458: 84862, 6459: 85084, 6460: 85162, 6461: 85258, 6462: 85267, 6463: 84938, 6464: 84835, 6465: 85042, 6466: 84934, 6467: 84855, 6468: 85312, 6469: 84949, 6470: 85237, 6471: 85480, 6472: 85207, 6473: 85195, 6474: 85242, 6475: 85393, 6476: 85139, 6477: 85389, 6478: 85409, 6479: 84950, 6480: 85157, 6481: 85185, 6482: 85187, 6483: 85470, 6484: 85112, 6486: 85340, 6487: 85520, 6488: 85149, 6489: 85307, 6490: 85391, 6491: 85181, 6492: 85423, 6493: 85365, 6494: 85442, 6495: 85302, 6496: 85397, 6497: 85333, 6498: 85355, 6499: 85313, 6500: 85727, 6501: 85543, 6502: 85385, 6503: 85589, 6504: 85474, 6505: 85751, 6506: 85382, 6507: 85537, 6508: 85696, 6509: 85379, 6510: 85792, 6511: 85290, 6512: 85622, 6513: 85788, 6514: 85317, 6516: 85667, 6517: 85786, 6518: 85235, 6519: 85755, 6520: 85783, 6521: 85666, 6522: 85839, 6523: 85889, 6524: 85749, 6525: 86088, 6526: 85693, 6527: 85927, 6528: 85715, 6529: 84769, 6530: 86064, 6531: 85688, 6532: 85826, 6533: 85790, 6534: 85922, 6535: 86011, 6536: 85670, 6537: 86092, 6538: 85810, 6539: 86098, 6540: 85692, 6541: 85912, 6542: 85930, 6543: 85934, 6544: 86019, 6545: 86060, 6546: 86170, 6547: 86248, 6548: 85998, 6549: 86305, 6550: 85888, 6551: 85987, 6552: 88274, 6553: 86228, 6554: 85819, 6555: 85829, 6556: 86032, 6557: 86246, 6558: 86311, 6559: 86118, 6560: 85923, 6561: 86263, 6562: 86266, 6563: 86096, 6564: 86130, 6565: 86815, 6566: 85805, 6567: 86284, 6568: 86313, 6569: 86486, 6570: 86178, 6571: 86254, 6572: 86552, 6573: 86036, 6574: 86182, 6575: 86391, 6576: 86628, 6577: 86373, 6578: 86476, 6579: 86362, 6580: 86670, 6581: 86565, 6582: 86929, 6583: 86698, 6584: 86462, 6585: 86796, 6586: 86871, 6587: 86716, 6588: 86414, 6589: 86546, 6590: 86575, 6591: 86506, 6592: 86537, 6593: 86725, 6594: 86623, 6595: 86736, 6596: 86201, 6597: 86847, 6598: 86184, 6599: 86501, 6600: 86769, 6601: 86768, 6602: 86667, 6603: 86742, 6604: 86732, 6605: 86499, 6606: 86219, 6607: 86561, 6608: 86731, 6609: 86831, 6610: 86835, 6611: 86809, 6612: 86713, 6613: 87042, 6614: 87174, 6615: 87073, 6616: 87072, 6617: 87099, 6618: 86782, 6619: 86925, 6620: 87074, 6621: 87163, 6622: 87314, 6623: 86974, 6624: 87393, 6625: 86939, 6626: 86946, 6627: 87044, 6628: 87220, 6629: 87108, 6630: 87261, 6631: 87294, 6632: 87379, 6633: 87150, 6634: 87564, 6635: 87926, 6636: 86614, 6637: 86620, 6638: 87158, 6639: 87224, 6640: 87404, 6641: 87045, 6642: 87192, 6643: 87390, 6644: 87194, 6645: 87380, 6646: 88599, 6647: 87460, 6648: 87472, 6649: 87523, 6650: 87335, 6651: 87516, 6652: 87529, 6653: 87532, 6654: 87308, 6655: 87341, 6656: 87212, 6657: 87567, 6658: 87569, 6659: 87428, 6660: 0, 6661: 87495, 6662: 87616, 6663: 87624, 6664: 87280, 6665: 87427, 6666: 87540, 6667: 87491, 6668: 87671, 6669: 87382, 6670: 87558, 6671: 87722, 6672: 87706, 6673: 87445, 6674: 87434, 6675: 87846, 6676: 87655, 6677: 87563, 6678: 87865, 6679: 87782, 6680: 87836, 6681: 87813, 6682: 87936, 6683: 87948, 6684: 87812, 6685: 87747, 6686: 87847, 6687: 87777, 6688: 87585, 6689: 87875, 6690: 87866, 6691: 88038, 6692: 88012, 6693: 88060, 6695: 87808, 6696: 87910, 6697: 87895, 6698: 88048, 6699: 87744, 6700: 88116, 6701: 87234, 6702: 87850, 6703: 87933, 6704: 88125, 6705: 87833, 6706: 88101, 6707: 87998, 6708: 88294, 6709: 88148, 6710: 88175, 6711: 88020, 6712: 88149, 6713: 88128, 6714: 88192, 6715: 88258, 6716: 88298, 6717: 87293, 6718: 88030, 6719: 88213, 6720: 88172, 6721: 92824, 6722: 88217, 6723: 88290, 6724: 88380, 6725: 87728, 6726: 88190, 6727: 88396, 6728: 88122, 6729: 88267, 6731: 89115, 6732: 88374, 6733: 88404, 6735: 87670, 6736: 88469, 6737: 88277, 6738: 88331, 6739: 88550, 6740: 88815, 6741: 88346, 6742: 88567, 6743: 88714, 6744: 88429, 6745: 88866, 6746: 88635, 6747: 88522, 6748: 88694, 6749: 88726, 6751: 89234, 6752: 88601, 6753: 88415, 6754: 88528, 6755: 88670, 6756: 88684, 6757: 88671, 6758: 88627, 6759: 88859, 6760: 89006, 6761: 89042, 6762: 88760, 6763: 88604, 6764: 88533, 6765: 88657, 6766: 88839, 6767: 88565, 6768: 88636, 6769: 88816, 6770: 88765, 6771: 88771, 6772: 88947, 6773: 88905, 6774: 89382, 6775: 88745, 6776: 88814, 6777: 89010, 6778: 89096, 6779: 88794, 6780: 89020, 6781: 88818, 6782: 88817, 6783: 89112, 6784: 88862, 6785: 88981, 6786: 89099, 6787: 88886, 6788: 89086, 6789: 85822, 6790: 88732, 6791: 88788, 6792: 88754, 6793: 88836, 6794: 88899, 6795: 88964, 6796: 89370, 6797: 89000, 6798: 89114, 6799: 88929, 6800: 89065, 6801: 89153, 6802: 89178, 6803: 89023, 6804: 89290, 6805: 89487, 6806: 88972, 6807: 89008, 6808: 89674, 6809: 88127, 6810: 88136, 6811: 85699, 6812: 89341, 6813: 89288, 6814: 89156, 6815: 89172, 6816: 89369, 6817: 89047, 6818: 89507, 6819: 89605, 6820: 89298, 6821: 89597, 6822: 89439, 6823: 89440, 6824: 89246, 6825: 89470, 6826: 89279, 6827: 89104, 6828: 89805, 6829: 90133, 6830: 89512, 6831: 89408, 6832: 89642, 6833: 89637, 6834: 89527, 6835: 89622, 6836: 89620, 6837: 90606, 6838: 89609, 6839: 89726, 6840: 89587, 6841: 89630, 6842: 89678, 6843: 89623, 6844: 89601, 6845: 89482, 6846: 89731, 6847: 89474, 6848: 0, 6849: 89401, 6850: 89348, 6851: 89677, 6852: 89684, 6853: 89604, 6854: 89683, 6855: 90098, 6856: 89920, 6857: 89772, 6858: 89851, 6859: 89931, 6860: 89773, 6861: 89980, 6862: 90037, 6863: 89968, 6864: 90012, 6865: 89448, 6866: 89918, 6867: 89827, 6868: 89861, 6869: 89962, 6870: 90074, 6871: 90304, 6872: 89826, 6873: 89977, 6874: 90124, 6875: 90200, 6876: 89925, 6877: 89935, 6878: 90083, 6879: 90185, 6880: 89835, 6881: 90096, 6882: 90023, 6883: 90052, 6884: 90135, 6885: 90067, 6886: 89943, 6887: 90086, 6888: 90260, 6889: 90290, 6890: 90174, 6891: 89981, 6892: 90238, 6893: 90336, 6894: 90414, 6895: 90139, 6896: 90289, 6897: 90422, 6898: 90253, 6899: 90930, 6900: 90271, 6901: 90163, 6902: 90313, 6903: 90191, 6904: 90256, 6905: 90568, 6906: 90337, 6907: 90485, 6908: 90664, 6909: 90478, 6910: 90541, 6911: 90146, 6912: 91723, 6913: 90496, 6914: 90510, 6915: 90597, 6916: 90797, 6917: 90342, 6918: 90441, 6919: 90494, 6920: 89908, 6921: 90604, 6922: 90662, 6923: 90156, 6924: 90398, 6925: 90487, 6926: 90576, 6927: 89937, 6928: 90497, 6929: 90610, 6930: 90595, 6931: 90759, 6932: 90651, 6933: 90687, 6934: 90830, 6935: 90642, 6936: 90763, 6937: 90842, 6938: 90853, 6939: 90981, 6940: 90692, 6941: 90676, 6942: 90887, 6943: 90637, 6944: 90806, 6945: 90344, 6946: 90804, 6947: 90836, 6948: 90936, 6949: 90476, 6950: 90729, 6951: 90982, 6952: 90968, 6953: 90969, 6954: 91062, 6955: 90762, 6956: 90884, 6957: 90844, 6958: 90858, 6959: 90913, 6960: 91014, 6961: 91004, 6962: 90991, 6963: 90967, 6964: 92586, 6965: 91066, 6966: 90915, 6967: 90971, 6968: 90923, 6969: 91132, 6970: 91105, 6971: 90970, 6972: 91172, 6973: 91117, 6974: 90865, 6975: 91041, 6976: 91090, 6977: 91118, 6978: 90905, 6979: 90780, 6980: 91139, 6981: 91159, 6982: 91792, 6983: 91013, 6984: 91119, 6985: 91217, 6986: 91461, 6987: 91237, 6988: 91347, 6989: 91369, 6990: 91405, 6991: 91494, 6992: 91286, 6993: 91322, 6994: 92233, 6995: 91279, 6996: 91733, 6997: 91235, 6998: 91438, 6999: 91394, 7000: 91384, 7001: 91262, 7002: 91389, 7003: 91250, 7004: 91854, 7005: 91662, 7006: 90647, 7007: 91532, 7008: 91499, 7009: 91373, 7010: 91523, 7011: 91689, 7012: 92024, 7013: 91196, 7014: 91677, 7015: 92008, 7016: 91533, 7017: 91491, 7018: 91315, 7019: 91552, 7020: 91726, 7021: 91875, 7022: 92021, 7023: 91781, 7024: 91751, 7025: 90182, 7026: 91901, 7027: 92394, 7028: 91525, 7029: 91918, 7030: 91675, 7031: 91989, 7032: 91845, 7033: 91707, 7034: 91880, 7035: 91974, 7036: 92294, 7037: 92157, 7038: 92016, 7039: 92041, 7040: 91975, 7041: 91820, 7042: 91606, 7043: 91852, 7044: 91883, 7045: 92079, 7046: 92111, 7047: 91958, 7048: 92027, 7049: 91755, 7050: 92226, 7051: 91919, 7053: 91926, 7055: 92136, 7056: 91971, 7057: 91973, 7058: 92036, 7059: 92117, 7060: 91843, 7061: 92043, 7062: 92308, 7063: 92175, 7064: 92088, 7065: 92367, 7066: 92202, 7067: 92131, 7068: 92382, 7069: 92161, 7070: 92350, 7071: 91985, 7072: 92301, 7073: 92098, 7074: 92609, 7075: 91915, 7076: 92254, 7077: 92384, 7078: 92390, 7079: 92270, 7080: 92156, 7081: 92243, 7082: 91811, 7083: 92391, 7084: 92133, 7085: 92386, 7086: 92312, 7087: 92646, 7088: 92480, 7089: 92442, 7090: 92219, 7091: 92346, 7092: 92630, 7093: 92687, 7094: 92488, 7095: 92676, 7096: 92269, 7097: 92669, 7098: 92396, 7099: 92475, 7100: 92398, 7101: 92524, 7102: 92405, 7103: 92635, 7104: 92643, 7105: 92649, 7106: 92420, 7107: 93015, 7108: 92799, 7109: 92593, 7110: 92674, 7111: 93012, 7112: 92550, 7113: 92614, 7114: 92747, 7115: 92551, 7116: 92761, 7117: 92056, 7118: 92599, 7119: 92814, 7120: 92845, 7121: 92855, 7122: 92953, 7123: 92549, 7124: 92112, 7125: 92512, 7126: 92882, 7127: 93163, 7128: 92931, 7129: 92989, 7130: 93287, 7131: 92728, 7132: 92768, 7133: 92818, 7134: 93148, 7135: 92872, 7136: 93074, 7137: 92689, 7138: 92771, 7139: 92791, 7140: 92833, 7141: 92946, 7142: 92951, 7143: 92963, 7144: 92968, 7145: 93057, 7146: 92831, 7147: 92934, 7148: 92937, 7149: 93026, 7150: 93085, 7151: 93134, 7152: 93174, 7153: 92731, 7154: 92822, 7155: 93132, 7156: 93209, 7157: 92862, 7158: 93051, 7159: 93140, 7160: 92040, 7161: 93574, 7162: 93017, 7163: 93138, 7164: 93234, 7165: 93124, 7166: 93225, 7167: 93179, 7168: 93315, 7169: 93368, 7170: 93371, 7171: 93175, 7172: 93203, 7173: 93218, 7174: 93104, 7175: 92997, 7176: 93244, 7177: 93470, 7178: 93194, 7179: 93177, 7180: 92782, 7181: 93256, 7182: 93423, 7183: 93270, 7184: 93068, 7185: 93210, 7186: 93418, 7187: 92969, 7188: 93542, 7190: 93624, 7191: 93053, 7192: 93279, 7193: 93429, 7194: 93506, 7195: 93498, 7196: 93197, 7197: 93552, 7198: 93374, 7199: 92717, 7200: 93404, 7201: 93309, 7202: 93393, 7203: 93537, 7204: 93354, 7205: 93543, 7206: 93419, 7207: 93443, 7208: 93488, 7209: 93526, 7210: 93299, 7211: 93667, 7212: 93437, 7213: 93815, 7214: 93580, 7215: 93408, 7216: 93523, 7217: 93683, 7218: 93340, 7219: 93647, 7220: 93666, 7221: 94150, 7222: 93603, 7223: 93862, 7224: 93187, 7225: 93717, 7226: 93825, 7228: 104382, 7229: 93466, 7230: 93763, 7231: 93743, 7232: 93858, 7233: 94054, 7234: 93864, 7235: 93747, 7236: 93805, 7237: 93718, 7238: 93720, 7239: 93855, 7240: 93925, 7241: 93887, 7242: 94005, 7243: 93820, 7244: 93770, 7245: 93892, 7246: 93993, 7247: 93097, 7248: 93867, 7249: 93996, 7250: 93845, 7251: 93713, 7252: 93733, 7253: 93843, 7254: 94114, 7255: 94130, 7256: 94124, 7257: 94157, 7258: 93808, 7259: 94160, 7260: 93966, 7261: 93917, 7262: 93903, 7263: 93975, 7264: 94141, 7265: 94144, 7266: 94068, 7267: 94034, 7268: 94243, 7269: 94149, 7270: 94272, 7271: 94398, 7272: 94076, 7273: 94344, 7274: 94724, 7275: 94013, 7276: 94372, 7277: 94434, 7278: 94789, 7279: 94385, 7280: 94290, 7281: 94556, 7282: 94437, 7283: 94311, 7284: 94280, 7285: 94377, 7286: 94382, 7287: 94477, 7288: 94478, 7289: 94712, 7290: 94140, 7291: 94645, 7292: 94643, 7293: 94336, 7295: 94302, 7296: 94730, 7297: 94858, 7298: 94481, 7299: 94598, 7300: 94624, 7301: 94620, 7302: 94630, 7303: 94727, 7304: 94820, 7305: 94679, 7306: 94703, 7307: 94720, 7308: 94685, 7309: 94490, 7310: 94376, 7311: 94623, 7312: 94083, 7313: 94833, 7314: 94713, 7315: 94834, 7316: 94986, 7317: 94929, 7318: 94827, 7319: 94885, 7320: 95369, 7321: 94913, 7322: 94755, 7323: 95054, 7324: 94852, 7325: 94916, 7326: 94910, 7327: 95077, 7328: 94779, 7329: 95261, 7330: 95149, 7331: 94982, 7332: 95002, 7333: 95066, 7334: 95211, 7335: 94934, 7336: 95073, 7337: 95241, 7338: 94932, 7339: 95159, 7340: 95168, 7341: 94890, 7342: 95176, 7343: 95294, 7344: 95188, 7345: 94981, 7346: 95067, 7347: 95219, 7348: 95347, 7349: 95222, 7350: 95396, 7351: 94974, 7352: 94648, 7353: 95281, 7354: 95253, 7355: 95408, 7356: 95038, 7357: 95287, 7358: 95260, 7359: 95235, 7360: 95456, 7361: 94993, 7362: 95477, 7363: 95503, 7364: 95340, 7365: 95143, 7366: 95453, 7367: 95485, 7368: 95319, 7369: 95398, 7370: 95690, 7371: 95081, 7372: 95372, 7373: 95447, 7374: 95400, 7375: 95564, 7376: 95370, 7377: 95501, 7378: 95557, 7379: 95574, 7380: 95619, 7381: 95306, 7382: 95352, 7383: 95999, 7384: 95487, 7385: 95498, 7386: 95492, 7387: 95585, 7388: 95866, 7389: 95572, 7390: 95560, 7391: 95582, 7392: 95823, 7393: 95932, 7394: 84535, 7395: 95556, 7396: 95664, 7397: 95732, 7398: 95865, 7399: 95905, 7400: 95793, 7401: 95520, 7402: 95820, 7403: 95673, 7404: 95812, 7405: 95771, 7406: 95785, 7407: 95822, 7408: 95656, 7409: 95818, 7410: 95965, 7411: 96141, 7412: 95898, 7413: 95576, 7414: 95937, 7415: 95929, 7416: 96178, 7417: 95947, 7418: 95951, 7419: 95953, 7420: 95853, 7421: 96016, 7422: 96234, 7423: 95167, 7424: 96341, 7425: 94604, 7426: 96052, 7427: 96014, 7428: 96003, 7429: 96229, 7430: 96327, 7431: 96406, 7432: 96356, 7433: 96365, 7434: 96607, 7435: 96734, 7436: 96195, 7437: 96275, 7438: 96351, 7439: 96440, 7440: 96465, 7441: 96302, 7442: 96198, 7443: 96496, 7444: 96288, 7445: 96416, 7446: 96483, 7447: 96468, 7448: 96164, 7449: 96428, 7450: 95978, 7451: 96258, 7452: 96417, 7453: 96286, 7454: 96536, 7455: 96882, 7456: 96481, 7457: 96387, 7458: 96458, 7459: 96781, 7460: 96556, 7461: 96721, 7462: 96100, 7463: 96516, 7464: 96739, 7465: 96396, 7466: 96503, 7467: 96491, 7468: 96459, 7469: 96441, 7470: 96729, 7471: 96630, 7472: 96601, 7473: 96760, 7474: 96665, 7475: 96688, 7476: 96808, 7477: 96572, 7478: 96683, 7479: 96757, 7480: 96807, 7481: 96724, 7482: 96789, 7483: 96693, 7484: 96620, 7485: 96801, 7486: 96840, 7487: 96706, 7488: 96837, 7489: 96950, 7490: 96856, 7491: 97067, 7492: 96805, 7493: 96931, 7494: 98086, 7495: 96825, 7496: 97063, 7497: 96957, 7498: 97534, 7499: 96907, 7500: 96773, 7501: 96988, 7502: 96977, 7503: 96895, 7504: 96901, 7505: 97028, 7506: 97077, 7507: 97260, 7508: 97091, 7509: 96919, 7510: 97421, 7511: 97139, 7512: 97087, 7513: 97569, 7514: 97081, 7515: 97290, 7516: 97244, 7517: 97118, 7518: 97150, 7519: 97229, 7520: 97142, 7521: 97491, 7522: 97033, 7523: 97151, 7524: 97674, 7525: 97278, 7526: 97070, 7527: 97611, 7528: 97165, 7529: 97228, 7530: 97242, 7531: 97646, 7532: 97423, 7533: 97319, 7534: 97295, 7535: 97307, 7536: 97365, 7537: 97598, 7538: 97515, 7540: 97402, 7541: 97499, 7542: 97454, 7543: 97376, 7544: 97473, 7545: 97122, 7546: 97496, 7547: 97372, 7548: 97816, 7549: 97819, 7550: 97477, 7551: 97485, 7552: 97749, 7553: 97650, 7554: 97607, 7555: 97538, 7556: 97572, 7557: 97649, 7558: 97971, 7559: 97687, 7560: 97675, 7561: 97783, 7562: 97697, 7563: 97326, 7564: 97629, 7565: 97679, 7566: 97630, 7567: 97634, 7568: 97651, 7569: 97767, 7570: 97804, 7571: 97849, 7572: 97787, 7573: 97765, 7574: 97796, 7575: 97871, 7576: 97635, 7577: 97700, 7578: 97944, 7579: 98332, 7580: 97877, 7581: 98032, 7582: 97433, 7583: 97789, 7584: 97928, 7585: 98012, 7586: 98163, 7587: 98174, 7588: 98346, 7589: 97757, 7590: 98495, 7591: 97774, 7592: 97886, 7593: 97966, 7594: 97967, 7595: 97938, 7596: 97980, 7597: 98066, 7598: 97970, 7599: 98038, 7600: 97845, 7601: 97961, 7602: 98036, 7603: 98478, 7604: 98162, 7605: 98226, 7606: 97985, 7607: 98017, 7608: 97870, 7609: 98085, 7610: 98103, 7611: 97892, 7612: 98624, 7613: 98068, 7614: 98258, 7615: 98110, 7616: 98169, 7617: 98351, 7618: 98353, 7619: 98055, 7620: 98143, 7621: 98482, 7622: 98234, 7623: 98412, 7624: 98421, 7625: 98608, 7626: 98044, 7627: 98485, 7628: 98194, 7629: 98461, 7630: 98512, 7631: 98470, 7632: 98146, 7633: 98073, 7634: 98111, 7635: 98337, 7636: 98385, 7637: 98416, 7638: 98253, 7639: 98579, 7640: 98325, 7641: 98375, 7642: 98320, 7643: 98575, 7644: 98959, 7645: 98438, 7646: 98383, 7647: 98425, 7648: 98526, 7649: 98633, 7650: 98688, 7651: 98379, 7652: 98761, 7653: 98543, 7654: 98308, 7655: 98563, 7656: 98609, 7657: 98636, 7658: 98785, 7659: 98842, 7660: 98571, 7661: 98809, 7662: 98738, 7663: 99200, 7664: 98754, 7665: 99240, 7666: 98333, 7667: 98844, 7668: 98960, 7669: 98823, 7670: 98767, 7671: 98910, 7672: 98819, 7673: 99120, 7674: 99137, 7675: 98953, 7676: 98583, 7677: 98897, 7678: 98863, 7679: 98920, 7680: 98954, 7681: 99024, 7682: 98658, 7683: 98921, 7684: 98858, 7685: 98702, 7686: 98401, 7687: 98915, 7688: 99080, 7689: 99031, 7690: 99171, 7691: 99453, 7692: 98946, 7693: 99158, 7694: 99221, 7695: 98872, 7696: 99176, 7697: 99026, 7698: 100697, 7699: 99145, 7700: 99234, 7701: 98962, 7702: 99102, 7703: 99461, 7704: 98890, 7705: 99352, 7706: 99570, 7707: 99762, 7708: 99303, 7709: 99457, 7710: 99473, 7711: 99404, 7712: 99529, 7713: 99445, 7714: 99747, 7715: 99572, 7716: 99479, 7717: 99585, 7718: 99518, 7719: 99531, 7720: 99631, 7721: 99539, 7722: 99825, 7723: 99679, 7724: 99742, 7725: 99878, 7726: 99579, 7727: 99500, 7728: 99920, 7729: 99926, 7730: 99639, 7731: 99738, 7732: 100165, 7733: 99685, 7734: 99719, 7735: 99675, 7736: 99770, 7737: 99749, 7738: 99918, 7739: 99824, 7740: 99655, 7741: 99853, 7742: 99663, 7743: 99841, 7744: 99874, 7745: 100151, 7746: 99913, 7747: 100027, 7748: 100062, 7749: 100184, 7750: 99255, 7751: 99848, 7752: 99893, 7753: 99951, 7754: 100064, 7755: 99870, 7756: 99889, 7757: 99929, 7758: 100300, 7759: 99968, 7760: 100018, 7761: 100195, 7762: 100016, 7763: 100044, 7764: 100288, 7765: 100332, 7766: 100412, 7767: 100069, 7768: 100232, 7769: 100108, 7770: 100122, 7771: 100208, 7772: 100283, 7773: 100310, 7774: 100256, 7775: 100325, 7776: 100345, 7777: 100142, 7778: 100274, 7779: 100469, 7780: 100276, 7781: 100097, 7782: 100239, 7783: 100017, 7784: 100268, 7785: 101427, 7786: 100250, 7787: 100591, 7788: 100524, 7789: 100435, 7790: 100751, 7791: 100295, 7792: 100221, 7793: 100511, 7794: 100541, 7795: 100434, 7796: 100453, 7797: 100486, 7798: 100437, 7799: 100764, 7800: 100501, 7801: 100738, 7802: 100515, 7803: 100664, 7804: 100261, 7805: 100357, 7806: 100587, 7807: 100574, 7808: 100852, 7809: 100758, 7810: 100762, 7811: 100754, 7812: 101843, 7813: 100779, 7814: 100881, 7815: 100651, 7816: 100807, 7817: 101017, 7818: 100714, 7819: 101011, 7820: 100953, 7821: 100977, 7822: 101027, 7823: 100866, 7824: 100969, 7825: 101090, 7826: 100907, 7827: 100808, 7828: 100859, 7829: 101120, 7830: 101123, 7831: 101101, 7832: 101211, 7833: 101070, 7834: 101076, 7835: 101067, 7836: 101160, 7837: 101221, 7838: 101626, 7839: 101213, 7840: 101235, 7841: 101133, 7842: 101384, 7843: 101084, 7844: 101138, 7845: 101345, 7846: 101477, 7847: 101214, 7848: 101612, 7849: 101300, 7850: 101093, 7851: 101243, 7852: 101421, 7853: 101558, 7854: 101245, 7855: 101507, 7856: 101552, 7857: 101473, 7858: 101483, 7859: 101773, 7860: 101285, 7861: 101383, 7862: 101505, 7863: 102162, 7864: 102125, 7865: 101608, 7866: 101474, 7867: 101467, 7868: 101134, 7869: 101772, 7870: 101475, 7871: 101589, 7872: 101965, 7873: 101692, 7874: 101641, 7875: 101983, 7876: 101556, 7877: 101808, 7878: 101746, 7879: 101260, 7880: 101716, 7881: 102157, 7882: 101769, 7883: 101800, 7884: 101847, 7885: 101765, 7886: 101810, 7887: 101768, 7888: 101756, 7889: 101923, 7890: 101921, 7891: 101867, 7892: 101882, 7893: 102014, 7894: 101868, 7895: 101870, 7896: 101916, 7897: 101936, 7898: 101997, 7899: 101909, 7900: 101984, 7901: 100965, 7902: 102032, 7903: 101919, 7904: 101899, 7905: 102026, 7906: 101958, 7907: 101966, 7908: 101082, 7909: 102092, 7910: 102094, 7911: 101949, 7912: 101934, 7913: 102395, 7914: 102040, 7915: 102230, 7916: 101900, 7917: 102033, 7918: 102080, 7919: 101986, 7920: 102333, 7921: 102066, 7922: 102062, 7923: 102158, 7924: 102098, 7925: 102011, 7926: 102155, 7927: 102195, 7928: 102281, 7929: 102177, 7930: 101044, 7931: 102414, 7932: 102276, 7933: 102497, 7934: 102773, 7935: 102496, 7936: 102485, 7937: 102487, 7938: 102216, 7939: 102388, 7940: 102258, 7941: 102440, 7942: 102453, 7943: 102693, 7944: 102358, 7945: 102253, 7946: 102561, 7947: 102531, 7948: 102532, 7949: 102488, 7950: 102618, 7951: 102624, 7952: 102790, 7953: 102633, 7954: 102631, 7955: 102431, 7956: 102571, 7957: 102422, 7958: 102530, 7959: 102962, 7961: 102772, 7962: 102499, 7963: 102589, 7964: 102780, 7965: 102831, 7966: 102585, 7967: 102370, 7968: 102950, 7969: 102635, 7970: 102901, 7971: 102916, 7972: 102642, 7973: 102805, 7974: 102819, 7975: 102833, 7976: 102891, 7977: 102724, 7978: 102712, 7979: 102989, 7980: 102978, 7981: 102878, 7982: 102945, 7983: 102827, 7984: 102843, 7985: 103005, 7986: 103227, 7987: 103127, 7988: 102949, 7989: 103071, 7990: 103045, 7991: 103103, 7992: 103206, 7993: 102771, 7994: 103077, 7995: 103004, 7996: 102993, 7997: 103168, 7998: 103154, 7999: 103079, 8000: 103226, 8001: 103089, 8002: 102208, 8003: 103094, 8004: 103108, 8005: 103145, 8006: 103261, 8007: 103191, 8008: 103200, 8009: 103189, 8010: 103301, 8011: 103294, 8012: 103298, 8013: 103389, 8014: 103347, 8015: 103401, 8016: 102599, 8017: 103414, 8018: 103460, 8019: 103752, 8020: 103312, 8021: 104043, 8022: 103343, 8023: 103371, 8024: 103545, 8025: 103359, 8026: 103360, 8027: 103673, 8028: 103413, 8029: 103346, 8030: 103527, 8031: 103646, 8032: 103511, 8033: 103616, 8034: 103569, 8035: 103519, 8036: 103532, 8037: 103635, 8038: 103652, 8039: 103738, 8040: 103530, 8041: 103682, 8042: 103814, 8043: 103219, 8044: 103675, 8045: 103777, 8046: 103836, 8047: 103632, 8048: 103882, 8049: 103598, 8050: 103902, 8051: 103734, 8052: 104385, 8053: 103732, 8054: 103889, 8055: 104085, 8056: 103892, 8057: 103891, 8058: 103981, 8060: 104019, 8061: 104440, 8062: 103828, 8063: 103894, 8064: 103871, 8065: 103810, 8066: 104031, 8067: 104041, 8068: 104048, 8069: 104177, 8070: 104148, 8071: 103963, 8072: 103929, 8073: 104364, 8074: 103949, 8075: 104139, 8076: 104174, 8077: 104101, 8078: 103956, 8079: 104060, 8080: 104234, 8081: 104597, 8082: 104172, 8083: 104297, 8084: 104185, 8085: 104214, 8086: 104217, 8087: 104365, 8088: 104281, 8089: 104194, 8090: 104357, 8091: 104452, 8092: 104755, 8093: 104459, 8094: 104371, 8095: 104481, 8096: 104557, 8097: 104521, 8098: 104538, 8099: 104171, 8100: 104680, 8101: 104539, 8102: 104634, 8103: 104483, 8104: 104738, 8105: 104579, 8106: 104516, 8107: 104537, 8108: 104752, 8109: 104449, 8110: 104750, 8111: 105169, 8112: 104105, 8113: 104451, 8114: 104978, 8115: 104732, 8116: 104771, 8117: 104925, 8118: 104839, 8119: 104642, 8120: 104765, 8121: 104872, 8122: 104914, 8123: 104858, 8124: 104980, 8125: 105148, 8126: 104822, 8127: 104963, 8128: 104974, 8129: 106320, 8130: 104887, 8131: 104987, 8132: 105046, 8133: 104788, 8134: 105079, 8135: 105140, 8136: 104962, 8137: 105143, 8138: 105034, 8139: 105168, 8140: 105319, 8141: 105164, 8142: 105228, 8143: 105102, 8144: 105101, 8145: 105334, 8146: 105138, 8147: 105064, 8148: 105312, 8149: 105224, 8150: 105080, 8151: 105382, 8152: 105425, 8153: 105091, 8154: 105186, 8155: 105229, 8156: 105678, 8157: 105269, 8158: 105344, 8159: 105768, 8160: 105412, 8161: 105282, 8162: 105199, 8163: 105413, 8164: 105259, 8165: 105411, 8166: 105390, 8167: 105515, 8168: 104968, 8169: 105432, 8170: 105406, 8171: 105268, 8172: 105576, 8173: 105502, 8174: 104756, 8175: 105574, 8176: 106474, 8177: 105685, 8178: 105570, 8179: 105370, 8180: 105696, 8181: 105858, 8182: 105558, 8183: 105665, 8184: 105662, 8185: 105497, 8186: 105561, 8187: 105668, 8188: 105841, 8189: 105607, 8190: 105652, 8191: 105695, 8192: 105729, 8193: 105637, 8194: 105660, 8195: 105761, 8196: 106044, 8197: 105689, 8198: 105703, 8199: 105767, 8200: 105854, 8201: 106424, 8202: 105913, 8203: 105819, 8204: 105881, 8205: 105864, 8206: 105733, 8207: 105928, 8208: 105769, 8209: 105811, 8210: 105860, 8211: 106065, 8212: 106007, 8213: 106039, 8214: 106067, 8215: 105942, 8216: 105898, 8217: 105966, 8218: 105891, 8219: 106021, 8220: 106003, 8221: 106064, 8222: 106143, 8223: 106062, 8224: 105949, 8225: 106140, 8226: 106052, 8227: 105972, 8228: 106093, 8229: 106327, 8230: 106340, 8231: 106243, 8232: 106278, 8233: 106419, 8234: 106881, 8235: 106363, 8236: 106429, 8237: 106171, 8238: 106032, 8239: 105727, 8240: 106355, 8241: 106500, 8242: 106267, 8243: 106227, 8244: 106564, 8245: 106559, 8246: 106393, 8247: 106590, 8248: 106420, 8249: 106818, 8250: 106527, 8251: 106592, 8252: 106481, 8253: 106654, 8254: 107089, 8255: 106551, 8256: 106703, 8257: 106595, 8258: 106605, 8259: 106518, 8260: 106723, 8261: 106661, 8262: 106642, 8263: 106758, 8264: 106786, 8265: 106781, 8266: 106711, 8267: 106787, 8268: 106907, 8269: 106978, 8270: 106856, 8271: 107030, 8272: 106752, 8273: 106938, 8274: 106872, 8275: 106771, 8276: 106897, 8277: 106944, 8278: 106985, 8279: 106801, 8280: 107843, 8281: 106886, 8282: 107112, 8283: 107095, 8284: 106999, 8285: 107128, 8286: 107423, 8287: 107144, 8288: 107188, 8289: 107151, 8290: 107041, 8291: 107097, 8292: 107173, 8293: 107238, 8294: 112355, 8295: 107232, 8297: 107129, 8298: 107140, 8299: 107344, 8300: 107162, 8301: 107136, 8302: 107302, 8303: 107409, 8304: 107186, 8305: 107380, 8306: 107235, 8307: 107253, 8308: 107315, 8309: 107310, 8311: 107382, 8312: 107197, 8313: 107348, 8314: 107350, 8315: 107354, 8316: 107259, 8317: 107119, 8318: 107487, 8319: 107517, 8320: 107445, 8321: 107472, 8322: 107556, 8323: 107649, 8324: 107230, 8325: 107502, 8326: 107608, 8327: 107374, 8328: 107575, 8329: 107488, 8330: 107558, 8331: 107773, 8332: 107596, 8333: 107835, 8334: 107418, 8335: 107533, 8336: 107637, 8337: 107750, 8338: 107664, 8339: 107586, 8340: 107797, 8341: 107734, 8342: 107555, 8343: 107763, 8344: 107788, 8345: 107749, 8346: 107901, 8347: 107723, 8348: 107887, 8349: 107856, 8350: 107956, 8351: 108036, 8352: 108195, 8353: 108085, 8354: 107975, 8355: 108058, 8356: 108022, 8357: 107930, 8358: 108060, 8359: 108090, 8360: 108102, 8361: 107893, 8362: 108281, 8363: 108144, 8364: 108121, 8365: 108259, 8366: 108294, 8367: 108347, 8368: 108431, 8369: 108478, 8370: 108759, 8371: 108165, 8372: 108296, 8373: 108339, 8374: 108209, 8375: 108226, 8376: 108453, 8377: 108348, 8378: 108494, 8379: 108543, 8380: 108849, 8381: 108626, 8382: 108506, 8383: 108317, 8384: 108364, 8385: 108612, 8386: 108661, 8387: 108870, 8388: 108505, 8389: 108553, 8390: 108691, 8391: 108632, 8392: 108693, 8393: 108699, 8394: 108784, 8395: 108420, 8396: 108797, 8397: 108766, 8398: 108871, 8399: 108650, 8400: 108535, 8401: 108868, 8402: 108874, 8403: 108758, 8404: 108875, 8405: 108952, 8406: 108772, 8407: 108845, 8408: 108975, 8409: 109081, 8410: 108991, 8411: 109111, 8412: 108969, 8413: 109068, 8414: 109074, 8415: 109023, 8416: 108924, 8417: 108917, 8418: 109139, 8419: 109056, 8420: 109477, 8421: 109033, 8422: 109079, 8423: 108456, 8424: 109102, 8425: 109268, 8426: 109005, 8427: 109082, 8428: 109017, 8429: 109121, 8430: 109176, 8431: 109285, 8432: 109584, 8433: 109289, 8434: 109124, 8435: 109209, 8436: 109212, 8437: 109306, 8438: 109240, 8439: 109332, 8440: 109408, 8441: 109276, 8442: 109190, 8443: 109205, 8444: 109404, 8445: 109272, 8446: 109412, 8447: 109422, 8448: 109303, 8449: 109352, 8450: 109427, 8451: 109442, 8452: 109472, 8453: 109466, 8454: 109410, 8455: 109439, 8456: 109445, 8457: 109509, 8458: 109471, 8459: 109458, 8460: 109493, 8461: 109577, 8462: 109624, 8463: 109521, 8464: 109667, 8465: 109492, 8466: 109602, 8467: 109647, 8468: 109400, 8469: 109556, 8470: 109737, 8471: 110078, 8472: 109572, 8473: 109434, 8474: 109474, 8475: 109654, 8476: 109585, 8477: 109821, 8478: 109789, 8479: 109592, 8480: 109786, 8481: 110256, 8482: 109730, 8483: 109620, 8484: 109902, 8485: 109754, 8486: 109908, 8487: 109745, 8488: 109973, 8489: 109831, 8490: 109746, 8491: 109939, 8492: 109990, 8493: 109657, 8494: 109857, 8495: 109984, 8496: 110000, 8497: 110018, 8498: 109937, 8499: 110003, 8500: 110009, 8501: 110109, 8502: 110130, 8503: 109977, 8504: 110023, 8505: 111196, 8506: 109972, 8507: 110091, 8508: 110179, 8509: 110311, 8510: 110171, 8511: 110103, 8512: 110273, 8513: 110298, 8514: 110341, 8515: 110618, 8516: 110391, 8517: 110346, 8518: 110395, 8519: 110316, 8520: 110386, 8521: 110478, 8522: 110371, 8523: 110351, 8524: 110506, 8525: 110116, 8526: 110712, 8527: 110668, 8528: 110408, 8529: 110529, 8530: 110532, 8531: 110649, 8532: 110548, 8533: 110578, 8534: 110602, 8535: 110497, 8536: 110566, 8537: 110498, 8538: 110538, 8539: 110672, 8540: 110838, 8541: 110609, 8542: 110746, 8543: 110696, 8544: 110778, 8546: 109693, 8547: 110935, 8548: 110785, 8549: 110790, 8550: 110550, 8551: 110882, 8552: 110936, 8553: 110849, 8554: 110807, 8555: 110873, 8556: 110997, 8557: 110725, 8558: 110960, 8560: 111043, 8561: 110817, 8562: 110986, 8563: 111045, 8564: 110992, 8565: 111066, 8566: 111062, 8567: 111086, 8568: 110919, 8569: 111041, 8570: 111138, 8571: 110991, 8572: 111022, 8573: 111123, 8574: 111068, 8575: 111072, 8576: 111188, 8577: 111504, 8578: 110787, 8579: 111104, 8580: 111171, 8581: 111170, 8582: 111310, 8583: 111200, 8584: 111191, 8585: 111169, 8586: 111278, 8587: 111296, 8588: 111259, 8589: 111247, 8590: 111394, 8591: 111056, 8592: 111449, 8593: 111548, 8594: 111362, 8595: 111314, 8596: 111515, 8597: 111497, 8598: 111325, 8599: 111242, 8600: 111594, 8601: 111600, 8602: 111643, 8603: 111544, 8604: 111567, 8605: 111601, 8606: 111550, 8607: 111543, 8608: 111649, 8609: 111627, 8610: 111710, 8611: 111833, 8612: 111761, 8613: 111674, 8614: 111801, 8615: 111532, 8616: 111809, 8617: 111753, 8618: 111810, 8619: 111879, 8620: 111967, 8621: 111795, 8622: 111841, 8623: 111934, 8624: 111884, 8625: 111660, 8626: 111869, 8627: 111797, 8628: 111954, 8629: 111965, 8630: 112405, 8631: 111974, 8632: 111944, 8633: 111925, 8634: 112029, 8635: 112117, 8636: 112122, 8637: 112102, 8638: 112032, 8639: 112127, 8640: 112031, 8641: 112051, 8642: 112067, 8643: 112041, 8644: 112203, 8645: 112168, 8646: 112259, 8647: 112179, 8648: 112098, 8649: 112211, 8650: 112158, 8651: 112144, 8652: 112170, 8653: 112222, 8654: 112241, 8655: 112374, 8656: 112242, 8657: 112381, 8658: 112414, 8659: 112432, 8660: 112358, 8661: 112300, 8662: 112449, 8663: 112781, 8664: 112725, 8665: 112447, 8666: 112417, 8667: 112440, 8668: 112501, 8669: 112603, 8670: 112529, 8671: 112554, 8672: 112694, 8673: 112542, 8674: 112567, 8675: 112623, 8676: 112615, 8677: 112508, 8678: 112590, 8679: 112716, 8680: 112746, 8681: 112714, 8682: 112641, 8683: 112635, 8684: 112748, 8685: 112832, 8686: 112895, 8687: 112670, 8688: 112731, 8689: 112924, 8690: 112778, 8691: 112803, 8692: 112761, 8693: 112862, 8694: 112724, 8695: 112948, 8696: 112864, 8697: 112935, 8698: 112961, 8699: 112917, 8700: 113044, 8701: 113137, 8702: 112519, 8703: 112997, 8704: 113031, 8705: 112986, 8706: 113009, 8707: 112998, 8708: 113048, 8709: 113136, 8710: 113127, 8711: 113148, 8712: 113084, 8713: 113189, 8714: 113131, 8715: 113167, 8716: 113184, 8717: 113186, 8718: 113174, 8719: 113234, 8720: 113246, 8721: 113283, 8722: 113307, 8723: 113222, 8724: 113311, 8725: 113281, 8726: 113288, 8727: 113345, 8728: 113368, 8729: 113357, 8730: 113360, 8731: 113327, 8732: 113447, 8733: 113371, 8734: 113421, 8735: 113433, 8736: 112833, 8737: 113445, 8738: 113465, 8739: 113503, 8740: 113532, 8741: 113531, 8742: 113521, 8743: 113562, 8744: 113501, 8745: 113498, 8746: 113593, 8747: 113638, 8748: 113116, 8749: 113657, 8750: 113610, 8751: 113622, 8752: 113561, 8753: 113621, 8754: 113669, 8755: 113603, 8756: 113673, 8757: 113674, 8758: 113640, 8759: 113686, 8760: 113784, 8761: 113684, 8762: 113726, 8763: 113781, 8764: 113801, 8765: 113783, 8766: 113788, 8767: 113860, 8768: 113802, 8769: 113969, 8770: 113797, 8771: 113902, 8772: 113896, 8773: 113889, 8774: 113957, 8775: 113881, 8776: 113904, 8777: 113853, 8778: 113852, 8779: 113864, 8780: 113919, 8781: 113963, 8782: 113996, 8783: 113998, 8784: 113994, 8785: 114005, 8786: 114258, 8787: 114131, 8788: 114081, 8789: 114119, 8790: 114132, 8791: 114159, 8792: 114096, 8793: 114167, 8794: 114268, 8795: 114144, 8796: 114155, 8797: 114104, 8798: 114187, 8799: 114189, 8800: 114174, 8801: 114162, 8802: 114254, 8803: 114163, 8804: 114200, 8805: 114210, 8806: 114237, 8807: 114273, 8808: 114212, 8809: 114408, 8810: 114550, 8811: 114227, 8812: 114341, 8813: 114366, 8814: 114382, 8815: 114347, 8816: 114371, 8817: 114375, 8818: 114407, 8819: 114222, 8820: 114421, 8821: 114389, 8822: 114365, 8823: 114453, 8824: 114449, 8825: 114430, 8826: 114520, 8827: 114526, 8828: 114626, 8829: 114699, 8830: 114570, 8831: 114607, 8832: 114622, 8833: 114641, 8834: 114724, 8835: 114775, 8836: 114750, 8837: 114714, 8838: 114725, 8839: 114742, 8840: 114822, 8841: 114855, 8842: 114844, 8843: 114948, 8844: 114745, 8845: 114863, 8846: 114921, 8847: 114937, 8848: 114996, 8849: 115129, 8850: 114939, 8851: 114831, 8852: 114971, 8853: 114924, 8854: 114904, 8855: 115062, 8856: 115015, 8857: 114981, 8858: 115033, 8859: 115054, 8860: 115022, 8861: 115036, 8862: 115836, 8863: 115102, 8864: 115065, 8865: 115115, 8866: 115126, 8867: 114984, 8868: 115142, 8869: 115144, 8870: 115120, 8871: 115178, 8872: 115088, 8873: 115148, 8874: 115152, 8875: 115171, 8876: 115191, 8877: 115272, 8878: 115227, 8879: 115257, 8880: 115250, 8881: 115218, 8882: 115271, 8883: 115312, 8884: 115261, 8885: 115280, 8886: 115245, 8887: 115355, 8888: 115360, 8889: 115433, 8890: 115404, 8891: 115407, 8892: 115438, 8893: 115444, 8894: 115395, 8895: 115510, 8896: 115496, 8897: 115476, 8898: 115537, 8899: 115500, 8900: 115528, 8901: 115620, 8902: 115567, 8903: 115591, 8904: 115590, 8905: 115623, 8906: 115669, 8907: 115713, 8908: 115756, 8909: 115769, 8910: 115759, 8911: 115738, 8912: 115768, 8913: 115755, 8914: 115833, 8915: 115806, 8916: 115830, 8917: 115839, 8918: 115770, 8919: 115908, 8920: 115907, 8921: 115906, 8922: 115915, 8923: 115919, 8924: 115953, 8925: 115996, 8926: 115990, 8927: 116030, 8928: 116060, 8929: 116097, 8930: 116076, 8931: 116106, 8932: 116118, 8933: 116119, 8934: 116146, 8935: 116250, 8936: 116181, 8937: 116231, 8938: 115746, 8939: 116247, 8940: 116264, 8941: 116292, 8942: 116307, 8943: 116310, 8944: 116323, 8945: 116399, 8946: 116368, 8947: 116354, 8948: 116355, 8949: 116389, 8950: 116365, 8951: 116428, 8952: 116380, 8953: 116465, 8954: 116495, 8955: 116505, 8956: 116539, 8957: 116653, 8958: 116591, 8959: 116602, 8960: 116592, 8961: 116584, 8962: 116582, 8963: 116611, 8964: 116613, 8965: 116631, 8966: 116737, 8967: 116709, 8968: 116758, 8969: 116771, 8970: 116768, 8971: 116714, 8972: 116728, 8973: 116781, 8974: 116727, 8975: 116820, 8976: 116805, 8977: 116824, 8978: 116851, 8979: 116853, 8980: 116889, 8981: 116876, 8982: 116901, 8983: 116918, 8984: 116928, 8985: 116912, 8986: 116941, 8987: 116957, 8988: 116971, 8989: 116948, 8990: 116962, 8991: 117020, 8992: 117054, 8993: 117075, 8994: 117105, 8995: 117125, 8996: 117088, 8997: 117073, 8998: 117089, 8999: 117107, 9000: 117133, 9001: 117219, 9002: 117218, 9003: 117221, 9004: 117245, 9005: 117265, 9006: 117315, 9007: 117324, 9008: 117301, 9009: 117314, 9010: 117299, 9011: 117340, 9012: 117375, 9013: 117371, 9014: 117420, 9015: 117445, 9016: 117452, 9017: 117430, 9018: 117447, 9019: 117450, 9020: 117472, 9021: 117494, 9022: 117491, 9023: 117507, 9024: 117503, 9025: 117500, 9026: 117510, 9027: 117541, 9028: 117551, 9029: 117567, 9030: 117628, 9031: 117629, 9032: 117689, 9033: 117683, 9034: 117681, 9035: 117710, 9036: 117718, 9037: 117722, 9038: 117712, 9039: 117730, 9040: 117756, 9041: 117761, 9042: 117774, 9043: 117797, 9044: 117860, 9045: 117863, 9046: 117880, 9047: 117887, 9048: 117927, 9049: 117931, 9050: 117932, 9051: 117939, 9052: 117957, 9053: 117956, 9054: 118029, 9055: 118048, 9056: 118027, 9057: 118071, 9058: 118076, 9059: 118077, 9060: 118092, 9061: 118114, 9062: 118121, 9063: 118116, 9064: 118131, 9065: 118178, 9066: 118188, 9067: 118209, 9068: 118224, 9069: 118234, 9070: 118214, 9071: 118243, 9072: 118268, 9073: 118277, 9074: 118281, 9076: 118322, 9077: 25, 9078: 34, 9079: 43, 9080: 63, 9081: 88, 9082: 107, 9083: 106, 9084: 122, 9085: 124, 9086: 137, 9087: 145, 9088: 171, 9089: 154, 9090: 0, 9091: 183, 9092: 186, 9093: 194, 9094: 207, 9095: 238, 9096: 253, 9097: 274, 9098: 301, 9099: 302, 9100: 330, 9101: 343, 9102: 345, 9103: 355, 9104: 379, 9105: 365, 9106: 357, 9107: 394, 9108: 377, 9109: 399, 9110: 418}
	data := []byte(`   1          BD+44 4550      3 36042          46           000001.1+444022000509.9+451345114.44-16.88 6.70  +0.07 +0.08         A1Vn               -0.012-0.018      -018      195  4.2  21.6AC   3
   2          BD-01 4525      6128569                       235956.2-010330000503.8-003011 98.33-61.14 6.29  +1.10 +1.02        gG9                 +0.045-0.060      +014V
   3 33    PscBD-06 6357     281285721002I         Var?     000013.0-061601000520.1-054227 93.75-65.93 4.61  +1.04 +0.89 +0.54   K0IIIbCN-0.5       -0.009+0.089 +.014-006SB1O < 17  2.5   0.0     3*
   4 86    PegBD+12 5063     87 917012004                   000033.8+125023000542.0+132346106.19-47.98 5.51  +0.90               G5III              +0.045-0.012      -002V?
   5          BD+57 2865    123 21085          61  V640 Cas 000101.8+575245000616.0+582612117.03-03.92 5.96  +0.67 +0.20         G5V                +0.263+0.030 +.047-012V          0.8   1.4      *
   6          CD-4914337    142214963      W                000108.4-493751000619.0-490430321.61-66.38 5.70  +0.52 +0.05         G1IV               +0.565-0.038 +.050+003SB         5.7   5.4      *
   7 10    CasBD+63 2107    144 109782005                   000114.4+633822000626.5+641146118.06  1.75 5.59  -0.03 -0.19         B9III             e+0.008 0.000      -000V     153                 *
   8          BD+28 4704    166 73743          69     33    000125.2+282811000636.8+290117111.26-32.83 6.13  +0.75 +0.33         K0V                +0.380-0.182 +.067-008V          2.6 158.6AB   4*
   9          CD-23    4    2031660531003                   000143.0-233947000650.1-230627 52.21-79.14 6.18  +0.38 +0.05         A7V                +0.100-0.045      +003V
  10          BD-18 6428    256147090                       000211.8-175639000718.2-172311 74.36-75.90 6.19  +0.14 +0.10         A6Vn               -0.018+0.036      -009V?    195                 *
  11          BD-03    2    315128595              Var?     000236.7-030620000744.1-023256 98.02-63.29 6.43  -0.14 -0.47         B8IIIpSi           +0.027-0.002      +013                          *
  12          CD-23   13    319166066          89     46    000240.3-230352000746.8-223032 55.56-79.07 5.94  +0.14 +0.06         A2Vp:              +0.052-0.044      -013V          5.1   1.9      *
  13          CD-34   17    344192367                       000258.6-340510000803.5-333146355.91-78.67 5.68  +1.12               K1III              -0.037 0.000      +007
  14          BD-03    3    3521286022006I         AP Psc   000304.8-030015000812.1-022652 98.34-63.24 6.07  +1.38 +1.14         K2III+F            +0.009-0.003      +001SB1O   22  2.0   0.0      *
  15 21Alp AndBD+28    4    358 73765   1I     94  Alp And  000313.0+283218000823.3+290526111.73-32.84 2.06  -0.11 -0.46 -0.10   B8IVpMnHg         v+0.136-0.163 +.032-012SBO    56  8.5  81.5      *
  16          BD-09    5    360128604                       000310.9-092247000817.4-084926 91.79-69.04 5.99  +1.04 +0.83        gG8                 -0.052-0.033      +020
  17          BD+35    8    400 53677                       000331.9+360426000841.0+363736113.45-25.45 6.19  +0.48 -0.09         F8IV               -0.100-0.145 +.044-014    =<  6
  18          BD-18    3    402147103    I            51    000327.1-180801000833.4-173439 74.69-76.25 6.06  +1.67 +1.97         M0III               0.000-0.025      -017V?                        *
  19          BD+24    3    417 73769                       000342.2+245419000852.2+252746110.97-36.42 6.23  +0.97 +0.73         K0III              +0.114+0.031      +015
  20          BD+78    1    431  4048         102           000348.5+790933000920.2+794253120.98 17.00 6.01  +0.19 +0.10         A7IV               +0.102-0.027 +.002+001       90  0.2   0.6      *
  21 11Bet CasBD+58    3    432 21133   2I    107  Bet Cas  000350.2+583554000910.7+590859117.52-03.27 2.27  +0.34 +0.11 +0.20   F2III-IV           +0.525-0.181 +.072+012SB     70 11.3  31.3      *
  22 87    PegBD+17    7    448 91734                       000352.8+173922000902.4+181243108.99-43.51 5.53  +1.04               G9III              +0.137-0.024      -023V?
  23          CP-54   19    469231943      W                000359.7-543333000902.4-540007316.25-62.02 6.33  +0.74 +0.39         G4IV               +0.051+0.016D+.009+001SB         1.3   0.2      *
  24   Kap1SclCD-28   16    493166083         111           000415.2-283240000921.0-275916 25.24-80.63 5.42  +0.42 +0.08         F2V                +0.072-0.004 +.023+009      131  0.2   1.4AB   3*
  25   Eps PheCD-46   18    496214983   3                   000420.2-461757000924.7-454451324.34-69.60 3.88  +1.03 +0.84 +0.52   K0III              +0.124-0.181 +.066-009
  26 34    PscBD+10    8    560 91750         122           000453.8+103521001002.3+110844106.87-50.43 5.51  -0.07 -0.24         B9Vn              e+0.041-0.003D+.017+014V     275  4.4   7.7      *
  27 22    AndBD+45   17    571 36123   4                   000507.2+453057001019.3+460420115.52-16.21 5.03  +0.40 +0.25 +0.29   F2II               +0.008 0.000 -.002-005       47                 *
  28          BD+56   11    584 21162                       000515.0+563632001029.7+570956117.38-05.26 6.74  -0.08 -0.41         B7IV               +0.023+0.006      +002
  29          BD-06   11    587128621      D                000511.6-054815001018.8-051455 96.99-66.03 5.84  +0.98 +0.74         K1III              +0.036-0.033      +024                          *
  30   Gam3OctCP-82    4    6362582153971                   000530.4-824648001002.1-821326304.63-34.77 5.28  +1.05 +0.92         G8III              -0.029-0.020      +015
  31          BD-13   13    6451471272008                   000535.5-130807001042.8-123448 87.69-72.60 5.85  +1.01 +0.80         K0IV               +0.150-0.038      +006V?
  32          CP-73    4    661255642      W                000544.4-734653001038.6-731328306.98-43.58 6.64  +0.37 +0.06         F2V+F6V            +0.123+0.016      -014           1.1   0.5AB   3*
  33  6    CetBD-16   17    693147133                       000610.5-160101001115.9-152805 82.24-75.06 4.89  +0.49 -0.03         F7V                -0.077-0.268 +.067+014V?      0
  34   Kap2SclCD-28   26    720166103   5I W                000629.8-282124001134.4-274759 26.30-81.13 5.41  +1.34 +1.46         K5III              +0.010+0.016      -006          13.8  46.0
  35   The SclCD-35   42    739192388   6                   000639.0-354134001144.0-350759347.16-78.34 5.25  +0.44               F4V                +0.173+0.119 +.034-002        0
  36          BD+47   21    743 36148    I                  000645.1+473545001159.1+480909116.16-14.20 6.16  +1.45              gK4                 +0.058+0.017      +016
  37          BD-18   14    787147144    I                  000704.0-182938001210.0-175618 76.32-77.10 5.25  +1.48 +1.63         K5III              +0.056-0.031 -.015-008V    < 19:
  38          BD+36   12    829 53725                       000738.2+370815001250.4+374136114.55-24.55 6.73  -0.13 -0.70         B2V                +0.028-0.012      -010                          *
  39 88Gam PegBD+14   14    886 91781   7I'W       Gam Peg  000805.1+143740001314.2+151101109.43-46.68 2.83  -0.23 -0.87 -0.19   B2IV               +0.003-0.012 -.002+004SBO     3  8.9 163.4AB   3*
  40          BD+26   13    895 73823         161           000813.4+262556001324.0+265914112.56-35.12 6.30  +0.65 +0.33         G0III              -0.008-0.047D+.006-013           1.1   0.1AB   3*
  41 23    AndBD+40   29    905 361732010                   000819.0+402900001330.8+410207115.27-21.27 5.72  +0.31 -0.02         F0IV               -0.123-0.148 +.024-029       25
  42          CD-26   56    942166130    I            96    000837.8-263435001342.1-260119 38.28-81.49 5.94  +1.55               K5III              +0.026-0.069      -030V
  43          CD-26   57    943166131    I                  000840.2-265029001344.2-261705 36.52-81.54 6.31  +1.48 +1.65         K5III              -0.020+0.012      +018V
  44          BD+32   21    952 537442012                   000850.6+323901001402.3+331222113.99-29.02 6.25  -0.01 -0.05         A1V                -0.006-0.018      +001       56
  45 89Chi PegBD+19   27   1013 917921004I            99    000925.6+193902001436.2+201224111.30-41.83 4.80  +1.57 +1.93 +1.13   M2+III            e+0.093 0.000 +.015-046V                         *
  46          BD-08   26   1014128655    I    180  AD Cet   000920.8-082013001427.6-074650 96.87-68.76 5.12  +1.62 +1.81 +1.03E  M3+III             +0.059+0.004 +.003-002V?         6.0   3.0      *
  47          CP-85    2   10322582173972                   000932.1-853302001319.4-845939303.91-32.06 5.77  +1.72 +2.10         M0-1III            +0.004+0.008      +004
  48  7    CetBD-19   21   1038147169    I         AE Cet   000933.7-192913001438.4-185558 75.11-78.23 4.44  +1.66 +1.99 +1.14   M3III             e-0.024-0.067 +.029-023                          *
  49          BD+21   13   1048 73838                       000945.3+214343001456.1+221703111.92-39.81 6.24  -0.01 +0.12         A1pSi              +0.065-0.011      -015                          *
  50 35    PscBD+08   19   1061109087         191  UU Psc   000949.7+081556001458.8+084915107.86-52.98 5.79  +0.31 +0.04         F0IV               +0.097-0.028D+.017+001SB2O   86  0.0   0.0 A   3*
  51          BD-10   30   1064128660                       000948.2-100731001454.5-093411 95.06-70.44 5.75  -0.08               B9V                +0.028-0.011      +017V?
  52          BD+30   26   1075 53755    I                  000954.9+305848001507.0+313209113.93-30.70 6.45R                     K5                 +0.035-0.005      +002
  53          BD+26   23   1083 73842      W                000959.3+264340001510.6+271659113.10-34.90 6.35  -0.02 +0.02         A1Vn               +0.021-0.031      -004SB    230  6.8  29.2      *
  54          CD-35   65   1089192418                       000955.3-352736001458.2-345416345.80-78.99 6.17  +1.34               K3III              +0.073-0.023      -052
  55          BD+76    5   1141  4071         207           001033.1+762342001614.0+765703120.89 14.22 6.35  -0.07 -0.26         B8Vnn              +0.018-0.002D+.013-007V          0.3   0.8      *
  56          BD+42   41   1185 36221         215           001106.2+430224001621.6+433541116.23-18.82 6.15  +0.05 +0.03         A2VpSi             +0.043-0.030      +002      100  3.9   9.2      *
  57          CD-32   72   11871924302014I W                001105.4-320006001608.9-312647  1.53-81.18 5.67  +1.35 +1.50         K5III              +0.122-0.030      +026           9.9  64.0      *
  58          CP-76   19   1221255650                       001120.4-762803001555.2-755441305.79-41.02 6.49  +1.00 +0.72         G8-K0III           -0.013+0.006      +016
  59 36    PscBD+07   27   12271091002015                   001125.7+074105001634.1+081424108.28-53.64 6.11  +0.92 +0.66         G8II-III           -0.020-0.019      +001
  60          BD+60   21   1239 11084         222           001134.6+605839001657.1+613200118.83-01.06 5.74  +0.88 +0.59         G8III              -0.001-0.001      -004           6.4  19.4      *
  61          BD-21   24   1256166167                113    001138.1-204557001642.5-201238 72.13-79.47 6.47  -0.09 -0.48         B8III              +0.012-0.014      +019V                         *
  62          BD+47   50   1279 362362016                   001152.3+472331001709.1+475651117.01-14.53 5.89  -0.09 -0.44         B7III              +0.008 0.000      -009V?      0
  63 24The AndBD+37   34   1280 53777                116    001151.9+380735001705.5+384054115.62-23.70 4.61  +0.06 +0.04 +0.01   A2V                -0.050-0.019 +.022+001V     107                 *
  64          CP-79    7   1324255652                       001222.8-792007001649.0-784650305.07-38.20 6.77  +0.46 +0.08         F3III              +0.077-0.043      -013
  65          BD+50   46   1337 21273    I         AO Cas   001224.8+505239001743.0+512559117.59-11.09 6.14  -0.13 -0.97         O9IIInn            -0.005-0.004      -035SB2O  135                 *
  66          BD-19   30   1343147205                       001228.2-193624001732.6-190304 77.21-78.78 6.45  +0.37               F4IV-V             +0.015+0.003      -005
  67          BD+00   28   13671091192017                   001239.5+010758001747.7+014120105.89-60.07 6.17  +0.94 +0.73         K0II               +0.095+0.012      -009
  68 25Sig AndBD+35   44   1404 537981005            118    001306.0+361351001819.7+364707115.59-25.61 4.52  +0.05 +0.07  0.00   A2V                -0.063-0.041 +.023-008SB    103                 *
  69          BD+10   25   1419 91832                       001308.3+103905001817.2+111221110.00-50.83 6.05  +1.03               K0III              -0.036-0.038      +009
  70 26    AndBD+42   48   1438 36256         254    119    001325.7+431409001842.1+434728116.70-18.68 6.11  -0.08 -0.35         B8V                +0.020-0.003      +007           4.1   6.2      *
  71          BD+30   35   1439 538031006                   001324.7+305743001838.3+313102114.79-30.84 5.87  -0.01  0.00         A0IV               +0.058-0.006      -005       53
  72          BD-08   38   1461128690                       001333.0-083616001841.8-080310 99.29-69.40 6.46  +0.68 +0.29         G0V                +0.413-0.139 +.053-010V?                        *
  73          CD-43   64   1483215047                       001343.7-434728001842.6-431407323.20-72.58 6.33  +1.21               K2III              +0.056+0.012      +012
  74  8Iot CetBD-09   48   1522128694   9I W         123    001419.9-092242001925.7-084926 98.98-70.19 3.56  +1.22 +1.25 +0.59   K1.5III            -0.014-0.036 +.013+019     < 17  4.8 108.8AC   3*
  75          BD+39   56   1527 36269                       001425.9+401029001941.6+404347116.46-21.74 6.33  +1.18               K1III              -0.026-0.010      -038
  76          BD+48   79   1561 36272                       001446.3+481838002005.2+485155117.63-13.68 6.52                      A0V s              +0.005-0.017      -003V      57
  77   Zet TucCP-65   13   1581248163  10I'                 001451.7-652745002004.3-645229308.35-51.90 4.23  +0.58 +0.02 +0.33   F9V                +1.705+1.164 +.134+009        0
  78          BD+30   42   1606 53820                128    001510.7+302250002024.4+305609115.13-31.47 5.90  -0.10 -0.46         B7V                +0.021+0.004      +004      125
  79          BD+32   45   1632 53825    I                  001531.6+322124002045.5+325441115.53-29.52 5.79  +1.59       +1.00   K5III              -0.026-0.014      -036
  80 41    PscBD+07   36   16351091521008I           132    001527.0+073806002035.9+081125109.91-53.90 5.37  +1.34 +1.55        gK3                 -0.002+0.008      +016
  81          BD+10   32   1663 91858         287           001545.4+102522002054.5+105837110.93-51.18 6.56                      A0V                -0.046-0.028D+.005-018V      56  0.2   0.6      *
  82 27Rho AndBD+37   45   1671 538281009                   001551.1+372453002107.3+375807116.37-24.52 5.18  +0.42 +0.05         F5III              +0.060-0.040 +.017+009       41
  83   Pi  TucCP-70   12   16852481672018                   001600.9-701048002039.0-693730306.87-47.27 5.51  -0.05 -0.14         B9V                -0.007 0.000 +.011+012
  84   Iot SclCD-29   86   1737166207                       001629.7-293204002131.2-285854 15.52-83.15 5.18  +1.00 +0.82         G5III              +0.040-0.074 +.027+021
  85          BD-20   50   1760166210    I         T Cet    001642.6-203645002146.3-200328 77.51-80.20 5.12  +1.82 +1.78         M5IIe              +0.071-0.005      +029V                         *
  86 42    PscBD+12   25   1796 918662020     303           001714.9+125538002225.5+132857112.19-48.78 6.23  +1.22               K3III              +0.062+0.021      +003V          4.6  28.9      *
  87          CP-78    9   1801255663                       001712.7-775854002128.6-772537305.04-39.58 5.97  +1.40 +1.40         K3III              +0.002-0.010      +021V
  88  9    CetBD-13   60   1835147237    I'W       BE Cet   001744.3-124557002251.8-121234 97.34-73.64 6.39  +0.66 +0.24         G2V                +0.394+0.061 +.049-007V       6  5.4 202.1      *
  89          CD-31  138   1909192495                       001812.3-313526002312.6-310210358.89-82.70 6.55   0.00 -0.31         B9IVMn             +0.029-0.003      +001SB1    25                 *
  90          BD+37   58   1967 53860    I:W       R And    001844.8+380125002402.0+383438117.07-23.98 7.39  +1.97 +1.25 +2.21   S6.5IIIeZr6Ti2     -0.007-0.023      -011V          4.9  84.6      *
  91          BD+51   62   1976 21366         328    155    001852.2+512757002415.6+520112118.68-10.63 5.57  -0.12 -0.60         B5IV               +0.013-0.004D+.004-007SBO   230  2.5   0.2      *
  92 NOVA 1572                                     B Cas                                                                                                                                            *
  93 12    CasBD+61   69   2011 11172                       001916.2+611637002447.5+614952119.79-00.88 5.40   0.00 -0.16         B9III              +0.012-0.001 +.015-004V     154
  94          BD-03   49   20231287432021                   001923.0-024620002429.7-021309107.27-64.27 6.07  +1.22 +1.23         K1III              -0.033-0.044      +015
  95 47    Tuc                                                                                                                                                                                      *
  96          BD+52   61   2054 213812022            159    001941.6+522934002506.4+530249118.92-09.62 5.74  -0.06 -0.31         B9IV               +0.023-0.002      +010V     210                 *
  97 44    PscBD+01   57   21141091921010  W                002016.5+012310002524.2+015623109.73-60.26 5.77  +0.86 +0.57         G5III              -0.013-0.015      -004     < 20                 *
  98   Bet HyiCP-77   16   2151255670  11I'          161    002030.0-774903002545.1-771515304.81-39.77 2.80  +0.62 +0.11 +0.34   G2IV               +2.215+0.324 +.153+023                          *
  99   Alp PheCD-42  116   2261215093  12I A                002120.5-425056002617.0-421822320.02-73.98 2.39  +1.09 +0.88 +0.60   K0III              +0.203-0.396 +.035+075SB1O             0.1      *
 100   Kap PheCD-44  101   2262215092                       002117.1-441405002612.2-434048318.42-72.68 3.94  +0.17 +0.11 +0.08   A7V                +0.109+0.029 +.072+011      219
 101 10    CetBD-00   63   2273128760              Var?     002129.6-003612002637.4-000259109.50-62.27 6.19  +0.90 +0.55         G8III              +0.073-0.001      -023V                         *
 102          CD-26  138   23631662822025                   002214.0-260601002714.7-253250 47.84-84.35 5.98  +1.03 +0.88         G8III              +0.036-0.019      -003
 103 47    PscBD+17   55   2411 91910    I         TV Psc   002250.0+172021002802.9+175335115.10-44.62 5.06  +1.65 +1.82 +1.54   M3III             v+0.117+0.021 +.016+006V                         *
 104          BD+43   92   2421 363902027                   002251.1+435029002813.7+442340118.57-18.28 5.17  +0.03 -0.01         A2V s              +0.089-0.016 +.013+002SB1O   36                 *
 105   Eta SclCD-33  152   24291925452026          Eta Scl  002258.2-333333002755.7-330026342.36-82.23 4.81  +1.64 +1.81 +1.38   M4III              -0.019-0.052 -.014+011                          *
 106 48    PscBD+15   63   2436 919121012I                  002300.9+155332002812.7+162642114.89-46.06 6.06  +1.58 +2.00         K5III              +0.019-0.015      -007SB
 107          BD+09   47   24541092242028                   002309.8+093832002820.1+101123113.62-52.26 6.04  +0.43 -0.06         F6Va vw            +0.046-0.207 +.033-010V       8                 *
 108          BD-21   57   24751662962029  W                002320.2-205307002821.1-202006 83.87-81.40 6.43  +0.59 -0.03         G0V                -0.116-0.114D+.034+006V          0.1   0.2      *
 109          CD-40   93   2490215103    I                  002330.7-402802002826.4-395454321.94-76.35 5.43  +1.56 +1.90         M0III              +0.130-0.036 +.015+032                          *
 110          BD+36   66   2507 53928                       002337.9+362048002856.6+365400117.94-25.75 6.26  +0.92               G5III              -0.011+0.003      +010
 111          CD-51  113   2529232055                       002352.8-510509002843.1-503158311.91-66.20 6.26  +1.09               K0III              +0.150-0.002      +006
 112          BD+76   10   2589  41302031                   002429.4+762804003055.0+770110121.74 14.20 6.21  +0.84 +0.54         K0IV               +0.337-0.038 +.023+019                          *
 113          BD+59   68   2626 21457         412           002444.9+592530003019.9+595838120.29-02.79 5.94  +0.01 -0.36         B9IIIn             +0.017-0.012D+.002-017V     225: 2.2   0.5AB   3*
 114 28    AndBD+28   75   2628 74041         409  GN And   002450.6+291201003007.3+294506117.42-32.89 5.23  +0.24 +0.09         A7III              +0.041-0.054 +.001-010       21  7.1 139.3AC   3*
 115          BD-15   84   26301473072030                   002447.8-152458002951.9-145151 99.58-76.75 6.14  +0.37               F2III              +0.144-0.032      +003       42                 *
 116          CD-32  154   2632192567                       002451.6-324005002948.9-320700344.92-83.15 6.57  +1.34               K4III              -0.004-0.054      +001
 117 12    CetBD-04   54   2637128791  13I    410           002456.1-043035003002.4-035726109.57-66.27 5.72  +1.56 +1.91         M0III              +0.011-0.012      +005V?         5.6  10.6      *
 118          CD-24  179   2696166318  14                   002522.7-242027003022.7-234716 66.87-84.19 5.19  +0.12               A5Vn               -0.024+0.013 +.018-000SB                        *
 119          CD-41  116   2724215120              BB Phe   002534.7-412934003027.8-405622318.97-75.53 6.19  +0.33 +0.14         F2III              -0.001+0.021      -005                          *
 120          CD-48  102   2726215119                       002535.8-484555003026.1-481254312.52-68.52 5.69  +0.35 +0.05         F2V                +0.129-0.088      +002
 121 13    CasBD+65   67   2729 11243                       002539.8+655802003125.3+663110120.94  3.73 6.18  -0.10 -0.47         B6V                +0.023-0.006      -010                          *
 122          BD+32   80   2767 53956      W                002606.8+330147003125.6+333454118.16-29.10 5.87  +1.14               K1III              +0.047-0.019      +009SB         2.4  56.4
 123 14Lam CasBD+53   82   2772 21489         434           002615.1+535813003146.4+543120120.05-08.24 4.73  -0.10 -0.35 -0.12   B8Vn               +0.045-0.010 +.030-012V          0.2   0.5      *
 124          BD+52   92   2774 21486    I                  002612.4+521715003141.2+525022119.91-09.92 5.60  +1.15 +1.13         K2III              -0.053-0.019 +.008-052SB
 125   Lam1PheCD-49  115   2834215131  15  W                002635.6-492123003125.0-484813311.75-67.98 4.77  +0.02 +0.04         A0V                +0.143+0.017 +.027-002SB    109  8.8  30.3      *
 126   Bet1TucCP-63   50   2884248201      W     A          002657.7-633033003132.7-625729306.78-54.02 4.37  -0.07 -0.17 -0.07   B9V                +0.090-0.054 +.030+014SB    173  0.2  27.0AC   6*
 127   Bet2TucCP-63   50   2885248202      W     C          002658.4-633100003133.6-625757306.78-54.01 4.54  +0.15 +0.03 +0.08   A2V+A7V            +0.101-0.056 +.030+010       50  0.2  27.0AC   6*
 128          BD+42   99   2888 36453                       002703.0+425635003226.8+432941119.29-19.24 6.70  -0.01 -0.20         A1Vn               +0.018-0.010      -015SB    265                 *
 129          BD+70   24   2904  41422034                   002720.9+702548003319.3+705854121.44  8.16 6.42  -0.01 -0.06         A0Vn               +0.028-0.004      -010      154
 130 15Kap CasBD+62  102   2905 11256  16          Kap Cas  002718.6+622248003300.0+625554120.84  0.14 4.16  +0.14 -0.80 +0.06   B1Ia              e+0.003-0.003      -002SB     62                 *
 131 52    PscBD+19   79   2910 740842033     452           002720.8+194437003235.5+201740116.94-42.36 5.38  +1.08 +0.96         K0III              +0.135-0.047 +.032-013V          6.8  49.6
 132 51    PscBD+06   64   29131092622032     449  Var?     002714.1+062412003223.8+065720114.55-55.61 5.67   0.00 -0.17         B9.5V              +0.035+0.002      +017      125  0.7   0.0 O   4*
 133          BD+26   76   2924 74083                       002718.2+270144003234.5+273450117.82-35.10 6.67                      A2IV               -0.017-0.015      +002
 134          BD+27   84   2942 74090         455           002732.4+274341003249.1+281649117.96-34.41 6.30  +1.00 +0.73         K0III              -0.006+0.007 +.006-012SB         4.9   8.5AB   3*
 135          BD+54  101   2952 21512                       002737.3+542039003310.4+545342120.28-07.88 5.93  +1.04 +0.84         K0III              +0.071-0.042      -035
 136   Bet3TucCP-63   52   3003248208      W                002810.4-633456003243.8-630152306.54-53.97 5.09  +0.04 +0.02         A0V                +0.081-0.038 +.049+005V      84  0.2   0.1      *
 137 16    CasBD+65   70   3038 11265                       002835.1+661156003424.9+664501121.25  3.93 6.48  -0.08 -0.32         B9III              +0.021 0.000      -016SB
 138          CD-30  156   30591663671013I                  002844.3-300633003341.1-293330  0.01-85.40 5.55  +1.27 +1.23         K2III              -0.026-0.030      -009SB                        *
 139   The TucCP-71   20   31122556792035          The Tuc  002908.9-714904003323.3-711558305.01-45.79 6.13  +0.23 +0.20         A7IV               +0.067-0.012      +002       52                 *
 140          CP-53  117   31582320911014                   002942.3-525532003427.8-522223308.97-64.55 5.57  +0.47  0.00         F3IV-V             +0.230+0.032 +.038+035
 141          BD+12   57   3166 91980                       002944.1+124917003455.3+132216116.77-49.30 6.40                      K0                 -0.090-0.057      -019V
 142 13    CetBD-04   62   3196128839         490    212    003006.0-040836003514.9-033534112.89-66.15 5.20  +0.56 +0.08         F8V                +0.413-0.026 +.060+009SBO    18  0.7   0.3AB   3*
 143 14    CetBD-01   68   32291288432036                   003024.7-010318003532.8-003020114.12-63.10 5.93  +0.44 -0.02         F5IV               +0.146-0.066      +006V   =< 10
 144          BD+53  102   3240 21551                       003034.1+533703003608.3+541007120.67-08.64 5.08  -0.11 -0.36         B7III              +0.022 0.000      +001V      58
 145          BD+12   59   3268 91990                       003043.9+123940003554.7+131224117.11-49.49 6.41  +0.52 -0.05         F7V                -0.143-0.189 +.020-025    =< 15
 146          BD+59   84   3283 11291                       003046.0+594632003627.3+601934121.08-02.49 5.79  +0.29 +0.29         A4III              -0.001-0.003      -009V?                        *
 147   Lam2PheCD-48  121   3302215157                       003055.0-483257003541.1-480003310.27-68.90 5.51  +0.44 -0.01         F6V                +0.045-0.104      +008        0
 148          CP-55  117   3303232099                219    003053.4-552217003533.4-544919307.83-62.16 6.06  +1.01               K0IV               +0.027-0.054      -010
 149          BD+26   91   3322 74136                221    003102.6+264215003620.0+271517118.81-35.50 6.50  -0.09 -0.39         B8IIIpHgMn:        +0.011-0.016      +001SB?    28                 *
 150          BD-15  109   3325147360                       003101.5-153120003603.0-145825105.75-77.33 6.45  +1.06 +0.91         G9III              -0.083-0.079      +012V?
 151          CD-23  220   33261664002037          BG Cet   003108.3-232331003606.9-225033 83.10-84.48 6.06  +0.29               A6pSr              -0.072-0.052      +013                          *
 152          BD+43  113   3346 36509    I                  003120.1+435613003646.6+442919120.18-18.31 5.13  +1.60 +1.97         K5-M0III           -0.023+0.033 +.007-033V    < 17
 153 17Zet CasBD+53  105   3360 21566  17            225    003123.7+532048003658.3+535349120.78-08.91 3.66  -0.20 -0.87 -0.21   B2IV               +0.019-0.009 +.004+002SB?    18                 *
 154 29Pi  AndBD+32  101   3369 54033  18     513    227    003132.2+331008003652.9+334310119.47-29.05 4.36  -0.14 -0.55 -0.12   B5V                +0.015-0.004 -.003+009SB1O   34  4.2  35.2AB   3*
 155 53    PscBD+14   76   3379 919952039          AG Psc   003134.6+144053003647.3+151354117.70-47.49 5.89  -0.15 -0.66         B2.5IV            v+0.002-0.015      -009SB                        *
 156          BD+23   84   3411 74148                       003151.1+232754003707.2+240051118.74-38.74 6.47R                     K2III              -0.014-0.044      -001
 157          BD+34   86   3421 54038                       003159.7+345057003721.1+352358119.70-27.38 5.48  +0.88 +0.48         G2.5IIa            -0.016-0.005      -000
 158          BD+81   13   3440   1063941            235    003212.4+815630003947.3+822938122.53 19.63 6.40  +0.55 -0.01        dF6                 -0.111+0.078 +.028-033V
 159          CD-25  225   3443166418         520           003212.5-251903003720.7-244602 68.68-86.03 5.57  +0.72 +0.24 +0.27E  G8V                +1.390-0.012 +.076+017SB2     2: 0.2   0.5      *
 160          CP-65   58   3444248225                       003213.2-654029003637.4-650729305.46-51.93 6.42  +1.26 +1.38         K2-3III            +0.029-0.020      +020
 161          BD+02   80   34571093152040                   003221.5+023512003730.5+030807116.06-59.55 6.39  +1.33 +1.52 +0.54E  K4III              +0.095-0.061      +004V                         *
 162          CP-55  130   3488232114                       003238.7-545639003718.1-542339307.41-62.61 6.41  +1.00               K0II-IIICNIV       +0.056-0.021      -006
 163 30Eps AndBD+28  103   3546 74164  19I                  003316.1+284608003833.3+291842119.57-33.47 4.37  +0.87 +0.47 +0.51   G6IIIFe-3CH1       -0.227-0.254 +.031-084V       9
 164          BD+48  192   3574 36550    I    546           003338.0+484818003909.9+492116120.88-13.47 5.43  +1.64               K7III              +0.008-0.011      -008V          5.3  13.3      *
 165 31Del AndBD+30   91   3627 54058  20I:   548           003358.7+301850003919.7+305139119.87-31.94 3.27  +1.28 +1.48 +0.66   K3III              +0.136-0.091 +.028-007SB1O < 17  8.2  28.7AB   3*
 166 54    PscBD+20   85   3651 74175      W         245    003409.5+204240003921.8+211502119.18-41.53 5.87  +0.85 +0.57 +0.39   K0+V               -0.464-0.370 +.099-034V          4.9 122.0      *
 167 55    PscBD+20   87   3690 74182    I    558           003439.6+205323003955.6+212618119.36-41.35 5.36  +1.16 +1.05         K0III+F3V          +0.028-0.035 +.009-017V          3.2   6.6      *
 168 18Alp CasBD+55  139   3712 21609  21I    561  Alp Cas  003449.7+555920004030.5+563214121.42-06.30 2.23  +1.17 +1.13 +0.60   K0IIIa             +0.053-0.032 +.016-004V?     21  6.7  64.4AD   4*
 169          CP-73   42   3719255690                       003445.3-734115003840.8-730814304.22-43.96 6.85  +0.11 +0.12 +0.06C  A1m                -0.017+0.020      +021V?
 170          CD-34  224   3735192663              Z Scl    003502.6-343028003957.9-335742322.13-82.73 6.69  +0.51 -0.03         F8V               v+0.333-0.116 +.020-011                          *
 171          CD-45  201   3750215185                       003505.9-452046003952.0-444748309.65-72.18 6.01  +1.14 +1.09         K1IIICNII          +0.045 0.000      +008
 172          BD-17  109   3794147395      W                003527.6-170354004028.6-163101108.92-79.09 6.49  +0.92 +0.53         G5                 +0.037-0.040      +048V          2.5 105.0AB   3
 173          CD-24  263   3795166475                       003530.7-242040004032.9-234816 85.79-85.86 6.14  +0.70 +0.21         G3V                +0.644-0.333 +.043-053V
 174          BD-05  101   38071288912042  W                003536.9-045402004042.4-042107116.05-67.08 5.91  +1.10 +0.89         K0III              -0.013-0.013      +035V          2.5  64.3
 175 32    AndBD+38   90   3817 540792043                   003541.7+385435004107.2+392731120.76-23.37 5.33  +0.89 +0.60 +0.46   G8III              -0.010-0.007 +.010-005V    < 17                 *
 176          CP-60   46   3823232143                       003545.2-600058004026.4-592716305.56-57.61 5.89  +0.55 +0.01         G1V                +0.889+0.448 +.058+002
 177          BD+65   83   3856 113362045                   003605.6+653557004203.4+660851121.98  3.29 5.83  +1.04 +0.87         G9III-IV           -0.013-0.009      -003
 178          BD+23   94   3883 742002044  W         258    003617.6+240451004136.0+243745120.09-38.19 6.04  +0.26 +0.22 +0.11   A7m                +0.107-0.016      -015       25                 *
 179 19Xi  CasBD+49  164   3901 216372046            260    003629.0+495751004203.9+503045121.41-12.33 4.80  -0.11 -0.63 -0.12   B2V                +0.015-0.005      -010SB1   211                 *
 180   Mu  PheCD-46  180   39192151941015                   003636.0-463802004119.6-460506308.31-70.94 4.59  +0.97 +0.72 +0.52   G8III              -0.014-0.002 +.004+019V?                        *
 181          BD+57  132   3924 21642                       003645.8+581219004231.1+584512121.77-04.10 6.17  -0.01 -0.11         B9.5III            +0.036-0.007      -003      100
 182 M 31  And                                     S And                                                                                                                                            *
 183   Xi  PheCP-57  143   3980232152      W       Xi  Phe  003712.9-570305004146.4-563006305.65-60.57 5.70  +0.20               ApSrEuCr           +0.085+0.044      +010           4.4  13.2      *
 184 20Pi  CasBD+46  146   4058 36602                268    003755.8+462839004328.1+470129121.52-15.82 4.94  +0.18 +0.09         A5V                -0.020-0.034 +.023+013SB2O   58                 *
 185   Lam1SclCD-39  175   4065192690      W                003754.2-390041004242.9-382748311.54-78.52 6.06  -0.03 -0.13         A0V                +0.003-0.005      +010V          0.3   0.6      *
 186          CP-60   48   4088248238                       003812.5-604835004241.8-601545304.92-56.83 5.98  +1.32 +1.45         K5III              +0.255-0.045      +026                          *
 187   Rho TucCP-66   47   40892482372047                   003812.0-660103004228.4-652805304.43-51.63 5.39  +0.50 +0.02         F6V                +0.053+0.042 +.038+014SBO     0                 *
 188 16Bet CetBD-18  115   4128147420  22I           272    003834.2-183208004335.4-175912111.31-80.68 2.04  +1.02 +0.87 +0.51   G9.5IIICH-1        +0.234+0.033 +.061+013V?     18                 *
 189          BD+47  181   4142 36617                       003852.8+471858004426.4+475151121.72-14.99 5.67  -0.12 -0.56         B5V                -0.026+0.015      -060SB    177                 *
 190          BD-12  126   4145147422                       003847.9-123314004350.2-120042115.84-74.78 6.02  +1.10 +1.09         K2III-IV           +0.007-0.204      -009V?
 191   Eta PheCP-58   42   4150232162  23  W                003851.7-580041004321.2-572747305.08-59.63 4.36   0.00 -0.02  0.00   A0IV               -0.006+0.011 +.046+002V      96  7.0  19.8
 192 21    CasBD+74   27   4161  4216  24     624  YZ Cas   003902.2+742629004539.0+745917122.55 12.12 5.66  +0.05 +0.07         A2IV               -0.018-0.026      +009SBO    27  5.5  36.0      *
 193 22Omi CasBD+47  183   4180 36620  25     622  Omi Cas  003908.9+474414004443.5+481704121.78-14.57 4.54  -0.07 -0.51 -0.05   B5IIIe             +0.019-0.008      -017SB1O  260  7.2  33.6      *
 194 17Phi1CetBD-11  128   41881474232048I           278    003908.7-110915004411.4-103634116.69-73.39 4.76  +1.01 +0.84 +0.51   K0III             v-0.009-0.113 +.022+001     < 17                 *
 195   Lam2SclCD-39  181   4211192703  26                   003922.0-385821004412.1-382518310.14-78.60 5.90  +1.16 +1.18         K1III              +0.244+0.113 +.010+027
 196          BD+54  143   4222 21677         625           003935.0+544027004517.2+551318122.05-07.64 5.42  +0.04 +0.04         A2V s              -0.025+0.002 +.012-009V      59  5.4  89.0AC   3*
 197          BD-22  127   4247166528                       003947.5-223321004444.4-220022106.02-84.66 5.24  +0.33               F1III-IV           -0.066+0.083 +.048+012       32
 198          CD-43  207   42932152211017                   004013.4-431316004457.1-424036307.36-74.40 5.94  +0.28               F0V                -0.086-0.102      +010SB
 199          CP-63   72   4294248243      W                004010.5-630243004432.3-622952304.31-54.61 6.07  +0.44 +0.13         F5III-IV           +0.143-0.005      -004V?         1.8   2.5      *
 200          BD+68   49   4295 11380                       004022.1+684642004639.0+691930122.50  6.46 6.33R                     F3V                +0.196+0.001      -014       15
 201          BD-05  120   4301128935    I                  004018.6-051038004524.1-043745119.00-67.46 6.15  +1.62 +2.01         M0III              +0.028+0.033      +007V
 202          CP-54  166   43042321682049                   004024.0-541544004500.0-534254305.07-63.39 6.15  +0.53 -0.08         F7III              +0.230 0.000      +014                          
 203 18    CetBD-13  128   4307147432      W                004027.3-132520004528.7-125251117.05-75.68 6.15  +0.61 +0.10         G0V                -0.034-0.201      -013           6.3  69.5      *
 204          BD+54  148   4321 21689                       004031.6+544531004615.1+551819122.19-07.56 6.52                      A2III              +0.015-0.012      -008
 205          BD+44  160   4335 36640                       004038.4+441853004610.9+445141121.95-18.00 6.05  -0.07 -0.27         B9.5IIIpHgMnCr     +0.028-0.009      +000SB     25                 *
 206          BD-17  132   4338147436         636           004041.7-165816004541.7-162527115.55-79.21 6.47  +0.31               F1IVn              +0.014-0.005      -000           3.0   3.0
 207          BD+58  101   4362 21693                       004052.4+590140004642.4+593428122.33-03.29 6.39  +1.09 +0.77         G0Ib               -0.008-0.001      -015V
 208 23    CasBD+74   29   4382  4226                       004105.0+741805004746.1+745051122.69 11.98 5.41  -0.08 -0.39         B8III              +0.015-0.009      -003SB2O    4                 *
 209          CD-48  176   4391215232      W                004104.1-480604004545.6-473306305.68-69.54 5.80  +0.64               G1V                +0.184+0.086 +.049-011           7.7  14.3
 210          CD-23  293   4398166555                       004113.6-230407004611.8-223119108.14-85.24 5.50  +0.98               G3V                +0.201-0.006 +.021-015V                         *
 211 57    PscBD+14  111   4408 92072    I           293    004118.9+145549004633.0+152832121.19-47.38 5.38  +1.65 +1.76         M4IIIa             -0.027-0.044 +.015-027V                         *
 212          BD+71   37   4440  4229                       004136.8+720741004809.1+724030122.68  9.80 5.87  +1.01 +0.82         K0IV               +0.130+0.024      +001V
 213 58    PscBD+11   96   4482 920802050                   004148.4+112542004701.5+115826121.22-50.88 5.50  +0.97               G8II               +0.058-0.036      -001
 214 59    PscBD+18  101   4490 920822051          XX Psc   004156.4+190156004713.6+193444121.57-43.28 6.13  +0.27 +0.18         F0Vn               +0.104+0.011      +000V?    163                 *
 215 34Zet AndBD+23  106   4502 74267  27I W       Zet And  004202.1+234324004720.3+241602121.74-38.59 4.06  +1.12 +0.90 +0.59   K1IIe              -0.100-0.083 +.037-024SB1O   40  5.6 162.7AD   4*
 216 60    PscBD+05  104   4526109461                       004213.2+061142004723.6+064427121.13-56.12 5.99  +0.94 +0.70         G8III              +0.014-0.015      +014V
 217 61    PscBD+20  105   4568 74280                       004236.4+202244004754.8+205531121.82-41.94 6.54  +0.50 +0.03         F8V                +0.157+0.006 +.022+001V                         *
 218          BD-18  127   4585147451    I                  004244.3-183630004743.3-180341117.34-80.89 5.70  +1.30 +1.46         K3III              +0.043+0.031      +002V
 219 24Eta CasBD+57  150   4614 21732    I:   671           004303.0+571706004906.0+574857122.60-05.05 3.44  +0.57 +0.01 +0.36   F9V+dM0            +1.100-0.529 +.176+009SB1O=<  6  4.0  11.6AB   8*
 220          BD-22  134   46221665851018                   004304.1-221605004801.1-214321114.55-84.54 5.57  -0.06 -0.12         B9V                +0.028-0.011      +021V      53
 221 62    PscBD+06  105   4627109470                       004306.0+064514004817.4+071800121.55-55.56 5.93  +1.10 +1.00         G8III              +0.111+0.004      -000V?
 222          BD+04  123   46281094711019  W                004308.2+044600004823.0+051650121.49-57.57 5.75  +0.88 +0.59 +0.47   K2V                +0.758-1.146 +.145-013V          4.6 181.2
 223 25Nu  CasBD+50  147   4636 21729                302    004309.8+502522004850.1+505806122.51-11.90 4.89  -0.11 -0.42         B9III              +0.035-0.006      +009V     210
 224 63Del PscBD+06  107   4656109474  28I W                004329.6+070227004841.0+073506121.73-55.28 4.43  +1.50 +1.86 +0.87   K4IIIb             +0.085-0.052 +.019+032V?   < 19  8.6 113.3      *
 225 64    PscBD+16   76   4676 920991020  W                004343.3+162403004858.7+165626122.09-45.92 5.07  +0.51  0.00         F8V                -0.003-0.204 +.047+002SB2O < 17  7.4  76.7AB   3*
 226 35Nu  AndBD+40  171   4727 366991021                   004417.7+403204004948.8+410444122.60-21.79 4.53  -0.15 -0.58 -0.15   B5V+F8V            +0.023-0.019      -024SB2O   80                 *
 227          BD-14  145   47301474642054I    680           004424.2-140613004925.6-133341120.84-76.42 5.59  +1.30 +1.54         K5III              +0.105-0.104      +004V          6.2   1.3      *
 228          CD-24  345   4732166602         679           004418.2-244050004913.9-240811113.36-86.97 5.90  +0.95 +0.72         K2III              +0.078-0.051      +023V          7.7   8.8
 229          CD-47  229   47372152452053                   004418.4-471437004856.7-464152304.21-70.42 6.27  +0.90               G8III              -0.008+0.017      +012
 230 65    PscBD+26  131   4757 74295         683B          004430.6+270958004952.8+274239122.51-35.16 7.0   +0.36 +0.06         F4III              +0.084-0.010 +.011+005V      82  0.0   4.5      *
 231 65    PscBD+26  131   4758 74296         683A          004430.9+270955004953.2+274237122.51-35.16 7.1                       F5III              +0.091-0.009 +.011+007       82  0.0   4.5      *
 232          CD-24  347   4772166608                       004438.0-235425004933.4-232142116.39-86.21 6.28  +0.13 +0.13         A3V                -0.008 0.000      +012V
 233          BD+63   99   4775 11424  29  S         317    004439.2+634211005043.6+641451122.85  1.38 5.39  +0.49 +0.14         G0III-IV+B9.5V     +0.030-0.012      +003SBO     0                 *
 234          BD+44  176   4778 367022055          GO And   004443.0+442726005018.3+450008122.72-17.87 6.15  +0.01 -0.03         A0pCr             v+0.065+0.006      +002                          *
 235 19Phi2CetBD-11  153   4813147470  30            316    004507.1-111058005007.6-103840121.81-73.51 5.19  +0.50 -0.02 +0.28   F7IV-V             -0.226-0.229 +.065+008V       0                 *
 236   Lam HyiCP-75   64   4815255710  31                   004507.2-752804004835.4-745524303.18-42.20 5.07  +1.37 +1.68 +0.52E  K5III              +0.130-0.033 +.025-009V
 237          BD+61  178   4817 11430    I           319    004517.5+611540005116.4+614821122.91-01.07 6.07  +1.88 +1.84         K2Ib-IICN-2        -0.002+0.006      -006V?                        *
 238          BD+50  161   4818 21767              V526 Cas 004513.9+505748005057.4+513029122.85-11.36 6.39  +0.28 +0.12         F2IV               +0.135-0.003      +002V?                        *
 239          CD-44  216   4849215254              AZ Phe   004522.6-435624005003.7-432341303.82-73.73 6.48  +0.29 +0.12         A9-F0III           +0.011+0.016      +015                          *
 240          BD+82   20   4853   1433942                   004529.8+830952005453.1+834226123.03 20.84 5.62  +0.09 +0.10         A4V                +0.062-0.022      +012       65
 241          BD+50  164   4881 21775                       004551.3+510138005133.7+513416122.95-11.30 6.21                      B9.5V              -0.010-0.019      -014       68
 242   Rho PheCD-51  209   49192322032056          Rho Phe  004608.0-513157005041.2-505913303.23-66.14 5.22  +0.36 +0.16         F2III              +0.067+0.039 +.005+022                          *
 243          BD+02  118   4928109507                       004609.3+025033005118.3+032306122.87-59.49 6.37  +1.07 +0.89         K0III              +0.016-0.067      +006V?                        *
 244          BD+60  124   5015 11444         721           004705.8+603433005304.1+610727123.13-01.75 4.82  +0.53 +0.12 +0.30   F8V                -0.071+0.172 +.066+021SB1O    6  5.5 129.7AB   5*
 245          CD-44  226   5042215270      W                004712.1-441510005152.1-434233302.66-73.42 6.90  +0.35               F2V+F5V            +0.025-0.013D+.007+013V?         0.4   0.9      *
 246          BD+37  159   5066 54225                       004722.9+380020005253.4+383255123.24-24.32 6.69  +0.02 +0.10         A2V                +0.029-0.021      +016SB
 247          CD-24  376   5098166647    I    726           004746.0-243301005240.6-240021128.10-86.87 5.46  +1.24 +1.21         K2III              +0.035+0.035D+.007+034SB         4.4  11.0      *
 248 20    CetBD-01  114   51121290091022I                  004753.8-014114005300.5-010839123.83-64.01 4.77  +1.57 +1.93 +0.91   M0III              +0.008-0.017 -.003+016
 249          BD+36  148   5118 54237                       004758.5+365234005328.2+372505123.38-25.45 6.06  +1.14              gK3                 +0.013-0.046      -006V?
 250          BD+51  179   5128 21814    I    735           004801.1+520849005347.6+524121123.29-10.18 6.27  +0.19 +0.15 +0.08   A5m                +0.076-0.026D+.013-001       30  3.4   8.2AB   4*
 251          CD-25  338   5156166651         733A          004818.0-251917005312.4-244637132.59-87.62 6.46  +0.44 +0.03         F6IV-V             +0.098+0.037D+.006-004V          2.5   5.4      *
 252   Lam1TucCP-70   37   5190248269      W         334    004836.3-700244005224.3-693016302.81-47.62 6.22  +0.56 +0.08         F7IV-V             -0.007-0.086      +030           0.7  20.6
 253 26Ups1CasBD+58  134   5234 21832    I    748           004903.7+582553005500.1+585822123.39-03.90 4.83  +1.21 +1.26 +0.61   K2III              -0.032-0.043 +.006-023V?   < 17  7.6  94.1AC   3*
 254 66    PscBD+18  122   5267 92145         746           004917.4+183846005435.2+191118123.96-43.68 5.74R -0.01 -0.14         A1VN               +0.018-0.014 +.014+010SB    175  0.8   0.5AB   3*
 255 21    CetBD-09  181   5268129019                       004914.8-091655005417.6-084427125.17-71.60 6.16  +0.92 +0.54         G5IV               +0.018-0.052      +045V
 256          BD+47  242   5273 36763    I                  004924.0+480811005505.2+484043123.55-14.19 6.27  +1.68 +2.00         M2.5IIIa           -0.022-0.006      -052
 257          CP-63   83   52762482762057          BQ Tuc   004928.3-632452005337.9-625217302.51-54.25 5.70  +1.56 +1.77         M4III              +0.073+0.004      -010                          *
 258 36    AndBD+22  146   5286 74359         755    343    004936.6+230513005458.0+233742123.97-39.24 5.47  +1.00 +0.90 +0.50   K1IV               +0.131-0.031D+.022+002           0.4   0.6AB   3*
 259          BD+23  126   5316 74365    I           346    004953.4+240055005514.7+243325124.03-38.31 6.20  +1.62 +1.75         M4IIIab            +0.020-0.015      -010
 260          BD+57  172   5343 21846    I                  005016.3+572719005612.9+575948123.57-04.87 6.21  +1.37               K3III              +0.037-0.014      -030
 261          BD+67   81   5357 11481                       005027.0+681406005655.6+684634123.43  5.91 6.37  +0.38 -0.02         F4III              +0.137-0.021      -008       34
 262 67    PscBD+26  151   5382 743732059                   005035.6+264002005558.5+271234124.17-35.65 6.09  +0.14 +0.09         A5IV               -0.020+0.013      -008      130
 263          BD-08  167   53841290322058I                  005039.2-075317005542.4-072050126.06-70.19 5.85  +1.52 +1.93 +0.64E  K5III              -0.007-0.046      +002V                         *
 264 27Gam CasBD+59  144   5394 11482  32I    782  Gam Cas  005040.1+601031005642.5+604300123.58-02.15 2.47  -0.15 -1.08 -0.08   B0IVe             v+0.026-0.005 +.016-007SB    300: 6.3   2.3AB   3*
 265 28Ups2CasBD+58  138   5395 21855                       005042.2+583827005639.8+591052123.60-03.68 4.63  +0.96 +0.69 +0.50   G8IIIbFe-0.5       -0.092-0.046 +.051-047V    < 17                 *
 266          BD+59  146   5408 11484         784           005045.2+594917005647.1+602146123.59-02.50 5.55  -0.07 -0.32         B9IVn              +0.032-0.008D+.007-005SB2O  178  0.5   0.2AB   3*
 267 22Phi3CetBD-12  162   5437147519    I                  005100.6-114829005601.5-111600127.04-74.10 5.31  +1.52 +1.78         K4III              -0.020-0.008 +.009-026V?
 268          CD-28  288   5445166686    I                  005104.5-281903005555.5-274632246.23-88.81 6.10  +1.67 +1.97         M0III               0.000+0.002      +022V
 269 37Mu  AndBD+37  175   5448 54281  33     788           005112.0+375725005645.2+382958124.07-24.36 3.87  +0.13 +0.15 +0.08   A5V                +0.152+0.033 +.039+008       72  6.8 266.7AD   4*
 270   Lam2TucCP-70   40   5457248281  34                   005116.2-700404005500.3-693137302.47-47.60 5.45  +1.09 +1.00         K2III              +0.006-0.045 +.035+005
 271 38Eta AndBD+22  153   5516 743882060I W                005151.8+225240005712.4+232503124.65-39.43 4.42  +0.94 +0.69 +0.48   G8IIIb             -0.044-0.048 +.009-010SB2O < 17  6.9 131.7      *
 272          BD+45  237   5526 36795                       005159.9+451755005739.7+455023124.07-17.02 6.12  +1.02 +0.89         K2III              +0.013+0.005      +005
 273          BD+65  115   5550 11502                       005210.9+654842005831.1+662108123.64  3.49 5.97  -0.02 -0.13         A0III              +0.043-0.006      -010
 274 68    PscBD+28  157   5575 743951023I                  005225.3+282706005750.2+285932124.62-33.86 5.42  +1.08              gG6                 +0.008-0.008      -001
 275          BD+33  140   5608 543062061                   005245.2+332445005814.2+335703124.54-28.90 5.98  +1.00               K0                 +0.035-0.072      -017
 276          BD+12  119   5612 92183                       005239.5+130919005754.5+134145125.34-49.15 6.32  +0.89 +0.57 +0.46   G8III              -0.011-0.013      +015V                         *
 277          BD+20  131   5641 74402         805           005259.2+205151005818.9+212416125.07-41.44 6.37R +0.09 +0.11         A2V                -0.006-0.003D+.009-006V?         1.4   0.4      *
 278          BD+70   65   5715  42882063                   005347.8+702637010031.0+705859123.68  8.12 6.39  +0.13 +0.11         A4IV               +0.082 0.000      +006V     101                 *
 279 23Phi4CetBD-12  173   5722147546                       005343.4-115511005843.9-112248129.49-74.16 5.61  +0.97 +0.66         G7III              -0.034-0.019      -019V?
 280   Alp SclCD-30  297   5737166716  35            359    005347.3-295352005836.4-292127268.05-87.27 4.31  -0.16 -0.56 -0.13   B7IIIp             +0.022+0.004      +010SBO     7                 *
 281          CP-61   50   5771248293                       005412.8-611413005822.4-604147301.40-56.41 6.23  +0.10 +0.13         Am                 +0.050+0.018      +007
 282          BD+43  193   5788 36832         824B          005423.5+441020010003.4+444240124.54-18.13 6.84H                     A2Vn               +0.011-0.024D+.005+017SB    250  0.8   7.8      *
 283          BD+43  193   5789 36833         824A          005423.7+441028010003.6+444248124.54-18.13 6.04H -0.01 -0.04         B9.5Vn             +0.016-0.023D+.005+001SB    298  0.8   7.8      *
 284          BD+05  131   5820109581    I W       WW Psc   005438.6+055638005949.7+062859126.69-56.33 6.11  +1.67 +1.92 +1.02E  M2III              +0.023-0.008 +.001-015V?         7.1  31.0      *
 285          BD+85   19   5848   181 906                   005501.5+854314010844.7+861525123.24 23.40 4.25  +1.21 +1.33 +0.60   K2II-III           +0.076-0.012      +009                          *
 286          BD+88    4   5914   209                       005536.8+882915013350.4+890056123.13 26.16 6.46  +0.11 +0.07         A3V                +0.062-0.036      -010
 287          BD+50  202   6028 21938                       005629.2+502950010218.4+510206124.68-11.80 6.47                      A3V                -0.004-0.013      +006
 288   Xi  SclCD-39  260   60551928702064                   005638.0-392723010118.3-385500293.64-78.03 5.59  +1.18               K0III              +0.069+0.045 +.034-031
 289          BD+46  243   6114 36875         862           005715.9+465020010301.5+472234124.97-15.45 6.45  +0.25 +0.09         A9IV               +0.088-0.015 +.012+003SB         1.0   0.9      *
 290 39    AndBD+40  209   6116 36874         863           005716.9+404827010254.3+412042125.24-21.47 5.98  +0.16 +0.10 +0.05   A5m                -0.020-0.010      +004V?     39  6.2  20.9      *
 291 69Sig PscBD+31  168   6118 54374                       005720.3+311603010249.1+314816125.75-31.00 5.50  -0.05 -0.18         B9.5V              +0.015-0.028      +010SB2O   46                 *
 292          BD+60  157   6130 11551         868           005727.2+603215010337.0+610430124.40-01.76 5.92  +0.49               F0II               -0.006+0.004      -001V      20  3.1   1.2
 293   Sig SclCD-32  410   61781928841026                   005739.9-320525010226.4-313307275.15-84.97 5.50  +0.08 +0.13         A2V                +0.082+0.012      -008V
 294 71Eps PscBD+07  153   6186109627  36I D                005745.1+072106010256.6+075324127.89-54.87 4.28  +0.96 +0.70 +0.52   K0III              -0.078+0.023 +.027+007V?   < 17  0.0   0.2      *
 295   Ome PheCP-57  220   61922322681027                   005748.1-573227010201.8-570009300.04-60.07 6.11  +0.94               G8III              +0.006+0.015      +013
 296 25    CetBD-05  177   6203129094    I W                005759.0-052217010302.6-045012130.52-67.54 5.43  +1.11 +1.04         K0III-IV           -0.108-0.106      +015           6.4  99.9
 297          BD+60  158   6210 11557                       005808.7+610238010419.6+613449124.47-01.25 5.84  +0.54 +0.11         F6V                -0.070-0.022      -016       30
 298          BD+51  220   6211 219672066I                  005809.5+515800010402.4+523008124.88-10.32 5.99  +1.47               K2                 +0.001-0.059      -007
 299          CD-47  313   62452153432065                   005818.8-465607010249.2-462351297.02-70.60 5.36  +0.90               G8III               0.000+0.002 +.025-001
 300          CD-30  325   6269166774                       005831.2-300345010317.6-293133256.19-86.46 6.29  +0.93 +0.59         G5IV               -0.114-0.030      +057
 301 26    CetBD+00  174   6288109643  37     875           005840.1+004951010349.0+012200129.39-61.35 6.04  +0.27 +0.13         F1V                +0.122-0.042D+.024+006           3.4  16.0AB   3*
 302          BD+50  212   6300 21988                       005855.7+502824010446.8+510036125.08-11.81 6.54  -0.08 -0.57         B3V                +0.019-0.002      -003                          *
 303          BD+28  174   6301 74471                       005859.5+290731010427.6+293931126.31-33.13 6.19  +0.43 -0.05         F7IV-V             +0.084-0.116      +000       12
 304          CP-66   80   6311248308              CC Tuc   005853.1-655936010242.9-652722301.05-51.63 6.21  +1.64 +1.91 +1.33   M2III              +0.004 0.000      +035V                         *
 305          BD+39  249   6314 54398                       005858.8+392718010436.4+395928125.67-22.81 6.72  +0.31 +0.04         F0Vn               +0.077-0.022      +011
 306          BD+86   17   6319   193                       005906.3+863648011613.5+870843123.27 24.29 6.25  +1.12 +0.99        gK2                 +0.051-0.027 +.011-005
 307 73    PscBD+04  172   6386109656    I                  005941.7+050713010452.6+053923129.09-57.05 6.00  +1.51 +1.86        gK5                 +0.028-0.014      -015V
 308 72    PscBD+14  163   6397 922301028     889           005948.5+142430010505.4+145646127.84-47.80 5.68  +0.41 -0.06         F4II-III           +0.006+0.054      +004SB2 =< 10  7.4  55.2      *
 309          BD+61  206   6416 11571                       010004.0+621336010622.8+624542124.64-00.06 6.54  +0.19 +0.06         A5Vn               +0.102-0.024      +011SB    125                 *
 310 74Psi1PscBD+20  156   6456 74482         899A   397    010019.1+205616010540.9+212824127.34-41.28 5.34  -0.03 -0.06         A1Vn               +0.050-0.016D+.016-003V     246  0.3  29.9AB   3*
 311 74Psi1PscBD+20  157   6457 74483         899B          010019.9+205548010541.7+212755127.35-41.28 5.56  -0.07 -0.16         A0Vn               +0.047-0.018D+.016-004V     267  0.3  29.9AB   3*
 312          BD+79   29   6473  4331                       010039.5+792841010912.3+800042123.74 17.17 6.29R                   sgG6                 -0.030-0.038      -027V
 313 77    PscBD+04  175   6479109666         903A          010038.7+042233010549.2+045430129.66-57.77 6.35  +0.38 -0.05         F3V                +0.022-0.121      -007           1.0  33.0AB   5*
 314 77    PscBD+04  176   6480109667         903B          010040.9+042237010551.4+045434129.68-57.77 7.25  +0.49 -0.04         F6V                +0.029-0.119      -010V          1.0  33.0AB   5*
 315 27    CetBD-10  229   6482147601                       010036.2-103051010536.8-095845134.63-72.53 6.12  +1.01 +0.86 +0.53   K0III              -0.034-0.033      +012V?                        *
 316          BD+56  196   6497 220152069                   010055.6+562410010700.2+565606125.06-05.87 6.43  +1.18 +1.26 +0.42E  K2III              +0.100-0.110      -096                          *
 317 28    CetBD-10  230   6530147606                       010104.1-102230010605.1-095022134.92-72.37 5.58  +0.01               A1V                +0.030+0.004      +011SB2    65                 *
 318          BD+52  262   6540 22021    I    915           010112.4+525748010709.5+532954125.30-09.30 6.38R                     K2III              +0.023-0.001      +007           4.0  21.7AB   3*
 319 75    PscBD+12  135   6557 922502068                   010117.8+122512010633.6+125722128.64-49.75 6.12  +0.96               G8III              +0.019+0.023      +008V
 320          CD-24  484   65591667991029                   010117.1-243137010607.7-235933170.31-85.44 6.14  +1.09 +1.00         G8-K0III           -0.031-0.041      +033V
 321 30Mu  CasBD+54  223   6582 220241030  W         405    010136.8+542547010816.4+545513125.32-07.85 5.17  +0.69 +0.09 +0.42   G5Vb               +3.424-1.596 +.136-097SBO     1  5.8 190.5AB   6*
 322   Bet PheCD-47  324   6595215365      W         400    010137.3-471515010605.0-464307295.51-70.20 3.31  +0.89 +0.57 +0.50   G8III             v-0.027-0.001 +.021-001           0.2   1.0AB   3*
 323          CD-36  417   6619192925                       010145.3-361143010626.5-353939283.32-80.89 6.61  +0.14 +0.16 +0.07   A1V                +0.069-0.023      +018SBO                       *
 324 41    AndBD+43  234   6658 369502071                   010216.2+432434010800.9+435631126.08-18.83 5.03  +0.11 +0.14 +0.04   A3m                +0.165-0.070 +.031+009       91                 *
 325          CD-24  496   6668166814                       010221.8-243149010713.1-235947172.43-85.26 6.37  +0.23 +0.09         A7V                +0.098-0.036      +015V
 326          BD+57  200   6676 22032                       010225.6+574347010833.3+581549125.19-04.53 5.79  -0.01 -0.27         B8V                +0.007-0.005      -004
 327 78    PscBD+31  185   6680 544452072                   010229.1+312843010801.3+320044127.02-30.73 6.25  +0.40 -0.01         F5IV               +0.204-0.030      +014V     101                 *
 328 79Psi2PscBD+19  185   6695 74506                       010235.2+201227010757.2+204421128.13-41.96 5.55  +0.13 +0.13         A3V                +0.087-0.094      -002V     125
 329 30    CetBD-10  238   6706147622                       010244.5-101914010746.2-094708136.22-72.24 5.82  +0.43 +0.02         F7IV               +0.152+0.017      +022V?
 330 80    PscBD+04  190   67631096972073  W         410    010313.0+050715010822.2+053859130.68-56.96 5.52  +0.34  0.00 +0.18   F0III-IV           -0.264-0.185 +.036+007           3.8 159.3AC   3*
 331   Ups PheCD-42  391   67672153741031  W                010313.8-420118010747.9-412913290.82-75.25 5.21  +0.16 +0.09 +0.09   A3IV               +0.038+0.010 +.014+014V          0.4   0.2      *
 332   Iot TucCP-62   89   6793248324  39                   010321.0-621834010718.7-614631299.64-55.25 5.37  +0.88               G5III              +0.073-0.012 +.033-008
 333          BD+78   34   6798  4341  41                   010337.2+790830011216.7+794026123.91 16.84 5.64  +0.01 +0.03         A3V                +0.091-0.005      +018      175
 334 31Eta CetBD-10  240   6805147632  40I W                010333.5-104244010835.4-101056137.15-72.58 3.45  +1.16 +1.19 +0.58   K1.5IIICN1Fe0.5    +0.218-0.138 +.041+012V    < 17  6.7 233.5      *
 335 42Phi AndBD+46  275   6811 36972         940  Var?     010341.7+464231010930.2+471431126.11-15.52 4.25  -0.07 -0.34 -0.05   B7Ve               +0.010-0.009 +.003-000V      71  1.2   0.4      *
 336 31    CasBD+68   77   6829 116122074  W                010353.1+681447011039.3+684643124.68  5.97 5.29  -0.02 -0.06         A0Vnn              +0.036-0.031 +.017+001V     215                 *
 337 43Bet AndBD+34  198   6860 54471  42I    949    414    010407.8+350526010943.9+353714127.10-27.10 2.06  +1.58 +1.96 +1.00   M0+IIIa           e+0.178-0.114 +.049+003V         10.1  80.4AD   5*
 338   Zet PheCP-55  241   6882232306      W       Zet Phe  010410.9-554649010823.1-551445297.83-61.71 3.92  -0.08 -0.41 -0.12   B6V+B9V            +0.019+0.025D+.013+015SB2O  127  4.0   6.6ABxC 3*
 339 81Psi3PscBD+18  153   6903 92283              Var?     010428.3+190730010949.2+193931128.85-43.00 5.55  +0.69               G0III              -0.006+0.009      -008V?     91
 340 44    AndBD+41  219   6920 369842075                   010437.8+413259011018.8+420453126.67-20.65 5.65  +0.60               F8V                -0.133-0.043      -011V   =< 10
 341          BD+24  186   6953 74530    I                  010453.8+245541011019.4+252728128.29-37.22 5.80  +1.47 +1.82         K7III               0.000-0.114      +005V?
 342          BD+63  149   6960 11615                424    010457.5+634015011125.6+641210125.10  1.42 5.55  -0.07 -0.13         B9.5V              +0.040-0.013      -010V      47                 *
 343 33The CasBD+54  236   6961 22070      W         423    010500.5+543705011106.2+550859125.76-07.61 4.33  +0.17 +0.12 +0.07   A7V                +0.229-0.022 +.014+009SB    102  5.9 145.6      *
 344          BD+14  175   6966 92288    I                  010453.2+150831011011.5+154027129.55-46.96 6.06  +1.49               M0III              +0.031-0.021      -003V
 345 32    CasBD+64  127   6972 116172078          RU Cas   010510.3+642914011141.4+650108125.07  2.23 5.57  -0.09 -0.28         B9IV               +0.023-0.016      -002V      70                 *
 346 32    CetBD-09  227   6976129154                       010511.0-092616011012.0-085422137.48-71.25 6.40  +1.04 +0.87         K0III              -0.013-0.036      -020V?
 347 33    CetBD+01  221   7014109715    I           422    010524.7+015449011033.6+022644132.54-60.07 5.95  +1.51 +1.87         K4III               0.000-0.015      -003V?                        *
 348 45    AndBD+36  201   7019 54494      W                010532.9+371131011110.3+374327127.24-24.98 5.81  -0.10 -0.40         B7III-IV           -0.012-0.007 +.003-001       70  0.0   0.4      *
 349 82    PscBD+30  181   7034 54493                       010535.7+305334011106.8+312529127.84-31.26 5.16  +0.23               F0V                -0.011-0.014      +002V      84
 350          CP-58   81   70822323242077                   010602.5-581325011007.4-574140298.05-59.26 6.41  +0.89               G6II-III           -0.006-0.121      +088
 351 84Chi PscBD+20  172   7087 745441032I                  010604.6+203011011127.2+210205129.18-41.59 4.66  +1.03 +0.82 +0.54   G8.5III-IIIa       +0.038-0.011 +.015+016     < 19:
 352 83Tau PscBD+29  190   7106 74546  43I                  010609.0+293332011139.6+300523128.12-32.58 4.51  +1.09 +1.01 +0.58   K0.5IIIb           +0.072-0.035 +.023+030SB   < 17
 353 34    CetBD-03  161   7147129169    I                  010638.3-024656011143.5-021504134.84-64.65 5.94  +1.40 +1.68         K4III              -0.062-0.026      -009                          *
 354          BD+60  186   7157 11637         987           010648.3+611031011309.9+614221125.50-01.06 6.41  +0.01 -0.27         B9V                +0.033-0.021      -002V?    160  2.8   1.2AB   3
 355          BD+44  261   7158 37023    I                  010646.3+444821011234.0+452016126.82-17.37 6.11  +1.64 +2.03         M1III              +0.021+0.026      +022V
 356          BD+29  195   7229 74561         990           010728.8+293203011259.5+300351128.47-32.58 6.19  +1.00 +0.75         G9III+G1V          +0.020-0.034      +036SB         4.1  10.8AB   4*
 357          BD+79   36   7238  4354                       010739.1+792244011630.7+795436124.08 17.09 6.26  +0.42 -0.07         F5V s              -0.055+0.059      -043V?     45
 358          CD-31  484   7259192977                       010739.8-311953011223.4-304808252.89-84.13 6.52  +0.48 +0.03         F7IV               +0.061-0.077      +014
 359          CD-38  420   7312192980  44          AI Scl   010809.0-382311011245.4-375123281.60-78.37 5.92  +0.28 +0.12         F0III              +0.090-0.034      +010SB                        *
 360 85Phi PscBD+23  158   7318 745712082I    995           010819.0+240315011344.9+243501129.37-38.01 4.65  +1.04 +0.85 +0.53   K0III              +0.017-0.023 +.007+006SB   < 17  5.2   7.9AB   3*
 361 86Zet PscBD+06  174   73441097391033     996A          010830.3+070248011343.9+073431132.56-54.88 5.24  +0.32 +0.09         A7IV               +0.144-0.056 +.026+009SB    262  0.1   0.1 O   5*
 362 86Zet PscBD+06  175   7345109740         996B   442    010831.7+070258011345.3+073442132.57-54.87 6.30  +0.49 +0.01         F7V                +0.147-0.052 +.026+011SBO =< 25  0.1   0.1 O   5*
 363          BD+27  196   7351 74576    I           444    010835.0+280004011405.0+283147128.94-34.08 6.43  +1.70 +1.87 +0.98E  S3+/2-             +0.079-0.049      +002SB                        *
 364 87    PscBD+15  177   7374 923262083            445    010848.8+153615011407.6+160801130.84-46.39 5.98  -0.08 -0.41         B8III              -0.027-0.026      -016V      28                 *
 365          BD+70   90   7389  4358                       010900.3+711253011612.1+714438124.89  8.97 7.83  +0.78 +0.35         K1V                +0.006+0.004      -017                          *
 366 37    CetBD-08  216   7439129193        1003A   446    010921.7-082737011424.0-075523139.80-70.04 5.13  +0.46 -0.05         F5V                +0.127+0.273 +.056+022V      12  2.7  49.8      *
 367 88    PscBD+06  181   7446109753                       010930.2+062759011442.4+065943133.14-55.41 6.03  +1.08 +1.02        gG6                 -0.011-0.024      -009V
 368 38    CetBD-01  162   74761291962084  W                010942.8-013032011449.2-005826136.01-63.25 5.70  +0.42 -0.02         F5V                -0.012+0.201 +.028+026V   =< 10  6.3 114.1AB   3*
 369          BD+47  357   7546 370672085                   011030.5+473314011624.5+480456127.24-14.58 6.61  -0.05 -0.34         B9IIIpSi           +0.010+0.001      -014V?
 370   Nu  PheCD-46  346   7570215428                       011040.4-460356011511.1-453153290.10-71.00 4.96  +0.58 +0.11 +0.36   F8V                +0.669+0.183 +.069+012                          *
 371          BD+32  223   7578 54567    I:                 011044.1+323514011618.8+330653128.91-29.47 6.02  +1.15               K1III              +0.016-0.033      +006                          *
 372          BD+44  271   7647 37077    I                  011115.6+442231011705.1+445407127.70-17.73 6.34R                     K5                 +0.014-0.045      -052
 373 39    CetBD-03  172   7672129204      W       AY Cet   011131.6-030136011636.3-023001137.75-64.64 5.41  +0.90 +0.44         G5IIIe+dA          -0.104-0.063 +.015-020V          5.7 177.6      *
 374          BD+30  196   7724 54580                       011150.9+311301011724.1+314441129.36-30.80 6.73R                     K0                 -0.052+0.003      -034
 375          BD+76   40   7732  4376                       011159.1+770231012019.5+773414124.54 14.79 6.31  +0.92               G5III              -0.018+0.086      -074
 376          BD+46  319   7758 37096                       011216.2+465334011810.2+472511127.61-15.21 6.25  +1.35               K0                 +0.018-0.002      -001
 377   Kap TucCP-69   45   7788248346      W     A   454    011222.7-692426011546.1-685234299.66-48.10 4.86  +0.47 +0.03 +0.22   F6IV               +0.399+0.107 +.061+009      111  2.2   5.2AB   4*
 378 89    PscBD+02  185   78041097931034                   011238.4+030517011748.0+033652135.63-58.61 5.16  +0.07 +0.08 +0.03   A3V                -0.046-0.025 +.013+005SB    121                 *
 379          BD+36  220   7853 54592        1055           011306.8+365136011847.0+372310128.93-25.17 6.46  +0.23 +0.14 +0.11   A5m                -0.011-0.014D+.008+005SB     35  3.1   6.2
 380          CP-67   81   7916248350      W                011335.5-665531011703.6-662353298.90-50.53 6.24  +0.05 -0.02         A0V                +0.044+0.002D+.010+013           2.4   2.4
 381          BD+75   59   7925  43872087                   011351.5+754253012159.1+761420124.79 13.48 6.38  +0.28 +0.07         F0IVn              +0.063-0.032      -016
 382 34Phi CasBD+57  260   7927 22191    I   1073           011347.2+574221012004.9+581354126.71-04.43 4.98  +0.68 +0.49 +0.56   F0Ia                0.000-0.002 -.003-024SB     27  1.9 133.8AC   5*
 383 90Ups PscBD+26  220   7964 74637  45                   011358.0+264419011928.0+271551130.56-35.19 4.76  +0.03 +0.10 +0.05   A3V                +0.025-0.013 +.019+008SB     93
 384 35    CasBD+63  176   8003 11712        1088    474    011424.1+640802012105.2+643930126.10  1.97 6.34  +0.09               A2Vnn              +0.057-0.019      -015V     240  2.1  55.5
 385 42    CetBD-01  171   8036129235        1081           011441.5-010202011948.3-003032138.45-62.52 5.87  +0.64 +0.32         G8III+A7V          +0.012-0.014D+.009+014SB         0.7   1.6AxBC 3*
 386          BD+77   49   8065  4391                       011457.4+781208012346.8+784333124.57 15.96 6.07  +0.39 -0.08         A0Iab              -0.009-0.003      -075
 387          BD-04  185   8120129239                       011529.9-034619012034.5-031449140.45-65.12 6.23  +1.02 +0.84         K1III              -0.013+0.006      -003
 388          BD-11  248   81211477462088                   011529.8-114543012027.8-111420147.57-72.71 6.15  +1.11 +1.03         K1III              -0.047-0.074      -011V?
 389 91    PscBD+27  215   8126 746472089I                  011535.4+281257012107.4+284417130.75-33.68 5.23  +1.39              gK5                 +0.027-0.078      -036V?
 390 46Xi  AndBD+44  287   8207 371551035I                  011626.9+450017012220.4+453144128.58-17.00 4.88  +1.08 +0.99 +0.53   K0-IIIb            +0.032+0.009 +.026-013SB   < 17                 *
 391          BD+57  274   8272 22230        1105           011659.5+573721012321.6+580835127.14-04.47 6.45                      F4V                +0.139-0.094D+.005+007           0.2   0.5AB   4*
 392          BD+00  223   83341098342091I                  011728.0+011216012237.0+014335138.75-60.18 6.20  +1.52 +1.86 +0.63E  K5IIIab            +0.066-0.048      -016V
 393 43    CetBD-01  179   8335129262                487    011727.8-005821012234.8-002659139.87-62.29 6.49  +1.08 +1.01        gK0                 +0.023-0.014      +014
 394          BD-19  229   8350147767        1106           011739.5-193609012230.5-190453166.13-79.24 6.35  +0.48               F6IV               -0.056-0.071D+.008-005V?     19  2.2   5.0      *
 395 47    AndBD+36  237   8374 546552093                   011757.2+371135012340.6+374254129.94-24.72 5.58  +0.26 +0.13 +0.12   A2Vm               +0.083-0.022      +013SB2O   15                 *
 396          BD+33  220   8375 54654                       011757.1+334312012337.5+341445130.47-28.16 6.29  +0.83 +0.48         G8IV               +0.229+0.116      +003V
 397          BD+19  226   8388 74682    I                  011800.6+195648012324.9+202808132.99-41.78 5.97  +1.71               K5                 -0.009-0.007      -011
 398          BD+70  102   8424  4401                       011824.9+702732012546.5+705848125.75  8.30 6.49R                     A0Vnn              +0.016-0.013      +011      195
 399 36Psi CasBD+67  123   8491 11751  46I   1129           011851.7+673629012556.0+680748126.15  5.47 4.74  +1.05 +0.94 +0.52   K0III              +0.074+0.026 +.013-012V    < 17  5.2  23.2AC   4*
 400          CD-31  562   84981930901036I                  011851.7-312800012331.0-305644243.40-82.02 5.84  +1.61 +2.05 +0.80E  K5III              -0.012-0.046      -016                          *
 401 44    CetBD-08  243   8511129275      W       AV Cet   011900.7-083139012402.5-080027146.42-69.38 6.21  +0.23 +0.09         F0V                +0.174-0.070      +006      157  4.7  84.6      *
 402 45The CetBD-08  244   8512129274  47I   1118           011901.5-084158012401.4-081100146.59-69.54 3.60  +1.06 +0.93 +0.56   K0III-IIIb         -0.079-0.218 +.041+017V?   < 17 11.0  65.4      *
 403 37Del CasBD+59  248   8538 22268  48I W       Del Cas  011916.1+594256012549.0+601407127.19-02.35 2.68  +0.13 +0.12 +0.09   A5III-IV          v+0.297-0.051 +.037+007SB    113  8.7 131.7      *
 404          BD-07  223   8556129277        1123           011918.8-072611012420.6-065453145.55-68.33 5.91  +0.41 -0.05         F3V                +0.042+0.003D+.023+032SBO        0.2   0.2      *
 405          BD-16  237   8589147793                       011945.7-161053012439.8-153937158.54-76.18 6.14  +0.91 +0.54         G5                 +0.012-0.005      +013V                         *
 406          BD-03  195   85991292801037                   011943.8-032209012448.7-025055142.56-64.44 6.15  +0.96 +0.71         G8III              +0.013-0.030      -027
 407          BD+22  226   8634 747072096                   012007.7+225928012535.7+233042132.98-38.69 6.18  +0.43  0.00         F5III              +0.032-0.021      -016SBO    34                 *
 408          CD-42  493   86512154922095                   012015.2-420046012440.8-412933279.66-74.10 5.42  +1.04 +0.86 +0.49   K0III              +0.022-0.038 -.012+074
 409          BD+42  302   8671 37204                       012025.6+425620012618.6+432728129.61-18.96 5.96  +0.49 +0.01         F7V                +0.103-0.060      +031    =<  6
 410          BD+33  228   8673 54695                       012027.1+340342012608.7+343447131.00-27.75 6.31  +0.47 +0.01         F7V                +0.230-0.087      +017       32                 *
 411          CD-45  463   8681215494                       012021.3-450257012441.9-443142284.11-71.36 6.26  +1.14 +1.08 +0.52   K1II               -0.009-0.018      -010V
 412 46    CetBD-15  266   8705147803    I                  012042.1-150708012537.2-143556157.07-75.16 4.90  +1.23 +1.26         K2.5IIIb           +0.039-0.018 +.007-023V    < 17
 413 93Rho PscBD+18  187   8723 92436                506    012051.7+183907012615.3+191020134.19-42.94 5.38  +0.39 -0.10         F2V:              v-0.024+0.010 +.038-009V      61
 414 94    PscBD+18  189   8763 924441039                   012117.4+184320012641.7+191425134.31-42.85 5.50  +1.11 +1.05         K1III              +0.050-0.061 +.013-042V                         *
 415          BD+33  234   8774 54705                       012125.1+335129012706.2+342239131.25-27.92 6.27  +0.46 +0.02         F7IVb vs           +0.154-0.011      +015       23                 *
 416          BD-01  189   8779129300                       012120.2-005507012627.3-002355141.83-61.98 6.41  +1.24 +1.27         K0IV               +0.044-0.001 +.023-006V
 417 48Ome AndBD+44  307   8799 372281040    1152           012140.1+445326012739.4+452424129.56-17.00 4.83  +0.42  0.00 +0.23   F5IV               +0.351-0.107 +.030+011       69  6.6 119.0AC   4*
 418          BD+40  289   8801 37227        1151           012137.5+403452012726.6+410602130.20-21.26 6.46  +0.27 +0.03 +0.16   A7m                +0.003-0.001      +001       70  5.9  15.3      *
 419          BD+02  211   8803109895        1148           012143.4+030059012653.5+033207139.88-58.16 6.58  -0.04 -0.19         B9V+A8V            +0.004-0.035 -.016+015V     210  2.5   6.0      *
 420          CP-65  130   88102483811038                   012137.8-645322012505.3-642210296.98-52.37 5.93  +1.56 +1.92         K5III              +0.023-0.019      +023
 421 47    CetBD-13  262   88291478121041            512    012155.5-133435012651.6-130324155.16-73.67 5.66  +0.35 +0.02         F0IIIDel Del       +0.016+0.009      +010
 422          BD+39  334   8837 37234                       012158.8+394901012747.0+402008130.39-22.01 6.60  -0.04 -0.17         A0III              +0.015-0.019      -001V?    135                 *
 423          CD-33  525   8879193122    I W       R Scl    012221.9-330342012658.1-323235250.18-80.59 5.79  +3.86 +7.4  +1.38   C6II               -0.011-0.031      -008          11.9  10.0      *
 424  1Alp UMiBD+88    8   8890   308 907    1477  Alp UMi  012233.7+884626023148.7+891551123.28 26.46 2.02  +0.60 +0.38 +0.31   F7:Ib-II          v+0.038-0.015 +.007-017SBO    17  6.8  18.4AB   5*
 425          BD-11  272   8921147819        1162           012247.5-112515012746.6-105406152.44-71.66 6.13  +1.32 +1.36         K4III-IV           +0.180+0.019      +004           3.7   1.5
 426          BD+07  213   8949109907      W         519    012308.2+072636012822.9+075741138.54-53.79 6.20  +1.12 +1.08         K1III              +0.112-0.003      +002           2.1  69.0      *
 427 38    CasBD+69  102   9021  44221042                   012346.8+694500013113.8+701553126.30  7.65 5.81  +0.47 -0.11         F6V                +0.137-0.077 +.039+005SBO                       *
 428          BD+65  175   9030 11788                       012352.7+653454013052.3+660553126.92  3.53 6.14  +0.08 +0.05         A2V s              +0.084-0.010 -.001+009SB     75                 *
 429   Gam PheCD-43  449   9053215516  49I:        Gam Phe  012401.4-434950012821.9-431906280.52-72.17 3.41  +1.57 +1.85 +0.98   M0-IIIa            -0.014-0.209 +.000+026SB1O                      *
 430 49    AndBD+46  370   9057 37275    I                  012406.0+462929013006.2+470026129.75-15.35 5.27  +1.00 +0.82         K0III              +0.008-0.044 +.015-011     < 19:
 431          CD-34  576   9065193136              WZ Scl   012409.6-341651012843.3-334549254.77-79.59 6.58  +0.30 +0.07         F0IV               -0.055-0.025      -005                          *
 432 97    PscBD+17  210   9100 92463              VX Psc   012428.8+175019012952.9+182120135.56-43.57 6.02  +0.14 +0.14         A4IV               +0.063-0.003      +004      108                 *
 433 48    CetBD-22  254   91321670861043    1184           012448.3-220848012936.1-213746182.65-79.72 5.12  +0.02 +0.04         A1V                +0.056+0.006 +.028+001V          7.5  22.0
 434 98Mu  PscBD+05  194   91381099262099I W                012456.6+053742013011.1+060838140.07-55.43 4.84  +1.37 +1.54 +0.74   K4III              +0.294-0.048 +.018+034V    < 17  5.6 182.5AB   3*
 435          CD-47  440   9184215525              AW Phe   012516.5-471625012930.4-464523284.52-68.99 6.31  +1.66 +1.87 +1.39   M2III              -0.012+0.013      +018                          *
 436          CD-26  502   9228167093    I   1193           012540.1-264327013022.9-261228209.10-81.25 5.93  +1.34 +1.41         K4III              +0.045+0.002      -001           5.0   2.9      *
 437 99Eta PscBD+14  231   9270 92484  50I   1199    532    012607.8+144949013129.0+152045137.01-46.43 3.62  +0.97 +0.75 +0.50   G7IIIa             +0.028-0.006 +.015+015V    < 19: 3.6   0.8      *
 438          BD+34  265   9298 547622100                   012625.3+341706013207.6+344800132.33-27.33 6.39  -0.12 -0.42         B7IIIpHgMn         -0.008-0.002      -001       60
 439          BD+57  320   9352 22389    I S                012656.0+574848013325.8+581939128.44-04.09 5.70  +1.52 +1.15         K0Ib+B9V           +0.011+0.002      -001    =< 50                 *
 440   Del PheCD-49  425   93622155361044                   012705.3-493532013115.1-490422286.27-66.75 3.95  +0.99 +0.70 +0.51   K0III              +0.142+0.151 +.031-007
 441          CD-30  506   9377193163                       012705.8-304747013143.3-301700234.96-80.63 5.82  +1.08               K0III              +0.009-0.067      +003V?
 442 39Chi CasBD+58  260   9408 22397    I                  012723.4+584308013355.9+591355128.35-03.19 4.71  +1.00 +0.76 +0.53   G9IIIb             -0.037-0.021 +.021+006     < 19:
 443          CD-46  420   9414215539      I                012724.1-460526013139.1-453432282.11-69.89 6.17  +0.06 +0.09 +0.03   A2V                 0.000+0.006      +008           4.2   1.4      *
 444          BD-09  298   9484129363              Var      012804.3-093144013303.5-090053153.38-69.39 6.59  -0.04 -0.10         A0III              +0.029+0.003      +010V?                        *
 445          CD-37  589   95251931732101                   012827.6-372243013256.0-365155263.48-76.89 5.51  +1.02               K0III              +0.002-0.023 +.007+013V?
 446          BD+36  277   9531 54788              KK And   012830.0+364328013416.6+371414132.32-24.85 5.88  -0.07 -0.30         B9IV               +0.009-0.015 +.018-004V     175                 *
 447          CD-50  411   9544215547                       012830.5-501422013236.3-494340286.40-66.06 6.28  +0.46               F4V                -0.030-0.077      +002
 448          BD-07  256   9562129371                       012840.9-073211013342.9-070131151.37-67.53 5.76  +0.64 +0.21         G2IV               +0.180-0.081      -015V?  =< 10
 449          BD+73   81   9612  4448                       012909.4+734723013722.5+741803126.09 11.71 6.58  +0.03 -0.16         B9Ve               +0.027-0.009      -008V?
 450          BD+17  224   9640 92520    I                  012924.3+175700013449.1+182738137.11-43.21 5.89  +1.52 +1.80         M2IIIab            +0.033-0.069      -026
 451 49    CetBD-16  265   9672147886                       012944.6-161119013437.8-154034166.32-74.78 5.63  +0.07 +0.05         A3V                +0.096+0.003      +009V?
 452          BD+40  328   9712 37344                       012958.4+403353013552.5+410435131.88-21.02 6.38  +1.11 +1.00         K1III              +0.139-0.002      +065
 453          CD-32  613   9742193188                       013016.9-322412013450.7-315332242.30-79.43 6.12  +1.11               K0-1III            -0.077-0.036      -011
 454          BD+47  460   9746 37351              OP And   013020.2+481243013627.2+484322130.54-13.48 5.92  +1.21              gK1                 -0.012-0.017      -043V                         *
 455101    PscBD+13  240   9766 92530                559    013025.5+140901013546.4+143941138.71-46.85 6.22  -0.04 -0.21         B9.5III            +0.004-0.011      -016V     220
 456 40    CasBD+72   86   9774  4453  51    1268    567    013030.9+723149013830.9+730224126.40 10.49 5.28  +0.96 +0.72         G8III              -0.013-0.015 +.017-005V    < 19: 6.4  53.3
 457          BD+16  176   9780 92531                       013030.0+165519013554.8+172601137.79-44.15 5.80R                     F0IV               +0.152+0.010      +003SB     97
 458 50Ups AndBD+40  332   9826 373621045I W                013055.5+405419013647.8+412420132.00-20.66 4.09  +0.54 +0.06 +0.29   F8V                -0.171-0.382 +.062-028SBO     8  8.4 114.0AB   3*
 459 50    CetBD-16  270   98561479012104I                  013106.3-155442013559.0-152401166.61-74.36 5.42  +1.21 +1.20         K2IIIaBa0.2CN1     +0.023+0.011      +024V    < 19:
 460          CP-58  123   9896232470                561    013130.3-583900013515.2-580822292.01-58.05 6.01  +0.38               F2V                +0.277-0.035 +.011+001?                         *
 461          BD+57  349   9900 224562108I                  013135.0+572805013807.6+575839129.11-04.33 5.56  +1.38 +1.43 +0.71   G5II               +0.007-0.006      -008V?                        *
 462   Tau SclCD-30  540   9906193201      W                013131.2-302510013608.4-295427231.37-79.80 5.69  +0.33               F2V                +0.100+0.040D+.025+005       63  1.1   0.9      *
 463102Pi  PscBD+11  205   9919 925361046                   013147.7+113749013705.9+120830140.16-49.20 5.57  +0.35 -0.02 +0.21   F0V                -0.066+0.043      -001V      96
 464 51    AndBD+47  467   9927 37375  52I                  013151.0+480718013759.6+483742130.81-13.53 3.57  +1.28 +1.45 +0.65   K3-III             +0.065-0.113 +.021+016V    < 17                 *
 465          BD+44  341   9996 37393              GY And   013230.2+445326013831.7+452400131.53-16.68 6.36  +0.04 -0.10         B9pCrEu           v-0.034+0.014      +003SB1O   19                 *
 466          BD-10  343  100091294122109  W                013237.7-095458013737.7-092414156.70-69.19 6.24  +0.53  0.00         F7V                +0.259+0.093 +.024+048V?         0.4   0.2      *
 467          CP-79   40  10042255794  53  W                013259.1-790045013339.2-783017300.26-38.41 6.11  +0.97 +0.70         K0III              -0.034-0.136      -001           7.4  50.1      *
 468          CP-58  126  10052232477                       013304.9-584651013644.8-581615291.74-57.86 6.18  +1.61 +1.89         M3III              +0.022+0.003      -002
 469 52Chi AndBD+43  343  10072 374062112I                  013320.9+435239013921.0+442310131.89-17.65 4.98  +0.89 +0.55         G8III              -0.022+0.012 +.009+007SB   < 17
 470          BD+53  363  10110 22493    I                  013351.4+532139014013.1+535206130.16-08.32 6.39  +1.61 +1.98         K5III              -0.011-0.002      -062
 471          CD-37  620  10142193224                       013400.9-370200013827.4-363142259.18-76.31 5.94  +1.05 +0.90 +0.37E  K0IV               -0.018-0.128      +030                          *
 472   Alp EriCP-57  334  10144232481  54          Var      013359.4-574441013742.9-571412290.84-58.79 0.46  -0.16 -0.66 -0.11   B3Vpe             v+0.095-0.035 +.026+016V     251                 *
 473          BD-22  272  101481671982113                   013405.0-214705013851.8-211631187.04-77.72 5.58  +0.34               A4n                +0.131+0.036      +014V
 474          CD-25  670  10161167199        1298           013408.4-253150013850.0-250119204.39-79.16 6.70  -0.08 -0.17         B9Vn               +0.001+0.011      +015V          6.2  20.3
 475105    PscBD+15  245  10164 92558                       013416.9+155355013940.8+162421139.37-44.92 5.97  +1.12               K2III              +0.077-0.015      +017V
 476          BD+42  345  10204 37419                       013439.7+424730014039.8+431752132.35-18.67 5.61  +0.21 +0.14       sgA9                 +0.127-0.037      +017V      82                 *
 477 53Tau AndBD+39  378  10205 37418        1311    584    013440.4+400414014034.8+403437132.92-21.34 4.94  -0.09 -0.41 -0.07   B8III              +0.013-0.025      -014SB     91  5.9  52.3      *
 478 43    CasBD+67  149  10221 11919  55          V557 Cas 013455.6+673214014220.5+680235127.68  5.64 5.59  -0.07 -0.23         A0pSiSr            +0.052-0.009      +005       28                 *
 479          CP-54  358  10241232483      W                013455.7-535644013848.2-532620287.70-62.27 6.84  +0.44 +0.11         F5IV-V             -0.027-0.046      -003           1.3  10.4      *
 480 42    CasBD+69  114  10250  4470                590    013510.0+700702014255.9+703721127.21  8.18 5.18  -0.04               B9V                +0.077-0.013 +.004+006SB    117                 *
 481          BD+60  308  10260 11920                       013515.7+603157014203.0+610218129.02-01.23 6.71  -0.03 -0.41         B8IIIpSi           +0.015-0.007      -013V                         *
 482          BD+57  370  10293 22520        1334           013539.3+580720014217.8+583740129.52-03.59 6.37  -0.02 -0.45         B8III              +0.019-0.010      -004V?     35  2.4  20.1AB   3*
 483          BD+41  328  10307 37434      A                013541.6+420642014147.2+423649132.70-19.30 4.95  +0.62 +0.12 +0.33   G1.5V              +0.809-0.154 +.081+004V?      2        0.1      *
 484          BD+25  276  10308 748702115    1326           013543.5+251427014118.4+254445136.80-35.78 6.17  +0.44  0.00         F2III              +0.118-0.040 +.023+005SB2O   12  4.4  10.6AB   3*
 485          BD+29  286  10348 74872                       013559.9+293230014139.2+300250135.69-31.58 5.99  +1.01 +0.84 +0.46   K0III              -0.015-0.006      +005                          *
 486          CP-56  329  10360232492      W     B          013559.9-564215013947.4-561153289.60-59.66 5.87  +0.90 +0.59         K0V                +0.310-0.020 +.152+023SB         0.1  10.8      *
 487          CP-56  329  10361232490      W     A          013600.5-564208013947.8-561141289.60-59.66 5.76  +0.87 +0.53 +0.48   K5V                +0.279+0.020 +.152+020           0.1  10.8      *
 488          BD+60  312  10362 11931                       013608.7+605459014258.4+612518129.06-00.84 6.34  +0.01 -0.45         B7II               +0.004+0.011      -005                          *
 489106Nu  PscBD+04  293  10380110065  56I                  013613.5+045854014125.9+052915145.12-55.22 4.44  +1.36 +1.57 +0.71   K3IIIbBa0.1        -0.021+0.002 +.041+000     < 19:
 490          BD+34  297  10390 549121047                   013616.5+344427014203.5+351444134.46-26.49 5.64  -0.07 -0.20         B9IV-V             +0.049-0.028      -002V      70
 491 44    CasBD+59  307  10425 11941        1344           013633.2+600249014319.7+603304129.27-01.68 5.78  -0.02 -0.32         B8IIIn             +0.008-0.020      -033SB    130: 5.1  66.0AC   4*
 492          BD-12  315  10453147962        1339           013648.6-114907014144.7-111929162.14-70.26 5.75  +0.44 -0.01         F5V+F7V            +0.043-0.412 +.035-010V          1.1   1.8      *
 493107    PscBD+19  279  10476 74883      W         600    013703.9+194657014229.8+201607138.88-41.01 5.24  +0.84 +0.49 +0.43   K1V                -0.296-0.672 +.119-034V?   < 19: 6.4  19.0AB   3
 494          CD-38  584  10481193255                       013703.9-383824014127.3-380759262.44-74.80 6.17  +0.42 +0.04         F2V                +0.050+0.052      +011
 495          BD+44  354  10486 37460                       013711.0+444905014316.5+451920132.40-16.59 6.34  +1.01 +0.92 +0.48   K2IV               +0.146-0.017      +012                          *
 496   Phi PerBD+49  444  10516 22554  57I         Phi Per  013723.3+501106014339.6+504119131.32-11.33 4.07  -0.04 -0.93 +0.02   B2Vep              +0.026-0.014 +.025+001SB2O  450                 *
 497   Pi  SclCD-32  666  105371932631048I                  013737.8-324952014208.6-321937241.27-77.84 5.25  +1.05 +0.79 +0.54   K0III              -0.065-0.025 -.005+010V?                        *
 498          CD-37  650  10538193261  58                   013738.5-372012014203.0-364957258.27-75.56 5.72  -0.01       -0.01   A1V                -0.028-0.021      +017V
 499          BD+56  330  10543 22566        1359           013741.3+570202014417.9+573211130.00-04.61 6.21  +0.10 +0.03         A3V                +0.039-0.040 -.003+005       70  1.4   1.2      *
 500          BD-04  260  105501294651049I                  013740.1-041138014243.5-034125152.75-63.55 4.99  +1.38 +1.58         K3II-III           -0.007-0.032 +.009-034     < 19:
 501          CD-50  461  10553232497                       013741.8-503236014141.1-500220283.51-65.15 6.64  +0.12 +0.12 +0.07C  A3V                -0.023-0.020      -007V?
 502          BD+56  334  10587 22578                       013810.9+563510014446.1+570521130.16-05.03 6.25  +0.04 +0.06         A2V                +0.014-0.004      +005      170
 503          BD+31  301  10588 54939              Var?     013807.2+314118014350.0+321130135.65-29.38 6.34R                     G8III-IV           -0.022-0.007      -005SBO                       *
 504          BD+45  432  10597 37475    I                  013820.2+453815014426.5+460823132.43-15.75 6.35R                     K5III              -0.009-0.028      -019
 505          CP-61  130  106152484212116                   013822.5-611734014148.0-604722292.17-55.27 5.71  +1.27               K2-3III            +0.015-0.041      +002
 506          CP-54  365  10647232501                       013838.1-541432014229.3-534426286.90-61.77 5.52  +0.53  0.00         F9V                +0.171-0.093      +013
 507          BD-05  309  10658129477    I                  013852.6-051604014354.8-044556154.47-64.37 6.19  +1.54 +1.90         K0                 -0.020-0.032      +020
 508109    PscBD+19  282  10697 92611    I                  013928.0+193501014455.8+200459139.67-41.04 6.27  +0.75 +0.23         G3Va               -0.041-0.109 +.027-044V?
 509 52Tau CetBD-16  295  10700147986  59I W                013925.3-162751014404.1-155615173.19-73.43 3.50  +0.72 +0.21 +0.47   G8V                -1.718+0.856 +.275-016V       2  9.5  89.8
 510110Omi PscBD+08  273  10761110110  60I           615    014006.7+083916014523.6+090928144.61-51.43 4.26  +0.96 +0.71 +0.48   G8III              +0.073+0.048 +.023+014     < 17                 *
 511          BD+63  238  10780 11983      W                014030.5+632132014744.8+635108129.08  1.66 5.63  +0.81 +0.40 +0.39   K0V                +0.582-0.255 +.107+003V?         3.1  91.1      *
 512          CP-83   27  10800258271    I'                 014033.8-832902013755.6-825830301.23-34.00 5.87  +0.61 +0.10         G2V                +0.119+0.120      -005V
 513          BD-06  336  108241294902117I                  014058.0-061401014559.3-054400156.61-64.97 5.34  +1.52 +1.88         K4III              -0.007-0.033      +011V?
 514   Eps SclCD-25  704  10830167275  61    1394           014057.7-253308014538.8-250309206.28-77.66 5.31  +0.39 +0.02         F1V                +0.162-0.057 +.037+015V      86  3.2   4.9AB   4*
 515          BD+16  196  10845 92622              VY Psc   014109.5+165443014635.3+172447141.20-43.49 6.55  +0.25 +0.17         A9III              +0.053+0.011      -001SB     67                 *
 516   Tau1HyiCP-79   44  10859255806      W                014117.5-793908014121.3-790854299.99-37.69 6.33  +0.94 +0.69         G6-8III            +0.052+0.009      +002           6.0  16.0      *
 517          CD-27  595  10863167279                       014123.5-275054014601.0-272057217.10-77.87 6.39  +0.36               F2Vn               +0.095-0.052      +006V
 518          BD+45  447  10874 375132119                   014139.5+454354014748.0+461347133.00-15.54 6.32  +0.43  0.00         F6V                +0.014-0.056      -003       27
 519          CD-51  419  10934232519    I'                 014210.7-511858014605.8-504859282.92-64.14 5.49  +1.60 +1.95 +1.27   M3III              +0.034-0.024 -.023-002
 520          CP-54  377  109392325202118                   014217.5-540126014606.3-533119285.68-61.74 5.04  +0.04 +0.05         A1V                +0.127+0.056 +.023+010       56
 521          BD+37  372  10975 54991                       014244.6+372718014838.9+375710135.20-23.55 5.94  +0.97 +0.71         K0III              +0.112-0.029      +037V?
 522  4    AriBD+16  203  10982 926371050                   014245.3+162727014810.9+165720141.89-43.82 5.84  -0.03 -0.12         B9.5V              +0.053-0.036      +010SB     22
 523          BD+31  316  11007 549942120                   014257.7+321101014841.6+324125136.65-28.66 5.79  +0.55 -0.01         F8V                -0.167+0.298 +.038-027V   =<  6
 524          CD-42  633  11022215650                       014302.5-421538014716.8-414536268.52-71.43 6.18  +1.54 +1.91         K5-M0III           +0.024+0.042      -013
 525          CP-85   17  11025258273 916                   014308.3-851629013728.0-844611301.70-32.25 5.69  +0.94 +0.66         G8III              +0.034+0.021      +018
 526          BD+47  508  11031 37536        1438           014302.9+472356014915.7+475349132.86-13.86 5.82  +0.29 +0.07         A3V                -0.010-0.002D+.008-002SB2        0.7   1.9AB   4*
 527          BD+02  270  110371101382121                   014315.1+031111014826.0+034108149.17-56.25 5.91  +0.97 +0.74         G9III              -0.006+0.017      +003V?
 528          CD-37  687  11050193303                       014325.8-373932014747.8-370935256.59-74.45 6.32  +1.02               K0III              +0.008+0.011      +015
 529          BD+51  416  11151 22678                       014432.7+512625015057.1+515600132.18-09.87 5.90  +0.43 +0.02         F5IV               +0.041-0.120      -017       40
 530  1    AriBD+21  243  11154 74966        1457           014436.9+214644015008.5+221631140.39-38.60 5.86  +0.74 +0.50 +0.49   K1III+A6V          -0.017-0.008 +.017+003SB         1.1   2.8      *
 531 53Chi CetBD-11  352  111711480361051  W                014440.3-111052014935.1-104111165.48-68.61 4.67  +0.33 +0.03 +0.16   F3III              -0.146-0.093 +.046-001       61  2.1 184.0      *
 532          CD-31  753  11183193320                       014448.0-313410014919.5-310422233.70-76.77 6.34  +1.22               K2-3III            +0.101-0.026      +020
 533  1    PerBD+54  396  11241 226902122          V436 Per 014525.0+543908015159.3+550851131.57-06.70 5.52  -0.18 -0.83         B1.5V              +0.014-0.008      -003SBO   198                 *
 534          BD+10  252  11257 92659      D         635    014533.5+103254015052.0+110236145.58-49.20 5.94  +0.30 -0.03 +0.17   F2V w              -0.070-0.027      +011V          0.0   0.1      *
 535          CD-39  553  11262193326      W                014529.9-385425014948.8-382414259.30-73.37 6.37  +0.52               F8V                -0.003+0.233 +.022+022V?         7.0  28.0
 536  2    PerBD+50  379  11291 226961052                   014547.5+501755015209.4+504734132.64-10.93 5.79  -0.07 -0.30         B9IIIpHgMn         +0.017-0.026      +010SB1O   22                 *
 537          CD-48  487  11332215673                       014619.4-481849015020.2-474859277.77-66.33 6.14  +1.01 +0.88 +0.34E  K1IV               +0.124+0.051      +012                          *
 538          BD+50  381  11335 22705                       014626.9+505851015250.8+512829132.58-10.24 6.26                      A3V                -0.011-0.009      +006V     135
 539 55Zet CetBD-11  359  11353148059  62I W         638    014631.4-104945015127.6-102006165.88-68.05 3.73  +1.14 +1.07 +0.55   K0IIIBa0.1         +0.041-0.039 +.031+009SB1O < 19: 6.2 187.0      *
 540          BD+54  408  11408 22716                       014710.9+550617015348.5+553553131.72-06.21 6.45  +0.18 +0.11 +0.07   A5m                +0.049+0.001      +008       57
 541          CD-50  514  114132325422123          BD Phe   014701.6-504204015054.5-501222280.70-64.28 5.94  +0.14               A1Vp               -0.042-0.003      +007                          *
 542 45Eps CasBD+62  320  11415 12031  63            652    014711.7+631040015423.7+634012129.84  1.65 3.38  -0.15 -0.60 -0.12   B3III              +0.032-0.021 +.010-008V      19                 *
 543 55    AndBD+40  394  11428 375872124I W                014717.3+401411015317.3+404347135.40-20.64 5.40  +1.32 +1.41        gK1                 -0.010-0.006 +.006-007V?         5.3  59.8
 544  2Alp TriBD+28  312  11443 74996  64I W                014722.7+290530015304.9+293444138.64-31.40 3.41  +0.49 +0.06 +0.28   F6IV               +0.011-0.235 +.057-013SB1O   93  8.6 222.3AC   3*
 545  5Gam1AriBD+18  243  11503 92680        1507B Var?     014802.4+184821015331.8+191745142.55-41.20 4.83H                     B9V               v+0.078-0.112 +.026+004V     152  0.1   7.8AB   3*
 546  5Gam2AriBD+18  243  11502 92681        1507A Gam Ari  014802.4+184813015331.8+191737142.55-41.20 4.75H -0.04 -0.13 -0.05   A1pSi              +0.079-0.104 +.026-001V      63  0.1   7.8AB   3*
 547          BD-17  336  115221480782125          BK Cet   014803.6-172517015252.1-165545180.42-72.52 5.80  +0.27 +0.09         F0IIIn             +0.042-0.053      -004                          *
 548 46Ome CasBD+67  169  11529 120382129            659    014813.5+681139015600.0+684107128.77  6.55 4.99  -0.10 -0.40         B8III              +0.013-0.011      -024SBO    34                 *
 549111Xi  PscBD+02  290  11559110206  65I           654    014822.6+024138015333.3+031115151.64-56.18 4.62  +0.94 +0.72 +0.47   K0III              +0.022+0.023 +.005+030SBO  < 17                 *
 550   Tau2HyiCP-80   35  11604258280      W                014842.4-804015014746.5-801036299.96-36.63 6.06  +0.34 +0.04         F0IV               -0.074-0.071      +030           7.4  39.7
 551          BD+39  434  11613 37607                       014852.5+401244015453.8+404207135.72-20.59 6.24  +1.25               K2                 +0.047-0.066      +032SBO                       *
 552          BD+36  346  11624 55082    I:                 014902.9+363813015457.5+370742136.75-24.03 6.26  +1.17               K0                 +0.021-0.010      -002
 553  6Bet AriBD+20  306  11636 75012  66I           658    014906.8+201909015438.4+204829142.24-39.68 2.64  +0.13 +0.10 +0.08   A5V                +0.096-0.111 +.074-002SBO    79                 *
 554          CD-39  573  116431933682127                   014904.9-390517015323.2-383541258.35-72.71 6.10  +1.12               K1II               +0.131+0.027      +022
 555   Psi PheCD-46  552  11695215696  67I         Psi Phe  014938.2-464732015338.8-461809274.35-67.22 4.41  +1.59 +1.70 +1.52   M4III              -0.086-0.087 +.011+002SB                        *
 556          BD+36  354  11727 55102    I   1534B          014959.3+364715015554.4+371640136.91-23.84 5.89  +1.63              gM0                 +0.004-0.003      +007V   =< 25  0.2 190.4AB   4*
 557 56    AndBD+36  355  11749 55107        1534A          015012.7+364540015609.3+371506136.96-23.85 5.67  +1.06 +0.91         K0III              +0.182+0.006      +059    =< 25  0.2 190.4AB   4*
 558   Phi PheCD-43  583  117532156971053                   015013.0-425915015422.0-422949267.17-69.99 5.11  -0.06 -0.15 -0.06   A3V                -0.028-0.032      +012SB     13                 *
 559  7    AriBD+22  284  11763 750302130I         RR Ari   015016.3+230514015551.0+233438141.48-36.96 5.74  +1.19 +1.04         K1III              +0.009-0.013      +014                          *
 560          BD+01  347  11803110235        1538    667    015043.6+012116015553.8+015059153.67-57.13 6.01  +0.56 +0.04         F7V+G0V            +0.158+0.184 +.025+030SB         0.1   1.5      *
 561          BD+60  398  11857 12065                678    015127.5+611236015833.3+614153130.80-00.14 6.02  -0.04 -0.40         B5III              +0.014-0.008      +007V                         *
 562          BD+40  407  11905 37653                       015152.2+411223015756.4+414140136.03-19.48 6.78  -0.06 -0.39         B8III              +0.012-0.014      +003V       8                 *
 563  8Iot AriBD+17  289  11909 927212132I                  015153.1+171946015721.1+174903144.34-42.30 5.10  +0.92 +0.70         K1p                +0.037-0.018      -005SBO  < 17: 1.0   0.0      *
 564          BD+27  310  11928 75048    I                  015202.7+271904015743.8+274816140.40-32.81 5.82  +1.60 +1.89         M2III              +0.026-0.058      -003V
 565 56    CetCD-23  721  119301674162131I                  015159.2-230054015640.2-223137199.22-74.52 4.85  +1.42 +1.67 +0.56E  K4III              +0.062-0.029 -.012+027
 566   Chi EriCP-52  241  11937232573  68  W         671    015204.0-520623015557.5-513632280.88-62.67 3.70  +0.85 +0.46 +0.45   G8IIIbCNIV         +0.681+0.292 +.058-006           6.5   4.8      *
 567          BD+63  265  11946 12076        1571           015215.1+640806015938.0+643717130.16  2.71 5.26  +0.01 -0.05         A0Vn               +0.033-0.020      +005V     360  5.6  41.5      *
 568  3    PerBD+48  576  11949 37665                       015212.0+484256015833.6+491215134.06-12.21 5.69  +1.01               K0IV               +0.010+0.038      -000
 569  9Lam AriBD+22  288  11973 75051        1563A   680    015221.2+230630015755.7+233546142.04-36.80 4.79  +0.28 +0.09 +0.16   F0V                -0.090-0.013 +.028-001SB     99  2.5  37.4AB   4*
 570   Eta2HyiCP-68  101  11977248460  69                   015224.0-680821015456.1-673850293.93-48.35 4.69  +0.95 +0.64 +0.49   G8.5III            +0.074+0.074      -016                          *
 571          CP-61  157  11995248464                       015234.6-612107015546.4-605141289.48-54.55 6.06  +0.37 +0.01         F0IV-V             +0.004+0.056      +015
 572          BD+77   73  12005  4555                       015249.0+772554020257.2+775459126.76 15.58 6.04  +1.14               K0                 -0.004-0.002      -003
 573          CP-52  242  12042232581                       015311.2-521541015700.1-514558280.76-62.45 6.10  +0.48 -0.04         F6-7V              +0.359+0.248 +.056+004
 574          CD-47  597  12055215715                       015312.2-475225015710.0-472306274.84-66.00 4.83  +0.88 +0.52 +0.45   G8III              +0.103+0.016 +.021+012
 575 48    CasBD+70  153  12111  4554        1598    694    015344.0+702520020157.4+705425128.69  8.83 4.54  +0.16 +0.06 +0.08   A3IV               -0.067+0.004 +.035-005SBO    67  1.7   0.5AB   4*
 576          CD-33  682  121351934222135  W                015401.9-333311015826.7-330400239.29-74.36 6.35  +1.02               K0III              +0.008-0.020      +001V          6.9   6.1
 577          BD+20  322  12139 75077        1582D          015402.5+203422015935.7+210330143.55-39.07 5.87  +1.03               K0III-IV           +0.139-0.022      -002           3.3 188.1AD   4*
 578          BD+11  261  12140 927392136                   015404.5+114835015925.9+121741147.82-47.26 6.09  +0.19 +0.10         A6V                +0.008-0.042      -012V?    129
 579          BD+73  108  12173  4559        1606           015418.7+732201020310.5+735102127.95 11.68 6.23R                     A5III              -0.029-0.005D+.004-005           3.1   5.5
 580 50    CasBD+71  117  12216  4560  70                   015453.1+715615020326.1+722517128.38 10.31 3.98  -0.01 +0.03  0.00   A2V                -0.045+0.022      -014SB2    84
 581 47    CasBD+76   63  12230  45622139  W                015505.6+764803020507.4+771653127.06 15.00 5.38  +0.31 +0.05         F0Vn               +0.125-0.056 +.028-026V      95  6.0  95.6
 582112    PscBD+02  311  12235110266                691    015457.0+023708020009.2+030550154.33-55.53 5.88  +0.62 +0.19         G2IV               +0.231-0.251 +.034-017V?  =< 10
 583 57    CetBD-21  356  12255167466    I                  015503.9-211837015946.1-204928194.60-73.19 5.41  +1.65 +1.98         M1III              +0.008+0.019      -015V
 584          CP-66  123  12270248472                       015510.2-655439015753.6-652529292.20-50.30 6.37  +0.90 +0.62         G8II-III           -0.010-0.013      +012
 585 59Ups CetBD-21  358  12274167471  71I                  015517.6-213344020000.3-210440195.47-73.25 4.00  +1.57 +1.91 +1.04   M0III             e+0.135-0.024 +.007+018                          *
 586 52    CasBD+64  282  12279 12095                       015525.1+642507020252.7+645405130.42  3.07 6.00  +0.03 +0.02         A1Vn               -0.002-0.015      -025SB    305
 587          BD-09  380  12292129624    I W       AR Cet   015529.0-090029020026.9-083125167.40-65.25 5.51  +1.52 +1.34         M3III              +0.096-0.010      +006V          4.1  62.2      *
 588          CD-42  684  12296215726                       015531.5-423047015938.8-420150264.25-69.60 5.57  +1.06               K1III              -0.054-0.103 +.018+027
 589 53    CasBD+63  274  12301 12097                       015535.7+635425020300.3+642324130.57  2.59 5.58  +0.38 -0.27 +0.31   B8Ib               +0.005-0.005      -020
 590  4    PerBD+53  439  12303 228591054                   015538.2+540015020218.1+542915133.19-06.97 5.04  -0.08 -0.32 -0.05   B8III              +0.035-0.002      -002SB     93
 591   Alp HyiCP-62  162  12311248474  72                   015537.1-620323015846.2-613411289.45-53.76 2.86  +0.28 +0.14 +0.14   F0V                +0.264+0.027 +.048+001V     153                 *
 592 49    CasBD+75   86  12339  4565        1625           015557.4+753804020531.2+760654127.44 13.90 5.22  +0.95 +0.75         G8III              -0.022-0.021 +.022+000V    < 17  7.0   5.4AB   3*
 593   Sig HyiCP-78   42  123632558352134                   015600.7-785014015550.5-782054298.84-38.26 6.16  +0.44 +0.01         F5-6IV-V           +0.096+0.060      +006
 594   Pi  ForCD-30  703  12438193455                       015646.8-302856020114.7-300006227.66-74.42 5.35  +0.88 +0.46         G5III              -0.104-0.109 +.017+024
 595113Alp PscBD+02  317  12446              1615B          015652.3+021651020202.8+024549155.35-55.60 5.23H                     A3m                +0.033-0.005 +.005+009SB?    24  1.0   2.0      *
 596113Alp PscBD+02  317  12447110291        1615A   705    015652.3+021651020202.8+024549155.35-55.60 4.33H +0.03 -0.05  0.00   A0pSiSr            +0.033-0.005 +.005+009SB     42  1.0   4.0      *
 597          BD+80   64  12467   3443943                   015704.5+804903020925.3+811745125.99 18.89 6.05  +0.11 +0.06         A1V                -0.036+0.006      -013V     130
 598          BD+64  285  12468 12105                       015708.5+643724020440.1+650612130.54  3.32 6.52   0.00 -0.05         A0V                +0.051-0.032      -004V     130
 599  3Eps TriBD+32  369  12471 55218        1621    707    015707.2+324808020258.0+331702139.71-27.25 5.50  +0.03 +0.06 +0.03   A2V                -0.016-0.013 +.010+003SB?    92  6.0   4.2      *
 600          CP-66  125  124772484762137                   015703.6-663304015941.1-660359292.34-49.64 6.10  +1.17 +1.21         K1III              +0.032+0.006      +005
 601          BD+12  271  12479 92763    I                  015712.0+125940020235.2+132836148.19-45.88 5.94  +1.59 +1.83         M2III              +0.016-0.010      -007                          *
 602   Chi PheCD-45  659  125242157392138                   015741.8-451142020142.4-444249268.87-67.48 5.14  +1.49 +1.82 +0.89   K5III              -0.028-0.053 -.019-031
 603 57Gam1AndBD+41  395  12533 37734  73I   1630A          015745.4+415100020354.0+421947136.96-18.56 2.26  +1.37 +1.58 +0.68   K3-IIb             +0.045-0.052 +.013-012SB   < 17  2.8   9.6AxBC 4*
 604 57Gam2AndBD+41  395  12534 37735        1630BC         015746.3+415104020354.7+421951136.97-18.56 4.84  +0.03 -0.12         B8V+A0V            +0.037-0.055 +.013-014SB2O   70  2.8   9.6AxBC 4*
 605 10    AriBD+25  341  12558 75114        1631           015758.6+252714020339.3+255608142.61-34.16 5.63  +0.54 +0.02         F8IV               +0.134+0.022 +.025+016V          1.5   0.8AB   3*
 606          CD-30  714  12563167511                       015759.5-300852020228.1-293954226.33-74.19 6.42  +0.14 +0.19 +0.04   A3III              -0.033+0.027      +012        0
 607 60    CetBD-00  307  125731296552142                   015803.9-002113020311.7+000742158.18-57.75 5.43  +0.15 +0.13 +0.08   A5III              +0.079+0.017      +013
 608          BD-16  356  125831481702141                   015809.7-154716020258.6-151821181.14-69.60 5.86  +0.98 +0.74         G3IV               +0.028+0.011      +006V
 609          BD+17  307  12594 92774    I D                015813.4+174622020342.6+181512146.03-41.35 6.21  +1.42              gK4                 -0.007-0.018      +010V                         *
 610 61    CetBD-01  285  12641129667        1634    711    015840.9-004911020348.2-002025158.88-58.07 5.93  +0.88 +0.50         G5II-III+G5V       +0.079-0.046      +024           3.8  42.9AB   3*
 611          BD-04  324  126421296652143I                  015838.2-043457020340.5-040613162.91-61.23 5.62  +1.59 +1.97         K5I:               +0.006-0.068      +025V?
 612   Nu  ForCD-29  706  127671675321055          Nu  For  020000.5-294636020429.4-291749224.89-73.78 4.69  -0.17 -0.51 -0.14   B9.5pSi            +0.012+0.008      +019       87                 *
 613 12Kap AriBD+21  279  12869 75146                       020058.0+221018020633.9+223854144.78-37.02 5.03  +0.11 +0.13 +0.03   A2m                +0.018-0.035      +012SB2O   29                 *
 614          BD+07  324  12872110337    I         WZ Psc   020055.3+074614020612.3+081451152.67-50.26 6.31  +1.65 +1.82 +1.00E  M2III:             -0.005-0.036      -026                          *
 615 11    AriBD+25  349  12885 75149        1658           020108.9+251340020649.2+254217143.52-34.14 6.15  -0.03 -0.26         B9IV-Vn            +0.017-0.011      -009      300  5.5   1.6
 616          BD-00  318  12923129695                       020121.8-002632020629.3+000206159.58-57.40 6.28  +0.90 +0.63         K0                 +0.064-0.010      +021V
 617 13Alp AriBD+22  306  12929 75151  74I           725    020132.0+225923020710.4+232745144.57-36.20 2.00  +1.15 +1.12 +0.62   K2-IIICa-1         +0.190-0.148 +.049-014SB   < 17                 *
 618          BD+57  494  12953 22959              V472 Per 020141.3+575652020840.5+582525132.92-02.94 5.67  +0.61 -0.02 +0.50   A1Ia              t-0.011+0.003      -036V      18                 *
 619          BD+43  431  13013 37794                       020218.3+435907020833.6+442734137.14-16.27 6.42R                     G8III-IV           +0.016-0.051      +024
 620 58    AndBD+37  486  13041 552892145                   020226.9+372305020829.3+375133139.31-22.55 4.82  +0.12 +0.14 +0.07   A5IV-V             +0.157-0.044 +.006+008SB    133
 621          BD+53  460  13137 229932146                   020324.8+532214021007.8+535035134.48-07.25 6.31  +0.95               G8III              +0.027-0.043      +010
 622  4Bet TriBD+34  381  13161 55306  75I                  020335.4+343052020932.6+345914140.55-25.20 3.00  +0.14 +0.10 +0.08   A5III              +0.150-0.040 +.022+010SB2O   76                 *
 623 14    AriBD+25  355  13174 75171      W                020343.6+252801020925.3+255623144.08-33.71 4.98  +0.33 +0.15         F2III              +0.075-0.039      +001V     154  2.5 105.9AC   3
 624          BD+16  247  13201 92810                       020353.3+164519020923.1+171328148.23-41.79 6.43                      F5V                +0.142-0.181      +011    =< 10
 625          BD-18  374  13215148237    I           732    020401.1-181511020845.7-174646189.29-69.87 6.10  +1.61 +2.02 +0.95E  M2III              -0.002-0.032      +014                          *
 626          BD+73  121  13222  45992149                   020407.8+733327021321.2+740140128.58 12.06 6.29  +0.91               G8III              +0.048-0.037      -037V
 627  5    PerBD+56  438  13267 23011        1685  Var?     020431.0+571024021129.0+573845133.51-03.57 6.36  +0.33 -0.43 +0.26   B5Ia               -0.013+0.005      -034V      53  6.9  60.0AC   3*
 628 59    AndBD+38  425  13294 55330        1683A          020448.6+383403021052.8+390222139.38-21.28 5.63  -0.02 -0.09         B9V                -0.015-0.018D+.010+001V     192  0.8  16.2      *
 629 59    AndBD+38  425  13295 55331        1683B          020449.5+383417021053.7+390235139.38-21.28 6.10  -0.07 -0.17         A1Vn               -0.009-0.026D+.010+015V?    305  0.8  16.2      *
 630          CD-24  921  133051675992147                   020459.9-244904020934.8-242045208.31-72.20 6.48  +0.29               F4III              -0.027-0.044      +020
 631 15    AriBD+18  277  13325 928221056I           738    020504.9+190143021037.6+193001147.40-39.58 5.70  +1.65 +1.91         M3IIIab            +0.088-0.028      +060V
 632          CD-44  632  13336215785                       020509.7-435919020909.3-433100264.22-67.31 5.85  +1.19 +1.08         K1III              -0.048-0.053      -026
 633 16    AriBD+25  362  13363 75188    I                  020530.5+252755021112.0+255613144.53-33.57 6.02  +1.36              gK4                 -0.007-0.010      -019
 634  5    TriBD+30  347  13372 553382148            740    020534.2+310319021125.0+313135142.27-28.32 6.23  +0.11 +0.14         A1m                +0.035-0.013      +011SB2    30                 *
 635 64    CetBD+07  347  13421110390                       020604.2+080606021121.1+083411154.20-49.39 5.63  +0.56 +0.14         G0IV               -0.139-0.112 +.034-018V?  =< 10
 636          CD-44  638  13423215794                       020605.5-441715021004.9-434856264.54-66.99 6.32  +0.90               G8III              +0.059-0.009      +041V?
 637          CD-51  532  13445232658                       020623.7-511852021025.6-504928275.99-61.97 6.12  +0.82 +0.45 +0.32E  K1V                +2.119+0.641 +.090+053
 638          BD-10  447  134561482622150                   020628.0-103106021122.2-100308174.54-64.57 6.01  +0.39 -0.03         F5V                -0.022-0.166      +011    =< 10:
 639 63    CetBD-02  375  134681297392151                   020631.1-021744021135.8-014931163.52-58.23 5.93  +0.97 +0.70        gG9                 -0.008-0.031      +032
 640 55    CasBD+65  239  13474 12180  76  S         753    020637.7+660321021429.1+663128131.08  4.98 6.07  +0.63 +0.33         G0II-III+B9V       -0.006-0.006      -012V      70                 *
 641          BD+57  519  13476 23044                       020637.7+580529021341.5+583340133.50-02.61 6.44  +0.60 +0.23 +0.51   A3Iab              -0.012+0.020      -041V?  =< 45                 *
 642  6    TriBD+29  371  13480 55347    I   1697  TZ Tri   020634.0+295004021222.3+301811142.98-29.40 4.94  +0.78 +0.31         G5III+F5V          -0.064-0.063 +.004-018SBO    36  1.6   3.8      *
 643 60    AndBD+43  447  13520 378672153I S         746    020656.8+434545021313.3+441354138.04-16.23 4.83  +1.48 +1.74 +0.60E  K3.5IIIBa0.5:      -0.020-0.016      -046SB1O < 17                 *
 644          BD+23  297  13522 75203    I                  020658.0+234153021237.5+241004145.69-35.09 5.96  +1.37               K0                 +0.041-0.006      -001
 645          BD+50  481  13530 23047  77  W         747    020657.0+503605021336.3+510357135.85-09.73 5.31  +0.93 +0.62 +0.34E  G8III:            v+0.346-0.171 +.009+027SBO        4.4  27.3      *
 646 17Eta AriBD+20  348  13555 75204                       020712.0+204428021248.1+211239147.14-37.81 5.27  +0.43 -0.05         F5V                +0.161+0.003      +006V?      9                 *
 647          BD+46  536  13594 37878        1709           020737.7+470102021402.6+472903137.10-13.10 6.06  +0.40 -0.07 +0.25   F4V                -0.063-0.059 +.036-008V    < 26  0.6   1.0      *
 648 19    AriBD+14  357  13596 928411057I           748    020735.9+144841021303.3+151647150.40-43.20 5.71  +1.55 +1.94 +0.97   M0III              +0.097-0.023      +023SB                        *
 649 65Xi 1CetBD+08  345  136111104081058I D         749    020741.9+082240021300.0+085048154.55-48.95 4.37  +0.89 +0.60 +0.49   G6II-IIICN-2       -0.022-0.009 +.022-004SBO  < 17  2.0   0.0      *
 650 66    CetBD-03  336  13612129752        1703           020740.9-025140021247.5-022337164.60-58.52 5.54  +0.57 +0.09         F8V                +0.370-0.069 +.045-003V?         2.1  16.4AB   3*
 651          BD-21  396  13692167637                       020821.2-212812021300.9-210001199.11-70.43 5.86  +1.01 +0.78         G9III              +0.050+0.035      +038
 652   Mu  ForCD-31  882  13709193573  78                   020830.2-311134021254.5-304326229.19-71.84 5.28  -0.02 -0.06         B9V                +0.018+0.006 +.064+010V
 653          BD+47  590  13818 37905                       020929.9+472051021557.9+474841137.30-12.69 6.33R                     G9III-IV           +0.062-0.071      +016                          *
 654          BD+56  471  13854 23115      W       Var?     020952.0+563525021651.7+570319134.38-03.91 6.48  +0.28 -0.66 +0.19   B1Iab             e+0.001-0.002      -042V      57  0.9 103.0      *
 655  7    TriBD+32  409  13869 55397                       021001.2+325339021556.2+332132142.55-26.27 5.28  -0.01 -0.03         A0V                -0.017-0.038      -001V     122
 656 20    AriBD+25  373  13871 75239                       021002.3+251908021546.0+254659145.74-33.33 5.79  +0.44 +0.04 +0.21   F6IV-V             +0.176-0.063      +026V?     12                 *
 657 21    AriBD+24  329  13872 752381059  W                021002.2+243447021542.8+250235146.07-34.01 5.58  +0.49 +0.01         F6V                -0.088-0.087D+.015-044           0.3   0.2      *
 658          BD-10  460  13936129781                       021033.5-095555021528.3-092756175.21-63.45 6.55  -0.02 -0.07         A0V                -0.003+0.031      -001      118
 659          CD-41  621  139402158311060                   021029.1-413757021432.0-411000257.70-67.91 5.91  +0.97               G8III              -0.013-0.028      +014SB
 660  8Del TriBD+33  395  13974 55420        1739           021056.8+334600021703.2+341327142.43-25.39 4.87  +0.61 +0.02 +0.24E  G0.5V              +1.154-0.245 +.093-006SB2O=< 10  8.5  65.4      *
 661  8    PerBD+57  535  13982 231432157                   021054.8+572610021759.9+575359134.25-03.06 5.75  +1.17               K3III              +0.063+0.005      +003
 662  7    PerBD+56  486  13994 23149        1753           021102.1+570310021804.5+573100134.39-03.42 5.98  +1.05 +0.79         G7III              -0.019+0.006      -011V          2.6 121.9AD   4*
 663          BD+43  461a 14028                    W And    021112.9+435035021733.4+441825138.78-15.90 6.7                       M7-8Se            v+0.041+0.083      -045V                         *
 664  9Gam TriBD+33  397  14055 55427  79                   021122.0+332305021718.9+335050142.65-25.71 4.01  +0.02 +0.02  0.00   A1Vnn              +0.047-0.051 +.039+014V     208
 665          BD+23  307  14067 752622156                   021131.1+231818021710.4+234604147.05-35.06 6.55  +1.02 +0.85         G9III              -0.033-0.039      -013
 666 67    CetBD-07  393  14129129798  80                   021159.7-065258021659.0-062520171.27-60.99 5.51  +0.96 +0.76         G8.5III            +0.093-0.108      +007V?
 667   Pi 1HyiCP-68  126  14141248518                767    021209.0-681830021414.7-675030291.57-47.42 5.55  +1.55 +1.85         M1III              +0.037+0.037 +.013+026                          *
 668          BD+63  320  14171 12226                       021231.5+635233022012.9+642014132.36  3.10 6.60  -0.03 -0.09         B9.5V              -0.019+0.018      -026    =< 41
 669 22The AriBD+19  340  14191 92877  81                   021233.6+192619021807.5+195404149.27-38.50 5.62  +0.01 +0.03         A1Vn               -0.013-0.002      +006SB    175
 670 62    AndBD+46  552  14212 379481063                   021249.4+465507021916.8+472248137.99-12.91 5.30  -0.01  0.00         A1V                -0.059-0.008 +.015-030V      74
 671          BD+45  589  14213 37947                       021245.6+460040021910.9+462821138.29-13.77 6.21                      A4V                -0.014-0.009      -015
 672          BD+01  410  142141104561061                   021249.6+011706021801.4+014528162.20-54.38 5.58  +0.60 +0.10         G0.5IVb            +0.368+0.369 +.038+027SBO                       *
 673          BD+48  648  14221 37949                       021251.1+482931021922.7+485719137.46-11.42 6.37R                     F4V                -0.093+0.068      -019       15
 674   Phi EriCP-52  285  14228232696  82  W                021256.2-515830021630.6-513044275.35-60.82 3.56  -0.12 -0.39 -0.11   B8V-IV             +0.096-0.027      +010      247  5.0  87.6      *
 675 10    TriBD+27  360  14252 75276        1770           021309.1+281052021857.0+283833145.22-30.41 5.03  +0.04 +0.05         A2V                +0.011-0.006      +003V      29  7.5  57.1
 676          BD+22  329  14262 75277                       021319.0+224224021858.0+231004147.80-35.44 6.46  +0.34 +0.08         A1V+F3III           0.000-0.005      -013SBO   116                 *
 677          BD+39  521  14272 55453                       021327.6+392228021937.3+395006140.79-19.96 6.63  -0.09 -0.37         B8V                +0.033-0.010      -008V?     15
 678   Pi 2HyiCP-68  128  14287248521                       021323.4-681235021528.5-674447291.35-47.45 5.69  +1.30 +1.48         K2III              +0.035-0.016      +017
 679          BD+46  557  14372 37955                       021412.9+465105022041.4+471839138.24-12.89 6.11  -0.08 -0.47         B5V                +0.007-0.007      +002V
 680          BD+29  392  14373 75287                       021413.5+294344022004.4+301118144.79-28.89 6.47R                     K0                 +0.016-0.023      -001
 681 68Omi CetBD-03  353  14386129825    I   1778  Omi Cet  021417.6-032554021920.7-025839167.75-57.98 3.04  +1.42 +1.09 +1.90   M7IIIe+Bep        v-0.008-0.237 +.024+064V          7.3 118.7AC   4*
 682 63    AndBD+49  640  14392 37960                790    021420.7+494134022058.2+500905137.29-10.21 5.59  -0.13 -0.38         B9pSi              +0.034-0.027      -002V     105                 *
 683          CD-26  828  14412167697                       021429.6-262508021858.5-255644214.47-70.41 6.34  +0.73 +0.18         G5V                -0.214+0.448 +.089+007V?
 684          BD-05  438  14417129830                       021439.2-044820021940.8-042044169.55-58.98 6.50  +0.08 +0.09         A3V                +0.041+0.005      +018
 685  9    PerBD+55  598  14489 232562159    1802  V474 Per 021522.8+552317022221.4+555044135.51-04.79 5.17  +0.37 -0.11 +0.32   A2Ia              v-0.002-0.002      -015V      25  7.8  11.6      *
 686          CD-42  785  14509215860    I                  021525.4-421832021924.7-415054257.92-66.77 6.37  +1.16               K2III              +0.009+0.023      +023
 687          BD+40  500  14622 379862161  W                021636.7+405633022250.3+412347140.80-18.28 5.82  +0.27 +0.05         F0III-IV           -0.077-0.096      -035V?         1.8 295.5AC   3
 688          CP-56  413  146412327172158  W                021639.8-562414021954.3-555641280.10-57.09 5.81  +1.55 +1.95         K5III              +0.026+0.020      +049           5.1  33.8
 689 69    CetBD-00  355  14652110495    I           805    021649.1-000340022156.6+002345164.95-54.92 5.28  +1.65 +1.87         M2III              -0.013-0.007      +023
 690          BD+54  535  14662 23283        1820  V440 Per 021654.0+545434022351.8+552152135.87-05.17 6.28  +0.86 +0.64         F7Ib               +0.006-0.024      -026V      12  8.1  10.0      *
 691 70    CetBD-01  322  146901298582160                   021706.6-012025022212.4-005306166.40-55.90 5.42  +0.31 +0.10         F0Vn               -0.019-0.048      +020SB
 692          BD-11  448  14691148354                       021708.0-111355022201.4-104640179.78-63.15 5.46  +0.35 +0.02         F0V                +0.140-0.085      +012V     101
 693          BD-18  409  147281483561064                   021722.2-180702022205.0-173944193.30-67.06 5.87  +1.23 +1.35 +0.44E  K0III              +0.011-0.060      -003                          *
 694 64    AndBD+49  649  14770 38005    I                  021746.0+493311022424.9+500024137.86-10.15 5.19  +0.98 +0.76         G8III              +0.024-0.039 -.001-013V?   < 17
 695   Kap ForCD-24 1038  14802167736  83                   021758.0-241614022232.6-234859208.85-69.21 5.20  +0.60 +0.12 +0.36   G0Va               +0.200-0.062 +.077+018        4
 696 10    PerBD+55  612  14818 23304              Var      021812.1+560922022516.0+563636135.62-03.93 6.25  +0.31 -0.62 +0.20   B2Ia              e-0.002+0.008      -046V   =< 45                 *
 697          BD-19  444  14830148366                       021815.2-184824022257.8-182116195.13-67.20 6.22  +0.94               G5                 +0.149-0.109      +020V?
 698          CD-43  724  148322158782162                   021816.5-433925022211.8-431200260.03-65.60 6.31  +1.00               K0III              +0.064+0.035      -007
 699 65    AndBD+49  656  14872 233192165I W                021857.0+494934022537.4+501643137.95-09.83 4.71  +1.53 +1.89 +0.87   K4+III             +0.027-0.010 +.011-005V?   < 17  5.7 191.7AC   3
 700          CD-38  797  14890193679                       021859.1-380148022306.5-373435247.15-68.09 6.53  +1.61 +1.94         K2III              -0.014-0.037      +011
 701          CD-51  571  14943232736                       021923.5-513255022254.6-510532273.28-60.44 5.92  +0.22               A5V                +0.022+0.061      +005
 702 24Xi  AriBD+09  316  14951 929322164                   021927.3+100928022449.0+103638157.03-45.99 5.47  -0.10 -0.48         B7IV               +0.020-0.017      +004SB    175
 703          CD-26  857  149881677572163                   021950.5-261804022420.1-255051214.69-69.20 6.44  +1.32               K3III              +0.036+0.013      +034
 704 71    CetBD-03  374  15004129888                       021955.1-031357022458.4-024648169.53-56.93 6.33   0.00 -0.14         A0III              -0.001-0.007      +012V                         *
 705   Del HyiCP-69  113  150082485451065                   021958.1-690652022144.9-683934291.25-46.37 4.09  +0.03 +0.05 +0.01   A3V                -0.050+0.002 +.050+006V?    163
 706          CD-41  681  15064215892                       022032.3-411746022433.8-405026254.50-66.45 6.18  +0.66               G2V                +0.222+0.111      +001SB1O                      *
 707   Iot CasBD+66  213  15089 12298        1860  Iot Cas  022049.2+665711022904.0+672409132.12  6.29 4.52  +0.12 +0.06 +0.06   A5pSr             v-0.017+0.011 +.023+001V?     46  2.2   2.3AB   4*
 708 72Rho CetBD-12  451  151301483851066                   022107.1-124429022557.0-121726183.75-63.35 4.89  -0.03 -0.07 -0.03   B9.5Vn             -0.010-0.009 +.029+010V     191
 709 66    AndBD+49  666  15138 23353                       022109.0+500722022751.8+503411138.18-09.42 6.12  +0.41               F4V                +0.036-0.093      -004SBO                       *
 710          BD-15  426  15144148386        1849  AB Cet   022115.1-154727022600.3-152028189.53-65.08 5.83  +0.15 +0.10 +0.04   A6VpSrCr           -0.058-0.047 +.015-008SBO    14  3.   12. AB   3*
 711          BD+26  409  15152 75370    I                  022120.3+263354022707.0+270048147.91-31.18 6.18  +1.43 +1.82         K5III              -0.055-0.065      -048
 712 11    TriBD+31  427  15176 55570                       022132.1+312108022727.7+314805145.72-26.79 5.54  +1.12              gK1                 -0.027-0.028      -039V
 713          BD-20  455  152201677952167                   022156.3-202944022635.2-200234200.01-67.12 5.88  +1.26 +1.33         K2III              +0.088+0.099      +042V
 714   Lam HorCP-60  199  15233248555  84                   022206.1-604535022453.9-601843283.79-53.20 5.35  +0.39 +0.06         F2III              -0.068-0.133 +.023+006V     106
 715   Kap HyiCP-74  194  152482558801067                   022216.2-740555022252.3-733845294.47-41.95 5.01  +1.09 +1.04         K1III              -0.083+0.010      +022
 716          BD+54  557  15253 23369        1878           022222.4+550520022925.0+553211136.55-04.72 6.51  +0.09  0.00         A2pShell          v+0.033-0.010      +002           1.3   2.8      *
 717 12    TriBD+29  417  15257 753821068                   022218.0+291323022810.0+294010146.86-28.67 5.29        +0.10         F0III              -0.015-0.087      -025V      78
 718 73Xi 2CetBD+07  388  15318110543  85I'                 022250.4+080043022809.5+082736159.69-47.39 4.28  -0.06 -0.12 -0.05   B9III              +0.040-0.009 +.030+011SB?    63
 719          BD+01  431  15328110542      W                022250.2+013046022800.0+015739165.42-52.76 6.45  +0.97 +0.79         K0III              -0.002-0.012      +018           0.4   0.5
 720 13    TriBD+29  423  15335 75391                       022256.3+292855022848.5+295555146.89-28.37 5.89  +0.58 +0.01?        G0V                -0.069+0.074      +040V   =<  6
 721   Kap EriCD-48  637  15371215906  86I'                 022319.2-480909022659.1-474214267.12-62.24 4.25  -0.14 -0.50 -0.13   B5IV               +0.023-0.010      +028SB?    24                 *
 722          CP-67  154  15379248556              TZ Hor   022322.4-665639022526.3-662941289.17-48.04 6.41  +1.54 +1.46 +1.74   M5III              -0.029-0.019      +002                          *
 723          BD+22  354  15385 75398                       022331.4+230121022913.6+232808150.24-34.16 6.19  +0.15 +0.14 +0.02   A5m                +0.081-0.028      +021V      21
 724   Phi ForCD-34  905  154271937232168                   022347.8-341532022801.7-334840236.68-68.20 5.14  +0.10               A2V                +0.020+0.007 +.015+019V     127
 725          BD+08  385  15453110565        1896           022414.8+090709022935.3+093357159.26-46.27 6.07  +1.02 +0.85         K2III              -0.012+0.012      -011           4.5   1.6      *
 726          BD+33  445  15464 55611                       022415.4+332323023016.6+335002145.40-24.69 6.25  +1.07               K1III              +0.070-0.057      +007
 727          CD-31  990  15471193727                       022416.8-313256022835.4-310609229.36-68.46 6.11  +1.11 +1.07         K2IIICNII          -0.034-0.019      -002
 728          BD+24  358  15524 754072171    1904           022447.0+244731023032.4+251406149.62-32.45 5.92  +0.41 +0.06         F6IV               +0.066-0.078D+.022-011V      50  4.5 113.1AC   3*
 729 26    AriBD+19  365  15550 929792172  D       UU Ari   022501.8+192441023038.4+195119152.63-37.23 6.15  +0.25 +0.10         A9V                +0.078-0.033      +019V     152                 *
 730          CD-23  942  15588167832        1906           022521.3-230742022955.4-224058207.10-67.27 6.77  +0.19 +0.15 +0.05   A9V                +0.077+0.006      +013           5.7  28.4
 731 27    AriBD+17  380  15596 929831069                   022521.4+171542023054.4+174214154.01-39.09 6.23  +0.90 +0.55         G5III-IV           +0.031-0.084      -118V?
 732          BD-00  378  15633110583                846    022538.1-001114023045.2+001519168.08-53.68 6.00  +0.17 +0.17         A7III-IV s         -0.053-0.071      -002V?     42                 *
 733          CD-25  979  15634167837              TY For   022544.0-253756023013.8-251111213.52-67.78 6.51  +0.29              dA9n                +0.090+0.031      +025V?                        *
 734          CP-64  174  156462485672170                   022544.2-644447022804.4-641759287.00-49.73 6.37  -0.04 -0.05         A0V                +0.037+0.001      +010
 735          CD-23  947  15652167839    I                  022559.2-225920023032.8-223244206.86-67.09 6.10  +1.61 +2.00 +0.84E  M1III              +0.008-0.033      -019                          *
 736 14    TriBD+35  497  15656 556351070I                  022559.8+354214023206.2+360850144.74-22.43 5.15  +1.47 +1.78         K5III              +0.046+0.012      -036     < 19:                *
 737          BD+01  438  156941105892173I                  022619.7+014927023130.1+021602166.25-51.98 5.25  +1.27 +1.41         K3III              +0.027-0.004 +.041+026V    < 19:
 738          BD+33  454  15755 55650                       022650.1+340603023252.5+343233145.62-23.82 5.83  +1.07               K0III              -0.066-0.012      -002V
 739 75    CetBD-01  353  15779129959                       022704.0-012835023209.4-010206169.94-54.44 5.35  +1.02 +0.84        gG3                 -0.026-0.037      -005V?
 740 76Sig CetBD-15  449  157981484451071                   022720.8-154101023205.2-151441191.12-63.78 4.75  +0.45 -0.02 +0.27   F4IV               -0.070-0.120 +.035-029       12
 741 29    AriBD+14  419  15814 929982176                   022725.4+143531023254.1+150205156.28-41.18 6.04  +0.54 +0.04         F8V                -0.028+0.037      +006SB  =< 10
 742          CD-36  957  15889193759                       022807.5-365209023214.8-362539242.77-66.74 6.30  +1.02 +0.74 +0.37E  G8III              +0.065+0.009      +013                          *
 743          BD+72  140  15920  4694  87                   022830.9+722252023802.0+724906130.71 11.58 5.16  +0.88 +0.58         G8III              -0.030+0.016 +.011-002V?   < 19:
 744   Lam1ForCD-35  877  15975193763  88                   022856.8-350523023307.0-343900238.27-67.00 5.90  +1.06               K0III              -0.015-0.020      +013
 745          BD-20  480  15996167878                       022903.3-202622023340.2-200007201.48-65.57 6.21  +1.10               K0                 -0.041-0.079      -006V
 746          BD+39  573  16004 55680        1961           022912.5+391337023527.9+393952143.84-18.95 6.36  -0.10 -0.34         B9pHgMn            +0.026-0.026      -008V?     30  5.6  16.3AB   3*
 747          BD+65  280  16024 123612178I                  022925.7+651833023736.1+654444133.55  5.09 5.78  +1.56 +1.85         K5III              +0.042-0.005      +041
 748          BD+36  519  16028 55684    I   1964           022928.9+365229023538.8+371844144.92-21.07 5.71  +1.39               K3III              +0.004-0.009      -006           5.2  18.3AB   3
 749   Ome ForCD-28  819  16046167882        1954A          022927.9-284018023350.7-281357221.70-67.34 4.90  -0.05 -0.13 -0.07   B9.5V              -0.019-0.013      +010V      72  2.8  10.8      *
 750 15    TriBD+34  469  16058 55687    I W         866    022942.5+341505023546.8+344115146.14-23.44 5.35  +1.66 +1.93 +1.00E  M3IIIa             +0.030-0.049 +.016-010V          1.2 141.3      *
 751          BD+06  392  16060110625                       022946.3+070210023504.1+072817162.60-47.26 6.18  +1.06 +0.89        gG6                 -0.006-0.108      -025                          *
 752 77    CetBD-08  484  16074129984    I W                022946.5-081746023442.7-075134179.49-58.87 5.75  +1.40 +1.65        gK4                 +0.063-0.065      +025           6.7  94.7      *
 753          BD+06  398  161601106361073  W                023035.7+062435023604.9+065313163.39-47.62 5.82  +0.98 +0.81 +0.53   K3V                +1.811+1.451 +.147+023SB         5.0   2.5AP   3*
 754 78Nu  CetBD+04  418  161611106351072I   1971           023037.5+050925023552.5+053536164.48-48.67 4.86  +0.87 +0.53 +0.49   G8III              -0.026-0.025 +.002+005SB   < 17  4.8   8.1      *
 755          CD-51  611  161702328182177                   023030.2-513153023354.6-510537270.92-59.19 6.24  +0.52 +0.07         F6V                -0.004-0.030      -008
 756          BD+38  515  16176 55705                       023042.4+381808023657.2+384358144.53-19.67 5.90  +0.48               F5V                +0.145-0.194      +001       27
 757          BD+30  418  16187 55703                       023045.2+311018023642.9+313627147.80-26.13 6.10  +1.05 +0.79         K0                 -0.042-0.004      +003V
 758          BD+33  470  16210                    R Tri    023057.7+334953023702.5+341550146.59-23.71 5.3   +1.55 +0.84         M4IIIe             +0.058-0.029 +.008+067V                         *
 759 80    CetBD-08  489  162121300041074I W                023104.6-081559023600.0-074954179.86-58.61 5.53  +1.59 +1.93 +0.87E  M0III              -0.033-0.059      +014           3.6 146.7AC   3
 760          BD+39  582  16219 55715                       023104.3+392740023720.8+395345144.09-18.59 6.54  -0.12 -0.48         B5V                +0.006-0.019      +016SBO    35                 *
 761          BD+32  473  16220 55711                       023105.2+322717023706.4+325331147.26-24.94 6.25  +0.48 +0.01         F8V                +0.065+0.063      +000
 762          CP-63  169  16226248595                       023106.3-630132023333.6-623513284.65-50.76 6.77  -0.06               B9V                +0.026+0.010      +001V
 763 31    AriBD+11  360  16234 930222179  D                023110.6+120051023637.9+122651159.13-42.91 5.68  +0.49 -0.08         F7V                +0.286-0.089 +.035+007V   =< 10  0.1   0.0      *
 764 30    AriBD+24  375  16232 75470        1982B          023111.5+241246023657.7+243854151.47-32.31 7.09  +0.50 +0.02         F4V                +0.143 0.000 +.018+017       30  0.6  38.3      *
 765 30    AriBD+24  376  16246 75471        1982A          023114.3+241244023700.5+243851151.49-32.31 6.50  +0.41 -0.02         F6III              +0.136-0.006 +.018+015SBO    60  0.6  38.3      *
 766          BD+07  402  16247110640                872    023117.1+071741023635.1+074347162.83-46.83 5.81  +1.04 +0.86        gK0                 -0.049-0.031      -025
 767   Iot1ForCD-30  958  16307193795                       023150.5-302850023609.3-300241226.39-66.88 5.75  +1.02               G8-K0III           -0.013+0.001      +002V?
 768          BD+37  588  16327 55729        1996           023206.5+371739023817.8+374336145.24-20.47 6.18  +0.47 +0.08         F6III              -0.040-0.044      +009       43  5.0  20.8AC   4*
 769          BD+37  591  16350 55735                       023215.3+373920023827.8+380522145.10-20.13 6.30  -0.03 -0.03         B9.5V              -0.009+0.001      +002    =< 41                 *
 770          BD+07  405  16399110653                       023241.8+071545023800.8+074143163.28-46.66 6.39  +0.44  0.00         F6IV               +0.086-0.044      +013       12
 771 81    CetBD-04  436  164001300262180                   023239.4-034944023741.8-032346174.49-55.25 5.65  +1.02 +0.85        gG5                 +0.041-0.045      +008
 772   Lam2ForCD-35  903  16417193811                       023249.4-350019023658.6-343442237.68-66.24 5.79  +0.66 +0.22         G5IV               -0.015-0.270 +.056+010V?
 773 32Nu  AriBD+21  362  16432 75495  89                   023308.1+213145023849.0+215741153.46-34.47 5.43  +0.16 +0.13         A7V                -0.008-0.017 +.013+008SB?   110
 774          BD+80   86  16458   4333945                   023320.8+810129024747.6+812654127.33 19.59 5.78  +1.30               G8pBa3             +0.010-0.076 +.013+018V?                        *
 775          BD+02  406  16467110655      W                023324.5+030037023836.9+032635167.32-49.96 6.21  +1.00 +0.76         G9III              +0.043+0.002      +002           3.2   0.8
 776   Mu  HyiCP-79   66  16522255898  90                   023347.1-793245023140.5-790634297.19-36.88 5.28  +0.98 +0.73         G8III              +0.121-0.056 +.001-015
 777   Iot2ForCD-30  973  165381938292183                   023359.9-303729023818.7-301139226.75-66.41 5.83  +0.48 -0.01         F4V                +0.100-0.077      +029
 778   Eta HorCP-53  457  165552328352182                   023406.4-525833023724.4-523235272.31-57.82 5.31  +0.27               A6V                +0.100-0.019 +.025-003      239
 779 82Del CetBD-00  406  16582110665  91          Del Cet  023421.3-000610023929.0+001943170.76-52.21 4.07  -0.22 -0.87 -0.21   B2IV               +0.014-0.004 +.001+013SB     13                 *
 780          CD-38  875  165891938342184                   023422.8-382516023824.8-375926245.54-65.10 6.49  +0.52               G0IV               +0.111-0.063      +043SBO                       *
 781 83Eps CetBD-12  501  16620148528      W                023443.6-121748023933.8-115220187.19-60.41 4.84  +0.45 -0.01         F5V+F6V            +0.146-0.236 +.072+015SBO    13  0.2   0.1      *
 782 33    AriBD+26  443  16628 75510        2033           023450.2+263754024041.1+270339151.02-29.79 5.30  +0.09 +0.13         A3V                +0.067-0.031 +.002+017SB    104  4.2  28.6      *
 783          BD+05  374  16647110673                       023459.2+054054024015.7+060643165.32-47.60 6.25  +0.40 -0.03         F3V:               +0.052-0.003      +018V      18
 784          BD-10  525  16673130047                       023520.4-095251024012.3-092711183.54-58.84 5.78  +0.52  0.00         F6V                -0.151-0.086      -004V   =<  6
 785 11    PerBD+54  598  16727 235552188                   023553.2+544045024302.8+550621138.51-04.35 5.77  -0.13 -0.48         B7IIIpHg           +0.034-0.027      -002                          *
 786          CD-31 1081  16733193846                       023545.9-310343024002.5-303802227.82-66.03 6.52  +1.04               K0III              -0.004-0.068      -004V?
 787          BD+52  616  16735 23556        2059           023556.5+530559024259.7+533134139.16-05.79 5.84  +1.12               K0II-III           +0.068-0.030      -012V          9.0  12.4      *
 788 12    PerBD+39  610  16739 557932187  S         896    023556.0+394616024214.9+401138144.84-17.91 4.91  +0.59 +0.14 +0.30   F9V                -0.019-0.186 +.043-023SB2O < 25  0.3   0.1      *
 789          CD-43  814  167542159962185  W                023559.2-431915023948.0-425330255.67-62.98 4.75  +0.06 +0.06 +0.01   A2V                +0.106-0.027 +.028+018V     190:10.0  23.8
 790 84    CetBD-01  377  16765130055        2046    893    023606.5-010715024113.9-004145172.42-52.68 5.71  +0.52 -0.02         F7IV               +0.214-0.136 +.036+008           3.6   4.0      *
 791          BD+67  224  16769 12421  92                   023612.9+672359024449.7+674929133.34  7.27 5.95  +0.10 +0.17         A5III              +0.015-0.033      +005SBO    35                 *
 792          BD+47  683  16780 38274        2064           023619.0+475018024301.9+481556141.40-10.56 6.48R                     G5II               +0.007+0.002      -005           5.0  72.8AC   3*
 793 34Mu  AriBD+19  403  16811 93062        2062           023643.5+193508024222.0+200042155.53-35.74 5.69R -0.02 -0.03         A0Vnp:             +0.029-0.043      -007V     195  0.8   0.1 O   4*
 794   Iot EriCD-40  689  168152159991075I                  023643.2-401700024040.0-395120249.30-64.07 4.11  +1.02 +0.74 +0.56   K0III              +0.138-0.032 +.038-009
 795          BD-03  421  16824130061                       023646.1-033828024148.3-031248175.53-54.40 6.05  +1.17 +1.13        gG9                 -0.019+0.002      +004V?
 796          BD-15  478  16825148550                       023649.4-145843024134.0-143258192.38-61.45 5.98  +0.43 -0.02         F5III:             -0.032+0.042      +002V?
 797          BD+10  360  16861 93067                899    023705.8+101856024228.9+104430162.07-43.53 6.30  +0.06 +0.06 +0.02   A2V                -0.025-0.022      +006V      33                 *
 798          CP-64  192  16891248616                       023722.6-644241023931.7-641655285.44-49.01 6.55  -0.08 -0.24         B8V                +0.025+0.013      -001V?                        *
 799 13The PerBD+48  746  16895 38288  93I   2081    902    023721.9+484820024412.0+491342141.16-09.61 4.12  +0.49  0.00 +0.30   F8V                +0.336-0.089 +.077+025V?      6  5.6  19.6AB   3*
 800 14    PerBD+43  566  16901 382891077                   023734.2+435219024405.2+441749143.31-14.07 5.43  +0.90 +0.65         G0Ib-IICa1CH-1     +0.004-0.007      -003SB?
 801 35    AriBD+27  424  16908 75532  94                   023734.8+271654024327.1+274226151.29-28.93 4.66  -0.13 -0.62 -0.13   B3V                +0.008-0.012      +013SB1O  132                 *
 802   Zet HorCP-55  446  169202328571076                   023732.9-545841024039.6-543300274.43-56.11 5.21  +0.40 -0.01         F4IV               +0.036-0.001 +.031-001SB2O                      *
 803          BD+25  441  16955 75539        2082           023802.9+251247024351.2+253817152.53-30.69 6.35  +0.08 +0.11         A2Vp:              -0.011-0.005 -.000-011           4.1   2.9AB   3*
 804 86Gam CetBD+02  422  16970110707        2080           023807.1+024852024318.0+031409168.92-49.38 3.47  +0.09 +0.07 +0.04   A3V                -0.142-0.152 +.052-005V     183  2.7   2.8AB   3*
 805          CD-38  894  16975193873                       023807.6-384837024206.6-382302245.90-64.29 6.01  +0.92               G8III              +0.018+0.001      +017
 806   Eps HyiCP-68  161  16978248621  95                   023802.9-684143023935.4-681601288.96-45.82 4.11  -0.06 -0.14 -0.07   B9V                +0.086-0.002      +006      110                 *
 807          CD-47  832  170062160132189                   023832.4-465652024208.5-463128261.96-60.83 6.10  +0.88               K1III              +0.015-0.100      +013
 808 36    AriBD+17  426  17017 930812190                   023844.1+172027024419.1+174550157.47-37.42 6.46  +1.07 +1.07 +0.39E  K2III              +0.039-0.038      -032V                         *
 809 37Omi AriBD+14  457  17036 93082                       023902.2+145319024432.9+151842159.22-39.46 5.77  -0.01 -0.19         B9Vn               -0.002-0.019      -007      350
 810   Iot HorCD-51  641  17051232864                       023909.2-511354024233.5-504801268.82-58.33 5.41  +0.56 +0.10         G0V                +0.334+0.222 +.069+016
 811 89Pi  CetBD-14  519  17081148575  97                   023921.7-141656024407.4-135131191.81-60.57 4.25  -0.14 -0.45 -0.14   B7V                -0.008-0.015      +015SB     18
 812 38    AriBD+11  377  17093 93083              UV Ari   023930.5+120130024457.6+122645161.43-41.79 5.18  +0.24 +0.10         A7III-IV           +0.122-0.084 +.027-002V      83                 *
 813 87Mu  CetBD+09  359  17094110723  98  D         909    023932.1+094131024456.5+100651163.25-43.70 4.27  +0.31 +0.08 +0.19   F0IV               +0.284-0.036 +.046+030SBO    54  1.4   0.1      *
 814          CD-41  769  17098216019      W                023927.3-405710024320.3-403139250.30-63.35 6.36  -0.02               B9V                +0.019+0.024D+.009+016           0.3   1.9      *
 815          BD+69  179  17138 12445              RZ Cas   023954.1+691249024855.5+693803132.89  9.07 6.18  +0.18 +0.10         A3V                 0.000+0.032      -039SBO    82                 *
 816          BD+04  437  17163110730                       024006.2+041726024520.9+044242168.09-47.92 6.03  +0.31 +0.08        gF0                 +0.068-0.043      +020SB     75
 817          CD-33  943  17168193888                       024008.8-325650024420.5-323130232.25-65.00 6.22  +0.04               A1V                +0.020-0.031      +021SB     48
 818  1Tau1EriBD-19  518  17206148584                       024026.1-185945024506.2-183421200.83-62.53 4.47  +0.48  0.00 +0.27   F6V                +0.330+0.038 +.072+026SB1O   22                 *
 819          BD+35  553  17228 55868                       024047.4+353348024658.3+355901147.73-21.26 6.25  +0.93 +0.62         G8III              +0.050-0.005      +021                          *
 820          BD+34  513  17240 55872        2117           024054.3+350809024703.5+353318147.96-21.64 6.30R                     A9V                -0.043-0.047D+.013-004       45  2.1   1.5AB   3*
 821          CP-53  475  17254232871                       024058.1-525934024410.7-523414271.10-57.02 6.15  +0.09               A2V                -0.012-0.015      +010
 822          CD-46  797  17325216036                       024141.3-464229024516.5-461714260.97-60.49 6.85  +1.36               K2III              -0.003-0.014      +032V
 823          CP-67  181  17326248632      W         911    024141.7-670806024326.6-664252287.19-46.84 6.26  +0.53 +0.01         F5V                +0.108-0.070 +.010-020           0.0   0.4      *
 824 39    AriBD+28  462  17361 75578    I                  024157.1+284955024754.5+291450151.42-27.10 4.51  +1.11 +1.05 +0.58   K1.5III            +0.149-0.123 +.027-015V    < 17
 825          BD+56  718  17378 23637      S       V480 Per 024207.7+564001024930.8+570503138.47-02.18 6.25  +0.89 +0.50 +0.76   A5Ia               +0.004-0.010      -038V      37                 *
 826          BD-22  479  17390168045                       024212.4-220334024645.2-213823207.44-63.24 6.49  +0.38               F3IV               +0.091+0.009      +007       67
 827          CD-23 1061  17438168051                       024240.8-225412024711.2-222908209.30-63.39 6.47  +0.39               F2                 -0.021-0.038      +003V?
 828 40    AriBD+17  442  17459 93118    I D         937    024255.5+175202024832.1+181701158.18-36.45 5.82  +1.20 +1.13 +0.44E  K1III              +0.044-0.036      +047V          0.0   0.2      *
 829          BD+68  200  17463 124722195          SU Cas   024302.8+682827025158.7+685319133.47  8.52 5.80  +0.64 +0.48         F5:Ib-II           -0.006-0.004 +.010-007SB                        *
 830          BD+24  396  17471 75588              VZ Ari   024256.9+244615024845.9+251117153.91-30.53 5.86R -0.07 -0.08         A0V                +0.059-0.004      +014       60                 *
 831          BD+36  566  17484 55910                       024312.5+365436024927.1+371934147.54-19.84 6.45  +0.43 +0.14         F6III-IV           +0.009-0.016      +012SB     17
 832          BD-13  530  17491148612    I         Z Eri    024308.4-125238024756.0-122738190.42-59.04 6.90  +1.56 +1.37 +1.39E  M4III             v+0.007-0.038      -014                          *
 833   Gam HorCP-64  196  17504248642      W                024319.2-640726024527.5-634216284.11-49.05 5.74  +0.93 +0.63         G8III-IV           +0.022-0.009      -011SB         7.3  20.0
 834 15Eta PerBD+55  714  17506 23655  99I   2157           024323.8+552850025041.8+555344139.14-03.18 3.76  +1.68 +1.89 +0.89   K3-Ib-IIa         e+0.017-0.014 +.006-001SB  =< 54  4.7  28.3AB   6*
 835   Eta1ForCD-36 1050  17528193916                       024330.3-355803024733.7-353303238.99-63.93 6.51  +0.96               K0-1III            -0.041-0.039      +003
 836 42Pi  AriBD+16  355  17543 93127        2151    944    024342.6+170254024917.5+172751158.93-37.04 5.22  -0.06 -0.47         B6V                -0.001-0.015      +009SBO    79  3.0   0.0 A   4*
 837   Zet HyiCP-68  169  175662486442191                   024400.0-680213024532.6-673700287.76-46.00 4.84  +0.06 +0.09 +0.07   A2IV-V             +0.069+0.046 +.009+004V?    100
 838 41    AriBD+26  471  17573 75596 100    2159    951    024405.7+265054024959.0+271538152.98-28.61 3.63  -0.10 -0.37 -0.11   B8Vn               +0.067-0.118 +.034+004SB    180  4.8 124.9AD   4*
 839          BD+57  651  17581 23662      W                024415.2+575402025145.5+581853138.20-00.94 6.45  +0.10 +0.08 +0.03   A1m                -0.066+0.015      -005SBO    33  2.8 192.7      *
 840 16    PerBD+37  646  17584 559282194  W         956    024416.0+375425025035.1+381907147.25-18.86 4.23  +0.34 +0.08 +0.23   F2III              +0.197-0.105 +.024+014      149  4.7 249.3AC   3*
 841   Bet ForCD-32 1025  17652193931 101I W                024454.3-324933024905.4-322421231.79-64.02 4.46  +0.99 +0.69 +0.54   K0-IIIFe-0.5       +0.090+0.155 +.026+017           9.5   4.7      *
 842          BD+46  648  17656 383972197                   024459.8+462546025141.7+465031143.38-11.20 5.88  +0.89 +0.58         G8III              -0.025-0.028      -012V?
 843 17    PerBD+34  527  17709 559462198I A         963    024521.0+343854025130.8+350335149.07-21.64 4.53  +1.56 +1.92 +0.95   K5+III             +0.009-0.061 +.004+014V    < 17                 *
 844   Gam1ForCD-25 1120  17713168081        2167           024525.1-245817024951.0-243337214.12-63.29 6.14  +1.07               K1III              -0.043-0.127      -006           4.4  40.9AC   3*
 845   Gam2ForCD-28  903  17729168082                       024534.2-282125024954.2-275630221.64-63.79 5.39  +0.02               A0V                +0.047+0.028 +.025+024      261
 846          BD+52  640  17743 23674        2185    971    024546.1+523511025252.0+525952140.73-05.63 6.36  +0.07 -0.27         B8III              +0.002-0.011 -.004+001V?         0.2   1.6ABxC 3*
 847 43Sig AriBD+14  480  17769 931441079                   024558.2+144013025129.6+150455161.17-38.72 5.49  -0.09 -0.43         B7V                +0.031-0.026      +017V     195
 848   Eta2ForCD-36 1067  17793193940      W                024612.1-361528025014.8-355037239.41-63.34 5.92  +0.90 +0.62         K0III              +0.058+0.021      +022V          4.3   5.0      *
 849          BD+47  723  17818 38418        2192           024632.1+480935025321.2+483410142.83-09.54 6.26  +1.18 +0.94         G5I                +0.015-0.027      -001           5.7   6.7
 850  2Tau2EriBD-21  509  17824168094 102I   2179           024630.1-212458025102.3-210015206.80-62.09 4.75  +0.91 +0.63 +0.47   K0III              -0.046-0.019 +.031-009          10.2  46.7      *
 851   Eta3ForCD-36 1070  17829193944                       024637.9-360514025040.4-354034239.01-63.28 5.47  +1.25 +1.31 +0.49E  K5III              +0.006-0.061 +.017+012                          *
 852   Nu  HorCP-63  188  178482486562196                   024648.3-631317024901.5-624824282.75-49.46 5.26  +0.10 +0.06         A2V                +0.091+0.021 +.001+031      167
 853          CD-40  736  17864216065                       024655.8-402040025047.9-395554248.01-62.21 6.36  +0.05               A0V                +0.055+0.007      +019V?
 854 18Tau PerBD+52  641  17878 23685 103I   2202  Tau Per  024709.8+522112025415.5+524545141.02-05.74 3.95  +0.74 +0.46 +0.44   G4III+A4V           0.000-0.005 +.019+002SB1O < 25  8.6  51.7AxBC 4*
 855 20    PerBD+37  655  17904 55975        2200    977    024723.7+375548025342.6+382015147.81-18.55 5.33  +0.41 +0.04         F4IV               +0.047-0.077 +.011+006SBO    63  0.6   0.2AB   4*
 856          BD+15  400  17918 93164                       024737.5+160431025311.7+162900160.59-37.33 6.31                      F5III              +0.055-0.062      +009      120
 857          BD-13  544  17925148647                975    024742.9-131030025232.1-124610192.06-58.26 6.04  +0.87 +0.56 +0.45   K2V                +0.394-0.175 +.121+019V
 858          CD-31 1148  17926193951                       024743.1-311342025155.3-304852228.14-63.47 6.40  +0.48               F8IV-V             -0.116+0.106      +008
 859          BD-10  569  17943130160                       024758.0-095108025250.5-092628187.02-56.35 6.32  +0.19 +0.12         A7IV               +0.074+0.050      +035      120                 *
 860          BD+60  591  17948 125172201  W                024801.0+610647025556.9+613116137.21  2.15 5.59  +0.45 -0.10         F4V                +0.142+0.032      +029           5.5  89.6AC   3
 861          BD+63  369  17958 12519    I W         985    024809.2+635533025624.8+641957135.96  4.67 6.24  +2.03 +2.41         K3Ib              v+0.006-0.002      -022           3.4 115.6      *
 862          BD-22  503  18071168121                       024905.1-224657025335.3-222235209.96-61.94 5.95  +1.04               K0III              +0.090-0.080      +049                          *
 863   Psi ForCD-38  948  181491939652200                   024939.2-385045025334.4-382613244.62-62.12 5.92  +0.44 +0.07         F6IV               +0.063+0.031      +021V
 864          BD+50  665  18153 23721                       024949.8+505125025650.6+511539142.08-06.89 6.22  +1.57 +1.90         K5III              -0.011-0.039      +005
 865          BD+46  658  18155 38455      W                024947.8+464531025633.4+470950143.97-10.53 6.02  +1.34              gK3                 +0.004 0.000      -013           5.5  24.9AC   3*
 866          CP-63  197  18185248673                982    025009.9-631906025219.2-625435282.43-49.13 6.03  +1.25               K1III              +0.072+0.001      +017
 867 45Rho2AriBD+17  457  18191 93189              RZ Ari   025011.2+175535025548.5+181954159.93-35.45 5.91  +1.47 +1.12 +2.17   M6-III:            -0.008-0.017 +.014+046V?                        *
 868          CD-50  860  18242                    R Hor    025033.2-501752025352.9-495325265.46-57.38 4.0 H +2.11 +0.43 +3.09   M7IIIe             +0.135+0.014 +.013+060V                         *
 869 46Rho3AriBD+17  458  18256 931952204                   025047.3+173728025626.1+180123160.29-35.62 5.63  +0.43 -0.02         F6V                +0.276-0.213 +.038+015SB1O   15                 *
 870          BD+07  450  18262110851                       025052.7+075846025613.8+082254167.75-43.37 5.97  +0.48 +0.06         F7IV               +0.068-0.087      +029V   =< 10
 871          CD-51  683  182652329332202                   025052.2-511641025406.5-505217266.92-56.80 6.21  +1.58 +1.88         K4III              +0.010+0.006      -013
 872   Nu  HyiCP-75  204  182932559292199                   025106.7-752832025028.5-750401293.35-39.77 4.75  +1.33 +1.56 +0.66   K3III              -0.037-0.027 +.011+005                          *
 873 21    PerBD+31  509  18296 560312205          LT Per   025112.9+313154025717.3+315603151.88-23.77 5.11  -0.01 -0.23         B9pSi             v+0.003-0.031 +.003+008SB     19                 *
 874  3Eta EriBD-09  553  18322130197 104I           988    025132.5-091746025625.7-085353187.15-55.31 3.89  +1.11 +1.00 +0.58   K1-IIIbBa0.2:      +0.079-0.220 +.033-020     < 17                 *
 875          BD-04  502  183311301991080                   025136.6-040653025637.4-034244180.36-52.04 5.17  +0.08 +0.05 +0.05   A1Vn               -0.035-0.044 +.026-015SB    231                 *
 876          BD+38  599  18339 56036    I                  025141.8+381247025802.3+383654148.45-17.90 6.04  +1.41              gK3                 -0.007-0.016      -041V?
 877          BD+03  410  183451108652206I           990    025150.2+040550025704.6+043004171.55-46.17 6.11  +1.69 +1.81         M4IIIab            +0.017+0.023      +052
 878 47    AriBD+20  480  18404 756621081                   025221.6+201604025805.2+204007158.88-33.22 5.80  +0.41 +0.01 +0.20   F5IV               +0.233-0.031 +.030+028       20                 *
 879 22Pi  PerBD+39  681  18411 560472207                   025221.8+391545025845.7+393946148.04-16.92 4.70  +0.06 +0.12 +0.04   A2Vn               +0.024-0.043 +.009+014SB    168                 *
 880          CP-64  206  18423248681                       025226.9-645026025420.9-642608283.75-47.86 6.56  +1.39 +1.59         K2-3III            +0.009-0.004      +006
 881          BD+78  103  18438  4810 105I   2294           025246.6+790125030607.8+792507129.08 18.19 5.49  +1.57               M1III+F7IV         -0.039+0.009D+.008-038SB         4.4   4.7      *
 882 24    PerBD+34  550  18449 560521082I                  025251.7+344657025903.7+351059150.45-20.78 4.93  +1.23 +1.29 +0.64   K2III              -0.046+0.006      -036V?   < 17
 883  4    EriCD-24 1336  18454168183                       025256.9-241547025723.7-235143213.46-61.47 5.45  +0.23               A5V                +0.098-0.030 +.029+029V      81
 884          CD-30 1122  18466194002                       025259.1-301526025713.1-295119226.08-62.31 6.29  +0.47               A2-3V:+G:          +0.012-0.005      +027SB2                       *
 885          BD+46  669  18474 38493                       025302.5+464913025949.9+471315144.44-10.22 5.47  +0.89 +0.61         G5:III*            +0.021+0.024 -.007+007V?                        *
 886          BD+40  639  18482 38492    I          1004    025312.0+403804025939.9+410159147.50-15.64 5.89  +1.44               K2                 +0.019-0.039      +032
 887 48Eps AriBD+20  484  18519 75673        2257B  1001    025329.5+205626025912.7+212025158.69-32.51 4.63  +0.04 +0.08 +0.02   A2V s              -0.017-0.005 +.005-006V      66  0.3   1.5AB   3*
 888 48Eps AriBD+20  484  18520 75673        2257A  1001    025329.5+205626025912.7+212025158.69-32.51 4.63  +0.04 +0.08 +0.02   A2V s              -0.017-0.005 +.005-008SB?        0.3   1.5AB   3*
 889  6    EriCD-24 1343  18535168191    I                  025338.8-240030025805.7-233622213.02-61.25 5.84  +1.33               K2III              +0.060+0.050      +007V
 890          BD+51  665  18537 23763        2270A          025344.3+515715030052.2+522106142.11-05.63 5.28  -0.05 -0.45         B7V                +0.028-0.026      -005SB    210  1.4  11.9      *
 891          BD+51  665  18538 23765        2270B          025345.7+515716030053.4+522108142.11-05.63 6.74   0.00 -0.15         B9V                +0.014-0.021      -002SB    200  1.4  11.9      *
 892          BD-03  470  18543130215      W                025339.7-031053025842.0-024657179.79-51.03 5.23   0.00 +0.04         A2IV               -0.030-0.056 +.016-007V?     63  7.3   2.7
 893          CD-38  976  18546194007                       025338.6-383533025732.7-381128243.71-61.42 6.41  -0.03               A0Vn               +0.007+0.005      +020
 894          BD+37  675  18552 56067                       025351.7+374401030011.8+380754149.09-18.11 6.11  -0.06 -0.38         B8Vne              +0.006-0.028      -016SB    305                 *
 895          BD-10  585  18557130216                       025356.3-101034025847.4-094635189.01-55.34 6.14  +0.22 +0.13 +0.14   A2m                +0.010-0.011      -018V      28
 896 91Lam CetBD+08  455  186041108891083                   025421.2+083032025942.9+085427168.21-42.41 4.70  -0.12 -0.45 -0.11   B6III              +0.004-0.014      +010      150                 *
 897   The1EriCD-40  771  18622216113 106  W     A  1002    025428.1-404219025815.7-401817247.86-60.74 3.24  +0.14 +0.14 +0.08   A4III              -0.045+0.019 +.035+012SB2    74  1.1   8.3      *
 898   The2EriCD-40  771  18623216114      W     B  1002    025428.9-404218025816.3-401816247.85-60.73 4.35  +0.08 +0.12         A1V                -0.064+0.018 +.035+019      100  1.1   8.3      *
 899  5    EriBD-03  475  186331302282209           1008    025438.2-025147025941.2-022754179.67-50.64 5.56  -0.08 -0.18         B9.5V              -0.002-0.022      +018SB    182:
 900          CD-29 1106  18650168202                       025451.0-291817025906.6-285425224.10-61.85 6.14  +1.04               K1III              +0.018-0.044      +014V
 901   Zet ForCD-25 1191  18692168209                       025511.9-254029025936.1-251627216.57-61.27 5.71  +0.40               F2V                +0.180+0.087 +.029+021       90                 *
 902          BD+10  401  18700 932322213I                  025518.6+102827030044.1+105213166.80-40.74 5.95  +1.59 +1.95         K6                 +0.080-0.039      +018
 903          CD-33 1042  187351940232211                   025530.9-325420025938.3-323026231.73-61.79 6.31   0.00               A0V                +0.010+0.011      +016V?
 904  7    EriBD-03  478  18760130242    I         CV Eri   025548.5-031632030051.0-025243180.47-50.70 6.11  +1.77 +2.07 +0.91E  M1III              +0.020+0.006      +080V                         *
 905 49    AriBD+25  477  18769 756932214           1021    025600.4+260400030154.1+262744156.02-27.90 5.90  +0.14 +0.15 +0.05   A3m                -0.012 0.000      -004V      43
 906          BD+80   97  18778   5003946    2348   1042    025610.9+810502031142.8+812814128.14 20.05 5.95  +0.15 +0.09 +0.07   A7III-IV           -0.040-0.009 +.018-003SB1O   41  5.0  24.2      *
 907  8Rho1EriBD-08  562  18784130243                       025614.9-080325030110.0-073946186.62-53.64 5.75  +1.05 +0.87         K0II               +0.098-0.070      +014V                         *
 908          BD+04  485  18832110915                       025636.2+045626030152.3+052010172.01-44.75 6.25  +1.05 +0.82         K0                 +0.036+0.015      -059
 909   Bet HorCP-64  215  188662487012212                   025654.3-642808025847.8-640417282.85-47.79 4.99  +0.13 +0.15 +0.05   A3-5IIIm:          +0.022+0.001      +024V?     84
 910 93    CetBD+03  420  18883110921                       025708.2+035731030222.5+042110173.08-45.39 5.61  -0.10 -0.41         B7V                +0.014+0.005      +012       80
 911 92Alp CetBD+03  419  18884110920 107I         Alp Cet  025703.0+034151030216.8+040523173.32-45.59 2.53  +1.64 +1.94 +1.16   M1.5IIIa          v-0.009-0.078 +.009-026V                         *
 912          BD-10  594  188851487212216                   025705.3-102121030156.1-095741190.03-54.80 5.83  +1.11              gG6                 +0.046 0.000      +012V?
 913          BD-07  537  18894130251                       025712.4-065306030209.3-062941185.31-52.74 6.19  +0.60 +0.15         G0IV-V             +0.087-0.148      +016SB2O   27                 *
 914   Eps ForCD-28  987  189071682382215                   025718.9-282827030137.7-280530222.48-61.23 5.89  +0.79               G5IV               +0.279-0.437 +.039+038V
 915 23Gam PerBD+52  654  18925 23789 108I   2324           025732.9+530654030447.8+533023142.07-04.34 2.93  +0.70 +0.45 +0.45   G8III+A2V           0.000-0.005 +.016+003SB2O=< 50  8.6  57.0AB   3*
 916          BD+27  468  18928 75709                       025731.7+275239030330.3+281611155.26-26.19 6.36                      F0V                +0.083-0.020      +011      157
 917  9Rho2EriBD-08  568  18953130254    I   2312   1024    025747.7-080443030242.3-074107187.04-53.34 5.32  +0.94 +0.73         K0II-III           +0.047-0.001      +025           4.2   1.9      *
 918          BD+56  767  18970 237912217I                  025801.3+561848030532.4+564221140.59-01.50 4.76  +1.02 +0.86         G9.5III            -0.014+0.067 +.005-045V    < 19:
 919 11Tau3EriCD-24 1387  189781682491085                   025758.9-240059030223.5-233728213.54-60.29 4.09  +0.16 +0.08 +0.09   A4IV               -0.145-0.054 +.059-010V?    144                 *
 920          BD+55  738  18991 23793                       025812.0+554045030539.9+560407140.92-02.04 6.11  +1.02               G9III              -0.003-0.039      -011
 921 25Rho PerBD+38  630  19058 56138 109I         Rho Per  025845.9+382710030510.6+385025149.60-17.01 3.39  +1.65 +1.79 +1.62   M4II               +0.130-0.106 +.011+028                          *
 922          BD+63  390  19065 126082219                   025856.6+634009030719.0+640328137.14  5.00 5.89  -0.03 -0.10         B9V                -0.015+0.006      -002V       0
 923          BD+40  664  19066 38559                       025852.7+401132030520.8+403457148.70-15.49 6.05  +1.01               K0III              -0.053+0.001      -034
 924          BD+15  430  19080 93260                       025906.6+152805030440.7+155122163.84-36.22 6.49R                     K3III              -0.001-0.093      -032
 925 10Rho3EriBD-08  572  19107130269                       025921.7-075931030416.4-073603187.30-52.98 5.26  +0.20 +0.09         A8V                +0.056+0.013 +.026+015V?    152
 926          BD+01  534  191211109452218                   025927.7+012824030438.1+015149176.17-46.77 6.05  +1.04 +0.86         K0III              +0.028-0.003      +001V?
 927 52    AriBD+24  431  19134 75723        2336A          025934.6+245158030526.7+251519157.53-28.47 6.8   -0.02 -0.38         B7V                +0.002-0.008D+.004+009           0.0   0.2AB   5*
 928 52    AriBD+24  431  19135 75723        2336B          025934.6+245158030526.7+251519157.53-28.47 7.0                       B7V                +0.003-0.008D+.004+009?          0.0   0.2AB   5*
 929          CD-47  932  191412161501086                   025930.7-472201030255.9-465830259.36-57.50 5.82  +1.30               K2-3III            +0.029+0.003      +016V
 930          BD+51  681  19268 23825                       030052.8+514941030803.9+521248143.14-05.21 6.31  -0.01 -0.43         B5V                +0.032-0.026      +006
 931          BD+12  436  19270 932762220                   030054.2+124806030623.7+131114166.31-38.06 5.62  +1.08               K3III               0.000-0.063      -015SB
 932          BD+73  168  19275  48402222                   030105.2+740049031156.3+742337132.11 14.08 4.87  +0.02 +0.05 -0.01   A2Vnn              +0.012-0.090 +.032+010V     247
 933          BD+46  692  19279 38587                       030056.3+465521030747.4+471830145.59-09.47 6.41  +0.12 +0.11         A3Vnn              +0.005-0.008      -010
 934   Mu  HorCP-60  236  19319232981 110                   030115.4-600732030336.8-594416277.43-50.34 5.11  +0.34 -0.03         F0IV               -0.073-0.067 +.025+017       97
 935          BD-06  606  193491302842221I W                030136.6-062830030633.5-060519185.87-51.62 5.27  +1.60 +1.78         M3III              +0.006-0.008      +017           7.2  15.9
 936 26Bet PerBD+40  673  19356 38592 111I   2362  Bet Per  030139.5+403414030810.1+405720148.98-14.90 2.12  -0.05 -0.37 -0.03   B8V                +0.004-0.001 +.045+004SBO    65  8.3  81.9AD   6*
 937   Iot PerBD+49  857  19373 38597 112I W                030150.8+491353030904.0+493648144.58-07.39 4.05  +0.59 +0.12 +0.29   G0V                +1.263-0.091 +.084+050V?  =< 10  8.2 146.2      *
 938 53    AriBD+17  493  19374 93284      D       UW Ari   030147.7+172939030725.7+175248162.98-34.21 6.11  -0.12 -0.79         B1.5V              -0.025+0.008      +021SB     18  0.0   0.0      *
 939   The HyiCP-72  219  19400255945 113  W                030202.8-721735030215.4-715409289.93-41.76 5.53  -0.14 -0.51         B3V+A0IV           +0.025+0.014D+.103+012       44  0.0   0.1      *
 940 54    AriBD+18  414  19460 93293    I                  030240.9+182441030821.2+184742162.53-33.35 6.27  +1.58 +1.97 +0.70E  M0III              +0.047-0.016 +.032+043V?                        *
 941 27Kap PerBD+44  631  19476 38609    I   2368   1055    030244.8+442843030929.8+445126147.12-11.43 3.80  +0.98 +0.83 +0.50   K0III              +0.177-0.157 +.033+029SB   < 17  8.4  27.7      *
 942          BD+07  478  19525111002                       030317.3+080508030838.7+082815170.83-41.30 6.28  +1.06 +0.88         G9III              -0.015+0.069      +038
 943          CD-28 1028  19545168321                       030334.5-281252030750.9-274952222.28-59.84 6.19  +0.16               A5V                +0.075-0.020      +012
 944 55    AriBD+28  499  19548 757571088                   030335.6+284143030936.7+290437156.01-24.78 5.72  +0.12 -0.15         B8III              +0.019-0.014      -002V                         *
 945          BD+27  480  19600 75762                       030410.3+272624031008.8+274912156.90-25.75 6.42  +0.01 -0.01         A0V               v+0.005-0.048      -005       62                 *
 946          BD+26  516  19637 75771                       030430.6+263049031027.0+265347157.54-26.48 6.02  +1.28 +1.28         K3III              +0.004+0.070      -016
 947 28Ome PerBD+39  724  19656 56224    I W                030449.8+391355031117.4+393642150.23-15.73 4.63  +1.11 +1.03 +0.57   K1III              -0.023+0.005 +.023+007V    < 19: 6.1 177.4      *
 948          BD+11  445  19698 93320      D                030511.0+112936031038.9+115221168.40-38.41 5.98  -0.06 -0.27         B8V                +0.041-0.026      +001      350                 *
 949          BD+47  779  19735 386382225I W                030531.1+472101031226.4+474333146.06-08.71 6.33  +1.43 +1.64         K5III              +0.074-0.083      -036           3.9 202.1AD   5*
 950          BD+41  631  19736 38635                       030533.1+415954031209.6+422234148.87-13.30 6.15  -0.09 -0.56         B4V                +0.028-0.013      +006V                         *
 951 57Del AriBD+19  477  19787 93328 114I          1066    030554.5+192055031137.8+194336162.60-32.14 4.35  +1.03 +0.87 +0.51   K2III             v+0.152-0.011 +.027+025V?   < 17                 *
 952          BD+12  452  19789 93327      D                030552.3+124007031121.9+130252167.61-37.40 6.12  +1.02 +0.84         K0IIIp             -0.013+0.015      +011                          *
 953          CD-24 1480  19826168354                       030611.0-240706031035.4-234418214.63-58.50 6.38  +0.93               K0IV               +0.058+0.033      +009V
 954 56    AriBD+26  523  19832 75788              SX Ari   030616.6+265248031214.2+271525157.68-25.95 5.79  -0.12 -0.42         B9pSi             v+0.010-0.017      +011V     200                 *
 955          BD-04  540  198361303282224I   2389           030618.3-041122031118.8-034842184.19-49.29 6.05  +1.66 +1.94         M1III              -0.006-0.014      +024           5.9  24.4      *
 956          BD+47  782  19845 38652                       030627.0+474804031324.0+481037145.96-08.25 5.90  +0.97 +0.81         G9III              +0.030-0.022      -010                          *
 957          BD-16  587  19887148821                       030638.0-162410031116.8-160131201.29-55.78 6.26  +1.20 +1.27         K0                 -0.014-0.020      -015V?
 958          BD+06  496  199261110442226I                  030707.9+061704031226.4+063939173.40-41.99 5.56  +1.08 +0.66         K1IIIep+A6V        -0.009-0.004      +005SB  =< 50                 *
 959          CP-69  174  19940248742                       030702.4-693846030749.2-691556287.07-43.42 6.15  +1.02 +0.83         K1III              -0.025-0.016      +011V?
 960          CD-49  884  19948216197                       030712.2-490642031027.4-484403261.22-55.59 6.12  +1.12 +1.03         K1III              +0.034-0.027      +014
 961          BD+77  115  19978  4875 115    2450   1094    030737.2+772203032019.7+774405130.66 17.16 5.45  +0.19 +0.11         A6V                +0.059-0.061 +.014+004V      37  6.8   1.2AB   3*
 962 94    CetBD-01  457  19994130355 116    2406           030740.1-013412031246.4-011146181.50-47.34 5.06  +0.57 +0.12         F8V                +0.196-0.067 +.057+018        7  6.4   3.2      *
 963   Alp ForCD-29 1177  20010168373    I   2402   1074    030749.3-292252031204.3-285913224.75-59.04 3.87  +0.52 +0.02 +0.31   F8V                +0.339+0.640 +.075-021        0  2.7   3.6      *
 964          BD+56  798  20041 23903        2424           030808.0+564605031548.0+570827141.57-00.41 5.79  +0.73 +0.09         A0Ia               -0.004+0.003      -012           8.2  10.8      *
 965          BD+84   59  20084   5501636                   030834.9+843327033220.1+845440126.50 23.20 5.61  +0.92 +0.49         G3IIp:+F0:V        +0.063-0.136 +.026+033SB     80                 *
 966          BD+41  638  20063 38674                       030818.4+420749031456.7+423014149.24-12.92 6.07R +1.07      ?        K2III              +0.070+0.012      +022                          *
 967          BD+65  338  20104 12686        2436           030845.7+651716031731.6+653931137.24  6.93 6.36  +0.09 +0.07         A3V                -0.010+0.010D+.005-006           0.4   0.5AB   3*
 968          CD-44 1025  201212162092228  W                030855.0-444740031225.8-442511253.82-56.96 5.93  +0.44               F7III+A0V          +0.093-0.017D+.025+017SB    151  0.4   0.2AB   3*
 969          BD+50  729  20123 23914    I                  030903.0+503400031612.2+505616144.89-05.66 5.03  +1.15 +0.83         G6Ib-IIa           -0.007-0.016 +.010+002SB   < 17                 *
 970          CD-36 1208  20144194163                       030906.4-361905031301.5-355638238.21-58.76 6.27  -0.08               B8V                +0.003-0.001      +022V
 971          BD+30  512  20149 56285                       030914.9+301104031520.5+303324156.23-22.85 5.52R +0.01 +0.09         A1V s              -0.006+0.007      -003SB?    24
 972 58Zet AriBD+20  527  20150 758101089                   030909.1+204026031454.1+210240162.38-30.62 4.89  -0.01 -0.01 -0.03   A1V                -0.028-0.073 +.020+007V     128
 973          BD+44  648  20162 38685    I                  030916.7+445833031604.7+452045147.87-10.41 6.16  +1.67 +2.00 +1.02E  M2III              +0.054-0.045      -003
 974          CD-30 1238  20176168397               1082    030927.6-301039031338.0-294815226.36-58.76 6.16  +1.05               K1IICNIII          +0.027-0.002      +023V?                        *
 975          BD+32  591  20193 56293        2431           030936.0+322906031547.0+325123154.91-20.90 6.31  +0.37 -0.02         F4V vw             -0.035+0.009      +014       19  6.5  44.1AC   3
 976          BD+34  610  20210 56296        2433  V423 Per 030945.6+341907031601.9+344119153.87-19.35 6.25  +0.28 +0.12         A1m                +0.043-0.034      +025SBO        7.2  29.3      *
 977          CP-57  513  20234233037 118          TW Hor   031001.1-574145031233.2-571918273.30-50.90 5.74  +2.28 +2.83 +1.38   C5II               +0.019+0.009      +014                          *
 978          BD+31  576  20277 56308                       031025.4+314900031635.1+321101155.47-21.36 6.06  +0.99 +0.68         G8IV               -0.018-0.108      +019V
 979          BD+39  743  20283 38700        2443           031039.1+400653031711.4+402900150.73-14.39 6.45                      B9VpSi             +0.015-0.023D+.004-008           0.9   3.8      *
 980          CD-26 1210  20293168406                       031042.2-262815031500.2-260601219.41-58.00 6.25  +0.04               A5V                -0.019-0.015      -002      270
 981          CP-79   91  20313255962      W       BN Hyi   031056.1-792208030732.0-785922295.34-36.05 5.57  +0.30 +0.09         F2II-III           +0.071+0.066      +011SB     47  2.4  15.2      *
 982 30    PerBD+43  674  20315 38704                       031103.4+433927031747.4+440130148.85-11.36 5.47  -0.06 -0.35         B8V                +0.032-0.029      -001SB    269                 *
 983          BD-06  636  20319130388        2440   1090    031104.1-061718031600.9-055507187.86-49.63 6.17  -0.02 -0.24         B9V                +0.007-0.004 +.039+007V          0.2   0.9      *
 984 13Zet EriBD-09  624  203201303871091           1088    031058.5-091128031550.0-084911191.58-51.31 4.80  +0.23 +0.09 +0.13   A5m                -0.002+0.046 +.025-004SBO    66                 *
 985          BD+65  340  20336 12704      W       BK Cam   031111.1+651712031959.4+653908137.46  7.06 4.84  -0.15 -0.77 -0.13   B2.5Ven           v+0.015-0.013 +.009-003SB    351  7.7 120.9      *
 986          BD+38  690  20346 563222230                   031116.7+385457031745.8+391700151.51-15.34 5.96  +0.07 +0.11         A2IV               +0.022-0.020      +027SB?    35
 987 29    PerBD+49  899  20365 23944                       031130.2+495121031837.8+501320145.60-06.06 5.15  -0.06 -0.56         B3V                +0.029-0.026      -004SB    156                 *
 988 14    EriBD-09  627  20395130395                       031144.9-093128031635.7-090916192.19-51.33 6.14  +0.40 -0.04         F1V                -0.005+0.048      -005    =< 10:
 989 31    PerBD+49  902  20418 38714                       031200.4+494347031907.6+500542145.74-06.12 5.03  -0.06 -0.54         B5V                +0.023-0.025      +003V     298                 *
 990          CD-31 1303  20423194197                       031203.9-311148031611.3-304939228.37-58.26 6.65  -0.07               A0V                +0.030+0.004      +020V?
 991          BD+33  619  20468 563402232I                  031228.4+335125031843.8+341322154.63-19.43 4.82  +1.49 +1.57 +0.76   K2IICN1            +0.003-0.006 +.012+002     < 17                 *
 992 95    CetBD-01  469  20559130408    I   2459   1097    031315.3-011740031822.4-005549182.52-46.10 5.38  +1.04 +0.81         G9IV               +0.252-0.052 +.017+020V          2.4   1.2      *
 993          CD-29 1216  20606168449                       031349.9-290943031802.8-284749224.61-57.71 5.91  +0.33               F3V+A8-9           +0.185-0.015      +042
 994 15    EriBD-22 1146  20610168452    I   2463           031356.8-225236031822.1-223041213.26-56.46 4.88  +0.90               G6III              +0.018+0.012 +.028+024V          1.3   0.2      *
 995 59    AriBD+26  540  20618 75863                       031357.3+264235031955.8+270416159.34-25.11 5.90  +0.86 +0.51         G6IV               -0.026-0.075      -000V
 996 96Kap1CetBD+02  518  206301111201093  W        1100    031406.9+030013031921.7+032213178.22-43.08 4.83  +0.68 +0.19 +0.36   G5V               v+0.271+0.092 +.108+020V?   < 17  4.5 268.7AB   3*
 997          BD-19  651  20631148897        2465           031407.1-185522031841.2-183335206.62-55.15 5.71  +0.37               F0IV               +0.136-0.053 +.029+018SB         2.3   7.2      *
 998          CD-48  900  206402162462233                   031411.0-480705031726.6-474506258.81-54.94 5.85  +1.24               K2III              -0.005+0.021      -009
 999          BD+28  516  20644 758712234I                  031417.1+284110032020.4+290254158.14-23.46 4.47  +1.55 +1.79 +0.88   K2II-III           -0.008-0.016 -.008-002V?   < 17
1000 60    AriBD+25  536  20663 75875                       031429.7+251809032025.6+253946160.37-26.17 6.12  +1.24 +1.27        gK3                 +0.017-0.087      +026V?
1001          BD+48  893  20675 38753      W                031446.7+484242032152.6+490415146.67-06.74 5.93  +0.43 -0.02         F6V                +0.191-0.070 +.015+024V?         3.5 205.8      *
1002 32    PerBD+42  750  20677 387502236           1107    031444.4+425806032126.5+431946149.81-11.58 4.95  +0.04 +0.07  0.00   A3V                -0.063-0.006 +.026-007SB?   228
1003 16Tau4EriBD-22  584  20720168460    I   2472  Tau4 Eri 031504.1-220718031931.0-214528212.09-56.00 3.69  +1.62 +1.81 +1.46   M3.5IIIaCa-1      e+0.052+0.033 -.013+042           6.2   5.5AB   6*
1004          CD-24 1578  20729168462    I          1104    031513.2-242906031934.8-240723216.21-56.59 5.61  +1.66 +2.00 +0.96E  M2III              +0.001-0.025      +015V                         *
1005 61Tau1AriBD+20  543  20756 758861094  W                031527.1+204712032113.6+210849163.66-29.64 5.28  -0.07 -0.52         B5IV               +0.025-0.024      +014V      20  2.5   0.2 O   3*
1006   Zet1RetCP-63  217  20766248770    I'W                031535.8-625728031746.2-623431279.12-47.22 5.54  +0.64 +0.06 +0.34   G2.5VFe1Hdel 1     +1.339+0.648 +.097+012           0.3 309.6      *
1007 97Kap2CetBD+03  461  20791111142                       031552.8+031856032106.8+034032178.32-42.54 5.69  +0.97 +0.76         G8.5III            +0.051-0.026      +011V
1008          CD-43 1028  20794216263 119I'                 031556.0-432708031955.7-430411250.78-56.11 4.27  +0.71 +0.22 +0.40   G8V                +3.043+0.725 +.156+087                          *
1009          BD+64  391  20797 127431096I          1121    031559.3+641343032440.5+643510138.46  6.44 5.23  +2.08 +2.06 +1.23   M0II               -0.005+0.001 +.009-021
1010   Zet2RetCP-62  265  20807248774      W                031602.3-625317031812.9-623023279.00-47.23 5.24  +0.60  0.00 +0.34   G1V                +1.336+0.645 +.091+012           0.3 309.6      *
1011          BD+48  899  20809 38768                       031607.8+485120032313.2+491248146.78-06.50 5.29  -0.07 -0.54         B5V                +0.022-0.020      +004SB?   244                 *
1012 62    AriBD+27  500  20825 75892                       031611.5+271454032211.9+273627159.43-24.38 5.52  +1.10 +0.91         G5III              +0.009-0.014      +006
1013          CD-27 1183  20853168475                       031628.9-265804032045.2-263623220.75-56.82 6.39  +0.54               F7V                +0.043+0.024      +025V?
1014          CP-67  217  208882487762235                   031650.9-671724031759.0-665537283.83-44.38 6.05  +0.13 +0.10         A3V                +0.055+0.007      +024
1015 63Tau2AriBD+20  551  20893 75899    I                  031659.8+202304032245.2+204431164.27-29.73 5.09  +1.24 +1.27         K3III              -0.044-0.022 +.008+002V?   < 19:
1016          CD-24 1600  20894168482                       031701.9-235938032124.0-233807215.54-56.07 5.52  +0.88 +0.59 +0.47   G6.5IIbCN-1Hdel -1 -0.021-0.029      +008V                         *
1017 33Alp PerBD+49  917  20902 38787 120I W        1125    031710.8+493019032419.4+495140146.57-05.86 1.79  +0.48 +0.37 +0.33   F5Ib               +0.024-0.025 +.016-002V      18 10.0 167.0      *
1018          CD-26 1257  209801684932237                   031758.3-255644032216.3-253516219.04-56.30 6.35  +0.01               A0V                +0.009-0.002      +025V
1019          BD+33  636  20995 56419        2514           031814.3+331053032429.7+333209156.09-19.30 5.61R -0.03 -0.16         A0V                +0.038-0.028      +002      145  3.5   4.3AB   3*
1020          BD+53  657  21004 24024                       031819.4+533407032548.4+535518144.50-02.36 6.51  +0.29 +0.07         A9III-IV           +0.088-0.030      -004       97
1021          CD-48  930  21011216278                       031819.5-480804032133.3-474637258.40-54.29 6.39  +1.00               K0III              -0.002-0.023      -002
1022 64    AriBD+24  481  21017 759122240I                  031824.0+242212032418.5+244327161.78-26.39 5.50  +1.19              gK4                 +0.015-0.050      +013V?
1023          BD+04  532  21018111161        2509  Var?     031823.2+043134032339.1+045255177.72-41.26 6.38  +0.86 +0.51         G5III              +0.006-0.008      +003SB         3.2   0.9      *
1024          BD-08  643  21019130457        2507           031824.7-080840032317.8-074739191.80-49.20 6.20  +0.70 +0.16         G2V                +0.010-0.222 +.034+043V?  =< 10  6.4   3.9      *
1025   Iot HyiCP-77  134  210242559731095                   031826.6-774512031557.6-772318293.66-37.01 5.52  +0.44  0.00 +0.24C  F4III              +0.109+0.061      +019V       0
1026          BD+40  736  21038 38809                       031832.2+405410032509.4+411526151.58-12.91 6.51                      A0V                -0.005+0.001      -019SB     91
1027 65    AriBD+20  556  21050 75915                       031840.0+202655032426.1+204813164.58-29.43 6.08  -0.04 -0.08         A1V                -0.002-0.006      -009V      44
1028          BD+12  473  21051 934162241                   031839.9+121630032410.1+123746170.87-35.66 6.04  +1.23 +1.09         K0III-IV           +0.028-0.021      +021
1029          BD+48  913  21071 38817                       031851.2+484605032557.4+490715147.20-06.33 6.09  -0.07 -0.49         B7V                +0.026-0.022      -001V?     43                 *
1030  1Omi TauBD+08  511  21120111172 121I          1134    031925.8+084037032448.8+090144174.12-38.16 3.60  +0.89 +0.61 +0.45   G6IIIFe-1          -0.066-0.078 +.016-021SB1O < 17                 *
1031          CD-33 1202  21149194268                       031943.7-330343032344.6-324226231.95-56.71 6.50  +1.37               K3III              +0.023-0.005      +018
1032          BD+71  201  21179  4917    I                  031956.8+713056033019.5+715150134.70 12.72 6.32  +1.80 +1.88         M2III              +0.010+0.005      -023
1033          BD+59  657  21203 12770        2538           032015.3+595423032823.6+601520141.25  3.09 6.49  +0.02 -0.23         B9V+A1             +0.018-0.021D+.006+005           1.0   0.2AB   3*
1034          BD+48  920  21278 38849                       032056.3+484251032803.1+490346147.52-06.19 4.98  -0.09 -0.56 -0.10   B5V                +0.024-0.028      +002SB1O   53                 *
1035          BD+59  660  21291 24054 122    2544   1152    032058.0+593531032904.1+595625141.50  2.88 4.21  +0.41 -0.24 +0.38   B9Ia               -0.003-0.004      -007       29  4.3   2.4      *
1036          BD+18  484  21335 934362242  D                032120.6+182424032703.2+184523166.64-30.61 6.57  +0.15 +0.10         A3Vnp              +0.043-0.006      +031      214  0.0   0.1      *
1037          BD+49  944  21362 38862                       032141.8+493004032852.4+495054147.19-05.46 5.58  -0.04 -0.45         B6Vn               +0.028-0.026      -002SB    355                 *
1038  2Xi  TauBD+09  439  21364111195 123                   032144.9+092303032710.2+094358174.01-37.25 3.74  -0.09 -0.33 -0.09   B9Vn               +0.060-0.039 -.017-002SB2    33                 *
1039          BD+12  477  21379 93439                       032148.1+122310032718.7+124406171.47-35.06 6.28  -0.02 -0.07         A0V s              +0.007-0.019      +015SB     59
1040          BD+58  607  21389 24061    I                  032155.4+583155032954.9+585243142.19  2.06 4.54  +0.56 -0.11 +0.50   A0Ia              e+0.006-0.002      -006SB      6                 *
1041          BD+33  656  21402 56475                       032203.7+332740032820.8+334827156.60-18.61 5.61R +0.02 +0.14         A2V                +0.037-0.057      +006SB
1042   Chi1ForCD-36 1290  21423194289               1142    032203.9-361616032555.8-355515237.74-56.16 6.39  +0.08               A1IV               +0.029-0.007      +025V
1043          BD+58  608  21427 24062        2563   1158    032207.9+590116033011.3+592158141.94  2.49 6.13  +0.08               A2V                +0.036-0.047D+.004+011V          1.4   2.5
1044 34    PerBD+49  945  21428 38872        2558           032212.8+490946032922.1+493032147.45-05.70 4.67  -0.09 -0.57 -0.11   B3V                +0.026-0.027 +.018-002V     202  2.1   0.8      *
1045          CD-27 1228  21430168544                       032209.2-274008032622.5-271903222.37-55.70 5.93  +0.94               G5III              +0.008+0.049      +013V
1046          BD+54  684  21447 24064        2565   1159    032222.4+550621033000.2+552707144.14-00.75 5.09  +0.05 +0.04 -0.01   A1V                -0.046-0.009 +.025+000V     179  4.8  14.8AB   3*
1047          BD+46  760  21455 38874        2560           032227.5+463531032926.2+465616148.93-07.80 6.24  +0.13 -0.25         B7V                +0.018-0.035      +002      150  5.1  27.9AB   6*
1048 66    AriBD+22  495  21467 75945        2552           032235.7+222734032826.6+224815163.95-27.29 6.03  +0.95               K0IV               -0.001-0.104      +049SB         1.4   0.0 A   4*
1049          CD-42 1115  21473216316                       032237.3-415914032611.7-413813247.78-55.30 6.32  +0.06 +0.05         A0V                +0.015+0.027      +012
1050          BD-11  667  21530148972                       032314.8-113758032801.0-111712197.35-50.01 5.73  +1.10 +1.04         K2III              +0.002-0.049      -002V
1051          BD+47  844  21551 38893                       032333.0+474535033037.0+480613148.42-06.73 5.82  -0.04 -0.32         B8V                +0.032-0.029      +005SB    334                 *
1052 35Sig PerBD+47  843  21552 38890 124I          1167    032331.2+473900033034.5+475943148.48-06.83 4.36  +1.35 +1.54 +0.74   K3III              +0.004+0.020 +.004+016     < 17                 *
1053          CP-70  230  21563248797                       032336.7-695834032402.5-693729286.07-42.15 6.15  +0.48 +0.25         G0-5+A3-5V         +0.029+0.015      +020
1054   Chi2ForCD-36 1306  215741943122244                   032340.9-360144032733.4-354053237.27-55.85 5.71  +1.29               K2III              +0.081+0.002      +030
1055          BD+72  178  21610  49362251                   032414.4+730029033512.4+732049134.10 14.14 6.57  +0.03 +0.02         A0Vn               +0.013-0.026      -009      145
1056          BD+48  938  21620 38906               1173    032420.8+485201033129.4+491235147.90-05.74 6.29  +0.09 +0.12         A0Vn               -0.005+0.001      -023      230                 *
1057 NOVA 1901                                     GK Per                                                                                                             +029SBO                       *
1058   Chi3ForCD-36 1310  21635194318      W                032420.0-361158032811.5-355112237.56-55.71 6.50  +0.13 +0.08         A1V                +0.030-0.003      +015           4.0   6.3
1059          BD+48  942  21661 38912                       032439.4+490332033149.1+492403147.84-05.56 6.39  +0.09 -0.14         B9III              +0.012-0.005      -006       25                 *
1060          BD-07  606  21665130520                       032445.2-070847032939.1-064818191.86-47.34 5.99  +1.02 +0.87         G5                 -0.096-0.104      +026
1061  4    TauBD+10  452  21686 93463                       032456.4+105937033024.5+112011173.32-35.55 5.14  -0.03 -0.07         A0Vn               -0.008-0.018 +.002+000V     236
1062          BD-13  662  216881489852246                   032452.5-130109032936.0-124029199.55-50.32 5.59  +0.17 +0.11         A5IV               +0.005+0.004      +015      181
1063          BD+47  847  21699 38917              V396 Per 032504.4+474057033208.6+480125148.68-06.65 5.47  -0.10 -0.57         B8IIIpMn           +0.020-0.025      +001V      59                 *
1064          CP-69  192  217222488042243                   032508.2-694110032536.2-692011285.65-42.24 5.96  +0.42 +0.01         F3-5IV             -0.002+0.063      +014V
1065          BD+27  515  21743 75970        2582A          032518.4+271350033120.8+273419161.21-23.17 5.96R                     A2V                +0.041-0.029D+.009+006      100  0.4  11.4      *
1066  5    TauBD+12  486  21754 93469 125I                  032521.0+123539033052.4+125612172.07-34.32 4.11  +1.12 +1.02 +0.54   K0II-IIIFe-0.5     +0.020-0.002 -.012+015SB1O < 19: 2.0   0.0      *
1067          BD+05  502  217551112302247                   032526.6+055048033045.4+061119178.04-39.07 5.94  +0.96 +0.64         G8III              +0.045-0.016      +011
1068          BD+58  619  21769 24093        2592A          032531.3+582536033332.1+584554142.64  2.24 6.40  +0.14 +0.15         A4III              +0.008-0.057 +.009+007           1.7  20.4AB   3*
1069 36    PerBD+45  778  21770 389242249           1182    032530.4+454305033226.3+460325149.87-08.22 5.31  +0.40 -0.02 +0.22   F4III              -0.054-0.079 +.031-049       29
1070 17    EriBD-05  674  217901305281097                   032539.3-052505033037.1-050431189.96-46.18 4.73  -0.09 -0.27 -0.08   B9V s              +0.014+0.007      +015V?     96
1071          BD+57  730  21794 240982252                   032546.3+573142033341.2+575208143.17  1.52 6.37  +0.51 +0.03         F7V                +0.007+0.040      -072    =< 10
1072          BD+44  734  21803 38927              KP Per   032547.0+443056033239.1+445120150.61-09.18 6.41  +0.03 -0.72         B2IV               +0.011-0.013      +002V                         *
1073          BD+54  693  21819 24099                       032601.9+543808033339.0+545829144.84-00.84 5.98  +0.11               A3V                -0.048-0.005      +014SB2   185
1074          BD+34  674  21856 565311098          Var?     032617.8+350718033240.0+352742156.32-16.75 5.90  -0.06 -0.85         B1V                -0.010+0.001      +025V     150                 *
1075          CD-43 1085  21882216347                       032625.9-425837032955.0-423803249.21-54.41 5.78  +0.22               A5V                -0.075-0.001      +012      198
1076          CD-41 1029  218992163502248           1179    032639.7-414226033013.6-412212247.06-54.61 6.12  +0.48               F7V                -0.009-0.175      +016
1077          BD+59  675  21903 24111        2612           032650.6+594213033500.8+600228142.05  3.39 6.46  +0.39 -0.07         F3V                -0.035+0.013D+.021+021           0.9   1.1AB   3*
1078          BD+39  811  21912 565382254          IW Per   032659.3+393344033335.1+395358153.72-13.09 5.81  +0.11 +0.12         A5m                +0.019-0.046      +004SB1O  101                 *
1079  6    TauBD+08  528  21933111246                       032711.2+090208033236.0+092225175.51-36.55 5.77  -0.08 -0.30         B9IV               +0.033-0.044      +007V      90
1080          BD+75  143  21970  4953                       032721.1+752426033924.8+754423132.81 16.20 6.27  +0.97 +0.65         G9III-IV           -0.015+0.006      +028
1081          CD-47 1071  219812163572250          TU Hor   032724.4-474300033037.0-472231256.87-53.00 5.99  +0.11               A1V                +0.069+0.011      -012V                         *
1082          CD-26 1333  21997168612                       032737.1-255712033153.9-253651219.83-54.18 6.38  +0.12               A3V                +0.082-0.019      +018V?
1083   Kap RetCP-63  234  22001248819 126  W                032737.7-631724032922.7-625615278.36-45.94 4.72  +0.40 -0.04 +0.18   F5IV-V             +0.383+0.372 +.060+012        0  6.0  60.0      *
1084 18Eps EriBD-09  697  22049130564 127I A                032813.1-094748033255.8-092730195.86-48.04 3.73  +0.88 +0.59 +0.47   K2V                -0.974+0.021 +.303+015V?   < 17  3.0   2.0      *
1085          BD+17  575  22072 93494                       032826.4+173018033408.3+174958168.80-30.19 6.17  +0.89 +0.54         K1IVFe-1           +0.089-0.315 +.022+011V?
1086  7    TauBD+23  473  22091 75999        2616   1192    032831.1+240745033426.6+242752163.95-25.14 5.92  +0.13 +0.17         A3V+A3V            +0.008-0.024 +.006+026       31  0.2   0.7AB   3*
1087 37Psi PerBD+47  857  22192 38980    I         Psi Per  032922.8+475136033629.4+481134149.17-06.09 4.23  -0.06 -0.57 -0.01   B5Ve              v+0.024-0.026      -001V     369                 *
1088 19Tau5EriBD-22  628  222031686341099                   032922.2-215805033347.3-213758213.52-52.78 4.27  -0.11 -0.35 -0.12   B8V+B8V            +0.048-0.027      +014SB2O   48                 *
1089          BD+05  511  22211111273                       032930.2+060500033449.2+062504178.70-38.17 6.49  +0.63 +0.16         G0                 -0.007-0.017      -011    =< 10
1090          CD-50 1071  22231233152 128                   032935.8-504304033234.8-502243261.29-51.65 5.68  +1.10 +1.16         K2III              +0.097+0.079      +037V                         *
1091          BD-10  704  22243149047                       032948.9-101211033437.4-095207196.69-47.90 6.25  +0.02               A2V                +0.018-0.004      +011V?
1092          CP-66  195  22252248825                       032950.0-664941033051.6-662923282.23-43.68 5.83  -0.06 -0.33         B8V                +0.010-0.002      +029
1093          CD-31 1450  22262194375      W                032954.4-312502033356.8-310449229.23-54.48 6.20  +0.48               F5V                -0.021+0.062D+.024-033           0.4   0.1      *
1094          BD+56  826  22316 24133                       033027.9+563612033819.7+565558144.23  1.13 6.30  -0.13 -0.35         B9p               v+0.022-0.032      -000V?      0:                *
1095          CD-32 1358  22322194379      W                033033.0-321232033433.5-315229230.61-54.40 6.40  +1.40               K3III              +0.014+0.001      +014           3.8   1.4
1096          CP-61  267  223822488342256                   033057.1-612113033251.6-610101275.72-46.68 6.41  +1.04 +0.91         K0III              +0.080+0.036      +031
1097          BD+42  795  22402 38999                       033114.3+421513033800.2+423459152.76-10.44 6.42  -0.06 -0.37         B8Vn               +0.024-0.021      -001
1098          BD-11  696  22409149057                       033111.7-113141033557.7-111137198.66-48.26 5.57  +0.91              gG7                 +0.030+0.084      +037V
1099          BD+00  616  22468111291        2644  V711 Tau 033139.3+001541033647.3+003516184.91-41.57 5.71  +0.92 +0.46 +0.39E  G9V                -0.026-0.164 +.032-023SB2O       2.7   6.6      *
1100 20    EriBD-17  699  224701490631100          EG Eri   033144.0-174753033617.4-172801207.47-50.85 5.23  -0.13 -0.49         B9.5p              +0.029-0.010 +.014+014       98                 *
1101 10    TauBD-00  572  224841112921101I                  033146.1+000504033652.4+002406185.12-41.66 4.28  +0.58 +0.07 +0.32   F9IV-V             -0.233-0.483 +.054+028V?      0 10.0 396.0      *
1102          BD+14  586  22522 93524      D                033210.9+150607033747.8+152551171.46-31.35 6.39  +0.17 -0.06:        A5IV               +0.015-0.020      +033       67  0.0   0.1
1103          BD+20  602  22615 76045               1213    033311.7+203523033900.1+205457167.43-27.13 6.50  +0.15 +0.19 +0.07   Am,A5-F0           +0.005-0.026      -009       30                 *
1104          CP-66  199  22634248842                       033317.0-660546033424.7-654552281.13-43.84 6.75  +0.17 +0.15         A3V                +0.044+0.006      +011V
1105          BD+62  597  22649 12874 129I         BD Cam   033328.3+625333034209.3+631300140.84  6.44 5.10  +1.63 +1.82 +1.39   S3.5/2             -0.020+0.017 +.002-022SB                        *
1106          CD-40 1008  22663216405 130                   033330.3-403609033705.7-401629244.85-53.51 4.58  +1.04 +0.77 +0.57   K1III              -0.005-0.030 +.011+012
1107          BD+86   51  22701   6233947                   033355.0+861957041001.5+863734125.76 24.91 5.86  +0.37  0.00         F5IV               +0.139-0.084 +.022-004V
1108          BD-07  647  22675130628                       033336.2-074302033829.2-072330194.29-45.81 5.85  +0.98 +0.76        gG5                 -0.009-0.055      -030
1109          CP-78  101  22676255998                       033337.2-784112032958.8-782107293.79-35.80 5.70  +0.93 +0.65         G8III              -0.017-0.028      +010SB
1110          BD+16  484  22695 935362257    2661           033346.3+161241033925.7+163212170.90-30.28 6.16  +1.00 +0.69         K0III              +0.041-0.034      +014           0.0   0.3 O   3*
1111 21    EriBD-06  713  22713130634                       033404.8-055648033901.1-053734192.27-44.75 5.96  +0.92 +0.66         K1V                -0.015-0.205 +.028+040V?
1112          BD+59  699  22764 24169    I   2691           033428.0+593850034242.7+595810142.87  3.90 5.76  +1.71 +1.82 +1.02   K3IIb              -0.005+0.002      -010           2.4  54.7AD   5*
1113          BD+37  811  22780 56628               1225    033437.1+371526034107.9+373448156.37-14.04 5.57  -0.07 -0.41         B7Vne              +0.023-0.030      -001      360                 *
1114   Tau ForCD-28 1225  227891687011102                   033438.0-281611033847.7-275635224.16-53.08 6.01  -0.02               A0V                +0.025+0.023      +039
1115 12    TauBD+02  581  227961113342258                   033438.5+024354033951.1+030325183.01-39.42 5.57  +0.94 +0.67        gG6                 -0.041+0.008      +021V
1116          BD-03  591  22798130639                       033437.8-034258033938.3-032335189.82-43.37 6.23  +1.04 +0.89         G5                 -0.022-0.073      -018
1117          BD-10  717  22799149082                       033438.5-104535033925.4-102614198.28-47.14 6.19  +1.03               G5                 -0.021-0.104      +023
1118 11    TauBD+24  529  22805 760731103  D                033447.8+250022034046.3+251946164.54-23.55 6.11  +0.06 +0.16         A2IV               +0.008-0.015      -005SBO    70  2.0   0.0      *
1119          BD-01  519  22819130645                       033454.4-012644033959.5-010714187.38-41.97 6.12  +1.00 +0.77         K1III-IV           +0.024+0.016      +026
1120          BD-15  634  229051490942259                   033534.0-153300034011.4-151336204.83-49.12 6.33  +0.88               G5                 -0.005-0.014      -019V
1121 22    EriBD-05  715  229201306522260                   033541.2-053201034038.3-051238192.10-44.18 5.53  -0.15 -0.57         B9IIIpSi4200       -0.003-0.001      +016      121                 *
1122 39Del PerBD+47  876  22928 39053 131  W       Del Per  033548.1+472804034255.5+474715150.28-05.77 3.01  -0.13 -0.51 -0.11   B5IIIe            t+0.028-0.034 +.016+004SB    259  7.3  99.1      *
1123 40    PerBD+33  698  22951 56646        2699           033602.1+333839034222.5+335754158.92-16.70 4.97  -0.01 -0.84         B0.5V              -0.005-0.009      +020SB     51  5.0  20.0      *
1124          BD+66  284  23005 128902267                   033632.7+665317034600.9+671206138.68  9.84 5.80  +0.35 +0.07         F0IV               +0.081-0.113 +.004+006
1125          BD-12  689  23010149107      W                033629.0-120730034113.8-114811200.36-47.40 6.49  +0.36 +0.02?        F5II               +0.073+0.013      +029       82  7.7  14.7      *
1126 13    TauBD+19  578  23016 935572263                   033632.8+192248034218.9+194201169.00-27.51 5.69  -0.01 -0.27         B9Vne              +0.001-0.014      -010SB    350                 *
1127          BD+48  984  23049 39063    I   2712           033655.7+481220034406.4+483125149.99-05.06 6.06  +1.55 +1.83         K4III              -0.005-0.010      -012           7.1  18.6AC   3*
1128          BD-20  687  23055149114                       033654.2-195420034122.4-193505211.27-50.46 6.59  +0.09               A3V                -0.002-0.007      -008V?
1129          BD+62  604  23089 12891    I S                033717.3+630146034602.3+632042141.10  6.81 4.80  +0.80 +0.25 +0.53   G0III+A3V          -0.004-0.013 -.000-002SBO    50  1.0   0.1      *
1130          BD+45  804  23139 390712266                   033740.0+454700034440.9+460559151.56-06.92 6.11  +0.28 +0.21         A7IV               -0.004-0.024      +009V      81
1131 38Omi PerBD+31  642  23180 56673        2726  Omi Per  033802.7+315818034419.1+321718160.36-17.74 3.83  +0.05 -0.75  0.00   B1III              +0.007-0.011 +.023+019SB2O   85  2.8   1.0      *
1132 14    TauBD+19  582  23183 93568                       033800.1+192056034347.2+193954169.31-27.30 6.14  +1.01 +0.74         G8III              +0.116-0.054      +078SB                        *
1133          BD+36  742  23193 566752268                   033802.9+360840034431.4+362736157.64-14.49 5.59  +0.06 +0.15 +0.01   A2m                +0.048-0.040      +022       45
1134   Del ForCD-32 1430  23227194467 133                   033816.2-321528034214.9-315618230.88-52.77 5.00  -0.16 -0.60 -0.15   B5IV               +0.009+0.014      +026SB    217
1135 41Nu  PerBD+42  815  23230 39078 134I   2738   1261    033823.8+421546034511.6+423443153.83-09.63 3.77  +0.42 +0.31 +0.26   F5II               -0.014-0.002 +.020-013       44  8.1  31.4
1136 23Del EriBD-10  728  23249130686 135I         Del Eri  033827.4-100606034314.9-094548198.10-46.00 3.54  +0.92 +0.69 +0.50   K0+IV             e-0.092+0.745 +.109-006     < 17                 *
1137          BD+20  621  23258 76121                       033838.9+203646034428.1+205543168.47-26.26 6.10  +0.01 +0.06         A0V                +0.014-0.020      +015SB     87
1138          BD+70  257  23277  5000                       033849.3+703342034913.8+705216136.55 12.86 5.44  +0.09 +0.12 +0.03   A2m                +0.024-0.065 +.017+017SB2O   25                 *
1139          BD-10  729  23281149132                       033847.2-104808034333.8-102908199.05-46.28 5.60  +0.22 +0.10 +0.12   A5m                -0.014-0.017      +016       70
1140 16    TauBD+23  505  23288 76126      D        1262    033851.4+235830034448.2+241722166.04-23.73 5.46  -0.04 -0.33 -0.05   B7IV               +0.011-0.046 +.012+003V?    246  2.0   0.0      *
1141          BD+45  811  23300 39085        2746           033859.3+452205034559.3+454055152.00-07.11 5.66  -0.07 -0.45         B6V                +0.023-0.018      +002           8.4   6.5      *
1142 17    TauBD+23  507  23302 76131 136  D                033856.1+234757034452.5+240648166.18-23.85 3.70  -0.11 -0.40 -0.10   B6III             e+0.019-0.046 +.020+012SB1O  215  2.0   0.0      *
1143          CD-37 1415  233191944752265  W                033907.6-373744034250.1-371849239.76-52.69 4.59  +1.20 +1.31 +0.42E  K2.5III            -0.088-0.077 +.028+010           7.6   5.4
1144 18    TauBD+24  546  23324 76137      D                033911.6+243132034509.7+245021165.70-23.26 5.64  -0.07 -0.36 -0.07   B8V                +0.020-0.047      -002SB2   246  0.0   0.1      *
1145 19    TauBD+24  547  23338 76140      W        1264    033915.2+240913034512.5+242802165.98-23.53 4.30  -0.11 -0.46 -0.09   B6IV               +0.018-0.045      +006SB1   133  1.5   0.0 A   3*
1146 24    EriBD-01  526  23363130698 137                   033925.7-012842034430.5-010947188.32-41.08 5.25  -0.10 -0.39         B7V                +0.002-0.007      +027SB    207
1147          BD+55  824  23383 24215                       033941.8+553639034732.2+555521145.85  1.10 6.10  -0.03 -0.15         B9Vnn              +0.031-0.015      -016V     415
1148   Gam CamBD+70  259  23401  5006 138  W                033947.7+710127035021.5+711956136.31 13.27 4.63  +0.03 +0.07 +0.04   A2IVn              +0.017-0.042 +.012-001V     189  3.9 106.3AC   3
1149 20    TauBD+23  516  23408 76155      D        1279    033952.4+240319034549.6+242204166.17-23.51 3.87  -0.07 -0.40 -0.06   B8III             v+0.020-0.046      +008SB     39  0.0   0.0      *
1150 25    EriBD-00  593  23413130704    I                  033949.6-003641034456.5-001748187.48-40.49 5.55  +1.42 +1.72         K4III              +0.057-0.001      +071V                         *
1151 21    TauBD+24  553  23432 76159               1283    033956.8+241432034554.4+243317166.05-23.36 5.76  -0.04 -0.23 -0.04   B8V                +0.011-0.042      +009SB    194                 *
1152 22    TauBD+24  556  23441 76164      D                034005.3+241257034602.9+243141166.09-23.36 6.43  -0.02 -0.15 -0.02   A0Vn               +0.014-0.044      +000V     236  0.0   0.1      *
1153 29    TauBD+05  539  234661114001104    2750           034021.5+054413034540.4+060300181.28-36.39 5.35  -0.12 -0.61         B3V                +0.019-0.013      +017SB2O  142  7.3  66.2      *
1154          CP-78  105  234742560052261                   034018.1-783845033630.0-781923293.48-35.59 6.29  +1.15 +1.03         K2III              -0.021+0.001      +003
1155          BD+65  369  23475 12916    I         BE Cam   034021.6+651301034931.2+653134140.02  8.75 4.47  +1.88 +2.13 +1.42   M2+IIab           e-0.007-0.013 +.014-003V                         *
1156 23    TauBD+23  522  23480 76172      W       V971 Tau 034023.3+233813034619.6+235654166.57-23.75 4.18  -0.06 -0.42 -0.04   B6IVe             v+0.021-0.045      +006V     282                 *
1157          CD-41 1119  23508216459      W                034034.0-405822034406.2-403937245.19-52.14 6.45  +1.06 +1.01 +0.38E  K1III              +0.008-0.086      +019           3.0   5.1      *
1158          BD+62  612  23523 12917                       034050.0+625923034936.7+631750141.45  7.03 5.85  +0.18               A5Vn               -0.018-0.051      -014SB    222
1159          BD+06  583  23526111407      W                034049.1+062933034609.5+064811180.67-35.81 5.91  +0.99 +0.76         G9III              +0.019-0.071      -026V?
1160          BD+50  825  23552 24231        2769           034055.8+502536034818.3+504412149.16-02.90 6.14  +0.06 -0.32         B8Vne              +0.014-0.006      -024V     250  6.2   7.0      *
1161          BD+56  846  23594 24244      W                034121.0+564837034919.6+570705145.30  2.19 6.46  +0.06 +0.02         A0Vn               +0.018-0.027      -002      157  0.8  58.3
1162 26Pi  EriBD-12  707  23614149158    I         Pi  Eri  034124.8-122455034608.5-120606201.55-46.47 4.42  +1.63 +2.01 +1.05   M2III              +0.048+0.058 -.000+046                          *
1163          BD+33  717  23625 56709        2772   1305    034131.9+331724034752.6+333600160.08-16.25 6.57  +0.08 -0.61         B2.5V              -0.006-0.002D+.005+020SB2O  170  4.0   3.2      *
1164          BD+31  650  23626 56707                       034132.3+315311034748.9+321142161.02-17.33 6.25  +0.47               G0                 -0.032-0.047      -004SB
1165 25Eta TauBD+23  541  23630 76199 139I W                034132.3+234746034729.1+240618166.67-23.45 2.87  -0.09 -0.34 -0.04   B7IIIe             +0.019-0.046 +.008+010V?    215  1.6   0.0 O   5*
1166          BD+68  286  23662 12929                       034152.7+681208035141.8+683027138.26 11.18 6.32  -0.08 -0.23         B9IVp              +0.017-0.016      -004V?
1167          CD-48 1069  23670216469                       034146.7-482219034450.6-480341256.76-50.51 6.49  +1.02               G8-K0III           +0.028-0.081      +004
1168          CP-54  589  23697233252      W                034200.8-543519034433.8-541626265.80-48.46 6.30  +1.04 +0.89         K1III              +0.023+0.065      +033           3.0   5.0
1169          CD-47 1147  23719216470                       034209.4-474016034515.8-472135255.67-50.63 5.73  +0.96               K1III              -0.011-0.020      -003
1170          BD+43  818  23728 39128              V376 Per 034214.0+433916034908.2+435747153.52-08.10 6.02  +0.29 +0.09         A9IV               +0.001+0.021      -015       94                 *
1171   Sig ForCD-29 1413  23738168820      I                034222.7-293856034627.4-292017226.81-51.61 5.90  +0.12               A3V                +0.004+0.012      -013           5.2   5.0      *
1172          BD+22  563  23753 76215      D        1321    034225.5+230650034820.9+232516167.33-23.83 5.45  -0.07 -0.32 -0.06   B8V                +0.023-0.050      +004SB?   272  2.0   0.1      *
1173 27Tau6EriCD-23 1565  23754168827 140                   034232.7-233242034650.9-231459217.34-50.32 4.23  +0.42  0.00 +0.22   F3III              -0.158-0.529 +.059+007        6
1174 30    TauBD+10  486  23793 93611        2778   1327    034247.1+105009034816.3+110836177.17-32.53 5.07  -0.13 -0.60         B3V+F5V            +0.028-0.022D+.011+016V?     52  4.4   9.2      *
1175   Bet RetCP-65  263  23817248877 141  W                034256.6-650717034412.0-644825279.24-43.54 3.85  +1.13 +1.10 +0.56   K2+III             +0.313+0.076 +.042+051SBO        4.21480.       *
1176          BD+44  801  23838 39134              Var?     034306.5+443945035004.5+445804153.02-07.22 5.66  +0.76       +0.29E  G2III+F2:V         -0.028-0.028      +014SB  =< 50                 *
1177 42    PerBD+32  667  23848 56727              V467 Per 034313.2+324705034932.6+330529160.70-16.42 5.11  +0.07 +0.11         A3V                -0.029-0.001 -.003-014SBO    82                 *
1178 27    TauBD+23  557  23850 76228 142    2786   1345    034312.8+234452034909.7+240312167.01-23.23 3.63  -0.09 -0.36 -0.05   B8III              +0.018-0.047 -.026+009SB1O  212  1.5   0.0 A   3*
1179          CD-30 1494  23856194531                       034315.7-301232034720.1-295407227.75-51.50 6.55  +0.49 -0.03         F8V                +0.182-0.069      +004V?
1180 28    TauBD+23  558  23862 76229              BU Tau   034314.1+234952034911.2+240812166.96-23.17 5.09  -0.08 -0.28 -0.07   B8Vpe             v+0.013-0.050 -.012+004SB    329                 *
1181 28Tau7EriCD-24 1877  23878168836              Var      034321.5-241104034739.6-235229218.37-50.30 5.24  +0.07 +0.15         A2                 +0.043+0.048 +.014+029V?      0                 *
1182          BD-00  602  238871114332273                   034331.0-000445034838.9+001340187.65-39.43 5.91  +1.23 +1.31         K3III              +0.063-0.006      +066
1183          BD+23  563  23923 76244                       034347.3+232426034943.5+234242167.37-23.40 6.17  -0.05 -0.19 -0.05   B8V                +0.014-0.049      +009      273                 *
1184   Rho ForCD-30 1497  239401945352272           1337    034353.6-302808034756.0-301004228.19-51.40 5.54  +0.98 +0.66         G6III              +0.028-0.230 +.008+053
1185          BD+21  535  23950 76250                       034402.2+215624034955.0+221440168.50-24.44 6.07  -0.01 -0.32 -0.01   B8III              +0.015-0.037      -003SB?   165                 *
1186          CD-36 1453  23958194537                       034403.8-362449034749.6-360621237.74-51.73 6.21  -0.10               B8V                +0.021+0.008      +011V
1187          BD-21  703  23978168851    I                  034411.5-211233034835.8-205411214.04-49.27 5.81  +1.64               K5III              -0.015-0.025      +003V
1188          BD+25  624  23985 76256        2799   1366    034418.1+251640035018.9+253446166.10-21.93 5.26  +0.21 +0.08         A2V+A5V            +0.038-0.109D+.018+004V      69  0.4   0.7      *
1189          CD-38 1297  24071194550      W     B          034454.1-375540034835.4-373720240.17-51.53 5.40  +0.01 +0.01         A1V                +0.059-0.013 +.024+016        0  0.6   8.0      *
1190          CD-38 1297  24072194551      W     A  1359    034454.4-375533034835.9-373714240.17-51.53 4.73  -0.03 -0.07 -0.04   B9V                +0.079-0.025 +.024+016      218  0.6   8.0      *
1191          BD+33  728  24131 56761      W                034530.0+340327035153.7+342133160.23-15.14 5.77   0.00 -0.80         B1V                +0.002-0.007      +018V     140  7.5  15.1      *
1192          BD+57  752  24141 242761105                   034535.9+574041035343.3+575830145.22  3.22 5.80  +0.18 +0.11 +0.04   A5m                +0.081-0.096      -005       55
1193          BD+21  539  24154 76275      D                034544.0+214349035136.6+220154168.97-24.32 6.83R                     K0III              +0.013-0.019      +063                          *
1194          BD+12  516  24155 93637              V766 Tau 034542.6+124440035115.8+130245176.12-30.70 6.30  -0.07 -0.48         B9pSi              +0.022-0.030      +016       52                 *
1195          CD-36 1467  24160194559 143I'                 034542.7-363011034927.3-361201237.88-51.40 4.17  +0.95 +0.69 +0.50   G9II-III           -0.046-0.051 +.025+002                          *
1196          BD+71  222  24164  50402277                   034542.6+713130035630.2+714918136.36 13.96 6.34  +0.30  0.00         A5m                -0.057-0.008      -002       49
1197          BD+30  582  24167 56762                       034549.6+305206035204.3+311006162.44-17.52 6.25  +0.20 +0.14         A5V                -0.026-0.045      -038      151                 *
1198          BD+48 1015  24240 39175                       034623.6+482108035338.7+483902151.15-03.98 5.76  +1.05 +0.95         K0III              +0.036-0.029 +.020+008V
1199 31    TauBD+06  594  242631114692275  W                034640.2+061404035200.2+063205182.07-34.88 5.67  +0.06 -0.43         B5V                +0.009-0.008      +016           0.1   0.6
1200          CD-36 1476  24305194570                       034653.5-364339035037.6-362531238.24-51.16 6.86  -0.04               B9.5V              +0.004+0.020      +006
1201          BD+16  523  24357 936501106                   034726.8+170146035310.0+171937172.94-27.41 5.97  +0.34  0.00 +0.17   F4V                +0.142-0.030 +.030+035V?     59                 *
1202 30    EriBD-05  769  24388130789        2832           034745.2-053935035241.6-052141194.49-41.74 5.48  -0.10 -0.41         B8V                -0.011-0.011D+.018+015V     191  6.7   8.3      *
1203 44Zet PerBD+31  666  24398 56799 144I   2843   1397    034750.6+313512035407.9+315301162.29-16.69 2.85  +0.12 -0.77 +0.09   B1Ib               +0.006-0.010 +.010+020SB     59  6.5  12.9AB   5*
1204          BD+62  628  24479 129692281                   034835.6+624645035725.5+630420142.28  7.42 5.03  -0.09 -0.16 -0.08   B9.5V             e+0.010+0.005      +005V     113
1205          BD+60  768  24480 12968    I   2867           034836.3+604858035708.3+610632143.53  5.90 5.00  +1.45 +1.26         K3I-II+A3V:        -0.004-0.015 +.015-002V          3.2   1.7      *
1206          BD-18  691  244971492292276                   034843.5-184354035313.0-182604211.06-47.44 6.22  +0.88              gK+F                +0.011 0.000      +020SB2    15                 *
1207          BD+47  912  24504 391952279                   034845.8+473440035558.2+475217151.94-04.33 5.37  -0.08 -0.47         B6V                +0.019-0.018      +005SB    286                 *
1208   Gam HyiCP-74  276  24512256029 146                   034847.0-743244034714.3-741420289.13-37.80 3.24  +1.62 +1.99 +1.09   M2III              +0.047+0.114 +.005+016                          *
1209          BD+30  591  24534 56815    I   2859  X Per    034907.9+304505035523.0+310245163.08-17.14 6.10  +0.29 -0.82         O9.5ep            v-0.013-0.007 -.002+015SBO   150  5.8  22.5      *
1210 43    PerBD+50  860  24546 24314    I W                034910.1+502421035636.5+504143150.20-02.10 5.28  +0.41  0.00         F5IV               +0.091-0.130 +.036+027SB2O < 17  4.8  75.3AB   4*
1211 32    EriBD-03  631  24554130805        2850B          034916.0-031454035417.4-025710192.09-40.10 6.14  +0.09 +0.07         A2V                +0.029+0.008 +.004+018V     180  1.5   6.8AB   3*
1212 32    EriBD-03  631  24555130806    I   2850A          034916.1-031501035417.5-025717192.10-40.10 4.79  +0.94 +0.69 +0.40   G8III              +0.029+0.002 +.004+027SB   < 17  1.5   6.8AB   3*
1213 33Tau8EriCD-24 1945  24587168925              Tau8 Eri 034927.3-245429035342.6-243645220.00-49.14 4.65  -0.13 -0.48 -0.14   B6V                +0.023-0.016      +023SB     48
1214          CD-35 1455  246261946082278                   034950.2-350140035338.9-344356235.57-50.53 5.11  -0.13               B6V                +0.031-0.004      +021V?     35
1215          BD+34  768  24640 56824               1418    035001.8+344719035628.7+350452160.47-13.97 5.49  -0.03 -0.75         B1.5V              +0.005-0.001      +017SB    137                 *
1216          CD-47 1187  247062165401108                   035027.4-471116035333.3-465337254.43-49.37 5.93  +1.24 +1.35 +0.56C  K2III              +0.034-0.043      -001V                         *
1217          BD-12  752  24712149251              DO Eri   035033.9-122329035516.3-120557202.97-44.47 6.00  +0.32 +0.03?        A5pSrCrEu         v-0.055-0.040      +022    =<  6                 *
1218 32    TauBD+22  605  24740 763392283                   035057.4+221124035652.1+222841169.59-23.16 5.63  +0.30 -0.02         F2IV               +0.069-0.116 +.030+032V
1219          CD-40 1128  247442165462280  W        1414    035052.9-403904035423.2-402126244.38-50.23 5.71  +0.60               K0III+A3V          -0.012-0.003      +002SB         0.0   0.1AB   3*
1220 45Eps PerBD+39  895  24760 56840 147    2888  Eps Per  035108.4+394316035751.2+400037157.35-10.09 2.89  -0.18 -0.99 -0.18   B0.5V+A2V          +0.018-0.026 +.009+001SB2   153  5.1   8.8AB   3*
1221 33    TauBD+22  607  24769 76343      D       V817 Tau 035107.9+225307035703.8+231032169.10-22.63 6.06  +0.02 -0.01         B9.5IV             +0.009-0.018      +025SBO    70  2.0   0.0      *
1222          BD+24  599  24802 76350                       035127.5+241020035726.4+242743168.19-21.64 6.16  +1.37               K0                 +0.001-0.012      -013
1223          BD+34  773  24809 56847              V386 Per 035136.6+343132035803.1+344852160.90-13.96 6.53  +0.23?              A8V                +0.013-0.018      -002SB2    80                 *
1224          BD+05  564  24817111516                       035142.3+054506035701.7+060224183.49-34.22 6.09  +0.06 +0.05         A2Vn               +0.036-0.067      +008V?    190
1225          BD-10  793  248321308332284          DL Eri   035150.1-100229035637.9-094503200.29-43.10 6.19  +0.28 +0.14         F1V                +0.042+0.006      +014      120                 *
1226          BD+38  827  24843 56849                       035149.9+383310035829.1+385025158.23-10.89 6.30  +1.08 +0.98        gK1                 +0.038-0.041      +022
1227          CP-53  628  248632333212282  W        1419    035156.3-525855035434.0-524126262.79-47.64 6.46  +0.16 +0.13         A4V                +0.031-0.041      +021           6.1  22.8      *
1228 46Xi  PerBD+35  775  24912 56856 148I         Xi  Per  035228.4+353013035857.9+354728160.37-13.11 4.04  +0.01 -0.92 -0.01   O7.5III(n)((f))   e+0.002 0.000 -.004+070SB    216                 *
1229          BD+38  829  24982 56866        2910           035300.9+383204035940.0+384914158.42-10.75 6.38R +0.10 +0.13         A1Vp:              -0.006 0.000D+.005-002       68  2.7   1.5
1230          BD+80  125  25007   650        2963           035317.0+802525041002.8+804155130.51 20.91 5.10  +0.56 +0.30         G8III+A6V          -0.017-0.002 +.005+004       12  0.8   0.8      *
1231 34Gam EriBD-13  781  25025149283 149I   2904  Gam Eri  035321.8-134734035801.8-133031205.16-44.47 2.95  +1.59 +1.96 +1.00   M0.5IIICa-1Cr-1   e+0.061-0.111 +.010+062           9.5  52.8      *
1232          BD-05  789  25069130860                       035356.6-054504035852.3-052812195.68-40.48 5.83  +1.00 +0.85         G9V                -0.055-0.180      +036V?
1233          BD+09  524  25102 93710                       035411.5+100245035940.7+101951180.07-30.98 6.37  +0.42  0.00 +0.22   F5V                +0.170+0.004      +040V      54                 *
1234          BD+36  805  25152 56899                       035441.2+364224040114.7+365924159.90-11.91 6.41  -0.01 -0.07         A0V                -0.022+0.015      -020V      68                 *
1235          BD-12  766  25165149299    I                  035448.7-125128035930.1-123427204.20-43.75 5.60  +1.48 +1.76         K5III              -0.011-0.032      -005
1236          CP-63  275  25170248912                       035446.0-634511035604.0-632749276.74-43.11 6.14  +1.10 +1.02         K1-2III            +0.072+0.047      -000V
1237          BD+16  544  25175 93716                       035454.0+170052040036.9+171748174.34-26.15 6.32  +0.06 +0.01         A0V                 0.000-0.033      +029V      20
1238          BD+17  666  25202 937212288  W                035502.9+175443040048.8+181138173.65-25.50 5.89  +0.32 +0.04 +0.17   F4V                +0.132-0.028      +025SB2   146  3.3 164.2      *
1239 35Lam TauBD+12  539  25204 93719 150I'        Lam Tau  035508.3+121228040040.8+122925178.37-29.38 3.47  -0.12 -0.62 -0.09   B3V+A4IV           -0.006-0.012 +.002+018SB2O   87                 *
1240 36Tau9EriCD-24 2022  252671690172287          Tau9 Eri 035539.6-241759035955.5-240059219.65-47.62 4.66  -0.13 -0.42 -0.13   B6V+B9.5V          +0.015+0.015 +.015+024SBO    25                 *
1241          BD+68  303  25274 130062291I                  035600.5+682413040603.1+684048139.15 12.18 5.87  +1.54               K2III              +0.002-0.001      -047V
1242          BD+58  690  25291 243842290                   035606.9+585240040427.2+590920145.51  5.03 5.06  +0.50 +0.49 +0.39   F0II               -0.002+0.001      -020        9
1243          BD+09  528  25330111566        2938           035618.7+094303040146.1+095952180.76-30.80 5.67  +0.02 -0.41         B5V                +0.005-0.005      +003           5.2  12.0AB   3*
1244 35    EriBD-01  572  253401308781111                   035627.9-014947040132.0-013259191.88-37.81 5.28  -0.15 -0.55         B5V                +0.025-0.016      +017V     179                 *
1245          CP-57  606  253462333471109                   035634.0-572310035842.9-570609268.50-45.53 6.05  +0.44 -0.01         F2IV               +0.029+0.006      +012
1246          CD-30 1597  253711946892289                   035641.2-304620040040.7-302927229.23-48.71 5.93  +0.04               A2V                +0.052+0.004      +021      110
1247   Del RetCP-61  290  254222489181110                   035709.7-614057035844.7-612401274.02-43.78 4.56  +1.62 +1.96 +1.03   M2IIIab            +0.011-0.018 -.001-001
1248          BD+65  391  25425 13015                       035717.3+651450040639.0+653115141.38  9.91 6.17  +0.14 +0.17 +0.06   A3m                +0.030-0.021      -003V?     23
1249          BD-00  632  25457130893                       035729.2-003225040236.7-001608190.74-36.87 5.38  +0.50  0.00         F5V                +0.149-0.251 +.057+018V      23
1250          CD-51  975  25470233354              XY Dor   035734.3-515046040015.7-513353260.82-47.14 6.51  +1.64 +1.95 +0.96E  M1III              +0.027+0.014      +007                          *
1251 38Nu  TauBD+05  581  25490111579 151                   035750.1+054243040309.4+055921184.68-33.06 3.91  +0.03 +0.07  0.00   A1V                +0.005-0.003 +.030-006       69
1252 36    TauBD+23  609  25555 76425    I   2965           035822.7+234951040421.7+240621169.67-20.79 5.47  +0.86               G0III+A4V          -0.004-0.014 +.014+018V      50  0.0   0.0 O   3*
1253 40    TauBD+05  584  25558111585                       035826.4+050935040344.6+052608185.31-33.28 5.33  -0.08 -0.57         B3V                +0.003-0.010      +011       51                 *
1254          BD+07  592  25570111586                       035831.7+075515040356.6+081150182.78-31.53 5.46  +0.37  0.00 +0.21   F2V                +0.169+0.023 +.030+036V?     40                 *
1255          BD+53  732  25602 24411                       035849.0+534419040636.6+540031149.18  1.40 6.31  +0.99 +0.75         K0III-IV           +0.055-0.099      -008
1256 37    TauBD+21  585  25604 764301112I W                035846.8+214832040441.7+220455171.28-22.15 4.36  +1.07 +0.95 +0.53   K0+III-IIIaFe-0.2  +0.090-0.059 +.013+009     < 17  5.0 137.2      *
1257          BD+02  645  256211115902292                   035856.0+023319040409.9+024937187.90-34.77 5.36  +0.50 +0.04         F6IV               +0.147-0.125 +.020-018       15                 *
1258          BD-20  769  25631169071              Var?     035900.3-202511040324.7-200839214.57-45.74 6.46  -0.18 -0.80         B2.5V              +0.003-0.014      +020                          *
1259          BD-20  770  25661169078                       035912.3-202602040336.8-200930214.62-45.70 7.01  +1.22               K2II               +0.020+0.006      +024
1260          BD+61  676  25638 13031        2984A SZ Cam   035902.7+620333040751.1+621948143.68  7.66 6.99  +0.42 -0.50         B0III              -0.023-0.010      -009SBO   258  0.3  17.9AB  12*
1261 47Lam PerBD+49 1101  25642 244121113                   035907.8+500448040635.0+502105151.63-01.32 4.29  +0.02 -0.04 +0.02   A0IVn              -0.015-0.036      +006V?    196
1262 39    TauBD+21  587  25680 76438      W        1452    035924.9+214421040520.2+220032171.44-22.10 5.90  +0.62 +0.12         G5V                +0.169-0.136 +.063+026V?      3  2.7 170.1AB   3*
1263          BD-16  770  257001493452293                   035935.6-165141040408.7-163520209.94-44.34 6.39  +1.26               K2                 +0.093-0.063      +032V?
1264   Gam RetCP-62  312  25705248925    I'        Gam Ret  035926.9-622618040053.8-620934274.81-43.21 4.51  +1.65 +1.81 +1.45   M4III              +0.002+0.026      -007SB                        *
1265          BD-13  806  25723149351                       035941.9-130401040422.7-124733205.17-42.77 5.61  +1.08 +0.97        gK0                 +0.010+0.013      +032V
1266   Iot RetCP-61  293  25728248927                       035940.8-612132040118.2-610444273.44-43.64 4.97  +1.42 +1.70 +0.73   K4III              +0.067+0.092 +.014+061                          *
1267          BD-20  774  25803169095                       040017.0-203917040441.0-202254215.03-45.53 6.13  +1.16               K0                 +0.028-0.009      -004V?
1268 41    TauBD+27  633  25823 76455              GS Tau   040028.2+271950040636.4+273600167.43-17.96 5.20  -0.13 -0.47         B9pSi             v+0.019-0.053      -002SB1O   20                 *
1269 42Psi TauBD+28  619  25867 764612295                   040049.3+284352040700.5+290005166.46-16.90 5.23  +0.34 -0.04         F1V                -0.092+0.007      +009V      68
1270          BD+59  759  25877 24436                       040059.1+593827040927.6+595429145.48  6.01 6.28  +1.14 +0.92         G8IIa               0.000-0.005      -014
1271          CP-85   44  25887258356      W                040055.2-853332034232.1-851544299.20-30.56 6.41  -0.01 -0.10         B9.5IV             +0.013+0.010D+.008+021V          1.5   2.2
1272          BD-09  811  259101309482294                   040107.4-090736040556.5-085122200.70-40.65 6.26  +0.06 +0.09         A3V                +0.024-0.008      +028
1273 48    PerBD+47  939  25940 39336 152I         MX Per   040123.9+472644040839.7+474245153.65-03.04 4.04  -0.03 -0.55 -0.02   B3Ve               +0.021-0.030 +.020+001V?    217                 *
1274          BD-20  780  25944169111                       040123.5-204700040546.7-203044215.32-45.33 6.34  +0.92               G5                 -0.020+0.012      +014
1275          CD-27 1540  25945169110 153                   040130.1-275531040537.4-273906225.31-47.18 5.59  +0.31 +0.05         F1IV               +0.205+0.094 +.045+063V      98
1276          BD+54  740  25948 24440                       040128.4+543352040922.4+544944148.92  2.27 6.18R                     F5V                +0.081-0.093      -005       15
1277 49    PerBD+37  881  25975 57000                       040138.6+372754040815.3+374340160.43-10.42 6.09  +0.95 +0.75         K1III              -0.112-0.191 +.020-040
1278 50    PerBD+37  882  25998 570062297                   040156.5+374641040836.6+380223160.27-10.15 5.51  +0.46 -0.03         F7V                +0.167-0.202 +.049+025       19                 *
1279          BD+14  657  26015 93775        2999   1466    040202.3+145343040742.0+150946177.38-26.33 6.01  +0.40 +0.02 +0.21   F3V                +0.132-0.024D+.007+036SB1?   25  3.2   3.8      *
1280          BD+16  560  26038 93777    I   3006           040215.7+170421040759.4+172023175.63-24.83 5.89  +1.50               K5IIIb             +0.007-0.015D+.006-031           0.0   0.1 O   3*
1281          BD+71  239  26076  5125                       040234.1+715158041344.9+720735137.15 15.10 6.03  +1.01               K1III              +0.013-0.027      -004
1282          BD+68  310  26101 13061                       040247.6+681421041251.6+683006139.73 12.48 6.32  +1.18               K0                 -0.045+0.026      -024
1283 43Ome1TauBD+19  672  26162 937851115                   040320.3+192042040910.0+193633174.00-23.09 5.50  +1.07               K2III              +0.104-0.033 +.023+025V
1284          BD+13  648  26171 937842298                   040326.4+130800040901.6+132354179.11-27.24 5.95  +0.05 -0.06         B9.5V              +0.016-0.014      -025V      27
1285          CD-43 1304  26262216655                       040407.5-431100040725.1-425501247.93-47.57 6.59  +0.93 +0.61 +0.44C  K0III              -0.005+0.004      -007
1286          BD+33  807  26311 570472300I                  040433.7+331931041059.0+333512163.77-13.03 5.72  +1.44 +1.47 +0.79   K1II-III           -0.004-0.009      +020V
1287 44    TauBD+26  686  26322 764851116          IM Tau   040444.3+261312041049.9+262851168.96-18.08 5.41  +0.34 +0.06         F2IV-V             -0.033-0.036      +019        0                 *
1288          BD-16  796  26326149412                       040445.4-163858040917.8-162309210.32-43.12 5.37  -0.15 -0.53         B4V                +0.002+0.005      +014V      31
1289          BD+83  104  26356   685                       040459.3+833352042813.0+834828128.40 23.40 5.57  -0.13 -0.51         B5V                -0.014+0.010 +.003-007V     320:                *
1290 37    EriBD-07  758  26409131001                       040529.7-071106041022.5-065526199.19-38.76 5.44  +0.94 +0.67         G8III              -0.003-0.013      -010V?
1291          CD-46 1314  264132166672299  W                040527.7-460744040833.9-455153252.21-47.02 6.59  +0.38  0.00 +0.22C  F2V                +0.075+0.021      +008           4.0   1.2AB   3
1292 45    TauBD+05  601  264621116482301  W                040600.8+051546041120.3+053123186.58-31.73 5.72  +0.36 +0.01 +0.19   F4V                +0.149+0.005 +.032+037V?      6  4.1 124.1      *
1293          BD-09  837  26464131005                       040558.7-090450041047.7-084911201.40-39.58 5.70  +1.06 +0.93        gG9                 +0.038+0.009      +030
1294          CP-64  305  26491248945                       040616.1-642944040721.6-641321276.89-41.65 6.38  +0.64 +0.11         G3V                +0.213+0.324 +.038+028
1295          BD+16  569  26546 93810                       040647.1+170113041231.4+171639176.47-24.06 6.09  +1.08 +0.96         K0III              +0.052-0.018      +027V
1296          BD+57  785  26553 244952304          Var?     040649.2+571219041501.8+572737147.69  4.73 6.08  +0.61 +0.50         A4III              -0.002+0.003      -023       50                 *
1297          BD+22  649  26571 76505                       040655.3+220923041251.2+222449172.42-20.55 6.12  +0.19 -0.27         B9IIIp:Si:        v-0.005-0.010      +013SB                        *
1298 38Omi1EriBD-07  764  26574131019 154          Omi1 Eri 040659.0-070554041151.9-065015199.32-38.39 4.04  +0.33 +0.13 +0.16   F2II-III           +0.010+0.082 +.033+011V      96                 *
1299          CD-35 1588  26575194810                       040702.7-353157041045.8-351626236.63-47.05 6.44  +1.07 +0.96 +0.56   K1+IIIaCH-1.5      -0.019-0.035      +028                          *
1300          BD-20  801  265911692062302                   040712.7-203658041136.2-202122215.72-43.99 5.79  +0.18 +0.09 +0.05   A2m                +0.050+0.052      -013SB2O   30                 *
1301          BD+37  897  26605 57081                       040720.5+374241041359.6+375801161.10-09.47 6.45R                     G9III              -0.058 0.000      +029
1302   Del HorCD-42 1400  26612216682                       040728.5-421516041050.6-415937246.50-47.02 4.93  +0.33 +0.07 +0.21   A9V                +0.196+0.063 +.012+037      193                 *
1303 51Mu  PerBD+48 1063  26630 394041117I   3071   1518    040733.1+480919041453.9+482434153.94-01.82 4.14  +0.95 +0.64 +0.54   G0Ib               +0.005-0.018 +.017+008SB1O   14  6.0  83.8AC   5*
1304          BD+82  113  26659   693                       040759.1+830600043000.1+832026128.84 23.15 5.46  +0.87 +0.46         G8III              -0.055+0.105 +.015-038
1305          BD+61  687  26670 13075                       040804.5+613556041653.6+615101144.78  8.04 5.70  -0.14 -0.51         B5Vn               +0.011-0.010      -002V?
1306 52    PerBD+40  912  26673 394092306I                  040804.8+401350041453.3+402901159.45-07.54 4.71  +1.01 +0.65 +0.57   G5Ib+A2V           +0.008-0.026 +.004-002SBO =< 50                 *
1307          BD+09  549  26676 93821                       040805.9+095728041334.8+101244182.69-28.42 6.23  +0.05 -0.33         B8Vn               +0.045-0.026      +003V?    300                 *
1308          BD+08  651  26677111671        3063           040805.6+083809041331.3+085326183.86-29.25 6.51  +0.16 +0.13         A2m                +0.016-0.027      +008           6.3  54.3AC   3*
1309 46    TauBD+07  617  26690111672        3064           040809.9+072738041333.1+074258184.93-29.96 5.29  +0.36 +0.01 +0.19   F2V+F5V            -0.002+0.006 +.030+004SB2?   58  0.2   0.1      *
1310          BD+12  564  26703 93823                       040815.8+122957041349.9+124512180.51-26.76 6.25  +1.16 +1.12 +0.39E  K0                 +0.019-0.028      +048                          *
1311 47    TauBD+08  652  26722111674    I   3072           040830.0+090037041356.4+091549183.60-28.94 4.84  +0.80 +0.50 +0.45   G5III+A7V?         -0.010-0.038 +.029-007           2.2   1.2AB   3*
1312          BD-01  600  267391310412305                   040833.4-012416041338.2-010859193.54-35.07 6.44  -0.13 -0.56         B5IV               +0.007-0.004      +018
1313          BD+57  787  26755 24514                       040851.4+573639041708.2+575138147.62  5.21 5.71  +1.09               K1III              +0.030-0.034      -038
1314          BD+53  750  26764 245122310                   040854.8+532139041643.1+533643150.54  2.13 5.19  +0.05               A2Vn               -0.017+0.002 +.011-003SB    191
1315          BD+09  550  26793111680                       040908.2+094532041436.3+100040183.04-28.34 5.22  -0.10 -0.34         B9Vn               +0.002-0.026      +007      350
1316          CD-44 1450  26820216694                       040920.6-443725041231.6-442206249.91-46.51 6.71  +1.48 +1.81 +0.71C  K3-4III            +0.045+0.003      +041
1317          BD+80  133  26836   6912321I                  040937.5+803508042702.8+804927130.89 21.50 5.43  +1.17              gG6                 +0.003-0.019 +.014-009V
1318 39    EriBD-10  867  26846149478    I   3079           040938.2-103016041423.7-101523203.58-39.45 4.87  +1.17 +1.15 +0.57   K3III              -0.009-0.162 +.015+007     < 19: 3.7   6.4AB   3*
1319 48    TauBD+15  603  26911 93836      W        1537    041005.5+150902041546.3+152402178.59-24.70 6.32  +0.40 +0.02 +0.21   F5V                +0.116-0.029      +036V      53  4.7 151.4AC   3*
1320 49Mu  TauBD+08  657  269121116961118                   041006.1+083831041532.1+085332184.21-28.85 4.29  -0.06 -0.53 -0.06   B3IV               +0.021-0.024      +017SB?    89                 *
1321          BD+05  613  26913111695        3085B V891 Tau 041006.4+055706041525.8+061159186.66-30.50 6.93  +0.70 +0.22         G5IV               -0.106-0.107      -008V   =<  6  0.6  64.7AB   4*
1322          BD+05  614  26923111698        3085A V774 Tau 041009.5+055620041529.2+061112186.68-30.50 6.31  +0.59 +0.05         G0IV               -0.060-0.116      -008        4  0.6  64.7AB   4*
1323          CD-40 1286  269272167052307                   041009.9-403642041335.7-402128244.07-46.58 6.37  +1.46               K3III              +0.016+0.028      +059
1324          BD+49 1150  26961 24531              b Per    041043.2+500259041814.6+501744153.02-00.09 4.61  +0.04 +0.03 +0.05   A2V                +0.047-0.054      +020SB2O   98                 *
1325 40Omi2EriBD-07  780  26965131063    I   3093  DY Eri   041040.2-074830041516.3-073910200.72-38.00 4.43  +0.82 +0.45 +0.45   K1-V               -2.242-3.414 +.209-042V          5.0  83.4AB   5*
1326   Alp HorCD-42 1425  26967216710 155I:                 041041.2-423227041400.1-421740246.88-46.41 3.86  +1.10 +1.00 +0.59   K1III              +0.046-0.209 +.026+022                          *
1327          BD+64  433  27022 130982315                   041115.6+645347042040.3+650826142.72 10.66 5.27  +0.81 +0.47 +0.43   G5IIb              -0.036-0.003      -019       10                 *
1328          BD+41  844  27026 39447                       041112.6+415343041808.2+420828158.72-05.93 6.22  -0.08 -0.29         B9V                +0.029-0.034      +007V
1329 50Ome2TauBD+20  724  27045 76532      D       Var?     041123.9+201957041715.6+203443174.61-21.03 4.94  +0.26 +0.11 +0.12   A3m                -0.044-0.058      +016SB     64  2.0   0.1      *
1330          BD+49 1155  27084 39457                       041142.7+494818041913.3+500256153.30-00.16 5.45  +0.22 +0.12         A7V                +0.061-0.052      -017      120                 *
1331 51    TauBD+21  618  27176 765412313  W                041228.0+212006041823.2+213445173.99-20.17 5.65  +0.28 +0.08 +0.16   F0V                +0.094-0.037      +035SBO    97  2.0   0.1     3*
1332          BD-06  862  27179131092                       041225.8-064307041719.2-062819199.76-37.03 5.94  +1.08 +0.94        gG8                 -0.019+0.002      -002
1333          BD+50  973  27192 245442316           1550    041236.7+504041042011.5+505515152.80  0.57 5.55  -0.01 -0.75         B1.5IV              0.000-0.006      -016SB      0                 *
1334          BD+09  558  27236111729                       041257.5+091434041824.5+092912184.17-27.93 6.54  +0.16 +0.17         A4III              -0.023-0.034      +028
1335          BD+60  800  27245 131132317I          1558    041305.5+602952042147.6+604408146.00  7.67 5.39  +1.49 +1.81         M0III              +0.054-0.117      +029V?
1336   Alp RetCP-62  332  27256248969 156  W                041308.1-624327041425.5-622826274.32-41.65 3.35  +0.91 +0.63 +0.44   G8II-III           +0.045+0.045 +.013+036SB?        8.6  48.5
1337          BD+41  852  27278 39468                       041319.8+413359042014.4+414829159.24-05.89 5.92  +0.94               K0III              +0.011-0.026      +024V
1338   Gam DorCD-51 1066  27290233457 157          Gam Dor  041324.3-514419041601.6-512912259.84-44.79 4.25  +0.30 +0.03 +0.16   F4III              +0.106+0.182 +.061+025V?     69                 *
1339 53    TauBD+20  733  27295 76548                       041332.3+205402041926.1+210832174.52-20.28 5.35  -0.08 -0.25         B9IV               +0.030-0.042      +012SBO     2                 *
1340          CP-62  334  27304248973      W                041329.3-622634041448.7-621131273.94-41.72 5.45  +1.10 +1.03 +0.39E  K0III              +0.010+0.088 +.006+036           5.6  11.0      *
1341 56    TauBD+21  623  27309 76551              V724 Tau 041341.4+213155041936.7+214625174.04-19.83 5.38  -0.14 -0.39         A0pSi              +0.030-0.040      +012       53                 *
1342          BD+56  905  27322 24563                       041343.2+561558042151.8+563024149.03  4.70 5.88R                     A3V                -0.014+0.015      -018V     145
1343 54    PerBD+34  860  27348 57171 158I W                041354.9+341931042024.6+343400164.47-10.95 4.93  +0.94 +0.69         G8III              -0.025-0.005      -027     < 19: 7.9 110.0
1344          BD+31  757  27349 57166    I                  041347.9+314242042010.0+315712166.35-12.81 6.16  +1.71               K5                 +0.002-0.008      -018
1345          BD-21  831  27362169317    I          1548    041354.4-205736041815.9-204256216.87-42.62 6.00  +1.67 +1.82         G8III              -0.013+0.012      +025V                         *
1346 54Gam TauBD+15  612  27371 93868 159I S        1553    041406.0+152311041947.6+153739179.08-23.82 3.65  +0.99 +0.82 +0.47   K0-IIIabCN1        +0.115-0.025 +.028+039SB1 =<  8                 *
1347 41Ups4EriCD-34 1614  27376194902      W       Var      041406.5-340232041753.7-334754234.67-45.47 3.56  -0.12 -0.37 -0.11   B9V                +0.067-0.008      +018SB2O   31  1.0   0.2AB   3*
1348 52Phi TauBD+27  655  27382 76558    I   3137           041412.1+270642042021.3+272103169.83-15.94 4.95  +1.15 +1.08         K1III              -0.025-0.080      +003     < 19: 3.7  50.0      *
1349          BD+09  562  27386 93866    I A                041408.9+095249041937.5+100717183.81-27.30 6.31  +1.43 +1.60         K0                 +0.002-0.040      -027                          *
1350 53    PerBD+46  872  27396 394832319          V469 Per 041419.0+461536042133.2+462956156.08-02.40 4.85  -0.03 -0.52 -0.05   B4IV               +0.025-0.035      +001       19                 *
1351 57    TauBD+13  663  27397 93872      W       V483 Tau 041419.7+134739041957.7+140207180.46-24.80 5.59  +0.28 +0.08 +0.16   F0IV               +0.116-0.021      +042SB1?  109  7.9  34.9      *
1352          BD+59  793  27402 24577        3146   1566    041425.1+592245042257.9+593659146.91  6.99 6.19                      A4V                +0.030-0.038      +012V     150  3.1  32.3ABxC 3*
1353          CD-23 1856  27411169325                       041421.0-231251041837.4-225813219.84-43.21 6.07  +0.30 +0.20 +0.14   Am                 +0.047+0.029      +039V      56                 *
1354          BD+18  624  27429 93874                       041436.3+183010042025.1+184433176.61-21.69 6.12  +0.37 +0.02 +0.20   F3:V               +0.108-0.043      +042SB    132                 *
1355   Eps RetCP-59  324  27442233463      W                041445.4-593233041628.9-591807270.20-42.56 4.44  +1.08 +1.07 +0.53   K2IVa              -0.051-0.164 +.067+029           8.1  13.8
1356 58    TauBD+14  682  27459 93876              V696 Tau 041455.9+145121042036.3+150543179.67-24.01 5.26  +0.22 +0.10 +0.12   F0V                +0.108-0.024      +036SB1    65                 *
1357          CP-61  317  27463248977      W       TT Ret   041450.3-611139041621.1-605655272.30-42.01 6.37  +0.07 +0.01         ApEuCrSr:          +0.041+0.019D+.015+022           0.5   0.4      *
1358          BD+13  665  27483 93878                       041515.1+133731042052.8+135151180.76-24.74 6.17  +0.46 +0.01 +0.25   F6V                +0.113-0.025      +037SB2O   12                 *
1359          CD-34 1626  27490194923      W     A          041516.8-340847041903.0-335418234.86-45.24 6.37  +0.13 +0.11         A3V                +0.017-0.006D+.007+016      165  2.1   6.0      *
1360          BD+05  631  274971117562320                   041521.3+055333042041.2+060751187.61-29.49 5.77  +0.92 +0.68         G8III-IV           -0.012-0.051      +007V
1361          BD+08  672  27505111759                       041522.0+085906042049.0+091332184.81-27.62 6.53  +0.15 +0.11         A4V                +0.054+0.039      +039
1362          BD-06  875  27536131129              EK Eri   041544.1-062902042038.7-061444200.02-36.20 6.27  +0.91 +0.56         G8IV:              +0.103-0.044      +008                          *
1363          BD-07  798  275631311322322          EM Eri   041551.8-074953042042.8-073533201.50-36.83 5.85  -0.13 -0.49         B5III              +0.002-0.001      +011V?                        *
1364          CD-44 1503  27588216749      W                041606.7-443026041916.6-441605249.61-45.32 5.34  +1.08 +0.95 +0.51C  K2III              +0.063-0.046 +.005+024           2.4  70.4      *
1365          CP-53  679  276042334762318  W                041611.9-530611041840.0-525136261.59-44.09 6.09  +0.49               F7IV-V             +0.051+0.073      +021           3.0   0.6
1366          BD-00  687  27611131140    I   3152           041620.3-001957042127.1-000553193.75-32.85 5.86  +1.32 +1.52         K3III              -0.016-0.123 +.008-027           5.0 197.8AC   3*
1367          BD-20  831  27616169354 161                   041617.2-205241042039.0-203823217.01-42.07 5.38  -0.02               A2V                +0.025-0.011 +.007+032V     164
1368 60    TauBD+13  668  27628 93892      W        V775 Tau041625.3+135027042203.5+140438180.78-24.38 5.72  +0.32 +0.10 +0.17   A3m                +0.113-0.026      +041SB1O   25  7.1 109.2AC   3*
1369 59Chi TauBD+25  707  27638 76573        3161           041629.7+252337042234.9+253745171.51-16.74 5.37  -0.05 -0.08         B9V                +0.017-0.023      +017V     350  3.0  19.4     3*
1370          BD+20  744  27639 76571    I   3158           041629.6+203506042222.8+204917175.26-19.97 5.91  +1.66               M0IIIab+A,F        +0.001+0.001 +.015-009           3.0   2.0      *
1371          BD+42  946  27650 39501        3172           041638.0+421138042335.9+422541159.24-05.01 6.23   0.00 -0.15         A1pSi              +0.024-0.032D+.002-012V      40  0.5   0.4
1372   The RetCP-63  316  27657248986      W        1556    041633.0-632953041740.2-631520275.09-41.01 5.87  -0.07 -0.26         B9III-IV           +0.002+0.027      +003V          1.8   3.9
1373 61Del1TauBD+17  712  27697 93897 162I W        1582    041709.9+171829042256.1+173233178.01-22.01 3.76  +0.98 +0.82 +0.47   K0-IIICN0.5        +0.107-0.030 +.021+039SBO =<  8  8.7 109.6AB   3*
1374          CD-26 1642  27710169368        3159AB 1574    041722.2-255749042131.3-254342223.73-43.30 6.01  +0.35  0.00         F0IV-V             +0.048-0.050 +.057+018V      46  0.3   0.6AB   4*
1375          BD+20  751  27742 76585      D                041738.7+204456042332.4+205856175.32-19.66 5.99  +0.03 -0.26         B8IV-V             +0.016-0.028      +018V     175  0.0   0.2
1376 63    TauBD+16  586  27749 93900              Var?     041740.7+163238042325.0+164638178.73-22.41 5.64  +0.30 +0.13 +0.16   A1m                +0.104-0.028      +035SB1O   10  2.0   0.0      *
1377 55    PerBD+33  853  27777 57212                       041759.7+335357042429.2+340750165.38-10.65 5.73  -0.06 -0.34         B8V                +0.019-0.044      +005      250
1378 62    TauBD+23  684  27778 76591        3179A          041757.8+240405042359.8+241804172.76-17.39 6.36  +0.17 -0.35         B3V                +0.006-0.010      +016V          1.9  29.1AB   4*
1379 56    PerBD+33  854  27786 57216        3188   1592    041808.3+334346042437.4+335735165.53-10.75 5.76  +0.40 -0.12         F4V                +0.043-0.072 +.005-032           3.6   4.9      *
1380 64Del2TauBD+17  714  27819 93907      W                041819.7+171245042405.8+172638178.29-21.86 4.80  +0.15 +0.12 +0.07   A7V                +0.110-0.038 +.017+039SB     59  8.5 143.0      *
1381 66    TauBD+09  570  27820111791        3182           041824.6+091342042351.9+092739185.11-26.88 5.12  +0.07 +0.11 +0.05   A3V                -0.015-0.008 +.001-004V      83  0.1   0.3      *
1382          BD+57  800  27855 246152324    3203           041842.4+572124042700.9+573507148.74  5.96 6.32  +0.04 +0.02         A0III              +0.009-0.012      -001           5.9  20.6      *
1383 42Xi  EriBD-04  818  278611311761120           1590    041842.0-035836042340.8-034444197.82-34.30 5.17  +0.08 +0.08 +0.03   A2V                -0.050-0.057 +.013-011SB?   173                 *
1384          CD-25 1862  278811693912323I          1588    041854.8-250730042305.7-245332222.75-42.75 5.83  +1.51               K5                 +0.018-0.015      +029V                         *
1385          BD+18  633  27901 93918                       041907.3+184844042457.1+190230177.12-20.68 5.98  +0.37 +0.04 +0.22   F4V                +0.105-0.044      +037SB?   125                 *
1386          CD-35 1687  27941194971      W                041927.5-354639042307.7-353242237.28-44.56 6.39  +1.24               K1III              -0.006 0.000      +017           7.0  18.7
1387 65Kap1TauBD+21  642  27934 76601        3201   1593    041924.4+220354042522.1+221738174.56-18.48 4.22  +0.13 +0.13 +0.05   A7IV-V             +0.096-0.048 +.025+040SB?    81  0.0   0.1 O   6*
1388 67Kap2TauBD+21  643  27946 76602        3201   1594    041927.4+215817042525.0+221159174.65-18.54 5.28  +0.25 +0.10 +0.14   A7V                +0.109-0.054      +032V     153  0.0   0.1 O   6*
1389 68Del3TauBD+17  719  27962 93923        3206  V776 Tau 041942.1+174158042529.4+175541178.12-21.30 4.29  +0.05 +0.08 +0.01   A2IV               +0.108-0.028D+.019+035SB     18  3.7   1.5AB   3*
1390          BD+31  776  27971 572292325                   041944.2+311248042606.3+312620167.62-12.24 5.28  +0.97 +0.80         K1III              +0.076-0.110      +028V    < 19:
1391 70    TauBD+15  621  27991 93925      W                041954.7+154245042537.3+155627179.79-22.54 6.46  +0.49 +0.02 +0.25   F7V                +0.107-0.028 +.038+038SB?    15  0.0   0.1AP   3*
1392 69Ups TauBD+22  696  28024 766082326  W       Ups Tau  042019.3+223513042618.5+224849174.30-17.98 4.28  +0.26 +0.14 +0.14   A8Vn               +0.107-0.046 +.036+035SB1   196  1.9   0.0     3*
1393 43    EriCD-34 1664  280281949841121                   042016.8-341456042402.2-340101235.17-44.23 3.96  +1.49 +1.80 +0.83   K4III              +0.069+0.051 -.008+024                          *
1394 71    TauBD+15  625  28052 93932      W       V777 Tau 042038.8+152329042620.8+153706180.18-22.60 4.49  +0.25 +0.14 +0.15   F0V                +0.112-0.023 +.007+038SBO   192  2.0   0.1     3*
1395   Eta RetCP-63  324  28093249009 163                   042048.3-633725042153.3-632311275.01-40.52 5.24  +0.96 +0.69         G8III              +0.089+0.173 +.016+045                          *
1396 73Pi  TauBD+14  697  28100 93935    I                  042057.3+142916042636.4+144249180.99-23.12 4.69  +0.98 +0.72 +0.51   G7IIIaFe-1         -0.002-0.033 +.014+032     < 17
1397          BD+08  687  281141118172327                   042055.5+082147042621.1+083525186.30-26.90 6.06  +0.02 -0.42         B6IV               -0.001-0.006      +014       25
1398          CD-35 1704  28143194996      W                042114.1-345859042456.4-344528236.23-44.12 6.55  +0.44               F2III              -0.013-0.110      +007V          3.4  39.5AB   3*
1399 72    TauBD+22  699  28149 76613      D                042118.5+224615042717.5+225947174.31-17.69 5.53  -0.10 -0.48         B7V                +0.006-0.017      +005V     230                 *
1400          BD+01  753  28191111827                       042148.6+015119042700.7+020446192.50-30.50 6.23  +1.09 +1.03         K1III              +0.065-0.047      +021
1401          BD+72  227  28204  52382333    3267           042154.6+721847043330.7+723143137.89 16.48 5.94  +0.30 +0.18         A8m                +0.027-0.089      +010SB1O   31  6.6  26.0      *
1402          BD+10  577  28217 93943        3228           042157.4+105916042728.8+111245184.16-25.11 5.88  +0.05 -0.33         B8IV               -0.003-0.003D+.003+004V      70  1.8   0.4      *
1403          BD+21  647  28226 76618      W                042204.5+212349042800.8+213712175.52-18.46 5.72  +0.27 +0.10 +0.14   Am                 +0.095-0.038      +036SB2    88  0.8   0.0 A   3*
1404          CD-44 1546  28246216790                       042210.4-442323042519.1-440939249.35-44.24 6.39  +0.44               F6V                +0.035+0.067      +013
1405          CP-57  659  28255233506      W     A          042215.1-571751042412.2-570417266.91-42.27 6.29  +0.66 +0.19         G4V                -0.098-0.077 +.042-004           0.4   5.8      *
1406          BD+30  665  28271 57249        3243A          042233.0+300822042851.9+302141168.85-12.53 6.40  +0.52 +0.08         F7V                +0.010-0.023D+.016-037SB     40  1.8  14.7AB   4*
1407 75    TauBD+16  605  28292 93950    I D        1609    042243.3+160810042826.4+162135179.90-21.75 4.97  +1.13 +1.10         K2III             v+0.006+0.028      +018V    < 19: 2.5   0.0      *
1408 76    TauBD+14  702  28294 93948                       042243.3+143107042823.4+144427181.26-22.77 5.90  +0.32 +0.06 +0.19   F0IV               +0.108-0.024      +044V     102                 *
1409 74Eps TauBD+18  640  28305 93954 164I W                042246.5+185731042837.0+191049177.60-19.92 3.53  +1.01 +0.88 +0.50   G9.5IIICN0.5       +0.107-0.038 +.020+039V?  =<  8  7.0 181.6      *
1410          CD-24 2343  28312169455        3230           042244.6-241820042657.0-240453222.00-41.69 6.11  +0.14               A2                 -0.003-0.015D+.019-004           0.2   0.4      *
1411 77The1TauBD+15  631  28307 93955    I W                042251.6+154425042834.5+155744180.26-21.97 3.84  +0.95 +0.73 +0.47   K0IIIbFe-0.5       +0.103-0.027 +.038+040SB1 =<  8  0.4 337.4AB   4*
1412 78The2TauBD+15  632  28319 93957      W       The2 Tau 042257.0+153857042839.7+155215180.35-22.01 3.40  +0.18 +0.13 +0.09   A7III              +0.103-0.025 +.029+040SB1O   78  0.4 337.4AB   4*
1413          BD+01  755  28322111840                       042252.3+013807042803.6+015131192.88-30.40 6.15  +1.02 +0.81         G9III              +0.027+0.004      +030
1414 79    TauBD+12  598  28355 939602330                   042313.9+124934042850.2+130251182.78-23.73 5.03  +0.23 +0.12 +0.08   A7V                +0.105-0.016      +033V?    104                 *
1415          BD+01  757  283751118451123                   042321.9+010933042832.1+012251193.42-30.57 5.55  -0.10 -0.55         B3V                +0.019-0.021      +018SB     26                 *
1416          CP-61  335  28413249016                       042341.9-612752042505.3-611418272.18-40.92 5.94  +1.54 +1.85         K4-5III            -0.024+0.014      -019
1417  1    CamBD+53  779  28446 24672 165    3274A Var?     042406.4+534137043201.8+535439151.91  3.95 5.77  +0.18 -0.73         B0III               0.000-0.003D+.006-007SB    275  1.1  10.3AB   3*
1418          CD-47 1383  284542168092329                   042409.6-470946042706.0-465651253.17-43.71 6.10  +0.46               F5-6V              +0.068-0.275 +.032+016                          *
1419          BD+32  806  28459 57269      W                042413.1+321422043038.3+322729167.52-10.85 6.21  -0.04 -0.13         B9.5Vn             -0.004-0.011      +023V     350                 *
1420          BD+10  583  28475 93968                       042412.9+101807042942.9+103118185.13-25.09 6.79  +0.09 -0.37         B5V                +0.004+0.001      +015V
1421          BD-19  931  28479149668                       042415.0-194036042839.0-192730216.34-39.91 5.96  +1.22               K1III              +0.028-0.090      +026SB
1422 80    TauBD+15  636  28485 93970        3264           042426.4+152511043008.6+153817180.78-21.88 5.58  +0.32 +0.10 +0.19   F0V                +0.102-0.024 +.031+030SB1   134  2.2   1.7      *
1423          BD-13  893  284971496742332          DU Eri   042428.2-131607042906.9-130254208.78-37.40 5.60  -0.23 -0.89 -0.10   B1Vne             v+0.002 0.000      +022      340                 *
1424          BD+39 1013  28503 57278        3273A          042433.9+394735043124.0+400037162.01-05.62 6.26  +0.04 -0.36         B8V                -0.006-0.014      -012V          0.5   9.3      *
1425          BD+09  590  28505 93971                       042433.1+100243043002.4+101545185.41-25.18 6.48  +1.03 +0.82         G8III              -0.007-0.067      -063
1426   Del MenCP-80  116  28525258372 166           1591    042444.0-802654041759.1-801250293.80-32.90 5.69  +0.84 +0.53         K2-3III+A          +0.018+0.061      -020SB
1427          BD+15  637  28527 93975      W        1627    042450.1+155835043033.7+161138180.38-21.45 4.78  +0.17 +0.13 +0.07   A6IV               +0.107-0.027 +.053+038SB     71  0.0   0.0 O   3*
1428 81    TauBD+15  639  28546 93978      W                042456.5+152828043038.9+154131180.82-21.75 5.48  +0.26 +0.10 +0.12   A5m                +0.103-0.023      +039V?     21  3.9 162.0      *
1429          CD-42 1510  28552216821               1615    042452.9-421051042809.4-415736246.27-43.80 6.44  +1.64 +1.95         M1III              -0.011-0.006      +097
1430 83    TauBD+13  690  28556 93979      W                042459.5+133025043037.3+134328182.49-22.97 5.40  +0.26 +0.10 +0.14   F0V                +0.106-0.023      +039V      95  5.8 111.9      *
1431          BD-13  896  28625149682                       042532.5-134837043009.7-133532209.54-37.39 6.24  +1.00 +0.73         G5                 -0.025+0.009      +014V?
1432 85    TauBD+15  645  28677 93993               1640    042608.9+153813043151.8+155106180.88-21.42 6.02  +0.34 +0.04 +0.20   F4V                +0.101-0.027      +036SB1   109                 *
1433          CD-46 1427  28700216832                       042622.4-464403042920.0-463055252.54-43.37 6.16  +1.06               K1III              +0.046+0.030      +011
1434 57    PerBD+42  990  28704 396041124  W                042622.6+425102043324.9+430350160.02-03.27 6.09  +0.38 +0.01         F0V                +0.004+0.004      -023V          0.7 116.2AB   4
1435          CP-62  357  28732249029      W                042636.1-624427042745.9-623115273.64-40.20 5.75  +1.01 +0.82         K0III              -0.019+0.014      +001           6.7  10.2
1436          BD+05  674  28736111879                       042645.2+051144043204.8+052436190.14-27.59 6.39  +0.42  0.00 +0.25   F5V                +0.114+0.009      +040V      35                 *
1437 45    EriBD-00  713  28749131270    I                  042645.6-001530043152.7-000238195.34-30.63 4.91  +1.32 +1.44 +0.67   K3II-III           +0.003-0.009 -.009+017     < 17
1438          BD-13  904  28763149702        3284           042648.8-135129043125.9-133842209.75-37.13 6.21  +0.12 +0.09         A3V                -0.011-0.063      -015           3.2  29.1
1439          CD-35 1768  287761950852334  W                042701.7-355213043040.3-353913237.62-43.04 5.96  +1.00               K0II                0.000+0.031      -008           8.3  36.6
1440          BD+63  515  28780 131962336                   042701.5+640309043624.2+641542144.54 11.30 5.94  -0.03 -0.02         A1V                -0.026-0.009      -016V   =< 41
1441          BD-03  809  28843131279      W       DZ Eri   042737.2-032519043237.4-031233198.62-32.10 5.81  -0.14 -0.54         B9III             v-0.005-0.006      +018V?         3.4 124.5      *
1442          BD+17  750  28867 94002        3297           042745.5+174820043332.9+180100179.34-19.76 6.25  +0.07 -0.10         B9IVn              +0.011-0.024D+.002+015V          0.1   3.1      *
1443   Del CaeCD-45 1567  28873216850 167          Var?     042746.3-451006043050.1-445714250.37-43.22 5.07  -0.19 -0.78         B2IV-V             +0.007-0.008      +014       36
1444 86Rho TauBD+14  720  28910 940071125          Rho Tau  042810.3+143803043350.9+145040182.05-21.67 4.65  +0.25 +0.08 +0.12   A8V                +0.103-0.027 +.026+040SB2O  117                 *
1445          BD+28  666  28929 76654        3304           042822.4+284507043438.0+285740170.77-12.53 5.88  -0.06 -0.36         B9pHg              +0.002-0.025      +013V      76  4.9  25.3AB   3*
1446          BD+09  600  28930111890                       042820.6+091213043348.1+092447186.77-24.93 6.01  +1.06 +0.82         G8III              -0.016-0.043      -026
1447          BD-11  900  28970149725    I                  042838.5-105949043322.0-104709206.78-35.50 6.06  +1.38 +1.45         K0                 -0.005+0.015      -003V?
1448          BD+05  679  289781118962335                   042849.1+052132043408.3+053407190.32-27.08 5.68  +0.05 +0.11 +0.04   A2V s              -0.010-0.005      -008       36                 *
1449 46    EriBD-07  838  29009131309        3305  EH Eri   042902.2-065655043354.8-064420202.47-33.55 5.72  -0.13 -0.46         B9pSi               0.000-0.007      +002V          4.8   1.2      *
1450          BD-07  841  29063131317    I                  042922.1-070245043414.2-065016202.62-33.52 6.09  +1.38 +1.64         K2III              -0.018-0.037      -004
1451 47    EriBD-08  887  290641313152337I         DV Eri   042922.6-082626043411.6-081353204.10-34.18 5.11  +1.70 +2.02 +0.97E  M3III              -0.018-0.001 +.006-012V                         *
1452          BD-09  930  29065131316    I                  042924.4-091034043411.7-085813204.90-34.51 5.26  +1.47 +1.73         K4III              -0.037-0.111 +.010-027V    < 19:
1453 50Ups1EriCD-30 1883  29085169570    I          1650    042935.1-295807043330.6-294600229.85-41.59 4.51  +0.98 +0.72 +0.54   K0IIICN-1          -0.102-0.275 +.028+021
1454 58    PerBD+40 1000  29094 396392338I A                042945.5+410334043641.4+411553161.76-04.03 4.25  +1.22 +0.82 +0.69   K4III+A3V          -0.005-0.016 +.016+005SBO =< 50        0.1      *
1455          BD+19  742  29104 94022        3316           042950.7+194031043542.7+195254178.14-18.19 6.36                      G5II-III+A-F       -0.001-0.017      -002SB         0.5   0.3      *
1456   Nu  MenCP-81  115  29116258378                       042949.1-814825042057.9-813448295.07-32.05 5.79  +0.35 +0.05         F0-2III            +0.003+0.127      +014                          *
1457 87Alp TauBD+16  629  29139 94027 168I   3321  Alp Tau  043010.9+161830043555.2+163033180.97-20.25 0.85  +1.54 +1.90 +0.94   K5+III            v+0.063-0.190 +.048+054SB   < 17  9.8 121.7AC   6*
1458 88    TauBD+09  607  29140 94026        3317   1658    043009.5+095720043539.3+100939186.40-24.13 4.25  +0.18 +0.11 +0.10   A5m                +0.055-0.046 +.036+029SB2O   35  3.6  69.7      *
1459          BD+23  715  29169 76670               1663    043027.6+230813043629.2+232027175.47-15.85 6.02  +0.38 +0.02 +0.21   F5IV               +0.112-0.055      +043V      80                 *
1460          BD-10  959  29173131335        3318A          043028.1-095633043514.0-094412205.87-34.63 6.37  +0.11 +0.10         A1m                +0.021-0.023      +026V      15  1.0  12.8      *
1461          BD-20  880  29184169584                       043037.8-200747043500.5-195514217.55-38.65 6.13  +1.17               K3III              +0.081+0.081      +074V?
1462          BD-03  830  29227131344        3328           043102.4-034900043601.6-033643199.53-31.57 6.33  -0.10 -0.43         B7III              -0.007-0.013      +020           4.7  17.9AB   4*
1463 48Nu  EriBD-03  834  29248131346 169  W       Nu  Eri  043119.3-033325043619.1-032109199.31-31.38 3.93  -0.21 -0.89 -0.19   B2III              +0.002-0.005      +015SB     25  9.2  50.9      *
1464 52Ups2EriCD-30 1901  29291195148 170I                  043139.7-304601043533.0-303344231.01-41.31 3.82  +0.98 +0.72 +0.49   G8IIIa             -0.045-0.012 -.014-004                          *
1465   Alp DorCP-55  663  29305233564 171  W       Alp Dor  043150.0-551506043359.8-550242263.83-41.42 3.27  -0.10 -0.35 -0.09   A0IIISi            +0.051-0.003 +.018+026       82  0.5   0.2     3*
1466  2    CamBD+53  794  29316 24744        3358           043202.4+531634043958.1+532823153.03  4.53 5.35  +0.32 +0.07         A8V                +0.046-0.092 +.019+020V     112  1.6   0.3AB   4*
1467  3    CamBD+52  865  29317 24743    I   3359   1681    043201.9+525249043954.7+530447153.32  4.26 5.05  +1.07 +0.89         K0III              -0.002-0.013 -.008-041SB1O < 17  7.0   3.8      *
1468          BD+76  174  29329  5289                       043208.1+762520044600.3+763640135.05 19.65 6.49  +0.51               F7V                +0.064-0.136 +.015-006       64
1469          BD+00  798  293351119282339           1671    043204.4+004745043713.7+005954195.13-28.94 5.31  -0.12 -0.45         B7V                -0.004-0.004      +024V     136                 *
1470          BD+26  731  29364 76682        3353   1676    043218.8+264427043829.6+265624172.92-13.20 6.47R                     F2V:               +0.047-0.058D+.021+004SB         0.1   4.0      *
1471          BD+20  785  29365 766802341          HU Tau   043221.7+202902043815.8+204105177.88-17.22 5.92  -0.05 -0.35         B8V                -0.015-0.007      -002SBO    70  1.5   0.0      *
1472 89    TauBD+15  661  29375 94043      W                043225.8+154958043809.4+160200181.72-20.12 5.79  +0.31 +0.06 +0.16   F0V                +0.093-0.023      +038V     115  5.9 140.5      *
1473 90    TauBD+12  618  29388 940442342  W                043234.0+121837043809.5+123039184.74-22.24 4.27  +0.12 +0.13 +0.05   A6V                +0.103-0.013 +.024+045SB1    79  5.7 119.8AC   3*
1474 51    EriBD-02  963  29391131358        3350           043234.1-024024043736.1-022824198.61-30.66 5.23  +0.28 +0.04         F0V                +0.042-0.061 +.034+021       95  5.9  80.0AC   3*
1475          CP-63  342  29399249054      W        1659    043231.2-630146043334.0-624925273.74-39.47 5.79  +1.04 +0.90         K1III              -0.115-0.023      +031           3.6  31.9
1476          CD-30 1911  29435195163                       043257.9-305508043650.9-304300231.28-41.06 6.30  -0.10               B9IV-V             -0.011+0.013      +015
1477          BD+24  674  29459 766892343                   043317.1+250111043923.1+251306174.41-14.15 6.22  +0.16 +0.11         A5Vn               +0.017-0.015      +021V     160
1478 91Sig1TauBD+15  665  29479 94051      W                043326.5+153611043909.2+154759182.07-20.07 5.07  +0.15 +0.20         A4m                +0.039-0.072 +.021+019SBO    60  0.4 431.2AB   3*
1479 92Sig2TauBD+15  666  29488 940542345  W                043333.2+154312043916.5+155505181.99-19.97 4.69  +0.15 +0.13 +0.07   A5Vn               +0.083-0.018 +.020+036SB2   117  0.4 431.2AB   3*
1480          BD+07  681  294991119542344  W                043341.1+074020043906.2+075215188.99-24.76 5.39  +0.26 +0.13 +0.12   A5m                +0.094-0.003      +036V      55  3.5 297.8AC   3*
1481 53    EriBD-14  933  29503149781 172I W                043336.0-142958043810.8-141814211.32-35.89 3.87  +1.09 +1.01 +0.56   K2IIIb             -0.075-0.156 +.044+042SB1  < 17  3.4   0.8      *
1482          BD+48 1128  29526 396882347                   043356.4+480623044124.1+481803157.06  1.28 5.67  -0.02 -0.02         A0V                +0.042-0.042      +023V      65
1483          BD-12  955  29573149789                       043413.7-121916043853.6-120723208.96-34.85 5.01  +0.07 +0.09 +0.05   A2IV               -0.053-0.011 +.028+007V      43
1484 93    TauBD+11  639  29589 94063                       043429.2+120005044003.4+121152185.31-22.05 5.46  -0.12 -0.49         B8IV               -0.001-0.008      +023V      73                 *
1485          CP-83   91  295982583791658  W                043429.0-830656042250.9-825357296.31-31.26 6.76  +0.20 +0.10         A7IV-V             -0.018+0.003      -005           5.7  47.7      *
1486          BD+59  826  29606 24777        3391           043438.1+591946044318.1+593115148.73  8.83 6.50R                     A7IV               +0.033-0.049D+.005+010           0.0   0.1      *
1487          BD-14  936  29613149797                       043443.7-143311043919.7-142133211.52-35.66 5.45  +1.06               K1IVa              +0.122-0.127 +.007+056V
1488          BD-01  689  29610131392                       043442.2-011456043947.2-010310197.52-29.46 6.10  +0.94 +0.70         K0                 +0.025-0.018      +034
1489          BD+37  954  29645 573772348                   043502.3+380520044150.3+381649164.66-05.27 5.99  +0.57 +0.06         G0V                +0.241-0.093 +.022+047V?  =< 10
1490          BD+28  680  29646 767071126    3379           043504.1+282517044119.8+283654172.02-11.64 5.78  -0.03 -0.01         A2V                +0.034-0.030      +025V          5.2  43.5AB   3*
1491          BD+75  189  29678  5309 173  W                043522.2+754533044850.3+755628135.73 19.40 6.06  +0.28 -0.02         A9IV               +0.035-0.135 +.021-006SB     97  6.0  96.9
1492          CP-62  372  29712249066    I W       R Dor    043535.6-621627043645.6-620439272.67-39.34 5.40  +1.58 +0.86 +3.16   M8eIII:            -0.064-0.084      +026           6.5  36.5AC   3*
1493          BD+49 1230  29721 397021128  W                043545.4+494658044321.6+495826156.02  2.62 5.87  +0.02 -0.25         B9III              -0.004-0.016      +003           7.3  20.8
1494 59    PerBD+43 1043  29722 396992349                   043548.5+431029044254.3+432154160.95-01.78 5.29   0.00 +0.02         A1Vn               +0.039-0.052 +.018+009V     197
1495          CD-24 2488  297371696501127                   043557.2-244040044006.8-242857223.58-38.91 5.58  +0.92               G6III              -0.067+0.018      -018SB
1496 54    EriBD-19  988  29755149818    I   3380  DM Eri   043604.0-195148044026.5-194018217.79-37.36 4.32  +1.61 +1.81 +1.38   M4III              +0.021-0.096 +.008-033           0.3   0.1      *
1497 94Tau TauBD+22  739  29763 76721 174  W                043614.5+224555044214.7+225725176.64-15.07 4.28  -0.13 -0.57 -0.14   B3V                -0.002-0.016 +.017+015SB2O  187  0.7   0.1     5*
1498          CD-51 1207  29805233605                       043635.6-515207043904.3-514022259.24-41.23 6.44  +1.32               K2III              +0.017+0.017      +037
1499 95    TauBD+23  733  29859 76727                       043710.4+235358044313.8+240520175.88-14.19 6.13  +0.54 +0.07         F7IV-V             +0.017-0.022      +008V   =< 10
1500          BD+40 1032  29866 39715                       043717.4+403554044412.9+404713163.06-03.28 6.08  +0.06 -0.28         B8IVne             +0.003-0.007      +041      300                 *
1501          BD+32  827  29867 57393                       043720.3+324039044348.3+325155169.06-08.49 6.45R                     A8V                -0.035-0.042      +000       73
1502   Alp CaeCD-42 1587  298752169261129  W       Var?     043720.3-420316044033.7-415150246.15-41.49 4.45  +0.34 +0.01 +0.21   F2V                -0.141-0.077 +.045-001V      52  8.0   6.6      *
1503   Bet CaeCD-37 1867  299921952391130                   043831.2-372022044203.5-370840239.92-40.91 5.05  +0.37 +0.04         F1V                +0.049+0.193 +.058+027V?    140
1504          CP-59  370  30003233622      W                043839.2-590825044018.3-585637268.60-39.73 6.53  +0.68 +0.24         G5V                +0.070+0.186 +.064+010           0.2   2.7      *
1505 55    EriBD-09  969  30020131442        3409B DW Eri   043846.8-085852044334.6-084737205.96-32.37 6.82  +0.39 +0.14         F4IIIpSr           +0.021-0.012D+.008+040SB     60  0.1   9.3      *
1506 55    EriBD-09  970  30021131443        3409A          043847.2-085859044335.1-084746205.96-32.37 6.70  +0.92 +0.59         G8III              +0.032-0.024D+.008+048SB  =< 25  0.1   9.3      *
1507          BD+10  621  30034 94095      W        1705    043853.2+105735044425.8+110846186.90-21.80 5.40  +0.25 +0.08 +0.13   F0V                +0.100-0.015      +040       86  5.8  74.7AB   3*
1508 56    EriBD-08  929  300761314511131          DX Eri   043917.0-084125044405.3-083013205.72-32.12 5.90  -0.11 -0.81 -0.03   B2Ve               +0.003-0.002      +015      240                 *
1509          CD-30 1968  30080195250    I                  043917.3-305705044309.3-304556231.68-39.74 5.68  +1.41 +1.60         K2III              -0.030-0.067      -004V?
1510          BD+70  322  30085  53262358                   043924.5+704547045036.4+705630140.05 16.55 6.37  -0.08 -0.24         A0IV               +0.001-0.020      +007V?  =< 41
1511  4    CamBD+56  973  30121 24829 175    3432           043940.2+563447044800.3+564526151.27  7.54 5.30  +0.25 +0.15 +0.08   A3m                +0.050-0.148 +.013+019       63  7.5  99.0AC   3*
1512          BD+23  739  30122 76737                       043940.1+232639044542.5+233741176.62-14.03 6.35  +0.06 -0.44         B5III              +0.007-0.015      +019V?                        *
1513          BD-18  906  301271498562352                   043943.0-185108044408.0-184000216.98-36.20 5.53  +0.02 +0.04         A3V                +0.051-0.018      +005
1514          BD+40 1045  30138 39746                       043950.3+400751044644.4+401846163.73-03.23 5.97  +0.93               G9III              -0.001-0.029      +034
1515          BD+55  928  30144 24834                       043954.8+552527044807.0+553609152.18  6.81 6.26R                     F0                 +0.075-0.102      +022       53                 *
1516   Lam PicCD-50 1471  301852336382351                   044012.5-504010044246.4-502853257.58-40.79 5.31  +0.98 +0.74         K0-1III            -0.031+0.030 -.000+005
1517          BD+18  719  30197 94112                       044026.3+183314044616.8+184406180.71-16.93 6.01  +1.21 +1.32 +0.63   K4III              +0.072-0.065      +038                          *
1518          CD-41 1549  30202216961                       044026.9-411503044344.2-410353245.12-40.87 6.25  +1.46 +1.78         K3-4III            +0.003+0.010      -004
1519          BD+11  646  30210 941112353  W                044027.9+113122044601.7+114220186.65-21.16 5.37  +0.19 +0.14 +0.08   A2m                +0.068-0.006 +.005+041SB1?   47  7.5  93.8AC   3*
1520 57Mu  EriBD-03  876  30211131468 176                   044030.0-032616044530.1-031517200.53-29.34 4.02  -0.15 -0.60 -0.14   B5IV               +0.016-0.013      +009SB1O  140                 *
1521          BD-21  966  30238169727    I                  044046.0-212800044504.2-211701220.15-36.87 5.72  +1.47              gK2                 +0.018-0.025      +022V
1522          BD-03  884  30321131481                       044123.4-030805044624.1-025716200.36-28.99 6.33  +0.04 +0.06         A2V                +0.012-0.047      +019
1523          BD+80  155  30338   7833948                   044137.3+810141050020.7+811138131.38 22.78 5.07  +1.28 +1.47         K3III              -0.001+0.024 +.020-008V?   < 17
1524          CD-34 1859  30397195275                       044207.5-341113044549.6-340018235.96-39.75 6.86   0.00               A0V                +0.040+0.018      +028
1525          CD-28 1735  304221697521132                   044226.1-281606044625.8-280515228.51-38.47 6.19  +0.19               A3IV               +0.001+0.016      +019      157
1526          CD-39 1624  304321952782355                   044232.6-393213044555.4-392124242.91-40.35 6.05  +1.07               K1III              -0.059-0.020      -006
1527          BD+63  543  30442 132912362I W                044243.4+632005045205.2+633019146.25 12.16 5.44  +1.57 +1.76         M3IIIab            +0.037-0.096 +.001-036V          4.5 115.4      *
1528          BD+32  840  30453 57444      S                044250.8+322446044919.0+323518170.03-07.78 5.86  +0.24 +0.14 +0.11   A8m                +0.018-0.031      +021SB1O   15                 *
1529          BD+31  816  30454 574412359I                  044248.1+311549044912.8+312614170.91-08.52 5.58  +1.12 +1.03         K2III              +0.017-0.108      +023V
1530   Kap DorCP-59  376  304782336642354          Var      044250.5-595458044421.1-594358269.44-39.05 5.27  +0.20               A8-9III-IV         +0.033+0.039 +.024+000V     177
1531          CP-77  181  304792561162350                   044253.7-775041043821.7-773922290.58-33.35 6.05  +1.10 +0.95 +0.52C  K2III              -0.010-0.018      +009
1532 58    EriBD-17  954  30495149888                       044306.8-170703044736.3-165604215.37-34.81 5.51  +0.63 +0.12 +0.34   G2.5V              +0.133+0.170 +.079+023V                         *
1533          BD+37  969  30504 574471133I                  044310.6+371842044954.6+372918166.30-04.57 4.88  +1.44 +1.70 +0.58E  K3.5IIIBa0.2:      -0.038+0.040 +.025-023V?   < 19:                *
1534          BD+03  681  30545112098                       044329.5+032445044844.6+033518194.39-25.12 6.03  +1.19 +1.15         K1III              -0.001-0.017      -019
1535          BD+48 1162  30557 397992361                   044338.0+483405045109.3+484427157.78  2.79 5.66  +0.99 +0.80         G9III              -0.040-0.031      +029
1536          BD-05 1044  30562131504                       044339.7-055037044836.3-054026203.39-29.83 5.78  +0.62 +0.20         F8V                +0.305-0.243 +.038+079V                         *
1537 96    TauBD+15  687  30605 94151    I   3464           044400.7+154347044944.1+155415183.60-17.97 6.08  +1.60 +1.72        gK3                 +0.005-0.010      +013V?         5.5  29.3AB   3
1538 59    EriBD-16  956  30606149901                       044402.5-163024044832.5-161946214.78-34.37 5.77  +0.55 +0.09         F6V                +0.004+0.036      +035
1539   Zet CaeCD-30 2011  30608195300                       044355.4-301200044749.6-300113231.01-38.60 6.37  +1.07               K1IV               +0.039+0.094      +006V
1540          CP-63  365  306102490952356           1718    044402.3-632435044457.9-631347273.75-38.12 6.46  +1.08 +1.00         K0-1III            -0.013-0.005      +016
1541   Mu  MenCP-71  282  30612256122 177                   044403.6-710652044303.9-705552282.97-35.83 5.54  -0.12 -0.46         B8II-IIISi:        +0.008+0.033      -001V?
1542  9Alp CamBD+66  358  30614 13298 178                   044406.3+661023045403.0+662034144.07 14.04 4.29  +0.03 -0.88  0.00   O9.5Ia            e 0.000+0.006 -.002+006V?     95                 *
1543  1Pi 3OriBD+06  762  306521121061134I W        1731    044424.6+064712044950.4+065741191.45-23.07 3.19  +0.45 -0.01 +0.26   F6V                +0.466+0.012 +.125+024SB2    17  5.5  94.6      *
1544  2Pi 2OriBD+08  777  30739112124    I                  044509.6+084343045036.7+085401189.82-21.83 4.36  +0.01  0.00  0.00   A1Vn               +0.004-0.032 +.036+024SB    212                 *
1545          BD-14  970  307431499162363                   044507.0-135618044942.2-134611212.07-33.12 6.26  +0.44 -0.05         F5V                -0.122-0.172      -003    =< 15
1546          BD+52  891  30752 24894                       044515.1+524020045309.8+525027154.80  5.63 6.41                      A2V                -0.008-0.021      -013
1547 97    TauBD+18  743  30780 941641135  W       V480 Tau 044531.3+184011045122.5+185023181.38-15.91 5.10  +0.21 +0.12 +0.12   A7IV-V             +0.080-0.035 +.022+037V     141  5.3 177.0      *
1548          CD-44 1720  307882170042360                   044529.1-440918044833.8-435848248.99-40.07 6.72  +0.95 +0.66         G8III              +0.018+0.022      +019
1549 60    EriBD-16  964  30814149924    I          1740    044541.1-162327045011.6-161302214.84-33.96 5.03  +0.98 +0.77         K0III             v+0.041+0.054 +.023+037
1550          BD+42 1081  30823 39826                       044543.8+422503045247.8+423512162.71-00.90 5.71  +0.11 +0.14         A3III              -0.006-0.001      -001V
1551  2    AurBD+36  952  30834 57475    I                  044556.2+363203045238.0+364211167.26-04.65 4.78  +1.41 +1.58 +0.78   K2.5IIIbBa0.4      -0.027-0.007      -017     < 17                 *
1552  3Pi 4OriBD+05  745  30836112142 179           1742    044552.7+052603045112.4+053618192.89-23.52 3.69  -0.17 -0.81 -0.16   B2III+B2IV         -0.001+0.001 +.001+023SBO    40                 *
1553          BD+09  668  30870112150                       044613.8+094820045143.4+095830189.03-21.00 6.11  +0.08 -0.44         B5V                +0.001-0.008      +011                          *
1554          BD+27  701  30912 768022365                   044632.2+274349045247.1+275351174.20-10.13 5.97  +0.37 +0.16         F2IV               +0.041-0.032      +038      135                 *
1555  5    CamBD+55  941  30958 249042367    3508           044652.4+550539045503.1+551533153.07  7.36 5.52  +0.04  0.00         B9.5V              -0.012-0.010      +002V?     71  6.4  12.9      *
1556  4Omi1OriBD+14  777  30959 941761136I         Omi1 Ori 044652.4+140503045232.0+141502185.43-18.39 4.74  +1.84 +2.03 +1.17E  S3.5/1-             0.000-0.057 +.004-008V                         *
1557          CD-41 1593  30985217032      W                044701.0-412936045016.2-411915245.54-39.66 6.07  +0.37               F2-3V              +0.006+0.065      +025           4.6  13.4      *
1558          BD+43 1116  31069 39851                       044740.4+435352045451.3+440339161.81  0.32 6.08  -0.02 -0.09         A0V                +0.027-0.053      +003V     121
1559          CD-35 1962  310931953572364  W                044749.8-350426045128.2-345423237.34-38.74 5.86  +0.08 +0.09         A1Vn               +0.026-0.022      +023SB         0.4   0.1
1560 61Ome EriBD-05 1068  311091315682366                   044758.8-053711045253.7-052710203.75-28.78 4.39  +0.25 +0.16 +0.17   F4III+A6III        -0.023+0.021 +.013-006SBO   153                 *
1561          BD+52  898  31134 24919                       044811.4+524225045607.1+525210155.05  6.00 5.75  +0.11 +0.10         A2V s              -0.003+0.008      -022SB     40
1562  5    OriBD+02  800  31139112179    I          1755    044809.8+022035045322.8+023029196.08-24.71 5.33  +1.64 +1.95 +0.91E  M1III              +0.034-0.019 +.010+013
1563   Iot PicCP-53  760  31203233709      W                044841.3-533754045055.2-532741261.28-39.22 5.61  +0.33               F0IV               -0.091+0.083      +016V      57  0.8  12.3      *
1564   Iot PicCP-53  760  31204233710      W                044842.5-533747045056.3-532735261.27-39.22 6.42                      F4V:               -0.092+0.081      +010V          0.8  12.3      *
1565          BD+01  847  31209112191                       044845.1+012419045355.8+013410197.04-25.08 6.61  +0.04 +0.01         A1Vn               +0.009-0.003      +021      139
1566          BD+19  811  31236 94199                       044905.5+191925045458.3+192907181.36-14.84 6.37  +0.29 +0.06 +0.16   F3IV               +0.060-0.038      +035V?    102                 *
1567  8Pi 5OriBD+02  810  31237112197 180          Pi5 Ori  044902.5+021637045415.1+022626196.27-24.56 3.72  -0.18 -0.83 -0.20   B3III+B0V           0.000 0.000 +.003+023SBO    93                 *
1568  7    CamBD+53  829  31278 24929        3536           044916.0+533532045717.2+534508154.47  6.68 4.47  -0.02 -0.01 -0.01   A1V                -0.024+0.007 +.000-008SB1O   45  3.4   0.4AB   3*
1569  6    OriBD+11  675  31283 941972368  S                044914.0+111546045446.9+112534188.21-19.56 5.19  +0.12 +0.11         A3V                -0.009+0.032      +009V     127
1570  7Pi 1OriBD+09  683  31295 94201      W                044923.4+095931045453.8+100903189.34-20.25 4.65  +0.09 +0.09 +0.03   A0Vp               +0.046-0.132      +013V     104  4.2 171.6AB   3*
1571          BD+07  755  312961122032369I          1763    044923.3+073702045447.8+074645191.44-21.59 5.33  +1.22 +1.18        gK1                 -0.021-0.027      -005
1572          BD+74  229  31312  5385    I                  044938.5+740653050220.1+741609137.71 19.17 6.06  +1.57 +1.83         K5III              +0.021+0.034      -052V?
1573          BD+35  930  31327 57511                       044939.4+360029045619.9+361008168.14-04.40 6.07  +0.41 -0.43         B2Ib               -0.017+0.013      -005V?  =< 32                 *
1574          BD+00  893  31331112206                       044942.5+001819045450.7+002803198.22-25.45 5.99  -0.12 -0.56         B5V                 0.000+0.003      +017SB    200
1575          BD+24  709  31362 76848                       045010.0+242557045615.6+243532177.35-11.54 6.37  +0.33 -0.08         F0                 -0.026-0.017      -009V?     60
1576          BD+14  787  31373 94212                       045008.6+145249045550.3+150224185.24-17.28 5.81  -0.08 -0.45         B9V                +0.018-0.021      +007V      90
1577  3Iot AurBD+32  855  31398 57522 181I          1778    045028.7+330028045659.6+330958170.59-06.16 2.69  +1.53 +1.78 +0.82   K3II               +0.003-0.018 +.021+018V    < 17                 *
1578          BD+05  769  31411112220                       045039.2+051423045558.4+052357193.77-22.63 6.50  +0.02 -0.01         A0V                -0.004-0.009      +022      118
1579          BD-16  991  31414149985                       045038.1-165405045506.8-164426215.95-33.06 5.70  +0.96              gG9                 +0.008+0.001      +010
1580  9Omi2OriBD+13  740  31421 94218    I   3540           045044.8+132124045622.3+133052186.63-18.05 4.07  +1.15 +1.11 +0.63   K2-IIIFe-1         -0.074-0.047 +.019+001     < 17  7.5 100.4AC   3*
1581          BD-16  992  314441499882370          R Eri    045049.1-163446045518.6-162504215.61-32.90 5.72  +0.88              gG4                 +0.010+0.045      +032V                         *
1582 62    EriBD-05 1091  31512131614      W                045128.6-051946045624.2-051017203.93-27.88 5.51  -0.13 -0.56         B6V                -0.008-0.007      +024      104  3.6  67.3
1583          CD-25 2115  31517169889                       045124.6-255316045530.2-254340226.30-35.91 6.72  +0.27               F0-2III            +0.018+0.028      +025
1584          CD-39 1691  315291954002371                   045133.8-394721045454.8-393743243.46-38.65 6.10  +1.42               K3III              -0.003+0.022      +029
1585          BD+16  672  31539 94227    I D                045135.7+165949045722.3+170913183.67-15.75 5.48  +1.31              gK1                 -0.013-0.007      +025V          0.0   0.1      *
1586 99    TauBD+23  777  31553 76858        3557           045144.5+234733045748.7+235655178.09-11.64 5.79  +1.11              gG8                 +0.008-0.007      +004           6.4 103.5AC   3
1587          BD+73  264  31563  5403    I                  045145.0+733656050413.0+734550138.24 19.00 6.66R                     K0                 +0.021-0.016      +022
1588  8    CamBD+52  906  31579 24943    I                  045148.2+530006045946.3+530920155.17  6.61 6.08  +1.46               K4III              -0.014-0.009      -002
1589          BD+73  265  31590  5407      W                045203.1+735510050439.8+740401137.98 19.19 5.96R                     A1V                +0.013-0.016D+.010-009V      93  1.0   0.6      *
1590 98    TauBD+24  717  31592 768622374    3547           045202.1+245346045809.4+250301177.24-10.92 5.81   0.00 -0.02         A0V                +0.022-0.048      +026SB     50  5.8  94.6AC   4*
1591          BD-01  762  316231316252373                   045212.6-011323045717.2-010402200.03-25.70 6.23  +0.42 +0.06         F2                 -0.037-0.028      +009       67
1592  4Ome AurBD+37 1005  31647 57548        3572           045228.0+374421045915.4+375325167.15-02.88 4.94  +0.04 +0.02 +0.03   A1V                +0.040-0.101 +.005+005V      83  2.8   5.1      *
1593          BD+60  853  31662 13341        3590           045236.3+605555050135.9+610441148.91 11.58 6.03  +0.41               F4V                +0.008-0.175      +011       34  5.5   5.4      *
1594          BD+66  370  31675 13344                       045242.2+664059050250.4+664922144.19 15.03 6.19  +0.48 -0.04        dF6                 +0.066-0.345 +.034+017V?                        *
1595          BD-14 1003  31726150029                       045310.1-142311045744.8-141353213.50-31.51 6.15  -0.21               B1V                +0.010+0.011      +011
1596          BD-02 1080  31739131640        3570           045308.6-022202045810.8-021245201.26-26.07 6.35  +0.10 +0.11         A2V                -0.003+0.025      +037           5.5  21.0
1597          CP-58  437  31746233733                       045314.5-584227045453.0-583250267.62-37.96 6.12  +0.44 -0.06         F3V                +0.101+0.077      +022
1598          CP-66  338  317542491382372           1777    045321.2-665007045330.5-664032277.61-36.31 6.41  +1.63 +1.96         M0-1III             0.000+0.010      -024V                         *
1599  5    AurBD+39 1133  31761 57559        3589           045325.7+391436050018.3+392341166.09-01.79 5.95  +0.41 -0.03         F5V                -0.017+0.004 +.021+006           3.4   3.7      *
1600          BD+14  796  31764 94240        3579A  1788    045318.8+142325045859.4+143234186.12-16.94 6.09  +0.04 -0.27 +0.01   B7V                +0.006-0.017      +004      125  1.5  39.3AB   4*
1601 10Pi 6OriBD+01  872  31767112281    I          1786    045321.8+013338045832.9+014251197.56-24.02 4.47  +1.40 +1.55 +0.70   K2-II              -0.002-0.003 +.018+014V?   < 17
1602  6    AurBD+39 1134  31780 57560    I                  045329.5+393013050023.2+393918165.90-01.61 6.58R                     K4I                +0.002+0.006      -024V
1603 10Bet CamBD+60  856  31910 13351 182I   3615A          045431.1+601746050325.1+602632149.57 11.38 4.03  +0.92 +0.63 +0.45   G1Ib-IIa           -0.006-0.016 +.009-002       19  3.4  80.8AB   3*
1604          BD-16 1013  31925150052        3588   1793    045432.9-163153045901.3-162233215.97-32.06 5.66  +0.43 -0.08         F3V+F9V            -0.141+0.148 +.028+030V?         1.5   0.2AB   3*
1605  7Eps AurBD+43 1166  31964 39955 183I   3605  Eps Aur  045447.4+434032050158.1+434924162.79  1.18 2.99  +0.54 +0.33 +0.45   F0Iae+B           v-0.001-0.004 +.007-003SB1O   29  6.2 207.6AE   6*
1606          CP-72  332  31975256139                       045445.5-723425045305.5-722427284.32-34.57 6.28  +0.52 +0.01         F6V                -0.052+0.276      +025
1607          BD-15  915  31996150058    I         R Lep    045503.2-145724045936.5-144821214.32-31.33 7.71  +5.74       +1.47   C6IIe              +0.021+0.020      +032V                         *
1608 63    EriBD-10 1066  320081500602375                   045506.4-102435045950.4-101548209.55-29.41 5.38  +0.80 +0.35         G4V                +0.020-0.135      -012V
1609          BD+03  736  32039112304        3597B  1800    045517.1+032759050032.6+033655196.07-22.61 7.03                      B9Vn               +0.025+0.001      +031      340  0.4  21.3      *
1610          BD+03  737  32040112305        3597A  1800    045518.5+032803050033.9+033658196.07-22.60 6.66  -0.07 -0.28         B9Vn               +0.007-0.012      +042      352  0.4  21.3      *
1611 64    EriBD-12 1047  32045150064              S Eri    045516.9-124105045955.8-123215211.94-30.34 4.79  +0.26 +0.16 +0.14   F0IV               +0.045-0.091 +.022-009V?    173                 *
1612  8Zet AurBD+40 1142  32068 399661137I         Zet Aur  045529.1+405548050228.7+410433165.02-00.43 3.75  +1.22 +0.38 +0.87   K4II+B8V           +0.009-0.022 +.005+013SBO    19:                *
1613          BD-02 1095  32115131684                       045537.3-021252050039.8-020356201.45-25.45 6.32  +0.28 +0.04         A8IV               -0.019+0.008      +021V
1614          BD-05 1123  32147131688                       045551.1-055216050049.0-054512205.07-27.18 6.22  +1.06 +1.02 +0.49   K3V                +0.551-1.094 +.110+027SB
1615          BD+41 1044  32188 39979               1810    045617.6+411750050318.6+412630164.82-00.08 6.14  +0.16               A2IIIShell?        +0.002-0.004      -001V?                        *
1616          BD+85   74  32196   8431637                   045617.8+854946053148.0+855619127.17 25.68 6.51  +0.32 +0.10         A5m                +0.013-0.079      -006SB                        *
1617 65Psi EriBD-07  948  32249131700                       045635.4-071914050126.3-071026206.59-27.69 4.81  -0.19 -0.74 -0.19   B3V                +0.003+0.008      +025V?     74
1618          BD+00  923  322631123342376           1806    045641.5+003437050150.3+004320198.95-23.81 5.92  +1.27 +1.39         K0                 +0.001-0.012      +021
1619          BD+01  886  32273112340        3623A          045649.2+012747050200.0+013632198.14-23.33 6.24  -0.04 -0.41         B8V                -0.009+0.012D+.013+025V      53  1.3  14.2AxBC 4*
1620102Iot TauBD+21  751  32301 76920 184  D                045707.0+212650050305.7+213524180.77-12.06 4.64  +0.16 +0.15 +0.09   A7V                +0.066-0.042 +.019+041      126  0.0   0.1      *
1621          BD-20  990  32309169981                       045705.2-201151050125.6-200307220.27-32.85 4.91  -0.05 -0.14 -0.05   B9.5Vn             +0.034-0.015      +024      237:                *
1622 11    CamBD+58  804  32343 25001      W       BV Cam   045726.7+584958050608.5+585821150.99 10.80 5.08  -0.08 -0.69         B2.5Ve             -0.005-0.008      -011V?    131  1.0 180.5AB   4*
1623 12    CamBD+58  805  32357 25003      W       BM Cam   045729.9+585256050612.2+590116150.95 10.83 6.08  +1.12 +0.85?        K0III              -0.001-0.031      -008SBO =< 25  1.0 180.5AB   4*
1624          BD+60  857  32356 13369    I                  045727.4+610157050629.7+611012149.19 12.11 6.04  +1.37               K5II               +0.036-0.075      -040
1625          BD-04 1019  32393131715                       045747.4-042113050245.5-041235203.82-26.03 5.85  +1.21 +1.36         K3                 +0.055+0.016      +038V?                        *
1626          BD+30  772  32406 57610                       045751.2+302112050414.5+302941173.66-06.57 6.14  +1.21               K0II-III           +0.001-0.005      +018
1627          BD+32  879  32428 57614                       045808.0+321053050436.9+321913172.25-05.41 6.62  +0.27 +0.13 +0.13   A4m                -0.011-0.070      -008                          *
1628          CD-26 1975  324361699972377I                  045805.7-262500050209.8-261630227.45-34.62 5.02  +1.07 +0.97         K1III              +0.089-0.073 +.007+027
1629   Eta MenCP-75  290  324402561451138                   045803.5-750526045511.2-745613287.12-33.55 5.47  +1.52 +1.83         K4III              +0.024+0.059 +.026+026
1630          BD+54  859  32445 25007        3667           045814.4+541559050622.0+542421154.75  8.14 7.24R                     G5                 -0.014+0.028                     3.7   6.7AB   3*
1631          CD-39 1744  32453195501                       045814.7-395148050134.5-394305243.74-37.38 6.03  +0.88               G5III              -0.001+0.030      +006
1632          BD+27  723  32480 76941                       045822.7+273324050437.9+274146175.98-08.17 6.60  +0.24 +0.12         F0III              +0.019-0.031      +022      105
1633          BD+21  755  32482 76939                       045823.9+210816050421.6+211641181.21-12.01 6.19  +1.32 +1.47 +0.48E  K0                 +0.020-0.015      +048
1634  1    LepBD-22 1960  32503170010    I                  045831.6-225617050244.9-224742223.49-33.46 5.75  +1.20 +1.14         K1IV               +0.059+0.023      +033                          *
1635          CD-31 2163  325151955091139                   045835.7-315500050222.8-314617234.00-35.92 5.94  +1.17               G8III              -0.002+0.082      +029
1636          BD+69  302  32518 133822382                   045840.0+693024050936.7+693822142.12 17.12 6.41  +1.11 +1.03         K1III              +0.056-0.063 +.007-008
1637  9    AurBD+51 1024  32537 25019        3675           045850.6+512754050640.6+513552157.06  6.51 5.00  +0.33 -0.01 +0.20   F0V                -0.030-0.173 +.052-005SBO    14  4.4  90.1AC   5*
1638 11    OriBD+15  732  32549 942901140          V1032 Ori045851.2+151554050434.1+152415186.18-15.35 4.68  -0.06 -0.09 -0.02   A0pSi              +0.016-0.034 +.018+017V      36                 *
1639          BD+35  973  32608 576292379                   045920.0+354756050600.9+355611169.53-03.00 6.52  +0.16 +0.09         A5V                -0.011-0.010      +014       80
1640          BD-14 1027  32612150109                       045918.0-143038050352.0-142210214.33-30.21 6.41  -0.18 -0.77         B2.5IV             -0.009+0.033      +016
1641 10Eta AurBD+41 1058  32630 40026 185           1822    045930.0+410558050630.9+411404165.35  0.27 3.17  -0.18 -0.67 -0.17   B3V                +0.029-0.068 +.022+007V?    132                 *
1642          BD+19  847  32642 94306        3672           045938.3+194009050532.1+194823182.60-12.64 6.44  +0.21 +0.24 +0.14   A5m                -0.009-0.022D+.008-017       53  0.6   1.1
1643          BD+73  274  32650  54552387          BN Cam   045945.3+734904051222.4+735648138.39 19.58 5.43  -0.12?-0.36?        B9pSi               0.000-0.025 +.024+009V      37                 *
1644          BD+42 1170  32655 40029                       045941.0+430218050649.6+431029163.84  1.49 6.20  +0.43 +0.30         F2IIp:             +0.005 0.000      -014    =< 23
1645          CD-24 2795  32667170029                       045944.6-243137050353.3-242316225.40-33.70 5.61  +0.10 +0.07  0.00   A3m                +0.024-0.028      +007SB    216
1646          BD-03  998  326861317422378                   045954.2-031041050454.5-030223202.96-25.00 6.05  -0.11 -0.53         B5IV               +0.001-0.001      +027                          *
1647          BD+64  500  32715 133882384                   050000.8+644731050944.5+645510146.23 14.54 6.41  +0.40 -0.02         F6V                -0.015-0.182 +.024+000V
1648          BD+00  939  32736112406    I         W Ori    050013.8+010225050523.7+011039199.01-22.82 6.17  +3.45 +6.5  +1.48   C6II               +0.012-0.007      +017V?                        *
1649   Eta1PicCD-49 1541  32743217140      W                050011.7-491734050248.6-490905255.64-37.63 5.38  +0.42 +0.01         F2V                -0.044+0.024 +.055+021        0  7.6  10.4
1650          BD+76  190  32781  5464        3738   1854    050030.1+762048051435.6+762822136.14 20.98 6.37  -0.02 -0.08         A0V                -0.007-0.008      +007       34  4.5   1.5      *
1651          CD-41 1690  32820217153                       050042.2-415317050354.0-414442246.33-37.15 6.31  +0.53 +0.04         F8V                +0.027+0.152      +029
1652   Gam1CaeCD-35 2089  32831195532      W       Var?     050048.5-353711050424.4-352900238.60-36.23 4.55  +1.20 +1.20 +0.65   K3III              +0.129-0.051 +.013-003V          4.1   3.0      *
1653   Gam2CaeCD-35 2090  32846195534              X Cae    050051.8-355039050426.1-354219238.87-36.26 6.34  +0.30 +0.08         F1III              +0.028+0.038      +006                          *
1654  2Eps LepBD-22 1000  32887170051 186I          1826    050113.6-223019050527.7-222216223.25-32.73 3.19  +1.46 +1.78 +0.81   K5III             v+0.025-0.074 +.011+001
1655          CD-26 2005  32890170050                       050112.5-261714050516.2-260909227.54-33.91 5.73  +1.17               K2III              +0.007-0.065      -003V?
1656104    TauBD+18  779  32923 94332        3701           050132.3+183039050727.0+183842183.83-12.94 5.00  +0.65 +0.14         G4V                +0.537+0.016 +.061+020V?      2  0.1   0.1      *
1657 66    EriBD-04 1044  32964131777        3698   1831    050148.9-044721050645.7-043918204.78-25.35 5.12  -0.06 -0.17 -0.10   B9V+A1V            +0.016+0.008 +.024+031SB2O=< 41  5.4  52.8      *
1658106    TauBD+20  885  32977 769712383                   050153.2+201712050748.4+202506182.40-11.85 5.30  +0.09 +0.10         A5V                -0.047-0.034 +.007-002V      95                 *
1659103    TauBD+24  755  32990 76974        3709           050200.9+240759050806.6+241555179.25-09.56 5.50  +0.06 -0.57         B2V               e 0.000-0.004      +016SBO    98  5.7  35.3AC   4*
1660105    TauBD+21  766  32991 76972      W                050156.6+213421050755.5+214217181.34-11.09 5.89  +0.19 -0.57         B2Ve               +0.005-0.006      +017V     220  0.0   0.3 O   3*
1661          BD-13 1063  32996150149                       050159.6-131525050636.7-130719213.32-29.10 6.05  -0.06               A0V                +0.019+0.035      +040V?  =< 41
1662 13    OriBD+09  736  33021112436      W                050209.4+092059050738.3+092819191.77-18.00 6.17  +0.62 +0.10         G1IV               -0.003-0.378 +.033-024V?  =<  6  3.8 124.0AB   3*
1663   Eta2PicCD-49 1562  33042217164 187           1827    050222.5-494248050458.0-493440256.17-37.28 5.03  +1.49 +1.88 +0.91   K5III              +0.071-0.003 +.009+036                          *
1664 14    OriBD+08  866  33054112440        3711           050226.1+082205050752.9+082954192.67-18.48 5.34  +0.33 +0.10 +0.18   Am                 +0.024-0.065 +.004+006V      52  0.8   0.7      *
1665          BD-12 1076  33093150159                       050245.6-123714050725.0-122926212.75-28.66 5.97  +0.60               F7V                +0.140-0.080      +050V?  =< 15
1666 67Bet EriBD-05 1162  33111131794 188I W        1841    050255.9-051256050751.0-050511205.34-25.31 2.79  +0.13 +0.10 +0.08   A3III              -0.095-0.081 +.050-009      179  8.0 116.7      *
1667          CP-54  768  331162338142381                   050254.8-543233050500.6-542427262.23-37.07 6.27  +1.54 +1.90         M2III              +0.002 0.000      +105
1668          BD+46  970  33167 40077                       050316.0+465019051042.9+465744161.20  4.29 5.68  +0.42 +0.02         F5V                +0.055-0.147 +.027+033V                         *
1669          BD+37 1067  33203 57704    I   3734           050332.6+371028051018.9+371807168.95-01.49 6.02  +0.72               K3:+B2II:          +0.004-0.006D+.002+009SB         0.1   1.5AB   3*
1670          BD+27  732  33204 769901141    3730A          050328.2+275413050945.1+280150176.38-07.06 6.01  +0.27 +0.04 +0.14   A5m                +0.054-0.060D+.008+041       27  2.5  11.6AxBC 3*
1671          BD-08 1037  33224131806        3722A          050332.7-084742050820.2-083954208.95-26.83 5.78  -0.06 -0.37         B8V                +0.005-0.012      +027V          3.2  21.6AB   3*
1672 16    OriBD+09  743  332541124671142  W        1849    050349.5+094204050919.6+094946191.69-17.46 5.43  +0.24 +0.16 +0.12   A2m                +0.064-0.007 +.011+037SBO    15  4.5 168.0AC   3*
1673 68    EriBD-04 1056  33256131813                       050346.2-043510050843.6-042722204.83-24.83 5.12  +0.44 -0.05         F2V                +0.044+0.017 +.050+009        0
1674   Zet DorCP-57  735  33262233822 189                   050347.7-573633050530.6-572822266.04-36.72 4.72  +0.52 -0.04 +0.29   F7V                -0.030+0.115 +.085-002        0
1675          BD+61  766  33266 13409                       050352.8+614333051303.2+615100149.07 13.14 6.17  +0.02 +0.08         A2III              +0.011+0.006      -004
1676 15    OriBD+15  752  33276 94359      W                050358.4+152811050942.0+153550186.74-14.21 4.82  +0.32 +0.19 +0.19   F2IV               +0.002-0.013 +.004+030       53                 *
1677   Bet MenCP-71  309  332852561542380                   050400.4-712703050243.0-711852282.77-34.21 5.31  +1.00 +0.77         G8III              -0.005+0.004 +.013-011
1678 14    CamBD+62  734  33296 13413                       050412.7+623405051331.3+624129148.38 13.65 6.50  +0.21 +0.12         A7Vn               -0.036+0.006      -004SB
1679 69Lam EriBD-08 1040  33328131824 190          Lam Eri  050421.6-085256050908.8-084515209.14-26.69 4.27  -0.19 -0.90 -0.20   B2IVne            v+0.002-0.004 -.003+003V     336                 *
1680          CD-35 2126  33377195581                       050441.1-355050050814.8-354306239.05-35.50 6.52  +1.08 +0.94         K1III              +0.009+0.001      -005
1681          BD-00  867  33419131834                       050457.3-004125051003.2-003355201.28-22.67 6.10  +1.10 +1.07         K0III              +0.001-0.053      +026
1682          CP-78  165  33519256153      W                050535.9-782614050013.2-781801290.71-32.05 6.29  +1.51 +1.88         K5-M0III           -0.023-0.012      +000           4.0  46.4
1683          BD+73  280  33541  5483                       050552.8+730912051813.3+731605139.23 19.59 5.74R                     A0V                -0.005-0.033      +000SB2    65
1684          BD+15  759  33554 943772391I                  050556.8+155520051141.6+160244186.63-13.57 5.18  +1.49 +1.86         K5III              +0.005+0.007      -006V?   < 19:
1685          BD-02 1161  33555131847                       050555.4-022227051058.0-021514203.00-23.29 6.25  +0.98 +0.81         K1-IVa             +0.075-0.145      +064
1686          BD+79  169  33564  5496 191    3864A          050604.1+790659052233.5+791352133.74 22.65 5.05  +0.47 -0.13         F6V                -0.082+0.161 +.054-010        0  3.7  10.4AB   3*
1687          BD-02 1165  336081318522390                   050617.1-023651051119.2-022927203.27-23.33 5.90  +0.46 +0.04         F5V                +0.079+0.001      +031    =< 10
1688          BD+59  857  33618 25088                       050623.5+591715051511.3+592420151.30 11.99 6.15  +1.18 +1.23         K2III-IV           +0.012-0.014      +003
1689 11Mu  AurBD+38 1063  33641 57755 192                   050635.0+382158051325.7+382904168.35-00.30 4.86  +0.18 +0.09 +0.10   A4Vm               -0.018-0.075 +.023+021V      86
1690          BD+00  974  33647112505        3767  V1085 Ori050632.9+002331051141.4+003053200.48-21.79 6.67  -0.07 -0.32 -0.12   B9Vn               +0.015+0.003      +020V      39  0.4   0.1      *
1691          BD+00  975  33646112509        3764           050635.8+005453051145.3+010213200.00-21.51 5.89  +0.66 +0.29         G5III              -0.012-0.015D+.002-019       23  1.4   1.6      *
1692          BD+53  872  33654 250872393                   050642.7+530543051444.3+531250156.46  8.45 6.20  +0.07 +0.05         A0V                 0.000+0.001      +001V?     54
1693          BD-12 1092  33664150206    I         RX Lep   050642.8-115825051122.8-115057212.55-27.51 5.68  +1.46 +1.23         M6III              +0.028+0.056      +046                          *
1694          CD-26 2045  33667170151                       050640.7-260205051044.5-255434227.69-32.67 6.41  +1.25               K1III              -0.015+0.059      +042V
1695          CP-63  420  336842491982389          WZ Dor   050647.0-633132050734.0-632359273.25-35.61 5.2   +1.65 +1.85 +1.42   M3III              +0.015-0.047 +.018+019                          *
1696  3Iot LepBD-12 1095  33802150223        3778           050737.9-115921051217.9-115209212.67-27.32 4.45  -0.10 -0.40 -0.09   B8V                +0.027-0.015      +025V     193  6.3  12.7      *
1697          BD-06 1109  33833131873                       050754.6-061034051248.2-060326206.91-24.67 5.91  +0.96 +0.72         G7III              +0.027-0.032      +023
1698 17Rho OriBD+02  888  33856112528    I   3797   1872    050803.7+024432051317.5+025140198.52-20.27 4.46  +1.19 +1.16 +0.59   K0.5III            +0.001-0.006 -.010+041SB1O < 17  3.9   6.9AB   3*
1699          CD-37 2071  33872195639                       050808.5-373058051135.9-372343241.22-35.12 6.57  +1.62 +1.95         K5III              +0.007+0.004      +001
1700          CP-73  286  338752561602388                   050809.9-731000050609.3-730216284.67-33.45 6.27  -0.01 +0.01         A1V                +0.016+0.064      +008V
1701          BD+01  938  33883112535        3799           050819.8+015057051331.6+015805199.37-20.66 6.09  +0.42 +0.28         A5V                +0.010+0.012D+.004+005      100  0.2   0.5AB   3*
1702  5Mu  LepBD-16 1072  339041502371144          Mu  Lep  050826.3-161925051255.9-161220217.25-28.91 3.31  -0.11 -0.39 -0.12   B9IIIpHgMn         +0.043-0.026 +.023+028       12                 *
1703          BD+00  988  33946112543    I W        1874    050838.7+002636051347.2+003337200.71-21.31 6.32  +1.46 +1.35         M0V                +0.003-0.025      -011
1704          BD-08 1059  33948131887                       050844.8-081556051333.3-080852209.07-25.44 6.37  -0.13 -0.54         B5V                +0.002 0.000      +025
1705  4Kap LepBD-13 1092  33949150239        3800           050836.8-130335051313.9-125629213.88-27.55 4.36  -0.10 -0.37 -0.09   B9V                -0.014-0.009 +.034+018V     124  2.7   2.6      *
1706 14    AurBD+32  922  33959 57799        3824A KW Aur   050853.5+323419051524.4+324116173.30-03.35 5.02  +0.23 +0.19         A9IVDel Del        -0.024+0.014D+.007-010SBO    24  2.9  14.3AC   4*
1707          BD+53  882  34019 25112    I   3845  R Aur    050913.2+532826051717.8+533510156.37  8.97 6.5   +1.66 +0.27         M7IIIe             +0.016-0.021      +008V          2.1  46.4      *
1708 13Alp AurBD+45 1077  34029 40186 193I   3841   1897    050918.0+455347051641.4+455953162.58  4.57 0.08  +0.80 +0.44 +0.44   G5IIIe+G0III      v+0.076-0.425 +.073+030SB2O       0.5   0.0AP  10*
1709          BD+04  877  340431125562394I          1880    050925.0+050225051444.1+050922196.62-18.79 5.50  +1.37 +1.55        gK4                 -0.001+0.004      -008
1710          BD-14 1074  34045150255                       050926.7-144324051359.9-143624215.69-28.05 6.21  +0.37               F2III              -0.003+0.009      +031       67
1711108    TauBD+22  864  34053 77057      W                050926.9+221013051527.7+221705181.88-09.32 6.27  +0.08 +0.14         A2V                -0.008-0.011      -011SB     90  6.2   1.9
1712          BD+34  980  34078 57816        3843  AE Aur   050941.7+341152051618.2+341843172.08-02.26 5.96  +0.22 -0.70         O9.5V             e+0.008+0.030 -.004+059V       5  3.3   8.4AB   3*
1713 19Bet OriBD-08 1063  34085131907 194I   3823   1882    050943.9-081901051432.3-081206209.24-25.25 0.12  -0.03 -0.66 -0.02   B8Ia:             e 0.000-0.001 +.013+021SB     33  6.5   9.5AB   4*
1714          BD+85   78  34109   873                       050952.0+853516054348.7+854005127.54 25.79 6.60R                     A2V                -0.016+0.002      -014
1715          CD-35 2176  34167195669               1877    051013.4-355624051346.5-354932239.43-34.41 6.98  +1.47               K4III              +0.036-0.051      -007
1716   Xi  MenCP-82  106  34172258395 917                   051014.9-823615045850.9-822814295.23-30.47 5.85  +0.93 +0.64         G8-K0III           -0.011+0.004      -005
1717          BD-01  837  34180131917                       051014.7-013127051518.4-012433202.76-21.93 6.15  +0.39 -0.03         F0IV               -0.043+0.037      +014V      50:
1718 18    OriBD+11  756  34203 944262395                   051030.7+111344051604.1+112029191.30-15.25 5.56  -0.04 -0.05         A0V                -0.009-0.005      -008       67
1719 15    CamBD+57  874  34233 25125                       051050.1+580035051927.8+580702152.71 11.75 6.13  -0.03 -0.47         B5V                +0.004-0.021      +007SB?                       *
1720          BD+62  742  34255 134602397I                  051102.3+623249052022.6+623913148.85 14.29 5.61  +1.75 +2.00         K4I:               -0.003-0.002      -006V
1721          CD-36 2127  34266195683                       051056.5-360529051428.8-355838239.65-34.30 5.76  +1.01               G8III              +0.016+0.004      +013V
1722          BD+42 1239  34269 40214    I         PU Aur   051107.1+424101051815.9+424732165.38  2.95 5.48  +1.65 +1.62         M4III              +0.038-0.023 +.007-038                          *
1723          CD-27 2161  34310170238                       051123.6-270319051524.3-265636229.22-31.97 5.07  -0.10               B9V                +0.010-0.018      +029      111
1724          BD+01  957  34317112588                       051129.4+015012051641.1+015650199.81-19.99 6.42  -0.02 +0.01 -0.03   A0V               e-0.011-0.009      -008V?     65                 *
1725          BD+40 1240  34332 40224                       051141.3+402126051840.4+402754167.33  1.68 6.18  +1.37               K0                 +0.017-0.015      -017
1726 16    AurBD+33 1000  34334 57853    I   3872   1909    051136.8+331602051810.7+332218173.07-02.48 4.54  +1.27 +1.27 +0.71   K2.5IIIbFe-1       +0.045-0.160 +.014-028SB1O < 17  5.8   4.2      *
1727          CP-52  677  34347233886                       051133.9-520840051353.3-520152259.20-35.84 6.05  +1.39 +1.60         K3III              -0.004-0.024      +034
1728 17    AurBD+33 1002  34364 57858              AR Aur   051143.9+333934051818.9+334602172.77-02.23 6.14  -0.06 -0.18         B9.5V              +0.012-0.030      +025SBO    58                 *
1729 15Lam AurBD+39 1248  34411 402331145    3886           051206.3+400037051908.5+400557167.67  1.54 4.71  +0.63 +0.12 +0.32   G1.5IV-VFe-1       +0.519-0.664 +.066+066        2  4.9 146.6AD   6*
1730          CD-35 2199  34435195694                       051211.5-350219051547.0-345536238.46-33.84 6.66  +0.15 +0.16 +0.08   A2III-IV           -0.018+0.026      +020
1731          BD-17 1069  34447150295                       051221.4-171507051648.1-170830218.64-28.40 6.56  -0.16               B3IV               -0.012+0.022      +012
1732          BD+33 1008  34452 57884              IQ Aur   051225.1+333832051900.0+334454172.86-02.13 5.41  -0.19 -0.55         A0pSi             v+0.011-0.025      +029SB?    53                 *
1733          BD+44 1170  34498 40242                       051246.5+441914052002.3+442532164.22  4.15 6.62R                     K0                 +0.004-0.010      +013
1734 18    AurBD+33 1010  34499 57893        3893           051247.8+335248051923.6+335908172.72-01.92 6.49  +0.24 +0.10         A7V                +0.009-0.010      +007      120  5.3   3.8
1735 20Tau OriBD-07 1028  34503131952 195    3877           051245.0-065709051736.4-065040208.28-23.96 3.60  -0.11 -0.47 -0.11   B5III              -0.015-0.008 +.006+020SB     46  7.2  36.0AD   4
1736          BD+46  998  34533 40251        3903           051311.5+465136052039.3+465750162.18  5.68 6.54  +0.60 +0.36         GIII+A2V           +0.002-0.002      +017           2.9  23.2      *
1737          BD-13 1116  34538150304               1913    051304.6-133735051740.2-133111214.97-26.80 5.50  +0.93 +0.62       sgG9                 -0.012-0.046      +075
1738          BD+40 1253  34557 40248                       051313.2+405900052014.8+410510166.99  2.28 5.52  +0.11 +0.05         A3V                -0.002-0.058 +.011+022V?    200
1739109    TauBD+21  816  34559 770972398I                  051316.0+215935051916.6+220547182.54-08.70 4.94  +0.93 +0.69         G8III              +0.012-0.083 +.010+019     < 17
1740 19    AurBD+33 1013  34578 579062399           1925    051325.3+335113052000.9+335729172.81-01.83 5.03  +0.27 +0.42         A5II               +0.001+0.002      -005       16
1741          BD+19  893  34579 77098        3894           051319.6+200147051914.7+200805184.18-09.80 6.08  +1.01 +0.82         G8II-III+G1IV-V    -0.039-0.027 +.015-047           3.5   9.0      *
1742          CP-52  683  34587233895                       051320.4-521733051538.9-521055259.39-35.57 6.49  +1.21               K2III              +0.022+0.027      +013
1743   Omi ColCD-35 2214  34642195721 197                   051352.6-345935051729.1-345343238.51-33.49 4.83  +1.00 +0.80 +0.55   K0IV               +0.090-0.337 +.018+021V?
1744   The DorCP-67  401  34649249225 196                   051349.9-671752051345.4-671107277.64-34.26 4.83  +1.28 +1.39 +0.63   K2.5IIIa           +0.019+0.036 -.006+011V
1745          BD+77  195  34653  55352404                   051401.7+775307052925.7+775839135.11 22.40 6.56  +0.14 +0.16         A7III              +0.011-0.017      -016V      67
1746 21    OriBD+02  916  34658112624               1922    051358.2+022932051911.2+023545199.54-19.12 5.34  +0.41 +0.09         F5II              v-0.017-0.047 +.022+011V      72                 *
1747          BD-18 1051  34721150326        3899           051423.5-181412051850.4-180748219.88-28.33 5.96  +0.58 +0.08         G0V                +0.381+0.058 +.065+040SB         7.8  46.0AB   3*
1748          BD-01  859  34748131983                       051431.3-013057051935.2-012444203.32-21.00 6.34  -0.11 -0.75 -0.14   B1.5Vn             -0.007-0.006      +019SB    280                 *
1749 20Rho AurBD+41 1162  34759 402692400                   051443.6+414217052148.4+414816166.56  2.93 5.23  -0.15 -0.57         B3V                +0.018-0.037      +013SBO   102                 *
1750          BD+27  758  34762 77121                       051442.5+275121052059.3+275726177.88-05.08 6.33  +0.04 -0.26         B9IV               -0.013-0.016      +007SBO   125  2.0   0.0      *
1751 16    CamBD+57  879  34787 251612402  W                051454.2+572650052327.8+573240153.49 11.88 5.28  -0.03 -0.07         A0Vn               +0.013-0.055 +.023+012SB    268  7.7 106.2
1752          BD+29  869  34790 77124                       051450.9+292806052112.7+293412176.57-04.12 5.76R +0.06 +0.13         A1V s              +0.004+0.004      -019SB2O   53  0.4   0.0      *
1753          BD-18 1055  34798150335        3910A          051454.2-183726051917.5-183112220.34-28.36 6.36  -0.16 -0.59         B3V                +0.012+0.010      +010       16  0.2  39.3      *
1754          BD-18 1056  34797150336        3910B TX Lep   051455.0-183648051918.3-183035220.33-28.36 6.54  -0.11 -0.44         B8/9III/IV         +0.005-0.002      +015       72  0.2  39.3      *
1755          BD+19  902  34810 94478                       051502.1+194247052056.6+194851184.68-09.64 6.18  +1.23               K0III              -0.006-0.015      +000
1756  6Lam LepBD-13 1127  348161503401146                   051458.0-131648051934.5-131036214.83-26.24 4.29  -0.26 -1.03 -0.28   B0.5IV             -0.001-0.003 +.000+020       67                 *
1757  7Nu  LepBD-12 1132  34863150345                       051520.5-122505051959.1-121856214.00-25.79 5.30  -0.12 -0.40         B7IVnn             -0.006+0.009      +016      370
1758          CD-27 2204  34868170311 198                   051524.5-272818051923.7-272208229.99-31.24 5.99  -0.04               A0V                 0.000-0.010      +018       69
1759          BD-05 1225  34880132004        3926           051531.5-052808052026.4-052200207.19-22.66 6.39  -0.03 -0.36         B8III              -0.011+0.029      +020       50  4.7   4.4      *
1760          BD+40 1268  34904 40290                       051548.8+405551052250.3+410146167.31  2.65 5.54  +0.12 +0.13         A3V                -0.013+0.005      -009V     150
1761          BD+03  857  34959112660                       051602.9+035445052119.3+040043198.53-17.95 6.57  -0.09 -0.49 -0.04   B5Vp              e-0.003-0.009      +005      300                 *
1762          BD-21 1135  34968170327        3930   1940    051610.7-212025052026.9-211422223.35-29.07 4.71  -0.05 -0.11 -0.05   A0V               v+0.005 0.000 +.002+030V      51  3.8   4.2
1763          BD+08  933  349891126672401                   051616.8+081946052143.6+082543194.62-15.61 5.80  -0.13 -0.88 -0.13   B1V                +0.003-0.006      +026SB     57                 *
1764          BD-00  929  35007132024        3941           051625.6-003057052131.8-002459202.64-20.09 5.68  -0.12 -0.65 -0.15   B3V                -0.003+0.002      +007V      40  5.9  37.6AC   3*
1765 22    OriBD-00  930  350391320281147                   051639.4-002852052145.7-002257202.63-20.03 4.73  -0.17 -0.79 -0.18   B2IV-V              0.000-0.001      +029SB1O   14                 *
1766          CD-34 2198  35046195763                       051644.5-344757052020.6-344156238.43-32.88 6.34  +0.33 +0.14         F2IV-V             -0.007-0.004      -010
1767   Zet PicCD-50 1723  35072233926 199                   051654.8-504248051922.1-503622257.47-34.98 5.45  +0.51 +0.01         F7III-IV           +0.028+0.224 +.023+045
1768 22    AurBD+28  788  35076 77139                       051702.8+285029052322.9+285612177.36-04.09 6.46  -0.04 -0.19         B9V s              +0.019-0.030      +009V      70
1769          BD-13 1135  35104150375                       051716.0-135113052151.0-134522215.67-25.96 6.56  -0.09 -0.48         B8V                +0.008-0.007      +014V
1770 23    OriBD+03  871  35149112697        3962A          051734.6+032654052250.0+033240199.16-17.86 5.00  -0.15 -0.86 -0.18   B1V                -0.004-0.002      +018      295  2.2  31.9      *
1771          CD-24 3023  35162170351    I   3954A          051739.9-245212052146.2-244623227.30-29.95 5.06  +0.67 +0.16         G7II-III+A7IV-V    -0.024-0.013 +.004+005           1.5   3.1AB   3*
1772          CD-34 2207  35165195770      W       Var?     051739.5-342636052116.9-342043238.07-32.61 6.09  -0.19 -0.70 -0.14   B5IVnpe            +0.009+0.001      +020V?         4.8   2.2      *
1773 21Sig AurBD+37 1175  35186 57981    I   3984           051751.3+371731052439.2+372308170.52  0.88 4.99  +1.42 +1.76 +0.53E  K4III              +0.001-0.008 +.003-019V    < 19: 7.0   8.7AB   4*
1774110    TauBD+16  765  35189 94514                       051751.1+163618052337.7+164158187.68-10.82 6.08  +0.14 +0.19         A2IV               -0.029-0.017      +021SB
1775          BD+31  954  35238 57987                       051811.6+310751052438.3+311326175.62-02.58 6.28  +1.24               K1III              -0.043-0.009      +040SB
1776          BD+31  955  35239 57988                       051811.8+310300052438.5+310835175.69-02.63 5.94  +0.04 -0.13         B9III              -0.011-0.007      +005V     285
1777          BD+05  905  35242112708                       051811.7+051341052331.1+051921197.64-16.82 6.35  +0.12 +0.09         A2V                -0.024-0.003      +009                          *
1778          BD-08 1107  35281132053        3978           051830.7-083036052318.5-082457210.50-23.39 5.9   -0.03 -0.36         B8III              +0.003-0.016D+.013+015V     119  1.7   6.0      *
1779          BD+34 1031  35295 57999        4000A          051834.0+344551052513.0+345119172.68-00.44 6.55  +1.11 +1.16         K1pIII-IV+F6V      -0.007-0.039      -015           1.8  31.2      *
1780111    TauBD+17  920  35296 94526      W                051835.2+171726052425.4+172300187.20-10.29 4.99  +0.53 -0.07         F8V                +0.246-0.009 +.067+037SB     16  2.9  85.7      *
1781          BD-00  936  35299132057                       051835.4-001514052342.3-000935202.68-19.49 5.70  -0.21 -0.87 -0.19   B1.5V              -0.009+0.003      +022V?     36                 *
1782          BD-01  882  35317132060        3991           051846.1-005738052351.4-005202203.35-19.80 6.11  +0.50 +0.01         F7V                -0.004-0.009D+.021+045SB2        0.0   2.7AxBC 3*
1783  8    LepBD-14 1119  35337150396                       051855.6-140116052330.2-135538216.02-25.67 5.25  -0.21 -0.87?        B2IV               +0.002+0.004      +018V      23
1784 29    OriBD-07 1064  35369132067    I                  051907.7-075359052356.8-074829209.98-22.98 4.14  +0.96 +0.69 +0.50   G8IIIFe-0.5        -0.015-0.043 +.011-018V?   < 17
1785          CD-26 2185  35386170375                       051910.8-264800052312.0-264221229.54-30.23 6.49  +0.50               F6V                +0.029+0.009      +051V
1786          BD+02  947  35407112729                       051923.4+021540052436.2+022110200.48-18.07 6.32  -0.15 -0.62 -0.16   B4IVn              +0.011-0.007      -008SB    398                 *
1787 27    OriBD-01  886  35410132070    I                  051923.8-005914052428.9-005329203.46-19.68 5.08  +0.96 +0.69         G9III-IVFe-1       -0.005+0.132 +.021+020V?   < 19:
1788 28Eta OriBD-02 1235  35411132071        4002  Eta Ori  051926.9-022921052428.6-022349204.87-20.39 3.36  -0.17 -0.92 -0.23   B1V+B2e            +0.002 0.000 +.007+020SB2O   46  1.4   1.5AB   5*
1789 25Psi1OriBD+01 1005  354391127342406          V1086 Ori051933.3+014517052444.8+015047200.96-18.29 4.95  -0.20 -0.92 -0.22   B1Vpe              +0.001+0.001      +019V     316                 *
1790 24Gam OriBD+06  919  35468112740 201I W        1972    051946.0+061533052507.9+062059196.93-15.95 1.64  -0.22 -0.87 -0.22   B2III              -0.009-0.014 +.029+018SB?    59 10.5 179.0      *
1791112Bet TauBD+28  795  35497 77168 202I W                051958.1+283123052617.5+283627177.99-03.74 1.65  -0.13 -0.49 -0.10   B7III              +0.022-0.175 +.028+009V      71       33.4      *
1792          BD-17 1117  35505150416                       052001.3-170400052428.4-165834219.25-26.64 5.65   0.00               A1V                +0.015-0.027      +022       89
1793          CD-39 1940  355151958072405          SW Col   052005.9-394616052324.0-394043244.40-33.22 5.71  +1.62               M1III              +0.028+0.003      +061                          *
1794          BD+35 1102  35519 580292408I                  052013.2+352210052654.3+352726172.38  0.18 6.15  +1.45 +1.68         K2                 -0.018-0.011      -021                          *
1795          BD+34 1040  35520 58028                       052011.3+341814052648.9+342330173.25-00.43 5.94  +0.14 +0.21         A0III              +0.007-0.011      +008       95                 *
1796          BD+33 1045  35521 58030                       052017.4+331028052651.3+331546174.19-01.05 6.15  +1.15               K0                 +0.013 0.000      -009
1797          CD-37 2176  35528195809                       052012.1-372544052339.0-372012241.66-32.75 6.82  +1.04               K1III              +0.034+0.008      +030
1798113    TauBD+16  775  35532 94543                       052018.9+163641052605.7+164201188.00-10.32 6.25  -0.08 -0.63         B2Vn               -0.009-0.003      +031
1799          BD-10 1178  35536150420    I                  052018.5-102508052501.6-101945212.58-23.84 5.61  +1.56 +1.79         K5III              -0.017-0.022      +057
1800          BD-00  945  35548132086        4020           052025.1-003800052531.2-003239203.26-19.28 6.57  -0.05 -0.18         B9pHgSi            +0.012-0.007D+.002-009V?      5  0.6   0.2      *
1801   Kap PicCP-56  840  355802339522403                   052031.5-561342052222.1-560804264.19-34.51 6.11  -0.10               B8-9V              -0.005+0.020      -005V?
1802 17    CamBD+62  759  35583 13518 203I          2003    052043.3+625901053010.2+630402149.08 15.46 5.42  +1.71 +2.00 +0.94E  M1IIIa             -0.006-0.005      -019V?
1803          BD+00 1056  35588112752               1975    052038.5+002552052547.0+003115202.31-18.71 6.16  -0.18 -0.75 -0.20   B2.5V              -0.005+0.019      -024SBO   150                 *
1804          BD+30  898  35600 58040                       052044.2+300718052708.3+301231176.76-02.70 5.74  +0.16 -0.18         B9Ib               +0.008-0.010      +017                          *
1805 24Phi AurBD+34 1048  35620 58051    I W                052101.0+342327052738.9+342833173.28-00.24 5.07  +1.40 +1.67 +0.47E  K3IIICN+2          -0.003-0.047 +.009+031V    < 17  3.2 206.8AD   4*
1806          BD-05 1247  35640132100                       052107.8-053624052602.4-053106208.02-21.49 6.23  -0.06 -0.23 -0.05   B9.5Vn             +0.013+0.008      +018V     280                 *
1807          BD+06  923  35656112767                       052115.6+064656052638.8+065209196.66-15.36 6.42  -0.02 -0.04         A0Vn               +0.003-0.012      +013V?    118
1808115    TauBD+17  928  35671 945541148    4038           052120.0+175235052710.1+175744187.07-09.42 5.42  -0.10 -0.53         B5V                +0.005-0.022      +018V?    155  1.2   0.1 O   4*
1809          BD+15  822  35693 94556                       052130.7+151018052713.8+151528189.39-10.86 6.16  +0.08 +0.14         A1IV               -0.014-0.002      +025       68
1810114    TauBD+21  847  35708 77184        4048           052137.6+215106052738.1+215613183.75-07.17 4.88  -0.15 -0.76 -0.15   B2.5IV             +0.002-0.007      +013V?     24  0.0   0.1 O   5*
1811 30Psi2OriBD+02  962  35715112775        4039  Psi Ori  052135.8+030033052650.2+030544200.09-17.22 4.59  -0.21 -0.93 -0.22   B2IV               +0.002-0.004 -.016+012SB2O  141  5.6   2.7AB   3*
1812          BD-19 1173  35736150442        4034A          052139.8-194657052559.8-194144222.23-27.31 5.89  +0.40 -0.05         F2V                +0.004-0.022      +006V          2.8  27.0      *
1813          CD-44 2036  357652173252407                   052156.8-441852052455.6-441333249.84-33.58 6.08  +1.20               K1III              +0.003+0.005      +015
1814116    TauBD+15  826  35770 94566                       052200.8+154723052745.6+155227188.93-10.42 5.50  +0.01 -0.05         B9.5Vn             +0.006-0.022      +016V     260
1815          CP-81  134  357982584053976                   052206.7-813847051225.7-813230294.02-30.39 6.51  +1.11 +1.01         K1III              +0.011+0.050      +019SB2
1816117    TauBD+17  931  35802 94573    I                  052213.2+170921052801.6+171420187.79-09.64 5.77  +1.63 +2.01         M1III              +0.020-0.049      -023V?
1817          BD-12 1169  358501504612409                   052225.2-115906052704.8-115403214.37-24.04 6.35  +0.51 +0.01        dF7                 +0.019-0.049      +019V?
1818   The PicCP-52  718  35860233965      W                052230.1-522413052446.2-521859259.56-34.18 6.27  +0.07 +0.07         A0V                -0.012-0.031D+.004-003SB2        0.5  38.2AC   3*
1819          BD+13  903  35909 94580                       052255.3+133547052834.8+134044190.93-11.42 6.35  +0.15 +0.14         A4V                +0.014-0.018      +025V?    153
1820          BD+01 1021  35912112794                       052251.2+011252052801.6+011754201.88-17.84 6.41  -0.18 -0.74 -0.18   B2V                +0.018+0.003      +034       32                 *
1821118    TauBD+25  839  35943 77201        4068A  2009    052307.1+250410052916.5+250902181.25-05.10 5.47  -0.04 -0.14         B8.5V              +0.010-0.032D+.014+015V?    133  0.8   4.8AB   3*
1822          BD+29  909  35984 77205                       052319.3+290624052940.6+291111177.92-02.81 6.24  +0.45 +0.01         F6III              +0.022-0.054      +013V      39
1823          BD-21 1174  35991170445                       052320.8-212737052736.5-212232224.15-27.56 6.07  +1.04 +0.86 +0.35E gG7                 +0.013+0.036      +034                          *
1824          BD+41 1206  36040 403872412                   052344.6+412302053048.6+412743167.79  4.13 6.00  +1.11 +1.12         K0IIIp             +0.002-0.035      +014                          *
1825          BD+39 1322  36041 58129      W                052347.4+394452053045.1+394933169.15  3.23 6.37  +0.97 +0.76         G9III              +0.021-0.041      +012           1.3  75.4      *
1826          BD-03 1115  36058132157        4078           052357.3-032318052856.7-031827206.28-19.83 6.39  -0.01 -0.06         A0Vn               -0.035-0.015D+.003+005      142  0.3   0.8      *
1827          CD-41 1884  360602173401149  W                052352.7-410148052705.3-405637246.03-32.73 5.87  +0.23 +0.18 +0.12   A7m                +0.011+0.096      +036           8.7  20.3
1828 18    CamBD+57  889  36066 252411150           2058    052359.8+570902053233.8+571316154.43 12.76 6.48  +0.57 +0.12         F8V                +0.113-0.222 +.012+037V?  =< 10
1829  9Bet LepBD-20 1096  36079170457 204I   4066   2008    052357.6-205021052814.7-204534223.56-27.20 2.84  +0.82 +0.46 +0.44   G5II               -0.004-0.089 +.020-014V?     11  4.5   2.6AB   5*
1830          BD-03 1116  36134132170               2022    052424.5-033134052923.6-032647206.46-19.80 5.79  +1.15 +1.02 +0.57   K1-III             -0.044-0.008      +023V
1831          BD+22  925  36160 77220                       052441.1+222305053043.4+222745183.70-06.29 6.29  +1.18 +1.15         K0                 +0.043-0.016      +002
1832          BD+15  837  36162 94596                       052442.7+151659053026.1+152137189.72-10.15 5.94  +0.14 +0.14         A3Vn               -0.027-0.050      -016V?    230
1833          BD+01 1032  36166112830                       052443.3+014237052954.8+014721201.67-17.19 5.78  -0.20 -0.84 -0.20   B2V                +0.008-0.004      +012V     175                 *
1834 31    OriBD-01  913  36167132176    I   4097  CI Ori   052439.2-011015052944.0-010532204.31-18.62 4.71  +1.57 +1.85 +0.69E  K5III              +0.004-0.023 +.006+008SB   < 19: 5.4  12.5      *
1835          CD-37 2220  36187195887                       052448.5-371850052815.3-371350241.75-31.83 5.57  +0.02 +0.03         A1V                +0.009+0.074      +050
1836   Lam DorCP-59  472  361892339812410                   052451.5-585948052619.3-585445267.52-33.87 5.14  +1.00       +0.48   G6III              -0.008+0.028 +.012+010
1837          BD+04  949  36217112837              CK Ori   052502.8+040738053019.8+041215199.53-15.91 6.21  +1.27 +1.38 +0.47E  K2IIIe            v+0.002-0.048      +015                          *
1838          CD-30 2421  36255195898                       052516.2-301147052906.7-300700233.75-29.96 6.75  +1.06               G8/K0III           -0.003+0.014      +059V
1839 32    OriBD+05  939  36267112849        4115           052525.9+055219053047.1+055653198.02-14.94 4.20  -0.14 -0.55 -0.14   B5V                +0.012-0.034 +.005+019V?    176  1.3   0.9      *
1840          BD-07 1099  36285132192                       052530.7-073045053020.7-072605210.38-21.39 6.33  -0.19 -0.83 -0.17   B2IV-V             -0.013+0.001      +011       10                 *
1841 NOVA 1891                                     T Aur                                                                                                                                            *
1842 33    OriBD+03  948  36351112861        4123           052559.6+031258053114.5+031732200.48-16.17 5.46  -0.18 -0.83 -0.20   B1IV+B1.5V         +0.001+0.003D+.001+020V?     38  1.0   1.9AB   3*
1843 25Chi AurBD+32 1024  36371 581641151                   052613.0+320705053243.7+321131175.77-00.61 4.76  +0.34 -0.46 +0.27   B5Iab              -0.002-0.003      -000SBO    41                 *
1844          BD+74  252  36384  5593 205I          2337    052620.9+745840053943.7+750238138.26 21.71 6.17  +1.57 +1.95         M0III              -0.008+0.025      -003V?
1845119    TauBD+18  875  36389 94628    I         CE Tau   052620.9+183112053212.8+183540187.18-08.07 4.38  +2.07 +2.21 +1.44   M2Iab-Ib          e+0.002-0.001 +.002+023V?                        *
1846          BD+41 1218  36404 40426                       052621.7+420210053328.7+420632167.51  4.90 6.55  -0.02 -0.27         B9IIIp:Hg:         -0.001-0.001      +001
1847          BD+16  794  36408 94630        4131A          052626.3+165903053214.2+170329188.50-08.88 5.46R -0.04 -0.23         B7IIIe             -0.002-0.009D+.009+015SB     55  0.4   9.6      *
1848          BD-06 1207  36430132210                       052629.2-064701053120.9-064230209.80-20.85 6.22  -0.17 -0.74 -0.17   B2V                +0.010 0.000      +023       25                 *
1849 10    LepBD-20 1105  36473170506                       052650.8-205615053107.6-205149223.94-26.61 5.55   0.00 +0.03         A0V                +0.005-0.044 +.016-011       60
1850          BD+32 1028  36484 58179                       052654.8+324350053327.5+324804175.34-00.15 6.48  +0.09 +0.17  0.00   A2m                +0.008-0.063      +034SB
1851 34Del OriBD-00  982  36485132221        4134C          052653.8-002130053200.5-001704203.84-17.73 6.85  -0.16 -0.71         B2V                +0.009-0.004 +.014+021V      77  4.6  51.7AC   3*
1852 34Del OriBD-00  983  36486132220 206I   4134A Del Ori  052653.8-002223053200.4-001757203.86-17.74 2.23  -0.22 -1.05 -0.22   O9.5II             +0.001-0.002 +.014+016SBO   152  4.6  51.7AC   3*
1853          BD+66  401  36496 13548      W                052701.6+663746053716.2+664146146.14 17.87 6.26                      A8Vn               -0.010-0.031      -015V?    209  0.8   0.3
1854          BD+34 1083  36499 58182                       052659.2+343915053338.0+344333173.74  0.93 6.27  +0.15 +0.17         A3IV               -0.038-0.008      -006SB    100
1855 36Ups OriBD-07 1106  36512132222                       052705.5-072231053155.8-071805210.44-20.98 4.62  -0.26 -1.07 -0.26   B0V                -0.001-0.005      +017V      20                 *
1856          CD-47 1884  365532173681152  W                052724.5-470858053009.5-470440253.35-32.98 5.46  +0.62 +0.21         G3IV               +0.025-0.137      +016           1.0 197.1AD   4
1857 19    CamBD+64  536  36570 13550        4177           052734.2+640523053715.1+640917148.49 16.68 6.15  +0.01 -0.07         A0V                +0.003-0.070 +.007-012       55  4.0   1.6      *
1858120    TauBD+18  877  36576 94649              V960 Tau 052739.9+182808053331.6+183225187.39-07.84 5.69  +0.01 -0.76         B2IV-Ve            -0.002 0.000      +041V     271                 *
1859          CP-68  375  36584249281      W                052731.3-684206052700.0-683722279.07-32.78 6.03  +0.34 +0.03 +0.21C  F0IV-V             +0.001-0.019D+.012-002           0.2   1.1
1860          BD+20  989  36589 77255               2132    052742.1+202412053338.8+202827185.76-06.78 6.18  -0.07 -0.39         B6V                -0.004-0.012      +027V      90
1861          BD-01  935  36591132234        4141   2107    052737.7-013951053241.3-013531205.14-18.20 5.35  -0.19 -0.94 -0.19   B1IV               -0.001-0.009      +034       29  4.5   2.0      *
1862   Eps ColCD-35 2348  365971959242413I                  052739.7-353237053112.7-352814239.89-30.88 3.87  +1.14 +1.08 +0.60   K1IIIa             +0.022-0.035 +.008-005V
1863          BD-01  939  36646132247        4150   2131    052804.0-014719053307.2-014306205.31-18.17 6.46  -0.09 -0.60 -0.12   B4Vn               -0.025-0.040D+.002+037      196  1.8   1.5AB   4*
1864 35    OriBD+14  947  36653 946522414                   052813.2+141408053354.3+141820191.08-09.99 5.64  -0.14 -0.62         B3V                -0.001-0.009      +019V     224
1865 11Alp LepBD-17 1166  36673150547 207I   4146   2128    052819.1-175338053243.8-174920220.95-25.14 2.58  +0.21 +0.23 +0.21   F0Ib               +0.001+0.002 +.007+024       13  8.5  35.8AB   3*
1866          BD+54  914  36678 252762416I                  052822.6+542144053635.2+542544157.19 11.82 5.73  +1.63 +1.98         M0III              -0.013-0.002      +001
1867          CP-62  479  36689249288                       052822.6-622321052917.4-621852271.57-33.29 6.59  +1.53 +1.85         K4-5III            +0.010-0.023      +014V
1868          BD-01  943  36695132255              VV Ori   052826.8-011335053331.4-010922204.84-17.81 5.34  -0.18 -0.90 -0.20   B1V                -0.004 0.000      +022SB1O  168                 *
1869          BD+47 1178  36719 404662417                   052842.7+473857053615.9+474255163.00  8.30 6.11  +0.26 +0.15        dF0                 +0.012-0.018      +014V      67
1870          CD-46 1892  36734217382                       052845.5-455955053136.1-455531252.02-32.61 5.86  +1.35 +1.54         K3III              +0.028+0.038      +007
1871          BD+01 1058  36741112901                       052847.0+012018053357.6+012428202.53-16.49 6.59  -0.17 -0.79 -0.21   B2V                +0.009+0.004      +014      198                 *
1872 38    OriBD+03  964  36777112904      W                052900.9+034155053416.7+034601200.43-15.27 5.36  +0.05 +0.07 +0.04   A2V                -0.031-0.018 +.014-004V     145        0.4      *
1873          BD-01  949  36779132269        4159           052858.9-010617053403.9-010208204.79-17.64 6.22  -0.17 -0.81 -0.18   B2.5V              +0.007+0.001      +004      227  4.8  27.5      *
1874          BD-01  950  36780132270    I                  052900.2-013219053404.0-012814205.20-17.84 5.93  +1.55 +1.88         K5III              -0.012-0.033      +091
1875121    TauBD+23  954  36819 772852415          Var?     052920.6+235824053527.1+240222182.95-04.52 5.38  -0.09 -0.62         B2.5IV             +0.010-0.024      +023SB    117                 *
1876 37Phi1OriBD+09  877  36822112914 208  S                052919.8+092519053449.2+092922195.40-12.29 4.41  -0.16 -0.97 -0.17   B0III              +0.002-0.004 +.000+033SBO    39                 *
1877          CD-38 2085  36848195948                       052929.3-383459053251.3-383047243.44-31.20 5.48  +1.22 +1.34         K2III              +0.040-0.002 -.008-001
1878          BD+27  806  36859 77295    I                  052938.8+273550053555.5+273944179.95-02.48 6.27  +1.54 +1.65         K0                 -0.025-0.039      -009SB
1879 39Lam OriBD+09  879  36861112921    I   4179A  2240    052937.7+095202053508.3+095603195.05-12.00 3.54  -0.18 -1.03 -0.17   O8III((f))          0.000-0.006 +.007+034       66  2.0   4.4AB   4*
1880 39Lam OriBD+09  879  36862              4179B  2240    052937.9+095205053508.5+095606195.05-12.00 5.61  +0.04 -0.77         B0.5V              +0.001-0.006 +.007+036V?    125  2.0   4.4AB   4*
1881          CD-35 2367  36874195952               2158    052932.7-351229053307.4-350823239.62-30.42 5.78  +1.07 +1.00         K0III              +0.097-0.041      +015V?
1882          CP-64  452  36876249297      W                052939.7-640008053015.9-635540273.48-33.04 6.19  +0.22 +0.16         F0IV               +0.009+0.054      +011           5.1   9.2
1883          BD+10  818  36881 94671        4181   2251    052942.0+101025053513.4+101424194.79-11.82 5.60  +0.12 -0.07         B9IIIpHg:Mn:       +0.022-0.008      +035V      15  4.2   2.9      *
1884          BD+40 1346  36891 40481                       052953.1+400704053652.4+401056169.49  4.41 6.09  +1.03 +0.69         G3Ib+F             -0.002-0.010      -018SB?
1885          BD+85   80  36905   9141638                   052954.5+850850060120.2+851056128.17 25.99 6.11  +1.57 +1.90         M0III              +0.011 0.000      -046V
1886          BD-06 1233  36959132298        4182B  2261    053007.7-060433053500.9-060033209.57-19.72 5.67  -0.23 -0.91 -0.21   B1V               v-0.011 0.000 +.005+030V?     26  0.9  36.2      *
1887          BD-06 1234  36960132301        4182A          053009.3-060407053502.7-060007209.56-19.71 4.78  -0.25 -1.02 -0.25   B0.5V              +0.004+0.002 +.005+028V?     38  0.9  36.2      *
1888          CD-29 2348  36965170561                       053000.8-295502053352.1-295056233.78-28.90 6.53  -0.02               A0V                +0.014+0.021      +015
1889          BD+25  879  36994 77310                       053018.2+255230053630.2+255622181.47-03.30 6.49  +0.43               F5III              +0.014+0.003      +003       76
1890          BD-04 1183  37017132317              V1046 Ori053025.0-043336053521.8-042936208.18-18.96 6.56  -0.13 -0.77 -0.13   B1.5V              -0.011+0.030      +029SB1O   85                 *
1891          BD-04 1184  37016132319      W                053025.3-042924053522.5-042531208.11-18.92 6.24  -0.15 -0.70 -0.17   B2.5V              +0.022-0.027      +031           0.7   0.2      *
1892 42    OriBD-04 1185  37018132320    I   4187   2318    053027.1-045414053523.2-045018208.50-19.11 4.59  -0.19 -0.94 -0.18   B1V                +0.004 0.000 +.006+030SB     71  3.2   1.5      *
1893 41The1OriBD-05 1315  37020              4186A V1016 Ori053021.0-052712053515.9-052314209.01-19.38 6.73  +0.02 -0.88 +0.21   B0.5V              +0.005+0.002 +.001+032SBO   112  0.1 117.0 *  12*
1894 41The1OriBD-05 1315  37021              4186B BM Ori   053021.3-052704053516.1-052307209.01-19.38 7.96  +0.24 -0.49         B0V                +0.005+0.002 +.001+024SBO   222  0.1 117.0 *  12*
1895 41The1OriBD-05 1315  37022132314    I   4186C  2294    053021.7-052720053516.5-052323209.01-19.38 5.13  +0.02 -0.95 +0.18   O6p               v+0.004+0.002 +.001+033SB?   127  0.1 117.0 *  12*
1896 41The1OriBD-05 1315  37023              4186D  2295    053022.5-052714053517.3-052316209.01-19.38 6.70  +0.09 -0.71 +0.20   B0.5Vp             +0.005+0.002 +.001+031SB    109  0.1 117.0 *  12*
1897 43The2OriBD-05 1319  37041132321    I:  4188A  2320    053028.2-052854053522.9-052458209.05-19.37 5.08  -0.09 -0.94         O9.5Vp            e+0.001+0.005 -.001+036SBO   170  0.1 117.0 *  12*
1898          BD-04 1186  37040132325        4192           053033.9-042546053531.1-042153208.07-18.86 6.38  -0.14 -0.74 -0.16   B2.5IV             +0.002-0.021      +030      140  2.0   4.2      *
1899 44Iot OriBD-06 1241  37043132323 209I   4193   2334    053032.4-055832053526.0-055436209.52-19.58 2.77  -0.24 -1.08 -0.20   O9III             e 0.000+0.001 +.025+022SB2O  130  4.5  11.4AB   3*
1900          BD-03 1146  37055132332      W        2350    053036.1-031905053535.8-031510207.04-18.33 6.40  -0.12 -0.63 -0.13   B3IV               -0.008 0.000      +024           6.2   1.1      *
1901 45    OriBD-04 1188  37077132336        4196           053043.5-045517053539.5-045121208.55-19.06 5.26  +0.24 +0.15 +0.14   F0III              +0.009+0.011 +.026-009V      66  9.0  19.1AC   3*
1902          BD+26  870  37098 77322        4208   2426    053054.0+265143053708.8+265527180.72-02.65 5.83  -0.07 -0.39         B9IV-V             +0.011-0.025D+.007+010V          0.1   1.1AB   4*
1903 46Eps OriBD-01  969  37128132346 210I W       Eps Ori  053108.3-011557053612.8-011207205.21-17.24 1.70  -0.19 -1.04 -0.17   B0Ia              e+0.001-0.002 -.002+026SB     87  8.7 179.9      *
1904          BD+33 1102  37138 58263                       053110.3+332950053745.7+333333175.19  1.02 6.33  +1.31               K0                 +0.003 0.000      +029
1905122    TauBD+16  822  37147 94700                       053115.4+165843053703.8+170225189.12-07.91 5.54  +0.22 +0.10 +0.12   F0V                +0.045-0.031 +.029+041      114                 *
1906          BD-05 1334  37150132351               2416    053120.9-054241053615.0-053853209.37-19.28 6.54  -0.18 -0.81         B3V               v 0.000-0.004      +011                          *
1907 40Phi2OriBD+09  898  37160112958    I                  053124.7+091411053654.3+091726195.84-11.95 4.09  +0.95 +0.64 +0.55   K0IIIbFe-2         +0.091-0.305 +.030+099                          *
1908          BD+10  828  37171 947022420I                  053130.8+105823053704.4+110206194.33-11.02 5.94  +1.58 +1.91 +0.94   K4III              +0.051-0.018      -111SB
1909          CD-33 2414  37192195986                       053134.4-330851053515.4-330448237.44-29.49 5.78  +1.12               K2IIIa             +0.020+0.109      +044
1910123Zet TauBD+21  908  37202 77336 211I D       Zet Tau  053140.0+210454053738.7+210833185.69-05.64 3.00  -0.19 -0.67 -0.10   B4IIIpe           v 0.000-0.021 +.008+020SB1O  310  2.0   0.0      *
1911          BD-06 1255  37209132359        4212   2433    053142.5-060739053635.7-060354209.81-19.39 5.72  -0.22 -0.91 -0.25   B1V                -0.005-0.012      +028SB     50  3.2   5.2      *
1912          CP-54  854  372262340262419  W                053145.3-545807053344.4-545408262.70-32.91 6.43  +0.55               F8V                +0.072+0.011      +038           3.6  54.7AB   3*
1913          BD+08 1016  37232112966                       053151.1+085325053719.3+085707196.20-12.03 6.12  -0.17 -0.83         B2IV-V             +0.001-0.001      +042      120                 *
1914 26    AurBD+30  963  37269 58280        4229   2485    053212.7+302559053838.1+302933177.88-00.47 5.40  +0.45 +0.25         F9III+B9.5V        -0.019-0.009D+.005+002SB         0.3   0.2AB   4*
1915          CD-28 2298  37286170587                       053215.6-284614053610.3-284228232.71-28.09 6.26  +0.13 +0.05 +0.05   A2/3III/V:         +0.023 0.000      +022
1916          BD+65  485  37289 135702426I                  053225.0+653837054226.4+654152147.32 17.88 5.60  +1.24              gK5                 -0.006-0.018      -019V
1917          CP-64  456  372972493092418                   053226.8-641737053259.6-641339273.80-32.72 5.34  +1.04 +0.85         G8-K0III           +0.041-0.010 +.010+010SBO                       *
1918          BD-06 1262  37303132375               2467    053233.8-055956053727.4-055618209.79-19.15 6.05  -0.22 -0.93 -0.25   B1V               v+0.015+0.002      +029      260                 *
1919          BD-11 1238  373061506172421                   053229.0-115010053708.8-114632215.37-21.75 6.11  +0.05               A2V                +0.020-0.020      +024V?
1920          BD+07  953  37320112979                       053236.2+072855053801.1+073229197.53-12.59 5.88  -0.07 -0.37         B8III              +0.007-0.017      +019V?     35
1921          BD+26  884  37329 77350      D                053243.3+263335053857.4+263704181.19-02.47 6.37R                     G9III              +0.024-0.018      +015                          *
1922   Bet DorCP-62  487  37350249311 212          Bet Dor  053245.4-623319053337.5-622923271.73-32.77 3.76  +0.82 +0.55 +0.43   F6Ia              v+0.002+0.009 +.012+007V       0                 *
1923          BD-04 1196  37356132387               2483    053257.3-045225053753.4-044849208.78-18.54 6.19  -0.04 -0.72 -0.05   B2IV-V             +0.004+0.021      +029       10                 *
1924          BD+29  947  37367 77354                       053256.7+290927053918.4+291255179.04-01.03 5.96  +0.16 -0.50         B2IV-V             +0.012-0.004      +020SB         2.0   0.0      *
1925          BD+53  934  37394 25319      W                053313.4+532626054120.3+532852158.38 11.95 6.23  +0.84 +0.51 +0.43   K1V                +0.002-0.518 +.081+002V          2.8 688.0AD   5*
1926   Nu 1ColCD-27 2389  37430170601                       053319.4-275547053716.5-275217231.89-27.60 6.16  +0.34               F0IV               +0.014-0.061      +027
1927          CD-47 1940  37434217422              TX Pic   053319.7-472230053602.7-471850253.76-32.01 6.11  +1.16               K2III+F            -0.014-0.017      +016SB2
1928125    TauBD+25  902  37438 773602424  D                053332.3+255028053944.2+255349181.90-02.71 5.18  -0.15 -0.68         B3IV               +0.012-0.024      +015SBO    65  2.0   0.0     3*
1929          BD+21  918  37439 77358                       053326.7+214224053927.0+214546185.38-04.95 6.34R +0.06 +0.10         A1Vn               -0.004-0.036      +015SB2   180
1930          CP-58  526  37462234037                       053335.6-585603053502.4-585216267.43-32.74 6.75  +1.45 +1.80         K4III              -0.035+0.017      +032
1931 48Sig OriBD-02 1326  37468132406        4241AB         053343.5-023928053844.8-023600206.82-17.34 3.81  -0.24 -1.01 -0.24   O9.5V              +0.002+0.001 +.007+029SB     94  0.8   0.2AB   5*
1932          BD-02 1327  37479132408        4241E V1030 Ori053345.9-023907053847.1-023539206.82-17.32 6.65  -0.19 -0.86         B2Vp              v-0.018+0.012 +.007+029           0.8   0.2AB   5*
1933          BD-06 1275  37481132405                       053346.0-063754053837.8-063426210.53-19.17 5.96  -0.23 -0.92         B1.5IV             -0.015+0.003      +015V     105                 *
1934 47Ome OriBD+04 1002  374901130012423          Ome Ori  053354.3+040352053911.1+040717200.73-14.03 4.57  -0.11 -0.76 -0.10   B3IIIe            v+0.001-0.003      +022      194                 *
1935   Nu 2ColCD-28 2321  37495170613                       053350.3-284458053744.6-284122232.81-27.75 5.31  +0.46 +0.10         F4V                -0.034+0.052 +.047+036V      31
1936          CP-61  488  37501249314                       053351.8-611415053457.6-611033270.16-32.69 6.32  +0.85               G5IV               +0.006-0.022      +008
1937 49    OriBD-07 1142  37507132411                       053402.7-071607053853.1-071247211.16-19.40 4.80  +0.13 +0.11 +0.06   A4V                -0.012-0.053 +.042-001SBO   170                 *
1938          BD+31 1048  37519 583192425           2537    053407.6+311813054035.9+312129177.37  0.35 6.04  +0.04 -0.21 +0.05   B9.5III-IVp:       -0.007-0.020      -007      250
1939          BD+31 1049  37536 58322    I         NO Aur   053411.9+315157054042.1+315515176.90  0.67 6.11  +2.12 +2.22 +1.42   M2IIIS             +0.010+0.006      +005                          *
1940          BD-03 1166  37594132424                       053432.2-033714053931.2-033353207.81-17.61 6.00  +0.27 +0.01         A8V s              -0.002-0.003      +022
1941 24    CamBD+56 1050  37601 25333                       053432.7+563145054301.6+563454155.74 13.68 6.05  +0.95 +0.73         K0III              +0.014+0.031      -029V?
1942          BD-09 1197  37635132425                       053446.2-094543053930.8-094224213.62-20.35 6.50  -0.10 -0.48         B7V                +0.008-0.014      +021                          *
1943 23    CamBD+61  816  37638 13590                       053456.6+612537054408.6+612836151.34 16.12 6.15  +0.90              gG5                 -0.005+0.001      -004V?
1944          BD-17 1199  37643150652        4254           053451.8-175419053916.3-175058221.62-23.71 6.38  -0.12 -0.40         B7V                +0.001+0.003      +021V          1.5   0.6AB   9*
1945          BD+29  953  37646 77393        4262A          053458.4+292607054121.0+292915179.04-00.50 6.43  -0.11 -0.39         B8IV               +0.019-0.029      +018SB    200  0.4  26.1      *
1946126    TauBD+16  841  37711 94759        4265   2560    053530.9+162856054117.7+163202190.09-07.31 4.86  -0.13 -0.63 -0.12   B3IV               +0.008-0.018 +.014+021SB?    97  0.4   0.4      *
1947          CD-40 1999  37717217450                       053530.8-404548053843.6-404227246.21-30.52 5.82  -0.09               B8V                -0.005+0.009      +004
1948 50Zet OriBD-02 1338  37742132444    I   4263A  2553    053542.7-015943054045.5-015634206.45-16.59 2.05H -0.21 -1.07 -0.20   O9.7Ib            e+0.003-0.002 +.024+018SB    140  2.2   2.4AB   3*
1949 50Zet OriBD-02 1338  37743          I   4263B  2553    053542.7-015943054045.6-015634206.45-16.59 4.21H                     B0III              +0.004-0.002 +.024+013SB         2.2   2.4AB   3*
1950          BD-02 1337  37744132441                       053536.5-025242054037.3-024930207.25-17.03 6.22  -0.20 -0.88         B1.5V              +0.008+0.010      +029       35                 *
1951          BD+23 1007  37752 77413                       053550.0+231631054154.6+231935184.35-03.65 6.59  -0.06 -0.53         B8p                -0.016-0.014      +026                          *
1952          BD-01 1004  37756132445               2556    053546.0-011053054050.6-010744205.71-16.19 4.95  -0.21 -0.84         B2IV-V             -0.012-0.006      +026SBO    77                 *
1953   Gam MenCP-76  333  37763256201 214  W                053550.3-762445053152.9-762028287.93-30.97 5.19  +1.13 +1.19 +0.55   K2III              +0.111+0.283 +.020+057V          6.0  38.2      *
1954          BD+22  996  37784 77420                       053601.1+223636054203.9+223937184.93-03.96 6.36R                     K2                 -0.011-0.021      -021
1955          BD+00 1152  37788113033                       053557.4+001705054105.6+002016204.40-15.44 5.93  +0.30 +0.04         F0IV               +0.015+0.028      -013V      52                 *
1956   Alp ColCD-34 2375  37795196059 215  W        2549    053601.6-340738053938.9-340427238.81-28.86 2.64  -0.12 -0.46 -0.09   B7IVe             v+0.007-0.026 +.001+035V?    176  9.7  13.5      *
1957          BD-10 1258  37808150680              V1051 Ori053603.3-102740054046.0-102434214.44-20.37 6.52  -0.16 -0.48         B9.5IIIpSi4200     -0.030-0.030      +020                          *
1958          CD-32 2479  37811196061                       053607.8-324054053949.8-323745237.23-28.44 5.45  +0.92       +0.47   G6-8III            -0.022-0.029      -008
1959          BD-02 1346  37904132465        4279           053639.4-025651054140.3-025346207.44-16.83 6.42  +0.30 +0.07         A9IV-V             +0.041+0.030      +030SB2        0.9   0.1      *
1960          CP-66  439  37935249322                       053654.8-663700053654.7-663337276.51-32.10 6.31  -0.07 -0.15         B9.5V             e-0.022+0.010      +020                          *
1961          BD+23 1015  37967 77450              V731 Tau 053715.1+230925054319.5+231215184.62-03.43 6.21  -0.06 -0.64         B2.5Ve             -0.001-0.022      +016                          *
1962          BD-16 1208  37971150703                       053714.2-164633054141.5-164332220.74-22.74 6.21  -0.13               B3IVp              -0.005+0.009      +016                          *
1963 51    OriBD+01 1105  379841130562427I                  053718.3+012534054228.6+012829203.54-14.59 4.91  +1.17 +1.06 +0.61   K1III              -0.058-0.015 +.015+088V?   < 17                 *
1964          CP-73  316  379932562082422          WX Men   053714.1-734802053444.7-734429284.90-31.30 5.78  +1.71 +1.83 +1.53   M3III              -0.003+0.033      -011                          *
1965          BD-17 1214  38054150716                       053749.3-173444054214.3-173149221.60-22.94 6.15  +1.38               K0                 -0.036+0.007      +062V
1966          CD-33 2483  38056196088                       053747.3-332700054127.0-332402238.18-28.32 6.34  -0.04               A0V                +0.014-0.002      +029
1967          BD-06 1293  38089132477        4299           053802.4-065043054253.8-064746211.24-18.32 6.02  +0.40 -0.05         F3V                -0.003+0.068D+.019-007SB         0.4   0.2AB   4*
1968 12    LepBD-22 1194  38090170715                       053801.5-222520054213.9-222225226.52-24.73 5.87  +0.11               A2                 -0.004+0.018      +028                          *
1969 26    CamBD+56 1058  38091 25362               2615    053804.6+560427054630.4+560656156.39 13.88 5.94  +0.17 +0.12         A4Vn               +0.019-0.056      +021SB    260
1970          BD-01 1012  38099132480    I         V1197 Ori053805.7-013934054309.3-013647206.44-15.90 6.31  +1.47 +1.77         K4III              +0.003-0.026      +030SBO                       *
1971 27Omi AurBD+49 1398  38104 40583 216                   053809.1+494658054554.0+494935161.98 10.75 5.47  +0.03 +0.07 +0.01   A2VpCr             -0.010+0.001 +.019-006V      30                 *
1972          CD-30 2571  381381960962428                   053822.8-303501054211.6-303208235.12-27.37 6.19  +0.01               A0V                +0.002+0.007      +027
1973          CD-34 2401  38170196098                       053840.0-344300054215.1-344004239.62-28.49 5.29  -0.05               B9.5V              -0.008+0.051      +035       41
1974          BD+40 1403  38189 40584                       053848.6+402752054549.5+403026170.11  6.03 6.58  +0.25 +0.02 +0.15   A3Del Del          -0.031-0.005      -004       67
1975          BD-18 1172  38206150739                       053858.9-183610054321.6-183327222.74-23.08 5.73  -0.01 -0.11         A0V s              +0.016-0.015      +025    =< 41
1976          BD+62  784  38284 13618        4376           053938.3+624614054904.9+624829150.37 17.24 6.13R                     A4V                -0.007-0.010D+.010-004V     125  0.9   0.9
1977          BD+20 1083  38307 77516    I         Y Tau    053941.7+203911054539.4+204142187.05-04.28 6.95  +3.03 +5.81 +1.52   C5II               -0.001 0.000      +017                          *
1978          BD+03 1025  38309113099        4333           053945.1+035757054501.9+040029201.57-12.81 6.09  +0.31 +0.09        gF0n                +0.020-0.019      +008       82  4.2  15.7AB   3*
1979          BD+42 1396  38358 406092431                   054005.1+422922054714.7+423136168.49  7.29 6.29  +1.35 +1.59         K0                 +0.009-0.088      -016
1980          BD-20 1171  38382170756                       054010.0-201014054428.4-200735224.43-23.43 6.34  +0.58               G0                 -0.016+0.046      +038V?
1981          CD-39 2140  383851961162430                   054011.9-392704054330.2-392425244.95-29.36 6.25  +0.37               F3V                +0.051-0.002      +013
1982          BD-22 1210  38392170757        4334B          054016.4-222717054426.5-222518226.76-24.26 6.15  +0.94 +0.74 +0.35   K2V                -0.309-0.353 +.128-010           2.5  96.5AB   3*
1983 13Gam LepBD-22 1211  38393170759 217I   4334A          054017.6-222851054427.8-222654226.79-24.27 3.60  +0.47  0.00 +0.26   F6V                -0.293-0.370 +.128-010       11  2.5  96.5AB   3*
1984          CD-45 2131  38458217497               2613    054050.8-455243054341.1-454959252.25-30.51 6.39  +0.29 +0.13         A9V                +0.029+0.091      +011                          *
1985129    TauBD+15  926  38478 94848                       054100.4+154700054645.5+154921191.39-06.55 6.00  -0.06 -0.44         B8IIIpHgMn:        +0.004-0.004      +019       80
1986          BD-04 1235  38495132515        4361           054105.4-041823054602.8-041606209.24-16.48 6.34  +1.05 +0.99         K1III+G0IV          0.000-0.068      -046           2.7   6.9      *
1987          BD+09  954  385271131242432    4369           054122.7+092908054652.1+093120196.90-09.71 5.79  +0.88 +0.58         G8III              -0.034-0.065      -026           5.9  17.2      *
1988          BD+01 1126  38529113119                       054125.4+010800054635.0+011005204.32-13.83 5.95  +0.78 +0.42         G4V                -0.073-0.148 +.027+029V                         *
1989131    TauBD+14 1025  38545 94855                       054131.3+142706054713.2+142918192.60-07.14 5.72  +0.04 +0.16         A3Vn               +0.010-0.039 +.021+021V
1990130    TauBD+17 1004  38558 94858 218                   054136.3+174130054726.2+174345189.82-05.44 5.49  +0.30 +0.27         F0III              -0.005-0.006      +009       51:
1991   Iot MenCP-78  195  38602256214              Iot Men  054143.6-785223053536.3-784915290.68-30.22 6.05  -0.02 -0.33         B8III              +0.008+0.017      +013V?                        *
1992 29    CamBD+56 1065  38618 25403        4412           054202.1+565309055034.0+565508155.93 14.75 6.54                      A4IV-V             -0.009-0.008      +004V?         4.2  25.1AB   3*
1993133    TauBD+13  979  38622 948642433    4381           054202.6+135148054742.9+135359193.17-07.33 5.29  -0.17 -0.66         B2IV-V             +0.002-0.012 -.002+029       70  6.1  24.9AC   3*
1994          BD+68  412  38645 136352438                   054210.0+682633055255.5+682817145.13 19.98 6.20  +0.95               G9III              +0.005-0.051 +.004-001
1995 29Tau AurBD+39 1418  38656 58465    I   4398           054214.7+390850054910.5+391052171.58  5.91 4.52  +0.94 +0.70 +0.49   G8IIIFe-1          -0.025-0.022 +.007-020     < 17  6.1  49.6AC   3*
1996   Mu  ColCD-32 2538  38666196149               2630    054216.9-322040054559.9-321823237.29-27.10 5.17  -0.28 -1.06 -0.27   O9.5V              +0.009-0.025      +109      153                 *
1997          BD+20 1105  38670 77578        4392           054224.1+205004054822.4+205210187.23-03.64 6.07  -0.08 -0.38         B9Vn               +0.014-0.016D+.001+007SB    175  1.4   0.5AB   3*
1998 14Zet LepBD-14 1232  38678150801 219                   054225.4-145133054657.3-144919219.40-20.83 3.55  +0.10 +0.07 +0.03   A3Vn               -0.016-0.001 +.049+020SB?   202
1999 52    OriBD+06 1027  38710113150        4390           054237.8+062508054800.2+062715199.75-10.98 5.27  +0.23 +0.17         A5V                +0.007-0.019 +.023+043SB2   110  0.1   1.4      *
2000          BD-16 1244  38713150806                       054239.4-161628054707.6-161416220.81-21.35 6.17  +0.89               G2Ib-II            -0.034+0.003      +007V
2001          BD-10 1281  38735150814      S       V1031 Ori054244.4-103407054726.7-103159215.30-18.94 6.03  +0.16               A4V                -0.029-0.024      -001SB3                       *
2002132    TauBD+24  970  38751 775922435I D        2650    054252.7+243202054901.0+243403184.13-01.61 4.86  +1.01 +0.81 +0.38E  G8III             v-0.004-0.023 +.005+016V?   < 19  0.0   0.1
2003          BD+51 1117  38765 254112437                   054259.5+512902055056.4+513053160.87 12.26 6.29  +1.05               K1III              +0.167-0.030      +026
2004 53Kap OriBD-09 1235  38771132542 220I          2641    054300.8-094218054745.4-094011214.51-18.50 2.06  -0.17 -1.03 -0.18   B0.5Ia            v+0.002-0.002 +.015+021V?     82                 *
2005          CD-28 2449  38804170836                       054310.1-284031054704.7-283821233.45-25.79 6.22  -0.15               B5V                +0.006+0.006      +018
2006 30    CamBD+58  863  38831 25419                       054328.3+585607055217.4+585751154.15 15.88 6.14  -0.04 -0.13         A0V s              +0.007-0.023      +012      400:
2007          BD-04 1244  388581325541155                   054336.7-040720054834.9-040541209.37-15.84 5.97  +0.64 +0.10         G4V                +0.062-0.225      +029V
2008          CD-46 1999  388712175212434  W                054341.0-463802054627.4-463550253.21-30.16 5.31  +1.04               K0-1II             +0.012+0.004 +.018+011           7.4  37.0
2009          CD-35 2509  38885196171                       054347.1-354240054718.6-354029241.02-27.75 6.32  +1.18               K1III              -0.008+0.041      +057
2010134    TauBD+12  912  38899 94888      W                054355.8+123711054932.9+123904194.49-07.57 4.91  -0.07 -0.17 -0.08   B9IV               -0.020-0.024      +018V      22  5.4  18.9
2011 31Ups AurBD+37 1336  38944 584962440I          2661    054413.1+371636055102.4+371820173.39  5.28 4.74  +1.62 +1.93 +1.07   M0+III-IIIbFe-0.5  +0.035-0.047 -.002+038                          *
2012 32Nu  AurBD+39 1429  39003 58502 221I   4440           054433.5+390709055129.4+390855171.84  6.28 3.97  +1.13 +1.09 +0.56   G9.5III*           -0.005+0.007 +.017+010V?   < 17  7.3  54.6      *
2013          BD+27  888  39004 77625        4474           054439.9+275617055058.1+275804181.44  0.51 5.56  +0.97              gG7                 -0.009+0.012      +008V          1.3   0.0 O   3*
2014          BD+09  978  39007113179                       054432.0+095026055002.6+095216196.98-08.85 5.80  +0.87 +0.62 +0.30E  G8III              +0.005-0.010      +044V?                        *
2015   Del DorCP-65  496  390142493461154                   054435.6-654623054446.4-654408275.47-31.37 4.35  +0.21 +0.12 +0.17   A7V                -0.031+0.008 +.027-003      206
2016135    TauBD+14 1041  39019 94904                       054447.5+141636055028.9+141820193.16-06.55 5.52  +1.01              gG9                 +0.006-0.035      +046V?
2017          CD-40 2085  39040217543                       054445.3-404111054758.1-403909246.55-28.79 6.61  +1.12               K1III              +0.007+0.032      +013SB2
2018          BD+32 1109  39045 58504    I   4443  Var      054454.6+320546055125.7+320730177.91  2.72 6.25  +1.75 +2.00 +1.40   M3III             v+0.001+0.007      +103           5.9  15.2      *
2019          BD+04 1052  390511131862439I                  054455.4+042339055013.1+042524201.83-11.48 5.97  +1.36 +1.57        gK2                 +0.005-0.039      +027V?
2020   Bet PicCD-51 1620  39060234134                       054454.9-510609054717.1-510359258.37-30.61 3.85  +0.17 +0.10 +0.16   A5V                +0.011+0.085 +.061+020V?    139                 *
2021          BD-14 1251  39070150845        4432           054503.8-143047054936.5-142901219.35-20.10 5.49  +0.88              gG6                 -0.025-0.044D+.009-002V          3.0   2.6      *
2022   Pi  MenCP-80  161  39091258421                       054506.8-803240053709.8-802809292.53-29.78 5.65  +0.60 +0.11 +0.19E  G1V                +0.305+1.054 +.044+009V?                        *
2023          CP-54  892  39110234137                       054512.0-542339054713.0-542139262.18-30.93 6.18  +1.40               K3III              -0.016-0.009      -003
2024          BD+01 1148  39118113198              Var      054518.0+015944055030.0+020128204.02-12.57 5.98  +0.91 +0.30         G8III+A0IV         +0.001-0.015      +007SBO =< 50                 *
2025          BD+39 1435  39182 58524                       054542.0+393255055239.5+393428171.58  6.69 6.45R +0.09 +0.17         A2V                -0.026-0.016      -019
2026          CD-23 3135  39190170892                       054542.9-230007054953.5-225818227.82-23.29 5.87  +0.06               A2                 -0.014+0.025      +044
2027 31    CamBD+59  920  39220 254472446          TU Cam   054600.3+595157055457.8+595318153.44 16.59 5.20  +0.02 +0.03         A2V                +0.001-0.019 +.009-003SB2O   76                 *
2028          BD+33 1179  39225 585282443I          2681    054602.8+335329055240.1+335503176.50  3.85 5.98  +1.60 +1.97         M1.5II-III         +0.020+0.008      +100                          *
2029 30Xi  AurBD+55 1027  39283 254501157                   054627.9+554101055450.8+554225157.33 14.72 4.99  +0.05 +0.12 +0.02   A2V                -0.013+0.020 +.012-012V?     72                 *
2030          BD+19 1110  39286 94942                       054627.9+195032055223.4+195205188.57-03.34 6.06  +0.54 -0.01         G2IIIe+B8III       -0.002+0.003      -003V?    230                 *
2031 55    OriBD-07 1187  392911325912442           2677    054632.3-073241055122.0-073105212.89-16.76 5.35  -0.20 -0.83         B2IV-V             -0.001+0.003      +020V     164                 *
2032          CD-44 2274  39312217559                       054639.7-445416054934.1-445231251.36-29.34 6.38  +1.27               K1III              -0.004+0.011      +029
2033137    TauBD+14 1060  39317 94945              V809 Tau 054641.4+140847055222.3+141018193.51-06.22 5.59  -0.05 -0.06         B9pSiCrEu          -0.020-0.005      -004V      35                 *
2034136    TauBD+27  899  39357 776751158  D        2696    054702.4+273519055319.6+273644182.01  0.77 4.58  -0.02 +0.03  0.00   A0V                +0.003-0.012 +.021-016SB2O   41  1.5   0.0      *
2035 15Del LepBD-20 1211  39364170926 222I                  054701.2-205315055119.3-205245225.82-22.22 3.81  +0.99 +0.68 +0.56   K0IIIFe-1.5CH0.5   +0.227-0.649 +.022+099        0                 *
2036          BD-22 1246  39385170931                       054717.8-225705055128.6-225534227.91-22.93 6.17  +1.02               K0                 -0.001-0.018      +026
2037 56    OriBD+01 1151  394001132202444I   4467   2690    054714.8+014951055226.4+015118204.41-12.22 4.78  +1.38 +1.47         K1.5IIb            -0.007-0.012 +.005+010     < 19: 8.4  43.4
2038          BD+20 1156  39417 77680                       054722.3+201633055319.1+201757188.31-02.94 6.71  -0.07 -0.33         B9V                +0.005-0.010      -006      175
2039          BD-09 1255  39421132603                       054721.7-090404055207.6-090229214.41-17.25 5.97  +0.10 +0.08         A2Vnp              -0.027+0.052      +039                          *
2040   Bet ColCD-35 2546  39425196240 223I:                 054726.0-354821055057.6-354606241.35-27.06 3.12  +1.16 +1.21 +0.58   K2III              +0.059+0.401 +.023+089V                         *
2041          BD+66  413  39429 136622448                   054725.7+660442055735.0+660546147.63 19.44 6.25  +1.36 +1.64         K0                 +0.048-0.016      -022
2042   Gam PicCP-56  946  395232341541156                   054800.6-561130054949.7-561000264.31-30.69 4.51  +1.10 +0.98 +0.54   K1III              +0.083-0.076 +.018+016                          *
2043          CD-29 2556  39543170945                       054807.6-292824055159.5-292655234.68-25.02 6.45  +1.48 +1.72         K3III              -0.004+0.020      +002
2044          CP-52  791  39547234162      W                054817.3-524738055028.6-524604260.39-30.30 6.35  +0.76               K0-1III+A4V+A      -0.025+0.015      +001           0.8   0.2
2045          BD+51 1128  39551 25467                       054817.5+514707055614.4+514814161.00 13.13 6.49R                     A5V                -0.009-0.008      -012
2046          BD+31 1139  39586 58569                       054829.5+314111055459.0+314206178.66  3.15 5.90  +0.14?+0.13?        A5IV               -0.043-0.176 +.023-021      105
2047 54Chi1OriBD+20 1162  39587 77705    I D                054827.6+201528055422.9+201634188.46-02.73 4.41  +0.59 +0.07 +0.31   G0V                -0.189-0.084 +.104-014SB?     6        0.6     3*
2048          BD+10  927  39632 94977    I                  054841.0+103357055413.4+103513196.87-07.61 6.12  +1.47 +1.43         G9II               +0.006+0.008      +013
2049          CP-52  794  396402341691159                   054837.4-520754055053.2-520632259.64-30.17 5.17  +0.99 +0.72 +0.42   G8III              +0.006-0.078 +.023+001
2050          BD+11  964  39662 94979      W                054857.0+114440055432.2+114545195.88-06.96 6.59  +0.03 +0.02         A2V                +0.008-0.068      +028           5.5  22.7      *
2051          BD+03 1071  39685113253                       054900.5+031224055415.7+031331203.40-11.17 6.31  +1.29 +1.47         K0                 +0.053-0.061      -004
2052 57    OriBD+19 1126  39698 949862447           2722    054901.4+194349055456.7+194459188.98-02.88 5.92  -0.17 -0.74         B2V                -0.003-0.004      +007SBO   121  2.2   0.0      *
2053          CD-37 2457  39720196266      W                054908.5-373909055233.2-373752243.46-27.23 5.63  +1.04 +0.87         K1III              +0.033-0.031      +032           6.1  16.5AB   3*
2054          BD+49 1423  39743 40720                       054923.5+490049055704.9+490146163.58 11.96 6.47  +0.99               G8III              +0.019-0.016      -002
2055          CD-38 2270  39752196274      W                054926.7-383250055247.7-383133244.45-27.40 6.70  +1.08               K1III              +0.004-0.004D+.003+004           3.5   1.2
2056   Lam ColCD-33 2599  39764196276              Lam Col  054929.0-334924055306.9-334805239.36-26.09 4.87  -0.15 -0.57 -0.15   B5V                -0.004+0.033 +.035+030V      88                 *
2057          BD+00 1208  39775113265    I                  054934.4+005658055444.1+005806205.49-12.14 6.00  +1.34 +1.45         K0III              +0.003-0.003      +022
2058          BD-04 1281  39777132635                       054936.8-040502055434.7-040350210.05-14.50 6.57  -0.18 -0.81         B1.5V              -0.001+0.020      +025       20                 *
2059          CP-84   75  397802584181659          TZ Men   054933.7-845007053013.9-844706297.35-28.83 6.20  -0.02 -0.11         B9.5IV-V           -0.008+0.046      -004SBO                       *
2060          BD-19 1293  39789150925                       054937.9-193930055357.4-193818224.85-21.18 6.69  +0.10               A4V                -0.025+0.004      +002
2061 58Alp OriBD+07 1055  39801113271 224I   4506  Alp Ori  054945.4+072319055510.3+072425199.79-08.96 0.50  +1.85 +2.06 +1.28   M1-2Ia-Iab        e+0.026+0.009 +.005+021SB         9.9 174.4AE   6*
2062   Lam MenCP-72  418  398102562392441                   054950.0-724349054748.1-724208283.55-30.52 6.53  +1.08 +0.97         K0III              -0.011+0.022      +015
2063          BD+20 1171a 39816 77730    I         U Ori    054952.8+200929055549.3+201030188.72-02.49 5.4   +0.99 +0.43         M6.5IIIe            0.000-0.014      -021V?                        *
2064   Eps DorCP-66  463  39844249368               2693    054959.8-665534054953.6-665404276.80-30.80 5.11  -0.14 -0.49         B6V                -0.016+0.016 +.012+016
2065          BD-11 1321  39853150932    I                  055003.6-114737055443.6-114626217.29-17.85 5.66  +1.53 +1.84         K5III              +0.066+0.039      +086V?                        *
2066          BD+28  952  39866 77744                       055012.6+285535055633.8+285632181.22  2.06 6.32  +0.30 +0.26         A2II               +0.003-0.001      +019
2067          BD+13 1036  39881 95004        4519           055020.4+135518055603.5+135531194.16-05.57 6.60  +0.65 +0.13         G5IV               +0.384-0.467 +.047-002V          1.9  47.5AB   4*
2068          CD-29 2595  39891170993      W                055021.4-290954055414.1-290852234.53-24.46 6.36  +0.37               F3V                -0.020-0.051      +018           0.0   0.1
2069          CD-42 2205  39901217599                       055019.7-425629055322.9-425517249.32-28.29 6.55  +1.37               K3III              +0.012+0.009      -010
2070          BD-04 1289  39910132652                       055033.4-043759055530.3-043659210.67-14.54 5.87  +1.18 +1.21        gK2                 +0.040-0.017      +026V?
2071          BD-04 1291  39927132653                       055039.3-044816055535.4-044718210.84-14.60 6.28  +0.06 +0.07         A2III              -0.011-0.019      -014
2072          CP-57  901  39937234181               2714    055039.6-571029055220.3-570922265.48-30.41 5.94  +0.66               F7IV               +0.024-0.078      +008V
2073          CP-64  486  39963249376                       055050.2-640320055123.0-640201273.46-30.72 6.36  +0.86 +0.57         G8III              -0.011+0.016      +013                          *
2074          BD+24 1033  39970 77750                       055048.6+241406055656.1+241459185.32-00.22 6.02  +0.39 -0.19         A0Ia               +0.002 0.000      +001V      60                 *
2075          BD+09 1016  39985113284                       055058.3+092940055628.0+093035198.09-07.65 5.99  -0.07 -0.15         A0IV                0.000-0.001      +013       83
2076          BD+11  975  40020 950272453                   055114.1+113029055649.5+113116196.37-06.59 5.87  +1.10               K2III              +0.106-0.058      +021
2077 33Del AurBD+54  970  40035 25502 225I W                055117.5+541637055931.6+541705158.95 14.69 3.72  +1.00 +0.87 +0.50   K0-III             +0.081-0.125 +.022+008     < 17  5.8 115.4AB   4
2078          BD+75  247  40055  57302459                   055121.8+753455060509.3+753509138.34 23.40 6.40R                     K5                 +0.009-0.010      +004
2079          BD+55 1036  40062 25504                       055125.4+551846055945.7+551915158.00 15.18 6.44  +0.31 +0.12 +0.13   A5m                -0.015-0.088      +045
2080          BD+54  971  40083 25505                       055133.0+543216055948.1+543250158.73 14.84 6.14  +1.20 +1.33         K2III              +0.011-0.035      -006
2081          BD+49 1428  40084 40743                       055135.7+495449055921.8+495528162.95 12.70 5.89  +1.23               G5III              -0.002-0.009      -004SBO                       *
2082          CD-39 2260  400911963092449                   055137.2-395829055452.5-395729246.12-27.36 5.57  +1.51 +1.85         M0III              -0.014+0.016      +114V
2083          CD-50 1977  40105234191                       055143.8-502342055410.7-502143257.72-29.44 6.52  +0.90 +0.63 +0.30E  G8IV               +0.086+0.569 +.034+002
2084139    TauBD+25 1052  40111 77775                       055147.3+255630055759.7+255714183.97  0.84 4.82  -0.06 -0.93 -0.11   B0.5II              0.000-0.002      +008SB    131                 *
2085 16Eta LepBD-14 1286  40136150957 226I                  055151.0-141109055624.3-141004219.76-18.47 3.71  +0.33 +0.01 +0.16   F1III              -0.042+0.139 +.066-002        0
2086          BD-22 1269  40151171034                       055202.3-225118055614.3-225025228.25-21.88 5.96  +1.11 +0.99?        K0V                +0.119+0.021      +034V
2087   Xi  ColCD-37 2487  40176196316                       055203.4-370807055529.9-370715243.07-26.53 4.97  +1.11               K1III              +0.032-0.026 -.011+060V
2088 34Bet AurBD+44 1328  40183 40750 227I   4556  Bet Aur  055211.5+445615055931.7+445651167.46 10.41 1.90  +0.03 +0.05 -0.01   A2IV               -0.057 0.000 +.041-018SB2O   37  8.9 184.6AB   3*
2089          CD-49 1945  402002176122450                   055211.0-493836055441.1-493737256.89-29.25 6.10  -0.13               B3V                -0.009+0.016      +012
2090          CD-23 3263  40235171045                       055224.3-231346055634.5-231256228.66-21.94 6.36  +1.07               K0                 +0.014+0.026      -005V
2091 35Pi  AurBD+45 1217  40239 40756    I         Pi  Aur  055230.7+455540055956.1+455613166.60 10.94 4.26  +1.72 +1.83 +1.48   M3II              e 0.000-0.005 +.003+001V                         *
2092   Sig ColCD-31 2848  402481963302454                   055235.1-312346055620.9-312257237.03-24.73 5.50                      F2III              -0.002+0.006      +019       79
2093          BD+01 1168  40282113306    I                  055244.4+011246055754.2+011328205.64-11.31 6.22  +1.48 +1.84         M0III              -0.069+0.008      +037
2094          CP-52  805  40292234199                       055237.9-523927055450.2-523807260.32-29.63 5.29  +0.31  0.00?        F0VDel Del         -0.005+0.248 +.034+027        0                 *
2095 37The AurBD+37 1380  40312 58636    I   4566  The Aur  055254.1+371221055943.3+371245174.34  6.73 2.62  -0.08 -0.18 -0.06   A0pSi              +0.048-0.081 +.022+030SB     49  4.5   3.5AB   4*
2096          BD+44 1332  40325 40769        4576           055300.1+443506060019.0+443531167.84 10.37 6.22                      K2III+K0III        -0.026-0.041      +002           4.0  34.2AC   3*
2097          BD-01 1078  40347132700                       055306.6-010014055811.7-005939207.67-12.29 6.22  +1.14 +1.20         K0                 +0.007-0.025      +061
2098          CD-31 2854  40359196335                       055305.0-315920055649.0-315834237.68-24.82 6.44  +1.07               G8III              +0.002+0.019      +029
2099          BD+12  968  40369 95075        4562           055315.6+124755055853.2+124831195.49-05.52 5.70  +0.89 +0.63?        K2III+A5V          -0.014+0.021D+.006+012V          1.0   0.4      *
2100 59    OriBD+01 1171  40372113315        4555  V1004 Ori055312.7+014938055824.4+015013205.14-10.92 5.90  +0.21 +0.17         A5mDel Del         +0.005-0.008      +045SBO    60  4.7  36.7      *
2101 36    AurBD+47 1227  40394 40778                       055323.3+475344060058.6+475407164.91 12.01 5.73  -0.01 -0.12         B9.5pSiFe          +0.006-0.018      +016
2102          CP-63  498  404092493902452                   055320.0-630712055406.1-630524272.38-30.42 4.65  +1.05 +0.96 +0.50   K1III-IV           +0.138+0.537 +.048+025                          *
2103 60    OriBD+00 1239  404461133211161  W                055341.0+003238055849.6+003311206.35-11.43 5.22  +0.01 +0.01         A1V s              -0.013+0.002 +.004+035SB     47  6.6  19.1
2104          CP-64  495  40455249391                       055344.6-642955055411.9-642856273.98-30.42 6.63  +0.38 +0.10         F0III              -0.003+0.064      +013
2105          BD+48 1333  40486 407892458I                  055402.2+485716060143.1+485734164.01 12.60 5.96  +1.45               K0                 +0.005-0.015      +011
2106   Gam ColCD-35 2612  404941963521160  W                055359.4-351738055732.2-351700241.23-25.64 4.36  -0.18 -0.66 -0.17   B2.5IV              0.000+0.009      +024       96  8.3  33.8
2107  1    MonBD-09 1284  40535132714              V474 Mon 055415.7-092327055901.0-092256215.50-15.87 6.12  +0.29 +0.10         F2IV               +0.014+0.012      +012SB     25                 *
2108  2    MonBD-09 1285  405361327152455                   055419.4-093354055904.3-093330215.67-15.93 5.03  +0.19 +0.16 +0.09   A6IIIm vs          +0.009-0.050 +.010+022SBO    23                 *
2109          BD-01 1083  40574132723                       055433.7-012704055937.7-012640208.25-12.18 6.63  -0.07 -0.39         B8IIIn             -0.009-0.020      -004
2110          BD+31 1164  40588 58657                       055442.4+310146060110.2+310205179.90  3.97 5.98R +0.08 +0.14         A2V                -0.001+0.009      -009      100
2111          BD+27  945  40589 77837        4589           055443.3+273402060100.4+273420182.90  2.23 6.05  +0.25 -0.26         B9Iab              -0.001-0.004      +017       57  6.0   9.5      *
2112          BD+49 1441  40626 40801                       055502.4+495415060248.7+495420163.22 13.19 6.05  -0.06 -0.10         B9.5IV             +0.024-0.046      +025       20
2113          BD-03 1256  406571327322457I          2770    055503.1-030441060003.4-030427209.79-12.83 4.53  +1.22 +1.21 +0.67   K1.5IIIFe-1.5      +0.012-0.076 +.022+026     < 17
2114          CP-53  978  40665234224      W        2753    055507.0-532606055714.4-532534261.27-29.37 6.45  +1.48               K5-M0III           +0.016-0.014      +073           1.0  55.9AB   3*
2115          BD+43 1421  40722 40808                       055539.8+432239060253.6+432243169.15 10.21 6.42R                     K0                 +0.020-0.024      -019
2116          BD+22 1140  40724 77858      D                055539.2+222353060141.6+222403187.47-00.20 6.37  -0.07 -0.28         B8V                -0.011-0.020      +024       40                 *
2117          CD-44 2363  40733217648                       055539.5-440230055837.6-440204250.77-27.59 5.81  +1.07 +0.87         G8II               -0.001+0.014      +012V
2118          BD-12 1337  40745151011                       055540.9-125413060017.7-125359218.96-17.08 6.22  +0.36               F0                 -0.007-0.039      +033       45
2119 38    AurBD+42 1473  40801 408182461                   055605.3+425453060318.0+425442169.60 10.05 6.10  +0.97 +0.78         K0II               +0.112-0.141 +.014+038V?                        *
2120   Eta ColCD-42 2266  40808217650 229                   055605.1-424914055908.8-424855249.46-27.23 3.96  +1.14 +1.08 +0.58   K0III              +0.022-0.014 +.019+017
2121          BD+59  937  40827 25551                       055614.9+592347060508.3+592335154.47 17.55 6.34  +1.10 +1.10         K1III-IV           +0.005-0.051      +031                          *
2122          BD+32 1166  40832 58688                       055621.5+323827060255.1+323809178.68  5.07 6.24  +0.42 -0.01         F4V                +0.076-0.209 +.028+034       12
2123          BD+51 1146  40873 255482463    4633   2804    055633.3+513433060429.1+513424161.80 14.17 6.45  +0.18 +0.16         A7III              +0.005-0.042      +020V      99  2.8  39.0AB   3*
2124 61Mu  OriBD+09 1064  40932113389        4617   2792    055652.8+093850060223.0+093851198.68-06.31 4.12  +0.16 +0.11 +0.10   A2V                +0.015-0.028 +.028+045SB1O   24  1.8   0.3AB   3*
2125   Kap MenCP-79  202  409532562482451                   055701.7-792243055016.7-792141291.13-29.44 5.47  -0.08 -0.24         B9V                -0.009+0.066      +010
2126          BD+63  630  40956 13715                       055704.8+632729060639.2+632713150.61 19.31 6.39R                     K0                 -0.030+0.006      -015
2127          BD+01 1195  40964113392                       055705.8+014136060217.1+014140205.74-10.13 6.59  -0.05 -0.27         B8V                +0.004+0.007      +003
2128  3    MonBD-10 1349  40967151037        4615           055708.1-103557060150.4-103553216.95-15.77 4.95  -0.12 -0.60 -0.09   B5III              -0.007 0.000D+.003+039SB     65  3.5   1.9      *
2129          CD-25 2865  409721711382460                   055709.3-252509060113.1-252504231.28-21.74 6.05  +0.02               A1V                -0.008-0.018      +012
2130 64    OriBD+19 1186  41040 95166      D        2803    055732.1+194132060327.3+194126190.03-01.18 5.14  -0.11 -0.43         B8III              -0.001-0.017      +012SB2    17  0.0   0.0     3*
2131          CD-33 2681  41047196413    I:                 055738.7-335445060116.4-335442240.02-24.51 5.55  +1.58 +1.94         K5III              +0.015-0.020      +019
2132 39    AurBD+42 1477  41074 40840               2812    055751.8+425921060503.4+425854169.69 10.37 5.87  +0.30 +0.06         F3V                -0.039-0.143      +034V?     82                 *
2133          BD+11 1009  41076 95170                       055749.8+114058060324.7+114051197.02-05.10 6.08  -0.04 -0.03         A0V s              -0.016-0.015      -013V      19
2134  1    GemBD+23 1170  41116 779151163I W       Var?     055802.4+231608060407.2+231548186.99  0.72 4.16  +0.82 +0.52 +0.45   G7III              -0.008-0.100 +.031+020SBO        0.4   0.2AB   4*
2135 62Chi2OriBD+20 1233  41117 77911      D        2809    055758.8+200827060355.2+200818189.69-00.86 4.63  +0.28 -0.68 +0.22   B2Ia              t+0.002-0.006 +.025+017V      36  0.0   0.0      *
2136          BD-14 1315  41125151052                       055801.0-142948060233.8-142950220.72-17.25 6.20  +0.96               G5                 +0.011+0.012      +045
2137          BD+37 1405  41162 58716        4649           055810.9+375806060502.6+375751174.18  8.02 6.34                      K0III+A2           +0.003-0.007      +005    =< 23  5.6  17.9AB   3*
2138          CD-51 1713  41214234251                       055828.4-511311060049.2-511259258.86-28.53 5.67  +0.20               Am                 -0.021+0.094      +005        0
2139          BD+33 1236  41269 58727                       055857.8+333618060534.0+333557178.10  6.02 6.23  -0.08 -0.26         B9pSi              +0.008-0.013      +025       75
2140          CD-26 2675  41312171180    I   4645           055913.7-261702060315.5-261704232.33-21.62 5.04  +1.34 +1.44 +0.55E  K3III              +0.056+0.092      +182V?         7.3   2.2AB   3*
2141          BD+35 1334  41330 58739      W                055927.2+352410060608.5+352315176.57  6.99 6.12  +0.60 +0.06         G0V                -0.131-0.302 +.046-012V   =< 10  3.4 122.0
2142          BD-06 1391  41335132793               2817    055921.9-064217060413.5-064233213.60-13.54 5.21  -0.06 -0.85 +0.04   B2Ven             v-0.005-0.003      +025SB    419                 *
2143 40    AurBD+38 1377  41357 587492465          Var?     055941.4+382930060635.1+382858173.87  8.53 5.36  +0.25 +0.15 +0.09   A4m                +0.007-0.049 +.004+018SB2O   26                 *
2144 63    OriBD+05 1085  41361113429                       055938.2+052532060458.2+052512202.73-07.78 5.67  +1.04 +0.83        gG7                  0.000+0.003      +020V?
2145 66    OriBD+04 1116  41380113430 230                   055941.3+040952060458.4+040931203.85-08.38 5.63  +1.04 +0.75         G4III              -0.001-0.005      +033V?
2146          BD+29 1112  41429 77958    I   4673           055959.3+293113060622.5+293045181.78  4.20 6.08  +1.73 +1.94         M3II+F7V           +0.015-0.006      -036           5.0  10.0      *
2147          BD+41 1357  41467 40868                       060019.7+415151060726.9+415115170.92 10.24 6.12  +1.22               K0III              +0.007-0.021      +006
2148 17    LepBD-16 1349  41511151093    I         SS Lep   060031.3-162839060459.1-162904222.87-17.53 4.93  +0.24 +0.12 +0.52   A2eShell          v-0.006-0.001 +.032+020SB2O   98                 *
2149          CD-32 2743  415341964592464  W                060037.1-321011060420.3-321021238.43-23.36 5.65  -0.19 -0.82         B2.5V              -0.015+0.119      +093V     128  0.0   0.1      *
2150          BD-10 1368  41547151098                       060043.7-101409060527.0-101434217.01-14.81 5.87  +0.37 +0.04        dF4                 +0.014+0.024      +033SB2
2151          CP-60  537  41586249424              SW Pic   060054.1-600536060209.2-600549268.97-29.34 6.45  +1.58 +1.81         M4III              +0.010+0.004      -051V?                        *
2152 37    CamBD+58  897  41597 25597    I                  060109.6+585655060959.1+585609155.18 17.94 5.36  +1.09 +0.92         G8III              +0.024+0.023 +.011+031     < 19:
2153          BD+41 1365  41636 40881                       060119.4+410408060823.0+410320171.72 10.03 6.36  +1.05 +0.83         G9III              -0.016-0.053      -087
2154          BD-04 1362  416921328412467    4698           060141.2-041101060638.7-041138211.57-11.87 5.38  -0.13 -0.54         B5IV               -0.007-0.004      +020       20  6.2  28.5
2155 18The LepBD-14 1331  416951511102466                   060137.7-145534060609.3-145607221.51-16.64 4.67  +0.05 +0.02  0.00   A1Vn               -0.018+0.014 +.012+032SB    211                 *
2156          CD-24 3679  41698171222    I         S Lep    060138.2-241110060545.7-241144230.46-20.34 6.95  +1.63 +1.30 +2.37   M6III              +0.024-0.018      +012                          *
2157          CD-45 2300  41700217702 231  W     C          060135.7-450210060428.4-450212252.13-26.78 6.35  +0.52  0.00         G0IV-V             -0.080+0.244 +.042+026           0.3 196.7AC   3*
2158          CD-45 2302  41742217706      W     AB         060147.5-450442060440.2-450444252.18-26.76 5.93  +0.49 -0.01         F4V                -0.070+0.255 +.033+027           0.3 196.7AC   3*
2159 67Nu  OriBD+14 1152  41753 95259 232                   060151.7+144650060734.3+144606194.81-02.72 4.42  -0.17 -0.66 -0.16   B3V                +0.006-0.021 -.032+024SB1O   41                 *
2160          CD-35 2684  41759196483                       060155.0-353018060527.1-353049241.98-24.16 5.80  +0.02               A1V                +0.007+0.018      +024
2161          BD-11 1386  41814151126                       060210.9-110945060651.9-111025218.04-14.90 6.66  -0.15               B3V                -0.002+0.001      +013
2162          CD-48 2124  41824217708      W        2827    060211.3-482657060446.9-482731255.90-27.43 6.58  +0.72 +0.27         G6V                -0.092-0.038 +.019+023V       8  0.3   2.5      *
2163          CD-23 3431  41841171236        4704           060221.8-230557060632.0-230638229.46-19.78 5.47  +0.08 +0.09 +0.03   Am                 -0.023-0.023 +.014-015       64  5.1  44.3
2164          CD-29 2769  418431712311164                   060214.5-294451060605.5-294531236.08-22.22 5.81  +0.05               A1V                +0.013-0.041      +007V
2165 36    CamBD+65  517  41927 13756 233I                  060247.4+654418061251.1+654306148.62 20.75 5.32  +1.34 +1.44         K1.5IIIbCN-0.5     +0.006-0.033 +.004+007SB   < 17
2166          BD-21 1353  41933171251    I          2833    060243.6-214802060657.6-214846228.22-19.19 5.78  +1.64 +1.79         M4III              +0.004-0.016      +008V                         *
2167          BD+08 1202  42035113503                       060319.2+084104060847.2+084012200.31-05.40 6.55  -0.07 -0.17         B9V                +0.032+0.005      +035V
2168 19    LepBD-19 1361  42042151142    I                  060320.6-190914060741.6-190957225.72-18.01 5.31  +1.68 +1.96         M2III              +0.013+0.054      +029V
2169          BD+22 1198  42049 78045    I                  060330.6+221223060932.4+221124188.54  1.28 5.93  +1.63               K4I                -0.018-0.013      +008V
2170          CD-34 2655  42054196503               2836    060327.4-341757060703.7-341843240.83-23.48 5.83  -0.13 -0.55 -0.09   B4IVe              +0.003+0.003      +018
2171   Pi 1ColCD-42 2343  42078217720                       060335.5-421710060641.0-421755249.26-25.76 6.12  +0.25 +0.12 +0.14   Am                 -0.033+0.003      +002V      69
2172          BD+52 1041  42083 25613                       060343.5+524004061146.0+523850161.29 15.63 6.30  +0.14 +0.08 +0.02   A5IIIm             +0.011-0.070      +011SB2O   30                 *
2173  3    GemBD+23 1226  42087 78050        4751   2846    060339.6+230747060944.0+230648187.75  1.77 5.75  +0.21 -0.63         B2.5Ib            e+0.007-0.001      +016V      37  2.5   0.7AB   3*
2174          BD+02 1139  42111113507        4749           060344.7+023056060857.9+022958205.81-08.28 5.73  +0.07 +0.06         A3Vn              v+0.004-0.020      +034      250  1.2  29.3AB   3*
2175 41    AurBD+48 1352  42126 40924        4773B          060356.8+484401061136.5+484247164.97 13.95 6.82H                     A8V                +0.012-0.066D+.010+029      120  0.8   7.7      *
2176 41    AurBD+48 1352  42127 40925        4773A          060356.9+484353061136.6+484240164.97 13.95 6.09H +0.10 +0.08         A3V                +0.019-0.050D+.010+031      115  0.8   7.7      *
2177   The ColCD-37 2609  421671965142468                   060405.8-371420060731.6-371511243.94-24.26 5.02  -0.11               B9IV               +0.002-0.005      +045                          *
2178          CD-45 2317  42168217724                       060407.8-450445060701.9-450529252.30-26.36 6.51  +1.16 +1.13         K1III              +0.085+0.044      +082
2179          BD-05 1523  42278132900                       060441.8-054139060936.2-054240213.29-11.90 6.17  +0.34 +0.01         F3IV w             +0.058+0.014      -001       27
2180          BD-22 1327  423011712931165                   060445.6-222434060857.9-222539229.01-19.00 5.50  -0.01               A0IV               +0.002-0.039 +.012+044
2181   Pi 2ColCD-42 2351  42303217730      W                060446.5-420817060752.9-420914249.16-25.51 5.50   0.00               A0V                +0.001-0.014 +.016+031      248  0.1   0.1      *
2182          BD-18 1316  42327151173      W                060456.5-180630060920.3-180734224.88-17.24 6.35  -0.03               B9Vn               +0.013-0.015      +033           5.0  23.0
2183          BD-14 1348  42341151178    I                  060502.0-143404060934.3-143503221.53-15.74 5.56  +1.16 +1.20         K2III              -0.047+0.045      +031
2184          BD+18 1112  42351 95337                       060510.6+180901061101.8+180746192.26-00.37 6.33  +1.35 +1.44?        K1II               -0.002-0.047      -003V
2185  5    GemBD+24 1151  42398 78079                       060524.3+242632061132.3+242513186.80  2.76 5.80  +1.11               K0III              -0.004-0.054      +022SB                        *
2186          BD-22 1330  42443171317      W                060536.0-224526060947.9-224627229.43-18.95 5.71  +0.44 -0.05         F6V                +0.080+0.067D+.019+022V          0.1   0.2      *
2187          CD-44 2452  42448217735      W                060537.9-442021060834.6-442122251.57-25.92 6.27  -0.15               B8II               -0.013+0.010      +017V?         6.8  33.1
2188          BD+51 1163  42466 25630                       060552.0+511153061345.3+511021162.82 15.30 6.04  +1.06               K1III              -0.007-0.065      +011
2189          BD+32 1217  42471 588522470I                  060547.1+324256061220.1+324136179.58  6.84 5.78  +1.66               M2IIIa             -0.008-0.001      -051V?
2190          BD+21 1146  42475 78092    I         TV Gem   060550.4+215323061151.4+215207189.08  1.60 6.56  +2.25 +1.77 +1.48   M0-1Iab            +0.002+0.015      +017                          *
2191          BD+13 1151  42477 95352                       060548.1+133936061127.9+133819196.25-02.44 6.04   0.00 +0.04         A0Vnn              -0.005-0.017      +013      160
2192          CD-26 2761  42486171318                       060547.0-264057060947.2-264203233.30-20.40 6.27  +1.01 +0.78         G8-K0III           -0.007+0.021      -014V
2193 68    OriBD+19 1253  42509 953592471  W                060605.9+194846061201.3+194726190.92  0.63 5.75  -0.07 -0.11         B9.5V              -0.001-0.012      +031       83  3.5  86.3
2194   Eta1DorCP-66  493  42525249448                       060602.3-660132060609.4-660223275.80-29.19 5.71  -0.03 -0.07         A0V                +0.017+0.027      +018
2195          BD-06 1439  42536132924              V653 Mon 060609.7-064359061101.3-064515214.41-12.05 6.15  +0.01 +0.03         A0pSrCr            +0.004-0.003      +005V                         *
2196          CP-62  582  425402494512469           2845    060608.6-620812060703.4-620917271.36-28.89 5.05  +1.25 +1.35 +0.64   K2-3III            +0.018-0.072 +.013+022
2197  6    GemBD+22 1220  42543 78098    I         BU Gem   060615.3+225552061219.1+225430188.22  2.19 6.39  +2.24 +2.48 +1.48   M1-2Ia-Iab         +0.002-0.004 +.003+022SB                        *
2198 69    OriBD+16 1035  42545 95365                       060617.2+160911061203.3+160750194.13-01.12 4.95  -0.14 -0.58 -0.14   B5Vn               +0.009-0.014      +022SB    303                 *
2199 70Xi  OriBD+14 1187  42560 95362      W                060615.2+141353061156.4+141232195.81-02.06 4.48  -0.18 -0.65 -0.16   B3IV               +0.003-0.020 -.001+024SB1O  223  7.9  40.0AB   3*
2200          CD-27 2780  42621171339               2866    060635.8-270755061034.6-270915233.82-20.40 5.72  +1.07 +0.98         K1III              -0.016-0.046      +001
2201 40    CamBD+60  938  42633 13772    I W                060641.5+600137061540.6+595957154.43 19.01 5.35  +1.34               K3III              +0.033-0.021      +012V?         5.1 102.7
2202          BD-04 1393  42657132941        4799  V638 Mon 060647.3-043834061143.7-043956212.58-10.95 6.18  -0.08 -0.37         B9pHgMn            -0.011-0.009      +023       74  1.7   0.8      *
2203          CD-40 2291  42682217753               2864    060656.6-402006061010.3-402113247.37-24.63 5.58  +1.68 +2.00         M2II-III           -0.026+0.076      -019
2204          CD-49 2084  42683217747                       060652.8-493243060923.3-493346257.28-26.90 6.49  +0.52               F7V                -0.024+0.080      +060
2205          BD-06 1446  42690132944                       060659.8-063139061151.8-063301214.32-11.77 5.05  -0.20 -0.78         B2V                -0.005-0.005      +029       25
2206          CD-26 2784  42729171356                       060712.8-262736061113.5-262856233.21-20.02 6.09  -0.04               A0V                -0.017+0.003      +024V
2207          BD+18 1129  42784 95397                       060740.7+184224061333.4+184049192.07  0.42 6.58  -0.08 -0.43         B8Vnn              +0.023-0.017      +020V?    350                 *
2208          BD+10 1050  42807 95394                       060739.5+103938061312.6+103739199.10-03.50 6.45  +0.67 +0.15         G2V                +0.092-0.285 +.055+003
2209          BD+69  371  42818 13788 234                   060749.6+692119061850.8+691911145.16 22.46 4.80  +0.03  0.00 -0.01   A0Vn               +0.002-0.107 +.008-007SB    320
2210          BD-02 1512  42824132965                       060743.2-022848061244.4-023016210.74-09.75 6.62  +0.05 +0.08         A2V                -0.045+0.012      -004
2211          CD-45 2349  428342177582472                   060747.4-451535061039.9-451655252.67-25.77 6.31  -0.03 -0.04         A0V                +0.005+0.005      +020
2212   Del PicCP-54  980  42933234359 235          Del Pic  060821.0-545646061017.9-545807263.30-27.68 4.81  -0.23 -1.03 -0.26   B3III+O9V          -0.003+0.005      +031SB2O  221                 *
2213          BD-17 1398  42927151227                       060821.7-174416061246.3-174547224.88-16.35 6.52  -0.17               B3III              -0.008+0.017      +008
2214          BD+17 1182  42954 95419      W                060838.1+175605061428.6+175423192.85  0.24 5.88  +0.22 +0.15 +0.11   A5Vm               -0.009-0.012      +029SB     47  0.0   0.5      *
2215  1    LynBD+61  869  42973 137872479I         UW Lyn   060841.6+613252061754.8+613055153.03 19.81 4.98  +1.83 +2.06         M3IIIab            -0.014-0.006 +.006+011                          *
2216  7Eta GemBD+22 1241  42995 78135    I   4841  Eta Gem  060850.4+223209061452.6+223024188.85  2.52 3.28  +1.60 +1.66 +1.31   M3III              -0.068-0.012 +.014+019SBO        2.0   0.0 O   3*
2217          BD+36 1388  43017 589051167    4849A          060854.6+361042061539.0+360855176.81  9.05 6.92  +0.45 -0.02         F4IV               -0.065+0.003      +007           0.6  11.3AB   4*
2218          BD-03 1345  43023132994                       060855.7-034251061354.3-034429211.99-10.05 5.83  +0.94 +0.64 +0.32E  G8III              -0.011+0.025      +050V                         *
2219 44Kap AurBD+29 1154  43039 781431168I          2877    060900.3+293206061522.7+292953182.72  5.92 4.35  +1.02 +0.80 +0.54   G8.5IIIb           -0.073-0.262 +.018+020V?   < 19:
2220 71    OriBD+19 1270  43042 95432        4842           060857.8+191126061450.9+190923191.80  0.92 5.20  +0.44  0.00         F6V                -0.098-0.188 +.041+036       13  5.6  25.4AB   4*
2221   Nu  DorCP-68  474  431072494611166                   060922.7-684919060844.2-685036279.01-29.00 5.06  -0.08 -0.21         B8V                -0.052+0.018      +018V?    121
2222          BD+13 1173  43112 954442474I:W        2878    060928.0+135251061508.5+135104196.49-01.55 5.91  -0.23 -0.95         B1V                +0.023+0.006      +036V?         6.6  21.3      *
2223 72    OriBD+16 1060  43153 95447                       060939.1+161026061525.1+160835194.51-00.40 5.30  -0.14 -0.46         B7V                +0.003-0.012      +022V      91
2224          BD-04 1421  43157133003        4846           060940.0-043220061436.7-043406212.83-10.27 5.83  -0.17 -0.65         B5V                -0.008+0.006      +020           6.0   1.7
2225          CD-23 3577  43162171428                       060937.3-235012061345.1-235143230.86-18.52 6.39  +0.72               G5                 -0.069+0.118      +022V?
2226          CD-29 2883  43179171425                       060941.3-292203061333.3-292346236.31-20.57 6.54  -0.10               B7V                -0.025-0.013      +018V
2227  5Gam MonBD-06 1469  432321330122475I   4853           060958.6-061439061451.3-061629214.40-10.98 3.98  +1.32 +1.41 +0.64   K1.5IIIBa0.3       -0.008-0.019 +.013-005V?         9.0  51.4      *
2228 42    AurBD+46 1122  43244 40999                       061007.4+462726061734.7+462527167.55 13.89 6.52  +0.27 +0.09 +0.14   F0V                -0.043+0.019      -008V     195                 *
2229 73    OriBD+12 1081  43247 95457                       061007.9+123457061545.0+123304197.71-02.04 5.33  -0.02 -0.13         B9II-III           -0.001+0.001      +013V      59
2230  8    GemBD+24 1182  43261 78168      D                061012.4+240009061619.0+235812187.72  3.50 6.08  +0.90 +0.62 +0.49   G8III              -0.015-0.009      -021           0.0   0.1      *
2231          BD+06 1172  43285113650                       061018.6+060553061540.1+060358203.42-05.13 6.07  -0.13 -0.52         B6Ve               -0.008-0.024      +026      290
2232          BD+04 1181  433171136532478                   061029.6+041856061547.0+041701205.02-05.94 6.64  -0.17 -0.65         B3IV               -0.001-0.004      +013SB
2233          BD-00 1234  433181330282477                   061029.1-002828061534.3-003044209.27-08.21 5.65  +0.50  0.00         F6V                -0.158-0.219 +.036-037V   =<  6
2234          BD-04 1431  43319133027        4865           061033.6-045256061529.7-045455213.24-10.23 5.99  +0.09 +0.10         A5IV s             +0.025-0.045      +031V?     95  4.2   4.1
2235          BD+17 1191  43335 95473    I                  061035.1+171253061623.8+171053193.71  0.30 6.39  +1.58 +1.79         K5II               -0.002-0.014      +039V?
2236          BD+01 1275  43358113656      W                061043.9+011202061554.0+011009207.81-07.36 6.37  +0.46 -0.01         F5IV:              -0.005+0.034      +003SB2    22  0.0   0.2      *
2237          BD-08 1368  43362133029        4866           061040.1-090015061526.0-090207217.00-12.07 6.10  -0.08 -0.31         B9III              -0.035+0.015 -.003+019V          0.0   0.2      *
2238  2    LynBD+59  959  43378 25665 237          UZ Lyn   061048.0+590250061937.4+590039155.59 19.12 4.48  +0.01 +0.03 -0.01   A2V s             v-0.008+0.026 +.035-004V?     20                 *
2239 43    AurBD+46 1124  43380 41010                       061049.4+462358061816.9+462138167.66 13.97 6.38  +1.11 +1.12         K2III              +0.012-0.126      +000                          *
2240  9    GemBD+23 1275  43384 78176                       061052.6+234629061658.7+234427187.99  3.53 6.25  +0.45 -0.38 +0.40   B3Ib                0.000-0.001      +013V      51                 *
2241 74    OriBD+12 1084  43386 954761169  W                061049.6+121800061626.6+121620198.04-02.03 5.04  +0.42 -0.02         F5IV-V             +0.082+0.186 +.041+009V?     17  4.2 204.0AC   3
2242          BD-20 1336  43396171458    I                  061050.3-201429061508.4-201620227.51-16.84 5.91  +1.32 +2.43         K0                 +0.005+0.030      +052V
2243          BD-18 1352  43429151274                       061054.6-182637061517.5-182836225.81-16.09 5.99  +1.06 +0.96         K0                 +0.016-0.048      +065V
2244          BD-13 1411  43445151283                       061109.9-134108061544.9-134306221.37-14.02 5.01  -0.08 -0.23 -0.05   B9Vn               +0.011-0.009      +038V     235
2245   Eta2DorCP-65  561  434552494692473           2873    061102.0-653356061115.0-653522275.31-28.65 5.01  +1.62 +1.86 +1.24   M2.5III            -0.026+0.112 +.009+035
2246          BD+01 1278  43461113668                       061111.1+010650061621.2+010449207.94-07.30 6.63  -0.05 -0.44         B6V                +0.016-0.008      +022
2247 75    OriBD+09 1173  435251136752480    4890           061135.8+095844061706.6+095633200.16-02.99 5.39  +0.10 +0.09         A2V                +0.007-0.061 +.013+012      192  0.2   0.1AP   5*
2248          BD+07 1216  43526113673                       061134.6+070515061658.4+070311202.70-04.38 6.57  -0.12 -0.51         B7III              -0.011+0.009      +041
2249          BD-16 1415  43544151294               2891    061140.1-163504061607.7-163704224.13-15.15 5.92  -0.17 -0.75 -0.14   B2.5Vn            e-0.002+0.008      +014                          *
2250          BD+14 1233  43583 95494                       061152.5+140540061733.3+140330196.59-00.94 6.59  -0.03 -0.10         A0V                 0.000-0.012      +003       74
2251          BD+05 1168  43587113683      W        2896    061158.2+050753061716.1+050601204.47-05.23 5.71  +0.61 +0.10         F9V                -0.216+0.159 +.050+013V?  =<  6  3.9 189.7AB   4
2252          CD-29 2936  43636171482                       061206.1-294516061557.2-294718236.90-20.22 6.67  +1.55               K4III              +0.004+0.008      +036V?
2253          BD+14 1235  43683 955022481    4901           061224.0+142511061805.6+142258196.37-00.67 6.16  +0.01 +0.03         A3V                -0.007+0.004      +008V?    120  6.9  25.4AC   3*
2254          BD-22 1364  43745171502      W                061251.1-224018061703.5-224254230.04-17.38 6.07  +0.56 +0.07         G0V                +0.119-0.251 +.018-004V          4.4  47.6      *
2255  6    MonBD-10 1455  43760151318                       061253.0-104117061735.2-104331218.79-12.33 6.75  +0.35 +0.20 +0.18   F0Del Del          -0.002-0.007      +027V?  =< 15
2256   Kap ColCD-35 2800  43785196643 238           2897    061259.6-350626061633.1-350826242.35-21.89 4.37  +1.00 +0.83 +0.51   K0.5IIIa            0.000+0.086 +.025+024                          *
2257  4    LynBD+59  964  43812 25678        4950           061311.1+592454062203.6+592220155.35 19.54 5.94R                     A3V                -0.001+0.004D+.005-024V?    155  1.1   0.7AB   5*
2258          BD+17 1203  43819 95519              V1155 Ori061312.9+172152061901.8+171930193.88  0.92 6.32  -0.07 -0.30         B9IIIpSiCr:        -0.009-0.009      +000V      35                 *
2259          BD+09 1184  43821113710                       061311.7+090513061840.5+090250201.14-03.07 6.24  +0.87 +0.52         K0                 +0.030-0.044      -014SB1O                      *
2260          BD-16 1426  43827151322    I          2903    061314.7-164642061741.7-164857224.48-14.89 5.14  +1.30 +1.23         K3III              -0.011+0.008 +.017-008                          *
2261   Alp MenCP-74  374  43834256274 239                   061313.2-744307061014.4-744511285.75-28.80 5.09  +0.72 +0.33 +0.32   G6V                +0.118-0.213 +.115+035
2262          CD-39 2491  43847196646                       061317.3-391339061635.6-391552246.60-23.14 6.00  +0.16               A2Vm:              -0.013-0.022      -004
2263          CD-37 2707  43899196653                       061337.0-374209061701.2-374415245.05-22.61 5.53  +1.14 +1.11         K1III              +0.002+0.081      +067                          *
2264 45    AurBD+53 1008  43905 256812484                   061338.6+532952062146.1+532708161.14 17.33 5.36  +0.43 +0.12         F5III              +0.020-0.084 +.022-001SB1O   14                 *
2265          CD-37 2708  43940196659                       061343.6-371255061709.5-371511244.56-22.43 5.87  +0.14               A3V                -0.011-0.010      +021V?
2266          BD-19 1407  439551513342483                   061354.8-195541061813.7-195801227.52-16.05 5.52  -0.18               B2V                -0.005+0.007      +023SB     88
2267          BD-09 1411  43993133091    I                  061405.2-092058061850.6-092325217.70-11.47 5.36  +1.24 +1.32         K1.5III            +0.003-0.033      +007
2268          BD-14 1400  44021151346    I          2912    061417.1-145907061848.8-150129222.91-13.90 6.06  +1.66 +1.87 +0.86E  M0III              -0.003+0.022      +051                          *
2269          BD+14 1247  44033 95543    I          2917    061422.0+144136062004.2+143904196.36-00.12 5.69  +1.61 +1.87         K3Ib               -0.003-0.014      +036                          *
2270          BD-08 1386  44037133098                       061420.8-083244061907.9-083511216.99-11.05 6.22  -0.04 -0.14         B9V                -0.022-0.010      +004
2271          BD-20 1355  44081171549               2915    061442.6-205305061859.0-205534228.51-16.27 5.81  -0.15               B4IV               -0.009-0.009      +031                          *
2272          BD+29 1190  44092 78259                       061448.9+293509062112.1+293228183.28  7.06 6.43  +0.06 +0.01         A1V s              +0.035-0.039      +021V      54
2273  7    MonBD-07 1373  441121331141170  S                061453.7-074651061942.8-074923216.36-10.59 5.27  -0.19 -0.75         B2.5V              -0.004 0.000      +029SB    152
2274          CP-59  619  44120234418      W                061455.5-591000061618.5-591249268.17-27.47 6.43  +0.59 +0.14         G0V                -0.057-0.325 +.043-003           1.0  38.3AB   3*
2275          BD-02 1564  44131133118    I          2918    061459.1-025407061959.6-025640211.97-08.34 4.90  +1.60 +1.96         M1III              -0.013+0.004 -.001+047V?
2276          BD+11 1128  44173 95563                       061517.0+114801062052.3+114523199.00-01.32 6.54  -0.10 -0.56         B5III              +0.023+0.003      +019
2277          BD+17 1214  44234 95572                       061535.7+174835062125.9+174549193.76  1.63 6.35R                   g:G9                 +0.019-0.039      +010
2278          CP-52  902  44267234430                       061539.3-524133061751.7-524359261.05-26.20 6.41  +1.46               K2-3III            +0.003-0.006      -005
2279          CD-34 2795  44323196686                       061604.7-342112061941.0-342348241.81-21.04 5.78  -0.08 -0.26         B9V                +0.014+0.001      +026
2280          BD+02 1197  44333113758        4971   2932    061613.2+021854062125.8+021607207.47-05.63 6.31  +0.24 +0.09         A4.5V              -0.016-0.026D+.012-028V     160  0.6   0.3      *
2281          CD-50 2169  44362234442                       061620.1-501900061846.8-502133258.49-25.58 7.04  +0.83               G2Ib               +0.006+0.003      +017
2282  1Zet CMaCD-30 3038  44402196698 240  W        2927    061628.4-300108062018.8-300348237.52-19.43 3.02  -0.19 -0.72 -0.18   B2.5V              +0.009+0.003 +.004+032SBO    63  4.6 175.5      *
2283          CP-71  426  44447256285                       061641.8-713957061505.9-714210282.28-28.49 6.64  +0.56 +0.01         F8V                -0.025+0.061      +014
2284          BD-11 1460  44458151401        4978  FR CMa   061645.1-114337062124.7-114624220.17-11.94 5.64  -0.02 -0.85 +0.06   B1Vpe             v-0.009-0.005      +016SB    265  4.2   4.2AB   3*
2285          BD+70  401  44472  5861        5039           061650.3+703523062814.6+703208144.14 23.58 5.97R                     A4V                +0.002+0.023      -030V      80  5.9   5.7      *
2286 13Mu  GemBD+22 1304  44478 78297 241I   4990  Mu  Gem  061654.6+223354062257.6+223049189.72  4.17 2.88  +1.64 +1.85 +1.38   M3IIIab           e+0.054-0.111 +.020+055V?         8.1 121.7AxBC 3*
2287          BD+12 1123  44497 95599                       061659.6+123709062236.5+123412198.48-00.56 6.00                      F0III              -0.033-0.037      +021V      75
2288          CD-34 2806  44506196707               2931    061659.1-340557062036.3-340838241.63-20.78 5.53  -0.19 -0.88         B1.5Ve             +0.005+0.028      +054V     211                 *
2289 46Psi1AurBD+49 1488  44537 41076 242I         Psi1 Aur 061711.8+492020062453.9+491717165.35 16.17 4.91  +1.97 +2.29 +1.07   K5-M0Iab-Ib        -0.002-0.001 +.007+005SB                        *
2290          CD-48 2259  445942178612486I'                 061728.1-484117062006.1-484428256.80-25.01 6.60  +0.66 +0.20         G3V                +0.231-0.271 +.041+060
2291          BD+56 1125  44691 257312491          RR Lyn   061759.3+562018062625.8+561706158.63 19.00 5.64  +0.24 +0.12 +0.10   A3Vm               -0.026+0.022 +.023-010SB1O   37                 *
2292          BD+03 1221  44700113801                       061802.3+034854062318.5+034552206.36-04.52 6.40  -0.15 -0.63         B3V                +0.006-0.013      +030SB                        *
2293  5    LynBD+58  927  44708 25733    I   5036           061805.0+582819062648.9+582502156.53 19.78 5.21  +1.53 +1.88         K4III              -0.005-0.009 +.005-003SB   < 19: 2.6  95.4AC   3*
2294  2Bet CMaBD-17 1467  44743151428 243I W       Bet CMa  061817.7-175422062242.0-175721226.06-14.27 1.98  -0.23 -0.98 -0.24   B1II-III           -0.006 0.000 +.019+034SB     36  7.8 185.9      *
2295          BD-04 1484  44756133199                       061826.3-043811062322.7-044114213.93-08.37 6.67  +0.06 +0.01         A2V                -0.013-0.007      +017
2296   Del ColCD-33 2927  44762196735                       061827.5-332309062206.8-332611241.03-20.24 3.85  +0.88 +0.52 +0.47   G7II               -0.024-0.055 +.019-003SBO                       *
2297          BD+29 1213  44766 78333                       061829.4+294536062452.6+294226183.50  7.85 6.71  -0.06 -0.39         B8IIIn             -0.032-0.009      +029      250
2298  8Eps MonBD+04 1236  44769113810 244    5012A          061828.1+043838062346.1+043534205.68-04.03 4.44  +0.18 +0.13 +0.10   A5IV               -0.018+0.011 +.025+015SB    124  2.2  12.9AB   3*
2299          BD+04 1237  44770113811        5012B          061828.5+043850062346.5+043544205.67-04.03 6.72  +0.45 -0.05         F5V                -0.021-0.010 +.025+016    =< 25  2.2  12.9AB   3*
2300          BD+08 1316  44783113817                       061834.1+085613062402.3+085305201.90-01.97 6.26  -0.08 -0.31         B8Vn               -0.005-0.018      +003V      65                 *
2301          BD-09 1444  44816133203    I                  061851.7-094924062336.0-095228218.67-10.63 6.19  +1.84 +2.05         K5                 +0.008+0.018      +029V
2302          BD+16 1135  44867 95641                       061906.9+160642062452.9+160325195.66  1.56 6.33  +1.08 +0.91         G9III              +0.040-0.046      +077SB
2303          BD-14 1428  44891151450    I                  061914.5-150109062346.0-150418223.47-12.83 6.24  +1.53               K2                 -0.015-0.007      +037V?
2304          BD+23 1347  44927 78349      D                061928.2+232256062532.9+231937189.28  5.07 6.06R -0.03 +0.07         A2Vn               -0.010-0.023      -027V?    220  0.4   0.1      *
2305          BD-11 1478  449511514581171I                  061930.4-112834062410.3-113149220.25-11.22 5.22  +1.24 +1.20         K3III              -0.054-0.035 +.016-026     < 19:
2306          BD-19 1435  44953151453        5023   2944    061928.1-194400062347.6-194707227.89-14.78 6.60  -0.15 -0.64         B8IIIHe wk         -0.017+0.014D+.003+024           0.3   0.8      *
2307          CD-31 3245  44956196759                       061929.8-314419062314.4-314724239.47-19.45 6.34  +0.89               G8III              -0.056+0.013      +015
2308          BD+14 1283  44984 95659    I         BL Ori   061945.8+144637062528.1+144319196.91  1.06 6.24  +2.34 +3.40 +1.28   C5II               -0.014-0.002      +013                          *
2309          BD-12 1470  44996151461        5030           061943.8-125431062420.5-125745221.58-11.80 6.12  -0.08 -0.63         B5Ve               -0.012-0.012      +008V     120  4.4  23.2      *
2310          BD+07 1273  44990113845              T Mon    061949.0+070825062513.1+070509203.63-02.55 5.98  +1.22 +1.04 +0.65   F7Iab-K1Iab        +0.017+0.004 +.005+032SB     21                 *
2311          CD-25 3189  450181716882490I                  061952.1-253124062355.9-253440233.44-17.04 5.63  +1.56 +1.90         K5III              +0.008-0.042      +034V?
2312          BD+01 1332  45050113855      W                062007.6+013324062518.4+013004208.60-05.12 6.66  -0.04 -0.22         B9V                -0.017-0.011      +007      150  0.0   0.1
2313          BD-00 1287  45067133229                       062009.7-005305062516.4-005646210.78-06.25 5.87  +0.56 +0.08 +0.20E  F8V                +0.224-0.225 +.027+044    =< 10
2314          BD+47 1299  45105 41109                       062019.1+472747062751.1+472419167.36 15.89 6.56  -0.04 -0.12         A0V                -0.004+0.020      -012       31
2315          BD+02 1227  45137113868                       062033.8+021942062546.6+021620207.97-04.66 6.51  -0.03 -0.07         A0V                +0.005-0.002      +024      120
2316          CD-36 2873  45145196774      W                062032.8-363919062401.0-364228244.46-20.96 5.62  +1.04 +0.87         G6III              -0.015+0.055      +053           1.2  64.8AC   3
2317          BD-03 1425  45168133238                       062048.8-034957062547.1-035321213.48-07.48 6.35  +1.02 +0.77         G9III              -0.020-0.005      +012SBO                       *
2318          CD-28 2981  45184171711    I'                 062050.7-284317062443.9-284648236.62-18.07 6.39  +0.62               G2IV               -0.159-0.115      -005V?                        *
2319          BD+32 1300  45192 59096                       062103.1+323726062735.5+323347181.17  9.64 6.43  +1.26               K0                 -0.002-0.058      +057
2320   Nu  PicCP-56 1072  45229234473                       062108.7-561857062255.8-562212265.18-26.16 5.61  +0.24 +0.13 +0.14   Am                 -0.034-0.024      +007V?      0
2321          BD-07 1422  45239133246                       062109.8-075016062558.8-075339217.13-09.23 6.40  +0.14 +0.12         A4V                -0.004+0.019      +031
2322          CP-52  913  45291234475                       062121.8-520734062337.7-521052260.64-25.23 5.98  +1.04               G8III              -0.025-0.023      +011
2323          CD-40 2440  453062179102492                   062129.7-401340062444.5-401703248.16-21.95 6.31  -0.05               B9V                -0.003-0.009      +023V
2324          BD-01 1242  453201332602493                   062135.7-012653062639.6-013026211.45-06.20 5.87  +0.08 +0.09         A3Vn               -0.007-0.026      +010V     270
2325          BD-04 1510  45321133257                       062137.7-043220062634.5-043550214.21-07.62 6.15  -0.14 -0.62         B2.5V              +0.007-0.002      +010      185                 *
2326   Alp CarCP-52  914  45348234480 245I                  062143.9-523827062357.1-524145261.21-25.29-0.72  +0.15 +0.10 +0.18   F0II               +0.022+0.021 +.028+021        0                 *
2327          BD+00 1421  45357113896                       062149.5+005401062658.7+005027209.38-05.05 6.71  +0.04  0.00         A1Vn               -0.020-0.004      +010      143
2328          BD-07 1429  45380133263        5070           062154.9-072708062644.9-073041216.86-08.89 6.27  -0.04 -0.10         A0Vn               +0.006-0.001      +020      146  2.3  21.1      *
2329          CD-34 2864  45383196805      W                062155.8-350020062529.9-350350242.91-20.13 6.25  +1.36 +1.59         K3III              -0.005-0.035      +019           1.1 132.2AB   3
2330 16    GemBD+20 1428  45394 78402                       062159.8+203323062756.6+202946192.06  4.26 6.22  +0.01 +0.18         A2V s              -0.035+0.007      +039
2331  6    LynBD+58  932  45410 25771      W                062206.0+581410063047.2+580946156.97 20.19 5.88  +0.94 +0.66         K0III-IV           -0.026-0.335 +.025+036           3.0 179.6
2332 48    AurBD+30 1238  45412 59128              RT Aur   062208.4+303318062834.1+302935183.15  8.92 5.55  +0.68 +0.50         F8Ib              v+0.002-0.013 +.005+022V       0                 *
2333          BD+02 1237  45415113906                       062206.6+025804062720.4+025429207.59-04.02 5.55  +1.04 +0.86 +0.36E  G9III              -0.046+0.008      +053
2334          BD+00 1426  45416113905    I                  062205.6+002133062713.8+001757209.90-05.25 5.20  +1.18 +1.13         K1II               +0.007-0.008 +.005+033                          *
2335          BD-00 1299  45433133269    I                  062208.8-001257062715.6-001634210.41-05.50 5.55  +1.38 +1.61         K5III               0.000-0.008      +039V?                        *
2336          CP-58  692  45450234482                       062216.5-582919062346.9-583238267.59-26.42 6.48  +0.12               A3V                +0.010+0.021      +023SB2                       *
2337          CP-63  561  45461249531                       062222.9-633743062301.3-634059273.28-27.23 6.27  +1.60 +2.01 +0.90E  M1III              -0.024+0.013      -014
2338 47    AurBD+46 1149  45466 411302496I                  062234.7+464457063003.0+464108168.20 15.95 5.90  +1.44              gK4                 -0.006+0.003 +.017-047
2339          BD+27 1122  45504 78417      D                062240.7+270156062856.6+265804186.37  7.41 6.47R                     F5                 +0.116-0.067      -070    =< 10  0.9   0.0      *
2340          BD+16 1159  45506 957192494                   062242.6+161806062828.0+161418195.90  2.41 6.23  +0.90 +0.60         G5                 -0.095-0.047      +041
2341          CP-52  919  45509234488                       062235.9-524457062448.2-524823261.36-25.19 6.51  +1.70               K5-M0III           -0.007+0.002      +036
2342          BD+10 1149  45512 95718                       062247.0+102201062818.8+101814201.14-00.38 6.15  +1.15 +1.18         K2III-IV           +0.026-0.039      -020
2343 18Nu  GemBD+20 1441  45542 784231173    5103           062301.5+201632062857.8+201244192.42  4.35 4.15  -0.13 -0.48 -0.11   B6IIIe             -0.007-0.014 +.009+039SB    219  1.7   0.2AA'  8*
2344 10    MonBD-04 1526  45546133290 246  W                062301.2-044201062757.6-044544214.52-07.39 5.06  -0.17 -0.76 -0.17   B2V                -0.005-0.001      +025V?     79  4.2  77.2AB   3*
2345          CP-60  608  45557249535                       062259.5-601330062413.7-601652269.52-26.63 5.80   0.00 -0.01         A0V                -0.041+0.036      +010
2346          BD+79  208  45560  5911                       062305.5+794033064016.8+793558134.52 26.15 6.54R                     A1V                +0.001+0.017      -007       92
2347          BD+02 1244  45563113929                       062304.9+015831062816.8+015444208.58-04.27 6.48  -0.07 -0.16         B9V                -0.002-0.027      +042       80
2348          CD-48 2308  45572217922      W                062304.7-480702062543.6-481038256.45-23.97 5.76  -0.06 -0.16 -0.03C  B9V                -0.005-0.032D+.003+006           2.5   1.2
2349          CD-25 3237  45588171774      W                062309.5-254722062711.2-255124234.00-16.46 6.07  +0.53 +0.05         F9V                -0.186-0.218 +.045+036V?         6.5  43.
2350          BD+82  177  45618  10433950                   062322.6+821157064430.2+820655131.75 26.61 6.65  +0.17 +0.06         A5V                +0.010-0.052      +006       55
2351          BD+11 1193  45638 95732                       062326.9+110500062900.0+110110200.58  0.10 6.59                      A9IV               -0.027-0.010      +041       42
2352   Pi 1DorCP-69  607  45669249532                       062334.9-695544062238.2-695903280.35-27.82 5.56  +1.51 +1.83         K5III              +0.020+0.029      +016V?
2353          CD-37 2837  45680196842      W                062343.5-375006062707.5-375344245.88-20.76 6.48  +0.39               F3V                -0.011+0.042      +030           7.0  32.8
2354          CP-63  568  45701249539    I'                 062344.5-632203062426.3-632544273.01-27.04 6.46  +0.66               G3III-IV           -0.007-0.103      +023
2355          BD+02 1253  45724113940    I W                062401.2+024242062914.9+023846208.04-03.72 6.16  +1.54 +1.86        gM1                 +0.009-0.040      +009           4.0  19.3
2356 11Bet MonBD-06 1574  45725133316        5107A  2977    062358.0-065808062849.0-070158216.66-08.21 4.60  -0.10 -0.63 -0.16   B3Ve              v-0.017 0.000 +.021+020SBO   346  0.5   7.1AB   4*
2357 11Bet MonBD-06 1575  45726133317        5107B  2977    062358.5-065814062849.5-070204216.66-08.21 5.40  -0.07 -0.52         B3ne               -0.017+0.005 +.021+018      123  0.5   7.1AB   4*
2358 11Bet MonBD-06 1575  45727              5107C  2977    062358.5-065814062849.5-070204216.66-08.21 5.60H                     B3e                -0.016+0.005 +.021+023      331  0.5   7.1AB   4*
2359          BD-17 1506  45765151546                       062411.7-172408062837.3-172758226.20-12.78 5.77  +1.12               G5                 -0.007+0.004      +018V
2360          CP-63  572  45796249542                       062418.5-634608062455.6-634940273.47-27.04 6.27  -0.13 -0.45         B6V                -0.014+0.026      -001
2361   Lam CMaCD-32 3066  45813196857                       062427.7-323101062810.1-323448240.64-18.76 4.48  -0.17 -0.61 -0.16   B4V                -0.018+0.024 +.013+041      135
2362          BD+09 1259  45827113957                       062437.1+090546063005.5+090145202.47-00.58 6.57  +0.14 -0.06         A0III              -0.017-0.026      +014SB?                       *
2363          BD+78  227  45866  59192507I                  062456.8+780431064028.8+775945136.29 25.92 5.73  +1.47              gK5                 +0.009-0.012 +.018-014
2364          CD-32 3072  45871196861      W       Var      062455.9-321824062839.2-322216240.47-18.59 5.74  -0.17 -0.65         B4Vnpe             -0.003+0.017D+.006+023           2.0   1.2      *
2365          BD+73  340  45947  59092503                   062517.4+734622063754.9+734144140.95 25.02 6.24  +0.38 +0.01         F2                 -0.155-0.018 +.039+006       30
2366          BD+17 1275  45951 95765        5146           062522.2+170030063110.1+165619195.57  3.31 6.20  +1.16 +1.06         K2III              +0.001-0.049      +027SB         4.0   8.0AB   3*
2367          BD-09 1493  459761515732497I                  062527.5-100050063011.3-100453219.58-09.26 5.93  +1.38               K0                 -0.011+0.002      +020V?
2368          CD-40 2482  45983217951                       062530.2-410034062842.3-410428249.22-21.47 6.32  +0.40               F2-3V              +0.030+0.028      +017
2369          CP-57 1001  45984234519                       062529.1-575618062704.0-580008267.08-25.90 5.82  +1.28               K3III              -0.015-0.014      +013
2370          BD+11 1204  45995 95766        5153A          062535.7+111913063109.5+111503200.62  0.68 6.14  -0.08 -0.86         B2V:nne            -0.005-0.016D+.009+020      293  2.8  16.1      *
2371 19    GemBD+16 1178  46031 95778                       062552.2+155825063137.4+155412196.55  2.93 6.40R                     A8V                -0.016-0.015      +021       97
2372          BD+32 1324  46052 591942500          WW Aur   062555.6+323134063227.2+322717181.72 10.52 5.87  +0.14 +0.15 +0.02   A3m+A3m:           -0.031-0.022      -009SB2O   35                 *
2373          BD-13 1519  46064151585        5148           062558.4-130447063034.7-130853222.42-10.51 6.16  -0.15               B1.5V              -0.024+0.002      +002           5.0  33.2      *
2374          BD+11 1207  46075 95783                       062604.3+115145063139.2+114732200.20  1.04 6.65  -0.12 -0.42         B6III              -0.030-0.012      +010       70                 *
2375          BD+11 1209  46089 95788                       062613.6+113650063148.3+113240200.44  0.95 5.23  +0.15 +0.13         A3V                +0.013+0.032 +.010-003V     124                 *
2376  7    LynBD+55 1093  46101 25814    I                  062613.5+552535063432.8+552111159.97 19.75 6.45R                     K0III:             +0.002+0.004      -020
2377   Pi 2DorCP-69  614  461162495502495                   062619.8-693759062528.6-694125280.04-27.55 5.38  +0.97 +0.67 +0.36E  G8III              -0.016+0.195 +.007+009                          *
2378          BD+11 1213  46178 95803        5170           062648.4+114446063223.3+114025200.39  1.14 6.03  +1.07               K0III              +0.010-0.031      -021           4.2  31.1      *
2379          BD-12 1518  46184151602    I W                062644.4-121915063123.0-122330221.81-10.01 5.15  +1.27 +1.38         K3III              +0.040-0.016 +.022+017V?   < 19: 8.3  14.8
2380          CD-27 3051  461891718662499                   062648.7-274200063046.3-274610236.17-16.46 5.93  -0.15 -0.67         B2.5V              -0.007+0.011      +020V
2381          BD-08 1462  46229133369    I                  062701.7-080511063150.1-080929218.02-08.05 5.43  +1.38 +1.52        gK2                 -0.012-0.008      +003V
2382 12    MonBD+04 1304  46241114023               3017    062700.7+045540063219.2+045121206.43-02.02 5.84  +0.99 +0.79 +0.53   K0V                -0.037-0.006      +021V                         *
2383          BD+33 1356  46251 59222                       062709.0+330552063342.7+330127181.32 11.00 6.42R +0.03 +0.13         A3V                +0.011+0.010      -009
2384          CD-50 2241  46273234539      W                062721.5-501006062949.1-501421258.81-23.84 5.27  +0.41               F2V                -0.054-0.058 +.027+002SB    133  0.1   0.5AB   4*
2385 13    MonBD+07 1337  463001140341174                   062729.7+072423063254.2+071959204.30-00.75 4.50   0.00 -0.18 +0.04   A0Ib               -0.001-0.006 +.007+012       17                 *
2386          BD-05 1678  46304133382      W                062729.3-054743063223.1-055208216.01-06.90 5.60  +0.25 +0.07         F0Vnn              -0.001-0.042      -021           7.7   4.2
2387  4Xi 1CMaCD-23 3991  46328171895        5176  Xi1 CMa  062741.3-232047063151.3-232506232.11-14.53 4.33  -0.24 -0.99 -0.27   B0.5IV             -0.006+0.012      +027SB     27  9.6  24.6AB   3*
2388          CD-35 2947  46349196897                       062739.3-351116063113.0-351533243.53-19.10 5.84  +0.81 +0.53         G8III              -0.011 0.000      +014V                         *
2389          CP-56 1095  463552345412498           3000    062744.4-564703062928.5-565110265.89-25.36 5.22  +1.09               K0III              -0.040+0.031 +.010+013
2390          CD-40 2512  46365217986                       062747.3-405043063059.8-405459249.20-21.01 6.20  +1.40               K3III              -0.017 0.000      +051V?
2391          BD+14 1339  46374 95830                       062755.4+141356063336.1+140919198.32  2.55 5.53  +1.11              gK2                 -0.019-0.085      -012V?
2392          BD-11 1520  46407151625               3024    062805.7-110534063246.9-110959220.85-09.17 6.24  +1.11 +0.78 +0.48   G9.5III:Ba3        -0.007+0.001      -011V                         *
2393          CD-36 2962  464311969062501I:        SX Col   062807.1-365211063135.0-365624245.23-19.61 6.34  +1.63 +1.91         M1III              +0.005+0.072      +054                          *
2394  8    LynBD+61  893  46480 13897 247  W                062833.1+613409063741.4+612852153.87 22.04 5.94  +0.89 +0.52         G8IV-V             -0.209-0.274 +.027-046           3.0 157.0AB   4
2395          BD-01 1274  464871334041175                   062833.3-010840063337.9-011313211.99-04.52 5.10  -0.14 -0.56         B5Vn               -0.003-0.021      +025V     300
2396          BD+71  359  46509  59252511                   062843.2+714956064032.2+714456143.09 24.82 5.92  +1.19               K0III              -0.002+0.015      -023
2397          CD-31 3407  46547196919      W       Var      062854.4-315722063239.0-320150240.46-17.68 5.69  -0.17 -0.79         B2IVe              -0.001-0.002      +020V     120  3.0  24.7      *
2398 49    AurBD+28 1168  46553 785242504           3032    062854.1+280601063512.1+280120186.04  9.13 5.27  -0.03 -0.08         A0Vnn              -0.002-0.015 +.000+017      149
2399          CD-37 2889  46568196917                       062855.7-373713063221.3-374148246.04-19.72 5.24  +1.00               G8III              +0.065-0.077 +.016+039
2400          CD-51 1946  46569234551                       062858.1-514521063118.3-514934260.56-24.00 5.60  +0.54 +0.04         F8V                +0.105+0.094      +016
2401          BD+79  212  46588  5946 248                   062910.2+794022064614.1+793353134.59 26.41 5.45  +0.50 -0.02         F8V                -0.102-0.604 +.049+013SB1O                      *
2402 11    LynBD+56 1136  46590 25842                       062908.3+565617063738.5+565127158.61 20.64 5.85                      A2V                 0.000+0.012      +000V?
2403          BD-20 1437  46602171940                       062910.0-205050063326.6-205526229.92-13.18 6.40  +0.83               G5                 -0.030-0.026      -002V
2404 14    MonBD+07 1357  46642114085        5211   3031    062921.4+073901063446.4+073421204.30-00.23 6.45  -0.02 -0.03         A0V s              -0.010-0.004      +038       56  4.0  10.5      *
2405          BD+38 1539  46687 59280    I W       UU Aur   062940.3+383134063632.9+382643176.52 13.78 5.29  +2.61       +1.43   C5II               +0.005-0.026 +.005+012           5.6  17.9      *
2406          BD+10 1186  46709 958702505I                  062946.8+100402063517.6+095918202.21  1.00 5.88  +1.51 +1.85         K4III              -0.001-0.010      +039V
2407          CD-38 2730  46727196930                       062948.5-383254063310.3-383731247.03-19.88 6.44  +1.00               G8III              +0.006-0.022      +029
2408          CP-65  610  467302495682502                   062946.8-652951063003.0-653406275.48-26.71 6.29  +0.33 +0.06         F0III              +0.002+0.058      +006SB2                       *
2409          BD+00 1491  46769114097                       063006.3+005810063515.8+005324210.29-03.19 5.80   0.00 -0.45         B8Ib               -0.001-0.006      +010
2410          CP-61  669  46792249572                       063010.9-614820063110.6-615247271.43-26.05 6.15  -0.15               B3V                -0.013+0.008      +034SBO                       *
2411          CD-36 2990  46815196945    I                  063019.1-360926063349.5-361356244.69-18.94 5.42  +1.44 +1.72         K3III              -0.014+0.094 +.030+032
2412   Mu  PicCP-58  722  46860234564      W                063028.9-584042063158.4-584515268.03-25.40 5.70  -0.06 -0.18         B9Ve              t+0.004-0.011      +002           3.2   2.4      *
2413          BD+04 1335  46885114115                       063042.2+043442063600.0+042951207.17-01.37 6.55  -0.06 -0.28         B9III              -0.021-0.005      +013V      35                 *
2414  5Xi 2CMaBD-22 1458  46933171982 249                   063051.9-225308063503.4-225753231.99-13.68 4.54  -0.05 -0.03 -0.02   A0V                +0.011+0.016 +.025+026V     138
2415          CD-32 3168  469361969552506                   063052.8-323814063435.3-324259241.29-17.54 5.62  -0.08 -0.23         B8V                +0.005-0.001      +028V
2416          CP-52  947  47001234574                       063110.4-521507063326.1-521944261.18-23.80 6.19  +1.09               G8III              -0.044+0.040      +006
2417          BD+24 1328  47020 78557                       063119.4+244026063727.2+243527189.38  8.07 6.44R +0.10 +0.05         A3V                -0.016+0.008      -002
2418          BD-05 1710  470541334692509                   063139.9-050741063635.3-051240215.89-05.67 5.52  -0.08 -0.39         B8V               e 0.000-0.017      +027      245                 *
2419 51    AurBD+39 1690  47070 59316 250I                  063143.7+392845063839.5+392327175.79 14.54 5.69  +1.35 +1.56         K5III              -0.023-0.109      +033                          *
2420 52Psi3AurBD+40 1665  47100 59319                       063151.3+395918063849.2+395409175.32 14.77 5.20  -0.07 -0.40         B8III              -0.009-0.012      +009V?    169
2421 24Gam GemBD+16 1223  47105 95912 251I W                063156.1+162905063742.7+162357196.77  4.45 1.93   0.00 +0.04 -0.01   A0IV               +0.042-0.042 +.033-013SB     32  9.0 143.5AC   3*
2422          BD+06 1309  47129114146              V640 Mon 063202.4+061310063724.1+060807205.87-00.31 6.06  +0.05 -0.88         O8p                +0.006-0.001      +025SB2O   80                 *
2423  6Nu 1CMaBD-18 1480  47138151694        5253A          063200.1-183439063622.8-183936228.11-11.61 5.70  +0.85 +0.49         G8III+F3IV-V       -0.011+0.017      +025SB         1.9  17.4      *
2424          CD-36 3009  47144196978      W                063155.6-364157063524.1-364648245.35-18.84 5.59  -0.14 -0.48         A0V                -0.011+0.018D+.011+023V          0.8   1.5AB   3*
2425 53    AurBD+29 1293  47152 78571      D                063202.5+290412063823.0+285903185.47 10.17 5.79   0.00 -0.07         B9npEu             -0.021-0.021      +018V?     29  0.0   0.1      *
2426          BD+10 1201  47156 95914                       063204.4+105619063736.9+105111201.71  1.90 6.38                      K0                 -0.039-0.039      +002
2427 50Psi2AurBD+42 1585  47174 41239    I W                063211.4+423437063919.9+422920172.89 15.86 4.79  +1.23 +1.29 +0.60   K3III              +0.006-0.058 +.004+017V    < 17  5.5  52.1AB   3
2428          BD-13 1570  47182151700    I                  063210.4-131412063646.7-131915223.24-09.23 5.97  +1.56               K5                 +0.011-0.023      -012
2429  7Nu 2CMaBD-19 1502  472051517022510I          3047    063219.3-191014063641.0-191521228.69-11.80 3.95  +1.06 +1.01 +0.51   K1III              +0.059-0.062 +.058+003     < 17
2430          BD+02 1315  472201141542512                   063226.9+024725063740.3+024215208.95-01.82 6.17  +1.08 +0.96         K1III              -0.038-0.049      -008
2431          CD-35 3005  47230196987      W                063223.3-360012063554.0-360520244.70-18.50 6.35  +0.48               G0V                -0.058-0.095D+.028+028SB         0.4   0.2      *
2432          BD+05 1334  47240114162                       063233.8+050231063752.7+045726206.98-00.75 6.15  +0.14 -0.74         B1Ib               -0.004+0.020      +033V     126:                *
2433          BD-22 1472  47247172021        5260           063228.7-223152063640.9-223654231.82-13.19 6.35  -0.11 -0.54         B5V                -0.031-0.006D+.012+034V          3.6   9.1      *
2434          BD+44 1506  47270 41245                       063242.6+440607063958.0+440050171.46 16.55 6.41R                     K1III              +0.033-0.003      -030SB
2435          CP-52  953  473062345892508                   063246.3-525338063458.6-525832261.93-23.73 4.39  -0.02 -0.15 +0.14   A0II               -0.003+0.007      +023       45
2436          BD+22 1416  47358 785862513                   063304.3+220708063905.3+220151191.86  7.27 6.04  +1.03               G9III              +0.011-0.025      -009
2437          BD-12 1566  47366151725                       063303.7-125345063740.9-125906223.03-08.89 6.12  +1.00               K0                 +0.030-0.124      +009V?
2438 54    AurBD+28 1196  47395 78593        5289   3065    063314.7+282105063933.1+281547186.24 10.10 6.03  -0.08 -0.48         B7III              -0.006-0.014D+.002+019SB     70  1.6   0.8
2439          BD+24 1343  47415 78596                       063323.4+244109063931.4+243600189.58  8.50 6.38  +0.53               F8IV               +0.006+0.084      +018SB
2440          BD-02 1691  47420133511    I                  063319.0-022726063820.5-023237213.71-04.07 6.14  +1.48 +1.77         K2                 -0.018+0.015      +027
2441          BD+04 1365  47431114194                       063331.1+044718063849.5+044202207.31-00.65 6.57  -0.07 -0.33         B8IIIn             +0.012-0.008      +033                          *
2442          BD+01 1443  47432114191               3060    063327.0+014203063838.1+013649210.03-02.11 6.21  +0.15 -0.82         O9.5II             -0.007-0.005      +058V?    100                 *
2443  8Nu 3CMaBD-18 1492  47442151730    I                  063329.5-180903063753.4-181415227.87-11.11 4.43  +1.15 +1.04 +0.60   K1-II-III          -0.010-0.007 +.013-002     < 17                 *
2444          CD-38 2782  47463197011      W                063338.1-380344063701.9-380848246.82-19.00 6.04  +1.03 +0.84         K0III              +0.006+0.040      +029           4.0  25.0AC   4
2445          CD-41 2488  47475218060                       063340.8-412819063651.3-413325250.22-20.17 6.34  +1.15               K0II               +0.008+0.008      +014
2446          CD-36 3031  47500197014      W                063345.8-365418063713.8-365926245.69-18.57 5.71  -0.12 -0.51         B7IV               -0.014+0.017      +021V          0.8   0.6
2447          CD-32 3216  47536197019    I                  063402.8-321517063747.6-322023241.18-16.78 5.27  +1.18 +1.12         K2III              +0.105+0.068 +.017+079                          *
2448          BD-16 1554  47561151737                       063408.0-164706063835.4-165225226.69-10.37 6.03  +0.03 +0.10         A1V s              -0.008-0.023      +010       44
2449          BD+13 1356  47575 95963        5302           063409.9+130422063947.6+125859200.05  3.35 5.97  +0.06 +0.11         A2V                -0.034-0.001      -020       60  6.0   2.9
2450          BD-14 1525  476671517512515I                  063442.4-140322063916.7-140845224.26-09.04 4.82  +1.50 +1.68 +0.73   K2II               +0.004-0.004 -.003+029     < 17                 *
2451   Nu  PupCD-43 2576  47670218071 252           3062    063442.0-430629063745.7-431146251.94-20.54 3.17  -0.11 -0.41 -0.07   B8III              +0.002-0.006      +028SB    228                 *
2452          BD+36 1482  47703 593652516                   063454.8+360131064137.7+355555179.30 13.69 6.46  +0.49               F8III              -0.017-0.022      +086    =< 10
2453 25    GemBD+28 1207  47731 78636      W                063502.7+281721064121.0+281147186.47 10.42 6.42  +1.09 +0.85         G5Ib               +0.008-0.009      -006           3.9  55.9AC   3
2454          BD+06 1338  47756114244                       063509.8+062746064031.8+062218206.02  0.49 6.51  -0.14 -0.47         B8IIISi            -0.029+0.012      +035       35
2455          CD-23 4172  47827172102      W                063526.6-233617063936.3-234144233.11-13.02 6.05  -0.03 -0.06         A0                 +0.013-0.002      +026           0.4   0.1
2456 15    MonBD+10 1220  47839114258    I   5322  S Mon    063528.2+095918064058.7+095344202.94  2.20 4.66  -0.25 -1.07 -0.22   O7V((f))           +0.003-0.006 -.004+033SB     63  3.5   2.9AB  13*
2457          BD+16 1242  47863 95996                       063535.7+162927064121.8+162351197.17  5.23 6.28  -0.01 -0.07         A1V                -0.022-0.008      +022       38
2458          BD+11 1273  47886 95997    I                  063544.1+110546064117.2+110012201.98  2.77 6.11  +1.68               M1III              -0.002+0.009      +016
2459 55Psi4AurBD+44 1518  47914 412882517I                  063548.3+443714064305.0+443128171.18 17.26 5.02  +1.48 +1.83         K5III              -0.042-0.028 +.019-073V    < 17
2460          CD-30 3386  47946197057                       063552.8-302226063942.7-302813239.54-15.69 5.71  +1.14 +1.10         K0III              -0.006-0.182 +.019-024
2461          BD+00 1546  47964114269                       063556.9+003519064105.4+002943211.31-02.08 5.79  -0.10 -0.37         B8III              -0.016-0.003      +023       95
2462          CD-48 2417  47973218093      W        3072    063557.6-480750063837.6-481313257.12-21.91 4.93  +0.87 +0.60 +0.43C  G8III              +0.008+0.012 +.012+028           2.3  13.0      *
2463          BD+53 1056  47979 25916                       063606.9+532354064411.6+531747162.53 20.41 6.27  +1.08               K0                 +0.050-0.177      +019
2464          BD+37 1567  48073 59406                       063626.3+371439064313.8+370849178.29 14.47 6.19  +1.03               K0                 +0.036-0.041      -041
2465          CD-38 2817  48087197064                       063632.4-380356063956.9-380932247.04-18.47 6.58  +1.18               K1IIICNIb          +0.056-0.027      +018
2466 26    GemBD+17 1357  48097 96015                       063634.9+174435064224.3+173843196.16  6.02 5.21  +0.06 +0.01         A2V                +0.005-0.087      +015SB     99
2467          BD+06 1351  48099114293    I                  063637.1+062625064159.3+062042206.21  0.80 6.37  -0.05 -0.94 -0.05   O6.5V             e+0.010-0.006      +031V      70:                *
2468          CP-61  688  48189249604      W                063656.2-612641063800.6-613159271.21-25.19 6.18  +0.62 +0.12         G1-2V              -0.011+0.084      +032           2.0   2.1      *
2469          BD-09 1601  48217133585    I                  063709.9-090414064156.4-091002220.05-06.26 5.19  +1.53 +1.89 +0.68E  M0III              +0.043-0.040 +.031+001V?                        *
2470 12    LynBD+59 1015  48250 25939        5400AB         063724.0+593234064614.1+592630156.31 22.48 4.87  +0.08 +0.08 +0.03   A3V                -0.023-0.002D+.011-003V      90  0.6   1.7AB   4*
2471          BD+36 1494  48272 59425                       063729.0+361229064412.6+360635179.35 14.24 6.31R +0.06 +0.11         A2V                +0.014+0.014      -015V?     90
2472 NOVA 1903                                     DM Gem                                                                                                                                           *
2473 27Eps GemBD+25 1406  48329 78682 254I   5381   3183    063746.7+251349064355.9+250752189.54  9.63 2.98  +1.40 +1.46 +0.61   G8Ib              e-0.006-0.013 +.017+010SB   < 17  6.0 110.3      *
2474          BD+03 1371  48348114312    I                  063752.1+030755064306.5+030200209.28-00.47 6.19  +1.37 +1.60         K0                 -0.001-0.021      +031
2475          CD-40 2625  48383218126      W                063758.5-401516064114.1-402059249.30-18.99 6.12  -0.14 -0.59         B4V                -0.014+0.015      +017      217  3.4  15.4      *
2476          CD-47 2521  48403218124                       063806.4-473444064049.3-474029256.68-21.40 6.65  +1.58 +1.92 +0.98E  M2III              -0.008-0.009      +021
2477 13    LynBD+57 1004  48432 259472520                   063818.0+571623064649.5+571009158.69 21.93 5.35  +0.96 +0.73         K0III              +0.012-0.038      +019     < 19:
2478 30    GemBD+13 1390  48433 96051    I   5387           063820.9+131944064359.3+131340200.30  4.37 4.49  +1.16 +1.16 +0.60   K0IIICN1Ca1        -0.003-0.058 +.007+014     < 19: 8.4  27.2      *
2479          BD+04 1414  48434114324                       063822.1+040154064338.7+035556208.54  0.06 5.90  -0.02 -0.89         B0III              +0.006-0.006      +035       70                 *
2480 28    GemBD+29 1327  48450 786922518I          3188    063825.2+290420064445.5+285815186.08 11.43 5.44  +1.45 +1.64         K4III              -0.004-0.026      +016V?
2481          BD-22 1505  48501172204        5377A  3181    063833.3-222111064245.8-222657232.26-11.84 6.13  +0.34 -0.02         F0                 -0.085+0.078      +044           2.1  18.2      *
2482          CD-38 2844  48543197108      W                063853.2-381803064216.4-382355247.45-18.13 6.29  +0.34 +0.13         A3V+F/G            -0.015+0.005D+.003+063V          1.5   7.9      *
2483 56Psi5AurBD+43 1595  48682 41330 255    5425           063931.9+434037064644.3+433439172.36 17.52 5.25  +0.56 +0.05         G0V                -0.002+0.165 +.068-024    =<  6  3.4  36.2      *
2484 31Xi  GemBD+13 1396  48737 96074 256I:         3193    063940.6+130013064517.4+125344200.74  4.50 3.36  +0.43 +0.06 +0.23   F5III              -0.116-0.191 +.055+025V?     70                 *
2485          BD+55 1122  48767 25963        5436B  3202    063952.1+554849064812.9+554216160.26 21.69 6.33H                    dF6                 +0.057-0.101 +.032+012SB         0.1   4.7AB   3*
2486          BD+55 1122  48766 25962        5436A  3202    063951.5+554848064812.3+554216160.26 21.69 6.28H +0.47              dF5                 +0.058-0.101 +.032+009V          0.1   4.7AB   3*
2487 57Psi6AurBD+48 1436  48781 413461176I                  064002.1+485343064739.6+484722167.26 19.47 5.22  +1.12 +1.04         K0III              -0.001+0.006      -008V    < 19:
2488          CD-39 2798  487971971342519                   064003.0-390531064323.3-391136248.31-18.20 6.30  +0.26               A8:mDel Del        -0.001-0.019      +010V                         *
2489 32    GemBD+12 1275  48843 96084               3197    064017.1+124752064554.2+124137200.99  4.54 6.46                      A9III              +0.001+0.002      +009    =< 15
2490 42    CamBD+67  454  48879 139732523                   064031.5+674056065057.1+673419147.80 24.88 5.14  -0.17 -0.63         B4IV               +0.002+0.002 +.010+005SB    126
2491  9Alp CMaBD-16 1591  48915151881 257I   5423           064044.6-163444064508.9-164258227.22-08.88-1.46   0.00 -0.05 -0.03   A1Vm               -0.553-1.205 +.375-008SBO    13 10.3  11.2AB   4*
2492 10    CMaCD-30 3484  48917197149      W       FT CMa   064040.1-305804064428.4-310414240.53-14.98 5.20  -0.12 -0.93 -0.15   B2IIIe            v-0.015+0.008      +034      200  5.4  36.3      *
2493          CD-27 3248  48938172264                       064052.4-271447064451.9-272029237.03-13.42 6.45  +0.54 -0.02         G2V                -0.011+0.308 +.062-012V?
2494 16    MonBD+08 1486  489771143881177                   064105.1+084135064632.4+083514204.73  2.82 5.93  -0.17 -0.69         B2.5V              -0.006-0.007      +010V                         *
2495          CD-23 4325  49001172273                       064113.0-232131064523.3-232743233.46-11.72 6.05  +1.20 +1.29         K0                 -0.038+0.042      +034V?
2496  NGC 2281                                                                                                                                                                                      *
2497          CD-30 3495  49028197163      W                064112.7-302856064502.4-303510240.11-14.68 6.54  -0.11 -0.49         B8IV               -0.026+0.017      +018       60  3.9   4.4      *
2498          BD-14 1573  49048151895                       064126.6-144124064559.3-144746225.57-07.87 5.32  +0.07 +0.12         A2V                -0.025-0.017 +.024-019      205
2499          BD+18 1349  49059 96111        5447           064132.8+181807064723.5+181136196.20  7.32 6.20R +0.07 +0.01         A2V                +0.003-0.043 +.016+021V      77  0.2   0.5      *
2500          CD-31 3640  49095197171                       064138.4-314047064522.8-314737241.29-15.08 5.92  +0.48 -0.03         F6V                -0.220-0.320 +.051+032V
2501          CD-30 3505  49131197177      W       HP CMa   064142.4-305038064531.3-305656240.50-14.73 5.80  -0.19 -0.88         B2III             e+0.010+0.002D+.005+007SB         2.3   4.9      *
2502          BD-09 1644  491471519112521                   064155.0-100002064639.0-100626221.42-05.64 5.66  -0.05 -0.11         B9.5V              -0.017+0.003      +017V     149
2503 17    MonBD+08 1496  49161114410    I                  064154.0+080843064719.8+080214205.31  2.75 4.77  +1.40 +1.65 +0.70   K4III              -0.026-0.013 +.013+047     < 19:
2504 11    CMaBD-14 1584  49229151919                       064217.3-141907064651.1-142533225.33-07.52 5.29  -0.04 -0.25         B9III              -0.002+0.012      +015V     245
2505          CP-71  476  49268256326      W                064227.8-714024064057.8-714632282.50-26.48 6.51  +1.11 +1.07         K1IIICNII          +0.031-0.051      +021           3.5  10.7
2506 18    MonBD+02 1397  49293114428 258I                  064238.7+023118064751.6+022444210.38  0.30 4.47  +1.11 +1.04 +0.55   K0+IIIaBa0.2       -0.018-0.012 +.017+011SB   < 17                 *
2507          CD-39 2831  49319197192                       064244.0-392600064603.3-393224248.84-17.84 6.62  -0.13 -0.50         B6Vnne             +0.003+0.014      +016                          *
2508          BD-08 1558  49331133679    I          3213    064250.5-085322064737.1-085954220.53-04.93 5.07  +1.80 +1.88         M1+Ib-IIa          -0.016+0.005 +.002+024V                         *
2509 12    CMaBD-20 1576  49333172318              HK CMa   064244.4-205426064701.5-210056231.36-10.34 6.08  -0.19 -0.62         B7IIIn             -0.014+0.005      +015                          *
2510          CD-37 3065  49336197195              V339 Pup 064246.3-374005064612.1-374632247.13-17.18 6.21  -0.13 -0.64         B4Vne              -0.008+0.002      +015                          *
2511 43    CamBD+69  394  49340 13986 259                   064255.3+690017065342.2+685318146.42 25.38 5.12  -0.13 -0.43         B7III              +0.003+0.008 +.002-021V?    188
2512          BD+32 1414  49380 595212525I         IS Gem   064310.3+324313064941.3+323624183.13 13.89 5.71  +1.29               K3II               -0.036-0.047      -016                          *
2513          CP-52  996  49396234693                       064307.4-520539064526.1-521204261.57-22.00 6.57  +1.08               G6Iab              +0.007-0.007      +017
2514          BD-01 1386  49434133687                       064314.8-011227064819.0-011909213.75-01.29 5.75  +0.28 +0.05         F1V                -0.041-0.042      -014       82
2515          CP-52  998  49517234699                       064336.4-521807064553.7-522436261.81-21.99 5.80  +1.53 +1.82         K3III              -0.004-0.003      +023
2516 58Psi7AurBD+41 1536  49520 413802527I W                064341.7+415356065045.9+414653174.39 17.57 5.02  +1.27 +1.35 +0.46E  K3III              -0.024-0.136 -.003+061V?   < 19: 5.0  40.9AB   3*
2517          BD+01 1531  49567114465                       064353.9+010651064903.7+010007211.77-00.07 6.15  -0.13 -0.68         B3II-III           +0.007-0.004      +023       87
2518          CD-37 3080  495911972151178  W                064356.0-374909064721.4-375547247.37-17.02 5.26  -0.08 -0.25         B9IV               -0.007-0.015      +047      173  5.8  65.4
2519 33    GemBD+16 1298  49606 961612526  W       OV Gem   064404.4+161859064949.8+161210198.26  6.96 5.85  -0.13 -0.52         B7III              -0.017-0.012      +013       35  7.3  27.5      *
2520 14    LynBD+59 1028  49618 26012        5514           064415.9+593402065305.1+592655156.54 23.32 5.33  +0.65               G4III+A2V          -0.006-0.039 +.005+013V?     26: 1.0   0.9AB   4*
2521          BD-02 1776  496431337181179  W                064414.2-020933064916.4-021619214.71-01.52 5.74  -0.10 +0.46         B8IIIn             -0.009-0.004      +010V          0.0   0.1
2522          BD-14 1599  49662151962        5487           064425.7-150155064857.8-150841226.21-07.38 5.39  -0.10 -0.51         B6V                +0.002-0.004D+.004+023SB    157  2.7   0.7
2523          CD-51 2078  49689234705                       064428.5-510910064652.8-511557260.67-21.52 5.40  +1.34 +1.24         K1II-III+G:p       -0.003-0.097 +.018-005
2524          CP-54 1115  49705234704      W                064439.4-543507064641.5-544141264.22-22.50 6.46  +0.86               G6III              -0.050+0.024D+.004+003           3.4   1.8
2525 35    GemBD+13 1434  49738 96179    I                  064446.8+133141065025.5+132448200.84  5.85 5.65  +1.34              gK3                 -0.002-0.002      +026V?
2526          CP-55 1063  498772347102524           3220    064521.7-552544064718.7-553224265.13-22.65 5.61  +1.52               K5III              +0.003+0.023      +035V                         *
2527          BD+77  266  49878  6022 260I                  064529.1+770618070004.0+765839137.55 26.86 4.55  +1.36 +1.66 +0.71   K4III              +0.071-0.013 +.023-026SB   < 17                 *
2528          CD-23 4438  49891172389        5498           064534.9-235739064944.0-240433234.45-11.07 6.33  +0.04 +0.08         A1                 +0.002-0.003D+.002-002           1.3   1.6AB   4*
2529 36    GemBD+21 1405  49908 788052530    5511           064533.4+215245065133.0+214540193.38  9.76 5.27  -0.02 +0.01         A2V                -0.008-0.038 +.010+034V     135  8.8  11.3      *
2530          BD-00 1462  49933133760        5505           064543.4-002510065049.9-003227213.34-00.38 5.77  +0.39 -0.07         F2V                +0.030-0.185 +.036-015V   =< 10: 5.5   5.9      *
2531          CP-72  522  499472563302522                   064539.7-730028064336.8-730705284.02-26.43 6.37  +0.96 +0.66         G8III              +0.012-0.093      +016
2532          BD+45 1359  49949 41406                       064550.1+445739065307.6+445022171.53 19.04 6.26  +0.21 +0.12         A8Vn               +0.008-0.081      +006SB?   260
2533          BD+23 1518  49968 78816    I                  064555.7+234312065200.0+233606191.74 10.64 5.65  +1.45 +1.96         K5III              -0.040-0.009      +040V?                        *
2534          BD-07 1592  49976133761              V592 Mon 064553.3-075529065042.3-080228220.02-03.82 6.29   0.00 +0.02         A2pSrCrEu         v-0.006 0.000      +022                          *
2535          BD-16 1624  49980151992    I                  064554.5-165806065021.9-170502228.12-07.93 5.79  +1.44               K0                 +0.013+0.015      +023V
2536          CP-70  560  50002256331                       064556.1-701927064456.1-702602281.07-25.98 6.11  +1.33 +1.50         K3III              +0.007+0.007      +005
2537          CD-27 3310  500121724032528           3234    064606.0-271305065006.0-272002237.51-12.36 7.04  -0.18 -0.82         B2IV               -0.004+0.002      +033V?                        *
2538 13Kap CMaCD-32 3404  500131972581180          Kap CMa  064606.3-322335064950.5-323031242.36-14.49 3.96  -0.23 -0.92 -0.16   B1.5IVne          v-0.007+0.004      +014      199                 *
2539 59    AurBD+39 1771  50018 59571        5534  OX Aur   064608.6+385919065301.5+385209177.40 16.91 6.12  +0.37 +0.12         F2VDel Del        v+0.005+0.008      +001V     105  3.9  22.3AB   3*
2540 34The GemBD+34 1481  50019 59570 261    5532           064611.9+340455065247.3+335740182.10 15.02 3.60  +0.10 +0.14 +0.08   A3III              -0.002-0.048 +.021+021SB    128  7.9  78.7AB   3
2541 60    AurBD+38 1636  50037 59576      S                064621.9+383346065313.4+382617177.83 16.79 6.30  +0.49              dF5                 +0.034-0.176 +.005+032                          *
2542          BD+35 1511  50056 59574    I          3250    064621.9+355429065303.4+354719180.38 15.77 6.01  +1.45               K3III:             -0.020+0.015      +006
2543          BD+03 1437  50062114525                       064624.9+030940065139.3+030230210.25  1.44 6.38  +0.04 +0.09         A2V s              -0.010-0.042      +024V      70
2544          CD-25 3691  50093172420                       064632.5-253941065036.9-254641236.11-11.61 6.33  -0.18 -0.77         B2V                -0.006+0.014      +021
2545          CD-31 3717  50123197263      W                064636.5-313522065023.3-314222241.64-14.07 5.70  +0.14 -0.52         B6Vnpe            v-0.002+0.012      +020V     270  2.1  42.9AB   3*
2546          CD-45 2773  50196218236      W        3236    064702.7-451954064957.8-452700254.96-19.21 6.55  +1.51 +1.83 +0.82C  K5-M0III           +0.019-0.044      +015           4.6   6.6      *
2547 61Psi8AurBD+38 1638  50204 59589                       064705.7+383739065357.0+383018177.82 16.95 6.48  -0.05 -0.19         B9.5pSi            -0.010-0.026      +027                          *
2548          CD-46 2703  502232182352529                   064705.2-463029064954.6-463653256.12-19.60 5.14  +0.45 -0.02 +0.26C  F5V                +0.004+0.370 +.049+020        0                 *
2549          CD-34 3140  50235197277                       064714.1-341457065052.4-342202244.21-15.02 4.99  +1.38 +1.56         K5III              +0.007+0.006 -.013+030
2550   Alp PicCP-61  720  50241249647 262                   064709.8-615002064811.4-615629271.92-24.10 3.27  +0.21 +0.13 +0.13   A7IV               -0.069+0.269 +.052+021      205
2551          BD+08 1543  50277114556    I:                 064723.0+083007065249.4+082249205.62  4.12 5.77  +0.26 +0.10         F0Vn               -0.050-0.032      +027      195
2552          BD-05 1845  50282133807                       064727.6-051144065222.9-051858217.77-02.21 6.30  +0.99 +0.76         K0                 -0.020-0.008      +031
2553   Tau PupCD-50 2415  50310234735 263                   064727.2-502943064956.2-503653260.16-20.86 2.93  +1.20 +1.21 +0.60   K1III              +0.036-0.070      +036SB1O                      *
2554          CP-53 1168  50337234737              V415 Car 064740.9-533019064951.3-533720263.24-21.76 4.40  +0.92 +0.61 +0.45   G6II                0.000+0.030 +.025+026SBO                       *
2555          BD+11 1344  50371 96241                       064749.5+110720065322.5+105947203.33  5.42 6.24  +0.97               K0III              +0.011-0.136      -034
2556          BD+46 1202  50384 41426                       064753.8+455709065515.2+454935170.67 19.72 6.34  +0.94 +0.65         K0III-IV           -0.037-0.067      +031                          *
2557          BD+44 1551  50420 41429              V352 Aur 064801.6+440205065514.7+435436172.59 19.08 6.13  +0.32 +0.19 +0.19   A9III              +0.010-0.009      -007V      30                 *
2558          CD-36 3189  50445197291                       064810.8-360629065142.3-361349246.06-15.58 5.96  +0.18               A3V                -0.048-0.064      +004
2559   Zet MenCP-80  196  50506258451 264                   064822.5-804230064002.7-804849292.60-27.21 5.64  +0.20 +0.15         A5III              -0.016+0.055      +007V     157
2560 15    LynBD+58  982  50522 26051    I   5586           064837.0+583314065716.5+582521157.77 23.60 4.35  +0.85 +0.52 +0.44   G5III-IV           -0.005-0.135 +.014+009V    < 25  1.2   0.9AB   4*
2561          BD+57 1017  50551 26050               3280    064840.4+574125065713.2+573348158.68 23.38 6.05  +1.49 +1.75         K3III              +0.018+0.022      -054V
2562          CP-60  712  50571249654                       064841.5-600759065001.1-601457270.18-23.49 6.11  +0.46               F7III-IV           +0.009+0.109      +023
2563          CD-48 2556  50621218261                       064851.1-481017065132.8-481733257.90-19.89 6.42  +1.21 +1.23         K2-3III            +0.039-0.009      -009
2564 38    GemBD+13 1462  50635 96265        5559   3266    064900.1+131818065438.7+131040201.51  6.67 4.65  +0.30 +0.07 +0.20   F0Vp               +0.078-0.083 +.043+024SB    126  2.9   7.0AB   3*
2565          BD-18 1591  50644152055                       064856.6-185435065318.8-190158230.20-08.15 5.64  +0.28 +0.14         F1V                -0.049+0.003      +030
2566          BD-18 1594  50643152056                       064859.1-184838065321.7-185600230.11-08.10 6.14  +0.15 +0.13 +0.05   A5m                -0.028+0.023      +041
2567          CD-26 3529  50648172504    I   5548   3259    064859.1-264959065300.1-265727237.43-11.61 6.40  +1.53 +2.29         M4III              -0.031-0.061      -013V?         7.5  11.9AB   3*
2568   Psi9AurBD+46 1203  50658 41446              Var?     064908.4+462404065632.3+461627170.30 20.07 5.87  -0.06 -0.45         B8IIIe             +0.018+0.013      -041      270                 *
2569 37    GemBD+25 1496  50692 78866                       064909.7+253003065518.6+252232190.42 12.06 5.73  +0.57  0.00         G0V                -0.042+0.022 +.055-011V   =<  3
2570          BD-05 1863  50700133855        5557           064914.5-054340065408.5-055109218.45-02.07 6.41  +0.17 +0.16         A6Vn               -0.020-0.001D+.003+017           0.1   1.2
2571 15    CMaBD-20 1616  507071725202532          EY CMa   064913.4-200602065332.9-201327231.30-08.62 4.83  -0.21 -0.96 -0.22   B1IV               -0.009+0.004      +028V      49                 *
2572          BD-00 1487  507471338702533                   064919.7-010006065424.6-010737214.27  0.15 5.45  +0.18 +0.16         A4IV               +0.003-0.012 +.009-009       70                 *
2573          BD+46 1205  50763 41450                       064931.3+465009065656.0+464218169.89 20.28 5.86  +1.08              gK0                 -0.102-0.090      +039
2574 14The CMaBD-11 1681  50778152071 266I                  064932.6-115448065411.4-120219223.98-04.85 4.07  +1.43 +1.70 +0.78   K4III              -0.137-0.013 +.022+097V?   < 19:                *
2575          CD-42 2793  50785218272                       064931.6-422252065239.7-423016252.22-17.73 6.52  +0.42 +0.12 +0.22C  F5II-III           -0.002-0.008      +041
2576          CD-28 3554  50806172524                       064935.1-282412065333.9-283223238.95-12.16 6.04  +0.71 +0.27 +0.24E  G5IV               +0.274-0.438 +.042+072V                         *
2577          BD-01 1446  50820133881               3271    064938.5-013751065442.1-014523214.87-00.08 6.21  +0.56 -0.35 +0.66   B3IVe+K2II         +0.006+0.006      +009SB    130                 *
2578          CD-24 4565  50853172531                       064947.4-242450065355.3-243221235.29-10.40 6.21  +0.01 +0.10         A1                 -0.027-0.012      +027V?                        *
2579          CD-43 2756  50860218278      W                064945.2-435108065247.1-435833253.67-18.23 6.46  -0.10 -0.38 -0.05C  B8V                -0.005+0.001      +026           6.0  10.0
2580 16Omi1CMaCD-24 4567  50877172542    I         Omi1 CMa 064958.9-240332065407.9-241102234.98-10.21 3.87  +1.73 +1.99 +0.82   K2+Iab             -0.008+0.013 +.002+036     < 19:                *
2581          BD+70  430  50885  6041    I W                064959.8+705634070121.5+704829144.42 26.31 5.68  +1.33              gK4                 +0.017-0.019      -017           5.3 117.0
2582          BD-02 1827  50890133890                       064957.8-024038065458.8-024813215.83-00.49 6.04  +1.10 +0.85        gG6                 -0.016+0.002      +020
2583          CD-23 4553  50896172546              EZ CMa   065003.3-234810065413.0-235542234.76-10.08 6.91  -0.28 -0.89 +0.06   WN5-B             v-0.009+0.003      +130SBO                       *
2584          BD+08 1562  50931114626                       065008.2+082706065534.6+081929205.98  4.70 6.29  +0.04 +0.06         A0V                -0.019+0.017      +020V?     70
2585 16    LynBD+45 1367  50973 414632537           3293    065019.3+451327065737.1+450539171.55 19.87 4.90  +0.03 +0.05 +0.01   A2Vn               -0.021-0.004 +.006-008V     201                 *
2586          BD+33 1433  51000 59631               3289    065026.5+334837065700.5+334052182.72 15.72 5.89  +0.88               G5III              -0.019 0.000      -010V
2587          CP-53 1177  51043234762                       065039.2-535757065246.9-540524263.86-21.48 6.57  +1.08               G5Ib-II            -0.019+0.016      +014
2588 17    CMaBD-20 1624  51055172569        5585           065043.6-201638065502.7-202417231.62-08.39 5.74  +0.08               A3IV               -0.003-0.004      -015           3.2  50.5AC   4*
2589          BD+10 1335  51104 962942535                   065055.6+100510065625.8+095723204.61  5.62 5.92  -0.08 -0.35         B8Vn               -0.015-0.014      +033V?    175
2590 19Pi  CMaBD-19 1610  51199172579        5602   3285    065117.2-200032065537.4-200811231.44-08.15 4.68  +0.37 +0.05 +0.19  gF2                 +0.047+0.043 +.034+008      120  5.0  11.6      *
2591          CD-42 2818  512082182962534          NP Pup   065117.7-421419065426.7-422156252.21-17.38 6.32  +2.24 +2.79 +1.30   C3II               +0.002+0.014      +032                          *
2592          CP-59  716  51210234764                       065117.3-591301065245.4-592028269.31-22.93 6.41  +0.18               A3m                -0.012+0.039      -002SB2                       *
2593 18Mu  CMaBD-13 1741  51250152123    I   5605           065131.6-135451065606.6-140237225.99-05.34 5.00  +1.18               G5III+A2           -0.001+0.005 -.002+019V          2.5   2.8AB   4*
2594          CD-50 2458  51266234774                       065133.6-502926065402.3-503642260.38-20.24 6.26  +0.99               K0-1III            -0.049+0.218 -.003+044
2595          BD-22 1602  51283172588                       065134.4-224844065546.8-225629234.01-09.33 5.30  -0.18 -0.80         B3II-III           -0.017+0.003      +038      248                 *
2596 20Iot CMaBD-16 1661  513091521262536          Iot CMa  065140.6-165528065608.2-170315228.70-06.68 4.37  -0.07 -0.70 -0.07   B3II               -0.003+0.002 +.005+041       29                 *
2597          BD+12 1361  51330 96314                       065150.7+120221065725.7+115427202.96  6.71 6.27  +0.34?+0.27?        F2Ib-II            +0.019 0.000      +009
2598          CD-31 3808  51411197371      W                065207.8-313937065554.8-314724242.21-13.03 6.36                      B3V                -0.006+0.013      +021SB2        7.6  18.8      *
2599          BD-07 1642  51424133937                       065211.3-080251065700.1-081044220.85-02.49 6.34  +0.64 +0.26         K0II-III+A2V       -0.007+0.002      -005
2600 62    AurBD+38 1656  51440 596582538I                  065214.0+381122065902.8+380308178.64 17.72 6.00  +1.23 +1.17         K2III              -0.043-0.122      +025V
2601 39    GemBD+26 1405  51530 78929      W                065237.6+261245065847.4+260452190.09 13.07 6.10  +0.46 +0.02         F7V                -0.168+0.093 +.025+006    =< 10  6.0  28.8
2602   Iot VolCP-70  572  51557256344 267                   065235.7-705020065127.0-705748281.74-25.53 5.40  -0.11 -0.38         B7IV               +0.001+0.023      +019      145                 *
2603          BD-22 1616  51630172631              HH CMa   065300.2-220413065714.8-221212233.48-08.71 6.61  -0.19 -0.80         B1-2III:           -0.006-0.015      +032V                         *
2604          CD-35 3225  51682197388                       065310.2-351233065645.7-352030245.63-14.29 6.29  +1.29               K2III              -0.001-0.003      +018
2605 40    GemBD+26 1411  51688 78947      D                065317.4+260300065927.9+255451190.31 13.14 6.40  -0.11 -0.47         B8III              -0.011-0.008      +012V      67  0.0   0.0      *
2606          BD+07 1539  51693114713                       065314.2+074527065839.0+073719206.95  5.06 6.27  +0.11 +0.10         A3V                -0.016-0.030      -012V?    115
2607          CD-24 4648  51733172644        5629           065326.3-243000065733.9-243750235.73-09.70 5.46  +0.36 +0.03        dF0                 -0.058+0.106 +.016+020V?     92  1.3   1.1
2608          CD-48 2601  51799218324                       065336.2-483521065616.0-484316258.60-19.29 4.95  +1.69 +1.92 +1.18   M1III              +0.006+0.010 -.003+022
2609          BD+87   51  51802  1168 909          OV Cep   065344.3+871220074030.5+870112126.22 27.72 5.07  +1.63 +1.97 +0.88E  M2IIIab            -0.045-0.028 +.003-025V                         *
2610          BD+03 1488  518141147222540    5648           065341.4+034417065857.0+033608210.58  3.31 5.97  +1.06 +0.88         G8III              -0.002-0.004      +017           6.7   3.6
2611          CD-27 3460  51823172650                       065342.7-272413065742.4-273215238.42-10.91 6.23  -0.14 -0.66         B2.5V              -0.034+0.005      +024SB     80                 *
2612          CD-35 3233  51825197402      W                065342.9-352227065717.6-353027245.83-14.26 6.23  +0.46 -0.04         F8IV-V             -0.037+0.008 +.069+010V?         0.2   0.2      *
2613          BD+07 1544  51892114731                       065356.1+072712065920.1+071901207.30  5.08 6.35  -0.11 -0.47         B7III              -0.019-0.011      +005V      70
2614          CD-26 3646  51925172656      W        3316    065406.6-270146065807.6-270953238.12-10.67 6.37  -0.19 -0.82         B2.5III            +0.006-0.008      +018V     205  0.4   0.2      *
2615 41    GemBD+16 1354  52005 96363    I                  065431.0+161302070015.8+160444199.48  9.16 5.68  +1.66 +1.82 +0.85   K3Ib               -0.007-0.005      +022
2616          CD-25 3864  52018172669        5651           065430.0-251642065835.9-252450236.55-09.82 5.59  -0.16 -0.70         B3V                -0.010+0.015      +028SB    125  7.0  10.2      *
2617          BD+70  432  52030  6074    I                  065433.0+705236070551.8+704355144.56 26.67 6.50R                     K0III              +0.037-0.008      +020
2618 21Eps CMaCD-28 3666  52089172676 268I   5654           065441.7-285009065837.5-285820239.83-11.33 1.50  -0.21 -0.93 -0.21   B2II               +0.004+0.003D+.001+027       44  6.4   7.5      *
2619          CD-33 3389  520921974272541                   065445.4-335833065825.1-340642244.61-13.49 5.06  -0.16 -0.65         B3V                -0.006+0.005      +019V      36
2620          BD+32 1460  52100 59697        5680           065447.4+323316070117.2+322452184.30 16.06 6.59  +0.27 +0.14         A9IV               +0.011-0.020      -028       90  5.8  16.1AB   5*
2621          CD-30 3757  52140197432      W                065454.0-305142065843.8-305952241.72-12.16 6.42  -0.14 -0.61         B3V                +0.004+0.021      +014           2.6  35.0AB   3*
2622          BD-05 1910  52265134031                       065523.4-051349070018.0-052201218.72-00.48 6.30  +0.57 +0.09         G0III-IV           -0.118+0.088      +054V?
2623          BD-21 1689  52273172700                       065523.0-212753065939.3-213612233.19-07.94 6.26  -0.15 -0.78         B2-3IV-V:          -0.002-0.010      +033
2624          BD-08 1662  523121340361181  W                065535.4-081603070023.7-082425221.43-01.85 5.96  -0.08 -0.34         B9III              -0.009+0.001      +010V          6.7   1.5
2625          BD-19 1644  52348172718                       065548.3-200109070008.1-200932231.93-07.21 6.31  -0.15 -0.62         B4III              -0.028-0.008      +032V
2626          CD-45 2850  52362218360                       065547.2-453747065841.8-454605255.83-17.88 6.22   0.00 -0.06 -0.01C  A0V                -0.005-0.021      +034SB2                       *
2627          BD-08 1667  52382134041                       065552.8-090347070039.3-091211222.17-02.15 6.49  +0.21 -0.75         B1Ib               -0.007-0.006      +051       87                 *
2628          BD-21 1695  52437172725        5687  FU CMa   065604.4-215845070019.3-220710233.72-08.03 6.53  -0.17 -0.82 -0.18   B2IV-Vne           -0.006-0.018      +009           0.0   0.1AP   5*
2629          BD+05 1513  52479114798                       065623.0+045736070141.4+044905209.80  4.47 6.63  +0.06 +0.10         A3V s              -0.006-0.001      -011
2630 42Ome GemBD+24 1502  52497 789991182          Ome Gem  065619.2+242129070224.8+241255192.18 13.06 5.18  +0.94 +0.68         G5IIa-Ib           -0.009-0.001 +.012-009     < 19:                *
2631          BD+17 1479  52554 964072543I D       NP Gem   065636.5+175352070225.5+174520198.17 10.35 5.94  +1.63               M1                 +0.016+0.031      +023V          0.0   0.0      *
2632          BD+15 1431  52556 96403                       065634.5+152846070217.4+152010200.37  9.28 5.74R                    gK1                  0.000-0.016      -014
2633          BD+05 1514  52559114801                       065635.2+054159070155.0+053327209.17  4.86 6.59  -0.02 -0.64         B2IV-V             -0.041+0.010      +034                          *
2634          CP-55 1116  52603234826                       065642.6-553517065839.6-554346265.81-21.16 6.27  +1.16 +1.16         K2III              -0.073-0.090      -001
2635          BD+16 1363  52609 96409    I                  065647.1+164905070233.5+164027199.18  9.91 5.82  +1.68 +1.92         M2III              +0.017-0.015      +035V?
2636          BD-01 1509  52611134073      W                065648.3-011208070152.9-012044215.32  1.71 6.17  +1.28 +1.36         G5II               +0.010-0.027      -045           4.7  24.5
2637          CD-28 3711  52619172749                       065645.3-282050070042.6-282922239.58-10.71 6.27  +0.45               F5V                -0.006-0.034      +010
2638          CP-56 1211  52622234825                       065643.5-561519065836.2-562341266.49-21.37 6.45  +0.39               F2II               -0.030-0.028      -003
2639          BD-05 1926  52666134076    I          3341    065701.9-053446070156.4-054320219.22-00.28 5.20  +1.68 +2.05         M2III              -0.016+0.001 -.003+003V
2640          CD-25 3911  52670172763                       065659.3-250426070105.9-251255236.62-09.23 5.63  -0.17               B2V                -0.008+0.021      +006SB     67
2641          CD-33 3415  52703197475                       065707.5-331934070049.8-332755244.21-12.77 6.40  +1.05               G8II-III           +0.019+0.091      +028V
2642          BD+60 1026  52708 26141                       065711.5+595659070601.3+594807156.56 25.00 6.44R                     G8III:             +0.026+0.001      +022
2643          BD+29 1441  52711 79009                       065709.1+293018070330.4+292014187.43 15.32 5.93  +0.60 +0.04         G4V                +0.153-0.826 +.059+022V   =<  6                 *
2644          BD+52 1165  52859 26144        5746           065742.8+525430070539.8+524529164.07 23.36 6.12  +0.11 +0.88         A3V s              -0.024-0.072D+.011+027           0.2   4.2AB   3*
2645          BD+47 1391  52860 41538                       065739.2+475521070509.0+474630169.25 21.93 6.38  -0.04 -0.23         B9IIIn             -0.001-0.004      -017V?
2646 22Sig CMaCD-27 3544  528771727971183I   5719  Sig CMa  065744.1-274729070143.1-275605239.17-10.27 3.47  +1.73 +1.88 +1.00   K7Ib              e-0.005+0.005 +.024+022          10.5  10.0      *
2647          BD+09 1496  52913114835                       065749.8+091701070317.9+090818206.10  6.77 5.97  +0.12 +0.11         A3V s              -0.023+0.006      -012SB     85
2648 19    MonBD-04 1788  529181341062547          V637 Mon 065756.8-040539070254.8-041421218.01  0.61 4.99  -0.20 -0.93 -0.19   B1V               e-0.003+0.003      +025V?    336                 *
2649          BD+11 1428  52960 96429    I                  065805.6+110554070338.0+105706204.50  7.65 5.13  +1.39 +1.62         K3III              -0.008-0.019 +.011+021V    < 19:
2650 43Zet GemBD+20 1687  52973 79031 269I   5742  Zet Gem  065810.6+204302070406.5+203413195.75 11.90 3.79  +0.79 +0.62 +0.41   F7-G3Ib           v-0.009 0.000 -.006+007SB  =< 54  0.0   0.1 O   6*
2651          BD+12 1406  52976 96432    I                  065815.3+124428070351.6+123540203.04  8.42 5.98  +1.58               K5                 +0.002+0.001      -016
2652          CD-51 2224  530472348542544          Var?     065825.7-511535070051.5-512409261.54-19.48 5.14  +1.61 +1.92 +1.15   M1III              -0.015+0.016 -.014+005V?                        *
2653 24Omi2CMaCD-23 4797  53138172839 270                   065850.9-234113070301.5-235000235.55-08.23 3.02  -0.08 -0.80 -0.09   B3Iab             v-0.004+0.003      +048SB     44                 *
2654          BD+01 1665  53205114867      W                065909.3+013813070420.2+012918213.08  3.55 6.57  +0.01 -0.07         A0V                +0.002-0.012      +007       62  0.9  90.4
2655          BD-05 1943  53208134133    I                  065909.7-051034070405.2-051925219.11  0.38 5.62  +1.29 +1.24         K3III              -0.007+0.009      +040V
2656          BD-09 1818  53240152308                       065912.9-095834070357.3-100727223.36-01.85 6.45  -0.08 -0.31         B9IIIn            e-0.025-0.011      +024                          *
2657 23Gam CMaBD-15 1625  53244152303 271                   065914.0-152908070345.5-153800228.25-04.41 4.12  -0.12 -0.48 -0.10   B8II               -0.002-0.008      +032V      27                 *
2658          CD-43 2882  53253218396                       065909.8-431529070215.6-432415253.77-16.43 6.43  -0.04 -0.09         A0V                +0.015-0.012      +031
2659 44    GemBD+22 1566  53257 79042                       065917.1+224714070518.4+223814193.94 13.01 6.02  -0.03 -0.08         B8Vn               -0.006-0.013      -011      345
2660          BD+34 1524  53329 59773               3373    065936.1+343734070611.6+342826182.69 17.78 5.55  +0.91 +0.55         G8IV               -0.055-0.048 +.036+005
2661          CP-58  820  533492348632546           3349    065932.1-584757070105.1-585624269.21-21.79 6.02  +0.30               A8III              -0.058+0.144      +010                          *
2662          CP-67  686  53501249704                       070000.8-674644065950.5-675458278.61-24.22 5.17  +1.40 +1.65 +0.70   K3III              -0.033+0.241 +.037+039                          *
2663          BD+09 1510  53510114899    I                  070010.4+092014070539.1+091109206.32  7.31 5.78  +1.52 +1.87         M0III              +0.048-0.020      +046V?
2664          BD-21 1732  53629172906                       070031.5-215249070447.1-220156234.11-07.07 6.09  +1.22 +1.35         K0                 +0.020-0.058      +077
2665          BD+34 1530  53686 59794    I                  070048.2+340950070722.3+340034183.24 17.84 5.91  +1.51              gK4                 -0.028-0.025      +014SB
2666          CD-42 2929  537042184241184          Var?     070052.5-421122070402.8-422014252.87-15.72 5.20  +0.20 +0.15 +0.12   Am                 -0.008+0.072 +.025+028SB2    62                 *
2667          CD-43 2906  53705218421    I'W     A          070053.2-432809070357.3-433629254.09-16.22 5.54  +0.64 +0.04 +0.34   G3V                -0.105+0.386 +.052+086SB         1.2  21.0AB   3*
2668          CD-43 2907  53706218423    I'W     B          070054.8-432820070358.8-433642254.09-16.22 6.79  +0.80 +0.36 +0.28E  K0V                -0.109+0.371 +.052+090           1.2  21.0AB   3*
2669          BD+28 1314  53744 79066                       070108.5+281951070725.0+281038188.89 15.66 6.48  -0.09 -0.25         B9V                +0.006+0.020      -016V?    160                 *
2670          BD-10 1862  53755152363        5782  V569 Mon 070106.3-103029070549.7-103940224.05-01.69 6.49  -0.05 -0.89         B0.5V+F5III         0.000-0.029      +016SB    412  3.6  38.3AC   4*
2671          BD+22 1577  53791 79070    I W       R Gem    070120.1+225128070721.4+224213194.08 13.47 7.68  +2.21 +1.85 +3.36   S3.9e             v+0.003-0.002      -041V          4.9 172.5      *
2672          CD-49 2587  53811218427                       070117.6-492616070353.7-493502259.91-18.41 4.93  +0.13 +0.12         A4IV               -0.045+0.146 +.020+027       59                 *
2673          BD+34 1533  53899 598062549                   070139.7+335920070813.3+334956183.48 17.94 6.28  +1.34 +1.54         K1                 -0.010-0.030      -003
2674          CP-58  826  53921234890      W                070143.0-590142070315.6-591041269.54-21.60 5.50  -0.11 -0.47         B9IV               +0.022+0.009D+.014+007V          0.8   1.6
2675          BD+37 1660  53925 59812                       070150.4+373606070836.3+372642179.94 19.28 6.16  +1.21               K1III              -0.008-0.013      +010
2676          BD+05 1543  53929114935                       070148.0+050355070706.4+045437210.33  5.72 6.11  -0.13 -0.47         B9.5III            -0.033-0.020      +006V?     25
2677          CD-34 3327  53952197557      W                070154.0-343734070532.0-344640245.85-12.42 6.14  +0.37 +0.03         F2V                -0.043+0.041D+.034+019           1.3   3.1AB   5*
2678          BD-11 1790  53974152394        5795  FN CMa   070158.9-110823070640.7-111739224.71-01.79 5.39  +0.05 -0.85         B0.5IV             -0.011-0.005      +031V?    153  1.3   0.5AB   3*
2679          BD-12 1788  53975152393                       070156.7-121424070635.9-122338225.68-02.32 6.48  -0.10 -0.99         O7.5V              -0.018+0.007      +033      162                 *
2680          CD-30 3907  54031197566                       070209.1-303008070600.6-303920242.08-10.59 6.34  -0.16 -0.63         B3V                 0.000+0.009      +005SB
2681          BD+72  352  54070  61152552                   070219.3+715845071358.1+714900143.41 27.42 6.35  +1.12               K0                 +0.025+0.020      -068
2682          BD+07 1607  540791149471185                   070225.1+073742070749.5+072816208.11  7.03 5.75  +1.18 +1.08        gK0                 +0.007-0.036      +024V?
2683          CP-56 1232  54118234902 272          V386 Car 070226.4-563552070418.3-564459267.11-20.73 5.17  -0.04               A0pSi              +0.005-0.001 -.015+030        0                 *
2684 45    GemBD+16 1397  54131 96535        5812           070237.9+160526070822.0+155551200.47 10.86 5.44  +1.03 +0.80 +0.36E  G8III              -0.008-0.099 +.007-017V?         5.4  10.4AB   3*
2685          CD-38 3163  54153197572                       070236.2-381344070602.3-382258249.27-13.81 6.11  +0.70               G0IIaCH1           +0.014+0.017      +022V
2686          CD-24 4868  54173172986                       070244.7-244819070652.3-245738236.97-07.94 6.08  +1.34 +1.51         K2                 -0.031+0.007      +019V
2687          CD-50 2561  54179234907                       070243.9-501228070516.5-502137260.76-18.47 6.46                      K3III              -0.011+0.032      +013
2688          CD-26 3880  54224172989                       070256.8-263004070700.1-263928238.52-08.66 6.62  -0.17 -0.77         B2IV-V             +0.016-0.032      +004V                         *
2689   The MenCP-79  238  542392563552545                   070254.0-791636065634.4-792513291.11-26.40 5.45  +0.05 -0.07         B9.5V              -0.007 0.000      +006
2690          CD-23 4908  54309173002              FV CMa   070311.7-234104070722.6-235025236.01-07.34 5.71  -0.10 -0.89 -0.12   B2IVe             v-0.005+0.023      +029V?    290                 *
2691          CD-40 2930  54475218465              Var      070350.5-404411070707.1-405336251.72-14.63 5.79  -0.17 -0.67         B3V               t-0.007+0.004      +006                          *
2692          BD+21 1528  54563 79131                       070410.6+212516071006.7+211449195.71 13.46 6.43  +0.89 +0.50         G9V                -0.167-0.478 +.029-015V                         *
2693 25Del CMaCD-26 3916  54605173047 273I          3424    070419.5-261404070823.5-262336238.42-08.27 1.84  +0.68 +0.54 +0.33   F8Ia               -0.003+0.004 -.011+034SB     28                 *
2694          BD-10 1892  54662152470                       070436.1-101111070920.3-102050224.17-00.78 6.21  +0.03 -0.89         O6.5V              +0.005-0.020      +058       91                 *
2695          CD-23 4949  54669173064              Var?     070439.0-235302070849.3-240239236.34-07.13 6.65  -0.19 -0.81         B2V                -0.013-0.009      +057V                         *
2696 63    AurBD+39 1882  54716 59866 274I                  070446.6+392901071139.3+391914178.27 20.46 4.90  +1.45 +1.74 +0.56E  K4III-IIIa         +0.044+0.003 +.023-027SB?  < 17
2697 46Tau GemBD+30 1439  54719 59858    I   5846   3437    070446.5+302433071108.4+301443187.22 17.20 4.41  +1.26 +1.41 +0.63   K2-III             -0.030-0.045 +.007+022     < 17  6.5   1.9AB   3*
2698          CD-51 2306  54732234940                       070449.4-514841070713.3-515804262.46-18.75 5.96  +1.00 +0.82         K0III              -0.008+0.057      +029V
2699          BD-16 1802  54764152477        5837           070503.1-160424070933.2-161405229.42-03.44 6.03  +0.06 -0.79         B1II               -0.022-0.009      +006           5.3  32.5      *
2700 47    GemBD+27 1327  54801 791412553                   070510.9+270116071123.1+265124190.51 15.96 5.78  +0.10 +0.13         A4IV               -0.023-0.036      +039       80                 *
2701 20    MonBD-04 1840  548101342821186I W                070515.6-040451071013.7-041414218.85  2.23 4.92  +1.03 +0.78 +0.52   K0III               0.000+0.215 +.030+079     < 19: 4.9 186.2AC   4
2702          CD-39 3105  548931976322551           3431    070529.5-392942070851.1-393921250.69-13.83 4.83  -0.18 -0.69 -0.15   B2IV-V             -0.009+0.002      +020V       0                 *
2703          BD+51 1295  54895 262232555I         UY Lyn   070535.6+513541071323.4+512544165.80 24.18 5.47  +1.67 +1.90         M3-III             +0.006+0.014 +.003-051V                         *
2704          CD-25 4120  54912173101      W       Var?     070535.7-250409070942.9-251352237.50-07.48 5.69  -0.16 -0.77         B2.5IV             -0.012+0.006      +027SB         6.9   9.2      *
2705          BD-18 1711  54958152497      W                070545.0-183123071009.3-184107231.68-04.43 6.23  +0.40               F0                 -0.010+0.016      +033V   =< 15  5.0   0.8AB   3
2706 48    GemBD+24 1558  55052 79162                       070621.8+241745071226.4+240742193.21 15.11 5.85  +0.36 +0.09         F5III-IV           -0.022-0.048      +013V      74
2707 21    MonBD-00 1634  55057134316              V571 Mon 070616.9-000813071123.6-001807215.48  4.30 5.45  +0.29 +0.14         A8Vn-F3Vn         v-0.031-0.011 +.021+030V     121                 *
2708          CD-27 3710  55070173122                       070618.3-271941071019.4-272929239.61-08.37 5.46  +1.00               G8III              -0.003+0.009      +015
2709          BD+81  242  55075  1164                       070624.1+812622072521.9+811527132.74 28.08 6.31  -0.04 -0.09         A0III              -0.005-0.026      -008
2710          BD+05 1577  55111115062                       070631.3+054914071151.3+053917210.20  7.11 6.09  -0.02 -0.05         A1V                -0.017-0.008      +042SB     69
2711          BD+27 1337  55130 79170        5871   3453    070635.7+272340071249.0+271329190.28 16.40 6.43  +0.49 -0.02         F8V                +0.010-0.104 +.027-013V?  =< 10  0.1   1.1AB   3*
2712          CP-68  591  55151249740                       070636.3-684042070614.1-685014279.73-23.86 6.47  +1.04 +0.87         K0III              -0.004+0.008      -014
2713          BD+05 1580  55184115065                       070647.9+053827071207.4+052829210.39  7.09 6.16  +1.14 +0.97         K0III              -0.036+0.005      +020
2714 22Del MonBD-00 1636  551851343301187    5864           070645.4-001937071151.9-002934215.70  4.32 4.15  -0.01 +0.02 +0.01   A2V                -0.001+0.005 +.016+015V     142  9.2  32.0
2715 18    LynBD+59 1065  55280 262482558I                  070710.9+594857071554.9+593815156.99 26.20 5.20  +1.07 +1.01         K2III              -0.101-0.259 +.042+024V?   < 19:
2716          BD-20 1767  553441731682556                   070722.8-204302071141.6-205259233.80-05.11 5.84  -0.04 -0.05         A0V                -0.003+0.022      +018V      71
2717 51    GemBD+16 1417  55383 966381188I W       BQ Gem   070737.7+161943071322.3+160932200.78 12.04 5.00  +1.66 +1.82 +1.25E  M4IIIab            +0.013-0.042 +.011-009V          5.2 149.2AB   3*
2718 26    CMaCD-25 4191  55522173193               3458    070806.6-254630071212.2-255633238.40-07.30 5.92  -0.17 -0.71         B2V                -0.009+0.011      +022                          *
2719          CD-48 2765  555262185142554                   070806.3-484617071047.5-485556259.70-17.13 5.14  +1.24 +1.29 +0.47E  K2III              -0.012+0.192 +.031+064                          *
2720          CD-30 4081  55568197686                       070812.8-303916071204.1-304918242.80-09.49 6.10  +0.27 +0.06 +0.14   A8V                -0.039+0.015      +006                          *
2721          BD+47 1419  55575 416441190                   070824.6+472503071550.1+471424170.34 23.52 5.58  +0.58 +0.03         G0V                +0.030-0.183 +.037+085V   =<  6
2722          BD+24 1576  55579 79191      W                070820.4+245256071426.6+244239192.84 15.76 6.89  -0.03 -0.07         A1pSrCr            +0.003-0.016      +003           3.5  27.5AB   4
2723          BD-11 1849  555891525702557I                  070825.0-110456071307.2-111505225.40-00.37 5.78  +1.51 +1.76         K0                 -0.003+0.008      +009
2724          CD-27 3761  55595173200              HN CMa   070822.9-271822071224.1-272827239.80-07.95 6.59  +0.19               A5IV-V             -0.025+0.008      -008                          *
2725 52    GemBD+25 1618  55621 79199    I   5909   3469    070835.0+250331071441.9+245306192.70 15.88 5.82  +1.56 +1.83         M1III             v+0.047-0.092      +047V?         6.3  23.9      *
2726          CD-36 3421  55718197694      W        3464    070852.6-362232071225.7-363240248.09-11.89 5.96  -0.14 -0.62         B3V                -0.022+0.004D+.004+017           2.5   2.5      *
2727          CD-40 2987  55719218525              Var?     070857.0-401947071215.8-402956251.76-13.57 5.31  +0.06 +0.09         A3pSrEuCr          -0.018-0.015 +.009+017SBO    66                 *
2728          BD+12 1469  55730 966722559                   070858.1+121716071432.6+120657204.63 10.56 5.62  +1.01 +0.77         G6III              -0.050-0.020      +030
2729          BD+03 1609  55751115119    I                  070905.6+031658071420.0+030641212.76  6.51 5.35  +1.19 +1.18         K2II               -0.006-0.002      +037
2730          BD-22 1756  55762173230    I          3467    070909.4-223009071323.9-224024235.58-05.58 6.01  +1.48 +1.69         K2III              +0.006-0.008      -005
2731          BD-03 1804  55775134391    I   5911           070912.2-034349071411.0-035405219.01  3.27 5.75  +1.58 +1.89         K5III              -0.015+0.007      +022V          5.0   2.7
2732          BD-09 1921  55832134395    I                  070930.3-094633071415.5-095651224.38  0.48 5.90  +1.52 +1.72         K3III              -0.004+0.004      +043V?
2733          BD-22 1761  55856173247        5912           070934.6-224404071348.3-225423235.84-05.60 6.36  -0.21 -0.83         B2IV+G5IV          -0.012-0.017      +017           1.8  19.7      *
2734          CD-27 3789  55857173244              GY CMa   070934.7-271107071336.3-272123239.81-07.65 6.12  -0.23 -1.01         B0.5V              -0.025+0.006      +043V                         *
2735   Gam1VolCP-70  600  55864256373      W     B          070933.4-702004070842.2-702950281.55-24.05 5.69  +0.40 +0.04         F2V                +0.007+0.101 +.016-003SB         1.9  13.6      *
2736   Gam2VolCP-70  600  558652563741189  W     A  3448    070935.8-702011070844.9-702956281.56-24.04 3.78  +1.04 +0.88         K0III              +0.023+0.106 +.016+003V?         1.9  13.6      *
2737          BD+52 1188  55866 26260                       070942.8+521827071733.7+520751165.21 24.97 5.92  +1.26              gK1                 -0.001-0.025      -007
2738 53    GemBD+28 1350  55870 79221    I          3485    070942.4+280417071557.1+275351189.91 17.30 5.71  +1.63 +2.00         M1IIIa             -0.016+0.001 +.008+024V?
2739          BD-10 1933  55879152598                       070943.8-100838071428.2-101900224.73  0.35 6.03  -0.18 -0.97         B0III              -0.004-0.011      +033       50                 *
2740          CD-46 2977  55892218537 275          QW Pup   070942.5-463532071233.6-464534257.72-16.02 4.49  +0.32 -0.01 +0.22   F0IV               -0.132+0.103 +.048-001       54                 *
2741          CD-30 4143  55958197719              GG CMa   070956.3-305443071347.1-310502243.20-09.28 6.60  -0.17 -0.76         B2IV               -0.014-0.002      +028SB?                       *
2742          BD+82  201  55966  11793951I         VZ Cam   071003.1+823616073104.4+822441131.42 28.18 4.96  +1.66 +1.80 +1.25E  M4IIIa             -0.006-0.042 +.001+014V                         *
2743          CD-30 4146  55985197722                       071004.1-301004071357.2-302024242.54-08.92 6.33  -0.20 -0.83         B2IV-V             -0.015-0.006      +024SB                        *
2744 24    MonBD+00 1871  56003134414        5933           071012.3+000044071519.3-000941215.81  5.24 6.41  +0.90 +0.57         G5III              -0.020-0.002D+.011-010V?         5.8   3.8
2745 27    CMaCD-26 4057  56014173264      W       EW CMa   071010.6-261048071415.2-262109238.97-07.07 4.66  -0.19 -0.71 -0.12   B3IIIe             -0.009+0.007      +000SB2   139  0.0   0.1      *
2746          CD-44 3223  56022218546              OU Pup   071013.8-450032071313.4-451059256.26-15.30 4.89  -0.02 -0.07         A0pSi              -0.019-0.099 +.010+004V?     28                 *
2747          BD+08 1712  56031115159    I          3486    071013.9+080907071539.4+075840208.53  8.99 5.82  +1.60 +1.71         M4-IIIab           +0.029+0.007      -009                          *
2748          CD-44 3227  56096218549      W       L2 Pup   071029.1-442836071332.4-443823255.77-15.04 5.10  +1.56       +2.42   M5IIIe             +0.114+0.330 +.022+053V?         6.4  62.0      *
2749 28Ome CMaCD-26 4073  56139173282              Ome CMa  071045.2-263556071448.7-264622239.41-07.15 3.85  -0.17 -0.73 -0.12   B2IV-Ve           v-0.008+0.006      +026SB?   120                 *
2750          CD-26 4074  56160173283    I                  071048.6-265148071451.1-270217239.65-07.26 5.58  +1.22 +1.29         K4III              -0.049-0.030      +015V
2751          BD+49 1612  56169 41681                       071056.1+493835071831.9+492754168.11 24.51 5.05  +0.08 +0.09 +0.06   A4IIIn             -0.004+0.014 +.008-012V     222
2752          BD-10 1945  56207152621                       071059.2-102430071543.1-103502225.11  0.50 5.95  +1.18 +1.18         K0                 +0.011-0.007      -013
2753 64    AurBD+41 1630  56221 41679 276                   071105.0+410340071802.2+405300177.08 22.11 5.78  +0.17 +0.12         A5Vn               -0.015+0.013      -010V     185                 *
2754          CP-62  789  56239249774      W                071105.4-630108071202.0-631124274.00-21.77 6.02  -0.02               A0IV-V             -0.003 0.000D+.005-000           0.4   0.4
2755          CD-23 5173  56341173319                       071135.6-233353071547.5-234426236.79-05.58 6.32R                     A0                 +0.017+0.007      +034
2756          CD-30 4184  56342197756                       071128.8-303041071521.0-304111242.99-08.80 5.36  -0.17 -0.65         B3V                -0.018+0.013      +033        0                 *
2757          BD+31 1529  56386 599642563                   071140.6+310807071804.1+305721187.09 18.85 6.24  -0.03 -0.10         A0Vn               -0.004-0.028      +027V     325
2758          BD-15 1734  564051526412561                   071142.7-152431071614.5-153509229.59-01.71 5.46  +0.08 +0.06         A2Vnp              -0.050-0.015 +.041+010V?    152                 *
2759          CD-41 2906  56410218568                       071141.3-411505071457.1-412533252.84-13.49 5.94  -0.16               B4III-IV           +0.009+0.022      +021V?
2760          BD+06 1594  56446115204                       071155.4+065133071717.8+064050209.88  8.78 6.65  -0.11 -0.41         B8III              +0.001-0.015      +009V?    200
2761          CD-46 3000  56455218570              PR Pup   071153.6-464028071445.9-465059257.96-15.71 5.72  -0.11 -0.46         A0pSi              -0.018-0.008      +040                          *
2762          CD-48 2807  56456218567      W                071152.6-480549071438.2-481618259.31-16.29 4.76  -0.10 -0.29 -0.08   B8-9V              +0.004+0.004      +039V          8.6  18.5      *
2763 54Lam GemBD+16 1443  56537 96746 277    5961   3512    071220.7+164315071805.6+163225200.92 13.23 3.58  +0.11 +0.10 +0.05   A3V                -0.048-0.037 +.047-009SB    154  1.0   0.0 O   3*
2764          CD-23 5189  56577173349    I   5951   3503    071223.8-230816071636.8-231856236.50-05.21 4.79  +1.71 +1.87 +0.96   K3Ib-II            -0.010+0.002 +.019+028V?         1.2  26.8      *
2765          BD-06 2032  56614134474    I                  071239.1-063004071731.7-064048221.86  2.72 6.29  +1.62 +1.96         K2                 -0.014+0.011      +009
2766          CD-27 3852  566181733602562I                  071234.4-274216071635.0-275252240.58-07.30 4.64  +1.60 +1.88 +1.25   M3III              -0.010+0.043 +.012+042                          *
2767          CP-52 1123  56705235040                       071258.9-521932071521.0-522959263.45-17.78 5.97  +1.10               K1III              -0.058+0.103      +039
2768          CD-30 4234  56731197789      W                071305.3-304302071657.2-305348243.34-08.59 6.32  +0.20 +0.19 +0.15   A9II               +0.005-0.021      -005V          1.7  37.9      *
2769          CD-38 3288  56733197785                       071304.3-380826071631.8-381908250.08-11.91 5.80  -0.13 -0.59         B4V                -0.015+0.005      +045V                         *
2770          CD-36 3485  56779197790              Var?     071315.8-362449071649.5-363534248.52-11.12 5.03  -0.17 -0.69         B2IV-V             +0.001-0.002      +009SB    132                 *
2771          CD-46 3023  56813218589                       071322.3-463549071615.5-464628257.99-15.45 5.66  +1.44 +1.67         K4III              +0.009+0.035      +020
2772 47    CamBD+60 1048  56820 26298        5995           071331.7+600514072217.2+595407156.84 27.03 6.35                      A8m                +0.003+0.014D+.015+007V          4.5   2.2
2773   Pi  PupCD-36 3489  56855197795 278  W        3515    071336.6-365504071708.6-370551249.01-11.28 2.70  +1.62 +1.24 +0.91   K3Ib               -0.009+0.004 +.032+016           5.3  69.2      *
2774          CD-26 4140  56876173411                       071344.2-263700071747.9-264751239.73-06.57 6.46  -0.15 -0.61         B2IV-V             -0.010+0.001      +015V                         *
2775          BD+42 1699  56941 41705                       071359.4+425030072103.1+423920175.42 23.16 6.35  +1.46 +1.58         K0                 -0.019-0.046      +046
2776          BD+45 1422  56963 417082567                   071403.1+452447072117.5+451341172.73 23.90 5.77  +0.32  0.00         A7 s               -0.048+0.004      +025
2777 55Del GemBD+22 1645  56986 79294 279I   5983           071409.0+221000072007.4+215856195.98 15.89 3.53  +0.34 +0.04 +0.19   F2IV               -0.026-0.012 +.061+004SBO   111  2.0   0.2 O   3*
2778          BD+02 1640  569891152632564                   071408.8+025527071922.4+024426213.67  7.46 5.89  +1.07 +0.82         G9III               0.000-0.013      +024
2779          BD+07 1684  570061152692565                   071423.9+071942071947.6+070834209.74  9.54 5.91  +0.53 +0.07         F8V                +0.060-0.063      +022    =< 10
2780          BD+15 1541  57049 96783                       071425.1+151939072006.9+150834202.43 13.08 6.45  -0.02 +0.07         A2Vn               +0.011-0.017      +013SB
2781 29    CMaCD-24 5173  57060173444    I         UW CMa   071430.5-242234071840.3-243332237.82-05.37 4.98  -0.15 -1.01 -0.13   O7Ia:fp           v-0.006 0.000      -011SB2O  136                 *
2782 30Tau CMaCD-24 5176  57061173446    I   5977   3528    071433.6-244618071842.4-245715238.18-05.54 4.40  -0.15 -0.99 -0.18   O9Ib               -0.009+0.010      +040SBO   112  0.0   0.2AP   5*
2783 19    LynBD+55 1192  57102 26311        6012B          071441.2+552822072250.9+551704161.96 26.36 6.53H                     B9V                +0.004-0.024 +.011+010SB    250  1.0  15.1AB   4*
2784 19    LynBD+55 1192  57103 26312 280    6012A          071442.5+552812072252.1+551653161.96 26.37 5.45                      B8V                -0.003-0.029 +.011+005SB2O   80  1.0  15.1AB   4*
2785          BD-19 1813  57118152710                       071438.5-190550071902.0-191649233.17-02.85 6.09  +0.62 +0.35         F0Iab-Ib           +0.008+0.002      +062SB?    23
2786          CD-26 4164  57146173453    I                  071446.9-262411071851.2-263509239.65-06.26 5.28  +0.96 +0.65         G2Ib               -0.017+0.008 +.017+032V                         *
2787          CD-36 3512  57150197824      W       NV Pup   071445.0-363306071818.4-364403248.78-10.91 4.66  -0.10 -0.80 -0.05   B2V+B3IVne         -0.003+0.001      +019      277  0.4 239.9AB   4*
2788          BD-16 1898  57167152724              R CMa    071456.6-161227071928.1-162342230.67-01.41 5.70  +0.35 +0.04         F1V                +0.157-0.132 +.040-039SBO    98                 *
2789          CD-43 3093  57197218618                       071458.4-434815071804.2-435912255.49-14.03 5.85  -0.12 -0.48         B8II-III           -0.015+0.001      +013V
2790          CD-36 3519  57219197837              NW Pup   071504.8-363335071838.2-364434248.82-10.86 5.11  -0.16 -0.66 -0.21   B2IVne             -0.002+0.003      +018V     124                 *
2791          CD-38 3309  57240197836                       071508.8-390138071833.6-391237251.08-11.94 5.25  +0.01 +0.07         A1V                +0.010+0.007 +.014+032       29
2792          BD+39 1927  57263 60012                       071523.8+391105072213.4+385946179.29 22.31 6.40  +1.22 +1.23         K0                 -0.003-0.038      +003
2793 65    AurBD+37 1707  57264 600102568I   6009           071521.7+365655072202.6+364538181.57 21.58 5.13  +1.08 +0.94         K0III              -0.098-0.023 +.022+023     < 17  5.5  11.4AB   3*
2794          CD-33 3696  572991978422566                   071530.3-333233071913.7-334338246.12-09.42 6.30  +1.31               K2-3III            -0.013-0.013      -002
2795 56    GemBD+20 1775  57423 79328    I   6016   3550    071602.8+203757072156.8+202637197.63 15.66 5.10  +1.52 +1.87         M0IIIab            -0.066-0.027 +.002+004           7.4  16.5AB   3*
2796          BD-14 1846  57478152749                       071623.0-141023072058.2-142136229.04-00.13 5.45  +0.98 +0.63        gG5                 -0.018+0.016      +013
2797          BD+81  252  57508  1194                       071627.2+810559073439.7+805348133.11 28.47 6.41R                    gG7                 -0.010+0.003      -002
2798          BD-08 1862  57517134563                       071628.7-084110072116.9-085242224.24  2.51 6.55  +0.54 +0.04         F5                 +0.036-0.156      -007V?  =< 10
2799          BD-22 1823  57573173517              Var      071638.6-223946072053.2-225106236.54-04.13 6.61  -0.15               B2.5V              +0.006-0.042      +010                          *
2800          CD-26 4223  57593173522        6015  HQ CMa   071651.3-264634072055.0-265749240.20-06.03 6.01  -0.17 -0.70         B2.5V              +0.010+0.019      +018V          7.2   8.2      *
2801          BD+00 1915  57608115335                       071655.5+002159072203.5+001038216.28  6.89 5.99  -0.07 -0.29         B8III              +0.001+0.004      -010V       5
2802          CD-25 4400  57615173529    I                  071657.8-254214072104.3-255329239.26-05.50 5.87  +1.64 +1.90         M4III              -0.012+0.025      +023
2803   Del VolCP-67  730  57623249809 281                   071652.8-674627071649.8-675726279.09-22.69 3.98  +0.79 +0.45 +0.38   F6II               -0.007+0.005 +.004+023        0
2804          BD+52 1205  57646 26333    I                  071709.3+520452072457.2+515314165.74 26.03 5.80  +1.61 +1.96 +0.92   K5III              +0.014-0.035      +018V
2805 66    AurBD+40 1852  57669 417381191I                  071713.0+405154072408.5+404020177.67 23.15 5.19  +1.23 +1.24         K1+IIIaCN1         -0.006-0.023 +.007+021SB?  < 19:
2806          BD-08 1872  57682134580                       071714.3-084725072202.0-085845224.41  2.63 6.43  -0.19 -1.04         O9V                +0.006+0.021      +023       35                 *
2807          BD-02 2079  57708134585                       071717.6-024721072218.5-025844219.13  5.49 6.23  +0.68 +0.35         F5                 -0.016+0.003      -015V   =< 10
2808 57    GemBD+25 1660  57727 79352      D                071722.7+251434072328.5+250302193.34 17.78 5.03  +0.90 +0.57         G8III              -0.069-0.023 +.022+006     < 19:                *
2809          BD+66  502  57742 141952571                   071731.9+663143072725.8+661954149.69 28.26 6.47  -0.09 -0.19         B9III              -0.012-0.032      +007V?      0
2810 58    GemBD+23 1698  57744 79354                       071727.6+230816072328.2+225643195.38 16.97 6.02R -0.01 -0.01         A1V                -0.022-0.036      +015      125
2811          BD-05 2089  57749134588               3560    071731.1-054732072225.4-055858221.81  4.12 5.82  +0.35 +0.19         F3IV               -0.011-0.004      +011V      42                 *
2812          BD-18 1806  578211527762570                   071749.2-184933072213.5-190100233.29-02.06 4.96  -0.04 -0.39 -0.04   B7IV               -0.006-0.008      +027      117
2813          CP-52 1153  57852235110      W     A          071756.7-520737072021.4-521842263.57-17.00 6.05  +0.42 -0.02         F0-2IV-V           -0.036+0.134 +.038+022           0.6   9.3      *
2814          CP-52 1153  57853235111      W     B          071757.0-520728072021.7-521835263.57-17.00 6.60  +0.59 +0.05         G0V:e              -0.040+0.124 +.038+035V          0.6   9.3      *
2815          CD-51 2445  57917235116                       071812.4-515348072038.7-520510263.37-16.87 5.39  -0.07               B9II-III           -0.019-0.007      +018SB     29:
2816 59    GemBD+27 1374  57927 79366                       071820.1+274952072433.4+273817190.91 18.97 5.76  +0.34 +0.08       g:F0                 +0.012+0.017      -005V      67
2817          BD+15 1564  58050 96866              OT Gem   071845.3+154241072427.7+153102202.53 14.19 6.41  -0.18 -0.84         B2Ve              v+0.011+0.004      +037      140                 *
2818 21    LynBD+49 1623  58142 417642572           3585    071910.2+492436072642.8+491241168.71 25.75 4.64  -0.02 -0.01 -0.05   A1V                -0.013-0.050 +.012+027V      16                 *
2819          CD-31 4437  58155197925                       071911.2-314351072300.6-315526244.86-07.90 5.43  -0.15 -0.68 -0.13   B5IIIn            e-0.014+0.004      +024SB                        *
2820  1    CMiBD+11 1578  58187 96871                       071924.8+115156072458.2+114011206.17 12.67 5.30  +0.10 +0.13         A5IV               -0.017-0.014 +.015-001V?    165
2821 60Iot GemBD+28 1385  58207 79374 282I                  071930.9+275949072543.6+274753190.85 19.27 3.79  +1.03 +0.85 +0.50   G9IIIbHdel -1      -0.123-0.086 +.032+008V       9                 *
2822          CD-27 4020  58215173622    I                  071927.3-273825072329.1-275003241.24-05.92 5.38  +1.53 +1.83         K2III              +0.045+0.005 +.013+048SB
2823          CD-31 4454  58286197938                       071943.2-320029072331.9-321208245.16-07.93 5.39  -0.18 -0.69         B3III-IV           -0.009+0.004      +021SB     29                 *
2824          CD-29 4322  58325197944                       071959.7-300120072354.3-301301243.41-06.94 6.60  -0.20 -0.79         B2IV-V             -0.019+0.009      +007                          *
2825          BD-15 1810  58343152834              FW CMa   072008.9-160018072440.1-161204231.09-00.21 5.33  -0.05 -0.60 -0.02   B2.5IVe           v-0.014-0.010      +002V      33                 *
2826          BD-22 1855  58346173656                       072002.5-224303072417.2-225446236.96-03.46 6.19  -0.09 -0.36         B8                 -0.016 0.000      +032V?                        *
2827 31Eta CMaCD-29 4328  58350173651 283I W                072008.3-290629072405.7-291811242.62-06.49 2.45  -0.08 -0.72 -0.06   B5Ia               -0.004+0.005      +041V      45  4.5 178.7      *
2828  2Eps CMiBD+09 1643  58367115425    I                  072010.9+092824072538.9+091634208.45 11.78 4.99  +1.01 +0.78 +0.50   G6.5IIb            -0.006-0.011 +.008-008V?   < 17                 *
2829          CD-35 3569  58420197951      W                072021.1-353834072358.3-355016248.48-09.49 6.31  -0.15 -0.55         B7III              -0.018+0.018      +028           6.2  25.0      *
2830          BD+68  480  58425 14211 284I                  072028.6+684012073052.7+682756147.28 28.69 5.64  +1.11 +0.89        gK2                 -0.010-0.038 +.011+056
2831          BD-18 1825  58439152837      W                072026.4-184857072450.7-190044233.58-01.51 6.24  +0.29  0.00?        A2Ib               -0.021+0.005      +070           4.9   2.7
2832          BD-13 2001  584611528401192  W                072032.7-133318072508.3-134507228.99  1.05 5.78  +0.42               F0V                -0.204-0.004 +.037+007V?
2833          BD-05 2112  58526134654                       072056.3-053436072551.0-054630222.02  4.97 5.97  +0.92 +0.62         G3Ib               -0.029-0.004      +014V
2834          CD-31 4482  58535197964    I W        3584    072053.9-313644072443.8-314832244.92-07.52 5.35  +1.07 +0.91         G8II               -0.020+0.010 -.021+020           2.1  99.2AC   4*
2835          BD+21 1596  58551 79386                       072055.6+214408072650.2+213209197.05 17.15 6.54  +0.46               F6V                -0.315-0.020 +.038+047SB  =< 10:
2836          BD+10 1532  58552 96899                       072057.1+104825072627.9+103630207.31 12.54 6.37  +0.09 +0.06 +0.02   A2IV               -0.021+0.005      -010V      20
2837 61    GemBD+20 1805  58579 79391      D        3592    072102.6+202727072656.3+201526198.29 16.66 5.93  +0.30 +0.10         F2Vn               -0.005-0.019      +009SB                        *
2838          BD-04 1943  58580134658                       072105.7-042018072603.5-043215220.95  5.59 6.76  -0.01 -0.11         A0IV               +0.009-0.018      -002SB2    71                 *
2839          BD-21 1925  58585173691      W                072102.8-214706072519.9-215858236.26-02.81 6.05  +0.23               A8I-II:            -0.010+0.003      +041V          7.4   2.4
2840          BD+11 1588  58599 96901      W                072109.4+111231072641.3+110033206.97 12.76 6.41  -0.13 -0.46         B6IV               +0.009-0.002      +029       50                 *
2841          CD-24 5366  586121736972574                   072116.5-250111072525.3-251304239.12-04.32 5.78  -0.10 -0.39         B7III              -0.005-0.002      +024V                         *
2842          CD-37 3549  58634197974      W     B          072114.5-370535072447.2-371724249.87-10.00 6.97  +0.22 +0.15         F0V:               +0.016+0.009D+.003+026           0.1   7.0      *
2843          CD-37 3549  58635197975      W     A          072114.9-370541072447.3-371727249.87-10.00 6.84  +0.27 +0.15         F0V:               -0.020+0.044D+.003+018           0.1   7.0      *
2844          BD+48 1538  58661 41797        6095           072124.5+482314072851.6+481102169.91 25.87 5.72  -0.10 -0.41         B9pHgMn            +0.007-0.049      +021V      32  4.6  16.5ABxC 3*
2845  3Bet CMiBD+08 1774  58715115456 285  W       Bet CMi  072143.6+082927072709.0+081722209.52 11.68 2.90  -0.09 -0.28 -0.06   B8Ve               -0.052-0.038 +.019+022SB    276  7.8 138.7AE   5*
2846 63    GemBD+21 1602  58728 79403        6089           072148.2+213859072744.4+212642197.23 17.30 5.22  +0.39 -0.04         F5V+F5V            -0.058-0.121 +.041+025SB2O   38  0.5   0.0 O   5*
2847          CD-31 4506  58766197986              Var?     072152.6-313223072543.0-314419244.96-07.30 6.31  -0.17 -0.70         B2V                 0.000+0.007      +010                          *
2848          CP-86  105  588052584601661                   072201.5-865211064658.7-870130299.59-27.15 6.47  +0.42 -0.07         F3V                +0.006+0.004      +004
2849 22    LynBD+49 1630  58855 41808      W                072220.5+495245072956.0+494021168.33 26.35 5.36  +0.45 -0.06         F6V                +0.114-0.080 +.042-027V      12  4.8 169.3AB   3*
2850          CD-23 5477  58907173743                       072227.9-233043072640.7-234244237.93-03.36 6.56  +0.01               A1                 -0.006+0.023      +027
2851  5Eta CMiBD+07 1729  58923115477        6101           072239.3+070845072802.1+065631210.85 11.28 5.25  +0.22 +0.17         F0III              -0.004-0.044 +.010+018       67  5.2   4.0      *
2852 62Rho GemBD+32 1562  58946 60118 286    6109           072240.8+315900072906.7+314704187.14 21.34 4.18  +0.32 -0.03 +0.19   F0V                +0.155+0.175 +.062-004SB     68  3.4 220.4AC   4*
2853          BD-17 1980  58954152894        6093           072240.6-173948072708.0-175152232.83-00.49 5.63  +0.32               A5n                +0.002+0.015      -029      157  5.3   1.6
2854  4Gam CMiBD+09 1660  58972115478    I   6100           072243.1+090741072809.8+085532209.05 12.18 4.32  +1.43 +1.54 +0.79   K3-IIIFe-0.5       -0.062+0.017 +.019+047SB1O < 17  5.9 130. AD   4*
2855          BD-22 1874  58978173752              FY CMa   072245.0-225305072659.4-230510237.41-03.00 5.61  -0.13 -0.98 -0.03   B0IV:pe            -0.012+0.004      +025V?    244                 *
2856          CD-33 3813  59026198000                       072259.3-335622072642.3-340827247.20-08.22 5.90  -0.17 -0.68         B2IV-V             -0.027 0.000      +007
2857 64    GemBD+28 1396  59037 79427                       072306.6+281928072920.4+280705190.83 20.13 5.05  +0.11 +0.12         A4V                -0.041-0.056 +.008+035V     202                 *
2858          BD+15 1579  59059 96930                       072306.1+151851072847.3+150634203.36 14.97 6.22  -0.05 -0.11         B9IV               +0.004-0.025      +034
2859          BD-11 1951  59067152909        6104AB 3600    072309.6-112114072751.7-113325227.36  2.67 5.79  +0.58 -0.21         G8Ib-II+B2+B8V     -0.004-0.001      +015V?         2.0   0.9AB   5*
2860          BD-22 1878  59136173778                       072327.8-223922072742.9-225135237.29-02.74 5.95  -0.09 -0.41         B5V:               -0.005-0.017      +026V?
2861 65    GemBD+28 1400  59148 79434    I   6119           072335.5+280721072948.7+275458191.07 20.15 5.01  +1.11 +1.03 +0.37E  K2III              -0.034-0.024 +.018+036SB2  < 17  8.5  12.8      *
2862          CD-50 2761  592192351922575                   072347.8-504900072621.9-510106262.72-15.63 5.10  +1.06               K0III              +0.001+0.008 +.007+008
2863          CD-28 4383  59256173799              Var?     072400.9-285707072759.2-290921242.88-05.67 5.54  -0.08 -0.15         B9VpSi:            -0.006+0.008      +004
2864  6    CMiBD+12 1567  59294 969521193I          3609    072413.8+121248072947.8+120024206.38 13.88 4.54  +1.28 +1.37 +0.64   K1+IIIBa0.4         0.000-0.019 +.029-015     < 19:
2865          BD-01 1738  593111347402578I                  072415.3-014157072918.7-015419218.99  7.54 5.59  +1.50 +1.76         K5III              -0.015-0.005      -005V
2866          BD-07 1996  59380134742                       072434.1-072054072925.7-073304224.02  4.91 5.86  +0.48 -0.02         F8V                +0.064+0.131      +009V?  =< 10
2867          BD-10 2067  593811529412579I                  072437.1-100713072922.1-101936226.46  3.58 5.75  +1.62 +1.99         K5III              -0.001+0.001      -007
2868          BD-14 1925  59438152943        6126           072448.8-144708072921.9-145957230.57  1.36 6.05  +0.47 -0.07         F7V                -0.185-0.254 +.027-006V          1.4   2.2AB   5*
2869          CD-37 3601  59466198031                       072451.7-373612072822.8-374837250.66-09.59 6.58  +0.06 +0.04         A1IV               -0.020-0.050      -001V?
2870          CD-31 4590  59499198038      W     A          072500.6-313833072851.3-315054245.36-06.76 6.38  -0.17 -0.70         B3V                +0.008+0.003      +002           0.7   8.9      *
2871          CD-31 4590  59500198039      W     B          072501.2-313827072851.5-315049245.36-06.76 7.13  -0.15 -0.60         B4V                -0.036+0.003      +004           0.7   8.9
2872          BD+39 1958  59507 60148                       072508.4+390623073155.7+385347179.99 24.08 6.54  +0.07 +0.04         A2V                -0.035-0.014      +007
2873          CD-31 4593  59550198042                       072513.2-311459072904.9-312723245.04-06.54 5.77  -0.19 -0.75         B2V                -0.014+0.002      +008SB
2874          BD-22 1897  59612173864      W                072536.5-224859072951.4-230128237.67-02.39 4.85  +0.23 +0.14 +0.27   A5Ib               -0.004-0.006 -.003+037       26  5.9   3.0
2875          CD-38 3400  596351980452580                   072537.9-383619072905.7-384844251.64-09.93 5.43  -0.16               B5Vp               -0.016+0.011      +024V      38
2876          BD-04 1979  596691347742582                   072554.9-050101073051.1-051335222.12  6.32 6.24  +1.18 +1.06         K0                 -0.006-0.002      +017
2877          BD+17 1596  59686 969852585I                  072602.4+171756073148.4+170510201.80 16.44 5.42  +1.13              gK2                 +0.043-0.078 +.001-040V?
2878   Sig PupCD-43 3260  597172187551194  W                072603.4-430556072913.8-431805255.74-11.91 3.25  +1.51 +1.78 +0.92   K5III              -0.055+0.187 +.020+088SB1O       5.3  22.3      *
2879          BD+23 1744  59878 79489        6160           072651.0+230603073250.6+225316196.30 18.95 6.54  +1.01 +0.77         K0II-III+F8V       -0.025-0.012      +030SB         2.2  11.5      *
2880  7Del1CMiBD+02 1691  598811155812587                   072654.2+020735073205.9+015452215.88  9.92 5.25  +0.22 +0.20         F0III              -0.003-0.003 +.016+029V?     75
2881          CD-30 4620  59890198064    I                  072649.3-304507073042.5-305744244.76-06.00 4.65  +0.93 +0.64 +0.47   G3Ib               -0.021 0.000 +.023+014
2882          CD-37 3637  59967198069    I'                 072709.9-370748073042.4-372023250.46-08.97 6.65  +0.63               G4V                -0.098+0.042      +009
2883          BD-08 1964  59984134806        6158           072718.1-083951073205.8-085251225.50  4.87 5.90  +0.54 -0.08         F5V                -0.090-0.157      +055V   =< 10  2.7  23.4      *
2884          CP-52 1198  60060235239                       072734.1-522632072959.6-523904264.50-15.79 5.87  +1.01 +0.83         K0III              -0.031+0.058      +026
2885          CD-35 3650  60098198086               3629    072748.4-355628073125.8-360910249.46-08.29 6.68  -0.12 -0.57         B4V                -0.004+0.022      +021       83
2886 68    GemBD+16 1510  60107 97016      D        3642    072754.0+160230073336.5+154936203.19 16.32 5.25  +0.05 +0.06         A1Vn               -0.019-0.012 +.011+012V?    163  1.6   0.2      *
2887  8Del2CMiBD+03 1715  60111115610                       072757.1+033013073311.6+031725214.76 10.79 5.59  +0.31 +0.10         F2V                -0.018+0.038      +001      101
2888          CP-64  721  601502498642584                   072802.5-641800072851.3-643036275.98-20.45 6.39  +1.56 +1.85         K5III              -0.011-0.016      +014
2889          CD-35 3652  60168198093      W       PS Pup   072804.4-354030073142.8-355316249.25-08.12 6.61  -0.08 -0.36         A0V                +0.011+0.007      -002SB2        4.4  91.5AB   3*
2890 66Alp GemBD+32 1581  60178 60198 287I   6175B          072813.0+320627073436.0+315319187.44 22.48 2.88  +0.04 +0.02         A2Vm               -0.172-0.099 +.067-001SB1O   30  0.9   2.0AB   6*
2891 66Alp GemBD+32 1581  60179          I   6175A YY Gem   072813.0+320627073436.0+315318187.44 22.48 1.98  +0.03 +0.01 -0.01   A1V                -0.171-0.098 +.067+006SB1O   14  0.9   2.0AB   6*
2892          CP-54 1294  602282352522586                   072815.4-541118073030.9-542358266.20-16.43 5.96  +1.56 +1.96         M1III              +0.018+0.021      +049
2893          BD+10 1563  60275 97021                       072834.6+104704073405.1+103406208.18 14.21 6.28  -0.01 -0.06         A1V                -0.005-0.009      -003      135
2894          BD+56 1227  60294 26423                       072838.6+555831073647.0+554519161.77 28.38 5.92  +1.12               K2III              -0.015-0.035      +001V
2895          CD-35 3659  60312198104      W                072844.1-354449073222.3-355741249.38-08.04 6.3   -0.10 -0.36         B9V                -0.012+0.005      +023V          1.2   0.3
2896          BD+31 1620  60318 60204        6185           072847.5+311042073508.7+305740188.43 22.29 5.33  +1.01 +0.85         K0III              -0.033+0.013D+.003-006     < 19: 0.4   0.4AB   3*
2897          BD-14 1966  60325153061                       072846.1-140722073322.1-142018230.45  2.52 6.21  -0.04 -0.71         B1V                -0.010-0.004      +022                          *
2898          BD+43 1711  60335 41877        6191           072853.7+431503073556.0+430152175.80 25.89 6.30R                     F0                 -0.024-0.050D+.005+021    =< 15  1.1   2.2
2899          BD-19 1944  60341153062                       072855.2-191142073319.5-192445234.89  0.06 5.66  +1.12 +1.08 +0.38E  K3III              +0.022-0.067      +016V                         *
2900          CD-24 5566  60345173987                       072858.9-242945073309.7-244239239.51-02.54 5.85  +0.17 +0.11         A7V:               -0.010+0.013      +015
2901  9Del3CMiBD+03 1719  60357115644      W                072901.0+033519073415.8+032217214.81 11.06 5.81  -0.02 -0.09         A0Vnn              -0.012-0.014      +034V     195  5.2  90.0AB   3
2902          BD-14 1971  60414153072    I         KQ Pup   072912.3-141827073347.9-143126230.67  2.52 4.97  +1.41 +0.29 +1.33   M2Iabpe+B2V       v-0.012+0.004 -.003+022SBO                       *
2903          BD+46 1286  60437 418791195I                  072916.1+462403073631.6+461049172.41 26.71 5.65  +1.56 +1.90 +0.78E  M0III              -0.028-0.032      +029
2904          BD+03 1723  60489115653               3648    072932.9+025637073446.0+024330215.46 10.88 6.55  +0.22 +0.15         A7III              -0.043-0.027      +046                          *
2905 69Ups GemBD+27 1424  60522 795331196I W        3652    072945.6+270705073555.3+265345192.60 21.06 4.06  +1.54 +1.94 +0.91   M0III-IIIb        e-0.034-0.106 +.014-021    =< 19: 9.0  46.1      *
2906          BD-21 2007  60532174009 288                   072946.3-220448073403.2-221746237.50-01.19 4.45  +0.51 +0.06 +0.27   F6IV               -0.040+0.046 +.054+061        0                 *
2907          CD-39 3398  60559198120                       072949.1-395032073313.5-400332253.13-09.78 6.26  -0.12 -0.48         B8IVpSi            +0.020-0.010      +014V
2908          CD-42 3325  60574218792                       073000.7-425208073313.3-430511255.87-11.16 6.52  +0.92               G8III              +0.005-0.035      -048
2909          CD-23 5709  60584174019        6190A          073005.0-231520073418.6-232825238.56-01.71 5.83  +0.44 +0.03        dF4                 -0.095+0.002 +.023-005SB         0.0   9.6      *
2910          CD-23 5709  60585174020        6190B          073005.7-231523073419.1-232829238.56-01.71 5.87                     dF6                 -0.111-0.010 +.023-006V          0.0   9.6      *
2911          CD-36 3715  60606198130              OW Pup   073013.9-360715073351.0-362018249.85-07.95 5.54  -0.06 -0.72 -0.01   B3Vne              -0.012+0.001      -009SB    297                 *
2912          CD-25 4719  60629174028                       073021.6-255351073428.8-260700240.89-02.95 6.65  -0.01 -0.05         A0V                -0.019-0.025      +028
2913          CD-33 3926  60646198138                       073026.8-331449073412.8-332748247.33-06.53 6.11  +0.29               A9IV               -0.056+0.070      +025
2914          BD+49 1653  60652 41893                       073026.4+485949073753.9+484625169.61 27.45 5.92  +0.22 +0.16 +0.05   A5m                -0.026-0.032      +010       60
2915          BD+40 1903  60654 418872589I                  073027.2+401455073717.8+400131179.10 25.39 6.38R                     M1                 -0.011-0.050      +010
2916          CD-26 4574  60666174033                       073030.1-264744073434.9-270043241.69-03.37 5.77  +1.06               K1III              -0.039+0.079      -006V?
2917          CD-39 3407  60686198137                       073033.7-394120073358.5-395421253.06-09.59 6.76  +1.13               K2III              -0.017+0.045      +030V
2918          BD+06 1729  60803115693                       073115.2+060500073634.7+055142212.81 12.70 5.91  +0.60 +0.10         G0V                -0.111+0.011      +004SB2?=< 10
2919   Eps MenCP-78  265  608162564152583                   073108.4-785305072538.0-790539291.02-25.01 5.53  +1.28 +1.42         K2-3III            -0.033+0.003 -.002+011
2920          BD-07 2065  60853134883    I                  073126.7-080523073616.6-081841225.50  6.04 6.27  +1.54 +1.89         K2                 +0.035+0.002      +005
2921          BD-14 1999  60855153118        6208  Var      073128.0-141616073603.9-142934230.91  3.02 5.70  -0.12 -0.71 -0.10   B2Ve               -0.010+0.003      +021      284  3.8  19.6ABxC 3*
2922          CD-28 4566  60863174058        6205           073122.0-280854073522.8-282210242.96-03.87 4.64  -0.11 -0.40 -0.11   B8V                -0.074-0.019      +003V?    222  4.5  38.4AB   3*
2923          BD-21 2030  60951174092                       073149.8-215617073607.8-220938237.61-00.70 6.34  +0.99 +0.76         G5                 +0.044-0.018      +028SB2
2924 70    GemBD+35 1662  60986 60243        6229           073159.1+351621073832.8+350255184.45 24.24 5.56  +0.93               K0III              +0.037+0.030      -036V          4.6 159.9AC   5
2925          CD-51 2571  61031235310                       073205.8-511515073439.5-512829263.71-14.64 6.28  +0.08               A1III-IV           -0.006-0.001      +028
2926          BD+24 1727  61035 79562                       073211.3+243505073814.5+242137195.34 20.64 6.27R                     F0                 +0.025+0.013      +007      112
2927 25    MonBD-03 1979  61064134899 289  W        3665    073218.3-035316073716.7-040640221.89  8.26 5.13  +0.44 +0.12         F6III              -0.068+0.017 +.028+046       20  5.3 121.7AC   3*
2928          BD-19 1967  610681531491197          PT Pup   073217.3-192846073641.0-194208235.53  0.61 5.74  -0.19 -0.89         B2III              -0.007+0.005      +022V?     38                 *
2929 23    LynBD+57 1093  61106 264592593                   073233.2+571840074049.5+570458160.34 29.09 6.06  +1.46 +1.65         K5III              -0.018-0.005      -013V
2930 71Omi GemBD+34 1649  61110 602472592                   073238.3+344850073909.9+343503184.97 24.23 4.90  +0.40 +0.11 +0.22   F3III              -0.037-0.117 +.021+007V      89
2931          BD+24 1730  61219 79580                       073309.5+242658073911.9+241321195.56 20.79 6.17  +0.01 +0.04         A2V                -0.012+0.002      -011V     110
2932          BD-14 2053  61224153172      W                073302.7-141257073738.9-142628231.05  3.38 6.53  -0.01 -0.30         B9IIIe             +0.014-0.004      +016      275  2.8  65.0      *
2933          CD-23 5791  61227174141                       073303.2-233303073716.8-234630239.15-01.26 6.37  +0.54 +0.44         F0II               -0.015+0.022      +018V
2934          CP-52 1231  612482353361198           3661    073311.3-521838073539.7-523202264.77-14.96 4.94  +1.40 +1.63 +0.73   K3III              +0.024-0.016 +.012+062SB                        *
2935          BD+38 1803  61294 60257    I                  073330.9+383424074014.6+382040181.06 25.49 5.73  +1.63 +1.95         M0III              -0.050-0.009      +046V?
2936          BD+32 1599  61295 60254                       073330.3+321420073954.1+320035187.71 23.58 6.17  +0.35 +0.13         F6II               -0.019-0.045      +025SB  =< 23
2937          CD-34 3755  61330198195 290  W                073340.0-344437073722.1-345807248.98-06.67 4.53  -0.09 -0.31 -0.07   B8IV               -0.022+0.014      +024       80  0.5   0.1AB   3*
2938 74    GemBD+18 1701  61338 97120    I D        3671    073342.1+175408073928.6+174029202.01 18.35 5.05  +1.56 +1.92         K5IIIFe-0.5        +0.005+0.004 +.009+028V?         0.0   0.0      *
2939          BD+48 1561  61363 419342594                   073349.2+482154074112.4+480754170.43 27.88 5.56  +1.01       +0.52:  K0III              -0.054-0.128 +.009+040
2940          CD-48 3069  613912188412591                   073355.8-483619073643.9-484949261.40-13.19 5.72  -0.06               B9.5V              -0.001-0.003      -001
2941          CP-55 1282  61394235343                       073354.2-553949073601.7-555315267.95-16.32 6.39  +1.18               K1-2IIICNII        -0.002+0.010      +022
2942          CD-34 3760  61409198205                       073403.7-350305073744.8-351638249.29-06.75 6.60  +1.14               K1III              -0.027+0.012      +031
2943 10Alp CMiBD+05 1739  61421115756 291I   6251   3672    073404.0+052853073918.1+051330213.69 13.03 0.38  +0.42 +0.02 +0.23   F5IV-V             -0.710-1.023 +.288-003SBO     6 10.4   4.6AC   5*
2944          CD-25 4828  61429174175        6246  PU Pup   073408.2-250816073818.0-252153240.65-01.84 4.70  -0.11 -0.35 -0.09   B8IV               -0.007-0.007      +041V     293: 0.4   0.2      *
2945          CD-37 3736  61453198210                       073412.8-374708073745.2-380038251.71-08.05 6.38  +1.48               K4III              +0.007+0.058      +052                          *
2946 24    LynBD+59 1103  61497 26474 292    6285           073432.8+585640074300.4+584237158.51 29.53 4.99  +0.08 +0.08 +0.04   A3IVn              -0.037-0.051 +.018+009V     183  5.8  54.7
2947          BD-18 1946  61554153195                       073440.8-182700073907.1-184045234.92  1.61 6.72  -0.10 -0.50         B6Vn               -0.021-0.026      +036
2948          CD-26 4707  61555174198        6255A  3673    073443.4-263427073849.3-264806241.96-02.44 4.50H -0.17 -0.57 -0.16   B6V                -0.017+0.019D+.010+024       65  0.2   9.9AB   3*
2949          CD-26 4707  61556174199        6255B  3673    073443.9-263434073849.8-264813241.97-02.44 4.62H                     B5IVn              -0.029+0.028D+.010+033      193  0.2   9.9AB   3*
2950          BD+05 1742  61563115773        6263           073448.3+052741074007.0+051351213.79 13.20 6.02  -0.04 -0.18         A0III              -0.008-0.023 +.014+017SB    100  0.3   1.1AB   3*
2951          BD+23 1780  61603 796072595I                  073459.2+231459074058.5+230107196.92 20.73 5.89  +1.58 +1.86 +0.62E  K5                 -0.010-0.011      +039                          *
2952          CD-39 3463  61623198226                       073458.6-394550073824.2-395929253.54-08.88 6.59  -0.05 -0.17         A0                 +0.013+0.018      +015V
2953          BD+14 1721  61630 97136                       073509.7+140008074047.3+134615205.89 17.05 6.24                      K0                 +0.010-0.016      +005
2954          CD-36 3773  61641198237                       073506.4-361606073843.9-362949250.46-07.16 5.80  -0.16 -0.70         B2IV-V             -0.007+0.001      +019V?                        *
2955          CD-38 3521  61642198229                       073502.5-383308073832.6-384652252.47-08.28 6.19  +1.02 +0.77         G8III              +0.030-0.028      +026                          *
2956          CD-26 4722  61672174219                       073521.0-263800073926.9-265147242.09-02.35 6.50  -0.10 -0.48         B7V                -0.013+0.001      +033
2957          CD-48 3091  61715218852              MY Pup   073528.5-482223073818.2-483604261.31-12.86 5.68  +0.65 +0.45         F4Iab              +0.002+0.008      +011       19                 *
2958          BD-07 2118  61749134969                       073545.3-075713074035.5-081109225.90  7.04 6.01  +0.15 +0.11         A3IV               +0.016-0.037      -007V
2959          BD-14 2082  61772153227    I                  073548.8-150155074023.2-151549232.09  3.55 4.94  +1.56 +1.80         K3II               -0.010-0.022 +.005+000V
2960          BD-19 2003  61774153225        6273           073549.4-192549074013.6-193939235.90  1.36 5.93  +1.16 +1.07         K0                 +0.007+0.011      -003           5.2   8.3      *
2961          CD-38 3531  61831198253                       073555.9-380441073927.4-381830252.14-07.90 4.84  -0.19 -0.66 -0.16   B2.5V              -0.014+0.004      +026      138                 *
2962          BD+34 1657  61859 60291                       073614.7+341404074243.5+340001185.83 24.76 6.02R                     F7V                -0.080-0.003      -011SB2O                      *
2963          CD-37 3767  61878198265      W                073611.7-375432073943.8-380822252.01-07.77 5.73  -0.12 -0.49         B5Vn               -0.019+0.011D+.006+030           2.7   1.1      *
2964          CD-37 3768  61899198268                       073616.1-380148073947.8-381539252.13-07.82 5.76  -0.07 -0.54         B2.5V              -0.011+0.004      +023                          *
2965          BD+13 1737  61885 97154    I                  073615.4+134252074151.8+132850206.28 17.17 5.77  +1.67 +1.99         M2IIIab            -0.039-0.020      +007
2966          BD+03 1758  61887115813               3689    073619.8+035130074135.2+033729215.43 12.81 5.94  -0.04 -0.08         A0V                +0.015-0.025      -024V     165
2967          BD+14 1729  61913 971572597I         NZ Gem   073624.9+142633074203.2+141230205.61 17.51 5.56  +1.64 +1.91 +1.42   M3II-III           -0.004-0.012 +.006-016V?                        *
2968          CD-37 3770  61925198273                       073623.8-372053073958.0-373446251.54-07.46 6.00  -0.04 -0.46 +0.01   B6IVe              +0.001-0.010      +007SB                        *
2969          BD+50 1460  61931 264882599                   073629.8+504014074404.2+502602167.94 28.72 5.27   0.00  0.00         A0IIIn             -0.005-0.026 +.015+000SB    183
2970 26Alp MonBD-09 2172  61935134986 293I                  073628.1-091904074114.8-093304227.19  6.52 3.93  +1.02 +0.88 +0.52   K0III              -0.073-0.019 +.024+011     < 17                 *
2971          CP-52 1242  61966235391      W       V390 Car 073635.5-530235073900.4-531624265.68-14.82 6.06  -0.11 -0.41         B9IV-Vp:Si:        -0.009+0.008      +012           7.8  15.       *
2972          CD-27 4393  61987174273                       073640.1-274246074043.4-275645243.17-02.63 6.76  -0.16 -0.64         B3-5IV-V:          +0.007-0.017      +036
2973 75Sig GemBD+29 1590  62044 79638    I W       Sig Gem  073703.7+290732074318.7+285301191.19 23.27 4.28  +1.12 +0.97 +0.58   K1III             e+0.065-0.232 +.018+046SB1O   22  6.5 182.2      *
2974          CD-31 4910  62058198286              R Pup    073659.8-312541074052.7-313939246.43-04.43 6.56  +1.18 +0.85 +0.67   G20-Ia             +0.014+0.012      +068                          *
2975 51    CamBD+65  593  62066 143212602                   073706.6+654141074640.1+652721150.76 30.20 5.92  +1.18              gK2                 +0.027+0.018      -029
2976          BD-21 2077  62082174298    I                  073705.7-220612074123.6-222014238.36  0.27 6.18  +1.62 +1.95 +0.94E  M1III              +0.022-0.004      +032V
2977 49    CamBD+63  733  62140 14322              BC Cam   073724.3+630418074627.4+624950153.79 30.15 6.49  +0.26 +0.13         F0pSrEu            -0.036-0.057      +002                          *
2978          BD+22 1756  62141 79641      W     A          073724.8+223807074322.2+222358197.75 21.02 6.21  +0.93               K0III              -0.026+0.013      -003
2979          CP-73  457  62153256428      W     A          073724.6-740254073521.7-741632286.11-23.14 7.16H -0.02 -0.24         B9IV               -0.007+0.009D+.010+013SB2        0.1   1.8      *
2980          CP-73  457  62154            W     B          073724.6-740254073521.8-741632286.11-23.14 7.26H                     B9IV               -0.007+0.009D+.010+010           0.1   1.8      *
2981          CD-38 3556  62226198298                       073744.7-381759074115.8-383201252.51-07.69 5.42  -0.15 -0.56         B5V                -0.017+0.013      +040SB2   100                 *
2982          BD+00 2054  62264115839        6313           073757.4+002534074305.4+001122218.73 11.56 6.19  +1.02 +0.74         K0III              +0.008-0.012      +008           0.2   0.1AB   3
2983 76    GemBD+26 1633  62285 79650    I          3703    073800.9+260121074406.9+254703194.42 22.39 5.31  +1.54 +1.89         K4-5III            -0.020-0.016 +.007+003V    < 19:
2984          CD-44 3655  62318218905                       073813.5-442350074121.8-443756257.96-10.57 6.41   0.00 -0.18         B7IV-V             +0.010 0.000      +027V?
2985 77Kap GemBD+24 1759  62345 79653 294I   6321           073824.6+243816074426.8+242353195.85 21.97 3.57  +0.93 +0.69 +0.45   G8IIIa             -0.033-0.052 +.026+021        8  5.8   7.1      *
2986          CD-38 3564  62376198315                       073826.8-381735074158.0-383144252.57-07.57 6.54  -0.09 -0.40?        B7V                -0.017+0.002      +008
2987          BD+13 1750  62407 97199                       073838.9+130557074414.0+125134207.12 17.43 6.43                      K0                 -0.024-0.042      +027
2988          CD-26 4824  62412174356                       073840.4-260649074248.1-262104242.01-01.44 5.64  +0.99 +0.80         K1III              -0.005-0.027      -018
2989          BD+02 1761  62437115864              AZ CMi   073855.0+023839074407.4+022418216.84 12.81 6.47  +0.20 +0.14         F0III:             -0.048-0.022      +015                          *
2990 78Bet GemBD+28 1463  62509 79666 295I   6335   3712    073911.8+281604074518.9+280134192.23 23.41 1.14  +1.00 +0.85 +0.50   K0IIIb            e-0.628-0.046 +.094+003V    < 17 10.2 201.1AC   7*
2991 79    GemBD+20 1893  62510 79665                       073917.1+203322074509.3+201859199.99 20.62 6.33R  0.00  0.00         A1V                -0.014+0.008      -011       79
2992          CD-25 4966  62555174395      W                073929.0-251554074339.1-253014241.37-00.85 6.55  +0.06               A3                 +0.001-0.005      +023V?         7.5   5.3
2993  1    PupCD-28 4767  62576174391    I   6324   3708    073930.1-281023074332.4-282440243.88-02.32 4.59  +1.63 +1.96 +0.97   K3Ib               -0.014+0.025 +.018+033           8.9  26.2
2994          CD-35 3809  62578198343                       073932.3-354844074312.0-360301250.51-06.15 5.60  -0.13 -0.54         B5V                -0.019+0.008      -001
2995          CD-38 3583  62595198342                       073936.4-383731074307.0-385151252.98-07.54 6.89  +1.02 +0.77         G6/8III            +0.009-0.018      +000
2996  3    PupCD-28 4774  626231744002601I                  073947.6-284256074348.5-285717244.38-02.54 3.96  +0.18 -0.09 +0.22   A2Iabe             +0.002+0.005      +025SBO    73                 *
2997          BD+80  238  62613  1254                       073945.7+803059075617.3+801556133.68 29.44 6.56  +0.73 +0.28         G8V                -0.476+0.090      -008V
2998          CD-44 3675  62644218923                       073951.4-445509074257.2-451024258.57-10.57 5.06  +0.78 +0.32         G6IV               -0.066-0.558      +028SB
2999          BD+37 1769  62647 603281199I          3721    073958.9+374535074639.3+373103182.32 26.50 5.18  +1.58 +1.94 +0.90E  M2IIIb             +0.027+0.014      -035V
3000          CP-77  321  626892564312596                   074006.1-772412073604.2-773803289.62-24.11 6.18  +1.73 +2.03         M0III              +0.006+0.009      +007V?
3001          CD-37 3820  62712198352                       074010.2-375745074342.9-381207252.45-07.11 6.40  -0.16 -0.48         B9VpSi             -0.025+0.006      -006                          *
3002          CD-40 3377  627132189322603                   074017.6-404121074341.9-405602254.86-08.44 5.17  +1.10 +1.06         K1III              +0.133-0.175 +.026+053                          *
3003 81    GemBD+18 1733  62721 972211200I D                074020.0+184515074607.4+183036201.86 20.14 4.88  +1.45 +1.75 +0.83   K4III-IIIb         -0.076-0.058 +.014+081SB   < 17  0.0   0.1      *
3004          CD-24 5885  62747174433              Var?     074021.9-242601074434.0-244026240.75-00.26 5.62  -0.19               B1.5III            -0.030+0.016      +015                          *
3005          CD-49 3014  62756218928                       074022.8-494512074306.9-495934262.95-12.79 6.57  +0.07               A2V                -0.005-0.006      +003
3006          CP-58  967  627582354302600                   074020.2-582335074210.2-583751270.91-16.72 6.43  -0.10               B2.5V              -0.001+0.003      +011SB
3007          CD-35 3825  62781198366                       074030.3-354927074409.6-360346250.62-05.99 5.80  +0.31 +0.01         F2V                -0.083+0.076      +029
3008 11    CMiBD+11 1670  62832 972241201           3724    074045.9+110043074616.2+104606209.33 17.00 5.30  +0.01 -0.02         A1Vnn              -0.029-0.024 +.022+028SB    249
3009  2    PupBD-14 2193  62863153362        6348B PV Pup   074052.7-142636074528.7-144110232.19  4.91 6.89                      A8V                -0.016-0.018      +030SB2        0.8  16.8AB   3*
3010  2    PupBD-14 2194  62864153363        6348A          074053.1-142651074529.1-144127232.20  4.91 6.07  +0.11               A2V                -0.014-0.026      +024           0.8  16.8AB   3*
3011          CD-37 3841  62893198379                       074100.5-374207074434.2-375635252.31-06.84 5.88  -0.11 -0.36         B7V                -0.018+0.004      +037                          *
3012          CP-57 1305  62897235440                       074100.2-575929074253.3-581348270.57-16.46 6.21  +1.05 +0.84         K0III              -0.030+0.033      -022V
3013 80Pi  GemBD+33 1585  62898 60340 296I   6364   3729    074103.5+333940074730.3+332456186.76 25.53 5.14  +1.60 +1.95         M1IIIa             -0.018-0.029 +.012-012V?         5.2  21.0AB   3*
3014          BD-06 2281  62902135079    I                  074108.5-063136074602.2-064621225.31  8.90 5.49  +1.38 +1.68         K5III              +0.056-0.094      -033V
3015  4    PupBD-14 2199  629521533721202                   074120.5-141914074556.9-143350232.14  5.07 5.04  +0.33 +0.09         F0V                -0.012+0.006 +.031-002V     101
3016          CD-37 3861  62991198390                       074130.7-373843074504.6-375316252.31-06.73 6.54  -0.10 -0.63         B3IV               -0.023+0.007      +024                          *
3017          CD-37 3863  630321983982605           3723    074141.4-374333074515.3-375807252.39-06.74 3.61  +1.73 +1.72 +1.00   K2.5Ib-II          -0.010+0.002 +.006+017                          *
3018          CD-33 4113  63077198404                       074151.1-335832074535.0-341023249.13-04.81 5.37  +0.60 -0.06 +0.23E  G0V                -0.279+1.665 +.064+103V                         *
3019          BD-12 2135  63112153389                       074204.2-122549074644.9-124031230.59  6.17 6.39  -0.02 -0.08         A0V                +0.018 0.000      +008V?  =< 41
3020          CD-43 3534  63118218955                       074205.3-433032074518.1-434508257.51-09.53 6.03  -0.07 -0.41         B6IV               -0.004+0.002      +035      118
3021 82    GemBD+23 1812  63208 79704        6378           074234.9+232319074833.6+230828197.48 22.40 6.18R                     G2III+A4V          -0.011+0.008      -005V?         0.1   0.3AB   4
3022          CD-37 3886  63215198416                       074236.4-374121074610.4-375602252.45-06.56 5.88  -0.11 -0.47         B7V               t-0.035+0.004      +028                          *
3023          BD-22 2027  63271174533              Var?     074254.6-221624074712.5-223111239.19  1.34 5.90  -0.19 -0.81         B2IV-V             -0.013+0.007      +007       73                 *
3024   Zet VolCP-72  627  63295256438 297  W                074303.1-722157074149.2-723622284.57-22.16 3.95  +1.04 +0.83 +0.53   K0III              +0.030+0.018 +.017+048           5.8  16.7
3025          CD-39 3587  63308198424                       074306.5-394850074633.4-400335254.36-07.54 6.57  -0.13 -0.67         B2V                -0.006-0.001      +034                          *
3026          BD-15 2049  63302153404              QY Pup   074305.2-154437074738.5-155927233.59  4.71 6.34  +1.78 +1.92         K1Ia-Iab           -0.004+0.001      +031V?                        *
3027          BD-15 2052  63323153409    I W        3741    074312.0-154602074745.2-160052233.62  4.72 6.43  +1.70 +1.97         M2II-III            0.000-0.001      +014V?         0.1 130.5AB   3*
3028          BD+54 1177  63332 26535                       074313.1+542243075105.7+540745163.89 30.25 6.02  +0.46 +0.02         F6V                -0.041+0.051      -002V   =<  6
3029  5    PupBD-11 2106  63336153414        6381           074315.7-115650074756.7-121135230.32  6.66 5.48  +0.48 -0.02        dF5                 -0.111+0.059D+.013+027V      55  1.8   2.0      *
3030          BD+13 1772  63352 97273    I                  074325.5+133715074902.0+132215207.14 18.71 6.04  +1.38               K0                 +0.051-0.043      -057                          *
3031          CP-56 1420  63382235490      W                074330.1-562839074535.6-564321269.32-15.48 6.12  +0.39               F0II               +0.005+0.016      +019V?         5.0  36.2
3032          CD-39 3595  63401198435              OX Pup   074336.3-390503074705.8-391953253.77-07.09 6.31  -0.17 -0.55         B8III              -0.010-0.005      +022                          *
3033          BD+04 1826  63435115966                       074342.4+043459074858.9+041958215.64 14.77 6.53  +0.78 +0.38         G2III              -0.036-0.031      -006
3034   Omi PupCD-25 5081  63462174558        6384   3745    074355.7-254120074805.2-255614242.25-00.21 4.50  -0.05 -1.02 -0.01   B0V:pe:           v-0.009+0.006      +015V     368  8.0  27.0      *
3035          CD-38 3650  63465198442      W                074352.5-381549074725.0-383040253.08-06.63 5.08  -0.10 -0.65         B2.5III            -0.006 0.000 -.004+012       53  5.8  10.9      *
3036          CP-65  806  63513249944                       074405.7-654941074443.9-660419278.18-19.49 6.38  +0.95 +0.66         G6-8III            -0.015+0.043      +002
3037          CD-46 3435  635782190001203                   074430.1-462137074731.5-463631260.25-10.56 5.23  -0.14 -0.85         B1.5IV             -0.002+0.004      +034SB    154
3038          CP-69  770  63584249943                       074431.1-693437074413.0-694917281.85-20.97 6.18  -0.06 -0.09         A0IV-V             -0.050+0.011      +010
3039          BD+55 1228  63586 26543                       074437.3+552753075236.6+551234162.66 30.56 6.38                      A0Vn               -0.004-0.039      +008V     325
3040          BD+33 1601  63589 60379      W        3758    074436.7+332909075102.3+331401187.19 26.19 6.03  +0.15 +0.15 +0.02   A2Vm               -0.017+0.001      -010V      40  4.2  77.8
3041          CD-40 3490  63640219006              T Pup    074443.4-402408074808.5-403908255.03-07.57 6.14  +1.58 +1.90 +1.01E  M2III              -0.003-0.029      +023                          *
3042          BD-12 2164  63655153449                       074449.5-130606074928.6-132112231.52  6.41 6.23  -0.09 -0.47         B8III              -0.035-0.018      -007V
3043          CD-24 6022  63660174592                       074449.6-243944074901.7-245444241.47  0.49 5.33  +0.76               G0III              -0.030+0.019 +.036+002SB
3044  6    PupBD-16 2146  636971534542607I                  074510.1-165825074941.2-171342234.91  4.51 5.18  +1.28 +1.49         K3III              +0.052-0.117      +044V
3045  7Xi  PupCD-24 6030  637001746011204I   6393           074505.3-243631074917.7-245135241.46  0.57 3.34  +1.24 +1.16 +0.55   G6Iab-Ib           -0.003-0.002 +.003+003SB         9.5   5.1      *
3046          CD-46 3451  63744219018                       074521.6-464931074820.3-470440260.73-10.66 4.71  +1.06 +0.92 +0.57   K0III              -0.088-0.078 +.006-001SB
3047          BD-08 2096  63752135158    I                  074522.2-085552075010.6-091100227.95  8.62 5.61  +1.44 +1.56         K3III              -0.001+0.004      -007V
3048          BD-19 2085  63754174617        6398           074521.7-195708074945.2-201225237.49  3.02 6.56  +0.58 +0.12         G3V                -0.053-0.106      +045V          7.0   6.0
3049          CD-34 3970  63786198480               3756    074531.1-345931074914.7-351436250.42-04.69 5.93  -0.05 -0.19         B9V                +0.001+0.009      +034SB2                       *
3050          BD+03 1818  63799116014        6405           074532.5+033153075047.4+031638216.83 14.69 6.18  +1.12 +1.01         K1III              +0.054-0.031D+.003-048           0.2   0.2      *
3051          BD-19 2089  63822153468                       074540.3-191616075005.7-193125236.95  3.44 6.12  +1.26               K0                 -0.007-0.002      +032V
3052          CD-32 4451  638521984872608                   074546.2-330213074935.4-331720248.76-03.64 5.60  +1.61 +1.95         K5III              -0.034 0.000      +033
3053          BD+19 1854  63889 97318                       074607.8+193452075156.7+191931201.62 21.72 5.99  +1.13              gK1                 -0.055-0.032      +039
3054          BD-10 2253  638941534792609                   074611.0-105225075055.2-110743229.75  7.82 6.16  +1.13               K0                 -0.001-0.031      +044V
3055          CD-46 3458  63922219035      W                074611.4-460716074914.3-462224260.18-10.19 4.11  -0.18 -1.01 -0.17   B0III               0.000+0.006      +024       49  4.9  59.2      *
3056          CP-56 1437  63926235532      W                074611.3-561312074819.2-562816269.26-15.03 6.33  +1.01 +0.38         K0III+A1V          -0.026+0.003      +009           2.2   1.1
3057          CD-44 3762  63948219040                       074618.9-442958074928.2-444507258.76-09.36 6.32  +0.96 +0.72         K0III              -0.048+0.003      +032
3058          CD-46 3460  63949219034              QS Pup   074612.1-463621074912.9-465128260.61-10.42 5.84  -0.14 -0.85         B1.5IV              0.000+0.011      +025                          *
3059 13Zet CMiBD+02 1808  639751160431205                   074630.8+020120075142.0+014601218.32 14.20 5.14  -0.12 -0.49         B8II               -0.016-0.003 +.012+032V?     35                 *
3060          CD-24 6060  64042174658        6414           074646.6-241623075100.0-243142241.37  1.08 6.45  -0.01 -0.06         A0                 -0.012-0.023      -007           3.5   0.9
3061          BD+03 1824  64052116054    I         BC CMi   074652.4+033208075207.2+031638216.98 14.99 6.31  +1.59 +1.62 +1.26E  M4III              +0.047-0.084      -062                          *
3062          CP-56 1442  64067235539      W                074657.8-560928074906.7-562438269.26-14.91 5.59  +1.13 +0.84         G5II               -0.003+0.007      +022SB         8.0   6.9
3063  8    PupBD-12 2179  64077153499                       074700.2-123349075140.9-124910231.32  7.14 6.36  +0.39               F2                 -0.001-0.011      +021V     101
3064  9    PupBD-13 2267  64096153500        6420           074708.4-133757075146.3-135353232.27  6.62 5.17  +0.60 +0.06 +0.36   G0V:               -0.062-0.340 +.062-018SBO  < 17  0.7   0.4      *
3065 25    LynBD+47 1498  64106 42055                       074712.9+473841075429.3+472310171.70 29.95 6.25  +1.15 +1.07        gK2                 -0.017+0.006      -063V
3066 26    LynBD+47 1499  64144 42058 299I                  074725.9+474926075442.7+473353171.50 30.02 5.45  +1.46              gK4                 -0.047+0.002      +017
3067 83Phi GemBD+27 1499  64145 797741207                   074722.6+270129075329.8+264557194.19 24.70 4.97  +0.09 +0.10 +0.05   A3V                -0.035-0.031      +008SB    152
3068          BD-20 2235  64152174679                       074721.7-205506075142.9-211026238.56  2.93 5.63  +0.96 +0.70 +0.33E  G8III              -0.062+0.025      +032V?                        *
3069          CD-44 3780  64181219059                       074731.8-441930075042.4-443447258.72-09.09 6.45  +0.90               G6III              -0.022+0.021      +033V
3070          CP-59  908  641852499751206  W                074733.7-600203074912.9-601701272.87-16.62 5.78  +0.42 -0.01         F1V                -0.055+0.151      +020           6.9  23.3      *
3071          CD-50 3004  64225235552                       074740.6-501512075023.9-503035263.97-12.00 5.91  +1.09 +1.02         K2III              -0.060-0.049      -023
3072          BD-05 2280  642351352052611  W        3777    074751.6-051011075247.9-052541224.94 11.03 5.76  +0.41  0.00         F5IV               -0.017-0.023      -002       20: 0.3   0.2      *
3073 10    PupBD-14 2250  64238153520                       074742.7-143521075218.9-145047233.16  6.26 5.69  +0.37 +0.21?        F1Ia               -0.011-0.002      +017V
3074          CD-42 3601  64287219069                       074803.5-425020075120.5-430544257.47-08.26 6.32  -0.17 -0.76         B2IV-V             +0.011-0.010      +014
3075          BD+74  338  64307  6378 300I                  074813.8+741106080011.7+735505140.86 30.73 5.41  +1.42 +1.64         K3III              -0.009-0.037 +.022+035V?
3076          CP-59  910  64320235553                       074812.6-594747074954.8-600304272.69-16.44 6.72  +1.24               K0II                0.000+0.020      +022
3077          BD+56 1253  64347 26579                       074820.4+564603075626.8+563016161.19 31.18 6.72                      A2IV                0.000-0.024      +028SB
3078          CD-42 3610  64365219076              QU Pup   074822.7-423752075140.3-425318257.32-08.10 6.04  -0.18 -0.84         B2IV               -0.016+0.004      +032V                         *
3079          CD-34 4036  64379198540      W                074831.5-342714075215.7-344219250.27-03.88 5.01  +0.44 -0.06         F5V                -0.194+0.243 +.074+027V       0  3.6   3.0      *
3080          CD-40 3579  64440219082 301                   074846.7-401904075213.0-403433255.35-06.86 3.73  +1.04 +0.78 +0.56   K1-2II+A0          -0.010+0.003 +.031+024SB1O                      *
3081          CP-65  827  644842499782610                   074901.1-655625074941.0-661145278.52-19.08 5.79  -0.04 -0.16         B9V                +0.003-0.001      +011
3082          BD+79  265  64486  63922617  W                074904.4+794511080447.1+792847134.47 29.96 5.42  -0.06 -0.14         A0pSi              -0.027-0.049 +.021+003       35  8.3   6.7      *
3083          BD+35 1705  64491 604532613                   074909.7+354032075540.8+352446185.13 27.70 6.23  +0.28 -0.02         A3IVp:             -0.062-0.023      +022V      70
3084          CD-38 3769  64503198545              QZ Pup   074906.2-383614075238.7-385147253.90-05.93 4.49  -0.19 -0.69 -0.16   B2.5V              -0.002-0.006 +.012-031SB    187                 *
3085          CD-36 3989  64572198553                       074922.8-360614075303.5-362150251.78-04.59 5.43  +1.16               K0III              -0.005-0.005 +.035+012
3086 85    GemBD+20 1946  64648 797992614                   074949.7+200853075539.9+195302201.42 22.74 5.35  -0.04 -0.06         A0V s              -0.018-0.043 +.009+011SB     65
3087          BD+09 1815  64685116120                       075005.5+090743075531.4+085146212.16 18.24 5.86  +0.35 +0.02         F2IV               -0.013-0.087      +022
3088          CP-54 1420  64722235579              V372 Car 075006.5-540627075229.7-542202267.62-13.54 5.70  -0.15 -0.89         B1.5IV             -0.003+0.004      +018V     147                 *
3089          CD-49 3137  64740219106                       075014.5-492110075303.7-493647263.38-11.19 4.63  -0.23 -0.93 -0.21   B1.5Vp            v+0.002+0.016      +008      160                 *
3090          CD-47 3396  64760219111                       075021.8-475031075318.2-480611262.06-10.42 4.24  -0.14 -1.00 -0.12   B0.5Ib             +0.004+0.001      +041      238                 *
3091          CD-35 4002  64802198575                       075028.5-353655075411.0-355239251.48-04.14 5.49  -0.19 -0.73         B2V                -0.001-0.010      +028       81:                *
3092          CD-34 4091  64876198579    I:                 075054.2-343500075439.9-345049250.64-03.53 6.15  +1.53 +1.86         K4III              -0.012-0.020      +047
3093          BD+04 1860  64938116145                       075107.1+044503075623.9+042909216.37 16.49 6.17  +0.98 +0.74         G8III              -0.005 0.000      +017V?
3094          BD+44 1693  64958 420971209                   075115.2+441440075816.6+435839175.71 30.05 6.34   1.06 +0.91         K0                 +0.038+0.006      -049                          *
3095  1    CncBD+16 1590  64960 973991208I                  075118.8+160327075659.4+154725205.62 21.45 5.78  +1.28               K3+III             -0.028-0.042      +010V?
3096          CD-30 5275  64974198591                       075116.8-303916075513.7-305503247.33-01.40 6.44                      K4III              -0.047+0.043      +054V
3097          BD+09 1824  65066116162                       075150.4+085431075715.9+083829212.57 18.53 6.05  +1.00 +0.86         K0III              +0.006-0.017      -036V?
3098          BD+01 1959  65123116165        6483           075207.4+012339075716.2+010737219.58 15.14 6.35  +0.50  0.00         F7V                -0.172-0.002 +.005-000           0.2   0.3      *
3099          CD-29 5189  65183198609    I         PX Pup   075223.5-300104075622.8-301707246.92-00.86 6.33  +1.66 +1.50         M6III              +0.004-0.032      +021V                         *
3100          CP-52 1333  65189235612                       075226.1-521906075500.5-523459266.19-12.36 6.38  -0.01 -0.20         B8III              -0.008+0.017      +007V
3101          CD-43 3737  65211219157      W                075231.5-433445075546.5-435042258.53-07.93 6.02  -0.11 -0.49         B6V                -0.012+0.002      +014           0.2   0.7
3102 11    PupBD-22 2087  652281748522615I:                 075233.5-223647075651.5-225248240.64  3.07 4.20  +0.72 +0.42 +0.37   F7II               -0.026+0.009 +.023+014       21                 *
3103          BD+07 1879  65241116179               3829    075243.5+072857075805.8+071249214.02 18.09 6.41  -0.04 -0.07         A0V                -0.020-0.007      +035SB2    59                 *
3104          BD+16 1598  65257 97429    I D                075249.2+164717075831.5+163107205.06 22.08 5.99  +1.47               K0                 -0.006-0.001      -001           0.0   0.2      *
3105          CP-56 1468  65273235615                       075248.9-570218075453.2-571811270.45-14.61 5.63  +1.30 +1.44 +0.48E  K3-4III            -0.077+0.023 +.009+026                          *
3106          BD+59 1130  65301 26618                       075257.8+591908080120.7+590251158.24 31.91 5.77  +0.39 -0.04       d:F2                 +0.017+0.032      -040V?
3107          CD-40 3655  65315219169                       075257.3-402809075624.3-404411255.89-06.26 6.78  -0.18 -0.74         B2V                +0.020-0.002      +014
3108          BD+84  169  65299  13001639                   075301.7+842050081653.8+840328129.27 29.15 6.49  +0.03 +0.03         A3IV               -0.013-0.018      -003SB
3109 53    CamBD+60 1105  65339 14402 302  S       AX Cam   075310.0+603552080142.4+601928156.73 31.95 6.01  +0.14 +0.06         A2pSrCrEu         v-0.037-0.020      -005SBO    14:                *
3110 14    CMiBD+02 1833  65345116182      W                075309.5+022929075820.6+021329218.70 15.88 5.29  +0.92 +0.71         K0III              -0.158+0.099 +.027+046     < 19: 3.0  88.6AB   4
3111          CD-42 3717  65442219186                       075336.7-420818075657.9-422422257.39-07.02 6.09  +1.36 +1.61         K3III              +0.029+0.023      +058
3112          BD+63  749  65448 144072619  W                075333.1+632154080230.8+630525153.47 31.97 6.40  +0.59 +0.35         G1III              -0.012-0.017      +020V          2.4  48.6
3113          CD-29 5236  654561986361210           3830    075340.9-300356075740.1-302005247.10-00.65 4.79  +0.15 +0.18 +0.15   A2V               v-0.006+0.007 +.025+028        0                 *
3114          CD-43 3758  65460219189                       075341.1-431356075657.9-433001258.34-07.57 5.35  -0.18 -0.74         B2.5V              +0.001+0.013      +020SB    146:
3115          BD+13 1811  65522 97445    I                  075400.1+133051075935.1+131432208.40 20.99 6.02  +1.32               K2                 -0.019-0.011      +027
3116          CD-43 3766  655512191972616  W       Var      075404.0-435027075718.4-440635258.90-07.83 5.09  -0.17               B2.5IV             -0.008+0.009      +016        0  8.9  10.0AB   3*
3117   Chi CarCP-52 1343  65575235635 303          Chi Car  075414.1-524250075646.7-525856266.68-12.32 3.47  -0.18 -0.67 -0.18   B3IVp              -0.029+0.021 +.004+019V      95                 *
3118          CD-47 3457  65598219200      W                075421.6-473716075720.0-475325262.21-09.73 6.22  -0.10 -0.50         B5V                -0.020+0.014D+.007+012           0.6   0.5
3119          BD+57 1118  65626 26634              AE Lyn   075427.1+573302080235.8+571625160.33 32.05 6.49  +0.62 +0.16         F8V                -0.027-0.063 +.038+026SB2O   12                 *
3120          CP-60  935  65662250019                       075437.3-601528075618.6-603135273.50-15.94 5.74  +1.55 +1.80 +0.82   K3III              -0.011+0.014      +023                          *
3121          CD-45 3611  65685219218                       075443.1-451828075751.8-453440260.23-08.49 5.17  +1.27 +1.36         K2III              -0.006+0.020 +.009+051SB
3122 27    MonBD-03 2157  65695135345 304I                  075444.4-032425075944.1-034047224.24 13.39 4.93  +1.21 +1.21 +0.46E  K2III              -0.054-0.003 +.008-029     < 19:
3123 12    PupBD-22 2104  65699174932    I                  075448.3-230219075905.7-231838241.28  3.29 5.11  +1.12 +0.92        cK2                 -0.012+0.001 -.003+011                          *
3124  2Ome1CncBD+25 1812  65714 798611211                   075452.8+254000080055.9+252334196.23 25.82 5.83  +1.02 +0.88        gG8                 +0.014+0.006      +002V
3125          BD+20 1976  65735 97471                       075458.4+200525080048.0+194858201.98 23.84 6.25  +1.11               K1III              -0.001-0.013      +028
3126          CP-58 1028  65750235638              V341 Car 075458.2-585122075650.7-590735272.25-15.24 6.25  +2.08 +2.29 +1.04E  M0III             e-0.024-0.019      +024                          *
3127          BD+23 1866  65757 79864        6513           075502.4+235129080100.7+233459198.13 25.23 6.34R                     K1III-IV           -0.026-0.024D+.010+025           4.5   2.3
3128  3    CncBD+17 1731  65759 974722618I                  075503.4+173458080047.3+171831204.50 22.88 5.55  +1.32              gK3                 -0.012-0.002      +041                          *
3129          CD-48 3349  65818219226      W       V Pup    075521.9-485825075814.4-491442263.48-10.28 4.41  -0.17 -0.96         B1Vp+B3IV:         -0.011+0.008      +019SB2O       5.9  39.2AD   5*
3130          BD+35 1731  65801 60523    I                  075525.1+354120080155.1+352447185.50 28.93 6.34  +1.56               K0                 -0.043-0.007      -016
3131          BD-18 2118  658101536871212                   075523.1-180729075952.0-182357237.16  6.00 4.61  +0.08 +0.08 +0.06   A2Vn               -0.002-0.039 +.021-012      198                 *
3132  4Ome2CncBD+25 1816  65856 79869      W                075541.9+252153080143.8+250523196.62 25.89 6.31  +0.01 +0.03         A1V                -0.021+0.020      -009V     115  3.8 109.3AC   3
3133          CD-51 2784  65867235646               3841    075540.1-511039075821.4-512655265.43-11.36 6.44  +0.25               A8-9IV             -0.046+0.034      +010                          *
3134  5    CncBD+16 1612  65873 97485                       075548.3+164352080130.3+162719205.43 22.71 5.99  -0.02 -0.02         B9.5Vn             +0.004-0.005      -012SB    185
3135          BD-02 2379  65875135368               3857    075542.4-023626080044.1-025254223.64 13.99 6.51  -0.07 -0.81 -0.05   B2.5Ve            v-0.014+0.013      +046V     148                 *
3136          BD+05 1857  65900116244      W                075556.5+050918080113.8+045247216.58 17.74 5.65   0.00 +0.01         A1V                -0.038+0.009      +046    =< 41  6.7  30.1
3137          CD-44 3920  65904219240                       075551.3-445635075901.8-451258260.02-08.13 5.99  -0.14               B4V                -0.011-0.004      -003
3138          CP-59  944  65907250035    I'W                075556.0-600205075746.9-601812273.38-15.69 5.60  +0.57  0.00         G0V                +0.517+0.114 +.062+014SB         4.3  60. AB   3*
3139          CP-62  925  65908250030                       075556.1-630134075712.5-631749276.13-17.11 6.14  -0.10               B8V                -0.018+0.015      +023                          *
3140          CD-38 3908  65925198668                       075556.5-390120075928.4-391750254.96-05.01 5.24  +0.39 +0.02         F5III              -0.082-0.043 +.038-008      125
3141 28    MonBD-00 1882  659531353802620I         V645 Mon 075608.0-010654080113.3-012333222.35 14.81 4.68  +1.49 +1.78 +0.83   K4III              +0.068-0.073 +.009+027     < 19:                *
3142          CD-49 3243  66005219249      W     A          075622.9-494212075912.3-495836264.20-10.51 6.32                      B2IV-V              0.000+0.015      +013V?         0.0  16.4      *
3143          CD-49 3244  66006219250      W     B          075624.0-494201075913.4-495825264.20-10.51 6.34                      B2IV-V             -0.008+0.013      +023V?         0.0  16.4      *
3144          BD+09 1843  660111162512621          Var      075624.9+091124080150.7+085450212.83 19.67 6.22  +0.57 +0.13         G0IV               -0.002+0.022      +004       12                 *
3145          BD+02 1854  661411162602623I W                075703.7+023634080215.9+022004219.07 16.80 4.39  +1.25 +1.28 +0.67   K2III              -0.031+0.105 +.024+071V    < 19: 4.7 241.4
3146          CD-45 3662  66190219280                       075709.6-451053080019.6-452725260.35-08.06 6.61  +1.27               K1Ib-II            +0.008+0.001      +027
3147          CP-60 1006  66194250055              V374 Car 075710.4-603302075850.5-604928273.93-15.80 5.81  -0.09 -0.80 -0.06   B2IVpne           v-0.008+0.015      +011V     250                 *
3148          CD-48 3384  66210219283                       075720.5-484224080014.9-485853263.41-09.86 6.02  +0.04 +0.08         A2V                -0.015+0.047      +012
3149   Chi GemBD+28 1532  66216 79896 305I W                075722.6+280429080331.1+274739193.90 27.14 4.94  +1.12 +1.09         K1.5III            -0.026-0.042 +.016-011SB   < 19: 6.0  78.6AC   3*
3150          BD-05 2339  662421354061213                   075731.1-060330080226.0-062014226.95 12.67 6.33  +0.62 +0.13         G0III              +0.006-0.014 +.007-017V
3151          CD-48 3388  66255219292              PY Pup   075733.9-483544080028.9-485217263.34-09.77 6.12  -0.11               A0pSi              -0.014+0.012      +028                          *
3152          CP-59  954  66341250064                       075754.4-595554075940.2-601227273.41-15.43 6.33  -0.06 -0.40         B5III              +0.004+0.003      +023                          *
3153          CP-60 1018  66342250063                       075755.0-601841075937.6-603513273.76-15.61 5.17  +1.74 +1.91 +1.13   M1.5IIa            +0.003+0.011 -.001+023                          *
3154          CD-36 4116  66358198714                       075757.9-370021080137.4-371701253.46-03.60 5.95  +0.14 +0.14         A3IV               -0.017+0.008      -000
3155          CD-36 4120  66435198725               3872    075825.6-364617080206.2-370302253.31-03.39 6.34  +1.62 +1.88         M1III              +0.016-0.003      +059V?                        *
3156          CP-53 1505  664412356802622                   075822.4-535226080049.9-540905268.02-12.37 5.87  -0.14 -0.52         B5Vn               -0.023+0.008      +000V?
3157          CP-54 1470  66546235686      W        3871    075857.2-541412080122.9-543054268.39-12.48 6.10  -0.04 -0.65         B2IV-V             -0.018+0.021      +034SB    164  2.0  40.2      *
3158          BD+19 1911  66552 97537                       075858.5+190730080445.2+185032203.35 24.34 6.15R -0.05 -0.12         B9V                -0.035-0.016      +028       70
3159          CP-63  866  665912500692624                   075904.2-631725080020.0-633403276.55-16.92 4.82  -0.17 -0.62 -0.20   B3V                +0.001+0.017      +022        0                 *
3160          CD-32 4766  66598198743    I W                075909.9-321059080304.1-322750249.52-00.79 5.82  +1.22 +1.28         K2-3III            -0.017 0.000      +049           3.1  35.1
3161          CP-55 1419  66607235690                       075911.4-551032080131.5-552718269.24-12.92 6.28  -0.15 -0.62         B4V                -0.017-0.008      +006
3162          CD-40 3776  66624219339      W       V336 Pup 075918.4-410147080244.8-411836257.01-05.53 5.52  -0.15 -0.47         B9pSi              -0.002+0.010      +026V?    104  3.2  26.1      *
3163  8    CncBD+13 1831  66664 975422625                   075930.2+132412080504.5+130705209.11 22.16 5.12  +0.01 -0.01         A1V                -0.036-0.067 +.023+021SB    150
3164          BD+27 1536  66684 79928        6569   3892    075929.3+274849080537.0+273147194.34 27.50 6.21  +0.01 -0.09         A0V                 0.000-0.009D+.007-017V?    103  0.7   3.7      *
3165   Zet PupCD-39 3939  66811198752 306                   080004.1-394317080335.1-400012255.98-04.71 2.25  -0.26 -1.11 -0.22   O5f               e-0.027+0.012      -024V?    211                 *
3166          CD-42 3832  66812219355                       080009.1-423958080329.5-425655258.48-06.27 6.29  +1.01 +0.76         G8II               -0.026-0.013      +014                          *
3167 28    LynBD+43 1770  66824 421742627                   080014.3+433251080709.9+431537176.84 31.52 6.26R -0.02 -0.13         A1V                -0.010-0.034      +009SB2O   50                 *
3168 14    PupBD-19 2228  66834153796                       080015.0-192640080441.5-194341238.90  6.28 6.13  -0.16 -0.72         B3III              +0.002+0.003      +014
3169  9Mu 1CncBD+23 1887  66875 79940    I         BL Cnc   080022.9+225516080618.4+223808199.58 26.05 5.99  +1.66 +1.91 +1.01E  M3III              -0.008-0.005      +026V                         *
3170          CD-32 4796  66888198764    I         MZ Pup   080022.3-322330080416.2-324030249.83-00.69 5.31  +1.91 +1.95         M1Ib               -0.003+0.004 +.024+036                          *
3171          CP-72  654  66920256463                       080029.9-725756075916.1-731441285.73-21.21 6.34  +0.14 +0.13         A3III              +0.002-0.036      +024
3172          BD-00 1903  66950135488                       080043.2-001717080549.6-003425222.18 16.22 6.41  +1.05 +1.01         K0                 -0.022-0.013      +034
3173 27    LynBD+51 1391  67006 26687 307    6600           080056.2+514742080827.4+513024167.19 32.65 4.84  +0.05  0.00  0.00   A2V                -0.059-0.004 -.009+011V     168  7.1  45.6AB   3
3174          BD-08 2222  67159135505        6588           080138.4-085728080627.5-091442230.04 12.07 6.23  -0.07 -0.12         B9V                +0.001-0.012      +028V?         1.7  30.8AB   4*
3175          BD+58 1102  67224 26701    I                  080151.5+583228081003.8+581453159.17 33.04 5.93  +1.39 +1.50        gK4                 -0.023-0.075      +034
3176 10Mu 2CncBD+22 1862  67228 799592630I'                 080152.8+215220080745.8+213454200.81 25.99 5.30  +0.63 +0.21         G1IVb              +0.022-0.069 +.034-036    =< 10
3177          CD-33 4525  67243198791      W                080153.2-331659080544.9-333409250.76-00.91 6.14  +1.11 +0.88         G1Ib               -0.006+0.010      +033           2.4  22.1AB   3*
3178          CD-50 3138  672492357302626                   080154.1-501818080442.4-503526265.19-10.06 5.95  +1.21 +1.03         G5II               -0.001+0.001      +027
3179          CD-46 3764  67341219400                       080215.2-464132080520.4-465844262.12-08.10 6.19  -0.15 -0.67         B3Vnp              +0.005-0.009      +024SB
3180          CP-52 1376  67364235735                       080228.2-524917080504.0-530629267.42-11.30 5.53  +1.34 +1.60         K3-4III            +0.034-0.006 +.020+018
3181          BD+42 1819  67370 42199                       080231.1+424325080923.1+422550177.88 31.79 6.27  +1.27              gK3                 -0.008-0.072      +038V?
3182          BD+68  524  67447 144561215                   080251.8+684607081248.8+682827147.01 32.53 5.32  +1.04 +0.81 +0.48   G7+II               0.000+0.008 +.021-009V    < 19:                *
3183          BD-20 2395  67456175206      W                080253.2-201555080718.0-203317239.93  6.36 5.38  +0.10 +0.16 +0.09   A5II               -0.016-0.006 +.001+012        0  9.2  13.8      *
3184 12    CncBD+14 1831  67483 97594                       080307.1+135555080842.4+133827208.98 23.18 6.27  +0.43 +0.01         F3V                -0.001-0.017      -010V      40
3185 15Rho PupCD-23 6828  67523175217 308I W       Rho Pup  080317.1-240057080732.6-241815243.15  4.40 2.81  +0.43 +0.19 +0.21   F6IIpDel Del      e-0.083+0.049 +.035+046SB     14 10.9  29.6      *
3186          CP-62  953  67536250101      W       V375 Car 080317.5-623259080442.9-625010276.11-16.14 6.30  -0.10               B2.5Vn            e-0.009+0.017      +000V?    292  1.3  87.1      *
3187          CD-44 4051  67582219422               3905    080327.7-445838080640.4-451559260.77-07.01 5.05  +1.50 +1.73         K3III               0.000 0.000 -.011+025                          *
3188 29Zet MonBD-02 2450  67594135551    I   6617           080334.0-024133080835.6-025902224.72 15.66 4.34  +0.97 +0.69 +0.46   G2Ib               -0.017-0.005 +.003+030     < 17: 4.9  66.0AC   4*
3189          BD-10 2400  67725153887                       080412.1-110250080856.9-112023232.20 11.53 6.32  +0.01 -0.09         A0Vn               -0.004-0.012      +020V?    160
3190          BD-19 2262  67751175250      W                080418.2-200416080843.5-202147239.95  6.74 6.36  +0.16               A7IV               -0.012 0.000      +028           6.6  20.2
3191 14Psi CncBD+25 1865  67767 799952633  W                080425.8+254839081027.2+253026196.89 27.90 5.73  +0.81 +0.43 +0.28E  G7V                -0.075-0.348 +.029-043V          4.9 112.1AC   3*
3192 16    PupBD-18 2190  677971538902632                   080433.7-185708080901.6-191442239.04  7.40 4.40  -0.15 -0.60 -0.14   B5IV               -0.014-0.002      +019SB?   188
3193          BD+39 2065  67827 606252635                   080443.8+390143081121.6+384353182.23 31.50 6.58  +0.59 +0.09         G0                 -0.111-0.075      +026    =< 10
3194          BD-15 2280  67880153898        6632           080454.0-155719080928.5-161456236.53  9.08 5.68  -0.17 -0.74         B2.5V              -0.017-0.007      +033V?         6.7   5.3      *
3195          CD-37 4288  678881988482631          PQ Pup   080457.7-372320080837.6-374053254.54-02.63 6.37  -0.04 -0.56 +0.01   B4V               e-0.002+0.003      +030V?                        *
3196          CD-29 5620  67921198854      W                080505.2-300145080906.7-301921248.41  1.44 6.65  +1.40 +1.60         K5III              +0.024-0.001      -025           4.2   1.1
3197          BD+82  235  67934  13193952                   080512.4+824427082432.9+822551130.95 29.89 6.32  +0.01 +0.01         A2Vn               -0.022-0.025      -021V?    200
3198          BD+15 1775  67959 97628                       080521.8+145531081058.8+143746208.24 24.08 6.23  +0.02 +0.04         A1V                -0.028-0.014      +024    =< 41
3199          CD-35 4256  67977198859                       080523.2-350942080910.2-352718252.72-01.34 6.20  +0.89               G8III              +0.003+0.014      -014
3200          BD+56 1278  68077 267322640                   080552.0+564507081350.2+562708161.31 33.58 5.85  +1.01 +0.79         G9III              -0.022-0.033      +007V?
3201          BD+10 1746  68099116444                       080549.4+100704081116.6+094916213.03 22.16 6.07  -0.10 -0.42         B6III              -0.009-0.021      +030V      70                 *
3202 18    PupBD-13 2420  681461539242637  W                080601.7-133018081039.8-134757234.57 10.61 5.54  +0.49  0.00         F6V                -0.251+0.061 +.047+038           6.7  93.       *
3203          CD-48 3516  68161219493                       080610.6-482324080909.6-484104263.92-08.45 5.70  -0.12 -0.42         B8Ib-II             0.000-0.004      +027V?
3204          CD-43 3998  682172195022634          Var?     080618.0-434940080935.9-440722260.07-05.96 5.21  -0.19               B2IV-V             -0.007 0.000      +008      194
3205          CD-42 3944  68242219507      W                080624.2-422044080947.7-423826258.84-05.13 6.26  -0.04 -0.32         B7V                -0.004+0.010D+.017+012V?         1.1   5.6      *
3206   Gam1VelCD-46 3846  68243219501      W     B  3930    080624.4-470302080929.3-472045262.81-07.70 4.27  -0.23 -0.92 -0.18   B1IV               -0.004-0.005      +020SBO   119  2.3  41.2AB   5*
3207   Gam2VelCD-46 3847  68273219504 309  W     A Gam2 Vel 080627.0-470230080932.0-472012262.80-07.69 1.78  -0.22 -0.99 -0.13   WC8+O9I           v-0.004+0.006 +.017+035SB2O       2.3  41.2AB   5*
3208 16Zet1CncBD+18 1867  68257 97645        6650A          080628.6+175658081212.7+173852205.30 25.53 5.63  +0.54 +0.06         F8V                +0.066-0.135 +.039-006        0  0.4   0.9AB   5*
3209 16Zet1CncBD+18 1867  68256 97645        6650B          080628.6+175658081212.7+173852205.30 25.53 6.02  +0.54 +0.06         F9V                +0.066-0.135 +.039-006           0.4   0.9AB   5*
3210 16Zet2CncBD+18 1867  68255 97646        6650C          080629.0+175655081213.3+173852205.30 25.53 6.20  +0.60 +0.13         G5V                +0.097-0.109 +.039-011V          1.2   5.9ABxC 5*
3211 19    PupBD-12 2385  68290153942    I   6647           080634.7-123749081116.3-125537233.89 11.19 4.72  +0.95 +0.74 +0.48   G9III-IIIb         -0.025+0.012 +.031+036V    < 17  3.0  71.0AE   5
3212          BD-07 2378  68312135611                       080641.0-072828081133.0-074621229.40 13.91 5.36  +0.89 +0.60 +0.45   G6III              -0.041-0.021 +.012-011V    < 19:
3213          CD-47 3653  68324219515              IS Vel   080640.7-473831080943.2-475615263.33-07.98 5.23  -0.21 -0.89         B1IVe              +0.003+0.001      +005SB    188                 *
3214          BD+14 1850  68332 97647                       080646.5+141809081222.1+140014209.01 24.14 6.54                      A7III              -0.026-0.008      -009       71
3215 15    CncBD+30 1664  68351 800162641          BM Cnc   080657.0+295723081308.9+293924192.58 29.71 5.64  -0.07 -0.12         B9pSiCr            -0.009-0.018      +020SBO    27                 *
3216          BD+76  310  68375  6487 310                   080659.2+760344081932.2+754525138.47 31.61 5.54  +0.90               G8III              +0.030+0.016      +007V
3217          CP-63  896  68423250127                       080706.4-633023080824.5-634804277.20-16.24 6.28  -0.06               B6Ve               -0.017+0.002      +030V?                        *
3218          CP-55 1467  684342357842638                   080713.8-554726080933.6-560508270.38-12.26 5.66  +0.21               A3m                -0.007+0.034      +025
3219          CD-36 4291  68450198898      W                080719.9-365942081101.6-371732254.47-02.02 6.44  -0.01 -0.87         O9.5II             -0.012+0.003      +039       97  7.0  18.0AC   3*
3220          CP-60 1074  684562501312636                   080721.1-605955080900.7-611809274.97-14.95 4.76  +0.43 -0.03 +0.25   F5V                -0.158-0.297 +.055+025        0
3221          BD+60 1119  68457 14479        6680           080724.8+604058081550.5+602250156.59 33.69 6.45  +0.20 +0.11         A7Vm               -0.013+0.007      -016           4.1  48.6AC   4*
3222          BD+16 1662  68461 97653      D                080718.7+164851081259.8+163051206.54 25.27 6.01  +0.89               G8III              -0.009-0.019      -020SB                        *
3223   Eps VolCP-68  736  68520250128      W                080736.4-681924080755.8-683702281.62-18.56 4.35  -0.11 -0.46 -0.09   B6IV               -0.026+0.025      +010SBO    38  3.6   6.0      *
3224          BD+23 1913  68543 80024                       080746.2+232619081341.7+230816199.70 27.82 6.56  +0.11 +0.12         A4IVn              -0.036-0.010      -002
3225          CD-39 4084  68553198908              NS Pup   080747.1-391913081121.5-393707256.45-03.24 4.45  +1.62 +1.86 +0.90   K3Ib                0.000-0.004 -.006+016SB                        *
3226          CD-42 3979  68601219569      W                080803.1-424119081125.9-425914259.29-05.06 4.75  +0.18 +0.08 +0.22   A5Ib               +0.004+0.004 +.004+019        0  4.8  25.7
3227          CD-48 3576  68657219565                       080810.3-480948081111.1-482743263.91-08.05 5.82  -0.15 -0.63         B3V                +0.015-0.004      +015
3228          BD+18 1882  68703 97669        6673           080827.7+175839081411.1+174033205.47 25.98 6.47  +0.30 +0.14 +0.13   A0VnDel Del        -0.005+0.011      -003       67  3.0  63.0AC   3
3229 20    PupBD-15 2324  68752153993 311I                  080844.1-152912081320.0-154718236.63 10.11 4.99  +1.07 +0.78 +0.38E  G5II               -0.013-0.004 +.030+017V?
3230          CD-29 5738  68758175390                       080843.1-293639081246.0-295439248.50  2.33 6.52  +0.06 +0.07         A1V                -0.024+0.031      +028V?
3231          BD+13 1868  68776 97671      W                080847.4+132104081421.0+130254210.19 24.19 6.38                      G8III              -0.021-0.018      +025SB         5.6  22.8
3232          CD-46 3902  68808219587              AH Vel   080851.2-462040081200.0-463839262.44-06.96 5.76  +0.59 +0.39         F7Ib-II            -0.001+0.008      +026SB?                       *
3233          CD-37 4394  68862198942                       080911.3-373723081251.5-375528255.20-02.07 6.43  +0.10 +0.08         A3V                -0.011-0.002      +027                          *
3234          CD-45 3892  68895219602      W                080920.4-455748081230.9-461551262.17-06.68 6.03  -0.11 -0.57         B5V                +0.008+0.012      +013           0.8   0.4
3235 29    LynBD+60 1124  68930 267562645                   080932.0+595240081750.4+593416157.54 33.99 5.64  +0.16 +0.13         A7IV               -0.006+0.003      -015V      96
3236          BD+72  409  68951  6504    I   6724           080939.3+724303082040.3+722426142.29 32.46 5.98  +1.54 +1.46         M0III              +0.004-0.025      +012           3.7  42.9AB   3*
3237          CD-35 4349  68980198957              MX Pup   080942.9-353550081329.6-355359253.58-00.84 4.78  -0.11 -0.98 -0.08   B1.5IIIe          v+0.002-0.002      +035SB2   156                 *
3238          CD-33 4705  69002198960      W                080947.4-331601081341.1-333409251.66  0.48 6.37  +1.14               K2III              +0.033+0.019      +026           6.7   6.3
3239          CD-31 5742  690801989722644  W                081013.7-315014081411.0-320827250.52  1.35 6.06  -0.16 -0.74         B1.5V              -0.002+0.006      -049SB2   208  7.0  29.6
3240          CD-35 4358  69081198969      W       OS Pup   081012.9-360109081358.4-361921253.98-01.00 5.08  -0.19 -0.86         B1.5IV             +0.010-0.002      +018      213  0.9  66.9      *
3241          CD-35 4360  69082198970                       081013.3-360215081358.8-362028254.00-01.01 6.11  -0.18 -0.71         B2IV-V             +0.010-0.008      +014SB2                       *
3242          CD-35 4365  69123198979                       081025.7-351112081413.2-352926253.32-00.49 5.78  +1.02 +0.87         K1III              -0.055+0.008      +027
3243          CD-39 4128  69142219635      W                081029.8-400232081402.9-402053257.35-03.21 4.44  +1.17 +1.09 +0.64   K1II-III           +0.047-0.068 +.010+014SB1O       5.1  51.1      *
3244          CD-46 3929  69144219629      W       Var      081028.4-464118081336.2-465931262.88-06.92 5.13  -0.14 -0.63         B2.5IV             +0.003-0.004      +025SB     57  3.9  35.       *
3245          BD+62  991  69148 14500              Var      081035.1+624858081917.3+623026154.00 33.93 5.71  +0.89               G8III              -0.013+0.007      -002SB                        *
3246          BD+54 1215  69149 26760    I                  081033.8+542710081815.8+540837164.09 34.22 6.27  +1.54 +1.90         K5                 -0.024-0.038      +025
3247          CD-49 3430  69194235819               3957    081040.4-495333081334.1-501146265.60-08.65 5.51  +1.61 +2.00 +0.47   M1III              +0.009-0.004 +.007-007                          *
3248          BD+12 1803  69243 97694    I         R Cnc    081103.0+120201081633.9+114335211.75 24.14 7.13  +1.56 +0.40 +2.98   M7IIIe             +0.012-0.010 +.021+032V?                        *
3249 17Bet CncBD+09 1917  69267116569 312I   6704   3973    081105.5+092938081630.9+091108214.25 23.05 3.52  +1.48 +1.77 +0.78   K4IIIBa0.5         -0.044-0.049 +.012+022V?   < 17  9.4  29.2      *
3250          CD-45 3914  69302219657      W                081111.1-453147081423.9-455004261.98-06.17 5.83  -0.20 -0.79         B2IV-V             -0.004+0.008      +020           1.5   0.4AB   3
3251          CD-30 5946  69445199010      W                081151.9-303706081552.5-305533249.71  2.33 6.21  +0.78 +0.35         G5-8III+G:         -0.047-0.005D+.009+006V          1.9   2.2AB   3
3252          BD+09 1921  69478116585                       081206.7+091032081731.7+085158214.68 23.13 6.29  +0.98 +0.75         G8III              -0.012-0.013      +029
3253          CD-35 4401  69511199015                       081211.7-353545081558.9-355410253.86-00.43 6.16  +1.55 +1.70         K2III              +0.005+0.015      +030
3254 30    LynBD+58 1112  69548 26784                       081221.6+580318082026.1+574436159.73 34.43 5.89  +0.39 -0.10         F4V                +0.061+0.014      -015V
3255          BD-20 2467  69589175497                       081230.0-210044081654.2-211913241.80  7.83 6.60  +0.02               A0                 -0.016+0.026      +010
3256          CD-50 3227  69596235838                       081230.3-500830081523.3-502658265.98-08.55 6.44  +1.49 +1.87         K3-4III            -0.012-0.020      +010
3257 21    PupBD-15 2362  69665154076               3986    081248.0-155831081723.1-161706237.58 10.66 6.16  +0.01 +0.08         A2V                -0.020-0.008      +000
3258          BD+54 1217  69682 26788                       081251.8+535323082029.1+533428164.78 34.55 6.49  +0.28?              F0IV               -0.065-0.092      +011       34
3259          BD-12 2449  69830154093                       081339.1-121736081823.9-123755234.55 12.83 5.98  +0.76 +0.33 +0.38   G7.5V              +0.267-0.979 +.082+030V
3260          CP-62  985  69863250164      W                081345.0-623625081515.9-625457276.80-15.13 5.16  +0.09 +0.10         A2V                -0.023-0.022 +.031+004      199: 2.6   3.9      *
3261          CD-29 5897  69879175543                       081354.2-294133081758.3-300012249.20  3.22 6.45  +1.04               K0III              +0.041+0.014      -012
3262 18Chi CncBD+27 1589  69897 801041217                   081359.4+273230082003.9+271304195.78 30.47 5.14  +0.47 -0.06         F6V                -0.018-0.378 +.061+033        0
3263          BD+61 1043  69976 14522                       081419.9+605652082244.1+603752156.20 34.52 6.41R +0.97?              K0III              +0.008-0.002 +.015-006
3264          BD+21 1817  69994 801122646                   081431.0+210348082021.0+204452202.85 28.45 5.83  +1.13 +1.06         K1III              +0.060-0.047      -017V
3265          BD-09 2471  699971541051218          HQ Hya   081427.5-095113081915.1-100957232.52 14.30 6.32  +0.33 +0.17         F3IIIpDel Del      -0.041+0.038      +032                          *
3266          CD-35 4452  70002199061                       081428.3-350822081817.4-352706253.75  0.21 5.58  +1.24               K2III              +0.018+0.002      -009
3267          CD-36 4443  70003199059      W                081429.5-370346081812.6-372227255.33-00.88 6.70  +0.23 -0.03         A0IV+G:            -0.022+0.019D+.009+017SB2        0.8   1.9      *
3268 19Lam CncBD+24 1909  70011 80113                       081435.4+242014082032.1+240120199.33 29.59 5.98R -0.03 -0.12         B9.5V              -0.023-0.021      +023SB     82
3269          BD+04 1954  70013116630      W                081434.4+041545081949.9+035652219.71 21.43 6.05  +0.97 +0.64         G8III              +0.034-0.030      -047           0.0   0.2
3270          CD-36 4449  70060199070 313                   081448.7-362057081833.3-363934254.78-00.42 4.45  +0.22 +0.11 +0.13   A7III              -0.106+0.097 +.057+005      129
3271          BD-00 1966  70110135783                       081506.8-003532082013.1-005434224.31 19.20 6.18  +0.60 +0.15         F9V                +0.056-0.094      +010V?  =< 10
3272          BD-04 2303  70148135787                       081519.5-050048082017.1-051945228.35 17.02 6.13  +1.33 +1.52         K2III              +0.024-0.027      -037
3273          CD-34 4627  70235199084                       081537.8-341633081929.4-343525253.18  0.90 6.43  -0.08 -0.37         B9pHgMn            -0.018 0.000      +012                          *
3274          CP-58 1095  70267235872      W                081552.9-585110081755.8-591001273.67-12.91 6.42  +0.39  0.00         F5V                -0.044-0.032      -002           3.3  43.8AB   3
3275 31    LynBD+43 1815  70272 42319 314I          4030    081559.4+433032082250.1+431117177.39 34.34 4.25  +1.55 +1.90 +0.88   K4.5III-IIIb      e-0.021-0.096 +.023+024     < 19:                *
3276          BD-22 2233  70302175634      W                081606.3-223632082027.4-225529243.61  7.63 6.13  +1.04               K0                 -0.011+0.001      +026V          7.0   6.7
3277          BD+53 1246  70313 268192649                   081614.2+533231082348.5+531311165.21 35.05 5.51  +0.11 +0.08         A3V                -0.026-0.095 +.030+021
3278          BD-01 2017  70340135804        6762           081615.7-011703082120.2-013608225.10 19.11 6.50  +0.02 -0.02         A2Vpn:EuSrCr:Si    -0.017-0.041D+.009+029           0.7   0.5      *
3279          BD-19 2369  704421541502647                   081653.6-194539082121.2-200445241.33  9.38 5.58  +0.77 +0.50 +0.45   G2III+A3           -0.001-0.013      -008SB  =< 50                 *
3280          CP-65  907  70514250186                       081712.5-651756081818.9-653648279.39-16.22 5.07  +1.15 +1.19 +0.58   K1III              +0.027+0.020 +.013+000
3281          BD-17 2464  70523154159                       081722.3-171602082154.6-173511239.29 10.87 5.75  +1.05 +0.98         K1III              -0.096-0.014      +069                          *
3282          CD-32 5185  705551991181219I                  081726.7-324410082123.0-330316252.13  2.10 4.83  +1.45 +1.60 +0.75   K2.5II-III         -0.008+0.004 +.030+033
3283          CD-36 4513  70556199119      W                081734.3-360958082121.0-362904254.95  0.14 5.20  -0.19 -0.82         B2IV-V             -0.005-0.003      +016SB     71  6.8   6.8      *
3284 20    CncBD+18 1930  70569 977811220                   081738.3+183912082321.8+181956205.70 28.25 5.95  +0.17 +0.13         A9V                -0.054-0.028 +.011+036
3285          BD-05 2512  705741358322650                   081734.7-055135082230.2-061045229.41 17.06 6.15  +0.22 +0.13         A8IV               -0.062+0.003      +018      125
3286          CD-39 4245  706121991232648                   081747.0-391808082124.2-393715257.54-01.64 6.16  +0.17               A3V                +0.011+0.002      -012V?
3287          BD+42 1859  70647 42342    I W                081756.6+421937082442.8+420018178.86 34.54 6.02  +1.59 +1.94         K5III              +0.015+0.007      +027           2.5  79.2
3288          BD-07 2452  70652135840    I                  081800.9-071323082254.0-073236230.69 16.44 5.96  +1.67 +1.85         M1III              -0.023+0.002      +050
3289 22    PupBD-12 2490  70673154177               4046    081804.8-124400082246.8-130317235.52 13.50 6.11  +1.00 +0.89        gG7                 -0.051-0.043      -017
3290 21    CncBD+11 1830  70734 97788    I   6787           081827.0+105716082355.2+103755213.68 25.32 6.08  +1.49 +1.54         M2III              +0.004-0.023      +003           3.3   1.0
3291          CD-25 5988  70761175709        6782           081836.3-260139082249.9-262053246.77  6.16 5.90  +0.37 +0.27?        F3Ib               -0.007+0.008 -.009+065           0.1   0.1AB   3*
3292          BD+35 1819  70771 607942652                   081840.9+352005082504.9+350041187.20 33.45 6.06  +1.27               K0                 -0.012-0.012      +033
3293          CP-57 1490  70839235917              Var?     081858.6-573914082112.0-575823272.87-11.91 5.97  -0.09 -0.80         B1.5III            -0.007+0.009      +020V     178                 *
3294          CD-48 3734  70930219848      W       Var?     081927.0-481009082231.6-482925264.98-06.50 4.82  -0.15 -0.84 -0.13   B1V                -0.013+0.008      +027V     169  1.0   0.8
3295          BD-04 2328  70937135882                       081937.6-042331082436.4-044301228.37 18.25 6.01  +0.46 +0.04         F2V                +0.022-0.051      -035SB
3296          CD-37 4638  70946199166               4056    081935.1-375749082317.2-381709256.65-00.57 6.32  +1.63 +1.86         M1III              +0.028-0.003      -009                          *
3297  1    HyaBD-03 2333  70958135877                       081935.9-032537082435.0-034504227.50 18.74 5.61  +0.46 -0.06         F3V                -0.210-0.024 +.057+072SBO    35                 *
3298          CP-63  940  70982250202                       081944.0-634706082107.7-640622278.20-15.19 6.12  +0.93               G6-8III            -0.004-0.025      -003
3299 25    CncBD+17 1842  71030 97806                       082010.2+172233082549.9+170246207.30 28.32 6.14  +0.41 -0.03         F6V                -0.192-0.151 +.035+038
3300          CD-51 2980  71043235933                       082007.3-514807082255.2-520726268.05-08.49 5.85  +0.01               A0V                -0.010+0.008      +003
3301   Kap1VolCP-71  677  71046256497      W     A          082006.5-711147081949.0-713054284.83-19.05 5.37  -0.06 -0.31         B9III-IV           -0.013+0.030      +036SB2    50  0.3  65.0AB   3*
3302   Kap2VolCP-71  678  71066256499      W     B Var?     082017.7-711112082000.7-713019284.82-19.03 5.65  -0.10 -0.31         A0IVMn             -0.008+0.044      -006           0.3  65.0AB   3*
3303          BD+67  545  71088 145682662                   082020.3+673735082946.2+671751148.07 34.30 5.88  +0.97               G8III              -0.066+0.010      -003
3304 22Phi1CncBD+28 1602  71093 801812656I W                082022.8+281323082627.7+275337195.50 32.02 5.57  +1.40 +1.68         K5III              -0.033-0.114      +024V          4.3 130.6
3305          BD+02 1965  710951167472654I                  082024.0+022540082535.5+020608222.19 21.83 5.73  +1.53 +1.86         K5III              -0.020-0.006      +012V?
3306          BD+08 2053  711151167522655    6805           082032.9+075326082554.8+073352216.96 24.43 5.13  +0.94 +0.64 -0.06   G7II-IIIFe-1       -0.034-0.012 +.016+015V    < 19: 5.2  30.4
3307   Eps CarCP-59 1032  71129235932 315           4058    082027.7-591115082230.8-593035274.29-12.60 1.86  +1.28 +0.19 +0.89   K3III+B2:V         -0.026+0.014      +002                          *
3308          BD-22 2262  71141175777                       082034.1-224947082455.1-230913244.38  8.36 5.68  +0.06               A2                 -0.041+0.042      +026V?
3309          BD+46 1398  71148 423692660                   082038.4+455925082736.8+453911174.49 35.42 6.32  +0.62 +0.12         G5V                -0.017-0.350 +.046-034V   =< 10                 *
3310 23Phi2CncBD+27 1612  71150 80187        6815B  4078    082044.1+271540082646.8+265604196.61 31.82 6.32H +0.18 +0.09         A6V                -0.014+0.002D+.010-025V?    126  0.1   5.2      *
3311 23Phi2CncBD+27 1612  71151 80188        6815A  4078    082044.4+271544082647.0+265607196.61 31.82 6.30H +0.18 +0.09         A3V                -0.016-0.003D+.010-029V?    135  0.1   5.2      *
3312 24    CncBD+25 1920  71152 80184        6811A  4076    082042.9+245147082639.8+243203199.28 31.08 7.02  +0.32 +0.07         F0III              -0.041-0.080D+.021+015           0.6   5.7AB   3*
3313 24    CncBD+25 1920  71153 80185        6811B  4076    082043.2+245151082640.1+243207199.28 31.08 7.81  +0.52 -0.04         F7V                -0.045-0.085D+.021+019           0.6   5.7AB   3*
3314          BD-03 2339  71155135896 316                   082039.8-033448082539.6-035423227.78 18.89 3.90  -0.02 -0.02 -0.05   A0V                -0.066-0.023 +.026+010V     125
3315          CD-23 7277  711761757831221I   6800A          082044.6-234318082503.7-240246245.15  7.88 5.28  +1.48 +1.83         K5III+K1III        -0.028+0.026 +.031+026SB         3.5  41.0      *
3316          BD-20 2522  71196175792                       082053.3-204319082519.1-210245242.67  9.62 6.01  +0.40               F3V                -0.019+0.061      -002
3317          BD-16 2442  71231154240                       082105.7-170648082539.4-172622239.67 11.69 6.44                      K0                 -0.007-0.005      -013V
3318   Alp ChaCP-76  507  71243256496                       082106.7-763613081831.6-765511289.86-21.68 4.07  +0.39 -0.02 +0.23   F5III              +0.110+0.108 +.053-014       36
3319 27    CncBD+13 1912  71250 978192658I         BP Cnc   082112.1+125905082643.9+123916211.95 26.79 5.50  +1.60 +1.92 +1.41   M3III              -0.020-0.104 +.030-007V                         *
3320          BD-14 2517  712671542442657                   082116.8-143615082555.6-145547237.56 13.13 5.98  +0.17 +0.13 +0.06   A5m                -0.010+0.040      +009       30
3321  2    HyaBD-03 2345  71297135916      W       LM Hya   082127.4-033931082627.2-035915227.96 19.02 5.59  +0.22 +0.09         A5III-IV           -0.053-0.061 +.028+027       22  3.8  72.8      *
3322          CD-42 4219  71302219890      W                082129.6-422639082457.2-424610260.50-02.89 5.98  -0.17 -0.69         B3V                -0.011+0.019      +023           0.4   0.4
3323  1Omi UMaBD+61 1054  71369 14573 317I   6830   4093    082157.5+610309083015.9+604305155.97 35.43 3.36  +0.84 +0.52 +0.42   G5III              -0.133-0.107 +.009+020       17  7.1 143.0AC   4*
3324          BD-12 2524  71377154257    I          4085    082158.9-121221082641.9-123204235.59 14.59 5.54  +1.19 +1.14         K2III             v-0.091-0.018      +061V                         *
3325          BD-05 2530  71433135931                       082221.9-060447082717.2-062435230.26 17.96 6.59  +0.51 +0.06         F4III              -0.040-0.030      +007V      12                 *
3326          CD-41 4119  71459219910                       082221.9-414933082551.9-420912260.10-02.40 5.47  -0.15 -0.63         B3V                -0.009+0.003      +028       24                 *
3327          CD-38 4462  71487199222      W     A NO Pup   082237.5-384351082617.6-390334257.62-00.54 6.53  -0.09 -0.36         B9V                -0.017-0.008D+.005+028           0.6   8.0AxBC 3*
3328          CD-38 4462  71488199224      W     B          082238.0-384356082618.2-390337257.62-00.54 7.25  +0.08 +0.06         A2V:               -0.011+0.006D+.005+019V?         0.6   8.0AxBC 3*
3329 28    CncBD+24 1931  71496 80204              CX Cnc   082241.0+242837082836.8+240841199.88 31.38 6.10  +0.24 +0.15         F0Vn               -0.031-0.058      +009SB    130                 *
3330          CD-51 3004  71510235962      W                082238.4-512401082531.0-514341267.94-07.93 5.17  -0.16               B2Ve               -0.041-0.012      +018      158  4.2  25.7      *
3331          CD-28 6048  71523175851                       082243.4-285313082650.8-291255249.65  5.25 6.73  -0.05               B9.5V              -0.005+0.017      +037V
3332          BD+69  472  71553 14582                       082300.2+693920083253.4+691912145.61 34.14 6.31  +1.35               K0                 +0.030-0.027      -030
3333 29    CncBD+14 1899  71555 978431222  D                082302.5+143231082837.3+141239210.56 27.83 5.95  +0.19 +0.14         A5V                -0.014-0.011      +002SB    120  0.5   0.2 O   3*
3334   Eta VolCP-72  694  715762565052653  W                082258.5-730435082204.4-732400286.66-19.82 5.29  +0.01 -0.01         A0-1IV-V           -0.034+0.031 +.010+020           6.3  42.4AC   3
3335          BD-20 2538  71581175870      W       VV Pyx   082306.5-203049082733.3-205038242.80 10.16 6.56  +0.04               A2V                +0.010-0.008      +001SB2        1.1   0.3      *
3336          CD-31 6079  71622199246                       082315.2-312034082716.4-314023251.71  3.91 6.33  +0.90               G8III              -0.016-0.011      -001SB2                       *
3337          BD-02 2581  71663135958        6828  LO Hya   082326.3-021108082829.2-023102226.88 20.20 6.39  +0.33 +0.08         A5m                -0.005-0.016D+.015-014SBO        0.1   0.4AB   5*
3338          BD-08 2374  71665135956                       082328.7-082903082819.7-084858232.55 16.91 6.43  +1.20 +1.23         K0                 -0.001-0.040      +011V
3339          CD-25 6109  71688175883                       082338.9-254806082753.5-260757247.24  7.22 6.62  +0.11               A2IV-V             -0.055-0.002      +026V?
3340   The ChaCP-77  383  71701256503 318  W                082338.7-770943082038.5-772904290.46-21.84 4.35  +1.16 +1.20 +0.57   K2IIICN0.5         -0.134+0.042 +.036+022           7.7  31.
3341          CP-52 1474  717222359762661                   082339.4-522841082625.2-524827268.92-08.43 6.05  +0.06               A0V                -0.037-0.003      +030
3342          BD-09 2532  71766135965                       082401.8-092459082850.9-094454233.45 16.53 6.00  +0.42 +0.20         F2III              -0.012+0.008      +003
3343          CD-34 4842  71801199260      W        4092    082407.2-344657082759.3-350650254.60  2.03 5.75  -0.15 -0.70         B3V                -0.023+0.002      +025SB    158  0.0   0.1AP   3*
3344          BD-22 2286  71815175902                       082414.2-224420082835.9-230418244.80  9.10 6.51  +0.05 +0.08         A2                 -0.043-0.022      +054
3345          BD-20 2549  71833175912      W                082426.8-203703082853.3-205701243.07 10.36 6.67  -0.06 -0.41         B8III              -0.012-0.008      +007           5.8  18.9      *
3346          CP-64  878  71863250226                       082429.6-641617082551.6-643603278.92-15.01 5.97  +0.96 +0.73         G8-K0III           -0.012+0.007      +019V
3347   Bet VolCP-65  933  71878250228 319           4088    082439.0-654810082544.2-660813280.26-15.84 3.77  +1.13 +1.14 +0.56   K1III              -0.036-0.155 +.042+027
3348          BD+37 1870  71906 60866                       082451.0+373604083119.9+371557184.80 35.11 6.18  -0.03 -0.15         A0V                -0.016+0.001      +016V      56
3349          CP-54 1647  71919236001      W                082452.7-544048082727.4-550042270.85-09.55 6.53  -0.02               A0V                -0.018-0.010      +020           0.0   0.2
3350          CP-52 1484  71935236002              GU Vel   082452.0-524527082736.5-530519269.26-08.44 5.09  +0.25 +0.14         A9-F0III-IV        -0.068+0.013 +.021+025      122                 *
3351          BD+53 1259  71952 26896                       082503.3+532713083233.5+530653165.30 36.36 6.24  +1.01 +0.83 +0.51   K0IV               +0.013-0.077      +044V                         *
3352          BD+75  342  71973  6592        6872   4133    082510.4+750354083648.7+744325139.27 32.95 6.31                      A2m                -0.017-0.023      -006SBO    20  3.5   1.8      *
3353          CD-26 6103  71997175939               4104    082515.1-265954082927.6-271957248.44  6.81 6.70  -0.13 -0.55         B4V                 0.000-0.008      +006V
3354  2    UMaBD+65  638  72037 14590                       082539.1+652910083436.2+650842150.52 35.21 5.47  +0.18 +0.10 +0.04   A2m                -0.049-0.063 +.017-016       17
3355 30Ups1CncBD+24 1940  72041 802292666                   082535.7+242507083130.5+240452200.19 31.98 5.75  +0.28 +0.05         F0IIIn             -0.082-0.044      +019V      97
3356          CD-43 4337  720672199822665  W        4106    082543.5-434936082907.6-440938262.08-03.08 5.79  -0.16 -0.75 -0.15   B2Vn              t-0.003+0.004      +003V?         0.1   0.2      *
3357 31The CncBD+18 1963  72094 978812667I W        4117    082553.6+182557083135.7+180540206.77 29.98 5.35  +1.56 +1.93         K5III              -0.061-0.058      +045V?         0.0   0.3 O   3*
3358          CD-47 4004  72108219985      W                082555.2-473542082904.7-475545265.14-05.28 5.33  -0.14 -0.79         B2IV               -0.011-0.001      +029SB     66  0.0   0.2AP   4*
3359          CD-44 4462  72127219996      W                082605.3-442325082927.6-444330262.57-03.36 4.99  -0.16 -0.79 -0.17   B2IV               +0.004-0.003D+.000+022SB    163  1.7   4.6      *
3360          BD+38 1920  72184 60890 320                   082625.0+382134083255.0+380059183.95 35.54 5.90  +1.11 +1.17         K2III              -0.103-0.169 +.011+015V                         *
3361          BD+10 1816  72208116863                       082628.5+100907083154.6+094852215.44 26.74 6.83  -0.07 -0.17         B9p:Hg:            +0.003-0.007      +008SB
3362          CD-31 6165  72227199308    I R        4113    082627.9-314924083028.6-320934252.51  4.18 5.65  +1.49 +1.78         K2III              -0.016-0.005      +019           0.0   0.1      *
3363          CD-45 4183  72232220007                       082629.6-455948082945.6-461955263.91-04.26 5.99  -0.15               B5III              -0.029+0.002      +012
3364          CD-36 4715  72268199310               4115    082641.4-362306083029.6-364316256.21  1.50 6.69  +1.95 +2.09         M2Iab              -0.019+0.012      +025V                         *
3365 32    LynBD+36 1836  72291 60896                       082656.9+364632083321.8+362611185.89 35.37 6.24  +0.36 -0.08         F5Vb vw            -0.143+0.001 +.032+000V      20:
3366 33Eta CncBD+20 2109  72292 80243 321I                  082655.5+204652083242.5+202628204.33 31.06 5.33  +1.25 +1.39 +0.61   K3III              -0.047-0.043 +.013+024
3367          BD-19 2438  72310154359        6862           082701.3-191422083130.9-193439242.27 11.65 5.42  -0.06               A0V                -0.033-0.007 +.003+012V?     61  0.6   0.2      *
3368          CP-54 1667  72322236032      W                082701.5-545117082936.3-551128271.17-09.40 6.36  +0.80               G0V                -0.015-0.015      -012V          5.9   2.4
3369 32Ups2CncBD+24 1946  72324 80245                       082705.4+242530083300.1+240505200.31 32.31 6.36  +1.02 +0.88 +0.50   G9III              -0.068-0.047 +.027+075                          *
3370          CP-69  919  72337250235                       082702.6-694540082716.9-700536283.86-17.80 5.53  -0.03 -0.04         A0V                -0.008+0.048      +013V?
3371          CD-44 4477  72350220025      W                082716.7-442401083039.2-444414262.71-03.19 6.30  -0.02 -0.49 -0.06   B4IV               -0.013+0.011      +024V?         4.5   3.2      *
3372 34    CncBD+10 1818  72359 97902                       082713.3+102418083239.9+100358215.27 27.02 6.46  -0.02 -0.04         A1V                +0.003-0.005      -011SB     20
3373          CD-38 4566  72436199329      W                082743.0-384334083124.6-390351258.20  0.26 6.31  -0.15 -0.56         B4V                -0.017+0.004      +011SB         1.8   3.8AB   3
3374          BD-14 2564  72462154373      W                082754.4-144129083233.3-150146238.56 14.41 6.38  +0.27 +0.09         F0Vn               -0.050+0.052      +023V          2.1   0.3
3375          CD-47 4048  72485220039                       082800.1-473143083110.8-475200265.30-04.96 6.39  -0.15 -0.65         B2.5V              +0.001+0.002      +011
3376          BD+13 1940  72505 97913      D                082812.6+133558083345.1+131526212.12 28.59 6.28  +1.17 +1.23         K0III              -0.029-0.045      +028           0.0   0.1
3377 33    LynBD+36 1840  72524 60907                       082818.4+364546083443.8+362510185.96 35.64 5.78  +0.04 +0.02         A2Vnn              -0.031-0.044      +025
3378          BD+05 1997  72561116890                       082827.1+050554083343.5+044524220.68 24.88 5.87  +1.07 +0.87         G5III              -0.005-0.013      +001V?
3379          BD+74  370  72582  6605 322                   082835.7+735846083942.6+733747140.44 33.48 6.15  +1.02               G7III              -0.015-0.099 +.015+016V
3380          BD+08 2077  72617116894      W                082850.0+084741083413.3+082707217.09 26.66 6.03  +0.33 +0.04         F3IV               -0.003-0.031      +011       67  7.6  31.2
3381          CD-24 7089  72626176061        6871           082845.5-241555083304.8-243623246.67  9.05 6.19  +0.27               A8IV               -0.007-0.016 +.016-008V?         0.2   0.5AB   3*
3382          CP-53 1729  72650236055                       082849.0-540312083129.6-542339270.67-08.72 6.34  +1.31 +1.40         K3III              +0.020-0.047      -007
3383          BD-01 2074  726601360442669                   082858.1-014837083401.6-020906227.29 21.58 5.81   0.00  0.00         A1V                -0.032+0.020      +004    =< 41
3384          CD-31 6229  72673199352                       082857.2-311052083251.5-313003252.29  4.98 6.38  +0.79 +0.34 +0.28E  K0V                -1.110+0.765 +.086+016V
3385          CD-34 4959  72688199353              VX Pyx   082903.8-341735083258.6-343802254.82  3.14 6.36  +0.95               K0V                -0.010+0.004      +006V                         *
3386          CP-52 1517  72737236062      W                082918.3-525219083204.8-531244269.75-07.97 5.69  +0.58               K0III+A3           -0.021+0.007      +019           1.0   0.8      *
3387 35    CncBD+20 2118  72779 97928                       082934.5+195603083519.4+193524205.51 31.34 6.58  +0.68 +0.25 +0.37   G0III              -0.036-0.010      +036V      91                 *
3388          CD-37 4850  72787199363              Var?     082935.6-380145083319.9-382216257.87  0.98 6.49  -0.18 -0.65         B2.5V               0.000-0.007      +010                          *
3389          CD-38 4610  72832199370               4137    082955.6-383024083338.3-385056258.29  0.74 5.96  -0.14               B5III              -0.034+0.006      +028                          *
3390          CD-46 4300  72900220103                       083015.1-463741083330.4-465816264.81-04.11 6.24  +1.56               K3III              +0.011-0.009      +023
3391  3Pi 1UMaBD+65  643  72905 14609                       083019.1+652200083911.7+650115150.55 35.70 5.64  +0.62 +0.07 +0.33   G1.5Vb             -0.024+0.088 +.073-012V?      4                 *
3392          BD+03 2014  72908116920                       083012.3+030516083524.9+024437222.85 24.30 6.33  +1.02 +0.87         G9III              -0.006+0.007      -004
3393          CP-80  258  729222584962664                   083015.6-803511082419.8-805451293.85-23.26 5.69  +1.02 +0.74         G8III              -0.149+0.216      +027V?
3394          BD+15 1851  72943 97950                       083031.2+153935083607.7+151849210.22 29.94 6.32                      F0IV               +0.008-0.024      +004       65
3395          BD+07 1997  72945116929        6886A          083032.0+065808083551.0+063712219.12 26.20 5.99  +0.52 +0.04         F8V                -0.128-0.143 +.052+025SBO        1.2  10.3AB   5*
3396          BD+07 1997  72946116931        6886B          083032.3+065818083551.3+063721219.12 26.21 7.25  +0.71 +0.27         G5V                -0.128-0.143 +.052+027V          1.2  10.3AB   5*
3397          CD-32 5465  72954199388      W                083031.5-321504083431.7-323555253.37  4.61 6.43  +0.75 +0.23?        G5IV-V             -0.044-0.133      +008           0.0   0.2      *
3398  3    HyaBD-07 2540  729681360762672          HV Hya   083035.4-073817083528.2-075856232.80 18.85 5.72  -0.03 -0.01         A1pSrCrEu         v-0.020+0.013      +024SB?    16                 *
3399          CD-37 4873  72993199389      W        4143    083042.4-371603083429.3-373641257.39  1.61 6.30  +1.56 +1.74         K5III              -0.007+0.001      +023           2.3   1.9      *
3400          BD+53 1268  73017 26935                       083054.3+534459083822.2+532405164.89 37.21 5.66  +0.96               G8IV               -0.078-0.017      -043V
3401          BD+60 1148  73029 14612                       083101.8+601721083910.2+595622156.74 36.63 6.48                      A2Vn               -0.019-0.033      -014
3402          CD-26 6225  73072176131                       083114.2-262955083528.8-265037248.82  8.18 5.96  +0.37               A1V+G-KIII         -0.022+0.014      +009SB2
3403  4Pi 2UMaBD+64  698  73108 146162677I                  083128.5+644038084012.8+641940151.36 35.96 4.60  +1.17 +1.16 +0.63   K1+IIIb            -0.060+0.023 +.016+015     < 17                 *
3404          CD-39 4519  73121199402                       083131.9-393734083512.6-395812259.37  0.31 6.47  +0.58               G1V                +0.087+0.053      +034
3405          BD+53 1269  73131 26940                       083134.2+531630083900.0+525530165.48 37.34 6.42R                     K0                 -0.022-0.032      +039
3406 36    CncBD+10 1837  731431169532675                   083140.5+100011083705.8+093920216.22 27.83 5.88  +0.09 +0.10         A3V                -0.029-0.003      +017
3407          CD-49 3646  731552201382674                   083140.3-493559083443.6-495639267.33-05.71 5.01  +1.33 +1.38 +0.68   K1-2II             -0.012+0.016 +.009+004
3408          BD+53 1272  73171 26946 323                   083153.1+530343083917.6+524242165.75 37.39 5.91  +1.17 +1.05        gK1                 -0.029-0.028      +027V?                        *
3409          BD+33 1728  73192 60939                       083204.2+330904083819.0+324807190.51 35.69 5.94  +1.12              gK2                 -0.024-0.010      +004
3410  4Del HyaBD+06 2001  732621169651223  W                083221.7+060309083739.4+054213220.26 26.18 4.16   0.00 +0.01 +0.01   A1Vnn              -0.066-0.007 +.033+011SB    249  6.2 243.7
3411          BD-04 2401  73281136103                       083228.5-043508083727.1-045601230.32 20.88 6.19  +1.05 +0.89         K0                 -0.006+0.021      +018
3412 37    CncBD+10 1840  73316116975                       083240.2+095527083805.2+093429216.42 28.01 6.53  -0.02 -0.04         A1V                -0.039-0.007      +028       50
3413          CD-50 3417  73340236110              HV Vel   083252.8-503721083552.1-505812268.27-06.18 5.80  -0.13 -0.54         B8Si              v-0.004-0.004      +023V?                        *
3414          CP-57 1591  73389236106                       083257.9-573947083519.6-580033273.97-10.38 4.86  +1.00 +0.81 +0.49   K0III              +0.042+0.022 +.017+024
3415          CP-57 1590  73390236105                       083256.1-575241083515.4-581330274.14-10.52 5.26  -0.14 -0.61 -0.16   B3V+B3Vn           -0.032+0.006      +021V     140                 *
3416          BD-06 2669  73451136117                       083324.8-061844083820.3-063945232.02 20.16 6.51  +0.44 +0.17         A1V+G               0.000-0.005      +030V
3417          CP-72  713  73468256524                       083323.0-730050083242.2-732124287.03-19.14 6.12  +0.95 +0.66         G8III              -0.038+0.082      -027
3418  5Sig HyaBD+03 2026  734711169881224I                  083331.8+034133083845.4+032029222.71 25.31 4.44  +1.21 +1.28 +0.56   K1+III             -0.018-0.018 +.035+025V    < 17                 *
3419          CD-33 5257  73476199436      W                083331.5-332342083729.7-334445254.66  4.42 6.48  +0.33               F0IV-V             -0.021-0.046      +012V?         5.3  22.4AB   4
3420   Eta PyxCD-25 6356  734951761892676  W                083335.8-255417083752.2-261518248.66  8.96 5.27  -0.04               A0V                -0.020-0.006 +.038+031      203  7.8  16.0
3421          CD-39 4574  73524199438                       083342.5-394756083719.9-400851259.76  0.53 6.55  +0.60 +0.14         G4IV-V             -0.308+0.033 +.059-002
3422 34    LynBD+46 1422  73593 424901225I                  083406.5+461105084101.1+455002174.42 37.76 5.37  +0.99 +0.75         G8IV               +0.025+0.092 +.015-037
3423          BD+32 1776  73596 60970                       083406.3+321744084018.3+315631191.65 35.92 6.10R                     F5III              -0.035-0.028      +013SB
3424          BD+08 2099  73599116997                       083402.5+082213083924.6+080102218.16 27.62 6.45  +1.08 +0.99         K1III              -0.025-0.037      +017
3425          BD-19 2489  736031544922678I   6903           083409.7-192308083840.3-194413243.40 12.93 6.33  +1.59 +1.83         K5III              +0.014+0.006D+.005+000           3.0   4.3
3426          CD-42 4451  73634220204 324                   083407.6-423821083738.7-425921262.05-01.14 4.14  +0.11 +0.16 +0.16   A6II               -0.006+0.008 +.018+019        0
3427 39    CncBD+20 2158  73665 80333      W        4179    083421.3+202139084006.4+200028205.51 32.54 6.39  +0.98 +0.83 +0.47   G8III              -0.036-0.015      +035V       0  0.0 149.8AB   5*
3428          BD+20 2166  73710 98021        6921           083437.7+200125084022.1+194012205.89 32.48 6.44  +1.02 +0.90 +0.49   G9III              -0.035-0.017      +036SB   < 45  1.3  63.2AC   4*
3429 41Eps CncBD+20 2171  73731 98024      W                083442.9+195355084027.0+193242206.06 32.45 6.30  +0.17 +0.16 +0.06   A8Vn               -0.039-0.006      +030SB2    82  1.1 134.9      *
3430          BD-22 2345  73752176226        6914           083445.2-221918083908.0-223943245.90 11.31 5.05  +0.73 +0.36         G6IV               -0.235+0.436 +.063+043SB         1.3   0.8AB   3*
3431  6    HyaBD-11 2420  73840154515 325I                  083517.2-120718084001.5-122831237.40 17.34 4.98  +1.42 +1.62 +0.75   K4III              -0.081-0.002 +.023-011V?   < 19:                *
3432          CP-62 1058  738872502882679  W                083532.3-623006083718.8-625113278.15-12.99 5.47  +1.02 +0.83         K0III              -0.008-0.025 +.020+021V?         5.6   7.6
3433   Zet PyxCD-29 6544  738981762532680I   6923   4186    083533.4-291218083942.5-293340251.59  7.31 4.89  +0.90               G4III              -0.020-0.089 +.021-032           4.2  52.4
3434          CD-36 4872  73900199473      W                083532.4-361518083922.1-363625257.18  3.00 6.13  +0.42               F1IV               -0.171+0.040D+.024+009SB         1.5   0.9      *
3435          CP-52 1565  73952236142      W                083554.4-524417083844.9-530526270.25-07.09 6.47  -0.10 -0.35         B8Vn               -0.009+0.013      +011V     184  7.5  22.2      *
3436          BD+47 1606  73971 42508                       083602.9+471534084300.2+465404173.08 38.13 6.22  +0.96               G8III              -0.039-0.045      -007
3437          BD-08 2452  739971361762682                   083610.5-084148084101.6-090307234.54 19.43 6.63  -0.02 +0.02         A1Vn               -0.033+0.002      +010      112
3438   Bet PyxCD-34 5128  740061994902681  W                083611.2-345712084006.2-351830256.24  3.90 3.97  +0.94 +0.65 +0.48   G7Ib-II            +0.012-0.022 +.018-013SB         8.5  12.6
3439          CD-39 4653  74067220252      W        4193    083638.9-395432084019.3-401551260.19  0.91 5.20  -0.01 -0.14         B9V                -0.044-0.004      +021        0  3.7   4.4      *
3440          CP-53 1796  74071236151              HW Vel   083634.9-530510083923.8-532623270.59-07.22 5.48  -0.16 -0.55         B5V                -0.029+0.019      +015V?    128                 *
3441  9    HyaBD-15 2554  741371545522684I   6937           083704.9-153502084143.3-155636240.63 15.71 4.88  +1.06 +0.92 +0.53   G9.5III            +0.001-0.098 +.032-002V    < 19: 6.8  31.0      *
3442          CP-52 1579  74146236157      W        4194    083705.5-524200083957.6-530318270.33-06.92 5.19  -0.14 -0.57         B4IV               -0.022+0.012      +014SB     56  4.4  16.6      *
3443          CP-59 1075  74148250291      W                083705.8-595748083913.2-601902276.18-11.33 6.36   0.00               A0V                -0.032+0.026D+.013+009           0.6   1.2
3444          CD-44 4679  74167220261               4198    083710.7-445007084035.3-451129264.12-02.06 5.71  +1.65 +2.00         M0III              +0.013-0.003      +013
3445          CD-46 4438  741802202651226  W        4199    083718.4-461734084037.6-463856265.28-02.95 3.84  +0.71 +0.30 +0.60   F3Ia                0.000+0.003 +.031+025V      38  6.9  37.5      *
3446          BD-11 2432  74190154558                       083724.0-113629084209.8-115758237.26 18.06 6.45  +0.16 +0.13         A5m                -0.031-0.022      +002
3447          CP-52 1583  741952361641227          o Vel    083725.6-523400084017.6-525519270.25-06.80 3.62  -0.18 -0.64 -0.18   B3IV               -0.022+0.019      +016SB     40                 *
3448          CP-52 1584  74196236165               4197    083726.0-523937084017.4-530055270.33-06.86 5.61  -0.14 -0.50         B7Vn               -0.029+0.030      +015SB    130                 *
3449 43Gam CncBD+21 1895  74198 803781228  W                083729.9+214942084317.1+212807204.17 33.73 4.66  +0.02 +0.01  0.00   A1IV               -0.106-0.039 +.016+029SB     79  4.0 106.3AB   5*
3450 45    CncBD+13 1972  74228 980692686                   083741.7+130223084312.3+124051213.80 30.46 5.64  +0.39 +0.25         A3V+G0III          -0.012+0.002      -013SBO    30  0.0   0.1      *
3451          BD+37 1899  74243 61018                       083745.8+371649084410.1+365505185.71 37.59 6.33R                     F7V                -0.034-0.094      +004       20
3452          CD-46 4448  74272220284                       083756.3-465736084113.1-471901265.87-03.27 4.77  +0.12 +0.12 +0.19   A5II               -0.012+0.007 -.000+017SB      0
3453          CD-48 4020  74273220282                       083754.9-483357084105.3-485521267.13-04.27 5.90  -0.21 -0.90         B1.5V              -0.005+0.007      +018      181                 *
3454  7Eta HyaBD+03 2039  742801170502687          Eta Hya  083759.8+034528084313.5+032355223.25 26.32 4.30  -0.20 -0.74 -0.19   B3V                -0.019-0.001      +017SB?   128                 *
3455          CP-57 1644  74341236179      W                083816.6-571120084043.6-573243274.02-09.53 6.34  +0.20 +0.14         A3V                -0.015+0.025D+.022+014           1.9   3.8
3456          CD-44 4704  74371220300                       083832.7-450308084156.9-452439264.44-02.01 5.23  +0.21 -0.53         B6Iae              -0.003-0.002      +024       53                 *
3457          CP-59 1080  743752361812685  W       V343 Car 083824.4-592415084037.0-594540275.82-10.86 4.33  -0.11 -0.80 -0.13   B1.5III            -0.001 0.000      +013SB     48  8.9  15.7      *
3458          BD+04 2029  74393117069                       083844.5+044143084359.7+042005222.43 26.94 6.37  -0.06 -0.13         B9.5III-IV         -0.032-0.005      +034V
3459          BD-06 2708  74395136221    I W                083845.7-065225084340.4-071401233.29 20.97 4.62  +0.84 +0.49 +0.44   G1Ib               -0.002+0.003 +.007+031     < 17  4.1  78.9AB   3*
3460   The VolCP-69  946  744052565352683  W                083843.1-700147083905.2-702313284.68-17.12 5.20  +0.01 -0.03         A0V                +0.018-0.040 +.006+013       82  5.0  45.0AC   3
3461 47Del CncBD+18 2027  74442 98087 326I   6967           083900.1+183119084441.1+180915208.02 32.90 3.94  +1.08 +0.99 +0.54   K0III-IIIb         -0.018-0.228 +.025+017     < 17  7.5  38.4      *
3462          CD-47 4251  74455220313      W       HX Vel   083902.0-474424084216.0-480557266.60-03.61 5.51  -0.18 -0.88         B1.5Vn             -0.024-0.008      +042V     285  1.3   0.3      *
3463          CD-35 4976  74475199535                       083903.1-353502084257.0-355636257.10  3.97 6.42  +0.02 +0.01         A0V                 0.000+0.018      +008
3464 46    CncBD+31 1876  74485 610292690                   083913.4+310336084521.4+304152193.45 36.71 6.13  +0.94 +0.63 +0.49   G5III              -0.002-0.006 +.008-012V?                        *
3465 49    CncBD+10 1864  74521 980892688          BI Cnc   083919.4+102638084445.0+100454216.71 29.71 5.66  -0.11 -0.25         A1pEuCr            -0.016-0.019      +024SB     20                 *
3466          CP-52 1605  74535236202      W     BCKT Vel   083926.9-524428084218.9-530600270.58-06.67 5.52  -0.15 -0.54         B8Si               -0.029+0.023      +017           0.8  76.6AB   4*
3467          CP-52 1607  74560236205      W     A HY Vel   083933.3-524517084225.5-530650270.60-06.66 4.86  -0.17 -0.66 -0.19   B3IV               -0.015+0.022      +022SB     22  0.8  76.6AB   4*
3468   Alp PyxCD-32 5651  74575199546 327           4220    083934.4-324933084335.5-331111254.99  5.77 3.68  -0.18 -0.88 -0.16   B1.5III            -0.011+0.011      +015V?     19                 *
3469 10    HyaBD+06 2030  74591117088                       083943.5+060235084501.3+054050221.23 27.79 6.13  +0.20 +0.10         A6V                -0.001-0.005      -009SB    132
3470          BD+67  560  74604 14667                       083946.1+670430084849.4+664229148.23 36.25 6.20  -0.11 -0.40         B8V                -0.011-0.039      +005V?
3471          CP-55 1688  74622236206                       083943.3-552457084220.9-554627272.73-08.29 6.29  +1.18 +1.20         K2III              -0.072+0.053      +075
3472          BD-02 2676  74688136243        6977A          084017.7-021415084520.8-023603229.28 23.78 6.41  +0.52 +0.06         F2IV               +0.005-0.006D+.005-018V          0.7   4.7      *
3473          BD-20 2667  747061764041229                   084026.8-204818084455.2-211004245.47 13.28 6.11  +0.22               A5V                -0.020+0.001      -015      170
3474 48Iot CncBD+29 1823  74738 80415    I:  6988B  4238    084036.9+290751084640.0+284555195.87 36.54 6.57  +0.04 +0.04         A3V                -0.018-0.041 +.017+015      200  2.5  30.4      *
3475 48Iot CncBD+29 1824  74739 80416 328I   6988A  4238    084038.8+290733084641.8+284536195.88 36.54 4.02  +1.01 +0.78 +0.49   G7.5IIIaBa0.1      -0.025-0.042 +.017+016     < 19: 2.5  30.4      *
3476          CD-49 3761  747532203612689                   084032.4-492740084340.3-494922268.11-04.49 5.16  -0.20 -1.03         B0IIIn             +0.002+0.004      +028      288
3477          CD-42 4569  74772220371      W                084049.6-421713084424.0-423857262.54  0.05 4.07  +0.87 +0.52 +0.50   G5III              -0.012+0.018 +.051-002           7.1  45.3      *
3478          BD-01 2125  74794136254                       084058.1-014108084602.5-020256228.86 24.21 5.70  +1.10 +1.04        gK0                 +0.043+0.042      +010V?
3479          CD-36 4980  74824199573               4232    084100.8-364702084451.9-370850258.28  3.53 5.76  -0.15               B2III              -0.008+0.001      +012V     148                 *
3480          BD-10 2634  74860154615    I W                084119.0-103833084606.9-110023237.00 19.39 6.25  +1.61 +1.90         K5III              -0.018+0.027      +034V          3.5   2.3
3481 50    CncBD+12 1904  74873 98117                       084127.1+122837084656.0+120636214.84 31.06 5.87  +0.11 +0.05         A1V                -0.068-0.052      +023V      74
3482 11Eps HyaBD+06 2036  74874117112    I   6993  Eps Hya  084128.8+064709084646.6+062508220.72 28.53 3.38  +0.68 +0.36 +0.39   G5III              -0.189-0.051 +.027+036SBO    19  0.9   0.2AB   6*
3483          CD-24 7377  74879176439                       084129.7-250125084549.3-252315249.05 10.91 6.10  +0.08               A2V:               -0.029+0.032      +009V?
3484 12    HyaBD-13 2673  74918154622    I W                084139.0-131055084622.5-133252239.25 18.01 4.32  +0.90 +0.62 +0.46   G8-IIIFe-1         +0.016-0.011 +.013-008SB1  < 17  9.3  19.9      *
3485   Del VelCP-54 1788  74956236232      W                084156.5-542031084442.2-544230272.08-07.37 1.96  +0.04 +0.07 +0.02   A1V                +0.023-0.078 +.051+002V?     40  3.0   2.2AB   4*
3486          BD-01 2130  749881362762693                   084210.8-013150084715.0-015350228.89 24.55 5.29  +0.04 +0.08         A3V                -0.033+0.004 +.020+002SB    170
3487          CD-45 4517  75063220422                       084238.2-454032084601.7-460230265.38-01.84 3.91   0.00 -0.05 +0.13   A1III              -0.001 0.000 +.003+024       25
3488          CD-40 4602  75081220432                       084243.9-404533084623.8-410732261.58  1.29 6.21  -0.05 -0.19         B9Ve              t-0.018-0.001      +023                          *
3489          CP-58 1202  75086236241      W     A          084243.3-582133084505.3-584330275.33-09.78 6.21  -0.09 -0.47         B7III              -0.031-0.020D+.006+028           0.1   4.1AB   4*
3490          CD-34 5243  75112199607               4251    084250.9-341522084649.2-343722256.54  5.41 6.37  -0.13 -0.58         B4V                -0.021+0.007      +015V                         *
3491          CP-67  990  75116250315                       084257.5-675053084354.3-681242283.07-15.53 6.32  +1.50 +1.78         K3III:             +0.001+0.027      -017
3492 13Rho HyaBD+06 2040  75137117146        7006           084308.1+061226084826.0+055016221.52 28.62 4.36  -0.04 -0.04 -0.07   A0Vn               -0.017-0.034 +.014+033SB    126  8.7  12.4      *
3493          BD-06 2727  75140136287                       084308.6-061123084804.9-063331233.32 22.26 6.09  +1.28 +1.42         K0                 +0.012-0.021      -005
3494          CD-45 4526  75149220442                       084306.3-453244084630.6-455446265.33-01.69 5.46  +0.27 -0.53         B3Ia               +0.006-0.012      +026       73                 *
3495          CP-65 1013  751712503172691                   084306.6-652748084430.0-654932281.10-14.09 6.05  +0.20 +0.08         A4V                -0.062+0.101      +009
3496          CD-45 4541  75276220464                       084355.2-454714084718.9-460920265.61-01.73 5.75  +0.56 +0.38         F2Iab              +0.003-0.008      +032                          *
3497          CD-41 4507  75289220481                       084401.9-412142084740.5-414414262.21  1.09 6.36  +0.58 +0.10         G0Ia-0:             0.000-0.242 +.029+014V
3498          CP-56 1865  753112362682695          V344 Car 084407.3-562407084642.5-564611273.90-08.41 4.49  -0.17 -0.73 -0.18   B3Vne             v-0.010+0.008      +027      332                 *
3499          BD+33 1765  75332 61074                       084419.5+333932085032.2+331707190.53 38.30 6.25  +0.49 +0.01         F7Vn               -0.067-0.081      +005       11
3500 14    HyaBD-02 2699  753331363081230          KX Hya   084420.2-030419084921.7-032635230.64 24.19 5.31  -0.09 -0.35         B9pHgMn            -0.020-0.023      +033V?     35                 *
3501          CD-41 4516  75387220495                       084432.4-420538084808.8-422750262.83  0.70 6.43  -0.20 -0.77         B2IV-V             -0.003-0.009      +030                          *
3502   Eta ChaCP-78  372  75416256543 331                   084443.7-783601084119.5-785748292.40-21.65 5.47  -0.10 -0.35         B8V                -0.031+0.026      +014V     301
3503          CP-52 1675  75466236284                       084504.1-522851084800.2-525102270.91-05.84 6.30  -0.10 -0.32         B8V                -0.026+0.007      +012      204                 *
3504          BD+19 2110  75469 98162                       084503.6+191219085045.1+184956207.88 34.48 6.16R -0.01  0.00         A2V s              -0.015-0.020      +019
3505  5    UMaBD+62 1027  75486 146912701                   084508.3+622011085322.6+615744153.83 37.93 5.73  +0.28 +0.11         F2III              -0.010+0.020      -031V?    101
3506          BD+59 1198  75487 27026                       084512.2+592551085305.9+590322157.46 38.52 6.25R                     F5IV-V             +0.014+0.003      +009       30
3507          BD-20 2693  75495176535                       084515.4-204030084944.9-210255246.06 14.25 6.47  +0.23               A7Vn               +0.006-0.064      -017
3508 35    LynBD+44 1794  75506 425762700I                  084514.2+440556085156.8+434336177.19 39.64 5.15  +0.98 +0.68 +0.50   K0III              -0.015+0.048 +.018+015     < 17
3509          BD+45 1649  75523 42580                       084523.8+454115085211.6+451845175.13 39.71 5.99  +1.26 +1.33         K0III              -0.026-0.037      +012
3510 54    CncBD+15 1917  75528 981682699  D                084527.3+154318085101.5+152102211.81 33.26 6.38  +0.64               G1V                -0.112+0.075      +045    =< 10  1.4   0.1      *
3511          BD+42 1935  75556 42581                       084533.2+422243085210.0+420009179.43 39.62 5.99  +1.25               K0III              -0.039-0.071      +057
3512          CD-32 5770  756051996782696           4268    084547.7-322425084951.5-324650255.49  7.05 5.21  +0.87               G5III              +0.002-0.049 +.028-008V                         *
3513          CD-28 6600  75629176546                       084550.7-290526085002.2-292747252.90  9.15 5.87  +0.95               G8III              -0.014+0.007      -010
3514          CD-39 4838  75630220531                       084555.7-395652084939.2-401914261.34  2.27 5.48  +0.06 +0.13         A2/3IV             -0.008-0.014 +.036+017
3515 M 67                                                                                                                                                                                           *
3516          CD-28 6610  75649176554                       084608.1-281440085021.6-283705252.27  9.73 6.17  -0.08 -0.19         B9V                -0.021-0.022      +025
3517          CD-38 4925  756541996822697          HZ Vel   084605.8-384612084952.4-390830260.46  3.05 6.39  +0.23               A5III              -0.058+0.038      +009                          *
3518   Gam PyxCD-27 5986  75691176559 332I                  084617.2-272020085031.9-274236251.57 10.33 4.01  +1.27 +1.40 +0.67   K3-III             -0.130+0.087 +.033+025
3519 51Sig1CncBD+33 1770  75698 61102        7057           084624.0+325056085234.6+322827191.64 38.57 5.66  +0.22 +0.07 +0.08   A8Vma s            -0.003+0.016      -014SB     65  4.4  79.0AC   4*
3520          CD-44 4861  757102205402698                   084620.1-445608084947.7-451829265.22-00.86 4.93  +0.05 +0.15         A2III              -0.010+0.010 +.012+005SB     77
3521 53    CncBD+28 1659  75716 80476    I W       BO Cnc   084627.8+283805085228.6+281533196.87 37.66 6.23  +1.62               M3III              -0.017-0.004 +.001+011V          3.5  43.1      *
3522 55Rho1CncBD+28 1660  75732 80478                       084638.5+284246085235.8+281951196.79 37.71 5.95  +0.87 +0.63 +0.40   G8V                -0.484-0.234 +.074+027V          7.0  85.0      *
3523 15    HyaBD-06 2743  75737136345        7050   4282    084639.5-064809085134.5-071038234.39 22.64 5.54  +0.15 +0.14         A4m                -0.044+0.001D+.011+037SB2    25  1.8   0.9AB   4*
3524          CP-78  378  75747256549              RS Cha   084637.7-784214084312.4-790411292.55-21.63 6.05  +0.23 +0.08         A5V+A7V            -0.029+0.040      +026SBO                       *
3525          CD-41 4560  75759220552                       084642.6-414259085021.0-420524262.80  1.25 6.00  -0.10 -0.95         B1-2III            -0.011-0.007      +023SB2O   39                 *
3526          BD+05 2074  75811117214        7061           084707.3+054259085224.2+052024222.55 29.26 6.33  +0.12 +0.14         A5V                -0.004-0.015      -010V      43  3.2   0.8
3527          CD-46 4661  75821220561      W                084709.8-460918085033.5-463145266.25-01.54 5.10  -0.21 -0.99 -0.16   B0III              -0.001-0.002      +007SB     74  4.1   3.2      *
3528          BD+36 1883  75896 611172704                   084737.9+355459085355.7+353218187.81 39.34 6.14  +0.04 +0.11         A4III              -0.024-0.017      +025SB?    50
3529          BD-12 2716  75916154704        7062           084746.0-125124085230.7-131401239.88 19.41 6.13  +1.14 +1.14         K0                 +0.025-0.019      +014V?         5.3  33.6
3530          CD-42 4723  75926220581                       084750.5-420746085127.9-423016263.25  1.15 6.55  +0.04 +0.08         A1Vn               -0.022+0.027      +024V
3531  6    UMaBD+65  673  75958 14703                       084803.2+645912085637.5+643614150.48 37.60 5.58  +0.86 +0.57         G7IIIFe-0.5        -0.026-0.081      +003SB
3532 57    CncBD+31 1907  75959 61125    I   7071           084808.6+305730085414.7+303446194.11 38.55 5.39  +1.05               G7III              +0.038-0.019D+.005-060           0.2   1.5AB   3*
3533          CD-32 5814  76001199728                       084821.1-320756085226.1-323033255.62  7.64 6.50  +1.46               K2/3III            -0.008-0.006      +001
3534          CD-36 5125  76072199731      W        4288    084843.7-361004085238.6-363244258.79  5.12 6.42  +0.56               G8III+A3-5V        -0.015-0.017      +018           0.0   0.1
3535          CD-38 4980  76110199737                       084858.8-382048085248.0-384327260.50  3.75 5.82  +1.50 +1.89         M0III              -0.001+0.013      +021
3536          CP-57 1759  76113236339                       084903.5-571526085136.6-573801275.00-08.43 5.59  -0.11 -0.41         B8III              -0.011+0.014      +008       19
3537          CP-66  927  76143250347      W                084913.8-662511085034.8-664735282.28-14.18 5.35  +0.42 +0.07         F5IV               +0.089+0.097 +.031+042      106  6.6  20.
3538          BD-04 2490  76151136389                       084922.3-050321085417.9-052604233.21 24.17 6.00  +0.67 +0.22 +0.21E  G2V                -0.416+0.031 +.085+028V   =<  6
3539          CD-47 4460  76161220609                       084921.0-475852085238.6-482133267.89-02.43 5.91  -0.15 -0.59 -0.16   B3Vn               -0.009-0.006      +003V?                        *
3540 58Rho2CncBD+28 1666  76219 805112705I                  084940.3+281833085539.7+275539197.50 38.26 5.22  +1.00 +0.75 +0.49   G8II-III           -0.010-0.030 -.005+017V?   < 19:                *
3541          BD+17 1973  76221 98230    I         X Cnc    084944.9+173643085522.9+171353210.18 34.94 6.64  +3.36       +1.37   C6II               +0.001+0.009 +.007-001                          *
3542          CD-51 3303  76230236362      W                084938.9-514503085240.8-520745270.80-04.83 6.39   0.00 -0.13         A0V                -0.019+0.005      +018           1.6   2.8      *
3543          CP-79  352  76236256552                       084936.0-790804084555.2-793016293.02-21.76 5.79  +1.60 +1.97         K5III              -0.029+0.085      +007
3544          CP-72  747  76270256556                       084953.0-721032084950.3-723303287.05-17.65 6.11  +0.20 +0.17         Am                 -0.001+0.025      -003
3545          BD+46 1459  76291 42612                       085003.8+460055085650.0+453755174.71 40.52 5.74  +1.09 +1.06 +0.56   K1IV               -0.126-0.034      +059V?
3546          BD+40 2125  76292 426112706                   085001.1+403504085630.5+401206181.84 40.33 5.89  +0.35 +0.06         F3III              -0.083-0.033      +025       53                 *
3547 16Zet HyaBD+06 2060  76294117264 334I                  085006.4+061934085523.6+055644222.34 30.20 3.11  +1.00 +0.80 +0.49   G9II-III           -0.099+0.014 +.035+023     < 17
3548          CD-39 4924  76304220628                       085006.2-400404085350.7-402651261.95  2.81 6.47  +0.96               K0II-III+A5        -0.006+0.007      -001V
3549          CP-56 1918  76346236368                       085024.3-561616085303.8-563858274.35-07.66 6.03  -0.02 -0.08         A0V                -0.017+0.033      +024
3550 60    CncBD+12 1941  76351 98235    I D        4308    085027.9+120030085555.6+113734216.45 32.85 5.41  +1.46              gK5                 -0.010-0.010      +024V?                        *
3551          CD-47 4480  76360220636      W                085029.2-470824085350.7-473115267.37-01.74 5.33  +0.26               A9IV-V             +0.006-0.028 +.019-001SB2    99  0.0   0.1      *
3552 17    HyaBD-07 2661  76369136408        7093B          085035.5-073515085529.5-075813235.69 23.01 6.91H                     A7m                -0.001-0.038D+.004-020       50  0.3   4.1      *
3553 17    HyaBD-07 2661  76370136409        7093A          085035.5-073520085529.6-075816235.69 23.01 6.67H +0.22 +0.13         A2m                +0.002-0.024D+.004-010    =< 25  0.3   4.1      *
3554          BD-17 2691  763761547451231I W                085036.7-175136085512.4-181429244.54 16.97 5.75  +1.32               K2-3III            +0.020+0.004      +037V?         1.2  66.7
3555 59Sig2CncBD+33 1785  76398 61146                       085046.4+331743085656.6+325437191.30 39.55 5.45  +0.12 +0.12         A7IV               -0.048-0.068 +.011+005      145                 *
3556   Del PyxCD-27 6072  76483176697        7095           085114.1-271749085531.5-274055252.25 11.21 4.89  +0.11 +0.14 +0.05   A3IV               +0.079-0.102 +.037+005       73  9.1  23.8      *
3557          BD+04 2081  76494117287                       085122.2+043712085636.9+041412224.24 29.65 6.14  +1.00 +0.75         G8II-III           -0.020+0.005      -012
3558          BD+17 1979  76508 98245                       085130.9+173143085708.2+170838210.46 35.30 6.17  +1.00               K1III              -0.044-0.025      +019
3559          CD-23 7902  765121767112707                   085131.3-232612085555.9-234906249.22 13.69 6.39  +0.16 +0.12         A7V:n              -0.040+0.038      +021
3560          CP-59 1174  76538250369                       085131.4-595824085348.7-602115277.32-09.92 5.78  -0.08 -0.55         B5III              -0.007+0.002      +002
3561 62Omi1CncBD+15 1945  76543 98247                       085140.3+154223085714.9+151922212.53 34.64 5.20  +0.15               A5III              +0.056+0.017      -005SB     89
3562          CD-44 4951  76566220664      W       IY Vel   085148.5-443933085519.3-450230265.64  0.05 6.26  -0.16 -0.63         B3IV               +0.002+0.008      +022        5  6.5  35.0      *
3563 61    CncBD+30 1795  76572 61157        7107           085154.2+303705085758.7+301401194.76 39.26 6.29  +0.43 -0.02         F6V                +0.055+0.024      +008    =< 10  0.0   0.1      *
3564          BD-16 2639  76579154773    I          4321    085155.5-161928085634.1-164234243.46 18.15 5.96  +1.54               K0                 -0.010-0.047      +038                          *
3565 63Omi2CncBD+16 1864  76582 98250                       085200.1+155755085735.2+153453212.28 34.81 5.67  +0.20               F0IV               +0.058+0.029      -004V?    113
3566          BD+36 1889  76595 61161                       085210.0+361118085827.5+354809187.63 40.29 6.51R -0.01 +0.01         A2V                +0.008-0.011      -015
3567          BD+09 2093  766291173012710                   085218.4+094623085742.0+092316219.06 32.28 6.19  +0.98 +0.76         G8III              -0.002-0.007      -014
3568          CP-57 1790  76640236402                       085222.5-575126085454.0-581424275.75-08.47 6.38  -0.11 -0.49         B5V                -0.022+0.002      +013      119
3569  9Iot UMaBD+48 1707  76644 42630 335I   7114   4329    085221.8+482604085912.4+480230171.51 40.84 3.14  +0.19 +0.07 +0.07   A7IV               -0.444-0.226 +.075+009SB1O  151  7.2   3.9AB   4*
3570          CP-54 1925  76653236405                       085221.5-543449085511.9-545756273.23-06.35 5.71  +0.48  0.00         F6V                +0.037-0.082      -002
3571          CP-60 1243  76728250374 336  W                085246.8-601545085502.8-603841277.65-09.98 3.84  -0.10 -0.45 -0.11   B8-9II             -0.021+0.038      +025V      36  8.8  21.1      *
3572 65Alp CncBD+12 1948  76756 98267 337    7115   4327    085301.1+121442085829.2+115128216.51 33.52 4.25  +0.14 +0.15 +0.04   A5m                +0.034-0.031 +.024-014SB     68  0.0   0.1     3*
3573          BD+02 2112  767571173112712  W                085258.1+015540085808.2+013230227.14 28.65 6.59  +0.06 +0.09         A2V                -0.040-0.005      +026           5.5   2.5
3574          CP-52 1788  76805236417      W                085318.1-522020085619.4-524325271.62-04.79 4.69  -0.12 -0.47 -0.14   B5V                 0.000+0.007D+.020+022SB1O   48  3.0   2.8      *
3575 64Sig3CncBD+32 1821  76813 611771232I W                085324.2+324826085932.6+322507192.05 40.01 5.20  +0.93               G8III              -0.045-0.035      +023           3.8  89.3      *
3576  8Rho UMaBD+68  551  76827 14742 338I          4344    085331.9+680110090232.7+673747146.63 37.23 4.76  +1.53 +1.88 +1.26   M3III-IIIbCa1      -0.021+0.019 +.014+005V
3577          BD+18 2093  76830 98276    I          4332    085331.9+183127085910.8+180805209.54 36.11 6.38  +1.55 +1.69         M4III             v-0.044-0.068 -.002+021V?
3578          BD-15 2656  769321548042713                   085402.3-154504085843.9-160758243.30 18.91 5.86  +0.53 +0.08 +0.23E  F6V                +0.246+0.212 +.028+121V       0
3579          BD+42 1956  76943 42642 339I W                085409.0+421044090038.4+414658179.80 41.19 3.97  +0.44 +0.05 +0.22   F5V                -0.440-0.246 +.070+026SBO    26  2.0   0.3AB   5*
3580          BD+38 1986  76944 61191    I                  085409.4+375936090030.8+373616185.33 40.90 6.44R                     K5                 -0.007 0.000      -017
3581          BD+84  196  76990  14611640                   085432.3+843459091521.2+841052128.39 30.44 6.33  +0.30 +0.10         F2III              +0.027+0.015      -006SB2
3582          CP-58 1301  770022364361233  W     A V376 Car 085431.6-585035085658.4-591346276.70-08.90 4.92  -0.19 -0.77 -0.24   B2IV-V             -0.009+0.007      +027        0  1.9  40.2      *
3583          CD-48 4282  77020220703                       085436.5-481109085755.8-483424268.62-01.90 5.87  +1.07 +0.88         G8-K0II            +0.004-0.004      +018
3584          BD-18 2536  77084154815                       085505.8-184858085939.9-191229246.01 17.22 6.18  +0.46               F6V                -0.051-0.098      +020V
3585          CD-28 6793  77087176789                       085501.0-282504085915.7-284822253.67 11.13 6.25  +1.00               G8III              -0.061+0.015      +025
3586          BD+40 2138  77093 61203                       085514.1+400624090140.6+394248182.56 41.29 6.36  +0.30 +0.04         A9Vn               -0.053-0.079      -008      150
3587 66    CncBD+32 1829  77104 61202        7137           085516.2+323835090124.1+321509192.36 40.36 5.82R  0.00 +0.09         A2V                -0.004+0.003      -013V          2.3   4.6AB   3*
3588          CD-46 4810  77140220717      W       FZ Vel   085528.5-465050085852.3-471405267.71-00.90 5.18  +0.25 +0.19         Am                 -0.086+0.054 +.013+018       57  6.0  24.2      *
3589 67    CncBD+28 1674  77190 80585      W                085551.2+281748090148.9+275410197.94 39.58 6.07                      A8Vn               -0.056-0.076      +012SB    166  2.8 103.5      *
3590          BD+06 2087  77250117351                       085614.5+060158090131.4+053827223.50 31.40 6.07  +1.11 +1.02         K1II-III+F3IV      -0.028-0.007      +033           1.6 269.0      *
3591          CD-40 4810  772582207301234           4346    085621.4-405151090005.4-411514263.33  3.18 4.45  +0.65 +0.38 +0.41   G8-K1III+A         -0.039+0.045 +.030-007SB1O    0                 *
3592          BD+54 1272  77309 27105 340                   085640.9+544041090400.4+541702163.20 40.86 5.75  +0.02 +0.05         A2V                -0.003+0.004      -004V     110
3593          CD-42 4875  77320220738              IU Vel   085643.6-424658090022.2-431024264.82  1.96 6.07  -0.16 -0.82 -0.12   B2.5Vn            t-0.015+0.019      +027V     230                 *
3594 12Kap UMaBD+47 1633  77327 42661 341    7158   4357    085648.0+473308090337.5+470924172.63 41.63 3.60   0.00 +0.01 +0.01   A1Vn               -0.033-0.054 +.016+004V?    219  0.2   0.3      *
3595 69Nu  CncBD+25 2029  77350 805952714           4356    085653.5+245047090244.3+242710202.31 38.90 5.45  -0.04 -0.10 -0.04   A0pSi             v-0.005-0.006 +.003-015SBO    23                 *
3596          BD+00 2449  773531365111235                   085651.4-000531090158.0-002858229.68 28.43 5.67  +1.15 +1.04         K0III              -0.048+0.069      +073V
3597          CD-26 6647  77361176833                       085651.1-261613090111.4-263950252.26 12.83 6.20  +1.13               K1IIICNII          +0.026-0.056      -024V?
3598          CP-58 1327  77370236475      W                085656.8-584203085924.1-590501276.79-08.57 5.16  +0.42               F3V                -0.174+0.280 +.054+011       66  6.8  26.2      *
3599          BD+07 2066  77445117369                       085725.1+074132090244.8+071753221.94 32.44 5.85  +1.11 +0.98        gK3                 -0.016-0.005      +027
3600          CD-41 4720  77475220760              IZ Vel   085737.9-412818090120.8-415152263.95  2.96 5.55  -0.14 -0.58         B5V                -0.023+0.005      +021SB      0                 *
3601 70    CncBD+28 1683  77557 80609                       085812.3+281739090409.9+275354198.10 40.08 6.38R  0.00 +0.05         A1V                +0.001-0.002      -021V     180
3602          CD-38 5154  77580199902                       085816.4-390032090206.4-392409262.19  4.69 6.27  +1.00               K1III-IV           -0.037+0.023      +018
3603          BD+49 1801  77601 42682                       085830.1+485540090524.1+483149170.76 41.83 5.95  +0.45 +0.17         F6II-III           -0.012-0.016      -006SB    139                 *
3604          CP-60 1283  77615250417                       085827.6-603415090045.7-605750278.35-09.65 5.79  +1.21               G8II               -0.006 0.000      +018
3605          CD-51 3430  77653236518      W                085838.2-514742090144.6-521118271.75-03.81 5.23  -0.12 -0.45         B9Si               -0.007+0.018D+.008+032        0  1.2   0.9
3606          BD+32 1837  77660 61242                       085848.3+324632090455.2+322237192.36 41.12 6.46  +0.24 +0.10         A8V                -0.043-0.058      +016      131
3607          CD-25 6829  77665176881                       085846.0-250631090308.8-253016251.63 13.90 6.74  -0.03               B8                 -0.014-0.006      +027
3608          BD+59 1217  77692 271212716                   085857.2+594436090643.1+592040156.59 40.15 6.45  +0.04 +0.13         A2V                -0.014-0.010      +002SB     55
3609 11Sig1UMaBD+67  573  77800 14769    I                  085937.0+671630090823.6+665224147.27 38.03 5.14  +1.51 +1.83 +0.83   K5III              -0.018-0.038 +.011+015SB   < 19:
3610          CP-68  879  77887250421                       090000.0-681721090108.5-684102284.45-14.56 5.88  +1.63 +1.97         M1III              +0.012+0.005      +013
3611          CP-53 2072  77907236542                       090004.3-530914090305.2-533259272.91-04.56 6.40  -0.09 -0.35         B6V                -0.024+0.030      -007
3612          BD+39 2200  77912 612541237I                  090010.2+385107090631.8+382708184.33 42.15 4.56  +1.04 +0.82 +0.49   G7Ib-II            -0.028-0.014 +.021+017V?   < 19:                *
3613 18Ome HyaBD+05 2116  779961174202717I                  090042.5+052931090558.4+050532224.69 32.11 4.97  +1.22 +1.21 +0.57   K2II-III           -0.018-0.010 +.011+025V    < 19:
3614          CD-46 4883  78004220803 342                   090042.3-464158090409.3-470552268.20-00.14 3.75  +1.20 +1.22 +0.60   K2III              -0.044-0.013 +.020+024
3615   Alp VolCP-65 1065  78045250422 343                   090052.1-655948090226.8-662346282.71-13.00 4.00  +0.14 +0.13 +0.06   A2-3IVm            -0.002-0.096 +.051+005SB2    45                 *
3616 13Sig2UMaBD+67  577  78154 14788        7203   4392    090135.8+673226091023.2+670805146.87 38.11 4.80  +0.49 +0.02 +0.28   F6IV               -0.020-0.074 +.056-002        0  3.5   2.8AB   3*
3617          BD+23 2048  78175 80643        7187A          090141.1+232256090726.9+225852204.53 39.51 6.40                      F4V                -0.168+0.009 +.004+029           0.4   7.6AB   3*
3618          BD+02 2145  781961174322718I                  090149.9+015152090659.9+012746228.52 30.51 6.17  +1.65 +1.96 +0.97E  M1III              -0.011-0.015      +003
3619 15    UMaBD+52 1365  78209 271362721                   090149.1+520030090852.3+513617166.57 42.02 4.48  +0.27 +0.12 +0.11   F0IVm vs           -0.137-0.030      -000       37                 *
3620          BD+33 1810  78234 61276                       090158.0+325636090804.1+323226192.28 41.80 6.50  +0.36  0.00         F2V                -0.084-0.017      +041
3621 72Tau CncBD+30 1817  78235 806502719                   090159.9+300323090800.1+293915196.07 41.28 5.43  +0.89 +0.57 +0.45   G8III              -0.032+0.002      -013V?   < 19:
3622          CP-57 1859  78293236578      W                090207.9-572712090448.0-575109276.31-07.23 6.44  +0.25 +0.19         A8III              -0.026+0.025      +015           3.6   3.3      *
3623 76Kap CncBD+11 1984  78316 983781238  D       Kap Cnc  090219.8+110415090744.8+104005218.99 35.06 5.24  -0.11 -0.43         B8IIIpMn           -0.020-0.010      +024SB1O    9  0.2   0.3     3*
3624 14Tau UMaBD+64  723  78362 147962727    7211           090240.6+635514091055.1+633049151.21 39.42 4.67  +0.35 +0.15 +0.13   F3-4IIIm vs        +0.098-0.056      -009SBO    18  6.2  57.2AB   3*
3625          BD+34 1949  78366 612882723                   090242.9+341720090851.1+335256190.53 42.16 5.93  +0.60 +0.04         F9V                -0.187-0.114      +027V       5                 *
3626 75    CncBD+27 1715  78418 806592724  W                090254.4+270235090847.3+263745200.03 40.78 5.98  +0.66 +0.20         G5IV-V             -0.136-0.368 +.040+013SB2O       3.1 112.0AB   3*
3627 77Xi  CncBD+22 2061  78515 806661239  W                090336.6+222700090921.5+220244205.86 39.65 5.14  +0.97 +0.80 +0.48   G9IIIFe-1CH-0.5    +0.001+0.005      -007SB1O < 17  0.5   0.1      *
3628   Kap PyxCD-25 6895  78541177002    I   7202   4389    090339.4-252718090802.9-255130252.64 14.52 4.58  +1.59 +1.89 +0.90   K4III              +0.041+0.008 +.013-045           5.2   2.1
3629          CP-55 1957  78548236611                       090341.7-552401090634.0-554812274.93-05.68 6.11  -0.15 -0.67         B2IV-V             -0.012-0.023      +029      202
3630 19    HyaBD-08 2588  785561366042725  W                090348.5-081106090842.2-083522238.29 25.34 5.60  -0.06 -0.19         B9.5III            -0.019-0.002      +023SB    172  5.0   1.0
3631          CD-50 3849  785992366182722                   090401.2-504831090714.7-511243271.59-02.52 6.73  +1.62 +1.87         K4III              -0.004 0.000      +007V
3632          CP-63 1093  78632250441                       090411.8-640553090607.6-642959281.49-11.49 6.37  +1.35 +1.58         K3III              +0.020+0.029      +001
3633          BD+72  444  78633  67842729                   090419.8+720358091403.2+713921141.50 36.59 6.55  +0.96 +0.75         G8III-IV           -0.028-0.038      +006
3634   Lam VelCD-42 4990  78647220878 345I'W       Lam Vel  090419.0-430144090759.8-432557265.94  2.82 2.21  +1.66 +1.81 +0.94   K4.5Ib-II         e-0.019+0.013 +.022+018          12.6  18.2      *
3635          BD+12 1979  78661 98400                       090420.3+115818090946.4+113352218.25 35.90 6.48  +0.34 -0.10         F2Vp               -0.045-0.062      -016       53
3636          BD-11 2565  786681549531240                   090423.8-115709090911.5-122128241.70 23.21 5.77  +0.94               G6III              +0.019+0.003      -009SB
3637          CD-26 6766  786761770222726                   090422.4-262147090843.5-264604253.46 14.05 6.15  +0.17               A4IV               -0.045+0.009      +008V?
3638          BD-17 2765  78702154950                       090427.3-175526090904.3-181943246.74 19.51 5.73   0.00 +0.01         A2Vn               -0.052+0.009      +017V?
3639          BD+31 1946  78712 61306    I         RS Cnc   090436.2+312214091038.7+305747194.50 42.08 5.95  +1.67 +1.03 +2.33   M6IIIase           -0.021-0.034      +014V                         *
3640 79    CncBD+22 2063  78715 80674      W                090436.2+222409091020.9+215947206.01 39.86 6.01  +0.90               G5III              +0.005+0.006      -007V          0.0   0.1      *
3641 20    HyaBD-08 2593  78732136622                       090442.1-082254090935.5-084716238.61 25.40 5.46  +1.01 +0.78         G8II               -0.019-0.010      +026V?
3642          CP-70  861  78764256583              V345 Car 090449.5-700810090538.3-703220286.19-15.43 4.71  -0.15 -0.81 -0.14   B2IVe              -0.008-0.005      +019V     140                 *
3643          CP-72  779  787912565822720                   090453.0-721201090508.8-723610287.83-16.77 4.48  +0.61 +0.22 +0.31   F9II               -0.013-0.005 +.024+022       53
3644   Eps PyxCD-29 7194  789222000471241  W                090542.1-295726090956.4-302155256.42 11.88 5.59  +0.19 +0.16 +0.07   A4IV-V             +0.002-0.048      -008SB    129  4.3  17.6AxBC 4*
3645          BD+73  452  78935  6792                       090550.3+732136091552.6+725646139.99 36.14 5.96  +0.18 +0.13         F0III              -0.085-0.063 +.023+000V?     90
3646          BD-22 2512  789551770662728                   090554.5-224610091023.0-231036250.88 16.66 6.53   0.00               A1-2V:             -0.029+0.010      +014V
3647          CD-48 4471  79025220910                       090623.2-490058090945.1-492529270.54-01.02 6.48  +0.17               A9Vn               +0.014-0.041      -001                          *
3648 16    UMaBD+62 1058  79028 148192730  W                090626.4+615007091420.6+612524153.62 40.47 5.13  +0.58 +0.10         F9V                -0.006-0.030 +.047-014SBO     0                 *
3649          BD+06 2120  79066117487                       090639.9+055242091155.6+052806225.15 33.59 6.35  +0.32 +0.01         A9IVDel Del        -0.101-0.025      +003
3650 81Pi 1CncBD+15 2003  79096 98427      W                090649.2+152357091217.6+145946214.66 37.86 6.51  +0.73 +0.25 +0.38   G9V                -0.522+0.248 +.064+045V          0.0   0.1AP   4*
3651          BD+04 2139  79108117492                       090659.4+041637091212.9+035202226.86 32.86 6.14  -0.01 +0.01         A0Vp               -0.044-0.001      +020V     150                 *
3652 36    LynBD+43 1893  79158 42759 346                   090715.9+433748091348.2+431304177.86 43.61 5.32  -0.14 -0.48         B8IIIpMn           -0.028-0.032      +021V      29                 *
3653          BD-19 2644  791811549891242                   090723.9-192020091158.7-194452248.37 19.15 5.73  +0.98               G9III              -0.057+0.046      -001V?
3654          CD-44 5206  79186220928              GX Vel   090726.6-442731091104.4-445205267.36  2.25 5.00  +0.23 -0.57 +0.22   B5Ia               -0.001+0.002      +034V?     61                 *
3655 21    HyaBD-06 2845  79193136662              KW Hya   090729.7-064159091226.0-070635237.55 26.95 6.11  +0.22 +0.12 +0.12   A3III:m+A0V:       -0.016+0.019      -004SBO  < 12                 *
3656          CD-38 5358  79241200084                       090747.1-385057091141.0-391532263.33  6.16 6.00  -0.11               B6V                -0.010+0.006      +019V
3657          BD+21 1991  79248 80702                       090754.6+214143091337.3+211700207.20 40.36 6.48  +0.02 +0.06         A2V                -0.016-0.008      +009
3658          CD-46 4987  79275220937                       090800.9-461026091133.2-463502268.68  1.14 5.79  -0.21 -0.84 -0.12C  B2III/IV           -0.018+0.010      +007
3659          CP-58 1419  79351236693              V357 Car 090820.0-583326091058.0-585801277.69-07.37 3.44  -0.19 -0.70 -0.19   B2IV-V            e-0.026+0.008      +023SB2O   30                 *
3660 17    UMaBD+57 1211  79354 27185    I          4427    090825.4+570922091549.9+564429159.52 41.98 5.27  +1.56 +1.87 +0.88   K5III              -0.016-0.031      -030V?   < 19:
3661          CD-43 5041  79416220952      W       KL Vel   090848.5-431207091230.6-433649266.63  3.30 5.57  -0.11 -0.47 -0.06C  B8Vp:Si:           -0.012+0.002D+.005+015V?    384  0.7   2.8ABxC 3*
3662 18    UMaBD+54 1285  79439 271912734          DD UMa   090859.6+542606091611.3+540119163.08 42.66 4.83  +0.19 +0.08 +0.09   A5V                +0.049+0.061 +.042-019SB    157                 *
3663          CP-61 1201  79447250471                       090900.4-615424091116.6-621902280.22-09.61 3.97  -0.18 -0.67 -0.18   B3III             v-0.043+0.009      +018       21                 *
3664          BD+35 1966  79452 61361      W                090906.0+350246091514.3+343801189.73 43.56 5.97  +0.86 +0.37 +0.47   G6III              -0.148+0.053      +056           3.5 150.6
3665 22The HyaBD+02 2167  79469117527 347    7253   4425    090909.7+024411091421.9+021851228.77 32.53 3.88  -0.06 -0.12 -0.05   B9.5V              +0.129-0.310 +.027-010SB     81  7.7  27.1AB   4*
3666          BD+74  393  79517  6816                       090937.0+742612091955.8+740059138.66 35.89 6.50R                     G8III              -0.037-0.067      +056
3667          CD-38 5376  79523200119               4423    090929.5-381211091325.9-383659263.09  6.85 6.31   0.00 +0.01         A0V                +0.008-0.012      +004
3668          CD-41 4904  79524220962                       090932.3-415144091318.6-421625265.75  4.32 6.29  +1.25               K2III              -0.017+0.054      +029
3669 82Pi 2CncBD+15 2009  79554 984562733I                  090942.6+152123091513.8+145629215.07 38.49 5.34  +1.32 +1.33         K1III              -0.044-0.012      +026                          *
3670          CD-46 5010  79621220965                       091003.7-465531091334.5-472019269.46  0.87 5.92  -0.05 -0.10 -0.01C  B9Ve              v-0.016+0.003      +019                          *
3671 NGC 2808                                                                                                                                                                                       *
3672          CD-43 5068  796942209722731           4428    091027.1-434355091408.2-440845267.21  3.14 5.85  -0.12 -0.50 -0.06C  B5/6IV/V           -0.019+0.014      +022V     260                 *
3673          CP-58 1432  79698236723                       091018.5-590004091255.6-592452278.19-07.49 5.54  +0.85               G6II                0.000 0.000      +016
3674          CD-42 5086  79735220978      W                091040.5-424847091424.5-431339266.58  3.81 5.25  -0.14 -0.56 -0.07C  B4V                -0.021+0.010 +.011+015V?    201  4.1   5.8      *
3675          BD-14 2793  79752155032               4431    091041.2-143632091524.9-150129245.00 22.77 6.35  +0.02  0.00         A0V                -0.026-0.014      +032V?    148
3676          BD+47 1658  79763 427902738                   091048.7+471403091731.2+464902172.82 44.02 5.97  +0.04 +0.05         A1V                +0.017+0.014      -012SBO    53                 *
3677          CD-37 5578  79807200152                       091057.8-371112091457.2-373609262.55  7.76 5.86  +0.83               G0V                +0.011-0.013      -002
3678   Zet OctCP-85  183  79837258515 918                   091114.3-851547085641.1-853947298.85-24.81 5.42  +0.31 +0.07         A8-9IV             -0.117+0.037      -004       81
3679          CP-55 2035  798462367492736           4429    091120.0-550919091418.0-553411275.50-04.72 5.27  +0.99               G8II-III           -0.035+0.027 +.015+009                          *
3680          CD-45 4982  79900220998      W                091137.2-450821091514.6-453320268.37  2.31 6.25  -0.08 -0.26 -0.02C  B8V                -0.003-0.010D+.006+033           0.9   0.8      *
3681 23    HyaBD-05 2762  799101367252739I W                091143.7-055609091641.7-062111237.54 28.25 5.24  +1.17 +1.22         K2III              +0.024+0.006 +.008-008SB1O < 19: 5.6   1.6      *
3682          CD-38 5408  799172001592737                   091140.2-380911091536.7-383412263.35  7.19 4.94  +1.11 +1.06 +0.57   K0III              -0.068-0.013 +.027+002
3683 24    HyaBD-08 2623  79931136728                       091147.4-081938091641.3-084441239.73 26.85 5.47  -0.09 -0.30         B9III              -0.020+0.004      +010V?    121
3684          CD-36 5505  79940200163      W                091144.9-365946091545.1-372448262.52  8.00 4.62  +0.45 +0.10 +0.28   F5III              +0.026-0.010 +.064+006SB    100  9.8  11.3
3685   Bet CarCP-69 1023  80007250495 348                   091206.2-691819091312.0-694302285.98-14.41 1.68   0.00 +0.03 +0.02   A2IV              v-0.162+0.108 +.021-005V?    133                 *
3686          BD+35 1971  80024 61387        7286           091216.0+354702091826.0+352151188.81 44.28 5.75R                     A8V                -0.041-0.024D+.005+022V     126  0.2   1.8
3687          BD-13 2808  80050155060                       091223.0-140920091707.5-143425244.90 23.37 5.84  +1.06 +0.90        gK0                 -0.048+0.012      -037
3688          CD-44 5305  80057221010      W        4436    091224.2-442853091604.2-445355268.00  2.87 6.04  +0.28 -0.13 +0.22C  A1Ib               +0.017+0.009      +026           8.0   6.8      *
3689          BD+12 2009  80064 98476        7285           091225.8+115512091751.4+113004219.39 37.66 6.41  +0.07 +0.11         A2IV               -0.012-0.003      -007SB     60  5.9  21.8      *
3690 38    LynBD+37 1965  80081 61391        7292           091237.3+371333091850.7+364809186.81 44.47 3.82  +0.06 +0.06 +0.03   A3V                -0.030-0.122 +.042+004SB    165  2.1   2.7AB   4*
3691          CP-57 1949  80094236772              Var?     091233.2-575817091517.5-582319277.65-06.57 6.02  -0.10 -0.43 -0.11   B7IV               -0.022-0.006      +016V                         *
3692          CD-43 5103  80108221014               4437    091241.2-435052091623.1-441557267.58  3.35 5.12  +1.67 +1.94 +0.79C  K3Ib               +0.002-0.001 +.017-003                          *
3693          CP-57 1951  80126236777                       091245.7-570938091535.0-573441277.09-05.98 6.32  +1.04               G8II               +0.003-0.003      +015
3694          CD-38 5430  80170200185                       091301.8-385855091657.2-392405264.14  6.80 5.33  +1.17               K5III-IV           -0.009-0.031 +.027-000V                         *
3695          CP-76  574  801942565942735                   091313.9-761448091212.3-763947291.43-18.99 6.14  +1.09 +0.99         K1III              +0.031-0.044      +001
3696          CP-57 1961  80230236787               4440    091322.6-570722091612.2-573229277.12-05.89 4.34  +1.63 +1.98 +1.03   M1III              -0.014-0.007 +.025-005
3697          BD+51 1495  80290 27215        7303           091347.4+514103092043.8+511558166.60 43.90 6.13  +0.42 -0.11         F3V                -0.035+0.150 +.036-008           4.0 142.1AC   3*
3698          BD+57 1214  80390 272192743I         CG UMa   091422.9+570723092143.3+564157159.29 42.77 5.47  +1.61               M4IIIa             -0.010-0.008      +021V                         *
3699   Iot CarCP-58 1465  80404236808 351           4444    091424.8-585120091705.4-591631278.46-07.01 2.25  +0.18 +0.16 +0.20   A8Ib               -0.020+0.008 +.017+013        0                 *
3700          CP-53 2281  80435236819                       091437.0-540428091742.3-542943275.07-03.63 6.33  +1.41               K3III              -0.018-0.004      -009
3701          BD+38 2025  80441 61411        7307           091443.5+383643092059.3+381118184.89 44.96 6.12  +0.38 -0.06         F2V+F4V            -0.044-0.013D+.018+001V?  =< 10: 0.3   1.2AB   3*
3702          BD-10 2804  80447155090                       091444.0-105335091933.7-111851242.49 25.86 6.62  +0.08               A2V s              -0.084+0.036      +029
3703          CD-50 4001  80456236824                       091446.1-503749091805.8-510304272.64-01.18 5.26  -0.07 -0.38         B7-8III            -0.023+0.008      +057SB
3704          BD-15 2763  80479155091    I   7302           091449.9-152440091933.2-155004246.36 23.03 5.78  +1.29 +1.39         K4III              +0.052-0.046      -030V?         5.0   4.1      *
3705 40Alp LynBD+35 1979  80493 61414 352I          4456    091457.8+344856092103.3+342333190.24 44.73 3.13  +1.55 +1.94 +0.90   K7IIIab           e-0.221+0.019 +.025+038                          *
3706 26    HyaBD-11 2609  804991550962741I W                091457.3-113310091946.4-115830243.10 25.49 4.79  +0.93 +0.64 +0.47   G8III              -0.030+0.015 +.023-002V    < 19: 7.5   3.4
3707          BD+33 1848  80546 61424                       091523.3+331937092127.2+325407192.33 44.64 6.16  +1.09 +1.04         K3III              +0.011-0.035      +028
3708          CD-51 3693  80558236837               4454    091524.1-510819091842.2-513338273.07-01.47 5.87  +0.54 -0.38         B6Iae              -0.014 0.000      +022       73                 *
3709 27    HyaBD-08 2643  80586136768    I   7311A          091536.0-090753092029.0-093321241.08 27.11 4.80  +0.93 +0.67 +0.44   G8III-IV+F5V       -0.014-0.029 +.022+025SB   < 19: 1.9 231.8AB   3*
3710          CD-33 5973  805902002412742                   091539.0-334048091947.9-340612260.66 10.88 6.39  -0.10               B8V                -0.010-0.007      +026
3711          BD+15 2027  80613 98517                       091544.1+154745092115.4+152216215.29 40.00 6.53  -0.01 +0.03         A1V                -0.036-0.015      +018      130
3712          CP-68  918  80671250526      W                091553.5-681603091717.2-684122285.44-13.45 5.39  +0.42 -0.03         F4V                -0.109-0.020 +.058+032        0  0.0   0.1AB   3*
3713          CP-66 1002  80710250529                       091608.3-663746091751.7-670303284.25-12.30 6.11  +1.26 +1.10         K2III              -0.003+0.014      +001V
3714          BD-14 2828  807191551142744                   091611.5-151126092055.5-153704246.41 23.42 6.33  +0.46 +0.02        dF6                 +0.065-0.107      -001SB
3715          CD-31 7162  80773200258      W                091630.2-312009092044.2-314538259.07 12.64 6.82   0.00 -0.04         B9.5V              -0.034-0.011      +018V?         0.8   2.9      *
3716          CD-37 5668  80774200257                       091628.5-370924092029.6-373453263.30  8.57 6.05  +1.39               K3/4III            -0.017-0.018      +007
3717          CP-54 2186  80781236851                       091629.1-544547091932.6-551112275.75-03.92 6.28  -0.10 -0.57 -0.06   B5V                -0.004 0.000      +013       53                 *
3718   The PyxCD-25 7114  808741773221243I                  091703.9-253223092129.6-255756254.81 16.72 4.72  +1.63 +2.02 +1.08   M1III              -0.011-0.008 +.010+020
3719          BD+75  377  80930  68582749                   091721.3+753143092751.6+750554137.18 35.80 6.29  +0.07 +0.11         A5V s              -0.014+0.015      +001       55
3720          CP-74  579  80951256599      W                091736.1-742821091725.4-745340290.24-17.59 5.29  +0.02 +0.04         A1V                -0.015+0.030      +011        0  0.5   0.2AB   4*
3721          CP-74  580  80950256600                       091735.3-741846091727.4-744405290.11-17.49 5.86  -0.02 -0.04         A0V                -0.041+0.033
3722          BD+64  733  80953 14875    I                  091741.2+642216092544.2+635627149.91 40.80 6.28  +1.46 +1.74         K2III              -0.008-0.041      +007
3723          BD+25 2088  80956 80797                       091744.6+253637092331.8+251059203.02 43.64 6.41  +0.82 +0.52:        G5IV               -0.121+0.005      -001V
3724          BD-09 2816  81009136799        7334  KU Hya   091758.2-092440092250.9-095020241.74 27.40 6.53  +0.22 +0.11         A5pSrCrEu          -0.027-0.021      +023     < 10  0.1   0.1AB   3*
3725          BD+52 1389  81025 27246        7348           091759.6+520010092455.7+513426166.00 44.47 6.31  +0.75               G2III              +0.041-0.005      -016SB         2.2   1.4      *
3726          CD-41 5023  81034221103                       091800.4-414601092150.9-421142266.80  5.51 5.58  +1.63 +1.94         M1III              +0.053-0.060      +049
3727          BD+37 1978  81039 614562748                   091811.8+370059092422.5+363513187.21 45.57 6.67  +0.21 +0.10         F0V                -0.068-0.024      +015
3728          CP-61 1242  811012505442745                   091832.7-615842092056.8-622417281.06-08.85 4.81  +0.94 +0.62 +0.54   G6III              -0.013+0.002 +.017+051
3729          CD-39 5446  81134200299                       091840.6-392058092236.7-394629265.17  7.32 6.54  +1.13               K1III              -0.063+0.074      +023
3730          CD-45 5099  811362211092746           4473    091844.8-453714092224.0-460251269.60  2.86 5.75  +0.92 +0.65 +0.43C  G6-8III            +0.004+0.027      -007                          *
3731  1Kap LeoBD+26 1939  81146 808071244I   7351   4478    091849.9+263646092439.3+261056201.76 44.13 4.46  +1.23 +1.31 +0.63   K2III              -0.033-0.048 -.003+028     < 19: 5.1   2.5AB   3*
3732          CP-54 2213  81157236887                       091847.3-550523092150.0-553054276.21-03.92 5.63  +0.19 +0.11         A3IV s:            -0.065+0.060      +059       34
3733   Lam PyxCD-28 7196  811691773742747I                  091852.4-282422092312.3-285002257.26 15.04 4.69  +0.92 +0.63 +0.47   G8.5IIICN-1.5*     -0.123+0.019 +.038+010                          *
3734   Kap VelCP-54 2219  81188236891 353                   091901.0-543501092206.8-550039275.88-03.54 2.50  -0.18 -0.75 -0.19   B2IV-V             -0.008+0.009 +.013+022SBO    49                 *
3735          CD-37 5721  81309200320                       091943.6-371942092344.8-374526263.89  8.90 6.48  +0.18               A1mA5-F0           -0.083+0.017      +020
3736          BD+17 2078  81361 985612750                   092000.1+170102092532.5+163508214.32 41.41 6.29  +0.97              gG9                 -0.084-0.025 +.016+012V
3737          CD-38 5541  81411200330      W                092018.0-385941092416.3-392533265.15  7.80 6.06  +0.20 +0.10 +0.12   A7V:               +0.012-0.036      +008V          0.0   0.1      *
3738 28    HyaBD-04 2616  814201368321245I                  092024.0-044110092524.0-050703237.84 30.73 5.59  +1.52 +1.82         K5III              -0.012-0.008      +005V                         *
3739          CD-51 3767  81471236930                       092038.8-511825092359.5-514414273.78-01.01 6.08  +0.57 +0.37         A7Iab              +0.013-0.002      +018       62                 *
3740          CP-59 1374  81502250561                       092047.7-595221092327.4-601809279.76-07.16 6.30  +1.48 +1.68         K1-2II-III         -0.005-0.004      +022V?
3741          BD-00 2195  81567136844                       092116.8-010153092622.3-012750234.48 33.03 6.01  +1.32 +1.44         K2IIIb             -0.001-0.003      -015
3742          CP-61 1265  81613250563                       092133.5-611300092405.7-613856280.78-08.06 5.99  +1.06               K0III              +0.016-0.051      -008
3743          BD+46 1509  81688 428762751  W                092206.8+460223092840.0+453605174.22 46.07 5.41  +0.98 +0.74         K0III-IV           -0.007-0.128      +039           2.5  77.3AB   3*
3744 29    HyaBD-08 2678  81728136861        7382           092220.7-084723092714.6-091325241.94 28.63 6.54  +0.05 +0.02         A2V                -0.029+0.001      +030           0.0   0.3AB   3*
3745          CD-28 7271  81753177461        7379   4492    092223.3-282114092644.8-284715257.78 15.63 6.10  -0.10 -0.55         B6Ve              v-0.027-0.001D+.005+004V          0.7   0.5      *
3746          CD-39 5507  81780221167                       092232.0-400406092628.5-403007266.22  7.33 6.20  +0.25               A7III              +0.025+0.002      -003V
3747          BD+56 1388  81790 27271                       092238.8+561057092947.6+554444160.10 44.12 6.45R                     F3V s              -0.132-0.033      +010
3748 30Alp HyaBD-08 2680  81797136871 354I W        4496    092240.4-081330092735.2-083931241.49 29.05 1.98  +1.44 +1.72 +0.77   K3II-III           -0.014+0.033 +.022-004V?   < 17  7.5 283.1AB   3*
3749          BD-21 2802  81799177469    I                  092243.7-215418092718.4-222038252.94 20.14 4.69  +1.14 +1.15 +0.56   K2.5III            +0.180-0.161 +.030+029
3750          BD-05 2802  81809136872      W                092249.5-053803092746.8-060416239.15 30.65 5.38  +0.64 +0.13         G2V                -0.224-0.074 +.038+054SB2O=< 10  0.8   0.2      *
3751          BD+81  302  81817  1551 910                   092251.2+814607093705.2+811935130.65 32.65 4.29  +1.48 +1.72 +0.74   K3IIIa             -0.019-0.014 +.017-005V    < 19:
3752          CP-61 1271  81830250575      W                092257.8-613108092527.1-615701281.11-08.16 5.77  +0.15 +0.13         A4V                -0.129+0.061      +012           4.5   8.7      *
3753          CP-52 2360  81848236972                       092302.7-525643092618.1-532245275.18-01.94 5.11  -0.11 -0.50         B6V                -0.008 0.000      +024SB    201                 *
3754  2Ome LeoBD+09 2188  81858117717        7390           092306.1+092932092827.5+090324223.66 38.90 5.41  +0.60 +0.14         F9V                +0.055-0.004 +.034-006SBO        0.7   0.5      *
3755  3    LeoBD+08 2226  818731177182752    7391           092309.7+083729092829.2+081118224.64 38.50 5.71  +1.04 +0.90         K0III              -0.027-0.030      +022V          5.3  25.2      *
3756          CD-34 5895  81919200392      W                092329.2-343418092738.4-350028262.47 11.40 6.65  +0.20               A3                 -0.028-0.031      -028V          0.1   0.1
3757 23    UMaBD+63  845  81937 14908 355I   7402   4506    092338.9+632957093131.7+630343150.64 41.74 3.67  +0.33 +0.10 +0.18   F0IV               +0.109+0.028 +.041-010V     140  5.6  22.8AB   3*
3758          BD-00 2201  81980136894        7396           092356.9-004913092902.2-011525234.73 33.70 6.27  +0.27 +0.11         F0Vn               -0.073+0.001D+.012+000      176  5.0   1.3
3759 31Tau1HyaBD-02 2901  81997136895      W                092404.3-021955092908.9-024608236.23 32.86 4.60  +0.46  0.00 +0.22   F6V                +0.130-0.015 +.072+010V?     28  3.0  65.7      *
3760          BD-01 2268  82043136899                       092420.3-014605092924.5-021219235.72 33.23 6.14  +0.22 +0.16         F0III              -0.046-0.003      +016V
3761          CP-64 1037  82068250583                       092434.1-642946092644.2-645547283.34-10.17 6.05  +0.15 +0.10         A3Vn               -0.056+0.063      +008
3762          BD-03 2693  82074136900                       092431.1-034828092932.4-041450237.72 32.08 6.26  +0.84 +0.40         G6IV               -0.029-0.080      -012
3763          BD-20 2915  82077177521    I                  092436.4-201844092912.6-204455252.00 21.54 5.66  +1.60 +1.85         M1III              -0.017+0.022      -008SB
3764  7    LMiBD+34 1999  82087 615292755  W                092440.7+340543093043.2+333920191.56 46.65 5.85  +1.05              gG8                 -0.020-0.049      +002V          3.3  62.8AB   3
3765   Eps AntCD-35 5724  82150200416 356                   092507.0-353050092914.7-355705263.39 10.96 4.51  +1.44 +1.68 +0.76   K3IIIa             -0.022+0.001 +.005+022SB
3766          CD-37 5817  82165200419                       092514.6-375757092916.3-382414265.13  9.21 6.19  +0.22               A7Vn               -0.085-0.013      +001
3767          BD-22 2623  82180177541    I                  092517.8-225426092949.9-232043254.14 19.89 6.24  +1.57 +1.83         K0                 -0.002-0.003      -000V
3768 22    UMaBD+72  462  82189  6898              Var      092527.9+723859093453.6+721220139.93 37.74 5.72  +0.48 -0.01         F7V                +0.075-0.077      -038                          *
3769  8    LMiBD+35 2015  82198 61540    I          4510    092527.3+353244093132.4+350611189.47 46.93 5.37  +1.53 +1.81         M1IIIab            -0.056-0.102      +038
3770          CD-26 7117  82205177546    I   7405           092528.1-260905092954.5-263523256.64 17.66 5.48  +1.36 +1.52         K3III              -0.022+0.002      +012V          8.6  11.6      *
3771 24    UMaBD+70  565  82210  6897 357I         DK UMa   092538.7+701612093428.9+694949142.55 38.93 4.56  +0.77 +0.34 +0.41   G4III-IV           -0.063+0.077 +.042-027     < 19:                *
3772          BD-14 2867  82232155246                       092538.3-150812093022.6-153438247.99 25.19 5.85  +1.20 +1.22        gK3                 -0.066-0.063      +024V?
3773  4Lam LeoBD+23 2107  82308 808852756I          4514    092600.9+232433093143.2+225805206.69 44.86 4.31  +1.54 +1.89 +0.89   K5III             v-0.021-0.033 +.020+027V    < 17                 *
3774          BD+74  402  82327  6900                       092607.6+744547093606.8+741904137.63 36.68 6.46  -0.10 -0.33         B9V                -0.016-0.070      -011V?
3775 25The UMaBD+52 1401  82328 27289 358I   7420   4519    092610.3+520800093251.4+514038165.45 45.66 3.17  +0.46 +0.02 +0.27   F6IV               -0.954-0.531 +.052+015SB1O   12 10.7   4.1      *
3776          CP-61 1277  82347250590                       092616.2-615009092847.0-621623281.62-08.11 5.92  +1.10 +0.97         K1III              -0.076+0.031      +000
3777          CP-71  833  82350256623                       092607.3-711003092706.4-713608288.22-14.85 5.47  +1.08 +0.98 +0.38E  K2III              -0.102+0.075 +.036+003SB                        *
3778          BD+50 1657  82380 42914                       092624.2+495247093307.2+492619168.59 46.19 6.76                      A4V                -0.024+0.009      -010V?
3779  6    LeoBD+10 2014  82381117751    I   7416           092635.9+100924093157.6+094257223.42 39.97 5.07  +1.37 +1.53         K2.5IIIbFe-0.5     -0.003-0.013      +019SB   < 17  4.4  37.3      *
3780   Zet1AntCD-31 7355  82383200444      W     B          092628.7-312703093045.4-315329260.70 14.07 7.00  +0.09 +0.09         A2V                -0.013-0.026D+.001-003           0.7   8.0      *
3781   Zet1AntCD-31 7355  82384200445      W     A          092629.1-312656093046.1-315322260.70 14.07 6.18  +0.03 +0.03         A0V                +0.028-0.028D+.001-003V?         0.7   8.0      *
3782  5Xi  LeoBD+11 2053  82395 986271246I D        4518    092633.3+114434093156.7+111759221.56 40.68 4.97  +1.05 +0.86 +0.52   K0-III-IIIb        -0.094-0.082 +.028+029V    < 19: 1.9   0.0      *
3783          CP-66 1018  824062505912754                   092633.3-661554092830.6-664207284.74-11.30 5.91  +0.01 -0.02         A0V                -0.033+0.047      +029V
3784          CD-50 4204  82419237042                       092640.4-510441093005.2-513102274.31-00.20 5.45  -0.10 -0.42         B8V                -0.004+0.009      +010
3785          BD-09 2856  82428155257                       092646.4-100640093138.9-103308243.89 28.65 6.14  +0.24 +0.10         F0Vn               +0.014-0.020      -018SB    184
3786   Psi VelCD-39 5580  82434221234      W        4513    092645.6-400144093042.0-402800266.78  7.91 3.60  +0.36 -0.03 +0.22   F3IV+F0IV          -0.182+0.073 +.065+009V     201: 0.5   0.7      *
3787 32Tau2HyaBD-00 2211  82446136932                       092653.0-004437093158.9-011106235.15 34.34 4.57  +0.10 +0.09 +0.06   A3V                -0.012-0.011 +.027+006SB     70
3788          BD-09 2858  82477155262                       092703.6-095547093155.8-102214243.78 28.82 6.13  +1.19 +1.17         K0                 -0.086+0.010      +064V?
3789   Zet2AntCD-31 7369  82513200459                       092715.5-312551093132.2-315219260.80 14.20 5.93  +0.26 +0.16 +0.14   A9IV               -0.039+0.001      +022
3790          CD-35 5751  82514200462      W                092722.5-351608093132.9-354254263.56 11.45 5.87  +1.29 +1.47         K4III              +0.143-0.177 -.004+014           8.2  23.6AB   3
3791  9    LMiBD+37 1998  82522 61561                       092722.1+365549093330.3+362913187.47 47.39 6.18  +1.27              gK4                 +0.031-0.035      -017V
3792          BD+29 1913  82523 809002760    7426           092727.0+284840093318.3+282205199.29 46.46 6.53  +0.12 +0.10         A3Vnn              -0.044-0.034      +026V          5.2  30.5      *
3793          CP-57 2090  82536237056      W        4516    092728.2-575520093023.4-582142279.04-05.15 5.88  +1.70 +1.68         M2III              -0.031+0.028      +003V          7.7   2.3      *
3794          BD+02 2217  825431177572758  W                092731.2+021825093241.4+015151232.17 36.18 6.11  +0.62 +0.27         F7IV-V             -0.020-0.038      +028    =< 10  0.0   0.1
3795   Iot ChaCP-80  350  825542585302753                   092729.2-802118092409.1-804713295.18-21.22 5.36  +0.45 +0.06         F3-5IV            v-0.146+0.140      +007      123                 *
3796          BD-18 2708  825731552732757                   092741.5-185731093220.4-192401251.46 23.00 5.74  +0.14 +0.12 +0.04   A5V                -0.036+0.010      +027       74
3797          BD+47 1683  82582 42924                       092745.7+472045093419.6+465408172.15 46.86 6.52  +0.22 +0.09         F0V                -0.062-0.008      +011      142
3798          CD-28 7373  82610177619              S Ant    092755.7-281114093218.5-283741258.55 16.62 6.46  +0.33               A9V                -0.072+0.039      +015SBO   113                 *
3799 26    UMaBD+52 1402  82621 27298                       092758.5+522946093449.5+520305164.83 45.85 4.50  +0.01 +0.04 +0.01   A2V                -0.065-0.035 +.025+023V     180
3800 10    LMiBD+37 2004  82635 61570 360I         SU LMi   092805.9+365030093413.4+362351187.61 47.53 4.55  +0.92 +0.62 +0.46   G8.5III            +0.006-0.022      -012V    < 19:                *
3801          BD-07 2836  82638136945                       092806.7-080342093302.1-083019242.30 30.20 6.12  +0.98 +0.71         K0                 -0.020-0.034      +036
3802          BD-12 2926  826601552812761I                  092807.6-130427093255.8-133101246.70 27.00 5.94  +1.50 +1.74         K5III              -0.008+0.003      +028
3803          CP-56 2270  82668237067 361          N Vel    092810.9-563535093113.3-570204278.21-04.11 3.13  +1.55 +1.89 +0.89   K5III              -0.032+0.004 +.022-014                          *
3804          BD+24 2104  82670 80909    I                  092816.6+235359093359.1+232714206.23 45.50 6.25  +1.46              gK7                 -0.052-0.087      -006V?
3805          BD-06 2939  82674136951                       092822.8-064447093320.1-071124241.15 31.07 6.24  +1.17 +1.25         K0                 -0.033-0.012      +029SBO                       *
3806          BD+73  470  82685  6915        7446           092823.5+733139093756.2+730450138.84 37.47 6.42R                     F2V+F1V            -0.041-0.026D+.017+000           0.1   4.7      *
3807          CD-40 5284  826942212622759  W                092821.0-401225093219.3-403858267.13  7.99 5.35  +0.90               G8III              +0.011-0.012 +.024-001           3.6   1.1
3808          BD-20 2936  827341776421247I                  092836.1-204023093312.5-210657252.97 21.98 5.01  +1.02 +0.87         K0IV               -0.020+0.015 +.052+013
3809          BD+40 2224  82741 615782762I                  092849.6+400356093503.8+393717182.83 47.71 4.81  +0.99 +0.76 +0.53   G9.5IIIFe-1        -0.030+0.022      -012     < 19:
3810          BD-22 2645  82747177649                       092852.9-222519093326.2-225150254.38 20.82 5.91  +0.02               A0Vn               -0.049+0.060      +015
3811          BD+40 2226  82780 42931        7438A          092907.3+402428093522.4+395748182.32 47.76 6.76  +0.35  0.00         F2V                -0.016+0.012      -035SBO    45  1.4  24.9AB   3*
3812          CD-38 5676  82785200492      W                092905.9-384112093307.8-390744266.19  9.20 6.43  +0.33               F0IV               -0.029+0.049      +025           3.5  55.2
3813          CP-66 1025  82858250611                       092932.5-661636093132.9-664310284.97-11.11 6.27  +1.35 +1.38         K1III              -0.027-0.006      +004
3814 33    HyaBD-05 2840  828701369642763I                  092933.2-052807093432.7-055454240.18 32.09 5.56  +1.16 +1.13 +0.41E  K1III              +0.011-0.059      +013V                         *
3815 11    LMiBD+36 1979  82885 61586        7441  SV LMi   092939.7+361545093539.6+354837188.49 47.81 5.41  +0.77 +0.44 +0.37   G8V                -0.706-0.241 +.109+013V?   < 17  8.5   3.3      *
3816          CP-62 1253  82901250614    I:W       R Car    092943.6-622046093214.6-624720282.27-08.21 6.10  +1.43 +0.23 +2.51   M6-7IIIep         v-0.036+0.020      +028V          7.3   2.1      *
3817          CD-48 4802  82984221288      W                093009.1-483338093344.5-490018273.03  2.04 5.12  -0.12 -0.58         B4IV               -0.016+0.010D+.015+017V     117  0.8   2.1      *
3818  7    LeoBD+15 2077  83023 98662        7448A          093025.0+144934093552.9+142247218.36 42.85 6.36  +0.01 +0.03         A1V                -0.038 0.000 +.006+020       92  3.2  41.2     3*
3819          CD-50 4270  830582371072764                   093040.7-504836093408.8-511519274.60  0.42 5.01  -0.18 -0.94         B1.5IV             -0.003+0.004      +035SB    207
3820          BD+31 2011  83069 615942765I          4545    093046.9+313636093642.9+310942195.41 47.63 5.56  +1.59 +1.86         M2IIIa             +0.003-0.039      -020V?
3821          CP-72  835  83095256634 362                   093051.4-723814093136.3-730451289.55-15.64 5.47  +1.56 +1.75         K4III              -0.027+0.001      +014
3822          BD-18 2728  83104155323      W                093054.7-190809093533.7-193501252.17 23.43 6.31  +0.06 +0.03         A2V                -0.038-0.046      +016V?         3.2  51.5
3823          CD-35 5803  83108200531                       093102.2-352238093511.8-354926264.18 11.89 6.49  +0.42               F7III-IV           -0.072-0.002      +013
3824          BD+67  602  83126 149492769I                  093110.6+674322093927.9+671620145.15 40.59 5.94  +1.52               K5                 -0.026-0.040      +019
3825          CP-58 1576  83183237117                       093132.6-584701093426.7-591346280.02-05.43 4.08  +0.01 -0.56 +0.01   B5II               -0.012+0.014      +022V?      0                 *
3826  8    LeoBD+17 2109  83189 986732766I D                093131.5+165310093702.6+162616215.91 43.91 5.69  +1.25              gK1                 -0.014-0.006      +006                          *
3827 10    LeoBD+07 2160  832401178072767I          4551    093155.9+071703093712.7+065009227.52 39.73 5.00  +1.05 +0.87         K1III             v-0.060+0.001 +.019+020SB   < 17
3828          CD-24 8263  83261177738      W                093203.3-241523093633.7-244210256.32 20.05 6.53  +0.38               F2                 -0.119+0.060      +024           0.0   0.1
3829 42    LynBD+40 2232  83287 42958                       093207.2+404120093821.7+401423181.86 48.31 5.25  +0.22 +0.12         F0V                -0.017+0.006      -003V      96
3830          CD-24 8272  83332177748                       093230.3-245057093700.2-251748256.84 19.71 5.70  +1.12              gK1                 -0.062+0.043      +030
3831          CD-48 4831  83368221339      W       IM Vel   093247.7-481808093625.3-484505273.18  2.52 6.17  +0.27 +0.15         ApSrEuCr           -0.011-0.024D+.021-004       33  2.4   3.1      *
3832 34    HyaBD-08 2725  83373137011                       093257.3-085830093751.5-092528244.00 30.55 6.40  -0.04 -0.09         A1V                -0.062+0.002      +025    =< 41
3833          CD-31 7458  833802005611248                   093251.6-314345093709.9-321043261.91 14.81 5.63  +1.02               K0III              +0.034-0.020      -003
3834          BD+05 2207  834251178211249I                  093314.3+050604093827.3+043857230.16 38.88 4.68  +1.32 +1.46 +0.71   K3III              -0.163-0.051 +.011+045V    < 19:                *
3835          CD-35 5833  83441200573                       093318.2-353847093728.3-360545264.72 12.00 5.98  +1.12               K2III              -0.035-0.001      +020
3836          CD-48 4836  834462213442768  W                093314.8-485424093649.6-492119273.64  2.12 4.35  +0.17 +0.13 +0.10   A5IV-V             -0.125+0.025 +.019+021      144  8.7  25.7
3837          CP-52 2612  83465237144                       093324.1-522950093646.3-525639276.03-00.56 6.19  +1.05 +0.89         K0III              -0.120+0.081      +034
3838          BD+69  531  83489 14966 363                   093341.5+694134094214.8+691415142.76 39.82 5.69  +1.14              gG9                 -0.067-0.070      -009V?
3839 27    UMaBD+72  466  83506  69362772I                  093345.1+724226094257.2+721509139.45 38.23 5.17  +1.04 +0.90         K0III              -0.026-0.030      -017V?   < 19:
3840          CP-53 2646  83520237149      W                093351.8-531306093712.3-534007276.56-01.05 5.45  +0.15 +0.08         A2-3V              -0.054-0.017      -013           0.0   0.5      *
3841          CP-64 1049  83523250636                       093345.8-643009093605.1-645702284.08-09.50 6.56  +0.10 +0.11         A2V                -0.040+0.036      +016
3842          CD-42 5462  835482213552771                   093407.0-424422093801.5-431128269.65  6.84 5.50  +1.00 +0.68 +0.43C  G8II               +0.032-0.037 +.020+002                          *
3843          BD+78  317  83550  6948                       093410.8+783526094530.8+780805133.41 34.91 6.23  +1.35               K2III              +0.004+0.003      -027
3844          CD-39 5697  83610200590      W                093438.0-390939093840.7-393651267.32  9.58 6.70  +0.49               F5                 -0.044-0.062      +002V?         2.0   0.5      *
3845 35Iot HyaBD-00 2231  836181370351250I          4568    093444.9-004120093951.4-010834236.49 35.98 3.91  +1.32 +1.46 +0.67   K2.5III-IIIbBa0.3  +0.049-0.064 +.026+023V?   < 17                 *
3846 37    HyaBD-09 2898  83650155377                       093454.4-100705093947.4-103413245.37 30.18 6.31  -0.03 -0.14         A0Vn               -0.024+0.005      +014V     127
3847          BD+79  319  83727  69562780                   093527.0+793543094718.0+790812132.39 34.35 6.17  +0.25 +0.12         F0V:               -0.031-0.021      -007
3848          BD-10 2888  83731155387                       093527.2-101857094020.1-104609245.65 30.16 6.37  +0.08               A2V                -0.004-0.001      -011V
3849 38Kap HyaBD-13 2917  83754155388 364           4576    093530.7-135243094018.4-141956248.71 27.81 5.06  -0.15 -0.57 -0.15   B5V                -0.027-0.020      +018V?    185                 *
3850          BD+31 2026  83787 61633    I W       DR Leo   093540.2+314356094135.2+311641195.43 48.68 5.89  +1.60 +1.77 +0.97   K6III              +0.030-0.001      -013           7.5  28.4      *
3851 43    LynBD+40 2241  83805 616362773                   093548.7+401249094200.3+394528182.53 49.03 5.62  +0.95 +0.68 +0.46   G8III              -0.050-0.045 +.018+030V?   < 19:
3852 14Omi LeoBD+10 2044  83808 98709 365I   7480           093548.8+102050094109.0+095332224.60 42.06 3.52  +0.49 +0.21 +0.23   F6II+A1-5V         -0.142-0.037 +.034+027SBO =< 50  0.2   0.0     3*
3853 13    LeoBD+26 1991  83821 80970                       093553.3+262205094138.5+255446203.37 47.78 6.24  +1.25       +0.66  gK2                 -0.014-0.033 +.013-026V?
3854          BD+49 1868  83869 429902774                   093608.6+485312094243.1+482552169.53 47.95 6.39  -0.01  0.00         A1V                -0.030-0.017      -008SB2    86
3855          BD+55 1345  83886 273592775                   093612.0+544913094307.0+542149161.12 46.36 6.47  +0.13 +0.09 +0.02   A5m                -0.035-0.034      +021      100
3856          CP-60 1477  83944250653               4577    093634.8-605231093921.0-611941281.89-06.57 4.52  -0.07 -0.21 -0.08   B9IV-V             -0.041+0.021      +020V      68                 *
3857 13    LMiBD+35 2042  83951 616482776                   093641.1+353302094242.7+350536189.66 49.21 6.14  +0.36  0.00         F3V                -0.021-0.052      -008V      12
3858          BD-22 2684  83953177840      W                093643.2-230812094117.0-233530256.29 21.60 4.77  -0.12 -0.58 -0.09   B6Ve              v-0.028-0.004      +026      332  5.2  54.7      *
3859          BD+65  731  83962 14976                       093644.8+652626094436.7+645902147.47 42.17 6.17R                     F3Vn               -0.049+0.012      -028      116
3860   Zet ChaCP-80  365  83979258538              Zet Cha  093650.1-802931093353.2-805629295.57-21.04 5.11  -0.14 -0.57 -0.18   B5V                -0.043+0.007      -042SB     88                 *
3861 15    LeoBD+30 1901  84107 616561251                   093741.6+302603094333.3+295828197.47 48.93 5.64  +0.12 +0.08 +0.03   A2IV               -0.018-0.100      +016
3862          CD-23 8646  84117177866                       093743.6-232800094214.4-235456256.70 21.53 4.94  +0.53  0.00 +0.28   F9IV               -0.395+0.260 +.080+034V?
3863          CP-57 2228  84121237221      W                093737.7-573143094042.5-575901279.79-03.95 5.32  +0.20 +0.10         A3IV               -0.034+0.006 +.031+007SB     39  0.2   0.2      *
3864          CP-56 2435  84152237224                       093752.4-564811094102.2-571535279.35-03.38 5.80  +1.09 +0.98         K0-1III            +0.081-0.043      +039
3865 28    UMaBD+64  752  84179 14980      W                093813.9+640648094555.4+633912148.96 42.93 6.34  +0.31 -0.02         F2V                -0.012-0.043      -027SB    101  5.2   6.4      *
3866 16Psi LeoBD+14 2136  84194 987331252I W        4594    093817.2+142846094343.9+140118219.89 44.44 5.35  +1.63 +1.95         M2IIIab            +0.004-0.003      +008           5.3 281.8
3867          CD-34 6097  842242006522777                   093828.1-350242094241.4-353006265.10 13.16 6.41  -0.06 -0.37         ApSi               -0.014+0.007      +017
3868          CP-54 2594  84228237237                       093830.0-544527094147.8-551251278.08-01.77 6.00  -0.13 -0.58         B4V                -0.018-0.004      +014
3869          BD+19 2251  84252 98742      W                093856.4+191925094430.0+185149213.60 46.44 6.50R                     K0                 -0.007-0.054      -001           6.0  31.2
3870          BD+57 1231  84335 27377    I         CS UMa   093926.9+573514094631.7+570741157.15 45.82 5.2   +1.62 +1.80         M3IIIab            +0.004+0.034      +008V                         *
3871   The AntCD-27 6881  84367177908 366  W                093944.6-271842094412.1-274610259.89 19.06 4.79  +0.51 +0.35 +0.32   A8V+F7II-III       -0.047+0.035 +.053+024V?         0.2   0.1      *
3872          CD-50 4420  84400237260      W       IP Vel   093954.0-504611094327.5-511342275.67  1.41 6.15  -0.10 -0.46         B6V                -0.024-0.004      -023V          4.7   2.1      *
3873 17Eps LeoBD+24 2129  84441 81004 367I          4613    094010.5+241405094551.1+234627206.82 48.21 2.98  +0.80 +0.47 +0.44   G1II               -0.046-0.011 +.010+004V?   < 17                 *
3874          CD-38 5850  84447200681                       094010.6-390643094415.8-393416268.11 10.32 6.82  +0.30               F2-3V              -0.029+0.008      +005
3875          CP-53 2788  84461237268                       094018.5-532600094342.2-535330277.43-00.59 5.56  -0.04 -0.08 -0.07   A0IV               -0.068+0.022      +010V      16                 *
3876          BD+07 2181  845421178982781I                  094053.5+071013094610.0+064231229.14 41.59 5.79  +1.64 +1.96 +0.81E  M1.5IIIab          +0.004-0.022      +003V                         *
3877 18    LeoBD+12 2090  84561 98755    I                  094100.1+121615094623.3+114836223.07 44.08 5.63  +1.49              gK4                 -0.017+0.015      +030V
3878          CD-29 7758  84567177939                       094058.0-294433094521.8-301210261.83 17.45 6.45  -0.13 -0.94         B0.5IIIn           -0.032+0.001      +033                          *
3879          BD+02 2246  846071179012782                   094114.0+021453094623.6+014708234.63 38.99 5.65  +0.34 +0.12         F4IV               -0.056-0.041      +015SB
3880 19    LeoBD+12 2095  84722 98767      S                094203.4+120151094725.9+113406223.53 44.20 6.45  +0.43 +0.09         A7Vn               -0.053+0.005      -004           0.2   0.0      *
3881          BD+46 1551  84737 430461255                   094208.5+462913094835.4+460116172.78 49.43 5.09  +0.62 +0.10 +0.34   G0.5Va             +0.224-0.092 +.073+005V       3
3882          BD+12 2096  84748 98769    I         R Leo    094210.8+115334094733.5+112544223.72 44.16 6.02  +1.30 -0.20 +3.24   M8IIIe            v+0.002-0.042 +.020+013V?                        *
3883          CP-56 2499  84809237314               4623    094228.3-564328094540.5-571108279.78-02.92 6.46  -0.11 -0.58         B8III-IV           -0.031+0.017      +011V?                        *
3884          CP-61 1333  848102506831254          l Car    094230.0-620248094514.8-623028283.20-07.00 3.69  +1.22 +0.85 +0.68   G5Iab-Ib          v-0.014+0.007 +.027+003V                         *
3885          BD+66  637  84812 15000                       094233.3+660332095023.7+653536146.34 42.38 6.31  +0.28 +0.12         A9Vn               -0.052-0.029      -007      176
3886          CD-44 5846  84816221484              Var?     094236.3-441733094630.4-444518271.85  6.67 5.55  -0.18 -0.72 -0.15   B2.5IV             -0.008+0.002      -004SB    114                 *
3887          CP-58 1640  84850237321                       094251.5-582001094555.4-584739280.84-04.12 6.22  +0.46               F5IV               -0.123+0.059      +006
3888 29Ups UMaBD+59 1268  84999 27401 368I   7534  Ups UMa  094352.9+593033095059.4+590219154.31 45.58 3.80  +0.29 +0.10 +0.16   F2IV               -0.293-0.151 +.041+027V?    110  8.6  11.3      *
3889 20    LeoBD+21 2113  85040 810352785  W       DG Leo   094414.3+213844094950.1+211046210.95 48.37 6.09  +0.25 +0.18         A7IVn              -0.043-0.012      +026SB2O  139  0.3   0.3      *
3890   Ups CarCP-64 1084  85123250695      W     A          094436.1-643629094706.1-650419285.04-08.82 3.01  +0.28 +0.18         A6Ib               -0.012+0.011 +.027+014        0  3.0   5.0      *
3891   Ups CarCP-64 1084  85124250696      W     B          094436.7-643632094706.7-650421285.04-08.82 6.26  +0.08 -0.54         B7III              -0.011+0.023 +.027+014           3.0   5.0      *
3892          CD-36 5955  85206200777                       094516.4-364315094928.1-371111267.30 12.79 5.97  +1.24               K1III              -0.084+0.023      +012
3893  4    SexBD+05 2240  85217117937                       094517.8+044843095030.1+042037232.58 41.26 6.24  +0.48 -0.02         F7Vn               -0.147-0.055      +017SBO                       *
3894 30Phi UMaBD+54 1331  85235 27408        7545           094518.2+543154095206.4+540352160.87 47.70 4.59  +0.03 +0.08  0.00   A3IV               -0.004+0.021 +.032-012       30  0.2   0.3      *
3895          CP-55 2548  85250237370                       094522.7-555647094840.0-562443279.60-02.06 6.06  +0.94 +0.67         K0III              -0.056+0.014      +007                          *
3896 23    LeoBD+13 2164  85268 98809    I                  094537.3+133202095102.0+130358222.18 45.65 6.46  +1.58 +1.92         M0III              +0.028-0.008      -009V?
3897          CD-35 5961  85296200784                       094537.2-354807094951.3-361607266.74 13.54 6.37  +1.01               K0III              -0.022+0.002      +021V
3898          CD-45 5470  85355221523      W                094604.1-451557094957.1-454358272.95  6.31 5.08  -0.10               B7III              -0.026+0.004      +012      130: 4.5  66.6      *
3899  6    SexBD-03 2794  85364137183 370                   094611.6-034629095114.0-041436241.68 36.39 6.01  +0.17 +0.11         A8III              +0.013-0.028      -010V     101
3900 22    LeoBD+25 2169  85376 81054                       094612.5+245207095153.0+242343206.42 49.69 5.32  +0.23 +0.08         A5IV               +0.006-0.179 +.040-002V?    126                 *
3901          BD-05 2923  85380137185                       094622.6-054255095121.6-061054243.56 35.19 6.42  +0.58 +0.11         F8V                -0.125+0.060      +036V   =< 10
3902   Nu  ChaCP-76  598  853962566582784                   094618.5-761835094620.6-764634292.94-17.65 5.45  +0.89 +0.57         G8III              +0.079-0.056 +.019+011
3903 39Ups1HyaBD-14 2963  854441555422786I                  094640.0-142239095128.7-145048251.23 29.44 4.12  +0.92 +0.65 +0.47   G7-III             +0.017-0.026 +.021-015     < 19:                *
3904          CD-46 5558  85483221538                       094651.8-462800095042.0-465604273.82  5.46 5.73  +1.08 +1.02         K0IIICNIb          -0.034+0.016      +017V
3905 24Mu  LeoBD+26 2019  85503 81064 371I                  094704.6+262841095245.8+260025204.05 50.25 3.88  +1.22 +1.39 +0.58   K2IIICN1Ca1        -0.216-0.056 +.025+014V?   < 17                 *
3906  7    SexBD+03 2280  85504117959               4656    094702.6+025514095212.2+022715234.98 40.57 6.02  -0.04 -0.08         A0V s              -0.181+0.092 +.011+097V   =< 41
3907          BD+00 2573  855051179602788                   094704.4+003244095212.0+000432237.51 39.20 6.35  +0.94 +0.69         G9III              -0.022-0.032      +019
3908          BD-15 2920  85519155553                       094713.1-160350095159.6-163205252.72 28.36 6.08  +1.04 +2.12         K0                 +0.026-0.067      +005V
3909  8Gam SexBD-07 2909  85558137199        7555           094733.4-073803095230.4-080618245.57 34.17 5.05  +0.04 +0.06         A1V                -0.055-0.040 +.013+012V     114  0.5   0.4AB   3*
3910          CD-45 5499  855632215471256                   094727.4-454333095119.8-461138273.43  6.10 5.62  +1.17 +1.16         K2III              -0.043+0.033      +018
3911          BD+61 1151  85583 150232790  W                094745.7+613515095503.4+610658151.35 45.07 6.27  +1.05               K0                 +0.009-0.002      -011           6.8  12.2
3912          CD-45 5508  85622221553               4658    094748.8-460443095140.8-463252273.70  5.87 4.58  +1.20 +0.99 +0.63   G5Ib               -0.008-0.001 +.007+011SB1O                      *
3913          CP-58 1673  856552374182789                   094805.0-585719095112.1-592533281.77-04.18 5.79  +1.36 +1.28         K2IVCNIV-Vp        +0.035-0.045      +024V
3914          CP-62 1335  85656250721                       094806.8-621635095055.7-624443283.86-06.77 5.57  +1.32 +1.31         K1IIICNII          -0.007+0.007      +012
3915          BD+06 2224  85709117975    I                  094827.8+062546095342.9+055730231.32 42.80 5.95  +1.66 +1.94 +0.93E  M2III              -0.008-0.006      -001V?
3916          CD-26 7505  85725178130      W                094829.3-265151095258.0-271956261.10 20.70 6.30  +0.62               G1V                -0.284+0.093 +.015+023           4.0   1.3      *
3917 31    UMaBD+50 1698  85795 274302793          SY UMa   094911.4+501731095543.0+494912166.65 49.63 5.27  +0.07 +0.12         A3III              -0.003+0.022 +.025-006      147                 *
3918          BD+73  478  85841  7009 372                   094927.0+732118095822.8+725246137.93 38.80 5.83  +1.14              gK3                 -0.076-0.035      +004V?
3919          CD-25 7585  858591781582792I                  094940.4-252743095412.3-255557260.31 21.92 4.88  +1.23 +1.30 +0.45E  K2+III-IIIb        -0.180+0.054 +.031+051V?                        *
3920          CP-54 2816  85871237448               4668    094935.0-545407095300.0-552224279.41-00.87 6.48  -0.14 -0.86         B1IV               -0.018+0.001      -018SB    274                 *
3921          BD-21 2935  85905178164                       094953.5-220054095431.7-222918257.84 24.51 6.24  +0.04               A2                 -0.038-0.031      +009                          *
3922          BD+58 1224  85945 27438                       095015.4+575339095713.6+572506155.90 47.02 5.93  +0.89               G5III              +0.032-0.059 +.007-044V                         *
3923          BD-18 2810  85951155588 373I                  095009.2-183208095452.2-190034255.24 27.09 4.94  +1.57 +1.93 +0.74E  M1III              -0.047-0.037 +.011+050SB
3924          CD-50 4622  85953237464              Var?     095010.0-504028095350.2-510849276.87  2.51 5.93  -0.15 -0.73         B2III              -0.005-0.010      +010                          *
3925          CD-44 5987  85980221592      W                095021.2-444840095417.6-451702273.25  7.14 5.71  -0.11 -0.56 -0.11   B3V                -0.020-0.008      +013V     156  2.4   5.3      *
3926          BD+09 2262  860801180012794                   095107.9+092425095626.0+085559228.26 44.90 5.85  +1.13 +1.00        gK2                 -0.085+0.018 +.011+009
3927          CD-49 4801  86087237483                       095108.1-494614095451.3-501438276.43  3.32 5.72  -0.01 -0.01         A0V                -0.029+0.012      +014SB2                       *
3928 19    LMiBD+41 2033  86146 43115 374                   095133.7+413155095741.1+410320180.07 51.89 5.14  +0.46  0.00         F6V s              -0.117-0.025 +.040-010SB1O    0                 *
3929          BD+46 1566  86166 431172798                   095138.4+455327095756.9+452452173.16 51.16 6.30  +1.11 +0.95         K0III              +0.011-0.030      +005
3930          CD-40 5626  862112216172795                   095158.0-402057095605.4-404929270.68 10.82 6.41  +1.61 +1.99         M1III              -0.018-0.022      +058
3931          CD-25 7622  86266178214        7591           095214.4-260430095646.5-263301261.22 21.85 6.28  +0.22 +0.11         A4V                -0.114+0.006      +011           4.4   3.4
3932          CD-32 6895  862672008892796                   095213.4-325637095635.5-332507265.93 16.61 5.84  +1.20               K1III              +0.038+0.016      +004
3933          CD-26 7551  86301178216                       095223.0-270001095654.0-272830261.90 21.17 6.32  +0.17               A4V                -0.081+0.029      +014
3934          BD+84  225  86321  16373955          Var      095236.9+842404100834.3+835506127.59 31.66 6.37  +1.52 +1.64         K0                 -0.004+0.007      -012                          *
3935          CD-50 4662  863522375142797                   095240.8-505138095621.9-512010277.30  2.61 6.37  -0.17 -0.75         B2IV-V             -0.004-0.007      +009
3936          BD+28 1824  86358 81120                       095242.8+281412095826.1+274532201.68 51.82 6.30R                     F3V                -0.117-0.034      +036SB2    75                 *
3937 27Nu  LeoBD+13 2183  86360 988762800                   095250.6+125519095813.4+122641224.10 46.95 5.26  -0.04 -0.13         B9IV               -0.025-0.015 +.018+014SB     96
3938          BD+09 2269  86369118023    I                  095249.7+084729095807.6+081851229.31 44.95 6.04  +1.36 +1.55        gK3                 +0.011-0.024      -019V
3939          BD+57 1242  86378 274642802I                  095258.8+571725095951.7+564843156.48 47.60 5.48  +1.46 +1.82         K5III              -0.029-0.026      -013V
3940   Phi VelCP-53 3075  86440237522 375  W                095321.0-540530095651.8-543404279.35  0.11 3.54  -0.08 -0.62 -0.08   B5Ib               -0.010+0.003      +014       45  8.3  37.2      *
3941          CP-52 2980  86466237526              IV Vel   095333.4-520945095710.9-523820278.20  1.66 6.12  -0.13 -0.61         B3IV               -0.010-0.002      +019V?     30                 *
3942          BD+30 1946  86513 811292803                   095350.2+300728095936.2+293843198.70 52.34 5.73  +1.06              gG9                 -0.095-0.039      -001V?
3943          CD-47 5399  86523221644      W                095352.3-475613095742.5-482452275.67  5.04 6.05  -0.14               B3V                -0.016-0.015      +009SB         4.6  14.1      *
3944          CP-70  953  866062566722799                   095425.4-705446095609.7-712322289.82-13.13 6.35  -0.08 -0.93         B9Ib               -0.020-0.002      -030
3945 12    SexBD+04 2276  86611118041 376           4694    095431.8+035146095943.1+032305235.37 42.64 6.70  +0.27 +0.09         F0V                -0.066+0.022      -004SB    187
3946          CD-23 8898  86612178271              Var?     095429.1-232821095906.1-235701259.77 24.14 6.21  -0.10 -0.68 -0.06   B4Ve              v-0.049+0.023      +015      220                 *
3947   Eta AntCD-35 6050  86629200926 377  W                095434.8-352444095852.3-355328267.94 15.00 5.23  +0.31 +0.08         F1III-IV           -0.087-0.020 +.029+030           6.0  31.1      *
3948          CP-63 1233  866342507602801                   095432.7-640052095715.2-642922285.51-07.68 6.58  +1.13 +1.08         K0-1III            -0.130+0.084      +045
3949          CP-68 1011  86659250761                       095450.9-683729095659.7-690607288.40-11.31 6.20  -0.10 -0.62         B3V                -0.027 0.000      +016V?    135                 *
3950 29Pi  LeoBD+08 2301  86663118044 378I          4699    095455.7+083127100012.8+080239230.01 45.26 4.70  +1.60 +1.93 +1.08   M2-IIIab          e-0.030-0.023 +.017+023V                         *
3951 20    LMiBD+32 1964  86728 618081258  W                095514.8+322456100100.7+315525195.01 52.86 5.36  +0.66 +0.27         G3VaHdel 1         -0.528-0.428 +.053+056        3  3.2 407.       *
3952          BD+22 2164  87015 811542807                   095714.4+222554100248.9+215657211.17 51.48 5.66  -0.19 -0.73         B2.5IV             -0.017-0.006      +003
3953          CP-56 2746  87030237586      W                095710.4-562757100034.4-565648281.21-01.46 6.52  +0.98               K0III              -0.064+0.008      +010           6.5   7.6
3954          BD+54 1348  87141 275031259                   095757.7+542232100436.3+535330160.03 49.46 5.74  +0.48 +0.04         F5V                -0.022-0.002      -016       10
3955          CP-52 3087  87152237612              Var?     095802.7-525255100140.5-532152279.18  1.49 6.20  -0.13 -0.69         B2.5V              -0.036-0.017      +006V     218                 *
3956          CD-29 8034  871992009942808                   095821.0-300543100249.3-303439265.09 19.64 6.54  +1.19               K1III              -0.014+0.030      +014V?
3957          CP-56 2770  87238237621                       095834.2-565204100158.0-572059281.60-01.67 6.20  +1.12               K1II               -0.041+0.027      +007
3958          BD+53 1384  87243 27512                       095837.0+525121100510.5+522215162.14 50.15 6.14R                     A5IV               -0.004-0.020      -026SB    167
3959          BD-08 2836  87262137326    I                  095844.5-090523100341.0-093426249.14 35.24 6.12  +1.66 +2.01         K0                 +0.005-0.011      +009V?
3960          CP-59 1695  87283250782                       095849.1-595618100200.0-602515283.45-04.12 5.94  +0.26 +0.10         A9IV               -0.019+0.006      -002                          *
3961 13    SexBD+03 2311  87301118086                       095857.6+034116100408.4+031204236.44 43.45 6.45  +0.40 -0.02         F4V                -0.075-0.097      +000SB
3962          CD-24 8711  87318178357        7625           095905.3-244958100341.4-251900261.61 23.80 6.70   0.00  0.00         A0III-IV           -0.039+0.009      +012           5.5  16.1      *
3963          BD-17 3047  87344155704        7627AB         095916.9-173702100402.9-180606256.32 29.24 5.86  -0.06               A0V                -0.010-0.002      +019V      17  1.0  21.2AC   3*
3964          CD-46 5759  87363221728                       095922.9-460907100320.5-463810275.34  7.03 6.12  +0.02               A0V                -0.029-0.010      +003
3965          CD-23 8973  874271783671260                   095943.8-234805100421.0-241708261.00 24.68 5.70  +0.30               A8V                -0.114+0.023      +004V
3966          CP-59 1752  87436237640                       095946.9-594141100259.9-601043283.41-03.86 6.19  +0.17               A6II-III           -0.013+0.005      -004                          *
3967          CP-61 1431  87438250789               4720    095946.2-614023100249.4-620923284.58-05.45 6.42  +1.72 +1.90         K3Ib               -0.010+0.010      -008
3968          CD-39 6100  874772010202809                   100010.9-392927100423.4-395833271.42 12.46 6.43  +1.30               K1III              -0.034-0.003      +047
3969          BD+16 2077  87500 98944                       100015.4+161438100540.9+154527220.74 49.99 6.37  +0.37 +0.15         F2Vn               -0.078-0.017      +012      139
3970 40Ups2HyaBD-12 3073  875041557131261           4725    100015.2-123447100507.5-130353252.46 33.06 4.60  -0.09 -0.27 -0.09   B9III-IV           -0.037+0.018      +028SB2    76
3971          CP-61 1441  87543250795      W                100028.9-612358100334.3-615302284.49-05.18 6.14  -0.04 -0.43         B7IVne             -0.005+0.005D+.008+005           1.5   1.2      *
3972          CD-35 6130  87606201037                       100055.6-355353100515.2-362302269.30 15.41 6.27  +1.11 +1.07         K1III              -0.010+0.003      +028
3973 14    SexBD+06 2259  876821181112811  D                100133.6+060557100647.4+053641234.17 45.35 6.21  +0.94 +0.72         K1III              -0.034-0.013      +017V                         *
3974 21    LMiBD+35 2110  87696 618742812           4736    100131.9+354356100725.8+351441189.47 54.26 4.48  +0.18 +0.08 +0.07   A7V                +0.053+0.004 +.050-018V     148                 *
3975 30Eta LeoBD+17 2171  87737 98955 379  W        4738    100152.9+171501100720.0+164546219.53 50.75 3.52  -0.03 -0.21 +0.02   A0Ib               -0.002 0.000 +.003+003V      20  0.5   0.1      *
3976          CD-46 5806  87783221773      W                100213.6-465252100611.3-472212276.17  6.73 5.08  +0.88 +0.55         K1IV+G5V           +0.011-0.057 +.017+021           1.7   0.7      *
3977          BD-16 2974  878081557392813I                  100221.6-163908100709.5-170830256.19 30.45 5.60  +1.50 +1.68         K7III              +0.016-0.063      +011
3978          CD-51 4471  87816237690              R Vel    100223.0-514204100607.1-521117279.02  2.84 6.52  +0.98 +0.82         K1III              -0.068+0.015      +005                          *
3979          BD+32 1982  87822 61882        7651           100229.3+320542100815.9+313615195.72 54.37 6.24  +0.45 +0.01         F4V                -0.083-0.080      -008        8  0.3   0.2AB   4*
3980 31    LeoBD+10 2112  87837 98964    I   7649           100235.8+102916100754.3+095951228.95 47.89 4.37  +1.45 +1.75 +0.76   K3.5IIIbFe-1       -0.080-0.061      +041V?   < 19: 7.7   7.9      *
3981 15Alp SexBD+00 2615  878871373662814                   100249.1+000701100756.3-002218241.11 42.07 4.49  -0.04 -0.07 -0.05   A0III              -0.016-0.013 +.015+007V?      9
3982 32Alp LeoBD+12 2149  87901 98967 380I   7654A  4750    100302.8+122722100822.3+115802226.43 48.94 1.35  -0.11 -0.36 -0.10   B7V               e-0.248+0.006 +.045+006SB    329  6.8 177.6AB   4*
3983   Mu 1ChaCP-81  399  879712585543980                   100324.3-814351100043.7-821253297.28-21.29 5.52  +0.03 +0.05         A0IV               -0.031+0.032      +016SB2    76
3984          CD-36 6156  88013201081                       100342.7-365040100801.7-372001270.36 14.99 6.36  +0.98               K0III              -0.011+0.003      +001V
3985          BD-10 3000  88024155765                       100350.3-102342100845.7-105305251.36 35.23 6.53  +0.02 +0.02         A2V                +0.004-0.003      -009
3986          BD-14 3036  88025155763                       100345.8-150721100835.5-153642255.27 31.80 6.27  -0.01 -0.04         A0V                -0.027+0.014      +009V      68
3987          BD+41 2063  88161 43220                       100457.3+410911101058.9+403941180.10 54.43 6.32  +1.25              gK3                 -0.014-0.002      +014V?
3988          BD-11 2818  88182155777                       100502.3-113613100956.5-120545252.65 34.57 6.24  +0.18 +0.15 +0.09   A5m                -0.005-0.036      -014
3989 17    SexBD-07 2972  88195137385                       100509.4-075501101007.5-082430249.45 37.19 5.91  +0.02 -0.05         A1V                -0.026-0.001      +016V?    100                 *
3990          CD-51 4507  88206237736                       100508.7-511914100856.3-514840279.15  3.40 4.86  -0.12 -0.69 -0.09   B3IV               -0.010+0.001      +014V     229                 *
3991          BD-12 3101  88215155780                       100513.4-121917101005.9-124858253.29 34.08 5.31  +0.36  0.00         F5V                -0.125-0.113 +.028+023SB1O  148                 *
3992          CD-35 6194  882182011092816  W                100513.1-352158100931.8-355124269.69 16.36 6.13  +0.60 +0.16         F8V                -0.437+0.014 +.048+041           4.7   2.3      *
3993          BD+38 2110  88231 619142817I          4764    100517.3+375341101112.8+372407185.68 54.90 5.85  +1.29              gK3                 -0.021-0.031      +009
3994 41Lam HyaBD-11 2820  88284155785 381I   7671           100542.7-115135101035.3-122115253.01 34.50 3.61  +1.01 +0.92 +0.48   K0IIICN1           -0.202-0.089 +.027+019SB1O < 19: 7.7 112.2AC   3*
3995          CP-65 1248  88323250836      W                100555.3-651932100842.6-654855287.29-08.02 5.28  +0.98 +0.75         K0III              -0.072+0.042 -.014+000           4.2  63.3
3996 18    SexBD-07 2977  88333137395    I          4767    100557.5-075530101055.8-082506249.63 37.33 5.65  +1.30 +1.42         K2III              -0.006-0.044      +000V
3997   Mu 2ChaCP-80  423  88351258561               4744    100603.1-810443100407.6-813357296.93-20.72 6.60  +0.92 +0.53         G6-8III            -0.046+0.047      -000V
3998 34    LeoBD+14 2217  88355 98991        7674           100615.5+135056101138.2+132118225.08 50.28 6.44  +0.46 +0.02 +0.27   F7V                +0.040-0.036      -016           0.8   0.2      *
3999          CP-60 1701  88366250840              S Car    100610.6-610335100921.9-613257284.85-04.51 5.60  +1.60 +1.75 +1.54   M2-3IIIe           -0.094+0.073      +289V?                        *
4000          BD-06 3096  883721374002818                   100618.0-064925101117.8-071900248.70 38.16 6.25  +0.01 +0.01         A2Vn               +0.012-0.014      +013SB
4001          CD-41 5658  88399221832                       100625.3-411310101037.9-414254273.45 11.77 5.98  +1.24 +1.40         K2-3III            +0.050-0.120      +033
4002          CP-68 1034  88473250844      W                100701.7-681129100930.5-684100289.05-10.29 5.81  +0.02 -0.02         A0IV               -0.008+0.010D+.007+010           0.1   0.8AB   3
4003          CD-27 7266  88522178526        7681           100729.1-280644101202.9-283623265.45 22.43 6.28  +0.01 -0.01         A1V                -0.026-0.014      +018V?         0.1   0.1
4004 19    SexBD+05 2301  88547118164                       100736.1+050632101248.3+043653236.56 46.03 5.77  +1.18 +1.11        gK0                 -0.051-0.005      +032V?
4005          BD-18 2870  88595155820                       100753.2-183922101237.8-190913258.87 29.82 6.44  +0.50               F6V                -0.236-0.116      +034
4006          BD+27 1862  88639 81243                       100810.8+273751101349.8+270809203.65 55.08 6.04  +0.85               G3IIIFe-1          -0.018+0.002      +010SB                        *
4007          CP-58 1979  88647237773              V368 Car 100809.6-582005101135.2-584941283.50-02.12 6.40  +1.66 +1.75 +1.52   M3III              -0.051+0.020      -010                          *
4008          BD+60 1246  88651 151292821I         U UMa    100813.7+602852101507.7+595908150.91 47.77 6.25  +1.60               M0III             v+0.015+0.009      -021                          *
4009          CP-57 2781  886612377762819          QY Car   100817.3-573400101146.5-580338283.08-01.48 5.72  -0.08 -0.91 -0.04   B2IVpne           v-0.010+0.012      +031      360                 *
4010          CD-51 4560  88693237784                       100834.5-514012101223.0-520948279.79  3.41 6.16  +1.17               K2IIICNII          -0.041+0.046      +011
4011          CD-26 7752  88699178544                       100843.3-263205101319.4-270144264.63 23.84 6.25  +0.31               A9mA8-F3           -0.056+0.038      +018V                         *
4012          BD+21 2165  88737 81250                       100859.5+213957101429.7+211004213.71 53.85 6.02  +0.56 +0.12         F9V                -0.142-0.082      +017V?     11
4013          CD-32 7158  88742201186                       100859.6-323218101324.8-330155268.58 19.10 6.38  +0.59 +0.09         G1V                -0.360+0.061 +.055+042
4014 22    LMiBD+32 2005  88786 61953                       100921.8+315752101506.3+312805196.10 55.82 6.46R                     G8III              -0.037-0.001      +015V?
4015          CD-39 6222  88809221877      W                100930.5-395102101345.9-402045273.12 13.23 5.90  +1.21 +1.25         K1III              -0.078+0.017      +018V          7.0   4.7
4016          BD+73  489  88815  7098                       100935.7+733425101801.1+730424136.52 39.75 6.40  +0.23 -0.02         F2V                -0.056-0.079      +016SB     42
4017          CD-50 4924  88824237804                       100931.1-504413101322.8-511400279.38  4.27 5.28  +0.25               A7V                -0.041-0.026      +048      181
4018          CP-59 1974  88825237799                       100938.3-592522101301.3-595505284.27-02.92 6.10  -0.08 -0.48 -0.02   B4Ve               -0.006 0.000      -002V?                        *
4019          CD-39 6225  88836221883      W                100940.6-394852101356.6-401838273.13 13.28 6.35  +0.94               G8III              -0.026-0.002      +000           7.0  12.7
4020          CD-51 4578  88842237807      W        4788    100937.7-511538101328.0-514522279.69  3.84 5.78  +0.14               A3IV-V             -0.046+0.004      +018V?         1.5   0.3
4021          BD+71  534  88849  7099        7705A  4802    100947.4+713336101750.6+710338138.45 41.12 6.66  +0.32 +0.20         A7m                -0.031-0.048 +.026+011    =< 25  0.7  16.6AB   3*
4022          CP-61 1517  889072508752822                   101005.6-610947101321.2-613932285.30-04.33 6.41  -0.11 -0.66         B2V                -0.019-0.002      +013SB     55                 *
4023          CD-41 5713  88955221895 382I'                 101032.2-413735101444.2-420719274.33 11.88 3.85  +0.05 +0.06 +0.02   A2V                -0.145+0.045 +.034+007SB     91                 *
4024 23    LMiBD+30 1981  88960 812582823                   101033.6+294832101614.4+291838199.96 55.90 5.35  +0.01 +0.01         A0Vn               -0.075-0.023      +016V     139                 *
4025          CP-65 1273  88981250880                       101040.8-655236101330.6-662223288.01-08.19 5.16  +0.21 +0.18 +0.13   Am                 -0.039+0.004 +.012-015       24                 *
4026 32    UMaBD+65  767  88983 151351262                   101046.5+653626101802.0+650630144.61 45.02 5.82  +0.12 +0.10         A8III              -0.087-0.009      -007V     100
4027 24    LMiBD+29 2021  88986 81259                       101048.1+291058101628.1+284057201.09 55.88 6.49  +0.60 +0.16         G0V                -0.055-0.089      +030    =< 10
4028          BD+18 2338  88987 99032        7704           101049.3+181417101616.1+174425219.37 53.09 6.55R                     A9IV               -0.006+0.001D+.014-008V          0.3   1.1      *
4029          CD-35 6260  89015201211                       101057.8-360114101520.9-363105271.08 16.52 6.19  +1.06 +0.93         K0III              -0.033-0.005      +027
4030 35    LeoBD+24 2207  89010 81260    I'                 101100.1+240001101632.3+233011210.07 54.94 5.97  +0.67               G1.5IV-V           -0.205+0.036 +.034-033SB  =<  6
4031 36Zet LeoBD+24 2209  89025 81265 384I W        4804    101107.7+235457101641.4+232502210.23 54.95 3.44  +0.31 +0.20 +0.19   F0III              +0.018-0.007 +.017-016SB     84  2.3 325.9      *
4032          BD+26 2064  89024 81264                       101106.7+255207101641.9+252217206.90 55.40 5.84  +1.20 +1.22        gK2                 -0.107+0.029      +034V
4033 33Lam UMaBD+43 2005  89021 43268 383                   101104.0+432450101705.8+425452175.87 55.09 3.45  +0.03 +0.06 -0.01   A2IV               -0.164-0.038 +.030+018V      48                 *
4034          BD-10 3029  89033155855                       101113.4-104219101609.1-111212253.23 36.27 6.08  +1.10               K0                 -0.013+0.008      +032V
4035 37    LeoBD+14 2228  89056 990342824I                  101118.7+141337101640.7+134342225.42 51.54 5.41  +1.61 +1.95         M1.5IIIab          -0.023-0.016      +003
4036          CD-42 6074  89062221910                       101119.6-423647101531.5-430645275.02 11.15 5.60  +1.52               K4III              +0.009-0.064      +018
4037   Ome CarCP-69 1178  89080250885 385                   101121.6-693229101344.2-700217290.16-11.18 3.32  -0.08 -0.33 -0.03   B8III             t-0.039+0.007      +007V     225                 *
4038          CP-54 3356  89104237834              Var?     101134.2-542836101516.6-545827281.73  1.34 6.16  -0.17 -0.76         B2IV-V             -0.015 0.000      +008V     104                 *
4039 39    LeoBD+23 2207  89125 81270        7712           101144.5+233628101714.6+230622210.81 55.01 5.82  +0.50 -0.05 +0.27   F8Vb w             -0.408-0.100 +.062+038    =<  6  6.0   7.6      *
4040          BD-19 2964  89169178610        7711           101201.2-201014101645.6-204014260.84 29.29 6.57  +0.48               F3V                -0.127-0.038      -001V          3.5   1.3
4041          BD+28 1867  89239 81278                       101232.4+275454101810.3+272455203.42 56.08 6.52  -0.02 -0.10         A0V                -0.057+0.006      +007       83
4042 22Eps SexBD-07 3001  892541374691263                   101239.6-073410101737.8-080408250.79 38.76 5.24  +0.31 +0.13         F2III              -0.158+0.004 +.028+015V      76                 *
4043          CP-59 2008  89263237853      W                101237.9-592418101603.1-595412284.58-02.69 6.22  +0.20               A5V                -0.047+0.016D+.012+018           0.1   0.4      *
4044          BD+47 1761  89268 43281      W                101248.9+471543101859.0+464539169.34 54.28 6.43                      K1III              -0.021-0.030      -021           5.7  27.8
4045          CD-50 4990  89273237858              GY Vel   101246.2-504221101640.2-511218279.80  4.58 6.30  +1.54       +1.75   M4-5III            -0.028 0.000      +043                          *
4046          BD+49 1940  89319 432852827                   101314.1+485402101926.8+482349166.68 53.77 6.00  +1.02               K0                 -0.097-0.107      -006
4047          BD+69  568  89343 151472828                   101326.1+691501102103.4+684451140.50 42.89 5.96  +0.20 +0.12         A7Vn               -0.053-0.038      +004V     131
4048          BD+25 2231  89344 81285                       101326.5+251246101900.7+244242208.23 55.77 6.40R                     K0                 -0.047-0.008      +000
4049          CD-28 8070  893531786441265          AG Ant   101332.5-282931101807.6-285931266.85 22.93 5.34  +0.24               A0Ib-IIp           -0.010+0.012      -033                          *
4050          CP-60 1817  893882509051264  W       V337 Car 101344.5-604957101705.0-611956285.48-03.80 3.40  +1.54 +1.72 +0.77   K3IIa              -0.024+0.005 +.027+008           9.2  16.4AB   3*
4051          BD+54 1366  89389 27606                       101348.4+541646102014.8+534645158.57 51.58 6.45  +0.54 +0.08         F9V                -0.084+0.046      -021    =<  6
4052          BD+54 1367  89414 276092829                   101403.1+544308102031.2+541301157.93 51.40 6.00  +1.13              gK3                 -0.032-0.002      +009V?
4053          CD-36 6281  89442201266                       101414.0-361814101837.8-364817271.81 16.67 6.30  +1.28               K2-3III            -0.043+0.008      +002V
4054 40    LeoBD+20 2466  89449 99065               4822    101417.6+195842101944.1+192815217.12 54.48 4.79  +0.45 +0.01 +0.23   F6IV               -0.234-0.214 +.052+007V?     16                 *
4055          BD-11 2851  89455155894      W                101422.2-120134101916.8-123141255.04 35.83 6.00  +0.26               A9V                -0.014-0.019      -016      112  1.3   0.6
4056          CD-41 5765  89461221948                       101412.6-411004101828.2-414006274.65 12.65 5.96  -0.06               B9V                -0.027+0.006      +023
4057 41Gam1LeoBD+20 2467  89484 81298    I   7724A  4823    101427.6+202051101958.3+195030216.55 54.65 2.61  +1.15 +1.00 +0.62   K1-IIIbFe-0.5      +0.306-0.147 +.022-037SB   < 17  1.2   4.5AB   4*
4058 41Gam2LeoBD+20 2467  89485 81299    I   7724B  4823    101427.8+202049101958.6+195026216.55 54.65 3.80H                     G7IIIFe-1          +0.312-0.169 +.022-036V          1.2   4.5AB   4*
4059          BD-04 2840  89490137490                       101430.1-043609101932.2-050621248.44 41.16 6.37  +0.90 +0.60         K0                 -0.053-0.061      +016
4060          BD-08 2897  89565137495                       101501.5-083318101959.4-090332252.21 38.47 6.32  +0.33 +0.03         F1IV               -0.067-0.068      +015V      90
4061          CP-55 3220  89569237902                       101459.0-553644101837.6-560636282.76  0.66 5.81  +0.48               F6V                -0.253+0.127      +011
4062          BD+84  234  89571  1701                       101509.2+844537102941.5+841508126.85 31.76 5.50                      F0IV               -0.129-0.032 +.034+004V     105                 *
4063          CP-54 3474  896822379162830          GZ Vel   101551.2-543137101936.8-550146282.28  1.64 4.57  +1.62 +1.82 +0.87   K3II               -0.014+0.003 -.008+013                          *
4064 23    SexBD+03 2352  896881182481266          RS Sex   101552.1+024734102102.0+021723241.02 46.28 6.66  -0.08 -0.67         B2.5IV             -0.007-0.003      +005V                         *
4065          CP-64 1248  89715250917      W                101558.8-641028101904.8-644035287.53-06.46 5.67  +0.05 +0.05         A1V                -0.043+0.007      +010           0.1   2.4      *
4066          CD-47 5790  897362219702831                   101612.0-471147102016.7-474157278.33  7.82 5.65  +1.67               K5-M0III           -0.002+0.001      +016
4067          BD+41 2076  89744 43309                       101613.9+414414102210.5+411346178.39 56.39 5.76  +0.54 +0.08         F7V                -0.124-0.136 +.041-007V       8
4068          BD-17 3129  89747155920                       101619.0-172851102107.8-175906259.77 32.01 6.51  +0.40               F3III-IV           -0.051-0.028      +023V?     34
4069 34Mu  UMaBD+42 2115  89758 43310 386I          4829    101622.4+420009102219.7+412958177.90 56.36 3.05  +1.59 +1.89 +0.96   M0III             e-0.082+0.035 +.035-021SBO                       *
4070 42    LeoBD+15 2192  89774 99080               4828    101627.6+152847102150.3+145832224.49 53.21 6.12  +0.02 -0.02         A1V                -0.035-0.015      +009V      65                 *
4071          BD-22 2904  898161787212832                   101646.3-231227102128.7-234239263.97 27.59 6.50  +0.20 +0.15         A3                 -0.049+0.019      +013
4072          BD+66  664  89822 15163 387           4839    101655.5+660420102407.9+653359143.54 45.22 4.97  -0.06 -0.13 -0.06   A0pSiSr:Hg:        -0.008-0.022 +.043-000SB2O   15                 *
4073          BD-21 3045  89828178723        7739           101652.1-220129102136.0-223142263.18 28.54 6.51  +0.07               A0                 -0.030+0.006D+.006+012           1.5   1.8
4074          CP-55 3286  89890237959      W     A          101711.5-553222102054.8-560235282.99  0.89 4.50  -0.12 -0.58 -0.09   B3IIIe            v-0.016+0.003D+.008+010       81  3.7   7.1AB   3*
4075 27    LMiBD+34 2120  89904 620101267                   101720.8+342447102306.3+335429191.68 57.53 5.90  +0.14 +0.10         A6V                -0.013-0.005      -015V     150
4076          BD-19 2987  89911155935                       101725.9-192144102212.9-195201261.40 30.72 6.13  +0.03               A1V                -0.031-0.001      +011SB2    50                 *
4077 43    LeoBD+07 2289  89962118269                       101746.5+070302102300.4+063233236.37 49.21 6.07  +1.12 +1.14 +0.37E  K3III              -0.018-0.100      -024V                         *
4078          BD+30 2005  89993 81328                       101802.3+300718102341.8+293657199.69 57.54 6.39  +1.09 +0.94         G8III              -0.006-0.010      -013
4079          BD+06 2301  899951182712833  W                101803.0+061205102314.6+054139237.48 48.77 6.54  +0.46 -0.04         F6V                -0.241-0.073 +.007+030       23  2.5  58.6
4080          CD-41 5809  899982219981268           4837    101802.1-410848102219.6-413900275.25 13.07 4.83  +1.12 +1.08 +0.58   K1III              -0.022+0.056 +.024+021
4081 28    LMiBD+34 2123  90040 62019    I                  101823.7+341328102408.6+334307192.02 57.75 5.50  +1.18              gK1                 -0.019-0.003      -022V?
4082 25    SexBD-03 2911  90044137533 388          SS Sex   101823.2-033407102326.5-040427248.33 42.58 5.97  -0.10 -0.17         B9pSiCr:Sr:       v-0.050+0.005      +023                          *
4083          CD-29 8306  90071178759                       101837.9-293923102313.1-300944268.56 22.64 6.27  +0.31 +0.05         F0III              +0.006-0.011      -013V
4084          BD+83  297  90089  1714 911           4864    101854.9+830403103104.6+823331128.03 33.13 5.26  +0.37 -0.05         F2V                -0.083+0.031 +.047+007V     107
4085          BD+03 2358  90125118278        7755           101902.9+025230102413.1+022205241.65 46.96 6.32  +1.00 +0.74         G9V                +0.037-0.023      -014           0.3 212.2AC   3*
4086          CD-37 6509  901322013461269                   101906.5-373008102329.3-380036273.35 16.22 5.33  +0.25               A8V                -0.158-0.057 +.015+017      208
4087          CD-41 5833  90170222016                       101923.8-412650102340.4-415712275.64 12.96 6.27  +0.88 +0.60         G8IV               -0.130+0.011      +022                          *
4088 44    LeoBD+09 2351  902541182862835I         DE Leo   101959.0+091736102515.2+084705233.95 50.92 5.61  +1.62 +1.97         M3IIIabs           +0.014-0.042      -020V                         *
4089          CP-66 1243  902642509402834                   102000.2-662344102258.1-665406289.10-08.11 4.99  -0.13 -0.51         B8V               v-0.024+0.008      +012SB2    79:                *
4090 30    LMiBD+34 2128  90277 62038                       102011.0+341820102554.9+334746191.85 58.11 4.74  +0.25 +0.17 +0.14   F0V                -0.070-0.061      +014V      31
4091          CP-57 3127  90289238021               4846    102011.2-572649102350.9-575714284.35-00.51 6.35  +1.52 +1.78         K4III              -0.001-0.006      -013
4092          BD-06 3146  903621375572836I                  102044.3-063320102544.3-070335251.72 40.89 5.57  +1.52 +1.86 +0.95   K5:IIIbFe-0.5      -0.135+0.136      +032
4093          CD-41 5850  90393222041                       102059.6-415733102517.2-422805276.18 12.69 6.18  +1.00               G8III              -0.002-0.034      -005
4094 42Mu  HyaBD-16 3052  90432155980 389I                  102115.2-161933102605.4-165011259.99 33.65 3.81  +1.48 +1.82 +0.83   K4.5III            -0.128-0.080 +.018+040V    < 19:
4095          CP-57 3164  90454238046                       102121.8-580407102459.4-583435284.81-00.95 5.95  +0.32               F2III              -0.077+0.009      +005
4096          BD+42 2123  90470 43344                       102132.7+420644102728.0+413603177.30 57.27 6.02  +0.17 +0.08         A2V                -0.055-0.076      +007
4097          BD+20 2487  90472 991282837                   102134.2+195225102700.5+192152218.32 56.05 6.15  +1.14               K0                 -0.064-0.012      +032
4098          BD+49 1961  90508 43351      W                102153.6+491908102803.8+484705165.08 54.91 6.44  +0.60 +0.05 +0.24E  F9V                +0.079-0.884 +.053-007V       3  6.0   4.7      *
4099          CD-42 6222  90518222047                       102153.1-421343102609.5-424420276.47 12.55 6.13  +1.13 +1.08         K1III              -0.127-0.052      +023
4100 31Bet LMiBD+37 2080  90537 62053 390I   7780           102206.1+371311102753.0+364226186.28 58.28 4.21  +0.90 +0.64 +0.46   G9IIIab            -0.118-0.101 +.022+006SBO  < 19: 1.7   0.5      *
4101 45    LeoBD+10 2152  90569 99136        7781  CX Leo   102222.0+101620102739.0+094545233.13 51.94 6.04  -0.06 -0.09         A0pSiCr:           +0.007+0.002      -008SBO    26  6.3  37.2     3*
4102          CP-73  733  90589256710 391           4856    102224.7-733122102423.7-740154293.16-14.02 4.00  +0.35 -0.01 +0.19   F2IV               -0.021-0.026 +.086-004SB     55
4103          BD+45 1832  90602 433562838                   102235.0+454323102836.5+451244170.89 56.38 6.35  +1.32 +1.40         K0                 -0.021-0.019      -004
4104   Alp AntCD-30 8465  90610201405 392I          4862    102234.5-303331102709.1-310404269.88 22.39 4.25  +1.45 +1.63 +0.79   K4III              -0.075+0.011 +.024+012SB                        *
4105          CP-73  735  90630256711                       102244.3-732749102444.3-735818293.15-13.96 6.19  +0.07 +0.11         A2-3V              -0.032+0.017      -003V?
4106 35    UMaBD+66  671  90633 15196                       102248.4+660817102954.3+653734142.91 45.63 6.32  +1.14              gK2                  0.000-0.032      -025V?
4107          CP-54 3651  90677238067                       102257.6-542204102649.0-545239283.07  2.32 5.58  +1.56               K3II-III           -0.007-0.008      +006
4108          BD+64  789  90745 152002839                   102329.3+644615103026.6+641527144.32 46.59 6.12  +0.16 +0.12         A7III              -0.047-0.059      -013V     105
4109          BD-03 2929  90763137591              Var?     102339.8-031351102844.0-034433249.25 43.76 6.05  +0.05 +0.04         A1pSr:             +0.004-0.027      -026                          *
4110          CP-57 3256  90772238077              V399 Car 102340.9-570743102724.4-573820284.59  0.01 4.66  +0.51 +0.02 +0.48   A9Ia               -0.013 0.000 -.012-001       42                 *
4111          CD-48 5655  90798222081               4868    102356.2-485338102801.9-492420280.35  7.07 6.10  +1.51 +1.76         K4III              +0.006-0.036      +090                          *
4112 36    UMaBD+56 1459  90839 27670 394                   102413.8+562936103037.6+555850154.29 51.70 4.84  +0.52 -0.01 +0.28   F8V                -0.176-0.030 +.077+009        0  3.4 139. AC   3*
4113 32    LMiBD+39 2357  90840 62076                       102416.3+392613103006.4+385531181.98 58.36 5.77  +0.08 +0.14         A4V                -0.017-0.002      +002SB?    70
4114          CP-58 2227  90853238085 393           4869    102412.4-581343102752.7-584422285.22-00.90 3.82  +0.31 +0.24 +0.24   F2II               -0.013 0.000 +.010+009       22                 *
4115          CP-65 1354  90874250966                       102416.1-651141102725.3-654217288.83-06.86 6.01  +0.09 +0.11         A2V                -0.082+0.023      +006
4116 29Del SexBD-02 3155  908821376001270                   102423.9-021338102928.7-024421248.42 44.59 5.21  -0.06 -0.12         B9.5V              -0.048-0.014      +019V     191
4117          CD-29 8381  90957178888    I                  102451.7-290908102929.0-293949269.46 23.83 5.58  +1.42 +1.72         K5III              -0.059+0.015      -005V
4118   Del AntCD-29 8383  90972201442      W        4876    102459.1-300543102935.4-303626270.06 23.06 5.56  -0.04 -0.18         B9.5V              -0.033 0.000D+.002+014SB2     0  3.9  10.9      *
4119 30Bet SexBD+00 2663  909941376082841          Bet Sex  102510.7-000727103017.5-003813246.42 46.17 5.09  -0.14 -0.52 -0.14   B6V                -0.040-0.023      +012      114                 *
4120          CP-63 1440  91056250979                       102533.0-633938102852.6-641020288.16-05.47 5.29  +1.86 +2.10         M0III              -0.013+0.006 +.010-003                          *
4121          BD+81  343  91075  17352845                   102544.3+810036103601.7+802940129.37 34.87 6.52  +0.95 +0.74        gG4                 -0.027-0.006      -011
4122          BD-06 3173  91106137614    I   7808           102558.3-070728103058.7-073815253.49 41.37 6.20  +1.38 +1.18         K5III+F6V          -0.039+0.005      +007SB         2.3   2.8AB   3*
4123          BD-12 3181  91120156029              Var?     102605.0-130431103059.8-133518258.57 36.91 5.58  -0.04 -0.16 -0.04   B9Vn              e-0.045+0.009      +013SB    310                 *
4124 33    LMiBD+33 1999  91130 62101        7813           102610.8+325334103151.4+322246194.53 59.39 5.90  +0.11 +0.05         A0IV               +0.015+0.006      -015V     125  6.6  43.3      *
4125          CD-25 8084  91135178917                       102610.1-255816103051.4-262902267.74 26.62 6.51  +0.54               F7III              -0.065+0.015      +016
4126          BD+76  393  91190  7164 395I                  102636.2+761342103505.5+754247133.12 38.59 4.84  +0.96 +0.72         G8.5III            -0.030-0.003 +.027+017     < 19:
4127 46    LeoBD+14 2255  91232 991721272I          4883    102651.5+143902103211.8+140814227.68 55.08 5.46  +1.68 +2.04         M1.5IIIbCa-1       -0.042+0.024 -.008+034V?                        *
4128          CP-60 1945  91270250992      W                102705.6-605036103039.2-612122286.87-02.95 6.43  +1.76 +1.84         M2IIIe             -0.003+0.022      -004V          1.2  82.6      *
4129          CP-66 1291  91272250989               4881    102702.4-662817103008.7-665906289.74-07.80 6.19  +0.01 -0.53         B4IV               -0.025-0.008      -008V
4130          CD-27 7503  91280178938                       102709.2-274323103148.6-281415269.04 25.30 6.05  +0.51               F7V                -0.086-0.016      -021V
4131          BD+54 1381  91311 27682                       102727.4+540047103343.6+532951157.30 53.44 6.45R                     A1V                -0.047-0.025      +002      127                 *
4132          BD+41 2101  91312 433792844    7826           102723.9+405625103313.9+402532178.93 58.62 4.75  +0.23 +0.08 +0.09   A7IV               -0.140+0.006 +.031+009SBO   132  7.0  19.3      *
4133 47Rho LeoBD+10 2166  91316118355 396  W       Rho Leo  102732.7+094917103248.7+091824234.89 52.77 3.85  -0.14 -0.96 -0.16   B1Ib               -0.007-0.003 +.011+042SB     61  0.4   0.0      *
4134          CP-53 3909  91324238146      W                102728.6-531226103121.8-534256283.04  3.66 4.89  +0.50       +0.29   F6V                -0.418+0.205 +.050+020           3.5  46.4      *
4135          CD-44 6583  91355222126      W     A          102741.1-443309103157.5-450400278.63 11.12 5.74  -0.15 -0.71         B6II                0.000-0.002      -000           0.4  13.7      *
4136          CD-44 6582  91356222125      W     B          102740.2-443319103156.6-450410278.63 11.12 6.09  -0.11 -0.47         B8II               -0.013+0.006      -009V          0.4  13.7      *
4137 34    LMiBD+35 2154  91365 62121                       102747.9+353014103330.9+345919189.36 59.60 5.58R +0.02 +0.04         A2Vn               -0.029-0.006      +007V     153
4138          CP-71 1034  913752567222842                   102748.7-712842103020.1-715935292.41-12.07 4.74  +0.04 +0.07 +0.02   A1V                +0.025-0.037 +.014+008       20                 *
4139          CD-43 6347  91437222129                       102816.1-440611103233.6-443708278.49 11.56 5.91  +0.92               G6-8III            -0.011-0.032      -010
4140          CP-61 1704  91465251006 397          PP Car   102828.0-611015103201.4-614107287.18-03.15 3.32  -0.09 -0.72 -0.07   B4Vne             v-0.019+0.009      +026      303                 *
4141 37    UMaBD+57 1277  91480 27695 398                   102843.4+573552103509.7+570458152.26 51.57 5.16  +0.34 -0.02 +0.17   F1V                +0.067+0.039 +.036-010V      87                 *
4142          CP-72  981  91496256723      W        4886    102842.1-724226103102.0-731318293.12-13.08 4.93  +1.68 +1.83 +0.93   K4-5III            -0.017-0.001      +011           7.6  32.0      *
4143          CD-46 6205  915042221361273  W                102843.6-462918103256.9-470012279.80  9.55 5.02  +1.04 +0.59         K4III              -0.019+0.003 +.010+004           4.0  40.2      *
4144          CP-58 2285  91533238168                       102902.9-580905103247.6-584001285.73-00.50 6.00  +0.32 -0.07         A2Iab              -0.024 0.000      +010                          *
4145 44    HyaBD-22 2946  91550178979 399I   7834           102915.4-231347103400.9-234443266.62 29.27 5.08  +1.60 +1.82         K4III              -0.008+0.021 +.017-004V?         8.7  19.1
4146 48    LeoBD+07 2330  916121183762846I                  102935.0+072807103448.0+065713238.51 51.84 5.08  +0.94 +0.64 +0.48   G8.5IIIFe-1Ca-1    -0.102+0.056 +.034+005V?   < 19:                *
4147          CP-57 3431  91619238182              V369 Car 102938.5-574028103325.3-581125285.56-00.05 6.14  +0.35 -0.44         B7Iae             v-0.017-0.003      +007       52                 *
4148 49    LeoBD+09 2374  91636118380        7837  TX Leo   102947.3+091002103502.2+083901236.30 52.86 5.67  +0.05 +0.05         A2V                -0.055-0.006D+.005+017SB1O=< 39  2.0   0.0     3*
4149          BD-22 2952  91706178993                       103012.1-223937103457.7-231034266.45 29.86 6.10  +0.50 +0.01         F7V                -0.098+0.036      +012V
4150 35    LMiBD+37 2100  91752 62147                       103037.2+365044103621.4+361937186.58 60.01 6.28  +0.39 -0.03         F3V                +0.036-0.037      -024V       8
4151          CP-60 1983  91767251014               4898    103034.4-602815103412.9-605916287.05-02.42 6.23  +1.40 +1.55         K2-3III            +0.004-0.003      -001                          *
4152          BD-17 3187  91790156087                       103047.8-180307103538.9-183409263.42 33.69 6.49  +0.20               A7V                +0.009+0.008      -007
4153          CD-38 6579  91793201533              U Ant    103046.3-390244103512.9-393346276.22 16.14 5.38  +2.88 +7.1  +1.37   N0:               v-0.027-0.001      +037                          *
4154          CD-43 6395  91805222170      W                103050.3-430854103510.5-433953278.40 12.62 6.08  +0.94 +0.71  0.00   G8II-IIICNVp       -0.030+0.023      +008           8.4  14.7      *
4155          BD-09 3108  91858156092                       103118.8-100352103617.4-103500257.37 40.04 6.57  +0.29               F0V                +0.018-0.033      -022      112
4156   Phi2HyaBD-15 3087  91880156093    I   7847           103123.6-154935103616.7-162040261.94 35.55 6.03  +1.65 +1.86         M1III              -0.026-0.001      +016V          7.0   3.4      *
4157          CD-26 8022  91881179014        7846   4907    103121.5-260919103604.6-264030268.94 27.12 6.29  +0.48               F5V                +0.009-0.062D+.020-023SB         1.0   1.0      *
4158          BD-11 2918  918891560951274  W                103133.6-114136103632.4-121349258.78 38.82 5.70  +0.52  0.00         F7V                +0.268-0.673 +.048-006           5.1  14.4      *
4159          CP-56 3544  91942238222               4904    103144.7-570223103535.3-573328285.49  0.64 4.45  +1.62 +1.79 +0.84   K3-4II             -0.023 0.000 +.026+010SB
4160          BD-10 3094  91992156105                       103214.4-111344103711.6-114455258.56 39.29 6.52  +0.29               F0V                -0.051-0.032      +009      176
4161          CP-81  449  92029258581      W                103226.2-812417103151.0-815516298.01-20.42 7.07  -0.08 -0.51         B5III-IV           -0.033+0.008      +007           2.5  41.9      *
4162          CD-26 8033  920361790412848I                  103232.0-265340103713.7-272445269.64 26.65 4.89  +1.62 +1.95 +0.82E  M2III              -0.109+0.024 +.010+017
4163          BD-12 3218  92055156110    I         U Hya    103236.9-125152103733.2-132304259.97 38.07 4.82  +2.68 +5.78 +1.27   C5II               +0.034-0.032      -025V?                        *
4164          CP-58 2371  92063238242               4909    103235.8-590241103620.3-593353286.57-01.05 5.08  +1.18 +1.20         K1III              -0.059-0.053 +.038-013
4165          BD+54 1387  92095 277242849I                  103253.6+541125103905.7+534006156.32 54.01 5.52  +1.27 +1.34         K3III              -0.087-0.074      +045V?
4166 37    LMiBD+32 2061  92125 621731275I                  103305.6+322945103843.2+315834195.30 60.85 4.71  +0.81 +0.54 +0.38   G2.5IIa             0.000+0.008 +.021-007V    < 19:
4167          CD-47 6042  92139222199    I'W                103305.8-474222103718.1-481333281.06  8.87 3.84  +0.30 +0.07 +0.16   F4IV+F3            -0.155-0.017 +.040+019SB2O    0  0.9   0.3      *
4168 38    LMiBD+38 2166  92168 621782852                   103324.6+382553103907.6+375436183.25 60.29 5.85  +0.57 +0.17         F9V                -0.222-0.044 +.038+007SBO                       *
4169          CP-58 2411  92207238271              V370 Car 103338.3-581250103726.8-584400286.29-00.26 5.45  +0.50 -0.24         A0Iae             v-0.033+0.002      -009SB     56                 *
4170          CP-75  678  922092567302847                   103334.0-754726103524.8-761833295.03-15.57 6.30  +1.20 +1.27         K2III              -0.018+0.004      +018V
4171   Phi3HyaBD-16 3100  922141561222850I                  103342.5-162126103835.0-165236262.88 35.46 4.91  +0.92 +0.68         G8IIIFe-0.5        -0.097+0.027 +.014+018SB1O < 17                 *
4172          BD-11 2925  92245156124                       103353.7-115525103850.4-122637259.53 39.00 6.04   0.00               A0Vn               -0.057+0.002      +012      210
4173          CP-56 3588  92287238278                       103409.1-564411103802.5-571523285.63  1.07 5.91  -0.14 -0.63         B3IV               -0.028+0.002      -009SB                        *
4174   Gam ChaCP-77  622  92305256731 401I'         4913    103417.3-780521103528.1-783628296.28-17.53 4.11  +1.58 +1.95 +0.97   M0III              -0.042+0.014 +.009-022
4175          CD-42 6390  923282222222853  W                103426.7-421358103850.3-424513278.52 13.74 6.11  +0.66 +0.43         Am+Am:             -0.016-0.016      +016           0.1   0.2
4176          BD+69  583  92354 15260    I                  103441.7+685757104148.3+682636138.94 44.44 5.75  +1.30              gK3                 -0.026-0.026 -.000+005V?
4177          CP-58 2460  92397238295      W        4922    103456.5-583945103845.1-591059286.65-00.57 4.66  +1.48 +0.97 +0.90   K4-5III:           -0.012+0.001 +.016+011           3.4  14.6      *
4178 38    UMaBD+66  678  92424 152612855I          4936    103507.5+661425104156.6+654259141.57 46.45 5.12  +1.20 +1.28 +0.57   K2III             v-0.162-0.069 +.026-011SB   < 19:
4179          CP-58 2474  92436238304      W                103510.5-581747103859.4-584901286.50-00.24 5.92  +1.65 +1.96 +1.22   M1III              -0.086+0.012      +081           2.7  23.5      *
4180          CP-54 3915  92449238309 402  W                103519.5-550456103918.4-553612284.98  2.59 4.28  +1.04 +0.75 +0.47   G2-3Ib             -0.019+0.005 +.023+020           2.2  51.8AB   3*
4181          BD+69  586  92523 15269 403I                  103554.7+693558104304.1+690434138.24 44.04 5.00  +1.38 +1.54 +0.73   K3III-IIIb         +0.002-0.012 +.005-000     < 19:
4182 33    SexBD-00 2364  92588137728 404                   103618.9-011257104124.2-014430250.38 47.42 6.26  +0.88 +0.59 +0.45   K1IV               -0.137-0.121 +.026+043V?
4183          CD-35 6646  925892016312856  W                103617.5-351311104051.6-354430275.16 19.99 6.37  +0.92               G8-K0III+F-G       -0.021+0.001      +011V          2.5   0.6
4184          BD+32 2066  92620 62206    I W       RX LMi   103635.2+321313104211.3+314149195.85 61.59 6.02  +1.62 +1.83 +1.13E  M4III              +0.008-0.021      +016V          4.0 112.5      *
4185          CP-64 1403  92664251059              V364 Car 103643.5-643442104011.3-650602289.68-05.66 5.52  -0.16 -0.57         B9pSi              -0.027+0.003      +030       66                 *
4186          CP-73  758  92682256742                       103654.8-735818103916.6-742937294.30-13.88 6.07  +1.71 +1.91         K3II                0.000-0.006      -004
4187 39    UMaBD+57 1286  92728 27748                       103724.6+574328104343.3+571157150.95 52.42 5.80  -0.02 -0.09         A0V s              +0.017-0.053      -015V?     27
4188          CP-59 2450  92740238353               4939    103728.2-590915104117.6-594037287.17-00.85 6.42  +0.08 -0.82         WN7-A              -0.006-0.001      +033SBO                       *
4189 40    LMiBD+27 1927  92769 81485        7899           103732.4+265103104301.8+261932207.12 61.40 5.51  +0.17 +0.09         A4Vn               -0.105-0.063      +010      162  6.3  18.4AB   4*
4190          BD-13 3197  92770156170    I                  103735.2-132705104231.3-135830261.64 38.35 6.24  +1.54               K2                 -0.020-0.002      -009V
4191          BD+46 1657  92787 434441276  W                103740.4+464346104332.9+461214167.29 58.39 5.18  +0.33 +0.01         F5III              -0.269-0.066 +.021+004SB     52  2.8 286.9AB   5*
4192 41    LMiBD+23 2253  92825 81490 405                   103758.7+234243104325.0+231118213.52 60.84 5.08  +0.04 +0.05         A3Vn               -0.116+0.009 +.016+019V     154
4193 35    SexBD+05 2384  92841118449        7902A          103809.4+051621104320.9+044452243.46 52.16 5.79  +1.17 +1.09         K3III+K0III        +0.022-0.030D+.003-006SB         1.1   6.7AB   3*
4194          CD-32 7572  928452016651277  W                103805.0-321132104243.2-324257273.85 22.79 5.64   0.00  0.00         A0V                -0.029+0.002      +004      131  8.2  17.4
4195          BD+68  617  92839 15274    I         VY UMa   103807.4+675609104504.0+672441139.59 45.41 6.00  +2.39 +4.62 +1.27   C5II               +0.005-0.001      -005                          *
4196          CP-63 1589  92938251078                       103841.1-635635104214.0-642759289.56-05.00 4.82  -0.14 -0.58         B4V                -0.026+0.018      +024      138                 *
4197          BD+20 2514  92941 81496                       103851.6+201702104414.5+194531220.27 60.00 6.27  +0.17               A5V                -0.116-0.026      +005SB    168
4198          CP-58 2581  92964238379               4948    103848.6-584131104240.6-591257287.11-00.36 5.38  +0.26 -0.65 +0.36   B2.5Iae           v-0.009 0.000      -002       68                 *
4199   The CarCP-63 1599  93030251083 406I                  103923.3-635214104257.4-642340289.60-04.90 2.76  -0.22 -1.01 -0.23   B0Vp              v-0.023+0.010      +024SB    151                 *
4200          CP-59 2532  93070251090               4951    103943.8-600232104332.1-603400287.84-01.50 4.57  +1.71 +1.87 +0.99   K4III              -0.032+0.006 +.020+009SB                        *
4201 36    SexBD+03 2408  93102118473      D                104000.3+030050104509.4+022917246.68 51.02 6.28  +1.21 +1.28         K4III              -0.051-0.023      +011V          0.0   0.0      *
4202 41    UMaBD+58 1281  93132 27760    I                  104006.4+575336104622.5+572157150.36 52.58 6.34  +1.56 +1.90         M1III              -0.050-0.064      -002                          *
4203 42    LMiBD+31 2180  93152 62236 407  W                104018.3+311233104551.9+304056198.01 62.38 5.24  -0.06 -0.14         A1Vn               -0.023-0.037      +012SB    125  2.8 198.1
4204          CP-63 1619  93163251095              Var?     104015.4-634327104351.2-641456289.61-04.73 5.77   0.00 -0.57 -0.03   B2.5V              -0.014+0.006      +011       12                 *
4205          CP-63 1623  93194251096                       104029.7-632610104406.9-635740289.50-04.46 4.82  -0.13 -0.62 -0.13   B5Vn               -0.019+0.013      +026      260                 *
4206          CP-79  548  93237256745                       104048.2-791533104151.3-794700297.18-18.39 5.97  -0.07 -0.51         B4IVe              -0.031+0.010      +018V                         *
4207          BD+07 2356  932441184831278                   104053.3+065401104605.7+062223242.08 53.72 6.37  +1.12 +1.12        gK1                 -0.008-0.036      -009V                         *
4208 51    LeoBD+19 2371  93257 992811279I                  104101.3+192508104624.5+185329222.26 60.16 5.49  +1.12              gK3                 +0.097-0.038      -006
4209 52    LeoBD+14 2294  93291 992822860                   104107.5+144322104625.3+141141230.54 58.15 5.48  +0.91 +0.59        gG4                 -0.123-0.066      +035V
4210   Eta CarCP-59 2620  93308238429    I W       Eta Car  104110.8-590931104503.6-594103287.60-00.63 6.21  +0.61 -0.45 +0.49   pec                -0.004+0.009      -025           0.1   0.2AP  15*
4211          CP-70 1183  93344256750      W     A          104118.8-702002104419.4-705136292.82-10.53 6.26  +0.20 +0.17         A5IV-V             -0.075-0.007      +010           0.2  63.0AB   3*
4212          CP-70 1185  93359256751      W     B          104130.9-701946104432.1-705118292.83-10.51 6.46  +0.23 +0.14         A6IV               -0.060+0.007      +015           0.2  63.0AB   3*
4213          CP-71 1118  93372256752                       104139.9-715509104426.5-722638293.60-11.91 6.27  +0.49 +0.01         F6V                -0.175+0.040      +019
4214          BD-16 3124  93397156221                       104158.0-164610104652.0-171748265.16 36.28 5.42  +0.11 +0.14 +0.04   A3V                -0.020-0.013      -014V?
4215          BD+65  803  93427 15298                       104209.2+653936104850.0+650756141.38 47.36 6.39  -0.02 -0.04         A1V                +0.010 0.000      -005V     170
4216   Mu  VelCD-48 5913  93497222321    I'W                104228.0-485330104646.2-492512283.03  8.57 2.69  +0.90 +0.57 +0.49   G5III+G2V          +0.074-0.048D+.022+006SB         3.7   1.7      *
4217          CP-59 2671  93502251113      W                104226.3-600435104616.8-603612288.16-01.37 6.25  +0.04 +0.03         A0IV               -0.062-0.008      +008SB2        7.8   4.2      *
4218          BD-14 3186  93526156235        7930           104242.1-144403104738.0-151543263.88 38.05 6.67  -0.01               A0III              +0.002-0.014D+.012+022SB         1.2  74.7AC   3
4219          CP-63 1646  93540251115                       104239.1-635916104616.5-643054289.97-04.84 5.34  -0.10 -0.45         B6V               e-0.025-0.005      +032      228                 *
4220          CP-63 1649  93549251117      W                104250.9-634411104629.7-641548289.88-04.61 5.23  -0.07 -0.47         B7IV               -0.008+0.012      +012V     201  0.1   0.1      *
4221          CP-56 3800  93563238468              Var?     104255.3-561348104657.5-564526286.46  2.08 5.23  -0.08 -0.33         B8-9IIIe          v-0.016-0.002      +031      218                 *
4222          CP-63 1655  93607251120                       104312.6-635123104651.2-642300289.97-04.69 4.85  -0.15 -0.65 -0.18   B3IV               -0.023+0.012      +016      202                 *
4223 43    LMiBD+30 2072  93636 81533                       104326.3+295643104857.2+292457200.81 63.01 6.15  +1.15 +1.06       sgK1                 -0.086-0.041 +.038+010
4224          BD-01 2446  93655137800    I                  104334.5-012551104840.6-015732252.55 48.53 5.93  +1.60 +1.94         M2III              -0.012+0.008      +003
4225          CD-31 8536  93657201766                       104332.9-310935104814.2-314118274.38 24.27 5.88  +0.03 +0.04         A1V                -0.020-0.013      +021V
4226          CP-56 3821  93662238480               4975    104337.4-565623104738.7-572804286.87  1.49 6.36  +1.62 +1.86         M1II               +0.010-0.004      +002                          *
4227 53    LeoBD+11 2283  93702 99305 409                   104400.0+110428104915.4+103243237.00 56.82 5.34  +0.03 +0.02         A2V                -0.003-0.025 +.007-007SB    191
4228          CP-59 2720  937372384932862                   104410.4-592328104805.4-595509288.04-00.66 6.00  +0.27 -0.30         A0Ia-Iab           -0.001+0.006      -024                          *
4229 40    SexBD-03 2999  93742137808        7936           104413.0-032943104917.3-040127254.80 47.11 6.61  +0.22 +0.08         A2IV               -0.049-0.012D+.011+014           0.7   2.2
4230 44    LMiBD+28 1931  93765 815422863                   104424.0+283007104953.7+275826204.02 63.12 6.04  +0.37 -0.01         F3V                -0.011+0.031      +003V?     22
4231   Del1ChaCP-79  554  93779258592      W                104419.3-795630104515.7-802811297.68-18.91 5.47  +0.95 +0.75         K0III              -0.038-0.033D+.003+011           0.3   0.6
4232   Nu  HyaBD-15 3138  93813156256 410I                  104441.4-154013104937.5-161137265.05 37.56 3.11  +1.25 +1.30 +0.64   K2III              +0.094+0.200 +.028-001     < 17
4233          BD-09 3147  93833137815                       104443.1-091922104943.5-095110260.17 42.69 5.86  +1.07 +0.91        gG8                 +0.009-0.037      +040V?
4234   Del2ChaCP-79  556  93845258593 411                   104451.0-800046104546.8-803225297.74-18.96 4.45  -0.19 -0.70 -0.20   B2.5IV             -0.049+0.008      +023       55                 *
4235 43    UMaBD+57 1294  93859 27791                       104501.3+570642105111.1+563456150.63 53.59 5.67  +1.12               K2III              -0.058+0.003      +015
4236 42    UMaBD+60 1296  93875 277932865I                  104506.6+595104105123.7+591912147.28 51.76 5.58  +1.14 +1.15         K2III              -0.036-0.047      -017SB
4237 41    SexBD-08 3018  939031378231281    7942           104517.0-082205105018.1-085352259.52 43.53 5.79  +0.16 +0.13 +0.07   A3m                -0.004-0.015      -004SB1O   20  5.8  27.3AB   3*
4238          CD-33 7288  93905201805                       104517.7-333145104957.0-340329276.00 22.38 5.61  +0.04 +0.05         A1V                -0.044+0.015      -012V?
4239          CP-58 2755  93943238514      W                104526.7-584740104924.1-591925287.92-00.06 5.91   0.00 -0.09         B9.5IV-V           -0.043+0.001D+.010+004           1.2   1.1
4240          BD-02 3236  94014137834    I                  104600.4-023344105105.4-030533254.38 48.10 5.95  +1.48 +1.83         K2                 -0.046-0.006      +037
4241          BD+53 1439  94083 27809                       104630.3+530548105230.8+523355155.83 56.29 6.65R                    gG8                 -0.020-0.023      -008V
4242          BD+53 1440  94084 27810                       104632.1+530208105231.9+523013155.91 56.33 6.44  +1.11 +1.01 +0.53   K2III              -0.071-0.052      -003SB
4243          BD+70  634  94132  72292866  W                104640.1+702313105330.7+695114136.56 44.02 5.93  +0.99 +0.86         G9IV               -0.398-0.067 +.029+015V          3.0  75.1AB   4
4244          BD+01 2495  94180118550      W                104705.4+013321105213.7+010131250.31 51.29 6.38  +0.08 +0.13         A3V                -0.002+0.003      -013V?     75  5.8  39.2
4245          BD+00 2710  94237118555    I                  104728.7+001948105236.1-001205251.78 50.48 6.31  +1.50 +1.82 +0.62E  K5III              -0.009-0.018      +009                          *
4246 44    UMaBD+55 1418  94247 278152869I                  104731.2+550700105334.5+543506152.85 55.15 5.10  +1.36 +1.52 +0.47E  K3III              -0.064-0.011 +.016+001     < 19:                *
4247 46    LMiBD+34 2172  94264 62297 412I          4999    104743.2+344514105318.7+341254190.00 63.73 3.83  +1.04 +0.91 +0.54   K0+III-IV          +0.087-0.278 +.024+016V    < 19:                *
4248 45Ome UMaBD+43 2058  94334 435122870                   104813.4+434321105358.7+431124171.16 61.35 4.71  -0.05 -0.05 -0.04   A1V s              +0.045-0.021 +.014-017SBO    35                 *
4249          BD-01 2459  94363137863        7967           104819.8-014316105324.9-021519254.18 49.12 6.12  +0.90 +0.61         K0III+G0V          -0.138-0.083      +026SB         2.6  35.2      *
4250          CP-56 3947  943672385572868                   104825.0-564233105230.9-571426287.36  1.99 5.25  +0.16 -0.54         B9Ia              e-0.004+0.002      -024       41                 *
4251          BD-19 3125  94388156301      W        5006    104835.9-193601105329.5-200820268.69 34.78 5.24  +0.47 +0.06         F6V               v+0.078-0.239 +.054-005V      14  4.0 124.5AB   4*
4252          BD-14 3213  94386156302                       104835.8-145445105332.9-152644265.50 38.71 6.38  +1.16               K3IV               +0.067-0.043      +000V
4253          BD-01 2460  94402137871      W                104838.2-013552105343.7-020745254.14 49.26 5.45  +0.96 +0.77 +0.33E  G8III              -0.084+0.013      +015V?                        *
4254 48    LMiBD+26 2147  94480 81576                       104916.3+260123105442.2+252927209.85 63.85 6.20  +0.28 +0.12 +0.16   A8V                -0.054+0.003      +008V     140                 *
4255          BD-12 3293  944811563102871                   104919.8-131333105417.8-134529264.46 40.21 5.66  +0.83               G4III              -0.005+0.009      +005
4256          BD+34 2178  94497 62310                       104924.7+343407105458.2+340205190.31 64.10 5.72  +1.01              gG7                 -0.060-0.049      -028V?                        *
4257          CP-58 2834  94510238574      W        5011    104925.8-581919105329.6-585112288.18  0.60 3.78  +0.95 +0.65 +0.49   K1III              +0.067+0.030 +.059+008SB         2.7 153.9AB   3*
4258 46    UMaBD+34 2181  94600 62314    I                  105012.1+340227105544.4+333025191.47 64.32 5.03  +1.10 +1.00 +0.55   K1III              -0.109-0.027      -022V?   < 19:
4259 54    LeoBD+25 2314  94601 81583        7979A          105012.0+251659105536.8+244459211.59 63.91 4.50  +0.01 +0.02 -0.01   A1V                -0.072-0.012D+.015+005V     182  1.9   6.5      *
4260 54    LeoBD+25 2314  94602 81584        7979B          105012.4+251657105537.3+244456211.60 63.91 6.30H                     A2Vn               -0.074-0.026D+.015-001SB    233  1.9   6.5      *
4261          BD-19 3134  946191793342872                   105018.7-200755105511.6-203954269.45 34.54 6.44  +1.10               K0                 -0.010+0.004      -012V?
4262          CP-70 1246  94650256770      W                105025.2-701115105342.0-704313293.45-10.05 5.99  -0.02 -0.46         B6V               e-0.034-0.006D+.004+006           0.6   1.5      *
4263          CD-41 6220  94660222422              KQ Vel   105028.8-414305105501.0-421504280.98 15.60 6.11  -0.08 -0.25         A0pSiCr            -0.037-0.001      +022V                         *
4264          BD+42 2162  94669 43535                       105032.2+423239105614.5+420030173.09 62.21 6.03  +1.13 +1.08 +0.42E  K2III              +0.014-0.091 +.027-054                          *
4265 55    LeoBD+01 2501  94672118574        7982           105033.7+011613105542.4+004413251.63 51.69 5.91  +0.42 -0.02         F2/3III/V          +0.106 0.000 +.014-002SB         2.0   0.0     3*
4266          CP-61 1960  94683251173      W                105033.8-611737105429.6-614936289.59-02.02 5.93  +1.75 +1.97         K4III              +0.003-0.004      -003           5.0   7.1
4267 56    LeoBD+06 2369  94705118576    I         VY Leo   105049.9+064309105601.5+061107245.05 55.50 5.81  +1.45 +1.00 +2.09   M5.5III           v-0.022-0.008 -.003-013V?                        *
4268          CP-78  589  94717256768                       105049.3-790138105227.5-793334297.51-17.96 6.33  +1.46 +1.57         K2II-III           -0.040+0.002      +003
4269          BD+23 2279  94720 815892873I                  105053.8+225306105616.9+222106216.93 63.48 6.14  +1.55 +1.94         K2                 -0.037+0.017      +025
4270 50    LMiBD+26 2152  94747 81591                       105108.9+260203105634.4+253000210.00 64.26 6.35  +1.03 +0.89         K0                 -0.025-0.015      +030
4271          CP-59 2840  94776251178              T Car    105117.4-595909105517.2-603101289.11-00.80 5.92  +1.08 +0.88         K0III              -0.036+0.084      -026                          *
4272          BD+78  367  94860  7255 413                   105157.7+781821105956.8+774612130.13 37.76 6.20  +0.97               G9III              -0.072-0.023      -050V?
4273   Iot AntCD-36 6808  94890201927 414                   105203.3-363600105643.1-370816278.85 20.31 4.60  +1.03 +0.84 +0.53   K1III              +0.077-0.128 +.025-000
4274          CD-50 5534  949852386222874          IW Vel   105246.2-501351105707.9-504554285.14  8.11 5.91  +0.18               A4V                -0.031+0.013      +016                          *
4275          BD+52 1528  95057 278582876           5038    105323.5+522504105917.9+515256155.64 57.53 6.17  +1.38 +1.59         K0                 -0.012-0.003      -007                          *
4276          CP-59 2888  95109238635              U Car    105343.5-591150105748.4-594355289.06  0.04 6.11  +1.10 +1.10         G3Ia               +0.007+0.016      +000                          *
4277 47    UMaBD+41 2147  95128 435571282                   105352.0+405752105928.0+402549175.78 63.37 5.05  +0.61 +0.13 +0.31   G1-VFe-0.5         -0.317+0.057 +.072+013V?  =<  3
4278          BD+36 2139  95129 62345    I                  105358.0+363749105932.8+360535185.24 64.66 6.00  +1.59 +1.92 +1.29   M2III              +0.071-0.049 +.018-026V
4279          CP-74  755  952082567752875                   105424.9-743353105715.7-750559295.66-13.86 6.13  +1.53 +1.64         K1II               -0.017+0.003      +008
4280          BD+46 1680  95212 435622878I                  105430.2+460344110014.7+453134165.76 61.27 5.47  +1.47               K5III              +0.011+0.003      +009
4281          BD+12 2284  95216 993922877                   105427.6+121426105941.1+114221237.93 59.59 6.55  +0.43 -0.02         F5V                -0.228+0.044      +020        8
4282          CD-33 7401  95221201976      W        5041    105430.4-331201105913.9-334414277.67 23.57 5.71  +0.37               F2V                +0.016-0.043      -010           3.0   1.8
4283          BD+52 1529  95233 27868                       105432.8+520219110025.6+513007155.99 57.90 6.43                      G9III              -0.029-0.015      +000
4284          BD-15 3174  95234156372    I                  105434.3-154903105930.9-162114267.68 38.74 5.89  +1.63 +1.91         M2III              -0.046-0.016      -033V?
4285          BD+43 2068  95241 43564      W                105440.5+432705110020.6+425441170.63 62.52 6.02  +0.57 +0.02         F9V                -0.109-0.132 +.019-006V       6  5.6  37.1
4286          BD+64  824  95256 15360                       105446.8+635732110105.8+632516141.55 49.53 6.39  +0.16 +0.14 +0.04   A2Vm               -0.040-0.048      +007       50
4287  7Alp CrtBD-17 3273  952721563751283I                  105454.1-174558105946.5-181756269.06 37.12 4.08  +1.09 +1.00 +0.55   K0+III             -0.460+0.130 +.031+047V    < 19:                *
4288 49    UMaBD+39 2400  95310 62354                       105514.2+394457110050.4+391244178.15 64.04 5.08  +0.24 +0.17 +0.11   F0V s              -0.072-0.019      +003       72                 *
4289          BD-13 3271  95314156382    I                  105513.2-133246110011.6-140500266.26 40.74 5.88  +1.50 +1.82         K5III              -0.018-0.022      -006
4290          CP-60 2433  95324251205      W                105512.4-604704105914.0-611913289.89-01.32 6.16  -0.06 -0.29         B8IV               -0.031+0.019      +016           3.5   4.1
4291 58    LeoBD+04 2407  953451186101284I D                105523.7+040916110033.6+033703249.69 54.59 4.84  +1.16 +1.12 +0.56   K1IIICN-0.5        +0.014-0.016 +.007+006V?   < 19: 0.0   0.1      *
4292          CD-43 6692  95347222485                       105526.7-431615105959.4-434826282.54 14.61 5.81  -0.08               B8-9V              -0.058+0.011      -005
4293          CD-41 6276  95370222487 415                   105533.8-414122110009.3-421333281.86 16.05 4.39  +0.11 +0.12 +0.07   A3IV               +0.028+0.003 +.003-005      111
4294 59    LeoBD+06 2384  95382118615        8019           105533.8+063819110044.8+060605246.54 56.32 4.99  +0.16 +0.11         A5III              -0.048-0.025 +.035-012       71  7.5  44.8      *
4295 48Bet UMaBD+57 1302  95418 27876 416I          5053    105548.6+565507110150.5+562257149.17 54.80 2.37  -0.02 +0.01 -0.04   A1V                +0.082+0.034 +.053-012SB     39                 *
4296          CD-51 5220  95429238674      W                105546.6-511652110008.6-514904286.02  7.36 6.15  +0.18               A3III-IV           -0.038+0.004      +014           7.1  10.3
4297          BD-15 3178  95441156396                       105559.3-151517110057.2-154734267.67 39.40 6.34  +1.18               K0                 +0.055-0.040      -005V?
4298          CD-31 8696  95456201998                       105555.9-311820110040.8-315022277.02 25.40 6.07  +0.52 +0.09         F8V                -0.080+0.105      -002
4299 61    LeoBD-01 2471  955781379472879I          5059    105643.6-015646110149.7-022905256.87 50.32 4.74  +1.62 +1.95 +0.96   M0IIIBa0.2         +0.012-0.038 +.032-014V    < 19:
4300 60    LeoBD+20 2547  95608 816372880                   105659.5+204259110219.8+201047222.61 64.13 4.42  +0.05 +0.05 -0.02   A1m                -0.006+0.042 +.017-010       24
4301 50Alp UMaBD+62 1161  95689 15384 417I   8035   5070    105733.6+621727110343.7+614503142.85 51.01 1.79  +1.07 +0.92 +0.58   K0IIIa            e-0.119-0.067 +.038-009SBO  < 17  3.0   0.5      *
4302          CD-26 8302  95698179456        8028           105733.9-261725110224.4-264953274.77 30.01 6.23  +0.31 +0.09         F1V                +0.065-0.120      +001           0.1   0.2      *
4303          BD+00 2728  95771137963      D                105807.6-001240110314.6-004509255.51 51.87 6.14  +0.26 +0.06         F0V                -0.013-0.111      +004                          *
4304          CP-80  509  95788258599      W                105803.3-810116105912.9-813322298.73-19.62 6.71  +0.55 +0.04         F7IV               -0.160+0.073D+.010+011           0.1   0.4
4305          BD-10 3184  95808156421        8037           105814.7-104544110314.9-111813265.03 43.47 5.50  +0.94               G7-III             -0.070-0.108      -008V          5.9   3.7      *
4306 62    LeoBD+00 2729  95849118634                       105829.5+003215110336.6-000003254.81 52.49 5.95  +1.22 +1.32         K3III              -0.057+0.001      -008V
4307          CD-31 8726  958572020392881I                  105830.0-312518110316.1-315739277.62 25.54 6.46  +1.62 +1.98         M1III              -0.039-0.031      +080
4308          BD-12 3333  95870156427                       105837.1-125346110336.5-132604266.72 41.73 6.34  +0.88 +0.60         G5                 +0.003+0.010      -010V
4309 51    UMaBD+39 2414  95934 623872882    8046           105857.7+384649110431.2+381429179.75 65.03 6.00  +0.16 +0.10         A3III-IV           -0.064-0.003      +007V?         7.1   8.4AB   3*
4310 63Chi LeoBD+08 2455  96097118648 418  W        5079    105951.5+075236110501.0+072010246.14 57.94 4.63  +0.33 +0.08 +0.17   F2III-IV          v-0.341-0.047 +.029+005V      25  4.3 276.4AC   4
4311          CD-47 6466  961132225382884                   110001.2-470826110431.2-474045284.94 11.42 5.67  +0.24               A8III-IV           -0.110+0.027      -016      202
4312   Eta OctCP-83  386  961242586001664                   110001.3-840321105913.8-843538300.19-22.33 6.19  +0.11 +0.12         A1V                -0.068-0.005      -003
4313          CD-35 6954  961462020672885                   110010.7-351555110454.2-354817279.80 22.24 5.43  +0.03               A2V                -0.017+0.004      +011
4314   Chi1HyaCD-26 8338  96202179514 419  W                110030.7-264514110519.9-271737275.69 29.91 4.94  +0.36 +0.04         F3IV               -0.188-0.004 +.043+017V?    186  0.1   0.2      *
4315          BD-10 3190  962201564481286                   110032.8-103249110534.0-110520265.52 43.96 6.09  +0.30               F0Vn               +0.013-0.079      -020
4316          CD-48 6157  96224222548                       110035.2-485111110504.2-492333285.73  9.89 6.13  -0.02 -0.06         B9.5V              -0.023+0.004      +004
4317   Chi2HyaCD-26 8342  96314179522              Chi2 Hya 110106.5-264451110557.6-271716275.82 29.97 5.71  -0.06 -0.26         A0IV               +0.034-0.014      +030SBO   193                 *
4318          CD-50 5686  96407238763                       110138.3-504014110605.8-511245286.62  8.29 6.30  +0.95 +0.59         G8III              +0.005-0.065      +053
4319 65    LeoBD+02 2387  964361186681287    8060   5090    110148.1+022954110654.2+015720253.65 54.50 5.52  +0.97 +0.66         G9IIICN-1          -0.383-0.086 +.035+055           3.8   2.2      *
4320          CD-28 8657  96441179542                       110149.2-281111110638.7-284340276.73 28.76 6.77  +0.04               A1V                -0.057-0.032      +018
4321          CD-50 5693  96484238775                       110200.0-502459110627.4-505724286.57  8.55 6.32  +1.16 +1.22?        K2III              -0.067+0.006      +009
4322 64    LeoBD+24 2318  96528 81681                       110218.5+235152110739.7+231925216.37 66.27 6.46  +0.16 +0.12 +0.09   A5m                -0.014+0.008      -002SBO                       *
4323          CP-58 3112  96544238779      W        5091    110214.3-580804110629.3-584031289.64  1.46 6.02  +1.24               K2II-III           -0.015 0.000      +003           5.9  12. AB   3*
4324          CD-31 8776  96557202100                       110221.8-320242110708.4-323514278.74 25.34 6.59  +0.34 +0.03         F1III              -0.081-0.052      -002
4325          CP-61 2067  96566251269                       110226.3-615301110632.4-622527291.12-01.98 4.61  +1.03 +0.81 +0.45   G8III              -0.043+0.013 +.055-002
4326          CP-64 1630  96568251267                       110225.1-641757110624.2-645023292.06-04.21 6.41  +0.12 +0.13         A3V                -0.050+0.008      -006V?
4327          CD-41 6343  96616222581      W       V815 Cen 110239.0-420556110716.6-423819283.28 16.22 5.15  +0.03               ApSrCrEu           -0.096+0.043 -.001+002       56  2.5   1.4      *
4328          CD-29 8875  96700179558                       110309.0-293747110754.4-301029277.74 27.59 6.54  +0.60 +0.08         G2V                -0.513-0.138 +.049+013
4329          CP-70 1305  967062568001288  W       Var      110313.2-702013110649.9-705241294.51-09.74 5.57  -0.05 -0.63         B2V                -0.027 0.000      +007       62  8.2  15.       *
4330          BD+68  632  96707 15414              Var?     110319.4+674509110939.9+671237137.06 47.03 6.06  +0.22 +0.13         F0pSr             v-0.083-0.025      +005       45                 *
4331          CD-29 8877  96723179568                       110326.2-292551110815.8-295822277.71 27.80 6.49  +0.03 +0.05         A1V                +0.008-0.020      -013V?
4332 67    LeoBD+25 2344  96738 816922889    8071           110327.2+251159110849.1+243930213.26 66.83 5.68  +0.06 +0.12         A3IV               +0.003+0.004      -006V          6.6   4.8
4333          BD+37 2162  96813 624272892I         CO UMa   110349.1+365106110919.1+361834183.56 66.52 5.74  +1.51 +1.50 +1.32E  M3.5IIIab          -0.046-0.024      +022                          *
4334          CD-27 7886  96819179577               5101    110353.5-273218110843.9-280450276.87 29.54 5.44  +0.07 +0.06         A1V                -0.080-0.016 +.011+016V?    227
4335 52Psi UMaBD+45 1897  96833 43629 420I                  110402.6+450228110939.8+442955165.80 63.23 3.01  +1.14 +1.11 +0.57   K1III              -0.065-0.028      -004V?   < 19:
4336          BD+43 2083  96834 43627    I          5107    110402.6+434459110938.5+431227168.30 63.89 5.89  +1.57 +1.90 +0.97E  M2III              -0.064-0.011      +018                          *
4337          CP-58 3189  969182388131289I         V382 Car 110419.0-582600110835.4-585830290.01  1.29 3.91  +1.23 +0.94 +0.58   G40-Ia            v-0.007 0.000 -.001+007SB     23                 *
4338          CP-61 2075  969192512862891          V371 Car 110424.1-612420110834.0-615650291.15-01.45 5.13  +0.22 -0.44         B9Ia              t-0.006+0.002 +.009-022        0                 *
4339          CD-31 8816  97023202149               5111    110504.9-314928110953.4-322203279.22 25.79 5.81  +0.04               A2V                +0.023-0.029      -000
4340          BD+69  602  97138 15431               5119    110548.0+684853111211.0+681619135.92 46.27 6.40                      A3V                +0.032+0.010      -018
4341          BD+15 2301  97244 99492                       110629.1+145637111143.7+142401236.63 63.53 6.30  +0.21 +0.08         A5V                -0.066-0.009      +006       82
4342          CP-57 4387  97271238839                       110635.5-575445111054.7-582719290.09  1.89 6.88  -0.08 -0.44         B7III              -0.036+0.003      +017
4343 11Bet CrtBD-22 3095  97277179624 421                   110644.3-221647111139.5-224933274.78 34.53 4.48  +0.03 +0.06 +0.02   A2III              +0.003-0.100 +.051+006SB     58
4344          BD+55 1446  97302 27952                       110655.4+552616111244.5+545339149.04 56.94 6.63                      A4V                +0.001-0.002      -006V
4345          BD+36 2162  97334 62451      W                110705.8+362143111232.2+354849184.33 67.28 6.41  +0.61 +0.10         G0V                -0.271-0.170      -003V   =<  6  1.6 138.7AB   4
4346          CD-31 8847  973932021971290I          5129    110725.5-315326111214.8-322602279.75 25.93 6.38  +1.70 +1.85 +0.98E  M1III              +0.019+0.011      +027
4347   Psi CrtBD-17 3321  97411156528        8086           110732.7-175720111230.4-183000272.49 38.45 6.13  -0.01 -0.01         A0V                -0.019-0.030      +014       51  0.4   0.2      *
4348          BD-20 3374  97428179638                       110738.7-211221111234.6-214457274.41 35.59 6.40  +1.38               K0                 -0.011+0.012      -024V?
4349          CP-70 1336  97472256808                       110747.9-705334111129.5-712611295.08-10.10 6.35  +1.37 +1.54         K2-3III            -0.020+0.003      +026
4350          CD-48 6263  974952226391291                   110759.6-483329111233.1-490604286.76 10.64 5.36  +0.18               A3IV-V             -0.096+0.033      -010SB    128
4351          BD+41 2170  97501 43649        8093           110807.8+413757111340.2+410519171.83 65.55 6.33  +1.15 +1.12         K2III              -0.004+0.013      +012V          4.8   3.1      *
4352          CP-59 3190  97534251316      W        5137    110818.5-594625111236.0-601903290.99  0.24 4.60  +0.55 +0.05 +0.38   A6Iae              -0.007+0.004 +.005-008       34  6.8  21.7      *
4353          CD-49 5937  97550222643                       110823.0-491132111256.9-494411287.06 10.08 6.11  +1.06               G8II-III           -0.008-0.009      +007
4354          CD-43 6872  975762226472894                   110834.8-434941111314.7-442220285.02 15.06 5.80  +1.66 +2.08         K5-M0III           -0.001 0.000      +001V?
4355          CP-63 1860  975832513202893  W                110836.8-633733111245.2-641011292.44-03.33 5.23  -0.06 -0.25         B8V                -0.050+0.007      +021      206  6.7  19.1      *
4356 69    LeoBD+00 2761  97585118731                       110838.4+002829111345.6-000411258.13 54.08 5.42  -0.03 -0.05         A0V                -0.043-0.002 +.018+005V?    177
4357 68Del LeoBD+21 2298  97603 81727 422I W        5143    110847.4+210418111406.5+203125224.22 66.83 2.56  +0.12 +0.12 +0.03   A4V                +0.142-0.130 +.048-020V     181  6.0 191.4AB   3*
4358          BD+08 2476  976051187352895  W                110850.0+083629111401.8+080338247.99 60.08 5.79  +1.12 +1.13 +0.38E  K3III              +0.048-0.110      +017V?         6.0  21.5      *
4359 70The LeoBD+16 2234  97633 99512 423           5144    110859.6+155834111414.4+152546235.37 64.59 3.34  -0.01 +0.06 -0.03   A2V                -0.061-0.079 +.026+008V      20                 *
4360          CP-52 4350  97651238878                       110909.5-524118111339.3-531354288.48  6.87 5.76  +1.32 +1.42         K2III              -0.025+0.038      +047
4361          CP-58 3315  97670238877              Var      110911.0-590430111330.8-593710290.83  0.93 5.74  -0.10 -0.71         B1.5V              -0.009-0.006      +018       62                 *
4362 72    LeoBD+23 2322  97778 817362897I          5147    110953.1+233826111512.2+230544218.10 67.88 4.63  +1.66 +1.85 +1.31   M3IIb             e-0.024-0.006 +.010+016                          *
4363          BD+53 1480  97855 27970        8108A          111018.8+531900111604.0+524623151.12 58.82 6.50  +0.43 -0.12         F6V+F9V            +0.162+0.057 +.052-041V?  =<  6  1.7  12.7      *
4364          CD-43 6899  97866222671                       111012.9-431120111454.0-434403285.05 15.76 6.21  +1.61 +1.93         K4III              -0.025-0.016      +009
4365 73    LeoBD+14 2367  97907 99525    I S                111038.0+135111111551.9+131827239.93 63.74 5.32  +1.20 +1.09         K3III              -0.007-0.008      +018SB     19:                *
4366          BD+13 2379  97937 99527                       111044.3+132330111557.8+125041240.81 63.49 6.67  +0.27 +0.07         A9Vp               -0.034-0.055      -020SB    109                 *
4367          BD+50 1807  97989 43675 424                   111103.8+500120111641.9+492835155.72 61.19 5.88  +1.08              gK0                 -0.083-0.009      +000V?
4368 74Phi LeoBD-02 3315  980581381021292  W                111134.6-030618111639.7-033906262.66 51.65 4.47  +0.21 +0.14 +0.11   A7IVn              -0.108-0.036 +.019-003V?    225  4.7  96.9      *
4369          BD-06 3344  980881381062900    8115  SV Crt   111154.0-063521111658.2-070805265.81 48.81 6.14  +0.20 +0.15         A8IVpeSrCrSi       -0.006+0.009D+.004-008SB2O   25  3.8  57.2AC   3*
4370          CD-45 6837  98096222687      W                111148.8-452010111627.7-455248286.15 13.87 6.31  +0.40 -0.03         F3V                -0.136+0.066D+.016-002           0.3   2.4      *
4371 75    LeoBD+02 2409  981181187642902I                  111208.6+023338111717.4+020038257.02 56.25 5.18  +1.52 +1.84 +0.94   M0III-IIIb         +0.056-0.140 +.010-059V?
4372          CD-37 7146  98161202283                       111225.5-372807111711.8-380052283.19 21.22 6.27  +0.10               A3Vn               -0.088+0.010      -003
4373          CD-34 7345  98221202289                       111249.6-341128111739.2-344414281.91 24.28 6.45  +0.40 -0.03         F4V                -0.006+0.002      -009
4374 53Xi  UMaBD+32 2132  98230 62484    I:  8119B  5165    111250.9+320531111810.9+313145195.08 69.25 4.87H                     G0V                -0.432-0.586 +.137-016SB1O    3  0.5   3.1AB   4*
4375 53Xi  UMaBD+32 2132  98231 62484    I   8119A  5165    111250.9+320531111811.0+313145195.08 69.25 4.41H +0.59 +0.04 +0.34   G0V                -0.430-0.586 +.137-016SB1O    1  0.5   3.1AB   4*
4376          CD-35 7111  98233202291                       111255.3-355920111743.0-363204282.68 22.62 6.68  +0.98               G8-K0III           -0.077+0.026      +003V
4377 54Nu  UMaBD+33 2098  98262 62486 425I   8123           111304.7+333824111828.7+330539190.73 69.08 3.48  +1.40 +1.55 +0.70   K3-IIIBa0.3        -0.026+0.028 +.020-009SB   < 19: 6.0   7.2      *
4378          BD+12 2319  98280 99544                       111308.0+123156111821.0+115905243.13 63.44 6.66  +0.07 +0.05         A2V                +0.004-0.036      -035
4379          CP-67 1703  982922513412903  W        5163    111314.2-671637111719.0-674925294.21-06.56 6.06  +1.76 +1.93         M2III              +0.013-0.007      +026V          7.5  34.7      *
4380 55    UMaBD+38 2225  98353 624911293                   111340.9+384403111907.9+381108177.34 67.74 4.78  +0.12 +0.03 +0.03   A1Vp:              -0.057-0.068 +.025-003SB2O   47                 *
4381 76    LeoBD+02 2411  98366118778                       111347.0+021155111855.0+013902257.99 56.23 5.91  +1.04 +0.87        gK0                 -0.038-0.055      +005V?
4382 12Del CrtBD-13 3345  98430156605 426I                  111420.4-141414111920.5-144643272.07 42.51 3.56  +1.12 +0.97 +0.60   G8III-IV           -0.122+0.208 +.024-005SB   < 19:
4383          BD+67  692  98499 154782905                   111446.9+673858112053.8+670602135.79 47.70 6.21  +1.01               G8                 +0.049-0.052      -056
4384          CP-63 1881  98560251357                       111505.4-640211111916.5-643457293.25-03.46 5.99  +0.46  0.00         F6IV               -0.302+0.044      +006
4385          CP-78  638  986172568232904                   111538.5-790715111834.3-794007298.68-17.57 6.35  +0.26 +0.08         A8IIIm:            +0.031-0.026      -004
4386 77Sig LeoBD+06 2437  98664118804 427                   111558.8+063439112108.2+060146253.36 59.90 4.05  -0.06 -0.12 -0.07   B9.5V s            -0.092-0.012 +.001-005SB     65
4387          CP-74  801  98672256826                       111600.3-743542111936.3-750833297.03-13.33 6.27  -0.03 -0.12         B9.5-A0V           -0.038+0.004      +023V?
4388          BD+57 1316  98673 279992907                   111605.5+573721112149.3+570430144.73 56.10 6.43  +0.15 +0.09         A7Vn               -0.058+0.009      -010SB    230
4389          CP-71 1238  98695256828                       111610.8-712648112003.9-715940295.93-10.38 6.41  +0.05 -0.48         B4V                -0.039-0.005      -000
4390   Pi  CenCP-53 4498  98718238986 428  W                111626.7-535635112100.4-542928289.96  6.09 3.89  -0.15 -0.59 -0.17   B5Vn               -0.036-0.006D+.011+009V     326  0.7   0.2      *
4391          BD+65  828  98772 15486 429                   111654.9+645240112251.3+641950137.71 50.18 6.02  +0.09 +0.09         A1Vnp              -0.005+0.034      +001      160                 *
4392 56    UMaBD+44 2083  98839 437192908I                  111720.4+440153112249.6+432858164.73 65.77 4.99  +0.99 +0.80 +0.46   G7.5IIIa:Ba0.3     -0.038-0.013 -.002+003     < 19:
4393          CD-43 7006  98892222751                       111738.9-440548112223.1-443845286.70 15.40 6.12  +0.92 +0.68         G8III              -0.048-0.034      +001
4394          BD+00 2782  98960118825    I                  111810.7+004051112318.0+000754261.20 55.69 6.05  +1.46 +1.78         K3                 -0.037-0.018      +022
4395 13Lam CrtBD-17 3367  98991156646                       111824.3-181349112321.9-184648275.60 39.37 5.09  +0.42 -0.06         F5III              -0.312-0.034 +.023+011SB1O   14                 *
4396          CD-35 7163  989932023912909  W                111822.0-353658112312.7-360953283.65 23.38 5.00  +1.46 +1.63         K6III              -0.020 0.000 +.025-005SB         9.5  31.3
4397          CP-76  662  99015256832                       111834.8-770335112157.1-773630298.07-15.59 6.43  +0.20 +0.11         A5III-IV           -0.081-0.005      -006V?
4398          CP-56 4449  990222390172910                   111835.5-561350112308.1-564646291.02  4.04 5.79  +0.01               A4:pe              -0.030+0.006      +003                          *
4399 78Iot LeoBD+11 2348  99028 99587    I   8148   5184    111842.7+110449112355.5+103145247.60 63.55 3.94  +0.41 +0.07 +0.21   F4IV               +0.168-0.075 +.052-010SB1O   20  2.8   1.1      *
4400 79    LeoBD+02 2418  99055118831      D                111854.4+015724112402.3+012428260.09 56.83 5.39  +0.94 +0.66         G8IIICN-0.5        -0.020+0.005      -010V?         0.7   0.0      *
4401          CP-64 1657  99104251383      W                111902.0-642421112321.8-645718293.78-03.66 5.11  -0.08 -0.42         B5V                -0.001-0.005D+.010+019V?     78  1.2   2.6
4402 14Eps CrtBD-10 3260  99167156658    I                  111933.5-101839112436.6-105134271.04 46.57 4.83  +1.56 +1.96 +0.94   K5III              -0.028+0.027      +003V    < 19:
4403          CD-41 6529  991712227731294                   111934.9-420712112422.1-424009286.33 17.38 6.12  -0.18 -0.81         B2IV-V             -0.005+0.001      +010
4404          BD+12 2335  99196 995982912I                  111947.7+115847112458.9+112549246.45 64.34 5.80  +1.38 +1.58 +0.54E  K4III              -0.102-0.005      +038V?                        *
4405 15Gam CrtBD-16 3244  99211156661 431    8153           111953.1-170805112452.9-174102275.41 40.52 4.08  +0.21 +0.11 +0.11   A5V                -0.098+0.004 +.028+001V?    131  4.8   5.3      *
4406          CP-71 1248  992642568342911                   112011.8-714226112411.1-721524296.32-10.51 5.59  +0.06 -0.57  0.00   B2IV-V             -0.030-0.003      +002V                         *
4407          BD+56 1518  99283 28017                       112018.7+562355112557.1+555102145.26 57.42 5.75                      K0III              -0.066+0.048      -006V?
4408 81    LeoBD+17 2356  99285 996012913  W                112023.5+170023112536.4+162723236.75 67.45 5.57  +0.36 +0.02         F2V                -0.143-0.005      +018V      30  3.6  55.7
4409          CD-35 7189  99322202428      I                112038.4-353050112529.4-360347284.08 23.64 5.22  +0.99 +0.77 +0.31E  K0III              -0.113+0.020 +.032+004           4.6   1.3      *
4410 80    LeoBD+04 2463  99329118851                       112041.6+042439112550.0+035136257.90 59.03 6.37  +0.33 +0.05         F3IV               -0.080-0.041      -003SB    112
4411          CD-37 7235  99333202430      W                112042.5-371151112533.1-374452284.73 22.07 5.89  +1.54 +1.72         M3III              -0.042-0.014      +007           5.5   4.7
4412          BD+34 2222  99373 62551                       112104.4+335959112625.5+332702188.72 70.63 6.32  +0.43  0.00?        F7V                -0.040+0.016      -025        5
4413          CP-63 1893  99453251402                       112123.3-632515112543.2-635822293.70-02.65 5.17  +0.50 +0.04         F7V                -0.313-0.078 +.040-005SB2    84
4414 83    LeoBD+03 2502  994911188641296    8162A          112141.6+033329112645.3+030047259.28 58.52 6.50  +0.79 +0.49 +0.36   K0IV               -0.724+0.180 +.054-003V?         1.1  28.5AB   3*
4415          CP-60 2941  99556251406      W                112205.4-603354112635.3-610655292.87  0.09 5.30  -0.08 -0.54 -0.06   B3IV               -0.026+0.002      +009      205  8.5  13.0      *
4416 16Kap CrtBD-11 3098  995641566852914  W                112207.2-114826112709.5-122124272.85 45.53 5.94  +0.49 +0.03         F4III-IV           -0.097+0.026      +006           7.0  27.9
4417          CP-52 4567  99574239074      W                112207.6-523637112647.3-530936290.32  7.63 5.81  +0.52               K0III+A2-3V        -0.041+0.014      -008           0.4   0.1
4418 84Tau LeoBD+03 2504  996481188751297I W                112247.6+032425112756.2+025122259.90 58.57 4.95  +1.00 +0.79 +0.50   G7.5IIIa           +0.018-0.012 +.034-009     < 19: 2.6  91.1AB   4*
4419          BD-00 2442  99651138216      W                112247.0-010858112753.7-014200264.70 54.85 6.25  +1.04 +0.73        gK2                 -0.034-0.003      -010           1.4   0.3
4420          CD-34 7471  99712202478                       112305.3-344642112758.5-351943284.31 24.51 6.45                      K5III:             -0.006+0.012      -018V
4421          BD+62 1183  99747 155202915                   112321.8+621921112904.6+614642138.87 52.73 5.83  +0.36 -0.05         F5Va vw            -0.112+0.237      -008V
4422 57    UMaBD+40 2433  99787 62572        8175           112341.0+395314112904.2+392013172.23 68.97 5.31  +0.01 +0.01         A2V                -0.050+0.016 +.025-011SB    176  2.5 346.3AD   6*
4423          CD-41 6565  99803222813      W                112346.0-420725112835.1-424027287.10 17.64 5.08  -0.03               B9V                -0.040+0.008D+.007+003SB      0  2.7  13.2      *
4424          BD+57 1324  99859 28035                       112407.6+571722112943.5+564415143.51 57.01 6.28  +0.15 +0.11 +0.04   A4m                -0.089-0.034      +009       90                 *
4425          CP-71 1253  99872256843      W                112412.7-715523112818.2-722828296.69-10.62 6.09  +0.16 -0.42         B3V                -0.043-0.020      +012           0.1   0.3
4426 85    LeoBD+16 2266  99902 99629    I                  112429.3+155756112941.9+152448240.45 67.70 5.74  +1.32              gK4                 -0.028-0.045      -029
4427          BD+55 1468  99913 28038                       112438.7+545452113012.9+542142146.03 58.97 6.41R                     K0III              +0.010-0.057      -022
4428          CD-2310009  99922179935        8183           112440.1-235448112938.6-242748280.16 34.77 5.76  +0.07               A0V                -0.048+0.043D+.021-002           3.0   8.3AB   3
4429          BD+81  373  99945  18843956           5230    112447.9+814040113150.4+810738126.62 35.42 6.15  +0.24 +0.12         A2m                -0.149+0.032 +.020+003
4430          BD+47 1880  99967 43784              EE UMa   112457.4+471229113025.0+463927156.83 64.78 6.35  +1.27 +1.19         K2IIICN-1          +0.011+0.032      +027SBO                       *
4431 58    UMaBD+43 2122  99984 43787 432                   112506.5+434320113031.1+431024163.28 67.08 5.94  +0.49 -0.01         F4V                -0.050+0.083      -030    =<  6                 *
4432 87    LeoBD-02 3360  999981382382917I                  112512.3-022706113018.9-030013266.78 54.07 4.77  +1.54 +1.83 +0.84   K3.5IIIFe-1        +0.022-0.014 +.015+019V?   < 19:                *
4433 86    LeoBD+19 2459 100006 996372918I                  112515.9+185738113029.0+182435233.76 69.44 5.52  +1.05 +0.82 +0.37E  K0III              -0.088+0.020 +.026+027V                         *
4434  1Lam DraBD+70  665 100029 15532 433I          5231    112528.3+695259113124.2+691952132.98 46.20 3.84  +1.62 +1.97 +0.99   M0IIICa-1          -0.039-0.017 +.026+007                          *
4435          BD+48 1952 100030 43790                       112527.1+482857113052.9+475545154.61 63.94 6.42  +0.88 +0.54         G9IV               -0.229-0.070      +038
4436          BD+49 2062 100055 43793                       112542.0+492030113110.2+484721153.20 63.35 6.56  +0.93 +0.72         G9III              -0.030-0.040      +006
4437 88    LeoBD+15 2345 100180 99648        8196A          112635.1+145518113144.9+142152243.43 67.48 6.20  +0.57 +0.07         G0V                -0.330-0.189 +.035-004    =< 10  2.8  15.5      *
4438          CP-60 3011 100198251437              V809 Cen 112640.3-604336113115.0-611642293.45  0.11 6.38  +0.52 -0.11         A3Iae              -0.007+0.003      -004V                         *
4439          BD+61 1246 100203 15542        8197           112641.0+613811113220.8+610457138.90 53.52 5.48  +0.50 -0.01         F8V                +0.002-0.069 +.045-046SB      0  1.4   0.5      *
4440          BD-19 3285 1002191799642919                   112647.8-201332113147.6-204636278.98 38.36 6.24  +0.54               F7IV               -0.131+0.032      +010V?  =< 15
4441   Omi1CenCP-58 3692 100261239145    I W       Omi1 Cen 112708.6-585325113146.1-592632292.95  1.88 5.13  +1.08 +0.78 +0.66   G30-Ia             -0.005-0.005 +.024-020V          6.4  13.3      *
4442   Omi2CenCP-58 3693 1002622391462920          Omi2 Cen 112711.3-585749113148.8-593057292.98  1.81 5.15  +0.49 -0.11 +0.46   A2Ia               -0.008+0.001 +.014-017       43                 *
4443          CD-28 8928 100286179967        8202B          112718.5-284255113216.1-291548282.88 30.51 5.81  +0.53 +0.02         F8V                -0.014+0.138D+.013+010V?         0.1   9.3      *
4444          CD-28 8928 100287179968        8202A          112718.9-284247113216.3-291540282.88 30.51 5.64  +0.53 +0.04         F8V                -0.028+0.145D+.013+004V?         0.1   9.3      *
4445          CD-26 8620 100307179969    I                  112725.2-261143113223.3-264448281.86 32.87 6.16  +1.66 +1.99 +0.98E  M1III              -0.079+0.033      +035V?
4446          BD-07 3250 1003431382652921I                  112742.5-071631113247.5-074939271.56 50.19 5.95  +1.38 +1.64         K4III              -0.007+0.004      -001V?
4447          CD-39 7168 1003782228562922                   112755.9-395308113248.1-402611287.12 20.01 5.64  +1.58 +1.81         K6III              -0.069+0.052      +003
4448          CP-66 1605 100382251451                       112753.0-662435113219.9-665744295.29-05.27 5.90  +1.13 +1.12         K1IIICNIb-II       -0.004-0.001      -011
4449          CD-30 9303 100393202554    I          5246    112757.2-303207113254.1-310514283.75 28.84 5.04  +1.58 +1.95 +1.01   M2IIIb             -0.030+0.006 +.019+019
4450   Xi  HyaCD-31 9083 100407202558 434I W                112804.9-311815113300.1-315128284.08 28.13 3.54  +0.94 +0.71 +0.48   G7III              -0.206-0.040 +.027-005V          7.0  68.2      *
4451          BD-15 3295 100418156750                       112812.2-154337113314.6-161650277.05 42.63 6.05  +0.60               F9III              -0.002-0.043      -004
4452          BD+37 2195 100470 626042923                   112837.3+372210113356.3+364856177.50 70.98 6.40  +1.05 +0.93 +0.53   K0III              -0.129-0.046 -.014+018SB                        *
4453          CD-39 7175 100493222863      W                112844.7-400206113337.3-403513287.33 19.92 5.39  +0.12               A2IV-V             -0.071+0.024 +.028+009      137  0.0   1.0      *
4454          BD+11 2372 100518 99668                       112859.0+113437113410.0+110125250.77 65.72 6.55  +0.18 +0.10 +0.12   A2m                +0.045-0.031      -005V
4455 89    LeoBD+03 2521 1005631189292924                   112914.8+033656113422.0+030336262.18 59.69 5.77  +0.46 +0.01         F5V                -0.182-0.104 +.045+003V?     14
4456 90    LeoBD+17 2374 100600 99673        8220           112930.2+172059113442.5+164749239.18 69.47 5.95  -0.16 -0.64 -0.14   B4V                -0.011+0.003D+.003+019SB    117  1.1   3.6AB   3*
4457          BD+55 1473 100615 280642925                   112934.4+552016113504.9+544707144.44 59.04 5.63  +1.02               K0III              +0.008+0.008      +018V?
4458          CD-32 8179 100623202583                       112937.8-321805113429.5-324953284.79 27.31 5.98  +0.81 +0.39 +0.46   K0V                -0.674+0.826 +.106-023V
4459          BD+21 2331 100655 81886                       112950.7+205940113503.8+202629229.80 71.32 6.45  +1.01               G9III              -0.059-0.005      -007
4460          CP-53 4637 1006732391892926                   113001.0-534242113445.7-541551291.79  6.94 4.62  -0.08 -0.21 -0.53   B9Ve               -0.056+0.017      +003V     159                 *
4461  2    DraBD+70  670 100696 15567    I                  113011.3+695246113602.8+691922132.45 46.38 5.20  +1.01 +0.68 +0.50   K0III              +0.111-0.127 +.025-002     < 19:
4462          CD-48 6630 100708222883               5253    113009.5-483518113456.8-490812290.29 11.85 5.50  +1.04 +0.92         K1III-IV           -0.170+0.168 +.034-001
4463          CD-46 7199 100733222887      W       V763 Cen 113023.8-464910113513.2-472221289.79 13.55 5.71  +1.66 +1.93 +1.00E  M3III              -0.089-0.004 +.008+018           7.6   6.7      *
4464          BD+11 2377 100740 996832927                   113032.7+112753113543.4+105440251.62 65.92 6.56  +0.13 +0.10         A4Vn               +0.035-0.023      -005SB    232
4465          BD+28 2022 100808 81893        8231           113102.2+282003113617.9+274652206.35 73.33 5.80  +0.23 +0.07         F0V                +0.023+0.003D+.012+008SB         0.2   0.5AB   3*
4466          CD-46 7205 100825222895 435                   113104.6-470514113555.6-473830289.99 13.32 5.25  +0.25 +0.12         A7m                +0.036-0.049 +.022+005       43
4467   Lam CenCP-62 2127 100841251472 436  W                113110.0-622759113546.8-630111294.47-01.40 3.13  -0.04 -0.17 -0.01   B9III              -0.041-0.005      -001V       0  8.2  16.3      *
4468 21The CrtBD-08 3202 1008891382961299                   113136.5-091457113640.9-094808274.25 48.86 4.70  -0.08 -0.18 -0.07   B9.5Vn             -0.061+0.008      +001SB    192                 *
4469          CD-32 8199 100893202622      W                113137.2-330057113635.0-333412285.51 26.76 5.74  +1.02 +0.83         K0III              +0.040-0.037      +042           2.1   3.3AB   3
4470          CD-36 7291 100911202627                       113144.8-364102113640.8-371416286.81 23.28 6.31  +0.06 +0.06         A1V                -0.013-0.019      -002V
4471 91Ups LeoBD-00 2458 100920138298 437I D                113149.7-001618113656.9-004926267.26 56.80 4.30  +1.00 +0.75 +0.52   G8.5IIICN-0.5      +0.004+0.043 +.019+001     < 19: 4.5   0.1      *
4472          CP-60 3140 100929251479                       113142.2-602957113622.3-610308293.97  0.51 5.83  -0.09 -0.63         B2.5IV             -0.021+0.008      +011        0                 *
4473          CD-32 8202 100953202634                       113203.4-322557113701.2-325917285.40 27.34 6.29  +0.46 +0.06         F6III-IV           -0.011-0.079      +010
4474          BD+51 1679 101013 280812929                   113228.4+511021113753.0+503706148.77 62.66 6.14  +1.07 +0.76 +0.48   G9III:Ba2.5        -0.047-0.021      -004
4475          CP-60 3182 101021251486                       113222.8-604348113700.6-611700294.12  0.31 5.15  +1.12 +1.15         K0III              -0.224+0.006 +.002+003                          *
4476          CD-47 6997 101067222917                       113243.0-471139113733.9-474450290.29 13.30 5.44  +1.24 +1.30 +0.44E  K2III              -0.072+0.027 +.001-001                          *
4477 59    UMaBD+44 2110 101107 43837                       113300.9+441048113820.6+433732159.98 67.87 5.59  +0.33 +0.01         F2II-III           -0.145-0.033      +002V      51
4478          BD+09 2523 101112118961                       113300.5+092614113809.8+085303256.00 64.85 6.17  +1.08 +0.95 +0.36E  K1III              -0.059+0.019      +011                          *
4479   Pi  ChaCP-75  744 101132256857 438                   113308.1-752034113715.6-755348298.36-13.69 5.65  +0.35  0.00 +0.21C  F1III              -0.134+0.001      -010
4480 60    UMaBD+47 1894 101133 438392931                   113311.4+472320113833.5+465003154.16 65.63 6.10  +0.38 +0.14         F5III s            -0.037-0.033      -024
4481          BD+65  843 101150 15580        8249           113312.1+645402113849.2+642049135.31 51.00 6.46R                     A5IV               +0.016+0.006D+.004-022           1.3   1.8
4482          BD+34 2242 101151 62635                       113315.6+341047113832.2+333732186.02 73.02 6.27  +1.32 +1.48         K2III              -0.028-0.019      -006
4483  1Ome VirBD+08 2532 1011531189652932I         Ome Vir  113318.2+084116113827.6+080803257.26 64.32 5.36  +1.57 +1.53         M4III              -0.006+0.005 -.005+004V?                        *
4484          BD-01 2546 101154138314        8247           113317.5-015257113824.1-022610269.29 55.59 6.22  +1.12 +1.08         G9III              -0.031+0.009      -015V?         4.7   5.2      *
4485          CP-66 1629 1011622515012930                   113316.5-670359113748.4-673713295.99-05.75 5.96  +1.02 +0.83         K0III              -0.083-0.007      +034
4486          BD+45 1947 101177 43841        8250A          113329.1+453942113844.9+450631157.05 66.91 6.44  +0.56 +0.05         G0V                -0.594+0.026 +.049-018SBO        1.9   9.4AB   5*
4487          CP-61 2463 101189251505                       113326.9-611623113807.3-614935294.39-00.18 5.15  -0.02 -0.16         ApHgMn             -0.071+0.017 +.024+004        0                 *
4488 24Iot CrtBD-12 3466 101198156802      W                113335.2-123905113840.1-131207277.00 45.97 5.48  +0.52               F7V                +0.092+0.120      -024           5.4   1.4
4489          CD-24 9867 1012591800652933                   113359.0-240938113900.4-244316282.69 35.29 6.42  +0.82 +0.32         G5                 +0.033-0.241      +097V?
4490          BD-13 3420 1013691568202934    8259           113447.3-135451113951.1-142807278.08 44.92 6.21   0.00               A1V                -0.038-0.011      +005       68  6.0   7.8      *
4491          BD-15 3323 101370156819    I          5284    113446.9-160357113950.4-163713279.21 42.93 6.19  +1.62 +1.75         M3.5IIIBa0.2Ca1    -0.010-0.009      +026V?                        *
4492          CP-64 1685 101379251522      W       GT Mus   113451.4-645036113929.4-652352295.53-03.56 5.17  +0.80 +0.36 +0.63   G2III+A0V          -0.042-0.006 +.006+014SB         0.8   0.2AB   4*
4493          BD+58 1331 101391 28093                       113459.2+583127114027.4+575814140.07 56.73 6.37  -0.12 -0.34         B9pHgMn:           -0.012+0.021      +004
4494   Omi HyaCD-34 7610 101431202695 439                   113514.7-341125114012.8-344441286.72 25.87 4.70  -0.07 -0.22 -0.05   B9V                -0.042 0.000      +006      188:
4495 92    LeoBD+22 2391 101484 819412936I                  113535.1+215430114047.1+212110228.85 72.91 5.26  +0.98 +0.44?+0.48   K1III              -0.064-0.049      +009     < 17
4496 61    UMaBD+35 2270 101501 626551300  W        5291    113547.1+344600114103.0+341206183.53 73.32 5.33  +0.72 +0.25 +0.36   G8V                -0.012-0.381 +.119-005V    < 17  4.9 159.3
4497          CP-53 4691 101541239284               5289    113552.4-532449114042.6-535807292.55  7.47 5.96  +1.66 +2.02         M1III              +0.004-0.019      -012V?                        *
4498          CD-28 9027 101563180098                       113610.3-283852114108.4-291147284.99 31.21 6.44  +0.66 +0.19         G0V                -0.324+0.215 +.032-019SB
4499          CP-61 2514 101570251535                       113609.8-613208114053.6-620524294.78-00.34 4.94  +1.15 +0.81 +0.57   G3Ib               -0.020+0.008 +.025+014SB
4500          BD+55 1481 101604 28101    I                  113618.6+554334114143.6+551021142.47 59.23 6.27  +1.49               K5                 -0.015+0.023      -007
4501 62    UMaBD+32 2179 101606 626582938  W                113622.0+321759114134.3+314446191.85 74.13 5.73  +0.43 -0.07         F4V                -0.351+0.032 +.026+032V   =< 10  4.3  84.3      *
4502          CD-42 7155 1016152229602937                   113624.6-423229114119.8-430545289.57 17.95 5.55                      A0V                -0.082+0.003      +008SB      0
4503          CD-31 9181 101666202717    I W                113644.2-315638114144.0-322959286.29 28.11 5.22  +1.48 +1.78         K5III+F7V          +0.004-0.042 +.015+034           3.1  67.0AC   4*
4504  3    DraBD+67  714 101673 15606 440I                  113653.8+671754114228.4+664442133.20 48.99 5.30  +1.28 +1.24         K3III              -0.043+0.038      +003V?   < 17
4505          BD+23 2375 101688 81949                       113653.6+224602114205.2+221239226.49 73.51 6.59R                     F2IV-V             -0.092-0.063      -023       60
4506          BD-19 3326 101695180117                       113700.6-194417114203.5-201738281.62 39.69 6.22  +0.95               K0                 -0.015-0.048      -014V
4507          CP-82  469 1017822586213982  W                113733.2-823244114101.2-830600300.71-20.53 6.33  +1.08 +0.88         K0III              -0.071+0.015      +012           5.2  22.2
4508          CD-36 7371 1018832027442939                   113828.3-363804114327.2-371125288.20 23.73 5.98  +1.46 +1.71         K3III              -0.015-0.024      +036
4509          CP-78  677 101917256865                       113841.0-784504114255.4-791823299.66-16.87 6.39  +0.90 +0.59         K0III-IV           +0.132-0.008      +033
4510          BD-05 3340 101933138375                       113848.6-060715114355.1-064038274.69 52.45 6.07  +0.96 +0.71        gG8                 +0.059-0.040      -003V?
4511          CP-61 2559 101947251555    I         V810 Cen 113844.8-615604114331.2-622922295.18-00.64 5.03  +0.80 +0.35 +0.36   G00-IaFe1         v-0.008+0.003 +.037+010V                         *
4512          BD+26 2250 101980 819602940I   8285           113900.8+254623114413.2+251306216.37 74.81 6.02  +1.53 +1.85         K5III              -0.014+0.012      -003V?         4.7  36.7      *
4513          CP-62 2250 101995251559                       113906.1-621922114352.9-625242295.32-01.01 6.10  +0.06 +0.13         A2III-IV           +0.017-0.015      +002V?
4514 27Zet CrtBD-17 3460 1020701568691301I                  113941.6-174741114445.8-182103281.53 41.73 4.73  +0.97 +0.74 +0.48   G8IIIa             +0.031-0.030 +.030-005     < 19:                *
4515  2Xi  VirBD+09 2545 102124119029                       114007.7+084851114517.1+081530260.18 65.48 4.85  +0.18 +0.10         A4V                +0.061-0.018 +.039-001V     158
4516          CD-48 6777 102150222997                       114017.0-483054114512.6-490411291.91 12.38 6.26  +1.17               K1III              -0.031+0.014      +004
4517  3Nu  VirBD+07 2479 1022121190351302I          5318    114043.1+070523114551.6+063146262.86 64.18 4.03  +1.51 +1.79 +1.01   M1IIIab           e-0.018-0.184 +.014+051V?                        *
4518 63Chi UMaBD+48 1966 102224 43886 441I          5319    114046.3+482002114603.0+474646150.32 65.72 3.71  +1.18 +1.16 +0.60   K0.5IIIb           -0.137+0.030 +.019-009V    < 19:                *
4519          CD-44 7564 102232223009               5317    114046.9-450806114543.9-454124291.09 15.67 5.29  -0.12 -0.55 -0.04C  B6III              -0.047+0.009      -010V?     56
4520   Lam MusCP-66 1640 102249251575 442  W                114053.1-661027114536.4-664343296.49-04.68 3.64  +0.16 +0.15 +0.08   A7III              -0.103+0.037      +016       71  9.0  40.6      *
4521          BD+56 1544 102328 281422941I                  114134.5+561104114655.6+553742140.77 59.21 5.27  +1.27 +1.48 +0.60   K2.5IIIbCN1Ca1     +0.014-0.030      +002     < 19:
4522          CP-60 3325 102350251579 443  W        5325    114140.4-603721114630.8-611042295.19  0.71 4.11  +0.90 +0.58 +0.43   G5Ib-II            -0.027-0.015 +.011-003           8.8  25.       *
4523          CD-39 7301 102365223020    I'                 114144.9-395722114631.1-403002289.82 20.71 4.91  +0.66 +0.10 +0.38   G3V                -1.531+0.401 +.099+015                          *
4524          CD-35 7438 102397202805                       114205.7-352102114707.0-355425288.58 25.17 6.17  +0.96               G8III              +0.034-0.037      +009V
4525          CD-29 9337 102438202811                       114215.6-294328114715.7-301713286.84 30.58 6.48  +0.68 +0.24         G5V                -0.276-0.240 +.064+012
4526          CP-57 4989 1024612393732942           5334    114225.9-570829114719.1-574147294.42  4.11 5.41  +1.67 +1.99         K5III              -0.025+0.017 +.004-052                          *
4527 93    LeoBD+21 2358 102509 819981304I W       DQ Leo   114249.6+204629114759.1+201308235.01 73.93 4.53  +0.55 +0.28 +0.36   G5III-IVe+A7V      -0.149-0.003 +.022+000SBO =< 50  4.4  74.3      *
4528  4    VirBD+09 2549 102510119058      W                114246.6+084805114754.9+081445261.46 65.86 5.32  +0.02 +0.04         A1                 -0.057+0.012 +.017-001V      68  6.8 150.2AB   3
4529          BD-09 3366 102574156896      W                114318.3-094515114823.5-101848278.60 49.56 6.26  +0.58 +0.13         F7V                -0.099-0.115      +015V?  =< 10  3.3  88.4AB   3*
4530   Mu  MusCP-66 1649 102584251597              Mu  Mus  114325.7-661531114814.3-664853296.76-04.70 4.72  +1.54 +1.91 +0.88   K4III              +0.013-0.015 +.013+037                          *
4531          BD+15 2381 102590 99800        8311   5344    114329.9+145023114838.7+141703251.07 70.53 5.88  +0.29 +0.07         F0V                -0.107+0.007 +.053+006V          2.4   1.1      *
4532          CD-26 8789 1026201802081305I         II Hya   114341.9-261137114845.1-264459286.01 34.05 5.11  +1.60 +1.67         M4+III             -0.032-0.010 +.019+007                          *
4533          BD+00 2843 102634138420                       114355.3+001414114901.2-001907271.74 58.72 6.15  +0.52 +0.08         F7V                -0.217+0.009      +004V?      6
4534 94Bet LeoBD+15 2383 102647 99809 444I   8314  Bet Leo  114357.5+150752114903.6+143419250.65 70.81 2.14  +0.09 +0.07 +0.02   A3V                -0.497-0.114 +.082-000V     121  6.5 264.0AD   4*
4535          BD+17 2402 102660 99812                       114404.8+164801114914.9+161434246.81 71.94 6.04  +0.27 +0.14         A3m                +0.054-0.061      -023SB1O   22                 *
4536          BD+35 2284 102713 62718                       114430.0+352915114941.7+345554178.56 74.67 5.70  +0.46 +0.02         F5IV               -0.115+0.006      -007SBO                       *
4537          CP-63 1988 1027762516022944           5357    114448.9-631357114941.1-634718296.18-01.73 4.32  -0.15 -0.59 -0.14   B3Vne              -0.021+0.006      +029V     268                 *
4538          CP-69 1595 1028392516042945                   114509.3-694011114956.6-701333297.74-07.98 4.97  +1.40 +1.22 +0.75   G6Ib               -0.009+0.001 +.009+018
4539          BD-15 3363 102845156926                       114514.2-151829115019.5-155150282.11 44.52 6.13  +0.95               K0                 -0.013+0.009      -004V
4540  5Bet VirBD+02 2489 102870119076 445I W                114529.1+021942115041.7+014553270.50 60.75 3.61  +0.55 +0.11 +0.28   F9V                +0.743-0.271 +.104+005V       3  5.0 512.3AC   3*
4541          CP-61 2677 102878251605                       114533.3-620536115027.2-623858295.99-00.60 5.70  +0.26 -0.04         A2Iab              -0.012-0.002      -010                          *
4542          CD-26 8807 102888180232                       114533.9-264319115037.1-271640286.67 33.67 6.48  +0.98               G8III              -0.087+0.010      -003
4543          BD+13 2465 102910 998272946    8320           114547.4+125003115055.3+121644256.30 69.47 6.35  +0.27 +0.08         F0IVm s            -0.137+0.023      +007       60  4.9  14.9      *
4544          BD-04 3152 1029281384451306  D                114555.4-044638115102.2-052000276.47 54.41 5.64  +1.06 +0.88         K0III              +0.003 0.000      +012SB                        *
4545          BD+34 2264 102942 62731      W                114557.7+335550115109.4+332230183.62 75.58 6.27  +0.32 +0.11 +0.09   Am                 -0.021+0.019      +002V          2.5  46.6
4546          CD-44 7614 102964223062 446                   114608.6-443701115108.7-451025291.92 16.40 4.46  +1.30 +1.46 +0.67   K3III              -0.070-0.010 +.022+002
4547          BD-11 3190 1029901569402947  W        5365    114617.5-113758115121.9-121116280.66 48.07 6.35  +0.40               F1III-IV           -0.238+0.048      +009       28  7.2  30.7
4548          CD-30 9506 1030262028832948                   114638.0-301614115141.6-305006288.08 30.31 5.85  +0.56 +0.05         F7V                -0.010-0.295 +.019+033
4549          CP-64 1724 103079251617      W                114657.6-643858115151.2-651222296.73-03.05 4.90  -0.11 -0.54 -0.13   B4V                -0.039-0.010      +021V?     57  2.2   1.8      *
4550          BD+38 2285 103095 627381307          CF UMa   114713.0+382610115258.8+374307168.42 73.69 6.45  +0.75 +0.17 +0.45   G8Vp               +4.003-5.813 +.116-098V          5.5   1.7      *
4551          CP-56 4836 103101239443      W                114713.3-562556115210.1-565916294.89  4.96 5.57  +0.07               A2V                -0.119+0.033      +023V?         5.3  54.6AB   3
4552   Bet HyaCD-33 8018 103192202901      W       Bet Hya  114751.3-332106115254.6-335429289.27 27.41 4.28  -0.10 -0.33 -0.08   B9IIIpSi           -0.050+0.005D+.010-001       72  0.9   0.8      *
4553          CD-34 7760 1032662029102949                   114823.7-343034115326.8-350400289.72 26.31 6.17  +0.08 +0.09         A2V                -0.079-0.028      -009
4554 64Gam UMaBD+54 1475 103287 28179 447I          5379    114834.3+541503115349.8+534141140.84 61.38 2.44   0.00 +0.02 -0.03   A0Ve               +0.095+0.012 +.028-013SB    168                 *
4555          BD+01 2624 103313119100                       114843.1+010630115350.3+003307273.07 60.04 6.30  +0.20 +0.16         F0V                -0.039-0.007      +010       75
4556          CP-56 4861 103400239475                       114911.8-565113115411.5-572436295.26  4.61 6.06  +0.05               A0-1:III           -0.014+0.005      -014
4557          CD-37 7536 103437202926      W                114924.7-371138115425.8-374456290.66 23.76 6.46  +0.52 -0.03         F7V                -0.309+0.060 +.052-012           1.3   1.1      *
4558          CD-25 8930 1034621802882950                   114937.0-250934115442.5-254250287.21 35.42 5.30  +0.88 +0.53         G4III              +0.048+0.081 +.032-011V
4559  6    VirBD+09 2560 1034841191112951                   114955.3+090000115503.1+082638264.79 67.04 5.58  +0.94 +0.67 +0.46  gK0                 -0.031+0.016      -010
4560 65    UMaBD+47 1913 103483 43945        8347A DN UMa   114953.6+470159115505.7+462837149.12 67.68 6.54  +0.10 +0.08         A3Vn               +0.001+0.008D+.003-008SB    150  0.4  63.0ABxD 4*
4561 65    UMaBD+47 1914 103498 43946        8347D          114959.3+470134115511.2+462811149.10 67.70 7.03  +0.01 +0.02         A1pCrEuSr:         -0.009+0.006D+.003-007    =< 25  0.4  63.0ABxD 4*
4562          BD+37 2230 103500 62754    I                  115003.9+371851115514.1+364523170.44 74.71 6.49  +1.58 +1.83 +0.98E  M3III              -0.054-0.050      +019
4563          CP-62 2408 103516251640                       115001.4-624321115459.8-631644296.64-01.10 5.91  +0.19 +0.01         A3Ib               -0.026+0.011      -028
4564 95    LeoBD+16 2319 103578 998691308  W                115031.9+161212115540.5+153848251.67 72.70 5.53  +0.11 +0.12         A3V                +0.009+0.002 -.003-021SBO    54       90.       *
4565          CD-27 8384 103596180307    I                  115034.9-275511115540.1-282837288.34 32.81 5.93  +1.50 +1.86         K5III              +0.004-0.026      +011V
4566 66    UMaBD+57 1343 103605 281912953                   115045.0+570918115558.4+563555137.73 58.91 5.84  +1.10               K1III              +0.010 0.000      +013V
4567 30Eta CrtBD-16 3358 1036321569881309           5392    115055.1-163538115600.9-170903284.47 43.73 5.18  -0.02  0.00         A0V                -0.051-0.006 +.023+015V?     74
4568          CD-38 7410 103637202950                       115050.1-390756115554.7-394121291.45 21.94 6.13  +1.02               K0III              +0.048-0.008      -000
4569          BD+62 1204 103736 15666                       115140.1+620625115653.2+613257134.05 54.39 6.22  +0.96               G8III              -0.031-0.040      +017
4570          CD-46 7521 103746223127                       115141.3-463059115643.9-470421293.35 14.78 6.26  +0.40 +0.04 +0.22C  F5V                -0.126+0.016      +008
4571          CD-32 8413 103789202971              LV Hya   115158.8-324531115703.7-331855290.06 28.19 6.21  -0.04               A0V                -0.055+0.003      -010
4572          BD+41 2253 103799 439632954                   115205.9+405407115714.6+402037159.77 72.68 6.62  +0.46 -0.02         F6V                -0.160-0.062      +026        7
4573          CP-61 2829 103884251659               5397    115238.3-615330115740.1-622656296.76-00.22 5.57  -0.15 -0.66 -0.23   B3V                -0.022-0.006      +020V     185                 *
4574          BD+33 2174 103928 627741310    8368           115259.0+324956115807.2+321626185.75 77.34 6.42  +0.29               A9V                -0.109-0.060      +002V?     67  5.0 122.5AC   4*
4575          BD+62 1206 103953 15673                       115308.0+620117115820.6+612753133.83 54.53 6.76                      K0III              +0.042 0.000      -026
4576          CP-55 4751 1039612395332956           5402    115311.8-554538115815.2-561902295.57  5.79 5.44  -0.08               B8III              -0.021+0.006      -005
4577          CD-40 7041 103974223139      W                115315.5-402328115820.3-405650292.24 20.82 6.79  +0.97 +0.70         K1III              -0.057+0.021      +007           2.6   3.1
4578          CP-63 2073 104035251664                       115344.8-634658115847.7-642021297.27-02.05 5.61  +0.18 -0.11         A3Ib               -0.001+0.012      +002V
4579          CD-25 8963 104039180349        8371           115348.2-252105115854.4-255432288.40 35.48 6.43  +0.03 +0.04         A0-1V              -0.027-0.019      +013           0.2   0.2      *
4580          BD+01 2636 104055119147        8372           115356.4+010512115903.4+003150275.47 60.58 6.17  +1.26 +1.41         K2IV               -0.061+0.027      +012V?         7.7  14.2      *
4581          BD+33 2176 104075 62784        8374           115408.9+334327115917.6+331003181.58 77.23 5.96  +1.15 +1.11 +0.58   K1III              +0.004+0.003      -001V?         6.0   4.5      *
4582          CD-51 6236 1040812395482959                   115406.1-510823115910.9-514148294.74 10.34 6.05  +1.28               K1-2III            -0.012-0.003      +010
4583   Eps ChaCP-77  772 104174256894      W                115439.4-773954115937.3-781319300.21-15.62 4.91  -0.06 -0.16 -0.05   B9Vn               -0.057-0.002D+.012+013V     205  0.6   0.9      *
4584          BD+34 2279 104179 62786                       115449.3+343526115957.2+340206177.83 76.96 6.50  +0.22 +0.15         A9III              -0.064+0.042      -008      101
4585  7    VirBD+04 2556 1041811191562960                   115449.5+041244115956.9+033919273.05 63.50 5.37   0.00  0.00         A1V                -0.017-0.006 +.016-003V?     65
4586          BD+81  389 104216  19673957I          5415    115505.5+812440120018.6+805111125.42 36.03 6.17  +1.61 +1.90 +1.04E  M2III              -0.068-0.038      +032V?                        *
4587          BD-09 3413 104304157041                       115536.5-095234120044.5-102646283.11 50.47 5.55  +0.77 +0.44 +0.36   G8-K0IV            +0.126-0.476 +.080+000V?
4588          BD-21 3443 104307180374        8389           115535.3-211650120042.5-215014287.61 39.52 6.28  +1.22               K0IV               +0.028+0.006      +004V          7.1  11.7
4589  8Pi  VirBD+07 2502 1043211191641311  S                115544.9+071019120052.4+063651270.30 66.23 4.66  +0.13 +0.11 +0.04   A5V                +0.001-0.030 +.021-010SBO    74  2.0   0.0      *
4590          BD-18 3295 1043371570422961                   115544.2-190608120051.2-193932286.92 41.63 5.26  -0.20               B1.5V              -0.021+0.013      +002SBO   119                 *
4591          BD-00 2520 104356138533                       115554.5-011233120101.8-014605278.14 58.66 6.31  +1.21 +1.06        gG8                 -0.015-0.070      +036V?
4592          CP-56 4954 104430239589                       115623.4-565647120129.0-573013296.24  4.72 6.16   0.00 -0.03         A0V                -0.073-0.015      -004
4593          BD+36 2230 104438 628022963                   115632.4+363604120139.5+360231169.73 76.17 5.59  +1.01 +0.80 +0.51   K0III              -0.091-0.081 +.028+030V?
4594 67    UMaBD+43 2179 104513 440022965  W       DP UMa   115702.2+433602120206.8+430244151.89 71.20 5.21  +0.26 +0.07 +0.11   F0Vam              -0.326+0.070 +.019+006V?     76  1.7 310.5AB   4*
4595          CP-84  371 1045552586323983  W                115719.1-850430120220.1-853754301.93-22.86 6.05  +1.29 +1.54         K3III              -0.065+0.001      +017           4.3  25.0
4596          CP-70 1454 104570256897                       115722.6-705556120228.6-712920299.03-08.99 6.42  +1.17 +1.14         K1III              -0.064+0.018      -009
4597          CP-68 1604 1046002516992966                   115731.0-683806120237.7-691132298.60-06.73 5.89  -0.08 -0.28         B9V                -0.039-0.007      +008
4598          BD-06 3499 104625138551    I                  115744.5-070738120251.6-074101282.54 53.25 6.22  +1.49 +1.83         K5                 -0.036+0.020      -008
4599   The1CruCP-62 2543 104671251705      W                115755.8-624522120301.5-631846297.53-00.95 4.33  +0.27 +0.04 +0.18   Am                 -0.150+0.008 +.021-002SB2O   33  9.3   4.5      *
4600          CD-41 6938 104731223193 449                   115828.7-415226120339.6-422603293.60 19.57 5.15  +0.41 -0.03 +0.25C  F6V                +0.331-0.114 +.039+036        0
4601          CP-73  924 104752256898                       115835.2-733926120344.3-741250299.64-11.65 6.44  +1.23 +0.97 +0.60C  G6III              -0.027+0.007      +009
4602  2    ComBD+22 2437 104827 82123        8406           115909.3+220058120416.6+212733238.22 77.86 5.87  +0.22 +0.11 +0.11   F0IV-V             +0.036-0.004D+.005+006SB  =< 54  1.4   3.6      *
4603   The2CruCP-62 2561 104841251717              The2 Cru 115910.1-623632120419.2-630956297.64-00.78 4.72  -0.08 -0.61 -0.10   B2IV               -0.020+0.008      +016SBO    56                 *
4604          CP-67 1896 104878251720                       115929.2-674619120438.7-681945298.62-05.85 5.35  -0.01 -0.17         A0IV               -0.054-0.012 +.016+025SB    187                 *
4605   Kap ChaCP-75  777 104902256899                       115936.1-755749120446.5-763109300.15-13.90 5.04  +1.49 +1.78 +0.76   K4III              -0.082+0.048 +.003-002
4606          BD+86  176 104904  19751642                   115942.8+860829120428.1+853514123.98 31.45 6.27  +0.59               F6V                -0.056+0.092      +008V?
4607          CP-60 3697 104933251723                       115947.7-602441120457.1-605809297.32  1.40 5.96  +1.67 +2.04 +0.92E  M2III              -0.038-0.032      -000
4608  9Omi VirBD+09 2583 104979119213 450I                  120006.9+091718120512.5+084359270.07 68.60 4.12  +0.98 +0.63 +0.49   G8IIIaCN-1Ba1CH1   -0.220+0.046 +.039-030V?   < 19:                *
4609          BD+77  461 104985  7500 451                   120010.3+772754120515.1+765421126.32 39.92 5.80  +1.01 +0.85         G9III              +0.147-0.091 +.014-020V
4610          BD+63  999 105043 15710        8417   5452    120036.7+632930120539.7+625559131.64 53.41 6.13  +1.17 +1.15         K2III              -0.050-0.075      -026V?         5.4   2.3      *
4611          CP-64 1791 105071251734                       120042.2-645924120553.2-653250298.24-03.09 6.33  +0.22 -0.47         B6Iab-Ib           -0.031-0.012      -013V
4612          CD-35 7694 1050782031191312  W        5456    120048.1-350814120556.7-354138292.64 26.27 6.23  -0.08               B8V                -0.031-0.002      -018V          7.7  25.4
4613          BD-02 3460 1050891385852967                   120052.5-023427120559.8-030754281.26 57.82 6.37  +1.00 +0.79        gG8                 -0.028-0.019      +017V?
4614          CP-67 1903 105138251737                       120107.2-680540120619.8-683903298.83-06.14 6.23  +1.24 +1.04         G3Ib               -0.014+0.020      +006
4615          CP-65 1788 105151251738      W                120111.7-650908120623.1-654233298.32-03.24 6.06  +0.65 +0.40         G8-K0III+A3-5V     -0.049+0.002      -004           0.0   0.1AB   3*
4616   Eta CruCP-63 2145 1052112517422969  W                120139.8-640321120652.9-643649298.18-02.15 4.15  +0.34 +0.03 +0.16   F2III              +0.033-0.037 +.053+009SB     65  7.5  44.0
4617          CP-74  880 1053402569052971                   120233.6-744839120749.8-752201300.13-12.73 5.18  +1.30 +1.37 +0.60C  K2II-III           -0.098+0.019 +.020-045
4618          CD-49 6813 105382239687      W     B          120254.3-500615120805.2-503941295.94 11.62 4.47  -0.15 -0.67 -0.18   B6IIIe            v-0.035-0.010      +017      127  1.9 268.6AB   3*
4619          CD-50 6688 105383239686      W     C          120253.9-501223120804.8-504548295.96 11.52 6.37  -0.05               B9V                -0.037+0.002      +016V?         1.9 268.6AB   3*
4620          CD-47 7396 105416223235                       120303.8-480808120814.7-484134295.62 13.57 5.34  -0.01 -0.04  0.00C  B9V                -0.025-0.017      +003V                         *
4621   Del CenCD-50 6697 105435239689 452  W     A Del Cen  120310.4-500956120821.5-504321296.00 11.57 2.60  -0.12 -0.90 -0.12   B2IVne            v-0.034-0.008 +.026+011V     181  1.9 268.6AB   3*
4622          CP-60 3777 105437251757                       120311.4-601726120824.6-605050297.71  1.59 6.22  +1.74 +1.88         K3-4II             -0.013+0.003      +010
4623  1Alp CrvCD-2410174 105452180505                       120315.2-241016120824.8-244344290.66 37.12 4.02  +0.32 -0.02 +0.18   F2III-IV           +0.086-0.040 +.070+004V      16                 *
4624          CD-43 7502 105509223241              V788 Cen 120343.3-434605120853.8-441934294.96 17.89 5.75  +0.24 +0.16 +0.11C  A2mA5-F2          v-0.046-0.049      -009SB2                       *
4625          CD-40 7128 105521223242              V817 Cen 120344.1-404029120854.5-411353294.39 20.93 5.48  -0.07 -0.62 -0.08   B3IV              e-0.024-0.006      +000V?    204                 *
4626 10    VirBD+02 2517 1056391192452972  W                120433.8+022734120941.3+015352279.54 62.86 5.95  +1.12 +1.14         K3III              +0.043-0.178 +.009+003           7.3  39.5      *
4627          BD+75  469 105678  7512                       120455.9+751304120947.3+743941126.63 42.17 6.35  +0.50               F6IV               +0.001+0.006      -019       34
4628          CD-34 7956 105686203183      W                120452.5-340853121002.5-344218293.36 27.40 6.17  +0.03 -0.02         A0V                -0.052-0.018D+.006-010           1.8   3.4
4629 11    VirBD+06 2559 1057021192492973                   120457.6+062147121003.4+054825276.30 66.52 5.72  +0.35 +0.17 +0.15   F2-6IIIm vs        -0.158+0.016 +.019-009V?                        *
4630  2Eps CrvBD-21 3487 105707180531 453I          5483    120458.8-220349121007.5-223711290.59 39.26 3.00  +1.33 +1.47 +0.64   K2.5IIIaBa0.2:     -0.071+0.014 +.027+005                          *
4631          CD-37 7714 1057762031952974                   120522.4-371846121033.8-375213294.09 24.30 6.06  +0.20               A5V                +0.047-0.035      -007
4632  3    ComBD+17 2446 105778 999731313                   120525.8+172157121031.6+164833258.30 76.00 6.39  +0.06 +0.10         A4V                -0.017+0.001      -012V
4633          BD+28 2084 105805 82166               5485    120541.3+275017121046.1+271653209.63 80.96 6.01  +0.11 +0.09         A4Vn               -0.009-0.014      -009SB2   172                 *
4634          CP-60 3812 105841251778                       120550.4-604314121104.9-611639298.10  1.22 6.08  +0.39 +0.09         F2-3IV             -0.134-0.018      -003
4635  3    CrvBD-22 3305 105850180546                       120554.8-230244121103.9-233609291.12 38.34 5.46  +0.06               A2V                -0.063-0.016 +.023+011      124
4636          CD-44 7845 105852223266      W                120550.8-445159121102.9-452522295.54 16.87 6.61  +1.08 +0.79 +0.51C  K0III              -0.018+0.009      +006V          3.8   2.3
4637          CD-50 6752 105920239735                       120619.7-504804121131.4-512134296.61 11.02 6.23  +0.82 +0.49         G6III+G:           -0.208-0.067      +026
4638   Rho CenCD-51 6455 105937239737                       120625.4-514842121139.1-522207296.78 10.03 3.96  -0.15 -0.62 -0.17   B3V                -0.035-0.016 +.032+015V     140                 *
4639          BD+82  356 105943  1991      W                120630.2+821558121100.0+814236124.71 35.28 6.00  +1.62              gK5                 -0.029+0.007      -027V?         2.0  66.7
4640  4    ComBD+26 2316 105981 821782976I                  120646.7+262539121151.2+255213218.78 81.06 5.66  +1.41 +1.61         K4III              -0.038-0.031      +022SBO                       *
4641 68    UMaBD+57 1359 106002 28291    I                  120645.8+573640121144.9+570316133.49 59.25 6.43R                    gK5                 +0.007-0.013      +035
4642          BD+29 2265 106022 82181                       120656.1+290540121201.2+283210201.47 81.17 6.49  +0.36 +0.01         F5V                +0.097-0.071      -015V?                        *
4643  5    ComBD+21 2398 106057 821822977                   120704.1+210556121209.3+203231247.11 78.87 5.57  +0.95               K0II-III           -0.020-0.019      -025V
4644          CP-62 2624 106068251790                       120703.7-622340121222.0-625703298.51-00.42 5.92  +0.29 -0.33         B8Ia-Iab           -0.007+0.002      -002                          *
4645          CP-69 1646 106111251791              S Mus    120723.8-693543121246.8-700907299.64-07.53 6.17  +0.84 +0.57         F6Ib              v-0.022-0.009      +011SB                        *
4646          BD+78  412 106112  7522 454                   120731.0+781019121211.9+773659125.64 39.31 5.14  +0.33 +0.10         A5m                +0.010+0.022 +.031-000SB1O   79                 *
4647          CD-33 8252 106198203245               5503    120801.7-333409121313.0-340732293.97 28.08 6.5   +1.66 +1.46         M4III              -0.030-0.006      +003V                         *
4648          CD-38 7581 106231203250                       120812.9-382222121325.3-385545294.89 23.35 5.76  -0.13               B4IV               -0.003-0.006      -043V?                        *
4649          CP-77  804 106248256915                       120818.1-780102121355.7-783425301.01-15.84 6.35  +1.24 +1.41         K2-3IIICNII        -0.033-0.003      +035
4650 12    VirBD+11 2440 106251 999972978                   120820.3+104908121325.9+101544273.17 70.90 5.85  +0.27 +0.11 +0.11   A2m                -0.091-0.010      +002       35
4651          CD-33 8257 106257203252      W                120825.1-331411121336.7-334734294.00 28.42 6.33  +0.08               A0V                -0.006-0.011      -011V?         1.8   1.6      *
4652          CD-45 7630 106321223297      W                120848.9-451004121402.6-454326296.13 16.65 5.31  +1.43 +1.59 +0.70C  K3III              -0.042+0.007 +.007+007SB         1.3   2.8      *
4653          CP-63 2203 106343251803                       120855.7-635108121416.8-642431298.93-01.83 6.22  +0.12 -0.82         B1.5Ia             -0.013+0.004      -007      100
4654          BD+54 1504 106478 283091314                   120946.0+535928121443.4+532605134.94 62.83 6.16  +1.04              gK0                 -0.019-0.015      +000                          *
4655          BD-20 3606 106485180602                       120949.5-201718121459.6-205039291.58 41.22 5.83  +1.05              gG7                 +0.001+0.004      +016V
4656   Del CruCP-58 4189 106490239791 455          Del Cru  120949.9-581133121508.7-584456298.23  3.79 2.80  -0.23 -0.91 -0.24   B2IV               -0.041-0.009 +.003+022V?    194                 *
4657          BD-09 3468 106516157168      W                121001.7-094342121510.6-101845288.49 51.55 6.11  +0.46 -0.13 +0.16E  F5V                +0.034-1.017 +.034+008V       8  6.8  73.3      *
4658          CD-41 7056 106572223309                       121019.4-412108121530.5-415447295.81 20.46 6.26  +1.01 +0.76         K0III              -0.331-0.175 +.025+025
4659          BD+71  610 106574  75322980I                  121022.6+704524121508.5+701200127.39 46.61 5.71  +1.19              gK2                 -0.023-0.026 +.013-014V?
4660 69Del UMaBD+57 1363 106591 28315 456  W        5513    121028.7+573518121525.6+570157132.57 59.42 3.31  +0.08 +0.07  0.00   A3V                +0.104+0.009 +.061-013V     177  6.6 189.6AB   3*
4661          BD-22 3322 106612180619        8481           121036.0-224749121547.0-232113292.42 38.79 6.54  +0.45               F3V                +0.039-0.030D+.016+018V          1.2   1.6      *
4662  4Gam CrvBD-16 3424 106625157176 457I          5515    121039.7-165912121548.4-173231290.99 44.50 2.59  -0.11 -0.34 -0.09   B8IIIpHgMn         -0.161+0.023      -004SB     41                 *
4663  6    ComBD+15 2436 1066611000122981                   121055.5+152721121600.2+145356267.16 75.25 5.10  +0.06 +0.08         A3V                -0.080-0.034 +.026+010SB    170
4664          CP-71 1323 106676256919                       121052.3-720329121623.5-723653300.28-09.92 6.22  -0.01 -0.06         A0V                -0.053-0.026      +028
4665          BD+73  549 106677  7533              DK Dra   121059.5+730628121541.4+723303126.66 44.32 6.29  +1.14               K0IIIe+K0IIIe      -0.014-0.035      -048SB2O                      *
4666  2    CVnBD+41 2284 106690 44097 458I   8489           121106.9+411301121607.6+403937148.99 74.63 5.66  +1.55 +1.96         M1III+F7V          +0.017-0.032D+.004-016SB         3.1  11.4      *
4667  7    ComBD+24 2443 106714 822112982I                  121117.0+243005121620.5+235643232.90 81.47 4.95  +0.97 +0.68 +0.48   G8IIIFe-0.5        -0.027-0.007 +.006-028     < 17                 *
4668          BD+33 2213 106760 629282983I S       Var?     121128.5+333713121630.1+330341172.65 80.40 5.00  +1.14 +1.07 +0.58   K0.5IIIb           -0.053-0.111 +.022-042SB1O < 17                 *
4669          CP-64 1844 106797251826                       121141.2-650812121706.0-654134299.41-03.06 6.06  +0.03 -0.05         A0V                -0.057-0.001      +021V
4670          BD-15 3442 106819157184               5527    121153.8-160817121703.3-164137291.17 45.39 6.05  +0.10 +0.12         A2IV               -0.048+0.003      -012
4671   Eps MusCP-67 1931 106849251830    I:        Eps Mus  121209.9-672415121734.1-675739299.75-05.30 4.11  +1.58 +1.55 +1.69   M5III              -0.244-0.025 +.046+007SB                        *
4672          BD+54 1510 106884 28327    I                  121233.6+534453121729.5+531128134.24 63.19 5.81  +1.30 +1.52         K6III              +0.034-0.046      -041
4673          BD+29 2275 106887 82219        8501           121228.3+292930121730.5+285614197.38 82.30 5.70  +0.16 +0.12 +0.05   A4m:               -0.042+0.037      -007V      75  4.9   8.6      *
4674   Bet ChaCP-78  741 106911256924 459           5532    121228.6-784525121820.7-791844301.34-16.54 4.26  -0.12 -0.51 -0.10   B5Vn               -0.048+0.017      +023      255                 *
4675          CD-35 7842 106922203319      W                121233.8-353217121747.3-360538295.36 26.28 6.15  +0.01               A0V                -0.037-0.006D+.004-014           0.3   0.5
4676          BD+15 2442 106926100023                       121239.1+154205121744.3+150839268.05 75.68 6.34  +1.37               K4III              +0.044-0.063      -042SB
4677          BD-03 3262 106975138703        8505B          121301.3-032357121809.1-035716287.17 57.89 6.99  +0.36 +0.03         F3V                -0.017+0.016D+.016+001           0.4  20.1      *
4678          BD-03 3263 106976138704        8505A          121301.7-032338121809.6-035655287.17 57.90 6.54  +0.33 +0.01         F2V                -0.013+0.024D+.016-001           0.4  20.1      *
4679   Zet CruCP-63 2235 106983251841      W                121300.9-632650121826.1-640011299.33-01.36 4.04  -0.17 -0.69 -0.19   B2.5V              -0.050-0.010      +016      113  9.0  33.8      *
4680          BD+31 2350 107054 629432984                   121329.1+304828121831.6+301457187.60 82.14 6.23  +0.30 +0.02         A9.5III            +0.077-0.116      -018V     140
4681 13    VirBD+00 2920 107070138710                       121332.6-001353121840.3-004714285.85 60.99 5.90  +0.17 +0.13         A5Vn               +0.029-0.016      -014V     195
4682          CP-54 5113 1070792398382985  W        5544    121339.7-543514121859.7-550835298.27  7.43 5.00  +1.59 +1.95 +1.06   M1III              -0.076-0.014 +.010-007SB         7.0  35.7      *
4683          BD+87  107 107113  2012                       121355.8+865930121651.4+862610123.56 30.65 6.33  +0.43 -0.08         F4V                +0.210-0.003 +.037-006
4684          BD+26 2326 107131 82237              FM Com   121359.5+263350121902.1+260028219.89 82.67 6.48  +0.18 +0.09 +0.07   A5-7mIV-V          -0.004-0.022      +001SB    175                 *
4685  8    ComBD+23 2448 107168 82239                       121416.1+233525121919.1+230205240.56 81.66 6.27  +0.17 +0.15 +0.02   A5III vs           -0.022-0.012      +001V?  =< 12                 *
4686          BD+88   71 107192  20103958                   121423.5+881515121520.3+874200123.35 29.40 6.28  +0.35 -0.01         F2V                -0.028+0.054      -004V      67
4687          BD+75  470 107193  75402986                   121420.8+754256121849.9+750938125.72 41.79 5.38  -0.02 -0.05         A1V                -0.040+0.002 +.018-003V     217
4688  9    ComBD+28 2106 107213 82244                       121429.0+284256121929.6+280925202.80 82.85 6.33  +0.50 +0.08         F8V s              -0.197-0.129 +.025-008V       8                 *
4689 15Eta VirBD+00 2926 107259138721 460  D        5555    121447.3-000640121954.4-004001286.40 61.19 3.89  +0.02 +0.06  0.00   A2IV               -0.063-0.018 +.016+002SB2O   34  0.5   0.0     3*
4690  3    CVnBD+49 2130 107274 441271316I                  121453.2+493221121948.7+485903136.46 67.31 5.29  +1.66 +1.97 +1.06   M0III              -0.012+0.007      +008V
4691          BD-21 3511 107295180695        8515           121500.0-213711122010.7-221032293.46 40.12 5.97  +0.82               G3IV               -0.109-0.025D+.008-001V?         1.8   0.3
4692          CP-65 1842 107301251854                       121458.8-651714122028.0-655034299.77-03.16 6.21  -0.04 -0.15         B9V                -0.056-0.015      -008
4693          BD+27 2114 107325 82250               5559    121518.0+271040122019.7+263710215.35 83.05 5.54  +1.09 +1.07 +0.54   K2III-IV           -0.065-0.107      -013SBO                       *
4694          BD+26 2329 107326 82249               5557    121516.5+263323122017.7+260007220.37 82.95 6.15  +0.30 +0.08 +0.14   F0IV               -0.145+0.019 +.060+008V     109                 *
4695 16    VirBD+04 2604 1073281193411317I W                121516.2+035210122021.0+031845284.27 65.05 4.96  +1.16 +1.15 +0.61   K0-IIIbFe-1        -0.292-0.065 -.002+036V?   < 19: 0.0   0.6     3*
4696  5Zet CrvBD-21 3514 107348180700        8517           121522.8-213936122033.7-221257293.58 40.09 5.21  -0.10 -0.38 -0.13   B8Vne              -0.095-0.028      -006SB         8.4  11.2      *
4697 11    ComBD+18 2592 1073831000532987I   8521           121539.9+182044122043.0+174734264.15 78.28 4.74  +1.01 +0.79 +0.52   G8+III             -0.111+0.092      +042V    < 19: 8.0   9.1      *
4698          BD+27 2115 107398 82254        8519A          121539.4+273646122041.3+270317211.78 83.16 7.13  +0.36 -0.01         F3V+F3V            -0.001-0.114D+.008-015           0.1   8.5      *
4699          BD-12 3614 1074181572262988I W        5564    121545.8-130039122055.7-133356291.67 48.64 5.14  +1.05 +0.89 +0.52   K1III              -0.006+0.014 +.020+013V?   < 19: 8.1  48.6
4700   Eps CruCP-59 4188 1074462518622989           5568    121557.6-595055122121.6-602404299.23  2.25 3.59  +1.42 +1.63 +0.74   K3-4III            -0.175+0.087 +.026-005                          *
4701 70    UMaBD+58 1371 107465 28346    I                  121600.2+582516122050.8+575150130.79 58.79 5.55  +1.43 +1.71         K5III              +0.041-0.073      -043V
4702          CP-55 5019 107543239880                       121633.5-554910122157.5-562229298.83  6.26 5.92  +1.57 +1.64         K4III+F:           -0.008-0.012      -001
4703   Zet2MusCP-66 1747 1075662518662990  W                121633.6-665801122207.3-673119300.13-04.81 5.15  +0.19 +0.17         A5V                -0.031-0.003 +.007-017V?     86  5.4  32.4
4704   Zet1MusCP-67 1939 107567251868                       121636.7-674504122211.9-681827300.22-05.59 5.74  +1.04 +0.82         K0III              -0.016-0.057      +020V
4705          BD+25 2498 107655 82271                       121709.2+251943122210.8+244626231.00 83.02 6.19  -0.01 -0.04 -0.02   A0V                -0.062-0.008      -004       53                 *
4706          CP-56 5202 107696239901              Var?     121724.2-570716122249.3-574034299.10  4.98 5.39  -0.10 -0.41         B9V                -0.046-0.013      -005V                         *
4707 12    ComBD+26 2337 107700 822731318    8530   5581    121728.7+262404122230.3+255046222.50 83.40 4.81  +0.49 +0.26 +0.33   G0III-IV+A3V       -0.011-0.009 +.014+001SB2O   35  3.7  65.3AC   3*
4708 17    VirBD+06 2599 107705119360        8531   5579    121726.9+055142122232.0+051820284.15 67.12 6.40  +0.60 +0.08 +0.28   F8V                -0.166-0.053 +.027+005    =< 10  3.0  20.6      *
4709          CP-85  343 107739258644                       121737.1-853546122537.5-860902302.46-23.30 6.33  +1.08 +0.89         K0III              -0.017-0.001      +025
4710          CP-66 1752 107773251877                       121750.9-670503122313.8-673754300.26-04.91 6.36  +0.89 +0.53         G8-K0IV            -0.759+0.254 +.013+031
4711  6    CrvCD-2410314 107815180747                       121808.8-241708122321.6-245026294.89 37.59 5.68  +1.16              gK1                 -0.019-0.010      -002
4712          CD-34 8117 1078322034202992                   121819.7-345129122335.4-352446296.56 27.11 5.32  -0.08               B9III              -0.036-0.003      -010       45                 *
4713          CD-38 7700 107833203421                       121819.6-384453122336.9-391811297.08 23.25 6.40  +0.29               F2V                -0.009-0.016      -013
4714          CD-38 7701 107860203424                       121828.0-382123122344.9-385440297.06 23.64 5.79  -0.08               B9V                -0.036 0.000      -008
4715  4    CVnBD+43 2218 107904 441552993          AI CVn   121851.9+430548122347.0+423234141.22 73.59 6.06  +0.33 +0.18         F3III-IV           -0.078+0.015      -000SB     73                 *
4716  5    CVnBD+52 1626 107950 283662994I         Var      121910.0+520658122401.5+513344133.05 65.02 4.80  +0.87 +0.62 +0.44   G6IIIBa0.2         +0.014+0.013 +.038-012SB   < 17                 *
4717 13    ComBD+26 2344 107966 82291              GN Com   121917.5+263911122418.5+260555221.04 83.85 5.18  +0.08 +0.11 +0.02   A3V                -0.016-0.011 +.009+001V      54                 *
4718          CD-40 7281 107998223417      W                121926.7-404941122444.7-412303297.57 21.21 6.25  +1.18 +1.06         K2-3III            -0.064-0.071      +001           2.4  10.0      *
4719          BD+26 2345 108007 82293        8539           121925.6+260813122426.7+253458225.75 83.76 6.42  +0.27 +0.08 +0.10   A7V+F4V            -0.014-0.004D+.011-006V     148  1.0   1.4AB   3*
4720          CP-65 1862 108054251893                       121942.5-651246122517.3-654614300.25-03.03 6.30  +0.96 +0.74         G8-K0IV            -0.080-0.121      +025
4721          CD-41 7163 1080632234262995                   121950.4-415734122508.5-423052297.78 20.09 6.11  +0.65               G5III+F-G          -0.130-0.022      +034
4722          BD-10 3467 108107157272                       122002.1-110320122511.7-113637292.75 50.74 5.95  +0.03               A1V                -0.065-0.023      -003V?    160
4723          CD-27 8670 1081101807861319                   122004.1-271142122518.4-274457295.89 34.77 6.09  +1.27               K1III              +0.005-0.001      +010V?
4724          CD-34 8146 108114203450               5610    122005.4-343756122521.8-351111296.93 27.38 5.73  -0.06               B9III             v-0.034-0.006      -011                          *
4725          BD+24 2455 108123 82297                       122013.3+242853122515.1+235534239.94 83.28 6.03  +1.10 +1.00 +0.56   K0III              +0.058-0.038 +.007-005V?                        *
4726 71    UMaBD+57 1373 108135 28375    I                  122016.4+571956122503.2+564639130.16 59.98 5.81  +1.62 +1.83 +1.00E  M3IIIb             -0.014-0.018      -017V?
4727          BD+64  896 108150 158012996  W                122026.5+642124122506.4+634810127.76 53.08 6.32  +0.91               G8III              -0.015+0.002      -004           3.7  52.3
4728  6    CVnBD+39 2521 108225 63000 461I                  122055.4+393424122550.9+390107145.50 76.97 5.02  +0.96 +0.73 +0.46   G9III              -0.078-0.032 +.030-004V?   < 19:
4729          CP-62 2742 108250251903      W     C          122057.1-623404122630.9-630721300.12-00.39 4.86  -0.12 -0.59         B4IV               -0.040-0.025      +027SBO   137  0.4   4.4AB   3*
4730   Alp1CruCP-62 2745 108248251904 462  W     A          122102.0-623241122635.9-630557300.13-00.36 1.33  -0.24 -1.03         B0.5IV             -0.036-0.012D+.008-011SBO   117  0.4   4.4AB   3*
4731   Alp2CruCP-62 2745 108249            W     B          122102.5-623243122636.5-630558300.13-00.36 1.73  -0.26 -0.95         B1V                -0.034-0.007D+.008-001      197  0.4   4.4AB   3*
4732          CD-50 6975 108257239948      W                122107.5-505347122631.6-512703298.98 11.23 4.82  -0.14 -0.64 -0.16   B3Vn               -0.044-0.017      +005V     312  8.8  21.7      *
4733 14    ComBD+28 2115 108283 823102997I                  122124.0+274920122624.1+271606210.06 84.43 4.95  +0.27 +0.18 +0.15   F0p                -0.013-0.008 +.014-004SB    227                 *
4734          CD-48 7426 108309223443                       122131.2-482126122648.2-485448298.78 13.76 6.26  +0.68 +0.22         G3-5V              -0.633-0.082 +.040+030                          *
4735          CD-32 8713 108323203477 463                   122135.3-321632122651.7-324948296.99 29.76 5.55  +0.01               B9V                -0.005-0.027      -015
4736          CP-63 2283 108355251911                       122148.9-631406122724.6-634721300.28-01.04 6.00  +0.07               B8IV               -0.035-0.018      +042
4737 15Gam ComBD+29 2288 108381 823132999I                  122157.2+284928122656.3+281606199.63 84.46 4.36  +1.13 +1.15 +0.51   K1IIIFe0.5         -0.083-0.080 +.003+004V?   < 17                 *
4738 16    ComBD+27 2134 108382 82314                       122159.3+272246122659.3+264932214.72 84.55 5.00  +0.08 +0.13 +0.02   A4V                -0.006-0.011 +.024+002V?     89                 *
4739          CP-58 4289 108396239960              BL Cru   122158.0-582618122728.7-585931299.84  3.74 5.50  +1.45 +1.58 +1.88   M4-5III            -0.031+0.005 +.007+072                          *
4740          BD+72  565 108399  75632998                   122203.6+722903122624.2+715547125.67 45.06 6.24  +1.06 +0.85        gG8                 -0.153-0.023 +.005+006V
4741          BD+09 2628 108471119413                       122236.8+090951122742.1+083637284.96 70.65 6.37  +0.93 +0.68         G8III              +0.018-0.010      -006
4742          BD-15 3471 108477157299                       122237.8-160443122749.4-163755294.80 45.87 6.35  +0.84 +0.50         G3III              -0.018+0.003      -008
4743   Sig CenCD-49 7115 108483223454 464                   122237.8-494036122802.4-501350299.10 12.47 3.91  -0.19 -0.78 -0.20   B2V                -0.030-0.015      +008V     245                 *
4744          CP-63 2297 108501251920                       122242.0-634714122818.9-642029300.43-01.58 6.04  +0.02 -0.03         A0Vn               -0.068-0.027      +009
4745 73    UMaBD+56 1598 108502 283943000I                  122249.7+561559122735.1+554246129.89 61.09 5.70  +1.55               M2IIIb             -0.021-0.012      +017V?
4746          BD-03 3298 1085061387983001          FT Vir   122243.6-040343122751.6-043655291.87 57.75 6.22  +0.44 +0.08         A8n                -0.086+0.006      -012SB    139                 *
4747          CP-61 3209 108530251924                       122251.1-611429122825.5-614743300.22  0.96 6.22  +1.26 +1.32         K2III              -0.034-0.014      -014
4748          CD-38 7753 1085412035081320                   122303.3-382915122822.5-390229298.05 23.61 5.44  -0.08               B8V                -0.023-0.017      +005      174
4749          CP-55 5084 108570239977      W                122306.4-555053122833.4-562428299.75  6.33 6.15  +0.92 +0.64         K0-1III            -0.240-0.220 +.021+008           5.8  49.0AC   3*
4750          BD+27 2138 108642 82326                       122338.5+264649122838.1+261336221.68 84.83 6.54  +0.17 +0.12 +0.09   A2m                -0.020-0.014      +002SB1O=< 12                 *
4751          BD+26 2353 108651 82328        8568BC         122344.8+262711122844.6+255357225.26 84.78 6.65  +0.22 +0.08 +0.08   Am(A2/A9V/F0)      -0.018-0.021      -002SBO =< 12  1.4 145.4AB   3*
4752 17    ComBD+26 2354 108662 82330        8568A AI Com   122355.0+262759122854.7+255446225.22 84.82 5.29  -0.05 -0.12 -0.08   A1IVp              -0.025-0.020 +.021-003V      18  1.4 145.4AB   3*
4753 18    ComBD+24 2464 108722 82333                       122426.9+243943122926.9+240632243.07 84.20 5.48  +0.43 +0.09         F5III              -0.031-0.001      +025SB1O   93                 *
4754          CP-55 5097 108732239999                       122423.2-555820122953.9-563129299.95  6.22 5.80  +1.56 +1.82         M1III              -0.022+0.019      +042V
4755          CD-41 7219 108759223471               5655    122436.4-411057122957.9-414410298.64 20.96 6.02  +1.52 +1.54         M2II-III           +0.008-0.025      +003
4756 20    ComBD+21 2424 108765 82336 466                   122441.8+212700122943.2+205346263.27 82.04 5.69  +0.07 +0.09         A3V                +0.024-0.032      -005SB2
4757  7Del CrvBD-15 3482 108767157323 465I   8572A  5656    122441.3-155731122951.9-163056295.48 46.05 2.95  -0.05 -0.09 -0.04   B9.5V              -0.210-0.138 +.024+009V     148  5.3  24.4      *
4758          BD-12 3647 108799157326        8573           122455.4-125021123004.8-132335294.98 49.15 6.35  +0.56 +0.05         G0V                -0.248-0.041 +.049+000V?         2.7   1.5AB   4*
4759          BD-22 3383 108821180850    I          5662    122503.3-230837123017.5-234148296.71 38.92 5.63  +1.67 +2.03         M0III              -0.020-0.003      -011V
4760 74    UMaBD+59 1444 108844 28405 467                   122517.1+585721122957.3+582421128.32 58.50 5.35  +0.20 +0.14 +0.06   A5Del Del          -0.062+0.091 +.039+007V      87
4761  7    CVnBD+52 1631 108845 28407      W                122519.1+520515123002.9+513208130.89 65.26 6.21  +0.51 +0.03         F6-8V              -0.289+0.024 +.035+019V?  =< 10  2.8 229.0AC   3
4762 75    UMaBD+59 1446 108861 28408                       122523.4+591915123004.3+584603128.18 58.14 6.08  +0.98               G8III-IV           +0.031-0.024      -017
4763   Gam CruCP-56 5272 108903240019 468I W     A  5672    122537.0-563312123109.9-570648300.17  5.65 1.63  +1.59 +1.78 +1.41   M3.5III            +0.023-0.262      +021           5.1 110.6AB   3*
4764   Gam CruCP-56 5274 108925240022      W     B          122543.9-563142123116.7-570452300.18  5.68 6.42  +0.16               A3V                +0.002 0.000      +002           5.1 110.6AB   3
4765  4    DraBD+70  700 108907 15816    I         CQ Dra   122543.8+694519123006.7+691204125.75 47.81 4.95  +1.62 +1.81 +1.05E  M3IIIa             -0.059-0.052 +.016-013SB                        *
4766 21    ComBD+25 2517 108945 82346              UU Com   122600.9+250712123100.6+243402240.93 84.74 5.46  +0.05 +0.10 -0.03   A2p               v-0.012-0.009      +000V      63                 *
4767          BD+53 1554 108954 28413                       122604.8+533726123050.1+530436129.94 63.77 6.21  +0.54 +0.07         F8-G0V             +0.015+0.181      -021V?  =<  6
4768          CP-58 4344 108968240027              BG Cru   122605.0-585217123140.3-592526300.42  3.35 5.48  +0.63 +0.38         F7Ib-II           v-0.018-0.004 +.010-020SB                        *
4769          CP-72 1261 108970256954                       122607.2-722654123210.0-730006301.50-10.18 5.88  +1.11 +1.05 +0.49C  K1III              +0.038-0.024      +005
4770          BD+08 2609 1089851194533002I                  122616.5+080923123121.4+073615288.31 69.90 6.05  +1.52 +1.89         K5                 -0.018+0.010      -017
4771          CP-62 2805 109000251946                       122615.8-625714123155.8-633022300.76-00.72 5.95  +0.27               A8III              -0.063+0.008      +004
4772          BD-04 3296 109014138832                       122630.1-043004123138.7-050309293.74 57.47 6.19  +1.04 +0.86        gG9                 -0.038+0.031      +002V?
4773   Gam MusCP-71 1336 109026256955 469                   122629.4-713450123228.0-720759301.46-09.32 3.87  -0.15 -0.62 -0.17   B5V                -0.058-0.002      +003V?    188                 *
4774          CD-31 9746 109074203567                       122646.4-315852123204.5-323201298.21 30.16 6.46  +0.20 +0.18         A3V                -0.015-0.008      -009
4775  8Eta CrvBD-15 3489 109085157345               5690    122654.9-153832123204.2-161146296.19 46.42 4.31  +0.38 +0.01 +0.18   F2III-IV           -0.426-0.061 +.052-004SB2    59                 *
4776          BD-13 3552 109141157350                       122725.4-131820123236.0-135133295.99 48.76 5.74  +0.40 -0.02         F2V                -0.144-0.051      -001V
4777 20    VirBD+11 2473 1092171001463004                   122759.1+105051123302.9+101744287.63 72.62 6.26  +0.95 +0.70 +0.34E  G8III              -0.051+0.005      +001                          *
4778          BD-18 3416 109238157359               5715    122808.7-191424123322.4-194731297.13 42.88 6.26  +0.29               F2V                -0.013-0.002      +005       90
4779          BD-12 3659 1092721573611321                   122823.0-121648123334.3-124949296.18 49.81 5.58  +0.86 +0.45         G8III              -0.016+0.052      -016V?
4780 22    ComBD+25 2523 109307 82378                       122835.2+245006123334.2+241659247.20 85.07 6.29  +0.11 +0.10 +0.03   A4Vm               -0.018-0.004      +002V       8                 *
4781 21    VirBD-08 3372 109309138845                       122837.0-085401123346.8-092707295.66 53.17 5.48  -0.03 -0.10         A0V                -0.080+0.007 +.019-011V     110
4782          CD-49 7195 109312223516      W                122832.1-492124123359.2-495434300.05 12.87 6.38  +0.46               F3III-IV           -0.163-0.040      +018           6.5   7.8
4783          BD+34 2332 109317 630701322I                  122843.4+334800123338.9+331451153.97 82.78 5.42  +1.00 +0.83 +0.50   K0IIICN-1          +0.020-0.033 +.033-020V    < 19:
4784          BD+34 2333 109345 63072                       122852.1+335612123347.4+332305153.19 82.68 6.24  +1.05 +0.90 +0.52   K0III              +0.002-0.008 +.056-043V
4785  8Bet CVnBD+42 2321 109358 44230 470I S        5725    122859.6+415403123344.5+412127136.10 75.32 4.26  +0.59 +0.05 +0.31   G0V                -0.705+0.292 +.117+007SB1O=<  3                 *
4786  9Bet CrvBD-22 3401 109379180915 471I          5729    122907.9-225037123423.2-232348297.87 39.31 2.65  +0.89 +0.60 +0.44   G5II               +0.002-0.054 +.034-008V    < 17                 *
4787  5Kap DraBD+70  703 109387  7593 472I         Kap Dra  122912.9+702022123329.0+694718125.21 47.26 3.87  -0.13 -0.57 -0.08   B6IIIpe           v-0.059+0.012 +.013-011SBO   249                 *
4788          CD-43 7755 109409223527      W                122917.5-440657123442.3-444024299.80 18.10 5.77  +0.70 +0.28         G1V                -0.083-0.215 +.021+018           4.0   1.0
4789 23    ComBD+23 2475 109485 823901323  W                122952.1+231048123451.1+223745262.15 84.14 4.81   0.00 -0.01 -0.02   A0IV               -0.065+0.021 +.004-016SB2    66  2.4   0.4      *
4790          CP-61 3298 109492251962    I                  122950.8-611717123529.0-615031301.05  0.97 6.22  +0.73 +0.30         G3III              -0.304-0.084      +006
4791 24    ComBD+19 2584 109510100159        8600B  5745    123005.3+185540123506.3+182238278.83 80.48 6.56  +0.25 +0.11 +0.16   A9Vm               -0.003+0.024 +.002+005SB2O=< 25  1.5  20.2      *
4792 24    ComBD+19 2584 109511100160 473I   8600A  5748    123006.8+185539123507.8+182237278.86 80.48 5.02  +1.15 +1.11         K2III              -0.006+0.023 +.002+004    =< 25  1.5  20.2      *
4793          BD+22 2490 109519 82394    I                  123008.3+222559123508.1+215253266.81 83.58 5.85  +1.22               K1III              +0.014-0.019      -014V
4794          CD-40 7376 1095362235423005          Var?     123022.5-402815123545.5-410119299.75 21.75 5.13  +0.22  0.00 +0.11   A7III              -0.107-0.001 +.024-011       94                 *
4795  6    DraBD+70  705 109551  7600    I                  123030.2+703423123444.0+700119125.02 47.04 4.94  +1.31 +1.15         K3III*             -0.032 0.000      +005V                         *
4796          CD-39 7717 109573203621      W                123038.1-391904123601.2-395212299.72 22.91 5.80  +0.01               A0V                -0.034-0.037      +006           6.1  40.0AC   3
4797          BD-19 3521 109585180937              TU Crv   123044.0-195830123558.6-203138298.04 42.20 6.20  +0.33 +0.08         F0III              +0.014-0.037      -002V      67                 *
4798   Alp MusCP-68 1702 109668251974 474  W       Alp Mus  123113.0-683504123711.0-690808301.66-06.30 2.69  -0.20 -0.83 -0.24   B2IV-V             -0.048-0.013      +013V     147 10.1  29.6      *
4799 25    VirBD-05 3535 1097041388731324                   123138.2-051651123647.4-054955296.26 56.85 5.87  +0.07 +0.09         A3V                -0.028-0.018      -006V
4800          BD+60 1406 109729 28444    I         T UMa    123150.3+600216123623.3+592913126.49 57.54 5.5 H                     M4IIIe             -0.025-0.011      -091V                         *
4801 25    ComBD+17 2504 1097421001763007I                  123157.4+173826123658.3+170522283.77 79.42 5.68  +1.41 +1.75         K5III              -0.037-0.016      -008
4802   Tau CenCD-47 7745 109787223560                       123213.8-475926123742.2-483228300.59 14.27 3.86  +0.05 +0.03 +0.04   A2V                -0.184-0.005 +.024+005      281
4803          CD-26 9233 109799180965        8612           123224.0-263510123742.2-270820299.17 35.63 5.45  +0.32 +0.03 +0.19   F1IV               +0.076-0.091 +.037-001        0  5.9   1.4      *
4804          CP-74  955 109857256967      W                123249.3-744913123914.5-752210302.14-12.52 6.49  +0.08 -0.24 +0.08C  B8Vne              -0.032+0.031      -020           2.0   2.0      *
4805          BD+04 2631 109860119503                       123258.5+034958123804.4+031657294.73 65.94 6.33  +0.01 +0.01         A1V                -0.029-0.009      -008V?     24
4806          CP-66 1861 109867251980      W                123256.2-663834123852.5-671135301.71-04.35 6.25  +0.06 -0.85         B1Ia              t-0.006-0.002      -016       88  5.7  17.0      *
4807          BD+02 2560 1098961195083009I         FW Vir   123316.3+022419123822.4+015117295.32 64.53 5.71  +1.60 +1.83 +1.42   M3+IIIbCa0.5       -0.080-0.018 +.029-016V?                        *
4808          BD+07 2561 109914119509    I         R Vir    123325.4+073218123830.0+065918293.68 69.63 7.08  +1.44 +1.28 +1.74   M4.5IIIe           -0.026-0.004 +.014-026V                         *
4809          BD-17 3668 1099311574153010                   123331.2-174203123844.6-181501298.71 44.52 6.00  +0.29              dA6                 -0.119+0.025      -013V
4810          CD-29 9845 1099602036663011           5822    123344.1-295220123903.5-302520299.77 32.37 5.89  +1.21               K2-3III            -0.034-0.006      -013V
4811  9    CVnBD+41 2312 109980 44265                       123357.6+412529123846.3+405228132.88 76.01 6.37  +0.18 +0.07         A7Vn               -0.026-0.016      -015      220
4812          BD+23 2479 109996 82420      W                123404.1+231235123902.1+223934270.09 84.72 6.38  +1.10               K1III              -0.050-0.016      -027V          6.0  32.8
4813 26Chi VirBD-07 3452 110014138892 475I W                123405.0-072643123914.8-075944297.70 54.75 4.66  +1.23 +1.39 +0.61   K2III-IIIbCN1      -0.076-0.025 +.020-020     < 17  4.3 173.1AB   4*
4814          CP-65 1941 110020251987              FH Mus   123400.4-655740123955.6-663042301.78-03.67 6.26  -0.05 -0.20         B8V                -0.059-0.021      -003V                         *
4815 26    ComBD+21 2439 110024 824213012                   123408.8+213645123907.3+210345277.43 83.32 5.46  +0.96               G9III              -0.081-0.013      -021SBO                       *
4816          BD+36 2295 110066 631183013          AX CVn   123425.0+363006123916.9+355707138.51 80.81 6.45  +0.06 +0.03         A0pSrCrEu          +0.022-0.006      -015V?     13                 *
4817          CD-39 7748 110073203681               5835    123427.6-392613123952.5-395915300.53 22.83 4.64  -0.08 -0.42 -0.09   B8II/III           -0.047-0.030      +015SB1    57                 *
4818          CD-45 7944 1102872236011325                   123553.2-453553124123.0-460844301.12 16.69 5.84  +1.52 +1.72         K4III              -0.069+0.057      +007
4819   Gam CenCD-48 7597 110304223603      W                123559.9-482438124131.0-485735301.26 13.88 2.17  -0.01 -0.01  0.00   A1IV               -0.189-0.005 +.016-006SB     81  0.1   1.5AB   3*
4820          CP-68 1731 110311251996              R Mus    123558.4-685131124205.1-692427302.10-06.55 6.33  +0.78 +0.54         F7Ib-G2           v-0.004-0.002      +004                          *
4821          BD-12 3676 110317157447        8627B  5855    123604.3-122754124116.0-130049299.10 49.78 6.08H                     F3Vn               -0.113+0.019D+.014-014SBO        0.1   5.4AB   3*
4822          BD-12 3676 110318157448        8627A  5855    123604.6-122758124116.2-130054299.10 49.78 5.98H +0.42 +0.10         F5V                -0.127+0.008D+.014-011SB2O       0.1   5.4AB   3*
4823          CP-59 4393 110335240161               5858    123611.2-590813124156.6-594109301.73  3.16 4.93  -0.04 -0.39 -0.01   B6IVe              -0.025-0.002      +013SB    185:                *
4824 27    VirBD+11 2484 110377100207      W       GG Vir   123632.0+105830124134.4+102535294.55 73.14 6.19  +0.19 +0.10         A7Vn               -0.109+0.004      +009SB    153  3.8  85.5      *
4825 29Gam VirBD-00 2601 110379138917    I   8630A  5859    123635.5-005403124139.6-012658297.85 61.33 3.65H +0.36 -0.03 +0.19   F0V                -0.565+0.012 +.099-020SB     28  0.1   4.1AB   4*
4826 29Gam VirBD-00 2601 110380138917    I   8630B  5859    123635.5-005403124139.6-012658297.85 61.33 3.68H                     F0V                -0.565+0.012 +.099-020V      30  0.1   4.1AB   4*
4827          BD-18 3442 110385157451                       123635.2-191238124149.2-194531299.84 43.06 6.03  +0.39 +0.09         F3-5III            -0.212+0.027      -003       82                 *
4828 30Rho VirBD+11 2485 1104111002111326          Rho Vir  123649.3+104712124153.1+101408294.88 72.96 4.88  +0.09 +0.03 +0.02   A0V                +0.084-0.091      +002SB    173                 *
4829 31    VirBD+07 2568 110423119538        8633   5867    123653.0+072120124157.1+064824296.18 69.55 5.59   0.00 -0.02         A2V                -0.068-0.012 +.012+004V     136  5.8   4.0
4830          CP-62 2898 1104322520023015          BZ Cru   123658.1-623036124250.3-630331301.96-00.20 5.31  +0.27 -0.79 +0.21   B1IIIe            v-0.015 0.000      +035SB    275                 *
4831          CD-48 7608 110458223614                       123703.1-481549124235.4-484847301.43 14.03 4.66  +1.09 +1.01 +0.57   K0III              -0.127-0.032 +.025-012                          *
4832          CP-55 5194 110461240176                       123708.8-552353124249.6-555650301.72  6.90 6.08  -0.03 -0.19         B9V                -0.047-0.019      +015V?
4833 76    UMaBD+63 1026 110462 15871 478                   123711.8+631543124133.9+624247124.87 54.38 6.07  +0.01 +0.07         A2III              -0.032-0.017      -004
4834          CP-55 5197 110506240179                       123727.6-553739124309.1-561034301.77  6.68 6.00  -0.08 -0.30         B7-8V              -0.041-0.010      +010
4835          CP-58 4453 110532240183      W                123743.2-582117124328.2-585411301.90  3.95 6.40  +1.09 +1.00         K0-1III            -0.085-0.011      +017           3.3  29.8AC   3
4836          CD-39 7785 110575203746                       123758.9-393746124326.3-401040301.28 22.67 6.44  +0.25 +0.04 +0.14   A8V:               +0.027-0.014      -005                          *
4837          BD-00 2603 110646138933                       123829.8-010137124338.1-013437298.88 61.23 5.93  +0.86 +0.46 +0.49   G8IIIp             +0.050-0.076      +001                          *
4838          CD-35 8155 1106532037563016                   123833.8-354804124358.7-362057301.25 26.50 6.39  -0.06               A0V                +0.013-0.007      -001
4839          CD-27 8832 110666181063 479I                  123840.6-274630124400.5-281926300.95 34.52 5.48  +1.34 +1.50         K4III              -0.027-0.038      +007V
4840          BD+61 1312 110678 15877              Var      123841.0+614210124304.2+610920124.73 55.94 6.38  +1.26 +1.39         K0                 -0.047+0.026      -006                          *
4841          CP-68 1745 110716252012                       123852.9-681659124501.7-684952302.35-05.97 6.16  +0.69 +0.38         F6Ia               -0.032-0.015      -003V
4842   Iot CruCP-60 4273 110829252016      W                123945.0-602556124537.9-605852302.23  1.88 4.69  +1.05 +0.94 +0.52   K0III              +0.098-0.063 +.036+007           4.8  27.5      *
4843          BD+44 2221 110834 44307                       123943.9+443901124427.1+440611127.22 72.97 6.33  +0.43               F6IV               -0.027+0.005      -016      127
4844   Bet MusCP-67 2064 110879252019      W                124008.7-673338124616.9-680629302.45-05.24 3.05  -0.18 -0.74 -0.21   B2.5V              -0.039-0.017D+.015+042V     184  0.2   1.3      *
4845 10    CVnBD+40 2570 110897 63177                       124015.5+394920124459.5+391644128.81 77.78 5.95  +0.55 -0.03 +0.29   G0V                -0.355+0.139 +.063+081V?      1
4846          BD+46 1817 110914 443171327I         Y CVn    124025.9+455913124507.8+452625126.45 71.65 4.99  +2.54 +6.33 +1.39   C7I                -0.001+0.015 +.002+012                          *
4847 32    VirBD+08 2639 1109511195741328          FM Vir   124033.9+081313124537.1+074024298.61 70.50 5.22  +0.33 +0.15 +0.16   F0IIIm             -0.107+0.004 +.014-009SBO    66                 *
4848          CP-55 5215 1109562402353018  W                124038.0-555629124622.7-562920302.23  6.38 4.65  -0.16 -0.64 -0.18   B3V               v-0.036-0.013      +017V?     66  4.3  52.6      *
4849 33    VirBD+10 2468 111028119580      W                124117.6+100556124622.5+093224298.79 72.38 5.67  +0.99 +0.76         K1III-IV           +0.277-0.446 +.021+052V          3.1 171.5
4850          CD-32 8912 111032203797      W                124122.1-324605124646.0-331856301.81 29.55 5.86  +1.34 +1.60         K3III              -0.007-0.030      +031           6.0  65.
4851 27    ComBD+17 2533 111067100252    I                  124139.0+170726124638.7+163439296.68 79.39 5.12  +1.35 +1.54 +0.68   K3III              +0.009+0.007      +053V    < 17
4852          BD+81  402 111112  20803017                   124153.5+811009124426.0+803716123.29 36.50 6.40  +0.12 +0.13         A5m                +0.030-0.042      -028SB     30                 *
4853   Bet CruCP-59 4451 111123240259 481  W       Bet Cru  124152.5-590831124743.2-594119302.46  3.18 1.25  -0.23 -1.00 -0.26   B0.5III            -0.048-0.014      +016SB     38  6.0 371.6AC   3*
4854          BD+06 2660 111133119585              EP Vir   124157.5+062954124702.3+055703299.90 68.80 6.34  -0.05 -0.06         A0pSrEuCr         v+0.035-0.043      +016V      10                 *
4855 34    VirBD+12 2512 111164100260      W                124211.6+123017124713.6+115729299.00 74.80 6.07  +0.12 +0.13         A3V                +0.042-0.018      -002V?    120  3.2 139.4      *
4856          BD-05 3569 111199138967      W                124223.2-054516124733.4-061807301.18 56.56 6.26  +0.55 +0.07         F7V                +0.001-0.044      +013V?     12  6.1  16.3
4857          CD-2410540 1112261811051329                   124234.6-241824124753.7-245106301.91 38.01 6.44  -0.06 -0.45         B8III              -0.041+0.040      +049V
4858 35    VirBD+04 2653 1112391195961330I          5947    124245.8+040708124751.4+033422300.70 66.43 6.41  +1.60 +1.83 +0.96E  M3IIIb             -0.006-0.003      +008V?                        *
4859          BD+63 1034 111270 15901                       124302.8+631937124718.9+624651123.74 54.34 5.89R                     A9V                +0.016-0.001      -011SB    106
4860          CD-26 9340 111295181114                       124306.3-270259124826.4-273551302.12 35.27 5.66  +0.95 +0.61         G5III-IV           -0.139-0.063      -010V?
4861 28    ComBD+14 2546 111308100269                       124313.8+140559124814.3+133311299.63 76.40 6.56  +0.01 +0.01         A1V                -0.047-0.028      -000      130
4862          CP-71 1391 111315256983                       124315.9-712626124944.9-715911302.80-09.11 5.55  +1.17 +0.95         G8Ib-II            +0.001-0.005      -007V                         *
4863  7    DraBD+67  764 111335 159023020I                  124329.2+672011124734.4+664725123.53 50.33 5.43  +1.56              gK5                 +0.010-0.005 +.023+008
4864          BD+25 2568 111395 825113021                   124354.7+252319124847.0+245025288.28 87.64 6.31  +0.70 +0.25         G7V                -0.339-0.108 +.043-008V
4865 29    ComBD+14 2549 1113971002833022                   124353.5+144007124854.2+140721300.20 76.98 5.70  +0.02 +0.07 -0.01   A1V                +0.026-0.019      -008V?    145                 *
4866 11    CVnBD+49 2163 111421 44332                       124405.7+490043124841.8+482801124.18 68.66 6.27  +0.18 +0.17         A6m                -0.065+0.016      -002V?     37
4867          BD+61 1320 111456 15907                       124418.3+605155124839.4+601912123.56 56.81 5.85  +0.46 -0.04 +0.28   F5V                +0.105+0.001 +.041-012V      36                 *
4868          CP-59 4483 111463252047                       124417.0-595119125012.0-602403302.78  2.47 6.75  +0.35 +0.21         A3II               -0.007-0.017      -009V
4869 30    ComBD+28 2153 111469 82515        8674           124425.1+280549124917.4+273308171.10 89.36 5.78  +0.03 +0.06         A2V                -0.096+0.021      +000SB2        7.0  42.5      *
4870   Iot OctCP-84  407 111482258654 919  W                124427.3-843449125458.6-850724303.01-22.25 5.46  +1.02 +0.79         K0III              +0.062+0.028      +053           0.5   0.6      *
4871          CD-47 7893 111519223698                       124441.8-475453125019.6-482735302.74 14.41 6.24  +0.05               A0V                -0.037+0.002      -017
4872          CP-52 5947 1115882403143023                   124514.5-521433125057.9-524715302.86 10.08 5.73  +0.13               A5V                -0.026-0.012      -007
4873          BD+23 2502 111591 82523                       124521.0+232436125017.4+225148299.36 85.73 6.43  +1.00 +0.80         K0III              +0.109-0.069      +006                          *
4874          CD-33 8653 1115972038631331  W                124515.5-332715125041.2-335958302.75 28.87 4.91  -0.04 -0.11         A0IV               -0.028-0.020 +.020+004V     178:10.0  27.6      *
4875          BD+38 2373 111604 63217               5983    124525.5+380340125010.7+373101124.31 79.61 5.89  +0.15 +0.08         A3V                -0.097+0.027 -.000-014V     183                 *
4876          CP-59 4494 111613252054               5991    124522.1-594707125117.8-601947302.91  2.54 5.72  +0.38 -0.10         A2Iabe             -0.025+0.007      -021       18                 *
4877          BD-09 3569 111720157550        8684A  5994    124610.6-094738125122.9-102018302.91 52.53 6.41  +1.02 +0.79         G8III              -0.008-0.006      -017V          3.3  30.2AB   3*
4878 37    VirBD+03 2703 111765119633                       124631.4+033601125136.9+030324303.04 65.93 6.02  +1.29 +1.44        gK4                 -0.035+0.020      +003
4879          CD-39 7879 111774203881               5999    124626.4-390809125156.8-394051303.04 23.19 5.98  -0.10               B8V                -0.035-0.028      +005
4880          CD-47 7917 111775223720                       124627.0-473301125205.3-480539303.05 14.78 6.33  +0.03 +0.02         A0IV               -0.060+0.013      -002    =< 49
4881          CD-26 9369 111786181169                       124637.1-261143125157.9-264417303.08 36.13 6.15  +0.23               A0III              -0.111+0.040      -018                          *
4882          CP-53 5359 111790240338      W                124638.6-531709125224.6-534946303.08  9.04 6.24  +1.13               G8Ib-II            -0.020+0.012      -023           6.0   6.4
4883 31    ComBD+28 2156 111812 825371332I'        Var?     124649.7+280506125141.9+273226114.93 89.58 4.94  +0.67 +0.20 +0.35   G0III              -0.012-0.008 +.011-001V?     77                 *
4884 32    ComBD+17 2551 1118621003091333I W                124713.8+173704125212.3+170426303.98 79.94 6.32  +1.59 +1.99         M0III              +0.001-0.007      -001V?         0.6 196.5AB   3
4885          CP-54 5360 111884240347                       124716.9-542433125303.9-545709303.17  7.92 5.93  +1.31 +1.40 +0.50E  K2III              -0.119+0.011      -009                          *
4886          BD+16 2430 111893100312                       124729.0+164000125227.6+160721304.22 78.99 6.30  +0.16 +0.10         A7V                -0.043-0.019      -027      214
4887          CP-59 4529 111904252069               6008    124724.0-594706125321.8-601943303.17  2.54 5.76  +0.33 -0.42         B9Ia               -0.013 0.000      -019V?     81                 *
4888          CD-48 7753 1119152237313024                   124727.2-482357125306.9-485636303.22 13.93 4.33  +1.37 +1.58 +0.75   K3-4III            -0.076-0.030 +.020-002SB
4889          CD-39 7893 111968203907 482                   124753.7-393806125326.2-401044303.34 22.69 4.27  +0.21 +0.12 +0.14   A7III              +0.063-0.022 +.056-003       81
4890   Kap CruCP-59 4555 111973252077                       124750.5-594959125349.1-602237303.23  2.49 5.90  +0.20 -0.65         B5Ia              t+0.004-0.021      -004V?      0                 *
4891 38    VirBD-02 3593 1119981390223025  D                124803.8-030035125311.2-033311303.79 59.32 6.11  +0.50 +0.04         F5V                -0.262-0.003 +.042-007V                         *
4892          BD+84  289 112014  2101        8682B          124815.6+835742124906.6+832505123.01 33.71 5.85  -0.06 -0.12         A0V+A2V            -0.026+0.022 +.001+001SB2O   17  0.5  21.6AB   3*
4893          BD+84  290 112028  2102        8682A          124823.1+835724124913.6+832446123.01 33.72 5.28  -0.03 -0.06         A1IIIShell         -0.029+0.020 +.001+002V     275  0.5  21.6AB   3*
4894 35    ComBD+22 2519 112033 82550    I   8695           124822.3+214719125317.8+211442307.21 84.10 4.90  +0.90 +0.64 +0.45   G8III+F6V          -0.047-0.022 +.021-007SBO  < 17  2.1   1.0AB   4*
4895          CP-57 5776 112044240362              S Cru    124826.9-575315125422.0-582550303.32  4.44 6.58  +0.76 +0.60         F7Ib-II           v-0.014-0.001      -007                          *
4896          BD-03 3373 112048139027                       124828.6-034047125338.1-041326303.98 58.64 6.44  +1.10 +1.02         K0                 +0.001-0.041      +037V
4897   Lam CruCP-58 4584 112078240368              Lam Cru  124842.7-583612125439.2-590848303.35  3.72 4.62  -0.15 -0.61 -0.16   B4Vne              -0.036-0.010      +012      317                 *
4898   Mu 1CruCP-56 5487 112092240366      W     A          124842.8-563805125435.6-571040303.36  5.69 4.03  -0.17 -0.76 -0.24   B2IV-V             -0.033-0.006      +014       48  1.0  34.9      *
4899   Mu 2CruCP-56 5487 112091240367      W     B Mu2 Cru  124844.0-563731125436.8-571006303.37  5.70 5.17  -0.12 -0.51 -0.06   B5Vne              -0.035-0.001      +019V     201  1.0  34.9      *
4900 41    VirBD+13 2602 112097100322                       124848.6+125744125349.7+122507305.23 75.28 6.25  +0.27 +0.05 +0.15   A7III              +0.051-0.024      -010SB     67                 *
4901          BD-10 3570 112131157584      D        6019    124906.2-110622125418.7-113855304.06 51.22 6.00  +0.08               A2V                -0.141+0.013      -024           0.1   0.1      *
4902 40Psi VirBD-08 3449 1121421390331335I D       Psi Vir  124909.1-085945125421.2-093220304.14 53.33 4.79  +1.60 +1.53 +1.28   M3-IIICa-1        e-0.024-0.015 +.021+018V?         3.3   0.0      *
4903          CD-43 7953 112164223753                       124924.2-433612125458.5-440907303.61 18.72 5.89  +0.64 +0.25         G1V                -0.223-0.225 +.044+032
4904          BD+34 2369 112171 632443027                   124926.5+340434125413.1+333204117.74 83.57 6.26  +0.20 +0.09         A7IV               -0.090+0.032      +005      127
4905 77Eps UMaBD+56 1627 112185 28553 483I         Eps UMa  124937.8+563009125401.7+555735122.18 61.16 1.77  -0.02 +0.02 -0.03   A0pCr              +0.112-0.006 +.009-009SB?    38                 *
4906          CD-42 7975 1122132237603029                   124944.4-422223125519.4-425457303.69 19.95 5.47  +1.68 +2.05         M0III              -0.038-0.012      -007
4907          CP-71 1404 112219256992                       124950.5-713834125631.5-721107303.33-09.32 5.93  +1.13 +0.90         G8III              -0.027-0.011      +010
4908          CP-56 5498 112244240385      W        6024    125003.5-561737125557.0-565010303.55  6.03 5.32  +0.01 -0.84         O9Ib               -0.024-0.010      +018V     145  5.0  29.1      *
4909          BD+47 2003 112264 443833030I         TU CVn   125022.5+474421125456.5+471148121.20 69.92 5.84  +1.55 +1.57         M5III              -0.016-0.006      -017                          *
4910 43Del VirBD+04 2669 112300119674 484I W        6026    125033.9+035627125536.2+032351305.53 66.25 3.38  +1.58 +1.78 +1.33   M3+III            e-0.469-0.054 +.022-018V?         7.0 164.5      *
4911          BD-14 3605 1123041575993033                   125037.7-144707125553.3-151937304.52 47.53 6.17   0.00               A0Vn               -0.017+0.003      -003      148
4912          CD-25 9508 112374181244              LN Hya   125107.3-255505125630.1-262737304.34 36.40 6.62  +0.68               F3Ia               +0.013-0.015      -024V                         *
4913          CD-50 7394 112409240407                       125118.7-503925125704.4-511155303.83 11.66 5.16  -0.06 -0.32         B8-9V              -0.026-0.015      +025      255
4914 12Alp1CVnBD+39 2580 112412 63256        8706B          125119.7+385117125600.4+381853118.31 78.77 5.60  +0.34 -0.03 +0.23   F0V                -0.236+0.062 +.027-003V       8  2.7  19.4      *
4915 12Alp2CVnBD+39 2580 112413 63257 485I   8706A Alp2 CVn 125121.0+385130125601.7+381906118.29 78.77 2.90  -0.12 -0.32 -0.06   A0pSiEuHg          -0.234+0.056 +.027-003V      29  2.7  19.4      *
4916  8    DraBD+66  778 112429 15941 486                   125129.7+655851125528.5+652619122.26 51.68 5.24  +0.28 +0.02         F0V                -0.005-0.029 +.032+009      132                 *
4917          BD+54 1556 112486 28572        8710           125154.7+543827125617.6+540558121.36 63.01 5.82  +0.19 +0.08 +0.08   A5m                -0.077+0.001D+.012-005SB2O   20  1.8   3.5AB   3*
4918          BD-21 3635 112519181265                       125213.0-221244125733.2-224514304.78 40.10 6.31  +1.07               G5                 -0.052-0.029      +003V
4919          BD+46 1833 112570 44398                       125233.8+464309125707.7+461037119.92 70.92 6.12  +1.01 +0.80         K0III-IV           -0.022-0.047      +007V
4920 36    ComBD+18 2682 1127691003573036I          6046    125358.7+175654125855.4+172434313.41 80.13 4.78  +1.56 +1.96 +0.95   M1-IIIb            -0.039+0.034      -002
4921 44    VirBD-03 3384 1128461390861336    8727           125430.3-031621125939.5-034843306.92 59.00 5.79  +0.18 +0.08         A3V                -0.037+0.008      -018           4.9  20.9     3*
4922          CD-32 9083 1129352040293038                   125504.0-325750130032.6-333019305.11 29.33 6.02  +0.38               F0V                -0.078-0.079      -023V?
4923   Del MusCP-70 1548 112985257000 487                   125523.3-710034130216.2-713256303.80-08.69 3.62  +1.18 +1.26 +0.59   K2III              +0.256-0.021 +.030+037SBO                       *
4924 37    ComBD+31 2434 112989 632883039I   8731  Var?     125529.3+311928130016.5+304706 95.60 85.86 4.90  +1.17 +1.05 +0.58   G9IIICH-2Fe1Ca1    -0.018-0.013 +.016-013V?   < 19: 7.8   5.5      *
4925 46    VirBD-02 3609 112992139096        8732   6053    125526.9-024951130035.9-032207307.43 59.42 5.99  +1.12 +1.10         K2III              -0.025+0.050      +023V?         3.9   1.1AB   3*
4926          BD+19 2622 113022100366        8735           125544.3+185437130038.8+182223317.04 80.99 6.20  +0.42 +0.02         F6V s              -0.226+0.060 +.020+001V?     20  2.8 149.2AxBC 3*
4927          BD+76  473 113049  77143037                   125549.8+760042125847.3+752821122.32 41.65 6.01  +0.99 +0.75         K0III              +0.008+0.010      -015
4928  9    DraBD+67  773 113092 15960    I                  125608.4+670812125955.1+663550121.60 50.51 5.32  +1.29 +1.29         K2III              -0.139-0.013 +.005-030V?   < 17
4929 38    ComBD+17 2573 113095100374                       125612.9+173945130109.6+170723316.09 79.75 5.96  +0.96               K0III              -0.005-0.025      -006
4930          CP-70 1553 113120257003               6063    125617.1-705615130305.2-712834303.87-08.63 6.03  +0.05 -0.87         B1.5IIIne         v-0.023-0.014      -005V     300                 *
4931 78    UMaBD+57 1408 113139 28601        8739   6058    125626.4+565419130043.8+562159120.30 60.71 4.93  +0.36 +0.01 +0.21   F2V                +0.114-0.009 +.034-010V?     92  2.5   1.3      *
4932 47Eps VirBD+11 2529 113226100384 488I W        6064    125711.9+112948130210.6+105733312.34 73.63 2.83  +0.94 +0.73 +0.45   G8IIIab            -0.273+0.020 +.043-014V?   < 17  8.9 248.7      *
4933   Xi 1CenCD-48 7887 113314223870                       125745.8-485922130333.2-493138304.95 13.30 4.85  +0.02 +0.03         A0V                -0.059-0.014 +.012+000V     185                 *
4934          BD+64  927 113337 15962      W                125752.5+640851130146.8+633637121.00 53.48 6.00  +0.41 +0.02         F6V                -0.180+0.029 +.033-011V?  =<  6  9.4 119.
4935          BD-19 3629 113415181357        8757           125824.6-200247130346.1-203459306.83 42.20 5.58  +0.56 +0.07         F7V                +0.142+0.018D+.003+034V?         0.1   0.4      *
4936          BD+60 1439 113436 286133041                   125833.9+601514130240.4+594258120.31 57.35 6.53  +0.05 +0.06 +0.03   A3Vn               -0.029-0.016      -023SB?   200
4937 48    VirBD-02 3622 113459139131        8759           125845.2-030731130354.4-033948308.99 59.06 6.59  +0.29 +0.11         F0V                -0.035-0.036D+.005+003V     109  0.3   0.6
4938          CD-40 7662 113523223881              V789 Cen 125910.0-403933130448.1-411148305.64 21.61 6.26  +1.68 +1.83         M3-4III            -0.028-0.034      -033                          *
4939          CD-51 7248 113602240535               6084    125938.0-513446130530.8-520654305.13 10.70 6.43  +1.70 +1.99         M1III              -0.048+0.032      -031V?                        *
4940          CD-47 8088 113703223900      W                130028.8-475538130616.7-482749305.47 14.34 4.71  -0.14 -0.58 -0.14   B5V                -0.030-0.023      +006      189  6.5  11.4      *
4941          CD-40 7682 1137782239053044           6092    130055.0-410309130635.1-413519305.97 21.19 5.59  +1.05               K0II-III           +0.046-0.017      +002
4942   Xi 2CenCD-49 7644 113791223909 489  W                130104.2-492214130654.6-495422305.49 12.89 4.27  -0.19 -0.79 -0.20   B1.5V              -0.026-0.012      +014SB1O   51  5.1  25.1      *
4943 14    CVnBD+36 2337 113797 633381337           6089    130103.9+362002130544.5+354756104.46 80.81 5.25  -0.08 -0.20         B9V                -0.030+0.021 +.011-013V     175                 *
4944          CP-59 4740 113823240566      W                130114.0-591930130724.2-595138304.94  2.95 5.99  +0.48 +0.12         F8-G2+B9IV         -0.027-0.012      +004           0.2   0.9
4945          BD+46 1847 113847 444651338I   8775           130122.3+454812130552.3+451607114.85 71.63 5.63  +1.13               K1III              -0.012+0.028D+.012-020V          5.4   2.9
4946 39    ComBD+21 2487 113848 826501339  W                130128.8+214124130621.2+210912333.39 83.13 5.99  +0.39  0.00         F4V                -0.069-0.046 +.030+001       30  3.0   1.2      *
4947          CD-35 8441 113852204132                       130120.1-351928130654.3-355143306.44 26.91 6.54  -0.02 -0.02         A0IV-III           +0.041-0.078      +016    =< 49
4948          BD+29 2365 113865 82648        8777           130123.9+293354130610.2+290146 64.11 86.23 6.54  +0.05 +0.09         A3IV               -0.069+0.001      +005V          6.4   6.3AxBC 4*
4949 40    ComBD+23 2538 113866 82651    I         FS Com   130130.5+230910130622.6+223658340.64 84.36 5.60  +1.59 +1.63 +1.81   M5III              +0.025-0.049 +.045-005V?                        *
4950          BD+73  583 113889  7741        8772           130144.2+733337130449.7+730131121.57 44.07 6.31R                     F0V                -0.019+0.020D+.007-016       90  2.0   1.3AB   3
4951          CP-52 6194 1139022405731340                   130141.6-525527130738.3-532735305.37  9.34 5.71  -0.07 -0.27         B8V                -0.039-0.024      +022
4952   The MusCP-64 2183 113904252162      W       The Mus  130139.6-644617130807.0-651823304.67-02.49 5.51  -0.02 -0.87         B0Ia+WC5:         v-0.016-0.009      -028SBO   106: 1.7   5.3      *
4953          BD+62 1275 113994 15985                       130225.9+623441130622.7+620231119.88 55.00 6.14  +0.99 +0.65         G7III              +0.005-0.037      +015V
4954 41    ComBD+28 2185 113996 826593045I                  130222.8+280941130710.7+273729 41.94 86.47 4.80  +1.48 +1.85 +0.81   K5III              +0.031-0.070 +.008-016V?   < 17
4955 49    VirBD-09 3628 114038157739    I                  130239.4-101221130753.8-104425309.50 51.92 5.19  +1.14 +1.12 +0.58   K2III              +0.017-0.006 +.025-009     < 19:                *
4956          BD+28 2187 114092 82665                       130306.4+280531130753.6+273321 40.56 86.32 6.19  +1.36 +1.64 +0.48E  K4III              -0.041-0.064      -009
4957          BD-08 3491 1141131391753046I D                130319.5-082655130832.5-085904310.07 53.65 5.55  +1.18 +1.18         K3III              -0.032-0.062      +016V?         0.0   0.1
4958 45Psi HyaBD-22 3515 1141491814103048I                  130340.0-223500130903.3-230705308.19 39.57 4.95  +1.05 +0.93         K1III              -0.024-0.036 +.012-019V
4959          BD-08 3495 114203139183                       130400.9-090016130914.4-093218310.25 53.08 6.32  +1.02 +0.79         K0                 -0.033-0.009      +000V?
4960          BD+10 2516 114256100436                       130411.9+103321130912.4+100120317.53 72.39 5.78  +1.00               K0III              +0.017-0.003      -000
4961 50    VirBD-09 3636 114287157760    I          6114    130431.1-094745130945.3-101946310.31 52.28 5.94  +1.49 +1.69         K5III              -0.008-0.012      -007V?
4962          BD+17 2595 1143261004393049I                  130452.8+172255130947.8+165055326.33 78.88 5.91  +1.45 +1.65         K5III              -0.069-0.012      -017
4963 51The VirBD-04 3430 114330139189 490    8801           130446.2-050018130957.0-053220311.42 57.03 4.38  -0.01 -0.01  0.00   A1IV s+Am          -0.032-0.033 +.026-003SB     15  1.0   0.5     4*
4964          BD+38 2407 114357 63372                       130502.0+375721130938.7+372523103.61 79.01 6.02  +1.15               K3III              -0.103+0.005      -019
4965          CD-51 7329 114365240627              V824 Cen 130500.9-520202131058.4-523401305.95 10.19 6.06  -0.09 -0.38         ApSi               -0.041-0.017      +026V      76                 *
4966          CP-69 1772 114371252185      W                130457.3-692435131151.6-695631304.69-07.14 5.91  +0.42 -0.03         F3IV-V             +0.036+0.006      +007SB2        5.9  30.7      *
4967 15    CVnBD+39 2611 114376 63374        8805BC         130505.9+390400130942.0+383202105.55 77.97 6.28  -0.12 -0.48         B7III              -0.016+0.003      -006V     125  0.3 284.4AC   3*
4968 42Alp ComBD+18 2697 114378100443        8804A  6116    130507.4+180330130959.3+173146327.96 79.49 5.22H +0.45 -0.06         F5V                -0.431+0.136 +.054-018V?     28  0.1   0.2AB   3*
4969 42Alp ComBD+18 2697 114379100443        8804B  6116    130507.4+180330130959.3+173146327.96 79.49 5.22H                     F5V                -0.430+0.136 +.054-018V?         0.1   0.2AB   3*
4970          CD-41 7648 114435223960                       130526.9-414200131108.8-421359306.83 20.49 5.79  +0.52               F7IV               -0.080-0.024      -011
4971 17    CVnBD+39 2614 114447 63380 491    8805A          130527.7+390149131003.2+382956105.17 77.98 5.91  +0.29               A9III-IV           -0.070+0.044      +000SB2?   77: 0.3 284.4AC   3*
4972          CP-62 3046 114461252187                       130528.2-624615131153.1-631810305.23-00.52 6.33  +0.44 +0.35         A8II-III           -0.006+0.003      -003V
4973          CD-42 8175 114474223966                       130540.0-425009131123.2-432208306.77 19.35 5.25  +1.05 +0.92 +0.36E  K1-2III            -0.117-0.027 +.011-009                          *
4974          BD+63 1056 114504 159993050  W                130558.2+624543130950.2+621345119.22 54.77 6.54  -0.02 -0.02         A1V                -0.030-0.008      -017       68  3.2 107.8
4975          CP-59 4815 114529240645      W       V831 Cen 130602.8-592318131217.4-595515305.55  2.85 4.60  -0.08 -0.38 -0.09   B8V                -0.049-0.024D+.014+012SB    216: 0.4   0.1AB   4*
4976          CP-77  890 1145332570193054                   130557.9-775459131417.2-782650304.12-15.63 5.85  +1.07 +0.72         F8Ib               -0.018+0.004      -018
4977          CP-65 2201 114570252196                       130611.6-654144131248.9-661337305.09-03.44 5.90  +0.06 +0.04         A0Vm:              +0.024+0.006      +009
4978          CD-25 9653 1145761814461341  W                130612.8-260112131139.2-263306308.53 36.10 6.50  +0.19               A5Vn               -0.068+0.004      -021      177  0.1   0.2      *
4979          CD-37 8437 1146132042273051                   130628.4-371622131203.2-374811307.43 24.89 4.85  +0.70 +0.31 +0.36   G3V                -0.385+0.047 +.053-015V?
4980          CP-59 4827 114630240653      W                130640.2-591657131256.0-594900305.63  2.94 6.16  +0.60 +0.08         G0V                +0.014-0.107      +016SB2        3.2  25.3      *
4981 53    VirBD-15 3613 114642157788      W        6136    130644.1-153933131203.5-161155310.12 46.39 5.04  +0.46 +0.02         F5III-IV           +0.098-0.286 +.040-014       15  5.0 235.0AD   4*
4982          CD-42 8196 114707223980                       130706.9-421007131250.9-424159307.12 20.00 6.22  +1.06               K0III              -0.020+0.003      -001
4983 43Bet ComBD+28 2193 114710 82706 492I W                130712.4+282306131152.4+275241 43.33 85.40 4.26  +0.57 +0.07 +0.30   F9.5V              -0.801+0.882 +.120+006SB?     6  5.8  90.8      *
4984          BD+25 2610 114724 827083052                   130719.4+244725131208.4+241529  2.47 84.52 6.33  +0.98               K1III              -0.008-0.041      -024
4985          CD-50 7589 114772240655      W                130727.3-501007131323.5-504200306.48 12.02 5.89  -0.02               B9V                -0.019-0.027      -001           0.3   0.3
4986          BD+12 2565 1147801004603053I                  130734.3+120517131232.9+113322321.61 73.66 5.77  +1.51               M0III              -0.050-0.033      +025
4987          BD+19 2648 114793100461                       130742.9+191657131235.9+184506334.19 80.31 6.53  +0.88               G8III              -0.065-0.003      -020
4988          CP-58 4738 1148352406633055                   130759.0-580912131412.1-584102305.89  4.06 5.89  +1.08               K0-1III            -0.051-0.012      -002
4989          CP-58 4740 114837240666      W                130803.3-583407131414.8-590612305.87  3.64 4.92  +0.48       +0.27   F7IV               -0.265-0.157 +.053-065        0  5.3   2.7
4990 54    VirBD-18 3562 114846157798        8824A          130805.9-181744131326.8-184936310.15 43.74 6.28  +0.09 +0.01         B9III              -0.020-0.017D+.011-041SB         0.4   5.4      *
4991          CD-42 8213 114873223989                       130813.8-423634131357.5-430820307.29 19.54 6.16  +1.38 +1.62         K4III              -0.170+0.028      +030
4992          BD+19 2649 114889100467                       130820.5+191532131312.4+184337334.87 80.22 6.11  +1.20               G8III              -0.215-0.049      -024                          *
4993   Eta MusCP-67 2224 114911252224 493  W       Eta Mus  130828.0-672152131514.9-675340305.18-05.13 4.80  -0.08 -0.35 -0.08   B8V                -0.041-0.008      -008SBO   268  3.2  60.       *
4994          CP-69 1784 114912252225                       130830.3-690855131525.7-694047305.03-06.90 6.37  +1.21 +1.33         K2-3III            -0.101-0.048      -011
4995 55    VirBD-19 3651 114946157806                       130849.7-192420131410.9-195551310.21 42.62 5.33  +0.87 +0.44         G6V                -0.122+0.167 +.010-045V?
4996          CD-48 8050 114971224000                       130850.2-482529131443.1-485724306.86 13.74 5.89  +1.06 +0.93         K1III              -0.129-0.081      +056                          *
4997          BD+40 2633 115004 44519    I                  130910.9+404057131343.0+400910104.83 76.18 4.92  +1.06 +0.93 +0.49   G8IIIaCN0.5        -0.048+0.015 +.004-021V?   < 19:
4998          BD+12 2572 115046100473    I   8832           130931.9+115146131431.3+111954322.99 73.30 5.67  +1.50 +1.86 +0.90   M0III              +0.074-0.052      +012V          6.7  49.1      *
4999          CD-35 8547 115050204282                       130932.2-355029131509.7-362216308.26 26.26 6.19  +0.97               K1III              -0.012-0.015      -004
5000          CP-64 2316 115149252236                       131009.3-643629131644.8-650818305.59-02.40 6.07  +0.44 -0.02         F5V                -0.103-0.052      +018V
5001 57    VirBD-19 3653 115202157823    I                  131034.2-192441131558.8-195635310.77 42.56 5.22  +1.03 +0.87         K1III-IV           +0.306-0.116 +.048+034V                         *
5002          CP-66 2142 115211252240               6164    131028.5-661518131713.0-664701305.47-04.04 4.87  +1.50 +1.58 +0.75   K2Ib-II            -0.020-0.014 +.007-010                          *
5003          BD+73  587 115227  77823057                   131040.0+731944131332.0+724756120.66 44.23 6.59  +0.08 +0.07         A2V                +0.017-0.016      +002SB
5004 19    CVnBD+41 2374 115271 44531                       131102.2+412300131532.0+405119104.57 75.40 5.79  +0.19 +0.12         A7V                -0.108+0.015      -018       97
5005          BD-00 2674 1153081392543058          DK Vir   131117.5-005142131625.5-012326315.84 60.85 6.68  +0.31 +0.08         F1IV               -0.035-0.028      -012       51                 *
5006          CD-3010457 1153102043121342I                  131119.7-305837131653.1-313022309.26 31.05 5.10  +0.96 +0.61:        K1III              +0.034-0.048 +.016+013V?
5007          BD+19 2655 115319100484                       131122.4+193447131614.3+190306339.43 80.12 6.45  +0.98 +0.75         G8III              -0.102+0.002      -045
5008          CD-43 8165 1153312240321343                   131125.8-432705131713.9-435846307.82 18.65 5.84  +0.20 +0.15 +0.09   Am                  0.000-0.008      -012V                         *
5009          BD+81  416 115337  21643056  W        6153    131131.4+810002131225.4+802817121.85 36.61 6.25  +0.94               K0Ib               -0.009+0.012      -011V?         4.0   1.1      *
5010          BD+20 2814 115365 827513059  W                131141.1+201845131632.3+194707342.33 80.67 6.45  +0.25 +0.08         F0V                -0.110+0.023      -035      161  1.9 203.       *
5011 59    VirBD+10 2531 115383119847      W                131148.7+095648131646.5+092527322.80 71.31 5.22  +0.59 +0.10 +0.20E  G0V s              -0.334+0.190 +.079-026V?      7  9.1  34.3
5012          CP-71 1458 1154392570283061                   131200.2-713026131918.9-720208305.11-09.28 6.04  +1.35 +1.46         K3III              -0.013-0.056      -031
5013          BD+14 2591 115478100497    I                  131218.8+141207131715.6+134032328.35 75.26 5.33  +1.31 +1.51 +0.66   K3III              +0.007+0.039      -025V?   < 19:
5014          BD+00 3040 115488139264      W        6171    131222.6-000855131729.9-004036316.69 61.50 6.37  +0.26 +0.04         F0V                -0.058-0.025      -020      120  0.0   0.1
5015 60Sig VirBD+06 2722 1155211198551344I          6173    131233.2+055948131736.3+052811320.14 67.46 4.80  +1.67 +1.95 +0.84E  M1III              -0.005+0.013 +.011-027V?                        *
5016          CD-50 7660 115529240721                       131234.2-504531131834.6-511710307.25 11.36 6.19  +0.01               A0V                -0.048-0.023      -004
5017 20    CVnBD+41 2380 115604 44549 494          AO CVn   131303.5+410557131732.5+403421102.73 75.52 4.73  +0.30 +0.21 +0.15   F3III              -0.125+0.021 +.019+008V?     17                 *
5018          BD+69  694 115612 160333060                   131311.3+685606131628.6+682429119.46 48.55 6.20  -0.06 -0.16         B9.5V              -0.013+0.016      -007V     120:
5019 61    VirBD-17 3813 1156171578441345  W                131310.3-174518131824.3-181841311.88 44.10 4.74  +0.71 +0.26 +0.36   G6V                -1.070-1.065 +.115-009V?   < 17  5.5 231.5
5020 46Gam HyaBD-22 3554 115659181543 495I W        6180    131329.0-223838131855.3-231018311.10 39.26 3.00  +0.92 +0.66 +0.47   G8-IIIa            +0.064-0.045 +.027-005V?   < 17  6.7 138.4      *
5021          BD+04 2721 115709119867                       131347.1+041251131851.1+034116319.73 65.66 6.62  +0.06 +0.03         A1IV               -0.050-0.011      -001SB     47
5022          BD+34 2410 115723 63462    I                  131349.9+343728131827.8+340553 84.77 80.93 5.82  +1.35               K4.5III            +0.033+0.003      -020
5023 21    CVnBD+50 1994 115735 445563063          BK CVn   131359.3+501229131814.5+494055111.85 66.87 5.15  -0.07 -0.20         A0V                -0.033+0.015 +.020-003      106                 *
5024          CP-59 4912 115778240756      W                131412.8-591449132034.9-594624306.60  2.89 6.18  +0.43               F3-F5II            -0.042-0.024      -010           6.4  13.1
5025          BD+35 2435 115810 634683064    8861D          131428.3+353912131904.2+350741 88.19 80.06 6.02  +0.24               F0IV               -0.030+0.019      -002       82  3.6 319.0AD   4
5026          CP-52 6405 115823240762               6190    131432.9-521320132037.8-524453307.41  9.87 5.48  -0.13 -0.52 -0.15   B6V                -0.030-0.019      +010SB     73                 *
5027          CP-55 5504 1158422407653065           6193    131436.3-551632132048.3-554802307.08  6.83 6.02  +0.29 -0.69 +0.22   B0.5Ia            e-0.009+0.004      -003       90                 *
5028   Iot CenCD-36 8497 115892204371 496                   131458.4-361105132035.8-364244309.42 25.79 2.75  +0.04 +0.03 -0.01   A2V                -0.341-0.085 +.062+000       85
5029          CD-46 8580 115912224080                       131503.8-462121132057.7-465250308.17 15.69 5.77  +1.12 +1.01         K1III              -0.075+0.004      -003
5030          CP-71 1467 115967257033      W                131527.3-713720132252.6-720848305.37-09.42 6.05  +0.09 -0.34         B6V                -0.023-0.013      +033SB         1.4   0.3      *
5031          BD+03 2758 115995119889        8864           131537.1+032802132041.6+025630320.32 64.82 6.26  +0.10 +0.10         A3V                -0.059-0.029D+.005-004V      80  0.4   1.1
5032 23    CVnBD+40 2647 116010 445701346I                  131550.1+404031132019.0+400902100.09 75.67 5.60  +1.20               K1III              -0.058-0.002      -021
5033          BD-18 3587 116061157879               6206    131607.2-185754132129.9-192920312.61 42.82 6.21  +0.09 +0.08         A2V                -0.066+0.008      -008V?
5034          CP-60 4627 116072252283      W     B V790 Cen 131607.7-602652132235.7-605820306.70  1.67 6.18  +0.01 -0.58         B2.5Vn             -0.020-0.013      +003V     233  0.0   0.1AB   3*
5035          CP-60 4627 1160872522841347  W     A          131610.1-602751132237.9-605918306.71  1.65 4.53  -0.13 -0.60 -0.15   B3V                -0.039-0.014      +006V     241  0.0   0.1AB   3*
5036          CD-51 7465 116084240782                       131611.1-513932132216.2-521059307.73 10.40 5.83  +0.12 -0.72         B2.5Ib             -0.010+0.001      -015                          *
5037          BD+02 2664 1161601198993067                   131636.5+023646132141.6+020514320.35 63.93 5.69  +0.06 +0.03         A2V                -0.061-0.053      -007V     200
5038          CD-47 8260 116197224095      W                131654.8-472511132252.7-475635308.36 14.60 6.16  +0.18               A4V                +0.036+0.005      -000V?         0.3   0.6
5039          CD-47 8261 1162262240963068                   131703.9-480222132302.6-483346308.31 13.98 6.38  -0.07 -0.44         B6IV               -0.006-0.001      -041                          *
5040 64    VirBD+05 2737 116235119905                       131707.1+054045132209.7+050917322.72 66.84 5.87  +0.12 +0.10 +0.04   A2m                -0.065-0.036      -010V      20
5041          CP-63 2732 116243252293                       131717.0-640044132400.5-643209306.42-01.88 4.53  +0.85 +0.47 +0.43   G6II               +0.029-0.026 +.002+012
5042   Iot1MusCP-74 1057 1162442570413070                   131714.0-742142132507.1-745316305.17-12.16 5.05  +1.11 +1.01 +0.54   K0III              -0.115-0.130 +.016+028
5043          CD-32 9322 116278204420               6212    131731.6-324001132308.7-331124310.53 29.21 6.22  +1.65 +1.96 +0.94E  M1III              -0.005-0.005      -013                          *
5044 63    VirBD-16 3650 116292157899               6213    131739.6-171241132301.1-174407313.50 44.49 5.37  +0.99 +0.75 +0.49   KIII               -0.047-0.029 +.004-027
5045          BD+44 2269 116303 44582               6210    131741.4+442534132203.8+435411104.70 72.12 6.35  +0.24 +0.06 +0.14   A7m                -0.071+0.010      -001       36
5046          CD-49 7884 116338224104                       131751.4-491758132352.3-494923308.29 12.71 6.48  +0.96               G8III              -0.067-0.044      -032V
5047 65    VirBD-04 3469 1163651393083069I                  131807.9-042405132318.9-045528317.64 57.03 5.89  +1.43 +1.66         K3III              -0.019-0.015      +010
5048          CP-63 2743 1164572523043071  W                131832.1-635746132513.9-642907306.56-01.85 5.31  +0.40 -0.01         F2III              -0.157-0.022 +.008-002        0  5.7  25.7      *
5049          CP-69 1838 116458257042                       131832.5-700621132550.1-703739305.80-07.95 5.67  -0.03 -0.18         ApHgMn:            -0.067-0.009      +008V   =< 49                 *
5050 66    VirBD-04 3472 116568139324                       131920.8-043829132433.2-050950318.08 56.73 5.75  +0.42 -0.04         F3-4V s            +0.159-0.030      +014SB     40
5051   Iot2MusCP-74 1059 116579257047                       131921.4-741015132718.3-744131305.34-11.99 6.63  -0.06 -0.24         B9V                -0.049-0.017      -003
5052          BD+37 2404 116581 63514    I          6220    131921.9+373322132354.0+370202 90.25 77.95 6.07  +1.68               M3III              +0.022-0.006      +000
5053          BD+13 2663 116594100553                       131933.8+125709132430.5+122555332.35 73.39 6.44  +1.06 +0.91         K0III              -0.016+0.041      -006SB
5054 79Zet UMaBD+55 1598 116656 28737 497I   8891A  6224    131954.0+552651132355.5+545531113.11 61.58 2.27  +0.02 +0.03 -0.02   A1VpSrSi           +0.122-0.020 +.047-006SB2O   32  1.7   0.0AP   4*
5055 79Zet UMaBD+55 1598 116657 28738    I:  8891B  6225    131954.9+552639132356.4+545518113.11 61.58 3.95  +0.13 +0.09         A1m                +0.119-0.028 +.047-009SB1O   57  1.7   0.0AP   4*
5056 67Alp VirBD-10 3672 116658157923 498I W       Alp Vir  131955.4-103822132511.6-110941316.11 50.84 0.98  -0.23 -0.93 -0.24   B1III-IV+B2V       -0.041-0.028 +.023+001SB2O  159  1.5   0.0 O   5*
5057          BD+24 2578 116706 828253072                   132020.5+242232132506.7+235116 11.48 81.73 5.78  +0.06 +0.08         A3IV               -0.013-0.009      -003SB?    60
5058          CD-39 8246 1167132044653073                   132019.7-391359132607.8-394519310.14 22.63 5.09  +1.20 +1.02         K0.5III:Ba3        +0.184-0.052 +.012+067V                         *
5059          BD-00 2686 1168311393373074           6242    132104.0-004021132611.4-011133320.78 60.47 5.97  +0.19 +0.13         A7III              -0.111+0.004      -020      112                3*
5060          CD-40 7894 116835224148                       132106.8-405840132656.1-412953310.04 20.88 5.69  +1.47               K3III              -0.004-0.019      -024
5061          CD-48 8202 116836224149      W                132104.2-483726132706.3-490838308.91 13.31 6.31                      A0III-IV           -0.021-0.006      -018           0.0   0.1
5062 80    UMaBD+55 1603 116842 28751               6238    132113.1+553032132513.5+545917112.77 61.47 4.01  +0.16 +0.08 +0.07   A5V                +0.118-0.016 +.045-009SB    218                 *
5063          CD-48 8206 116862224151                       132117.8-485142132720.8-492251308.92 13.07 6.28  -0.12 -0.69         B3IV               +0.011+0.010      -006V?
5064 68    VirBD-11 3516 1168701579381348I          6244    132126.1-121114132643.2-124228316.18 49.25 5.25  +1.52 +1.75 +0.87   M0III             v-0.130-0.020      -029V?                        *
5065          CD-39 8260 116873204483      W                132127.2-393836132714.7-400947310.31 22.20 6.40  +0.98               G8/K0III           -0.007-0.002      +014V          5.7  45.4
5066          CP-68 1929 116890252321              EZ Mus   132130.9-690628132846.4-693741306.19-06.99 6.20  +0.02 -0.42         ApSi               -0.047-0.045      +016V                         *
5067          BD+46 1868 116957 44611                       132159.1+463254132616.6+460141105.14 69.88 5.88  +0.97              gK0                 +0.024-0.021      +004
5068 69    VirBD-15 3668 116976157946    I          6253    132207.0-152718132727.2-155825315.45 46.02 4.76  +1.09 +1.06 +0.53   K0-III-IVCN2Fe0.5 v-0.119+0.024 +.059-014V                         *
5069          CP-64 2418 117025252329                       132219.5-640926132907.6-644033306.95-02.10 6.11  +0.11 +0.05 +0.05   ApSrEuCr           -0.083-0.005      +015                          *
5070          BD+64  949 117043 16071                       132234.8+634629132559.9+631540116.41 53.43 6.50  +0.74 +0.30         G6V                -0.392+0.216 +.028-031V                         *
5071          CD-50 7812 117150240883                       132317.3-503849132925.2-510955308.99 11.26 5.06  +0.07 +0.07         A1V                -0.002-0.017 +.011-002V     243:
5072 70    VirBD+14 2621 1171761005821349  W                132332.3+141846132825.8+134644337.69 74.11 4.98  +0.71 +0.26 +0.39   G4V                -0.236-0.577 +.043+005V       1  3.6 286.4      *
5073          BD+73  592 117187  7814 499I W        6254    132335.0+725439132608.1+722329119.27 44.49 5.79  +1.63 +1.90         M1IIIab            +0.022-0.009      -048V          7.4  25.7
5074          BD+65  935 117200 16078      W     A          132342.6+651512132704.6+644408116.77 51.96 6.66  +0.37  0.00         F0                 -0.068+0.028      -013       21  0.4  47.0AB   3*
5075          BD+65  936 117201 16079      W     B          132348.6+651413132710.7+644310116.75 51.98 7.04  +0.39 -0.01         F0                 -0.063+0.032      -015    =< 30  0.4  47.0AB   3*
5076          BD+53 1622 117242 28763                       132358.4+531551132759.5+524445110.49 63.49 6.34  +0.23 +0.14         F0                 -0.114-0.008      -007      101
5077          BD+41 2400 117261 44621                       132401.4+411457132826.2+404347 95.99 74.40 6.47  +0.92               G8III              +0.009-0.056      -058
5078          BD-00 2694 117267139359                       132406.9-005043132914.9-012152322.16 60.09 6.43  +1.11 +1.11 +0.36E  K0III              -0.043-0.062      +039V?                        *
5079          BD+51 1846 117281 28766                       132405.8+510615132811.7+503514108.77 65.52 6.80R                     F1IV               -0.113+0.041      -016
5080          BD-22 3601 117287181695    I   8920  R Hya    132414.8-224552132942.8-231653314.22 38.75 4.97  +1.60 +0.68 +2.42   M7IIIe            v-0.056+0.016      -010V          8.5  21.2      *
5081 71    VirBD+11 2575 1173041005923076                   132415.7+112012132913.0+104906333.33 71.42 5.65  +1.05               K0III              -0.061-0.035      -001SB
5082          CP-76  767 117360257060      W       S Cha    132433.3-770259133314.8-773406305.25-14.88 6.48  +0.48 -0.06         F6V                -0.344-0.112 +.034-036V?      0  2.8  22.4      *
5083          BD+51 1847 117361 28771                       132438.9+511417132845.7+504306108.68 65.36 6.43  +0.37 +0.05         F0IV               +0.029-0.083      -007                          *
5084   Kap OctCP-85  384 1173742586741665                   132442.1-851625134055.5-854710303.91-23.01 5.58  +0.18 +0.16         Am                 -0.090-0.018      -009SB                        *
5085          BD+60 1461 117376 16080 500  W                132446.9+602744132827.1+595645114.52 56.57 5.40  -0.01 -0.02         A1Vn               -0.080+0.037 +.025-003V     144  3.3 182.3AC   3*
5086          BD+07 2655 1174041199623077I                  132459.1+074144133000.1+071044329.33 68.05 6.17  +1.47 +1.80         K5                 +0.004+0.003      -003
5087          BD+06 2750 117405119961                       132455.8+063143132957.6+060048328.11 66.98 6.51  +0.96 +0.59         K0                 -0.017+0.050      -019
5088 72    VirBD-05 3706 117436139370        8924   6281    132512.6-055715133025.7-062813320.03 55.10 6.09  +0.33 +0.07         F2V                +0.042+0.017      -009SB    131  4.9  29.8      *
5089          CD-38 8592 117440204545      W        6283    132514.6-385327133102.7-392427311.22 22.83 3.88  +1.17 +1.03 +0.59   G9Ib*              -0.012-0.015 +.012-002SB         0.1   0.2      *
5090          CD-27 9278 117558181723                       132559.7-273550133133.3-280646313.60 33.93 6.47  +0.10               A1V                -0.062+0.002      -004
5091          BD+79  422 117566  78213075                   132605.5+790938132656.7+783838120.71 38.33 5.77  +0.77 +0.35         G2.5IIIbCH1        -0.146+0.029      +015
5092          CD-37 8695 117597204561                       132617.8-375258133205.3-382357311.62 23.79 6.16  +1.04               K1III              +0.035-0.043      +004V
5093          CP-64 2465 117651252361                       132637.4-650704133335.7-653757307.27-03.12 6.37  -0.02 -0.08         A0V                -0.048-0.012      +012
5094 73    VirBD-17 3877 1176611579873078  W       HX Vir   132639.1-181248133202.8-184344316.15 43.10 6.01  +0.18 +0.16 +0.07   F0IV-V             -0.099-0.014      -018       60  0.2   0.1      *
5095 74    VirBD-05 3714 1176751393903079I          6297    132645.9-054422133157.9-061521320.78 55.21 4.69  +1.62 +1.95 +1.16   M2III              -0.104-0.045 +.020+018V
5096          BD+42 2405 117710 44637                       132655.4+423714133115.8+420622 96.97 72.96 6.08  +1.05               K2III              -0.093+0.026 +.012-020
5097          CD-2810128 1177161817373080                   132701.6-281039133235.9-284134313.74 33.31 5.69  +0.04               A1Vn               -0.095-0.022      -022
5098          CD-2810127 117718181735                       132659.0-290303133234.5-293355313.54 32.46 6.45  +0.44  0.00?        F3IV               -0.072+0.004      -020
5099 75    VirBD-14 3739 117789157998    I W                132730.9-145055133251.7-152147317.47 46.34 5.55  +1.23 +1.22         K1.5IIIb           -0.073-0.002      -040V          5.6  79.6AB   3
5100 76    VirBD-09 3711 1178181394013081I                  132741.9-093859133258.1-100954319.44 51.37 5.21  +0.96 +0.60 +0.50   K0III              -0.029-0.036 +.021-001     < 19:
5101          BD-06 3837 117833139403    I         S Vir    132746.7-064051133300.7-071142320.76 54.23 6.68  +1.28 +0.91 +1.86   M7IIIe             +0.050-0.005      +010                          *
5102          BD+25 2643 117876 82875        8937           132803.9+245158133248.1+242047 18.61 80.28 6.11  +0.96 +0.70 +0.53   G8III              +0.049-0.206      +006V?         6.7   1.5
5103          CD-47 8417 117919224254      W                132824.1-474532133428.9-481620310.29 13.98 6.33  -0.05               B8III              -0.029-0.012      -004           2.0   0.5
5104          CD-32 9459 1180102046123082           6315    132902.7-324752133443.6-331839313.24 28.69 6.44  +1.26               K2III              -0.049-0.012      -009
5105 78    VirBD+04 2764 1180221200041351          CW Vir   132903.8+041021133407.9+033932328.27 64.41 4.94  +0.03  0.00 -0.03   A1pSrCrEu          +0.044-0.024 +.021-012SB     15                 *
5106          BD-12 3843 118054158021        8954   6316    132921.3-124205133440.5-131252318.87 48.32 5.91  +0.02 -0.09         A2pSrEu:Cr:        -0.044-0.010D+.007-020V?         0.4   0.3AB   3*
5107 79Zet VirBD+00 3076 118098139420 501                   132935.8-000505133441.6-003545325.25 60.39 3.37  +0.11 +0.10 +0.06   A3V                -0.285+0.042 +.044-013      173                 *
5108          BD+39 2658 118156 63616        8956           132956.7+391804133421.8+384721 88.07 75.29 6.37  +0.21 +0.11         F0IV               +0.013+0.024      -012       97  4.3  30.6      *
5109 81    UMaBD+56 1667 118214 28803               6319    133016.6+555139133407.3+552055110.49 60.74 5.60  -0.03 -0.09         A0V                -0.022-0.002 +.013-007V     156
5110          BD+37 2426 118216 63623 502          BH CVn   133019.9+374141133447.8+371057 83.33 76.41 4.98  +0.40 +0.06 +0.30   F2IV               +0.086-0.009 +.022+007SBO     0                 *
5111 80    VirBD-04 3515 1182191394281352                   133019.0-045313133531.3-052346322.71 55.77 5.73  +0.95 +0.66         G6III              +0.017+0.085      -008
5112 24    CVnBD+49 2227 118232 446683083           6322    133022.1+493138133427.3+490058104.99 66.59 4.70  +0.12 +0.11 +0.02   A5V                -0.127+0.030 +.034-018V     178                 *
5113          CP-61 3841 118261252387      W                133025.2-611040133712.2-614131308.33  0.70 5.63  +0.50               F6V                +0.135-0.113D+.032+040SB    111  0.2   0.3AB   3*
5114          BD+10 2565 118266100630      W                133035.2+104305133533.3+101217336.59 70.14 6.49  +1.03 +0.90 +0.52   K1III+F6V          +0.077-0.061      +033V?         2.0  70.2      *
5115          CP-75  882 118285257069 503                   133038.2-751025133911.9-754102305.94-13.09 6.34  +0.01 -0.26         B8IV               -0.027-0.006      +018SB2                       *
5116          BD+44 2285 118295 446751353                   133058.7+444229133514.1+441149 98.48 70.79 6.84  +0.20 +0.25         A7-F0III           -0.016+0.015      -026      101
5117          CD-33 9189 118319204653                       133107.0-335722133650.5-342804313.47 27.47 6.50  +1.03 +0.85         K0III              -0.011-0.028      +022V?
5118          CD-43 8418 1183382242753084                   133107.7-433756133706.0-440836311.52 17.97 5.98  +0.94               G8-K0III           -0.050-0.020      +006
5119          CP-69 1898 1183442570703086                   133110.2-695603133845.7-702642306.91-07.94 6.10  +1.42 +1.73         K3III              -0.068-0.032      +002
5120          CD-25 9900 118349181790        8966A          133115.4-255906133648.4-262942315.38 35.28 5.78  +0.22 +0.19         A7III              -0.091+0.021 +.021-010           1.0  10.1AB   4*
5121          CD-45 8578 118354224283                       133120.6-455501133723.7-462542311.13 15.71 5.90  -0.12               B8V                -0.009-0.033      +003V?
5122          CP-57 6169 118384241013      W                133133.8-575414133807.6-582454309.03  3.90 6.42  +1.12 +0.93         K1III              -0.008-0.036      +001           4.6   2.5
5123          BD+25 2652 118508 829053085I          6342    133216.7+250724133659.1+243648 21.68 79.45 5.74  +1.55 +1.85         M2III              -0.024-0.005      -031V
5124          CP-56 5856 118520241026      W                133217.2-570649133849.1-573723309.26  4.67 6.01  +1.14               G5Ib               -0.006-0.014      -010           0.0   0.1
5125          CP-70 1653 118522257074                       133219.6-701644134000.6-704718306.94-08.30 6.59  +1.30 +1.09         K0III              -0.026-0.028      -003
5126          BD+50 2014 118536 44682                       133235.8+495949133639.8+492912104.68 66.01 6.49R                     K1III              -0.004-0.012      -010
5127 25    CVnBD+37 2433 118623 63648        8974           133301.0+364813133727.6+361742 78.96 76.61 4.82  +0.23 +0.09 +0.13   A7III              -0.099+0.027 +.030-006V     204  2.0   1.7AB   3*
5128          CD-2810181 118646181825                       133304.8-290300133842.0-293339315.08 32.19 5.83  +0.40               F5V                -0.078-0.069      +004
5129          BD+15 2602 118660100644                       133314.3+144841133807.9+141806345.58 73.18 6.52  +0.25 +0.11         A9V s              +0.036-0.019      -002       90                 *
5130          CP-63 2896 118666252400                       133310.5-640407134010.9-643437308.14-02.20 5.79  +0.39 +0.02         F3III-IV           -0.059-0.012      +004
5131          BD+77  516 118686  7848    I                  133323.5+770326133442.8+763248119.65 40.30 6.57R                     K5III:             -0.026-0.003      -014
5132   Eps CenCP-52 6655 118716241047 504  W       Eps Cen  133332.9-525728133953.2-532759310.19  8.72 2.30  -0.22 -0.92 -0.24   B1III              -0.028-0.016      +003      159  8.4  36.9      *
5133          BD+51 1856 118741 28819    I   8979           133342.6+511325133743.0+504253105.56 64.82 6.48  +1.55 +1.66 +1.05   M2II-III+K3III:    -0.012+0.001D+.003-048V          1.3   1.8      *
5134          CD-49 8095 118767224317    I         V744 Cen 133348.8-492633133959.7-495701310.89 12.17 6.00  +1.50 +1.15 +1.75E  M5III              -0.113+0.011      -011                          *
5135          CD-39 8390 118781204708              V765 Cen 133348.4-391422133940.8-394453312.92 22.19 6.27  +1.66 +1.78         M4III              -0.020-0.025      -048V                         *
5136          CD-39 8392 118799204712                       133355.9-393232133948.6-400307312.88 21.88 5.60  +1.30               K2/3III            -0.044-0.061      +001
5137          BD+19 2697 118839100650                       133413.5+184626133902.3+181555356.46 75.91 6.48  +1.20 +1.27         K3III              -0.045-0.018      -011
5138          BD+11 2589 118889100654        8987   6363    133438.7+111515133934.6+104446339.90 70.08 5.57  +0.33 +0.05         F0V                -0.112-0.013 +.011-018V     111: 0.2   0.1AB   3*
5139          BD+71  659 118904  7854 505I                  133446.8+714504133711.0+711432117.72 45.42 5.50  +1.20              gK2                 -0.038-0.006      +015
5140          CP-58 5059 118978241080                       133522.9-581651134201.1-584714309.46  3.44 5.38  -0.03 -0.23         B9III              -0.035-0.010      -008V     228:                *
5141          CP-53 5725 118991241076      W                133520.3-540309134144.7-543336310.25  7.60 5.01  -0.05 -0.23         B8Vn               -0.048-0.047D+.003+010V     281: 1.4   5.4      *
5142 82    UMaBD+53 1640 119024 28832                       133538.3+532536133930.4+525517107.04 62.70 5.46  +0.10 +0.04         A3Vn               -0.142+0.060 +.011-015V     181
5143          BD+31 2526 119035 63676                       133542.4+313058134015.6+310043 55.83 78.65 6.21  +0.96               G5II:              -0.078+0.087      -018SB
5144  1    BooBD+20 2858 119055 82942        8991           133553.8+202741134040.5+195720  3.08 76.64 5.75  +0.01  0.00         A1V                -0.049+0.023D+.018-026V      45  3.5   4.8AB   3*
5145          BD+28 2248 119081 82944      W                133602.2+283416134039.1+280355 40.68 79.06 6.23  +1.28 +1.50         K3III              -0.066+0.014      -063           3.3  91.2
5146          BD-22 3645 1190861818631354    8994           133559.6-225638134130.9-232659317.55 38.00 6.59  +0.06 +0.05         A0                 -0.009+0.003      -013           0.0   0.1AB   3*
5147          CD-32 9549 119090204739    I:        T Cen    133601.6-330530134145.7-333549314.80 28.11 6.05  +1.44 +1.10 +1.09   K0e-M4IIe         v-0.014+0.026      +028V                         *
5148          BD+51 1859 119124 28836        8992           133625.3+510126134023.2+503110104.46 64.82 6.32  +0.54 -0.03         F7-9V              -0.132+0.060 +.049-015V      12  4.3  17.6      *
5149  2    BooBD+23 2600 119126 82946                       133618.4+230009134102.3+222945 13.26 77.84 5.62  +1.01               G9III              -0.020-0.025      +005
5150 82    VirBD-07 3674 1191491394901355I          6390    133621.7-081154134136.8-084211323.42 52.16 5.01  +1.63 +1.95 +1.16   M1.5III            -0.094+0.040 +.013-037
5151          CP-56 5891 1191592410961356                   133623.7-561546134256.1-564605309.98  5.40 6.00  -0.08               B0.5III            -0.011-0.004      -023V
5152          CD-50 7983 119193241098                       133638.8-501706134254.6-504726311.19 11.25 6.41  +1.67               M0III              -0.007-0.027      -010
5153          BD+57 1456 119213 28838              CQ UMa   133641.9+574247134021.3+571227110.18 58.69 6.29  +0.10 +0.02         A2VpSrCrEu        v-0.057+0.024      -000                          *
5154 83    UMaBD+55 1625 119228 288433087I          6389    133656.7+551116134044.3+544054108.18 61.01 4.66  +1.64 +1.96 +1.12   M2IIIabBa0.5       -0.022-0.009 +.014-017
5155          CD-40 8096 119250224359                       133658.8-405342134255.0-412404313.20 20.44 5.98  +1.02               K0III              -0.059-0.049      +028
5156          BD+09 2798 119288120075              Var      133716.5+085344134212.7+082318337.96 67.74 6.16  +0.42 -0.04         F3Vp               -0.382-0.090 +.035-011V      12                 *
5157          CD-41 8089 119361224365      W                133741.9-413348134340.0-420403313.20 19.76 5.98  -0.08               B8III              -0.026-0.006      -021           0.0   0.1AB   3*
5158          CD-50 7998 119419241120              V827 Cen 133759.0-503030134415.9-510047311.35 10.99 6.47  -0.13               A0pSiCr            -0.036-0.041      +009                          *
5159 84    VirBD+04 2775 119425120082    I   9000   6401    133802.0+040238134303.7+033217332.78 63.38 5.36  +1.11 +1.04 +0.54   K2III              -0.291-0.071 +.006-041SB         2.8   3.0      *
5160          BD+42 2431 119445 44720                       133813.1+421041134228.8+414027 90.49 72.09 6.30  +0.86               G6III              -0.090+0.007      -033
5161          BD+35 2474 119458 637013089                   133816.0+352934134243.4+345920 71.71 76.51 5.98  +0.85               G5III              +0.013+0.012      -015SBO                       *
5162          BD+65  953 119476 161423088                   133823.1+651939134129.9+644921114.42 51.47 5.85  +0.07 +0.08         A2V                +0.054-0.012      -006SB    105
5163          BD-04 3540 119537139516                       133841.9-045943134354.3-052956326.12 54.99 6.51  +0.05 +0.01         A1V                -0.050-0.018      -022SB  =< 38
5164          BD+23 2606 119584 82969    I                  133901.6+231217134345.2+224201 15.32 77.35 6.13  +1.42 +1.70 +0.75   K4III              +0.049-0.036      +009                          *
5165 83    VirBD-15 3731 1196051581311357                   133906.0-154034134429.8-161045321.03 44.82 5.60  +0.81 +0.41 +0.40   G0Ib-IIa           +0.013-0.004      +001V                         *
5166          CD-2411057 119623181921                       133911.6-245952134445.7-253003317.81 35.83 6.21  +1.38               K0                 -0.053-0.016      +006V
5167          CD-25 9972 119752181931                       134001.9-253651134536.9-260658317.84 35.18 5.81  +0.02               A1Vn               -0.071-0.010      -010V
5168  1    CenCD-32 9603 119756204812 506          Var?     134000.2-323217134541.2-330238315.86 28.46 4.23  +0.38  0.00 +0.21   F3IV               -0.462-0.146 +.051-022SBO    86                 *
5169          BD+52 1733 119765 28866                       134002.1+523402134354.7+520352104.91 63.19 6.02   0.00 -0.01         A1V                -0.026-0.003      -007V?     77
5170 85    VirBD-15 3735 119786158147      W                134011.9-151554134535.1-154603321.56 45.14 6.19  +0.05               A2Vn               -0.044-0.024      -041SB         1.7   0.0 O   4*
5171          CP-61 4003 119796252448    I W       V766 Cen 134012.1-620520134710.7-623524309.30-00.41 6.51  +1.98 +1.19 +1.20   K00-Iae            -0.023-0.011      -036V          4.0   9.4      *
5172          CD-50 8017 119834241157      W        6434    134019.4-505551134639.3-512558311.63 10.50 4.65  +0.96 +0.72         G8-K0III           +0.006-0.033 +.018-006SB1O       6.3  40.       *
5173 86    VirBD-11 3591 119853158152        9018           134036.5-115532134556.3-122536323.16 48.29 5.51  +0.90               G8III              -0.017+0.008      -011V?         5.0   1.2AB   4*
5174          CD-35 8995 1199212048353091  W                134106.5-354504134656.4-361507315.28 25.29 5.15  -0.02               A0V                -0.011-0.009 +.019-010      437  7.3  26.3
5175          CD-49 8194 1199382411733092                   134109.6-494458134727.6-501458312.02 11.63 5.91  +0.29               A3mA5F0            +0.058+0.013      +016
5176          CD-49 8198 119971241177                       134122.3-494914134738.4-501916312.04 11.55 5.45  +1.36 +1.36         K2III              -0.151-0.023 +.013+028
5177          BD+56 1683 119992 28878                       134131.4+562326134513.2+555246107.97 59.66 6.50  +0.47 -0.06         F7IV-V             +0.102-0.360 +.024-004        9
5178          BD-08 3639 120033139544    I W                134156.2-091230134713.5-094233324.99 50.76 6.05  +1.42 +1.66         K5III              +0.008-0.037      +007V          1.6   0.4      *
5179          BD+41 2424 120047 44742                       134158.7+413525134613.5+410519 87.52 72.06 5.87  +0.21 +0.06         A5V                -0.114-0.043 +.012-013V?    199
5180          BD+39 2678 120048 63735                       134159.2+390015134619.0+383014 81.11 73.84 5.94  +0.94               G9III              -0.051-0.002      -014
5181 87    VirBD-17 3932 120052158165    I          6442    134158.9-172133134725.4-175136321.28 43.01 5.43  +1.62 +1.94 +0.89E  M2IIIab            +0.059-0.034      +064                          *
5182  3    BooBD+26 2494 120064 829931358                   134204.7+261214134643.3+254208 29.48 77.55 5.95  +0.49 +0.10         G5III+A7V:         -0.020-0.061      +008SB2O                      *
5183          BD+07 2690 120066120108                       134159.9+065112134657.1+062102337.86 65.38 6.33  +0.63 +0.16         G0-1IV-V           -0.510-0.108 +.026-031V?  =< 10  3.5 486.       *
5184          BD+78  466 120084  78763090                   134213.2+783354134239.3+780352119.56 38.73 5.91  +1.01              gG7                 -0.062+0.049 +.034-007V?                        *
5185  4Tau BooBD+18 2782 120136100706 507    9025   6444    134230.6+175719134715.7+172724358.95 73.88 4.50  +0.48 +0.04 +0.24   F6IV               -0.482+0.041 +.059-016V      14  7.5   5.0      *
5186          BD+39 2680 120164 637393094  W                134241.0+390234134659.8+383234 80.90 73.71 5.50  +1.03 +0.84 +0.52   K0III+F8V          -0.131-0.016      -010SB         3.2  71.3
5187 84    UMaBD+55 1634 120198 28885              CR UMa   134251.7+545557134635.7+542558106.38 60.89 5.70  -0.08 -0.08         B9pEuCr           v-0.019+0.001      -002V?     37                 *
5188          CP-82  585 1202132586833986                   134252.1-821014135538.7-823958305.09-20.06 5.95  +1.46 +1.57  0.00   K2.5IIIaCH-3Ba0.5  -0.036-0.023      -035V?                        *
5189          CD-35 9019 120237204867      W                134309.5-351203134855.1-354214315.87 25.72 6.53  +0.57 +0.05         G3IV-V             -0.516-0.174 +.039+004           3.0  11.5      *
5190   Nu  CenCD-41 8171 120307224469              Nu  Cen  134330.2-411122134930.3-414116314.41 19.89 3.41  -0.22 -0.84 -0.23   B2IV               -0.022-0.021      +009SB1O   91                 *
5191 85Eta UMaBD+50 2027 120315 44752 509I          6450    134336.0+494845134732.4+491848100.69 65.32 1.86  -0.19 -0.67 -0.18   B3V                -0.122-0.011 +.035-011SB?   205                 *
5192  2    CenCD-33 9358 120323204875    I         V806 Cen 134338.9-335705134926.7-342703316.32 26.91 4.19  +1.50 +1.45 +1.81   M4.5III            -0.042-0.057 +.031+041                          *
5193   Mu  CenCD-41 8172 120324224471 508  W       Mu  Cen  134335.4-415832134937.0-422826314.24 19.12 3.04  -0.17 -0.72 -0.11   B2IV-Ve           v-0.023-0.020      +009SB    175 10.7  48.       *
5194          CP-68 2014 1204042524813099                   134400.8-685417135147.4-692405308.23-07.15 5.75  +1.73 +2.02         K5-M0III           +0.004-0.003      +010V?
5195          BD+31 2547 120420 637603096                   134408.0+314113134838.7+311125 54.35 76.88 5.62  +1.01               K0III              -0.007+0.037      +011
5196 89    VirBD-17 3937 120452158186 510I                  134426.1-173810134952.3-180803321.93 42.57 4.97  +1.06 +0.92 +0.50   K0.5III-IIIb       -0.100-0.038 +.046-040     < 19:                *
5197          CD-2810277 1204551819993097           6460    134426.3-283501135006.5-290453318.07 32.06 6.18  -0.03 -0.12         A0V                -0.048-0.032      +000
5198          CD-39 8501 120457204888                       134421.9-392413135019.4-395404315.03 21.58 6.44  +0.99               K1III              +0.007-0.012      -004
5199          BD+40 2694 120499 63763    I         R CVn    134439.7+400224134857.2+393234 82.66 72.77 7.4                       M6IIIe            v+0.023-0.002 -.006-006V                         *
5200  5Ups BooBD+16 2564 120477100725    I          6458    134439.2+161738134928.6+154752355.81 72.40 4.07  +1.52 +1.87 +0.87   K5.5III           e-0.095+0.042 +.011-006V    < 17                 *
5201  6    BooBD+21 2578 120539 830153098I                  134459.0+214538134942.8+211551 12.25 75.49 4.91  +1.43 +1.65 +0.74   K4III              +0.024+0.017      -003     < 19:
5202          BD-19 3754 120544158192                       134505.8-192406135034.5-195350321.42 40.84 6.53  +0.51               G5III-IV:+A5:      -0.046+0.034      -030V      30
5203          BD+83  397 120565  22661643                   134510.2+831515134223.1+824509121.01 34.18 5.98  +1.01               G9III              +0.035-0.041      -050SB
5204          BD+37 2457 120600 63772                       134523.5+370743134945.0+363758 74.14 74.42 6.38R                     A7IV-V             -0.074+0.022      -012       82
5205          BD+06 2800 1206021201323100                   134523.3+055937135024.7+052950338.54 64.21 6.01  +0.90 +0.59         K0                 +0.009-0.003      -024
5206          CD-46 8909 120640224489                       134535.3-462410135147.2-465357313.53 14.73 5.77  -0.16 -0.75         B2Vp               -0.018-0.030      -005V?    100                 *
5207          CP-52 6787 120642241239      W                134538.1-521856135204.8-524842312.14  8.97 5.25  -0.09 -0.32         B9V                -0.045-0.035 -.001+027      186  2.3  18.1      *
5208          CD-35 9054 120672204911                       134545.4-355605135136.6-362600316.24 24.88 6.35  +0.48 +0.01         F6IV-V             -0.100-0.117      +000SB2                       *
5209          CD-2311329 120690182026                       134549.8-235313135120.4-242327319.96 36.49 6.45  +0.69 +0.27         G5V                -0.572-0.302 +.062+002                          *
5210  3    CenCD-32 9676 120709204916      W     A          134603.1-322953135149.6-325940317.28 28.19 4.56  -0.13 -0.60 -0.13   B5IIIp             -0.033-0.038D+.017+010       12  1.5   7.9      *
5211  3    CenCD-32 9676 120710204917      W     B          134603.8-322956135150.1-325941317.28 28.19 6.06  -0.04 -0.15         B8V                -0.056-0.027D+.017+000      180  1.5   7.9      *
5212          CD-3110706 120759204922      W                134616.5-310723135200.9-313710317.74 29.50 6.12  +0.48               F7V                -0.053-0.052D+.016+009           1.0   0.6
5213          BD+62 1318 120787 161863101                   134630.2+615917134945.5+612921111.06 54.27 5.96  +0.96 +0.64         G3V                +0.065-0.099      -011V
5214          BD+35 2492 120818 63779                       134639.7+351604135104.5+344621 67.50 75.13 6.65  +0.12 +0.07 +0.01   A5IV               +0.031-0.010      -012V     108                 *
5215          BD+35 2493 120819 63781    I          6468    134644.4+350939135109.2+343952 67.10 75.16 5.87  +1.62 +1.96         M2III              +0.005-0.054 +.014-040
5216          BD+59 1533 120874 28901                       134701.4+590204135027.7+583222108.83 56.94 6.46  +0.09 +0.08         A3V                -0.029+0.011      -040SB
5217          CP-52 6805 120908241262                       134713.3-525246135343.1-532225312.25  8.37 5.89  +0.01 -0.41  0.00   B5III              -0.024-0.029      +007       23                 *
5218          CP-67 2426 120913252511                       134712.5-670932135448.9-673909308.92-05.52 5.71  +1.49 +1.67         K2III              -0.034-0.027      -008
5219          BD+35 2496 120933 637933102I         AW CVn   134722.8+345622135147.5+342639 66.10 75.14 4.74  +1.66 +1.96 +1.28   K5III              -0.019-0.032 +.026-044                          *
5220          BD+12 2635 1209341007453104                   134724.4+123934135218.4+120955349.54 69.32 6.04  +0.04 +0.01         A1V                +0.028-0.005      -016       74
5221  4    CenCD-3110729 120955204944      W                134727.0-312601135312.5-315540317.93 29.14 4.73  -0.14 -0.54 -0.08   B4IV               -0.015-0.012      +005SBO    27  3.7  14.9      *
5222          CD-35 9090 120987204955      W                134741.9-351014135332.8-353951316.88 25.52 5.54  +0.44 -0.01         F4V                -0.078-0.019D+.009-008SB      0  0.2   0.9AB   5*
5223          CD-46 8931 120991224514      W       V767 Cen 134743.5-463806135356.9-470741313.84 14.42 6.10  -0.07 -0.92 -0.10   B2IIIe            v-0.037-0.001      -021       70  4.9  21.4      *
5224          CD-34 9223 121056204963                       134803.4-344910135352.2-351852317.06 25.84 6.19  +1.02 +0.86         K1IV-V             -0.270-0.075      +005V?
5225  7    BooBD+18 2795 121107100751                       134826.2+182532135312.9+175558  3.30 73.05 5.70  +0.84               G5III              -0.039+0.004      -010V?
5226 10    DraBD+65  963 121130 16199 511I   9039  CU Dra   134830.6+651302135125.9+644324112.77 51.21 4.65  +1.58 +1.89 +1.36   M3.5III            +0.001-0.002 +.014-011V?         7.7  90.2AC   3*
5227          BD+69  724 121146 161973103  W                134831.8+684839135059.2+681855114.79 47.86 6.40  +1.17       +0.60   K2IV               -0.172-0.055 +.025-045           1.8  79.1AB   3*
5228          CD-27 9478 121156182065                       134836.8-280432135416.6-283411319.27 32.30 6.04  +1.13               K2III              -0.170-0.069      -017V
5229          BD+29 2464 121164 830553106                   134838.1+290825135310.3+283853 42.88 76.28 5.90  +0.20 +0.14         A7V                -0.125+0.027      -012       57
5230          CD-51 7832 121190241294                       134845.0-514007135512.2-520940312.77  9.49 5.71  -0.08 -0.30         B9V                -0.027-0.024      +023V                         *
5231   Zet CenCD-46 8949 121263224538 512                   134917.9-464745135532.4-471718314.07 14.19 2.55  -0.22 -0.92 -0.20   B2.5IV             -0.057-0.042      +007SB2O  219                 *
5232 90    VirBD-00 2758 1212991396133107I                  134933.9-010040135442.1-013011333.47 57.57 5.15  +1.08 +1.08 +0.52   K2III              -0.088-0.018 +.027-007V?   < 17
5233          BD-07 3728 121325139618        9053           134943.3-073400135458.2-080332328.80 51.59 6.19  +0.53 +0.01         F8V+G0             -0.172-0.030 +.020-019V          0.9   3.4AB   4*
5234          CP-53 5805 121336241309      W                134946.1-533828135619.8-540755312.44  7.54 6.14  +0.07 +0.03         A1V                -0.038-0.014D+.012+013           1.0   1.8AB   3
5235  8Eta BooBD+19 2725 121370100766 513I W                134955.3+185356135441.1+182352  5.30 73.03 2.68  +0.58 +0.20 +0.29   G0IV               -0.063-0.358 +.108-000SB1O   13  6.0 112.6      *
5236          CP-54 5806 121384241315      W                134957.2-541230135633.0-544216312.32  6.98 6.00  +0.78               G6IV-V             -0.038-0.216      +005           6.9  33.
5237          CD-3011015 121397204994                       134959.4-304740135544.5-311706318.73 29.61 6.51  +0.90               G8III              -0.047+0.009      -018
5238 86    UMaBD+54 1630 121409 28928                       135010.2+541313135351.0+534343103.76 61.01 5.70  -0.05 -0.07         A0V                -0.030-0.003 +.012-016V?    225
5239          CD-45 8815 121416224555                       135007.3-460600135619.5-463533314.39 14.83 5.83  +1.14 +1.11         K1III              -0.164-0.076 +.025-001                          *
5240          CP-77  922 1214392571073111                   135013.4-780607140032.8-783524306.44-16.19 6.09  +0.03 -0.17         B9III              -0.021-0.005      +006V
5241          CP-63 3070 121474252531 514                   135024.5-631147135738.9-634112310.19-01.75 4.71  +1.11 +1.04 +0.40E  K1.5III            -0.044-0.030 +.033+022
5242          CP-65 2553 1215572525343110  W                135103.2-651840135831.2-654802309.74-03.82 6.20  +1.05 +0.81         K0III              -0.033-0.036      -029           4.0   6.4
5243          BD+14 2680 1215601007763109                   135100.8+143247135550.0+140323355.12 70.09 6.16  +0.50 -0.08         F6V                -0.292+0.007      -013    =<  6
5244 92    VirBD+01 2865 121607120185                       135122.1+013223135627.9+010302336.53 59.61 5.91  +0.20 +0.14         A8V                -0.026+0.014      -023      181
5245          BD+32 2411 121682 638371360                   135144.2+323114135610.5+320157 56.03 75.11 6.32  +0.37 +0.06         F4IV-V             -0.118+0.048 +.014-022        8
5246          BD-22 3687 121699182123    I                  135153.6-223204135727.7-230122322.14 37.37 6.14  +1.43               K0                 -0.041+0.006      +001V?
5247  9    BooBD+28 2278 121710 83084    I          6502    135200.2+275857135634.2+272931 38.12 75.54 5.01  +1.42 +1.72 +0.75   K3III             v+0.028-0.048 +.008-040     < 19:                *
5248   Phi CenCD-41 8329 121743224577               6509    135211.4-413644135816.3-420603315.98 19.07 3.83  -0.21 -0.83 -0.22   B2IV               -0.024-0.021      +006V     126                 *
5249   Ups1CenCD-44 9010 1217902245853112                   135230.0-441856135840.8-444813315.29 16.45 3.87  -0.20 -0.80 -0.21   B2IV-V             -0.025-0.024      +005V     122                 *
5250 47    HyaCD-2411202 121847182134 515                   135254.3-242903135831.1-245820321.66 35.45 5.15  -0.10 -0.40         B8VpShell          -0.049-0.029      +005SB                        *
5251          CD-49 8356 121853241352                       135252.1-495256135917.3-502212313.86 11.06 5.91  +0.96               G8-K0III           +0.011-0.034      -005
5252          CP-60 5135 121901252552                       135312.2-605938140017.3-612853311.06  0.30 6.49  +0.33 -0.04         F1III-IV           -0.072-0.040      +006
5253          CP-65 2573 121932252554      W                135320.0-654657140052.2-661607309.85-04.33 5.97  +0.35 +0.07         F2III              -0.136-0.016      -017           5.9  46.2
5254          BD+15 2651 121980100801    I                  135350.2+150815135839.9+143858357.75 69.98 6.00  +1.44 +1.71         K5III              -0.061-0.056      -041
5255 10    BooBD+22 2650 121996 831033113                   135357.7+221103135838.9+214146 17.02 73.78 5.76  -0.03 +0.02         A0V s              -0.009-0.045 +.013+006       60
5256          BD+62 1325 122064 16230                       135425.6+615826135732.1+612934109.61 53.90 6.37R                     K3V                -0.034+0.210      -025
5257 48    HyaCD-2411215 1220661821521361                   135424.0-243120140000.1-250037322.04 35.31 5.77  +0.48               F7V                -0.196-0.091 +.029-017V?
5258          BD-02 3768 1221061396661362                   135438.3-030345135949.3-033259333.84 55.17 6.40  +0.49 +0.06         F8V                -0.025-0.058      -008V?     12
5259          CD-39 8628 122210205096      W                135516.6-394416140119.0-401320317.12 20.72 6.13  +1.25 +1.29         K1III              -0.037-0.006      -045           6.2  10.
5260   Ups2CenCD-44 9040 122223224621                       135529.1-450708140143.4-453613315.60 15.53 4.34  +0.60 +0.27 +0.33   F6II               +0.002-0.019 +.013-001SB1O    0                 *
5261   The ApsCP-76  799 1222502571121363I:        The Aps  135534.5-761851140519.8-764748307.22-14.54 5.5   +1.55 +1.05 +1.77   M6.5III:           -0.094-0.032      +010                          *
5262          BD+09 2835 1223651202283114                   135623.4+092243140120.4+085341348.38 65.40 5.99  +0.09 +0.10         A2V                +0.036-0.007      -017      100
5263 11    BooBD+28 2287 122405 83130 517                   135638.4+275211140110.5+272312 37.90 74.51 6.23  +0.17               A7III              -0.080+0.017      -024V     102
5264 93Tau VirBD+02 2761 122408120238 516    9085A          135633.3+020142140138.8+013240339.22 59.38 4.26  +0.10 +0.12 +0.06   A3V                +0.018-0.021 +.024-002SB    150  5.2 129.0AB   3*
5265          CD-2610060 122430182182    I                  135641.4-265649140222.8-272548321.70 32.84 5.48  +1.34 +1.44         K3III              -0.033-0.008      +000
5266          CP-55 5846 122438241403                       135640.8-554350140326.2-561249312.87  5.26 5.92  +1.22 +1.18         K2III              -0.083-0.031      +017
5267   Bet CenCP-59 5365 122451252582 518  W       Bet Cen  135645.8-595326140349.4-602223311.77  1.25 0.61  -0.23 -0.98         B1III             v-0.032-0.019 +.009+006SB    139  3.2   1.3      *
5268          CD-3110859 122510205123      W                135713.2-311214140301.7-314102320.29 28.76 6.18  +0.48               F8V                +0.022+0.083 +.018+005           1.1   2.1      *
5269          CD-40 8373 1225322246411364          V828 Cen 135721.7-405628140327.5-412524317.16 19.45 6.11  -0.11 -0.39 -0.11   ApSi               -0.034-0.017      +004V                         *
5270          BD+10 2617 122563120251               6526    135737.4+101013140231.8+094111350.17 65.80 6.20  +0.90 +0.38 +0.58   F8IV               -0.196-0.063      -023                          *
5271          BD+46 1922 122675 448583115                   135813.8+461415140212.2+454513 90.56 66.67 6.27  +1.32 +1.45         K2III              +0.033-0.076      -049
5272          BD-21 3824 1227031822043117                   135818.3-215627140353.1-222518324.15 37.45 6.30  +0.45               F5III              -0.012+0.013      -019
5273          BD+11 2625 122742100832                       135837.5+111634140332.3+104712352.46 66.45 6.30  +0.74 +0.29 +0.40   G8V                +0.083-0.307 +.059-017SB                        *
5274          BD+08 2810 122744120261                       135838.8+080139140336.8+073247347.39 64.00 6.26  +0.94 +0.70         G9III              -0.038-0.012      -020
5275          BD+05 2836 1227971202653118                   135854.5+052254140355.8+045403343.99 61.85 6.24  +0.39 -0.02         F4V                -0.011-0.003      -023       50
5276          BD-04 3614 122815139711                       135901.2-045403140414.6-052253334.07 53.03 6.39  +1.32 +1.58         K0                 -0.009-0.011      -009
5277          BD-14 3863 1228371583251365                   135902.0-142928140427.0-145818327.95 44.32 6.28  +1.08 +0.95         G6III              -0.036-0.019      -015V
5278          CP-54 5887 1228442414303119                   135904.6-541122140546.5-544010313.63  6.65 6.17  +0.23               A5III-IV           -0.050-0.019      -036V
5279          CP-74 1142 1228622571163120                   135916.0-742240140827.1-745101308.00-12.75 6.02  +0.58 +0.06         G2-3IV             -0.252+0.179 +.041-022
5280          BD+51 1889 122866 28989                       135916.4+512710140259.7+505819 98.11 62.57 6.15  +0.01               A2V                -0.021 0.000      -010       60
5281          CP-59 5395 122879241439               6544    135922.0-591414140625.1-594256312.26  1.79 6.42  +0.12 -0.80         B0Ia               -0.011+0.019      +002                          *
5282          BD+69  733 122909 16254    I                  135938.2+690936140150.6+684043113.63 47.16 6.34  +1.40               K5                 -0.035-0.001      -022
5283          BD+02 2768 122910120269                       135933.3+024638140437.5+021751341.25 59.61 6.28  +1.02 +0.87         K0                 -0.030-0.003      -029
5284          BD-15 3805 122958158331                       135947.0-155125140514.0-162009327.46 42.99 6.56  +0.10               A4V                +0.002+0.006      -013
5285   Chi CenCD-40 8405 122980224673              Chi Cen  135956.3-404202140602.8-411047317.73 19.54 4.36  -0.19 -0.77 -0.22   B2V                -0.017-0.020      +012V?     33                 *
5286          CD-42 9027 123004224676                       140000.1-423644140610.9-430531317.15 17.71 6.20  +0.98               G8III              -0.006-0.037      +022
5287 49Pi  HyaCD-2610095 123123182244 519I                  140040.5-261202140622.3-264057323.00 33.25 3.27  +1.12 +1.04 +0.55   K2-III-IIIbFe-0.5  +0.044-0.139 +.049+027V
5288  5The CenCD-35 9260 123139205188 520I'W                140047.7-355241140641.0-362212319.46 24.08 2.06  +1.01 +0.87 +0.53   K0-IIIb            -0.519-0.519 +.065+001          12.0  69.9      *
5289          CP-62 3941 1231512526163121                   140051.1-624356140814.3-631229311.46-01.62 6.40  +1.02               G8-K0III           +0.025+0.038      +039
5290 95    VirBD-08 3697 123255139736                       140125.4-085011140642.8-091848332.12 49.24 5.46  +0.34 +0.08         F2IV               -0.139+0.011      -036V     120
5291 11Alp DraBD+65  978 123299 16273 521           6546    140140.8+645114140423.3+642233110.52 50.96 3.65  -0.05 -0.08 -0.07   A0III              -0.055+0.018 +.018-013SBO    18                 *
5292          CP-58 5383 123335241478                       140153.1-584803140856.3-591636312.70  2.12 6.34  +0.05 -0.51         B5IV               -0.010-0.011      -002V      89                 *
5293          CP-69 2012 1233772571193123  W                140210.6-694951141030.9-701820309.55-08.46 6.05  +1.74 +1.84         K4II               -0.006-0.010      -001           8.5   5.2AB   3
5294          CD-42 9065 123445224721      W                140239.1-425943140851.9-432816317.52 17.20 6.17  -0.06               B9V                -0.019-0.028      -002V          6.6  28.6      *
5295          CP-69 2014 123492252636                       140247.3-691443141101.9-694311309.77-07.91 6.06  +0.18 +0.15         A6IV               -0.035-0.026      -003
5296          CD-50 8294 1235152414913122  W        6565    140300.7-510148140935.0-513017315.12  9.51 6.00  -0.06 -0.27         B9IV               -0.030-0.007      +005SB1O < 39  3.0  64.1      *
5297          CP-52 7028 123569241496      W                140315.3-525745140954.8-532621314.58  7.65 4.75  +0.94 +0.72 +0.32E  G9-III             -0.146-0.096 +.017-016           9.2  27.4      *
5298 96    VirBD-09 3865 123630158385      D                140340.8-095139140900.6-102004332.20 48.08 6.47  +1.00 +0.73         G8III              -0.005+0.025      -020V?                        *
5299          BD+44 2325 123657 449011368I         BY Boo   140355.9+441948140755.8+435116 85.26 67.26 5.27  +1.59 +1.66 +1.66   M4.5:III           +0.011-0.028 +.024-036                          *
5300 13    BooBD+50 2047 123782 449053124I W       CF Boo   140433.1+495550140817.3+492729 94.58 63.23 5.25  +1.65 +1.92 +1.12   M1.5III            -0.061+0.059 +.020-013V          4.5  79.7      *
5301          BD-15 3817 123934158401    I         ET Vir   140522.6-154947141050.5-161807329.16 42.49 4.91  +1.72 +2.13 +0.85E  M2IIIa             +0.006-0.006 +.012+018V?                        *
5302          BD+60 1516 123977 290193126                   140540.1+594841140846.0+592016105.75 55.14 6.46  +1.02 +0.82 +0.52   K0III              -0.123-0.017      +011V?
5303   Eta ApsCP-80  706 1239982586933129                   140539.5-803220141813.8-810028306.43-18.73 4.91  +0.25 +0.11         A2m                -0.031-0.063      -009SB2    43                 *
5304 12    BooBD+25 2737 123999 83203 522                   140550.2+253355141023.9+250530 30.82 72.18 4.83  +0.54 +0.07 +0.29   F9IV w             -0.022-0.061 +.042+011SB2O   26                 *
5305  3    UMiBD+75  529 124063  79533125                   140608.9+750403140656.4+743537116.34 41.55 6.45  +0.14 +0.12         A7V                -0.052+0.012      -004
5306          CP-77  940 124099257131                       140622.2-771148141654.9-773950307.57-15.57 6.47  +1.42 +1.42         K2IIp              -0.017+0.009      -010                          *
5307          BD+02 2783 124115120334                       140626.6+014957141131.2+012144342.99 57.87 6.43  +0.48 +0.04         F7V                -0.125+0.028      -018       30
5308          CP-53 5912 124147241543                       140632.4-531144141316.4-533957314.98  7.28 5.56  +1.44               K5III+B-A          -0.012-0.018 +.009-003
5309          CD-2311551 124162182343                       140644.9-235335141224.5-242151325.54 34.93 6.34  +1.35               K0                 +0.011-0.028      +011V
5310          BD+32 2443 124186 63979                       140653.4+324556141115.1+321744 54.36 71.95 6.11  +1.26 +1.43         K4III              -0.025+0.022      -022
5311          CP-54 5933 124195241552              V716 Cen 140652.2-540922141339.9-543733314.73  6.35 6.11  +0.05 -0.38         B5Ve               -0.019-0.015      +066V                         *
5312 50    HyaCD-2610158 124206182349    I                  140702.1-264726141246.0-271540324.34 32.21 5.08  +1.16 +1.13         K3III              -0.012-0.034 +.048+027
5313          BD+03 2867 1242241203393127    9152  CU Vir   140712.0+025248141215.8+022434344.43 58.61 5.01  -0.12 -0.42 -0.13   A0VpSi            v-0.042-0.026 +.009-002SB    123  6.5  59.9      *
5314          CD-2610163 124281182354                       140730.0-260833141313.2-263644324.74 32.77 6.24  +1.08               K0III              +0.009-0.014      -010V
5315 98Kap VirBD-09 3878 124294158427 523I                  140733.6-094830141253.8-101625333.51 47.70 4.19  +1.33 +1.47 +0.75   K2.5IIIFe-0.5      +0.008+0.140 +.022-004     < 19:                *
5316          CP-56 6206 124367241563      W       V795 Cen 140759.0-563704141457.0-570509314.13  3.96 5.07  -0.08 -0.63 +0.04   B4Vne             v-0.039-0.009      +007V     242  5.8  33.9AB   3*
5317          BD-00 2796 124425139798              Var      140831.2-002224141340.8-005044341.52 55.74 5.91  +0.47 +0.03         F7V w              +0.207-0.141 +.027+018SBO    30                 *
5318          CD-41 8589 124433224791                       140831.9-412210141442.6-415015319.14 18.40 5.61  +0.93 +0.58         G8III              -0.133-0.021      -038
5319          CP-52 7087 124454241569                       140836.5-530234141521.2-533035315.33  7.33 6.39  +1.57               K3III              -0.030+0.002      -021
5320          CP-66 2490 124471252678      W                140844.4-660718141638.6-663516311.26-05.10 5.75  -0.06 -0.87         B1.5III            -0.015-0.005      -009V      57  7.1  23.8      *
5321  4    UMiBD+78  478 124547  7958 524I S                140913.9+780103140850.9+773251117.67 38.78 4.82  +1.36 +1.39 +0.52E  K3III              -0.032+0.034 +.010+006SB1O < 17                 *
5322          BD-05 3837 124553139806                       140908.7-052858141421.3-055651337.22 51.32 6.36  +0.60 +0.16         F9V                -0.306+0.088 +.023-032
5323 14    BooBD+13 2764 124570100925      S        6597    140916.5+132542141405.2+125734  0.78 66.03 5.54  +0.54 +0.09         F6IV               -0.256-0.052 +.022-039SBO     0                 *
5324          CD-2810528 124576182374                       140913.7-284853141501.3-291655324.02 30.14 6.08                      A1V                -0.030-0.015      -013
5325          CD-44 9181 124580224798                       140917.6-443150141538.8-450003318.20 15.36 6.31  +0.60               F9V                +0.137-0.142      +003
5326          CP-59 5476 124601241580    I W       R Cen    140921.8-592652141634.2-595450313.42  1.21 6.39  +2.04 +1.24 +1.73   M5IIe             v-0.019-0.018      -020V          6.9  28.0      *
5327          CP-82  601 124639258697                       140935.1-822316142423.3-825055305.94-20.53 6.42  +0.02 -0.33         B8Ve               +0.001-0.019      +027
5328 17Kap1BooBD+52 1782 124674 29045        9173B          140952.8+521520141327.7+514716 96.46 60.92 6.69  +0.39 -0.04         F1V                +0.054-0.019 +.010-020SB     40  2.2  13.4      *
5329 17Kap2BooBD+52 1782 124675 29046        9173A Kap2 Boo 140954.0+521527141329.0+514725 96.46 60.91 4.54  +0.20 +0.14 +0.12   A8IV               +0.063-0.002 +.010-017SB    127  2.2  13.4      *
5330 15    BooBD+10 2654 1246791009343131I W                140956.9+103418141450.8+100602356.00 64.00 5.29  +1.00 +0.77 +0.50   K1III              -0.028-0.167      +017V?   < 19: 2.6   1.1      *
5331          BD+04 2841 124681120364    I         FS Vir   140950.5+034811141453.0+032010346.51 58.96 6.45  +1.60 +1.72 +1.05E  M4IIIab            -0.045-0.014      -048                          *
5332          BD-17 4046 124683158448                       140953.3-174403141524.1-181203329.44 40.31 5.43  -0.03 -0.08         A1V                -0.039-0.012      -019V      74                 *
5333          BD+22 2678 124713 832423130                   141002.9+222024141441.0+215224 21.95 70.38 6.39  +0.18 +0.09         A7V                +0.039-0.006      -004       67
5334          BD+70  778 124730 163053128I          6593    141012.4+695407141204.0+692557112.88 46.09 5.24  +1.58 +1.84         M2IIIab            -0.027-0.047 +.016-023V?
5335          BD+42 2472 124755 44952                       141021.5+415917141423.5+413108 78.52 67.73 6.24  +1.04              gK3                 -0.024-0.105 +.030-010SB
5336   Eps ApsCP-79  755 124771257142                       141016.6-793850142222.7-800632306.93-17.95 5.06  -0.10 -0.57         B4V                -0.025-0.009      +005V     216                 *
5337          CD-32 9982 1247802053713132                   141023.0-324634141618.3-331429322.67 26.36 6.55  +0.30               F0V                +0.022+0.004      +003
5338 99Iot VirBD-05 3843 124850139824 525I          6604    141046.1-053124141600.9-060002337.75 51.07 4.08  +0.52 +0.04 +0.27   F6III              -0.004-0.432 +.043+012       15                 *
5339   Del OctCP-83  557 124882258698               6636    141051.6-831235142654.9-834004305.68-21.31 4.32  +1.31 +1.45 +0.49E  K2III              -0.105-0.007      +005
5340 16Alp BooBD+19 2777 124897100944 526I          6603    141106.0+194211141539.7+191057 15.14 69.11-0.04  +1.23 +1.27 +0.65   K1.5IIIFe-0.5     v-1.093-1.998 +.090-005V    < 17                 *
5341          BD-05 3845 124915139828                       141106.0-060924141621.5-063719337.36 50.49 6.44  +0.28 +0.09         A9III              -0.032-0.016      -034
5342          BD-02 3812 124931139830               6608    141119.1-024351141630.1-031147340.32 53.39 6.15  -0.01 -0.01         A1V                -0.025-0.038      +002       56
5343          BD+19 2779 124953100949              CN Boo   141122.0+192239141604.2+185443 14.49 68.90 5.98  +0.26 +0.05 +0.12   A8III              +0.043-0.028      +004SB?                       *
5344          BD-17 4053 124990158462                       141131.9-180717141703.8-183507329.69 39.80 6.22  +0.98               G5                 +0.001+0.005      -012V?
5345          BD+53 1699 125019 29059                       141146.6+530003141516.9+523209 97.00 60.16 6.58  +0.10 +0.09         A4V                -0.032-0.004      -015
5346          BD+20 2954 125040 83259        9192           141153.8+203518141632.8+200717 17.70 69.32 6.25  +0.49               F8V                -0.148-0.103D+.031-008V          2.0   4.4
5347          BD+40 2760 125111 64040                       141220.1+401230141624.2+394441 73.94 68.33 6.38  +0.36 -0.07         F2IV               -0.147+0.008      -024    =<  6
5348          CD-3210005 125150205412                       141228.6-324525141823.8-331314323.14 26.22 6.54  +0.27               F0V                -0.070-0.042      -029
5349          CP-60 5294 125158252703                       141231.2-604833141951.5-611623313.35-00.20 5.23  +0.29 +0.16 +0.11   Am                 -0.171-0.093 +.034+021       74                 *
5350 21Iot BooBD+52 1784 125161 29071 528    9198A Iot Boo  141237.4+514942141609.9+512202 95.23 60.96 4.75  +0.20 +0.06 +0.09   A9V                -0.150+0.092 +.048-019SB    137  3.5  38.7AB   3*
5351 19Lam BooBD+46 1949 125162 44965 527           6611    141234.9+463251141623.0+460518 86.97 64.67 4.18  +0.08 +0.05 +0.03   A0p                -0.187+0.161 +.045-008      110                 *
5352          BD+15 2690 125180100956    I          6613    141241.5+154334141728.4+151548  6.61 66.76 5.80  +1.71 +2.07 +0.87E  M3IIIa             +0.012+0.011      -010V?
5353          BD-06 3964 125184139856      W                141242.0-070424141800.6-073233337.18 49.49 6.47  +0.73 +0.35         G5IV               +0.254-0.240      -014V?         4.5  57.1      *
5354   Iot LupCD-45 9084 125238224833               6620    141259.9-453547141924.2-460328318.48 14.14 3.55  -0.18 -0.72 -0.14   B2.5IV             -0.013+0.002      +022      370                 *
5355          BD-18 3789 1252481584811369          CS Vir   141306.3-181511141838.3-184258330.06 39.52 5.90   0.00 -0.04         A0pCrEu           v-0.059-0.042      -009SBO     9                 *
5356          CD-2510271 125276182433        9212           141320.3-252150141900.8-254856326.53 33.01 5.87  +0.50 -0.09         F5V                -0.372+0.350 +.051-021V?         5.1  64.0AC   3*
5357          CD-36 9268 125283205430                       141320.9-363226141923.9-370013321.83 22.62 5.94  +0.08 +0.04         A2Vn               -0.047-0.059      +000
5358          CP-55 5984 125288241641 529                   141320.2-555532142019.5-562312315.05  4.38 4.33  +0.12 -0.43         B6Ib               -0.014-0.009      +004V?     10                 *
5359100Lam VirBD-12 4018 1253371584891371I:D        6621    141341.8-125439141906.6-132216333.40 44.25 4.52  +0.13 +0.12 +0.06   A2m                -0.015+0.030 +.017-011SB2O   16  0.1   0.0      *
5360          BD+51 1908 125349 29086                       141347.8+514611141721.1+511826 94.87 60.88 6.20  +0.04               A2IV               -0.025-0.005      -009V      85
5361          BD+36 2468 125351 640531370I                  141346.1+355815141759.8+353034 62.73 69.76 4.81  +1.06 +0.92 +0.53   K0III              +0.004+0.016      -026SB1O < 17                 *
5362          CD-42 9235 125383224838      W                141352.7-423556142009.7-430332319.69 16.91 5.56  +0.92 +0.60         G8III              -0.010+0.010      -019           2.6   3.5
5363          BD+48 2188 125406 44982                       141405.6+482753141749.2+480006 89.83 63.20 6.32  +0.44               F5                 -0.010-0.045      -017       30
5364          CD-44 9236 125442224843                       141419.5-444331142042.5-451114319.01 14.88 4.77  +0.31 +0.06         F0IV               +0.034-0.080 +.020+000SB     33
5365 18    BooBD+13 2782 1254511009751372  W                141425.8+132756141916.3+130015  2.76 65.08 5.41  +0.38 -0.03 +0.19   F5IV               +0.106-0.030      -003V?     39  4.9 156.4      *
5366102Ups VirBD-01 2938 1254541398663134I          6629    141423.2-014812141932.5-021556342.25 53.74 5.14  +1.02 +0.85 +0.51   G9III              -0.120-0.074 +.018-027V    < 19:
5367   Psi CenCD-37 9336 1254732054531373  W                141428.3-372531142033.4-375307321.72 21.71 4.05  -0.03 -0.11 -0.02   A0IV               -0.063-0.012 +.003-005V     101  9.5  36.0
5368          BD+01 2913 125489120400               6630    141434.6+005041141940.9+002303344.97 55.88 6.19  +0.20 +0.09         A7V                -0.041-0.013      -022V     157
5369          BD+39 2749 125538 64064                       141449.1+391337141855.7+384603 70.98 68.36 6.86  +1.06 +0.89         G9IV               -0.001+0.031      -010V?
5370 20    BooBD+16 2637 1255601009803135I          6631    141501.2+164554141945.2+161825  9.59 66.86 4.86  +1.23 +1.40 +0.60   K3III              -0.142+0.060      -008     < 17                 *
5371          CP-57 6619 125628241673      W                141527.1-580008142237.0-582734314.64  2.33 4.92  +0.86 +0.48         G8III+F5V          -0.052+0.013 +.024+015SB2        2.1   9.2AB   4*
5372          BD+55 1678 125632 29098                       141536.7+551925141855.8+545151 99.05 58.02 6.53R                     A5Vn               -0.028+0.011      -003      157
5373          BD+39 2750 125642 64072                       141541.3+391513141947.7+384738 70.85 68.19 6.33  +0.05 +0.05 -0.03   A2V                +0.022-0.013      -011V     142                 *
5374          BD+31 2605 125658 640743136           6633    141546.2+305317142008.7+302545 47.84 70.32 6.44  +0.15 +0.10 +0.05   A5III              -0.010-0.005      +001
5375          CD-47 9082 125721224870      W                141606.3-475146142238.7-481913318.20 11.83 6.09  -0.13 -0.91         B1III              -0.013-0.013      -018V?         3.7   4.2AB   3*
5376          CD-34 9570 1257452054853137                   141619.9-341947142219.7-344713323.33 24.46 5.56  -0.09               B8V                -0.025+0.001      -037
5377          CD-50 8501 1258102416863138  W                141639.8-501857142320.3-504620317.43  9.50 6.02  +1.36               K2III              -0.005-0.004      -007           3.6   1.3
5378          CD-38 9329 125823205497              V761 Cen 141652.4-390318142302.2-393044321.57 20.02 4.42  -0.18 -0.75 -0.17   B7IIIp            v-0.027-0.032      +008        2                 *
5379          CP-67 2574 125835252735 530                   141648.5-674425142506.3-681143311.48-06.89 5.61  +0.49 +0.01         A3Ib               -0.012-0.003      -022V      34
5380          CP-52 7195 125869241691                       141700.0-524313142348.4-531036316.64  7.22 6.00  +1.10               K1III              -0.071-0.024      +007V
5381 51    HyaCD-27 9803 125932182483    I          6648    141719.9-271742142305.8-274514326.60 30.87 4.77  +1.31 +1.54 +0.67   K4III              -0.193-0.112 +.040+020                          *
5382          CP-65 2718 125990252737                       141739.5-654307142539.5-661024312.26-05.02 6.36  +0.13 +0.06         A3V                -0.059-0.026      +018
5383  2    LibBD-11 3729 1260351585281374                   141802.6-111526142325.6-114251335.82 45.20 6.21  +0.99               G7III              -0.009-0.058      -001
5384          BD+01 2920 126053120424    I'                 141808.4+014236142315.3+011430347.17 56.02 6.27  +0.63 +0.09 +0.32   G1V                +0.226-0.477 +.061-019V       1
5385          BD+09 2882 126128              9247BC         141827.5+085400142322.6+082642356.56 61.33 6.86  +0.43 -0.01         F0V+F2V            -0.075-0.008 +.016-018SB     71  1.7   6.2AxBC 3*
5386          BD+09 2882 126129120426        9247A          141827.6+085406142322.7+082648356.56 61.33 5.12  -0.02 -0.07         A0V                -0.075-0.008 +.016-023V      93  1.7   6.2AxBC 3*
5387          BD+25 2770 126141 833213139           6653    141837.4+254728142306.8+252017 33.14 69.39 6.22  +0.37 -0.02         F5V                -0.155+0.064      -010        8
5388          BD+08 2857 126200120433               6654    141905.0+084154142400.9+081437356.48 61.08 5.95  +0.06 +0.08         A3V                +0.003-0.025      -007       90                 *
5389          CP-76  826 126209257163                       141902.4-761642142936.8-764345308.60-14.95 6.07  +1.18 +1.11         K0-1III            -0.039-0.026      -008V
5390          CD-2411469 1262181825171376                   141906.1-242109142448.6-244823328.45 33.40 5.32  +0.96 +0.71         G8III              -0.055-0.016 +.024-022V?
5391          CP-65 2732 1262412527453141                   141907.8-652210142707.1-654918312.52-04.75 5.85  +1.50 +1.76         K3III              -0.018-0.013      -020
5392          BD+06 2875 1262481204341375                   141912.7+061625142411.3+054912353.10 59.33 5.10  +0.12 +0.10         A5V                -0.077+0.005 +.023-010SB    194
5393          BD-11 3736 126251158550        9254           141918.2-111256142440.9-114011336.23 45.09 6.49  +0.42 +0.04         F4III              -0.071-0.029D+.012-036V          1.6   1.2      *
5394          BD+08 2858 126271120436               6655    141923.0+083228142418.3+080506356.35 60.92 6.19  +1.20 +1.29 +0.60   K4III              -0.114-0.092 +.015-031V?                        *
5395   Tau1LupCD-44 9322 1263412249191377  W       Tau1 Lup 141942.9-444609142608.2-451317319.92 14.50 4.56  -0.15 -0.79 -0.11   B2IV               -0.012-0.013      -022       30  4.7 158.2      *
5396   Tau2LupCD-44 9323 126354224920      W                141944.8-445538142610.8-452246319.87 14.35 4.35  +0.43 +0.19 +0.35   F4IV+A7:           +0.016-0.008 +.002-001SB1     0  0.1   0.1      *
5397          BD-19 3880 126367158558        9258A          141954.7-193102142529.8-195811331.22 37.68 6.61  +0.14 +0.11         A2V                -0.039-0.006D+.005-002       74  0.4  35.1AB   3*
5398          CD-41 8757 126386224923                       141956.5-415155142613.4-421909321.06 17.19 6.32  +1.19               K2III              -0.123-0.071      -033
5399          CD-2610280 126400182535                       142001.3-262355142547.7-265109327.67 31.45 6.48  +0.94               G7IV               -0.019-0.061      +007V
5400          CD-39 8918 126475205561                       142037.6-392521142649.9-395226322.14 19.41 6.35  -0.08               B7IV               -0.019-0.016      +008V
5401          CD-45 9188 126504224929      W                142045.2-454054142712.2-460804319.75 13.59 5.83  +0.31 +0.16 +0.16   A1mA5/7-F2         -0.153-0.082      -026           5.6  30.0      *
5402          BD+39 2764 126597 641373142                   142124.2+385041142529.2+382335 68.67 67.34 6.27  +1.23 +1.26        gK2                 +0.007-0.013      +025
5403          CP-58 5549 126610241737                       142124.8-584454142843.5-591152315.11  1.35 6.45  +0.14 +0.13         A0II-III           -0.011-0.015      +002V                         *
5404 23The BooBD+52 1804 126660 29137 531I W        6669    142147.5+521847142511.8+515103 93.84 59.65 4.05  +0.50 +0.01 +0.25   F7V                -0.236-0.397 +.072-011       34  7.0  69.5      *
5405 22    BooBD+19 2810 1266611010251378                   142148.2+194035142627.4+191337 18.06 66.82 5.39  +0.23 +0.23 +0.10   F0m                -0.071+0.025 +.007-028       29
5406104    VirBD-05 3880 126722139942                       142209.3-054009142724.4-060713341.39 49.45 6.17  +0.09 +0.11         A2IV               -0.071-0.058      -015V?
5407 52    HyaCD-2810712 126769182570 532    9270   6674    142218.9-290232142810.4-292930326.95 28.83 4.97  -0.07 -0.41         B8V                -0.024-0.023      +016V          0.0   0.1AB   4*
5408          CP-67 2595 126862252767      W                142255.3-671610143116.5-674302312.20-06.66 5.83  +1.00 +0.78         K1III              +0.035-0.066      +078           8.2  35.4
5409105Phi VirBD-01 2957 126868139951 533    9273   6675    142302.9-014647142812.1-021341345.22 52.51 4.81  +0.70 +0.21 +0.37   G2IV               -0.139-0.002 +.047-010SB      0  3.9   5.1AB   3*
5410106    VirBD-06 4009 1269271399573143I                  142325.1-062705142841.7-065402341.13 48.62 5.42  +1.49 +1.76         K5III              -0.019-0.052      -049
5411          BD+41 2504 126943 45058                       142329.9+412826142727.3+410130 74.26 65.88 6.63  +0.37 -0.05         F1IV               -0.056-0.019      -017       75
5412          CD-44 9383 126981224969      W                142341.4-445227143008.6-451918320.55 14.15 5.50  -0.08 -0.27 -0.04C  B8Vn               -0.046-0.040      +010V?    174  6.3  10.5      *
5413          CD-48 9098 126983224972      W                142341.1-490418143020.9-493109318.95 10.25 5.37  +0.05               A1V+B              -0.052-0.047      +000SB2O    0  6.5  22.1      *
5414          BD+28 2331 127043 83373        9277B          142407.1+284412142831.5+281721 41.69 68.52 7.62  -0.01  0.00         A1V                +0.028-0.013      -013SB     80  0.5  25.4      *
5415          BD+28 2332 127067 83374        9277A          142409.0+284416142833.3+281727 41.69 68.51 7.12  -0.03 -0.03         A1V                +0.017+0.008      -012      100  0.5  25.4      *
5416          BD+36 2495 127065 641613144                   142407.9+363839142816.4+361149 62.79 67.57 6.10R                     K0III              -0.024+0.002      -018
5417          CD-40 8729 127152224982                       142440.4-402400143056.5-405042322.51 18.21 6.39  +1.44               K3III              -0.030+0.003      -016
5418          BD+01 2941 1271671204993145                   142444.5+011628142950.5+004944348.95 54.66 5.94  +0.16 +0.11         A5IV               +0.010+0.002      -009      125
5419          CD-38 9430 1271932056373148                   142459.0-382533143110.8-385211323.38 20.01 5.97  +1.06               K1III              +0.005+0.023      +008
5420 24    BooBD+50 2084 127243 29165                       142509.0+501732142837.8+495041 90.16 60.64 5.59  +0.85 +0.44         G3IV               -0.305-0.046 +.022-006
5421          CP-56 6296 127297241777              V Cen    142522.8-562639143232.9-565316316.44  3.31 6.93  +0.91 +0.60         F5Ia               -0.027-0.012      -022                          *
5422          BD+32 2482 127304 64178        9288           142532.6+321410142949.7+314728 51.15 68.13 6.06  -0.03 -0.09         A0V s              -0.018+0.004      -009V   =< 41  5.0  25.8AB    *
5423          BD+42 2508 127334 450753146                   142540.3+421450142936.8+414745 75.46 65.16 6.35  +0.70 +0.21         G5V                +0.162-0.218 +.031-001V   =< 10
5424          BD+05 2886 1273371205043149I W                142545.0+051301143045.4+044620353.97 57.44 6.02  +1.42 +1.67        gK4                 +0.009-0.012      +006V          3.5  55.5      *
5425   Sig LupCD-49 8831 127381241781                       142552.6-500050143236.9-502725318.93  9.25 4.42  -0.19 -0.84         B2III              -0.043-0.010      -002V?     84                 *
5426          CP-54 6053 127486241792                       142630.8-543324143332.4-545954317.30  5.00 5.87  +0.48               F6IV-V             -0.095+0.004      -000
5427          CP-52 7301 127501241793      W                142636.6-521415143329.9-524048318.19  7.14 5.87  +1.09               K0III              -0.006-0.031      -042           8.1  20.
5428          CD-3011519 127624205681      W                142713.9-301621143309.6-304251327.47 27.27 6.09  +1.03 +0.84         K0III              +0.040-0.015D+.006+010V          3.3   2.5
5429 25Rho BooBD+31 2628 127665 64202 534I   9296   6697    142731.2+304837143149.8+302217 47.29 67.80 3.58  +1.30 +1.44 +0.65   K3-III             -0.100+0.119 +.029-014V?   < 17  8.2  42.2
5430  5    UMiBD+76  527 127700  80241379I   9286   6687    142743.8+760826142731.5+754146115.39 39.99 4.25  +1.44 +1.70 +0.75   K4-IIIBa0.3        +0.009+0.023 +.019+010V    < 17  7.8  58.8AC   3*
5431          CD-41 8890 127716225027                       142747.4-413931143408.0-420559322.56 16.83 6.60  +0.06               A2IV               -0.016-0.029      -056V?
5432          CP-59 5642 127724241811                       142749.0-593431143517.2-600057315.57  0.28 6.40  +1.26               K2III              -0.021-0.031      -007
5433          BD+27 2388 127726 83394        9301   6699    142754.6+270710143220.2+264038 37.58 67.54 6.01  +0.22 +0.07         A7Vn               -0.064-0.029 +.011-004      204  0.2   0.2      *
5434 26    BooBD+22 2715 127739 833953151                   142759.8+224201143232.5+221536 26.51 66.57 5.92  +0.35 +0.07         F2IV               -0.127+0.039      -012V                         *
5435 27Gam BooBD+38 2565 127762 64203 535I   9300  Gam Boo  142803.0+384444143204.7+381830 67.26 66.17 3.03  +0.19 +0.12 +0.08   A7III              -0.114+0.153 +.025-037V     139  9.5  33.4     3*
5436          BD+63 1136 127821 164063150                   142823.4+633740143046.1+631108105.61 50.47 6.09  +0.41 -0.04         F4IV               -0.178 0.000      -003       50
5437          BD+60 1547 127929 16411 536                   142859.8+603958143142.8+601332102.58 52.76 6.27  +0.24 +0.14         F0III              -0.049+0.021      -019V?     67
5438          BD-19 3903 127964182676                       142912.9-200002143450.7-202621333.37 36.25 6.50  +0.14               A4V                +0.017+0.002      -000
5439          CD-40 8794 127971225046      W                142911.8-410443143531.5-413102323.05 17.25 5.87  -0.08               B7V                -0.020-0.017      +010V          5.2  26.9      *
5440   Eta CenCD-41 8917 127972225044 537  I       Eta Cen  142909.3-414307143530.4-420928322.77 16.67 2.31  -0.19 -0.83 -0.20   B1.5Vne           v-0.035-0.035      -000SB    333                 *
5441          BD+37 2545 127986 64212              CP Boo   142915.0+372403143320.3+365734 63.89 66.37 6.43R                     F8IV w             -0.004-0.057      +002    =< 10                 *
5442          BD+56 1746 128000 29191    I                  142923.0+555019143230.9+552352 96.99 56.34 5.76  +1.50              gK5                 +0.006-0.015      +003
5443          CP-67 2616 1280202528243156                   142922.4-672916143746.3-675556312.69-07.10 6.04  +0.50 -0.02         F7V                -0.343-0.290 +.051-030
5444          CD-45 9293 128068225054      W        6724    142946.1-454831143619.0-461443321.20 12.87 5.55  +1.48 +1.72 +0.72C  K3III              -0.038+0.019 +.019-060           7.1  40.       *
5445          BD+33 2474 128093 642213153  W                142956.4+325823143411.7+323204 52.82 67.13 6.33  +0.40 -0.03         F5V                +0.108+0.007      -008       12  5.0  24.7
5446          CD-39 9047 128152205744                       143009.1-390935143624.2-393550324.04 18.93 6.13  +1.05               K1III              -0.037-0.018      -031V
5447 28Sig BooBD+30 2536 128167 834161380  W        6717    143019.5+301046143440.8+294442 45.62 67.20 4.46  +0.36 -0.08 +0.19   F2V                +0.189+0.132 +.068+000        3  5.3 237.1AB   3*
5448          BD+37 2551 128198 64227    I          6718    143033.2+370355143438.5+363733 62.91 66.22 6.03  +1.38              gK5                 -0.029-0.054      -012
5449          CD-39 9050 128207205751                       143027.3-394629143644.2-401242323.84 18.35 5.74  -0.12               B8n                -0.016-0.025      +002
5450          CD-45 9302 128266225062      W                143046.9-454151143720.1-460802321.41 12.90 5.41  +1.01 +0.73 +0.48C  G8III              -0.014-0.020 +.023-016           0.0   0.1AB   3*
5451          BD+57 1519 128332 292023154                   143114.6+573034143415.9+570355 98.69 54.95 6.48  +0.49 -0.03         F6-8V              +0.217-0.229 +.025-022    =<  6                 *
5452          BD+50 2095 128333 451213155I         CH Boo   143109.8+494815143439.6+492206 88.15 60.21 5.74  +1.56 +1.88 +0.72E  M1IIIab            -0.045+0.054      -020V?                        *
5453   Rho LupCD-48 9198 1283452250713158           6733    143109.4-485924143753.2-492533320.13  9.86 4.05  -0.15 -0.56 -0.11   B5V                -0.024-0.031      +008      195                 *
5454          BD+23 2710 128402 834273157                   143135.6+234108143606.9+231501 29.44 66.05 6.38  +1.06 +0.90         K0                 -0.012+0.027      +007
5455          BD-11 3770 1284291586771381                   143140.5-115249143659.8-121819339.35 42.97 6.20  +0.46 -0.03         F5V                -0.872+0.365 +.034-070V      15
5456          CD-38 9529 128488205786                       143204.9-382134143819.6-384739324.76 19.51 6.02  +1.43               K3III              +0.066-0.023      +028V
5457          CD-46 9469 128582225089                       143237.2-460844143911.0-463503321.52 12.36 6.07  +0.51 +0.03 +0.28C  F7V                -0.182-0.211 +.021-012
5458          CD-48 9218 128617225091                       143242.8-483709143924.7-490320320.52 10.10 6.39  +0.44 +0.01?        F3IV               -0.167-0.132 +.009-017                          *
5459   Alp1CenCP-60 5483 128620252838 538I W     A          143248.3-602522143935.9-605007315.78-00.71-0.01  +0.71 +0.24 +0.22E  G2V                -3.642+0.699 +.751-022SBO        1.4  20.9AB   3*
5460   Alp2CenCP-60 5483 128621          I'W     B          143248.3-602522143936.1-605008315.78-00.71 1.33  +0.88 +0.68 +0.30E  K1V                -3.646+0.700 +.751-021V?         1.4  20.9AB   3*
5461          CP-55 6107 1287132418983162                   143319.1-560033144032.8-562627317.63  3.29 6.30  +1.18               K0-1II             -0.007-0.011      -005
5462          BD+18 2906 1287501011213160                   143334.9+184359143814.0+181754 18.81 63.90 5.91  +1.10 +0.99 +0.56  gK2                 -0.029-0.068 +.038-014
5463   Alp CirCP-64 2977 128898252853 539  W       Alp Cir  143425.2-643223144230.4-645831314.34-04.59 3.19  +0.24 +0.12 +0.10   ApSrEuCr:          -0.192-0.232 +.056+007SB?     0  5.4  15.6      *
5464          BD+44 2376 128902 45145    I                  143427.1+440424143812.5+433831 77.43 62.89 5.70  +1.48 +1.68 +0.82   K2III              -0.113+0.030      -049V?
5465          CP-58 5672 128917241912                       143429.7-581108144155.7-583658316.91  1.23 6.22  +0.45               F4V                +0.019-0.042      +011
5466          CD-35 9702 128974205823                       143452.7-354218144101.4-360806326.50 21.67 5.67  -0.08 -0.38?        A0p:               -0.022-0.008      -023
5467          BD+54 1693 128998 292273161                   143504.8+542720143815.2+540124 94.17 56.73 5.85  -0.01  0.00         A1V                +0.018-0.016      -003V?     84
5468 33    BooBD+45 2204 129002 45153 540                   143506.9+445010143850.2+442416 78.76 62.42 5.39   0.00 -0.04         A1V                -0.067-0.017 +.012-013SB     79
5469   Alp LupCD-46 9501 129056225128 541  W       Alp Lup  143516.5-465732144155.8-472318321.61 11.44 2.30  -0.20 -0.89 -0.17   B1.5III/Vn         -0.021-0.018      +005SB     24 11.1  27.6      *
5470   Alp ApsCP-78  893 129078257193 542                   143525.6-783713144751.6-790241308.49-17.45 3.83  +1.43 +1.68 +0.53E  K3IIIa             -0.012-0.015 +.029-001
5471          CD-37 9618 129116205839                       143544.7-372151144157.6-374737325.90 20.10 4.00  -0.17 -0.70 -0.15   B3V                -0.023-0.032      +001V     187                 *
5472          BD+22 2731 129132 83458      S                143549.3+222415144021.9+215832 27.15 64.75 6.10  +0.40 +0.02         G0V                -0.018+0.034      +001SBO              0.1      *
5473          BD+14 2769 129153101137                       143555.4+135750144042.4+133203 10.34 61.10 5.91  +0.21 +0.04         F0V                +0.055-0.025      -008       94                 *
5474          CD-3011624 129161205841      W                143553.2-303015144151.1-305600329.23 26.24 6.37  -0.09               ApSi               -0.031-0.022D+.003-011           0.9   1.0
5475 29Pi 1BooBD+17 2768 129174101138        9338A  6762    143601.5+165049144043.6+162506 15.64 62.54 4.94  -0.03 -0.31 -0.02   B9pMnHgSi         v+0.015+0.015 +.002-001SB     22  0.9   5.7AB   3*
5476 29Pi 2BooBD+17 2768 129175101139        9338B  6763    143602.0+165048144043.9+162504 15.64 62.54 5.88  +0.24:+0.10:        A6V                +0.001+0.010 +.002-007SB    148  0.9   5.7AB   3*
5477 30Zet BooBD+14 2770 129246101145        9343B  6766    143622.3+140926144108.9+134342 10.80 61.12 4.83H                     A2III              +0.053-0.016 +.009-006           0.1   1.2AB   3*
5478 30Zet BooBD+14 2770 129247101145        9343A  6766    143622.3+140926144108.9+134342 10.80 61.12 4.43H +0.05 +0.05 -0.01   A2III              +0.053-0.016 +.009-006SB?   156  0.1   1.2AB   3*
5479          BD+80  448 129245  80543159                   143623.2+800532143338.3+793937117.41 36.35 6.26  +1.30 +1.46         K3III              -0.101+0.081 +.015-023V?                        *
5480 31    BooBD+08 2903 1293121206013163I          6769    143644.0+083522144138.8+080942  2.05 57.76 4.86  +1.00 +0.76 +0.48   G7+IIIHdel -0.5    -0.009+0.003 +.011-022V    < 17
5481 32    BooBD+12 2729 1293361011521382                   143655.2+120529144143.5+113938  7.46 59.86 5.56  +0.94 +0.67 +0.48   G8III              -0.157-0.114 +.023-023V                         *
5482          CP-62 4275 129422252869      W                143721.6-622656144517.3-625233315.50-02.81 5.36  +0.29               A9V                +0.063-0.084 +.011-001V?    213  5.5  36.3      *
5483          BD+21 2674 129430 83474                       143720.4+213308144154.2+210725 25.53 64.15 6.38R                     G8III-IV           -0.010-0.053      -011                          *
5484  4    LibCD-2411637 129433182795                       143726.6-243417144313.6-245951332.75 31.32 5.73  -0.01 -0.08         B9                v-0.019-0.001      -004V                         *
5485          CD-34 9868 129456205871 544                   143732.3-344435144339.4-351025327.48 22.29 4.05  +1.35 +1.53 +0.73   K3IIIb            e-0.064-0.180 +.014-038                          *
5486          CP-57 6772 129462241966                       143729.3-580306144455.5-582840317.32  1.19 6.11  +1.02               K0III              -0.083-0.053      -024
5487107Mu  VirBD-05 3936 129502140090 545                   143747.3-051325144303.6-053930346.55 47.54 3.88  +0.38 -0.02 +0.22   F2III              +0.109-0.316 +.045+005SB     54                 *
5488          CP-55 6150 129557241971      W       BU Cir   143758.2-551038144510.9-553607318.57  3.78 6.10  -0.06 -0.80         B2III              -0.015-0.013      -006       53  1.4  68.8      *
5489          CD-34 9888 129685205899                       143851.0-344606144459.2-351131327.73 22.15 4.92  +0.01 -0.03 +0.02   A0V                +0.003-0.004 +.019-005      455
5490 34    BooBD+27 2413 129712 834881383I         W Boo    143901.6+265710144325.4+263140 38.00 65.06 4.81  +1.66 +1.94 +1.22   M3-III             -0.013-0.017 +.002+006V                         *
5491          CP-87  235 129723258720 920          BP Oct   143858.8-874430152819.1-880759304.24-25.68 6.48  +0.30 +0.10         Am                 -0.091-0.077      -015       89                 *
5492          BD+61 1451 129798 16466        9357  DL Dra   143933.5+614117144203.2+611543102.09 51.16 6.25  +0.41 -0.01 +0.20   F2V                +0.070-0.030 +.017-006SB     42  2.2   4.1      *
5493          BD+41 2523 129846 451903166I                  143951.6+405256144344.4+402733 70.09 63.30 5.73  +1.39               K4III              -0.014+0.023      +013SB
5494          CD-46 9562 129858225178                       143947.6-470109144629.1-472628322.30 11.06 5.74  +0.07 +0.08 +0.03C  A1V                -0.022-0.015      -011
5495          CD-51 8457 129893241992 546  W                144001.4-515737144701.3-522301320.20  6.57 5.21  +0.98               G8III              -0.016-0.082 +.021-021           7.8   8.9
5496          BD-00 2867 129902140116    I                  144002.8-005942144511.7-012504351.32 50.45 6.07  +1.61 +1.94 +0.85E  M1III              -0.053-0.004      -047                          *
5497 54    HyaCD-2411661 129926182855        9375A          144012.6-250107144600.1-252635333.13 30.63 4.94  +0.35 +0.10         F2III-IV           -0.150-0.104 +.049-013      161  2.0   8.5      *
5498          CD-51 8461 129932241995      W                144013.1-514704144712.3-521220320.31  6.72 6.07  +0.09               A1III-IV           -0.025-0.017      -034           5.3  39.1
5499          BD-22 3844 1299441828573167                   144022.2-224348144606.8-230911334.50 32.60 5.81  +0.98              gG5                 +0.027-0.058      +007V
5500          CP-66 2645 129954252894      W                144016.2-661025144844.4-663538314.22-06.33 5.91  -0.07 -0.70         B2.5Ve             -0.021-0.025      +019           3.0  60. AB   3*
5501108    VirBD+01 2972 129956120642                       144024.5+010822144530.2+004302353.72 51.97 5.69  -0.03 -0.07         B9.5V              -0.038-0.007      -020V?     76
5502 35Omi BooBD+17 2780 129972101184    I                  144034.4+172316144514.5+165752 17.74 61.82 4.60  +0.98 +0.75 +0.48   G8.5III            -0.058-0.049 +.045-009     < 17                 *
5503  5    LibBD-14 4023 129978158788        9376           144026.8-150217144557.8-152735339.48 39.15 6.33  +1.19               K1III              -0.032 0.000      -040           4.7   3.1
5504          BD-20 4087 129980182858      W                144030.4-204507144610.9-211034335.73 34.29 6.40  +0.58 +0.15         F7V                -0.062-0.104D+.026-000V?         0.2   0.3      *
5505 36Eps BooBD+27 2417 129988              9372B          144037.1+272947144459.2+270430 39.38 64.78 5.12H                     A2V                -0.050+0.021 +.016-023SB2   147  2.6   2.8AB   3*
5506 36Eps BooBD+27 2417 129989 83500    I   9372A          144037.1+272945144459.2+270427 39.38 64.78 2.70H +0.97 +0.73 +0.52   K0-II-III          -0.049+0.021 +.016-017V          2.6   2.8AB   3*
5507          BD+19 2854 130025101187                       144043.3+191822144520.7+185305 21.51 62.60 6.13  +0.83               K0                 +0.032+0.014      -004
5508          CD-37 9686 130055205937                       144048.6-375205144705.1-381727326.62 19.20 5.94  +1.33               K3III              +0.052-0.090      +041V                         *
5509          CD-43 9326 130073225183                       144101.9-430812144732.1-433327324.22 14.46 6.30  +1.08 +0.88 +0.49C  G8III              -0.013-0.032      -023
5510          BD+33 2489 130084 643061384I                  144102.6+331243144513.7+324718 52.83 64.79 6.28  +1.58 +1.90         M1IIIb             +0.045-0.072      +030V
5511109    VirBD+02 2862 130109120648 547           6794    144111.5+021851144614.9+015334355.28 52.68 3.72  -0.01 -0.03 -0.02   A0V                -0.114-0.026 +.037-006V?    351
5512          BD+15 2758 1301441012003168I          6796    144123.1+153307144606.0+150755 14.55 60.79 5.63  +1.57 +1.31         M5IIIab            -0.084+0.020      -022
5513          BD-20 4093 130157182873    I                  144132.4-205419144713.7-211929335.89 34.03 6.06  +1.65 +1.98         K5III              -0.013+0.004      -024V
5514 55    HyaCD-2510534 130158182875                       144133.4-251216144722.5-253728333.34 30.32 5.63                      A0IIIp             -0.008-0.016      -018        0                 *
5515          CP-56 6441 130227242026               6807    144147.6-561447144906.9-564004318.61  2.58 6.23  +1.13 +1.06         K1III              -0.110-0.123      +013                          *
5516 56    HyaCD-2510537 1302591828821385                   144154.4-254006144744.8-260515333.15 29.87 5.24  +0.94 +0.65         G5III              +0.041-0.007 +.034-001
5517 57    HyaCD-2610519 130274182883                       144206.4-261338144757.5-263847332.88 29.36 5.77  -0.02 -0.09         B9V                -0.013-0.011      +006      183
5518          BD-12 4134 130325158808                       144227.4-122509144754.9-125023341.92 41.05 6.35  +1.10 +0.95         K0III              +0.043-0.085      +033V?                        *
5519          CD-36 9645 130328205966              V768 Cen 144225.6-361256144838.1-363805327.72 20.53 6.04  +1.54 +1.28 +1.61   M3III              -0.014-0.048      +041                          *
5520          CP-72 1604 130458257206      W                144313.2-724639145313.7-731124311.56-12.39 5.60  +0.82 +0.42         G7IIIa+F9IV        +0.021+0.033      +038           2.2   2.0      *
5521          CD-2311916 130529182898    I W                144331.9-235006144918.7-241506334.59 31.28 5.68  +1.30              gK1                 -0.023-0.003      -026           2.9  61.0AB   3
5522          BD-00 2886 1305571401523169                   144345.8-002554144854.1-005052353.00 50.25 6.14  -0.04 -0.10         B9VSi:Cr:         v-0.008+0.017      -016       71
5523  7Mu  LibBD-13 3986 130559158821        9396   6816    144350.0-134357144919.1-140856341.30 39.78 5.31  +0.07 -0.05         A1pSrCrEu          -0.061-0.014 +.006-004SB?    26  0.9   1.8AB   5*
5524          BD+24 2779 130603 83535        9389           144357.2+244655144823.3+242200 33.54 63.58 6.14  +0.48 +0.08         F2-6V              -0.110+0.045D+.010-031       38  1.1   1.9
5525   Pi 1OctCP-82  629 1306502587133987                   144413.7-824924150150.8-831340306.84-21.37 5.65  +0.95 +0.75         G8-K0III           +0.016+0.053      +014
5526 58    HyaCD-2710073 130694182911    I                  144424.8-273238145017.3-275737332.65 27.97 4.41  +1.40 +1.49 +0.76   K4III              -0.241-0.059 +.011-010                          *
5527          CP-63 3436 130701252928      I       AX Cir   144427.3-632348145235.0-634836315.83-04.01 5.87  +0.72 +0.22         G3II+B8V          v-0.024-0.009      -021SB?                       *
5528   Omi LupCD-43 9391 130807225248      W                144506.5-430941145138.4-433432324.90 14.11 4.32  -0.15 -0.61 -0.11   B5IV               -0.020-0.027      +007SB     84  0.0   0.1      *
5529          BD+38 2593 130817 643441386                   144511.1+381324144906.7+374840 63.78 63.12 6.16  +0.36 -0.07         F2V                -0.253+0.111 +.021-035V?     12
5530  8Alp1LibBD-15 3965 1308191588361387  W     B          144509.2-153453145041.2-155950340.31 38.07 5.15  +0.41 -0.03 +0.19   F4IV               -0.098-0.066 +.050-023V      22  0.4   0.0 O   3*
5531  9Alp2LibBD-15 3966 130841158840 548I W     A  6827    144520.7-153734145052.7-160230340.33 38.01 2.75  +0.15 +0.09 +0.04   A3IV               -0.106-0.067 +.058-010SB     84  0.4   0.0 O   3*
5532          BD+29 2581 130917 835513171  W                144540.0+290148144958.4+283657 43.16 63.83 5.80  +0.05 +0.08         A4V                +0.017+0.003      +001SB    175  4.9 111.2
5533 38    BooBD+46 1993 130945 45226                       144544.8+463158144918.7+460658 79.98 59.97 5.74  +0.48  0.00         F7IV w             -0.002-0.076      -005V?     20                 *
5534          BD+24 2786 130948 835533172                   144547.4+241929145015.8+235443 32.76 63.06 5.85  +0.56 +0.02         G0-2V              +0.143+0.034 +.069-002SB
5535 11    LibBD-01 2991 130952140176    I                  144549.8-015257145101.0-021757352.08 48.82 4.94  +0.98 +0.72 +0.51   G8III-IV           +0.084-0.123 +.016+083V    < 17
5536          BD+00 3253 130970140177    I                  144552.8+000921145100.1-001526354.23 50.31 6.18  +1.40 +1.69         K3III              -0.026+0.015      -020
5537          BD+51 1957 131040 29296        9405           144617.5+514718144932.3+512229 88.43 57.10 6.51  +0.40  0.00         F5IV               +0.012+0.008      -005SB         3.6  15.7AB   3*
5538 39    BooBD+49 2326 131041 45231        9406           144617.3+490754144941.3+484314 84.29 58.58 5.69  +0.47               F6V+F5V            -0.076+0.096 +.009-032SB2O       0.5   2.9      *
5539   Zet CirCP-65 2918 131058252951                       144614.1-653452145442.4-655929315.03-06.06 6.09  -0.06 -0.60         B3Vn               -0.022-0.022      -018
5540          CP-76  924 1311092572123175          R Aps    144628.8-761519145752.9-763946310.14-15.60 5.34  +1.45 +1.70         K4III:             -0.074-0.015 +.008-031                          *
5541          BD+37 2580 131111 64355    I                  144632.7+374056145029.6+371619 62.48 62.99 5.48  +1.02 +0.84 +0.54   K0III-IV           -0.211+0.094 +.026-067V    < 17                 *
5542          CD-3011780 131117206035                       144635.5-300954145233.1-303437331.67 25.45 6.29  +0.60 +0.13         G1V                -0.329-0.027 +.031-028V?                        *
5543          CD-37 9760 1311202060373173                   144634.4-372330145251.1-374812327.93 19.11 5.03  -0.16 -0.72         B7IIIp             -0.018-0.021      +006       95                 *
5544 37Xi  BooBD+19 2870 131156101250    I   9413  Xi  Boo  144646.4+193057145123.3+190604 23.09 61.36 4.55  +0.76 +0.28 +0.43   G8Ve+K4Ve          +0.137-0.098 +.156+003V?      3  2.1   7.2AB   4*
5545   Pi 2OctCP-82  636 131246258714                       144720.9-823814150446.7-830218307.03-21.25 5.65  +1.30 +1.15         G8Ib               -0.016-0.010      -021
5546          CP-59 5753 1313422421113174                   144751.7-594212145534.6-600651317.82-00.88 5.20  +1.16 +1.15         K2III              -0.125-0.115 +.031-014
5547          CP-76  931 131425257218                       144826.6-764525150011.8-770937310.01-16.09 5.93  +1.05 +0.81         G8II               -0.003+0.004      +002
5548 12    LibCD-2411735 131430182983    I                  144831.5-241400145420.1-243832335.49 30.35 5.30  +1.32 +1.54         K2III              -0.010-0.027 +.041+009V?
5549          CD-3210457 131432206079      W                144831.3-325333145438.0-331802330.61 22.87 5.82  +1.42               K2III               0.000+0.002      -024           4.0  58.1AC   3*
5550          BD+16 2705 131473101273        9425           144841.9+160647145323.3+154216 17.22 59.51 6.40  +0.57 +0.12         F9V                -0.022+0.009D+.021+021V?         0.5   1.5      *
5551   The CirCP-62 4337 131492252965      W       The Cir  144840.5-622229145644.0-624651316.71-03.31 5.11   0.00 -0.77 +0.08   B4Vnpe            v-0.011 0.000      +003V     199  0.0   0.1      *
5552          BD+59 1615 131507 29315 549I                  144854.0+594201145126.4+591738 98.56 51.78 5.46  +1.36 +1.60 +0.70   K4III              -0.122+0.139      +011V?
5553          BD+19 2881 131511101276               6847    144851.4+193319145323.7+190910 23.55 60.93 6.01  +0.83 +0.50 +0.45   K2V                -0.452+0.216 +.087-029SBO                       *
5554 13Xi 1LibBD-11 3827 131530158887                       144857.0-112925145422.9-115354344.34 40.88 5.80  +0.97 +0.72 +0.34E  G7III              -0.052-0.010      -024V?                        *
5555          CP-74 1281 131551257219      W                144908.5-743744145956.0-750157311.08-14.23 6.20  -0.04 -0.20         B9V                -0.006-0.014      -003           6.7  30.
5556          CP-52 7634 1315622421203176                   144909.9-522413145617.3-524835321.28  5.56 5.38  +0.13               A2III              +0.026+0.005      +007SB    120
5557   Ome OctCP-84  490 131596258717                       144921.9-842339151108.3-844715306.17-22.82 5.91  -0.06 -0.13         B9.5V              -0.008+0.009      -007V
5558          CD-3310169 1316252060991389  W                144936.3-332659145544.7-335121330.53 22.27 5.32  +0.04               A0V                +0.020 0.000 +.007+000SB2   202  7.2  24.3
5559          CD-47 9543 131657225306      W                144944.3-472824145632.0-475245323.63  9.90 5.64  -0.04 -0.27 -0.02C  B9V                -0.040-0.020      +015SB         0.8   2.2      *
5560          CD-50 8939 131705242128                       145000.5-510231145701.4-512649322.01  6.71 6.64  +1.70 +2.04 +1.03E  M2III              -0.076-0.009      +024
5561          CD-38 9785 131752206110                       145013.8-390038145635.8-392458327.79 17.34 6.36  +0.06               A0-1V              -0.026-0.024      -007
5562          CD-3210480 131774206112    I                  145025.2-321352145630.9-323812331.34 23.25 6.06  +1.41               K3III              -0.008-0.028      +012V
5563  7Bet UMiBD+74  595 131873  8102 550I W        6846    145059.5+743351145042.3+740920112.65 40.50 2.08  +1.47 +1.78 +0.76   K4-III             -0.031+0.012 +.039+017V    < 17  9.2 209.1      *
5564 15Xi 2LibBD-10 3989 1319181589151390I                  145120.4-110022145646.1-112435345.35 40.92 5.46  +1.49 +1.70         K4III              +0.011+0.008      +015V
5565          CD-2811055 131919183030    I:                 145115.5-284512145713.6-290928333.44 26.16 6.29  -0.04 -0.21         B9V                -0.027-0.031      +008
5566          CD-48 9494 1319232253283178                   145116.6-482704145808.8-485147323.40  8.91 6.35  +0.71 +0.28         G3-5V              -0.007-0.333 +.051+037                          *
5567          BD+15 2796 131951101293 551                   145129.9+145102145613.2+142647 15.69 58.32 5.77  -0.06 -0.08         A0V                -0.013 0.000      -022V?    150
5568          BD-20 4125 1319771830401391I   9446A          145137.4-205749145728.0-212456338.24 32.70 5.74  +1.11 +1.06 +0.56   K4V                +1.045-1.729 +.173+020V          2.3  22.8AB   6*
5569          BD+32 2531 132029 64408        9442           145150.2+324215145558.7+321801 51.36 62.57 6.12  +0.10 +0.07         A2V                -0.043+0.004      -012V          5.4   4.7      *
5570 16    LibBD-03 3696 1320521402403177                   145157.6-035621145711.0-042047351.73 46.27 4.49  +0.32 +0.05 +0.17   F0V                -0.094-0.150 +.047+022      117
5571   Bet LupCD-42 9853 132058225335 552                   145158.8-424352145831.9-430802326.25 13.91 2.68  -0.22 -0.87 -0.17   B2III/IV           -0.035-0.039      +000SB    127                 *
5572          CD-39 9402 132096206154                       145212.9-393012145836.8-395424327.89 16.72 6.15  +1.23 +1.24         K2III              -0.047-0.063      +024
5573          BD+00 3277 1321321207581393I W                145225.5+001407145733.3-001003356.13 49.22 5.53  +1.13 +1.11 +0.55   K1III              +0.062-0.026 +.034+020V?         2.9  86.3      *
5574          BD+22 2764 132145 835961392                   145233.2+215731145703.6+213319 28.80 60.93 6.49  -0.01 +0.01         A1V                -0.023-0.028      -011    =< 41
5575          BD+16 2715 132146101299                       145231.8+164727145711.7+162318 19.22 59.00 5.71  +0.94              gG5                 +0.005+0.001      -016
5576   Kap CenCD-41 9342 132200225344 553  W        6873    145239.2-414210145909.7-420615326.87 14.75 3.13  -0.20 -0.79 -0.16   B2IV               -0.019-0.024      +008SB     28  8.1   3.9      *
5577 59    HyaCD-2710148 132219183058        9453           145244.0-271521145839.2-273926334.61 27.28 5.65  +0.24 +0.14 +0.14   A4V+A6V            -0.042-0.010D+.008-010           0.3   0.4      *
5578 17    LibBD-10 3994 132230158935                       145248.1-104511145813.4-110918345.93 40.90 6.60  +0.01               A1V                -0.011-0.014      -017SB     71
5579          CD-37 9836 1322382061673180                   145254.9-372849145913.9-375253329.05 18.43 6.47  -0.08               B8V                -0.017-0.020      +015V?                        *
5580          CD-42 9871 132242225351                       145253.4-424533145927.1-430936326.39 13.81 6.10  +0.60 +0.30 +0.32C  F5III              -0.018-0.013      -010                          *
5581          BD+50 2126 132254 452883179                   145303.8+500215145623.0+493743 84.65 57.17 5.63  +0.50  0.00         F7V                +0.115-0.223 +.027-015V?      6
5582 18    LibBD-10 3999 132345158946        9456           145328.9-104431145853.6-110839346.11 40.80 5.87  +1.26 +1.44 +0.60   K3-IIICN1          -0.101-0.061      -012V          3.8  19.7AB   3*
5583          BD-04 3783 132375140256        9457           145340.0-043511145852.8-045921351.56 45.51 6.09  +0.50 +0.05         F8V                -0.357-0.097 +.026-029        8  7.2   9.8      *
5584          BD+05 2954 132525120774    I                  145423.5+045801145923.1+043404  2.14 52.07 5.93  +1.60 +1.90         M2III              +0.005-0.008      -012
5585          CD-37 9863 132604206208                       145452.6-373937150113.0-380330329.31 18.08 5.89  +1.24               K2-3III            +0.013-0.034      +041
5586 19Del LibBD-07 3938 1327421402701394  W       Del Lib  145537.7-080720150058.4-083108348.87 42.51 4.92   0.00 -0.10 +0.01   B9.5V              -0.063-0.005 +.020-039SBO    77  6.4 106.3AC   4*
5587          CD-3310244 132763206223                       145547.1-335745150158.1-342132331.45 21.19 6.22  +0.24 +0.13 +0.13   A8IV               -0.025-0.020      -003
5588 40    BooBD+39 2820 132772 644493182                   145546.8+393942145936.9+391555 65.66 60.77 5.64  +0.31 +0.03         F0-2IV-III         -0.032+0.042      +012       67
5589          BD+66  878 132813 16558 554I         RR UMi   145559.4+661951145735.0+655557104.87 46.53 4.60  +1.59 +1.59 +1.71   M4.5III            -0.079+0.032 +.010+007SBO                       *
5590          BD-02 3928 1328331402763183I          6889    145608.0-022130150119.8-024518354.40 46.72 5.52  +1.68 +2.02         M0III              +0.027-0.023      -015V
5591 60    HyaCD-2710183 1328511830993184                   145608.6-273952150206.4-280338335.10 26.53 5.85  +0.16 +0.16 +0.08   A4IV               +0.085-0.029      +008V
5592          BD+22 2772 132879 83616                       145623.3+222629150052.4+220244 30.28 60.22 6.38R                     K0                 +0.010+0.006      -026
5593   Eta CirCP-63 3493 132905253005                       145626.3-633820150448.2-640154316.91-04.84 5.17  +0.93               G8III              +0.099+0.004 +.032+045
5594          BD+00 3297 132933120798    I   9480   6890    145641.6+001518150148.9-000826357.28 48.48 5.71  +1.52 +1.56         M0.5IIb            +0.009-0.024D+.002-034           1.1   0.5AB   3*
5595          CD-3210560 1329552062393186  W                145651.9-321456150259.3-323836332.60 22.54 5.44  -0.12 -0.61         B3V                -0.015-0.021      +006       19  8.3  36.1      *
5596          BD+83  431 133002  24591644                   145702.9+825522145020.4+823043118.49 33.56 5.64  +0.68 +0.17         F9V                +0.176-0.222 +.016-043V
5597          BD+47 2192 133029 45326        9477  BX Boo   145714.0+474021150038.7+471640 80.19 57.71 6.37  -0.14 -0.27         B9pSiSrCr          -0.009+0.024      -014V    < 20  3.2  35.6      *
5598          CP-71 1729 133049257238                       145712.1-713053150708.7-715419313.13-11.77 6.52  +1.59 +1.86         K4III              +0.023-0.015      +009
5599          BD-02 3933 133112140286                       145733.1-023814150244.9-030153354.49 46.28 6.61  +0.20 +0.19 +0.13   A5m                -0.039-0.020      -022       82                 *
5600 41Ome BooBD+25 2861 133124 836243185I                  145743.6+252412150206.5+250029 36.25 60.66 4.81  +1.50 +1.83 +0.79   K4-IIIabCa0.5      -0.007-0.051 +.025+014V?   < 17
5601110    VirBD+02 2905 1331651208093190I                  145750.8+022902150254.0+020529  0.07 49.79 4.40  +1.04 +0.88 +0.54   K0.5IIIbFe-0.5     -0.054+0.016 +.025-016     < 17
5602 42Bet BooBD+40 2840 133208 45337 555I          6898    145810.7+404706150156.8+402326 67.62 60.04 3.50  +0.97 +0.72 +0.44   G8IIIaBa0.3Fe-0.5  -0.041-0.028 +.037-020V?   < 17                 *
5603 20Sig LibCD-2411834 133216183139 556I         Sig Lib  145812.9-245320150404.2-251655337.22 28.62 3.29  +1.70 +1.94 +1.29   M3-III            e-0.073-0.043 +.064-004                          *
5604          CD-40 9243 133220225422              GM Lup   145813.9-402809150442.9-405141328.45 15.32 6.41  +1.46               M6III              -0.015-0.039      -030                          *
5605   Pi  LupCD-46 9773 133242225426      W                145818.4-463935150507.2-470304325.33  9.93 4.72H -0.14 -0.59 -0.11   B5V                -0.020-0.019D+.009+005SB    161  0.1   1.8      *
5606   Pi  LupCD-46 9773 133243225426      W                145818.4-463935150507.2-470304325.33  9.93 4.82H                     B5IV               -0.019-0.019D+.009+005SB         0.1   1.8      *
5607          CD-40 9257 133340225435      W                145849.0-404037150519.1-410402328.44 15.08 5.15  +1.01               G8III              +0.020-0.003 +.019-003           3.7  30. AC   3
5608          BD+60 1582 133388 165743188           6899    145906.5+603550150127.1+601216 98.26 50.23 5.93  +0.09 +0.10         A4V                -0.017+0.015      -009
5609          BD+35 2642 133392 644763191           6908    145906.5+353550150306.1+351221 57.21 60.85 5.51  +1.02       +0.49  gG8                 -0.038+0.012      -027
5610          BD+06 2983 133408120822        9493A          145908.5+055304150406.4+052933  4.51 51.74 6.50  +0.31 +0.02         F0V                -0.010-0.041D+.017-008           0.2   9.9
5611          CP-64 3095 133456253024               6932    145922.6-645316150756.8-651632316.58-06.09 6.17  +1.47 +1.70         K3III              +0.007-0.009      -004
5612          BD+45 2251 133484 453483192  S                145934.5+450207150306.6+443840 75.33 58.43 6.65  +0.46 +0.04         F6IV               -0.093+0.024      -020       20        0.2
5613          BD+35 2644 133485 64484                       145935.2+345727150336.5+343358 55.89 60.82 6.59  +1.02               G8III-IV           -0.002-0.016      -025
5614          CD-2510710 133529183163                       145954.5-252401150547.7-254723337.27 27.98 6.67  -0.01 -0.38         B8V                -0.002-0.020      -032
5615          CD-3510035 133550206292                       145956.9-355233150613.9-361551331.19 19.10 6.27  +1.65               K5III              -0.017+0.008      -037
5616 43Psi BooBD+27 2447 133582 83645 557I                  150009.6+272015150426.7+265651 40.32 60.47 4.54  +1.24 +1.33 +0.65   K2III              -0.174-0.006 +.018-026     < 17
5617          CD-48 9630 133631225456                       150028.2-484207150725.9-490518324.63  7.97 5.77  +0.92               G8III              +0.014+0.017      -012V
5618 44    BooBD+48 2259 133640 45357        9494  i Boo    150029.4+480236150347.4+473916 80.36 57.06 4.76  +0.65 +0.11         F9-G1Vn            -0.407+0.033 +.086-025SB2O=< 16  0.8   0.7      *
5619          CD-3011960 133652206300                       150029.0-303148150633.3-305508334.30 23.60 5.96  -0.08               A0pSi             v-0.016-0.038      +002       69                 *
5620          BD-21 4030 133670183176               6931    150040.8-213834150627.1-220155339.86 30.99 6.17  +1.05              gK1                 +0.063-0.058      +005                          *
5621          CP-66 2725 1336832530313195                   150036.2-664157150929.9-670503315.80-07.73 5.76  +0.69 +0.39         F6II               -0.005-0.001      +009V
5622 21Nu  LibBD-15 4026 1337741590283193I                  150102.8-155209150637.6-161525344.00 35.62 5.20  +1.58 +1.92 +0.88   K5-III             -0.033-0.022 +.020-015
5623          CP-63 3518 133792253034                       150104.8-631528150925.5-633834317.55-04.76 6.28  +0.06               A0pSrCrEu          -0.015-0.018      +012
5624          CD-40 9305 1338802254743194          HR Lup   150142.8-401152150812.1-403502329.18 15.22 5.79  -0.14 -0.40         B8IVSi             -0.030-0.032      +003V                         *
5625          CD-4210050 133937225479                       150202.8-422858150839.1-425204328.04 13.22 5.85  -0.11 -0.45 -0.13   B7V                -0.027-0.018      +001SB    317                 *
5626   Lam LupCD-44 9889 133955225483      W                150206.3-445342150850.6-451647326.80 11.13 4.05  -0.18 -0.68 -0.15   B3V                -0.016-0.019D+.012+010SB    166  0.2   0.4      *
5627 47    BooBD+48 2262 133962 453701395    9500   6934    150207.1+483214150525.8+480904 80.95 56.60 5.57   0.00 -0.07         A1V                -0.066+0.031 +.026-013       59  7.7   6.2      *
5628          CP-72 1714 133981257247                       150216.1-722321151233.8-724613313.03-12.73 6.01   0.00 -0.24         B8-9III            -0.024-0.007      -013
5629          BD+66  887 133994 16587                       150226.3+661829150357.8+655511104.18 46.09 6.13R                     A2V s              +0.023-0.007      -005
5630          BD+37 2608 134044 64503                       150240.0+365026150635.1+362720 59.53 59.98 6.35  +0.52 +0.03         F8V                -0.063+0.028      -005V?  =<  6
5631          BD+06 3001 1340471208523196                   150242.6+055300150740.3+052953  5.39 51.04 6.16  +0.94 +0.71         K0III              -0.003-0.017      +003SB
5632          CP-60 5656 134060253043                       150242.8-610224151044.6-612521318.81-02.92 6.30  +0.63 +0.16         G2V                -0.198-0.021      +038
5633          BD+19 2924 134064101379        9505           150245.2+184941150720.4+182630 24.65 57.61 6.02  +0.06 +0.06         A3V                +0.043-0.055D+.015-006SB    165  0.1   0.1      *
5634 45    BooBD+25 2873 134083 836711396  W        6945    150254.5+251531150718.1+245209 36.45 59.48 4.93  +0.43 -0.02 +0.21   F5V                +0.185-0.165 +.064-007V      45  4.9 244.1AC   3*
5635          BD+55 1730 134190 294071397I                  150325.3+545628150616.7+543323 90.43 53.21 5.25  +0.96 +0.64 +0.49   G7.5IIIFe-1        +0.048+0.015 +.029+016V?   < 19:
5636          CD-3810020 134255206352                       150342.4-382437151007.5-384733330.48 16.55 5.98                      G8III              -0.006-0.023      -035
5637          CP-54 6367 1342702422873198  W                150348.5-545755151116.0-552046321.96  2.27 5.54  +1.14 +0.73         G2Ib-II            -0.002-0.008      -004           6.7  11.2      *
5638 46    BooBD+26 2656 134320 83682    I                  150404.7+264104150823.8+261804 39.31 59.50 5.67  +1.24 +1.26 +0.61  gK2                 +0.005-0.014      +021SB
5639          BD+13 2901 134323101403                       150409.6+133656150853.4+131406 16.44 55.04 6.10  +0.96 +0.62        dG6                 -0.056+0.064      -049V?
5640          BD+25 2876 134335 83685      W                150414.1+252928150835.5+250631 37.03 59.24 5.81  +1.24               K1III              -0.008-0.001      -016V?         4.1  57.5
5641          CD-2510758 134373183237                       150423.6-255705151018.6-261958337.87 26.96 5.76  +1.05               K0III              -0.032-0.010      -033V?
5642          CD-44 9922 134444225517      W     AB         150449.4-445354151134.9-451639327.22 10.88 6.44  +1.04 +0.85         K1III              -0.009+0.014      -004           0.9  32.5AC   3*
5643          CD-44 9921 134443225516      W     C          150446.1-445354151131.9-451646327.21 10.89 7.39  +1.08 +0.95         K0III              +0.025-0.055      +021V          0.9  32.5AC   3*
5644          CP-69 2267 134453253062    I         X TrA    150443.1-694208151419.1-700446314.60-10.52 5.81  +3.38 +6.69         C5,5               -0.001-0.006      -004                          *
5645          CP-61 4856 134468253059                       150452.6-612156151301.0-614438318.87-03.34 6.32  +1.90               K4Ib               -0.010-0.008      -031V
5646   Kap1LupCD-48 9704 1344812255251398  W                150458.8-482127151156.1-484416325.46  7.89 3.87  -0.05 -0.13 -0.01   B9.5Vne            -0.095-0.049      -006V     202  1.9  26.6      *
5647   Kap2LupCD-48 9705 134482225526      W                150500.4-482149151157.6-484437325.46  7.88 5.69  +0.14 +0.09         A3IV               -0.103-0.040      +000      186  1.9  26.6      *
5648          BD+50 2146 134493 294203197                   150509.1+502615150819.5+500318 83.57 55.29 6.39  +1.03               K0III              -0.004-0.018      -029
5649   Zet LupCD-51 8830 134505242304 558I W                150505.8-514307151217.1-520557323.76  4.98 3.41  +0.92 +0.66         G8III              -0.112-0.073 +.043-010           3.6  71.9      *
5650          CD-47 9779 134597225533                       150534.9-475024151231.3-481307325.81  8.29 6.33  +1.12               K2III              -0.014-0.028      -043
5651          CD-44 9932 134687225539              Var?     150606.4-440722151249.5-443002327.83 11.43 4.82  -0.17 -0.68 -0.16   B3IV               -0.034-0.028      +014SB1O   16                 *
5652 24Iot1LibBD-19 4047 134759159090 559    9532   6981    150631.1-192448151213.3-194730342.69 32.00 4.54  -0.08 -0.35 -0.09   A0pSi             v-0.036-0.039 +.030-012SB2O   54  0.6   0.1AP   4*
5653          CD-3510119 134837206418                       150649.1-354252151307.4-360529332.53 18.51 6.10  -0.08               B8V                -0.043-0.021      -003
5654          BD+19 2935 1349431014293199I W       FL Ser   150731.2+192108151204.3+185833 26.32 56.76 5.89  +1.53               M4IIIab            -0.017+0.005 +.006-035           1.9   0.5      *
5655          CD-2312133 134946183269      D                150726.6-233756151317.5-240030340.01 28.47 6.47  -0.04 -0.41         B9                 -0.011-0.020      +003           0.0   0.0      *
5656 25Iot2LibBD-19 4055 134967159105                       150737.3-191616151319.2-193851343.03 31.96 6.08  +0.12               A3Vn               -0.051-0.037      -004SB
5657 23    LibCD-2411928 134987183275                       150737.9-245556151328.7-251833339.19 27.38 6.45  +0.70               G4V                -0.398-0.071 +.034+003V?
5658          CD-2510801 135051183285        9538           150757.7-254908151353.3-261137338.69 26.61 5.84  +1.14              gG5                 -0.015 0.000      -028V?         4.5   1.8
5659          BD+19 2939 135101101437        9535A          150815.2+193911151243.5+191709 26.94 56.71 6.68  +0.68 +0.25 +0.33   G5V                -0.590+0.290 +.024-037V?         0.8  23.7      *
5660  1    LupCD-3111813 1351532064451399                   150829.6-310845151437.3-313109335.50 22.15 4.91  +0.37 +0.28 +0.31   F1II               -0.005+0.001 +.014-023        0
5661          CP-60 5698 135160253082      W                150831.8-603156151636.4-605414319.68-02.84 5.73  -0.08 -0.89         B0.5Ve            v-0.026 0.000      +006      196  2.9   1.2AB   3*
5662 26    LibBD-17 4285 135230159122                       150855.0-172342151433.7-174607344.68 33.26 6.17  -0.05 -0.26         B9.5III            -0.022-0.014      -026V?
5663          CD-47 9824 135235225590      W                150856.1-474205151553.8-480427326.37  8.11 5.95  +0.21 +0.11         A3m                +0.018-0.032      -009           4.2  13.3
5664   Del CirCP-60 5701 135240253084      W        6998    150851.4-603510151656.7-605727319.69-02.91 5.09  -0.06 -0.92         O8.5V              -0.019-0.009      +009SBO   189  8.3  50.       *
5665          BD+23 2789 135263 837233201                   150906.5+232116151331.9+225900 33.56 57.66 6.30  +0.06 +0.06         A2V                +0.049+0.093      -005
5666   Eps CirCP-63 3544 1352912530883205                   150911.8-631426151738.9-633638318.35-05.20 4.86  +1.25 +1.32 +0.27E  K2.5III            +0.001+0.008 +.004-005
5667          CD-41 9682 135345225600      I                150929.1-410711151604.0-412928329.98 13.66 5.16  +0.58 +0.10?        G5Ia+B             -0.005-0.006 -.003-027                          *
5668          CD-43 9749 135348225602                       150929.2-430649151610.5-432905328.90 11.97 6.04  -0.14 -0.63         B3IV               -0.007+0.003      -021
5669          BD-04 3840 135367140408    I                  150934.1-050750151450.7-053010355.07 42.40 6.28  +1.47 +1.81         K2                 -0.018+0.005      -027
5670   Bet CirCP-58 5875 135379242384 561                   150940.9-582540151730.8-584804320.89-01.12 4.07  +0.09 +0.09         A3V                -0.100-0.137 +.053+010       59
5671   Gam TrACP-68 2383 135382253097 560           7008    150934.1-681836151854.6-684046315.71-09.55 2.89   0.00 -0.02         A1V                -0.072-0.031 +.010-003V     214
5672          BD+68  823 135384 16630                       150940.4+680926151044.1+674653105.37 44.30 6.17R                     A8Vn               +0.001 0.000      -008V     200
5673          BD+38 2629 135402 645723202                   150947.0+383820151335.6+381553 62.59 58.32 6.20  +1.20              gK2                 -0.010-0.047      -062V?
5674          BD+32 2561 135438 64574    I W                151000.3+320940151406.1+314717 50.19 58.74 5.99  +1.52               K5                 +0.043-0.018      +004           1.5 122.1
5675  3    SerBD+05 2985 135482120916 562I                  151013.0+051838151511.4+045622  6.46 49.21 5.33  +1.09 +0.91 +0.58   K0III              -0.018+0.005 +.021-034
5676 48Chi BooBD+29 2640 135502 837293204                   151018.2+293207151429.2+290951 45.16 58.52 5.26  +0.03 +0.08         A2V                -0.066+0.030 +.026-016V?    101
5677          BD+42 2577 135530 45445    I                  151033.4+423238151410.3+421017 69.70 57.34 6.13  +1.66 +1.93 +0.92E  M2IIIa             +0.019-0.014      -007V?
5678          BD-21 4065 1355341833283206I                  151035.0-220146151623.0-222358341.76 29.34 5.50  +1.38 +1.48         K5III              -0.034+0.003      -005V
5679  4    SerBD+00 3327 135559120920                       151043.4+004432151549.1+002220  1.32 46.23 5.63  +0.18 +0.09         A4V                -0.107+0.015 +.020-012V     117
5680          CP-60 5720 135591253101      W                151045.7-600743151848.9-602947320.13-02.64 5.46  -0.10 -0.90?        O7.5III((f))       -0.027-0.002      -003V     121  5.5  44.5AC   3*
5681 49Del BooBD+33 2561 135722 64589 563I   9559A  7002    151128.2+334116151530.2+331853 53.12 58.43 3.47  +0.95 +0.66 +0.51   G8IIIFe-1          +0.086-0.112 +.030-012SB   < 19: 4.4 104.9      *
5682          CD-40 9481 135730225631                       151134.8-404137151809.4-410339330.56 13.81 6.28  +0.18 +0.14 +0.08   Am                 +0.028+0.007      -006V
5683   Mu  LupCD-47 9860 135734225638      W     AB         151134.4-473025151832.0-475230326.86  8.05 4.27  -0.08 -0.37 -0.04   B8Ve               -0.026-0.038D+.013+015      308: 0.1   1.3AB   3*
5684          CP-67 2836 135737253115      W                151132.1-670657152040.7-672853316.51-08.63 6.28  -0.09 -0.61         B3V                -0.003-0.008      -004V          1.9   0.8      *
5685 27Bet LibBD-08 3935 135742140430 564I          7009    151137.4-090050151700.4-092259352.02 39.23 2.61  -0.11 -0.36 -0.10   B8V                -0.096-0.019 -.003-035SB    230                 *
5686  2    LupCD-2911630 1357581833463207I          7012    151144.6-294651151749.9-300856336.96 22.88 4.34  +1.10 +1.07 +0.56   G9IIIaFe1          -0.008-0.010 +.018-004V?                        *
5687          CD-40 9496 135876225647              GG Lup   151222.8-402518151856.4-404718330.85 13.95 5.59  -0.10 -0.46         B7V                -0.014-0.024      +011SB2   150                 *
5688          CD-3012117 135896206509      W                151233.4-305035151841.3-311234336.46 21.91 6.18  +1.23               G6-8III            -0.006-0.008      -024           7.9  21.5
5689          CD-3610062 136014206523                       151309.1-364342151931.6-370548333.07 16.96 6.20  +0.96 +0.64         G6III-IV           -0.092-0.124      +011
5690          BD+00 3337 136028140444    I                  151318.3-000545151826.2-002741  1.02 45.19 5.89  +1.51 +1.82         K5III              +0.006-0.007      -013V?
5691          BD+67  876 136064 16660 565                   151329.3+674335151438.3+672048104.58 44.33 5.13  +0.53 +0.08         F9IV               +0.221-0.390 +.046-047    =<  6
5692          BD+21 2755 136138 837551400                   151355.4+205618151824.5+203422 29.93 55.89 5.70  +0.97               G8IIIaBa0.3        -0.012-0.020      -008V?
5693          BD+69  789 136174 16662                       151407.0+691849151451.9+685643106.15 43.21 6.51R                     A1Vn               +0.024-0.009      -011      151
5694  5    SerBD+02 2944 136202120946        9584  MQ Ser   151412.4+020837151918.8+014555  3.68 46.46 5.06  +0.54 +0.06 +0.27   F8III-IV           +0.374-0.514 +.037+054V       2  4.6  11.2AB   4*
5695   Del LupCD-40 9538 1362982256911402          Del Lup  151448.3-401707152122.3-403851331.32 13.82 3.22  -0.22 -0.89 -0.22   B1.5IV             -0.015-0.026      +000V?    221                 *
5696          CD-40 9539 136334225695                       151500.9-402316152135.3-404459331.30 13.71 6.20  +0.07               A1V                -0.010-0.032      -004
5697          CD-3710171 136347206543      W                151503.2-375125152130.1-381309332.75 15.81 6.48  -0.06 -0.30         A0pSi              -0.013-0.041      +007           2.6   5.7      *
5698   Nu 1LupCD-47 9922 1363512257033211                   151510.1-473349152208.3-475540327.34  7.67 5.00  +0.50 +0.04?        F8V                -0.140-0.137 +.032-011        0
5699   Nu 2LupCD-47 9919 136352225697                       151503.4-475657152148.2-481904327.10  7.37 5.65  +0.65 +0.05 +0.22E  G3-5V              -1.617-0.277 +.066-069
5700          CP-60 5760 136359253136                       151504.0-601749152310.4-603925320.49-03.07 5.67  +0.48  0.00         F7V                -0.090-0.020      +006
5701 28    LibBD-17 4312 136366159187                       151513.3-174743152053.7-180931345.78 32.01 6.17  +1.02 +0.73 +0.37E  G8III              -0.011-0.061      +003V?                        *
5702          BD+33 2574 136403 64630                       151528.0+325238151930.2+323055 51.58 57.60 6.32  +0.24 +0.09 +0.08   A2m                -0.015+0.016      -025SBO    18                 *
5703 29Omi LibBD-15 4083 136407159191      W                151525.8-151116152101.4-153254347.81 33.98 6.30  +0.41 +0.05         F0III              +0.024+0.026      +005V      82  2.2  44.4
5704   Gam CirCP-58 5908 136415242463      W                151524.5-585738152322.7-591915321.25-01.96 4.51  +0.19 -0.35         B5IV+F8            -0.016-0.038D+.010-017      278  0.3   0.9      *
5705   Phi1LupCD-3510236 136422206552 566I W                151527.5-355355152148.4-361541333.96 17.39 3.56  +1.54 +1.88 +0.87   K5III             e-0.089-0.084 +.014-029          10.9  17.3AC   3*
5706          BD-01 3047 136442140473                       151537.4-020250152047.1-022448359.53 43.45 6.35  +1.06 +1.07         K0V                -0.253-0.177 +.033-047V?
5707          BD-05 4057 1364791404743209  W                151550.5-052750152107.6-054929356.23 41.06 5.54  +1.04 +1.00         K1III              -0.050-0.008      -033V          7.1  12.2
5708   Eps LupCD-4410066 136504225712      W       Var?     151553.3-441947152240.9-444122329.23 10.32 3.37  -0.18 -0.75 -0.16   B2IV-V             -0.019-0.013D+.009+008SB2O  133  1.5   0.4AB   3*
5709  1Omi CrBBD+30 2647 136512 83768      W        7032    151600.2+295844152008.5+293658 46.21 57.33 5.51  +1.02 +0.77 +0.53   K0III              -0.120-0.047 +.001-053V?         3.9 147.3
5710  6    SerBD+01 3067 136514120955    I   9596   7037    151556.4+010444152102.0+004255  2.90 45.45 5.35  +1.19 +1.22 +0.60   K3III              -0.042-0.106 +.007+009V    < 17  4.0   3.1      *
5711          BD+25 2902 136643 83778                       151647.5+251906152106.9+245728 37.83 56.43 6.39  +1.23               K0                 -0.026-0.026      -002
5712   Phi2LupCD-3610103 1366642065801403                   151645.8-363000152309.4-365131333.84 16.75 4.54  -0.15 -0.63 -0.14   B4V                -0.018-0.023      +002      194                 *
5713          CP-67 2864 136672253155      W        7058    151648.6-675715152614.7-681833316.49-09.61 5.89  +1.02 +0.77         G8-K0III           +0.121+0.011      +025           6.0  50.
5714 11    UMiBD+72  678 136726  8207    I                  151710.2+721112151705.9+714926108.72 41.04 5.02  +1.37 +1.60         K4III              +0.007+0.011 +.015-016     < 17                 *
5715          BD+52 1869 136729 294873210  S                151708.6+521907152005.2+515731 84.96 52.77 5.66  +0.11 +0.11         A4V                +0.014+0.012      +008V     181        0.2
5716          BD+44 2453 136751 45497                       151714.5+444749152041.6+442603 73.01 55.55 6.19  +0.39 +0.04         F3-4IV s           +0.027-0.106      -000                          *
5717  7    SerBD+13 2928 1368311015083213                   151739.0+125532152223.2+123403 18.09 51.83 6.28  -0.02 -0.03         A0V                -0.004-0.004      +004V?     72
5718 50    BooBD+33 2581 136849 64656                       151748.3+331730152148.6+325602 52.36 57.12 5.37  -0.07 -0.21         B9Vn               -0.051+0.017 +.005-007SB    260
5719   Ups LupCD-39 9827 136933206597      W                151812.9-392113152444.9-394237332.42 14.23 5.37  -0.11               A0pSi             v-0.037-0.047 +.008-008        0  5.5   1.4      *
5720          BD-11 3940 1369561592273214                   151822.9-120045152352.2-122210351.02 35.90 5.72  +1.04              gG6                 -0.038-0.039      -026V?                        *
5721  8    SerBD-00 2961 1370061405021406                   151834.3-003957152343.7-010121  1.64 43.82 6.12  +0.25 +0.07 +0.12   F0V                +0.076-0.030      -002V     105                 *
5722          CD-3710225 137015206607      W                151838.7-374852152506.6-381010333.39 15.45 7.03  +0.09               A2V                -0.010-0.029      +003           0.0   0.2
5723 31Eps LibBD-09 4138 137052159234                       151846.5-095746152411.9-101920352.84 37.34 4.94  +0.44 +0.02         F5IV               -0.067-0.154 +.035-010SB1O    0                 *
5724          CD-3810289 137058206616      W                151851.0-382245152520.2-384401333.09 14.96 4.60   0.00 -0.06         A0IV               -0.052-0.016 +.012-003      231  4.0  92.4AxBC 5
5725          CP-64 3178 1370662531743217                   151850.2-641044152733.1-643153318.75-06.57 5.71  +1.64 +1.98         K5-M0III           -0.015-0.019      -009
5726          BD+40 2877 137071 64667    I                  151855.4+395618152237.4+393453 64.43 56.36 5.50  +1.59              gK4                 +0.002-0.015      -012
5727  2Eta CrBBD+30 2653 137107 64673        9617A  7054    151904.3+303856152312.3+301716 47.54 56.73 5.58H +0.58 +0.04 +0.28   G0V                +0.138-0.187 +.061-007SBO     3  0.3   0.5AB   4*
5728  2Eta CrBBD+30 2653 137108              9617B  7054    151904.3+303856152312.3+301716 47.54 56.73 6.08H                     G3V                +0.137-0.187 +.061-007           0.3   0.5AB   4*
5729   Rho OctCP-84  510 1373332587311666                   152012.0-840755154316.8-842755307.02-23.02 5.57  +0.11 +0.08         A2V                +0.131+0.097      -011      123
5730   Kap1ApsCP-72 1802 137387257289 567  W       Kap1 Aps 152036.6-730233153130.8-732322313.85-14.01 5.49  -0.12 -0.77 -0.02   B3IVe             v-0.006-0.013      +062V     300v 6.9  27.0      *
5731          BD+62 1410 137389 16712              Var?     152045.3+622410152237.3+620250 97.98 47.09 5.98  -0.03 -0.10         A0pSi              +0.010-0.034      -024                          *
5732          BD+45 2284 137390 455213216                   152042.7+453728152405.1+451616 74.08 54.73 6.01  +1.20 +1.23         K2III              -0.014+0.008      -010
5733 51Mu 1BooBD+37 2636 137391 64686 568    9626A  7063    152042.7+374340152429.4+372238 60.39 56.31 4.31  +0.31 +0.07 +0.15   F2IVa              -0.146+0.087 +.039-013SB     84  2.2 108.9AxBC 3*
5734 51Mu 2BooBD+37 2637 137392 64687        9626BC         152044.1+374153152430.9+372052 60.34 56.31 6.50  +0.59 +0.13         G1V                -0.144+0.094 +.039-009V?  =< 25  2.2 108.9AxBC 3*
5735 13Gam UMiBD+72  679 137422  8220 569I         Gam UMi  152052.9+721123152043.7+715002108.46 40.84 3.05  +0.05 +0.12 +0.06   A3II-III          v-0.019+0.020 +.003-004V     171                 *
5736          CD-3610161 137432206660      W                152053.7-362500152718.2-364604334.60 16.34 5.45  -0.15 -0.59 -0.17   B4Vp               -0.012-0.035      +004V     134  7.0  30. AC   3*
5737          BD+63 1192 137443 167133215I                  152058.1+634154152238.4+632029 99.48 46.30 5.79  +1.27              gK4                 -0.013-0.091 +.005-046V?
5738          CD-51 9132 137465242569      W                152109.3-511456152827.2-513551326.14  4.06 6.10  +1.09 +0.76         G2II               -0.007+0.005      -016           6.3  13.3
5739  9Tau1SerBD+15 2858 137471101545 570I          7074    152109.0+154647152547.4+152541 22.86 52.36 5.17  +1.66 +1.95 +1.04   M1III              -0.013-0.007 +.013-020V?
5740          BD+19 2966 137510101548                       152123.3+194955152553.3+192851 29.13 53.87 6.27  +0.60 +0.20         G0IV-V             -0.057-0.006      -003    =< 10                 *
5741          BD+34 2645 137704 647013218I                  152222.4+344103152617.4+342009 54.89 56.16 5.46  +1.40 +1.64 +0.77   K4III              -0.114+0.056      -048                          *
5742          CD-4610100 1377092258253220                   152226.5-462309152924.3-464358329.05  7.97 5.24  +1.74               K4III              -0.010-0.006 +.005-028
5743 32Zet1LibBD-16 4089 1377441592801407I                  152236.9-162204152815.4-164259348.46 31.96 5.64  +1.55 +1.84         K5III              +0.017-0.032      -021V?                        *
5744 12Iot DraBD+59 1654 137759 29520 571I W        7077    152242.2+591859152455.8+585758 93.97 48.63 3.29  +1.16 +1.22 +0.60   K2III              -0.009+0.017 +.040-011     < 17  5.7 254.6      *
5745          BD+25 2916 137853 838303219I                  152320.7+252659152738.9+250606 38.63 55.02 6.02  +1.62              gM1                 +0.007-0.014      -007
5746 10    SerBD+02 2965 1378981210203221                   152335.1+021122152838.2+015032  5.82 44.67 5.17  +0.23 +0.10         A8IV               -0.083-0.029 +.031-010SB    129
5747  3Bet CrBBD+29 2670 137909 83831 572  W       Bet CrB  152342.3+292701152749.7+290621 45.59 55.61 3.68  +0.28 +0.11 +0.05   F0p               v-0.179+0.086 +.032-019SBO    19  1.5   0.3      *
5748          BD+54 1747 137928 29527                       152349.1+542207152632.5+540112 87.20 50.95 6.45  +0.05               A2IV               +0.048-0.019      -005
5749          BD-20 4246 138105183533               7114    152449.3-202304153036.3-204342345.90 28.57 6.22  +0.17 +0.08 +0.06   A5m                +0.014-0.019      -011       30
5750 34Zet3LibBD-16 4099 138137159307                       152501.8-161559153040.4-163634349.05 31.66 5.82  +1.06              gG6                 +0.018+0.001      -002V?
5751          CD-3810425 138204206736                       152533.4-381646153204.4-383723334.26 14.29 6.25  +0.20               A7V                -0.026-0.089      -001
5752          BD+47 2227 138213 45562                       152531.3+473245152844.5+471205 76.76 53.35 6.15  +0.10 +0.12 +0.04   AmA3-F0V:          -0.015-0.002      -016SBO    30                 *
5753          CD-3210868 1382212067343223                   152535.1-323221153150.3-325252337.81 18.91 6.46  +0.09               B7V                -0.012-0.024      +005V?
5754          BD+62 1414 138245 16740                       152554.0+623717152740.8+621632 97.74 46.48 6.50  +0.14 +0.04         A5IV               +0.033-0.024      -005V
5755          BD+61 1509 138265 16741    I                  152552.3+610055152751.4+604013 95.79 47.38 5.90  +1.44 +1.64         K5III              -0.019-0.002      -047SB
5756          BD-19 4128 138268159317        9681A          152558.1-194922153143.4-200953346.54 28.82 6.22  +0.22 +0.14         A8V                -0.075-0.028      -040SB    225  0.0   0.0 O   4*
5757          CP-77 1134 1382892573033228                   152600.9-773450153918.3-775505311.39-17.90 6.18  +1.20 +1.36         K2IIICN0.5Ba-1     -0.085-0.123      +013
5758          BD+09 3055 1382901210381408                   152604.0+085516153055.4+083445 14.34 48.02 6.57  +0.37 -0.03         F4V w              +0.034-0.003      -001       23
5759          BD+55 1756 138338 295403222                   152621.8+553215152856.8+551142 88.57 50.09 6.43  +0.08 +0.09 +0.02   A3m                -0.007+0.036      -007       40
5760          BD+31 2742 138341 64736                       152620.0+313743153022.7+311710 49.50 55.25 6.46  +0.19 +0.14 +0.11   A4IV               -0.024-0.012      -004
5761          BD+37 2651 138383 64741                       152640.4+370843153027.9+364815 59.18 55.18 6.37R                     K0                 -0.059+0.023      +002
5762          BD-19 4135 1384131593303224                   152651.9-191948153236.7-194014347.09 29.06 5.52  +0.18 +0.20 +0.09   A2m                -0.021-0.035 +.018-033V?     27
5763 52Nu 1BooBD+41 2609 138481 45580 573I                  152720.2+411026153055.8+404959 66.11 54.58 5.02  +1.59 +1.90 +0.93   K4.5IIIbBa0.4      +0.011-0.007 +.020-010     < 17                 *
5764 35Zet4LibBD-16 4110 138485159335               7126    152716.1-163049153255.2-165110349.32 31.11 5.50  -0.14 -0.75 -0.17   B2Vn               -0.011-0.014      +009SB    256  2.0   0.0      *
5765          CD-2412155 138488183565        9689A          152714.3-240904153309.1-242922343.69 25.31 7.00  +0.27 +0.04         A3+F0V:            -0.003-0.038      -008           0.1   9.3AxBC 3*
5766          CP-65 3100 138498253222      W                152718.0-651634153617.4-653648318.88-07.98 6.51  +0.35 +0.03         F3IV-V             -0.092-0.063      -008           7.0   5.3
5767          CD-39 9970 138505206763                       152725.8-394340153401.6-400358333.69 12.90 5.82  +1.68 +1.93 +1.02E  M2III              -0.055-0.025      -040V?                        *
5768          BD+62 1416 138524 16746                       152734.8+622630152921.2+620559 97.36 46.42 6.38R                     K5                 -0.024-0.001      -040
5769          BD+37 2653 138525 64754                       152734.0+365726153122.4+363659 58.83 55.02 6.38  +0.50               F6III              +0.006-0.031      -050    =< 10
5770 12Tau2SerBD+16 2797 138527101600                       152733.1+162343153209.7+160322 24.80 51.22 6.22  -0.05 -0.23         B9V                +0.001+0.009      -018V     175
5771   Eps TrACP-65 3102 138538253226 574  W                152733.9-655850153643.2-661901318.49-08.57 4.11  +1.17 +1.16 +0.40E  K1-2III            +0.023-0.055 +.038-016V?         5.4  83.2      *
5772 11    SerBD-00 2982 1385621405963226                   152748.8-005049153257.9-011111  3.49 41.94 5.51  +1.09 +1.02         K0III              -0.017-0.042      -016
5773          CD-3810464 138564206769                       152747.0-390040153420.9-392058334.18 13.44 6.36  -0.03 -0.11         B9V                -0.028-0.047      +000                          *
5774 53Nu 2BooBD+41 2611 138629 45590        9688           152812.1+411419153147.0+405358 66.17 54.41 5.02  +0.07 +0.11 +0.04   A5V                -0.017-0.009 +.020-017V?    182  0.0   0.1AB   3*
5775 36    LibCD-2710443 138688183580    I                  152833.4-274236153437.3-280249341.50 22.35 5.15  +1.30 +1.16         K4III              +0.014-0.033 +.011+012SB
5776   Gam LupCD-40 9760 138690225938      W                152828.5-404950153508.5-411001333.19 11.89 2.78  -0.20 -0.82 -0.24   B2IV              e-0.015-0.028D+.008+002V     266  0.1   0.6      *
5777 37    LibBD-09 4171 1387161406091409I                  152842.6-094318153410.7-100352355.24 35.79 4.62  +1.01 +0.86 +0.52   K1III-IV           +0.309-0.234 +.027+048     < 19:                *
5778  4The CrBBD+31 2750 138749 64769 576  W        7134/5  152853.8+314148153255.8+312133 49.69 54.71 4.14  -0.13 -0.54 -0.11   B6Vnne             -0.019-0.011 +.027-025V     393  1.4   0.5      *
5779          BD-05 4100 1387631406133227                   152903.7-052134153420.8-054142359.28 38.75 6.51  +0.58 +0.04         F7V                -0.095+0.011      -038V
5780          BD-08 4010 138764140614                       152902.4-085049153426.6-091100356.08 36.36 5.17  -0.09 -0.45 -0.12   B6IV               -0.016-0.023      -002SB?    24                 *
5781          CD-4410239 138769225950      W       Var?     152859.9-443724153553.2-445731331.02  8.76 4.54  -0.18 -0.69 -0.22   B3IVp              -0.025-0.030D+.014+007SB    101  1.9   2.4      *
5782   Kap2ApsCP-73 1625 138800257307      W                152915.6-730659154021.2-732648314.34-14.44 5.65  -0.04 -0.38         B7III-IV           -0.021-0.024      -019           6.7  15. AB   3*
5783          BD+17 2880 138803101618                       152918.8+172829153352.8+170816 26.64 51.26 6.45R                     F3III              -0.042-0.038      -021       67
5784          CD-4310036 1388162259571410                   152920.9-440342153612.1-442349331.40  9.18 5.43  +1.50 +1.82         K4-5III            -0.045-0.057 +.017-019
5785          BD+64 1074 138852 167543225                   152931.3+643242153055.7+641231 99.67 45.05 5.79  +0.96               K0III-IV           -0.117+0.081 +.002+010V?                        *
5786          CP-75 1222 1388672573103235                   152936.9-754511154154.6-760455312.72-16.56 5.95  -0.04 -0.16         B9V                -0.018-0.041      +000
5787 38Gam LibBD-14 4237 138905159370 577I   9704           152955.8-142722153531.6-144722351.51 32.20 3.91  +1.01 +0.74 +0.55   G8.5III            +0.065+0.009 +.041-028V    < 17  0.2   0.1AP   3*
5788 13Del SerBD+11 2821 138917101623        9701B          153001.4+105219153448.1+103215 17.59 48.19 3.80  +0.26 +0.13         F0IV               -0.074-0.005 +.021-038V?     70  1.1   3.9AB   4*
5789 13Del SerBD+11 2821 138918101624        9701A Del Ser  153001.4+105223153448.1+103221 17.59 48.19 3.80  +0.26 +0.12 +0.11   F0IV               -0.074+0.012 +.021-042V      80  1.1   3.9AB   4*
5790          CD-3210930 138923206795      W        7149    152954.8-324533153611.4-330534338.43 18.19 6.24                      B8V                -0.016-0.017      -004           0.0   0.1
5791          BD+02 2977 138936121071                       153001.2+020014153504.6+014008  6.98 43.28 6.56  +0.28 +0.05         F0III              -0.074-0.035      -020       71
5792          CP-69 2422 1389652573083233                   153007.7-695350154011.5-701340316.36-11.89 6.44  +0.08 +0.08         A1V                -0.049-0.060      -002
5793  5Alp CrBBD+27 2512 139006 83893 578I         Alp CrB  153027.2+270304153441.3+264253 41.87 53.77 2.23  -0.02 -0.02 -0.04   A0V+G5V            +0.121-0.089 +.045+002SBO   133                 *
5794 39Ups LibCD-2710464 139063183619 579I   9705           153057.1-274814153701.5-280806341.89 21.94 3.58  +1.38 +1.58 +0.71   K3III             e-0.009+0.003 +.044-025           7.0   3.5      *
5795 15Tau3SerBD+18 3044 1390741016313230                   153100.8+175919153533.2+173920 27.65 51.08 6.12  +0.94              gG8                 -0.079-0.014      -022V
5796          BD+11 2826 1390871016343231                   153107.8+113553153553.4+111556 18.74 48.30 6.07  +1.09              gK0                 -0.030-0.012      -026V
5797   Ome LupCD-4210601 1391272260043232  W                153118.8-421421153803.2-423403332.78 10.44 4.33  +1.42 +1.72 +0.73   K4.5III            -0.153+0.057 +.017-007SB         6.7  11.8      *
5798          CD-51 9324 1391292427931411  W                153123.4-520234153849.5-522222326.99  2.50 5.44   0.00  0.00         A0V                -0.036-0.035 +.015-012        0  4.5  53.4AB   3
5799 14    SerBD-00 2988 139137140643                       153125.7-001347153633.7-003342  4.89 41.63 6.51  +0.72 +0.45         G8III+A            -0.027-0.022      -023SB
5800  6Mu  CrBBD+39 2889 139153 64790    I                  153134.7+392032153515.0+390036 62.81 54.03 5.11  +1.64 +2.01         M1.5III-IIIb       +0.029+0.006 +.003-019V?
5801          CD-2511000 139160183631                       153128.4-255656153728.5-261648343.25 23.31 6.19  +0.01 -0.42         B7IV               -0.010-0.023      +001V     152                 *
5802 16    SerBD+10 2884 139195101640                       153141.2+102041153629.6+100036 17.21 47.57 5.26  +0.95 +0.67 +0.45   K0III:CN1Ba0.7Sr2  +0.045-0.129 +.030+008V    < 17
5803          CP-59 6206 1392112428033236                   153145.4-593429153956.5-595430322.63-03.64 5.95  +0.48               F6V                -0.122-0.220 +.029-022
5804 18Tau5SerBD+16 2807 139225101642                       153152.7+162700153629.3+160708 25.54 50.29 5.93  +0.29 +0.04         F3V                +0.076-0.007      -002V                         *
5805          CD-3810532 1392332068263234                   153158.1-384952153832.7-390939334.96 13.10 6.57  -0.07 -0.19         B9V                -0.013-0.032      +005                          *
5806          BD-22 3989 139254183637                       153155.2-224836153748.1-230830345.55 25.65 5.78  +1.10 +0.89         K0III              -0.022-0.080      +007V?                        *
5807          CD-3810536 139271206827      W                153207.4-384757153842.3-390741335.01 13.11 6.04  +0.21               A3-5mA3-F2         +0.023-0.008      +014           8.0  10.9      *
5808          BD+38 2678 139284 64797                       153207.2+384219153549.3+382226 61.71 53.99 6.42R                     K2                 +0.029-0.006      +003
5809          CD-2710478 139290183641                       153210.8-275238153815.7-281224342.06 21.72 6.32  +1.17               K1III              +0.003-0.026      +029V
5810          BD-20 4285 139329183646                       153227.3-204109153816.3-210058347.21 27.17 5.84  +1.08              gG9                 +0.068-0.060      +032V?
5811          BD+54 1756 139357 29583                       153237.3+541512153516.3+535519 86.16 49.85 5.97  +1.18              gK4                 -0.011-0.004      -010
5812 40Tau LibCD-2911837 139365183649                       153230.7-292656153839.4-294640341.06 20.45 3.66  -0.17 -0.70 -0.19   B2.5V              -0.015-0.030      +003SB    149                 *
5813          BD+30 2682 139389 64808                       153247.9+301920153653.4+295928 47.47 53.75 6.52                      F5V:               +0.087-0.060      -013       23
5814 41    LibBD-18 4118 139446159411               7169    153309.0-185821153854.5-191807348.63 28.35 5.38  +0.86 +0.54 +0.46   G8III             v+0.085-0.076      +047
5815          BD-08 4031 139460140671        9728B          153316.0-082800153840.0-084740357.31 35.87 6.50  +0.53 -0.02         F6IV-V             +0.022-0.014D+.011+004V?  =< 25  0.2  11.9      *
5816          BD-08 4032 139461140672        9728A          153316.1-082748153840.1-084728357.32 35.87 6.48  +0.52 -0.02         F6V                +0.028-0.018D+.011+000SB  =< 25  0.2  11.9      *
5817          BD+52 1886 139478 29589                       153315.9+522351153604.1+520411 83.44 50.49 6.74  +0.32 +0.04         F4IIIp             -0.022+0.082      -016
5818          BD+55 1766 139493 29588      S        7163    153322.7+545738153557.1+543750 87.08 49.45 5.74R                     A2V                -0.035-0.010      -019                 0.5
5819          BD-22 3996 139518183665                       153328.2-224923153921.3-230901345.84 25.41 6.34  +0.02               A0V                -0.036-0.029      -009
5820  3Psi1LupCD-3310631 1395212068433237                   153324.8-340508153946.0-342443338.19 16.69 4.67  +0.99 +0.73 +0.49   G8III              +0.004-0.015 +.032-023
5821          CD-4710210 139599226059      W                153352.4-472437154058.3-474408330.05  6.01 6.23  +1.63 +1.98         K5-M0III           -0.027-0.025      -054           5.9  18.1
5822          CD-3012431 139613206858                       153403.1-305316154015.4-311249340.38 19.12 6.34  +1.43 +1.65         K2III              -0.061-0.020      -022
5823 54Phi BooBD+40 2907 139641 45643 580                   153414.0+404044153749.6+402112 64.95 53.37 5.24  +0.88 +0.53 +0.47   G7III-IVFe-2       +0.063+0.062 +.021-010V?   < 19:                *
5824 42    LibCD-2312458 1396631836863239I                  153422.0-232935154016.9-234905345.53 24.77 4.96  +1.33 +1.51 +0.68   K3IIICN1           -0.013-0.012 +.049-022V                         *
5825          CD-4410310 139664226064                       153418.7-441948154111.3-443940331.96  8.44 4.64  +0.40 -0.03 +0.20   F5IV-V             -0.177-0.262 +.060-005       87
5826 15The UMiBD+77  592 139669  82743229I                  153422.4+774057153124.9+772058112.85 36.46 4.96  +1.58 +1.89 +0.87   K5III              -0.049+0.010 +.021-025V    < 17
5827          BD+35 2711 139761 648253238                   153456.9+350004153848.9+344030 55.41 53.58 6.11  +1.02               K0                 -0.006-0.016      +004
5828          BD+54 1758 139778 29597                       153457.7+545010153732.0+543032 86.76 49.30 5.87  +1.07              gK1                 -0.037-0.015      -023
5829          BD+80  480 139777  2556        9696A          153458.7+804649152911.0+802655115.60 34.31 6.58  +0.67 +0.13         G0IV-V+G8IV-V      -0.228+0.113 +.039-014SB         0.7  31.4AB   3*
5830          BD+47 2253 139798 456501412                   153504.0+470738153816.2+464752 75.32 51.93 5.75  +0.36 -0.02 +0.18   F2V                +0.090-0.126 +.009-002V                         *
5831          BD+12 2875 139862101673        9740           153526.6+122237154010.4+120311 20.52 47.74 6.25  +0.94 +0.71         G7.5IIIaFe-0.5     -0.007-0.007      -021SB         4.2  16.5      *
5832          CD-49 9909 1398712260893241                   153522.8-491003154237.2-492922329.21  4.45 6.04  +1.31               K2III              -0.004-0.018      -020
5833  7Zet1CrBBD+37 2665 139891 64833        9737B          153536.2+365741153922.2+363812 58.70 53.41 6.00H                     B9V                -0.014-0.013 +.015-019       25  1.0   6.4      *
5834  7Zet2CrBBD+37 2665 139892 64834        9737A          153536.6+365737153922.7+363809 58.70 53.41 5.07H -0.12 -0.47         B7V                -0.013-0.006 +.015-024SB2O  100  1.0   6.4      *
5835          BD+50 2206 139906 29600                       153538.9+504458153834.3+502524 80.80 50.74 5.84  +0.83               G8III              +0.005-0.038      -014
5836          CP-59 6257 139915253269                       153537.4-595802154355.1-601714322.79-04.25 6.48  +1.05               G0Ib               -0.017-0.005      -037
5837          CD-3710441 1399802068873242                   153608.2-370614154238.3-372530336.71 13.97 5.24  +0.98               G8-K0III           -0.048-0.021 +.008-016
5838 43Kap LibBD-19 4188 1399971594421413I W        7200    153611.0-192117154156.8-194044348.94 27.58 4.74  +1.57 +1.95 +0.94   M0-IIIb            -0.037-0.103 +.040-004SB         5.0 172.0
5839  4Psi2LupCD-3410494 140008206889                       153618.5-342321154241.0-344238338.48 16.08 4.75  -0.14 -0.52 -0.16   B5V                -0.021-0.029      +004SB2O   80                 *
5840 19Tau6SerBD+16 2816 140027101678                       153623.2+162050154059.2+160129 26.07 49.26 6.01  +0.90 +0.61 +0.46   G8III              +0.028-0.013      +003                          *
5841          BD+58 1583 140117 29609               7192    153656.9+581451153909.5+575528 91.21 47.56 6.45  +1.09 +1.04         K1III              -0.005+0.011      -008                          *
5842 21Iot SerBD+20 3138 140159101682        9744           153705.5+195932154133.1+194013 31.42 50.45 4.52  +0.04 +0.03 -0.01   A1V                -0.057-0.046 +.011-017SB2   149  0.1   0.1AB   4*
5843 20Chi SerBD+13 2982 1401601016833243          Chi Ser  153705.0+131006154147.4+125051 21.83 47.75 5.33  +0.04 +0.05 -0.04   A0pSr             v+0.040-0.003 +.032+002SB     64                 *
5844          BD+69  806 140227 167943240I                  153722.4+693622153739.1+691700104.68 41.46 5.62  +1.35 +1.58         K5IIIb             -0.047+0.058      -029SB
5845 22Tau7SerBD+18 3059 140232101686                       153724.6+184657154154.7+182750 29.69 49.96 5.81  +0.20 +0.11 +0.10   A2m                -0.072+0.053 +.021-030V?     20                 *
5846          CD-4110245 140285226132      W                153737.7-413003154422.6-414910334.19 10.32 5.94   0.00 -0.20         A0V+B              -0.028-0.047      +019V          1.7   3.7      *
5847          BD-14 4266 140301159461                       153748.3-144321154324.9-150236352.91 30.68 6.31  +1.16 +0.96 +0.42E  K0IV               -0.004-0.096      +021V?                        *
5848 44Eta LibBD-15 4171 1404171594663246                   153826.7-152115154404.4-154022352.52 30.13 5.41  +0.23 +0.07 +0.09   F0IV               -0.034-0.066      -031      101
5849  8Gam CrBBD+26 2722 140436 83958        9757  Gam CrB  153832.5+263645154244.6+261744 41.74 51.92 3.84   0.00 -0.04 -0.02   B9IV+A3V           -0.105+0.045 +.033-011      112  1.5   0.2      *
5850          BD+14 2922 140438101699        9758           153830.3+135911154310.6+134004 23.15 47.81 6.48  +0.86 +0.52         G5III              -0.010-0.032D+.002-010SB         1.0   0.6
5851          CP-65 3139 140483253297      W     A          153846.0-650743154753.6-652633319.94-08.58 6.18  +0.22 +0.10         A5III-IV           -0.005-0.042      -030V?         0.1   1.9      *
5852          CP-65 3139 140484            W     B          153846.0-650743154753.7-652633319.94-08.58 6.39  +0.24 +0.10         F0-2IV             -0.005-0.042      -024           0.1   1.9      *
5853 23Psi SerBD+02 2989 1405381211523248    9763           153859.9+025008154401.8+023054  9.70 41.97 5.88  +0.68 +0.25 +0.35   G2.5V              -0.054-0.151 +.046+019SB         1.3 370. AE   5*
5854 24Alp SerBD+06 3088 140573121157 582I   9765           153920.5+064425154416.1+062532 14.20 44.08 2.65  +1.17 +1.24 +0.56   K2IIIbCN1          +0.137+0.047 +.053+003V?   < 17  8.3  58.2AB   3*
5855  9Pi  CrBBD+32 2621 140716 648703249                   154002.8+324953154359.3+323057 51.87 52.44 5.56  +1.06       +0.53  gG9                 -0.032-0.010      -004V?
5856          CD-2710550 140722183772        9775           154007.4-274452154612.8-280342343.60 20.69 6.51  +0.32               F2IV               -0.067-0.026      -021           0.0   0.6AB   3*
5857          BD+52 1898 140728 296283247          BP Boo   154007.5+524035154250.7+522139 83.25 49.42 5.51  -0.07 -0.10         B9pSiCr            -0.067+0.032 +.010-016SB     75                 *
5858 26Tau8SerBD+17 2906 140729101712      S                154009.5+173443154442.1+171551 28.33 48.91 6.14   0.00 -0.03         A0V                -0.026 0.000      -005SB     76        0.1
5859          BD+05 3072 140775121170                       154026.8+054540154523.5+052649 13.27 43.32 5.58  +0.04 +0.03 -0.01   A0V                +0.027-0.015      -010       65                 *
5860          CD-3410524 140784206962      W        7239    154020.8-342210154644.2-344057339.17 15.58 5.61  -0.11 -0.42         B7Vn               -0.021-0.028      +005V          0.3   0.1      *
5861          BD+01 3125 140815121173                       154034.0+011216154539.7+005329  8.25 40.69 6.33  +1.19 +1.20         K0III              +0.043+0.008      +014
5862          CD-3910157 140861226206                       154046.4-395255154725.4-401139335.68 11.23 6.42  +0.88               G5III-IV           -0.176-0.037      -028
5863 25    SerBD-01 3092 140873140740                       154055.2-012927154605.6-014816  5.53 38.99 5.40  -0.03 -0.40         B8III              -0.027-0.038      -012SBO   116                 *
5864          CD-3710500 140901206976      W                154059.3-373558154729.0-375459337.17 12.99 6.01  +0.72 +0.31         G6V                -0.430-0.217 +.075-005           6.6  14.6AB   3*
5865          CP-52 8912 140979243001      W                154119.2-520743154850.3-522617328.16  1.52 6.07  +1.38 +1.45         K2-3III            -0.007-0.002      -018           4.5   1.2
5866          BD-05 4161 1409861407513250                   154126.5-054833154645.4-060713  1.43 36.14 6.24  +1.17 +1.09         K0                 -0.009-0.002      -051V?
5867 28Bet SerBD+15 2911 141003101725 583    9778           154134.3+154405154611.3+152519 25.98 47.87 3.67  +0.06 +0.08 +0.03   A2IV               +0.067-0.045 +.041-001V     170  6.3  30.6AB   5*
5868 27Lam SerBD+07 3023 141004121186    I          7246    154135.3+073959154626.6+072111 15.70 44.10 4.43  +0.60 +0.11 +0.32   G0-V               -0.223-0.067 +.094-066SB1O=<  6                 *
5869          CP-52 8944 141168243022      W                154231.7-525407155006.9-531234327.83  0.80 5.77  -0.08 -0.23         B9Vn:              -0.048-0.030      +016V?         5.8  28.4      *
5870 31Ups SerBD+14 2939 141187101739                       154238.6+142525154717.3+140655 24.38 47.09 5.71  +0.09 +0.06         A3V                -0.060+0.039      -034                          *
5871          CD-4810349 141194226254                       154243.1-483619154957.5-485443330.51  4.16 5.84  +0.07 +0.06         A1V                -0.032-0.001      +012V
5872          CD-4510251 1412962262633251  W                154315.8-450542155016.3-452406332.77  6.86 6.12  +0.29 +0.03         A9III              +0.045-0.033      +002           4.8   2.9AxBC 3
5873          CP-54 6711 141318243044      W                154320.3-544502155106.8-550321326.79-00.73 5.73  +0.06 -0.70         B2II               -0.001-0.014      -001SB     54  3.4  18.0AB   4*
5874          BD+14 2940 141353101744    I                  154333.6+140558154813.3+134719 24.09 46.75 6.00  +1.25 +1.50        gK2                 -0.006-0.116      -054V
5875          BD-03 3829 141378140775                       154342.4-033043154856.8-034907  4.08 37.18 5.53  +0.12 +0.11 +0.04   A5IV               -0.033+0.009      -016      120
5876          CP-64 3286 141413253323                       154348.1-645100155256.7-650909320.54-08.69 6.54  +0.19 +0.12         A5IV               +0.035+0.008      +016
5877          BD+32 2631 141456 64915    I                  154403.7+320239154802.0+314409 50.70 51.53 6.44R                     K5                 +0.032-0.041      -020
5878          BD+55 1777 141472 29655    I                  154411.5+554652154634.8+552829 87.27 47.72 5.92  +1.39              gK3                 -0.125+0.076      -004
5879 35Kap SerBD+18 3074 141477101752 584I          7269    154414.2+182701154844.4+180830 30.10 48.33 4.09  +1.62 +1.95 +0.98   M0.5IIIab         e-0.051-0.088 +.021-039                          *
5880          BD+28 2477 141527 84015    I         R CrB    154427.1+282748154834.4+280924 45.05 50.98 5.85  +0.77 +0.29         G0Iep              -0.002-0.015 +.010+025V      18                 *
5881 32Mu  SerBD-02 4052 141513140787 585                   154424.0-030727154937.2-032549  4.59 37.29 3.53  -0.04 -0.10 -0.05   A0V                -0.086-0.024 +.007-009SB     87
5882          CD-4610430 141544226295                       154425.3-464524155131.5-470338331.88  5.44 6.01  +1.16 +1.12         K2III              -0.112-0.030      +030SB1O                      *
5883  5Chi LupCD-3310754 141556207040 586                   154436.1-331921155057.5-333738340.57 15.82 3.95  -0.04 -0.13 -0.07   B9IV               -0.006-0.030      +005SB2O    0                 *
5884          CP-62 4990 141585253328               7299    154440.0-621819155322.8-623625322.22-06.76 6.19  +1.47               K3III              -0.011-0.023      +008
5885  1    ScoCD-2511131 141637183854                       154457.7-252650155058.7-254505346.10 21.71 4.64  -0.05 -0.73 -0.10   B3V                -0.018-0.024      +003V     300                 *
5886          BD+63 1225 141653 16848 587                   154508.4+625431154640.0+623558 96.42 44.46 5.19  +0.04 +0.10         A2IV               +0.039-0.055 +.018-006V                         *
5887          BD+55 1779 141675 29668        9793           154512.9+554058154738.0+552236 87.05 47.62 5.86  +0.25 +0.16 +0.08   A3m                +0.011+0.010D+.013-002SB     36  3.5   2.1      *
5888 34Ome SerBD+02 3007 141680121215    I                  154514.5+023006155017.5+021147 10.54 40.50 5.23  +1.02 +0.82 +0.53   G8III              +0.031-0.051 +.027-004V?   < 17                 *
5889 10Del CrBBD+26 2737 141714 840193252I         Del CrB  154523.9+262228154935.7+260406 41.88 50.37 4.63  +0.80 +0.37 +0.42   G3.5III-IVFe-1     -0.076-0.063 +.014-019     < 17
5890          CD-50 9939 141724243078                       154527.7-501853155251.5-503655329.80  2.54 6.60  +1.53               K3III              +0.015+0.011      -000
5891   Kap TrACP-68 2585 1417672533423253                   154536.4-681817155529.6-683611318.47-11.48 5.09  +1.13 +0.94         G5IIa              -0.012-0.003 +.007+006
5892 37Eps SerBD+04 3069 141795121218 588                   154549.8+044643155049.0+042840 13.15 41.66 3.71  +0.15 +0.11 +0.06   A2Vm               +0.128+0.063 +.041-009V      37                 *
5893          CD-2912030 141832183873        9813   7301    154602.2-293459155212.8-295312343.36 18.46 6.40  +0.98 +0.85         K1III              -0.135-0.089      -017           6.4  29.9AB   3*
5894          BD+15 2918 141850101771    I         R Ser    154605.0+152614155041.7+150801 26.23 46.76 5.2                       M7IIIe             -0.005-0.041      +024V                         *
5895 36    SerBD-02 4058 141851140801      S                154603.0-024716155115.6-030526  5.25 37.18 5.11  +0.12 +0.07         A3Vnp              -0.087-0.027 +.010-008V                0.1      *
5896          BD-13 4269 141853159544                       154603.2-134955155138.4-140801355.27 29.89 6.19  +1.00               G8III              -0.026-0.005      -022V
5897   Bet TrACP-63 3723 141891253346 589  W                154619.6-630719155508.5-632550321.85-07.52 2.85  +0.29 +0.05 +0.15   F2III              -0.191-0.398 +.083+000       92 11.0 155.
5898          CP-60 6191 141913253344      W        7310    154624.9-602644155452.6-604436323.56-05.45 6.15  +0.10               B9II               -0.009+0.003      -005           2.3  45.0AC   4*
5899 38Rho SerBD+21 2829 141992 84037    I          7300    154652.3+211642155115.9+205840 34.44 48.70 4.76  +1.54 +1.88 +0.85   K5-III             -0.053+0.019 +.015-062V?   < 17
5900          CP-59 6428 142049253349      W        7318    154709.4-595245155532.3-601040323.99-05.06 5.77  +0.35 +0.14         G5II-III+A3        -0.052-0.075D+.028           56  3.0   3.9      *
5901 11Kap CrBBD+36 2652 142091 649481414I W                154727.8+355803155113.9+353927 57.02 51.04 4.82  +1.00 +0.87 +0.49   K1IVa              -0.007-0.347 +.034-024     < 17  6.7 134.6
5902 45Lam LibBD-19 4249 1420961838951415                   154731.6-195205155320.1-201002350.72 25.38 5.03  -0.01 -0.56 -0.05   B2.5V              -0.011-0.023      +006SB    197  2.0   0.0      *
5903 16Zet UMiBD+78  527 142105  8328 590           7263    154737.2+780608154403.5+774740112.68 35.66 4.32  +0.04 +0.05 +0.01   A3Vn               +0.020-0.001 +.016-013V     206                 *
5904  2    ScoCD-2412352 142114183896        9823           154736.3-250143155336.7-251938346.88 21.61 4.59  -0.07 -0.65 -0.09   B2.5Vn             -0.016-0.025D+.006-010      308  2.5   2.3      *
5905          CP-60 6208 142139253353                       154740.1-601106155606.0-602858323.85-05.34 5.76  +0.06 +0.07         A3V                -0.034-0.081      -011V      96                 *
5906          CD-2412354 142165183900      D                154755.4-241407155353.8-243159347.51 22.15 5.39  -0.02 -0.40 -0.02   B6IVn              -0.024-0.023      +002V     273                 *
5907          CD-2312569 142184183901               7314    154758.6-234048155355.8-235841347.93 22.55 5.42  -0.04 -0.61 -0.05   B2.5Vne            -0.015-0.025      -009V     349                 *
5908 46The LibBD-16 4174 142198159563    I                  154807.8-162609155349.5-164346353.54 27.72 4.15  +1.02 +0.81 +0.52   G8.5IIIb           +0.099+0.133 +.036+003     < 19:
5909          BD+17 2926 142244101789                       154824.8+174204155256.2+172413 29.61 47.13 6.36R                     K0                 -0.041+0.002      -012
5910          CD-2611096 142250183907                       154824.6-270230155430.0-272019345.57 20.00 6.14  -0.07 -0.44         B6Vp               -0.025-0.023      +004V?    106                 *
5911 39    SerBD+13 3024 1422671017923254  W        7313    154832.5+133035155312.1+131148 24.07 45.40 6.10  +0.60  0.00 +0.33   G0VFe-0.5          -0.151-0.561 +.041+036SB         5.6  98.3
5912  3    ScoCD-2412365 142301183914              V927 Sco 154839.2-245650155439.5-251437347.12 21.51 5.87  -0.06 -0.58         B8IIIp             -0.013-0.022      -009V      58                 *
5913          BD+16 2840 142357101796        9828           154900.2+162220155334.9+160430 27.89 46.49 6.09R                     F5II-III           +0.027-0.020      +002SB     30  6.4   3.6      *
5914  1Chi HerBD+42 2648 142373 457721416                   154913.0+424353155240.5+422706 67.68 50.33 4.62  +0.56  0.00 +0.32   F8VFe-2Hdel -1     +0.440+0.632 +.056-055        0                 *
5915 47    LibBD-18 4195 142378159572        9834   7327    154913.5-190515155500.3-192259351.65 25.66 5.94  -0.01 -0.53 -0.03   B5V                -0.016-0.020      -007SB    240  2.0   0.7      *
5916          CD-3012663 142407207119                       154914.9-304724155530.4-310501343.07 17.09 6.21  +1.37               K4III              +0.014+0.024      +044
5917  4    ScoCD-2511190 142445183931                       154927.3-255816155530.1-261557346.52 20.64 5.62  +0.15 +0.19         A3V                -0.045-0.021      -029
5918          CD-3910237 142448207128      W                154925.1-393413155606.5-395151337.19 10.42 6.03  +0.15               B6V                -0.035-0.018      -018V          5.9  18.       *
5919 40    SerBD+09 3116 1425001212543256          FP Ser   154950.3+085229155440.3+083449 18.57 42.96 6.29  +0.18 +0.09         A7Vn               +0.002+0.002      -025V?    230                 *
5920          CP-64 3320 142514253362      W                154947.4-644450155858.1-650216321.11-09.01 5.75  -0.06 -0.40         B7III              -0.013-0.019      +000           6.6   9.7      *
5921          CD-4710456 142529226392                       154951.0-475202155703.8-480944331.90  3.99 6.31  +0.38 -0.03         F1IV               -0.109-0.095      +002
5922          BD+56 1838 142531 296983255                   154956.9+560719155216.6+554936 87.29 46.84 5.81  +0.96              gG8                 -0.023+0.057      -030V
5923          CD-3112407 142542207134                       154956.2-312935155613.9-314709342.70 16.47 6.29  +0.43               F6V                +0.036+0.001      +018
5924          BD+20 3166 142574 840703257I                  155010.1+203615155434.6+201839 33.84 47.76 5.44  +1.59 +1.94 +0.97   M0III              -0.082+0.043 +.023-061V?                        *
5925   Xi 1LupCD-3310826 142629207144      W     A          155029.7-334024155653.5-335759341.30 14.75 5.12  +0.12 +0.08         A3V                +0.018-0.042 +.025-010       73  0.5  10.3      *
5926   Xi 2LupCD-3310826 142630207145      W     B          155030.3-334017155654.2-335751341.30 14.75 5.62  +0.05 +0.06         B9V                +0.020-0.039 +.025-012      191  0.5  10.3      *
5927          BD-13 4290 142640159584                       155037.9-140619155614.4-142358355.92 28.90 6.37  +0.48 +0.08         F7V:               +0.031-0.073      -006V
5928  5Rho ScoCD-2811714 1426691839573258    9846           155042.5-285519155653.1-291251344.63 18.27 3.88  -0.20 -0.82 -0.20   B2IV-V             -0.013-0.025      -000V?    156  8.8  38.3      *
5929          CD-3510611 142691207152                       155051.1-355339155721.3-361106339.85 13.03 5.80  +1.01               K0-1III            -0.018-0.002      -005                          *
5930          BD-14 4314 142703159587      D                155055.6-143213155633.4-144945355.62 28.55 6.13  +0.23 -0.03         A2III              +0.069-0.027      +012                          *
5931          BD+19 3036 142763101821                       155111.1+185447155539.8+183714 31.61 46.96 6.26  -0.11 -0.43         B8III              -0.014-0.015      -042V      80                 *
5932  2    HerBD+43 2542 142780 45788    I          7335    155117.8+432547155437.9+430819 68.68 49.87 5.37  +1.65 +1.97 +1.25   M3IIIBa0.3         -0.032+0.065 +.034-010
5933 41Gam SerBD+16 2849 142860101826 591I W        7350    155150.0+155916155627.2+153942 27.75 45.71 3.85  +0.48 -0.03 +0.24   F6V                +0.312-1.281 +.069+007V?      8  6.6 201.5AB   3*
5934          BD-20 4364 142883183972               7359    155149.6-204135155740.4-205859350.88 24.09 5.85  +0.02 -0.43         B3V                -0.016-0.018      +001SB2    98                 *
5935          CD-3710620 142889207178                       155156.3-371253155830.7-373011339.14 11.89 6.31  +1.01               K0III              -0.025-0.002      -014V
5936 12Lam CrBBD+38 2712 142908 649743259  W                155209.2+381408155547.6+375649 60.60 50.10 5.45  +0.33 +0.03         F0IV               +0.031+0.083 +.044-012V      74  4.5  94.3      *
5937          CP-53 6911 142919243219                       155209.8-534402155954.0-540116328.43-00.76 6.10   0.00 -0.48         B5IV               -0.023-0.017      -038
5938  4    HerBD+42 2652 142926 45790              Var?     155208.5+425125155530.6+423358 67.77 49.78 5.75  -0.11 -0.41 -0.18   B9pe              v-0.025+0.015      -017SBO   350                 *
5939          CP-63 3765 142941253377              S TrA    155211.8-632928160110.7-634636322.13-08.22 6.41  +0.78 +0.54         F8II               -0.009-0.003      +005V                         *
5940   Phi SerBD+14 2969 142980101834    I                  155237.7+144204155714.6+142452 26.21 45.02 5.54  +1.14 +1.16 +0.56   K1IV               -0.121+0.087      -068
5941 48    LibBD-13 4302 1429831596071417          FX Lib   155235.2-135927155811.4-141646356.39 28.63 4.88  -0.10 -0.20 -0.03   B5IIIp            t-0.012-0.015      -006SB?   393                 *
5942          CD-2412427 142990183982              V913 Sco 155234.9-243235155834.8-244953348.12 21.20 5.43  -0.09 -0.64         B5IV              v-0.014-0.018      -011V     200                 *
5943          CD-4110478 1430092264251418                   155241.5-412727155930.3-414440336.44  8.58 4.99  +1.00               K0II-III           -0.040-0.018 +.019-027
5944  6Pi  ScoCD-2511228 143018183987 592    9862   7371    155248.0-254934155851.1-260651347.22 20.23 2.89  -0.19 -0.91 -0.20   B1V+B2V            -0.011-0.026 +.010-003SB2O  100: 1.2   0.0     3*
5945          CD-4010113 143084226431               7375    155312.4-402202155958.0-403910337.23  9.34 6.49  +1.52               K1II-III           +0.021+0.008      -026
5946          CP-54 6922 143101243246                       155318.7-541733160106.4-543440328.20-01.29 6.13  +0.25               A5V                -0.043-0.039      -032
5947 13Eps CrBBD+27 2558 143107 84098 593I   9859           155326.8+271002155735.3+265240 43.65 48.78 4.15  +1.23 +1.28 +0.62   K2IIIab            -0.076-0.061 +.024-031V?   < 17  7.3 101.4AC   3*
5948   Eta LupCD-3810797 143118207208      W     A          155329.5-380639160007.3-382349338.77 11.01 3.41  -0.22 -0.83 -0.21   B2.5IV             -0.022-0.032D+.008+008V     230  4.4  15.0AB   3*
5949          BD+59 1691 143187 29723                       155353.1+591200155549.7+585442 91.08 45.17 6.31  -0.02 -0.18         A0V                -0.024+0.022      -008V?    180
5950          BD+40 2948 143209 64988                       155358.5+395851155729.9+394143 63.29 49.68 6.31  +1.07               K0                 -0.063+0.060      -014                          *
5951          CP-62 5122 143238253390                       155405.0-621533160252.4-623230323.10-07.43 6.25  -0.04               B9.5V              -0.042-0.023      +004
5952          CD-4010120 143248226442      W                155408.7-400910160053.7-402607337.51  9.39 6.21  +0.01 -0.01         A0V                +0.014+0.037      -015           3.8   8.0ABxC 3
5953  7Del ScoBD-22 4068 143275184014 594I S                155425.1-222014160020.0-223718350.10 22.49 2.32  -0.12 -0.91 -0.13   B0.3IV             -0.012-0.022      -007SB    181  0.2   0.2 O   4*
5954 49    LibBD-16 4196 1433331596251419  W                155442.7-161420160019.6-163200354.92 26.72 5.47  +0.52 +0.03         F8V                -0.630-0.394 +.035-025SB1O       5.6 155.1      *
5955          CP-72 1902 1433462573573269                   155446.9-720730160555.8-722403316.51-14.88 5.70  +1.17 +1.26         K1IIICNII          -0.037+0.069      +049
5956          CD-3112470 143404207234    I                  155500.8-313625160119.5-315322343.46 15.66 6.33  +1.45               K4III              +0.010-0.004      +001
5957          BD+37 2695 143435 65001    I                  155516.0+365539155857.7+363838 58.57 49.49 5.62  +1.49       +0.83  gK5                 +0.022+0.029      +011V?
5958          BD+26 2765 143454 84129    I W       T CrB    155519.0+261213155930.2+255513 42.37 48.16 2.0 H +0.1        +1.56 sdBe+gM3+Q           -0.005+0.013      -029SBO              0.2      *
5959 50    LibBD-07 4162 1434591408971420                   155523.6-080743160047.6-082441  2.00 32.00 5.55  +0.05 -0.05         A0V s              -0.014-0.013      -019    =< 41
5960          BD+55 1793 143466 29727 595          CL Dra   155524.9+550156155747.4+544459 85.41 46.50 4.95  +0.26 +0.05 +0.14   F0IV               -0.150+0.110 +.022-011SB    140                 *
5961   Iot1NorCP-57 7500 143474243279      W                155523.7-572936160331.9-574631326.35-03.92 4.63  +0.24 +0.09         A7IV               -0.124-0.085 +.023-014V     139  0.2   0.5AB   3*
5962   Eta NorCD-4810512 1435462264663266                   155551.7-485702160312.9-491347331.97  2.52 4.65  +0.92 +0.64 +0.47   G8III              +0.043+0.010 +.022-000
5963          BD+04 3096 1435531213153262                   155553.2+044226160051.1+042539 14.86 39.52 5.83  +1.00 +0.79 +0.53  gK0                 -0.041+0.075      -004
5964          BD+50 2239 143584 297373260                   155614.0+500957155904.4+495252 78.49 47.77 6.05  +0.29 +0.03         F0IV               +0.023-0.041      +004V?     63
5965          CD-2811817 143619184049        9896           155627.7-285122160239.4-290809345.65 17.47 6.03  +1.31               K3III              +0.036-0.013      +005V?         6.8  11.0
5966  5    HerBD+18 3101 1436661018793263I                  155644.6+180541160114.3+174906 31.20 45.44 5.12  +0.99 +0.74 +0.50   G8IIIbFe-0.5       -0.048+0.157 -.007-019V    < 17
5967          CD-3810832 1436992072763267                   155645.0-381924160324.2-383609339.12 10.43 4.89  -0.14 -0.58 -0.14   B6IV               -0.016-0.027      -002V?    201                 *
5968 15Rho CrBBD+33 2663 143761 65024      W                155713.2+333619160102.7+331813 53.51 48.92 5.41  +0.60 +0.08 +0.32   G0+VaFe-1          -0.192-0.766 +.040+018V       7  3.2  89.6      *
5969          CD-2511295 143787184068    I                  155717.9-253511160320.6-255155348.17 19.70 5.00  +1.22 +1.31 +0.46E  K5III              -0.071-0.038 +.032-039                          *
5970          CD-3112505 143790207282                       155715.2-314320160334.3-320002343.74 15.25 6.01  +0.47               F4IV               -0.044-0.028      +028
5971 14Iot CrBBD+30 2738 143807 841523264           7396    155726.1+300751160126.6+295104 48.30 48.46 4.99  -0.07 -0.19 -0.08   A0p:Hg:            -0.039-0.009 +.010-019SB     12                 *
5972 44Pi  SerBD+23 2886 143894 841553268                   155759.2+230455160217.7+224816 38.14 46.78 4.83  +0.07 +0.06 +0.01   A3V                +0.002+0.028 +.012-028SB    110
5973          CD-2412499 143900184075    I                  155754.2-242700160354.7-244335349.12 20.42 6.21  +1.38               K0                 +0.001+0.009      +026V
5974          CD-3211386 143902207292                       155755.1-325612160417.8-331252343.00 14.26 6.10  +0.34               A9IV               -0.031-0.055      -028
5975          CD-3710680 143928207297      W                155800.6-373503160436.7-375147339.81 10.81 5.90  +0.40               F3IV               -0.137-0.117      -006           7.0  40.7
5976 43    SerBD+05 3131 144046121339      W                155848.9+051543160345.7+045912 15.97 39.20 6.08  +0.96 +0.75 +0.32E  G9III              -0.041+0.012      -044V?         7.8  30.8      *
5977   Xi  ScoBD-10 4237 144069159665        9909B          155852.0-110551160422.1-112223  0.02 29.42 5.07H                     F5IV               -0.060-0.029 +.048-029           0.2   1.2AB   5*
5978   Xi  ScoBD-10 4237 144070159665        9909A          155852.0-110551160422.1-112223  0.02 29.42 4.77H +0.47 +0.01 +0.24   F5IV               -0.060-0.029 +.048-029SBO    27  0.2   1.2AB   5*
5979          CP-55 7079 144183243339                       155923.2-555511160724.0-561129327.80-03.09 6.16  +0.58               F2II               -0.003-0.011      +002V
5980   Del NorCD-4410625 144197226500 596                   155925.3-445407160629.4-451024335.09  5.17 4.72  +0.23 +0.15 +0.12   Am                 +0.002+0.028 +.035-016        0
5981          BD+53 1834 144204 29763    I                  155932.3+531137160205.5+525457 82.59 46.48 5.93  +1.48              gK5                 -0.002-0.034      -007V?
5982  6Ups HerBD+46 2142 144206 458653271                   155940.9+461851160247.9+460212 72.69 47.98 4.76  -0.11 -0.32 -0.10   B9III              +0.059-0.058      +003        6                 *
5983          BD+37 2708 144208 65049      S                155938.7+365427160319.4+363754 58.57 48.61 5.83  +0.56               A2V+F7III          +0.010-0.017      -001SB2O   23                 *
5984  8Bet1ScoBD-19 4307 144217159682 597I   9913A  7424    155937.2-193154160526.2-194820353.19 23.60 2.62  -0.07 -0.87 -0.09   B1V                -0.006-0.019 +.009-001SBO   130  1.3   0.0 O   5*
5985  8Bet2ScoBD-19 4308 144218159683        9913C  7424    155937.6-193142160526.5-194807353.20 23.60 4.92  -0.02 -0.70         B2V                -0.019-0.020      -004V      74  1.3   0.0 O   5*
5986 13The DraBD+58 1608 144284 29765 598I                  160000.8+584956160153.3+583355 90.18 44.58 4.01  +0.52 +0.10 +0.25   F8IV               -0.320+0.335 +.051-009SB1O   27                 *
5987   The LupCD-3610642 144294207332 599                   160001.4-363148160635.5-364808340.83 11.32 4.23  -0.17 -0.70 -0.17   B2.5Vn             -0.017-0.029      +015      309                 *
5988          CD-2312700 144334184113              V929 Sco 160008.4-232002160606.3-233623350.35 20.85 5.92  -0.08 -0.55         B8p               v-0.013-0.028      -007       44                 *
5989          BD-05 4234 144362140945        9918           160024.2-060110160544.5-061730  4.86 32.37 6.53  +0.48 +0.07         F2IV               +0.037-0.008D+.012-011SB     45  0.0   0.1AP   5*
5990          BD-05 4235 144390140947      W                160040.3-055206160559.8-060822  5.05 32.41 6.41  +1.03 +0.80         K0                 -0.030+0.004      -032V?         5.8   9.3
5991          CD-3610648 144415207341      W                160041.5-362903160716.4-364520340.97 11.27 5.73  +0.29 +0.02         F1IV               +0.066-0.043      +026           6.0  40.7
5992          BD+08 3134 144426121361                       160047.4+082204160537.8+080546 19.77 40.35 6.29  +0.08 +0.11 +0.06   A3m                -0.009-0.002      -021SBO    39                 *
5993  9Ome1ScoBD-20 4405 144470184123                       160057.3-202354160648.4-204009352.75 22.77 3.96  -0.04 -0.81 -0.08   B1V                -0.009-0.021      -003V?    142                 *
5994   Iot2NorCP-57 7613 1444802433683273                   160105.1-573955160918.6-575604326.81-04.55 5.57  -0.04               B9.5V              -0.013-0.063      +000       96
5995          BD+59 1697 144542 29777    I                  160120.2+594107160309.3+592439 91.19 44.11 6.19  +1.58 +1.91 +0.81E  M1III              -0.023-0.022      -005                          *
5996          BD-13 4342 144585159706                       160128.7-134808160703.4-140415358.18 27.16 6.32  +0.66 +0.24         G4IV-V             -0.258+0.024      -016V
5997 10Ome2ScoBD-20 4408 144608184135    I          7454    160132.3-203555160724.3-205207352.70 22.54 4.32  +0.84 +0.50 +0.43   G3II-III           +0.041-0.038 +.021-005                          *
5998          CD-2412552 144661184142                       160151.6-241137160751.9-242743349.99 19.97 6.33  -0.06 -0.53         B7IIIp:            -0.011-0.007      -005V      36                 *
5999          CD-3810893 144668207367      W     C V856 Sco 160152.1-385013160834.2-390619339.52  9.38 7.05  +0.36 +0.25         A7IVe              -0.019-0.031      +008V          0.4  44.1AC   4*
6000          CD-3810894 144667207368      W     A          160152.4-384929160834.4-390535339.53  9.38 6.65  -0.07 -0.44         A0-3III            -0.027-0.035      -003     < 20  0.4  44.1AC   4*
6001          CD-2511369 144690184144    I                  160201.8-260332160807.6-261936348.63 18.62 5.38  +1.65 +2.01 +0.95E  M2III              +0.108-0.003      -018V
6002 11    ScoBD-12 4425 144708159715        9924           160203.1-122835160736.4-124444359.41 27.93 5.78  +0.01               B9.5Vnn           t-0.045-0.029      -025SB    239  4.3   3.5      *
6003          CD-2312731 1448441841643275           7470    160245.2-232507160843.7-234108350.73 20.37 5.88  +0.02 -0.31         B9IVp:             -0.009-0.025      -014SB2   180                 *
6004 45    SerBD+10 2958 1448741019393274                   160251.0+100933160737.5+095330 22.16 40.77 5.63  +0.20 +0.17         A7V                -0.021-0.004      -028SB    157
6005          BD+22 2926 144889 84202    I                  160302.1+220528160722.2+214921 37.26 45.38 6.14  +1.37 +1.51 +0.76   K4III              -0.013-0.041      +057
6006          CD-3211456 144927207396      W                160309.5-322259160931.7-323859344.22 13.91 6.19  +0.79 +0.46         K0-2III            -0.026-0.053      -005           0.8   7.7      *
6007          CD-3310961 1449872074033277                   160327.8-331649160952.6-333245343.63 13.22 5.54  -0.10               B8V                -0.023-0.042      -021SB    181                 *
6008  7Kap HerBD+17 2964 1450011019511421    9933A  7471    160333.6+171848160804.5+170249 31.02 43.64 5.00  +0.95 +0.61 +0.46   G8III              -0.032-0.007 +.002-011V?     10  1.2  28.1AB   3*
6009  7Kap HerBD+17 2965 145000101952    I   9938B          160334.0+171917160804.9+170316 31.03 43.64 6.25  +1.14 +1.11         K1III              -0.031-0.028 +.002+038V   =< 25  1.2  28.1AB   3*
6010 47    SerBD+08 3141 145002121383    I         FS Ser   160338.6+084800160828.0+083203 20.71 39.95 5.73  +1.61 +1.69         M3.5IIIa           -0.017-0.010      -022V?                        *
6011          BD+03 3132 145085121390    I                  160359.1+034308160858.9+032716 15.17 37.28 5.91  +1.47 +1.82        gK5                 -0.028+0.016      +009V?
6012          BD-17 4502 145100159745                       160409.6-180430160955.2-182027355.15 23.82 6.47  +0.44               F4V                -0.084-0.076      -025V
6013  8    HerBD+17 2967 145122101962               7477    160416.1+172816160846.6+171221 31.31 43.54 6.14   0.00 -0.07         A0Vnn              -0.021-0.026      -017V     275
6014          BD+06 3169 1451481213921422                   160415.7+063949160911.2+062244 18.40 38.75 5.97  +1.00 +0.81 +0.50   K1.5IV             +0.236-0.738 +.030-004V
6015          CD-4010251 1451912265643278                   160428.5-405117161117.7-410711338.51  7.55 5.86  +0.27               F0IV               -0.077-0.127      -014                          *
6016          BD-03 3884 145206141001    I                  160436.4-031213160950.5-032801  8.28 33.24 5.37  +1.45 +1.46 +0.81   K4III              -0.030-0.007 +.008-046SB                        *
6017          CD-2912343 145250184197    I                  160449.1-290908161102.0-292459346.81 15.99 5.13  +1.12 +1.02         K3III              -0.093-0.087 +.025-027
6018 16Tau CrBBD+36 2699 145328 651081423I   9939   7488    160518.8+364442160858.3+362927 58.39 47.47 4.76  +1.01 +0.86 +0.54   K1-III-IV          -0.053+0.333 +.031-019SB   < 17  8.3   2.2      *
6019   Zet NorCP-55 7229 145361243449                       160524.7-551653161322.6-553227328.86-03.19 5.81  +0.34               F2III              -0.105-0.043      -046
6020   Del1ApsCP-78 1092 1453662573801424  W     A Del1 Aps 160523.5-782637162020.8-784145312.42-19.89 4.68  +1.69 +1.69 +1.33E  M5IIIb             -0.014-0.035 +.020-012           0.4 102.9AB   3*
6021   Del2ApsCP-78 1093 145388257381      W     B          160530.7-782457162026.8-784002312.44-19.88 5.27  +1.41 +1.62 +0.56E  K3III              -0.007-0.025 +.020-010           0.4 102.9AB   3*
6022          CP-53 7413 145384243448               7527    160529.4-532445161316.8-534018330.13-01.82 5.83  +1.96 +2.21         M0Ib-II+F-G        -0.017-0.016      -012V?                        *
6023 11Phi HerBD+45 2376 145389 45911 601           7490    160537.0+451149160846.2+445606 70.85 47.11 4.26  -0.07 -0.28 -0.09   B9p:Mn:            -0.025+0.038 +.020-016SB1O   10                 *
6024   Kap NorCP-54 7245 145397243454 600  W                160535.3-542218161328.7-543750329.49-02.53 4.94  +1.04 +0.78         G8III              -0.002-0.024 +.022-014           7.9  14.9
6025          BD+68  864 145454 169623276                   160602.8+680425160619.7+674837101.15 40.09 5.44  -0.02 -0.08         A0Vn               -0.035+0.066 +.010-018V?    204                 *
6026 14Nu  ScoBD-19 4332 145501159763        9951CD         160609.8-191125161158.6-192659354.61 22.71 6.30  +0.13 -0.37         B8V+B9VpSi         -0.010-0.009 +.030-014       67  2.5  41.1AC   5*
6027 14Nu  ScoBD-19 4333 145502159764        9951AB         160610.9-191203161159.7-192738354.61 22.70 4.01  +0.04 -0.65 +0.03   B3V                -0.010-0.023 +.030+002SBO   199  2.5  41.1AC   5*
6028 13    ScoCD-2710841 145482184221                       160608.5-274001161218.2-275535348.12 16.84 4.59  -0.16 -0.74 -0.17   B2V                -0.013-0.025      +010V     225                 *
6029 12    ScoCD-2811962 145483184217        9953   7520    160605.1-280926161215.9-282503347.75 16.50 5.67  +0.01 -0.20         B9V               v-0.032-0.043D+.021-003V?         2.1   3.9      *
6030   Del TrACP-63 3854 145544253474 602  W                160619.9-632548161526.3-634108323.36-09.23 3.85  +1.11 +0.86 +0.39E  G2Ib-IIa           +0.002-0.011 +.029-005           8.0  30.
6031 15Psi ScoBD-09 4324 1455701410223280                   160631.9-094818161200.0-100351  2.54 28.81 4.94  +0.09 +0.11 +0.07   A3IV               -0.001-0.014 +.020-006V      33                 *
6032          BD+10 2971 145589121414      W                160642.7+095818161129.7+094245 22.53 39.84 6.53  +0.24 +0.10         F0IV               +0.014-0.005      -027           0.0   0.1      *
6033 16    ScoBD-08 4180 145607141024                       160642.1-081721161207.3-083251  3.92 29.73 5.43  +0.12 +0.13         A4V                +0.042+0.003 -.009+005V     201
6034          BD+77  616 145622  8417                       160649.3+770338160331.3+764737110.90 35.44 5.56R                     A3V                -0.021+0.016      -016SB     80
6035          BD+17 2982 1456471019943279                   160657.0+165528161128.7+163956 30.94 42.74 6.08  +0.02  0.00         A0V                +0.003+0.004      -017V      24
6036          BD+58 1622 145674 29823        9944           160705.1+581154160902.9+575616 88.89 43.94 6.33R                     A1V                -0.024+0.022      -010SB    195  5.4  12.3      *
6037          CP-67 3054 145689253481                       160706.8-674110161705.5-675629320.42-12.36 5.75  +0.15 +0.09         A4V                -0.046-0.094      -009       89
6038          BD+56 1867 145694 29825                       160713.8+560520160926.0+554944 86.09 44.58 6.49R                     K0                 -0.033+0.025      -015
6039 10    HerBD+23 2909 145713 84240    I         LQ Her   160722.5+234512161138.0+232941 39.93 44.90 5.70  +1.56 +1.56 +1.81   M4.5IIIa           -0.029-0.012      -025                          *
6040          CP-57 7716 145782243509                       160733.7-573927161549.7-575444327.46-05.13 5.63  +0.14               A3III              -0.022-0.048      -015                          *
6041          BD-03 3891 145788141031                       160740.9-035750161256.5-041315  8.10 32.17 6.25  +0.13 +0.05         A1V                -0.032-0.015      -016    =< 41
6042          CD-2412623 145792184241        9967           160744.5-240958161345.7-242519351.01 19.03 6.41  +0.04 -0.44         B5V                +0.013-0.013      +006V?     23  3.8   1.5      *
6043          BD+33 2696 145802 65129        9958           160750.5+333600161139.7+332033 53.83 46.72 6.29  +1.20               K2III              +0.015+0.010D+.009+000V?         4.3   5.4AB   3*
6044          CD-3211525 145838207480                       160758.2-324521161422.3-330041344.70 12.94 5.92  +1.02               K0III              -0.015-0.034      -023
6045   The NorCD-4710611 145842226600                       160759.5-470702161515.3-472220334.70  2.53 5.14  -0.13 -0.40         B8V                -0.032-0.051      +001      251                 *
6046          BD+36 2706 145849 65132    I W                160808.6+364059161148.0+362530 58.33 46.90 5.63  +1.34               K3III              -0.011-0.036      -031SB1O       0.1   0.1      *
6047  9    HerBD+05 3165 1458921214313283I                  160818.5+051636161315.4+050116 17.55 37.18 5.48  +1.47 +1.78 +0.77  gK5                 +0.040-0.007 +.014-002V?
6048 17Chi ScoBD-11 4096 145897159793    I                  160818.9-113457161350.9-115015  1.30 27.34 5.22  +1.42 +1.54 +0.71   K3III              -0.010-0.006 +.029-025V?   < 17                 *
6049          CD-4211132 145921226603                       160826.6-423847161524.0-425359337.83  5.74 6.14  +1.11               K2III              -0.026-0.009      +004
6050          BD+42 2683 145931 459573281I   9962           160828.9+423750161147.6+422228 67.03 46.82 5.87  +1.45 +1.74 +0.77   K4II+F6-8V         -0.006+0.027      -021V?         4.3  23.6      *
6051          BD-20 4444 145964184253                       160836.0-205111161428.8-210627353.72 21.16 6.41  -0.01               A0V                -0.027-0.026      -008      254
6052          BD+27 2603 145976 84247        9966           160837.5+265538161245.3+264015 44.39 45.43 6.50  +0.39 -0.03         F3V                +0.049-0.025D+.012-008       30  3.8   2.6
6053          BD-18 4249 145997159807      D                160852.9-181644161439.1-183207355.81 22.85 6.32  +1.09               K2III              -0.106-0.109      -017V?         0.3   0.0      *
6054          CD-2511453 146001184258                       160849.7-251324161453.6-252837350.39 18.12 6.05  +0.04 -0.36         B7IV               +0.007-0.019      -008V?    181                 *
6055          CP-53 7594 146003243526               7574    160853.5-533336161643.1-534840330.40-02.27 5.44  +1.73 +1.86 +1.02E  M2III              -0.006+0.004 +.020-028                          *
6056  1Del OphBD-03 3903 146051141052 603I W        7556    160906.2-032613161420.7-034140  8.85 32.20 2.74  +1.58 +1.96 +1.03   M0.5III           e-0.044-0.143 +.034-020V         10.4  65.5      *
6057          BD+06 3184 146084121443                       160918.5+060921161413.5+055407 18.66 37.42 6.31  +1.15 +1.17         K2III              +0.029-0.012      -021
6058   Gam1NorCD-4910474 146143226619                       160931.5-494905161700.9-500406333.04  0.39 4.99  +0.80 +0.51         F9Ia               -0.005-0.004 -.001-017
6059          CP-52 9469 146145243544      W                160935.8-525007161720.9-530512330.98-01.81 6.33  +0.27 +0.14         A8IV               -0.059-0.058      -024           6.6  21.0
6060 18    ScoBD-07 4242 146233141066      W        7577    161011.0-080617161537.3-082210  4.70 29.16 5.50  +0.65 +0.17 +0.33   G2Va               +0.234-0.501 +.062+011V       2  7.7  25.8      *
6061          BD-14 4383 1462541598213286           7578    161012.7-143555161551.5-145057359.07 25.04 6.09  +0.06  0.00         A0V                +0.009+0.004      -011V     106
6062          CP-57 7821 146323243586              S Nor    161034.4-573910161851.9-575359327.75-05.40 6.49  +1.00 +0.66 +0.57   F8-G0Ib             0.000-0.001      +005V                         *
6063 17Sig CrBBD+34 2750 146361 65165        9979A TZ CrB   161055.9+340642161440.8+335131 54.67 46.14 5.64  +0.51  0.00         G0VCaIIe           -0.271-0.079 +.045-011SB2O=< 25  1.0   6.4AB   4*
6064 17Sig CrBBD+34 2750 146362              9979B          161055.9+340642161440.8+335130 54.67 46.14 6.66H                     G1V                -0.271-0.079 +.045-017V?  =< 30  1.0   6.4AB   4*
6065 16    HerBD+19 3075 146388102040                       161102.7+190338161528.7+184830 34.13 42.62 5.69  +1.12 +1.01         K3III              -0.064-0.077 +.010-018V
6066          BD-20 4454 146416184285                       161105.1-210318161658.8-211814353.98 20.60 6.61  +0.02 -0.15         B9V                -0.015-0.016      -008V?    300:                *
6067          BD-03 3910 146514141075                       161139.6-034220161655.3-035712  9.03 31.52 6.18  +0.31 +0.02         A9Vn               +0.040+0.009      -008V?
6068          BD+27 2613 146537 84269                       161142.5+274020161547.4+272520 45.63 44.93 6.14  +1.31               K2                 -0.029-0.033      -011
6069          BD+67  930 146603 16993                       161202.5+672351161225.1+670839100.03 39.88 6.21  +0.99 +0.72         G8III              -0.023-0.043      -010
6070          CD-2812037 1466241843013288                   161205.6-282155161817.9-283650348.56 15.41 4.78  +0.02  0.00 -0.05   A0V                -0.028-0.099 +.031-013       35
6071   Lam NorCD-4211188 146667226650      W                161219.9-422544161917.7-424026338.50  5.40 5.45  +0.10               A3Vn               +0.016-0.011      -015V          0.8   0.3      *
6072   Gam2NorCD-4910536 146686243643 604I'W                161221.3-495437161950.4-500920333.30  0.01 4.02  +1.08 +1.16 +0.58   G8III              -0.155-0.053 +.044-029    =< 12  6.0  44.9      *
6073          CP-54 7493 146690243654                       161225.8-545346162025.2-550824329.86-03.59 5.77  +0.97               K0III              +0.019-0.021      -000
6074 18Ups CrBBD+29 2803 146738 842813287    9990   7596    161244.4+292351161644.8+290901 48.09 45.06 5.78  +0.07 +0.10  0.00   A3V                +0.023-0.016      +007SB?    80  4.3 123.4AD   5*
6075  2Eps OphBD-04 4086 146791141086 605I W                161301.7-042656161819.3-044133  8.56 30.81 3.24  +0.96 +0.75 +0.49   G9.5IIIbFe-0.5     +0.085+0.041 +.043-010V    < 17  9.1 110.6      *
6076          BD-19 4357 146834184309                       161316.2-195826161907.7-201304355.21 20.95 6.29  +1.08 +0.78 +0.62   K5III              +0.011-0.005      +008V                         *
6077          CD-3013041 1468362075581426  W     A          161313.1-303951161932.7-305424347.03 13.63 5.49  +0.47 -0.01         F6III              +0.085+0.023 +.030-008SB      0  1.6  23.2      *
6078          BD-14 4398 146850159846    I                  161321.5-143745161900.4-145221359.59 24.44 5.94  +1.52 +1.95         K4III              -0.028+0.015      -042SB
6079 19    UMiBD+76  594 146926  8446 606                   161340.1+760746161049.5+755239109.67 35.61 5.48  -0.11 -0.36         B8V                -0.001+0.013      -001V
6080          CD-3910412 146954207574      W                161346.7-391115162032.6-392551340.98  7.53 6.12  -0.07 -0.22         B9V                -0.012-0.064      +003           4.7  15.6
6081 19Omi ScoCD-2312849 147084184329    I          7632    161437.0-235542162038.2-241010352.33 18.05 4.55  +0.84 +0.62 +0.84   A5II              v-0.003-0.023 +.005-008        7                 *
6082 20    UMiBD+75  586 147142  8452                       161502.6+752731161232.2+751238108.91 35.88 6.39R                     K2IV               -0.041+0.034      -026V?
6083          CD-4910591 147152226693                       161459.5-492000162228.0-493420334.01  0.13 5.33  -0.04 -0.23         B6IV               -0.011-0.024      -010      190                 *
6084 20Sig ScoCD-2511485 147165184336 607I  10009  Sig Sco  161506.5-252110162111.3-253534351.31 17.00 2.89  +0.13 -0.70 +0.11   B1III              -0.010-0.021      +003SBO    53  2.0   0.0 O   4*
6085          CD-4310724 147225226696      W                161525.8-434027162228.9-435444338.03  4.11 5.88  +1.16 +0.83         G3II               -0.027-0.014      -011           3.8  40.7      *
6086          BD+60 1665 147232 298743291I         AT Dra   161535.1+595951161715.3+594518 90.74 42.32 5.40  +1.63 +1.67         M4IIIa             +0.007+0.028      -036V?                        *
6087          BD+21 2902 147266 84306                       161543.9+212226162004.3+210757 37.59 42.35 6.05  +0.94 +0.64         G8IIIb             -0.017-0.050      -025V?
6088          BD+73  713 147321  84623289                   161611.7+733822161433.5+732342106.90 36.71 5.98  +0.08 +0.15         A3V                -0.005+0.040      -015
6089          CP-62 5325 147349253539                       161616.7-625331162522.0-630729324.55-09.63 6.15  +0.03               A1V                -0.027+0.023      +031
6090          BD+49 2491 147352 460253292I                  161623.5+491639161911.2+490217 76.31 44.75 5.91  +1.37              gK6                 -0.023+0.033      -032
6091          BD+40 3005 147365 652333293  W                161629.5+395652161955.1+394231 63.12 45.37 5.46  +0.40 -0.08         F3IV-V             -0.132+0.004 +.043-029V      50  4.5   2.2      *
6092 22Tau HerBD+46 2169 147394 46028 608   10010   7641    161644.0+463305161944.4+461848 72.48 45.04 3.89  -0.15 -0.56 -0.17   B5IV               -0.011+0.040 +.030-014V?     32 10.7   6.7      *
6093 50Sig SerBD+01 3215 1474491215401427                   161700.4+011550162204.4+010145 14.78 33.22 4.82  +0.34 +0.04 +0.15   F0V                -0.155+0.050 +.043-050V?     80                 *
6094          CD-3810983 1475132076223295           7680    161714.7-385733162401.3-391135341.62  7.21 5.40  +0.62 +0.15         G5V                +0.075+0.001 +.054+010                          *
6095 20Gam HerBD+19 3086 147547102107 609I  10022   7667    161730.4+192316162155.2+190911 35.26 41.30 3.75  +0.27 +0.18 +0.14   A9III              -0.047+0.043 +.024-035SB    141  6.4  41.6AB   3*
6096          BD-01 3174 147550141129                       161727.4-015040162238.9-020447 11.79 31.41 6.23  +0.07 -0.02         B9V                -0.013+0.003      -016
6097          CD-3211687 147553207625      W     A          161730.4-325754162356.7-331158345.99 11.37 6.47  +0.01 -0.02         A0V+A0V            -0.016-0.031D+.009-002           0.5   6.3AB   3*
6098   Zet TrACP-69 2558 147584253554 610                   161742.5-695132162828.1-700504319.53-14.57 4.91  +0.55 +0.04 +0.28   F9V                +0.196+0.108 +.094+009SBO                       *
6099          CD-4510633 147614226730      W                161744.1-450700162454.2-452057337.31  2.80 6.33  +0.19               A2-3V              +0.024-0.001      +001V          6.0   5.3
6100          CD-3710778 147628207637      W                161751.5-371957162431.7-373357342.87  8.27 5.42  -0.11 -0.40         B8IV               -0.020-0.024      +009V     160: 0.1   0.1      *
6101          BD+68  868 147662 17036                       161811.8+684733161809.4+683316101.34 38.77 6.41R                     K0                 -0.044+0.047 -.004-011
6102   Gam ApsCP-78 1103 147675257407 611                   161806.0-784021163327.0-785350312.69-20.51 3.89  +0.91 +0.62         G8-K0III           -0.131-0.076 +.056+005SB
6103 19Xi  CrBBD+31 2845 147677 652543294I W                161812.0+310726162205.8+305331 50.77 44.21 4.85  +0.97 +0.80 +0.46   K0III              -0.096+0.109 +.016-029     < 17  7.4 185.1      *
6104  4Psi OphBD-19 4365 147700159892    I                  161815.0-194812162406.2-200215356.17 20.18 4.50  +1.01 +0.82 +0.50   K0II-III           -0.027-0.047 -.002+000     < 17
6105          CD-2912513 147722184368       10035B          161822.7-292807162439.6-294212348.72 13.65 6.63  +0.59 +0.11         G0IV               +0.063-0.102D+.036-000           0.8   5.2      *
6106          CD-2912513 147723184369       10035A          161822.8-292814162439.7-294217348.72 13.65 5.84  +0.59 +0.13         G0IV               +0.066-0.080D+.036-003           0.8   5.2      *
6107 20Nu 1CrBBD+34 2773 147749 65257    I W        7676    161835.5+340204162221.4+334757 54.82 44.55 5.20  +1.60 +1.94 +0.87E  M2IIIab            +0.007-0.037 +.003-013V          0.1 364.4AB   4
6108 21Nu 2CrBBD+34 2774 147767 65259    I                  161843.1+335610162229.2+334213 54.69 44.51 5.39  +1.53 +1.88 +0.87   K5III              -0.007+0.059 +.021-040V    < 17
6109   Iot TrACP-63 3923 147787253555      W                161839.7-634950162757.3-640329324.06-10.48 5.27  +0.36 -0.02         F4IV               +0.046+0.027 +.029-005SBO     0  4.1  19.6      *
6110          BD+32 2716 147835 652621428   10031           161906.1+323359162256.5+321959 52.80 44.25 6.40  +0.08 +0.11         A4Vn               +0.023-0.007      -001SB    232  2.9  32.4      *
6111 21    HerBD+07 3164 1478691215681429                   161918.5+071045162410.8+065653 21.30 35.77 5.85  -0.01 +0.01         A2pSr:             +0.003+0.014      -033SB1O                      *
6112  5Rho OphCD-2312861 147933184382       10049A          161935.2-231301162535.2-232650353.69 17.69 5.02  +0.24 -0.55 +0.31   B2IV               +0.002-0.018D+.013-010V     303  0.7   3.2AB   5*
6113  5Rho OphCD-2312861 147934184381       10049B          161935.2-231257162535.1-232646353.69 17.69 5.92H                     B2V                -0.007-0.017D+.013-010      300  0.7   3.2AB   5*
6114          CP-58 6800 1479772438363297                   161948.9-582220162815.2-583559328.12-06.78 5.69   0.00               B9II-III           -0.021-0.038      +002
6115   Eps NorCD-4710765 147971226773      W                161950.9-471936162711.1-473318336.00  0.98 4.47  -0.07 -0.53 -0.04   B4V                -0.005-0.030      -012SB2O  165  2.7  22.8      *
6116 21Eta UMiBD+76  596 148048  8470 612  W                162025.2+755909161730.3+754519109.27 35.33 4.95  +0.37 +0.08         F5V                -0.085+0.252 +.043-010V      76 10.8 227.       *
6117 24Ome HerBD+14 3049 148112102153 613   10054  Ome Her  162048.0+141548162525.0+140200 29.46 38.63 4.57   0.00 -0.04 -0.04   B9pCr              +0.044-0.059 +.041-006V      38  8.1  28.4AC   4*
6118  7Chi OphBD-18 4282 1481841599183298I         Chi Oph  162113.6-181346162701.4-182723357.93 20.68 4.42  +0.28 -0.75 +0.22   B2IV:pe           v-0.002-0.022      -003SB1O  134                 *
6119          BD+19 3098 148206102160    I         U Her    162122.2+190712162547.7+185333 35.35 40.35 6.7               +2.95   M7IIIe             +0.022-0.003 +.010-028V?                        *
6120          CP-57 8035 148218243874                       162124.5-573200162945.0-574521328.87-06.34 6.06  +1.48               G8Ib               -0.018+0.015      +005
6121          BD+11 2984 148228102165                       162129.2+113802162611.5+112427 26.53 37.36 6.11  +1.03               G8III              -0.031+0.014      -021
6122          CD-3610783 148247207688                       162134.6-365717162814.7-371046343.67  8.00 5.79  +1.10               K1IIICNII          +0.033-0.008      -014
6123 25    HerBD+37 2750 148283 652903296  S                162150.3+373718162524.2+372338 59.90 44.23 5.54  +0.17 +0.09         A5V                +0.006-0.012      -001V     242        0.2      *
6124          BD+02 3106 148287121604                       162147.7+023427162650.1+022051 16.87 32.91 6.07  +0.92 +0.61         G8III              +0.023-0.027      +004V?
6125          CP-61 5701 1482912535823302                   162156.0-612443163049.4-613801326.10-09.08 5.20  +1.23               K0II-IIICNIb-II    -0.013-0.013 +.006+004V?
6126          BD+69  845 148293 17062    I                  162202.2+692026162148.7+690634101.80 38.23 5.25  +1.12 +1.11 +0.53   K2III              -0.019-0.004 +.009-008V    < 19:
6127          BD+55 1845 148330 29931 614          DQ Dra   162214.0+552557162425.3+551218 84.45 42.72 5.74   0.00 +0.01         A2Si3955 Sr       v+0.012+0.023      -004V      10                 *
6128          BD-07 4292 148349141186    I         V2105 Oph162220.0-072210162743.5-073553  7.43 27.22 5.23  +1.72 +2.07 +1.29   M3-III             +0.014-0.155 +.008+099V?                        *
6129  3Ups OphBD-08 4243 1483671411873299  W        7738    162223.5-080853162748.1-082218  6.74 26.74 4.63  +0.17 +0.08 +0.09   A3m                -0.087+0.021 +.028-031SBO    44  3.2   1.1      *
6130          BD+62 1478 148374 17073       10052           162227.5+615526162347.0+614148 92.82 40.91 5.67  +0.96               G8III              -0.037+0.034D+.003-024V?         1.0   1.1      *
6131          CD-4510697 1483792268133301          QU Nor   162227.4-460116162942.3-461436337.25  1.58 5.35  +0.56 -0.44 +0.47   B1.5Iape           -0.001-0.005      -012V      66                 *
6132 14Eta DraBD+61 1591 148387 17074    I  10058   7713    162238.1+614426162359.5+613051 92.58 40.95 2.74  +0.91 +0.70 +0.46   G8-IIIab           -0.017+0.061 +.051-014SB?  < 17  4.8 564.9ABxC 3*
6133          CP-87  259 148451258754                       162251.2-872333171559.3-873359305.41-26.12 6.57  +0.91 +0.56         G5III              -0.119-0.155      -004
6134 21Alp ScoCD-2611359 148478184415 616I  10074  Alp Sco  162316.4-261236162924.4-262555351.95 15.06 0.96  +1.83 +1.34 +1.23   M1.5Iab-Ib+B4Ve    -0.010-0.020 +.024-003SB  =< 20  4.2   2.9      *
6135          CP-70 2256 1484882574093306                   162316.8-704619163419.3-705917319.16-15.53 5.50  +1.22 +1.24         K1IIICNIa-Ib       -0.017-0.013      -017
6136          BD+00 3529 1485131216233300I                  162328.2+005320162834.0+003954 15.44 31.66 5.39  +1.46 +1.80 +0.78   K4IIIp             +0.004-0.067 +.006+007V   =< 19:                *
6137          BD-07 4299 148515141195       10072           162324.6-075418162848.9-080744  7.13 26.69 6.48  +0.40 +0.02         F2V                -0.058-0.073      +000SB         3.1   5.4AB   3*
6138          CP-82  687 148527258748                       162330.8-830212164553.6-831420309.22-23.47 6.57 ?+1.57?              K4III              +0.009+0.013      -017
6139          CP-86  333 148542258751 921                   162334.7-861043170058.5-862152306.50-25.40 6.04  +0.05 +0.02         A2V                +0.005-0.001      +007       89
6140          BD-14 4433 1486041599481430                   162407.7-141953162946.9-143303  1.65 22.62 5.68  +0.82               G2-6III            +0.037+0.014      -031
6141 22    ScoCD-2412695 148605184429                       162407.8-245342163012.4-250654353.10 15.80 4.79  -0.11 -0.72 -0.14   B2V                -0.008-0.022      +004V     232                 *
6142          CD-4110695 148688226855      W        7781    162444.7-413600163141.7-414901340.72  4.35 5.33  +0.33 -0.77 +0.30   B1Iae             v-0.016 0.000      -014       93  4.3  58.0AC   3*
6143          CD-3411044 1487032077321431                   162450.7-342911163122.9-344216345.94  9.22 4.23  -0.16 -0.80 -0.15   B2III-IV           -0.009-0.017      +001SB     90                 *
6144          BD-07 4305 148743141206                       162506.6-071748163029.9-073054  7.96 26.71 6.50  +0.37 +0.20         A7Ib               -0.015-0.011      +002SB?    30                 *
6145          CD-2611379 148760184437                       162514.4-261912163122.8-263216352.17 14.66 6.10  +1.08               K1III              -0.029-0.034      +039V
6146 30    HerBD+42 2714 148783 461083303I         g Her    162521.4+420606162838.5+415254 66.16 43.72 5.04  +1.52 +1.17 +2.23   M6-III             +0.030-0.009 +.018+003                          *
6147  8Phi OphBD-16 4298 148786159963    I  10086           162524.8-162341163108.3-163646  0.13 21.08 4.28  +0.92 +0.72 +0.44   G8+IIIa            -0.051-0.035 +.014-034V    < 19: 6.8 120.0AC   3*
6148 27Bet HerBD+21 2934 148856 84411 618I W        7778    162555.2+214227163013.2+212923 39.01 40.21 2.77  +0.94 +0.69 +0.47   G7IIIa             -0.098-0.015 +.024-026SB1O < 19: 7.3 256.2      *
6149 10Lam OphBD+02 3118 148857121658       10087   7784    162552.1+021210163054.8+015902 17.12 31.85 3.82  +0.01 +0.01 -0.01   A0V+A4V            -0.027-0.073 +.010-014SB    127  1.0   1.2AB   4*
6150          BD+51 2106 148880 299563304   10079           162610.3+513735162843.4+512428 79.20 42.86 6.29  +1.05               G9III              +0.034+0.006      -016           6.6   5.9
6151   The TrACP-65 3331 1488902536143312                   162606.8-651701163544.8-652943323.54-12.06 5.52  +0.93 +0.73         G8-K0III           +0.032-0.029 +.014+010
6152          BD+20 3283 148897 84416    I          7785    162613.0+204152163033.6+202845 37.79 39.82 5.25  +1.29 +1.22 +0.70   G8IIICN-2CH-1      -0.078-0.063 +.000+019V?   < 17                 *
6153  9Ome OphBD-21 4381 1488981844503307          Ome Oph  162612.4-211509163208.2-212759356.30 17.83 4.45  +0.13 +0.13 +0.02   A7p                +0.028+0.039 +.028+003       41                 *
6154          BD+22 2983 149009 84423    I                  162656.6+222437163113.4+221143 39.98 40.21 5.76  +1.61              gK5                 -0.012+0.003      -026V
6155   Mu  NorCD-4310900 1490382269003311          Mu  Nor  162658.5-435000163405.0-440243339.38  2.51 4.94  +0.05 -0.94         B0Ia               +0.005-0.004      +009V?    147                 *
6156 34    HerBD+49 2514 149081 46128                       162721.2+491043163006.0+485739 75.86 42.99 6.45  +0.03  0.00         A1V                -0.048-0.062      -008       59
6157          BD+35 2828 149084 653563308I                  162722.8+352625163102.8+351330 57.05 42.92 6.25  +1.64 +1.89         K5                 -0.008-0.016      +024
6158 28    HerBD+05 3223 1491211216763309                   162740.3+054403163235.7+053116 21.01 33.25 5.63  -0.06 -0.18         B9.5III            +0.015+0.001      -027        5
6159 29    HerBD+11 3008 1491611022343310I          7812    162755.4+114210163236.3+112917 27.46 35.97 4.84  +1.49 +1.83 +0.86   K7III              -0.180-0.078 -.012+003     < 19:                *
6160          CD-4410964 149174226926               7829    162756.3-450159163507.9-451441338.62  1.56 6.46  +1.34               K2-3III            -0.022-0.073      -056                          *
6161 15    DraBD+69  850 149212 17107 619                   162810.5+685904162759.0+684605101.11 37.87 5.00  -0.06 -0.12 -0.02   A0III              -0.025+0.036 +.014-007V     160
6162          BD+45 2422 149303 46147       10105           162846.5+454835163147.3+453554 71.24 43.01 5.65  +0.12 +0.13         A4Vn               -0.007+0.043D+.022-016V          3.3  16.3
6163   Bet ApsCP-77 1221 1493242574243319  W                162847.1-771829164304.6-773103314.18-20.06 4.24  +1.06 +0.95 +0.37E  K0III              -0.287-0.350 +.034-030           7.8  51.1      *
6164          CD-4211399 149404226953              V918 Sco 162920.4-423908163622.5-425132340.54  3.01 5.47  +0.40 -0.63 +0.35   O9Ia              e-0.006-0.009      -036SBO   140                 *
6165 23Tau ScoCD-2711015 149438184481 620                   162939.3-280031163553.0-281258351.54 12.81 2.82  -0.25 -1.03 -0.25   B0V                -0.008-0.022 +.020+002V      24                 *
6166          CD-3411112 149447207814    I          7844    162947.5-350258163622.5-351520346.22  8.09 4.16  +1.57 +1.94 +1.00   K6III              +0.022 0.000 +.022-002V?
6167          CP-60 6594 149485253638                       163001.8-604711163852.7-605925327.24-09.37 6.18  -0.08 -0.39         B7Vn               -0.005-0.023      -007      375
6168 35Sig HerBD+42 2724 149630 46161 621  S        7837    163052.7+423836163406.2+422613 66.91 42.70 4.20  -0.01 -0.10 -0.01   B9V                -0.011+0.046 +.010-011      270  3.    0.1      *
6169          BD+17 3053 1496321022593313  W                163057.3+171548163526.3+170326 34.18 37.56 6.41  +0.05  0.00         A2V                -0.007-0.004      -009SB2O   71  1.0 156.6      *
6170          BD+61 1598 149650 171301432                   163100.7+610158163225.7+604924 91.30 40.20 5.94  +0.02 +0.05         A2V                +0.022-0.010      -009V      80
6171 12    OphBD-02 4211 1496611412691433          V2133 Oph163106.2-020640163621.5-021929 13.73 28.42 5.75  +0.82 +0.48 +0.39   K2V                +0.458-0.310 +.093-015V                         *
6172   Eta1TrACP-68 2789 149671253649                       163103.9-680548164123.3-681746321.70-14.29 5.91  -0.08 -0.42         B7IVe              -0.006-0.012      -043SB?
6173          BD+79  498 149681  85273305                   163117.3+791038162543.1+785750112.31 33.28 5.56  +0.26 +0.05         F0V                -0.115+0.108 +.028-012
6174          CD-4310959 149711226989      W                163121.6-431145163826.2-432355340.39  2.37 5.83  -0.02 -0.59         B2.5IV             -0.022-0.027      +001           3.2  16.3      *
6175 13Zet OphBD-10 4350 149757160006 622I         Zet Oph  163139.0-102153163709.5-103402  6.28 23.59 2.56  +0.02 -0.86 -0.04   O9.5Vn            t+0.014+0.026 +.003-015V     379                 *
6176          BD+15 3029 149822102271              V773 Her 163210.4+154205163643.0+152953 32.51 36.69 6.30  -0.11 -0.18         B9pSiCrSr:         -0.009-0.008      +000       85                 *
6177          CP-60 6603 149837253651      W                163203.0-601444164050.5-602646327.81-09.20 6.18  +0.48               F2III-IV           +0.051-0.070      +004           2.7   1.4
6178          CD-3610879 1498862078783317  W                163223.5-370058163905.2-371303345.10  6.38 5.91   0.00 -0.12         B9-A0V+B           -0.012-0.047      +003           0.0   0.1AB   3*
6179          BD-06 4467 1499111412843315          Var      163240.1-062012163801.6-063217 10.04 25.74 6.09  +0.16 +0.13         A0pCrEu           v-0.007-0.008      -022                          *
6180          BD+72  734 150010  8544                       163300.0+724908163128.1+723643105.32 35.99 6.30  +1.32 +1.29         K2III              -0.043+0.041 +.011-033
6181          BD+13 3177 1500121022783316                   163311.4+135322163748.0+134113 30.58 35.73 6.31  +0.41 +0.02         F5IV               -0.035-0.063      -021       30                 *
6182          CP-67 3196 150026253673                       163314.5-671414164322.1-672557322.52-13.89 6.03  +0.02 +0.02         A0Vn               +0.012-0.037      -002
6183          BD+46 2194 150030 461843314                   163316.1+464856163611.2+463648 72.55 42.19 5.79  +1.04 +0.88?        G8II               -0.011+0.004      -015
6184 16    DraBD+53 1875 150100 30012       10129C          163349.4+530604163611.5+525401 80.94 41.48 5.53  -0.07 -0.15         B9.5Vn             -0.009+0.033D+.010-009V      83  0.0  90.3AC   4*
6185 17    DraBD+53 1876 150117 30013       10129A          163351.9+530731163613.7+525528 80.97 41.48 5.08  -0.04 -0.15         B9V                -0.009+0.027D+.010-010V     216  0.0  90.3AC   4*
6186 17    DraBD+53 1876 150118             10129B          163351.2+530730163614.1+525527 80.97 41.48 6.53  -0.06 -0.16         A1Vnn              -0.009+0.027D+.010-018      250  0.0  90.3AC   4*
6187          CD-4811070 150136227049      W     A          163350.8-483401164120.3-484547336.71-01.57 5.65  +0.13 -0.80         O5III(f)           -0.008-0.003D+.007+023SB    152  1.2   9.6AC   6*
6188          CD-4910890 1501682270583321                   163406.0-492722164140.2-493906336.08-02.20 5.65  -0.03 -0.87         B1Iab-Ib           -0.001-0.007      +006SB    154                 *
6189          BD-09 4430 150177141298                       163410.8-092111163939.1-093316  7.57 23.68 6.35  +0.48 -0.07         F3V                +0.007-0.145      -019V   =< 15
6190          BD-20 4537 150259184541    I:                 163440.7-201249164034.5-202431358.46 16.97 6.26  +1.08              gG9                 +0.012+0.034      +033V?
6191          BD+77  627 150275  8548 623                   163456.2+773844163038.8+772648110.58 33.82 6.34  +1.00 +0.71         K1III              -0.104+0.276 +.017-032V?                        *
6192          CD-3211913 150331207930                       163517.0-325701164145.5-330847348.57  8.64 5.87  +0.65 +0.24         G3III              -0.059-0.083      -008
6193          CD-2412765 1503661845493322                   163532.4-241627164136.2-242805355.34 14.23 6.09  +0.20 +0.06 +0.10   F0V                -0.049-0.008      -047       34
6194 36    HerBD+04 3234 150379121774       10149B          163537.2+042408164035.1+041226 20.79 30.87 6.93  +0.13 +0.12         A3IV               -0.002-0.011 +.001-028V?     80  1.2  69.6AB   4*
6195 37    HerBD+04 3235 150378121776       10149A          163540.7+042453164038.7+041311 20.82 30.87 5.77  -0.02 -0.03         A1V                -0.001-0.013 +.001-034      140  1.2  69.6AB   4*
6196          BD-17 4618 150416160046 624I                  163547.3-173255164134.4-174432  0.82 18.43 4.96  +1.11 +0.87 +0.60   G7.5IICN1Ba0.4     -0.020-0.001 +.047-025     < 19:                *
6197          CD-4510858 150421227092                       163546.3-455242164303.4-460414338.93-00.02 6.23  +0.85 +0.59         F5Iab              -0.015-0.018      -016                          *
6198          BD+63 1289 150429 17165    I                  163554.4+631626163655.0+630422 93.89 39.03 6.16  +1.53               K5                 +0.004-0.087      -042
6199          BD+56 1907 150449 300263320I                  163559.1+561239163800.4+560056 84.94 40.67 5.29  +1.08 +0.90 +0.52   K1III               0.000+0.066 +.015-019V?   < 17
6200 42    HerBD+49 2531 150450 462101434I  10144   7896    163601.9+490726163844.9+485542 75.61 41.58 4.90  +1.55 +1.76 +0.90E  M2.5IIIab          -0.047+0.031 +.017-055           5.4  25.6      *
6201          BD-00 3168 150451141310                       163602.1-004823164111.5-010002 15.73 28.10 6.24  +0.30 +0.07 +0.18   A7III              +0.006-0.015      -012       64
6202          BD-19 4406 150453160052    I:W                163600.9-194358164153.7-195528359.06 17.02 5.57  +0.43               F4IV-III           +0.022+0.041      +005           8.0  22.5
6203          BD+12 3063 150483102314                       163611.9+123520164051.4+122342 29.51 34.53 6.08  +0.05 +0.03         A3Vn               -0.026-0.010      -032V?    230
6204          CP-66 3009 150549253688      W       LP TrA   163635.9-665521164640.0-670635322.98-13.93 5.13  -0.08 -0.45         ApSi               -0.014-0.015 +.005-002       56  6.2  25. AB   3*
6205 14    OphBD+01 3290 1505571217903324                   163638.5+012220164142.5+011052 17.93 29.12 5.74  +0.32 +0.07         F2-4III-IV         -0.097+0.051 +.030-045       67                 *
6206          CD-4010649 150573227118      W                163649.2-405543164345.5-410708342.75  3.13 6.20  +0.14 +0.17         A4V                -0.046-0.016      -026           0.1  95.7      *
6207          CP-5210161 1505762441063327  W                163645.3-525747164439.8-530909333.74-04.85 5.96  +1.26               G8III              -0.009-0.018      -002V          6.0  40.
6208          BD+25 3115 150580 845363323I                  163651.7+250306164100.6+245131 44.12 38.82 6.06  +1.31               K2                 -0.034 0.000      -068
6209          CD-4010653 150591227123      W                163657.4-405524164353.9-410647342.77  3.12 6.12  -0.07 -0.52         B6-7V              -0.026-0.013      -002           0.1  95.7      *
6210          CD-3710942 150608207966      W                163701.9-375756164347.6-380924345.01  5.06 6.05  -0.06               B9II-III           -0.009-0.049      +007           6.2  50.0
6211          CD-3113161 1506382079673325                   163712.8-315457164338.7-320622349.63  9.01 6.46  -0.08               B8V                -0.002-0.023      -003V?
6212 40Zet HerBD+31 2884 150680 65485    I  10157   7915    163730.9+314701164117.2+313611 52.66 40.28 2.81  +0.65 +0.21 +0.32   G0IV               -0.470+0.394 +.102-070SB1O=< 10  2.6   1.1      *
6213 39    HerBD+27 2668 150682 84543      S                163733.2+270634164136.7+265501 46.72 39.22 5.92  +0.40 -0.07         F2III              +0.002-0.044      -012SB2O             0.1      *
6214          CD-4010661 1507422271463329  W                163746.9-403905164442.6-405023343.08  3.18 5.71  -0.12 -0.63         B3V                -0.010-0.024      +012           6.3   8.       *
6215          CP-58 6889 150745244122                       163748.9-581903164621.2-583013329.77-08.48 5.74  -0.09               B2IV-V             -0.008-0.016      -016      219
6216          CD-2711103 150768184591       10173   7935    163804.9-271606164417.3-272722353.35 11.87 6.58  +0.10               A2V                -0.019-0.007D+.012-026           4.0   2.0AB   3*
6217   Alp TrACP-68 2822 150798253700 625                   163804.3-685038164839.9-690140321.54-15.26 1.92  +1.44 +1.56         K2IIb-IIIa         +0.014-0.034 +.031-003
6218          CD-2812358 150894184602      W        7944    163844.8-281924164500.2-283035352.63 11.08 6.02  +0.09               A3IV               -0.028+0.001      -055           7.7   5.6
6219          CP-58 6893 150898244133                       163848.4-580927164719.5-582029329.98-08.47 5.58  -0.08 -1.00         B0.5Ia             -0.019-0.020      -053V     128                 *
6220 44Eta HerBD+39 3029 150997 65504 626I W        7934    163928.0+390644164253.8+385520 62.28 40.90 3.53  +0.92 +0.60 +0.48   G7.5IIIbFe-1       +0.037-0.083 +.034+008V?      8      113.5      *
6221          CD-3910677 151078208020                       163957.4-391137164647.8-392238344.46  3.82 5.48  +0.98               K0III              -0.032-0.034      -009
6222          BD+34 2830 151087 655123330                   164010.2+341323164351.7+340220 55.92 40.16 5.99  +0.29 +0.06         F2-3III-IV         -0.071+0.058      -010V?
6223 18    DraBD+64 1145 151101 171883326I                  164013.4+644643164055.1+643521 95.57 38.16 4.83  +1.22 +1.26 +0.61   K0III*             +0.001-0.013 +.009+000     < 19:                *
6224 16    OphBD+01 3298 151133121834                       164024.7+011213164529.7+010113 18.32 28.22 6.03  -0.02 -0.14         B9.5III            +0.002+0.013      -013V     100
6225 25    ScoCD-2511667 151179184630                       164043.9-252047164651.3-253143355.26 12.64 6.71  +1.18              gG6                 -0.005-0.014      +002V
6226          BD+55 1872 151199 30062               7945    164055.7+555226164258.4+554125 84.35 40.04 6.16  +0.07 +0.11         A2VpSr             +0.049+0.085      -045V     110
6227          BD+16 3013 151203102365    I          7950    164050.7+155548164522.5+154443 33.78 34.86 5.56  +1.66               M3IIIab            +0.027-0.043 +.023-019V?
6228 43    HerBD+08 3271 1512171218433332I W        7952    164101.7+084553164549.9+083457 26.03 31.77 5.15  +1.53 +1.94 +0.90   K5III               0.000+0.015 +.010-021V    < 17  3.9  82.5AB   3
6229   Eta AraCP-58 6906 1512492441681435  W                164108.8-585146164947.1-590229329.64-09.16 3.76  +1.57 +1.94         K5III              +0.038-0.028 +.025+009           9.5  23.6
6230          BD+43 2642 151388 46262    I                  164203.5+432402164511.7+431302 67.98 40.67 6.05  +1.40               K4III              -0.015-0.045      -013SB
6231          CP-67 3232 151404253718                       164203.3-673022165217.4-674055322.86-14.71 6.32  +1.28 +1.51         K2III              -0.073-0.064      +008
6232 19    OphBD+02 3175 1514311218591436   10207   7962    164207.2+021441164709.8+020352 19.58 28.39 6.10  +0.14 +0.14         A3V                -0.010-0.014      -006SB2        3.6  23.4AB   3
6233          CP-65 3365 1514412537173337                   164210.7-651203165153.9-652232324.72-13.28 6.13  -0.02 -0.28         B8II-III           -0.004-0.014      -010
6234 45    HerBD+05 3272 1515251218653333  W       V776 Her 164250.9+052534164746.4+051448 22.85 29.80 5.24  -0.02 -0.02         B9pCr:             -0.016-0.041 +.014-016SB     42  5.2 122.7      *
6235          BD-14 4486 1515271601043335                   164246.1-144354164827.0-145434  4.25 18.81 6.03  +0.20               A0Vn               -0.011-0.004      -026V     139                 *
6236          CD-4910998 151566227281      W                164257.7-495212165035.9-500244336.71-03.55 6.47  +0.31 +0.07         A5III+F7III        -0.018-0.023D+.008+025           0.1   3.2      *
6237          BD+57 1702 151613 30076 627  S                164323.9+565738164517.8+564655 85.67 39.53 4.85  +0.38 -0.06 +0.21   F2V                +0.018+0.066 +.046-004SBO    53                 *
6238          BD+79  511 151623  8592                       164333.7+790622163752.9+785506111.93 32.79 6.32  +1.14 +1.20 +0.58   G9III              -0.020+0.038 +.047-020
6239          BD+13 3225 151627102393                       164332.2+134604164808.9+133526 31.70 33.39 6.35  +0.87               G5III              +0.016-0.016      +001     < 20
6240          BD-15 4395 151676160116              V1010 Oph164344.9-152935164927.8-154003  3.76 18.16 6.10                      A5V                +0.012+0.031      -002SBO   155                 *
6241 26Eps ScoCD-3411285 151680208078 628           7983    164341.1-340642165009.8-341736348.82  6.56 2.29  +1.15 +1.27 +0.60   K2.5III            -0.611-0.255 +.022-003
6242          BD+42 2749 151732 462883336I         V636 Her 164407.7+422501164719.8+421420 66.70 40.25 5.87  +1.61               M4+III-IIIa        +0.003-0.026      -007V?                        *
6243 20    OphBD-10 4394 1517691601181438                   164418.0-103622164950.0-104659  8.04 20.93 4.65  +0.47 +0.07 +0.25   F7IV               +0.096-0.093 +.016-001SB1O   13                 *
6244          CD-3711023 151771208089      W       Var?     164415.4-372028165059.8-373052346.42  4.39 6.11  +0.12 -0.14         B9pSi*             -0.029-0.009D+.006-017SB?        2.2   6.7      *
6245          CD-4110957 151804227313               7992    164435.2-410331165133.7-411350343.62  1.94 5.22  +0.07 -0.84         O8Iaf              +0.001-0.001      -065V?    124                 *
6246          BD+13 3233 151862102410       10225           164457.5+132608164934.6+131541 31.51 32.94 5.91  +0.03 -0.02         A1V                -0.029-0.021D+.013-023V?     74  4.0   5.3AB   4
6247   Mu 1ScoCD-3711033 1518902081021439          Mu1 Sco  164505.7-375233165152.2-380251346.12  3.91 3.08  -0.20 -0.87 -0.19   B1.5V+B6.5V        -0.011-0.025      -025SB2O  239  0.4 390.       *
6248          BD-02 4259 151900141393                       164509.1-022850165022.3-023914 15.51 25.28 6.32  +0.42 -0.05         F1III-IV           +0.009-0.030      +031       40
6249          CD-4110972 151932227328    I:        V919 Sco 164518.2-414103165219.1-415116343.22  1.43 6.49  +0.23 -0.65 +0.39   WN7-A              -0.018-0.001      +025V                         *
6250 47    HerBD+07 3256 151956121895               7990    164528.0+072513165019.4+071452 25.23 30.18 5.49  +0.10 +0.11 +0.05   A3m                +0.053-0.011 +.025-004SB?    26                 *
6251          CP-57 8157 151967244245                       164530.9-574417165400.5-575434330.88-08.89 5.94  +1.59 +1.92         M1III              -0.032-0.126      -041
6252   Mu 2ScoCD-3711037 151985208116                       164533.6-375049165220.1-380103346.20  3.86 3.57  -0.21 -0.85 -0.22   B2IV               -0.011-0.024      +001V?     57  0.4 390.       *
6253          CP-63 4032 152082253734      W                164604.7-630612165524.7-631611326.67-12.31 6.02  +0.05               A0III              +0.008-0.023      -009           6.9   7.       *
6254 52    HerBD+46 2220 152107 46305       10227  V637 Her 164618.5+460927164914.2+455900 71.61 39.96 4.82  +0.09 +0.04 +0.01   A2VpSrCrEu        v+0.026-0.055 +.010-001V      31  4.6   1.8AxBC 5*
6255 21    OphBD+01 3323 152127121911       10230           164620.5+012311165124.9+011258 19.35 27.04 5.51  +0.05 +0.03         A2V s              -0.023-0.005 +.021-026V      59  1.5   0.4      *
6256          BD+43 2654 152153 46311                       164634.0+433609164940.5+432550 68.28 39.86 6.13  +1.26 +1.31         K0IV               -0.017-0.007      -020
6257          CD-4211627 152161227359               8012    164636.4-425259165342.5-430303342.46  0.48 5.96  +1.73               M3II-III           -0.016-0.027      -012V?                        *
6258 50    HerBD+30 2884 152173 846413339I                  164644.7+295837165039.0+294824 50.91 37.96 5.72  +1.60 +1.92         M1IIIa             -0.005 0.000      -010V
6259          BD+32 2795 152224 65595                       164657.0+324321165043.1+323313 54.34 38.50 6.13  +1.01 +0.78         K0III              +0.019+0.046      -030SB
6260          CD-4111024 152234227377      W                164700.7-413824165401.8-414823343.46  1.22 5.45  +0.19 -0.72         B0.5Ia             -0.008-0.008      -006SB     99  1.5   0.4AB   8*
6261          CD-4111021 152235227374              V900 Sco 164657.0-414940165358.8-415941343.31  1.10 6.32  +0.50 -0.45         B1Ia              e-0.002-0.018      -036                          *
6262   Zet1ScoCD-4211633 152236227375              Zet1 Sco 164656.3-421144165359.7-422144343.03  0.87 4.73  +0.49 -0.56         B1Iape            v+0.004-0.004      -026V      57                 *
6263          CD-4111036 152249227383               8023    164710.3-414100165411.8-415101343.45  1.16 6.45  +0.20 -0.74         O9Ib               +0.012-0.039      -024SB2O  100                 *
6264          BD+42 2753 152262 46317                       164723.8+420352165036.1+415348 66.29 39.63 6.29  +1.08              gK3                 -0.074+0.070      -037
6265          CD-4111041 152270227390      W       Var?     164718.5-413915165419.6-414912343.49  1.16 6.59  +0.21 -0.54 +0.33   WC7+O5-8           -0.011-0.011      -044SB2O       6.6   4.3      *
6266          CD-4211642 152293227392                       164723.0-421848165426.9-422844342.99  0.73 5.88  +0.64 +0.22         F5Ib-II            -0.003-0.005      -024V                         *
6267          BD+77  634 152303  8612       10214           164732.8+774111164306.0+773051110.29 33.19 5.98  +0.42 -0.05         F4V                +0.048+0.212 +.030+007V?         3.3   2.6AB   3*
6268 49    HerBD+15 3066 152308102435 629          V823 Her 164731.6+150831165204.9+145827 33.67 33.07 6.52  -0.05 -0.07         B9.5pCr:           +0.017+0.004      -023V?
6269          BD-20 4572 152311184754                       164730.9-201454165325.2-202456  0.37 14.58 5.88  +0.68               G3V                -0.050-0.029      -017V?
6270 51    HerBD+24 3069 152326 846511440I                  164736.5+244928165145.3+243923 44.73 36.42 5.04  +1.25 +1.29 +0.61   K0.5IIIaCa0.5      +0.011+0.006 +.010-016
6271   Zet2ScoCD-4211646 152334227402               8028    164732.7-421124165435.0-422141343.10  0.79 3.62  +1.37 +1.65 +0.68   K4III              -0.124-0.236 +.028-019                          *
6272          CD-4010919 152408227425      W        8031    164759.8-405912165458.4-410904344.08  1.49 5.77  +0.15 -0.75         O8:Iafpe           -0.017-0.013      -140V     140: 7.0   5.4      *
6273          CD-3013594 152431208176                       164813.0-302523165436.0-303514352.31  8.16 6.35  +0.21 +0.17 +0.11   A7IIIm:            +0.039+0.002      +001V?
6274          CD-5010905 152478244280               8047    164825.9-503044165608.9-504030336.78-04.64 6.33  -0.02               B3Vnep             -0.006-0.019      +019V?                        *
6275          CP-5210333 152527244285                       164836.8-520716165628.7-521702335.55-05.68 5.94  -0.08               B9V                -0.045-0.046      -006
6276          CP-69 2666 152564253756                       164849.5-690636165933.8-691606321.93-16.18 5.79  -0.10 -0.44         ApSi               -0.022-0.016      -001
6277          BD-01 3268 152569141427      W                164859.7-012647165410.6-013644 17.04 25.01 6.25  +0.28 +0.11         F0V                +0.015-0.071      -020      176  7.0  15.0AB   3*
6278          BD-11 4231 152585160159                       164906.2-113744165440.3-114733  7.87 19.37 6.57  +0.14               A2IV              e+0.022-0.013      -005                          *
6279 53    HerBD+31 2925 152598 656271441  W                164910.4+315201165258.1+314206 53.39 37.87 5.32  +0.29 -0.02         F0-2V              -0.091-0.017 +.012-022       59  6.7  70.9
6280 23    OphBD-05 4374 1526011414313342I                  164914.9-055925165435.7-060914 12.89 22.52 5.25  +1.08 +1.06 +0.53   K2III              -0.031-0.020 +.021-017V?   < 19:
6281 25Iot OphBD+10 3092 1526141024581442           8034    164916.5+101948165400.5+100955 28.73 30.66 4.38  -0.08 -0.32 -0.08   B8V                -0.051-0.036 +.029-021SB2   118
6282          CD-3311570 152636208205                       164925.8-332045165557.8-333026350.19  6.12 6.37  +1.71 +2.06         K5III              +0.023-0.002      -093
6283          CD-4010975 152667227473              V861 Sco 164938.3-403947165636.0-404925344.53  1.46 6.15  +0.26 -0.67         B0.5Ia            e+0.005-0.009      +004SB1O  161                 *
6284          BD-16 4371 152781160171      W                165015.4-163849165601.8-164822  3.76 16.22 6.37  +0.92 +0.68         K0IV               +0.081+0.040      -003V?         7.3  21.4
6285   Zet AraCP-55 7766 152786244315 631                   165020.5-554955165837.2-555925332.80-08.20 3.13  +1.60 +1.97         K3III              -0.019-0.036 +.044-006
6286          BD+47 2400 152812 46349    I S                165030.0+473437165317.6+472501 73.45 39.24 6.00  +1.32 +1.44 +0.70   K2III              -0.037+0.105      -063                 0.3
6287          BD+21 3002 152815 846873343                   165036.6+210710165455.2+205731 40.68 34.59 5.41  +0.97 +0.71 +0.49   G8III              +0.056+0.006 +.011-003V?   < 17
6288 27    ScoCD-3311590 152820208232                       165039.9-330603165711.1-331534350.54  6.07 5.48  +1.59               K5III              -0.009-0.005      -076V
6289          CD-5010924 1528242443131444                   165034.7-502859165818.0-503828337.02-04.88 5.55  +0.02               B9IV               -0.009-0.041      -044V
6290          BD+13 3258 152830102474              V644 Her 165039.5+134653165516.0+133711 32.56 31.82 6.34  +0.34 +0.06         F3V s              +0.034-0.036      -005SBO    23                 *
6291 24    OphBD-22 4249 152849184822       10265           165046.1-225929165648.0-230900358.60 12.31 5.58  -0.02               A0V                -0.004+0.001D+.009-026           0.3   1.0
6292 56    HerBD+25 3156 152863 84692       10259           165056.5+255330165502.1+254350 46.27 36.01 6.08  +0.92 +0.62 +0.48   G5III              +0.014-0.021      +001SB         4.6  17.9      *
6293 54    HerBD+18 3266 152879102476    I  10262           165058.4+183535165522.2+182600 37.86 33.62 5.35  +1.41 +1.66 +0.75   K4III              -0.110+0.014 +.020+012V?         4.2   2.5AB   3*
6294          BD-19 4471 152909160180       10266A          165111.3-192254165704.0-193224  1.62 14.41 6.27  +0.07 -0.30         B6V+B7V            -0.010-0.015D+.008-027           1.1   4.5
6295   Eps1AraCP-5210372 152980244331 632                   165136.7-530023165935.1-530938335.15-06.59 4.06  +1.45 +1.71 +0.54E  K4IIIab            +0.004+0.017 +.005+023
6296          BD-10 4417 153021160186                       165153.9-104816165726.1-105748  9.00 19.28 6.19  +1.00 +0.76         G8-K0III-IV        +0.021-0.081      -103
6297          CP-54 7947 153053244338      W                165158.7-542630170006.2-543549334.05-07.52 5.65  +0.19 +0.10         A5IV-V             -0.015-0.064      -020           6.1  20.1
6298          CD-3711131 153072208259      W                165206.1-372753165852.4-373716347.32  3.10 6.09  +0.19 +0.11         A3V                +0.024-0.061D+.018-027           0.2   0.2      *
6299 27Kap OphBD+09 3298 153210121962 633I         Kap Oph  165256.0+093149165740.1+092230 28.37 29.50 3.20  +1.15 +1.18 +0.54   K2III              -0.291-0.010 +.031-056V    < 17                 *
6300          CD-4811360 153221227542      W                165254.0-482937170027.1-483852338.81-03.93 6.00  +0.88               G8-K0III+G         -0.003-0.079      -029           1.1   1.0
6301          BD+14 3155 1532261024963346                   165257.1+140215165732.0+135303 33.10 31.42 6.37  +0.94 +0.70         K0V                -0.077+0.070      -031                          *
6302          BD-14 4509 1532291602053348                   165300.2-144254165841.6-145211  5.80 16.82 6.59  +0.40 +0.01         F3IV               -0.002-0.030      -013       53
6303          CD-4511123 153258227547                       165314.6-451801170032.3-452706341.35-01.98 6.65  +1.80               K4III              -0.004 0.000      -000V?
6304          CP-58 6964 153261244362              V828 Ara 165306.8-584828170147.4-585730330.66-10.33 6.11  -0.03               B2IVne            v-0.003-0.011      -006      201                 *
6305 57    HerBD+25 3166 153287 84720                       165324.6+253026165731.0+252110 46.01 35.36 6.28  +0.90 +0.55        gG5                 +0.004+0.013      +009
6306          BD+50 2345 153299 30161    I                  165332.7+501143165606.4+500220 76.80 38.69 6.56  +1.62 +1.98         M2IIIab            -0.021-0.008      -031V?
6307          BD+24 3095 153312 847263347                   165333.2+243210165742.3+242253 44.89 35.04 6.32  +1.10 +0.95         K0III              +0.006-0.018      -022
6308          CD-2412997 153336184892    I                  165350.2-245625165957.6-250531357.46 10.57 5.86  +1.62 +1.96 +0.97E  M3III              -0.005-0.006      -032V
6309 NOVA 1848                                     V841 Oph                                                                                                           -096                          *
6310 26    OphCD-2413002 153363184897      D                165401.9-245011170009.5-245921357.57 10.59 5.75  +0.41               F4V                +0.061-0.057      +019V?                        *
6311          CD-3511236 153368208293                       165356.6-354654170036.9-355603348.86  3.87 5.97  +1.16               K2IIICNII          -0.013-0.070      +017
6312          CD-5010955 153370244370      W                165359.7-505850170146.1-510751336.97-05.62 6.45  +0.27               A7V                -0.027-0.039      -028           0.3   0.1      *
6313          BD+42 2774 153472 464013349                   165441.0+423959165750.2+423045 67.19 38.33 6.34  +1.28 +1.41         K3III              -0.007-0.046      +028SB
6314   Eps2AraCP-53 8316 153580244388      W                165509.2-530512170308.8-531413335.41-07.06 5.29  +0.48               F6V                -0.003-0.144 +.039+007       58  7.6  25.
6315 19    DraBD+65 1157 153597 17281               8078    165528.6+651715165601.7+650805 95.68 36.47 4.89  +0.48 -0.03 +0.27   F6V               v+0.240+0.050 +.062-023SB1O    0                 *
6316          CD-3113473 1536132083243354  W                165524.5-315941170152.7-320837352.04  5.97 5.03  -0.10 -0.35         B8V                -0.007-0.051      +005V          7.5  23.8AB   4*
6317          BD+06 3332 1536531219953352  S                165536.7+064401170029.4+063501 25.88 27.63 6.59  +0.23 +0.09         A7V                +0.044-0.048      -010SB    158        0.1
6318 30    OphBD-04 4215 1536871414831445I W        8111    165547.1-040422170103.6-041321 15.58 22.18 4.82  +1.48 +1.83 +0.79   K4III              -0.041-0.075 +.015-007     < 19: 4.8  94.1
6319 20    DraBD+65 1159 153697 17285       10279   8081    165555.4+651128165625.2+650221 95.55 36.45 6.41  +0.38 +0.02         F1-3V              -0.041+0.032 +.029-021       90  0.2   1.0      *
6320          CP-57 8265 1537162444013355                   165553.6-573404170424.7-574244331.89-09.87 5.73  -0.10 -0.56         B5IV              e-0.008-0.022      +003V?    181                 *
6321 29    OphBD-18 4381 153727160231                       165600.1-184418170151.2-185308  2.85 13.88 6.26  +1.38 +1.26         K0III              -0.038-0.018      +044
6322 22Eps UMiBD+82  498 153751  2770 912   10242  Eps UMi  165612.2+821208164558.1+820214115.00 31.05 4.23  +0.89 +0.55 +0.47   G5III              +0.017+0.006 +.010-011SB1O   23  8.3  76.9      *
6323          CD-4611191 153791227588      W                165615.4-470059170341.7-470936340.33-03.45 6.06                      A2-3V              +0.005+0.029      +000           6.0   6.2      *
6324 58Eps HerBD+31 2947 153808 65716 634                   165627.7+310425170017.4+305535 52.85 36.17 3.92  -0.01 -0.10 -0.04   A0V                -0.047+0.028 +.028-025SBO    78                 *
6325          BD+22 3045 153834 84758    I                  165644.7+224646170058.1+223756 43.15 33.80 5.65  +1.33              gK3                 -0.011-0.021      +011
6326          BD+15 3095 153882102536       10310  V451 Her 165700.2+150544170133.0+145658 34.69 30.95 6.31  +0.04 +0.04         B9pCrEu           v+0.004-0.008      -032V      26  4.6  19.0      *
6327          CD-3711201 153890208355              V923 Sco 165701.6-380031170350.8-380908347.50  2.00 5.91  +0.38 -0.02 +0.18   F4IV+F2V           +0.071-0.023      -015SBO                       *
6328          BD+27 2738 153897 84765                       165708.7+272039170109.6+271147 48.46 35.08 6.55  +0.41 -0.08         F5V                -0.014-0.063      -031       40
6329          BD+08 3337 153914122023       10312           165710.7+083544170159.1+082702 27.95 28.14 6.33  +0.09 +0.06         A4V                +0.039+0.003D+.010+006SB    115  0.8   1.2      *
6330          BD+56 1934 153956 301903353                   165731.3+565007165921.5+564119 85.18 37.64 6.03  +1.16              gK1                 -0.051+0.037      -015
6331          CD-4511188 154025227601                       165746.9-452137170505.3-453006341.79-02.64 6.28  +0.07 +0.10         A2V                -0.031-0.017      -005
6332 59    HerBD+33 2817 154029 657361446                   165754.8+334246170136.4+333406 56.13 36.45 5.25  +0.02 +0.02         A3IV               +0.003+0.002 +.023-012V?     35
6333          BD+25 3183 154084 847763356                   165812.7+253847170218.7+253020 46.57 34.37 5.75  +1.02 +0.79        gG7                 +0.055+0.097 +.014-050
6334          CD-3311706 154090208377      W                165814.5-335856170449.4-340722350.83  4.29 4.87  +0.26 -0.68 +0.17   B1Ia              t+0.008-0.002      +004V?     91  9.1  20.       *
6335          BD+73  751 154099  86673351                   165815.6+731644165616.8+730740105.04 34.11 6.30  +0.24 +0.11         F0V                 0.000-0.021      -013V     177
6336          BD+32 2835 154126 65745                       165830.0+320136170217.2+315305 54.12 35.97 6.36  +1.13               K0                 +0.045+0.037      -013
6337          BD+14 3179 154143102553    I          8142    165832.9+141409170307.8+140531 33.95 30.26 4.98  +1.60 +1.92 +1.21   M3III              +0.023-0.064 +.005+043V                         *
6338          CD-4311396 154153227615                       165835.7-435800170548.6-440618342.98-01.90 6.19  +0.28 +0.02         A4III              +0.028+0.019      -033
6339          BD+14 3180 154160102554               8143    165837.8+143930170310.4+143040 34.41 30.42 6.52  +0.76 +0.45         G5IV:              -0.169-0.185 +.029-056
6340          BD-20 4627 154204184999                       165849.8-202115170445.3-202941  1.91 12.39 6.30  -0.02               B6IV               -0.006-0.024      -011V
6341          BD+13 3292 154228102564      W        8152    165903.5+134450170339.3+133619 33.50 29.95 5.93   0.00 -0.04         A1V                -0.023-0.035      -032V      29  0.2 299.2AB   4
6342          BD+13 3295 154278102569                       165921.8+134240170358.0+133403 33.49 29.87 6.08  +1.03 +0.84         K1III              +0.022-0.129      +046V?                        *
6343          BD+19 3218 154301102571    I  10323   8158    165931.9+194949170352.7+194126 40.11 32.18 6.35  +1.52               K4III              -0.019+0.016      -039SB         4.3 108.1AC   3*
6344          CD-3711274 154310208406      W                165934.6-370523170620.3-371339348.53  2.17 5.98  +0.07 +0.10         A2IV               +0.012-0.023      -021           4.9   5.8AB   3
6345          BD+69  884 154319 17305                       165935.8+691959165902.6+691111100.40 35.11 6.40R                     K0                  0.000-0.038 +.017-027
6346 61    HerBD+35 2911 154356 65761    I          8159    165954.7+353319170330.2+352451 58.48 36.40 6.69R                     M4IIIab            +0.033-0.035 +.010-012V?
6347          CD-3511306 154368208410      W                165948.9-351851170628.4-352704349.97  3.22 6.13  +0.50 -0.53         O9.5Iab            +0.013-0.006      +002V?    100  6.5   2.8      *
6348          BD+60 1728 154391 17312                       170000.5+604725170116.7+603857 90.05 36.81 6.13  +1.00 +0.82         K1III              -0.055+0.058      -017
6349          BD+00 3629 154417122056              V2213 Oph170011.3+005058170516.9+004209 20.77 23.78 6.01  +0.58 +0.06         F8.5IV-V          e-0.003-0.338 +.046-018V   =<  6                 *
6350          BD-21 4512 1544181850243358                   170013.4-212533170611.8-213353  1.22 11.49 6.30  +0.13               A3m                -0.011-0.077      -045
6351          BD+34 2890 154431 65766                       170016.8+345546170353.5+344725 57.73 36.21 6.04R +0.22?              A5V                -0.068-0.002      -020SB2   125
6352          BD+19 3220 154441102579       10326           170020.1+194414170441.3+193557 40.08 31.97 6.17R -0.01 -0.09         B9.5V              +0.009+0.001      -024V?     81  3.0   1.9
6353          BD-00 3224 1544451415133357                   170022.9-004518170532.3-005331 19.30 22.93 5.64  +0.16 -0.64 +0.05   B1V                +0.002+0.004      +016V     174                 *
6354          CD-2611896 1544811850331447                   170041.3-262239170653.2-263047357.24  8.47 6.29  -0.04 -0.24         A0III              +0.001-0.011      -024V
6355 60    HerBD+12 3142 154494102584 635   10334           170044.4+125241170522.7+124427 32.78 29.22 4.91  +0.12 +0.05 +0.02   A4IV               +0.051-0.010 +.022-004V?    111  6.1  56.0      *
6356          CP-61 5842 1545552538183362                   170058.6-613239171006.3-614032329.02-12.75 6.39  -0.04               B8II-III           -0.001-0.010      +004
6357          CP-70 2361 154556257472                       170101.1-703525171219.8-704316321.29-17.89 6.22  +1.06 +1.04         K1IVCNIII          +0.041-0.079      -024
6358          BD+09 3322 154610122075    I                  170124.6+095208170609.7+094400 29.77 27.78 6.37  +1.45 +1.73         K5III              +0.023-0.008      -006
6359          BD+10 3142 154619102592                       170129.5+103520170613.1+102715 30.51 28.07 6.37  +0.87 +0.57         G8III-IV           +0.049+0.018      -024
6360          BD+64 1170 154633 17324                       170142.0+644421170215.3+643602 94.84 35.94 6.10  +0.96 +0.71         G5V                -0.050+0.025 +.016-025V?
6361          BD-01 3292 154660141522       10347           170141.7-013117170652.9-013923 18.76 22.26 6.38  +0.20 +0.10         A9V                +0.033-0.036      +007V          3.4  20.3      *
6362          BD+44 2652 154713 464781448                   170202.1+435653170505.0+434844 68.93 37.09 6.43  +0.09 +0.16         A3IV               +0.006 0.000      -009
6363          BD+49 2583 154732 464763359           8185    170210.5+485631170449.8+484815 75.20 37.31 6.09  +1.09 +1.00         K1III              +0.026-0.066      +012SBO                       *
6364          BD+22 3073 154733 848353360I  10343           170204.1+221310170618.1+220503 43.00 32.45 5.56  +1.30 +1.52 +0.67   K3III              -0.094-0.038 +.011-096V          7.9  30.3      *
6365          BD-17 4717 1547791603051449                   170226.4-172836170814.9-173633  4.82 13.37 5.99  +1.01               K0III              +0.006-0.032      -014V?
6366          CD-3013840 1547832084673363                   170224.2-301616170847.5-302413354.33  5.83 5.97  +0.25 +0.33         A/FmDel Del        -0.015-0.062      -030                          *
6367          BD-00 3230 154895141528       10355           170304.0-005651170813.6-010446 19.49 22.26 6.06  +0.08 +0.04         A1V+F3V            -0.020-0.034D+.008-021SB         2.0   0.3      *
6368          CP-67 3296 154903253827      W                170304.3-670410171317.5-671148324.46-16.09 5.89  +1.06 +0.90         K0-1III            -0.180-0.093      -008           3.2  34.1      *
6369 21Mu  DraBD+54 1857 154905 30239       10345B          170315.3+543607170519.8+542813 82.30 37.02 5.83H                     F7V                -0.073+0.085 +.046-018       23  0.1   2.0AB   3*
6370 21Mu  DraBD+54 1857 154906 30239       10345A          170315.3+543607170519.7+542813 82.30 37.02 5.80H +0.48 +0.05 +0.24   F7V                -0.073+0.085 +.046-015       13  0.1   2.0AB   3*
6371          CD-4411502 154948227688      W                170327.4-442543171042.3-443327343.15-02.87 5.08  +0.86 +0.58 +0.43C  G8-K0III+G         -0.033-0.056 +.017-007           7.9  13.2
6372          BD-03 4063 154962141535                       170338.8-034457170854.5-035258 16.98 20.69 6.36  +0.69 +0.36         G5-8IV-V           -0.059-0.158      +015                          *
6373          CP-74 1610 1549722574783368                   170339.1-742440171635.6-743159318.01-20.12 6.25  -0.01  0.00         A0V                -0.027-0.058      +000
6374          CD-4811492 155035227699                       170402.4-484452171138.9-485227339.74-05.54 5.84  +1.84 +1.85         M1-2III            +0.029-0.030      +015V
6375          BD-10 4445 1550781603241450                   170416.2-102334170948.0-103124 11.15 16.99 5.56  +0.52 +0.02         F5IV               +0.057-0.105 +.014-003V?
6376          BD+40 3103 155102 46502 636                   170430.9+403848170746.7+403058 64.90 36.29 6.34  +0.03 +0.04         A2IV               -0.038-0.033      -007       45
6377          BD+36 2827 155103 65812       10360           170429.3+360354170802.0+355607 59.31 35.58 5.39  +0.31 +0.05 +0.14   A5-F1III/IVm s     -0.024-0.010 +.015-030       82  0.1   0.1AB   3*
6378 35Eta OphBD-15 4467 155125160332    I  10374           170438.5-153604171022.7-154329  6.72 14.01 2.43  +0.06 +0.09 +0.01   A2V                +0.039+0.098 +.052-001SB     26  0.4   0.3AB   4*
6379          BD+75  613 155154  8697               8183    170448.6+752609170140.2+751750107.35 33.04 6.21R                     F0IV-Vn            +0.016-0.079      +001SB    157                 *
6380   Eta ScoCD-4311485 155203227707 638                   170459.3-430626171209.2-431421344.37-02.30 3.33  +0.41 +0.09 +0.20   F3III-IVp          +0.024-0.287 +.063-028      150                 *
6381          CD-3911182 1552592085213366                   170521.7-392254171216.2-393025347.40-00.12 5.67  +0.04               A1V                -0.013-0.075      +012V?
6382          CD-3811632 155276208522                       170524.3-384152171216.5-384920347.95  0.28 6.30  +1.05               K1IIICNII          +0.019-0.043      -032
6383          BD+51 2178 155328 30262       10369           170549.8+505808170817.1+505032 77.74 36.73 6.46   0.00 -0.13         A1V                -0.002+0.029      -015       65  5.1  18.3     3*
6384          CP-56 8098 155341244539              V829 Ara 170546.4-564604171413.2-565318333.35-10.48 6.09  +1.78 +1.34         M1-2II-III+A       -0.016+0.003      -034V                         *
6385          BD+12 3161 155375102632                       170607.1+123530171045.8+122802 33.11 27.91 6.57  +0.08 +0.09 +0.05   A1m                +0.037-0.007      +005SBO                       *
6386          CD-2512018 155379185138                       170604.9-250753171213.6-251517358.98  8.23 6.54  -0.04               A0pHg              -0.014-0.033      -003                          *
6387          CD-2711516 155401185142                       170609.2-273819171225.0-274543356.95  6.74 6.14  -0.04               B9Vn:              -0.011-0.040      +001V                         *
6388          BD+40 3109 155410 465243365I S                170618.7+405408170933.3+404638 65.27 35.99 5.08  +1.28 +1.39 +0.64   K3III              -0.042+0.012 +.019-056SBO  < 17                 *
6389          CD-3212460 155450208539      W                170628.4-321902171258.7-322619353.20  3.91 6.01  +0.07 -0.77         B1II               +0.013-0.012      +017SB    114  7.5  30.       *
6390          BD+08 3367 1555001221641451                   170655.8+080059171145.2+075341 28.60 25.73 6.33  +1.04 +0.86         K0III              +0.035+0.014      -006
6391 63    HerBD+24 3140 155514 84896              V620 Her 170654.7+242135171103.2+241416 45.82 32.10 6.19  +0.23?              A8V                -0.011+0.033      -002      163                 *
6392          CD-3911212 155603208569    I W       V915 Sco 170731.8-393854171427.7-394601347.43-00.62 6.60  +2.21 +1.35 +1.08   G5Ia               +0.005-0.011      -006           4.2  14.4AB   3*
6393 37    OphBD+10 3165 1556441026463369I W                170744.9+104222171227.8+103507 31.38 26.74 5.33  +1.59 +1.64         M2IIIa             +0.001-0.026      +026V?         8.7  30.3
6394          BD+00 3654 155646122182                       170747.8+002825171254.4+002107 21.45 21.95 6.65  +0.50 +0.05 +0.27   F6III              +0.019-0.078      +058    =< 10                 *
6395          BD+52 2032 155711 302773367                   170813.1+523152171030.6+522432 79.68 36.35 6.29   0.00 -0.18         B9V                -0.014-0.010      -034V      50
6396 22Zet DraBD+65 1170 155763 17365 639  S                170829.7+655016170847.2+654253 96.01 35.04 3.17  -0.12 -0.43 -0.12   B6III              -0.020+0.022 +.023-017V      31                 *
6397          CD-3311875 155806208585               8388    170845.3-332558171519.3-333254352.59  2.87 5.53  -0.01 -0.91         O7.5V[n]e          +0.009 0.000      +004V?    162                 *
6398          CD-3811686 155826208591      W                170845.9-382802171535.9-383538348.52-00.11 5.96  +0.58 +0.11         F7V+G2V            -0.182-0.412 +.042-054           0.1   0.1      *
6399          BD+49 2604 155860 46561       10397   8327    170907.1+495155171140.2+494448 76.38 36.19 6.04R                     A5III              +0.020+0.035      -011           3.4   5.2AB   3*
6400          CP-69 2715 155875253870      W                170906.4-695549172012.8-700244322.26-18.11 6.53  +0.60 +0.15         G0-1IV:            -0.045-0.201      -005           2.7   1.4
6401 36    OphCD-2612026 155885185199    I  10417B          170911.7-262721171521.0-263610358.30  6.88 5.11  +0.86 +0.53         K1V                -0.458-1.142 +.188+000V?         0.0   4.6AB   5*
6402 36    OphCD-2612026 155886185198    I  10417A          170911.8-262717171520.8-263605358.30  6.88 5.07  +0.85 +0.53         K0V                -0.491-1.133 +.188-001SB         0.0   4.6AB   5*
6403          CD-3013968 155940208606                       170928.1-300543171551.5-301238355.38  4.71 6.21  -0.03 -0.09         B9.5V              -0.001-0.037      -004
6404          BD-14 4585 155970160402       10419           170939.1-142809171520.3-143502  8.38 13.64 5.99  +1.10               K1III              -0.012-0.001      -003V?         5.1   4.2      *
6405          CD-3511426 155974208610      W                170941.1-353738171621.5-354458350.92  1.42 6.12  +0.48 -0.02         G0IV-V             -0.118-0.319 +.023-001           7.4  36.5
6406 64Alp1HerBD+14 3207 156014102680    I  10418A Alp Her  171005.2+143015171438.9+142325 35.53 27.82 3.48H +1.44 +1.01 +2.14   M5Ib-II           e-0.006+0.036 -.002-033V      21  2.2   4.9AB   4*
6407 64Alp2HerBD+14 3207 156015102681       10418B          171005.5+143013171439.2+142324 35.53 27.82 5.39H                     G5III+F2V          -0.003+0.045 -.002-037SBO =< 50  2.2   4.9AB   4*
6408          CP-59 6954 156091244613      W                171021.3-593507171912.4-594140331.34-12.58 5.91  +1.37               K2IIICNIb-II       -0.012-0.008      -005           7.0  20.
6409          CD-3212545 1560982086261452                   171033.2-323259171703.7-323946353.52  3.08 5.55  +0.50 +0.05         F6IV               -0.096-0.056      -036        0                 *
6410 65Del HerBD+25 3221 156164 84951 641I  10424   8419    171055.4+245725171501.9+245021 46.83 31.42 3.14  +0.08 +0.08 +0.03   A3IV               -0.021-0.157 +.044-040SB    290  5.6   8.5AB   4*
6411   Iot ApsCP-69 2719 156190257491 642  W        8509    171056.5-700104172205.9-700724322.28-18.29 5.41  -0.04 -0.23         B9V+B9.5V          -0.006-0.012      -004           0.0   0.1      *
6412          BD+02 3283 156208122224      S        8438    171111.9+021754171614.2+021110 23.62 22.10 6.17  +0.22 +0.16         A2V                +0.001-0.021      +004       25        0.1
6413          BD-06 4575 1562271415853371                   171121.2-060802171642.8-061442 15.90 17.80 6.09  +1.10 +0.97         K0                 -0.004-0.005      -021
6414          BD+01 3408 1562471222261453   10428  U Oph    171127.2+011919171631.7+011238 22.73 21.57 5.88  +0.06 -0.45         B5Vnn+B5V          -0.001-0.013 +.002-011SB2O  105  7.3  20.7      *
6415 41    OphBD-00 3255 156266141586    I  10429           171128.6-001957171636.7-002643 21.20 20.75 4.73  +1.14 +1.13 +0.59   K2III              -0.023-0.066 +.010-002V?   < 17  3.0   1.0      *
6416          CD-4611370 156274227816      W                171127.7-463155171903.2-463802342.29-05.26 5.48  +0.80 +0.38 +0.46   G8-K0V             +0.979+0.224 +.131+023           3.1   7.5AB   4*
6417   Zet ApsCP-67 3310 1562772538823374                   171132.3-673958172159.4-674614324.40-17.10 4.78  +1.21 +1.27         K2-III             -0.042-0.006 +.014+013
6418 67Pi  HerBD+36 2844 156283 65890 643I          8431    171133.8+365518171502.8+364833 60.66 34.34 3.16  +1.44 +1.66 +0.72   K3IIab            e-0.026+0.004 +.025-026V?   < 17                 *
6419          BD+23 3070 156284 84955    I                  171132.1+235115171541.6+234434 45.66 30.94 5.96  +1.31               K2III              -0.025+0.022      -042V?
6420          CD-4311572 156293227813                       171133.6-440116171848.0-440747344.34-03.80 5.76  -0.05 -0.13 -0.02C  B9V                 0.000-0.021      -012
6421          BD+63 1336 156295 173913370                   171140.7+625918171232.6+625228 92.50 35.14 5.56  +0.21 +0.06         F0IV               +0.014+0.048 +.014-005V?    114
6422          CD-3212573 156325208657      W       Var?     171149.6-322640171820.4-323312353.77  2.93 6.36  +0.15 -0.36         B5Vn              e-0.013-0.019      -014V?         5.6  19.5      *
6423          CD-4911324 156331244631      W                171146.7-495720171930.3-500348339.50-07.27 6.27  +0.41               F8III+B9V          -0.014-0.015      +011           0.0   0.1      *
6424 39Omi OphCD-2413255 156349185238    I  10442A          171154.7-241041171800.7-241713  0.54  7.70 5.20  +1.10 +0.90         K0II-III           -0.054-0.008 +.013-029           1.5  10.2      *
6425 39Omi OphCD-2413255 156350185237       10442B          171154.6-241030171800.5-241703  0.55  7.71 6.80  +0.51 +0.05         F6IV-V             -0.073-0.009 +.013-028V          1.5  10.2      *
6426          CD-3411626 156384208670      W        8482    171208.7-345240171857.2-345923351.83  1.44 5.91  +1.04 +0.82 +0.59   K3V+K5V            +1.170-0.164 +.141+000           1.0   0.5AB   4*
6427          CD-4411595 156398227821      W                171209.8-440657171924.5-441323344.32-03.94 6.65  +0.20 +0.01 +0.15C  B9.5V+A6III        -0.007-0.015      -015           0.3   0.2      *
6428          BD-16 4470 156462160440    I                  171233.6-161217171819.2-161843  7.30 12.09 6.43  +1.66 +2.05         M2IIIFe1           -0.005+0.010      -040V
6429          CP-80  828 1565132587691455           8609    171245.1-804559173127.3-805133312.32-23.69 5.88  +1.67 +1.81         M3III              -0.013-0.044      -019
6430          BD+23 3074 156593 84982    I                  171324.5+231153171735.9+230527 45.11 30.32 6.45R                     K2                 -0.002-0.001      -015
6431 68    HerBD+33 2864 156633 65913       10449  u Her    171337.8+331228171719.5+330600 56.40 33.14 4.82  -0.17 -0.76 -0.19   B1.5Vp+B5III       -0.008-0.003 +.009-021SB2O  116  5.1   4.4      *
6432          BD+17 3216 156653102724                       171338.5+172529171805.0+171905 38.95 28.19 6.00  +0.01 +0.02         A1V                +0.014-0.011      +004SB     24
6433          BD+11 3156 1566811027253372I                  171354.8+105822171837.0+105152 32.38 25.49 5.03  +1.55 +1.89 +0.83   K4II-III           +0.010-0.095 -.002+040V?   < 17
6434          BD+06 3386 156697122270              V2112 Oph171359.4+061126171852.8+060507 27.69 23.34 6.51  +0.39 +0.15         F0-2IV-Vn          +0.020+0.006      -025      139                 *
6435          BD-17 4773 156717160462       10465           171404.0-173907171953.3-174523  6.28 10.99 6.02  +0.04               A2Vnn              -0.010-0.021D+.012-004           1.5   2.0AB   3*
6436 69    HerBD+37 2864 156729 65921               8489    171413.3+372346171740.3+371730 61.35 33.91 4.65  +0.05 -0.03  0.00   A2V                -0.034+0.063 +.018-010SB    149
6437          BD+49 2614 156753 46605                       171415.8+494755171648.6+494128 76.33 35.36 7.48R                     K2                 +0.023+0.002
6438          CP-57 8478 156768244678      W                171417.8-575436172255.0-580037333.05-12.07 5.88  +1.07 +0.86         G8Ib-II            -0.021-0.011      -010           3.6   2.0
6439          BD-05 4426 156826141611                       171438.3-054833171959.5-055502 16.64 17.27 6.32  +0.85 +0.47         G9V                +0.040-0.180 +.017-031V
6440          CP-62 5558 1568382539033378                   171437.5-624556172401.1-625151328.89-14.75 5.70  -0.14 -0.76         B2IV               -0.001-0.008      +004V?                        *
6441          BD-19 4605 156846160474       10476           171441.9-191339172034.2-191958  5.04  9.98 6.52  +0.58               G3IV               -0.151-0.102      -069           7.5   5.1      *
6442          CP-56 8191 156854244685                       171441.0-562534172306.9-563131334.34-11.29 5.80  +1.00 +0.77         G8-K0III           -0.008+0.009      -006
6443          BD+28 2719 156874 850013375                   171453.1+285539171848.5+284923 51.59 31.77 5.65  +0.98       +0.49   K0III              +0.050-0.005 +.016-014
6444          BD+38 2910 156891 659303373                   171501.7+385450171823.3+384841 63.18 34.03 5.94  +1.00 +0.81        gG7                 -0.014+0.077      -038V?
6445 40Xi  OphBD-20 4731 156897185296      W                171500.6-210020172100.2-210646  3.59  8.92 4.39  +0.39 -0.05 +0.22   F1III-IV           +0.232-0.206 +.061-009        0  4.5   3.7
6446 53Nu  SerBD-12 4722 1569281604793376   10481           171512.1-124444172049.7-125049 10.63 13.46 4.33  +0.03 +0.05  0.00   A2V                +0.042+0.004 +.028+005V     115  4.8  46.3      *
6447          CP-60 6800 156942253908                       171517.6-603435172418.7-604025330.84-13.64 5.77  -0.08               B8Ib-II            -0.008-0.002      -010
6448          BD+60 1743 156947 17414              VW Dra   171516.6+604636171629.4+604014 89.76 34.96 6.32  +1.09               K1.5IIIb           -0.042+0.014 +.003+017V?                        *
6449          BD-10 4477 156971160482      W                171520.2-103541172052.7-104146 12.51 14.60 6.46  +0.33 +0.01         F1III              +0.065-0.009      +012       28  7.8   8.1
6450          CD-3711507 157038208740      W                171549.9-374222172239.4-374818349.95-00.79 6.41  +0.64 -0.32         B4Ia               +0.017-0.013      -014           5.8   2.7AB   3*
6451   Iot AraCD-4711484 1570422278863379  W       Iot Ara  171545.6-472211172316.1-472806342.01-06.33 5.25  -0.11 -0.82 -0.08   B2IIIne           v-0.010-0.024      -019      369  5.3  42.8      *
6452          BD+18 3351 1570491027571454I         V656 Her 171554.4+180935172018.9+180326 39.96 27.97 5.00  +1.62 +2.06 +0.77E  M2IIIab            +0.009-0.055 +.006-046V?                        *
6453 42The OphCD-2413292 157056185320 644          The Oph  171552.0-245359172200.6-245958  0.46  6.55 3.27  -0.22 -0.86 -0.21   B2IV               -0.004-0.020      -002SB     35  2.0   0.0      *
6454          CD-3511505 157060208741                       171554.9-354853172238.0-355436351.51  0.28 6.47  +0.54               F7V                +0.080+0.112      -006V
6455          BD+25 3246 157087 85016                       171605.2+253821172009.8+253215 48.01 30.53 5.38  +0.03 +0.10         A3III              +0.021-0.015 +.011-005SB     11
6456          CD-3711512 157097208747      W                171607.4-370719172254.9-371314350.47-00.51 5.93  +1.08               K1III              +0.033-0.026      -036           8.4  20.2
6457 70    HerBD+24 3167 157198 85021      W                171647.0+243556172054.2+242958 46.93 30.05 5.12  -0.03 +0.02         A2V                -0.022+0.002 +.010-018SB     99  3.5 222.9AB   3
6458 72    HerBD+32 2896 157214 659631456   10488   8553    171655.0+323547172039.6+322804 55.88 32.31 5.39  +0.62 +0.07 +0.35   G0V                +0.137-1.041 +.069-078V       0  5.1 230.0AB   3*
6459 43    OphCD-2813081 157236185350    I                  171703.9-280245172321.6-280835358.02  4.54 5.35  +1.55 +1.80         K5III              +0.004-0.031 +.028-014
6460          CD-4411669 157243227911                       171658.0-440359172413.0-440945344.86-04.62 5.12  -0.06 -0.40 -0.01C  B7III              -0.011-0.028      +013V     150:                *
6461   Bet AraCP-55 8100 157244244725 645I'                 171659.1-552607172518.0-553148335.37-11.01 2.85  +1.46 +1.56 +0.53E  K3Ib-IIa           -0.008-0.025 +.034-000                          *
6462   Gam AraCP-56 8225 157246244726      W                171658.5-561700172523.6-562239334.64-11.48 3.34  -0.13 -0.96         B1Ib               -0.002-0.012      -003V     281  7.0  17.9AB   3*
6463          BD+16 3163 157257102770    I                  171705.7+164948172133.4+164351 38.69 27.20 6.35  +1.61 +1.98         M2.5IIIab          -0.018-0.024      +039
6464 74    HerBD+46 2293 157325 466453377I          8556    171731.6+462020172021.1+461427 72.16 34.58 5.59  +1.57 +1.86         M0III              -0.029+0.043      -057V?                        *
6465          BD-02 4343 157347141642                       171738.2-021721172251.3-022318 20.22 18.43 6.29  +0.68 +0.24         G5IV               +0.045-0.112      -035
6466          BD+28 2728 157358 85028      W                171736.1+285121172131.2+284529 51.71 31.18 6.35  +0.70               G0III              +0.006+0.006      -006SB     11  2.2   0.6
6467          BD+48 2506 157373 46651                       171751.6+481715172033.6+481118 74.53 34.68 6.43  +0.43 -0.08         F4V                +0.190-0.019 +.007+031    =< 15
6468   Kap AraCD-5011269 157457244734      W                171812.1-503231172600.0-503801339.60-08.45 5.23  +1.06               G8III              +0.007+0.001 +.030+017           8.3  30. AC   3
6469          BD+40 3136 157482 46664      S       V819 Her 171826.6+400422172143.6+395828 64.69 33.58 5.51  +0.68 +0.21         F9Vn:             e+0.009-0.063 +.021+003SB2    12        0.0      *
6470          CD-3411674 157486208786                       171824.2-343611172502.7-344147352.79  0.56 6.16  +0.02               A0VSiCr            +0.004-0.027      -018V
6471          CP-62 5590 157524253928                       171841.7-625650172807.7-630211328.99-15.24 6.24  -0.09               B7-8V              -0.023-0.027      -003
6472          BD-21 4597 157527185367       10522           171843.1-212053172442.0-212630  3.79  8.01 5.85  +0.93 +0.67 +0.32E  G8III              -0.021-0.028 +.013-056V?         6.0   4.3      *
6473          BD-18 4516 1575461605233382                   171845.7-182110172437.1-182645  6.32  9.67 6.21  +0.03 -0.13         B9Vn               +0.018-0.004      -021V
6474          CD-2413325 157588185374                       171859.4-240908172506.2-241437  1.49  6.39 6.19  +1.10              gK1                 +0.016+0.012      +020V
6475          CD-5110881 157599244749                       171900.5-515132172656.3-515657338.56-09.29 6.19  -0.03               B8-9V              +0.005-0.021      +001V?
6476          BD+08 3405 157617122346    I                  171910.8+085644172357.6+085110 31.00 23.44 5.77  +1.25 +1.27        gK1                 +0.009-0.002      +016
6477          CD-4511531 157661227971      W        8637    171928.8-454512172651.5-455037343.72-05.94 5.29  -0.07 -0.34 -0.03C  B7V+B9.5V          -0.040-0.040D+.009-009V?         0.8   2.3AB   3*
6478          CD-5011283 157662244755      W                171924.5-503229172712.4-503749339.71-08.61 5.92  +0.08               B9II               -0.007-0.004      +011           4.7  76.1AB   4
6479          BD+53 1937 157681 303543381I                  171935.5+533056172145.4+532514 80.90 34.64 5.67  +1.47               K4III              +0.021+0.005      -008V
6480 73    HerBD+23 3100 157728 850623383           8606    171955.4+230311172406.6+225737 45.53 28.86 5.74  +0.21 +0.05         F0IV               -0.046-0.032      -020       67                 *
6481          BD+16 3174 157740102805                       172002.7+162335172431.5+161804 38.56 26.37 5.71  +0.07 +0.12         A3V                +0.008-0.026      +011                          *
6482          BD+15 3179 157741102806       10528           172003.3+154149172433.8+153622 37.84 26.10 6.35  -0.02 -0.18         B9V                +0.008+0.015      -019      338  4.3   3.9      *
6483          CP-5210662 1577532447633386  W                171959.8-521230172757.6-521750338.35-09.61 5.75  +1.17               K2III              -0.006-0.065      -009V?         7.2  17. AB   3
6484 75Rho HerBD+37 2878 157778 66000       10526B          172013.6+371419172340.7+370848 61.44 32.71 5.47H                     A0Vn               -0.041+0.004 -.002-019      190  1.0   4.2AB   3*
6485 75Rho HerBD+37 2878 157779 66001       10526A  8605    172013.9+371416172341.0+370845 61.44 32.71 4.52H -0.03 -0.05 -0.01   B9.5III            -0.037+0.006 -.002-021       81  1.0   4.2AB   3*
6486 44    OphCD-2413337 1577921854011457           8640    172015.7-240500172622.2-241031  1.71  6.19 4.17  +0.28 +0.12 +0.12   A3m                 0.000-0.116 +.045-037       59                 *
6487          CP-55 8144 157819244770      W                172022.0-550500172838.7-551011335.94-11.22 5.94  +1.11               G8II-III           -0.017-0.012      -013           6.5  37.0AC   3*
6488          BD+38 2928 157853 66006       10531           172040.3+384022172402.2+383458 63.14 32.91 6.49  +0.73 +0.21         F8IV               -0.012+0.040D+.001+027           0.1   0.1      *
6489          BD-01 3329 1578561416611458                   172046.3-013352172557.9-013906 21.29 18.12 6.44  +0.46 -0.02         F3V                +0.063+0.051      -024        8
6490          CD-2512160 157864185406                       172043.9-255119172655.2-255636  0.30  5.11 6.44  -0.06               A0V                -0.028-0.026      -008V?
6491          BD+37 2882 157910 66014       10535           172059.2+370227172427.1+365707 61.25 32.52 6.28  +0.88 +0.53 +0.45   G5III+F0V          -0.026+0.047      -016V?         3.6  33.2      *
6492 45    OphCD-2913557 157919185412 646                   172058.0-294635172721.3-295201357.08  2.85 4.29  +0.40 +0.09 +0.18   F5IVDel Sct*       +0.020-0.139 +.021+037       25                 *
6493          BD-04 4275 157950141665 647           8662    172119.4-045954172637.9-050512 18.26 16.27 4.54  +0.39 -0.03 +0.21   F3V                -0.092-0.043 +.032+000SB2O   52                 *
6494          CD-2913563 157955185417                       172114.9-293816172737.5-294328357.23  2.88 6.00   0.00               B9IV               -0.008-0.026      -021V
6495          BD+17 3241 157967102819    I W       V640 Her 172127.3+170018172554.4+165503 39.33 26.30 5.98  +1.62               M4IIIab            -0.008+0.011      -010V?         7.5  23.1AC   3*
6496          BD-12 4750 157968160553                       172125.3-122527172702.1-123045 11.74 12.36 6.21  +0.51               F7V                +0.037-0.062      -040V?                        *
6497          BD+07 3368 1579781223813385                   172129.3+074059172619.0+073544 30.06 22.36 6.06  +0.58 +0.28 +0.43   B9.5V+G0V           0.000-0.005      -011SBO                       *
6498 49Sig OphBD+04 3422 1579991223871459I          8664    172133.1+041338172630.9+040825 26.77 20.75 4.34  +1.50 +1.62 +0.77   K2II               +0.004+0.007 +.008-027     < 19:
6499          BD+27 2809 158067 850803387  W                172200.4+265755172600.9+265244 49.95 29.69 6.41  +0.10 +0.12         A5IV               +0.008+0.023      -027       58  5.0  49.4AB   3
6500   Del AraCP-60 6842 158094253945 648  W                172204.2-603601173105.9-604102331.27-14.36 3.62  -0.10 -0.31         B8Vn               -0.059-0.096      +010      260: 8.0  47.4      *
6501          CD-3611546 158105208860                       172210.0-364141172856.1-364642351.50-01.25 6.02  +1.11               K0III              +0.002-0.009      -028
6502          BD+20 3481 158148 850953388                   172230.1+200958172649.1+200451 42.71 27.27 5.54  -0.13 -0.56         B5V                +0.001+0.015      -030V     251
6503          CD-3811927 158156208870      W                172233.0-382605172925.7-383100350.11-02.30 6.39  +0.09               A2V                +0.032+0.020      +008           0.3   0.4
6504          BD-08 4444 158170141679                       172236.5-080715172802.3-081230 15.66 14.38 6.37  +0.58 +0.16         F5IV               -0.081-0.128      -064    =< 15
6505          CP-56 8304 158220244808                       172253.0-565028173123.0-565514334.62-12.46 5.95  -0.08               B7II-III           -0.018+0.008      -003    =< 41
6506          BD+34 2971 158261 660543389                   172310.5+344646172646.2+344145 58.77 31.59 5.94  -0.01 -0.06         A0V                -0.024+0.047      -022SBO    10                 *
6507          BD+00 3697 1583521224183391                   172343.5+002442172849.7+001950 23.49 18.45 5.44  +0.22 +0.11         A8V               e-0.062+0.023 +.012-036      184                 *
6508 34Ups ScoCD-3711638 158408208896 649                   172357.8-371257173045.8-371745351.27-01.84 2.69  -0.22 -0.82 -0.23   B2IV               -0.001-0.031      +008SB     73                 *
6509 77    HerBD+48 2517 158414 46723 650                   172405.1+482038172644.2+481536 74.70 33.66 5.85  +0.12 +0.15         A4V                +0.002-0.004 +.014-009SB    120
6510   Alp AraCD-4911511 158427228069 651  W        8999    172406.6-494748173150.5-495234340.76-08.83 2.95  -0.17 -0.69 -0.24   B2Vne             v-0.031-0.070 +.007+000SB    298  8.0  55.6      *
6511          BD+60 1754 158460 17472                       172423.7+600756172541.3+600254 88.89 33.88 5.65  +0.03 +0.01         A1Vn               -0.010+0.027      +007SB    180
6512          BD-05 4450 158463141691       10583   8918    172426.5-055016172947.4-055511 17.92 15.17 6.37  +0.93 +0.67         K0III              -0.022-0.073D+.008+004           2.1   1.3      *
6513          CD-4511626 158476228075      W                172424.6-455734173149.1-460211344.03-06.77 6.03  +0.83               F8-G0Ib            +0.001+0.007      -029V          4.2  18.0
6514          BD+58 1731 158485 303873390                   172434.8+584407172604.9+583907 87.21 33.91 6.51  +0.14 +0.09 +0.05   A4V                -0.007+0.015      -030
6515 NOVA 1604                                     V843 Oph                                                                                                                                         *
6516          BD-00 3300 158614141702       10598           172514.8-005848173023.8-010345 22.40 17.44 5.31  +0.72 +0.31 +0.36   G9IV-VHdel 1       -0.117-0.170 +.054-077SBO        0.1   0.7      *
6517          CD-3312149 158619208921                       172511.8-333734173147.4-334210354.39-00.05 6.44  +1.19               K2III              -0.005-0.015      -034
6518          BD+67 1014 158633 17474                       172518.4+672326172500.2+671823 97.56 33.16 6.43  +0.76 +0.29         K0V                -0.525-0.004 +.078-040SB
6519 51    OphCD-2313412 158643185470               9037    172518.8-235307173125.0-235746  2.52  5.34 4.81   0.00 -0.06 +0.02   B9.5Ve             +0.004-0.027 +.040-012V
6520          CD-2612152 158704185474                       172531.7-261135173144.4-261611  0.62  4.02 6.05  -0.06 -0.37         B9p                -0.004-0.025      +000                          *
6521          BD+12 3234 158716102869                       172543.2+120004173022.4+115530 34.76 23.32 6.39R +0.06 +0.01         A1V                -0.026+0.053      -025    =< 41
6522          CD-3411757 158741208935              V949 Sco 172547.1-341213173224.5-341647353.98-00.47 6.17  +0.35               F2V                -0.017-0.039      -050
6523          CD-4111742 158799228110                       172604.2-410557173307.4-411025348.27-04.34 5.84  +0.06 -0.17         B9Ib-II            +0.004-0.024      -019
6524          BD+02 3337 158837122465       10607           172620.4+024758173121.3+024328 26.03 19.03 5.59  +0.84 +0.58         G8III+A7           -0.014+0.019      -029SB         0.0   0.1      *
6525          CP-59 7071 158895244866                       172639.7-594633173534.9-595046332.31-14.42 6.28  -0.08 -0.46         B5II-III           -0.007-0.009      -009
6526 76Lam HerBD+26 3034 158899 851631460I          9070    172641.7+261109173044.3+260638 49.48 28.44 4.41  +1.44 +1.68 +0.75   K3.5III            +0.020+0.018 +.016-026     < 17                 *
6527 35Lam ScoCD-3711673 158926208954 652  W       Lam Sco  172649.0-370151173336.5-370614351.74-02.21 1.63  -0.22 -0.89 -0.28   B2IV+B             -0.001-0.029      -003SB2   163 10.3  94.9AC   3*
6528          BD+31 3047 158974 66103                       172708.1+311358173055.4+310930 55.03 29.87 5.61  +0.95               G8III              +0.009+0.022      -026V?
6529          BD+80  544 158996  28593384I                  172711.3+801330171937.0+800811112.37 30.58 5.72  +1.50 +1.80         K5III              +0.016+0.004      -007
6530          CP-53 8682 159018244870                       172714.5-531658173520.0-532110338.04-11.11 6.10  +0.02               B9III               0.000-0.016      +007V?
6531          BD+39 3147 159026 66102                       172719.7+385725173040.3+385256 63.77 31.69 6.43  +0.49 +0.22         F6III              +0.013+0.015      -027      139
6532          BD+12 3241 159082102897                       172735.5+120009173215.0+115549 34.98 22.90 6.42  -0.01 -0.20         B9.5VpHgMn         +0.024+0.025      -012SBO    32                 *
6533 78    HerBD+28 2767 159139 85182                       172753.9+282847173149.6+282427 52.06 28.91 5.62R  0.00 -0.05         A1V                +0.006+0.028      -026SB?   245
6534          BD-05 4461 1591701417303392                   172809.4-054017173329.9-054441 18.56 14.46 5.62  +0.18 +0.09         A5V                -0.039-0.091      -026      255
6535          CD-3212935 159176208977      W        9167    172810.4-323045173442.5-323454355.67  0.05 5.70  +0.04 -0.86         O7V+O7V            +0.008-0.005      -004SB2O  161  4.8   5.4AB   3*
6536 23Bet DraBD+52 2065 159181 30429 653I  10611           172810.3+522231173026.0+521805 79.58 33.31 2.79  +0.98 +0.64 +0.48   G2Ib-IIa           -0.016+0.015 +.013-020V      13  8.9   4.2AB   3*
6537  1Sig AraCD-4611661 159217228162                       172812.8-462612173539.6-463020343.98-07.58 4.59  -0.03 -0.09 -0.05   A0V                -0.028-0.035 -.014+004
6538          BD+34 2989 159222 66118                       172825.9+342029173201.1+341615 58.58 30.43 6.56  +0.65 +0.17 +0.32   G5V                -0.233+0.050 +.055-052V?                        *
6539          CD-3711702 159312209001                       172854.3-372218173543.0-372624351.69-02.75 6.48  +0.01               A0V                -0.013-0.038      -008
6540          BD+57 1774 159330 30434                       172907.3+575700173043.8+575235 86.25 33.33 6.40R                     K2III              +0.030-0.030      -014SB
6541          BD+19 3354 1593321029123393                   172902.0+191944173322.8+191524 42.48 25.53 5.64  +0.48 -0.02         F6V                -0.030-0.090 +.032-059V   =< 10
6542          BD+16 3218 1593531029173394                   172910.9+162318173339.4+161903 39.51 24.36 5.69  +1.01 +0.82 +0.49  gK0                 -0.001-0.051 +.030-022
6543          BD+14 3279 159354102918    I         V642 Her 172910.4+145446173342.8+145030 38.03 23.77 6.48  +1.60 +1.58 +1.28   M4IIIa             +0.023-0.070      +030                          *
6544          BD-11 4411 1593581606531461  W                172912.6-111027173446.4-111431 13.86 11.40 5.55  +0.02 -0.17         B8Vn               +0.005+0.006      -011           3.0  54.6      *
6545 52    OphBD-21 4659 159376185526              V2125 Oph172917.5-215835173518.5-220238  4.63  5.62 6.57  +0.01 -0.18         B8p                -0.004-0.009      -012V                         *
6546          CD-3812044 159433209019               9205    172939.5-383352173632.8-383807350.77-03.53 4.29  +1.09 +0.90 +0.55   K0IIIb             -0.012-0.201 +.018-049V
6547          CD-4911577 159463244894                       172941.5-495935173727.2-500336341.08-09.69 5.93  +1.10               K0III              -0.009-0.099      +003
6548 53    OphBD+09 3424 159480122526       10635A          172951.8+093915173436.7+093512 32.95 21.39 5.81  +0.02 -0.01         A2V                +0.005-0.008      -014       16  2.0  41.3AB   4*
6549   Pi  AraCP-54 8403 159492244896                       172952.7-542559173805.6-543001337.24-12.04 5.25  +0.20 +0.08         A5IV-V             -0.039-0.150 +.057-003       48
6550          BD+41 2850 159501 467921462                   172956.9+411851173307.3+411437 66.60 31.66 5.74  +1.09 +0.96        gK1                 -0.068-0.060 +.015-029V?
6551          BD+16 3220 159503102928                       172959.2+163418173427.2+163014 39.78 24.26 6.40R                     A8Vn               -0.004-0.015      -041      142
6552          CP-85  469 1595172587793989                   172959.3-851034180134.1-851253308.13-26.00 6.45  +0.44 +0.02         F5IV               -0.007-0.142      -023
6553   The ScoCD-4212312 159532228201 654           9233    173007.9-425603173719.2-425952347.14-05.98 1.87  +0.40 +0.22 +0.20   F1II               +0.015-0.002 +.027+001      105                 *
6554 24Nu 1DraBD+55 1944 159541 30447 655   10628B  9163    173012.3+551509173210.6+551103 83.03 33.14 4.88  +0.26 +0.04 +0.13   A6V                +0.147+0.057 +.035-015SB     66  0.0  62.3      *
6555 25Nu 2DraBD+55 1945 159560 30450 657   10628A  9164    173017.7+551428173216.0+551023 83.02 33.12 4.87  +0.28 +0.06 +0.13   A4m               v+0.148+0.056 +.035-016SBO    50  0.0  62.3      *
6556 55Alp OphBD+12 3252 159561102932 656I A        9189    173017.5+123758173456.1+123336 35.90 22.57 2.08  +0.15 +0.10 +0.08   A5III              +0.120-0.226 +.067+013SB?   219        0.1      *
6557          CD-3711723 159633209034                       173035.7-380012173726.8-380356351.34-03.37 6.26  +1.25               G2Ib               -0.013+0.026      +012
6558          CD-4212327 159707228220                       173057.9-424904173808.5-425250347.32-06.04 6.10  -0.06 -0.26 -0.02C  B8V                +0.002-0.041      +013
6559          BD+21 3157 159834 85232       10655           173143.1+210336173559.6+205946 44.53 25.59 6.10  +0.19 +0.17         A7IV               +0.012-0.015      -017SB     22  3.5  10.4      *
6560          BD+57 1780 159870 30464      S                173153.1+573729173331.7+573332 85.87 32.96 6.17  +0.59 +0.37 +0.38   G5III+A5V          +0.015+0.014      -001SBO    28        0.1      *
6561 55Xi  SerBD-15 4621 159876160700 658I W        9270    173151.6-152008173735.2-152355 10.61  8.66 3.54  +0.26 +0.14 +0.13   F0IVDel Sct        -0.042-0.058 +.030-043SBO    32  9.4  22.0      *
6562          BD-15 4622 159877160701                       173151.9-153035173736.2-153416 10.46  8.57 5.94  +0.37               F0IV               -0.009-0.001      -008       30
6563          BD+37 2908 159925 661653395                   173216.1+372152173542.4+371806 62.20 30.40 6.10  +0.98 +0.82:        G9III              +0.015+0.003      +004
6564          BD+28 2787 159926 85236                       173211.9+281449173607.9+281105 52.14 27.94 6.38R                     K5                 -0.031+0.017      -034
6565          CP-72 2086 159964257525      W                173218.0-721017174419.7-721315321.16-20.84 6.49  +0.48 -0.04         F7IV+F5V           -0.003+0.113D+.032+035SB         0.3   0.5      *
6566 27    DraBD+68  938 159966 17526 659I W                173221.7+681156173157.9+680806 98.42 32.40 5.05  +1.08 +0.92 +0.53   K0III              -0.012+0.134 +.020-073SB   < 17  6.2 163.4
6567 57Mu  OphBD-08 4472 1599751417723399                   173224.5-080328173750.7-080708 17.00 12.34 4.62  +0.11 -0.20 +0.11   B8II-IIIp:Mn       -0.009-0.017      -019      134
6568          BD-10 4528 160018160708    I                  173236.6-105159173809.5-105535 14.57 10.85 5.75  +1.23              gK0                 -0.013-0.010      -033
6569   Lam AraCD-4911616 160032228257              Var?     173240.3-492113174023.6-492456341.89-09.77 4.77  +0.40 -0.04 +0.22   F3IV               +0.078-0.176 +.044+004        0                 *
6570          BD+30 3033 160054 661753397                   173248.3+305048173636.7+304707 55.00 28.59 6.02  +0.16 +0.09         A5V                +0.030-0.005      -017      101
6571 79    HerBD+24 3218 160181 852643400  S                173323.9+242210173731.1+241836 48.12 26.41 5.77  +0.11 +0.07         A2Vn               -0.012+0.008      -004V     210        0.1
6572          CD-4611747 1602632282793404                   173346.6-465202174116.3-465519344.13-08.62 5.79  -0.01 -0.06 +0.01C  A0V                +0.012-0.009      -012
6573 26    DraBD+61 1678 160269 17546       10660           173357.4+615704173459.5+615230 91.00 32.65 5.23  +0.61 +0.10         G0Va               +0.260-0.507 +.069-013V?     41  2.8   0.6AB   3*
6574 82    HerBD+48 2542 160290 46838    I                  173400.7+483837173637.6+483508 75.26 32.06 5.37  +1.15 +1.06 +0.61  gK1                 +0.031+0.065 +.017+029V?
6575          BD+02 3373 160315122607      W                173405.6+020507173908.5+020141 26.33 16.98 6.26  +1.03 +0.83 +0.53   K0III+F4IV         +0.040-0.021      +000SB         1.1 111.2AB   3*
6576          CD-5011474 160342244954              V626 Ara 173414.9-502726174203.9-503038341.06-10.56 6.24  +1.76 +1.89 +0.85E  M3+II-III          +0.024-0.012      -027                          *
6577          BD+13 3421 160365102999                       173422.1+132305173857.8+131945 37.09 21.99 6.12  +0.56               F6III              -0.023+0.038      +000SB     82
6578          BD-02 4425 160471141798    I          9461    173459.6-020552174011.8-020909 22.64 14.77 6.19  +1.67 +1.89         K2.5Ib             -0.023-0.018      -049V
6579          BD+32 2964 160507 66214                       173507.8+324744173849.7+324422 57.29 28.67 6.37R                    gG5                 +0.017-0.011      -015
6580   Kap ScoCD-3812137 160578209163 660          Kap Sco  173534.1-385842174229.3-390148351.04-04.72 2.41  -0.22 -0.89 -0.22   B1.5III            -0.006-0.027      -014SB    131                 *
6581 56Omi SerBD-12 4808 1606131607473405          Omi Ser  173547.6-124919174124.9-125231 13.28  9.17 4.26  +0.08 +0.10 +0.02   A2V                -0.069-0.052 +.006-030SB2   125                 *
6582   Eta PavCP-64 3662 160635254020 661                   173555.0-644033174544.0-644326328.40-17.75 3.62  +1.19 +1.17         K2II               -0.014-0.054 +.025-008
6583          CD-3611804 160668209172                       173603.6-365342174251.0-365645352.86-03.69 5.54  +1.55               K5III              +0.001-0.034      -004
6584          BD+31 3075 160677 66231    I                  173610.8+311519173957.5+311209 55.68 28.02 6.03  +1.58 +1.98         M2IIIab            -0.012+0.015      -009V?
6585   Mu  AraCD-5111094 160691244981 662                   173612.2-514650174408.7-515003340.06-11.50 5.15  +0.70 +0.24 +0.34   G3IV-V             -0.014-0.194 +.077-009
6586          CP-57 8703 160720244989      W                173619.0-572953174455.8-573243334.99-14.37 6.01  +0.90               G8III              +0.010-0.002      -006           0.0   0.2AP   3
6587          CD-3213208 160748209176                       173632.9-330008174306.9-330304356.21-01.70 6.40  +1.81 +2.00         M1III              +0.008+0.003      -003
6588 85Iot HerBD+46 2349 160762 46872 663  W       Iot Her  173638.4+460334173927.9+460023 72.32 31.27 3.80  -0.18 -0.69 -0.17   B3IV               -0.005+0.005 +.005-020SB1O   11  8.3 116.0      *
6589          BD+15 3246 160765103020                       173639.8+151347174111.0+151041 39.15 22.25 6.34  +0.03 +0.02         A1V                -0.012-0.015      -023       85
6590          BD+06 3498 160781122646                       173639.7+062150174132.3+061846 30.62 18.41 5.95  +1.26 +1.18         G7III              -0.005-0.010      -031
6591          BD+31 3076 160822 66243      W                173655.2+312027174041.2+311715 55.83 27.90 6.28  +1.05 +0.88         K0III              -0.067-0.073      -006           2.1 115.0
6592          BD+24 3225 160835 85310       10715           173659.0+243346174105.5+243048 48.64 25.71 6.36  +1.20 +1.18 +0.59   K1III+F4V          -0.019+0.058 +.002-032SB         3.0  16.3AB   3*
6593          CD-2711850 160839185655      D                173659.9-275009174317.8-275303  0.63  0.99 6.36  +0.47               A9V:               +0.010-0.006      -026                          *
6594          BD+16 3256 160910103033       10723           173729.3+155954174158.7+155707 40.00 22.38 5.52  +0.38 -0.05         F4V w              +0.009+0.101 +.036-044V?     30  2.8   0.6AB   3*
6595 58    OphBD-21 4712 1609151856601463  D                173726.2-213804174325.8-214100  5.94  4.21 4.87  +0.47 -0.03 +0.24   F6V                -0.095-0.043 +.060+011V       0  1.8   0.0      *
6596 28Ome DraBD+68  949 160922 17576 664  W                173732.1+684815173657.1+684529 99.08 31.87 4.80  +0.43 -0.01 +0.22   F5V                +0.002+0.323 +.045-014SBO    26  8.3  72.3      *
6597          CD-4212431 160928228363      W                173731.6-424101174442.0-424344348.08-06.99 5.87  +0.16 +0.09         A1V+F0IV           +0.005+0.019      -009           0.1   0.1      *
6598          BD+69  933 160933 17572                       173731.0+693756173639.7+693415100.04 31.79 6.42  +0.58 +0.09         F9V                -0.055-0.208 +.018-053V
6599          BD+43 2781 160950 46883                       173735.9+433111174037.6+432815 69.44 30.67 6.59R                     K2                 +0.056+0.061      -029
6600          BD-13 4732 161023160784                       173809.5-132734174348.6-133031 13.03  8.35 6.39  +0.37 -0.07         F0V                -0.048-0.109      -007    =< 15
6601          BD-07 4487 161056141832                       173823.1-070200174347.0-070446 18.67 11.58 6.30  +0.38 -0.48 +0.27   B1.5V              -0.001-0.006      -026V
6602 83    HerBD+24 3231 161074 85344    I W                173822.3+243652174228.4+243351 48.81 25.43 5.52  +1.46 +1.69 +0.80   K4III              -0.055-0.105 +.020-027           3.6 159.6
6603 60Bet OphBD+04 3489 161096122671 665I                  173831.9+043632174328.4+043402 29.21 17.19 2.77  +1.16 +1.24 +0.57   K2III              -0.040+0.159 +.033-012V   =< 17                 *
6604          BD+14 3321 1611491030523407                   173848.6+142025174322.0+141742 38.51 21.41 6.24  +0.42 +0.18         F5II               +0.006+0.023      -042                          *
6605          BD+57 1791 161162 305153406                   173856.4+572131174036.2+571837 85.58 32.00 6.77  +0.92 +0.58         K0                 -0.004+0.027      -014
6606          BD+72  800 161178  8862                       173902.4+723030173708.8+722721103.38 31.35 5.86  +1.01               G9III              +0.021+0.022 -.009+008V?
6607          BD+51 2243 161193 30522                       173904.1+515159174121.8+514905 79.13 31.60 5.99  +1.05              gK0                 -0.028-0.014      -009V?
6608 84    HerBD+24 3237 161239 85360                       173915.3+242216174321.6+241940 48.64 25.16 5.71  +0.65 +0.27 +0.33   G2IIIb             -0.115+0.071 +.006-026V?  =< 10
6609 61    OphBD+02 3390 161270122690       10750A          173932.7+023721174434.0+023446 27.50 16.04 6.17  +0.07 -0.01         A1IV-V             +0.004+0.014      -031SB    100  0.4  20.6AB   3*
6610          BD+02 3391 161289122691       10750B          173934.0+023720174435.4+023444 27.50 16.03 6.56                      A0V                +0.004+0.010      -030      115  0.4  20.6AB   3*
6611          BD+14 3329 161321103069       10749  V624 Her 173944.1+142712174417.3+142437 38.72 21.25 6.19  +0.21 +0.20 +0.12   A3m                +0.001+0.018      -031SB2O   35  5.7  39.7      *
6612          BD+44 2757 161369 469063408I                  174007.6+440741174305.6+440504 70.23 30.34 6.34  +1.54 +1.84         K4III              -0.040+0.034      -060
6613          CD-3812189 161390209246      W                174015.4-380418174707.3-380642352.30-05.02 6.43  -0.02               B9V+A1V            -0.002-0.007D+.009+001           0.3   0.1      *
6614          CP-55 8312 1614202450443412                   174017.7-552154174838.1-552406337.19-13.80 6.11  +0.28               A9IV               -0.018+0.036      -021
6615   Iot1ScoCD-4011838 161471228420 666  W                174035.4-400517174735.1-400737350.61-06.13 3.03  +0.51 +0.27         F2Iae               0.000-0.008 +.019-028SB     36  9.9  37.5
6616  3    SgrCD-2711930 1615921857551464I         X Sgr    174115.9-274734174733.6-274951  1.17  0.21 4.54  +0.80 +0.50         F7II               -0.001-0.009 +.035-013V      24                 *
6617          BD-22 4423 161664185765    I          9682    174143.0-222626174745.6-222841  5.77  2.94 6.18  +1.49               G6IbHdel 1         +0.003-0.011      -022V
6618          BD+53 1978 161693 30538                       174153.8+535037174359.3+534806 81.49 31.36 5.75  +0.01 +0.05  0.00   A2V                +0.022-0.015      -002V     130                 *
6619          BD+31 3090 161695 663113410                   174154.5+313239174540.3+313017 56.40 26.94 6.23   0.00 -0.22         A0Ib               +0.011-0.005      +002
6620          BD-14 4770 1617011608223413                   174154.5-144118174736.8-144333 12.44  6.93 5.94  +0.05 -0.32         B9pHgMn            -0.019-0.020      -025SB2O   16                 *
6621          CD-2612367 161756185779              V3894 Sgr174212.6-265621174827.8-265830  2.00  0.48 6.35  +0.12 -0.42         B4IVe               0.000-0.012      +009V                         *
6622          CP-53 8799 161783245065      W       V539 Ara 174220.0-533445175028.2-533644338.94-13.20 5.92  -0.08 -0.64         B2V+B3V            -0.014-0.006      -008SB2O   91  3.0  12.3      *
6623 86Mu  HerBD+27 2888 161797 85397 667I  10786           174232.6+274645174627.5+274314 52.45 25.63 3.42  +0.75 +0.39 +0.38   G5IV               -0.311-0.751 +.108-016V      20  6.4  33.8AxBC 4*
6624          CP-60 6950 161814254057      W                174235.6-600756175135.5-600952332.99-16.33 5.78  +1.00               K0III              -0.005-0.028      +017           7.2  30.
6625          BD+38 2997 161815 663153411                   174233.9+385514174553.7+385253 64.48 28.79 6.52  +1.00 +0.68         K0                 +0.003-0.028      -012
6626          BD+39 3219 161832 66317       10782  V826 Her 174240.3+392136174558.5+391921 64.97 28.88 6.68  +1.39 +1.46         K3III+F7V          +0.012+0.016      -032SBO        3.1   7.9AB   3*
6627          BD+17 3334 161833103106       10795           174243.1+174402174708.0+174150 42.25 21.93 5.72R +0.01  0.00         A1V                +0.011-0.007D+.004+003SB     54  1.3   0.6      *
6628          CD-3114609 161840209303                       174240.7-314008174910.5-314212358.03-02.09 4.83  -0.04 -0.29         B8V                +0.011-0.015      -013       45
6629 62Gam OphBD+02 3403 161868122754 668                   174252.6+024441174753.6+024226 28.02 15.36 3.75  +0.04 +0.04  0.00   A0Vnp              -0.022-0.074 +.039-007SBO   205                 *
6630          CD-3711907 161892209318 669  W                174303.0-370041174951.5-370236353.50-04.94 3.21  +1.17 +1.19         K2III              +0.049+0.033 +.040+025          11.5  41.7AC   3
6631   Iot2ScoCD-4011886 161912228466      W                174311.4-400329175011.2-400526350.89-06.54 4.81  +0.26  0.00         A2Ib               +0.007-0.008 -.001-010V?     39  6.1  32.6
6632          CP-53 8812 161917245072                       174305.6-530556175111.0-530750339.43-13.06 6.09   0.00               B9.5III-IV         +0.006-0.013      +019                          *
6633          BD+03 3493 161941122766                       174321.6+035018174820.2+034815 29.08 15.76 6.22  +0.16 -0.02         B9.5V              +0.007+0.004      -044    =< 41
6634          CP-65 3507 161955254063                       174318.6-652730175318.4-652921328.03-18.80 6.49  +1.08 +1.00         K0-1III            -0.017-0.083      -045
6635          CP-76 1226 161988257542      W                174331.9-760918175741.8-761039317.57-23.16 6.07  +1.20 +1.28         K2III              +0.002+0.017      +037           7.9  25.5
6636 31Psi1DraBD+72  804 162003  8890 670   10759A          174342.9+721153174156.3+720856102.97 31.03 4.58  +0.42 +0.01 +0.23   F5IV-V             +0.025-0.267 +.054-010V?     14  1.2  30.2AB   4*
6637 31Psi1DraBD+72  805 162004  8891       10759B          174344.6+721222174158.0+720925102.98 31.03 5.79  +0.53 +0.02         G0V                +0.030-0.278 +.054-010V?  =< 25  1.2  30.2AB   4*
6638          BD+20 3570 162076 854291465                   174407.3+203555174824.8+203356 45.25 22.73 5.69  +0.94 +0.71 +0.47   G5IV               +0.026+0.008      -026
6639          BD+02 3406 162113122787                       174416.3+015929174918.9+015740 27.50 14.70 6.47  +1.24 +1.26         K0III              -0.025+0.065      -058
6640          CD-4511958 162123228490                       174420.8-453417175144.5-453602346.21-09.52 6.11  +0.95               G6III              +0.009-0.005      -007
6641          BD+47 2537 162132 46954      S                174427.0+473848174708.1+473644 74.39 30.19 6.43  +0.11 +0.06         A2V s              -0.004+0.005      -026SBO              0.1      *
6642          BD+19 3435 162161103131                       174427.2+191714174847.9+191519 43.97 22.16 6.12  +0.02 -0.09         A1V                -0.006+0.022      -025SB?    66
6643          CD-4011905 162189228489      W                174430.6-404431175132.8-404621350.43-07.10 5.96  +1.59 +1.83 +0.95E  M2III              -0.028-0.053      -085           6.5  23.6
6644 87    HerBD+25 3353 162211 854373415I                  174445.8+253922174849.2+253722 50.44 24.44 5.12  +1.16 +1.12 +0.58   K2III              -0.003-0.039 +.016-026     < 17
6645          CD-3014802 162220209357      W                174446.3-303139175112.5-303326359.24-01.88 6.66  +0.04 -0.07         A0Vn               +0.002-0.011      -009           1.5  10.2      *
6646          CP-81  799 1623372587873990                   174526.2-812827180526.7-812911312.15-25.13 6.35  +1.50 +1.75         K3-4III            +0.018-0.040      -003
6647          CD-3412165 162374209383              V957 Sco 174533.5-344619175213.6-344757355.69-04.22 5.90  -0.10 -0.64         B6V               v+0.004-0.004      -013V    < 40                 *
6648          CD-3412170 162391209390                       174540.9-342324175219.7-342500356.03-04.04 5.84  +1.13               G8III              -0.001+0.003      -021V
6649          CD-4112139 162396228510                       174543.7-415755175252.7-415948349.48-07.92 6.20  +0.54               F8IV-V             +0.154-0.196 +.027-014
6650          BD+12 3305 162468103145                       174604.5+115832175043.5+115648 37.02 18.81 6.17  +1.25 +1.26         K1III-IV           -0.022-0.024      -049
6651          CD-3412186 162496209397                       174611.4-340521175249.2-340651356.34-03.98 6.06  +1.23               K1III              +0.001+0.017      -015V?
6652          CD-3412187 162515209401                       174614.8-345937175255.9-350107355.57-04.45 6.45  +0.02 -0.07         B9.5V              +0.015+0.007      -009SB2O   95                 *
6653          CD-3512013 162517209404                       174614.5-353551175257.9-353727355.05-04.76 6.03  +0.34 +0.09         F2III-IV           +0.046-0.047      -021
6654          BD+29 3126 162555 85464                       174630.2+292056175022.9+291920 54.42 25.32 5.50  +1.05       +0.55   K1III              +0.029+0.049      -015
6655          BD+22 3227 162570 85468                       174635.7+222039175048.4+221859 47.23 22.86 5.98R                     A9V                +0.014-0.016      +004      212
6656 30    DraBD+50 2468 162579 305913416  S                174641.0+504817174904.3+504652 78.07 30.30 5.02  +0.02 +0.01         A2V                -0.048+0.211 -.002-055V     140        0.1
6657          CD-3412200 162586209411      W                174639.6-344223175319.6-344350355.86-04.38 6.17  -0.03 -0.35         B8V                +0.005-0.001D+.005-012       37  0.0   0.1      *
6658          CD-3412203 162587209416      W                174642.9-345215175323.3-345343355.72-04.47 5.60  +1.14 +1.01         K3III              -0.009-0.010D+.003-007SB         0.1   0.4      *
6659          BD-01 3412 1625961419133418                   174649.2-011240175159.5-011412 24.91 12.61 6.35  +1.12 +0.89         K0                 -0.002+0.003      -044SBO                       *
6660          CD-3412219 162678209425                       174705.3-344545175345.5-344709355.86-04.48 6.38   0.00 -0.06         B9V                +0.009-0.003      -015V      37                 *
6661          BD-06 4672 162714141926    I:        Y Oph    174717.0-060709175238.8-060837 20.60 10.12 6.21  +1.40 +1.01 +0.95   F8Ib-G3Ib         v+0.008-0.009 -.006-005SB                        *
6662          CD-3412226 162724209428      W       V906 Sco 174714.7-344347175354.9-344509355.90-04.49 5.96  -0.01 -0.10         B9V+B9V            +0.021-0.003      -005SB2O   95  0.8   0.3      *
6663          CD-3412228 162725209430              V951 Sco 174717.8-344830175358.1-344954355.84-04.54 6.42  +0.01 -0.06         ApSi*              +0.007-0.023      -017       34                 *
6664 88    HerBD+48 2581 162732 469973417          V744 Her 174726.3+482516175003.3+482339 75.36 29.82 6.68  -0.11 -0.41         BepShell          v+0.004+0.012      -012SBO   300                 *
6665          BD+15 3292 162734103161       10850           174727.7+152100175158.5+151933 40.41 19.92 6.46R                     K0III               0.000+0.031D+.004-043V?         0.2   0.8AB   6*
6666          BD-10 4560 162757160885                       174730.1-105230175303.6-105359 16.45  7.70 6.18  +1.11              gK1                 +0.055-0.033      -035V?
6667          BD+01 3528 162774122861    I          9800    174731.3+011946175235.4+011818 27.29 13.67 5.95  +1.58 +1.92         K5III              -0.047-0.017      -065
6668          CD-3412244 162817209446                       174748.0-342642175427.2-342759356.21-04.44 5.96  -0.01 -0.10         B9.5V              +0.010+0.006      -016V?     65                 *
6669          BD+40 3225 162826 47009                       174759.1+400550175114.0+400421 66.06 28.06 6.46R                     G0V                -0.012+0.012      +002    =<  6
6670          BD+06 3566 1629171228803420                   174821.9+060718175314.2+060605 31.78 15.71 5.77  +0.42 -0.01         F3-5IV-V           -0.122+0.069 +.038-033SB     30
6671          CD-3612008 162926209458                       174821.9-362720175507.9-362833354.53-05.57 6.06  +0.08               B9.5III            -0.014-0.009      -015
6672          CD-2413615 162978185928                       174844.7-245202175454.0-245314  4.54  0.30 6.20  +0.04 -0.89         O7.5II((f))        -0.005-0.001      -011       50:                *
6673          BD+40 3228 162989 664023419I                  174849.4+400014175204.7+395856 66.01 27.88 6.04  +1.33 +1.49        gK4                 -0.011+0.054 +.015-066SB
6674          BD+46 2379 163075 47023                       174914.7+464010175201.0+463836 73.44 29.22 6.38  +1.09 +0.77         K0III              +0.039-0.122      -028
6675          CD-4412201 1631452285623425                   174929.5-441931175647.4-442032347.76-09.69 4.86  +1.21 +1.22         K2III              -0.006-0.017 +.017+045V?
6676          BD+11 3283 1631511031943422  W                174933.5+110918175414.2+110750 36.62 17.68 6.38  +0.45               F5Vn               -0.058-0.165 +.007-041SB    120  0.2   0.1      *
6677 90    HerBD+40 3233 163217 47037    I  10875   9834    175002.7+400136175318.1+400029 66.09 27.66 5.16  +1.18 +1.12 +0.61   K1IIIbCN-1         +0.011+0.053 +.014-035V?   < 19: 3.3   1.6      *
6678          CD-4012001 163234228564                       174955.0-401723175655.8-401820351.35-07.76 6.43  +1.43 +1.65         K3III              +0.016+0.005      -006
6679          BD-18 4686 1632451609093423                   175002.0-184704175555.0-184808  9.92  3.16 6.52  +0.05 +0.05         A4V                +0.016-0.017      +004V                         *
6680          CD-2813878 163318185975                       175023.0-280257175641.9-280355  1.99-01.65 5.80  +0.21               A7III/V:           +0.040-0.017      +007
6681          BD-15 4722 163336160915       10891           175033.8-154741175619.0-154845 12.56  4.57 5.89  +0.05               A1V                -0.010-0.063      -024V      56  3.2  20.6AB   3*
6682          CD-4112231 163376228578                       175041.4-414207175747.7-414258350.18-08.59 4.88  +1.65 +1.97 +0.81E  M0III              -0.015-0.010 -.005+004
6683          CD-3912058 163433209524      W                175101.8-390721175757.8-390813352.48-07.37 6.29  +0.01               A0IV-V             -0.005-0.039      -001           1.5   0.7
6684          BD+00 3813 163472122935              V2052 Oph175112.6+004108175618.4+004013 27.16 12.55 5.82  +0.09 -0.65         B2IV-V             -0.005+0.003      -017SB    120                 *
6685 89    HerBD+26 3120 163506 855451468I         V441 Her 175123.0+260357175525.2+260300 51.43 23.19 5.46  +0.34 +0.27 +0.21   F2Ib              e+0.004+0.006 -.004-029SB     23                 *
6686          BD-04 4376 163532141979    I                  175130.9-040403175647.7-040455 22.94 10.21 5.47  +1.16 +1.06        gG9                 -0.012-0.008      -039V
6687          BD+22 3237 163547 855523427I                  175138.7+222846175550.8+222751 47.84 21.83 5.58  +1.24              gK3                 -0.002+0.002      -044
6688 32Xi  DraBD+56 2033 163588 30631 671I W                175147.9+565318175331.7+565222 85.17 30.23 3.75  +1.18 +1.21 +0.59   K2-III             +0.094+0.080 +.035-026V?   < 17 11.1 316.       *
6689          BD+00 3816 163624122950       10912           175156.9+000449175704.3+000359 26.70 12.10 5.97  +0.10 +0.12         A3V                +0.028-0.016D+.004-011           0.2   0.2
6690          BD+06 3578 163641122949                       175203.6+063004175656.0+062916 32.57 15.06 6.29   0.00 -0.25         B9III              +0.026+0.002      -014       80
6691          CD-3612060 163652209545      W                175208.0-365053175855.6-365130354.57-06.42 5.74  +0.90               G8III              +0.008+0.013      -087V?         2.4  51.7      *
6692          CD-2813936 163685186025      D                175218.3-284453175839.1-284533  1.61-02.37 6.01  -0.08 -0.56         B3IV                0.000-0.009      -009           1.0   0.3      *
6693          CD-3015035 163755          I W     A  9929    175239.9-301434175905.2-301511  0.36-03.19 5.16  +1.77 +1.95         M1Ib               +0.002-0.010 +.012-020SB         1.8   5.5AB   3*
6694          CD-3015035 163756209553      W     B  9929    175240.3-301436175905.5-301512  0.36-03.19 7.04  +1.06 +0.80         G8II               -0.020-0.002 +.012-023V          1.8   5.5AB   3*
6695 91The HerBD+37 2982 163770 66485 672I          9890    175249.3+371549175615.2+371502 63.26 26.42 3.86  +1.35 +1.46 +0.63   K1IIaCN+2         e+0.004+0.006 +.002-027V    < 19:                *
6696          BD+11 3299 163772103243                       175245.7+110324175726.9+110239 36.89 16.93 6.36  +0.11 +0.08         A1V                -0.017-0.024      -016SB    112
6697          BD+24 3283 163840 85575      S                175306.5+240020175714.3+235945 49.51 22.08 6.30  +0.63               G2V                -0.024+0.076      -034SB  =<  6  1.5   0.1      *
6698 64Nu  OphBD-09 4632 163917142004 673I                  175331.2-094541175901.6-094625 18.17  6.98 3.34  +0.99 +0.88 +0.48   K0IIIaCN-1         -0.007-0.116 +.021+013     < 17
6699          BD+55 1995 163929 306453428                   175333.7+555853175523.7+555817 84.15 29.90 6.10  +0.32 +0.08         F0IV               +0.037+0.126      -027       82                 *
6700  4    SgrCD-2313731 1639551860613430                   175341.2-234825175947.6-234858  6.02-00.13 4.76  -0.04 -0.05 -0.02   B9V                +0.001-0.047 +.010-018V     191
6701 35    DraBD+76  667 163989  8939 675                   175355.4+765834174927.0+765746108.41 29.86 5.04  +0.49 +0.08         F6IV-V s           +0.037+0.248 +.032-023       11
6702          BD+45 2627 163990 47080    I         OP Her   175356.2+452147175648.4+452103 72.16 28.17 6.02  +1.64 +1.52 +1.90   M5IIbS             +0.008-0.028      +013V                         *
6703 92Xi  HerBD+29 3156 163993 85590 674I          9927    175352.7+291531175745.9+291452 54.91 23.77 3.70  +0.94 +0.70 +0.46   G8+III             +0.084-0.017 +.021-002     < 19:
6704          BD-20 4940 164028186070      D                175403.1-201954180000.1-202021  9.06  1.56 6.21  +1.40               K0II-III           +0.005-0.007      -008V                         *
6705 33Gam DraBD+51 2282 164058 30653 676I  10923           175417.0+513002175636.4+512920 79.06 29.22 2.23  +1.52 +1.87 +0.85   K5III             v-0.008-0.019 +.025-028     < 17  8.4 140.1AG   7*
6706          BD-04 4384 164064142012    I                  175418.1-044841175936.7-044917 22.62  9.24 5.87  +1.56 +1.92         K5III              -0.014-0.093      -032                          *
6707 94Nu  HerBD+30 3093 164136 66524              Nu  Her  175440.5+301151175830.2+301122 55.94 23.92 4.41  +0.39 +0.15 +0.23   F2II                0.000+0.007 +.007-022SB?    28                 *
6708          CD-3612115 164245209608                       175502.5-362225180148.3-362240355.28-06.69 6.30  -0.03 -0.33         B7IV               -0.007-0.017      +014
6709          BD+00 3832 164258123004              V2126 Oph175509.6+003806180015.5+003746 27.59 11.66 6.37  +0.15 +0.18         A3pSrCrEu         v-0.006+0.004      -034V?     65                 *
6710 57Zet SerBD-03 4217 164259142025                       175511.9-034102180029.0-034125 23.73  9.59 4.62  +0.38  0.00 +0.19   F2IV               +0.147-0.045 +.048-043SB     70
6711          BD+36 2986 164280 66531                       175512.9+361748175842.3+361716 62.37 25.69 6.00  +0.94              gG5                 +0.007-0.055      +010
6712 66    OphBD+04 3570 164284123005              V2048 Oph175518.6+042227180015.8+042207 30.99 13.37 4.64  -0.03 -0.83 -0.02   B2Ve               +0.003-0.013      -013SB    221                 *
6713 93    HerBD+16 3335 1643491032851469I                  175536.3+164523180003.4+164503 42.62 18.72 4.67  +1.26 +1.22 +0.59   K0.5IIb            -0.007-0.010 +.004-024     < 17                 *
6714 67    OphBD+02 3458 164353123013 677   10966           175538.1+025611180038.7+025554 29.73 12.63 3.97  +0.02 -0.62  0.00   B5Ib               +0.002-0.008 -.002-004V      22  4.3  54.5AC   5*
6715  6    SgrBD-17 4987 1643581609981470I                  175534.4-170911180123.1-170925 11.99  2.85 6.28  +1.80               K3III              +0.003-0.005      -022V?
6716          BD-22 4503 164402186135    I: 10983   9996    175550.8-224639180154.4-224650  7.16-00.03 5.77   0.00 -0.89         B0Ib               -0.003-0.008      -013      100  7.1   8.2AB   3*
6717          BD+78  616 164428  8946    I          9859    175548.4+781924175010.5+781824109.94 29.62 6.24  +1.44               K5                 +0.018+0.018      -007
6718          BD+45 2635 164429 47106              V771 Her 175600.8+452854175852.3+452834 72.38 27.84 6.48  -0.08 -0.22         B9pSiSr            +0.002+0.027      -019V?    200:                *
6719          BD+06 3597 1644321230173432                   175600.1+061620180052.9+061606 32.81 14.09 6.34  -0.08 -0.75         B2IV               +0.002-0.008      -014SB?
6720          BD+19 3494 164447103291              Var?     175607.5+193036180027.7+193021 45.34 19.71 6.50  -0.06 -0.39         B8Vne              +0.015-0.001      -029      210:                *
6721   Chi OctCP-87  274 164461258799 922                   175601.9-873951185446.9-873621305.62-27.14 5.28  +1.28 +1.60         K3III              -0.039-0.139      +034
6722          BD+15 3327 164507103299                       175626.2+150558180057.2+150536 41.12 17.85 6.26  +0.69 +0.24         G5IV               -0.059-0.108      +004
6723 68    OphBD+01 3560 164577123035       10990  10009    175640.7+011827180145.2+011819 28.38 11.64 4.45  +0.02  0.00 +0.01   A2Vn               +0.015-0.012 +.021+006SB    252  2.5   0.6      *
6724  7    SgrCD-2413793 164584186163                       175643.4-241653180251.1-241656  5.96-00.96 5.34  +0.52 +0.26 +0.36   F3III              +0.005-0.008 +.007-012V      36                 *
6725 34Psi2DraBD+72  818 164613  89613429                   175655.0+720053175511.2+720018102.68 30.03 5.45  +0.30 +0.15         F2-3II-III         +0.013-0.001      -002SB     45
6726          BD+33 3006 164614 66551    I                  175656.5+331302180036.4+331250 59.24 24.43 5.99  +1.51              gK6                 -0.003-0.019      -016
6727          BD-22 4516 164637186169                       175658.3-224307180301.7-224306  7.34-00.23 6.74  -0.05 -0.87         B0.5III            +0.003+0.007      -007SB     50                 *
6728          BD+45 2638 164646 471213433I          9980    175704.8+453021175956.2+453005 72.45 27.66 5.67  +1.57 +1.91         M0IIIab            +0.008-0.025 +.026-010V?
6729 95    HerBD+21 3280 164668 85647    I  10993B 10014    175715.3+213545180129.9+213543 47.50 20.29 5.18  +0.95 +0.57         G8III              +0.011+0.032 +.000-031V?  =< 25  0.1   6.3      *
6730 95    HerBD+21 3280 164669 85648       10993A 10014    175715.8+213546180130.4+213544 47.50 20.29 4.96  +0.12 +0.13         A5IIIn             +0.012+0.035 +.000-030V?    180  0.1   6.3      *
6731          CP-75 1410 164712257569 678  W                175716.2-755337181115.7-755329318.15-23.85 5.86  +1.24 +1.43         K2III              +0.010-0.295 +.016+015           7.3  25.9
6732          BD-05 4560 164716142045                       175726.3-052129180246.3-052131 22.52  8.29 6.76  +0.16 -0.10         B9V                +0.011-0.035      -014
6733 69Tau OphBD-08 4549 164764142050       11005B 10073    175738.1-081049180305.0-081049 20.06  6.87 5.94                      F5V                +0.029-0.037 +.058-038           0.7   1.9AB   3*
6734 69Tau OphBD-08 4549 164765142050       11005A 10073    175738.1-081049180304.9-081050 20.06  6.87 5.24  +0.38 +0.04         F2V                +0.028-0.037 +.058-040SB     25  0.7   1.9AB   3*
6735          BD+75  647 164780  8962                       175744.4+751047175426.6+751015106.32 29.79 6.36  +0.98               K0                 -0.013+0.023 -.019-018
6736  9    SgrCD-2413814 164794186204    I                  175744.5-242146180352.4-242138  6.01-01.20 5.97   0.00 -0.89 +0.02   O4V((f))           +0.002+0.003      +009SB    140                 *
6737          BD+33 3009 164824 665623434I  10998           175756.1+331840180135.9+331841 59.41 24.26 6.15  +1.55               K5III              +0.025+0.023      -010V          6.0  14.2AB   3*
6738 96    HerBD+20 3649 164852 85672              V820 Her 175806.5+205000180223.1+205001 46.83 19.81 5.28  -0.09 -0.61         B3IV               +0.005-0.010 +.004-015SB2O  205                 *
6739          CD-3512229 1648702096713435  W                175806.3-355415180450.4-355405355.99-07.00 6.00  +1.16               K2III              -0.016-0.035      -016           5.2  12.5
6740          CP-64 3796 164871254141                       175800.3-643320180748.3-643300329.59-19.84 6.41  +1.26 +1.41         K2-3III            -0.015-0.053      -022
6741 97    HerBD+22 3260 164900 85676                       175819.2+225521180230.1+225523 48.91 20.57 6.21  -0.10 -0.64         B3Vn               -0.004-0.008      -036SB
6742   Gam1SgrCD-2914447 164975186237       11029  W Sgr    175837.9-293504180501.3-293448  1.58-03.98 4.69  +0.78 +0.52 +0.45   F4-G1Ib           v+0.013 0.000      -028SBO    25  0.0   0.1     4*
6743   The AraCD-5011720 1650242452421471                   175850.7-500552180637.9-500530343.33-13.82 3.66  -0.08 -0.85         B2Ib               -0.010-0.014      +003      117                 *
6744          BD+19 3508 165029103334                       175854.8+193638180314.7+193647 45.72 19.15 6.50R +0.01 -0.01         A0V                +0.024-0.008      -032      171
6745   Pi  PavCP-63 4292 1650402541473437          10204    175857.1-634020180834.8-634006330.49-19.59 4.35  +0.22 +0.18         A7pSr              +0.013-0.188 +.036-016SB     20                 *
6746 10Gam2SgrCD-3015215 165135209696 679I         10173    175923.0-302531180548.5-302527  0.92-04.54 2.99  +1.00 +0.77 +0.51   K0III              -0.053-0.185 +.025+022SB                        *
6747          BD+01 3578 165174123090              V986 Oph 175934.4+015451180437.3+015509 29.27 11.29 6.14   0.00 -0.91         B0IIIn             -0.002-0.003      +017SB?   434                 *
6748          CD-3612214 165185209710                       175938.2-360139180623.7-360111356.04-07.33 5.95  +0.62 +0.07         G5V                +0.110+0.015 +.059+013
6749          CD-4312272 165189228708      W     A          175935.9-432548180649.8-432529349.45-10.86 5.77H +0.22 +0.16         A5V                -0.001-0.099D+.021-006SB         0.1   1.6      *
6750          CD-4312272 165190            W     B          175935.9-432548180649.8-432529349.45-10.86 5.77H                     A5V                -0.001-0.099D+.021-006           0.1   1.6      *
6751          CP-73 1888 165259257571      W                175952.0-734050181234.5-734018320.49-23.29 5.85  +0.46 +0.05         F5V                -0.053-0.231 +.031+013           3.0   2.8
6752 70    OphBD+02 3482 165341123107    I  11046           180024.1+023122180527.3+022958 29.91 11.38 4.03  +0.86 +0.54 +0.46   K0V                +0.266-1.093 +.201-007SBO    16  1.8   1.9AB  11*
6753          BD+48 2627 165358 47173       11028           180032.1+482734180309.0+482752 75.85 27.69 6.21  +0.03 +0.06         A2V                +0.029+0.015      -013           2.7  26.8AB   3*
6754          BD+23 3254 165373 85706                       180032.0+235615180440.2+235633 50.11 20.48 6.34  +0.30 +0.03         F0IV-V             +0.011-0.057      -033SB     70
6755          BD-08 4558 1654021420833436                   180040.5-081954180607.4-081926 20.30  6.14 5.85  +0.21 -0.20         B8III-IV           -0.008-0.016      -023SB                        *
6756          BD-04 4395 165438142085                       180055.7-044533180615.2-044505 23.48  7.82 5.77  +0.96 +0.81         K1+IV              +0.141-0.032      -019V
6757          BD-00 3414 165462142084                       180059.1-002716180607.4-002648 27.31  9.86 6.34  +1.06 +0.76         G8IIp              -0.020-0.029      -009
6758          BD+12 3383 165475103373       11056           180104.3+115944180543.3+120014 38.69 15.51 7.04  +0.30 +0.12         A3IV               -0.008+0.001D+.009+013SB    218  0.5   6.9AB   3*
6759          CD-4512215 165493228734      W                180105.4-454642180830.1-454602347.45-12.19 6.15  -0.08 -0.49         B7-8II             -0.009-0.025      -035V?         3.6   4.0      *
6760          CP-59 7231 165497245273      W                180107.7-590313180957.6-590224335.07-17.98 6.38  +1.55               K4III              -0.021-0.001      -010           5.6  40.
6761   Iot PavCP-62 5797 165499254157                       180108.5-620121181026.1-620008332.21-19.17 5.49  +0.58 +0.09         G0V                -0.087+0.223 +.053+030SB
6762          BD-21 4855 1655161863023438                   180111.4-212715180711.4-212638  8.93-00.44 6.28  +0.12 -0.78         B0.5Ib             +0.004 0.000      -009V                         *
6763          BD+21 3300 165524 85718                       180115.7+213818180530.2+213849 47.92 19.44 6.15  +1.23              gK3                 +0.015+0.009      -035
6764          BD+40 3276 165567 47193                       180128.0+400434180443.2+400503 66.78 25.56 6.52  +0.46 +0.02         F7V                +0.030+0.025      -001       12
6765 98    HerBD+22 3273 165625 85725    I         10208    180149.1+221234180601.9+221308 48.53 19.55 5.06  +1.58 +1.93 +0.98E  M3-IIIZrO 0+       -0.010-0.008 +.014-020V
6766          CD-2814174 1656341863283439I D                180144.9-282806180805.0-282726  2.89-04.02 4.57  +0.94 +0.74 +0.53   G7:IIIb*           +0.027-0.030 +.023-005V?         0.8   0.3      *
6767          BD+41 2968 165645 47195       11054           180153.6+415607180500.8+415648 68.79 25.97 6.34  +0.26 +0.04         F0V                -0.021+0.106      -020SB    139  3.6  23.4
6768          BD+32 3047 165683 66626                       180206.3+321318180549.6+321350 58.59 23.08 5.71  +1.16 +1.08         K0III              +0.012-0.026      +001
6769          BD-17 5028 165687161093    I                  180200.4-171004180748.4-170915 12.75  1.51 5.52  +1.11 +1.08 +0.56   K1III              -0.099+0.059      -032V?
6770 71    OphBD+08 3582 165760123140    I                  180231.4+084316180718.4+084402 35.82 13.74 4.64  +0.96 +0.74 +0.50   G8III              +0.011+0.032 +.021-003     < 19:
6771 72    OphBD+09 3564 165777123142 680   11076           180236.5+093258180721.0+093350 36.59 14.09 3.73  +0.12 +0.10 +0.05   A4IV s             -0.060+0.080 +.047-024SB2    80  7.7  54.4AD   4*
6772          CD-3612265 165793209779                       180235.6-364113180922.4-364021355.75-08.17 6.58  -0.03 -0.86         B1II               -0.011-0.004      +039V?
6773          CD-2512793 165814186350      W       V3792 Sgr180243.0-252913180854.1-252821  5.59-02.74 6.61  +0.02 -0.47         B3III              +0.003+0.018      -006SBO        2.0  13.3      *
6774          CP-70 2507 1658612575753446                   180254.1-704618181424.1-704505323.56-22.54 6.73  -0.03 -0.36         B7-8II-III         -0.002-0.023      -007
6775 99    HerBD+30 3128 165908 66648       11077           180313.7+303251180701.5+303343 56.97 22.30 5.04  +0.52 -0.09 +0.32   F7V                -0.094+0.071 +.060+001SB      3  3.4   1.7AB   3*
6776          BD+13 3529 165910103406       11086           180311.9+130327180748.4+130416 39.92 15.50 6.63  +0.07 +0.09         A2Vn               +0.016+0.008      -017V?         3.8  42.3      *
6777          CD-3213814 165978209794                       180326.7-324355180959.9-324311359.32-06.42 6.43  +1.02               K0IV               +0.001-0.149      +010
6778          CD-4712098 166006228774      W                180331.1-473148181104.5-473047346.05-13.35 6.07  +1.20 +1.45?        K1IIICNII          +0.007-0.029      -015           5.4   1.7
6779103Omi HerBD+28 2925 166014 85750 681  S       Omi Her  180338.4+284455180732.6+284545 55.19 21.59 3.83  -0.03 -0.07  0.00   B9.5V             e+0.001+0.009 +.005-030SB    134                 *
6780          CD-3015316 166023209803      W                180338.8-304440181005.7-304343  1.10-05.49 5.53  +0.97 +0.70         K1II               +0.003-0.027D+.004-022           2.3   3.9      *
6781100    HerBD+26 3178 166045 85753       11089A 10253    180347.6+260511180749.5+260605 52.54 20.60 5.86  +0.12 +0.08         A3V                -0.006+0.033D+.012-017V?    176  0.0  14.3AB   4*
6782100    HerBD+26 3178 166046 85752       11089B 10253    180347.5+260456180749.5+260551 52.53 20.59 5.90  +0.14 +0.08         A3V                -0.002+0.033D+.012-016      168  0.0  14.3AB   4*
6783   Eps TelCD-4512251 1660632287771473  W                180348.4-455818181113.8-455716347.50-12.69 4.53  +1.01 +0.78 +0.48   K0III              -0.017-0.037 +.023-026           8.4  21.2
6784          BD+14 3427 1660951034143442                   180400.3+141610180833.7+141705 41.14 15.84 6.37  +0.17 +0.23 +0.08   A5m                -0.005+0.004      -009V      20
6785          BD-13 4863 1661031611251472I                  180402.7-135704180943.4-135604 15.79  2.67 6.39  +1.42               K0                 +0.016-0.003      -020V
6786          CD-4112491 1661142287783444                   180400.2-412235181105.6-412133351.68-10.62 5.86  +0.29 +0.07         F2V                +0.032-0.035      -004V
6787102    HerBD+20 3674 166182 857693443   11102           180428.8+204755180845.5+204852 47.42 18.42 4.36  -0.16 -0.81 -0.18   B2IV               -0.001-0.005 -.010-015V?     33  7.9  23.4      *
6788          CD-3312917 166197209817              10304    180418.4-334907181055.2-334759358.45-07.10 6.16  -0.14 -0.86         B1V                -0.011+0.013      -025SB    212
6789 23Del UMiBD+86  269 166205  2937 913                   180432.7+863648173212.9+863511119.29 28.24 4.36  +0.02 +0.03 -0.01   A1Vn               +0.009+0.056 +.004-008V?    174                 *
6790          BD+50 2525 166207 307513441                   180429.9+504823180653.5+504922 78.59 27.52 6.29  +1.04 +0.96 +0.52   K0III              +0.002+0.096      -057                          *
6791          BD+43 2892 166208 47237    I                  180427.9+432656180728.8+432742 70.55 25.90 5.00  +0.91 +0.71 +0.47   G8IIICN-1CH-3      +0.002-0.060 +.012-016SB?  < 17
6792          BD+49 2732 166228 47233       11090           180436.4+494144180706.3+494238 77.37 27.29 6.32R                     A2V                -0.002+0.022D+.008-021SB    210  4.1   2.4
6793          BD+36 3027 166229 66666    I                  180434.1+362328180802.2+362405 63.08 23.92 5.48  +1.17 +1.21 +0.56   K2.5III            -0.095-0.183 +.009-007
6794101    HerBD+20 3675 166230 85770                       180434.1+200146180852.9+200243 46.68 18.10 5.10  +0.15 +0.18         A8III              +0.008-0.020 +.015-016V?     54
6795 73    OphBD+03 3610 166233123187       11111  10287    180435.5+035835180933.8+035936 31.72 11.13 5.73  +0.37 +0.03         F2V                +0.038-0.007 +.030-017V      94  1.2   0.3AB   3*
6796          CP-63 4334 166251254179      W                180439.1-634240181416.2-634122330.71-20.19 6.47  +1.41               K4III              -0.033-0.051      -044           6.8  19.       *
6797          BD+03 3613 166285123198       11113           180453.8+030626180954.0+030711 30.97 10.66 5.69  +0.47 -0.01         F5V                +0.022-0.192 +.041-014SB2 =< 10  5.2 103.5AC   3
6798          BD-19 4886 166393161153       11127           180519.2-195141181114.8-195032 10.79-00.50 6.36  +0.16               A4V                -0.002-0.028D+.008-032           0.4   1.2AB   3*
6799          BD+30 3138 166411 66682                       180520.7+302653180910.2+302810 57.04 21.83 6.38  +1.19 +1.22 +0.61  gK1                 +0.071+0.133      -080
6800          BD+03 3620 1664601232123445I                  180540.5+031816181040.3+031927 31.24 10.58 5.51  +1.19 +1.22 +0.61   K2III              +0.021-0.003      +010
6801 11    SgrCD-2314047 166464186437    I  11133           180537.2-234317181143.4-234204  7.46-02.45 4.98  +1.05 +0.90         K0III              +0.017-0.019 +.014+004           5.7  42.1      *
6802          CD-2814268 166469186444              V4045 Sgr180536.8-285522181158.2-285405  2.90-04.98 6.51  -0.01               B9IVpSrEuCr        +0.017-0.005      -014                          *
6803          BD+16 3390 166479103443       11123  10311    180540.8+162727181008.7+162836 43.39 16.40 6.09R                     B9V+F7III          -0.004-0.007D+.004-013           0.8   1.2      *
6804          CD-4112534 166596228815              V692 CrA 180607.8-412134181312.6-412010351.89-10.97 5.47  -0.17               B2.5III            -0.013-0.004      -015V?    197:                *
6805          CP-63 4343 166599254189      W                180611.0-630452181540.9-630320331.40-20.11 5.60  +0.92               K0III-IV           -0.023-0.031      -007           5.6  41.9      *
6806          BD+38 3095 166620 66700              10316    180618.7+382705180937.5+382727 65.34 24.21 6.40  +0.87 +0.59 +0.49   K2V                -0.303-0.472 +.092-019V?                        *
6807          BD+36 3039 166640 66703                       180630.1+362646180959.0+362759 63.27 23.56 5.58  +0.91               G8III              +0.011+0.015      -026
6808          CP-68 3081 1668412541963452  W                180719.1-681534181800.9-681345326.26-22.05 6.33  -0.04 -0.23         B9V                -0.007-0.019      +022      224  3.5   2.0
6809 40    DraBD+79  570 166865  8994       11061B 10205    180731.5+795917180003.4+800003111.78 28.91 6.04  +0.51 -0.01         F7                 +0.045+0.129 +.025+004SBO =< 25  0.4  19.3AB   3*
6810 41    DraBD+79  571 166866  8996       11061A 10205    180737.7+795928180009.2+800015111.78 28.90 5.68  +0.50 -0.01         F7                 +0.042+0.122 +.025+010SB      0  0.4  19.3AB   3*
6811 24    UMiBD+86  272 166926  2940                       180747.5+865938173048.0+865805119.70 28.14 5.79  +0.25 +0.07         A2m                +0.059+0.010      +001       55
6812 13Mu  SgrBD-21 4908 166937186497 682   11169  Mu  Sgr  180746.9-210506181345.8-210332 10.00-01.60 3.86  +0.23 -0.49 +0.20   B8Iap             e+0.002+0.001 +.012-006SBO =< 54  2.9   0.0     6*
6813          BD-04 4415 1669601421593450                   180753.1-040216181310.0-040042 24.95  6.64 6.59  +0.27 +0.14 +0.14   A2m                +0.015+0.022      -011V      25
6814          BD+33 3044 166988 66733       11149  10363    180805.7+332521181145.1+332649 60.27 22.29 5.88R +0.01 +0.11         A3V                +0.012+0.010      -019SB    100  0.3   0.1AB   3*
6815104    HerBD+31 3199 167006 667373448I         V669 Her 180808.3+312249181154.2+312419 58.20 21.59 4.97  +1.65 +1.94 +0.97E  M3III              -0.013+0.030 +.007-000V                         *
6816 14    SgrBD-21 4916 167036186509    I         10393    180815.4-214424181415.9-214247  9.48-02.02 5.44  +1.52 +1.69         K3III              -0.008-0.022      -059V                         *
6817          BD+54 1950 167042 307843447                   180828.6+541524181031.6+541712 82.54 27.55 5.95  +0.94 +0.72 +0.48   K1III              +0.104+0.245 +.019-016V?
6818          CD-4412456 167096228851                       180834.7-441412181552.7-441224349.47-12.67 5.46  +0.96 +0.72?        G8-K0III           +0.074+0.012      -027V
6819          CP-56 8706 1671282453691474          Var?     180842.0-560316181707.5-560124338.39-17.69 5.33  -0.05 -0.69 -0.03   B3IIIep           v-0.006-0.013      +015SB     50                 *
6820          BD+21 3347 167193 85836    I                  180902.2+215106181316.5+215249 48.88 17.87 6.12  +1.47 +1.72 +0.80   K4III              +0.062+0.051      -066
6821          CD-5111460 1672572453723453                   180908.2-510559181700.9-510406343.13-15.73 6.06  -0.06               B9V                -0.002-0.008      -005V?
6822 15    SgrBD-20 5054 167264186543                       180914.9-204529181512.9-204342 10.46-01.74 5.38  +0.07 -0.86         B0Ia               +0.003+0.003      -006SB     83                 *
6823 16    SgrBD-20 5055 167263186544       11191           180915.9-202504181512.9-202317 10.76-01.58 5.95  +0.02 -0.87         O9.5II            v-0.002+0.001      -005SB2   160  7.0   6.0      *
6824          BD+41 3011 167304 47305                       180931.9+410716181242.6+410849 68.35 24.38 6.36  +1.03 +0.98         K0III              -0.014-0.046      -048
6825          BD-18 4864 167356161227       11196           180938.2-184131181530.8-183941 12.31-00.82 6.07  +0.20 -0.34?        A0Ia               +0.014+0.006      +002V          5.7  10.1      *
6826          BD+38 3113 167370 667653451                   180944.6+384444181304.8+384625 65.87 23.65 6.04  -0.08 -0.22         B9IIIn             -0.004+0.010      -009SB    250
6827          BD+60 1813 167387 178033449                   180954.8+602302181107.1+602434 89.43 28.26 6.49   0.00 -0.06         A1Vnn              -0.010 0.000      -018      300
6828          CP-63 4370 167425254209      W                180959.9-635454181940.2-635313330.73-20.81 6.18  +0.58 +0.07         F9V                +0.043-0.285 +.017+000           4.6   7.7
6829   Phi OctCP-75 1417 167468257584                       181009.0-750509182336.1-750239319.27-24.38 5.47  +0.02 +0.04         A0V                -0.006+0.030      +005      224
6830          BD-03 4259 167564142185                       181042.0-033901181558.0-033703 25.63  6.21 6.36  +0.20 +0.17         A4V                +0.015+0.019      -022
6831          BD+29 3213 167588 85856                       181051.0+291059181444.0+291226 56.22 20.27 6.56  +0.54 +0.01         F8V                +0.016-0.249 +.014+003    =<  6
6832   Eta SgrCD-3612423 167618209957 683I'W       Eta Sgr  181051.6-364730181737.6-364542356.44-09.68 3.11  +1.56 +1.71 +1.32   M3.5III           e-0.128-0.167 +.045+001V?         4.9   3.5AB   4*
6833          CD-3412673 167647209959      W       RS Sgr   181058.4-340830181736.1-340626358.83-08.47 6.16  -0.11 -0.60         B3V+A              -0.015-0.009      +010SBO   186: 2.6  94.1AC   4*
6834          BD+02 3547 167654123308    I         10466    181103.7+022043181605.6+022240 31.01  8.95 6.01  +1.59 +1.56         M4IIIab            -0.003-0.015      +022V?                        *
6835          CD-2814407 167666186594                       181103.6-284110181724.0-283909  3.69-05.92 6.19  +0.18               A5/7V              +0.004-0.027      -034
6836          CD-2814408 167665186593                       181103.4-281910181723.7-281721  4.01-05.74 6.40  +0.54  0.00         G0V                +0.134-0.156      +007SB
6837          CP-80  849 167714258796                       181118.0-801649182919.5-801358313.74-25.77 5.95  +1.16 +1.27         K2III              -0.037-0.062      -014
6838          BD-17 5112 1677201612603454I                  181122.5-172429181711.6-172226 13.63-00.56 5.75  +1.58 +1.70         K4II-III           -0.003-0.015      -007
6839          CD-4213101 167756228895                       181131.3-421930181840.0-421718351.47-12.30 6.30  -0.11 -0.99         B0.5Ia             -0.007-0.004      -025
6840          BD-03 4263 167768142189                       181138.5-030204181653.1-030026 26.29  6.29 6.00  +0.89 +0.54 +0.34E  G3III              +0.012-0.268 +.005+002V                         *
6841          BD-18 4886 167771161267      W                181136.6-182955181728.5-182748 12.70-01.13 6.54  +0.11 -0.84         O7III:(n)((f))     -0.003 0.000      +001SBO    90: 6.6   8.4      *
6842          CD-2712684 167818186612    I                  181147.6-270443181803.2-270233  5.18-05.29 4.65  +1.66 +1.81 +0.93   K3II               +0.008+0.005 +.033-017
6843          BD-09 4678 1678331421921475                   181153.7-094733181724.2-094531 20.36  3.01 6.31  +0.38 +0.27         A8V                +0.009-0.056      -014      120
6844          BD+00 3907 167858123320                       181159.8+005816181704.8+010021 29.89  8.10 6.63  +0.31 +0.05         F2V                -0.011-0.026      -034V   =< 15
6845          BD+42 3035 167965 47342 684                   181232.1+420730181538.8+420934 69.58 24.12 5.59  -0.10 -0.46         B7IV               +0.001 0.000      -021SB    201
6846          CD-2512995 167979186629    I:                 181230.4-253830181841.7-253617  6.53-04.75 6.51  +1.34 +1.40        gK1                 -0.004-0.030      -046V
6847          BD+45 2684 168009 47343                       181240.1+451042181532.6+451234 72.82 24.92 6.29  +0.62 +0.12         G2V                -0.068-0.110 +.044-064V?  =<  6
6848          BD-18 4896 168021161304       11240  10543    181250.8-183928181843.3-183710 12.70-01.47 6.84  +0.31 -0.65         B0Ib               +0.013+0.006D+.020+004V      76  0.5   0.4AB   5*
6849          BD+56 2080 168092 30836       11213           181255.9+563314181441.1+563518 85.21 27.31 6.37  +0.34 +0.02         F1V                -0.011+0.029      -008SB2O   55  3.4  95.6AB   4*
6850 36    DraBD+64 1252 168151 17828 685                   181319.2+642148181353.8+642350 93.97 28.29 5.03  +0.38 -0.04         F5V                +0.349+0.035 +.047-035        8
6851          BD+13 3593 1681991035783456  S                181328.2+134420181802.9+134637 41.67 13.54 6.30  -0.03 -0.51         B5V                -0.001-0.010      -021                 0.1
6852          BD+18 3623 168270103581                       181343.9+180535181807.7+180753 45.76 15.33 5.99R +0.03 -0.10         B9V                -0.006-0.004      -027SB     90
6853          BD+40 3332 168322 47359                       181356.4+405350181706.8+405612 68.38 23.52 6.11  +0.99 +0.70 +0.53   G8.5IIIbFe-1CH0.5  -0.163+0.069 +.010-073                          *
6854          BD+23 3299 168323 85906    I W                181357.6+231530181807.7+231748 50.71 17.38 6.63R                     K5                 +0.007-0.018      +003           2.3 155.4AC   3
6855   Xi  PavCP-61 6140 168339254226 686  W       10673    181400.6-613221182313.6-612938333.29-20.40 4.36  +1.48 +1.55         K4III              +0.001+0.003 +.016+012SBO        4.6   3.5AB   3*
6856          CD-3712457 168357210017                       181405.5-373148182055.3-372915356.07-10.59 6.45  +1.31               K2II               +0.011-0.008      +002
6857          BD+07 3629 168387123353    I  11254           181419.3+071309181909.5+071535 35.79 10.46 5.39  +1.07 +1.06 +0.55   K2III-IV           -0.047+0.002      -008V          6.7  42.2      *
6858          BD-15 4927 168415161348    I                  181423.3-155221182008.8-154954 15.32-00.45 5.39  +1.47 +1.64         K4III              +0.037-0.033      +031
6859 19Del SgrCD-2914834 168454186681 687I  11264           181435.5-295214182059.7-294941  3.00-07.15 2.70  +1.38 +1.55 +0.68   K3-IIIa*           +0.035-0.028 +.047-020V?        10.2  58.1AD   4*
6860105    HerBD+24 3381 168532 859213457I                  181503.7+242416181910.7+242646 51.92 17.60 5.27  +1.53 +1.74 +0.79   K3III:Ba0.4        +0.009+0.006 -.002-014SBO                       *
6861          CD-2414219 168574186699    I         V4028 Sgr181522.0-245736182131.4-245454  7.44-04.99 6.25  +1.84 +2.03 +1.43E  M5III              +0.007+0.008      +003V                         *
6862          CD-3812729 168592210048                       181524.9-384207182218.6-383925355.12-11.35 5.10  +1.49               K4-5III            -0.033-0.028 +.011+018
6863          BD-18 4926 168608161376              Y Sgr    181530.0-185416182123.1-185136 12.79-02.14 5.75  +0.94 +0.62         F8I                +0.011-0.009 -.006-003V      18                 *
6864          CD-2814495 168646186704                       181540.4-282832182200.2-282548  4.35-06.71 6.16  +0.26 +0.26         A3III              +0.014-0.001      -012                          *
6865 37    DraBD+68  984 168653 178373455                   181551.9+684311181517.0+684521 98.93 28.40 5.95  +1.06              gK1                 +0.015-0.061 -.002-010V
6866 74    OphBD+03 3680 1686561233771476I  11271           181552.5+031956182052.1+032238 32.46  8.34 4.86  +0.91 +0.62 +0.45   G8III               0.000+0.010 +.019+005V?   < 19: 7.3  28.1AB   3*
6867          BD+29 3236 168694 85933    I                  181600.5+293722181952.1+293958 57.09 19.38 5.99  +1.29              gK4                  0.000-0.001      -036
6868106    HerBD+21 3390 168720 85941    I                  181604.0+215508182017.9+215741 49.63 16.40 4.95  +1.59 +1.99 +1.00   M1III              +0.017-0.056 +.016-033SB
6869 58Eta SerBD-02 4599 168723142241 688I W       10675    181608.1-025529182118.6-025356 26.91  5.36 3.26  +0.94 +0.66 +0.50   K0III-IV           -0.547-0.700 +.058+009V?   < 19: 8.7 179.7      *
6870          CD-3612524 168733210061              V4050 Sgr181606.4-364258182253.1-364010356.99-10.58 5.34  -0.14               Ap                  0.000-0.018      -012SB?     0                 *
6871          CP-63 4406 168740254237                       181602.4-630409182531.5-630117331.84-21.15 6.14  +0.20               A3V                -0.007-0.099      -021      123                 *
6872  1Kap LyrBD+36 3094 168775 668691477I         10657    181621.4+360110181951.7+360352 63.52 21.55 4.33  +1.17 +1.19 +0.55   K2IIIabCN0.5       -0.015+0.043 +.011-022     < 17                 *
6873          BD+05 3704 168797123385              10688    181633.6+052324182128.5+052609 34.39  9.13 6.13  -0.04 -0.62         B3Ve               +0.013-0.004      -018SB    300                 *
6874          CD-3612537 168838210075                       181643.7-361713182328.9-361418357.44-10.50 5.55  +1.02               K0III              +0.017-0.006      -006
6875          CD-4412569 1689052289823461  W       10724    181701.7-440935182418.2-440637350.22-14.00 5.25  -0.19 -0.70         B2.5Vn             -0.002-0.021      +000V?    297  4.6  75.3      *
6876108    HerBD+29 3241 168913 85956                       181705.9+294841182057.1+295132 57.37 19.23 5.63  +0.21 +0.03 +0.11   A5m                +0.014+0.057 +.013-020SB2O   20                 *
6877107    HerBD+28 2981 168914 85957              10690    181706.9+284920182101.0+285212 56.40 18.86 5.12  +0.20 +0.15         A7V                +0.005+0.050 +.021-030SB    183
6878          BD-10 4673 169009161412                       181730.6-101604182302.2-101307 20.61  1.57 6.33  +0.14               B9.5V              +0.024-0.007      -024V?  =< 41
6879 20Eps SgrCD-3412784 169022210091 689  W                181732.0-342555182410.3-342305359.20-09.81 1.85  -0.03 -0.13 -0.01   B9.5III            -0.038-0.124 +.023-015      140:12.3  36.1      *
6880          BD+51 2357 169028 30897                       181735.4+511813181956.1+512052 79.61 25.61 6.30  +1.10 +1.08        gK1                 -0.033-0.052 +.037-010SB
6881          BD-12 5024 1690331614153459                   181736.4-120349182312.2-120053 19.04  0.69 5.73  +0.01 -0.26         B8IV-Ve            -0.007-0.014      -011V     250
6882          BD+23 3316 169110 85975    I                  181758.4+231404182208.7+231707 51.07 16.53 5.41  +1.60       +1.01   M0IIIab            +0.012+0.077      -058
6883          BD+11 3442 169111103648                       181756.1+115848182235.3+120144 40.53 11.79 5.89  +0.04 +0.08         A2V                +0.012-0.008      -055
6884   Zet SctBD-09 4712 169156142267    I                  181810.8-085911182339.5-085603 21.81  2.03 4.68  +0.95 +0.72 +0.47   G9-IIIbFe-0.5      +0.045+0.046 +.015-006SB1O < 17                 *
6885          BD+17 3555 1691911036553460I                  181823.9+174634182249.0+174936 45.95 14.19 5.25  +1.27 +1.32 +0.65   K3III              +0.071+0.016 +.023-019V    < 17
6886          BD+49 2776 169221 47411                       181836.7+494036182107.1+494332 77.90 25.07 6.40  +1.07               K1III              -0.007+0.025      -017
6887          BD+16 3478 169223103660                       181835.4+163816182302.9+164117 44.91 13.67 6.22  +1.20               K0                 -0.019-0.016      +015SB
6888 18    SgrCD-3015661 169233210116    I                  181835.7-304827182501.5-304524  2.57-08.35 5.60  +1.14               K0III-IV           -0.123-0.072      -019
6889          CD-3612589 169236210119                       181837.8-360244182521.7-355930357.83-10.73 6.15  +1.00               K0III              -0.033+0.013      +004
6890          BD-03 4277 169268142274                       181847.9-033802182403.5-033500 26.61  4.44 6.38  +0.34 -0.01         F6III-IV           -0.025-0.055      -017SB2    34                 *
6891          BD+49 2782 169305 474173458I                  181859.1+490415182132.7+490718 77.27 24.87 5.05  +1.66 +1.94 +0.90E  M2IIIab            -0.020+0.053 +.000+014
6892          BD-07 4598 169370142288                       181917.2-070743182442.1-070431 23.58  2.67 6.31  +1.16 +1.09         K0                 +0.134-0.004      -025
6893          CD-3412802 169398210135                       181917.6-340001182554.6-335643359.75-09.94 6.30  -0.08 -0.51         B5IV               +0.002+0.006      -006
6894          CD-4812505 169405229021                       181918.1-481018182653.9-480701346.61-16.02 5.46  +0.84               K0-1III+F-G        -0.007-0.042 +.026+004SB
6895109    HerBD+21 3411 169414 86003 690I W       10742    181926.1+214327182341.9+214611 49.77 15.60 3.84  +1.18 +1.17 +0.60   K2.5IIIab          +0.196-0.242 +.025-058V?   < 17  6.5 221.8
6896 21    SgrBD-20 5134 169420186794    I  11325           181923.6-203543182521.0-203230 11.74-03.75 4.81  +1.31 +0.92 +0.84   K2II               +0.006-0.025 +.013-012V?         3.6   1.7      *
6897   Alp TelCD-4612379 169467229023 691                   181933.5-460124182658.4-455806348.67-15.18 3.51  -0.17 -0.64 -0.21   B3IV               -0.016-0.054      -000V?     35                 *
6898          BD-01 3486 169493142294       11324           181946.0-013801182457.0-013446 28.49  5.16 6.15  +0.37 +0.15         F6III+A9III        -0.017-0.010D+.011-010V?         0.4   0.8      *
6899          CP-74 1682 1695702576013466                   182004.9-740138183255.3-735756320.58-24.73 5.89  +0.98 +0.83         K0III              -0.002-0.090      -025
6900          BD+05 3730 169578123453                       182013.2+050146182508.8+050505 34.49  8.16 6.74  +0.02 -0.21         B9V                -0.003+0.008      -023V     250
6901          BD+38 3160 169646 66936    I  11320  10756    182036.4+384105182357.4+384421 66.53 21.62 6.36R                     K4III              +0.027+0.006      -040V          6.1  61.5AC   3*
6902          BD+07 3682 1696891234621478                   182050.1+075833182538.8+080155 37.21  9.36 5.65  +0.92 +0.45         G8III-IV+A0V       -0.002-0.006      -008SBO =< 50                 *
6903  2Mu  LyrBD+39 3410 169702 669433463                   182056.1+392709182413.8+393026 67.33 21.80 5.12  +0.03 +0.08         A3IVn              -0.023-0.002 +.004-024SB    165                 *
6904          BD+27 3016 169718 86019       11334           182059.8+272022182458.5+272343 55.30 17.51 6.27  +0.05 +0.02         A0V+A4V            +0.002+0.008D+.004-029SB    105  0.9   0.6AB   3*
6905   Zet TelCD-4912153 169767229047                       182107.7-490729182849.9-490415345.82-16.68 4.13  +1.02 +0.82 +0.34E  G8-K0III           +0.144-0.242 +.027-031                          *
6906          BD+14 3533 169820103709      S                182123.3+145434182555.4+145800 43.60 12.32 6.37  +0.01 -0.31         B9V                +0.012-0.006      -021V                0.1
6907          CD-2914965 169830186838                       182125.8-295237182749.5-294859  3.69-08.47 5.92  +0.52               F9V                -0.003+0.032      -017V
6908          CP-57 9063 1698362455103464  W                182119.8-573504182956.7-573123337.57-19.87 5.76  +0.98               K0III              +0.051-0.025      -000           4.7  33.9AC   3
6909          CD-2613184 169851186837       11354           182129.6-264137182743.8-263805  6.55-07.02 6.31  +0.26               A8V+F2V            +0.003-0.025D+.009-029           0.0   1.1      *
6910          CD-3912626 169853210197                       182132.0-390318182827.1-385944355.33-12.57 5.64  +0.13               A2mA2-F0           -0.002-0.038      -021
6911          BD+53 2079 169885 30943                       182138.5+531446182347.8+531803 81.86 25.44 6.32  +0.16 +0.14 +0.06   A3m                +0.008-0.015      -004       40
6912          CP-81  813 169904258804                       182142.5-815306184214.1-814828312.09-26.50 6.27  -0.13 -0.37         B8V                +0.001-0.006      -006V?
6913 22Lam SgrCD-2513149 169916186841 692I                  182147.9-252837182758.2-252518  7.66-06.52 2.81  +1.04 +0.89 +0.56   K1+IIIb            -0.044-0.185 +.053-043V?                        *
6914          CD-2613192 169938186843                       182151.7-264901182806.2-264526  6.47-07.15 6.27  +0.16               A3/4V               0.000-0.036      -030
6915          CD-4312564 169943229056                       182157.8-435431182912.9-435045350.84-14.70 6.36  +0.90               G8III              -0.002+0.028      -009
6916   Nu  PavCP-62 5879 169978254273              10889    182202.0-622030183122.4-621642332.82-21.56 4.64  -0.11 -0.39         B7-8III            -0.003-0.045      +059       91                 *
6917          BD+29 3259 169981 860431479                   182207.3+294616182558.8+294944 57.76 18.20 5.83  +0.06 +0.09 +0.04   A2IV               +0.026-0.022      +009SBO                       *
6918 59    SerBD+00 3936 169985123497       11353  d Ser    182205.5+000812182712.5+001146 30.34  5.48 5.21  +0.50 +0.21 +0.38   G0III+A6V          -0.008-0.002 +.016-023SBO   270  2.3   3.8AB   3*
6919          BD-17 5203 169990161493    I:        10835    182206.4-175139182756.5-174800 14.45-03.02 6.20  +0.01 -0.30         B8V                -0.004+0.007      -035V
6920 43Phi DraBD+71  889 170000  9084       11311  Phi Dra  182211.4+711705182045.5+712016101.88 28.04 4.22  -0.10 -0.33 -0.10   A0pSi:             -0.004+0.040 +.012-016SB1O   88  1.7   0.2AB   3*
6921          CD-3812824 170040210213                       182222.3-385449182916.8-385104355.54-12.66 6.63  -0.02               B8                 -0.004-0.007      -028
6922          CD-4712319 170069229064                       182224.6-471701182955.8-471313347.68-16.14 5.70  +1.26               K2III              +0.013-0.003      -018
6923 39    DraBD+58 1809 170073 30949       11336           182226.9+584434182354.5+584802 87.88 26.44 4.98  +0.08 +0.04 +0.02   A1V                -0.039+0.060 +.035-015SB    175  2.8   3.8AB   8*
6924          BD+26 3259 170111 86060       11356           182239.5+262322182640.9+262658 54.53 16.80 6.53  -0.11 -0.58         B3V                -0.003 0.000      -018SB         2.7  62.3AC   3*
6925          BD+03 3716 170137123513    I                  182251.5+034115182750.3+034455 33.59  6.96 6.07  +1.62 +1.84         K3III              -0.005-0.005      -019
6926          CD-2613206 170141186863      W       10864    182243.4-263840182857.4-263454  6.72-07.24 6.5   +0.11               A3III              +0.003+0.009      -017           1.7  41.9      *
6927 44Chi DraBD+72  839 170153  9087 695  W       10749    182251.5+724122182103.4+724358103.46 28.06 3.57  +0.49 -0.06 +0.31   F7V                +0.532-0.350 +.120+033SB2O   11  2.    0.1 *   4*
6928          BD+06 3790 170200123516      S                182305.7+060758182758.8+061139 35.81  8.03 5.73  -0.03 -0.35         B8III-IV            0.000-0.024      -012SBO    30        0.1      *
6929          CD-2513170 170235186873              V4031 Sgr182311.8-251912182922.0-251523  7.95-06.73 6.59  +0.07 -0.68         B2IVpe            v+0.004-0.001      -004      130                 *
6930   Gam SctBD-14 5071 170296161520 696                   182329.8-143747182911.9-143357 17.46-01.79 4.70  +0.06 +0.06 +0.05   A3Vn               +0.003-0.002 +.025-041V?    223
6931          CD-4112871 170384229080                       182356.3-415846183103.0-415449352.81-14.23 6.04  +0.14               A3V                 0.000-0.022      -014
6932          BD-14 5077 170397161528              V432 Sct 182404.6-143851182946.8-143454 17.51-01.92 5.96  -0.04 -0.14         B9pSiCr            +0.021+0.025      -016V      46                 *
6933          BD-18 4982 170433161540                       182419.1-184732183011.9-184344 13.88-03.92 5.66  +1.06              gK0                 +0.038-0.095 +.011-001
6934   Del1TelCD-4512550 170465229092                       182420.9-455855183145.4-455454349.06-15.93 4.96  -0.11 -0.42         B6IV               -0.008-0.032      +007SB1O   33                 *
6935 60    SerBD-02 4641 1704741423481480                   182428.7-020300182941.0-015907 28.68  3.93 5.39  +0.96 +0.76 +0.48   K0III              +0.032-0.033 +.013+028SB1  < 17                 *
6936          CD-3313281 170479210257      W                182431.2-330319183104.8-325921  1.11-10.49 5.34  +0.16 +0.07         A3m                -0.001-0.045 +.023+009      120  4.5   3.4AB   3*
6937          CD-4312600 170521229094                       182442.8-433433183156.2-433028351.37-15.01 5.72  +1.34               K2III              -0.005-0.019      +007
6938   Del2TelCD-4512556 170523229095                       182438.2-454933183202.0-454526349.23-15.91 5.07  -0.14 -0.56         B3III              +0.004-0.008      +008SB1O   24                 *
6939          CP-58 7418 170525245547                       182443.3-584635183329.5-584233336.54-20.70 6.44  +0.68 +0.18         G5IV               +0.027-0.124      +033                          *
6940          BD-05 4675 1705471423533468                   182453.5-054726183014.3-054327 25.42  2.08 6.28  +0.95 +0.65         G8II-III           +0.004-0.018      +027SBO                       *
6941          BD+03 3727 1705801235713469   11399           182507.0+035955183005.1+040355 34.13  6.60 6.69  +0.11 -0.51         B2V                 0.000-0.009      -022       95  4.2  20.1AB   5*
6942          CD-3912696 170642210277                       182523.4-394622183221.4-394214354.99-13.55 5.16  +0.08               A3Vn               +0.036-0.040 +.022-006      151
6943          BD+23 3347 170650 86115                       182526.9+234758182935.7+235158 52.32 15.19 5.90  -0.10 -0.51         B6IV                0.000-0.007      -017SB    200
6944          BD-18 4988 170680161564       11411           182534.7-182816183126.3-182410 14.30-04.03 5.14   0.00               A0Vp               -0.003-0.025 +.017-037V     213  8.9  25.2      *
6945 42    DraBD+65 1271 170693 178883465I                  182541.8+653006182559.1+653349 95.42 27.13 4.82  +1.19 +1.11 +0.63   K1.5IIIFe-0.5      +0.105-0.023 +.021+032V    < 17
6946          BD-10 4713 170740161569       11414           182552.9-105153183125.7-104745 21.06-00.53 5.72  +0.24 -0.45         B2V                 0.000-0.015      -015V?         3.6  12.3      *
6947          BD-19 5047 170764161571      W       U Sgr    182559.9-191142183153.3-190730 13.71-04.46 6.68  +1.13 +0.69 +0.71   G1.5Ib             -0.009+0.001      +002SB?        2.0 430.7AG  17*
6948          CD-3912704 1707732102863470                   182601.8-395740183300.9-395332354.87-13.74 6.22  +0.42               F5IV               +0.086-0.086      -016
6949          BD+59 1899 170811 30990                       182619.6+592857182742.3+593257 88.79 26.09 6.43                      K0IV               +0.052+0.045      -010
6950          BD+20 3821 170829 86142              Var?     182624.4+204511183041.6+204855 49.55 13.72 6.50  +0.79 +0.40         G8IV               +0.011-0.258 +.029-059SBO                       *
6951   The CrACD-4213378 170845229111 697                   182621.7-422304183330.2-421845352.62-14.80 4.64  +1.01 +0.76 +0.50   G8III              +0.031-0.022 -.001-002V?
6952   Kap1CrACD-3812896 170868210296      W     B          182629.3-384731183323.3-384313356.00-13.33 6.32                      A0III              +0.021-0.028      -016           0.7  21.4      *
6953   Kap2CrACD-3812895 170867210295      W     A          182629.3-384753183323.1-384334355.99-13.33 5.65  -0.06               B9V                -0.003-0.030      -020           0.7  21.4      *
6954          CP-5211158 170873245559                       182628.6-525752183431.2-525331342.43-18.91 6.22  +1.25               K2III              -0.011-0.054      +024
6955          BD+16 3529 1708781038001481                   182637.5+165133183104.4+165543 45.96 12.02 5.77  +0.04 +0.07         A2V                -0.032-0.021      -009SB?
6956          BD-14 5098 170902161580      S                182638.5-144254183220.8-143839 17.75-02.50 6.37  +0.22               A4V                +0.019-0.017      -038                 0.0
6957 61    SerBD-01 3504 170920142372                       182647.1-010426183157.0-010011 29.81  3.88 5.94  +0.16 +0.16 +0.12   A4III              +0.010-0.006      -027                          *
6958          BD+03 3737 170973123610              MV Ser   182707.8+033516183207.0+033935 34.00  5.96 6.43  -0.04 -0.26         A0pSiCr            +0.031+0.015      -008V?     15                 *
6959          BD-14 5099 170975161587    I                  182700.6-145617183243.3-145156 17.59-02.68 5.50  +1.97 +2.17         K3Ib-IICN1         -0.004+0.001      +005
6960          CD-3313338 1710342103123471                   182724.1-330526183357.8-330100  1.35-11.05 5.28  -0.11 -0.69         B2IV-V             +0.004-0.010      +004V     172                 *
6961 24    SgrCD-2414472 171115186981    I                  182746.9-240625183353.5-240157  9.52-07.10 5.49  +1.79               K4Ib               +0.002-0.010      -014V?
6962          BD-14 5106 171130161605                       182756.2-145541183339.0-145113 17.70-02.88 5.76  +0.04               A2V                +0.025-0.013      -021V
6963          BD-06 4791 171149142386                       182801.6-055907183322.9-055441 25.61  1.30 6.36  +0.02 -0.04         A0Vn               +0.012-0.017      -024      210
6964          CP-83  663 1711612588103992                   182758.8-832445185157.9-831859310.43-26.94 7.16  +1.26 +1.30         K1IIICNII:         -0.008-0.021      +011V
6965 25    SgrCD-2414479 171237186995              11035    182825.7-241755183432.8-241321  9.42-07.31 6.51  +0.54 +0.46         F3II-III           +0.001-0.001      +008       17                 *
6966          BD+23 3363 171245 861753472I W                182836.4+233232183246.2+233701 52.39 14.42 5.84  +1.46              gK5                 +0.010+0.016      -004V?                        *
6967          BD+08 3741 171247123634       11448           182835.0+081136183323.3+081606 38.29  7.75 6.42  -0.03 -0.34         B8IIIpSiSr:        +0.007-0.003      -022V?         4.0  38.7      *
6968          BD+30 3223 171301 67090       11446           182900.5+302844183249.9+303315 59.04 17.09 5.48  -0.10 -0.34         B8IV               +0.010+0.009 +.009-010V      74  7.3   7.1
6969          BD-20 5189 171369187012                       182923.2-205507183521.3-205027 12.54-05.96 6.48  +0.28               F0IV               +0.030-0.013      -010
6970          BD-11 4681 1713911616323477                   182928.9-110319183502.4-105838 21.31-01.40 5.14  +0.92 +0.60 +0.47   G8III              +0.051-0.004 +.012+007V    < 19:
6971          BD+30 3227 171406 67102              Var?     182934.7+304857183323.1+305333 59.41 17.11 6.59  -0.12 -0.51         B4Ve              v+0.011+0.009      -004                          *
6972          CD-2915123 1714161870243478                   182936.6-294642183559.7-294157  4.58-10.01 6.37  +1.27               K1III              +0.009-0.009      -050V?
6973   Alp SctBD-08 4638 1714431424081482I         11056    182945.9-081851183512.4-081439 23.76-00.17 3.85  +1.33 +1.54 +0.68   K3-III-IIIb        -0.015-0.312 +.016+036V?   < 17                 *
6974          BD+52 2232 171461 310323474                   182954.0+520225183211.4+520656 80.90 23.94 6.56  -0.07 -0.18         B9.5V              -0.004+0.006      -023V?     59
6975          BD+20 3847 171487 862033476                   183001.4+202319183419.6+202759 49.58 12.81 6.57  +0.11 +0.07         A3V                +0.008-0.006      -009SB
6976          BD+10 3573 171505103867                       183005.4+104848183447.5+105330 40.82  8.60 6.40R +0.08 +0.05         A1V                +0.004-0.007      -036      124
6977          BD+18 3740 171623103879      S                183048.5+180724183512.6+181212 47.56 11.67 5.78R  0.00 -0.10         A0Vn               +0.014+0.006      -021SB    280        0.2
6978 45    DraBD+56 2113 171635 310393475                   183051.0+565808183234.5+570244 86.20 24.99 4.77  +0.61 +0.44 +0.31   F7Ib               +0.001-0.006 +.013-012V      18
6979          BD+65 1276 171653 17912      W                183056.8+652131183114.8+652610 95.35 26.57 6.59  +0.30?              A8m                -0.019+0.071      -009SB2O   20  4.2  26.6      *
6980          BD+23 3385 171745 86224       11479           183120.6+233128183530.4+233620 52.64 13.84 5.61  +1.00 +0.83         G9III+G7III        +0.004+0.005D+.006+016V          0.2   0.6      *
6981          BD+16 3560 171746103886       11483  11085    183125.7+165345183553.2+165832 46.50 11.00 6.21  +0.53 +0.01         G2V+G2V            +0.049-0.068 +.020+010V?         0.2   1.8AB   3*
6982   Zet PavCP-71 2353 171759257620 698  W                183121.1-713050184302.1-712541323.49-24.95 4.01  +1.14 +1.02 +0.42E  K0III              -0.005-0.156 +.035-016    =<  9  7.9  55.6      *
6983          BD+52 2238 171779 31051    I  11468           183140.5+521626183356.7+522113 81.23 23.73 5.36  +1.09 +0.96 +0.54   K0III              -0.002+0.008 +.004-024V?   < 17  0.1   0.3AB   3*
6984          BD+34 3245 171780 67134      S       Var      183136.6+342237183513.5+342728 63.04 18.05 6.10  -0.11 -0.55         B5Vne             v+0.006+0.003 -.021-027SB    310        0.2      *
6985          BD+09 3783 1718021236901484                   183141.6+090237183627.8+090721 39.40  7.45 5.39  +0.37 -0.02         F5III              -0.002-0.129 +.033-022       14                 *
6986          CD-4812644 1718192291653482                   183140.2-475945183914.3-475435347.62-17.86 5.86  +0.23               A7IV-V             +0.028+0.015      -009V
6987          BD+06 3855 171834123693       11496           183147.2+063535183639.1+064019 37.22  6.31 5.45  +0.37 -0.04         F3V                -0.025-0.141 +.032-021SB     55  6.7  74.8AC   4*
6988          BD-21 5076 1718561870711485  D                183155.1-212849183754.4-212352 12.31-06.75 5.94  +0.19               A8IIIn              0.000-0.067      +006V          0.0   0.1      *
6989          BD-14 5139 171957161687       11512           183224.0-140524183804.6-140017 18.95-03.44 6.47  +0.21 -0.19         B9IV               +0.019+0.002      -019V?         4.8   2.0AB   3*
6990          CD-2314572 171961187080                       183225.8-233525183830.7-233018 10.47-07.81 5.81  +0.02 -0.41         B8III              -0.003-0.020      -033V                         *
6991          CD-4312699 171967229172                       183224.0-431618183935.0-431110352.23-16.17 5.37  +1.68 +1.95 +0.95E  M2III              -0.052-0.052 +.004+029
6992          BD+11 3530 171975103912                       183231.9+112015183712.6+112518 41.56  8.30 6.42R -0.02 -0.22         B9V                -0.021-0.011      -027      200
6993          BD-00 3521 1719781424443480          11122    183227.6-002337183736.0-001834 31.08  2.94 5.75  +0.07 +0.06 +0.04   A1V+A1V            +0.016-0.019      +012SB2O   26                 *
6994          CP-77 1314 171990257625                       183227.7-775810184749.4-775202316.48-26.29 6.39  +0.60 +0.13         F8V                -0.022+0.188      +015
6995          BD+16 3563 171994103913                       183239.8+160645183709.0+161154 45.92 10.40 6.29  +0.90 +0.56         G8IV               +0.010+0.047      -046
6996          CP-64 3942 172021254343                       183236.4-644358184222.5-643835330.72-23.39 6.37  +0.15 +0.14         A5V                -0.009-0.032      -009V?
6997          BD+33 3154 172044 67164       11504  11113    183257.2+332305183637.2+332808 62.18 17.42 5.42  -0.10 -0.50         B8II-IIIpHg        -0.019 0.000 +.007-026SBO    46  4.6   7.3      *
6998          BD-21 5081 172051187086                       183255.6-210804183853.4-210307 12.73-06.80 5.86  +0.68 +0.14         G4V                -0.078-0.152 +.072+036V
6999          BD-03 4331 172088142461       11520           183308.9-031652183823.7-031137 28.60  1.44 6.49  +0.55 +0.06         F9IV               -0.009+0.032 +.027-021V          0.1   0.1AB   3*
7000          BD-01 3529 172103142460                       183309.2-011158183819.1-010648 30.44  2.41 6.66  +0.42 +0.01         F1IV-V             -0.027-0.012      -025       23
7001  3Alp LyrBD+38 3238 172167 67174 699I  11510  Alp Lyr  183333.1+384126183656.3+384701 67.44 19.24 0.03   0.00 -0.01 -0.03   A0Va               +0.202+0.286 +.123-014V      15 10.4  62.8AB   5*
7002          BD+08 3780 172171123744    I  11524  X Oph    183334.1+084448183821.0+085002 39.35  6.90 6.4 H +1.32 +0.89         M6IIIe+K1III       -0.014+0.006 +.011-071V          2.2   0.4      *
7003          BD+43 3027 172187 476443481                   183341.8+430812183645.6+431319 71.90 20.72 6.20  +0.24 +0.09         F0V                +0.023-0.006      +005SB    185
7004          CP-64 3943 172211254353                       183352.3-643838184337.2-643304330.86-23.50 5.78  +0.96 +0.77         K0III              +0.003-0.036      -011
7005          CD-4812668 172223229194              11183    183356.3-481058184130.6-480542347.58-18.28 6.49  +1.22               K2III-IVCNII       -0.025-0.122      -046                          *
7006          BD+77  699 172340  9151 700                   183434.8+772809182944.9+773249108.91 27.61 5.64  +1.18              gK4                 -0.001+0.001 +.014+001V?
7007          BD-07 4648 172348142480    I  11552           183434.9-075248184000.5-074726 24.70-01.02 5.84  +1.55 +1.83         K4III              +0.004-0.032      -023V?         5.0  19.4      *
7008          BD+05 3891 1723651237783485                   183441.5+051027183936.9+051551 36.28  5.02 6.38  +0.78 +0.42         F8Ib-II            +0.006+0.001      -019SB?                       *
7009          BD+39 3476 172380 67193    I         XY Lyr   183448.2+393447183806.5+394005 68.41 19.32 6.04  +1.65 +1.42 +1.96   M4.5-M5+II         +0.006+0.006      -019                          *
7010          BD+07 3798 172424123782       11555           183501.1+071609183951.6+072130 38.19  5.91 6.28  +0.96 +0.69         G8III              +0.007-0.056      -041           5.3  23.1AB   4*
7011 26    SgrCD-2314625 1725461871463486                   183545.7-235535184151.6-235000 10.51-08.64 6.23  +0.23               Am                 +0.032-0.024      +002V
7012          CP-64 3948 1725552543583489                   183537.9-645754184526.9-645217330.57-23.76 4.79  +0.20 +0.08         A5IV-V             +0.027-0.149 +.035+002      134
7013          BD+65 1283 172569 17947 701                   183554.4+652357183613.3+652919 95.49 26.07 6.06  +0.28 +0.09         F0V                +0.019+0.083 +.009-023SB    135                 *
7014          BD-14 5156 172594161730                       183600.7-143930184142.5-143351 18.85-04.48 6.42  +0.81               F2Ib               +0.006+0.001      -003V      17
7015          CP-61 6229 172630254359                       183605.0-611134184511.5-610542334.54-22.80 6.04  +1.46               K3III              +0.023-0.023      -020
7016          BD+30 3262 172631 67226                       183613.2+304523184002.0+305058 59.93 15.77 6.36R                     K0                 -0.005+0.032      -050
7017          BD+40 3446 172671 47676      W                183619.6+405036183933.1+405606 69.77 19.49 6.25  -0.06 -0.17         B9V                +0.027-0.002      -009SB    130  0.4   0.2      *
7018          BD+62 1637 172728 179583484                   183639.3+622608183733.5+623136 92.27 25.43 5.74  -0.06 -0.17         A0V                -0.005+0.051      -011V   =< 41
7019          BD+38 3254 172741 67233                       183648.7+381626184012.2+382202 67.27 18.50 6.45  +0.21 +0.14 +0.09   A6m                +0.026+0.001      +017                          *
7020   Del SctBD-09 4796 1727481425151486   11581  Del Sct  183647.9-090854184216.4-090309 23.83-02.10 4.72  +0.35 +0.14 +0.19   F2IIIpDel Del      +0.009+0.002 +.025-045SB     32  4.5  52.6AC   3*
7021   Lam CrACD-3813036 172777210501      W       11247    183655.3-382510184346.9-381925357.21-15.04 5.13  +0.09               A2Vn               +0.003-0.057 +.024-026V     131  4.5  29.2AB   3*
7022          CP-57 9180 172781245681                       183655.0-565851184523.7-565254338.92-21.66 6.22  +1.38               K3III              -0.054-0.007      +069
7023          BD-19 5134 172816161754    I D       V3879 Sgr183701.6-192248184255.2-191702 14.74-06.86 6.35  +1.75 +1.56 +1.52E  M4III              -0.001-0.024      -033V?         2.6   0.6      *
7024          BD-07 4670 172831142525                       183712.3-071012184236.1-070424 25.63-01.27 6.15  +1.00 +0.89         K0-1III            -0.003+0.005      +026V
7025          BD+83  536 172864  30561646                   183721.8+830606182409.2+831031115.27 27.73 6.17  +0.05 +0.06         A2V                +0.015-0.027      -011
7026          CD-3612946 172875210507                       183722.0-364856184407.7-364307358.77-14.47 6.32  +0.98               K0III               0.000-0.060      -028V?
7027          CP-73 1939 172881257630      W                183725.5-730602184943.5-725941321.87-25.73 6.06   0.00 -0.09         B9.5IV-V           -0.004+0.024      +003SB3        2.0   1.8      *
7028          BD+52 2263 172883 31093                       183734.8+520606183952.8+521146 81.31 22.82 6.00  -0.07 -0.21         A0p:Hg:            +0.004+0.028      -018V      65                 *
7029          CD-3512876 1729102105093490  W                183737.5-354425184419.4-353831359.80-14.08 4.87  -0.18 -0.72 -0.16   B2.5V               0.000-0.031      +004       65  7.8   9.4      *
7030          BD+31 3332 172958 672563487                   183754.7+313116184141.3+313704 60.81 15.74 6.41  -0.04 -0.20         B8V                +0.008+0.009      -016      175
7031          CD-3912864 172991210518                       183800.4-394711184457.2-394111356.00-15.77 5.43  +0.87 +0.37?+0.60?  K3II+B7            +0.008-0.011 +.012-017SB                        *
7032   Eps SctBD-08 4686 173009142546 702I  11601           183804.4-082227184331.3-081631 24.67-02.02 4.90  +1.12 +0.87 +0.61   G8IIb              +0.022+0.008 +.016-011V    < 19: 8.6  37.6AC   4*
7033          BD+34 3285 173087 67265       11593A          183831.7+343856184208.1+344448 63.87 16.83 6.47  -0.13 -0.57         B5V                +0.010+0.008      -024SB    130  1.0   0.2AP   4*
7034          BD-06 4859 173093142557                       183827.8-065459184351.4-064907 26.00-01.43 6.31  +0.48 +0.02         F7V                +0.048-0.055      -066       30
7035          CD-2513394 173117187216      D                183840.7-250640184449.6-250040  9.73-09.76 5.83  +0.05 -0.35         B5:V               +0.005-0.022      +015V          0.2   0.0      *
7036   The PavCP-65 3754 173168254374                       183848.0-651052184837.8-650440330.43-24.14 5.73  +0.24 +0.11         A8V                -0.045-0.083      -001      190
7037          CD-5012135 173263245702                       183914.4-501151184659.1-500540345.91-19.80 6.54  +0.28               F0V                -0.001-0.034      -032
7038          BD-21 5131 173282187234                       183920.5-210611184518.7-210005 13.44-08.12 6.36  +0.45               F5V                +0.027-0.019      +002V
7039 27Phi SgrCD-2713170 1733001872391487  D                183924.5-270537184539.4-265927  8.00-10.77 3.17  -0.11 -0.36 -0.11   B8III              +0.053 0.000      +022SB     68  0.0   0.1      *
7040  4    AqlBD+01 3766 173370123879              11290    183947.1+015730184449.9+020336 34.01  2.41 5.02  -0.06 -0.26         B9V               e+0.009-0.018 +.005-013V     300                 *
7041          BD+39 3505 173383 67287    I W       11271    183956.6+391159184316.7+391801 68.41 18.26 6.45  +1.59 +2.00         K5                 +0.016-0.002      -034SB         3.8  60.2
7042          BD+62 1641 173398 17975                       184003.8+623900184056.3+624459 92.60 25.09 6.09  +0.98               K0III               0.000+0.061      -026
7043          BD+36 3246 173416 67292                       184005.7+362714184336.1+363324 65.74 17.22 6.01  +1.04               G8                 +0.024+0.065      -061
7044          BD+31 3348 173417 67293                       184006.1+314943184351.6+315536 61.29 15.43 5.70  +0.34 +0.05         F1III-IV           -0.033-0.129 +.026-002V
7045          BD-19 5154 173425161803    I         11309    184006.9-194238184601.2-193623 14.78-07.65 6.42  +1.65               M4III              -0.002+0.004      -040V                         *
7046 28    SgrBD-22 4854 173460187255    I  11652           184018.8-222949184620.6-222332 12.28-08.94 5.37  +1.64 +1.90         K4III              +0.031 0.000      -003V          7.7  13.7
7047          BD+23 3439 173494 86393                       184030.0+232922184440.2+233523 53.51 11.92 6.31  +0.40  0.00         F6V                +0.009-0.087 +.019-012
7048          BD+05 3941 173495123886       11640           184033.4+052346184528.4+052959 37.16  3.83 5.83  +0.04 +0.03         A1V+A1V            +0.015-0.011D+.006-011SB    130  0.3   1.3AB   4*
7049 46    DraBD+55 2107 173524 311193491  W       11273    184041.7+552617184237.9+553222 84.94 23.29 5.04  -0.09 -0.30         B9.5pHg:           -0.002+0.024 +.011-030SB2O   19  5.6 146.8AB   3*
7050   Mu  CrACD-4012807 1735402292853492                   184045.1-403045184744.6-402422355.51-16.54 5.24  +0.78               G5-6III            +0.030-0.017 +.023-018V?
7051  4Eps1LyrBD+39 3509 173582 67310       11635A          184101.5+393355184420.4+394012 68.85 18.20 5.06H +0.16 +0.06         A4V                +0.017+0.059 +.021-031V     200  0.0 209.3AC  10*
7052  4Eps1LyrBD+39 3509 173583 67309       11635B          184101.6+393358184420.3+394016 68.85 18.20 6.02H                     F1V                +0.003+0.059 +.021-033V     150  0.0 209.3AC  10*
7053  5Eps2LyrBD+39 3510 173607 67315       11635C 11301    184103.9+393029184422.9+393647 68.79 18.17 5.14H +0.19 +0.08         A8Vn               +0.008+0.064 +.021-024V?    177  0.0 209.3AC  10*
7054  5Eps2LyrBD+39 3510 173608             11635D 11301    184103.9+393029184422.9+393646 68.79 18.17 5.37H                     F0Vn               +0.007+0.064 +.021-028V?    212  0.0 209.3AC  10*
7055          BD-10 4797 173638161817       11670  11333    184112.4-101352184643.3-100730 23.38-03.56 5.71  +0.59 +0.54         F2Ib-II            -0.005+0.002      +010SB     16  8.2   3.4      *
7056  6Zet1LyrBD+37 3222 173648 67321       11639A 11308    184119.6+373002184446.4+373618 66.85 17.38 4.36  +0.19 +0.16 +0.08   A4m               v+0.028+0.024 +.031-026SB1O   27  1.4  43.7AD   5*
7057  7Zet2LyrBD+37 3223 173649 67324       11639D 11311    184121.5+372924184448.2+373540 66.85 17.37 5.73  +0.28 +0.06         F0IV              v+0.022+0.019 +.031-025SB    230  1.4  43.7AD   5*
7058          BD+21 3550 173650 86405              V535 Her 184121.0+215246184535.7+215906 52.11 11.05 6.51  +0.02 -0.08         B9pSiCr:          v+0.015+0.012      -017SB?    16                 *
7059  5    AqlBD-01 3559 173654142606       11667           184118.7-010401184628.5-005742 31.51  0.67 5.90  +0.13 +0.12 +0.07   A2Vm               +0.013-0.022D+.009+017SB? =< 25  1.6  12.8AB   3*
7060          BD+53 2126 173664 31133                       184121.1+534611184329.0+535219 83.22 22.74 6.11R +0.12?              A2IV               +0.006-0.008      +000
7061110    HerBD+20 3926 173667 86406 703I  11658  11323    184121.4+202702184539.7+203247 50.79 10.43 4.19  +0.46 +0.01 +0.26   F6V                -0.008-0.335 +.052+024       14  9.2  63.8AC   5*
7062   Eta1CrACD-4312841 1737152292991490                   184137.5-434719184850.5-434048352.40-17.92 5.49  +0.13               A3V                +0.025-0.019      -004
7063   Bet SctBD-04 4582 1737641426181489I                  184152.1-045118184710.5-044452 28.22-01.22 4.22  +1.10 +0.81 +0.57   G4IIa             e-0.004-0.016 +.019-022SB1O   10                 *
7064          BD+26 3349 173780 864181488I                  184202.6+263318184604.5+263944 56.50 12.90 4.83  +1.20 +1.23 +0.61   K3III              +0.018+0.024 +.023-017V?   < 17
7065          CD-4512779 1737912293063495                   184204.2-455520184927.4-454837350.33-18.76 5.81  +0.90 +0.51         G8III              +0.076+0.051      +010
7066          BD-05 4760 173819142620    I         R Sct    184208.6-054845184729.0-054218 27.40-01.72 5.20  +1.47 +1.64 +0.77   K0Ibp             v-0.041-0.028 +.001+044V                         *
7067          BD+18 3817 173833104087    I                  184218.1+183557184641.4+184221 49.21  9.42 6.17  +1.59               K5                 +0.028-0.022      -013
7068   Eta2CrACD-4312854 173861229307                       184223.4-433240184935.0-432602352.69-17.95 5.61  -0.08               B9IV               -0.003-0.024      -023V?
7069111    HerBD+18 3823 1738801040931491  W                184236.2+180412184701.3+181053 48.76  9.12 4.36  +0.13 +0.07 +0.03   A5III              +0.073+0.116 +.032-045SB?    79  5.5 121.3AC   4*
7070          CD-3413128 173902210600                       184238.4-345125184917.2-344455  1.06-14.66 6.62  +1.08               K1IVCNIII          +0.046-0.101      -061
7071          BD+54 2034 173920 31151                       184254.2+544728184455.4+545348 84.36 22.81 6.23  +0.82               G5III              +0.007-0.019      +007
7072          BD-18 5079 173928161848      W                184253.7-184242184845.3-183605 15.98-07.79 6.47                      G5III:+A0V:        -0.007-0.004      -031V?         0.7   0.4
7073          BD+41 3137 173936 477793493                   184301.1+412002184613.0+412630 70.73 18.48 6.07  -0.13 -0.49         B6V                +0.001-0.001      -019      125
7074   Lam PavCP-62 5983 173948254393 704  W       Lam Pav  184257.1-621807185213.0-621115333.61-23.87 4.22  -0.14 -0.89 -0.16   B2II-IIIe         v-0.006-0.014      +009V     189  8.0  63.1      *
7075          BD+60 1845 173949 17995       11661           184307.5+605631184418.2+610253 90.85 24.36 5.99  +0.96 +0.73         G7IV               -0.005+0.018D+.006-025V?         2.8   1.2
7076          BD+04 3884 1739541239283494I                  184304.9+040752184802.7+041429 36.32  2.69 6.21  +1.51 +1.79         K5                v-0.002+0.014      -001                          *
7077          BD-19 5182 174115161871      D       Var?     184342.4-191518184935.5-190832 15.57-08.21 6.75  +0.20 +0.15 +0.11   A1m                +0.017+0.010      -043SB2        0.0   0.2      *
7078 29    SgrBD-20 5277 174116187324    I  11713  11372    184344.1-202618184940.1-201929 14.50-08.74 5.24  +1.41               K4III              +0.005+0.036 +.034-018SB         8.6  16.6
7079          BD+23 3461 174160 86451                       184405.6+232412184816.4+233051 53.79 11.14 6.15  +0.49 +0.02?+0.60?  F8V                +0.024-0.030      -000    =< 15                 *
7080          BD+46 2551 174177 47798                       184408.5+461219184659.0+461854 75.64 19.98 6.52  +0.07 +0.17         A2IV               -0.002-0.010      -001V?
7081          BD+31 3369 174179 67396                       184410.8+313845184757.5+314525 61.48 14.57 6.06  -0.13 -0.66         B3IVp              +0.009-0.002      -015                          *
7082          BD+70 1023 174205  9222                       184418.3+704114184310.2+704734101.44 26.18 6.44R                     K2                 -0.002-0.004      -005
7083          BD-06 4922 174208142661    I  11719           184419.8-060133184941.0-055446 27.47-02.30 5.99  +1.60 +1.65         K2Ib               +0.005-0.006      -007SBO        2.5 113.7AC   3*
7084          BD+52 2280 174237 311651492          CX Dra   184429.0+525241184643.1+525917 82.45 22.03 5.88  -0.09 -0.73         B2.5Ve            v+0.010-0.002      -016SBO   170                 *
7085          BD+00 4027 174240123947                       184431.5+004323184937.1+005009 33.47  0.79 6.25  +0.04 +0.01         A1V                -0.001-0.024      -040SB     72
7086          BD+19 3798 174262104129                       184431.6+191259184853.4+191943 50.00  9.22 5.88  +0.03 +0.02         A1V                +0.017-0.021      +006SB     74
7087   Kap TelCP-5211268 1742952457723499                   184443.6-521319185239.6-520627344.16-21.27 5.17  +0.94               G8-K0III           +0.041-0.103 +.014-044
7088 30    SgrBD-22 4881 1743091873421493   11731           184449.8-221636185050.5-220944 12.95-09.78 6.61  +0.38 +0.29         A7III              -0.019-0.033      -035V?    120  6.8  21.1      *
7089          BD-08 4726 174325142674    I  11726  S Sct    184454.3-080120185020.0-075427 25.76-03.35 6.80  +3.09 +4.04         C5II               +0.003+0.003      +000           4.0  14.4AB   3*
7090          BD+48 2767 174366 47815      S                184503.3+485745184740.0+490430 78.48 20.74 6.40R                     A1V                +0.005+0.019      -017       71        0.5
7091          BD+24 3545 174369 86462      S                184507.9+245558184914.4+250247 55.30 11.58 6.59R                     A1V                +0.003-0.014      -009SBO    74        0.2      *
7092          CD-4612669 174387229336                       184501.1-464245185227.1-463543349.73-19.51 5.54  +1.63               M0III              +0.019-0.003 +.009-028
7093          CP-5211273 174430245783                       184517.5-520259185312.1-515552344.37-21.30 6.31  -0.09               B4III              +0.010-0.006      -023
7094          BD-09 4859 174464142692                       184528.4-095325185058.5-094627 24.16-04.34 5.83  +0.61 +0.35         F2Ib               +0.002-0.002      -018V?     19
7095          CD-4812769 174474229342                       184528.0-482839185302.5-482136347.99-20.18 6.19  +0.14 +0.08         A2V                -0.003-0.046      -044
7096          BD+48 2770 174481 478233497                   184537.8+483910184816.1+484603 78.20 20.55 6.12  +0.21 +0.08         A7III              -0.019+0.050      -030      185
7097          CD-4612676 174500229343                       184534.2-464219185259.6-463509349.78-19.60 6.19  +0.03 +0.05         A1IV-V             -0.028+0.023      +035
7098          BD+31 3373 174567 67438                       184557.0+313051184944.0+313745 61.51 14.17 6.64  +0.02 -0.10         A0V s              -0.003-0.012      -003    =<  7
7099          BD+10 3685 174569104170    I  11750           184603.3+105134185045.6+105835 42.65  5.13 6.55R                     K5III+K3III        +0.023+0.009D+.005-024SB         1.1   3.5      *
7100  8Nu 1LyrBD+32 3227 174585 67441       11732  11400    184602.5+324151184945.9+324846 62.64 14.63 5.91  -0.16 -0.71         B3IV               +0.006-0.008      -017V          5.9  58.7AC   6*
7101  8    AqlBD-03 4392 1745891427063500                   184607.0-032605185122.1-031904 29.97-01.50 6.10  +0.30 +0.09         F2III               0.000-0.020      +012V
7102  9Nu 2LyrBD+32 3228 174602 67446       11737           184608.8+322608184952.9+323303 62.40 14.50 5.25  +0.08 +0.11         A3V                -0.009-0.014 +.020+010V?    141  7.8  19.0      *
7103          CD-2613562 174630187388                       184615.2-264605185228.5-263901  8.96-12.01 6.29  +0.94               G8/K0III           +0.013-0.046      -025V?
7104          CD-2915449 174631187389                       184615.9-292951185237.0-292246  6.43-13.16 6.13  +1.35               K1III              -0.002-0.040      -056V
7105          CD-3016356 1746322106633502                   184616.4-305110185241.7-304403  5.16-13.72 6.63  -0.05 -0.27         B8V                -0.002-0.023      -050V
7106 10Bet LyrBD+33 3223 174638 67451 705I  11745A Bet Lyr  184623.2+331447185004.8+332146 63.19 14.78 3.45   0.00 -0.56 +0.02   B8IIpe             +0.003-0.003 -.002-019SBO        5.2  45.7AB   6*
7107   Kap PavCP-67 3603 1746942544133505          Kap Pav  184638.4-672131185657.0-671401328.29-25.39 4.44  +0.71 +0.60         F5I-II             -0.010+0.012 +.010+038V                         *
7108          CD-5012206 174730229358                       184649.9-500002185432.3-495243346.53-20.88 6.60  +0.08               A2V                -0.010-0.012      -008
7109          BD+13 3787 1748531041963503  S       V822 Her 184726.8+135045185201.9+135756 45.48  6.19 6.14   0.00 -0.31         B8Vnn              -0.011-0.005      -029SB2O  125        0.1      *
7110          BD-09 4876 174866142741      S                184732.1-094150185301.9-093434 24.57-04.70 6.34  +0.20 +0.11         A7Vn               +0.040-0.002      -048                 0.2
7111          CP-62 6002 174877254415                       184733.2-625540185654.7-624804333.08-24.54 6.48  +1.53               K3III               0.000+0.032      +027
7112          BD+28 3104 174881 86512                       184739.9+283950185135.9+284701 59.00 12.65 6.18  +1.18               K1II-III           +0.012+0.008      -022
7113112    HerBD+21 3582 174933 86521                       184800.1+211816185216.4+212531 52.27  9.41 5.48  -0.07 -0.42         B9II-IIIpHg        -0.006-0.008 +.007-020SB2O   13                 *
7114 33    SgrBD-21 5176 174947187422    I                  184801.5-212856185400.1-212135 14.00-10.10 5.69  +1.23 +1.05?+0.65?  K1Ib               +0.008-0.015      -007V
7115          BD+36 3295 174959 67485                       184805.6+362508185136.5+363219 66.35 15.72 6.09  -0.11 -0.48         B6IV               -0.007-0.022      -021       60
7116 32Nu 1SgrBD-22 4907 174974187426    I  11794           184807.9-225204185410.2-224442 12.74-10.72 4.83  +1.41 +1.23 +0.69   K2I+B9V:           +0.010-0.010 +.030-012V?         5.8   2.5AB   3*
7117          BD+73  835 174980  9241                       184816.3+735811184546.7+740508105.11 26.39 5.27  +0.92 +0.80         K0II-III           +0.005+0.077 +.020+003V    < 17
7118          BD+41 3167 175132 47874                       184854.5+411541185207.1+412300 71.08 17.43 6.28  -0.09 -0.31         B9IIIpSi           -0.016-0.001      -023       68                 *
7119          BD-15 5143 175156161964                       184859.2-154340185443.1-153611 19.33-07.75 5.10  +0.17 -0.39         B5II               -0.008-0.005      -003V?     19                 *
7120 35Nu 2SgrBD-22 4915 175190187445    I                  184904.4-224746185507.1-224017 12.91-10.89 4.99  +1.33 +1.51 +0.66   K3II-IIIBa0.8CN2   +0.103-0.026 +.042-110SB
7121 34Sig SgrCD-2613595 175191187448 706I W                184903.9-262515185515.9-261748  9.56-12.43 2.02  -0.22 -0.75 -0.21   B2.5V              +0.013-0.054      -011V     201  7.4 309.0      *
7122          CD-4213761 175219229383                       184909.4-425013185616.9-424238353.85-18.85 5.36  +1.00               K0III              -0.035-0.026 +.010-021
7123          BD+52 2294 175225 31217                       184920.5+525045185135.0+525830 82.65 21.33 5.51  +0.84 +0.51 +0.42   G9IVa              -0.046+0.271 +.037+002V?
7124 50    DraBD+75  682 175286  92501494                   184936.1+751858184622.2+752602106.61 26.50 5.35  +0.05 +0.04         A1Vn               -0.015+0.073 +.008-008SB2O   57                 *
7125 47Omi DraBD+59 1925 175306 31218 707I  11779  Omi Dra  184943.5+591558185112.1+592318 89.31 23.14 4.66  +1.19 +1.04 +0.64   G9IIIFe-0.5        +0.079+0.027 +.006-020SB1O   20  3.4  34.6AB   3*
7126          BD-16 5078 1753171619841495                   184945.4-162954185531.0-162236 18.71-08.26 5.79  +0.36 +0.06         F4V                -0.025-0.185      -042V?                        
7127   Ome PavCP-60 7213 1753292544233511                   184943.2-601955185836.4-601202335.92-24.18 5.14  +1.37 +1.44 +0.52E  K2IIICNIV          -0.126+0.031 +.019+180
7128          CD-2314844 175360187468                       184957.4-231804185600.6-231025 12.53-11.29 5.93  -0.02 -0.41         B8                 -0.002-0.009      -016V?
7129          CD-3712982 175362210734              V686 CrA 184953.6-372815185640.5-372036359.16-17.02 5.38  -0.14 -0.71         B8IVSi            v+0.009-0.026      +001V     170                 *
7130          CP-66 3404 175401254426                       184954.2-664705190003.5-663912328.98-25.59 6.01  +0.97 +0.80         K0III              +0.003-0.040      -020V
7131 11Del1LyrBD+36 3307 175426 675373506  W       11504    185013.9+365048185343.6+365818 66.93 15.49 5.58  -0.15 -0.66         B2.5V              +0.004-0.002      -026SBO   123  3.7 174.6      *
7132          BD+27 3150 175443 86558    I                  185014.6+274708185413.2+275434 58.43 11.76 5.62  +1.35               K4III              -0.016-0.074      +015
7133113    HerBD+22 3524 175492 865673508I  11820           185031.5+223106185444.9+223842 53.63  9.42 4.59  +0.78 +0.49 +0.46   G4III+A6V           0.000+0.001 +.013-024SBO    50  7.9  35.7AB   3*
7134   Lam TelCP-53 9402 175510245834 708                   185027.8-530410185827.7-525619343.57-22.35 4.87  -0.05               A0V                +0.010-0.011      -002SB     92
7135          BD+06 3978 1755151240503509I         11536    185034.8+062924185527.5+063655 39.28  2.12 5.57  +1.04 +0.88 +0.55   K0III              +0.021-0.087 +.011+023SBO                       *
7136          CD-3913012 175529210754                       185038.3-395716185734.7-394924356.80-18.08 6.31  +0.20               A3                 +0.037+0.029      +005
7137          BD+50 2686 175535 31241    I                  185044.9+503501185313.6+504230 80.43 20.40 4.92  +0.90 +0.57 +0.45   G7IIIaFe-1         +0.005-0.025 +.024+008V?   < 19:
7138          BD+41 3174 175576 47902                       185100.3+410554185414.3+411332 71.08 17.00 7.30R                     F5                 +0.034+0.017
7139 12Del2LyrBD+36 3319 175588 67559    I  11825  Del2 Lyr 185100.3+364617185430.2+365356 66.93 15.32 4.30  +1.68 +1.65 +1.63   M4II               -0.009+0.010 -.001-026           7.0  86.2AB   3*
7140          BD+33 3257 175635 67566       11834           185112.6+335027185452.5+335807 64.18 14.11 6.02  +0.91 +0.64 +0.50   G8III+A2           -0.008+0.003      -016SB         1.0  45.4AC   4*
7141 63The1SerBD+04 3916 175638124068 709   11853A 11557    185114.9+040424185613.2+041213 37.22  0.85 4.62  +0.17 +0.09         A5V                +0.049+0.031 +.030-046V?    143  0.4  22.2AB   3*
7142 63The2SerBD+04 3917 175639124070       11853B          185116.3+040419185614.6+041207 37.22  0.85 4.98  +0.20 +0.08 +0.07   A5Vn               +0.049+0.026 +.030-053      196  0.4  22.2AB   3*
7143          BD-01 3602 1756401428253510                   185111.0-015543185622.7-014800 31.89-01.92 6.22  -0.05 -0.30         B9III              -0.005-0.022      -026V?                        *
7144          BD+02 3730 175679124073      W                185123.6+022029185625.6+022816 35.70  0.02 6.15  +0.97 +0.71         G8III              +0.007-0.011      -015V          5.7 100. AB   3
7145 36Xi 1SgrBD-20 5339 175687187498                       185123.9-204713185720.5-203923 14.99-10.50 5.08  +0.13 -0.14?        A0II               -0.001-0.007 -.006+002       14
7146          BD+41 3177 175740 47909    I  11840           185140.0+412828185452.2+413610 71.49 17.03 5.44  +1.03               K0III              +0.002-0.002      -009V          6.9  23.6AC   3
7147          BD+17 3778 175744104271              V828 Her 185138.4+175157185603.9+175942 49.54  7.11 6.63  -0.04 -0.44         B9pSi              +0.011-0.018      -025       59                 *
7148          BD+17 3779 175743104272                       185141.4+175848185606.2+180619 49.65  7.15 5.69  +1.09 +1.06         K1III              -0.041-0.162 +.004+044SB                        *
7149   Eta SctBD-06 4976 175751142838    I                  185142.4-055834185703.7-055046 28.36-03.91 4.83  +1.08 +1.02 +0.53   K2III              +0.066-0.036 +.039-093V    < 19:
7150 37Xi 2SgrBD-21 5201 175775187504 710I                  185145.8-211417185743.8-210624 14.61-10.78 3.51  +1.18 +1.13 +0.59   K1III              +0.033-0.012 +.011-020                          *
7151          CD-3116189 175794210773                       185155.9-311002185821.3-310210  5.37-14.96 6.12  +1.35               K3III              -0.045-0.056      +085
7152   Eps CrACD-3713001 1758132107813512          Eps CrA  185158.7-371416185843.4-370627359.54-17.32 4.87  +0.41 +0.03         F2V                -0.132-0.105 +.032+054SBO   132:                *
7153          BD+57 1915 175823 31252                       185202.2+572134185346.3+572913 87.41 22.31 6.22  +1.23              gK5                 +0.018-0.006      -005
7154          BD+48 2793 175824 47913       11846           185208.8+484403185447.1+485135 78.66 19.56 5.77  +0.43 +0.02         F3III              -0.061-0.118 +.017-011V?     50  4.7   1.5AB   3*
7155          CD-2513574 175852187517      D                185212.6-250035185820.5-245236 11.17-12.48 6.62  +0.08               A0                 +0.007+0.003      -017           0.0   0.1      *
7156          CD-3913032 175855210786      W                185216.3-394004185911.1-393205357.20-18.27 6.49  -0.04               A0                 +0.009-0.035      -015           5.3  15.9
7157 13    LyrBD+43 3117 175865 47919 711I         R Lyr    185217.5+434851185520.1+435646 73.81 17.79 4.04  +1.59 +1.41 +1.91   M5III              +0.023+0.083 +.000-028SBO                       *
7158 64    SerBD+02 3738 175869124089                       185214.8+022414185716.6+023207 35.85-00.14 5.57   0.00 -0.27         B9IIIep:Hg:       v-0.005-0.012      -010V     105                 *
7159          BD-22 4928 175892187519                       185223.4-223947185824.6-223146 13.36-11.52 6.14  +0.09               A2V                -0.022+0.017      -007
7160          BD+79  604 175938  92563501                   185241.7+794920184538.1+795633111.64 26.93 6.39  +0.28 +0.04         A8V                +0.015+0.057      -005V?    113
7161          CP-68 3180 175986254446      W                185248.5-685342190329.7-684519326.72-26.23 5.88  +0.56 +0.13         F8V                -0.015-0.003      +025           0.0   0.2      *
7162          BD+32 3267 176051 67612       11871           185316.6+324623185701.6+325405 63.35 13.27 5.22  +0.59 +0.03 +0.34   F9V                +0.172-0.157 +.057-047V       4  2.2   0.7AB   6*
7163          BD+06 3989 176095124112                       185330.4+060630185823.8+061425 39.28  1.30 6.21  +0.46 +0.01         F5IV               -0.003-0.104      -009    =< 10
7164          BD-18 5155 176123162050      D                185335.7-184206185926.8-183401 17.12-10.06 6.37  +0.99               G3II               +0.003-0.043      -014V                         *
7165          BD+17 3799 176155104296       11884  FF Aql   185347.7+171335185814.7+172139 49.21  6.36 5.38  +0.80 +0.43         F8Ib               -0.003-0.007 -.007-022SBO    17  5.3   6.6      *
7166          BD-13 5172 176162162052      W       Var      185346.7-125834185923.8-125026 22.33-07.55 5.53  -0.04               B5IV               +0.002-0.018D+.006-013SB    185  0.5   0.4      *
7167 10    AqlBD+13 3838 176232104303              V1286 Aql185411.4+134620185846.9+135424 46.16  4.70 5.89  +0.25 +0.09         F0pSrEu            -0.002-0.052      +015    =<  6                 *
7168          CD-2513614 176246187562                       185416.6-250452190024.8-245632 11.31-12.94 6.36  +1.25              gK0                 +0.055+0.044      -025
7169          CD-3713017 176269210815    I W     B          185417.7-371156190103.2-370339359.76-17.73 6.69                      B9V                +0.011-0.020      +012SB    172  0.2  12.8      *
7170          CD-3713018 176270210816      W     A          185418.8-371158190104.3-370343359.76-17.74 6.40  -0.03               B8V-IV             +0.010-0.040      -015SB  =< 39  0.2  12.8      *
7171          BD+19 3858 176301104306                       185424.1+193930185845.1+194739 51.45  7.34 6.50  -0.04 -0.43         B7III-IV           +0.004-0.004      -001      125
7172 11    AqlBD+13 3841 176303104308       11902  11618    185429.4+132921185905.7+133721 45.94  4.50 5.23  +0.53 +0.07 +0.30   F8V                +0.014-0.123 +.039+016V?     26  4.3  17.8AB   3*
7173          BD+09 3951 176304104313                       185432.9+100015185917.5+100827 42.85  2.88 6.75  +0.25 -0.43         B2Vp               +0.025-0.011      -015V?     49
7174          BD+38 3373 176318 67642                       185436.5+380751185801.9+381558 68.51 15.21 5.89  -0.17 -0.52         B7IV               +0.001 0.000      -028SB1O  125                 *
7175 48    DraBD+57 1922 176408 312843513                   185503.5+574056185645.0+574854 87.87 22.02 5.66  +1.15 +1.19         K1III              -0.029-0.062 +.018-034V?
7176 13Eps AqlBD+14 3736 176411104318 712I W                185505.0+145557185937.4+150406 47.30  5.04 4.02  +1.08 +1.04 +0.52   K1-IIICN0.5Ba0.2   -0.051-0.073 +.028-048SB   < 17  5.9 131.1AB   3*
7177          CD-4213839 176425229446                       185504.5-420304190208.7-415436355.04-19.60 6.23   0.00               A0V                +0.048+0.002      -013
7178 14Gam LyrBD+32 3286 176437 67663 713   11908  11624    185512.1+323308185856.6+324122 63.32 12.81 3.24  -0.05 -0.09 -0.01   B9III              -0.002+0.002 +.021-021V      76  8.3 176.9AC   4*
7179          BD+40 3544 176502 47965       11910           185530.2+403230185846.6+404045 70.88 16.00 6.22  -0.16 -0.65         B3V                +0.008 0.000      -014SB         3.5  19.0      *
7180 52Ups DraBD+71  915 176524  9283 714I                  185537.4+710949185423.9+711750102.15 25.36 4.82  +1.15 +1.10 +0.56   K0IIIBa0.2         +0.049+0.044 +.014-007SB   < 17
7181          BD+26 3418 176527 86673    I                  185541.0+260531185945.5+261350 57.41  9.94 5.27  +1.24 +1.27         K2III              +0.088-0.008 -.002-024SB   < 17
7182          BD-22 4946 176537187584    I                  185536.0-225011190137.8-224144 13.53-12.26 6.24  +1.66               K5                 +0.004 0.000      -014V?
7183          BD+22 3549 176541 86675    I                  185545.0+224031185958.1+224853 54.32  8.41 6.29  +1.75 +2.00         M3.5IIIab          -0.022+0.012      -053
7184          BD+58 1849 176560 31292       11897           185549.5+580515185728.4+581331 88.33 22.04 6.46  +0.08 +0.05         A2IV               +0.014+0.047 -.008-008V?    105  0.2   0.8      *
7185          BD+39 3602 176582 67675                       185550.2+390445185912.3+391304 69.51 15.36 6.41  -0.17 -0.70         B5IV               +0.002+0.012      -014V                         *
7186          BD-15 5185 176593162097                       185550.6-152525190133.5-151657 20.35-09.10 6.32  +1.00 +0.78        gG6                  0.000+0.003      +020V?
7187          BD+65 1309 176598 180793514                   185559.0+650725185625.6+651529 95.68 23.95 5.63  +0.95 +0.69         G8III              -0.028-0.032 -.002-005V?
7188   Zet CrACD-4213855 1766382294613519                   185601.9-421413190306.9-420543354.91-19.83 4.75  -0.02 -0.07         B9.5V              +0.061-0.049 +.031-013
7189 NOVA 1899                                     V1016 Sgr                                                                                                                                        *
7190          CD-5111893 176664245899      W                185609.8-510931190357.4-510107345.82-22.64 5.93  +1.24 +1.38         K0-1III            +0.028-0.146      -061           6.5  22.0AC   3
7191          BD+62 1669 176668 18082       11901           185617.0+621541185717.4+622348 92.68 23.18 6.45  +0.93 +0.66         G5IV+G8V           +0.006-0.039      -008V?         2.8  17.0AB   4*
7192 15Lam LyrBD+31 3424 176670 67682    I         11641    185614.4+320020190000.9+320844 62.91 12.38 4.93  +1.47 +1.67 +0.73   K2.5IIIBa0.5       +0.013+0.013 -.003-016V    < 19:
7193 12    AqlBD-05 4840 176678142931    I         11655    185620.4-055248190140.8-054420 28.98-04.89 4.02  +1.09 +1.04 +0.54   K1III             v-0.019-0.033 +.021-044V    < 17
7194 38Zet SgrCD-3016575 176687187600    I  11950           185615.0-300123190236.7-295249  6.84-15.35 2.60  +0.08 +0.06 +0.01   A2III+A4IV         -0.015-0.002 +.025+022SB     72  0.3   0.5AB   3*
7195          CD-2513655 176704187599      D                185620.4-245905190227.7-245049 11.60-13.32 5.65  +1.23               K3III              -0.022-0.175      +002V?         0.0   0.1      *
7196          BD+50 2705 176707 31304                       185630.3+504012185859.6+504834 80.84 19.57 6.30  +0.98               G8III              +0.024+0.018      -021
7197          CD-3813300 176723210859                       185628.1-382350190317.7-381512358.75-18.57 5.74  +0.32               F0IIIn             +0.013+0.009      +004V?    203                 *
7198          BD+19 3879 176776104362                       185643.1+191005190105.5+191835 51.26  6.63 6.39R                     K1III              +0.018+0.002      -029
7199          BD+75  683 176795  9286       11870           185655.3+753914185333.2+754715107.07 26.10 6.22R                     A1V                +0.013+0.024 -.004-017SB    100  0.8   5.6      *
7200          BD+20 4022 176819 86704              11659    185704.1+204127190122.6+205001 52.67  7.25 6.69  +0.02 -0.69         B2IV-V             +0.007+0.017      -010SBO                       *
7201          BD+40 3555 176844 47993    I         11652    185702.5+403236190019.0+404103 71.00 15.73 6.65R                     M4IIIa             +0.002-0.011      -005
7202          BD+26 3429 176871 867073518                   185713.6+260857190117.4+261729 57.62  9.65 5.69  -0.08 -0.54         B5V                 0.000-0.011 +.009-014V     296
7203          BD-19 5273 176884162130       11972           185711.1-192323190303.8-191443 16.87-11.12 6.05  +1.29 +1.09         G6III              +0.007+0.006      -020V?         3.5   7.4AB   4*
7204          BD+33 3287 176896 67699                       185714.0+333936190055.2+334808 64.53 12.89 6.01  +0.97              gK0                 +0.007+0.003      -028V
7205          BD-19 5275 176903162133      D                185714.6-191451190307.0-190612 17.00-11.07 6.37  +0.48               F5IV-V             +0.007-0.015      +019V?         0.0   0.1      *
7206          BD+24 3608 176939 86714                       185727.6+245258190134.9+250133 56.49  9.05 6.72R                     K2                 -0.001 0.000      -021
7207          BD+22 3561 176971 86716                       185734.6+220714190149.5+221550 54.01  7.78 6.40R +0.16?              A4V                +0.021-0.008      -038V?
7208          BD+08 3951 176981124184                       185732.9+081346190221.6+082227 41.63  1.40 6.30  +1.67 +1.73         K2III              +0.026+0.022      -009
7209 14    AqlBD-03 4460 176984142959      W                185738.5-035038190254.5-034156 30.94-04.23 5.42   0.00 -0.07         A1V                +0.023+0.007      -039V?     56  0.0   0.1      *
7210          BD+50 2708 177003 313113515                   185742.8+502330190013.7+503201 80.64 19.29 5.38  -0.18 -0.75         B2.5IV             +0.009+0.005      -019SB     13
7211          CD-3116306 177074210883                       185759.8-311137190425.1-310249  5.88-16.16 5.50  +0.03               A0IV               +0.014-0.016      -026V?    107
7212          BD+33 3295 177109 67721       11965           185806.5+332838190148.3+333717 64.44 12.65 6.39  -0.12 -0.62         B5IV               -0.003+0.003      -023           6.2  25.4
7213   Rho TelCP-5211356 177171245921                       185824.9-522915190619.9-522027344.54-23.35 5.16  +0.53 +0.04         F7V                +0.030-0.118 +.022+002SB2    46                 *
7214          BD+01 3865 177178124203                       185828.7+014027190332.2+014908 35.93-01.86 5.83  +0.18 +0.10         A4V                +0.006-0.067      -022SB2   160
7215 16    LyrBD+46 2602 177196 48011       11964  11677    185836.5+464735190126.4+465605 77.15 17.86 5.01  +0.19 +0.08 +0.09   A7V                +0.020-0.084 +.036+008V     121  6.3 200. AC   3*
7216          BD+19 3888 1771991044053521                   185831.1+193055190252.6+193940 51.77  6.41 6.09  +1.34               K1III              +0.004-0.001      -007
7217 39Omi SgrBD-21 5237 177241187643    I  11996  11703    185841.4-215317190441.0-214430 14.71-12.51 3.77  +1.01 +0.85 +0.53   G9IIIb             +0.080-0.060 +.044+025           9.9  35.7
7218 49    DraBD+55 2137 177249 31323                       185844.6+553054190043.4+553930 85.83 20.86 5.48  +0.86               G5.5IIbFe-0.5      -0.016-0.006      +010V
7219          BD+03 3882 177332124219                       185910.5+031055190410.7+031950 37.35-01.31 6.73  +0.13 +0.14 +0.09   A5m                +0.007+0.013      -013
7220          BD-05 4858 177336142985    I         V Aql    185903.7-054959190424.2-054106 29.33-05.47 6.90  +4.19               C5II               +0.020-0.005      +037V?                        *
7221          CP-68 3185 177389254475                       185916.7-683441190952.7-682529327.17-26.76 5.33  +0.91 +0.61         G8-K0III-IV        +0.145-0.052 +.030-010
7222          BD+21 3648 177392 86753              LT Vul   185924.9+210714190342.5+211604 53.30  6.95 6.52  +0.32?              F2III              +0.022-0.021      +005      120                 *
7223          CD-4812901 1774062294933523                   185923.3-482701190655.6-481757348.80-22.36 5.97  -0.02 -0.02         A0V                +0.016-0.016      -006
7224          BD+69 1018 177410 181033517          EE Dra   185929.9+692320185852.6+693152100.31 24.65 6.52  -0.15 -0.53         A0pSi              +0.005+0.004      -015                          *
7225 15    AqlBD-04 4684 177463142996    I  12007A          185940.9-041049190457.6-040153 30.88-04.84 5.42  +1.12 +1.01 +0.61   K1III              +0.020-0.027 +.002-018V?  =< 25  1.6  38.5      *
7226   Gam CrACD-3713048 177474210928      W                185939.5-371225190625.1-370348  0.16-18.73 4.93  +0.52 -0.00         F8V                +0.096-0.274 +.054-052SB      0  0.1   1.6      *
7227   Gam CrACD-3713048 177475210928      W                185939.5-371225190625.1-370348  0.16-18.73 4.99  +0.52 +0.00         F8V                +0.096-0.274 +.054-052           0.1   1.6      *
7228   Sig OctCP-89   47 177482258857 923          Sig Oct  185944.1-891517210846.2-885723303.91-27.71 5.47  +0.27 +0.13         F0III              +0.023+0.005      +012      108                 *
7229          BD+52 2326 177483 31337       11979           185945.9+520656190207.0+521540 82.47 19.59 6.31  +1.00               G8III              -0.007-0.026D+.005+004V?         3.4   5.2AxBC 3*
7230          BD-15 5223 1775171621773522  W       Var?     185957.5-154839190541.2-153937 20.44-10.16 5.97  -0.02 -0.25         B9V                +0.003-0.007      -026SB     95  6.2  46.9      *
7231          BD-01 3642 177552143003                       190007.7-013947190518.6-013046 33.17-03.78 6.53  +0.35 -0.04         F1V                -0.003-0.007      -032       45
7232          CD-3713049 177565210937                       190006.6-375710190652.5-374837359.46-19.08 6.16  +0.72 +0.28         G5IV               -0.184-0.352 +.063+059
7233          CP-55 9001 177693245937                       190035.4-555221190852.1-554313341.05-24.51 6.49  +1.10               K1III              +0.031-0.110      -021
7234 40Tau SgrCD-2713564 1777161876831496I                  190041.8-274900190656.4-274014  9.34-15.37 3.32  +1.19 +1.15 +0.59   K1+IIIb            -0.053-0.251 +.044+045SB                        *
7235 17Zet AqlBD+13 3899 177724104461 716I  12026  11724    190048.8+134253190524.6+135148 46.86  3.25 2.99  +0.01 -0.01  0.00   A0Vn               -0.005-0.096 +.045-025SB    331  8.4 158.6AC   3*
7236 16Lam AqlBD-05 4876 177756143021 717I'                 190056.5-050157190614.9-045257 30.26-05.51 3.44  -0.09 -0.27 -0.09   B9Vn               -0.017-0.090 +.032-012V     176
7237          BD+31 3453 177808 67782    I                  190109.2+313542190457.9+314440 62.98 11.26 5.56  +1.54 +1.91 +0.91   M0III              +0.076-0.067 +.018+006V?
7238          BD+30 3409 177809 67781    I                  190106.9+303458190458.3+304400 62.05 10.83 6.06  +1.55 +1.88         M2.5IIIab          +0.030-0.022      -016SB
7239          BD-16 5153 177817162201       12039           190107.1-162257190652.3-161344 20.04-10.66 6.03  -0.04 -0.35         B8IV               +0.021+0.004      -017V?         4.0   6.4      *
7240          CD-2815403 177846187701    I                  190113.1-284727190730.9-283813  8.46-15.86 6.04  +1.61               K3III              +0.008-0.010      +005V
7241          BD-18 5206 177863162204              11743    190117.2-185330190708.5-184416 17.75-11.78 6.29  -0.04 -0.41         B8III              +0.019-0.004      -011V
7242   Del CrACD-4013061 177873229513                       190123.3-403906190820.9-402948356.86-20.24 4.59  +1.09 +1.03         K1III              +0.037-0.026 +.016+020
7243          BD+08 3970 177940124266    I W       R Aql    190133.2+080443190622.2+081348 41.95  0.45 6.09  +1.60 +0.37         M7IIIe            v+0.006-0.071      +032V          5.0 180.7AC   3*
7244          BD+29 3472 178003 86796    I                  190153.4+294608190547.1+295518 61.38 10.32 6.31  +1.65 +1.98         M0III              -0.005-0.009      -028
7245          BD+00 4106 178065124282                       190202.9+002911190709.1+003830 35.30-03.20 6.56  +0.05 -0.28         B9III              +0.003+0.009      -011V       5                 *
7246          CD-2415041 178075187718                       190208.0-244848190814.6-243925 12.32-14.45 6.30  +0.04 -0.24         B9III              +0.018+0.005      -013
7247          BD+76  712 178089  9296                       190209.1+765430185757.2+770303108.50 26.02 6.54  +0.38 -0.06         F2V                -0.039-0.064      -027V?     15
7248 18    AqlBD+10 3787 1781251044883525          Y Aql    190216.1+105502190658.6+110417 44.55  1.63 5.09  -0.07 -0.39         B8III              +0.005-0.026 +.016-019SBO    57                 *
7249          BD-19 5312 178175162229              V4024 Sgr190224.2-192649190816.7-191724 17.35-12.26 5.54  -0.11 -0.78 -0.04   B2Ve              v+0.006+0.008      -020SB    176                 *
7250          BD+24 3640 178187 86817                       190228.2+240544190638.4+241503 56.30  7.67 5.77  +0.09?              A4III              +0.055+0.015      -022V?
7251 51    DraBD+53 2178 178207 31371                       190240.3+531435190455.2+532348 83.76 19.56 5.38  -0.01 -0.08         A0Vn               -0.003+0.025 +.023-024SB    173
7252          BD+49 2929 178208 48071       12034           190234.8+494611190509.9+495524 80.32 18.34 6.43R                     K3III              -0.004+0.014      +008           5.3  80. AC   3
7253          BD+28 3193 178233 868191498                   190239.6+282816190637.7+283743 60.28  9.60 5.55  +0.29 +0.04         F0III              +0.076+0.085 +.025-024SB    158
7254   Alp CrACD-3813350 178253210990 718                   190240.2-380336190928.3-375416359.54-19.59 4.11  +0.04 +0.08  0.00   A2V                +0.084-0.098 +.036-018      201                 *
7255          CD-4013074 178254210994                       190245.2-395907190939.7-394941357.62-20.26 6.46  +1.06               K0III              -0.001-0.049      +017
7256          CD-3613355 1782992109963528                   190255.1-361924190936.4-360953  1.28-19.02 6.56  -0.02               A0IV               -0.008-0.013      -011SB2                       *
7257          CD-4213933 178322229531                       190255.0-420304190957.8-415333355.55-20.98 5.88  -0.08 -0.46         B6V+F1IV           +0.010-0.020      +013SB2O  102                 *
7258          BD+41 3232 178329 480843526                   190302.5+411532190617.0+412450 72.15 14.98 6.49  -0.15 -0.64         B3V                +0.003-0.002      -021SBO                       *
7259   Bet CrACD-3913146 178345211005                       190309.0-392958191001.7-392027358.14-20.17 4.11  +1.20 +1.07 +0.61   K0II               +0.003-0.038 +.016+003
7260          BD+16 3752 178428104511      W       Var?     190328.1+164216190757.3+165112 49.80  4.07 6.07  +0.70 +0.27 +0.36   G5V                +0.060-0.306 +.059+014SB1O       4.5  21.5      *
7261 17    LyrBD+32 3326 178449 67835       12061           190338.6+322038190725.6+323006 63.91 11.11 5.23  +0.34 +0.07         F0V                +0.130+0.023 +.017+004SB1O  127  4.3   3.5AB  10*
7262 18Iot LyrBD+35 3485 178475 67834 719  S                190343.9+355636190718.1+360601 67.23 12.65 5.28  -0.11 -0.51         B6IV               +0.001-0.004      -018V     250                 *
7263          BD+21 3672 178476 86843      S                190346.6+213220190803.6+214156 54.15  6.24 6.23  +0.40 +0.01         F3V                +0.051+0.074 +.008-040       50        0.2      *
7264 41Pi  SgrBD-21 5275 178524187756 720I W       11769    190349.0-211057190945.8-210125 15.89-13.29 2.89  +0.35 +0.22 +0.24   F2II                0.000-0.035 +.026-010       28  0.0   0.1AB   3*
7265          BD-20 5415 178555162260       12096           190354.3-195741190948.1-194813 17.03-12.80 6.13  +1.16 +1.13         K1III              +0.020-0.079D+.004+030V          0.2   0.1      *
7266 19    AqlBD+05 4040 1785961243183530                   190406.0+055457190859.9+060424 40.34-01.12 5.22  +0.35 +0.04         F0III-IV           -0.006-0.076 +.032-047V     104
7267          BD+16 3758 1786191045243529                   190411.6+164142190840.2+165105 49.88  3.91 6.48  +0.52 +0.25         F5IV-V             -0.036-0.104 +.018+010SB2O                      *
7268          CD-3913156 178628211017                       190410.6-390959191101.8-390018358.54-20.25 6.36  +0.01               B7IIIMn            -0.011-0.018      -005SB                        *
7269          BD-00 3662 178744143087                       190443.0-003521190951.6-002541 34.65-04.29 6.34  -0.04 -0.49         B5Vn               +0.006-0.006      -011
7270          CD-2915804 178840187786                       190458.7-293954191118.9-293008  7.95-16.96 6.30  -0.04               B9                 +0.033-0.018      -014
7271          CD-5012377 178845245976      W                190503.0-503901191246.1-502911346.77-23.85 6.13  +0.95               G8III              +0.040-0.044      -026           5.3   7.
7272          BD+34 3439 178911 67879       12101           190524.5+342603190904.4+343602 65.99 11.69 6.74  +0.63 +0.15         G1V                +0.053+0.195 +.025-041V?         1.5  16.4AB   4*
7273          CD-3713090 178937211037                       190523.8-374451191209.8-373458  0.05-19.99 6.57  +1.02               G2III              +0.010+0.008      -039V?
7274   Tau PavCP-69 2962 179009254508                       190544.7-692136191628.5-691126326.38-27.43 6.27  +0.17 +0.14         A6IV-V              0.000-0.024      -018
7275          BD+52 2350 179094 31413    I         V1762 Cyg190605.2+521558190825.8+522532 82.98 18.73 5.81  +1.09 +0.87         K1IV               -0.099-0.059 +.002+004SBO                       *
7276          BD-21 5292 179201187816                       190629.5-214926191228.0-213929 15.55-14.13 6.41  +1.13              gG8                 +0.027-0.012      -005SB
7277          CD-2613936 1793231878353533I                  190704.1-260428191313.7-255424 11.59-15.98 5.80  +1.38 +1.42?        K1I                -0.002-0.006      +003
7278          CP-66 3417 179366254515      W                190708.8-665000191712.0-663941329.22-27.27 5.53  +0.18 +0.13         A5V+A8V            -0.002-0.021      +012           0.3   0.4      *
7279 20    AqlBD-08 4887 1794061431341500          11808    190715.2-080624191240.7-075622 28.23-08.31 5.34  +0.13 -0.44         B3V                +0.016-0.006      -015V     187
7280          BD+26 3474 179422 86912                       190727.3+263414191130.9+264409 59.04  7.79 6.36  +0.41 -0.03         F5V                +0.023-0.036      -027       40
7281          CD-4513054 179433229573                       190723.1-452144191439.7-451136352.43-22.76 5.92  +0.90               G8III              +0.059-0.036 +.022-035
7282          BD-12 5311 1794971623263534I         11815    190740.1-122701191315.5-121657 24.34-10.35 5.51  +1.44              gK4                 +0.012-0.027      -018
7283 19    LyrBD+31 3497 179527 679463532          V471 Lyr 190755.8+310659191146.0+311700 63.19  9.75 5.98  -0.07 -0.30         B9pSi              -0.002-0.003      -030       36                 *
7284          BD+40 3620 179583 48168                       190803.9+401547191123.1+402545 71.62 13.70 6.18  +0.09 +0.10         A3V                 0.000-0.023      +006
7285          BD+16 3775 179588104602       12160           190805.4+164043191234.4+165047 50.30  3.08 6.73  -0.01 -0.28         B9IV               -0.006-0.015 +.007-018V          1.4 113.4AC   5*
7286          BD+21 3690 179648 86930                       190819.3+212309191236.7+213316 54.50  5.23 5.93R +0.03 +0.02         A2Vn               +0.005-0.002      -013V?    160
7287 21    AqlBD+02 3824 1797611244083537   12182  V1288 Aql190840.1+020725191342.7+021737 37.52-03.90 5.15  -0.07 -0.41         B8II-III           +0.009-0.003 +.009-005V      19  7.5  36.2      *
7288          BD+05 4081 179791124410              11827    190848.6+052044191344.1+053057 40.39-02.42 6.49  +0.09 +0.11         A2Vp               +0.016-0.006      +009V?    175                 *
7289          CD-4513072 179886229584                       190904.9-453825191621.7-452759352.23-23.13 5.40  +1.35               K3III              -0.003+0.002 +.010+006
7290 55    DraBD+65 1326 179933 181873531                   190923.3+654840190945.8+655843 96.82 22.80 6.25   0.00 -0.02         A0V                +0.001+0.030      -022       68
7291          CD-2415161 179949187883                       190927.7-242059191533.2-241045 13.46-15.78 6.25  +0.54 +0.09         F9V                +0.116-0.095 +.023-026
7292 42Psi SgrCD-2513866 179950187882    I: 12214           190924.5-252545191532.4-251524 12.42-16.20 4.85  +0.56 +0.32 +0.34   G8:III+A8V         +0.045-0.025 +.005-033SB2O    0  0.3   0.1AB   3*
7293          BD+49 2959 179957 48192       12169B          190929.5+494002191204.6+495115 80.67 17.27 6.75  +0.65 +0.19         G4V                -0.190+0.637 +.044-041V   =< 54  0.2   8.1AB   4*
7294          BD+49 2959 179958 48193       12169A          190930.2+494009191205.0+495122 80.67 17.27 6.57  +0.65 +0.21         G4V                -0.210+0.635 +.044-038V   =< 54  0.2   8.1AB   4*
7295 53    DraBD+56 2209 180006 314683535I         11819    190947.1+564120191140.5+565133 87.56 19.81 5.12  +1.01 +0.84 +0.49   G8III              +0.039+0.048 +.012-016     < 19:
7296          CD-3314076 180093211117    I         RY Sgr   191000.9-334150191632.8-333118  4.43-19.45 6.25  +0.02 -0.07         G0Ipe(C1,0)        +0.012+0.024      -023V                         *
7297          CP-53 9513 180134246017                       191011.0-533343191809.4-532312343.88-25.32 6.38  +0.49  0.00         F7V                -0.013-0.065      -024
7298 20Eta LyrBD+38 3490 180163 68010       12197  11839    191021.2+385826191345.5+390846 70.62 12.75 4.39  -0.15 -0.65 -0.15   B2.5IV             +0.004+0.002 +.002-008SB1O   24  4.0  28.3AB   3*
7299          BD+19 3956 180242 86981                       191041.7+200145191502.6+201212 53.56  4.11 6.00  +0.88               G8III              -0.012 0.000      +007
7300          BD+14 3846 180262104655      W                191046.7+145434191520.1+150501 49.05  1.67 5.57  +1.07 +0.87 +0.54   G8II-III           +0.002-0.011      -025V          2.1  89.6      *
7301  1    SgeBD+20 4088 180317 86987                       191058.8+210326191517.4+211356 54.50  4.53 5.64  +0.11               A4V                +0.039+0.012      -023V     231                 *
7302          BD+30 3491 180450 68040    I                  191131.7+302106191524.8+303135 62.85  8.72 5.85  +1.67 +2.03         M0III              +0.029-0.023      -063
7303 22    AqlBD+04 4045 180482124455                       191134.0+043930191631.0+045005 40.10-03.35 5.59  +0.08 +0.11         A3IV               +0.015-0.016 +.016-023SB     73
7304 43    SgrBD-19 5379 180540162413 722  D                191147.0-190751191738.1-185711 18.61-14.13 4.96  +1.02 +0.80         G8II-III           -0.012-0.014 +.006+015V    < 19:                *
7305          BD+27 3313 180553 87005       12239           191155.1+271646191557.0+272720 60.14  7.23 6.54  -0.06 -0.59         B8V                +0.007-0.018D+.002-009SBO    50  0.4   0.9AB   4*
7306  1    VulBD+21 3713 180554 870103540   12243  11866    191155.0+211249191613.0+212325 54.74  4.41 4.77  -0.05 -0.54 -0.07   B4IV                0.000-0.009 +.017-017SB1O  130  7.1  39.1AB   3*
7307          BD+14 3852 180555104668       12248           191152.0+142203191626.8+143241 48.69  1.19 5.63  -0.02 -0.16         B9.5V              +0.010+0.009 +.021-019V?    147  3.8   8.2AB   4*
7308          BD+27 3314 180583 87008              V473 Lyr 191158.9+274459191559.4+275537 60.56  7.44 6.16  +0.61 +0.34         F6Ib-II            +0.001+0.022      -016V      17                 *
7309 54    DraBD+57 1968 180610 31497    I                  191208.0+573157191355.2+574218 88.52 19.81 4.99  +1.16 +1.17 +0.58   K2III              -0.010-0.070 +.025-027V    < 19:
7310 57Del DraBD+67 1129 180711 18222 723I W                191231.9+672908191233.3+673942 98.65 23.00 3.07  +1.00 +0.78 +0.51   G9III              +0.094+0.093 +.032+025V    < 17  9.2  88.1      *
7311          BD+49 2968 180756 48247       12240           191242.8+495339191519.3+500415 81.09 16.87 6.27R                     G8III              +0.005-0.003      +006V?         4.5   2.1AB   3*
7312 59    DraBD+76  717 180777  93413536          11824    191250.3+762339190909.8+763338108.08 25.31 5.13  +0.31  0.00 +0.17   A9V                +0.047-0.123 +.049-004       63                 *
7313          BD+01 3960 180782124478                       191245.1+015110191748.2+020154 37.76-04.93 6.19  +0.02 +0.01         A1Vn               -0.004-0.030      -023V?    215                 *
7314 21The LyrBD+37 3398 180809 68065 724I W       11873    191253.8+375720191622.1+380801 69.90 11.86 4.36  +1.26 +1.23 +0.59   K0+II              -0.001+0.004 +.013-031V    < 19: 4.7 100.2AB   3
7315 25Ome1AqlBD+11 3790 180868104691 725                   191307.3+112454191749.0+113543 46.24-00.49 5.28  +0.20 +0.22         F0IV               +0.003+0.013 +.014-014V     113
7316          CD-3513393 1808852111751501                   191302.4-353612191940.0-352517  2.76-20.69 5.59  -0.13 -0.56         B3V                +0.007-0.013      +002V     158:                *
7317          BD-15 5310 180928162462    I W                191318.0-154238191900.1-153211 21.95-13.00 6.06  +1.43 +1.57 +0.83   K3III              -0.091-0.267 +.041-018V          5.5  48.6
7318  2    VulBD+22 3648 180968 87036       12287  ES Vul   191329.7+225043191743.6+230132 56.36  4.85 5.43  +0.02 -0.79         B0.5IV             +0.001-0.005D+.007+001V     332  4.7   1.8AB   3*
7319 23    AqlBD+00 4168 180972124487    I  12289  11889    191327.2+005412191832.5+010507 37.00-05.53 5.10  +1.15 +1.01 +0.58   K2II-III          v+0.010+0.019 +.003-024V?   < 19: 4.1   3.1AB   3*
7320          CP-68 3218 181019254551                       191339.6-683332192405.4-682216327.36-28.06 6.34  +1.23 +1.25         K2III              -0.026-0.003      -020
7321 24    AqlBD+00 4170 181053124492      W                191344.0+000924191850.9+002021 36.37-05.94 6.41  +1.05 +0.79         K0-IIIa:Ba0.4CH1   +0.012+0.012 +.010-029V?         0.2 423.4AB   4
7322          BD+46 2658 181096 482783541                   191359.2+464840191651.4+465957 78.24 15.44 6.00  +0.44 -0.01         F6IV:              -0.008+0.295 +.018-044V?  =<  6
7323          CD-3215071 181109211191    I                  191400.3-320007192026.2-314904  6.43-19.62 6.58  +1.67 +2.00         M0III              -0.004+0.004      -007
7324          BD+30 3502 181119 68095                       191409.1+305025191800.8+310120 63.56  8.44 6.68  +0.08 +0.13         A3V                -0.004+0.020      -025
7325          BD+09 4057 181122124497                       191406.5+092612191852.7+093705 44.61-01.64 6.32  +1.05 +0.89         G9III              +0.003-0.039      -012                          *
7326          BD+19 3975 181182104711      W       U Sge    191425.7+192540191848.5+193638 53.44  3.05 6.58  +0.03 -0.38         B8III+K:           +0.022+0.002      -017SBO    76  3.0  92.0      *
7327          BD-22 5063 181240187992                       191438.6-223519192038.1-222409 15.63-16.16 5.58  +0.27               F0III:             -0.015+0.035      -006       67
7328  1Kap CygBD+53 2216 181276 31537 726I         11886    191447.5+531102191706.2+532207 84.40 17.85 3.77  +0.96 +0.74 +0.47   G9III              +0.059+0.125 +.024-029SB   < 17                 *
7329   Eta TelCP-54 9339 181296246055                       191447.0-543634192251.1-542525342.90-26.21 5.05  +0.02               A0Vn               +0.015-0.072 +.020+013      323
7330          CD-3513422 181321211206                       191453.2-351002192129.9-345902  3.33-20.90 6.48  +0.63 +0.11         G5V                +0.106-0.099      -010                          *
7331 28    AqlBD+12 3879 181333104722      W       V1208 Aql191459.3+121124191939.3+122229 47.14-00.52 5.53  +0.26 +0.18         F0III              +0.008+0.021 +.004+003V?     59  3.5  60.2AB   3*
7332 29Ome2AqlBD+11 3802 181383104728                       191510.8+112058191953.0+113206 46.42-00.96 6.02  +0.08 +0.07         A2V                +0.042+0.026 +.001-026      160
7333 26    AqlBD-05 4936 1813911432863544I W                191512.6-053610192032.9-052457 31.39-08.93 5.01  +0.92 +0.63 +0.50   G8III-IV           +0.116+0.048 +.030-019SB1O < 17  6.8 115.9      *
7334          CD-4214133 181401229643                       191508.8-421209192209.6-420058356.15-23.18 6.34  +1.14               K1III              -0.050-0.034      +093
7335          BD+33 3409 181409 68125                       191519.3+331221191903.8+332320 65.80  9.30 6.60  -0.19 -0.89         B2IVe             e-0.010-0.033      +010SB                        *
7336 27    AqlBD-01 3716 181440143292                       191526.1-010442192035.7-005332 35.47-06.89 5.49  -0.04 -0.23         B9III              +0.006+0.006 +.008-027SB     77
7337   Bet1SgrCD-4413277 1814542296461502  W                191527.0-443848192238.3-442732353.61-23.93 4.01  -0.10 -0.39         B9V                +0.009-0.020      -011       84  3.3  28.3      *
7338          BD+37 3413 181470 68129                       191529.9+371539191901.2+372643 69.49 11.09 6.22R -0.03 -0.09         A0III              +0.014+0.017      -014SBO     5                 *
7339          BD-19 5412 181558162511      W                191545.5-192517192137.1-191403 18.74-15.10 6.26  -0.10 -0.51         B5V                +0.007-0.001      -023           3.6  90.8AC   3*
7340 44Rho1SgrBD-18 5322 181577162512      D       Rho1 Sgr 191552.4-180208192140.4-175050 20.06-14.55 3.93  +0.22 +0.13 +0.10   F0IV-V             -0.024+0.024 +.042+001       68  2.5   0.0      *
7341          BD+49 2977 181597 48315                       191557.7+492302191837.7+493411 80.82 16.19 6.31  +1.12               K1III             v+0.010+0.054 +.003-014                          *
7342 46Ups SgrBD-16 5283 181615162518 727I:D       Ups Sgr  191600.0-160834192143.6-155718 21.84-13.77 4.61  +0.10 -0.53 +0.13   B2Vpe+A2IaShell   v+0.002-0.006      +009SBO    58  1.0   0.0      *
7343   Bet2SgrCD-4513171 181623229654                       191559.6-445916192313.2-444759353.28-24.11 4.29  +0.34 +0.07         F2III              +0.097-0.054 +.030+019      126                 *
7344 45Rho2SgrBD-18 5325 181645162521                       191600.9-182938192150.9-181830 19.64-14.77 5.87  +1.06              gG9                 +0.102-0.092      -013SB
7345          BD+37 3417 181655 68144                       191607.8+370900191938.9+371950 69.45 10.93 6.31  +0.68 +0.21         G8V                -0.066-0.187 +.038+002V?
7346          BD+34 3503 181828 681643545  W                191654.0+345954192033.1+351110 67.57  9.82 6.31  -0.12 -0.51         B9V                +0.014+0.006      -018      125       45.0
7347          BD-08 4950 181858143321                       191655.0-082324192220.7-081204 29.07-10.57 6.31  -0.10 -0.40         B3IVp              -0.013-0.024      -010V?     67
7348   Alp SgrCD-4013245 181869229659 728                   191657.5-404814192353.2-403658357.71-23.09 3.97  -0.10 -0.33         B8V                +0.030-0.123      -001SB     89                 *
7349          BD-00 3725 1819071433243546                   191713.0-002630192221.5-001509 36.25-06.99 5.83  +1.09 +0.97        gG8                 +0.061-0.025      -011V?
7350          CD-4313352 181925229660                       191713.4-435451192421.4-434323354.48-24.03 6.17  +1.60 +1.92         M1-2III            +0.024-0.037      -017V?
7351          BD+54 2123 181960 31574                       191723.9+541123191936.5+542234 85.53 17.88 6.26  +0.03 +0.04         A1V                +0.018-0.025      -005      125
7352 60Tau DraBD+73  857 181984  9366 729I         11898    191728.6+731012191533.0+732120104.73 24.21 4.45  +1.25 +1.45 +0.58   K2+IIIbCN1Fe1      -0.140+0.107 +.015-030SB   < 17
7353          BD-07 4942 182038143340    I                  191740.4-073528192304.6-072401 29.88-10.38 6.32  +1.45 +1.69         K0                 +0.042-0.007      +012
7354          BD+09 4081 1821011245643548                   191802.6+094307192248.4+095447 45.33-02.36 6.35  +0.44 -0.03         F6V                +0.011+0.109      -020       12
7355          CD-2815767 182180188079                       191816.1-280333192430.1-275156 10.68-19.04 6.04  -0.13 -0.67         B2Vn               +0.004-0.001      +006V                         *
7356          BD+57 1986 182190 31587    I                  191826.1+572721192016.1+573843 88.77 18.99 5.91  +1.58 +2.01         M1IIIab            +0.023+0.011      -021
7357          BD+14 3896 182239104779                       191834.2+144344192308.2+145516 49.78-00.07 6.64  +0.27?              F1V                -0.027-0.009      +009V?     49
7358  3    VulBD+25 3811 182255 87136              11966    191845.1+260413192250.9+261545 59.78  5.32 5.18  -0.12 -0.52 -0.16   B6III              -0.001-0.010      -012V      40
7359          BD+33 3434 182272 68215                       191848.7+331937192233.4+333105 66.25  8.71 6.06  +1.03               K0III              +0.008-0.033      -016
7360          CD-2916104 182286188093       12400           191846.1-293009192504.1-291833  9.30-19.67 5.93  +1.26               K3III              +0.002-0.051      +025V?         4.4  12.1      *
7361          BD+64 1344 182308 18287      W                191859.9+641206191946.1+642327 95.51 21.33 6.52  -0.04 -0.47         B9IVpHgMn          -0.002+0.010      -020V?  =<  5  3.5 111.9
7362 47Chi1SgrCD-2415303 182369188101      W                191911.4-244210192516.5-243031 14.03-17.95 5.03  +0.23               Am                 +0.056-0.054 +.035-042       46  0.0   0.1      *
7363 49Chi3SgrCD-2415307 182416188105    I         11992    191926.5-240930192529.7-235744 14.58-17.79 5.43  +1.43 +1.67         K4III              -0.014-0.005      +040V?
7364          BD+19 4000 182422 87148                       191925.5+200412192346.9+201552 54.57  2.32 6.40  +0.01 -0.07         B9.5V              -0.007+0.008      -029       65                 *
7365          BD+57 1993 182440 31604                       191936.0+573427192125.4+574601 88.95 18.89 6.43R                     K2                 +0.021+0.042      +007
7366          BD-05 4964 182475143373                       191943.0-050450192501.6-045303 32.38-09.69 6.52  +0.33 +0.12         A9V                +0.054+0.015      -039V     105
7367          BD-14 5428 1824771625953552I                  191942.6-140542192521.6-135349 24.14-13.70 5.69  +1.43 +1.35         K3III              +0.057+0.066      -034V?
7368          BD+32 3411 182488 68239                       191947.6+330123192334.0+331320 66.07  8.38 6.37  +0.81 +0.46         G8V                +0.085+0.168 +.057-021V?
7369  2    SgeBD+16 3839 182490104797      W                191952.6+164434192422.1+165616 51.70  0.63 6.25  +0.08 +0.04 +0.02   A2III-IV           -0.003-0.011      +012SBO    43  0.8 340.6      *
7370          CP-54 9371 1825092461101504  W                191946.2-543129192748.1-541931343.15-26.90 5.69  +1.40 +1.68         K4III              -0.006+0.009      -005           2.7  72.9AB   4*
7371 58Pi  DraBD+65 1345 182564 182993547                   192009.3+653119192040.1+654253 96.90 21.66 4.59  +0.02 +0.06 -0.03   A2III s            +0.012+0.045 +.020-029       27                 *
7372  2    CygBD+29 3584 182568 871593550  W                192010.9+292532192407.6+293717 62.90  6.63 4.97  -0.10 -0.71 -0.14   B3IV               +0.013+0.013      -021V?    158
7373 31    AqlBD+11 3833 1825721048071503  W       11994    192012.1+114349192458.2+115640 47.36-01.86 5.16  +0.77 +0.42         G8IVHdel 1         +0.722+0.643 +.059-100V    < 17  3.5 105.6AB   3*
7374          BD+27 3379 182618 87165                       192021.6+275331192422.4+280516 61.56  5.87 6.53  -0.08 -0.55         B5V                -0.021+0.003      -020
7375 50    SgrBD-22 5105 1826291881213553I                  192021.3-215829192619.2-214636 16.77-17.13 5.59  +1.22 +1.20         K3III              +0.040-0.005      -020
7376          BD+36 3539 182635 68258                       192030.7+361515192406.1+362707 69.04  9.74 6.36R                     K1III              +0.010+0.077      -033                          *
7377 30Del AqlBD+02 3879 182640124603 730I W       12004    192027.4+025455192529.9+030653 39.62-06.13 3.36  +0.32 +0.04 +0.16   F3IV               +0.257+0.082 +.072-030SB     85  7.5 108.9AB   3*
7378          BD-15 5348 182645162609                       192029.7-151505192611.1-150311 23.15-14.37 5.72  +0.02 -0.34         B8Vn              t+0.023+0.010      -007V                         *
7379          BD-14 5435 182678162615                       192044.2-144458192624.6-143304 23.64-14.21 6.70  +0.05               A1V                +0.039-0.006      -023       65
7380          CD-2916140 182681188127 731                   192037.3-295627192656.5-294436  9.02-20.21 5.67   0.00 -0.10         B9V                +0.018-0.048      +001V?
7381          BD+49 2994 182691 316233549                   192046.4+500432192323.8+501617 81.81 15.76 6.51  -0.08 -0.27         B9III               0.000+0.012      -024
7382          BD+43 3229 182694 48401                       192046.8+431135192356.5+432317 75.39 12.82 5.84  +0.92 +0.63         G7III-IIIa         +0.017-0.025      -000V?                        *
7383          CP-68 3251 1827092545903557                   192047.0-683815193110.9-682602327.32-28.72 5.96  +1.64 +1.93         K4-5III            +0.013-0.019      -014
7384          BD+19 4009 182761 87186                       192100.9+200428192522.4+201617 54.75  2.00 6.31  -0.02 -0.08         A0V                -0.007-0.027      -032V     130                 *
7385  4    VulBD+19 4010 182762104818    I  12425           192105.2+193609192528.6+194755 54.35  1.75 5.16  +0.98 +0.81 +0.51   K0III              +0.095-0.063 +.026-001V    < 19: 6.2  18.9AB   3*
7386          BD+24 3737 182807 87190      W                192117.4+244356192525.8+245446 58.86  4.18 6.19  +0.51 -0.03         F7V                -0.175-0.630 +.027-004V   =<  6  4.4  40.1AB   5*
7387 32Nu  AqlBD+00 4206 182835124628    I W       12021    192124.2+000821192631.1+002019 37.26-07.64 4.66  +0.60 +0.42 +0.46   F2Ib                0.000+0.001 -.008-001       13  4.2 201.0      *
7388          CP-55 9096 182893246125                       192143.6-553837192952.6-552629341.97-27.37 6.13  +0.98 +0.81         K0-1III            +0.023-0.057      -027
7389          BD+12 3907 182900104832                       192145.2+124921192624.1+130126 48.49-01.66 5.74  +0.47 +0.04         F6III              +0.010+0.062 +.032-034V      30                 *
7390  5    VulBD+19 4015 182919104831                       192151.2+195357192613.2+200552 54.69  1.74 5.63  -0.01 -0.04         A0V                +0.003-0.035      -021SB    150                 *
7391          BD+19 4017 1829551048391505I  12445           192206.1+194133192628.7+195329 54.54  1.59 5.81  +1.55 +1.99         M0III              -0.001-0.046      -034V          3.8 152.5AD   5*
7392          CD-4313395 183007229712                       192217.3-433845192923.9-432645355.03-24.84 5.71  +0.22 -0.01         Am                 +0.096-0.136 +.013-031SB1O < 39                 *
7393   Mu  TelCP-55 9100 183028246131                       192227.9-551854193034.5-550636342.35-27.42 6.30  +0.45               F5V                +0.033-0.015      +009V?
7394   Lam UMiBD+88  112 183030  3020 914  W                192229.3+885916171656.8+890216121.94 27.51 6.38  +1.57 +1.79         M1III              -0.023-0.004      +002V?         7.5  54.7
7395  4    CygBD+36 3557 183056 683013554          V1741 Cyg192233.0+360702192609.1+361904 69.11  9.31 5.15  -0.12 -0.43         B9pSi              +0.005+0.014 -.007-022SBO    34                 *
7396          BD+13 4020 1831441048623555          12049    192257.9+140449192733.9+141657 49.73-01.31 6.32  -0.06 -0.51         B4III              +0.009-0.002      +004
7397          BD+02 3892 183227124661                       192319.5+024336192820.8+025549 39.79-06.85 5.85  +0.01 -0.39         B6III              +0.004-0.006      -014V?     48
7398          CD-2714004 183275188192    I  12506           192341.0-271125192952.2-265908 12.00-19.83 5.52  +1.12 +1.35 +0.36E  K3III              +0.034-0.041 +.045-031           3.5   7.6      *
7399          CD-3215233 183312211315                       192348.4-321749193014.7-320532  6.91-21.67 6.60  +0.39               F3IV-V             +0.083-0.065      +008
7400 35    AqlBD+01 4010 183324124675                       192357.6+014446192901.0+015701 38.99-07.45 5.80  +0.08 +0.07         A0IVp              -0.003-0.033      +012V      73                 *
7401          BD+57 1999 183339 316521507                   192358.2+574932192546.7+580138 89.43 18.45 6.60  -0.15 -0.56         B8IVHe wk          -0.004+0.008      -022                          *
7402          BD-07 4968 183344143454       12503  U Aql    192358.4-071458192921.5-070238 30.91-11.62 6.61  +1.10 +0.70 +0.66   F7-G1I-II         v+0.024+0.003 +.013-007V          5.5   1.5AB   3*
7403          BD+37 3465 183362 68346                       192405.5+374415192736.5+375628 70.71  9.79 6.34  -0.14 -0.74         B3Ve               +0.022-0.013      -028      244                 *
7404          BD-00 3760 183387124681    I                  192410.8+000226192918.0+001446 37.50-08.30 6.25  +1.32 +1.38         K2                 +0.007-0.008      -060
7405  6Alp VulBD+24 3759 183439 872611508I W       12069    192432.6+242744192842.3+243954 58.99  3.40 4.44  +1.50 +1.81 +0.97   M0III              -0.125-0.106 +.014-086V?         1.4 413.7      *
7406  8    VulBD+24 3761 183491 87267                       192446.6+243344192857.0+244607 59.11  3.40 5.81  +1.03 +0.87         K0III               0.000+0.014 +.004-027V?
7407          BD+14 3936 183492104896                       192446.5+142325192922.1+143545 50.21-01.55 5.56  +1.05 +0.91         K0III              +0.045-0.026      -040V?
7408  7Iot1CygBD+52 2434 183534 31673                       192459.0+520700192725.9+521914 84.02 16.01 5.75   0.00 -0.01         A1V                -0.014-0.024      +002SB     54
7409  7    VulBD+19 4039 183537 87269              Var      192459.0+200424192920.9+201647 55.20  1.18 6.33  -0.10 -0.52         B5Vn               +0.001-0.020      -037V                         *
7410          BD-21 5410 183545188219                       192457.9-213112193054.0-211844 17.66-17.93 6.13  +0.12               A2                 +0.012-0.017      -021
7411          CP-53 9585 183552246151                       192500.1-532346193253.8-531109344.56-27.45 5.75  +0.30 +0.20 +0.17   FmDel Del          +0.029-0.013      +014V
7412          BD+02 3904 183589124698    I  12520  12088    192509.1+024147193010.5+025415 39.98-07.26 6.09  +1.82 +2.03         K5Ib               -0.005 0.000      -013V?         4.9  34.1AB   4*
7413          BD+62 1716 183611 183433556                   192517.1+622108192626.5+623326 93.93 20.02 6.38  +1.39 +1.71         K5III              +0.034+0.043      -040
7414 36    AqlBD-03 4612 1836301434821509I         12093    192526.0-025950193039.8-024720 34.93-10.00 5.03  +1.75 +2.05         M1III             v+0.023-0.010 +.029-011V?
7415          BD+03 4043 183656124704              V923 Aql 192532.8+031408193033.1+032640 40.50-07.09 6.05  +0.01 -0.37         A0eShell           +0.010+0.005      -026V     180                 *
7416          CD-4513296 1838062297413558          PW Tel   192609.2-452901193321.6-451619353.26-25.97 5.61  -0.04               ApCrEuSr           -0.012-0.031      -008                          *
7417  6Bet1CygBD+27 3410 183912 87301 732I  12540A 12105    192641.3+274458193043.3+275735 62.11  4.57 3.08  +1.13 +0.62 +0.66   K3II+B9.5V         +0.002-0.002 +.017-024V   =< 25  2.0  34.7AB   3*
7418  6Bet2CygBD+27 3411 183914 87302       12540B          192643.4+274518193045.3+275755 62.12  4.57 5.11  -0.10 -0.32         B8Ve               -0.009 0.000 +.017-018V     250  2.0  34.7AB   3*
7419          BD+35 3658 183986 68417       12545           192709.7+360105193046.9+361343 69.46  8.44 6.25  -0.04 -0.11         B9.5III            +0.006-0.011 +.005+011V      65  7.7  27.7AB   3
7420 10Iot2CygBD+51 2605 184006 31702 733                   192711.0+513100192942.3+514347 83.61 15.45 3.79  +0.14 +0.11 +0.07   A5Vn               +0.020+0.130 +.005-020V?    226
7421          BD+26 3573 184010 87314                       192715.8+262418193121.8+263702 60.99  3.81 5.87  +0.93 +0.63         K0III-IV           +0.032+0.027      -001V?
7422          CD-4013356 1840352297463560          V4089 Sgr192717.0-401458193408.5-400205358.92-24.81 5.89  +0.09               A3III              +0.005-0.005      +012SB1O   53                 *
7423          BD+79  628 184102  9414 734                   192744.8+792409192140.2+793610111.52 25.30 6.05  +0.07 +0.06         A3V                +0.013-0.034      -003SB
7424   Iot TelCD-4813161 184127229751 735                   192747.9-481853193513.0-480557350.25-26.89 4.90  +1.09               K0III              -0.009-0.038 +.002+022
7425          BD+83  552 184146  32093962  W                192757.0+831606191507.8+832746115.68 26.27 6.53  +0.10 +0.10         A3V                +0.008+0.012      -014      105  3.7  25.0
7426  8    CygBD+34 3590 184171 684471510                   192803.3+341425193146.3+342711 67.97  7.44 4.74  -0.14 -0.65 -0.16   B3IV               +0.001-0.003      -022V?     18
7427          BD+49 3034 184293 31737    I                  192840.4+500532193119.3+501824 82.38 14.62 5.53  +1.25       +0.65  gK1                 -0.029+0.048      -009V?
7428          BD+55 2215 184398 31741              V1817 Cyg192906.6+553108193113.6+554355 87.51 16.87 6.37  +1.16 +0.91         K2II-IIIe+A0V     v-0.002-0.011      -006SBO =< 50                 *
7429 38Mu  AqlBD+07 4132 1844061247991511I  12607  12155    192912.2+071000193405.4+072244 44.41-06.01 4.45  +1.17 +1.26 +0.61   K3-IIIbCN0.5       +0.216-0.157 +.045-024V?   < 19: 5.0 177.5AE   6*
7430 37    AqlBD-10 5122 1844921627923562I                  192936.6-104644193507.3-103337 28.30-14.44 5.12  +1.13 +0.89         G9IIIa             +0.009+0.003 +.007-031
7431 51    SgrCD-2415442 184552188326              12183    192957.4-245618193601.7-244309 14.79-20.30 5.65  +0.19 +0.17 +0.07   A7m                +0.012-0.023 +.018-031SBO    16  2.0   0.0      *
7432          BD-07 4998 184573143564                       193005.9-074043193529.8-072737 31.22-13.17 6.34  +1.12 +0.92         K0                 +0.030-0.039      -010
7433          BD-12 5461 184574162797                       192959.2-122818193533.6-121510 26.77-15.26 6.27  +1.09               K0                 +0.015-0.006      +006V?
7434          CP-58 7627 1845852461883566                   193001.4-581215193825.9-575900339.28-28.86 6.18  +0.97 +0.81         K0III              +0.026-0.065 +.015-011
7435          CP-66 3445 1845862546223567  W                192958.8-665430193952.1-664108329.34-29.51 6.39  +0.02 +0.04         A1V                +0.032-0.043      -014           6.1  19.7
7436          BD+38 3650 184603 68499      S                193007.4+383239193336.4+384544 72.00  9.12 6.61  +0.03 +0.10         A3Vn               +0.005+0.028      -017                 0.1
7437  9    VulBD+19 4063 184606104990       12622  12173    193011.3+193318193434.9+194624 55.35-00.14 5.00  -0.09 -0.43 -0.09   B8IIIn             +0.008+0.002      +005V     235  7.8 108.3AC   3
7438          BD+02 3932 184663124823                       193023.7+024133193525.2+025448 40.60-08.42 6.38  +0.41 -0.02         F6IV               +0.019+0.045      +004       40
7439          BD-19 5521 184705162809                       193036.3-190425193626.1-185110 20.57-18.16 6.11  +0.27?              F0III              +0.031-0.003      -042
7440 52    SgrCD-2514184 184707188337 736   12654  12191    193037.3-250616193642.4-245301 14.68-20.51 4.60  -0.07 -0.15 -0.06   B9                 +0.069-0.021D+.002-019V      68  4.5   2.7      *
7441  9    CygBD+29 3651 184759 87385      W                193052.5+291434193450.9+292747 63.87  4.50 5.38  +0.55 +0.33         F8III+A0V          +0.015+0.024 +.021-011SB1O=< 50  0.5   0.1      *
7442          BD+48 2914 184786 48589    I         V1743 Cyg193056.3+490240193341.6+491545 81.58 13.83 5.96  +1.64 +1.77         M4.5IIIaS          +0.005-0.003      -010V?                        *
7443          BD-18 5432 1848351628163565                   193115.2-182711193703.3-181352 21.23-18.05 5.64  +1.26 +1.08         K3III              +0.013-0.009 +.010-007V
7444          BD+42 3386 184875 48601                       193125.5+421136193441.2+422446 75.38 10.62 5.35  +0.05 +0.09         A2V                -0.004-0.022 +.001+002       87
7445          BD+10 3981 184884105032       12660           193124.5+105544193608.0+110900 47.98-04.65 6.68                      A3V                -0.004-0.010      -005           6.4   3.8
7446 39Kap AqlBD-07 5006 184915143600 737I'        12204    193130.7-071500193653.5-070139 31.77-13.29 4.95   0.00 -0.87 -0.04   B0.5III            +0.003-0.004      -019V     259                 *
7447 41Iot AqlBD-01 3782 184930143597       12663  12201    193132.9-013030193643.3-011711 36.99-10.66 4.36  -0.08 -0.44 -0.08   B5III              +0.004-0.018 +.004-021V      95  8.9  47.0      *
7448          BD+59 2060 184936 183953563I W                193135.9+595624193310.1+600931 91.89 18.38 6.29  +1.57 +1.98         K4III              +0.005+0.006      -019           2.0  76.2      *
7449          BD+14 3974 184944105036                       193139.4+141013193615.8+142330 50.84-03.11 6.38  +1.04 +0.89         K0II-III           +0.034-0.023      -042
7450          BD+70 1073 184958  94473561I                  193148.3+704619193100.2+705922102.62 22.41 6.07  +1.41               K2                 -0.016+0.037      -043
7451          BD+50 2815 184960 31782                       193144.4+510119193419.8+511412 83.46 14.59 5.73  +0.48  0.00 +0.28   F7V                +0.032-0.190 +.037+000V?      6                 *
7452          BD+22 3741 184961 874263564          12199    193151.6+222150193608.3+223509 57.99  0.91 6.32  -0.07 -0.28         B9pSi:Cr:          +0.001-0.010      -023       50                 *
7453          BD+47 2870 184977 48606                       193149.2+475647193439.9+480953 80.64 13.21 6.67R +0.24?              A9V                -0.015-0.073      -001
7454          BD-14 5479 184985162827                       193156.3-143117193734.4-141806 25.05-16.56 5.47  +0.50 +0.04         F5V                -0.105-0.142 +.047-021SB                        *
7455          CP-66 3447 184996254627                       193156.2-660450194137.5-655115330.29-29.68 6.09  +1.55 +1.96         M0III              +0.027-0.065      -042
7456          BD+10 3984 185018105045       12670           193209.1+110300193652.5+111624 48.18-04.75 5.98  +0.88               G0Ib               +0.018+0.005      -001           5.7  80. AD   5
7457 11    CygBD+36 3619 185037 68552                       193212.6+364321193548.3+365640 70.58  7.89 6.05  -0.11 -0.37         B8Vne              +0.008-0.002      -015V     400:                *
7458          BD+20 4200 185059 87447              U Vul    193215.4+200636193637.7+201958 56.07-00.29 7.14  +1.32 +1.00         F2-F8Iab           -0.003-0.011 -.013-011V                         *
7459          CP-54 9438 185075246204                       193219.1-543841194018.3-542503343.36-28.71 6.26  +1.00               K0III              +0.038-0.009      +015
7460 42    AqlBD-04 4861 185124143621                       193229.0-045215193747.3-043851 34.06-12.42 5.46  +0.43 +0.03         F3IV               +0.107-0.051 +.027-038V?
7461          CD-4513354 185139229800              QQ Tel   193230.8-453019193942.0-451641353.53-27.06 6.25  +0.28               F2IV               -0.001+0.020      +008                          *
7462 61Sig DraBD+69 1053 185144 18396    I W       12176    193233.2+692928193221.6+693940101.32 21.89 4.68  +0.79 +0.38 +0.41   K0V                +0.600-1.738 +.177+027V    < 17      315.6      *
7463  4Eps SgeBD+16 3918 185194105061       12693  12213    193245.7+161417193717.4+162746 52.77-02.32 5.66  +1.02 +0.81         G8III             v+0.020+0.015      -033V?         2.7  88.9AB   4*
7464          CD-3913371 185257211451              V4090 Sgr193307.0-393933193955.7-392600359.88-25.72 6.61  +0.23               A2m                +0.062-0.061      -036V                         *
7465          BD+49 3059 185264 31804                       193315.1+500051193555.9+501419 82.65 13.93 6.52  +1.06 +0.84         G9III              -0.005+0.038      +008V?
7466          BD+29 3670 185268 87462       12696           193310.6+290633193709.4+292001 64.00  3.99 6.43  -0.09 -0.50         B5V                -0.016-0.001      -020      300  2.7  20.6AB   8*
7467          BD+38 3677 185330 68585                       193325.7+380933193656.6+382302 71.97  8.37 6.50  -0.15 -0.61         B5II-III            0.000 0.000      -027                          *
7468          BD+44 3185 185351 48649                       193332.0+442824193637.9+444142 77.62 11.35 5.17  +0.93 +0.69         G9IIIbFe-0.5       -0.101-0.102 +.019-005     < 19:
7469 13The CygBD+49 3062 185395 31815 738I: 12695           193345.5+495922193626.5+501316 82.66 13.85 4.48  +0.38 -0.03 +0.21   F4V                -0.017+0.257 +.056-028        7  6.9  48.4AC   3*
7470 53    SgrCD-2315618 185404188407       12741           193348.9-233918193949.4-232540 16.41-20.64 6.34  +0.03               B9.5V+A3IV         +0.001-0.032      -011           0.2   0.2      *
7471          BD+03 4097 1854231248923569                   193348.5+030916193849.0+032254 41.43-08.94 6.35  +0.04 -0.55         B3III              +0.004 0.000      -001SB
7472          BD+20 4210 185436 87489                       193356.5+203327193817.5+204658 56.66-00.41 6.48  +1.08 +0.98         K0III              -0.055-0.048      +005
7473          CD-2315625 185467188419                       193406.5-233928194007.1-232543 16.43-20.70 5.97  +1.04              gK1                 +0.026+0.002      -028V?
7474 44Sig AqlBD+05 4225 185507124903       12737  Sig Aql  193415.5+051011193911.6+052352 43.27-08.07 5.17  +0.03 -0.60         B3V+B3V           v+0.001-0.001      -005SBO   121  6.6  47.8AB   3*
7475          BD+16 3936 185622105104    I  12750           193453.9+162032193925.4+163417 53.12-02.71 6.38  +2.07 +2.22         K4Ib               +0.011+0.003      -004SB         2.7  31.8      *
7476 54    SgrBD-16 5399 1856441628831512I  12767A 12275    193459.7-163121194043.4-161736 23.47-18.07 6.2   +1.13 +1.06         K2III+F8V          +0.071-0.048 +.037-058V?         0.0   0.1     4*
7477          BD+48 2922 185657 48673      W                193510.1+490310193756.7+491704 81.91 13.22 6.47  +0.99 +0.72         G6V                +0.036+0.144 +.006-085V          2.9  29.6AB   4*
7478 12Phi CygBD+29 3684 185734 686373570I S                193525.6+295522193922.6+300912 64.95  3.97 4.69  +0.97 +0.80 +0.47   G8III-IV           +0.004+0.037 +.007+006SB2O             0.0      *
7479  5Alp SgeBD+17 4042 185758105120    I  12766           193537.5+174702194005.8+180050 54.45-02.14 4.37  +0.78 +0.43 +0.37   G1II               +0.014-0.021 +.000+002V?      0  6.8  90. AD   4*
7480 45    AqlBD-00 3813 1857621436783573   12775           193534.2-005111194043.3-003716 38.06-11.24 5.67  +0.11 +0.09         A3IV               +0.017+0.017 +.018-046       96  7.2  42.0     3*
7481          BD+33 3547 185837 68654       12765           193559.1+334453193945.0+335845 68.34  5.76 6.10  +0.08 +0.13         A3IV               +0.007+0.017      -033       80  4.5   1.3AB   3*
7482          BD+20 4218 185859 87542                       193606.0+201444194028.3+202836 56.64-01.00 6.50  +0.39 -0.62         B0.5Iae            -0.002-0.022      +005                          
7483 14    CygBD+42 3413 185872 486913572                   193611.1+423513193926.5+424906 76.16 10.03 5.40  -0.08 -0.22         B9III              +0.022+0.031 +.019-028SB     45
7484          BD+54 2193 185912 318503571          V1143 Cyg193626.2+544422193841.2+545826 87.25 15.59 5.82  +0.44 -0.10         F6Va               +0.035+0.164 +.036-015SBO                       *
7485          BD+23 3733 185915 87551       12778           193625.2+232910194039.7+234303 59.48  0.56 6.64  +0.01 -0.39         B6IV               +0.019-0.020      -020SB         1.5 118.4AC   3*
7486          BD+13 4098 1859361051323574  W       QS Aql   193627.7+133500194105.5+134856 50.91-04.41 6.01  -0.08 -0.52         B5V                -0.002-0.010      -014SBO    58  0.2   0.1      *
7487          BD+45 2940 185955 48697                       193632.2+454332193934.4+455729 79.00 11.47 6.20  +0.90 +0.55         G8-K0II-III        -0.005+0.046      -010V?
7488  6Bet SgeBD+17 4048 1859581051331513I                  193633.4+171439194102.9+172834 54.10-02.60 4.37  +1.05 +0.89 +0.50   G8IIIaCN0.5        +0.010-0.032 +.011-022     < 19:                *
7489 55    SgrBD-16 5413 1860051629151514  W                193647.9-162130194231.1-160726 23.82-18.40 5.06  +0.33 +0.09         F1III              +0.068-0.009 +.040-027SB    134  0.0   0.2      *
7490          BD+22 3767 186021 87569                       193657.1+221310194114.9+222710 58.45-00.18 6.36  +1.53               K0I                +0.025-0.004      -023
7491          CD-3713322 1860422115063576                   193656.7-374627194337.6-373220  2.12-25.92 6.16  +0.01               B8                 +0.006-0.019      -029V?
7492          BD+42 3419 186121 48714    I                  193726.7+425040194041.2+430440 76.50  9.95 6.16  +1.56 +1.42 +1.01E  M2III              +0.012-0.006      -005V?
7493 46    AqlBD+11 3954 186122105156                       193731.3+115730194212.8+121136 49.62-05.44 6.34  -0.03 -0.41         B9IIIpHgMn         -0.002-0.004      -032        5
7494          CP-81  868 1861542588441667                   193736.9-813601195601.5-812059312.46-29.21 6.39  +1.40 +1.68         K3-4III            +0.009-0.001      -006
7495          BD+45 2949 186155 487183575  W       12300    193745.0+451716194050.2+453130 78.71 11.07 5.06  +0.40 +0.15         F5II-III           +0.086+0.127 +.021-020V      42  6.7  88.0
7496          BD-15 5444 186185162931      W                193751.3-154207194333.5-152812 24.55-18.36 5.49  +0.46 +0.02         F5IV               +0.154-0.180 +.045+013V      21  6.2  70.4      *
7497 47Chi AqlBD+11 3955 186203105168       12808  12325    193751.6+113528194234.0+114936 49.34-05.70 5.27  +0.57 -0.04         G0:III+A3V         -0.001-0.007 +.006-022SB      0  1.2   0.4AB   5*
7498          CP-72 2445 1862192577363580                   193753.5-724450194925.3-723012322.60-30.12 5.41  +0.22 +0.11 +0.13C  A4III               0.000+0.017 +.033-001SB    104                 *
7499          BD+39 3878 186307 48737      W                193832.3+400103194157.5+401514 74.10  8.40 6.23  +0.18 +0.05         A6V                -0.025+0.026 +.006-032      110  1.3   0.2
7500          BD+60 1991 186340 18461       12789           193839.1+601624194013.0+603026 92.59 17.72 6.51  +0.22?              A5V                -0.008-0.005D+.026-001      135  2.2  18.3
7501          BD+28 3447 186357 87612              V1276 Cyg193848.8+290536194249.1+291954 64.60  2.92 6.49  +0.33 +0.28         F1III              +0.061+0.047      -025       84                 *
7502          BD+32 3531 186377 687303577                   193853.6+321123194244.6+322536 67.29  4.46 5.94  +0.11 +0.14         A5III              +0.004+0.003      -007       27
7503 16    CygBD+50 2847 186408 31898       12815A          193909.4+501735194148.9+503131 83.34 13.21 5.96  +0.64 +0.19 +0.33   G1.5Vb             -0.149-0.153 +.039-026V       2  0.2  39.2AB   3*
7504          BD+50 2848 186427 31899       12815B          193912.2+501708194152.0+503103 83.34 13.20 6.20  +0.66 +0.20 +0.34   G2.5V              -0.134-0.162 +.039-027V       3  0.2  39.2AB   3*
7505          BD+30 3706 186440 68744                       193913.5+302623194309.5+304043 65.81  3.52 6.05   0.00 -0.01         A1V                -0.012+0.038      -028      155
7506 10    VulBD+25 3933 186486 876331515                   193933.4+253157194342.9+254619 61.61  0.98 5.49  +0.93 +0.67 +0.47   G8III              +0.012+0.019 +.023-009     < 17
7507          CD-3215443 1865002115411516                   193938.4-320859194601.2-315431  8.25-24.79 5.52  +0.02               B8V                +0.005-0.017      -031
7508          BD+26 3654 186518 87640    I  12850  PS Vul   193950.1+265346194355.9+270808 62.82  1.61 6.28  +1.10               B7V+G1:III         -0.009+0.004 -.008-012V          1.5   0.4      *
7509          BD+55 2245 186532 31906    I         V1351 Cyg193951.4+551337194204.1+552748 87.93 15.37 6.48  +1.61               M5IIIa             +0.013-0.039      -028V?                        *
7510   Nu  TelCP-56 9290 186543246271 739                   193951.3-563610194801.2-562145341.27-30.01 5.35  +0.20               A7III-IV           +0.088-0.137      -012V
7511 48Psi AqlBD+12 4059 186547105199                       193955.0+130346194434.1+131810 50.88-05.40 6.26  -0.03 -0.21         B9III-IV           -0.007-0.006      -018SB     35                 *
7512          BD+33 3572 186568 68764       12852           194005.5+335522194351.4+340945 68.92  5.11 6.05  -0.06 -0.30         B8III              -0.002+0.003      -011       15  7.3  33.9AC   3
7513          CP-67 3680 186584254658                       194004.5-670335194953.3-664846329.18-30.50 6.45  +1.48 +1.72         K4III              -0.012+0.016      -002
7514          BD+41 3469 186619 48771    I                  194024.8+413158194345.0+414623 75.61  8.83 5.84  +1.57 +1.99         M0IIIab            +0.016+0.010 +.014-041
7515 56    SgrBD-20 5698 1866481629641517I                  194031.7-200006194621.7-194540 20.65-20.67 4.86  +0.93 +0.96         K1III              -0.129-0.089 +.025+020V
7516          BD-03 4701 1866601437763579                   194038.5-030733194552.2-025300 36.61-13.42 6.48  +0.05 -0.54         B3III              +0.005-0.002      -017
7517 15    CygBD+37 3586 186675 68778 740I                  194040.1+370645194416.6+372116 71.76  6.60 4.89  +0.95 +0.69         G7+III             +0.075+0.035 +.021-024     < 19:
7518          BD+28 3460 186688 87659              SU   Cyg 194048.4+290123194448.7+291553 64.76  2.51 6.82  +0.64 +0.20 +0.39   F2I               v+0.001-0.002 -.010-036V                         *
7519 49Ups AqlBD+07 4210 186689125032                       194048.0+072215194539.9+073648 46.01-08.42 5.91  +0.18 +0.09         A3IV               +0.053+0.002 +.027-030       30
7520          BD+34 3691 186702 68783    I         12372    194052.8+341022194438.2+342450 69.22  5.10 6.57R                     M1III              +0.004-0.009      +009
7521          CP-53 9678 1867562462773581                   194108.4-530759194855.1-525317345.27-29.79 6.25  +1.13 +1.11         K1III              +0.009-0.050      -021
7522          BD+57 2057 186760 31923                       194117.6+574640194314.6+580059 90.39 16.33 6.22  +0.56 +0.06         G0V                +0.137-0.060 +.045-022    =< 10
7523          BD+40 3866 186776 48789    I         V973 Cyg 194125.3+402831194449.0+404300 74.77  8.15 6.34  +1.64 +1.94 +0.92E  M4-III             -0.064-0.023      -097                          *
7524          CP-65 3827 186786254669              NZ Pav   194127.1-655059195101.3-653618330.58-30.65 6.05  +0.30 +0.08         F2III-IV           +0.101-0.155      -041                          *
7525 50Gam AqlBD+10 4043 186791105223 741I W                194130.3+102210194615.6+103648 48.73-07.08 2.72  +1.52 +1.68 +0.75   K3II              e+0.018-0.002 +.016-002V    < 17  8.0 132.6      *
7526          BD+56 2291 186815 31933                       194137.0+564803194339.6+570233 89.50 15.86 6.27  +0.88               K2III              +0.010+0.020      -026
7527          CP-61 6413 186837254666                       194136.8-611835195021.7-610341335.85-30.57 6.21  -0.14 -0.55         B5V                -0.006-0.001      -015                          *
7528 18Del CygBD+44 3234 186882 48796    I  12880  12380/1  194150.9+445312194458.5+450751 78.71 10.24 2.87  -0.03 -0.10 -0.02   B9.5IV+F1V         +0.053+0.047 +.030-020SB    149  3.5   2.0AB   3*
7529          BD+35 3786 186901 68805       12893A          194159.4+355049194539.6+360528 70.79  5.74 6.43  -0.03 -0.13         B9.5VpSi           +0.004+0.014D+.018-019       35  0.5  14.8AB   6*
7530          BD+34 3701 186927 68810       12900           194207.6+344608194551.4+350046 69.87  5.17 6.09  +1.06 +1.00         K0II-III+A2V       +0.021-0.004      -019           2.4  44.6AC   3*
7531          CP-59 7534 186957246293      W                194215.7-592635195044.8-591135338.02-30.55 5.42  +0.08               A0IV               +0.015+0.004 +.013+004       50  1.5   0.5      *
7532          BD-14 5555 186984162998                       194225.9-135659194803.0-134212 26.72-18.63 6.11  +0.20 +0.13 +0.08   A6m                +0.026-0.012      -008       75
7533          BD+24 3877 186998 87706                       194227.5+245319194639.5+250802 61.39  0.08 6.62  +0.27               A7IV               +0.073-0.004      +013SB
7534 17    CygBD+33 3587 187013 68827       12913A 12398    194237.8+332941194625.6+334340 68.81  4.44 4.99  +0.47  0.00         F7V                +0.023-0.446 +.049+005V?      9  3.5  26.0AB   3*
7535          BD+32 3558 187038 68835       12920           194245.1+323835194635.0+325319 68.10  3.99 6.18  +1.13               K3III              -0.032-0.003D+.016-046           2.1  30.7AD   4*
7536  7Del SgeBD+18 4240 187076105259 743I W       Del Sge  194255.7+181715194723.3+183203 55.77-03.38 3.82  +1.41 +0.96 +1.29   M2II+A0V           +0.007+0.008 +.001+003SBO =< 50                 *
7537          CD-4713103 187086229887              12432    194255.3-474825195013.8-473326351.37-29.27 5.94  +1.70 +1.97         M1III              -0.017-0.008      -065V                         *
7538          CD-2916546 1870981886033582   12956           194257.2-290205194911.6-284720 11.75-24.46 6.05  +0.40               F2V                +0.126-0.096      -034                          *
7539 NOVA 1670                                     CK Vul                                                                                                                                           *
7540          BD+25 3972 187193 87729                       194337.1+250813194748.6+252302 61.74-00.02 5.95  +0.99               K0II-III           +0.082-0.026      -018
7541          BD-11 5131 187195163012                       194331.3-110710194902.2-105215 29.53-17.66 6.04  +1.24 +1.36         K5III              +0.036-0.020      -033V
7542          BD+10 4058 187203105278                       194345.4+102644194830.4+104139 49.07-07.52 6.44  +0.96 +0.69         F8Ib-II            -0.007-0.001      -005       28
7543          BD+38 3758 187235 68859       12944           194354.8+380936194727.8+382427 72.99  6.57 5.77  -0.06 -0.37         B8Vn               +0.015-0.003      -028      350  7.2  24.7
7544 52Pi  AqlBD+11 3994 187259105282       12962  12425    194359.3+113401194842.1+114857 50.08-07.01 5.72                      G2III:+A1V:        +0.018-0.004D+.003+013V?         0.5   1.4AB   3*
7545          BD+68 1079 187340 18508                       194427.0+690534194418.5+692013101.37 20.78 5.92  +0.05 +0.11         A2III              +0.016-0.023      +000                          *
7546  8Zet SgeBD+18 4254 187362105298       12973           194432.3+185328194858.7+190832 56.48-03.40 5.00  +0.10 +0.05 +0.06   A3V                +0.021+0.030 +.010-014V     225  0.5   0.2AB   4*
7547          BD+47 2916 187372 48842    I                  194431.4+473936194726.8+475428 81.40 11.19 6.12  +1.64 +1.99         M2III              -0.026-0.031      +003SB
7548          CP-55 9221 187420246311      W     A          194439.6-551331195237.6-545816342.94-30.54 5.74  +0.92 +0.65         G8-K0III           +0.008-0.002      -018           0.7  22.9      *
7549          CP-55 9222 187421246312      W     B          194441.0-551351195239.0-545836342.93-30.55 6.50  +0.10 +0.12         A2V                +0.008-0.003      -022           0.7  22.9      *
7550          BD+34 3727 187458 68893       12972           194459.9+350334194843.9+351841 70.42  4.81 6.53  +0.44 -0.09         F4V                +0.085+0.062 +.017-027V          0.6   0.6      *
7551          BD+33 3602 187459 68895              V1765 Cyg194501.6+331113194850.6+332614 68.81  3.85 6.44  +0.20 -0.74         B0.5Ib             +0.007-0.002      -010SB2O  126:                *
7552          CD-4013514 1874742299033583          V3961 Sgr194503.2-400740195150.6-395228  0.03-28.05 5.33  -0.06 -0.22         A0pCrSr:          v+0.023-0.015 +.020-002SB      6                 *
7553 51    AqlBD-11 5149 187532163036 744   13017           194516.7-110102195046.8-104549 29.83-18.00 5.39  +0.38               F0V                -0.030+0.033      +006V?     67  8.0  21.1      *
7554          BD+07 4252 187567125116      S       V1339 Aql194526.3+073900195017.5+075409 46.83-09.28 6.51  -0.10 -0.70         B2.5IVe            +0.005+0.001      -030                 0.1      *
7555          BD+38 3772 187638 68909       12992           194555.1+382730194927.5+384236 73.45  6.39 6.11  +0.90               G5-8III            +0.006-0.010      +011V?         4.9  11.6AB   3
7556          BD+28 3493 187640 87786              12454    194551.5+281114194954.6+282626 64.61  1.13 6.38  -0.06 -0.50         B5V                -0.021+0.027      -012SB     75
7557 53Alp AqlBD+08 4236 187642125122 745I  13009           194554.2+083615195047.0+085206 47.74-08.91 0.77  +0.22 +0.08 +0.14   A7V                +0.538+0.386 +.198-026      242  8.6 165.2AB   3*
7558          CP-61 6426 1876532546831518                   194556.7-612544195440.4-611015335.74-31.09 6.24  +0.16               A4V                -0.008+0.011      +011
7559          BD-02 5133 187660143853    I                  194558.4-024250195111.1-022739 37.62-14.40 6.13  +1.57 +1.82         K5III              -0.007-0.035      +009
7560 54Omi AqlBD+10 4073 187691105338       13012           194614.2+100955195101.6+102456 49.14-08.19 5.11  +0.55 +0.07 +0.29   F8V                +0.242-0.139 +.048-000V   =<  6  8.3  14.4AB   3*
7561 57    SgrBD-19 5631 187739163060                       194623.3-191756195212.0-190242 21.92-21.67 5.92  +0.98 +0.64?        G5III              +0.002-0.058 +.021-026SB
7562          BD+09 4288 187753125139                       194630.1+092233195117.7+093749 48.48-08.64 6.25  +0.10 +0.09 +0.06   A1m                -0.001-0.012      +021       10
7563          BD+68 1082 187764 18530              CN Dra   194637.5+681117194644.7+682618100.58 20.24 6.34  +0.28 +0.09         F0III              +0.005+0.011      -012       98                 *
7564   Chi CygBD+32 3593 187796 68943    I W       Chi Cyg  194643.2+323940195033.9+325451 68.54  3.28 4.23  +1.82 +0.96 +2.68   S6+/1e            v-0.027-0.040 +.012-002V?         5.2 158.0      *
7565 12    VulBD+22 3833 187811 878133585          Var      194645.7+222120195104.1+223636 59.72-02.07 4.95  -0.14 -0.68 -0.14   B2.5Ve            v+0.028-0.017      -031SB?   281                 *
7566 19    CygBD+38 3780 187849 68947    I  13014  V1509 Cyg194701.3+382754195034.0+384321 73.57  6.20 5.12  +1.69 +2.10 +0.93E  M2IIIa             +0.011+0.107 +.000-039V?         6.4  55.5AB   3*
7567          BD+40 3902 187879 488923584          V380 Cyg 194711.3+402043195037.3+403559 75.22  7.13 5.69  -0.04 -0.77         B1III+B3V          +0.001-0.004      -004SBO    79                 *
7568          BD+37 3636 187880 68953    I                  194711.2+373417195047.0+374935 72.81  5.72 6.06  +1.70 +1.85         M4IIb              +0.024+0.007 +.005-016V?
7569          BD+11 4019 187923105348      W       12490    194722.5+112252195203.5+113744 50.33-07.82 6.13  +0.65 +0.13         G0V                -0.335-0.320 +.022-017V   =< 10  5.5  90.5
7570 55Eta AqlBD+00 4337 187929125159 746I         Eta Aql  194722.7+004456195228.4+010020 40.93-13.07 3.90  +0.89 +0.51 +0.47   F6Ib              v+0.011-0.007 +.010-015SB      0                 *
7571          BD-14 5578 187949163080      S       V505 Sgr 194727.8-145134195306.4-143611 26.38-20.12 6.48  +0.14 +0.09         A0V+F8IV           -0.004-0.042      -002SBO    97        0.3      *
7572          BD+09 4295 187961105355       13041  12496    194729.5+100542195215.6+102105 49.23-08.49 6.54  -0.01 -0.38         B7V                +0.004-0.015      -013      300  4.0  13.5AB   4*
7573          BD+24 3914 187982 878403587          12498    194749.1+244407195201.6+245932 61.88-01.04 5.57  +0.71 +0.13         A1Ia               +0.003-0.005 +.006-003SBO                       *
7574  9    SgeBD+18 4276 188001105360              12503    194754.1+182453195221.8+184019 56.48-04.33 6.23  +0.01 -0.92         O7.5Iaf            -0.002-0.003      +009SB    104                 *
7575          BD-03 4742 1880411438831519          V1291 Aql194804.5-032225195318.7-030652 37.27-15.18 5.65  +0.20 +0.10         A5p               v+0.024+0.015 +.003-020V       2                 *
7576 20    CygBD+52 2547 188056 320423586I         12481    194807.1+524403195037.7+525917 86.23 13.14 5.03  +1.28 +1.53 +0.50E  K3IIICN1           -0.008-0.068 +.014-020V    < 19:                *
7577          BD+47 2937 188074 48907              Var?     194820.1+470712195119.4+472238 81.25 10.36 6.20  +0.36 +0.04         F2V                +0.018+0.019      -018SB                        *
7578          CD-2415668 188088188692       13072  Var?     194818.4-241123195417.7-235628 17.18-23.91 6.18  +1.02 +0.94 +0.35E  K2V                -0.131-0.414 +.075-007SB2 =<  8  4.0  31.5      *
7579          CP-69 3072 1880972546933589                   194821.5-692533195841.3-690950326.39-31.18 5.75  +0.23 +0.16 +0.08   Am                 +0.063-0.099 +.032-012       89                 *
7580          BD+04 4264 188107125182                       194823.9+040829195322.6+042402 44.10-11.65 6.53  +0.02 -0.11         B9.5Vn             +0.016+0.004      +002      260
7581   Iot SgrCD-4214549 1881142299271520                   194821.8-420751195515.7-415206357.97-29.11 4.13  +1.08 +0.90 +0.52C  K0II-III           +0.017+0.056 +.035+036
7582 63Eps DraBD+69 1070 188119  9540    I  13007  12465    194830.7+700048194810.4+701604102.43 20.83 3.83  +0.89 +0.52 +0.48   G7IIIbCN-2         +0.082+0.037 +.016+003SB   < 19: 3.2   3.2      *
7583          BD+36 3744 188149 69004    I                  194835.9+361027195216.3+362556 71.76  4.76 6.10  +1.43 +1.62         K4III              +0.002 0.000      -021
7584 56    AqlBD-08 5150 188154143894    I W                194842.7-085002195408.2-083427 32.28-17.80 5.79  +1.64 +2.00        gK5                 +0.005-0.018      -050V?         6.1  46.5
7585          CD-3314560 188158211653                       194840.6-331827195505.1-330247  7.66-26.95 6.46  +1.48               K2/3III            -0.012-0.002      +034V
7586          CP-58 7683 188161246348                       194841.5-581116195657.7-575532339.54-31.31 6.53  +1.53 +1.79         K4II               -0.009-0.036      +026
7587          CP-59 7550 188162246349                       194842.4-590952195706.2-585405338.40-31.36 5.26  -0.01               B9.5IV             +0.015-0.021 +.022-002       51
7588          CP-69 3073 188164254696                       194840.4-690133195853.0-684544326.85-31.23 6.39  +0.16 +0.09         A0V+F5IV           +0.034-0.067      -018SB2O                      *
7589          BD+46 2793 188209 489173588          Var      194858.2+464609195159.1+470139 80.99 10.09 5.62  -0.07 -0.97 -0.12   O9.5Ia             -0.003 0.000 +.009-006V      77                 *
7590   Eps PavCP-73 2086 188228257757 748          12598    194901.8-731028200035.5-725438322.02-30.90 3.96  -0.03 -0.05 -0.03C  A0V                +0.074-0.132 +.016-002V?    110
7591          BD+47 2939 188252 48920              12511    194910.4+474025195207.2+475555 81.81 10.51 5.91  -0.18 -0.85         B2III              -0.005-0.007      -018       74
7592 13    VulBD+23 3820 188260 87883      W                194912.6+234907195327.7+240447 61.27-01.79 4.58  -0.06 -0.14 -0.05   B9.5III            +0.027+0.038 +.009-028SB     50  3.2   1.0
7593 57    AqlBD-08 5154 188293143898       13087A          194912.8-082916195437.6-081338 32.66-17.76 5.71  -0.08 -0.48         B7Vn               +0.010-0.025 +.010-006SB    340  0.8  35.7      *
7594 57    AqlBD-08 5155 188294143899       13087B          194913.2-082951195438.1-081414 32.65-17.77 6.49  -0.04 -0.26         B8V                +0.021-0.027 +.010-005SB    266  0.8  35.7      *
7595 59Xi  AqlBD+08 4261 188310125210    I                  194924.1+081209195414.9+082741 47.81-09.85 4.71  +1.05 +0.91 +0.57   G9+IIIb            +0.101-0.081 +.023-042V?   < 19:
7596 58    AqlBD-00 3871 188350125219                       194937.4+000044195444.8+001625 40.55-13.92 5.61  +0.10 -0.01         A0III              +0.042-0.012      -050      130
7597 58Ome SgrCD-2614637 188376188722    I                  194942.9-263353195550.4-261758 14.86-25.04 4.70  +0.75 +0.32 +0.37   G5V                +0.207+0.083 +.062-021SB                        *
7598          BD+06 4351 188385125221       13093           194947.1+065242195440.3+070825 46.70-10.60 6.15  +0.03 +0.04         A2V                +0.034-0.003      -016           6.4  13.8      *
7599          BD-07 5102 188405143911       13104           194957.7-065945195519.6-064403 34.15-17.26 6.51  +0.39 +0.05         F2V                +0.036-0.046D+.010-007SB3        1.1   0.2      *
7600          BD+47 2945 188439 48940              V819 Cyg 195003.6+473251195301.2+474827 81.77 10.32 6.29  -0.11 -0.91         B0.5IIIn           -0.009-0.009      -065SB    400                 *
7601          BD+23 3829 188485 87908                       195016.4+240326195431.1+241910 61.60-01.87 5.52  -0.02 -0.16         A0III              +0.026 0.000 +.015-008V     117
7602 60Bet AqlBD+06 4357 188512125235 749I  13110  12557    195024.0+060925195518.8+062424 46.13-11.09 3.71  +0.86 +0.48 +0.49   G8IV               +0.048-0.482 +.070-040V   =< 16  7.7 175.0AC   3*
7603   Mu 1PavCP-67 3695 188584254702                       195039.1-671246200023.0-665658328.96-31.52 5.76  +1.04 +0.80         K0III              -0.020-0.197 +.037+029
7604 59    SgrCD-2714399 188603188742    I                  195048.7-272606195656.8-271012 14.05-25.56 4.52  +1.46 +1.57 +0.73   K2.5IIb            +0.008-0.013 +.014-016SB                        *
7605          CD-3813776 188642211686                       195100.8-381921195741.3-380330  2.33-28.74 6.55  +0.39               F4V                +0.089-0.083      -031
7606          BD+36 3766 188650 69072                       195109.0+364355195448.3+365946 72.50  4.61 5.76  +0.75 +0.35         G1Ib-IICH1Fe-1Ca-1 +0.014+0.021      -024                          *
7607          BD+29 3802 188651 69079       13117           195107.1+295552195506.5+301143 66.70  1.05 6.57  -0.08 -0.47 -0.14   B6V+A5V            +0.011+0.017      -012      200  2.9   9.3AB   4
7608 23    CygBD+57 2084 188665 32085                       195114.2+571541195317.4+573125 90.55 14.91 5.14  -0.13 -0.55         B5V                +0.009+0.013      -025V?    145                 *
7609 10    SgeBD+16 4067 188727105436              S Sge    195128.6+162211195601.3+163805 55.17-06.12 5.36  +0.67 +0.42 +0.45   G5Ib              v+0.004-0.004 +.009-010SBO    20:                *
7610 61Phi AqlBD+11 4055 1887281054383590                   195130.1+110929195614.3+112526 50.66-08.80 5.28  -0.01 -0.08         A1IV               +0.034+0.009 +.023-027SB1O   26                 *
7611          BD+59 2137 188793 32093      W                195148.0+592639195335.4+594232 92.58 15.87 6.06  +0.02 +0.03         A3V                +0.043+0.061      -014SB    110  3.3  71.4
7612   Mu 2PavCP-67 3698 188887254707                       195208.6-671251200152.4-665639328.95-31.67 5.31  +1.22 +1.29 +0.41E  K2IVCNII           +0.030-0.073 +.025+042                          *
7613 22    CygBD+38 3817 188892 69101                       195217.2+381315195551.7+382912 73.90  5.19 4.94  -0.08 -0.51 -0.10   B5IV                0.000+0.002      -030V      85                 *
7614 61    SgrBD-15 5516 1888991631411522                   195216.7-154524195757.0-152929 26.02-21.54 5.02  +0.05 +0.07         A3IV               +0.017-0.101 +.032+003V     102                 
7615 21Eta CygBD+34 3798 188947 691161521I  13149  12586    195233.2+344903195618.4+350500 71.02  3.36 3.89  +1.02 +0.89 +0.52   K0III              -0.032-0.027 +.015-027     < 19: 7.7  46.0AC   5*
7616          BD+20 4351 188971 87963                       195237.7+204352195700.2+205953 59.05-04.08 6.48R +0.07 -0.05         A2IV               -0.024-0.023      +008
7617          CD-3017525 188981211708                       195239.0-304822195856.4-303217 10.62-27.01 6.28  +1.05               K1III              +0.090-0.053      -051V
7618 60    SgrCD-2614682 189005188778    I                  195251.7-262759195857.2-261144 15.23-25.67 4.83  +0.90 +0.55 +0.47   G6IIIBa0.2         +0.035+0.033 +.014-049SB                        *
7619 24Psi CygBD+52 2572 189037 32114       13148           195302.6+521024195537.8+522620 86.11 12.21 4.92  +0.12 +0.06 +0.06   A4Vn               -0.037-0.030 +.005-010V     276  2.3   3.1AB   4*
7620          BD+35 3878 189066 69129                       195302.2+355859195644.2+361503 72.06  3.89 6.02  -0.15 -0.59         B5IV               +0.004+0.005      -023                          *
7621          CD-4912949 1890802299733595                   195302.2-493721200025.3-492104349.59-31.19 6.17  +1.06 +0.92         K0III              -0.082-0.006      +067
7622 11    SgeBD+16 4081 1890901054713592                   195312.9+163111195745.4+164721 55.52-06.40 5.53  -0.05 -0.17         B9III              +0.012+0.016 -.003-026V?     52
7623   The1SgrCD-3513831 189103211716 751                   195313.7-353248195944.2-351635  5.52-28.47 4.37  -0.15 -0.67         B3IV               +0.006-0.026      +001SBO    69                 *
7624   The2SgrCD-3513832 189118211717      W                195321.9-345801195951.3-344152  6.17-28.34 5.30  +0.17 +0.06 +0.07   A4/5IV             +0.100-0.075 +.033-018        0  5.5  30. AB   3
7625          CP-59 7564 1891242463893598          NU Pav   195319.2-593854200144.7-592234337.85-31.96 5.13  +1.53 +1.31 +1.55E  M6III              +0.017-0.033 +.005-010                          *
7626          BD+57 2092 189127 32116                       195323.0+575909195522.1+581501 91.35 15.00 6.09  +1.02               G9III              +0.021-0.071      -017
7627          CD-4313735 189140229977                       195329.4-431856200026.5-430236356.85-30.27 6.14  +1.64 +2.02         M0II-III           +0.023+0.013      -034
7628          BD+39 3968 189178 490113593  W                195345.4+400556195713.9+402205 75.65  5.93 5.45  -0.10 -0.52         B5V                +0.008+0.007      -026SB    115  3.0  64.6      *
7629          CD-3813802 189195211719      W                195337.8-375825200015.9-374208  2.86-29.15 5.95  +1.00 +0.77         G8III              +0.018-0.020      +023           7.6  23.0
7630          CD-4513549 1891982299813597                   195343.4-452309200048.3-450647354.49-30.68 5.81  +0.28 +0.11 +0.17C  A8III               0.000+0.002 +.017+003SB    123                 *
7631          CD-3414082 189245211724              12655    195353.6-335804200020.3-334214  7.29-28.18 5.66  +0.49 -0.02         F7V                +0.136-0.306 +.049-008SB     86                 *
7632          BD+50 2930 189253 32130                       195401.0+503801195645.1+505409 84.82 11.31 6.43  -0.01 -0.05         A1V                +0.005+0.008      -019V     118
7633          BD+58 2013 189276 321223591I                  195400.9+583444195555.4+585046 91.93 15.21 4.96  +1.59 +1.93         K5II-III           -0.009-0.019 +.004+005     < 19:
7634          BD+56 2331 189296 32128                       195408.6+562506195619.0+564113 89.98 14.15 6.12R                     A4Vn               +0.022+0.016      -031V     175
7635 12Gam SgeBD+19 4229 189319105500 752I         12638    195418.5+191314195845.4+192932 57.97-05.21 3.47  +1.57 +1.93 +0.92   M0-III            e+0.066+0.024 +.013-033V?   < 17                 *
7636          BD+00 4375 1893221253103596                   195417.6+010615195922.7+012240 42.11-14.42 6.17  +1.13 +0.90         G8III              +0.026+0.065      +006
7637          BD-10 5238 189340163168      W                195421.1-101310195947.4-095730 31.60-19.66 5.88  +0.58 +0.05         F8V                -0.269-0.400 +.040+023V?         0.0   0.1
7638          BD+41 3549 189377 49031       13186           195434.4+415926195756.2+421539 77.36  6.79 6.43R                     A3V                +0.001-0.003D+.003-006SB         1.2   0.1AB   3*
7639          CD-4113807 189388229993                       195438.2-410515200126.4-404851359.42-30.04 6.29  +0.11               A2-3V              -0.015-0.038      -031
7640          BD+30 3837 189395 691883594                   195440.0+304244195838.0+305901 67.77  0.82 5.49  -0.06 -0.19         B9Vn               +0.041+0.003 +.036-007V     175
7641 14    VulBD+22 3872 189410 88016                       195453.0+224944195910.5+230605 61.11-03.41 5.67R                     F0                 -0.071+0.006 +.019-038SB    120
7642          BD+37 3703 189432 69193       13198           195457.9+375002195834.4+380620 73.84  4.54 6.32  -0.09 -0.50         B5IV               +0.009-0.004D+.004-014       15  1.1   1.9
7643          CD-2315935 1895611888293600                   195527.3-230044200123.9-224414 19.04-25.02 6.01  +0.97 +0.77         G7V                +0.010-0.014      +008V?
7644          CP-67 3703 189567254721                       195532.3-673453200532.8-671915328.48-31.98 6.07  +0.64 +0.08 +0.23E  G3V                +0.844-0.672 +.054-012
7645 13    SgeBD+17 4183 189577105522    I  13230  VZ Sge   195532.3+171435200003.3+173100 56.43-06.49 5.37  +1.67 +1.75 +1.20E  M4IIIa             +0.004-0.011      -017V?         7.5  28.5      *
7646          BD+45 3025 189684 49058                       195611.9+452957195920.5+454620 80.53  8.37 5.92  +0.16 +0.15         A5III              +0.020-0.021      +006V
7647 25    CygBD+36 3806 189687 692313599          V1746 Cyg195615.1+364607195955.2+370234 73.07  3.76 5.19  -0.17 -0.69         B3IVe              +0.006+0.001      -004SB    229                 *
7648          BD+08 4300 189695125355    I                  195608.7+081658200058.9+083329 48.74-11.25 5.91  +1.52 +1.89         K5III              +0.008-0.009      -040V?
7649 63    SgrBD-14 5618 189741163195                       195622.5-135451200158.6-133813 28.27-21.69 5.71  +0.08               A3V                +0.038+0.018 +.020-042
7650 62    SgrCD-2816355 189763188844 753I         V3872 Sgr195630.6-275916200239.5-274235 13.92-26.93 4.58  +1.65 +1.80 +1.56   M4III              +0.036+0.017 +.028+010V                         *
7651          BD+51 2728 189775 32170                       195636.3+514654195915.4+520321 86.04 11.54 6.15  -0.19 -0.65         B5III              +0.015+0.006      -016V
7652          CD-3813828 1898312117673602                   195654.9-381302200333.5-375627  2.77-29.84 4.77  +1.41 +1.68 +0.56E  K5III              +0.059-0.092 +.020-038                          *
7653 15    VulBD+27 3587 189849 880711523          NT Vul   195658.9+272837200106.1+274513 65.30-01.34 4.64  +0.18 +0.16 +0.09   A4III              +0.059+0.005 +.030-021SBO    23                 *
7654          BD+63 1584 189900 18627                       195713.7+631540195828.7+633203 96.42 17.09 5.96R +0.09?              A3V                -0.005-0.026      -009
7655          BD+36 3820 189942 69267                       195734.8+364913200115.3+370556 73.26  3.56 6.20  +1.31               K0III              +0.044+0.051      -016
7656          BD+24 3975 189944 88088                       195730.4+243122200144.7+244801 62.87-03.02 5.88  -0.12 -0.51         B4V                +0.005-0.003      -015
7657 16    VulBD+24 3977 190004 88098       13277           195746.8+243928200201.4+245617 63.02-03.00 5.22  +0.36 +0.09         F2III              +0.088+0.070D+.004-037V     121  0.3   0.8
7658          BD-22 5318 190009188863      D                195748.8-225235200344.3-223544 19.39-25.48 6.45  +0.50               F6V                -0.042+0.020      +007V                         *
7659          CD-3215682 1900562117823606I W       12746    195759.2-322013200419.6-320323  9.34-28.55 4.99  +1.21 +1.16         K1III              +0.047-0.015 +.007-012           7.6  51.3
7660 26    CygBD+49 3158 190147 49098    I  13278A          195831.8+494935200121.6+500617 84.49 10.28 5.05  +1.11 +1.02 +0.55   K1II-III           +0.019+0.004 +.014+001V    < 19: 3.8  41.8AB   5
7661          BD-07 5159 190172144038                       195838.3-074459200401.2-072811 34.46-19.52 6.72  +0.35 +0.07         F4III              +0.015-0.048      +004V?
7662          BD+18 4365 190211105608    I W                195847.2+181315200316.4+183002 57.67-06.63 5.96  +1.42 +1.60         K3II-III+G8IV      +0.015-0.031      +009           3.3  47.1
7663          CP-66 3473 190222254731                       195849.4-663826200820.5-662117329.57-32.36 6.45  +1.58 +1.98         K5III              -0.010-0.005      +036
7664          BD+15 4033 190229105615                       195855.4+154502200330.0+160153 55.58-07.97 5.67  -0.10 -0.48         B9pHgMn            +0.003-0.005 +.008-022SB1    11                 *
7665   Del PavCP-66 3474 190248254733 754          12790    195854.9-662612200843.6-661055329.79-32.40 3.56  +0.76 +0.45 +0.34   G6-8IV             +1.207-1.131 +.170-022                          *
7666          BD+69 1084 190252  9592                       195856.5+700522195841.9+702201102.91 20.06 6.33  +0.88 +0.54         G8III              +0.045+0.060      -010                          *
7667 62    AqlBD-01 3887 190299144045    I                  195914.0-005918200423.2-004234 40.83-16.51 5.68  +1.30 +1.35         K4III              +0.003-0.118 +.004+000V?
7668          CD-3314700 190306211798      W                195909.4-331659200532.1-330000  8.38-29.04 6.53  -0.08               B8                 +0.018-0.018      -018           1.0   0.4AB   3
7669 63Tau AqlBD+06 4416 1903271254031524I                  195915.2+065944200408.3+071641 48.01-12.57 5.52  +1.06 +0.86        gK0                 +0.016+0.011 +.005-028
7670          BD+29 3872 190360 88133                       195930.7+293747200337.4+295348 67.41-00.66 5.71  +0.73 +0.37         G6IV+M6V           +0.683-0.526 +.047-045V          9.8 179.       *
7671          BD-12 5641 190390163245      W       V1401 Aql195934.1-115256200505.4-113558 30.60-21.53 6.34  +0.52               F1III              -0.001-0.012      -012V?     23       25.       *
7672 15    SgeBD+16 4121 190406105635      W       12757    195936.6+164757200406.2+170412 56.56-07.55 5.80  +0.61 +0.09         G1V                -0.395-0.414 +.060+004V       4  0.9 203.7AC   6
7673   Xi  TelCP-53 9794 190421246443 755          12783    195943.5-531001200723.2-525251345.55-32.57 4.94  +1.62 +1.84 +0.80E  M1IIab             -0.014+0.008 +.014+036SB                        *
7674          CP-55 9317 190422246444                       195942.7-551811200735.1-550059343.02-32.69 6.26  +0.53               F8V                +0.016+0.023      +013
7675 65    SgrBD-13 5569 190454163253                       195952.7-125652200526.4-123955 29.59-22.06 6.55  +0.04  0.00         A1Vn               +0.009-0.049      -006      201
7676 64    DraBD+64 1405 190544 186583604I W                200025.0+643227200128.5+644916 97.77 17.38 5.27  +1.56 +1.81         M1III-IIIb         +0.007-0.012 +.006-034               193.8
7677          BD+22 3913 190590 88163      S                200040.1+225533200458.6+231237 61.91-04.49 6.45R                     A5Vn               +0.006-0.002      -022                 0.1
7678          BD+31 3925 190603 69362       13335  V1768 Cyg200040.8+315606200436.1+321307 69.49  0.39 5.64  +0.54 -0.46 +0.42   B1.5Ia            e-0.011-0.012      +021V      29  4.5  32.1      *
7679 16Eta SgeBD+19 4277 1906081056593609I                  200043.3+194215200509.5+195928 59.18-06.23 5.10  +1.06 +0.98 +0.34E  K2III              +0.030+0.078 +.032-040V?   < 19:                *
7680          BD+15 4040 190658105663    I  13344           200050.4+151253200526.5+153001 55.36-08.64 6.34  +1.64 +1.76         M2.5III            +0.033+0.020      -112SB         4.0   2.7      *
7681          BD-04 5013 190664144062                       200056.0-042147200612.2-040442 37.92-18.48 6.47  +1.16 +1.00         K0                 +0.045-0.045      -002
7682 65    DraBD+64 1407 190713 18669      W                200113.6+642107200220.3+643804 97.64 17.21 6.57R                    gG7                 +0.046+0.010      +009V?         2.1  96.8AB   3
7683          BD+38 3896 190771 69377       13348           200130.9+381124200509.8+382842 74.84  3.64 6.19  +0.64 +0.22 +0.38   G5IV               +0.259+0.109 +.041-024           6.1  40. AC   3*
7684          BD+47 3004 190781 491523610  S                200129.2+475642200428.8+481347 83.12  8.88 6.16  +0.04 +0.04         A2IV               +0.011+0.004      -014                 0.3
7685 67Rho DraBD+67 1222 190940 186763608I W                200222.1+673518200249.1+675225100.70 18.64 4.51  +1.32 +1.51 +0.65   K3III              +0.016+0.046 +.017-009     < 19: 8.6 124. AB   3
7686 69    DraBD+76  771 190960  96063605I                  200225.1+761209195936.6+762853108.94 22.48 6.20  +1.61 +1.89 +0.94E  M3III              -0.029-0.055      -069V?
7687          BD+51 2763 190964 32272    I                  200224.1+513308200506.7+515022 86.31 10.65 6.14  +1.60 +1.95         M1IIIa             +0.029+0.034      -056
7688 17    VulBD+23 3896 190993 882123611                   200235.5+231934200653.4+233652 62.48-04.65 5.07  -0.18 -0.67         B3V                +0.015+0.003      -005SB    214
7689 27    CygBD+35 3959 191026 69413      W       Var?     200238.8+354149200621.8+355821 72.86  2.10 5.36  +0.85 +0.54 +0.44   K0IV               -0.224-0.438 +.032-033           4.0  11.9AD   4*
7690 64    AqlBD-01 3899 191067144095                       200252.0-005758200801.8-004042 41.30-17.29 5.99  +1.02 +0.90         K1IV               +0.116-0.070      -004V?
7691          CP-57 9622 191095246470      W                200300.3-574859201107.2-573126340.04-33.20 6.37  +0.06               B7V+A3V            -0.002-0.025      -007SB3        0.6   0.8      *
7692          BD+55 2324 191096 32276                       200305.1+560307200521.5+562029 90.30 12.90 6.21  +0.43 +0.03         F4V                -0.004+0.080      -012V?     40
7693          BD+08 4344 191104125478       13403           200301.2+090633200750.3+092359 50.35-12.29 6.43  +0.46 -0.02         F3V                +0.047+0.022D+.015-027SB         2.1   3.1      *
7694          BD-10 5285 191110163290                       200303.3-102108200831.3-100346 32.48-21.65 6.18  +0.06               B9.5III            +0.008-0.042      -016SB2O                      *
7695          BD+63 1593 191174 18692       13371           200329.2+633608200444.6+635326 97.08 16.64 6.26  +0.10 +0.04         A2II-III           +0.007+0.043D+.023-019V?         3.6   5.5AB   3
7696          BD+16 4153 191178105733    I                  200332.8+162225200806.5+163951 56.71-08.58 6.42  +1.91 +1.86         M3III              +0.005-0.002      +011V?
7697          BD+52 2623 191195 322863613  W                200335.8+525212200613.8+530957 87.56 11.18 5.85  +0.39 -0.01         F5V                +0.212+0.255 +.027-041V   =< 10  3.8 170.4
7698          CP-83  695 191220258856                       200333.7-833708202454.4-831838310.00-29.60 6.17  +0.20 +0.13         Am                 +0.011+0.012      +000                          *
7699          BD+34 3881 191243 694573615                   200351.7+340757200741.4+342523 71.69  1.03 6.11  +0.16 -0.45         B5Ib               +0.002 0.000      +010V?                        *
7700          BD+10 4189 1912631057433616                   200352.1+102604200838.3+104333 51.62-11.78 6.31  -0.12 -0.67         B3V                +0.004+0.001      -025                          *
7701 66    DraBD+61 1970 191277 187003612I                  200357.4+614218200532.8+615944 95.39 15.65 5.39  +1.18 +1.29 +0.59   K3III              +0.121+0.074 +.037+006
7702          BD+49 3195 191329 32296                       200420.2+495622200711.5+501345 85.08  9.54 6.54  +0.13               A3V                +0.018-0.020      +003
7703          CD-3613940 191408211885      W                200437.7-362109201111.9-360604  5.26-30.91 5.32  +0.87 +0.46 +0.49   K3V                +0.458-1.571 +.179-130V?         4.0      AD   4*
7704          BD+67 1226 191372 18699    I                  200427.6+674421200453.3+680138100.94 18.54 6.28  +1.65 +2.00 +0.97E  M3IIIa             -0.017-0.012      -042                          *
7705 17The SgeBD+20 4453 191570 88276       13442A          200531.8+203704200956.6+205455 60.55-06.68 6.48  +0.38 -0.04         F5IV               +0.054+0.101 +.029-040           0.6  83.9AC   4*
7706          CD-4313855 191584230100                       200531.5-430428201223.9-424650357.57-32.39 6.22  +1.23               K1III              -0.014-0.118      -001
7707          CP-63 4571 1916032547563619                   200533.3-634257201426.9-632457332.97-33.29 6.09  +0.31 +0.09         F0IV               -0.037+0.037      +019
7708 28    CygBD+36 3907 191610 695181525          V1624 Cyg200542.7+363242200925.6+365023 73.91  2.04 4.93  -0.13 -0.77 -0.14   B2.5Ve            v+0.005+0.014      -014SB?   310                 *
7709          BD-09 5382 1916391441443618          Var      200544.7-090818201110.1-085032 33.97-21.71 6.49  -0.15 -0.92         B1V                +0.006 0.000      -007                          *
7710 65The AqlBD-01 3911 191692144150 756I'W                200608.7-010706201118.3-004917 41.58-18.08 3.23  -0.07 -0.14 -0.05   B9.5III            +0.038+0.004 +.012-027SB2O   63  9.6 113.7      *
7711 18    VulBD+26 3815 191747 88295                       200623.0+263628201033.5+265415 65.71-03.57 5.52  +0.08 +0.13         A3III              +0.020+0.013 +.002-012SB2O   41                 *
7712  1Xi 1CapBD-12 5664 191753163328                       200625.4-124122201157.9-122333 30.58-23.39 6.34  +1.21               K0III              -0.005-0.017      +001
7713          BD+20 4462 191814 88309                       200639.3+205013201103.5+210805 60.90-06.79 6.22  +0.90               K0                 +0.017+0.034      -007
7714          CP-5211643 191829246495                       200644.2-524441201419.0-522644346.13-33.60 5.65  +1.50               K4III              +0.022-0.045 +.020+014V
7715  2Xi 2CapBD-13 5608 191862163337      W                200651.6-125438201225.9-123703 30.41-23.59 5.85  +0.48 -0.07         F7V                +0.197-0.193 +.037+023V?         6.2  71.6
7716          BD+21 4088 191877 88315                       200658.7+213440201121.1+215232 61.57-06.45 6.26  -0.02 -0.85         B1Ibe              -0.001+0.007      -018
7717          BD+00 4444 191984125567       13506A          200729.1+003407201235.2+005203 43.30-17.56 6.27  +0.02 -0.01         B9pCrEu:Sr:        +0.010-0.014D+.009-019           0.2   2.8      *
7718 19    VulBD+26 3825 192004 88330    I                  200737.2+263038201148.0+264832 65.78-03.85 5.49  +1.41 +1.51 +0.70   K3II-III           +0.007-0.010      -023                          *
7719 20    VulBD+26 3828 192044 88339              12890    200749.0+261048201200.7+262844 65.53-04.07 5.92  -0.11 -0.43         B7Ve               +0.008-0.011      -022V     350                 *
7720 66    AqlBD-01 3920 192107144181    I                  200804.0-011833201313.9-010034 41.65-18.60 5.47  +1.43 +1.30         K5III              +0.022-0.028 +.028-028V                         *
7721          BD+47 3045 192276 49314                       200859.4+472612201203.8+474413 83.37  7.54 6.92  -0.12 -0.45         B7V                +0.016+0.004      -019SBO                       *
7722          CD-2714659 192310189065              12933    200903.1-271953201517.4-270158 15.62-29.38 5.73  +0.88 +0.64 +0.45   K0V               v+1.244-0.174 +.124-054SB?
7723          BD+23 3935 192342 88377       13543           200922.9+235608201340.6+241420 63.85-05.61 6.56  +0.29 +0.12 +0.14   A1m                +0.050+0.031D+.011-037           3.3   2.7      *
7724 67Rho AqlBD+14 4227 1924251058781526                   200938.9+145334201416.6+151151 56.24-10.61 4.95  +0.08 +0.01  0.00   A2V                +0.057+0.057 +.023-023SB?   152
7725          CD-3017773 1924332119403621                   200937.9-301837201550.6-300019 12.38-30.36 6.30  +1.51               K4III              +0.037+0.013      -099V
7726          BD+51 2796 192439 32354       13535           200944.8+510945201231.8+512749 86.60  9.47 6.01  +1.14               K2.5III            -0.011-0.014      +013           5.6   4.0AB   5*
7727 68    DraBD+61 1983 192455 18751      W                200956.6+614633201134.9+620443 95.83 15.08 5.75  +0.47 +0.02         F5V                +0.123+0.083 +.031-015V?  =< 10  9.4  93.
7728          CD-3614011 192472211948      W       12949    200953.2-364532201623.7-362716  5.09-32.02 6.39  +1.54               M4III              +0.009-0.042      -014V          5.1  38.0AB   3
7729          CD-3514020 192486211950                       200959.8-353020201626.5-351152  6.54-31.76 6.53  +0.37               F2V                +0.009+0.081      -008
7730 30    CygBD+46 2881 192514 49332       13554D 12926    201009.4+463047201318.0+464857 82.71  6.87 4.83  +0.09 +0.15 +0.08   A5IIIn             +0.012-0.001 +.012-026V?    146  1.0 337.5AD   6*
7731 21    VulBD+28 3675 192518 88391              NU Vul   201007.9+282330201414.5+284141 67.66-03.27 5.18  +0.18 +0.16         A7IVn              +0.014-0.022 +.010+007V?    197                 *
7732          CP-63 4576 192531254772                       201013.7-633215201903.0-631352333.13-33.82 6.27  +1.04               K0III              -0.015-0.084      -008
7733          BD+42 3642 192535 493363620I  13555           201020.8+430432201342.8+432245 79.85  4.93 6.14  +1.50 +1.83         K4III               0.000+0.006      -024           6.4  57.3AC   4
7734          BD+36 3949 192538 69653              Var?     201020.1+361805201405.0+363619 74.23  1.14 6.45   0.00 -0.02         A0V                +0.024+0.002      -022      142
7735 31    CygBD+46 2882 192577 49337 757I  13554A V695 Cyg 201028.9+462616201337.9+464429 82.68  6.78 3.79  +1.28 +0.42 +0.76   K2II+B3V           +0.004+0.003 +.007-008SBO =< 25  1.0 337.5AD   6*
7736 29    CygBD+36 3955 192640 69678      W       V1644 Cyg201047.3+362959201432.0+364823 74.45  1.17 4.97  +0.14  0.00 +0.07   A2V                +0.068+0.070 +.036-017V      37  1.6 212.4AB   5*
7737          BD+41 3668 192659 49345       13572           201054.7+414759201421.6+420613 78.85  4.13 6.71  -0.04 -0.28         B9IV-V             +0.005-0.021      -018V          0.2   0.9AB   3*
7738  3    CapBD-12 5680 192666163402       13600           201050.6-123836201622.8-122013 31.12-24.35 6.32  +0.01 -0.17         B9pHg              +0.012-0.013      -020           7.3  26.9AB   3
7739          BD+25 4165 192685 88410       13589  QR Vul   201101.6+251711201515.9+253531 65.19-05.17 4.78  -0.18 -0.72 -0.20   B3Ve              v+0.008-0.006      -007SB?   249  2.4   0.7AB   3*
7740 33    CygBD+56 2376 192696 32378 758          12935    201104.4+561542201323.9+563404 91.08 12.06 4.30  +0.11 +0.08 +0.06   A3IV-Vn            +0.063+0.083 +.021-018SB    268                 *
7741 22    VulBD+23 3944 192713 88416    I         QS Vul   201111.0+231211201530.2+233031 63.47-06.36 5.15  +1.04 +0.71 +0.48   G3Ib-II             0.000-0.015 +.004-023SB1O   17:                *
7742          BD+60 2099 192781 18766    I                  201137.4+602005201327.7+603826 94.66 14.16 5.79  +1.47              gK5                 +0.044+0.058      -001V?
7743          BD+33 3827 192787 69701       13596           201130.7+332534201523.7+334346 71.99-00.68 5.66  +0.91 +0.66         K0III              -0.040-0.106      -010V          6.3   3.0      *
7744 23    VulBD+27 3666 192806 88428    I S                201137.4+273026201546.1+274851 67.11-04.04 4.52  +1.26 +1.12 +0.71   K3-IIIFe-1         -0.036+0.008 +.008+003     < 19:                *
7745          CD-4813509 1928272301441528                   201145.7-480115201856.0-474239351.87-34.08 6.31  +1.67 +2.02         M1III              +0.012-0.004      -044V?                        *
7746 18    SgeBD+21 4130 192836 88433                       201156.1+211730201619.7+213555 61.96-07.57 6.13  +1.04 +0.92 +0.53   K1III              +0.001-0.024      -004                          *
7747  5Alp1CapBD-12 5683 1928761634221527I  13632           201206.3-124902201738.9-123030 31.09-24.70 4.24  +1.07 +0.78 +0.53   G3Ib               +0.023+0.001 +.006-026SB  =< 54  0.4 377.7 *   9*
7748  4    CapBD-22 5384 1928791891141529                   201208.9-220708201801.4-214836 21.50-28.31 5.87  +1.00               K0III              +0.035-0.024 +.040-018V?
7749          CD-4713340 192886230150                       201206.5-475310201917.9-473449352.03-34.13 6.13  +0.46 +0.01 +0.26C  F5V                +0.200-0.185 +.026-031                          *
7750  1Kap CepBD+77  764 192907  9665 759   13524           201215.7+772437200853.3+774241110.38 22.50 4.39  -0.05 -0.11 -0.06   B9III              +0.011+0.024 +.001-023V      18  3.8   7.3AB   3*
7751 32    CygBD+47 3059 192909 49385    I W       V1488 Cyg201222.8+472425201528.3+474252 83.67  7.05 3.98  +1.52 +1.03 +0.92   K3Ib+B3V           +0.001+0.008 +.014-014SBO  < 25  5.5 208.9      *
7752          BD+38 3977 192934 69720      S                201225.4+383527201603.4+385352 76.36  2.08 6.27  +0.01 -0.01         A1V                -0.001-0.037      +008V?     73        0.2
7753 24    VulBD+24 4075 192944 88451 760                   201230.3+242147201647.1+244016 64.61-05.97 5.32  +0.95 +0.67         G8III              +0.016-0.018 -.003+015V    < 19:
7754  6Alp2CapBD-12 5685 192947163427 761I  13645           201230.4-125117201803.3-123241 31.10-24.81 3.57  +0.94 +0.69 +0.48   G8IIIb             +0.064+0.004 +.034+000SB?  < 17  0.4 377.7 *   9*
7755          BD+49 3236 192983 32400      S                201248.8+495529201543.4+501358 85.82  8.38 6.31R                     A2Vn               +0.024+0.010      -025V?    335        0.2
7756          BD+45 3119 192985 49395                       201246.1+451622201600.6+453446 81.93  5.80 5.91  +0.38 -0.04         F5V:               +0.008-0.052      -040    =< 10
7757          BD+36 3978 192987 69725                       201244.3+364452201628.2+370323 74.87  0.99 6.48  -0.09 -0.41         B6III              +0.013+0.001      -008      300                 *
7758          CP-55 9365 1930022465353626                   201245.6-552145202032.3-550303342.99-34.54 6.27  +1.59 +2.01         M0-1III            +0.009-0.033      -009
7759          BD+39 4114 193092 49410    I  13640  12981    201321.7+400320201655.3+402154 77.67  2.76 5.24  +1.65 +1.88 +0.89   K3.5IIab-IIb       +0.004-0.003 +.004-020V?   < 19: 5.5  12.8AB   3
7760          BD+28 3695 193094 88473       13648           201325.7+285012201731.5+290853 68.43-03.62 6.22  +1.01               G9III              -0.011+0.032D+.009-020           4.7   5.9
7761  7Sig CapBD-19 5776 1931501634453625I  13675           201337.4-192550201923.6-190707 24.48-27.66 5.28  +1.40 +1.53         K3II               +0.009-0.009 +.025-011V          3.5  56.5
7762          BD+42 3670 193217 49425    I                  201403.4+422440201729.1+424319 79.69  3.99 6.29  +1.64 +1.91         K4II:              +0.014 0.000      -017
7763 34    CygBD+37 3871 193237 69773    I         P Cyg    201406.0+374319201747.2+380159 75.83  1.32 4.81  +0.42 -0.58 +0.26   B2pe               -0.005-0.004 +.004-009V      75                 *
7764          CD-2916981 193281189164       13702           201418.4-293041202028.1-291150 13.60-31.11 6.30  +0.17               A4/7V              +0.028+0.004      +004V      82  1.1  27.2AC   5*
7765          CD-3614057 1933022120251530                   201424.5-355920202051.9-354025  6.22-32.75 6.46  +1.31               K3-4III            +0.039+0.018      -015
7766          CD-5012929 1933072465463630                   201425.3-501829202141.0-495958349.13-34.67 6.27  +0.55 -0.02         G0V                -0.357-0.250 +.045+018                          *
7767          BD+40 4103 193322 49438       13672           201434.4+402513201807.0+404356 78.10  2.78 5.84  +0.10 -0.78 +0.01   O9V                +0.001-0.003 +.007-007V     110: 2.5   2.6AB   5*
7768          BD-01 3951 193329144296                       201433.2-012335201943.3-010443 42.40-20.06 6.06  +1.09 +1.02         K0                 +0.042+0.033      -045V
7769 36    CygBD+36 3998 193369 69803                       201443.8+364112201828.6+370000 75.05  0.63 5.58  +0.06  0.00         A2V                +0.037+0.025      -008SB    105
7770 35    CygBD+34 3967 193370 698063627          12994    201448.6+344012201839.1+345858 73.40-00.54 5.17  +0.65 +0.48 +0.42   F5Ib               +0.001-0.003 +.007-014SBO    13                 *
7771          BD+12 4289 193373105961    I                  201447.6+125413201929.2+131301 55.22-12.74 6.21  +1.63               M1III              -0.029-0.012      +023V?
7772          BD-06 5451 1934291443133628I         13013    201506.7-064027202026.1-062142 37.47-22.67 6.63  +1.55 +1.84         K5                 -0.081-0.090      +043V                         *
7773  8Nu  CapBD-13 5642 193432163468       13714           201507.0-130426202039.8-124533 31.17-25.48 4.76  -0.05 -0.11 -0.06   B9.5V              +0.019-0.019 +.021-002V?     17  7.0  54.1      *
7774          BD+13 4360 193472105974                       201519.0+131401202000.2+133253 55.57-12.67 5.95  +0.31 +0.13 +0.16   A5-F2m             +0.012-0.004      -008SB2    93                 *
7775          BD-15 5626 193452163471       13717           201509.4-150601202046.6-144706 29.12-26.32 6.10  -0.02 -0.11         A0III              +0.043 0.000 +.010-018SB1O   97  0.3   0.0 *   8*
7776  9Bet CapBD-15 5629 193495163481 762I W                201523.6-150550202100.7-144653 29.15-26.37 3.08  +0.79 +0.28 +0.50   F8V+A0             +0.042+0.002 +.010-019SBO    54  0.3   0.0 *   8*
7777          BD+45 3139 193536 49462              V1773 Cyg201537.2+460031201849.6+461922 82.82  5.80 6.45  -0.13 -0.69         B2V                -0.005+0.010      -009SB2O                      *
7778          BD+14 4263 1935561059883629                   201541.4+141513202020.5+143409 56.50-12.19 6.13  +0.91 +0.64 +0.44   G8III              -0.001+0.004      +008
7779   Kap1SgrCD-4214836 193571230177 763  W                201540.2-422153202227.5-420259358.75-34.13 5.59   0.00  0.00 -0.01C  A0V                +0.036-0.090 +.035-012V          6.0  52.0AC   3
7780          BD+17 4294 193579105991    I                  201548.9+172842202021.4+174735 59.26-10.44 5.80  +1.49              gK5                 +0.020-0.033      -033
7781          BD+54 2329 193592 32455       13692           201556.5+550503201824.8+552350 90.45 10.84 5.76  +0.11 +0.03 +0.06   A2V s              -0.006-0.023 +.003+001SB         1.3   3.5AB   4*
7782          BD+36 4008 193621 69841                       201603.7+364903201948.3+370757 75.31  0.48 6.57   0.00 -0.07         A0III              +0.010-0.011      -019                          *
7783          BD+66 1281 193664 18796                       201632.3+663156201731.3+665114100.45 16.92 5.93  +0.58 +0.06         G3V                +0.466+0.300 +.067-005V
7784          BD+38 4021 193702 69856       13728           201637.7+390516202015.2+392412 77.23  1.69 6.23  +0.06 +0.03 +0.03   A1V                -0.002-0.022D+.003-001V     127  1.1   0.3AB   3*
7785          CP-81  901 1937212588613994  W                201639.3-811737203317.5-805754312.42-30.69 5.77  +1.14 +0.85         G6-8II             -0.004-0.019      +009V          5.7  26.8
7786          BD+46 2910 193722 49482              V1584 Cyg201645.3+463116201956.1+465015 83.35  5.93 6.50  -0.06 -0.37         B8/9IIIpSi         +0.001+0.010      -011      250                 *
7787   Kap2SgrCD-4214847 193807230184      W                201705.3-424438202353.2-422522358.34-34.44 5.64  +0.20 +0.10 +0.02   A5V                 0.000+0.022 +.016+003V?         0.9   0.6AB   4*
7788          BD-10 5369 1938961443613632                   201734.3-095827202300.8-093917 34.55-24.70 6.30  +0.91 +0.58         G5IIIa             +0.024-0.018      -017V?
7789 25    VulBD+23 3986 193911 88580                       201745.4+240737202203.4+242646 65.09-07.09 5.54  -0.06 -0.40         B8IIIne             0.000-0.007 -.008-013V     242                 *
7790   Alp PavCP-57 9674 193924246574 764  W       13060    201744.3-570320202538.9-564406340.91-35.19 1.94  -0.20 -0.71 -0.16   B2IV               +0.007-0.089      +002SBO    39  7.1 245.4AB   3*
7791          BD+53 2384 193944 32491    I  13743           201751.2+531641202030.4+533546 89.08  9.60 6.18  +1.56               K4III              -0.016+0.014      -004           6.2   6.5AC   4
7792 71    DraBD+61 2000 193964 18807              DE Dra   201756.6+615623201936.7+621527 96.48 14.36 5.72  -0.05 -0.21         B9V                +0.010+0.026      -019SBO     0                 *
7793          BD+14 4275 194012106042                       201812.4+141351202252.3+143305 56.82-12.71 6.17  +0.51 -0.07 +0.26   F8V                +0.075-0.001 +.034+002V?      6
7794          BD+04 4434 1940131257471531                   201813.4+050124202310.7+052035 48.75-17.63 5.31  +0.97 +0.77 +0.50   G8III-IV           -0.026-0.039 +.002-012     < 19:
7795          BD+40 4136 194069 49524                       201830.6+404845202203.0+410753 78.85  2.39 6.39  +1.07 +0.82         G5III+A            -0.004-0.031      -004                          *
7796 37Gam CygBD+39 4159 194093 49528 765I  13765  13048    201838.3+395611202213.7+401524 78.15  1.87 2.20  +0.68 +0.53 +0.34   F8Ib               +0.004 0.000 +.003-008V      20  8.5 141.2AxBC 4*
7797          BD+30 4005 194097 69917    I                  201835.9+305643202237.4+311554 70.81-03.33 6.09  +1.35               K2                 +0.005-0.027      +012
7798          BD+45 3152 194152 49531              13050    201849.5+452825202205.4+454742 82.70  5.04 5.58  +1.08 +1.02         K0III             v+0.028+0.044      -022SB
7799          CD-4114024 194184230197                       201907.1-410705202547.9-404747  0.35-34.58 6.09  +1.36 +1.61         K3III              -0.102-0.093      +042SB1O                      *
7800          BD+40 4141 194193 49546    I                  201912.3+404222202245.3+410134 78.84  2.22 5.93  +1.60 +1.98         K7III              +0.001-0.047      +001V?
7801          CD-2917049 1942151892641532  D                201919.5-285916202526.8-283948 14.55-32.01 5.85  +1.10 +0.94         G8II/III           +0.014+0.007 +.038-007SBO                       *
7802          BD+42 3721 194220 49550       13786  13053    201928.7+423938202255.5+425900 80.46  3.32 6.20  +0.95 +0.76         K0III             v+0.053+0.036D+.014-020V          1.8  93.2AC   4*
7803          BD+00 4495 194244125769       13811  13064    201931.7+004441202437.5+010407 45.03-20.09 6.15  -0.04 -0.14         B9V                +0.003+0.007      -009V?    265: 4.7  31.3AC   3*
7804          BD+68 1121 194258 188173631I         AC Dra   201939.4+683338202006.0+685249102.43 17.72 5.55  +1.59               M4.5-5III          +0.017+0.025      -043V                         *
7805          BD+63 1618 194298 18826    I  13769           201946.3+633934202111.4+635849 98.10 15.11 5.69  +1.56               K5III              -0.004+0.023      +030V?         6.9   4.3      *
7806 39    CygBD+31 4062 194317 699503633I                  201951.9+315202202351.7+321124 71.72-03.02 4.43  +1.33 +1.50 +0.67   K2.5IIIFe-0.5      +0.049-0.005 +.014-015V    < 19:                *
7807          BD+37 3916 194335 69951              Var?     201959.9+370912202344.4+372835 76.04  0.04 5.90  -0.20 -0.88         B2Ven              +0.004-0.004      -030SB                        *
7808          CD-3713741 194433212126      W                202024.3-374336202653.0-372411  4.46-34.26 6.25  +0.97 +0.77         K2IV-V             -0.242-0.116 +.042+023           1.4   1.1      *
7809          BD-03 4888 194454144412                       202029.5-030729202542.5-024801 41.54-22.19 6.11  +1.19 +1.19        gK1                 -0.007-0.030      +024V?
7810          BD+09 4526 194526125797    I         13076    202055.4+094352202544.0+100323 53.30-15.71 6.33  +1.56 +1.92 +0.66E  K5III             v+0.002-0.030      -077
7811          BD+20 4559 194577 886643634          Var      202115.2+210501202540.5+212435 63.02-09.48 5.66  +0.93               G6III              +0.010-0.011      -022                          *
7812          CP-81  906 194612258864                       202124.9-813737203818.6-811720311.99-30.75 5.91  +1.71 +2.02         K5III              -0.005-0.030      +000
7813          BD+19 4408 194616106100                       202132.3+193217202601.2+195154 61.77-10.41 6.41  +1.02               K0III              +0.029+0.002      -030
7814 10Pi  CapBD-18 5685 194636163592       13860           202135.8-183223202719.2-181242 26.22-29.08 5.25  -0.07 -0.41         B8II-III           +0.015-0.011      -013V?     93  3.4   3.3AB   3*
7815          BD+53 2397 194668 32534                       202150.7+531333202432.4+533307 89.38  9.08 6.51  +0.01 -0.13         B9.5III            +0.010+0.015      -019
7816          BD+16 4259 1946881061013635                   202149.0+165918202623.2+171856 59.66-11.91 6.22  +1.01               K0                 +0.008-0.006      -017
7817          CD-3614166 1947832121603637                   202221.8-355532202846.7-353545  6.69-34.31 6.10  -0.11               B8II/III          v+0.008-0.021      -009                          *
7818          BD+59 2228 194882 32562       13850           202300.6+591623202505.1+593600 94.55 12.38 6.44  +0.09 +0.12         A2IV s             +0.010-0.007D+.005-022SB         0.3   0.2      *
7819          BD-16 5609 194918163612                       202305.5-160421202843.6-154430 28.97-28.46 6.41  +0.99 +0.69         K0III              +0.010-0.008      -018V?
7820          BD+07 4477 1949371258413638                   202315.5+080623202807.5+082615 52.19-17.07 6.25  +1.08 +0.93         G9III              +0.043+0.017      -011
7821 68    AqlBD-03 4906 194939144468      W                202310.7-034117202824.9-032128 41.36-23.05 6.13  -0.06 -0.18         B9V                +0.023-0.015      -009V          7.7   9.8
7822 11Rho CapBD-18 5689 194943163614       13887           202309.4-180839202851.6-174849 26.80-29.27 4.78  +0.38 +0.04 +0.19   F2IV               -0.014-0.021 +.047+018V      91  1.7 249.5AD   5*
7823          BD+33 3910 194951 70044                       202313.4+335958202707.7+341944 73.86-02.34 6.39  +0.48 +0.45         F1II               -0.006-0.003      -014       23
7824          BD+02 4175 194953125843                       202314.3+023622202816.8+025613 47.24-19.95 6.21  +0.90 +0.57         G8III              +0.049-0.002      -022
7825          BD-22 5442 1950061893453639I                  202339.3-224323202931.3-222330 21.90-31.01 6.16  +1.55 +1.99         M1III              +0.011-0.031      +056
7826 40    CygBD+37 3941 195050 70056                       202351.9+380642202734.3+382625 77.26-00.02 5.62  +0.06 +0.06         A3V                -0.023-0.070 +.021+000V     133
7827          BD+56 2421 195066 32590       13870           202358.5+561832202623.5+563820 92.13 10.59 6.36   0.00  0.00         B9V                +0.014+0.010      -021SB    165  2.0  26.6      *
7828 43    CygBD+48 3128 195068 49643                       202358.9+490306202702.3+492300 86.14  6.40 5.69  +0.26 +0.04        dF0                 +0.071+0.060 +.012-020V?                        *
7829 12Omi CapBD-19 5830 195093163625       13902B          202408.6-185502202952.5-183512 26.07-29.78 6.74  +0.22 +0.04         A7V                +0.020-0.089      -016      144  0.8  21.9      *
7830 12Omi CapBD-19 5831 195094163626       13902A          202409.9-185451202953.9-183500 26.08-29.78 5.94  +0.08 +0.30         A3Vn               +0.026-0.082      -012      301  0.8  21.9      *
7831 69    AqlBD-03 4918 1951351444951533I                  202425.4-031305202939.0-025308 41.97-23.10 4.91  +1.15 +1.22 +0.56   K2III              +0.073-0.021 +.021-023V
7832          CD-2917122 195206189374                       202449.4-292651203056.8-290645 14.42-33.29 6.39  +0.21               A5V                +0.015+0.003      -020
7833          BD+19 4423 195217106156                       202452.6+194516202921.1+200516 62.41-10.94 6.55  +0.23 +0.10 +0.09   A3m                -0.025-0.002      +004
7834 41    CygBD+29 4057 195295 700951534I         13111    202518.5+300205202923.7+302207 70.91-05.04 4.01  +0.40 +0.27 +0.23   F5II               +0.007 0.000 +.014-018V?     13
7835 42    CygBD+35 4141 195324 700961535                   202531.5+360715202920.4+362717 75.85-01.47 5.88  +0.52 +0.10         A1Ib               +0.004-0.001      -018
7836  1    DelBD+10 4303 195325106172       13920  13114    202530.6+103339203018.0+105345 54.66-16.21 6.08  -0.03 -0.11         A1eShell           +0.021+0.003 -.000-016V?    320  1.8   0.9AB   3*
7837          BD-15 5696 195330163645      W                202528.2-152326203104.3-150323 29.94-28.71 6.12  +0.79 +0.42        gG5+F               -0.041-0.056      +030V?         0.0   0.1      *
7838          CP-70 2792 195402254820                       202556.6-695701203551.7-693640325.19-34.32 6.11  +1.29 +1.41         K2III              +0.029-0.061      -005
7839          BD+20 4602 195479 88783      W                202629.7+201605203058.1+203621 63.06-10.95 6.18  +0.15 +0.14 +0.04   A1m                +0.093+0.045      -040       15  4.0  11.8AB   3
7840          BD+10 4307 195483106196       13946           202626.5+105527203113.0+111538 55.10-16.21 7.11  +0.03 -0.39         B8V                 0.000-0.003      -011       91  0.3  16.7AxBC 4*
7841          BD+45 3196 195506 49704                       202641.2+453519202959.9+455543 83.60  3.99 6.41  +1.13 +1.07         K2III              +0.071+0.154 +.007-031
7842          CD-2514854 195549189416                       202655.1-251653203252.4-245638 19.33-32.54 6.36  -0.01               A0                 +0.010-0.041      -011
7843          BD+55 2411 195554 326273640  W       13115    202657.2+554357202927.1+560405 91.89  9.92 5.91  -0.04 -0.24         B9Vne              +0.002+0.005      -022      250  2.5   0.8      *
7844 45Ome1CygBD+48 3142 195556 497123641   13932           202657.6+483655203003.5+485706 86.07  5.74 4.95  -0.09 -0.63 -0.11   B2.5IV             +0.013+0.008      -022SB?   184  7.0  56.3AC   3*
7845          BD-10 5423 1955641636651536   13960           202655.4-101141203223.7-095112 35.45-26.86 5.65  +0.69 +0.21         G2.5IV             +0.308+0.104 +.033+009V?         4.2 103.2AC   3*
7846   Nu  MicCD-4414020 1955692302763643                   202702.8-445118203355.1-443058356.00-36.45 5.11  +1.01 +0.80         K0III              +0.011-0.043 +.007+009
7847 44    CygBD+36 4105 195593 70135       13949           202711.2+363556203059.2+365609 76.44-01.45 6.19  +1.01 +0.74 +0.69   F5Iab              +0.003-0.004      -022           3.4   2.0AB   4*
7848   Phi1PavCP-61 6492 195627254823                       202718.0-605506203534.8-603454336.04-36.09 4.76  +0.28 +0.05         F1III              +0.068-0.182 +.026-020      121                 *
7849          BD+25 4272 195692 88808       13964           202741.4+252801203158.2+254816 67.51-08.15 6.34  +0.26 +0.08 +0.13   Am                 +0.032-0.033D+.009-018       50  1.8   0.4      *
7850  2The CepBD+62 1821 195725 18897 767                   202754.2+623929202934.9+625939 97.76 13.80 4.22  +0.20 +0.16 +0.09   A7III              +0.044-0.013 +.038-007SB2O   59                 *
7851 46Ome2CygBD+48 3154 195774 49741    I W       13128    202813.6+485258203118.8+491313 86.41  5.73 5.44  +1.55 +1.92         M2IIIab            +0.012-0.028      -064V?         4.7  56.8      *
7852  2Eps DelBD+10 4321 195810106230 768          13137    202826.1+105748203312.8+111812 55.42-16.59 4.03  -0.13 -0.47 -0.11   B6III              +0.013-0.022 +.025-019V      50                 *
7853          CD-3814108 195814212246                       202824.8-382554203455.5-380523  3.94-35.94 6.44  +0.26               A8III:             +0.018-0.005      +021
7854          BD+51 2882 195820 32649                       202829.7+515808203121.1+521835 88.94  7.53 6.18  +1.01               K0III              +0.036+0.068      -010
7855          BD-14 5781 1958381636863645                   202837.7-140353203411.7-134316 31.68-28.88 6.13  +0.54 -0.02         F6V                +0.069+0.070      -043
7856          CD-3018013 195843212249                       202837.4-304855203447.4-302825 13.08-34.44 6.40  -0.08               B8V                +0.018-0.020      +003V?
7857          BD+09 4579 195922125960                       202904.6+094303203353.6+100335 54.42-17.41 6.56  +0.08 +0.05         A2Vnn              -0.007+0.007      -013
7858  3Eta DelBD+12 4378 195943106248              13147    202913.2+124104203357.0+130138 57.02-15.80 5.38  +0.07 +0.05 +0.02   A3IV s             +0.072+0.026 +.010-025V      70
7859   Rho PavCP-61 6495 1959612548353647          Rho Pav  202912.4-615224203735.3-613148334.83-36.18 4.88  +0.43 +0.19         F5Del Del          +0.057-0.072 +.003+008    =< 49                 *
7860          BD+56 2444 195964 32667    I                  202919.8+562624203146.5+564648 92.67 10.06 6.14  +1.43              gK5                 -0.018-0.007      -015V
7861          BD+42 3778 195986 497723644                   202923.4+425101203252.3+431130 81.69  1.96 6.60  -0.11 -0.56         B4III              +0.007+0.001      -017SBO                       *
7862          BD+20 4629 196035 88846                       202942.5+203833203410.0+205907 63.81-11.35 6.48  -0.14 -0.66         B3IV               +0.015-0.002      +003
7863   Mu 1OctCP-76 1434 1960512578383650                   202942.0-763150204202.9-761050317.51-32.78 6.00  +0.44 +0.11 +0.25C  F4III-IV           +0.180-0.006      -036
7864   Mu 2OctCP-75 1644 196067257836      W                202949.8-754147204143.7-752102318.45-33.05 6.55  +0.62 +0.25         G5III              +0.141-0.150      -013           0.6  17.0      *
7865          BD-17 6027 196078163712                       202952.7-165210203532.2-163133 28.86-30.27 6.19  +0.20 +0.10         A7V                +0.083-0.021      -001       90
7866 47    CygBD+34 4079 196093 70203    I W                203000.7+345430203354.2+351503 75.43-02.93 4.61  +1.60 +0.78 +0.99   K2Ib+B3V           +0.002-0.006 +.002-004SBO =< 50  5.6 117.6AC   3*
7867          BD+41 3805 196134 49796                       203014.6+412554203348.4+414620 80.66  0.98 6.49  +1.00 +0.78         K0III-IV           -0.015-0.081      +001
7868          BD+72  957 196142  97931538                   203026.5+721134203000.7+723154106.17 18.88 6.27  +1.35 +1.54        gK4                 -0.003-0.022      -043V?
7869   Alp IndCD-4713477 196171230300 769  W                203032.1-473824203734.0-471729352.57-37.21 3.11  +1.00 +0.79 +0.36E  K0IIICNIII-IV      +0.053+0.066 +.046-001           8.8  66. AB   3
7870          BD+46 2977 196178 49804      W                203038.3+462102203354.9+464138 84.62  3.89 5.78  -0.16 -0.53         B9pSi              +0.011-0.001      -022       55  4.1 117.7
7871  4Zet DelBD+14 4353 196180106274              13163    203037.9+141945203518.5+144027 58.63-15.16 4.68  +0.11 +0.11 +0.05   A3V                +0.046+0.013 +.015-025V     119
7872          CP-63 4602 196317254844                       203118.9-631519203951.4-625428333.09-36.20 6.22  +1.10               K1III              +0.012-0.078      +018V
7873 70    AqlBD-03 4961 1963211446243648I                  203131.2-025347203643.6-023300 43.21-24.49 4.89  +1.60 +1.95         K5II               +0.013-0.011 +.016-010
7874 26    VulBD+25 4299 196362 88884                       203151.1+253208203608.3+255257 68.13-08.87 6.41  +0.21 +0.16         A5III              +0.019+0.011      -019SBO =< 15                 *
7875   Phi2PavCP-60 7419 196378254846                       203145.7-605300204002.6-603256335.98-36.63 5.12  +0.53 +0.01         F8V                +0.307-0.570 +.048-032        0
7876          BD+51 2895 196379 327093646                   203155.6+513032203450.4+515115 88.89  6.83 6.11  +0.41 +0.42         A9II               -0.003-0.005      -013       34
7877          CD-2514920 196385189503                       203155.0-252726203752.1-250632 19.54-33.67 6.36  +0.33               F0III              +0.064-0.001      -026       30
7878          BD-00 4056 196426144632                       203210.8-001503203718.3+000549 45.81-23.32 6.22  -0.08 -0.39         B8IIIp              0.000-0.018      -023        5
7879 73    DraBD+74  872 196502  9802 770          AF Dra   203249.8+743643203130.4+745717108.43 20.04 5.20  +0.07 +0.11         A0pSrCrEu         v+0.005-0.015 +.012+009SB1     6                 *
7880 27    VulBD+25 4302 196504 889033649                   203248.6+260650203704.7+262743 68.73-08.70 5.59  -0.07 -0.22         B9V                +0.017-0.012      -022SB    300
7881   Ups PavCP-67 3754 196519254854              13208    203246.7-670647204157.1-664539328.39-35.58 5.15  -0.06 -0.28         B9III              +0.009-0.025      +007V                         *
7882  6Bet DelBD+14 4369 196524106316    I  14073           203251.5+141450203733.0+143543 58.88-15.65 3.63  +0.44 +0.08 +0.24   F5IV               +0.113-0.033 +.028-023SB1O   54  1.0   0.6AB   5*
7883  5Iot DelBD+10 4339 196544106322                       203302.0+110143203749.1+112240 56.13-17.50 5.43  +0.06 +0.04 +0.02   A2V                +0.040-0.007 +.024-004SBO    48                 *
7884 71    AqlBD-01 4016 196574144649    I  14081  13187    203310.4-012717203820.3-010619 44.81-24.14 4.32  +0.95 +0.69 +0.46   G7.5IIIa           +0.015-0.022 +.010-006SB1O < 19: 7.5  32.2      *
7885 48    CygBD+31 4159 196606 70287      W                203327.8+311323203731.8+313421 72.92-05.73 6.32  -0.09 -0.40         B8IIIn             +0.010 0.000      -019V?     80  0.1 180.8AB   3*
7886          BD+17 4370 196610106329    I         EU Del   203321.0+175500203754.6+181609 62.05-13.63 6.25  +1.48 +1.00 +2.48   M6III              +0.019+0.099      -066V?                        *
7887          BD+31 4160 196629 70289      W                203328.9+311024203732.6+313119 72.89-05.76 6.49  +0.34 -0.03         F0V                -0.045-0.033      +001V     154  0.1 180.8AB   3*
7888          BD+37 4002 196642 70288                       203337.8+375850203723.6+381943 78.31-01.64 6.20  +0.99 +0.81         K0III              +0.010-0.043      -037
7889 14Tau CapBD-15 5743 196662163771       14099           203340.9-151820203916.4-145717 30.94-30.50 5.22  -0.12               B6III              +0.007-0.022D+.005-005SB    151  0.6   0.3AB   4*
7890          BD-02 5328 196712144666                       203401.1-024553203913.2-022446 43.67-24.97 6.22  -0.10 -0.42         B7IIIne           v+0.006+0.009      -014SB2   250                 *
7891 29    VulBD+20 4658 196724 889441539                   203403.3+205100203831.3+211204 64.59-12.05 4.82  -0.02 -0.08 -0.04   A0V                +0.071+0.005 +.002-018V      54
7892  8The DelBD+12 4411 196725106342    I                  203400.7+125750203843.9+131854 57.95-16.61 5.72  +1.53 +1.73 +0.75   K3Ib               -0.001-0.004      -014V?
7893          CD-3315119 1967372123331540                   203403.5-334708204019.8-332555  9.85-36.23 5.47  +1.12 +1.08         K1III              +0.024+0.036      +014V?
7894 28    VulBD+23 4084 196740 88945                       203410.5+234554203831.9+240658 67.00-10.35 5.04  -0.14 -0.53         B5IV               +0.010-0.002      -022V?    330
7895          BD+23 4085 196753 88946                       203412.6+231946203835.1+234050 66.65-10.61 5.91  +0.98               K0II-III+A3V       +0.015-0.007      +009SB  =< 50                 *
7896  7Kap DelBD+09 4600 196755126059 772   14101           203416.3+094402203907.8+100510 55.18-18.47 5.05  +0.72 +0.21         G2IV               +0.321+0.023 +.026-052V    < 19: 4.8 203.6AC   3*
7897  1    AqrBD-00 4064 1967581260623651I  14108           203417.4+000805203924.9+002911 46.47-23.58 5.16  +1.06 +0.97         K1III              +0.101-0.015 +.010-043V    < 19: 7.1  59.7AB   3*
7898          CD-2416193 196761189549                       203414.7-240822204011.7-234626 21.25-33.77 6.37  +0.72 +0.23 +0.26E  G8V                +0.491+0.464 +.070-045V
7899          BD+15 4220 196775106347       14106           203426.6+152912203905.0+155017 60.16-15.25 5.97  -0.14 -0.70?        B3V                +0.001-0.017      +002           5.6  22.5AC   5*
7900 15Ups CapBD-18 5738 196777163779 773I                  203421.5-182927204003.0-180819 27.57-31.87 5.10  +1.66 +1.99 +0.89E  M2III              -0.022-0.021 +.019-013                          *
7901 75    DraBD+80  659 196787  34083963  W                203431.5+810450202814.6+812522114.41 23.36 5.46  +1.02 +0.84         G9III              +0.033+0.015 +.012-006V?         1.3 197.7AC   3
7902          CD-2714959 196815189559                       203436.1-265953204036.0-263842 17.97-34.70 6.51  +0.57               F7V                +0.033-0.017      -006SB2
7903          BD+21 4305 196821 88959              13201    203444.1+212753203910.6+214902 65.20-11.82 6.08  -0.06 -0.08         A0III              +0.019+0.014      -037SB?    20
7904          BD+29 4121 196852 70323                       203452.5+295902203859.5+302004 72.12-06.72 5.68  +1.09 +0.99         K2III              -0.032-0.058 +.013+013
7905          BD-16 5663 196857163783                       203455.4-162848204032.5-160727 29.82-31.23 5.80  +1.00 +0.74        gG7                 -0.067+0.071      -004
7906  9Alp DelBD+15 4222 196867106357 774   14121  13207    203459.5+153333203938.3+155443 60.30-15.32 3.77  -0.06 -0.21 -0.04   B9IV               +0.066-0.002 +.008-003SB    162  7.5  43.4AC   6*
7907          BD+10 4351 196885106360      W                203504.2+105338203951.8+111459 56.31-17.99 6.42  +0.55 +0.03         F8IV:              +0.060+0.088      -028    =< 15  4.0 191.9
7908 74    DraBD+80  660 196925  3413      W                203515.0+804430202927.5+810529114.11 23.16 5.96  +0.92 +0.60         K0III+F8V          +0.064+0.222 +.022-014SB         3.2 209.0AB   3*
7909          CD-3216130 1969172123453652I                  203511.6-315705204123.7-313554 12.13-36.07 5.76  +1.53 +1.85         M0III              +0.113-0.065 +.023-097
7910          CD-2615192 196947189575                       203526.1-262115204124.1-260000 18.79-34.69 6.28  +1.22               K2III              -0.002-0.029      -026
7911          BD+40 4266 197018 49899       14126           203553.7+401333203933.3+403446 80.35-00.61 6.06  -0.16 -0.54         B6IIIpMn           +0.007 0.000D+.003-015V?         0.1   0.8AB   4*
7912          BD+45 3233 197036 49898                       203600.7+451848203923.1+454001 84.37  2.52 6.58  -0.06 -0.55         B5IV               +0.005+0.001      -015V
7913   Bet PavCP-66 3501 197051254862 775                   203557.0-663345204457.5-661211328.96-36.01 3.42  +0.16 +0.12         A7III              -0.046+0.012 +.035+010       86                 *
7914          BD+19 4484 197076106373      W                203613.8+193418204045.2+195607 63.85-13.22 6.45  +0.63 +0.09 +0.34   G5V                +0.122+0.304 +.050-037V?      5  5.1  93.7      *
7915          CD-4013994 197093212369                       203620.1-395457204253.0-393331  2.37-37.67 6.29  +1.08               K1III              +0.024-0.006      -007
7916          BD+55 2444 197101 32763                       203624.9+553909203900.2+560018 92.63  8.80 6.48R                     F2Vn               -0.005-0.035      -001      120                 *
7917          BD+29 4131 197120 88997       14149           203627.2+292657204036.2+294819 71.90-07.32 6.08R +0.14 +0.15         A2V                +0.008+0.036D+.010-030SB    110  4.5   0.9      *
7918 10    DelBD+14 4393 197121106384              13227    203635.2+141337204116.2+143459 59.40-16.40 5.99  +1.24              gK4                 -0.006+0.002      -032
7919          BD+42 3818 197139 49912                       203633.4+430621204003.1+432731 82.69  1.08 5.95  +1.19               K2III              -0.070-0.063      -019
7920   Eta IndCP-5211752 197157246709 776                   203641.9-521641204402.3-515516346.69-38.16 4.51  +0.27 +0.09         A7III-IV-A9       v+0.158-0.058 +.049-002      122
7921 49    CygBD+31 4181 197177 70362       14158           203659.6+315705204102.6+321826 73.96-05.88 5.51  +0.88               G8IIb              +0.007-0.014D+.004-029SB2        2.3   2.5AB   3*
7922          BD+38 4187 197226 703673653  W                203715.7+384334204100.4+390456 79.34-01.74 6.51  -0.12 -0.49         B6III              +0.007 0.000      -015SBO   100  3.3  48.3      *
7923          BD+17 4382 197249106396                       203722.6+170947204158.2+173117 62.00-14.85 6.22  +0.94 +0.57         G8III              +0.022+0.044      -002
7924 50Alp CygBD+44 3541 197345 49941 777I  14172  Alp Cyg  203801.3+445522204125.9+451649 84.28  2.00 1.25  +0.09 -0.24 +0.10   A2Ia              t+0.003+0.002 -.006-005SBO    21 10.0  75.4      *
7925          BD+59 2272 197373 189793654                   203810.4+600837204017.9+603019 96.41 11.33 6.01  +0.46 -0.06         F6IV               +0.015+0.190 +.021-013       30
7926          BD+41 3856 197392 499463655                   203819.7+412131204156.5+414301 81.53-00.26 5.67  -0.12 -0.45         B8II-III           +0.006 0.000      -027SB     25
7927          BD+34 4127 197419 70406              V568 Cyg 203827.2+350552204222.2+352722 76.64-04.18 6.66  -0.16 -0.68         B2IV-Ve            -0.019-0.009      -007V     115                 *
7928 11Del DelBD+14 4403 197461106425 778          Del Del  203847.4+144257204327.5+150428 60.14-16.55 4.43  +0.32 +0.10 +0.17   A7IIIpDel Del      -0.019-0.043 +.013+009SB     41                 *
7929 51    CygBD+49 3353 197511 32809       14189           203907.6+495851204212.6+502024 88.37  4.99 5.39  -0.10 -0.63         B2V                +0.003+0.005      -003V?     40  6.0   3.0AB   5*
7930          BD+83  588 197508  3418                       203905.2+831645202903.1+833732116.56 24.35 6.19  +0.13?              A3/4Vm             +0.025-0.017      +010SB     38
7931          CD-2715014 197540189641                       203913.0-273634204513.2-271450 17.60-35.85 6.50  +0.94               G8III              +0.008+0.006      -003V
7932          BD+35 4234 197572 70423    I         X Cyg    203929.2+351338204324.2+353516 76.87-04.26 6.47  +1.23 +0.99 +0.64   F7Ib-G8Ib         v-0.010-0.003 -.008+010SB     22                 *
7933          CD-3913960 1976302124163658                   203949.1-393344204620.1-391157  2.91-38.29 5.50  -0.10               B8/9V              +0.049-0.030      -030
7934   Sig PavCP-69 3138 1976352548713660                   203950.3-690830204918.1-684635325.77-35.70 5.41  +1.12 +1.10 +0.32E  K0III              -0.069-0.049 -.000+019                          *
7935          CD-3614396 197649212417                       203956.5-362857204618.6-360713  6.79-37.90 6.49  +0.39               F3IV               +0.044-0.055      -014SBO                       *
7936 16Psi CapCD-2515018 197692189664 779                   204010.5-253749204605.7-251615 20.01-35.50 4.14  +0.43 +0.02 +0.20   F4V                -0.050-0.157 +.098+026V      37
7937 17    CapBD-22 5523 1977251896673659                   204022.2-215239204610.0-213051 24.38-34.37 5.93  +0.04 +0.10         A0                 +0.024-0.021      +018
7938          BD+60 2154 197734 18998                       204031.6+601427204239.7+603605 96.66 11.16 6.15  +0.01 +0.05         A2IV               -0.003-0.014      -007       55
7939 30    VulBD+24 4229 197752 890843657I                  204032.8+245446204452.5+251614 68.82-10.81 4.91  +1.18 +1.18 +0.58   K2III              -0.028-0.182 +.006+031SB   < 17                 *
7940          BD+56 2477 197770 32832                       204042.4+564509204313.5+570651 93.88  9.00 6.32  +0.33 -0.48         B2III              -0.013+0.003      -015SB
7941          BD+17 4401 197812106458    I         U Del    204053.4+174335204528.2+180525 62.98-15.20 6.38  +1.68 +1.24         M5II-III           +0.002+0.005      -021                          *
7942 52    CygBD+30 4167 197912 70467    I  14259           204132.0+302116204539.7+304311 73.30-07.63 4.22  +1.05 +0.89 +0.53   G9.5III            -0.010+0.028 +.015-001     < 17  5.0   6.1      *
7943   Iot MicCD-4414145 1979372303791542  W                204142.4-442111204829.2-435919356.83-39.02 5.11  +0.35 +0.06         F1IV               +0.185-0.106 +.051-015V?     84 10.4   4.3
7944          BD+55 2462 197939 32849    I                  204146.3+560729204422.0+562917 93.47  8.50 5.78  +1.67 +1.86         M3III              +0.006-0.015      -028V?
7945  4    CepBD+66 1318 197950 190043656                   204156.1+661738204311.0+663927101.72 14.68 5.58  +0.22 +0.05         A8V                +0.021+0.032 +.034-011      159
7946          BD-03 5018 197954144802    I                  204151.6-025109204703.6-022912 44.66-26.72 6.27  +1.55 +1.97         K2                 -0.016-0.016      +028V
7947 12Gam1DelBD+15 4255 197963106475       14279B          204200.3+154549204638.7+160728 61.49-16.57 5.14  +0.49 +0.08         F7V                -0.030-0.191 +.026-007V?  =< 25  1.0   9.8      *
7948 12Gam2DelBD+15 4255 1979641064761541I  14279A          204201.1+154550204639.5+160727 61.50-16.58 4.27  +1.04 +0.97 +0.48   K1IV               -0.032-0.197 +.026-007V?      0  1.0   9.8      *
7949 53Eps CygBD+33 4018 197989 70474 780I  14274           204209.8+333544204612.7+335813 75.94-05.71 2.46  +1.03 +0.87 +0.54   K0-III             +0.356+0.328 +.057-011SB?  < 17  9.3  54.9AB   3*
7950  2Eps AqrBD-10 5506 198001144810 781                   204215.8-095143204740.6-092945 37.68-30.10 3.77   0.00 +0.02  0.00   A1V                +0.035-0.034 +.021-016V?     98                 *
7951  3    AqrBD-05 5378 1980261448141543I         EN Aqr   204227.7-052338204744.2-050140 42.24-28.09 4.42  +1.65 +1.92 +1.31   M3III              +0.003-0.040 +.008-022V?                        *
7952   Zet IndCD-4613718 1980482303913661                   204236.0-463548204929.0-461337353.95-39.24 4.89  +1.52 +1.87 +0.64E  K5III              +0.040+0.024 +.012-005
7953 13    DelBD+05 4613 198069126222       14293  13299    204251.1+053827204748.3+060030 52.78-22.50 5.58  -0.02 -0.09         A0V                +0.010-0.006 +.002-007      202  3.6   1.4      *
7954          BD+02 4250 198070126221                       204245.7+025619204747.8+031824 50.29-23.93 6.40  -0.02 -0.09         A0Vn               +0.031+0.021      -021      142
7955          BD+57 2240 198084 32862 782  W                204252.1+571315204521.1+573447 94.43  9.06 4.51  +0.54 +0.10 +0.28   F8IV-V             -0.064-0.236 +.046-031SB2     0  5.7  66.4      *
7956          BD+33 4028 198134 70499    I  14290  T Cyg    204311.2+340023204710.8+342227 76.39-05.62 4.92  +1.32 +1.47         K3III              +0.047+0.004 +.005-023V?   < 19: 6.5   9.9AB   3*
7957  3Eta CepBD+61 2050 198149 19019 783I  14276           204315.3+612701204517.4+615020 97.86 11.64 3.43  +0.92 +0.62 +0.49   K0IV               +0.087+0.818 +.071-087     < 17  8.2  51.7      *
7958          BD+45 3270 198151 50050      W                204316.7+460954204638.5+463154 85.83  2.06 6.30  +0.03 +0.04         A3V                -0.024-0.013      -006V?    125  1.9   0.5
7959          CP-62 6180 198160254883      W     A          204317.6-624759205138.3-622545333.32-37.62 6.28  +0.15 +0.08         A2-3IV-V           +0.076-0.047D+.009-016           0.2   2.4      *
7960          CP-62 6180 198161254884      W     B          204318.1-624759205138.8-622545333.32-37.62 6.59  +0.17 +0.06         A2-3IV-V           +0.078-0.043D+.009-010           0.2   2.4      *
7961          CD-2615282 198174189733                       204321.4-260901204917.6-254653 19.65-36.34 5.86  -0.07 -0.46         B7IIIp             +0.015-0.021      -012
7962          BD+52 2799 198181 328771544                   204327.2+523752204621.2+525943 90.87  6.12 6.33  +1.12               K0                 -0.081-0.103 +.008-029
7963 54Lam CygBD+35 4267 198183 70505       14296           204330.7+360723204724.5+362927 78.08-04.34 4.53  -0.11 -0.49 -0.11   B5Ve               +0.009-0.005 +.008-023SB?   155  1.3   0.9AB   3*
7964          BD-18 5783 1982081639103663I W                204340.2-182418204920.5-180209 28.63-33.89 6.21  +1.42               K3III              +0.002-0.030      +044V?         6.5  31.6
7965   Alp MicCD-3414660 198232212472      W       13329    204343.4-340859204958.1-334647  9.88-38.26 4.90  +1.00 +0.73         G7III              +0.010-0.025 +.012-015           5.0  20.5
7966          BD+45 3275 198237 50060    I  14298           204354.9+451244204720.8+453447 85.16  1.37 6.40  +1.61 +2.02         K3III              +0.005-0.025      -006           6.3  12.9
7967          BD+69 1127 198236 19018                       204356.7+692313204433.1+694507104.41 16.37 6.41R                     G8III              -0.026-0.027 +.002-009
7968   Iot IndCP-5211782 198308246762                       204416.4-515850205130.1-513630346.98-39.34 5.05  +1.13 +1.05         K1II-III           +0.007-0.014 +.001+021
7969          BD+47 3188 198345 500733662I  14318           204431.7+472747204749.3+474955 86.97  2.72 5.57  +1.46 +1.78         K5III              -0.006-0.015      -030SB         9.0  18.3
7970          CD-3216236 198356212487    I                  204437.1-322530205047.1-320316 12.08-38.12 6.36  +1.54 +1.81         K5III              -0.016-0.061      -004
7971          CD-3814250 198357212488                       204435.5-381707205100.7-375448  4.68-39.06 5.52  +1.38 +1.63?        K3II               -0.011-0.022      +017SB
7972          BD+51 2954 198387 32897       14322           204453.6+520232204752.8+522426 90.55  5.58 6.27  +0.89 +0.55         G7IV               +0.068-0.158 +.018-041V          5.2  12.2      *
7973 15    DelBD+12 4472 198390106536      W                204451.8+121016204937.8+123243 58.86-19.23 5.98  +0.43 -0.11         F5V                +0.057+0.098 +.030+002    =< 10  5.2 108.0AC   4
7974 14    DelBD+07 4556 1983911262653664          13337    204454.1+072933204948.3+075151 54.75-21.90 6.33  +0.02 -0.02         A1V s              +0.032+0.011      -030SBO    32                 *
7975          BD+05 4626 198404126267      W                204500.8+051022204959.1+053241 52.67-23.20 6.21  +0.98 +0.79         K0                 +0.040+0.006      -022           2.5  79.9
7976          BD-13 5773 1984311639243665                   204511.1-125455205041.8-123242 34.83-32.07 5.88  +1.07 +0.94        gK1                 +0.126-0.070 +.021-044SB
7977 55    CygBD+45 3291 198478 50099       14337  V1661 Cyg204531.8+454435204856.3+460651 85.75  1.49 4.84  +0.41 -0.45 +0.31   B3Ia              e+0.004 0.000 +.017-007V      35  6.3  21.3      *
7978          BD+51 2957 198513 32908       14336           204541.5+513221204842.7+515438 90.24  5.17 6.29  -0.07 -0.56         B8Vnp              +0.006+0.007      -025V?    100  2.3   4.0      *
7979   Bet MicCD-3315245 198529212499                       204546.1-333307205158.8-331038 10.73-38.57 6.04  +0.03               A2Vn               +0.015+0.014      -012V?
7980 18Ome CapCD-2715082 1985421897811546I         13351    204551.3-271736205149.3-265509 18.46-37.18 4.11  +1.64 +1.93 +0.94   M0-III-IIIbBa0.2   -0.006-0.001 +.001+009V                         *
7981          BD+17 4431 198552106562                       204601.0+174040205037.0+180305 63.70-16.21 6.52R +0.04  0.00         A1V s              +0.076+0.016      +013       53
7982  4    AqrBD-06 5604 198571144877       14360  13350    204607.6-060002205125.7-053735 42.13-29.18 5.99  +0.46 +0.03         F5V+F7V            +0.099-0.001 +.028-025           0.9   1.1AB   4*
7983          BD+46 3067 198625 50119       14350           204631.7+461718204954.7+463940 86.28  1.71 6.33  -0.07 -0.57         B4Ve               +0.008 0.000      -015           5.3  19.2      *
7984 56    CygBD+43 3739 198639 501213666  W                204631.7+434057205004.9+440334 84.29  0.03 5.04  +0.20 +0.12 +0.09   A4mDel Del         +0.126+0.133 +.025-021V?     90  6.0  75.5      *
7985  5    AqrBD-06 5606 198667144889              13360    204651.4-055256205208.7-053025 42.35-29.28 5.55  -0.08 -0.27         B9III              +0.002 0.000 -.009-002V      50
7986   Bet IndCP-58 7788 198700246784 785  W                204659.8-584953205448.6-582715338.16-38.85 3.65  +1.25 +1.23         K1II               +0.017-0.026 -.005-005V?         8.8  17.3
7987          CD-4014078 198716212522                       204709.8-401103205340.2-394836  2.30-39.76 5.35  +1.32               K2III              +0.044-0.102 +.020+020
7988          BD+27 3890 198726 89216    I         T Vul    204713.4+275232205128.2+281502 72.13-10.15 5.77  +0.72 +0.40 +0.38   F5Ib               +0.003-0.003 +.006-001SB                        *
7989          CD-2416328 198732189801       14380           204709.3-240928205301.2-234659 22.33-36.58 6.33  +0.88 +0.56         K0III              +0.099-0.053      -040           2.1   1.8      *
7990  6Mu  AqrBD-09 5598 1987431448951547  S                204715.6-092131205239.2-085900 38.84-30.98 4.73  +0.32 +0.11 +0.17   A3m                +0.044-0.030 +.019-009SBO    46                 *
7991          CD-3117917 198751212521                       204718.4-310543205325.0-304307 13.89-38.40 6.35  +1.09               K1III              +0.036-0.008      -007V?
7992          CD-5112748 198766246786                       204726.7-510618205435.0-504340348.07-39.89 6.24  -0.12               B5IV               +0.002-0.036      -004                          *
7993          BD+63 1663 198781 190513667                   204732.0+634009204917.4+640232 99.94 12.61 6.45  +0.07 -0.77         B0.5V              -0.006-0.004      -027
7994          BD-12 5854 198802163953      S                204737.4-115706205305.6-113425 36.15-32.20 6.38  +0.67 +0.14         G1V                +0.050+0.046      -001V                0.2
7995 31    VulBD+26 4017 198809 89228    I         13373    204750.8+264321205207.7+270549 71.31-10.98 4.59  +0.83 +0.47 +0.46   G7IIIFe-1          -0.067-0.065 +.036+001SB   < 19:
7996          BD+32 3974 198820 70596                       204756.1+322823205200.3+325057 75.83-07.36 6.44  -0.15 -0.62         B3III              -0.009+0.006      -018V?
7997          CD-2816975 198853189815    I                  204806.8-281811205406.8-275532 17.39-37.93 6.41  +1.60 +1.90         M4III              +0.027-0.025      +025V?
7998          BD-07 5433 198949144915                       204838.6-071603205358.4-065323 41.19-30.33 6.44  +0.37  0.00         F1IV               +0.033-0.027      -008       67
7999          BD+29 4221 198976 89241                       204855.4+291622205307.4+293858 73.47-09.56 6.34  +1.09               K2                 -0.001-0.047      -010
8000 19    CapBD-18 5805 199012163975    I                  204908.8-181808205447.9-175523 29.33-35.06 5.78  +1.12 +1.00         K0III              -0.051-0.020      -039V
8001 57    CygBD+43 3755 199081 50180              13388    204942.5+440031205314.8+442314 84.90-00.19 4.78  -0.14 -0.58 -0.13   B5V                +0.010+0.001      -020SB2O   69                 *
8002 76    DraBD+81  718 199095  3458 915          13319    204950.5+820940204235.2+823152115.73 23.46 5.75   0.00 -0.04         A0V                +0.029+0.023      -020SB     27
8003          BD+44 3617 199098 50182              13391    204948.9+444810205318.6+451055 85.52  0.31 5.45  +1.10 +0.93         K0II               +0.014+0.003 -.005-024SB
8004          BD+41 3922 199099 50183                       204947.7+420153205326.4+422437 83.41-01.49 6.66  -0.04 +0.01         A1V                +0.011+0.002      -007       98
8005          BD+32 3980 199101 706453668I W                204950.7+330327205353.9+332616 76.54-07.29 5.47  +1.52              gK5                 -0.021+0.038      -010V          4.0 187.7
8006          BD-01 4075 199124144941              EM Aqr   204958.1-014516205508.1-012224 46.88-27.92 6.55  +0.29 +0.06         A9Vn               -0.002+0.020      -009      173                 *
8007          BD+27 3909 199140 89265              BW Vul   205007.6+280831205422.4+283119 72.75-10.48 6.56  -0.13 -0.90         B2IIIe            v+0.007-0.010      -006V      26                 *
8008 32    VulBD+27 3911 199169 89272 786I         13398    205017.8+274038205433.6+280327 72.41-10.80 5.01  +1.48 +1.79         K4III               0.000-0.002 +.003+008V?   < 17
8009          BD+40 4354 199218 50209       14413           205038.1+401920205422.3+404211 82.21-02.72 6.70  -0.07 -0.41         B8Vnne             +0.010+0.003      -022V?    350  3.9   5.8      *
8010          BD+03 4461 199223126373       14430           205040.2+040902205540.7+043158 52.58-24.94 6.05  +0.82 +0.49         G6III-IV           +0.070+0.010 -.012-031V          1.4   2.1      *
8011 17    DelBD+13 4572 1992531066653669I         13408    205052.6+132024205536.7+134317 60.77-19.74 5.17  +1.12 +0.96         K0III              +0.021-0.014 +.017-010     < 19:
8012 16    DelBD+12 4501 199254106666       14429           205052.2+121110205538.6+123407 59.78-20.42 5.58R +0.12 +0.07         A4V                +0.040+0.018      +002SB    135  6.5  36.0      *
8013          CD-2615344 199260189856              13412    205050.8-264039205647.3-261747 19.59-38.09 5.70  +0.50 -0.04         F7V                +0.093-0.065 +.039-016
8014          BD-04 5307 199280144957                       205104.3-035640205618.3-033341 44.88-29.25 6.57  -0.10 -0.29         B8Vn               +0.027+0.018      -029V?
8015  7    AqrBD-10 5553 199345144968    I  14449  13419    205129.8-100451205654.0-094151 38.63-32.24 5.51  +1.47 +1.71        gK5                 -0.003-0.010      -033V?         3.5 176.6AC   3*
8016          BD+80  672 199437  3467    I         13369    205207.9+801038204733.4+803308114.01 22.27 5.39  +1.12 +1.06         K1III              -0.031-0.037 +.011-026
8017          BD-00 4132 199442126396       14457           205203.5+000452205710.6+002749 48.95-27.42 6.05  +1.22 +1.36         K2III              +0.018-0.065      -026V?         5.4  26.2AB   3*
8018          BD-16 5741 1994431640131548                   205204.8-162459205740.6-160154 31.78-35.00 5.87  +0.18 +0.12 +0.06   A2m                +0.053+0.001      -001V      76                 *
8019          CP-68 3398 199475254918                       205218.9-683553210128.1-681235325.99-36.94 6.37  +0.10 +0.11         A2V                -0.010-0.002      +009
8020          BD+46 3111 199478 50246                       205227.1+470203205549.8+472504 87.51  1.42 5.67  +0.47 -0.33 +0.39   B8Ia              t-0.003-0.002      -015V                         *
8021   Alp OctCP-77 1474 199532257879 787                   205236.4-772418210443.0-770126315.96-33.65 5.15  +0.49 +0.13         A7III+G2III        +0.006-0.369 +.025+045SB2O   57                 *
8022          BD+50 3232 199578 33001                       205304.2+504126205612.9+510430 90.34  3.73 6.63  -0.10 -0.50         B5V                +0.017+0.004      -018
8023          BD+44 3639 199579 50263    I                  205303.1+443224205634.7+445530 85.70-00.30 5.96  +0.05 -0.85         O6V((f))           -0.005+0.008      -006SBO   170                 *
8024          BD-15 5848 199603164027              DV Aqr   205309.6-145210205841.9-142858 33.64-34.63 6.01  +0.23 +0.11         F0IV               -0.042-0.002      +010SBO                       *
8025          BD+50 3233 199611 330043670   14460           205315.0+502040205625.5+504343 90.10  3.49 5.81  +0.30 +0.04         F0III              +0.041-0.015      -015SB    109  7.7   7.2      *
8026          BD+48 3249 199612 50262                       205309.5+484840205625.9+491145 88.93  2.49 5.90  +1.04 +0.92         G8II-III           +0.009+0.007 +.004-015V
8027          CD-5112778 1996232468203673                   205314.7-513926210021.5-511555347.25-40.75 5.76  +0.47 -0.06         F5IV-V             -0.084+0.130      -023
8028 58Nu  CygBD+40 4364 199629 50274 788                   205326.6+404655205710.4+411002 82.91-02.82 3.94  +0.02  0.00  0.00   A1Vn               +0.012-0.016 +.010-028SB    241
8029          BD+56 2515 199661 330053671                   205336.5+563009205617.0+565315 94.82  7.47 6.23  -0.18 -0.70         B2.5IV             +0.004+0.002      -019                          *
8030 18    DelBD+10 4425 199665106712      W                205336.9+102712205825.9+105021 58.69-21.98 5.48  +0.93              gG6                 -0.051-0.036      +000V          4.6 198.2AB   3
8031          CD-3614530 1996842126083674                   205341.0-363059205959.7-360747  7.25-40.63 6.11  +0.39 -0.03         F5V                +0.106-0.061      +016
8032 33    VulBD+21 4424 199697 893321549I                  205348.0+215621205816.4+221933 68.36-15.04 5.31  +1.40       +0.74   K3.5III            -0.003-0.003      -028
8033 20    CapBD-19 5982 199728164043              AO Cap   205355.2-192522205936.1-190207 28.54-36.52 6.25  -0.13               B9pSi              +0.011-0.013      -010                          *
8034  1Eps EquBD+03 4473 199766126428       14499AB         205404.6+035436205904.4+041737 52.87-25.78 5.23  +0.46 +0.02 +0.28   F6IV               -0.109-0.148 +.021+018SB1O   54  0.3   1.0AB   4*
8035          BD+43 3777 199870 50298                       205444.6+440455205819.5+442818 85.55-00.83 5.55  +0.97 +0.83         K0IIIbFe-0.5       +0.106+0.072 +.016-023SBO                       *
8036          BD+41 3949 199892 50303                       205449.0+413307205830.8+415625 83.66-02.51 6.16  -0.08 -0.46         B7III              -0.001+0.013      -033SBO    25                 *
8037          BD+16 4425 199941106738      W                205511.6+162606205950.8+164927 64.05-18.71 6.66  +0.38 -0.13         F4III              +0.041-0.014      +002       40  2.7  70.7
8038          BD+06 4718 199942126447      W                205508.5+070734210003.9+073059 55.97-24.20 5.99  +0.26 +0.09         F1Vp               +0.029+0.020D+.009-020V     145  1.1   0.2      *
8039   Gam MicCD-3216353 1999512126361550I W                205509.5-323855210117.5-321528 12.34-40.34 4.67  +0.89 +0.54 +0.32E  G6III              -0.002+0.005 +.034+018           9.0  25.7      *
8040          BD+49 3426 199955 33034       14504           205517.8+500425205830.1+502744 90.11  3.06 5.61  -0.15 -0.51         B5Vn               +0.013+0.010 +.008-021V          0.9   1.8AB   3*
8041 11    AqrBD-05 5433 199960145022 789                   205517.9-050700210033.8-044349 44.30-30.75 6.21  +0.63 +0.24         G1V                +0.051-0.137 +.033-017V?  =< 10
8042          CD-4314325 200011230492    I'W                205535.5-432327210212.6-430007358.19-41.49 6.64  +0.68 +0.17         G3IV+K0IV          +0.060-0.103 +.018-031           0.3  57.5      *
8043          BD+75  764 200039  99113672                   205555.1+753219205444.3+755532110.14 19.35 6.05  +0.93               G5III              +0.029+0.043      -025
8044          BD+18 4675 2000441067473675I W       13454    205553.4+185626210027.7+191946 66.23-17.29 5.65  +1.61               M3IIIab            -0.016-0.063 +.010-015V?         3.9  46.6
8045          CD-2715197 200052189942                       205549.2-271619210145.3-265252 19.22-39.32 6.05  +0.06               A7V:m:             +0.015-0.023      -004                          *
8046          CD-3914079 200073212653                       205602.4-385508210227.2-383150  4.15-41.35 5.94  +1.11               K0IV               +0.163-0.158 +.018+039V
8047 59    CygBD+46 3133 200120 503351551   14526  V832 Cyg 205625.4+470750205949.6+473116 88.03  0.97 4.74  -0.05 -0.94 -0.01   B1ne              v+0.006+0.002      +001SB    374  4.4  20.1AB   4*
8048   Zet MicCD-3914089 200163212666 790                   205634.6-390119210258.0-383754  4.03-41.46 5.30  +0.41               F3V                -0.028-0.111 +.024+005V?     47
8049          BD+58 2201 200205 330483676I                  205657.4+590251205925.4+592619 97.06  8.79 5.51  +1.40 +1.64        gK4                 +0.044+0.015 +.002-017V
8050          CD-2817077 200245189966       14565           205712.4-280729210310.2-274355 18.25-39.83 6.25  +0.95 +0.74         K0III              +0.035-0.044      -030           0.6   0.1      *
8051          BD+35 4357 200253 707943677                   205713.9+353802210112.9+360134 79.51-06.77 5.97  +0.98               G5III              -0.005-0.005      -010SB
8052          CP-76 1473 200266257887                       205715.3-763640210847.9-761245316.70-34.23 6.58  +1.23 +1.18         K1III              -0.027-0.022      +037
8053 60    CygBD+45 3364 200310 50359       14549  V1931 Cyg205741.4+454547210110.9+460921 87.15-00.10 5.37  -0.21 -0.93         B1Ve              v+0.006+0.008      -012SB    320  4.5   2.6      *
8054          BD-01 4095 200340145050                       205750.3-011909210259.6-005529 48.45-29.38 6.50  -0.10 -0.49         B6V                +0.007-0.002      -012V
8055   Mu  IndCP-55 9509 2003652468543680                   205753.0-550721210514.2-544338342.58-40.96 5.16  +1.21 +1.22         K2III              +0.010-0.046 +.023+012
8056          BD+00 4648 200375126491       14573           205758.6+010819210303.0+013155 50.85-28.11 6.25  +0.48 +0.01         F5V                -0.116-0.053 +.035+007           0.5   1.3AB   3*
8057          BD+14 4518 200430106796    I                  205818.6+142006210301.8+144348 62.77-20.58 6.31  +1.67 +2.04         M1III              +0.021+0.005      -039
8058 12    AqrBD-06 5664 200496145064       14592           205847.3-061312210404.6-054924 43.66-32.04 7.31H                     A3V                +0.003+0.012D+.003-005           1.9   2.7     3*
8059 12    AqrBD-06 5664 200497145065       14592           205847.3-061309210404.7-054923 43.66-32.04 5.89H +0.68 +0.41         G4III              +0.018-0.009D+.003+001V          1.9   2.7     3*
8060 22Eta CapBD-20 6115 200499189986      W                205842.9-201501210424.3-195118 28.05-37.86 4.84  +0.17 +0.09 +0.07   A5V                -0.034-0.037 +.055+024V      71  1.3   0.3
8061          CP-73 2192 200525257890      W                205854.1-733352210922.3-731023320.01-35.61 5.68  +0.59 +0.10         F8-G0V             +0.432-0.331 +.048-011           0.0   0.1AB   3*
8062          BD+44 3679 200527 50381    I                  205849.7+442347210224.1+444728 86.27-01.17 6.19  +1.69 +1.84 +1.56   S4/1III            -0.014+0.003      +001                          *
8063          BD+38 4325 200577 70832                       205911.9+381543210304.8+383927 81.75-05.32 6.07  +1.01 +0.79         G8III              +0.016-0.008      -003V?
8064          BD+45 3374 200595 50390       14585           205917.4+452712210248.5+455056 87.11-00.52 6.48  -0.15 -0.55         B3Vn               -0.005-0.001 +.009-012SB         0.0   0.2AB   5*
8065          BD+56 2524 200614 33078       14575           205923.6+561629210209.1+564011 95.18  6.72 5.83  -0.07 -0.34         B8III              +0.016+0.004D+.004-021       60  0.6   1.6      *
8066  3    EquBD+04 4606 200644126518    I W                205935.8+050620210434.7+053010 54.83-26.26 5.61  +1.65 +2.02         K5III              +0.017-0.002 +.011-016           6.9  81.7      *
8067          BD+02 4297 200661126519                       205938.7+023240210441.7+025631 52.45-27.69 6.42  +1.05 +0.89         K0                 +0.013+0.001      -010
8068          BD+01 4418 2006631265223681                   205940.7+015227210445.4+021611 51.82-28.07 6.33  +0.97 +0.62         G5                 +0.098-0.065      -012
8069   Eta MicCD-4114379 2007022305233684  W                205955.2-414705210625.5-412310  0.37-42.25 5.53  +1.35 +1.52         K3III              +0.026-0.016      +011           2.8 132.9AC   3
8070   Del MicCD-3018382 2007182127093683                   205958.9-303120210601.2-300730 15.35-40.95 5.68  +1.03               K0/1III            +0.041-0.068      +022V?
8071          BD+41 3987 200723 50409      W                210007.4+411357210352.1+413741 84.08-03.47 6.33  +0.38 +0.13         F3IV               +0.001-0.054      -008V      75  2.3  57.3AB   3*
8072          BD+49 3448 200740 33091                       210010.3+495714210326.0+502107 90.55  2.40 6.37  +0.98 +0.76         K0                 +0.058+0.053      -022                          *
8073          CP-64 4094 200751254941                       210014.2-641948210832.9-635544330.81-39.04 5.76  +1.18 +1.15         K0III              +0.007-0.011      -020
8074          BD+46 3159 200753 50408                       210015.7+462804210343.3+465143 87.97  0.04 6.32  +0.25               F0IVn              -0.055-0.110      -015SB2   190
8075 23The CapBD-17 6174 2007611641321552                   210019.6-173749210556.8-171358 31.31-37.29 4.07  -0.01 +0.01 -0.02   A1V                +0.083-0.060 +.017-011SB    123                 *
8076          CD-3216398 200763212716    I                  210017.9-324429210624.7-322030 12.47-41.42 5.18  +1.10               K2III              -0.001+0.010 +.012+003
8077  4    EquBD+05 4697 200790126535      W                210029.3+053347210526.7+055730 55.39-26.18 5.94  +0.54 +0.06         F8V                -0.090-0.129 +.015-022V?  =<  6  6.4  34.9      *
8078          BD+52 2859 200817 330983682                   210044.3+525315210347.6+531710 92.78  4.30 5.90  +0.99               K0III              +0.040+0.037      -016V
8079 62Xi  CygBD+43 3800 200905 50424 792I         13518    210117.5+433144210455.9+435540 85.93-02.08 3.72  +1.65 +1.83 +0.90   K4.5Ib-II         e+0.008+0.001 +.007-020SB   < 17                 *
8080 24    CapCD-2515235 200914190025 791I  14632           210116.8-252420210707.7-250021 21.98-40.01 4.50  +1.61 +1.93 +0.98   M0.5III            -0.023-0.043 +.022+032V          7.1  26.2
8081          CP-73 2195 200924257893                       210116.8-725653211120.7-723239320.62-36.02 6.20  +1.08 +1.03         K1III              +0.026-0.022      -015V?
8082          BD+26 4073 201051 894593686                   210202.4+263126210623.5+265528 73.23-13.55 6.12  +1.05 +0.62         K0II-III           +0.044-0.017      -006
8083          BD-18 5862 201057164156              13549    210207.6-175124210744.6-172719 31.25-37.77 6.17  +0.02?              A0V                +0.018-0.027      -021V?    130
8084          BD+30 4318 201078 709173687I         DT Cyg   210218.4+304701210630.3+311105 76.55-10.78 5.82  +0.56       +0.32   F7.5Ib-II         v+0.007-0.004      +000V       9                 *
8085 61    CygBD+38 4343 201091 70919 793   14636A V1803 Cyg210224.8+381531210654.6+384445 82.25-05.81 5.21  +1.18 +1.11 +0.65   K5V                +4.136+3.203 +.292-064V    < 17  0.8  28.7AB   6*
8086 61    CygBD+38 4344 201092          I: 14636B 13546    210226.3+381510210655.3+384436 82.25-05.81 6.03  +1.37 +1.23 +0.83   K7V                +4.126+3.208 +.294-064V?  =< 25  0.8  28.7AB   6*
8087 25Chi CapBD-21 5933 2011841900503690  W                210250.0-213544210833.6-211137 26.84-39.21 5.30  +0.01 -0.03         A0V                +0.022-0.060 +.046-007V?    199: 5.7  67.0AB   4
8088          BD+15 4340 2011961068533689                   210251.4+151528210733.6+153931 64.27-20.89 6.34  +1.01 +0.78         K2IV               +0.049-0.059      -034
8089 63    CygBD+47 3292 201251 504563688I  14649  13548    210309.3+471447210636.1+473854 88.88  0.20 4.55  +1.57 +1.75 +0.82   K4Ib-IIa           +0.008+0.001 +.008-026V          8.3  15.7
8090          BD+06 4754 2012981265663692I                  210331.7+063507210828.2+065922 56.80-26.22 6.15  +1.66 +1.98         K5III              -0.006+0.003      +020
8091 27    CapBD-21 5940 201352190069              13566    210350.0-205729210933.0-203323 27.71-39.23 6.25  +0.38               F1IV               +0.120-0.124      -043V?
8092   Omi PavCP-70 2835 2013712578961554                   210358.0-703202211320.5-700735323.27-37.19 5.02  +1.58 +1.56 +0.90E  M1-2III            +0.040-0.025 +.008-019SB1                       *
8093 13Nu  AqrBD-11 5538 201381164182 794I                  210408.8-114636210935.7-112218 38.45-35.77 4.51  +0.94 +0.70 +0.46   G8III              +0.095-0.015 +.017-012     < 17
8094          BD+29 4324 201433 70968       14682  V389 Cyg 210424.3+294805210838.9+301221 76.11-11.77 5.59  -0.10 -0.26         B9VpSi             +0.025-0.017 -.006-026SBO    10  2.3   3.4AB   4*
8095          BD+02 4311 201507126587                       210455.0+023212210958.3+025636 53.26-28.80 6.45  +0.37 +0.06         F5IV               +0.040+0.017      -044
8096          BD-09 5674 201567145171                       210523.4-094535211046.9-092114 40.83-35.15 6.27  +1.16 +1.21         K0III              +0.114-0.056      -040
8097  5Gam EquBD+09 4732 2016011265931555   14702ABGam Equ  210528.7+094343211020.5+100754 59.93-24.76 4.69  +0.26 +0.10 +0.11   F0IIIp            v+0.057-0.153 +.028-017V       8  1.2 352.5AD   4*
8098  6    EquBD+09 4735 201616126597       14702D          210539.7+093828211031.2+100256 59.89-24.85 6.07  +0.02 +0.04         A2V s              -0.009+0.015 +.010+007       60  1.2 352.5AD   4
8099          BD+70 1164 201636  99573693                   210547.5+710151210623.3+712555106.95 15.95 5.87  +0.40  0.00         F3IV               -0.048-0.116 +.030+002       60
8100          CD-4014216 201647230575                       210548.4-404020211213.6-401610  1.93-43.33 5.83  +0.45 +0.01         F5IV               +0.047-0.222 +.042+011V?
8101          BD+21 4486 201671 89505       14710A          210601.7+220250211032.0+222717 70.33-17.15 6.68  +0.03 -0.06         A1V                +0.020-0.008      -012SB     86  1.0  18.0      *
8102          BD-15 5908 201707164204              EW Aqr   210609.9-145252211141.3-142820 35.19-37.52 6.48  +0.28 +0.14         F0Del Del:         +0.039+0.004      -039V?    120                 *
8103          BD+44 3718 201733 50521              Var?     210623.2+450542210958.6+453009 87.70-01.68 6.63  -0.16 -0.66         B4IVpe             +0.004-0.001      +009                          *
8104          CD-3914152 201772212793                       210639.2-394955211303.1-392531  3.10-43.46 5.26  +0.44               F5IV+F6V           +0.186-0.123 +.029-044SB2     0
8105          BD+35 4426 201819 71032       14724  Var      210702.5+355326211103.9+361757 81.05-08.08 6.54  -0.14 -0.92         B1Vp               +0.021-0.015      -006           5.7  21.5      *
8106          BD+52 2880 201834 33185                       210710.0+530917211015.6+533348 93.64  3.78 5.73  -0.12 -0.45         B9III              +0.026+0.001      -021SB
8107          BD+47 3322 201836 50536       14720           210702.6+471702211031.0+474131 89.36-00.25 6.46  -0.01 -0.35         B6IV              e+0.004-0.007      -009      120  0.8 136.1AC   6*
8108          CD-3614676 201852212800                       210704.2-365004211318.9-362526  7.22-43.34 5.96  +0.98 +0.74         K0III              +0.029-0.013      +000
8109          BD+62 1903 201888 19223                       210720.5+625315210928.9+631744100.81 10.41 6.54  -0.13 -0.48         B7III              +0.013+0.005      -024
8110          CD-2817178 2019011901291556I                  210721.6-280138211317.3-273710 19.06-41.98 5.42  +1.42 +1.69         K5III              +0.099-0.116      -042V
8111          CP-75 1697 2019062579043697                   210722.5-754540211816.1-752048317.29-35.15 6.63  +0.03  0.00         A1V                +0.013-0.035      +016
8112          BD+77  800 201908  9959 795                   210730.2+774315210529.3+780735112.40 20.19 5.91  -0.07 -0.24         B8Vn               +0.020+0.029      -016SB    210                 *
8113          BD+67 1291 202012 19229    I         T Cep    210813.1+680500210932.0+682925104.81 13.84 7.33  +1.49 +0.33         M7IIIe            v-0.032-0.067 +.002-012V                         *
8114          CP-5310015 202103246929 796  W                210837.4-534037211545.9-531547344.11-42.76 5.75  +0.19               A6IV               +0.035-0.017      -013SB    103  0.1   0.2      *
8115 64Zet CygBD+29 4348 202109 71070 797I W                210840.7+294900211256.2+301337 76.75-12.45 3.20  +0.99 +0.76 +0.48   G8+III-IIIaBa0.6   +0.001-0.056 +.027+017SB   < 17  7.9  91.2AC   4*
8116          BD+15 4375 202128106930       14761           210846.5+153414211328.8+155857 65.49-21.80 6.27  +0.24 +0.09         A7Vn               +0.043-0.023 -.005-030V          0.1   0.2      *
8117          CD-4114440 202135230596                       210849.1-405512211514.7-403023  1.61-43.90 6.21  +1.14 +1.10         K2III              +0.139-0.009      -042
8118          BD-11 5553 202149164240                       210852.1-110107211416.7-103619 39.92-36.48 6.77  -0.08 -0.31         B9pHgMn            +0.014+0.004      -004       35
8119          BD+59 2334 202214 33210       14749  13598    210915.4+593431211148.2+595911 98.52  7.99 5.64  +0.11 -0.77         B0II                0.000-0.005D+.002-016       46  1.0   1.0AB   5*
8120          BD+36 4470 202240 71086                       210925.2+361314211326.3+363801 81.63-08.20 6.05  +0.21 +0.21         F0III              -0.018+0.006      -013       23
8121          BD-00 4186 202259145229    I         13614    210929.2-001917211437.0+000532 51.22-31.32 6.38  +1.61 +1.92         M1III              +0.028-0.009      -124V                         *
8122          BD-17 6216 202261164249                       210930.9-174532211506.6-172042 32.19-39.37 6.04  +0.96 +0.68         G5                 -0.004-0.020      +004V?
8123  7Del EquBD+09 4746 202275126643       14773           210936.6+093606211428.9+100025 60.50-25.65 4.49  +0.50 -0.01 +0.28   F5V+G0V            +0.049-0.305 +.058-015SB2O   10  0.3   0.2AB   3*
8124          CD-3614699 2022872128433696                   210933.3-363732211546.8-361239  7.57-43.81 6.12  +1.37               K3III              +0.032-0.009      -016
8125          CP-65 3900 202299254966              13627    210943.7-650551211800.3-644054329.44-39.74 6.31  -0.07 -0.22         B8V                +0.006-0.037      +013
8126          BD+29 4354 202314 89549                       210953.8+292914211410.3+295404 76.69-12.87 6.17  +1.09 +0.81         G6Ib-IIaCa1Ba0.3   +0.001+0.005      -005
8127 28Phi CapBD-21 5974 202320190173    I                  210956.4-210400211537.9-203906 28.19-40.61 5.24  +1.17 +1.11        gG9                 +0.013+0.002 +.020-005V
8128 29    CapBD-15 5935 202369164263    I         13620    211012.8-153514211544.9-151017 34.86-38.69 5.28  +1.64 +1.90 +1.08E  M3III              +0.027+0.005 -.000-038V                         *
8129          CP-85  519 202418258899              13686    211033.0-851417213204.2-844836307.53-30.40 6.45  +1.40 +1.66         K3III              +0.045-0.027      +015                          *
8130 65Tau CygBD+37 4240 202444 71121    I  14787  Tau Cyg  211047.9+373706211447.5+380244 82.85-07.43 3.72  +0.39 +0.02 +0.24   F2IV               +0.160+0.435 +.055-021SB     89  2.5   1.0AB   7*
8131  8Alp EquBD+04 4635 202447126662 800I W                211049.5+045004211549.4+051452 56.38-28.71 3.92  +0.53 +0.29 +0.35   G0III+A5V          +0.059-0.088 +.021-016SBO =< 50        0.1      *
8132          BD-02 5495 202554145259                       211129.1-020129211639.6-013628 49.84-32.65 6.48  +0.98 +0.81         K0                 +0.036-0.014      -026
8133          BD+63 1708 202582 19257       14783           211139.1+635931211342.6+642414101.97 10.81 6.39  +0.60 +0.06         G2IV+G2IV          +0.027-0.105D+.024+030V          0.2   0.6      *
8134          BD-13 5897 202606164279                       211145.0-134148211713.5-131644 37.26-38.27 6.40  +0.04               A2V                -0.022-0.002      -001V?
8135   Eps MicCD-3216498 202627212874 801                   211152.5-323525211756.3-321021 13.20-43.80 4.71  +0.06 +0.02         A1V                +0.058-0.026 +.033-001V     127
8136          BD+47 3348 202654 50631                       211207.4+473326211536.8+475825 90.16-00.68 6.46  -0.15 -0.66         B4IV               -0.042-0.023      -026SB                        *
8137 30    CapBD-18 5903 202671164286      D                211220.8-182415211757.3-175907 31.73-40.23 5.43  -0.12 -0.50         B8III              +0.017-0.002      -011V?     35  0.0   0.1      *
8138          BD+41 4067 202720 50656                       211241.8+415003211629.6+421505 86.14-04.76 6.43R                     K2                 +0.003-0.028      +008
8139 31    CapBD-18 5904 202723164289                       211240.0-175254211815.7-172744 32.40-40.11 7.05  +0.34               F2V                +0.036+0.004      -007
8140   The IndCP-5310037 202730246965      W                211244.4-535206211952.0-532659343.69-43.32 4.39  +0.19 +0.12         A5V                +0.105-0.072 +.043-015      178  2.4   6.1      *
8141 15    AqrBD-05 5512 2027531452783698                   211256.3-045622211811.1-043110 47.08-34.47 5.82  -0.13 -0.51         B5V                +0.011+0.015 +.008-009                          *
8142          CD-2917692 202773190220                       211259.8-291103211854.4-284556 17.89-43.43 6.40  +0.97               G8IV               -0.181-0.058      +038
8143 67Sig CygBD+38 4431 202850 711651558          13640    211329.2+385832211725.0+392341 84.19-06.87 4.23  +0.12 -0.39 +0.14   B9Iab             v+0.001-0.003 +.011-004SB     28                 *
8144          BD+42 4046 202862 50671                       211336.2+421551211723.2+424100 86.57-04.58 6.19  -0.11 -0.43         B7Vn               +0.015-0.008      -020
8145          CD-4514302 202874230635              T Ind    211334.0-452637212009.5-450120355.26-44.65 6.00  +2.33 +3.75         C5II                0.000-0.010      +002                          *
8146 66Ups CygBD+34 4371 202904 711731559   14831  Ups Cyg  211348.3+342837211755.1+345349 80.98-10.05 4.43  -0.11 -0.82 -0.08   B2Vne              +0.014-0.002 +.019+004SB?   261  6.4  15.3AB   3*
8147          BD+53 2588 202923 33281                       211354.3+533438211702.0+535951 94.64  3.35 6.13  +0.05 +0.06         A1V                +0.031+0.033      -008V?     68
8148          CD-2615541 202940190236       14847           211358.8-264553211945.8-262111 21.19-43.11 6.56  +0.73 +0.23 +0.26E  G5V                -0.539-0.357 +.056-030SBO        3.0   2.5AB   5*
8149          BD+10 4516 2029511070203700I                  211401.4+104655211852.0+111212 62.27-25.79 5.96  +1.65               K5III              +0.026+0.015      -037                          *
8150          BD+55 2549 202987 332873699I                  211414.8+552240211714.3+554753 95.96  4.58 5.98  +1.45 +1.62         K3III              +0.018+0.017      -019V
8151   The1MicCD-4114475 203006230644 802          The1 Mic 211421.9-411356212045.6-404835  1.19-44.95 4.82  +0.02 -0.07 +0.04   ApCrEuSr           +0.069-0.005 +.012+002V?     48                 *
8152          CD-5013325 203010246983              13662    211425.1-502125212116.4-495616348.40-44.22 6.38  +1.32 +1.59         K3III              -0.021-0.151      +022
8153          BD+57 2309 203025 33288       14832  13646    211434.0+581129211718.8+583642 98.00  6.53 6.42  +0.20 -0.50         B2IIIe             +0.001+0.003      -017SBO   175  5.6   4.1AB   3*
8154 68    CygBD+43 3877 203064 506903701I         V1809 Cyg211443.4+433130211827.2+435645 87.61-03.84 5.00  -0.01 -0.94         O7.5III:n((f))    e+0.006-0.009      +001SB    328                 *
8155          BD+40 4485 203096 50699       14849           211503.3+403708211855.3+410227 85.59-05.93 6.15  +0.30?              A5IV               -0.006+0.009      +007       25  6.4  56. AC   3
8156          CP-70 2844 203133254990              Y Pav    211513.4-700936212416.7-694403323.20-38.20 6.41  +2.82 +3.40         C5II               +0.005-0.018      +001V                         *
8157          BD+37 4271 203156 71203       14859  V1334 Cyg211522.8+374855211922.2+381415 83.62-07.95 5.83  +0.50 +0.16         F1II               +0.011-0.006      +008SB     28  0.0   0.1      *
8158          BD+21 4521 203206 89628                       211541.4+213610212014.0+220135 71.53-19.13 6.29  -0.08 -0.49         B6IV               +0.003+0.010      -017       70
8159          CP-72 2598 203212257920      W                211546.4-721336212518.1-714758320.84-37.31 6.09  +1.26 +1.41         K2III              +0.018-0.007      +011           5.9  44.6
8160 16    AqrBD-05 5524 203222145317                       211549.7-045904212104.3-043336 47.47-35.11 5.87  +0.92 +0.64        gG7                 -0.012+0.011 +.014-006
8161          BD+48 3345 203245 507133702                   211602.2+490514211928.8+493037 91.70-00.06 5.76  -0.15 -0.48         B6V                +0.017+0.002 +.001-023SB
8162  5Alp CepBD+61 2111 203280 19302 803I  14858  13660    211611.5+620943211834.8+623508101.00  9.17 2.44  +0.22 +0.11 +0.11   A7V                +0.151+0.049 +.068-010V     246  9.1 206.8AB   4*
8163  9    EquBD+06 4802 203291126719    I                  211608.0+065550212104.8+072116 59.19-28.55 5.82  +1.66 +1.97         M2IIIa             +0.041-0.019      -020
8164          BD+58 2249 203338 33318    I  14864           211629.5+581201211915.8+583725 98.18  6.36 5.66  +1.38 +0.07 +1.33   M1Ibep+B2pe+B3V   v+0.005-0.001 +.004-021SB  =< 50  3.8   4.6AB   4*
8165          BD+23 4294 203344 896403704                   211633.1+232605212104.4+235121 73.10-18.06 5.57  +1.05 +0.91         K1III              +0.243-0.120 -.005-089V?                        *
8166          BD+31 4425 203358 71230       14889           211636.8+320145212050.1+322710 79.60-12.18 5.68  +1.08 +0.93         G8IV               +0.053-0.033D+.018-029V?         0.4   2.1AB   3*
8167 32Iot CapBD-17 6245 2033871643461561I                  211640.7-171538212214.8-165004 33.62-40.77 4.28  +0.90 +0.58 +0.48   G7IIIFe-1.5        +0.033+0.004 +.033+012V?   < 17
8168          BD+76  833 203399 10007    I                  211647.8+763529211542.2+770044111.85 19.06 5.95  +1.50               K5III              +0.013+0.010      +015
8169          BD+32 4134 203439 71237                       211709.1+321116212122.0+323646 79.80-12.15 6.04  +0.03 +0.10         A1V                +0.018-0.011 +.007-003SBO    50                 *
8170          BD+39 4529 203454 50739                       211706.9+395534212101.4+402044 85.37-06.70 6.40  +0.53 -0.01 +0.34   F8V                -0.019-0.207 +.034+001SBO    12                 *
8171  6    CepBD+64 1527 203467 193133703          Var      211717.7+642652211922.2+645219102.74 10.69 5.18  -0.04 -0.58         B3IVe              +0.006+0.004 -.015-018SB    148                 *
8172          CD-2316877 203475190285    I                  211716.6-230545212300.5-224008 26.31-42.85 5.60  +1.63 +2.04         M1III              +0.037+0.010      -007V?
8173  1    PegBD+19 4691 203504107073 804I  14909           211727.6+192236212205.2+194816 70.05-20.92 4.08  +1.11 +1.06 +0.54   K1III              +0.106+0.064 +.014-076SB   < 17  4.9  36.2AB   3*
8174          BD+80  690 203501  35343964                   211730.7+804841211321.5+811351115.25 21.83 6.15  +0.09 +0.09         A3IV               -0.009 0.000      -001
8175 17    AqrBD-09 5728 2035251453513705I                  211734.6-094444212256.3-091910 42.56-37.81 5.99  +1.54 +1.89         M0III              -0.025-0.026      +018V
8176          CP-83  716 2035322589023995                   211736.8-830707213354.4-824059309.46-31.74 6.38  +0.13 -0.36         B3IV               +0.018 0.000      -005
8177          CD-4713796 203548230665                       211741.9-470233212420.8-463654352.92-45.21 6.31  +0.20               A7mSr:             +0.042-0.022      -024                          *
8178 10Bet EquBD+06 4811 203562126749       14920           211755.8+062301212253.6+064840 59.00-29.23 5.16  +0.05 +0.07         A3V                +0.056+0.010 +.028-011SB     78  6.5  89.2AE   5*
8179          BD+60 2227 203574 19333                       211759.3+601954212033.5+604525 99.84  7.73 6.11  +1.00 +0.75         G5III              -0.040-0.001      -027
8180   The2MicCD-4114503 203585230667      W                211802.3-412607212424.8-410024  0.89-45.64 5.77  -0.05 -0.20         A0IIIpSi           +0.031 0.000D+.006+011           0.6   0.5AB   3
8181   Gam PavCP-65 3918 203608254999 805          13689    211810.8-654907212626.6-652158328.14-40.29 4.22  +0.49 -0.12 +0.30   F6V                +0.079+0.799 +.111-030        8
8182          BD+29 4397 203630 71259                       211824.0+295256212241.9+301835 78.29-13.94 6.05  +1.08 +1.07         K1III              +0.015+0.004      -025
8183 33    CapBD-21 6007 2036381902953707I         13683    211829.4-211637212409.6-205107 28.78-42.56 5.41  +1.16 +1.10 +0.41E  K0III              -0.007-0.125 +.021+022V?                        *
8184          CD-2316889 203639190293                       211824.5-231031212407.9-224449 26.31-43.12 6.38  +1.02              gK0                 -0.028-0.003      +014V
8185          BD+48 3357 203644 50761                       211831.9+485737212200.4+492320 91.90-00.44 5.69  +1.10 +1.02         K0III              +0.034+0.067 +.011-002SB
8186          BD+38 4471 203696 71266                       211847.3+381226212246.9+383803 84.38-08.15 6.63  +0.01 +0.02         A1V                +0.004-0.024      -015      130
8187 18    AqrBD-13 5923 2037051643641562  W       13684    211843.6-131826212411.5-125241 38.63-39.64 5.49  +0.29               F1V                +0.091+0.008      +008V     120  7.5  48.9
8188   Gam IndCP-55 9586 2037602470311563          13692    211907.5-550533212615.4-543938341.75-43.94 6.12  +0.34 +0.04         F1III              +0.004+0.036      +007
8189          BD+36 4537 203784 71276      W                211919.9+365838212323.0+372424 83.58-09.10 6.58                      F6II-III           +0.046+0.025      -027           0.0 365.4AB   5*
8190          BD+23 4300 203803 89678      W                211928.4+235040212358.8+241627 73.90-18.27 5.71  +0.32 +0.08         F1IV               +0.136+0.022 +.027-018V      97  4.8  53.9
8191          BD+09 4800 203842126774                       211931.6+094438212424.6+101027 62.30-27.50 6.35  +0.47 +0.12         F5III              +0.072+0.021      -033V?     84                 *
8192 20    AqrBD-04 5444 203843145376                       211938.9-034937212451.7-032354 49.29-35.33 6.36  +0.33 +0.13         F0III              -0.006-0.053      -023
8193          BD+36 4543 203857 71280      W                211945.5+365521212348.3+372105 83.60-09.20 6.47R                     K5                 -0.009-0.007      -003           0.0 365.4AB   5*
8194          BD+24 4394 203858 89680       14943           211939.6+245257212407.4+251844 74.72-17.59 6.15                      A2V                +0.039 0.000      -019SBO        5.6  66.8AE   6*
8195 19    AqrBD-10 5668 203875145382                       211950.6-101028212513.1-094455 42.40-38.50 5.70  +0.20               F0IV               +0.024-0.171 +.007-021
8196          CP-70 2850 2038812550033712          SX Pav   211948.6-695614212844.9-693019323.22-38.65 5.34  +1.55 +1.45 +1.51E  M5III              +0.075-0.047 +.023+043                          *
8197          BD+23 4305 203886 89682                       211953.8+240556212423.1+243144 74.16-18.17 6.32  +1.04 +0.94         K0III              +0.033 0.000      -024
8198          BD+25 4531 203925 89685                       212007.5+254439212434.0+261028 75.46-17.08 5.68  +0.31 +0.07         A8III              +0.048+0.002 +.013-003SB     53                 *
8199 21    AqrBD-04 5446 2039261453843708I                  212004.0-035908212517.0-033324 49.19-35.51 5.49  +1.46 +1.82         K4III              -0.008-0.065      -024
8200          CD-3814551 2039492129983710                   212008.0-381539212622.9-374946  5.45-46.01 5.63  +1.19 +1.22         K2III              +0.163-0.012      -076
8201          CP-80 1017 203955258904      W                212010.2-802833213320.6-800221311.99-33.30 6.47  +0.04 +0.05         A0V                +0.006-0.012      +021           4.8  24.4
8202          CD-4314539 204018230692      W                212036.8-425851212701.6-423252358.65-46.06 5.51  +0.39 +0.15 +0.19   Am                 -0.049+0.017      +018V?     53  2.7   2.8
8203          BD-00 4215 204041126789                       212044.3+000608212551.5+003204 53.47-33.44 6.46  +0.16 +0.05         A1IV               +0.040+0.013      -009       68
8204 34Zet CapBD-2215388 204075190341 806I  14971           212057.5-225040212640.0-222441 26.98-43.59 3.74  +1.00 +0.59 +0.43   G4Ib               +0.001+0.023 -.010+003SB   < 19  8.6  21.3      *
8205          BD+00 4726 2041211267943711                   212121.2+004031212628.1+010612 54.14-33.25 6.13  +0.44 +0.01         F5V                +0.113-0.163 +.028+011V?     12
8206          BD+48 3376 204131 50817       14962  V1934 Cyg212125.8+485331212455.5+491924 92.20-00.82 6.58  -0.03  0.00         B9pSi:Cr:Sr:       -0.015-0.007      +001           5.9  20.3
8207 35    CapBD-21 6020 204139190349    I                  212134.7-213743212714.8-211146 28.63-43.36 5.78  +1.44 +1.74 +0.76   K5III              -0.026-0.030      +023V                         *
8208          BD+46 3305 204153 50824                       212139.2+461652212519.6+464252 90.42-02.73 5.60  +0.32 -0.01         F0V                +0.195+0.048 +.039+001V
8209 69    CygBD+36 4557 204172 71329       14969           212141.7+361407212547.0+364003 83.39-09.96 5.94  -0.08 -0.94         B0Ib               +0.006-0.005      +003       80  5.3  53.4AC   3*
8210          BD+18 4794 204188107138              IK Peg   212147.9+185632212626.7+192232 70.43-21.98 6.07  +0.22 +0.06 +0.12   A8m                +0.081+0.014      -011SBO    83                 *
8211          CP-54 9872 204228247043                       212157.4-540825212900.3-534221342.89-44.57 6.39  +1.14               K2IIICNIb          +0.081-0.025      +010
8212          BD-12 6005 204363164415                       212249.0-120008212813.9-113406 40.72-39.98 6.61  +0.49               F3V                +0.009-0.048      +023V
8213 36    CapBD-22 5692 204381190374    I                  212301.3-221434212843.4-214826 27.97-43.87 4.51  +0.91 +0.60 +0.46   G5III              +0.138-0.006 +.023-022V                         *
8214  5    PsACD-3118291 204394213034                       212305.3-314027212903.8-311419 14.98-46.02 6.50  +0.03               A0                 +0.022-0.011      -000
8215 70    CygBD+36 4568 204403 713583714                   212316.6+364055212721.4+370700 83.94-09.86 5.31  -0.14 -0.64         B3V                 0.000+0.002      -020SB    135                 *
8216          BD+48 3390 204411 50867              13718    212318.5+482400212651.6+485006 92.09-01.39 5.31  +0.07 +0.16         A6pCrEu:           +0.060+0.023 +.008-013V       9                 *
8217 35    VulBD+26 4164 204414 897203715                   212315.5+271023212740.1+273631 77.04-16.60 5.41  +0.04 +0.05         A1V                +0.041+0.018 +.019-008V?     97
8218          BD+52 2939 204428 33434                       212327.0+522751212645.0+525355 94.89  1.55 6.03  -0.12 -0.46         B6V                +0.017+0.008 +.018-030
8219          BD+07 4696 2044451268181564I         13727    212329.0+074538212824.8+081144 61.21-29.49 6.40  +1.64 +1.90 +0.93E  M1                 +0.011-0.025      -006                          *
8220          BD+31 4462 204485 713713716                   212351.8+314715212808.3+321331 80.53-13.44 5.80  +0.32 +0.04         F0V                +0.137+0.083 +.023-024                          *
8221          BD+17 4592 204560107168              13738    212419.0+172809212859.9+175421 69.67-23.41 6.44  +1.39 +1.64         K5                 -0.015-0.001      -012
8222          BD-19 6107 204577164433                       212422.9-193503212959.6-190852 31.58-43.31 6.57  +0.41               F0V                +0.034-0.045      -012V?  =< 15
8223          BD+21 4555 204585 89737    I W       13740    212425.5+214432212859.9+221046 73.09-20.54 5.93  +1.53 +1.32         M4.5III-IIIa       +0.045+0.011      -022           2.7  41.1AB   3
8224          BD+59 2383 204599 334433717I  14998  13729    212439.5+591853212725.3+594500 99.73  6.42 6.10  +1.75 +1.72         M3IIIaBa0.2        -0.022-0.018      -016V?         4.7  11.2      *
8225  2    PegBD+23 4325 204724 897521565I  15027           212525.0+231202212956.9+233820 74.39-19.70 4.57  +1.62 +1.93 +1.09   M1+III             +0.024+0.004 +.013-019V          8.1  29.8
8226          BD+54 2544 204754 33458    I         13745    212544.0+545850212852.7+552507 96.86  3.16 6.12  +0.14 -0.28         B8III              +0.014+0.011      -007V?      0
8227  7    CepBD+66 1405 204770 194323718                   212550.2+662222212746.1+664833104.76 11.43 5.44  -0.11 -0.42         B7V                -0.014-0.019 +.006+003V     295
8228 71    CygBD+45 3558 204771 50934 807                   212545.5+460558212927.0+463226 90.81-03.35 5.24  +0.97 +0.80 +0.49   K0-III             +0.045+0.104 +.023-019V?   < 19:
8229   Xi  GruCD-4114550 2047832307263719                   212546.1-413712213205.9-411045  0.56-47.08 5.29  +1.10 +1.01         K0III              +0.018+0.015 +.012-008
8230  6    PsACD-3415110 2048542130781566  W       13762    212611.7-342307213214.6-335641 11.18-46.99 5.97  +0.05               A2IV              v-0.001-0.004      +016           7.3   6.8
8231          BD+11 4583 204862107195                       212619.4+114153213109.6+120815 65.20-27.55 6.08  -0.05 -0.21         B9.5V              +0.019-0.014      -010SB2   125
8232 22Bet AqrBD-06 5770 204867145457 808I  15050           212617.7-060040213133.5-053416 48.02-37.88 2.91  +0.83 +0.56 +0.43   G0Ib               +0.021-0.008 +.006+007V?     18  7.9  35.5AB   3*
8233          CP-5310092 204873247080                       212622.4-531044213317.7-524415343.97-45.45 6.41  +1.48               K4III              -0.005-0.009      +034
8234          CP-79 1158 2049042579423723                   212627.1-795316213856.2-792633312.39-33.86 6.18  +0.46 +0.02         F4IV               +0.059-0.022      -006
8235          CD-2515479 2049431904233720                   212647.6-250155213233.3-243526 24.54-45.47 6.43  +0.20               A7V                +0.073+0.007      -009
8236          CD-4514367 2049602307371567                   212654.8-451726213323.5-445055355.16-46.99 5.57  +1.04 +0.85         K1III              -0.017-0.007      +011V
8237          BD+52 2957 204965 33477                       212700.4+523104213020.4+525729 95.32  1.22 6.02  +0.08 +0.11         A2Vp               +0.024+0.015      -017SB     65                 *
8238  8Bet CepBD+69 1173 205021 10057 809   15032  Bet Cep  212722.2+700718212839.6+703339107.54 14.03 3.23  -0.22 -0.95 -0.22   B1IV              v+0.010+0.007 +.014-008SBO    28  4.6  13.4AB   3*
8239          BD+79  707 205072  3568                       212746.7+800520212449.7+803129114.98 21.03 5.97  +0.92              gG6                 +0.044-0.016      +003
8240          BD+22 4418 205087 89786              13774    212754.2+225709213227.1+232340 74.62-20.29 6.70  -0.09 -0.28         B9pSiSrCr          +0.036+0.002      -016V?     25
8241          CD-4314602 205096230741                       212754.2-432205213417.0-425530357.95-47.36 6.32  +1.09               K1IIICNII          -0.041-0.024      +009
8242          BD+51 3079 205114 33497                       212805.8+521042213127.5+523712 95.22  0.86 6.16  +0.90 +0.44         G2Ib+B9V           +0.009+0.003      -023SBO =< 50                 *
8243          BD+59 2395 205139 19466                       212814.4+600106213059.3+602734100.55  6.62 5.53  +0.12 -0.73         B1II               -0.002-0.001      -015V?     73                 *
8244          CD-3018703 2052651904583721                   212859.1-300825213453.0-294146 17.47-47.03 6.41  -0.11               B8III              +0.017-0.017      -015V
8245 37    CapBD-20 6237 205289190461                       212914.1-203148213451.0-200504 30.88-44.70 5.69  +0.40 -0.02         F1V                -0.037+0.031      +006V?
8246          BD+49 3553 205314 51019      S                212924.5+493202213256.6+495840 93.59-01.24 5.75  -0.03 -0.11         A0V                +0.020+0.015      -033SB    265        0.0
8247          CD-2416729 205342190469                       212932.8-235357213515.9-232715 26.34-45.78 6.40  +1.10              gG7                 +0.077-0.008      -015V?
8248          BD+45 3584 205349 51027    I         13784    212933.0+452437213317.9+455115 90.83-04.31 6.25  +1.81 +2.03         K1Ib              v-0.001+0.003      -005                          *
8249          CP-65 3937 205417255040                       213004.1-651618213802.9-644927328.10-41.63 6.20  +0.03 +0.05         A0-1IV             +0.032-0.010      -009V?
8250          BD+22 4431 205420 89807                       212959.8+221839213434.0+224517 74.48-21.08 6.47  +0.51 +0.03         F7V                +0.014-0.041      +014
8251          BD-04 5489 205423145510                       213004.4-042544213517.6-035859 50.35-37.86 5.77  +1.11 +1.05        gG9                 -0.005+0.001 +.001-002V?
8252 73Rho CygBD+44 3865 205435 510351568I         13787    213013.1+450859213358.9+453531 90.74-04.58 4.02  +0.89 +0.56 +0.50   G8IIIFe-0.5        -0.023-0.094 +.002+007     < 19:                *
8253  8    PsACD-2615702 205471190478      W                213023.1-263703213611.0-261017 22.61-46.64 5.73  +0.22               A7/8IV             +0.111-0.023      -019           8.2  18.4      *
8254   Nu  OctCP-77 1510 205478257948 810  A                213021.7-775002214128.5-772324314.29-35.19 3.76  +1.00 +0.89         K0III              +0.045-0.240 +.053+034SBO              0.1 *    *
8255 72    CygBD+37 4359 205512 714803722I                  213041.4+380508213446.6+383203 86.01-09.85 4.90  +1.08 +1.02 +0.54   K0.5III            +0.124+0.099 +.012-066V?   < 17
8256  7    PsACD-3315664 205529213136                       213048.5-332943213648.9-330253 12.63-47.85 6.11  +0.22               A7Vn               +0.089-0.003      -002
8257          BD+27 4107 205539 89815      S                213053.1+274508213519.0+281150 78.72-17.39 6.31  +0.35 -0.05         F0IV               +0.128-0.043      -042SB2O             0.2      *
8258          BD+23 4346 205541 89819       15115           213055.5+240022213527.0+242708 75.94-20.05 6.11                      A4V                +0.011-0.011 +.002-028SB2   170  0.3   0.3      *
8259          BD+51 3091 205551 33540                       213100.6+511510213427.5+514154 94.93-00.13 6.15  +0.02 -0.29         B9IIIe             +0.007 0.000 -.006-022V     175
8260 39Eps CapBD-20 6251 2056371645203724  W       Eps Cap  213128.9-195451213704.8-192758 31.94-44.99 4.68  -0.17 -0.65 -0.12   B2.5Vpe           v+0.015+0.003      -024SB?   293  1.3   0.0     3*
8261          BD+29 4456 205688 89834       15126           213152.5+293622213613.9+300320 80.23-16.20 6.36R                     G8III-IV           -0.063+0.062      -020           4.4   2.1
8262          BD+44 3877 205730 51079    I         W Cyg    213214.3+445536213602.4+452229 90.86-04.98 5.53  +1.58 +1.24 +2.14   M5IIIae            +0.051+0.006      -014V                         *
8263          BD-01 4180 205765145533       15142           213225.6-005020213733.8-002325 54.52-36.38 6.25  +0.06 +0.05 +0.02   A2V                -0.015-0.023      +017V?    156  3.3  31.5      *
8264 23Xi  AqrBD-08 5701 2057671455371569  S                213225.7-081810213745.1-075115 46.45-40.34 4.69  +0.17 +0.13 +0.10   A7V                +0.116-0.025 +.012-021SBO   154  2.0   0.1      *
8265  3    PegBD+05 4830 205811126940       15147A          213244.7+061008213743.7+063706 61.41-32.28 6.18  +0.02 +0.02         A2V                +0.060-0.007      +003SB     89  1.5  39.2AB   3*
8266 74    CygBD+39 4612 205835 51101 811                   213256.3+395751213657.0+402449 87.62-08.76 5.01  +0.18 +0.10         A5V                -0.001+0.013 +.020+007V?    171
8267  5    PegBD+18 4827 2058521072881570                   213304.6+185207213745.4+191907 72.34-23.99 5.45  +0.30 +0.14         F1IV               +0.101+0.013 +.007-025V?    134                 *
8268          CD-3415163 205872213168                       213305.4-340742213906.1-334045 11.74-48.38 6.28  +0.92               G8IV               +0.066-0.051      -020
8269          CP-5211911 2058772471283726                   213310.2-524838213959.7-522133344.10-46.53 6.21  +0.60 +0.32         F7III              -0.017+0.001      -000SB2
8270  4    PegBD+05 4834 205924126956       15157           213331.4+051913213831.9+054618 60.76-32.96 5.67  +0.25 +0.08         A9IV-Vn            +0.114+0.024 +.033-019V?    195  6.4  27.2      *
8271          CP-56 9700 205935247132                       213330.5-561125214033.6-554415339.43-45.56 6.33  +1.06               K0II-III           +0.010+0.023      +027
8272          BD+44 3889 205939 51109              CP Cyg   213337.8+441451213727.9+444148 90.59-05.65 6.20  +0.19               A7III              -0.001-0.028      +004V?     23                 *
8273          BD-11 5640 2060051645553728                   213405.6-110138213928.1-103437 43.53-42.00 6.08  +1.03 +0.89         K0                 +0.019-0.050      -009
8274          BD+24 4445 206027 89870      S                213414.7+250252213845.0+252956 77.29-19.84 6.16  +1.02 +0.82         G9III              -0.021+0.003      -014                 0.1
8275          BD+53 2659 206040 33596                       213418.9+533531213738.7+540232 96.86  1.29 6.15  +0.99 +0.74         K1III              -0.019 0.000      +002
8276          BD+19 4754 206043 89871                       213421.4+194850213901.1+201555 73.31-23.55 5.85  +0.32 +0.03         F2V                +0.113-0.003 +.021-013V?    105
8277 25    AqrBD+01 4517 2060671269653729I W                213428.8+014738213933.3+021437 57.54-35.28 5.10  +1.04 +0.90 +0.52   K0III              -0.026-0.082 +.006-035V    < 19: 6.3 136.8
8278 40Gam CapBD-17 6340 206088164560 812I                  213433.1-170651214005.5-163944 36.00-44.67 3.68  +0.32 +0.20 +0.14   F0p                +0.190-0.023 +.029-031SB     30                 *
8279  9    CepBD+61 2169 206165 195413725          V337 Cep 213514.2+613751213755.2+620455102.27  7.25 4.73  +0.30 -0.53 +0.18   B2Ib              v-0.003-0.002 -.008-013V?     36                 *
8280   Lam OctCP-83  722 206240258914      W                213536.4-831044215054.3-824309309.02-32.13 5.29  +0.75 +0.47         G8-K0III           +0.058-0.036D+.011-011           2.3   2.8      *
8281          BD+56 2617 206267 33626 813   15184           213551.4+570212213857.6+572921 99.29  3.74 5.62  +0.21 -0.74         O6.5V((f))         -0.001-0.003      -008SBO   154  2.2  11.8AC   5*
8282          CD-2515545 206291190555                       213602.3-253323214146.1-250607 24.56-47.63 6.49  +1.19               K0                 -0.021-0.003      +032V?
8283 42    CapBD-14 6102 206301164580    I'                 213606.6-142937214132.9-140251 39.56-43.97 5.18  +0.65 +0.20 +0.24E  G1V+G0V            -0.118-0.307 +.034-001SB1O   17  2.0   0.0      *
8284 75    CygBD+42 4177 206330 511673730I  15208  13834    213615.5+424911214011.1+431626 90.00-07.04 5.11  +1.60 +1.90         M1IIIab            +0.063+0.016 +.004-028V          4.2  57.9AC   3*
8285 41    CapCD-2317057 206356190559    I  15223           213619.1-234255214200.8-231546 27.21-47.22 5.24  +0.95 +0.75         G9III              +0.102-0.089 +.025-044V          6.2   5.1      *
8286          CP-71 2632 2063992579553735                   213636.8-712758214528.8-710032320.64-39.09 6.01  -0.10 -0.35         B8IV               +0.009-0.019      -004V?                        *
8287 26    AqrBD+00 4770 206445126997    I                  213704.1+004947214210.1+011707 57.05-36.37 5.67  +1.44 +1.67         K2III              +0.001-0.007      +010V                         *
8288 43Kap CapBD-19 6152 206453164593    I                  213704.5-191920214239.5-185159 33.38-46.03 4.73  +0.88 +0.52 +0.49   G8III              +0.147-0.006 +.028-003V?   < 19:
8289  7    PegBD+05 4850 2064871270023732I         13848    213715.3+051328214215.5+054048 61.36-33.75 5.30  +1.64 +1.95         M2IIIab            +0.014-0.006      -004V?
8290          BD+54 2595 206509 336563731                   213724.5+542502214043.3+545220 97.74  1.61 6.20  +1.16 +0.96         K0III              +0.010+0.005      +004                          *
8291 76    CygBD+40 4611 206538 51189      W                213732.8+402104214134.3+404819 88.54-09.06 6.11  +0.07 +0.07 +0.05   A2V                -0.012-0.046 +.013+003SB    150  4.0  62.1AB   3*
8292          BD+10 4604 206540107340                       213739.8+102206214233.0+104929 66.11-30.53 6.09  -0.11 -0.51         B5IV               +0.024-0.002      +006V?     20
8293          BD-20 6270 206546164601                       213737.9-200439214313.5-193715 32.41-46.41 6.22  +0.27 +0.15 +0.12   A3m                +0.077-0.011      -025SBO    47  0.1   0.0      *
8294          CP-89   53 2065532589311669          CG Oct   213737.1-891903224528.6-884906303.63-28.13 6.57  +0.28 +0.13         F0IV-V             +0.016-0.040      +015                          *
8295 44    CapBD-15 6046 206561164600      D                213737.1-145125214304.4-142359 39.32-44.45 5.88  +0.25               F0IV               -0.002+0.025      -013                          *
8296 NOVA 1876                                     Q Cyg                                                                                                                                            *
8297          BD+34 4500 206570 71613    I         V460 Cyg 213747.9+350315214201.1+353037 85.01-13.06 6.07  +2.52 +4.76 +1.40   C6.3               +0.007 0.000      +010                          *
8298          BD+45 3637 206632 51204    I         V1339 Cyg213818.9+451835214208.4+454557 91.92-05.40 6.17  +1.55               M4III:             +0.006-0.013 +.003+009                          *
8299          CD-3914405 2066422132483736                   213820.3-390022214429.5-383309  4.35-49.57 6.30  +1.12 +0.93         G5III              +0.081-0.168      -058
8300 77    CygBD+40 4615 206644 51207      W                213821.4+403713214222.9+410438 88.84-08.96 5.69  +0.01 -0.01         A0V                +0.021+0.003 +.003-025SB2O   45  0.2   0.1      *
8301 80Pi 1CygBD+50 3410 206672 336653733                   213832.5+504359214205.7+511123 95.48-01.30 4.67  -0.12 -0.68 -0.13   B3IV               +0.006-0.006      -008SBO   109                 *
8302 45    CapBD-15 6052 206677164612                       213833.4-151227214401.0-144458 39.00-44.80 5.99  +0.21 +0.07 +0.12   F0V                -0.023+0.008      -004SB
8303          CD-5013463 206690230822                       213841.2-495733214519.0-492955347.83-48.10 6.45  +1.15               K1III              +0.094+0.053      -051
8304          BD+48 3480 206731 51217                       213900.3+490836214238.9+493601 94.51-02.56 6.09  +1.00 +0.74         G8II               +0.010-0.002      -002V
8305  9Iot PsACD-3315734 206742213258 814  W                213859.5-332856214456.8-330133 12.88-49.55 4.34  -0.05 -0.11 -0.03   A0V                +0.034-0.094 +.041+002SB2    47  7.0  20.       *
8306          BD+40 4623 206749 51221    I         13857    213905.2+404152214306.4+410918 89.00-08.99 5.49  +1.59 +1.96         M2IIIab            -0.023-0.015      -023V                         *
8307 79    CygBD+37 4408 206774 71643      W                213917.4+374933214325.7+381702 87.11-11.18 5.65R  0.00 -0.02         A0V                +0.036+0.004      -023      165  1.3 151.2AB   5*
8308  8Eps PegBD+09 4891 206778127029 815I  15268  Eps Peg  213916.4+092459214411.2+095230 65.57-31.46 2.39  +1.53 +1.70 +0.76   K2Ib              e+0.031-0.001 +.006+005V    < 17  6.1 142.5AC   3*
8309 78Mu 1CygBD+28 4169 206826 89940       15270A          213940.1+281727214408.6+284434 80.58-18.34 4.73H +0.48 +0.01 +0.28   F6V                +0.292-0.241 +.049+018SB2    18  1.3   2.0AB   5*
8310 78Mu 2CygBD+28 4169 206827 89939       15270B          213940.3+281726214408.3+284435 80.58-18.34 6.08H +0.36?              G2V                +0.231-0.219 +.049+017V?         1.3   2.0AB   5*
8311 46    CapBD-09 5829 206834145637    I                  213940.3-093230214500.3-090457 46.17-42.51 5.09  +1.11 +0.95 +0.53   G8II-III           +0.021-0.005 +.002-005SB   < 17
8312          BD+58 2314 206842 33683                       213944.8+584847214245.4+591616100.84  4.75 6.08  +1.34               K1III              -0.003+0.010      -002
8313  9    PegBD+16 4582 206859107365    I         13864    213946.5+165328214430.7+172100 71.98-26.51 4.34  +1.17 +1.00 +0.55   G5Ib               +0.012-0.013 +.008-022V?     20
8314          BD+14 4668 2068601073643737          HN Peg   213942.0+141858214431.3+144619 69.86-28.27 5.94  +0.59 +0.04         G0V               v+0.234-0.119 +.065-019V?     11                 *
8315 10Kap PegBD+24 4463 206901 89949    I  15281           214006.9+251107214438.7+253842 78.41-20.67 4.13  +0.43 +0.03 +0.25   F5IV               +0.034+0.010 +.035-008SB1O   29  0.4   0.3AB   3*
8316   Mu  CepBD+58 2316 206936 33693    I  15271  Mu  Cep  214026.8+581917214330.4+584648100.60  4.32 4.08  +2.35 +2.42 +1.76   M2-Ia             e+0.003-0.003 +.003+019V          8.3  19.5AB   3*
8317 11    CepBD+70 1193 206952 10126 817I                  214027.4+705103214155.3+711841108.88 13.82 4.56  +1.10 +1.10 +0.54   K1III              +0.117+0.099 +.008-037     < 17
8318 47    CapBD-09 5833 207005145648    I         AG Cap   214056.2-094414214616.3-091633 46.14-42.88 6.00  +1.66 +1.87 +1.03E  M3III              +0.021+0.009      +021V?                        *
8319 48Lam CapBD-12 6087 207052164639 818                   214109.1-114938214632.1-112157 43.64-43.91 5.58  -0.01 -0.05         A1V                +0.031-0.008 +.015+001V     190
8320          BD+35 4626 207088 716751571                   214130.0+352345214544.5+355126 85.82-13.30 6.40  +1.00 +0.71         G8III              +0.094+0.012      -005
8321 12    PegBD+22 4472 207089 899723739I                  214128.2+222916214604.4+225656 76.64-22.83 5.29  +1.41 +1.33         K0IbHdel 0.5      e+0.010-0.005 +.006-012                          *
8322 49Del CapBD-16 5943 207098164644 819I  15314  Del Cap  214131.3-163452214702.4-160738 37.60-46.01 2.87  +0.29 +0.09 +0.17   Am                v+0.263-0.297 +.087-006SBO    87  2.0   0.0     4*
8323          CD-4713928 2071292308461573  W                214145.6-474531214815.8-471813350.88-49.10 5.58  +0.60 +0.08         G0V                +0.167-0.298 +.073-007           3.0  55.0
8324          BD+71 1082 207130 10131    I                  214151.1+715142214304.0+721913109.65 14.50 5.17  +1.05 +0.97         K0III              -0.044-0.038 +.015-039V?   < 17                 *
8325          BD+24 4473 207134 89976    I:                 214150.7+250601214624.0+253348 78.65-21.00 6.28  +1.21 +1.25         K3III:             +0.166+0.044 +.006-045V?
8326 10The PsACD-3118466 2071552132923741  W                214152.1-312140214744.2-305354 16.24-49.94 5.01  +0.04 +0.05         A2V                -0.032+0.002 +.006+014      139  0.1   0.1AB   3*
8327          BD+61 2193 207198 19621      W                214208.3+615959214453.3+622738103.14  6.99 5.95  +0.31 -0.64 +0.17   O9Ib-II           e-0.007-0.002      -018       76  3.5  17.0      *
8328 11    PegBD+02 4414 2072031270601574                   214209.7+021325214714.0+024110 59.38-36.56 5.64   0.00 -0.01         A1V                +0.010-0.003 +.005+017V?    139
8329          BD+42 4204 207218 51277                       214218.2+423554214616.6+430339 90.71-07.93 6.54  +0.28 +0.13         G0:III+A4V         +0.013+0.013      -019V     100                 *
8330          BD+16 4598 207223107395                       214219.3+164355214704.7+171139 72.32-27.05 6.21  +0.34 +0.01         F3V                +0.094-0.014      -019
8331          CP-65 3951 2072292550803742                   214215.0-651034215000.1-644245327.40-42.80 5.62  +1.02 +0.83         K0III              -0.003-0.042      -001
8332          BD-06 5827 207235145671                       214222.4-062249214738.1-055502 50.30-41.49 6.17  +0.22 +0.05         A7V                +0.049-0.002      -011      120
8333   Omi IndCP-70 2873 207241255087 820                   214219.9-700541215047.1-693746321.80-40.23 5.53  +1.37 +1.63         K2-3III            -0.031-0.002 +.029+020
8334 10Nu  CepBD+60 2288 207260 196241572I         Nu Cep   214233.7+603933214526.9+610715102.31  5.93 4.29  +0.52 +0.13 +0.44   A2Ia               -0.003-0.003 +.013-021V      33                 *
8335 81Pi 2CygBD+48 3504 207330 51293 821                   214305.8+485048214647.6+491834 94.83-03.22 4.23  -0.12 -0.71 -0.12   B3III              +0.004-0.002 -.001-012SBO    43                 *
8336          BD+35 4643 207446 71716    I                  214355.4+360658214808.3+363450 86.68-13.08 6.47R                     K5                 -0.028+0.002      -031
8337          BD-13 6027 207503164679                       214416.8-131121214941.1-124323 42.41-45.20 6.31  +0.22               A3m                -0.003+0.015      +000SB2    20                 *
8338          BD+37 4427 207516 71722                       214420.1+381101214829.4+383855 88.12-11.56 6.12  -0.08 -0.34         B8V                +0.022-0.002      -020      100
8339 12    CepBD+60 2294 207528 19642    I                  214428.2+601343214725.3+604134102.22  5.45 5.52  +1.52 +1.93         M1IIIb             -0.008 0.000 +.021-020
8340          BD-17 6389 207552164686                       214443.0-171840215013.1-165041 37.05-46.99 6.38  +1.42               K0                 +0.034 0.000      -001V
8341          BD+19 4793 207563 900273744                   214446.4+195948214926.9+202745 75.35-25.15 6.29  -0.10 -0.64         B2V                +0.003-0.002      -012V?    122
8342          BD+69 1198 207636 19646                       214515.3+694113214701.0+700903108.41 12.67 6.29R                     A0V                -0.004-0.026      -002V?    205
8343 14    PegBD+29 4525 207650 900401575                   214525.2+294230214950.7+301027 82.57-18.12 5.04  -0.03 +0.03         A1V s              +0.019-0.027 +.006-023SBO    68                 *
8344 13    PegBD+16 4612 207652107425      W       13891    214523.1+164914215008.7+171708 72.97-27.52 5.29  +0.37 +0.03 +0.18   F2III-IV           +0.076-0.061 +.015-004V      71  2.0   0.2      *
8345          BD+40 4648 207673 513443745                   214535.9+404057214940.1+410856 89.94-09.80 6.48  +0.42 +0.07         A2Ib               -0.002-0.007      -002V      21
8346          BD-19 6176 207760164697                       214608.6-190521215141.8-183723 34.79-47.96 6.16  +0.36 +0.06         F0III              +0.148-0.079      -042V?
8347          BD+60 2300 207780 19663    I                  214623.2+604824214919.0+611622102.77  5.75 6.17  +1.67 +2.03         M1II-III           +0.003-0.024      -019
8348          BD+19 4797 207840107445       15383           214652.4+192128215134.2+194936 75.25-25.96 5.77  -0.10 -0.40         B8III              +0.014+0.017      -020       20  5.6  19.3AB   3*
8349          BD+38 4621 207857 71767              V1619 Cyg214656.6+390406215104.9+393212 89.10-11.20 6.17  -0.08 -0.42         B9pHgMn            +0.006+0.005      +000V?     12                 *
8350          BD+20 5027 207932 90059    I         HO Peg   214738.3+204810215218.2+211623 76.50-25.04 6.89R                     M4III              +0.015+0.021      -022                          *
8351 51Mu  CapBD-14 6149 2079581647131577                   214750.7-140121215317.8-133306 41.88-46.35 5.08  +0.37 -0.01         F1III              +0.313+0.013 +.047-022V      87                 *
8352          CP-62 6277 207964255101      W                214749.0-622121215511.5-615311330.39-44.71 5.90  +0.39               F1III              +0.050-0.084D+.019+001           0.1   0.3      *
8353   Gam GruCD-3714536 207971213374 822                   214752.5-375006215355.7-372154  6.11-51.47 3.01  -0.12 -0.37 -0.06   B8III              +0.102-0.021 +.013-002V?     68
8354 15    PegBD+28 4215 207978 90065                       214802.0+281931215229.9+284736 82.05-19.54 5.53  +0.42 -0.13         F6IV-V vw          -0.059-0.067 +.040+019V?  =<  6                 *
8355          BD-10 5785 208008164717      W                214815.2-104657215336.0-101842 46.07-44.96 6.59  -0.12               B9V                +0.012-0.004      -011V          0.5   0.1AB   3*
8356 16    PegBD+25 4635 208057 90075 823                   214830.6+252716215303.8+255530 80.10-21.75 5.08  -0.17 -0.67         B3Ve               +0.009-0.002 +.003-012SB    152                 *
8357          BD+55 2639 208095 33819       15405A 13909    214837.9+551936215201.0+554749 99.56  1.28 5.71  -0.13 -0.46         B6IV-V             +0.013+0.003D+.005-007SB2O  120: 1.0  18.3AB   3*
8358          BD+18 4879 208108107474                       214855.0+191148215337.4+194006 75.51-26.41 5.68  +0.01 +0.01         A0V s              +0.013+0.013      +006    =< 41
8359          BD+06 4919 208110127141                       214858.1+062334215357.8+065152 64.70-35.25 6.15  +0.80 +0.27         G0III s            +0.081 0.000      -010     < 32                 *
8360          BD-04 5568 208111145731                       214857.0-044443215410.4-041634 53.33-41.99 5.71  +1.18 +1.27         K2III              +0.057-0.093 +.017-037
8361          BD+65 1664 208132 19686       15407           214907.2+651659215137.3+654510105.85  9.04 6.37                      A1m                -0.006-0.015D+.010+004V?         0.3   1.9AB   3*
8362   Pi  IndCP-58 7911 208149247230                       214913.0-582223215614.0-575358335.36-46.69 6.19  +0.21 +0.15         Am                 +0.016+0.011      +007V?                        *
8363          BD-03 5329 208177145735       15432           214924.0-034621215435.9-031804 54.49-41.54 6.20  +0.48 +0.07         F5IV               +0.018-0.028      -016           3.4  17.6      *
8364          BD+19 4814 208202107489       15431           214935.3+191447215417.4+194306 75.68-26.48 6.39  +0.97 +0.68         K0III+F7V          -0.034-0.008      +004V?         2.1  22.3AB   3*
8365          CD-3118541 208285213406                       215005.8-310445215555.6-303623 17.01-51.66 6.41  +0.93               G8III              +0.051-0.026      -085V
8366          CD-3714565 208321213414                       215021.7-374339215622.8-371513  6.25-51.96 5.46  +0.08               A2Vn               -0.016-0.002      +028V     222
8367          CD-3814801 208435213427              BZ Gru   215100.1-381319215702.2-374449  5.44-52.07 6.18  +0.33               F1III-IV           +0.022-0.003      +001                          *
8368   Del IndCP-55 9733 208450247244 824  W                215106.9-552805215755.1-545933339.11-48.12 4.40  +0.28 +0.10         F0IV               +0.048-0.007 +.020+015V?    130  0.1   0.1      *
8369   Kap1IndCP-59 7744 2084962472473752          BG Ind   215126.0-592920215830.1-590044333.72-46.45 6.12  +0.46 -0.02         F3V                +0.005+0.026      +040SB2                       *
8370          CP-78 1430 208500257990                       215126.0-780826220152.1-773945313.17-35.87 6.41  +0.22 +0.08         A5IV-V             -0.031-0.002      +001V
8371 13    CepBD+55 2644 208501 338643749          13963    215131.4+560815215453.2+563641100.39  1.68 5.80  +0.73 -0.02 +0.60   B8Ib              v-0.003-0.001      -015V?     53                 *
8372          BD+20 5046 208527 901121579I                  215143.3+204552215624.0+211423 77.24-25.72 6.40  +1.70 +2.03         K5V                +0.002+0.015 +.004+002
8373 17    PegBD+11 4696 2085651075283751                   215203.7+113605215656.4+120435 69.96-32.32 5.54  +0.05 +0.03         A2Vnn              -0.024-0.014      +015V     240
8374          BD+60 2318 208606 19738    I                  215220.9+610402215520.7+613231103.50  5.51 6.13  +1.60 +1.62         G8Ib               +0.004+0.007      -032SB                        *
8375          BD+64 1607 208682 19742       15467  13966    215253.6+645045215531.1+651915105.89  8.45 5.86  -0.06 -0.75         B2.5Ve             +0.007+0.003      -015V     350: 2.2   1.1      *
8376          BD-06 5878 208703145778                       215258.8-055356215813.3-052529 52.76-43.46 6.33  +0.37 -0.03         F5IV               +0.040-0.101      +001V      12
8377          BD+47 3618 208727 514773754                   215312.9+481133215702.2+484007 95.74-04.77 6.42  -0.08 -0.37         B8V                +0.005+0.002      -016V?    140
8378          BD-21 6131 2087351907863757I                  215309.2-213936215843.8-211058 31.90-50.35 6.12  +1.65 +1.83         M4III              +0.023-0.002      +003
8379          CD-3814820 208737213452                       215315.2-385222215917.9-382343  4.34-52.47 5.50  +1.00               K0III              +0.042-0.006      -010
8380          CP-76 1542 2087412579933760  W                215316.5-763547220303.8-760707314.54-36.95 5.95  +0.39 +0.11         F3III              +0.022-0.080      +007SB         4.2  34.6
8381          CP-56 9784 208796247262                       215334.8-562142220024.1-555258337.68-48.07 6.01  -0.10 -0.12         B9IV-V             +0.010+0.014      +003
8382          BD-05 5674 2088011457841580                   215342.0-045038215855.0-042223 54.10-43.03 6.22  +1.00 +0.87 +0.35E  K2V                +0.001-0.254 +.034-044V
8383          BD+62 2007 208816 197533756I W       VV Cep   215349.9+630857215639.1+633732104.92  7.05 4.91  +1.77 +0.39 +1.40   M2Iaep+B8Ve       v-0.005-0.004 +.004-019SBO     0: 1.7   0.1      *
8384          BD+65 1691 208947 19760    I                  215437.7+654044215711.1+660922106.55  9.00 6.43  -0.05 -0.68         B2V                +0.005+0.001      +002SBO   250  0.2  91.       *
8385 18    PegBD+06 4940 2090081272193759                   215508.1+061417220007.9+064303 65.80-36.51 6.00  -0.12 -0.57         B3III              +0.015+0.004      -007V?     78
8386 12Eta PsACD-2918119 209014190822       15536  13993    215505.6-285601220050.2-282713 20.70-52.44 5.42  -0.09 -0.37 -0.07   B8V               e+0.016+0.006D+.011-005V          1.0   1.9      *
8387   Eps IndCP-5710015 209100247287 825                   215542.6-571148220321.6-564710336.28-48.01 4.69  +1.06 +0.99 +0.55   K4-5V              +3.961-2.538 +.285-040                          *
8388          BD+62 2010 209112 19773    I                  215556.6+621307215853.4+624154104.54  6.16 5.93  +1.67 +1.81         M3IIIab            +0.004+0.026      -016V?
8389          BD+56 2670 209124 33943                       215601.9+571046215923.0+573930101.51  2.13 6.59  +0.03 -0.01         A0III-IV           +0.004-0.015      -003                          *
8390 28    AqrBD-00 4296 209128127235    I                  215558.0+000729220105.0+003618 59.95-40.57 5.58  +1.28 +1.40        gK4                 +0.010-0.013      +007
8391          BD+32 4316 209149 71932                       215603.0+323128220026.8+330022 86.30-17.44 6.46                      F5III              -0.006+0.064      -002       40                 *
8392 20    PegBD+12 4737 209166107587 826   15543           215613.0+123827220105.4+130711 71.68-32.32 5.60  +0.34 +0.05         F4III              +0.059-0.057 +.038+007V    < 20  6.4  54.7
8393 19    PegBD+07 4779 209167127239    I                  215611.4+074636220109.2+081526 67.43-35.67 5.65  +1.44 +1.77        gK5                 -0.009-0.002      -023V?
8394          BD-18 6056 2092401648273761                   215641.6-182300220211.9-175413 37.15-50.04 6.28  +1.01              gG7                 +0.123-0.060      -017
8395          BD+74  946 209258 102083758I                  215653.8+743105215751.0+745948112.28 15.84 6.35  +1.56 +1.61         K5                 -0.007-0.012      -017
8396 29    AqrBD-17 6422 209278164830       15562A DX Aqr   215658.3-172646220226.6-165751 38.54-49.74 6.37  +0.42               A2V+K0III          +0.015+0.001D+.006+015SBO        0.1   3.6AB   3*
8397          BD+10 4676 209288107599              14001    215706.6+102931220201.4+105826 70.03-33.98 6.37  -0.10 -0.61         B5IIIn             +0.022+0.003      -015V?
8398          CD-3018975 209335213502                       215730.1-302312220317.2-295415 18.43-53.16 7.10  +0.32               F0IIIn             +0.122-0.014      -012
8399          BD+61 2233 209339 19792      W       13998    215739.7+620024220039.3+622917104.58  5.87 6.66  +0.06 -0.82         B0IV               -0.001+0.007      -020V?    213  2.0   0.9      *
8400 16    CepBD+72 1009 209369 10216      W                215749.3+724214215915.0+731048111.17 14.38 5.03  +0.44 -0.03         F5V                -0.068-0.161 +.033-021V      26  6.8 131.4      *
8401 30    AqrBD-07 5688 209396145836                       215800.9-070020220316.4-063121 52.41-45.11 5.54  +0.96 +0.73         K0III              +0.045+0.006      +030
8402 31Omi AqrBD-02 5681 2094091458373765          Omi Aqr  215808.5-023817220318.9-020919 57.44-42.67 4.69  -0.06 -0.41 -0.05   B7IVe              +0.028-0.012      +012V?    227                 *
8403          BD+52 3083 209419 339853763                   215810.6+522400220150.6+525256 98.91-01.89 5.78  -0.11 -0.50         B5III              +0.011+0.006      -022V?                        *
8404 21    PegBD+10 4681 209459107625                       215824.6+105412220319.0+112311 70.66-33.92 5.80  -0.07 -0.20         B9.5V              +0.017-0.012      -000        4
8405 13    PsACD-3018985 209476213517    I                  215838.0-302403220423.9-295500 18.45-53.40 6.47  +1.63 +1.93         K5III              +0.014+0.001      +023V
8406 14    CepBD+57 2441 209481 33990              LZ Cep   215842.8+573104220204.6+580002102.01  2.18 5.56  +0.06 -0.86         O9Vn               -0.002 0.000 -.009-007SB2O  130:                *
8407          BD+43 4119 209515 51595       15578  V1942 Cyg215854.4+441003220256.7+443900 94.08-08.58 5.60  -0.03 -0.10 -0.01   A0IV               -0.018-0.033      -001V?     98  2.5   1.0AB   5*
8408          CD-2715757 2095221908643767          14011    215855.6-271823220436.8-264921 23.58-53.01 5.96  -0.14 -0.66 -0.15   B4IVne            v+0.017-0.015      -018V     300                 *
8409   Kap2IndCP-60 7541 209529247303                       215849.7-600711220550.9-593810332.26-46.96 5.62  +1.46 +1.81 +0.56E  K4III              +0.048-0.056      -011
8410 32    AqrBD-01 4242 209625145853                       215938.8-012324220447.4-005424 59.10-42.22 5.30  +0.23 +0.15 +0.11   A5m               v-0.015-0.052 +.014+020SB1O   19                 *
8411   Lam GruCD-4014639 2096882135431581                   220005.3-400133220606.9-393236  2.22-53.67 4.46  +1.37 +1.66 +0.78   K3III              -0.023-0.124 +.013+039
8412          BD+32 4329 209693 71998       15602           220009.2+322725220434.4+325631 86.97-18.03 6.38  +1.10 +0.80?        G5Ia               -0.007+0.001      -022V?         4.8  21.4
8413 22Nu  PegBD+04 4800 209747127285    I         14020    220038.1+043411220540.8+050331 65.37-38.64 4.84  +1.44 +1.81 +0.76   K4III              +0.111+0.100 +.014-016SB   < 17
8414 34Alp AqrBD-01 4246 209750145862 827I W                220038.9-004821220547.0-001911 59.93-42.07 2.96  +0.98 +0.74 +0.49   G2Ib              v+0.020-0.010 +.012+008V?   < 17  9.0 113.0      *
8415          BD+25 4671 209761 90214    I                  220036.2+261114220511.4+264026 82.85-22.98 5.78  +1.25               K2III              +0.034+0.032      -025V
8416 18    CepBD+62 2028 209772 19828    I         MO Cep   220053.1+623800220352.9+630711105.26  6.15 5.29  +1.58 +1.79 +1.26E  M5IIIab            +0.031+0.055 -.006-004                          *
8417 17Xi  CepBD+63 1802 209790 19827       15600A          220053.7+640826220347.4+643740106.15  7.37 4.29  +0.34 +0.09 +0.13   A3/6Vm             +0.209+0.085 +.032-006SB2O   20  2.0   7.7AB   3*
8418 33Iot AqrBD-14 6209 209819164861 828                   220102.2-142118220626.2-135211 43.51-49.37 4.27  -0.07 -0.29 -0.09   B9IV-V             +0.042-0.056      -010SB2   169
8419 23    PegBD+28 4284 209833 902173768                   220102.7+282841220534.7+285750 84.49-21.27 5.70  -0.06 -0.21         B9Vn               +0.031-0.012      -012SB    360
8420          CP-76 1547 209855258003                       220115.7-762212221042.5-755250314.37-37.45 6.55  +1.18 +1.30         K2IIICNII          +0.035-0.002      +051
8421          BD+46 3574 209857 51632    I         HT Lac   220118.0+461533220516.4+464441 95.68-07.14 6.13  +1.61 +1.76 +1.56   M4IIIab            -0.042-0.020      -013                          *
8422          BD+44 4041 209932 51640                       220147.5+443733220550.6+450644 94.77-08.52 6.44  -0.03 -0.14         A0V                +0.026-0.009      -004SB     73
8423          BD+82  673 209942  3673       15571A          220149.0+822319215812.7+825211117.67 21.84 6.98  +0.52 -0.02         F6IV-V             -0.141-0.041 +.011-022SB2    30  0.5  13.7AB   3*
8424          BD+44 4043 209945 516503769I         14028    220158.9+443140220602.0+450052 94.74-08.61 5.14  +1.57 +1.94 +0.92   K5III              -0.007-0.016 +.006-023V    < 17
8425   Alp GruCD-4714063 209952230992 829  W       14040    220155.9-472643220814.0-465740350.00-52.47 1.74  -0.13 -0.47 -0.09   B7IV              e+0.129-0.151 +.057+012      236 10.1  28.4      *
8426 20    CepBD+62 2029 209960 19847 830I         Var?     220158.0+621752220500.5+624708105.16  5.81 5.27  +1.41 +1.78         K4III              +0.015+0.058 -.000-021V?   < 17                 *
8427          BD+47 3692 209961 51645              V365 Lac 220155.8+474441220551.2+481354 96.64-06.01 6.27  -0.06 -0.71         B2V                -0.005+0.004      -018SB1O  160                 *
8428 19    CepBD+61 2246 209975 19849       15624           220203.8+614736220508.9+621648104.87  5.39 5.11  +0.08 -0.84 +0.03   O9.5Ib             +0.003 0.000      -013V      33  5.6  60.4AC   3*
8429          BD+44 4044 209993 51652                       220209.3+444542220612.4+451455 94.91-08.44 6.19  +0.09 +0.14         A3V                +0.036-0.010      -002
8430 24Iot PegBD+24 4533 210027 90238 831I W       14034    220221.2+245124220700.7+252042 82.26-24.26 3.76  +0.44 -0.04 +0.25   F5V                +0.298+0.025 +.082-004SB1O    7  7.4 103.7      *
8431 14Mu  PsACD-3315922 210049213576 832                   220233.0-332835220823.0-325919 13.34-54.46 4.50  +0.05 +0.05 +0.05   A2V                +0.080-0.031 +.030+012      226
8432          CP-76 1549 210056258006                       220228.7-763621221155.3-760658314.09-37.34 6.15  +1.00 +0.82         K0III              -0.044-0.043      +025
8433   Ups PsACD-3415421 210066213577    I                  220234.8-343153220826.0-340238 11.53-54.48 4.99  +1.48 +1.81 +0.60E  M1III              +0.008-0.049 +.009+020                          *
8434          BD+55 2679 210071 34055                       220242.0+555121220613.5+562035101.47  0.52 6.39  -0.10 -0.45         A0III              +0.014-0.011      -020V?                        *
8435          BD+18 4930 210074107675                       220242.7+185910220728.6+192832 78.06-28.77 5.75R                    dF2                 +0.125+0.037 +.016-015V?     49                 *
8436          BD+17 4693 210090107676    I         14039    220242.9+173048220730.0+180002 76.93-29.87 6.35  +1.64               M1                 +0.025-0.040      -010                          *
8437          CD-3315926 210111213583                       220253.0-333656220842.7-330732 13.11-54.53 6.37  +0.20               A2III/IV           +0.022+0.029      -004
8438 25    PegBD+21 4695 210129 90252                       220308.4+211258220750.3+214210 79.80-27.15 5.78  -0.10 -0.41         B7Vne             v-0.047-0.069      -052V     250                 *
8439 35    AqrBD-19 6227 210191164888                       220330.0-190033220859.0-183111 37.15-51.76 5.81  -0.17               B2.5IV             +0.003-0.012      -005
8440          CD-4814143 210204231005                       220338.5-483547220958.0-480626348.06-52.39 6.43  +1.39               K3III              +0.077-0.047      +019
8441          BD+24 4540 210210 90259                       220340.2+250319220817.2+253237 82.64-24.29 6.11  +0.27 +0.12         F1IV               -0.038-0.037      +002SB     75
8442          BD+58 2393 210220 340723770                   220347.3+582109220709.6+585027103.04  2.47 6.32  +0.88 +0.63         G6III              -0.013-0.022 +.017-010V?
8443          BD+52 3114 210221 34076                       220343.3+524907220725.5+531826 99.84-02.05 6.14  +0.43 +0.24         A3Ib               -0.007-0.009      -026       26
8444          CD-3415430 210271213599              14049    220405.3-343023220955.7-340053 11.58-54.79 5.37  +0.24               A5V                -0.017+0.038 +.027+002      201
8445          BD+49 3746 210289 51678    I  15659           220423.4+491827220816.5+494747 97.89-04.97 6.42R                     K5III              +0.029-0.023      +017V?         6.7  38.5
8446          CD-2817622 210300190930                       220418.1-284702221000.1-281733 21.44-54.42 6.44  +0.15               A5V                +0.032+0.026      +011
8447 15Tau PsACD-3315941 210302213602                       220417.3-330222221008.8-323254 14.12-54.81 4.92  +0.48 +0.01         F6V                +0.434+0.013 +.053-015
8448          BD+45 3813 210334 51684              AR Lac   220438.4+451503220841.0+454431 95.56-08.30 6.11  +0.72 +0.26 +0.33E  G2IV+K0III         -0.027+0.031      -035SBO                       *
8449 27Pi 1PegBD+32 4349 210354 72064 833   15672           220447.7+324101220913.6+331020 87.93-18.45 5.58  +1.00 +0.77         G8IIIb             -0.059-0.067 +.006-006V?         5.8  70.2AC   5*
8450 26The PegBD+05 4961 210418127340 834          14057    220509.3+054221221012.0+061152 67.41-38.71 3.53  +0.08 +0.10 +0.04   A2Vp               +0.276+0.027 +.049-006SB2   117                 *
8451          BD-04 5623 210419145914                       220509.0-042303221021.1-035339 56.90-45.11 6.27  -0.01 -0.07         A1Vnn              +0.019-0.050      -010      254:
8452 38    AqrBD-12 6196 2104241649103771                   220516.7-120325221037.5-113354 47.36-49.23 5.46  -0.12               B7III              +0.036+0.009      +003V?     18
8453          BD-04 5625 210434145916                       220520.9-044531221033.8-041602 56.51-45.37 6.01  +0.98 +0.83         K0III-IV           +0.076-0.003      -018V
8454 29Pi 2PegBD+32 4352 210459 72077 835                   220532.7+324115220959.2+331042 88.06-18.54 4.29  +0.46 +0.18 +0.27   F5III              -0.014-0.021 +.007+002      139                 *
8455          BD+18 4946 210460107706      S                220533.0+190739221019.0+193701 78.74-29.11 6.18  +0.69 +0.17         G0V                +0.086-0.075      +022V?                        *
8456          BD+13 4861 210461107707       15690           220530.6+140820221022.2+143748 74.85-32.81 6.33  +1.08 +0.94         K0III              +0.034-0.026      -042V?         5.1  21.5
8457          BD-21 6173 210464190945                       220529.5-214325221102.4-211357 33.20-53.10 6.09  +0.50               F6V                +0.122-0.033      -013V?
8458          BD+10 4701 2105021077123773I                  220542.9+110803221037.4+113728 72.39-35.02 5.78  +1.58 +1.94         M1III              -0.046-0.054      +017SB
8459 28    PegBD+20 5093 210516 902873772                   220546.5+202911221030.2+205841 79.80-28.11 6.46  +0.12 +0.11         A3III               0.000-0.008      +008
8460          BD+29 4604 210594 72095                       220620.6+300340221051.6+303311 86.51-20.74 6.32  +0.18 +0.16         A8IV               +0.008-0.009      +004       86
8461          BD+15 4592 210702107729                       220701.5+153252221151.3+160226 76.29-32.02 5.95  +0.95 +0.73 +0.49   K1III              -0.008-0.019      +011V?                        *
8462 39    AqrBD-14 6229 210705164923              14068    220702.2-144111221225.8-141138 44.03-50.82 6.03  +0.38  0.00         F2V                +0.028-0.042      +015V?
8463          BD+50 3602 210715 341433774   15708           220716.6+501945221109.9+504924 98.86-04.40 5.40  +0.15 +0.05         A5V                +0.139+0.041 +.008-008V?    126  4.1  73.8AD   4*
8464          CD-2616033 210739190967                       220719.4-264916221257.5-261940 24.97-54.75 6.17  +0.17               A3V                -0.019-0.033      +015
8465 21Zet CepBD+57 2475 210745 34137 836I         14066    220723.0+574230221051.3+581204103.06  1.67 3.35  +1.57 +1.71 +0.78   K1.5Ib             +0.015+0.004 +.017-018SB   < 17                 *
8466          BD+24 4548 210762 90317    I                  220729.0+242725221208.0+245700 82.97-25.30 5.92  +1.51               K0                 -0.004-0.022      -003
8467          BD-05 5732 210763145940                       220731.4-051250221243.8-044315 56.42-46.07 6.39  +0.50 +0.06         F7V                -0.051-0.035      +002SB2    12
8468 24    CepBD+71 1111 210807 10265 837I                  220753.1+715055220948.4+722028111.28 13.23 4.79  +0.92 +0.61 +0.48   G7II-III           +0.032+0.003 +.017-015V    < 19:
8469 22Lam CepBD+58 2402 210839 34149    I         14069    220806.8+585516221130.7+592452103.83  2.61 5.04  +0.25 -0.74 +0.15   O6I(n)fp           +0.003-0.011 +.006-074V?    285                 *
8470          CD-2515815 210848190976                       220807.4-254034221344.4-251051 26.98-54.70 5.58  +0.49 +0.22         F6III-II           +0.074+0.016      -028
8471   Psi OctCP-78 1442 2108532580203779                   220807.0-780033221750.5-773042312.56-36.59 5.51  +0.31 +0.12         F3III              -0.041+0.011      +017                          *
8472          BD+56 2727 210855 341513775  W                220811.9+562031221148.8+565022102.38  0.48 5.24  +0.51 +0.06         F8V                +0.237+0.130 +.024-019V       0  5.1  72.9AB   3*
8473          BD+71 1112 210873 10267                       220817.6+713708221015.3+720640111.17 13.03 6.37  -0.06 -0.18 -0.07   B9pHgMn:           -0.009-0.028      -003       41                 *
8474          BD+69 1228 210884 19922       15719           220821.8+693819221038.9+700758110.00 11.41 5.50  +0.38 -0.04         F2V                -0.060+0.028 +.038+001V?         3.1  14.6      *
8475          BD+33 4456 210889 721323776I                  220822.2+340641221247.8+343617 89.46-17.76 5.33  +1.13 +1.13         K2-III-IIIb        +0.020-0.050 +.003-007V?   < 19:
8476          BD+58 2403 210905 34158                       220829.7+583518221156.9+590505103.68  2.31 6.30  +1.13 +1.07         K0III              +0.129+0.083      -028                          *
8477          CD-4114804 210918231045    I'                 220832.0-415119221438.6-412254358.68-54.97 6.23  +0.65 +0.14         G5V                +0.567-0.790 +.046-018V?
8478 16Lam PsACD-2817653 210934190985 838                   220838.7-281545221418.8-274601 22.58-55.28 5.43  -0.16 -0.55         B8III              +0.026+0.002 +.012-006       38
8479          BD+60 2358 210939 19932    I W                220843.4+601552221201.9+604534104.65  3.68 5.35  +1.17 +1.15         K1III              -0.008+0.019      -003                          *
8480 41    AqrBD-21 6180 210960190986       15753           220846.7-213418221418.0-210427 33.83-53.78 5.32  +0.80 +0.43         K0III+F2V          +0.016+0.066 +.016-024           1.6   5.1AB   4*
8481   Eps OctCP-81  995 210967258928 839I         Eps Oct  220849.8-805615222001.5-802623310.04-34.50 5.10  +1.47 +1.09 +1.49E  M5III              +0.043-0.045      +012                          *
8482          BD+27 4280 211006 90337                       220903.6+280646221338.6+283630 85.75-22.64 5.89  +1.15 +1.22         K2III              +0.071-0.001      -019V?
8483          BD+62 2048 211029 19937    I                  220915.5+624748221222.3+631729106.14  5.73 5.79  +1.67 +1.93         M3IIIab            -0.006-0.010      -014V?
8484          CD-4514644 211053231053                       220927.8-445654221535.1-442707353.42-54.42 6.10  +1.02 +0.81 +0.46C  G8-K0III           +0.013-0.019      -004
8485          BD+38 4711 211073 721551583I  15758  14076    220935.1+391307221352.7+394254 92.76-13.75 4.49  +1.39 +1.45 +0.74   K3III              +0.038+0.013 +.020-011SB   < 17  6.3  28.4AB   4*
8486   Mu 1GruCD-4114810 2110882310553777                   220935.5-415039221536.9-412048358.65-55.16 4.79  +0.80 +0.47         G8III+G            +0.044+0.022 +.009-007SB
8487          BD+44 4073 211096 51783                       220942.3+445641221349.3+452627 96.12-09.07 5.53  +0.04  0.00         A0III              +0.081+0.009      -009      111
8488   Mu 2GruCD-4215846 211202231063                       221025.6-420729221626.6-413739358.11-55.26 5.10  +0.92               G8III              -0.005-0.013 +.010+013SB
8489          BD+42 4333 211211 517973778                   221032.2+422728221444.4+425714 94.81-11.20 5.71   0.00 -0.04         A2Vnn              +0.049-0.025      -038V?    245
8490          BD+62 2053 211242 19948                       221040.8+623958221349.5+630945106.20  5.53 6.11  -0.08 -0.43         B8Vn               -0.020-0.003      +012SB
8491          BD+07 4834 2112871274203780                   221101.0+080308221559.8+083258 70.86-38.13 6.21  +0.02 -0.04         A1Vn               +0.034-0.011      +000SB    305
8492          CD-2616057 211291191016                       221100.4-262346221637.4-255354 25.98-55.48 6.15  +1.11               K1II/III           +0.037-0.018      +026V?
8493          BD+72 1022 211300 10284       15764  14078    221104.1+724838221252.9+731826112.05 13.88 6.08  +1.01 +0.85         K0II-III+A3V       +0.023+0.020 +.015+001V?         2.3  28.7      *
8494 23Eps CepBD+56 2741 211336 34227      W       Eps Cep  221120.9+563241221502.0+570237102.86  0.40 4.19  +0.28 +0.04 +0.15   F0IV               +0.447+0.048 +.041-001SB     86  5.3 127.8      *
8495          BD-02 5726 211356145989                       221124.5-020540221633.6-013547 60.82-44.98 6.15  +0.19 +0.09         A5Vn               -0.029-0.011      +001      199
8496 42    AqrBD-13 6148 211361164974    I                  221126.8-131948221648.1-124953 46.72-51.16 5.34  +1.14 +1.07        gK0                 +0.015+0.006      +013V
8497          CD-2317344 211364191021                       221126.1-233814221659.8-230824 30.73-54.95 6.17  +1.05               G5                 +0.084-0.055      -002V
8498  1    LacBD+37 4526 211388 72191    I                  221136.5+371502221558.2+374456 91.93-15.59 4.13  +1.46 +1.63 +0.72   K3-II-III          +0.013+0.004 +.003-008V?   < 19:
8499 43The AqrBD-08 5845 211391145991 840I                  221133.4-081653221650.0-074700 53.51-48.62 4.16  +0.98 +0.81 +0.48   G8III-IV           +0.121-0.022 +.021-015V?   < 17                 *
8500          BD-09 5948 211392145992              14095    221135.8-093218221652.6-090224 51.90-49.30 5.79  +1.16 +1.14        gK3                 -0.047-0.009      +012
8501          CP-5410055 211415247400      W                221142.3-540631221815.6-533740339.03-51.39 5.37  +0.60 +0.08         G3V                +0.429-0.667 +.083-014           4.5   3.0
8502   Alp TucCP-60 7561 211416255193 841  A                221139.2-604529221830.1-601535330.22-47.96 2.86  +1.39 +1.54 +0.73   K3III              -0.072-0.043 +.026+042SBO              0.1      *
8503          BD+27 4288 211432 90379                       221153.4+271821221629.7+274815 85.75-23.66 6.37  +0.99               G9III              -0.009+0.002      +016
8504 44    AqrBD-06 5960 2114341459933782          14100    221153.2-055312221706.5-052314 56.54-47.35 5.75  +0.88 +0.51         G6III              -0.002+0.020      +007
8505   Ups OctCP-86  406 2115392589321670                   221235.4-862834223137.4-855802305.61-30.41 5.77  +1.02 +0.88         K0III              -0.041+0.059      +019V
8506          BD+56 2746 211554 34256                       221249.7+564316221626.5+571313103.12  0.43 5.88  +0.93               G8III              +0.043+0.006      -008V
8507          BD-00 4333 211575146004      S                221256.6-004410221804.3-001416 62.64-44.42 6.39  +0.44  0.00         F3V                -0.039-0.062      +017V      12        0.1
8508 45    AqrBD-14 6255 211676164996                       221338.7-134820221900.8-131818 46.45-51.85 5.95  +1.06 +0.94        gG7                 +0.079-0.012      +030
8509          CP-58 7942 211726247410                       221356.7-580041222036.2-573036333.47-49.71 6.34  +1.37 +1.71         K3-4III            +0.022-0.015      -034
8510          BD+37 4537 211797 72228       15828           221433.0+371601221856.2+374610 92.44-15.91 6.17  +0.28 +0.11         A9IIIp             +0.057+0.044D+.013+007           2.7  15.7AB   4*
8511 25    CepBD+62 2059 211833 19991    I                  221456.9+621811221812.7+624816106.41  4.96 5.75  +1.26 +1.37         K3III              +0.046+0.015      -002V
8512 46Rho AqrBD-08 5855 211838146023                       221456.2-081924222011.9-074916 54.15-49.34 5.37  -0.06 -0.37         B8IIIpMn:Hg:       +0.016-0.001      -009V      72
8513 30    PegBD+05 4998 211924127453       15847  14122    221525.6+051713222027.6+054722 69.30-40.86 5.37  -0.02 -0.47         B5IV               +0.023-0.001      -008V      32  5.9  14.8AC   3*
8514          BD+07 4853 211976127460                       221556.1+074059222055.8+081112 71.63-39.25 6.17  +0.44 -0.03         F6V                +0.048+0.023      +010        7
8515   Nu  IndCP-72 2690 211998258033      W                221602.0-724429222436.8-721520316.86-40.67 5.29  +0.65 -0.06 +0.41E  A3V:+F9V           +1.299-0.678 +.031+021V?         0.1   0.1      *
8516 47    AqrBD-22 5897 2120101910831584I                  221605.3-220558222135.6-213554 33.84-55.56 5.13  +1.07 +0.92         K2III              -0.008-0.085 +.015+049
8517          BD+26 4410 212047 90437    I         14127    221621.3+262556222100.1+265607 86.05-24.96 6.47  +1.59 +1.71         M4III              +0.015-0.006      -004
8518 48Gam AqrBD-02 5741 212061146044 842   15864  14132    221629.5-015329222139.4-012314 62.18-45.84 3.84  -0.05 -0.12 -0.04   A0V                +0.132+0.007 +.046-015SB     57  7.9  37.4      *
8519          BD+50 3673 212071 34335                       221642.0+502839222039.6+505851100.20-05.11 6.42R                     K2                 +0.012+0.004      -009
8520 31    PegBD+11 4784 212076107854 843          IN Peg   221635.7+114205222131.1+121219 75.27-36.43 5.01  -0.13 -0.81 -0.14   B2IV-Ve           v+0.009+0.006      +010V?    134                 *
8521   Pi 1GruCD-4614292 212087231105      W       Pi1 Gru  221637.4-462707222243.9-455652350.28-55.16 6.62  +2.01 +1.86 +2.64   S5,7e              +0.007-0.012      -020           4.3   2.7      *
8522 32    PegBD+27 4299 212097 90440       15863           221642.2+274936222119.3+281950 87.03-23.88 4.81   0.00 -0.20 -0.01   B9III              +0.018+0.002 +.023+008V?     81  6.3  72.6AB   5
8523  2    LacBD+45 3894 212120 51904       15862  14130    221653.6+460159222101.6+463212 97.79-08.86 4.57  -0.10 -0.51 -0.11   B6V                +0.023+0.005 +.040-010SB2O   47  6.4  48.2      *
8524   Pi 2GruCD-4614295 2121322311113786I:W                221659.5-462554222308.0-455543350.28-55.23 5.62  +0.36 +0.02 +0.22C  F3III-IV           +0.237-0.061 +.027+020V          5.4   4.9      *
8525          BD+75  820 212150 103123784                   221708.1+755908221820.4+762917114.26 16.27 6.66   0.00 -0.11         A1Vn               +0.005+0.003      -018      165:
8526          CP-75 1748 2121682580363789  W                221710.5-753120222551.0-750056314.28-38.73 6.04  +0.64 +0.14         G3V                +0.048+0.010      +014           2.7  20.4
8527          CP-71 2686 2122112580343788                   221721.5-705609222510.5-702554318.51-41.99 5.78  +0.38 +0.03         F3III              +0.115-0.062      +004
8528          BD+41 4469 212222 51918                       221734.0+413426222150.9+420442 95.41-12.66 6.41  -0.08 -0.51         B5V                +0.019 0.000      -018
8529 49    AqrCD-2515905 212271191105                       221756.5-251605222330.9-244545 28.53-56.78 5.53  +0.99 +0.80        gG9                 +0.103 0.000      -011V?
8530          BD-07 5765 212320146062      D                221817.4-074201222332.1-071140 55.66-49.69 5.93  +1.00 +0.69         G6IIIBaII          -0.001+0.007      -014                          *
8531          CP-58 7954 212330247441    I'W                221818.0-581739222456.4-574750332.61-50.04 5.32  +0.67 +0.13 +0.39   G3IV               +0.143-0.343 +.058+008           7.6  81.
8532 33    PegBD+20 5139 212395 90462       15896           221850.8+202034222339.6+205054 82.41-30.16 6.04R                     F7V                +0.337-0.015 +.037-023V          2.2  78.4AC   3*
8533 51    AqrBD-05 5780 212404146067       15902           221854.3-052035222406.9-045013 58.72-48.44 5.78  -0.04 -0.11         A0V                +0.033-0.004D+.006+006       65  0.0   0.3AB   5*
8534 50    AqrBD-14 6276 2124301650443791                   221905.7-140211222427.1-133146 47.12-53.13 5.76  +0.97 +0.71         G6.5III            +0.053+0.012      -021V?
8535          BD+56 2765 212454 34383                       221918.7+564643222300.2+571704103.90  0.00 6.16  -0.13 -0.55         B8III-IV           +0.015+0.001      -013                          *
8536          BD+37 4560 212487 72296                       221928.6+380350222354.2+383425 93.74-15.80 6.22  +0.49 -0.01         F5IV:              +0.255+0.119 -.005+005    =< 10
8537          BD+61 2291 212495 20042                       221938.5+615447222300.2+622512106.66  4.33 6.04  +0.05 +0.02         A1V                -0.005+0.040      -015V     165
8538  3Bet LacBD+51 3358 212496 34395 844I S                221937.5+514341222333.6+521345101.26-04.30 4.43  +1.02 +0.77 +0.57   G8.5IIIbCa1        -0.013-0.186 +.021-010     < 17
8539 52Pi  AqrBD+00 4872 2125711275201585I         Pi  Aqr  222010.2+005212222516.6+012239 66.01-44.74 4.66  -0.03 -0.98 +0.02   B1Ve              v+0.019+0.001      +004V     278                 *
8540   Del TucCP-65 4044 212581255222      W       14158    222013.4-652830222720.0-645759323.87-45.87 4.48  -0.03 -0.07         B9.5V              +0.071+0.003      +012      245  4.5   6.9      *
8541  4    LacBD+48 3715 212593 51970                       222027.6+485810222431.0+492835 99.90-06.71 4.57  +0.09 -0.34 +0.10   B9Iab              -0.007-0.003 +.003-026       27
8542          CD-2417171 2126431911443793                   222038.9-241126222610.7-234057 30.71-57.13 6.29  -0.03               A0V                +0.018-0.002      -015V
8543          BD+17 4746 2126701079061586                   222051.1+175608222540.7+182640 81.10-32.34 6.26  +1.22               K0                 +0.028+0.027      +022
8544 53    AqrBD-17 6520 212697165077       15934B          222108.3-171459222634.2-164429 42.55-54.98 6.57H                     G2V                +0.262-0.013 +.057-003V?      7  0.2   3.3AB   4*
8545 53    AqrBD-17 6521 212698165078       15934A          222108.7-171503222634.4-164433 42.55-54.98 6.35H +0.61 +0.09         G1V                +0.223-0.003 +.057-006        9  0.2   3.3AB   4*
8546          BD+85  383 212710  37211648                   222117.8+853617221310.6+860629120.22 24.10 5.27  -0.03 -0.11         B9.5Vn             +0.052+0.045 +.006+004SB2   219
8547          CP-68 3493 2127282552271587                   222116.0-675948222837.7-672921321.11-44.26 5.55  +0.20 +0.08         A4V                +0.149-0.072      -017
8548 34    PegBD+03 4705 212754127529       15935           222132.0+035301222637.4+042337 69.38-42.93 5.75  +0.52 +0.04         F7V                +0.301+0.048 +.041-018V       7  6.5   3.3AB   3*
8549          BD+36 4835 212883 72344       15942           222219.2+365605222645.7+372638 93.58-17.05 6.46  -0.13 -0.75         B2V                +0.013-0.003      -007       25  3.3   4.3      *
8550          BD+77  860 212937 10351                       222251.4+774406222341.3+781436115.55 17.55 6.76  -0.06 -0.16         B9                 -0.008+0.018      -018                          *
8551 35    PegBD+03 4710 2129431275403796I W                222247.8+041139222751.5+044144 69.98-42.94 4.79  +1.05 +0.89 +0.56   K0III              +0.081-0.308 +.025+054V    < 19: 4.9 181.5AC   3
8552   Nu  GruCD-3914723 212953213850 845  W                222247.6-393816222839.2-390755  1.71-58.04 5.47  +0.95               G9III              +0.039-0.164 +.015+011           7.0  27.3
8553          BD+39 4841 212978 72358      S                222303.4+391801222726.5+394835 95.04-15.14 6.14  -0.14 -0.77         B2V                +0.001-0.011      -017      115        0.2      *
8554          BD+55 2750 212986 344603795                   222311.8+555525222659.2+562600103.91-01.01 6.57  -0.10 -0.47         B5III              +0.019 0.000      -007V?
8555          BD+31 4701 212988 72366    I                  222311.4+311945222746.2+315025 90.46-21.82 5.98  +1.45               K2                 +0.045+0.034      +001
8556   Del1GruCD-4414931 213009231154 846  W       14169    222317.6-440023222916.2-432944353.77-57.05 3.97  +1.03 +0.80 +0.56   G6-8III            +0.028-0.005 +.024+005           8.8   5.6
8557          BD+70 1240 213022 103663794I                  222325.6+701541222600.8+704615111.45 11.22 5.47  +1.20              gK2                 +0.018+0.006      -017V
8558 55Zet1AqrBD-00 4365 213051146107    I: 15971B          222340.8-003153222849.7-000113 65.35-46.33 4.59H                     F6IV               +0.181+0.009 +.022+029V?     58  0.2   1.8     3*
8559 55Zet2AqrBD-00 4365 213052146108    I  15971A          222341.0-003156222850.1-000112 65.35-46.33 4.42H +0.38 -0.01 +0.23   F3V                +0.209+0.042 +.022+025       58  0.2   1.8     3*
8560   Del2GruCD-4414935 213080231161    I'W       Del2 Gru 222347.1-441539222945.5-434458353.28-57.06 4.11  +1.57 +1.71 +1.58   M4.5IIIa           -0.007 0.000 -.005+002V?         4.7  60.6      *
8561 26    CepBD+64 1664 213087 200753797                   222352.2+643720222705.3+650756108.50  6.39 5.46  +0.37 -0.59         B0.5Ibe:           +0.001-0.002      -015V                         *
8562 36    PegBD+08 4874 2131191275441588I                  222408.5+083706222908.0+090744 74.39-39.97 5.58  +1.55 +1.91         K5IIIa             +0.055-0.022      -030V?
8563          CD-2715932 2131351911823798          14175    222410.4-273707222946.0-270626 24.69-58.58 5.95  +0.33               F1V                +0.123-0.012      +003
8564          BD+26 4439 213179 905441589                   222429.0+261507222910.2+264547 87.57-26.17 5.79  +1.25               K2II               +0.026-0.004      -045V
8565          BD-13 6204 213198165123                       222440.7-132537223001.5-125454 49.10-54.04 6.40  +0.32 -0.02         F3IV               +0.168+0.001      -011                          *
8566 37    PegBD+03 4713 213235127551       15988           222454.6+035525222958.0+042554 70.23-43.50 5.48  +0.38 +0.09 +0.20   F2V+F2V            -0.023-0.144 +.032+001SB1O   66  1.4   0.9      *
8567 56    AqrBD-15 6231 213236165127                       222455.8-150549223017.4-143509 46.61-54.87 6.37  -0.04 -0.37         B8V s              +0.039-0.039      -027       25                 *
8568          BD+63 1852 213242 20088                       222459.6+633429222819.7+640508108.06  5.43 6.29  +1.08               K0                 +0.023-0.013      -026
8569          BD+34 4700 213272 72399                       222514.1+351253222943.9+354332 93.12-18.81 6.56                      A2V                -0.026-0.037      -002
8570   Zet PsACD-2616175 213296191196              14180    222520.2-263504223053.7-260425 26.73-58.67 6.43  +1.08               K1III              +0.034-0.070      -026
8571 27Del CepBD+57 2548 213306 34508 847I  15987A Del Cep  222527.3+575412222910.3+582455105.19  0.53 3.75  +0.60       +0.40   F5Ib-G2Ib         v+0.015+0.001 +.011-015SB      9  2.4  40.8AC   3*
8572  5    LacBD+46 3719 213310 520553799I S       14177    222521.7+471142222931.8+474225 99.66-08.65 4.36  +1.68 +1.11 +1.07   M0II+B8V           +0.002 0.000 +.006-004SBO =< 50                 *
8573 57Sig AqrBD-11 5850 2133201651341591                   222521.3-111123223038.8-104041 52.49-53.05 4.82  -0.06 -0.11 -0.04   A0IV s             +0.003-0.030 +.021+011SB     23                 *
8574 38    PegBD+31 4708 213323 724061590  S                222527.3+320339223001.8+323421 91.33-21.48 5.65  -0.04 -0.11         B9.5V              +0.034-0.014      -016V     104        0.2
8575          BD+48 3747 213389 52073      W       V350 Lac 222559.6+485041223006.5+492122100.61-07.29 6.40  +1.15 +1.00         K2IIIe             -0.024-0.043      +004SB1O       3.2  66.0      *
8576 17Bet PsACD-3217126 2133982138831592  W                222549.3-325132223130.3-322046 14.59-59.32 4.29  +0.01 +0.02 +0.02   A0V                +0.065-0.018 +.021+006       36  3.5  30.3AD   4*
8577          CP-79 1206 2134022580493807                   222555.3-791712223526.4-784618310.69-36.22 6.15  +1.38 +1.35         K1III              +0.046-0.006      -013
8578 28Rho1CepBD+78  796 213403 10375                       222557.4+781634222642.5+784709116.00 17.91 5.83  +0.15?              A2Vm               -0.016-0.043      -006V?     58
8579  6    LacBD+42 4420 213420 520793800                   222610.2+423639223029.3+430724 97.36-12.64 4.51  -0.09 -0.74 -0.11   B2IV               -0.001-0.005      -008SBO    74                 *
8580          BD-03 5460 2134281461363802                   222608.2-032525223118.4-025440 62.68-48.67 6.16  +1.08 +1.00         K0                 -0.005-0.036      +010V
8581          BD-07 5797 213429146135      S                222603.6-070355223118.4-063318 58.21-50.90 6.14  +0.56 +0.06         F7V                +0.173-0.106 +.044-006V   =<  6        0.1
8582   Nu  TucCP-62 6348 2134422552473803          Nu  Tuc  222614.5-622945223300.1-615856326.58-48.30 4.81  +1.61 +1.75         M4III              +0.039-0.025 +.007-003
8583 58    AqrBD-11 5855 213464165147                       222623.2-112505223141.3-105420 52.39-53.39 6.38  +0.28 +0.07 +0.16   A8III              +0.075-0.045      +004SB
8584          BD+28 4389 213534 90568              GX Peg   222656.1+290148223134.2+293234 89.80-24.18 6.35  +0.19?              A5m                -0.044-0.029      +002SBO    48                 *
8585  7Alp LacBD+49 3875 213558 34542 848   16021           222710.2+494606223117.5+501657101.26-06.60 3.77  +0.01  0.00 -0.03   A1V                +0.138+0.019 +.040-004V?    146  8.3  36.3      *
8586 39    PegBD+19 4949 2136171079863804                   222745.5+194252223235.5+201348 83.91-31.92 6.42  +0.32 +0.02         F1V                +0.163+0.029 +.013-019                          *
8587          BD+15 4670 2136441079893806                   222753.6+152052223246.9+155148 80.76-35.42 6.32  +1.19 +1.17         K0                 +0.022+0.026      -028
8588          BD+39 4871 213660 724463805   16031           222801.0+391555223226.4+394647 95.87-15.68 5.88  +0.16 +0.14         A6V                +0.004-0.011      +005V?    111  5.5  43.1AB   5
8589          BD+53 2910 213720 34555                       222820.6+533119223218.8+540215103.32-03.45 6.35R                     G8III              +0.035+0.027      -014
8590 60    AqrBD-02 5781 213789146160      W                222853.6-020520223402.9-013427 64.89-48.33 5.89  +0.99 +0.73         G6III              +0.036-0.040      -008           4.4  98.1AB   3
8591 29Rho2CepBD+78  801 213798 104021593                   222859.8+781840222952.9+784927116.16 17.86 5.50  +0.06 +0.07         A3V                +0.001-0.022 +.008+001SB    115                 *
8592 59Ups AqrBD-21 6251 213845191235 849                   222913.5-211314223441.6-204230 37.08-58.20 5.20  +0.44  0.00         F4IV               +0.222-0.145 +.046-002       18
8593          CP-58 7971 213884247505                       222924.5-582402223552.9-575301331.15-51.17 6.23  +0.20               A5V                +0.077-0.009      +005
8594          BD+55 2769 213930 34574    I         14213    222947.8+560627223340.6+563730104.80-01.32 5.71  +0.96               G8III-IV           +0.073+0.047      -011SB                        *
8595          BD+69 1262 213973 20150       16057           223008.8+692345223302.9+695449111.50 10.17 6.6   +0.32  0.00         A9III              +0.116+0.068 +.002-002V?     77  0.3   0.5AB   4*
8596          CD-2417232 213986191246                       223006.4-243029223536.5-235928 31.10-59.30 5.97  +0.99 +0.81        gK0                 +0.039-0.006      -003V
8597 62Eta AqrBD-00 4384 213998146181 850                   223013.0-003759223521.4-000703 66.84-47.61 4.02  -0.09 -0.26 -0.07   B9IV-Vn            +0.091-0.056 +.025-008V?    280
8598          BD+69 1263 214019 10418       16062           223027.2+695126223317.0+702226111.76 10.56 6.34   0.00 -0.06         A0V                +0.051+0.020 -.003-019      165  3.0   9.7AB   3*
8599          BD+75  836 214035 104141594                   223030.9+754240223216.2+761335114.82 15.60 5.68  +0.05 +0.08         A2V                -0.021-0.011      -022SB    100
8600   Sig1GruCD-4114959 2140852312113811                   223038.9-410555223629.3-403458358.33-59.17 6.28  +0.12 +0.13         A3Vn               +0.048-0.070      +001SB
8601          CD-3217161 214122213948      W                223057.7-321050223635.4-313950 15.94-60.40 5.82  +1.09 +1.00         K1III              -0.042-0.044      +013           1.8  91.7
8602   Sig2GruCD-4114963 214150231217      W                223108.7-410626223658.8-403528358.27-59.26 5.86  +0.06 +0.06         A1V                +0.038-0.075      +015       58  5.2   2.8      *
8603  8    LacBD+38 4808 214168 72509       16095A 14242    223125.2+390700223552.3+393803 96.38-16.15 5.73  -0.15 -0.90         B2Ve              v+0.005-0.006      -010SB    348  0.7  22.4AB   8*
8604          BD+34 4728 214200 72517      W                223135.2+350340223607.9+353438 94.20-19.63 6.10  +1.00               K0                  0.000-0.067      -016                          *
8605          BD+10 4781 214203108036                       223138.9+111042223636.4+114149 78.37-39.24 6.40   0.00 +0.06         A1III              -0.008+0.018      -008
8606          BD+49 3903 214240 52171                       223144.0+493310223553.4+500416101.79-07.15 6.29  -0.05 -0.54         B3V                +0.006+0.008      -015SB2O   60                 *
8607          BD+55 2779 214279 34605                       223156.6+553309223551.8+560412104.79-01.95 6.38  +0.12 +0.09         A3V                -0.003-0.019      -002
8608          BD+11 4838 214298108043    I                  223208.1+120333223704.7+123438 79.21-38.63 6.30                      K5                 -0.030-0.017      -019
8609          BD+34 4729 214313 725353812                   223215.9+350802223648.7+353909 94.37-19.64 6.30  +1.37               K5                 -0.004 0.000      +010
8610 63Kap AqrBD-04 5716 2143761462101595I W                223234.6-044438223745.4-041341 62.68-50.75 5.03  +1.14 +1.16 +0.55   K2III              -0.067-0.120 +.019+008     < 19: 3.5  98.3      *
8611          CP-5310326 214441247531      W       CC Gru   223257.6-531242223908.4-524132337.71-54.56 6.65  +0.35               F1III              +0.005-0.016      +011V?         4.7  33.8      *
8612          BD-08 5912 214448146216       16130  14255    223307.4-082502223822.2-075352 58.10-53.10 6.23  +0.78 +0.49         G0III+F0V          +0.076-0.004      -017V?         1.8   0.2      *
8613  9    LacBD+50 3770 214454 34628                       223315.8+510144223722.4+513243102.73-05.99 4.63  +0.24 +0.10 +0.14   A8IV               -0.054-0.101 +.022+012SB     87
8614          CD-2918414 214462191292                       223311.0-291604223844.7-284452 21.91-60.74 6.47  +1.04               K1III              -0.035+0.001      +003V?
8615 31    CepBD+72 1049 214470 10425 851                   223317.8+730727223546.1+733835113.64 13.27 5.08  +0.39 +0.16         F3III-IV           +0.172+0.023 +.008+000V      85                 *
8616          CD-3316160 2144842139813814                   223312.5-333606223851.5-330453 13.03-60.84 5.66  +0.05               A2Vp               +0.006+0.014      +004SB2     0
8617          BD+44 4185 214558 52211      S                223357.7+443947223817.5+451059 99.67-11.59 6.40R                     G2III+A4V          +0.003 0.000      -004    =< 50        0.1      *
8618 40    PegBD+18 5014 214567108064      W                223402.2+190017223852.6+193120 84.84-33.37 5.82  +0.92               G8II               -0.042-0.099 +.010-020V          5.6   1.5
8619          CD-2817873 214599191308       16149A          223410.2-285043223944.0-281931 22.82-60.92 6.31  +1.01               K0IV               +0.076-0.031D+.010-022           1.3  86.6AB   3*
8620          CP-58 7984 2146322475443816                   223426.5-575637224048.9-572520331.07-51.97 5.97  +1.46 +1.70         K4III              +0.055-0.007      -032
8621          BD+56 2821 214665 34651    I  16140  14260    223441.8+561634223837.9+564745105.48-01.51 5.21R +1.58 +1.70         M4+III             +0.053-0.029 -.013+008           5.2  30.9      *
8622 10    LacBD+38 4826 214680 72575 852   16148           223446.3+383147223915.7+390301 96.65-16.98 4.88  -0.20 -1.04 -0.22   O9V                +0.001-0.005      -010V?     31  5.2  62.2      *
8623          CD-3118920 214690214000    I         14266    223447.6-311028224022.3-303932 18.02-61.20 5.87  +1.30 +1.44         K3III              -0.103-0.209 +.014+079
8624 41    PegBD+18 5021 214698108078                       223456.2+190937223947.0+194052 85.16-33.36 6.21R  0.00 +0.08         A2V                -0.004-0.010      -011V?     80
8625          BD+74  978 214710 10440    I                  223504.2+745106223713.0+752218114.63 14.71 5.79  +1.57 +1.91         M1IIIab            +0.044+0.006      -007
8626          BD+36 4902 214714 72581       16154           223502.8+370419223934.3+373534 95.93-18.27 6.03  +0.86               G3Ib-IICN-1CH2Fe-1 +0.008-0.001      -007V          5.7  19.5      *
8627 30    CepBD+62 2102 214734 20190 853                   223506.0+630352223839.0+633504108.79  4.42 5.19  +0.06  0.00         A3IV               -0.003-0.025 +.008+011SB2   140
8628 18Eps PsACD-2716010 214748191318 854          14270    223507.5-273354224039.4-270237 25.49-60.97 4.17  -0.11 -0.37 -0.09   B8V               e+0.030-0.001      +003      290
8629          BD-04 5728 214810146239      W                223537.5-040429224048.0-033315 64.27-50.90 6.31  +0.52  0.00         F6V                +0.003-0.042D+.012+012V          0.5   0.3AB   3*
8630   Bet OctCP-82  889 214846258941 924                   223551.2-815421224603.3-812254308.35-34.38 4.15  +0.20 +0.11 +0.10   A9IV-V             -0.067-0.001      +024SB     71
8631          BD+13 4971 214850108094       16173           223555.3+140125224052.7+143258 81.67-37.65 5.71  +0.72 +0.22         G4V                +0.268+0.144 +.032-010V          0.5   0.4AB   3*
8632 11    LacBD+43 4266 214868 52251    I                  223607.6+434515224030.9+441635 99.56-12.57 4.46  +1.33 +1.36 +0.68   K2+III-IIIb        +0.094+0.012 +.010-010V?   < 17                 *
8633          BD+53 2950 214878 34682                       223614.2+531929224018.4+535046104.25-04.20 5.93  +0.93               K0III              +0.001-0.014      -006
8634 42Zet PegBD+10 4797 214923108103 855   16182           223628.4+101833224127.7+104953 78.86-40.66 3.40  -0.09 -0.25 -0.07   B8V                +0.080-0.013 +.023+007V?    194  8.9  64.3      *
8635          CD-4714307 214953231257      W       14277    223639.3-474329224236.9-471238345.74-57.80 5.98  +0.58 +0.08 +0.32C  G0V                +0.009-0.326 +.067+017           4.0   7.8      *
8636   Bet GruCD-4714308 214952231258 856I         Bet Gru  223641.8-472427224240.1-465305346.27-57.95 2.10  +1.60 +1.67 +1.65   M5III              +0.137-0.008 +.008+002                          *
8637 19    PsACD-3019267 214966191337    I                  223648.4-295301224222.1-292139 20.76-61.58 6.17  +1.54 +1.90         M5III              +0.028-0.015      -009V?
8638          BD+30 4771 214979 72625    I                  223650.9+302636224131.4+305757 92.62-24.15 6.34R                     K5                 +0.063 0.000      -035
8639          CD-4415017 214987231260              14278    223650.3-444620224243.1-441452350.85-59.06 6.07  +0.98 +0.69 +0.46C  K0III              +0.042+0.037      +006V?                        *
8640 12    LacBD+39 4912 214993 72627      W       DD Lac   223659.9+394211224128.6+401332 97.65-16.18 5.25  -0.14 -0.87         B2III              -0.005-0.001      -015SB     53  4.0  69.0      *
8641 43Omi PegBD+28 4436 214994 90717                       223703.6+284708224145.4+291827 91.71-25.59 4.79  -0.01 -0.01 -0.02   A1IV               -0.004-0.027 +.024+009V      12                 *
8642          BD+13 4974 214995108109                       223701.1+135939224157.4+143059 81.92-37.83 5.90  +1.11              gK0                 +0.092-0.021 -.010-026V
8643          BD+40 4885 215030 52270                       223707.8+410129224136.1+413258 98.35-15.04 5.94R                     G9III              +0.148+0.065 +.001-014
8644   Rho GruCD-4216049 2151042312653818  W                223742.1-415608224330.0-412452356.00-60.20 4.85  +1.03 +0.80 +0.65   K0III              +0.006-0.095 +.030+029V?         9.1  14.2      *
8645          BD-09 6038 215114146271       16208           223749.0-085006224303.5-081842 58.68-54.29 6.45  +0.17 +0.12         A5V                +0.024-0.006D+.004+005V?         1.1   2.5AB   4*
8646          CP-61 6676 215121255299                       223748.4-610127224416.5-602958326.89-50.30 6.30  +0.56               F3V                +0.004+0.025      -006V
8647 67    AqrBD-07 5838 2151431462733819                   223800.9-072911224314.3-065746 60.56-53.52 6.41  -0.04 -0.11         A0Vn               +0.030-0.008      +002      142
8648          BD+53 2960 215159 347133817I                  223815.3+532308224220.8+535432104.55-04.29 6.12  +1.62               K2                 -0.001-0.006      +009
8649 66    AqrBD-19 6324 215167165252    I         14291    223812.4-192113224335.3-184949 41.88-59.52 4.69  +1.37 +1.56 +0.72   K4III              -0.027-0.028 +.013+022     < 19:                *
8650 44Eta PegBD+29 4741 215182 90734 857I  16211  14285    223818.8+294153224300.1+301317 92.50-24.95 2.94  +0.86 +0.55 +0.48   G2II-III+F0V       +0.015-0.025 +.017+004SB1O    9  5.1  90.4AxBC 6*
8651          BD+37 4670 215191 72652                       223822.9+371645224255.5+374810 96.64-18.42 6.43  -0.09 -0.81         B1V                +0.011-0.009      -018SB                        *
8652          BD+46 3803 215242 52296       16214  14290    223845.3+463841224304.5+471007101.40-10.27 6.39  +0.47 -0.06         A1V+G:             +0.014-0.002D+.002-017SB         1.6   0.5AxBC 3*
8653          BD+10 4805 215243108131                       223843.7+102510224342.7+105621 79.52-40.92 6.51  +0.46  0.00         G8IV               +0.015-0.168      -002    =<  6                 *
8654          BD+38 4855 215359 72675    I  16228           223934.2+385629224405.2+392756 97.71-17.09 5.95  +1.48               K5III+K2III        +0.007-0.021 +.010-027SB         3.5   2.7AB   3*
8655   Eta GruCP-5410123 2153692475703821  W                223929.7-540134224537.9-533001335.60-54.90 4.85  +1.18 +1.17         K2IIICNIV          +0.029+0.014 +.005+028SB         6.6  24.4
8656 13    LacBD+41 4594 215373 52317 858I  16227           223937.8+411740224405.5+414909 98.91-15.04 5.08  +0.96 +0.78         K0III              -0.007+0.004 +.010+013V?   < 19: 5.8  14.6      *
8657          CD-4714320 215405231278                       223946.5-470421224540.7-463251346.39-58.57 5.51  +1.32 +1.43 +0.60C  K2III              -0.031-0.020 -.003+042V?
8658          CD-4913955 215456231285                       224007.4-493012224608.0-485844342.31-57.46 6.62  +0.63               F9V                +0.203-0.050      -019
8659          CD-5013788 215504231287                       224027.9-501215224628.3-494109341.14-57.14 6.48  +1.14               K1III              +0.089-0.277      +002
8660 45    PegBD+18 5046 2155101081541596                   224036.2+185021224528.2+192200 86.26-34.39 6.25  +1.05              gG6                 -0.026+0.056      -022V?
8661          BD+51 3460 215518 34757    I                  224038.9+515929224449.2+523102104.21-05.69 6.55R                     K2                 -0.001+0.012      +005
8662          CD-4714331 215545231290      W                224048.6-472758224643.7-465622345.57-58.54 6.56  +0.30 +0.11 +0.16C  A9IIIm:            +0.058+0.004      -001SB         3.0  10.5      *
8663   Xi  OctCP-80 1055 215573258946                       224103.3-803905225022.9-800727309.03-35.53 5.35  -0.15 -0.50         B6IV               +0.017-0.022      +016V?      0
8664          CP-77 1554 215631258069                       224121.6-773443224940.9-770302311.22-38.05 6.73  +0.11 +0.10         A4V                +0.071+0.013      -009
8665 46Xi  PegBD+11 4875 215648108165    I  16261           224141.8+113936224641.6+121022 81.28-40.39 4.19  +0.50 -0.03 +0.31   F6III-IV           +0.235-0.498 +.052-005V       7  7.5 145.0AC   3*
8666          BD+43 4300 215664 52348              14311    224143.9+440108224610.2+443246100.60-12.83 5.76  +0.36 +0.08         F0III-IV           +0.140+0.032 +.019-010                          *
8667 47Lam PegBD+22 4709 215665 90775 859I                  224142.8+230222224631.9+233356 89.26-31.02 3.95  +1.07 +0.91 +0.51   G8IIIa*            +0.058-0.010 +.042-004     < 19:                *
8668          CD-3415735 215669214081                       224142.6-344122224719.1-340940 10.46-62.51 6.28  +1.16               K1III              -0.066+0.048      +041
8669          CP-62 6369 215682255321                       224152.4-621242224821.4-614103325.03-49.83 6.37  +1.06 +0.90         K0III              +0.069+0.003      -018
8670 68    AqrBD-20 6486 2157211652931597I                  224210.9-200806224733.1-193648 41.10-60.68 5.26  +0.94 +0.59         G7III              -0.101-0.204 +.034+023
8671          CD-3815217 215724214087                       224206.1-384450224747.1-381319  1.85-61.88 6.71  +0.48 -0.02?        F4IV               -0.068-0.076      +027
8672          CP-71 2726 215729258070                       224208.9-705236224917.4-702052316.54-43.41 6.34  +0.07 +0.10         A2V                +0.005+0.040      +008
8673 69Tau1AqrBD-14 6346 215766165298       16268           224224.2-143501224742.8-140323 51.06-58.35 5.66  -0.05 -0.25         A0V                +0.032-0.009      +015SB     71  3.9  23.7AB   3*
8674          CD-2616324 2157821914123824                   224226.9-262611224756.2-255443 28.36-62.41 6.30  +0.91               G5III              +0.112-0.114      -026V
8675   Eps GruCD-5113389 215789247593 860          14320    224230.9-515034224833.3-511901338.31-56.53 3.49  +0.08 +0.10 +0.07   A3V                +0.108-0.071 +.044+000V     236
8676 70    AqrBD-11 5923 2158741653083825          FM Aqr   224314.5-110501224830.2-103320 56.80-56.68 6.19  +0.28               A9III-IV           +0.035+0.007      -006                          *
8677          BD+57 2612 215907 34824                       224324.7+575719224723.2+582858107.31-00.57 6.36  -0.04 -0.03         B9.5IV             +0.007+0.001      +004V      73
8678          BD+36 4934 215943 72732              14321    224336.2+365326224810.9+372500 97.40-19.27 5.90  +1.03 +0.83        gG8                 -0.055-0.058      -025
8679 71Tau2AqrBD-14 6354 216032165321 861I W       14329    224417.9-140714224935.5-133533 52.27-58.52 4.01  +1.57 +1.95 +0.95   M0III              -0.012-0.038 +.021+001           4.5 132.5      *
8680          CD-3316244 216042214119      W                224424.9-332002224959.1-324819 13.31-63.19 6.33  +0.31               F0V+F3V            -0.050-0.017D+.015+024           0.2   0.3      *
8681          BD+09 5111 216048108204                       224431.8+095703224932.3+102844 80.64-42.17 6.54  +0.29 +0.05         F0IV-V             +0.091-0.031      -008V?    129
8682          BD+53 2993 216057 34845                       224438.7+535311224847.8+542454105.62-04.28 6.12  -0.07 -0.49         B5Vne              -0.018+0.007      -019V?    358                 *
8683          BD+62 2115 216102 20253                       224457.8+622440224844.2+625618109.49  3.32 6.06  +1.20               K0                 +0.007-0.052      -027
8684 48Mu  PegBD+23 4615 216131 90816 862I                  224510.5+240425225000.2+243606 90.68-30.56 3.48  +0.93 +0.68 +0.47   G8+III             +0.147-0.042 +.040+014        7
8685          CD-3914848 2161492141341599                   224520.7-394111225102.2-390925359.57-62.25 5.42  +1.43 +1.69         M0III              +0.022-0.011 -.001+027
8686          CP-60 7610 2161692553363826                   224526.7-602441225144.9-595253326.58-51.40 6.46  +1.13               K1IIICNII          -0.046-0.009      +000
8687          BD+67 1468 216172 20259       16291A          224536.0+680222224900.7+683413112.09  8.31 6.19R                     F5V                +0.112+0.068 +.025+003V?         0.2   4.3AB   3*
8688          BD+55 2820 216174 34858    I                  224538.5+552221224946.2+555410106.42-03.02 5.43  +1.17 +1.12         K1III              +0.083+0.043      -036
8689          CP-63 4826 216187255339      W                224540.6-634304225209.9-631119322.94-49.05 6.12  +1.03 +0.94         K0III              +0.016-0.040      -004           3.0   0.9
8690 14    LacBD+41 4623 216200 52412      S       V360 Lac 224551.0+412526225021.8+415713100.04-15.47 5.92  +0.08 -0.51         B3IV:             e+0.010 0.000      -014SB2   225        1.0      *
8691          BD+18 5059 216201108211                       224545.4+183642225039.1+190827 87.35-35.25 6.40R                     K0                 +0.050-0.020      -039
8692          BD+49 3954 216206 34862                       224552.6+500850225010.2+504037104.10-07.71 6.21  +1.14 +0.89         G4Ib               +0.013+0.004      -009SB?
8693 21    PsACD-3019324 216210191444                       224550.1-300358225120.9-293210 20.59-63.54 5.97  +0.91 +0.59         G8/K0III           -0.004-0.011      +005V?
8694 32Iot CepBD+65 1814 216228 20268 863I                  224607.1+654028224940.8+661202111.07  6.18 3.52  +1.05 +0.90 +0.51   K0-III             -0.065-0.125 +.041-012     < 19:                *
8695 22Gam PsACD-3316270 216336214153      W       14338    224658.1-332421225231.6-325232 13.06-63.72 4.46  -0.04 -0.14 +0.02   A0III*             -0.030-0.025 +.043+017V?     55  3.6   4.2      *
8696          BD+60 2450 216380 20281       16317           224728.2+610954225122.8+614149109.20  2.07 5.60  +0.78 +0.39         G8III-IV+G2IV      +0.108+0.044 +.005+002           1.2   1.6AB   3*
8697 49Sig PegBD+09 5122 2163851278103828                   224719.9+091813225224.1+095008 80.87-43.10 5.16  +0.48 -0.01         F7IV               +0.522+0.044 +.043+012SB      0  8.4 248.       *
8698 73Lam AqrBD-08 5968 216386146362 864I         Lam Aqr  224723.8-080642225236.9-073447 62.19-55.73 3.74  +1.64 +1.74 +1.19   M2.5IIIaFe-1      e+0.011+0.037 +.017-009                          *
8699 15    LacBD+42 4521 216397 52436    I  16325           224731.3+424651225202.0+431845100.97-14.41 4.94  +1.56 +1.94 +0.72E  M0III              +0.110+0.029 +.021-017V          6.0 104.4AC   5*
8700   Tau1GruCD-4913988 216435231343    I'                 224742.7-490740225337.9-483553341.70-58.71 6.04  +0.62 +0.19         G0V                +0.216-0.078 +.042-001                          *
8701   Rho IndCP-70 2971 216437258084 865I'                 224742.3-703628225439.4-700425316.27-43.90 6.05  +0.66 +0.24         G2-3IV             -0.050+0.072      -003
8702          BD+82  703 216446  3794       16294           224752.7+823724224729.0+830914119.16 21.21 4.74  +1.26 +1.35 +0.67   K3III              +0.021+0.049 -.018-031     < 19: 4.7   3.5
8703          BD+16 4831 2164891082313829          IM Peg   224806.9+161838225302.3+165028 86.36-37.48 5.64  +1.12 +0.90         K1-2II-III        e-0.018-0.024      -012SB1O                      *
8704 74    AqrBD-12 6371 216494165359      S       HI Aqr   224812.8-120854225328.7-113700 56.42-58.28 5.80  -0.08 -0.31 -0.05   B9III              +0.025+0.001      -007SB2O    5  0.5   0.1      *
8705          BD+49 3962 216523 34917                       224832.3+495251225252.3+502443104.37-08.14 6.46  -0.04 -0.34         B8V                +0.019-0.011      -011V?
8706          BD+39 4957 216538 72812              14346    224837.1+393809225311.3+401002 99.67-17.30 6.34  -0.08 -0.44         B7III-IV           +0.001-0.009      +007       15
8707          BD+59 2595 216595 349213830I                  224904.3+593409225303.8+600604108.68  0.55 6.01  +1.75             sgK2                 +0.017+0.002      -007SB
8708          BD+43 4331 216608 52465       16345           224911.5+441302225340.1+444457101.91-13.26 5.81  +0.26 +0.10 +0.14   A3Vm+F6V           -0.016-0.013 +.018+012SB     40  1.5   0.9AB   3*
8709 76Del AqrBD-16 6173 216627165375 866          14356    224920.6-162110225439.0-154915 49.58-60.67 3.27  +0.05 +0.08 +0.04   A3V                -0.040-0.025 +.038+018V      71                 *
8710 78    AqrBD-07 5886 216637146382                       224921.7-074410225434.1-071217 63.25-55.87 6.19  +1.28 +1.41         K3III              -0.012-0.038      +009V?
8711 77    AqrBD-17 6619 216640165376              14358    224928.0-164808225445.5-161619 48.79-60.90 5.56  +1.14 +1.09         K2.5IIIb           -0.222-0.090 +.019-036V
8712          BD+39 4964 216646 52471                       224931.7+395037225407.0+402237 99.94-17.20 5.81  +1.13               K0III              +0.103+0.036      -006                          *
8713          CD-3714981 2166662141823831                   224938.2-365518225514.9-362319  5.02-63.75 6.40  +1.31               K2III              +0.046+0.002      -026
8714          BD+16 4833 216672108246    I         HR Peg   224939.8+162434225435.7+165630 86.83-37.61 6.12  +1.77 +1.86         S4+/1+             +0.019-0.008      +011                          *
8715  1    PscBD+00 4939 216701127836                       224952.5+003155225459.5+010353 73.39-50.26 6.11  +0.20 +0.15         A7III              +0.023+0.001      +013
8716          BD-05 5885 216718146388       16365  14360    224959.8-053114225511.0-045916 66.38-54.55 5.72  +0.88 +0.60         K0III-IV           +0.034-0.005D+.012-009V?         2.0   0.6      *
8717 50Rho PegBD+08 4961 216735127839                       225011.6+081657225513.7+084857 80.79-44.34 4.90   0.00  0.00 -0.02   A1V                +0.079+0.014 +.006-010SB?    97
8718          BD+36 4956 216756 728381600                   225023.5+363237225502.6+370437 98.51-20.21 5.91  +0.42 -0.08         F5II               +0.086+0.007      -028    =< 10
8719          CD-3217312 216761214187                       225020.3-320959225551.4-313759 15.79-64.50 6.10  +1.36 +1.52         K3III              -0.033-0.006      +022V
8720 23Del PsACD-3316303 216763214189    I W                225024.6-330426225556.9-323223 13.69-64.46 4.21  +0.97 +0.69 +0.61   G8III              +0.018+0.031 +.021-012           5.0   5.2      *
8721          CD-3217321 216803214197              TW PsA   225050.3-320541225624.0-313356 15.94-64.61 6.48  +1.10 +1.02 +0.59   K4V                +0.327-0.162 +.130+006V                         *
8722   Tau3GruCD-4814364 2168232313643832                   225058.1-483011225647.8-475809342.17-59.49 5.70  +0.22 +0.19 +0.12   Am                 -0.028-0.001      +006
8723          BD+35 4917 216831 72851       16376           225104.8+354905225544.5+362106 98.28-20.92 5.74  -0.05 -0.38         B7III              +0.022+0.005      +001       70  3.7  51.0      *
8724          BD+11 4904 216900108275       16389           225151.4+111852225651.5+115054 83.69-42.13 6.51  +0.18 +0.05         A3V s              +0.063-0.009D+.008+008           3.0   3.9
8725 16    LacBD+40 4949 216916 52512       16381  EN Lac   225149.6+410412225623.6+413614100.92-16.30 5.59  -0.14 -0.83         B2IV               -0.002 0.000      -007SB1O   23  3.6  62.1AC   3*
8726          BD+48 3887 216946 52516    I         14368    225202.8+491159225626.0+494401104.59-09.00 4.95  +1.78 +1.96 +1.05   K5Ib               +0.003-0.002 -.001-010V?                        *
8727          BD-05 5894 216953146404                       225206.6-052040225717.2-044836 67.22-54.82 6.31  +0.94 +0.72         G9III              -0.015+0.002      -009V?
8728 24Alp PsACD-3019370 216956191524 867I         14372    225207.6-300908225739.1-293720 20.49-64.90 1.16  +0.09 +0.08 +0.02   A3V                +0.333-0.165 +.149+007      100                 *
8729 51    PegBD+19 5036 217014 90896              14374    225233.1+201357225727.9+204608 90.06-34.73 5.49  +0.67 +0.21 +0.34   G2.5IVa            +0.204+0.059 +.074-031V       2                 *
8730          BD+03 4799 2170191278603834                   225227.2+031629225732.8+034837 76.91-48.62 6.28  +1.12 +1.08         K1III              +0.070+0.029      +011
8731          BD+47 3985 217050 525263833          EW Lac   225239.2+480859225704.5+484103104.22-09.99 5.43  -0.09 -0.53         B4IIIep           v+0.012-0.009      -022SB    350                 *
8732          CD-3615650 217096214215                       225301.0-360319225835.0-353123  6.64-64.59 6.13  +0.58               F8III-IV           -0.001-0.111      +021                          *
8733          BD+38 4904 217101 728833835                   225303.5+384627225740.7+391832100.07-18.46 6.18  -0.15 -0.80         B2IV-V             +0.002-0.008      -016      168                 *
8734          BD-03 5539 2171071464123836  D                225306.5-025548225815.5-022343 70.47-53.32 6.16  +0.74 +0.43         G8IV               -0.003-0.018      -012V?         1.3   0.0      *
8735          BD-02 5858 217131146415                       225314.6-015643225823.7-012437 71.66-52.64 6.37  +0.35 +0.02         F0-2V              +0.090-0.007      -014                          *
8736          BD+84  517 217157  3808                       225328.8+845017225102.2+852225120.41 23.10 5.90  +1.32               K5                 +0.003+0.107      -030
8737          BD+08 4973 217166127870       16417  14379    225330.9+084932225835.1+092125 82.17-44.40 6.43  +0.64 +0.16 +0.36   G2V+G4V            +0.403-0.141 +.035-027V          0.2   0.2AB   3*
8738          BD+06 5092 217186127874                       225340.0+064823225842.6+072023 80.50-46.04 6.33  +0.06 +0.02         A1V                -0.040-0.080      -001       56
8739 52    PegBD+10 4859 217232108307       16428           225411.6+111139225911.8+114344 84.23-42.55 5.75  +0.32 +0.08         A8V+F6V            +0.033-0.041D+.014+020V?         1.3   0.7      *
8740          CD-3019383 217236191550              14380    225407.6-295954225935.8-292744 20.89-65.33 5.51  +0.25 +0.19         A5n                -0.003+0.009      +000       90                 *
8741          BD-13 6318 217251165425    I                  225419.7-133625225935.7-130415 55.63-60.33 6.07  +1.45 +1.76         K5III              -0.010 0.000      +013V?
8742  2    PscBD+00 4950 217264127881       16431           225419.9+002544225927.4+005746 74.59-51.08 5.43  +0.98 +0.83        gK1                 +0.086-0.071      -013V?         7.7   3.9
8743          CD-2516220 217303191554    I                  225441.0-254154230005.8-250951 31.11-64.99 5.65  +1.25 +1.20        gK0                 +0.035-0.075      -035
8744          BD+51 3514 217314 350223838I                  225450.3+520704225910.3+523916106.24-06.54 6.29  +1.42               K2                 -0.025+0.033      +028
8745          BD+59 2615 217348 35026                       225503.1+591644225909.0+594853109.25-00.04 6.43  +0.03 -0.24         B9III              +0.003+0.005      -011V?
8746          CD-2616419 217358191565                       225459.1-260943230024.6-253736 30.06-65.14 6.29  +1.13               K1III              +0.086-0.034      -025
8747   Zet GruCP-5310382 217364247680 868                   225458.6-531725230052.8-524515334.11-57.16 4.12  +0.98 +0.70 +0.53   G8-K0III           -0.067-0.014 +.039-001SB
8748          BD+83  640 217382  38161649                   225512.8+834840225424.8+842046119.95 22.17 4.71  +1.43 +1.69 +0.73   K4III              +0.092+0.026 +.001+003     < 17                 *
8749          CD-5113446 217403247683                       225514.9-512912230107.5-505700336.68-58.33 5.68  +1.42 +1.08 +0.54E  K3III              +0.078-0.006      +008
8750  3    PscBD-00 4443 2174281464433840          Var?     225530.2-002104230037.9+001109 74.11-51.86 6.21  +0.89 +0.60         G4III              +0.044+0.013      -016                          *
8751          BD+02 4594 217459127894    I                  225537.2+022839230042.9+030042 77.06-49.74 5.83  +1.34 +1.57 +0.50E  K4III              +0.011-0.086      +019V                         *
8752          BD+56 2923 217476 350393839I         V509 Cas 225552.1+562432230005.1+565643108.16-02.70 5.00  +1.42 +1.16 +0.85   G40               v 0.000-0.003 +.002-058V?     35                 *
8753          BD+30 4859 217477 72924       16443           225556.1+303246230042.5+310459 96.64-26.07 6.60  -0.04 -0.35         B9IIIpMn           +0.031+0.005      +001V?     35  2.5   3.4      *
8754          CD-2918537 217484191581    I                  225551.8-292325230119.4-285113 22.39-65.68 5.55  +1.35 +1.51         K3III              +0.066-0.010      -025V
8755          BD+44 4302 217491 52587                       225602.6+445018230034.4+452230103.32-13.23 6.50                      A3V                +0.010+0.002      -004V?
8756          CD-2317706 2174981915853842                   225600.4-231938230123.0-224727 36.71-64.73 6.28  +0.13               A5V                +0.026-0.027      -012
8757 81    AqrBD-07 5910 217531146447    I                  225611.8-073553230123.6-070340 65.42-57.08 6.21  +1.41 +1.77         K5III              -0.015-0.011      -002V?
8758          BD+37 4744 217543 72929              14386    225615.4+381016230054.7+384229100.39-19.28 6.54  -0.11 -0.65         B3Vpe              +0.001-0.005      -016SB    370                 *
8759          BD-05 5910 217563146451      D                225621.1-051456230131.7-044241 68.60-55.53 5.94  +1.00 +0.75         K0                 +0.019+0.009      +006                          *
8760          CD-3715047 217642214261      W                225700.4-365728230234.0-362515  4.16-65.17 6.47  +0.94 +0.71         K1III              +0.031-0.035D+.003+000           3.0   2.0      *
8761          BD+56 2927 217673 35062    I                  225716.8+563405230130.7+570620108.40-02.63 6.20  +1.50 +1.53         K1.5II             -0.003-0.007      -006
8762  1Omi AndBD+41 4664 217675 52609 869  S       Omi And  225719.1+414719230155.3+421934102.21-16.10 3.62  -0.09 -0.53 -0.08   B6IIIpe+A2p       v+0.023-0.006 +.015-014SB2O  330  0.5   0.3 S   4*
8763 82    AqrBD-07 5913 217701146465    I                  225721.0-070639230232.6-063427 66.44-56.97 6.15  +1.58 +1.90 +0.87E  M2III              -0.004-0.037      -008V?                        *
8764          BD-21 6354 217703191604                       225723.9-212418230244.3-205214 41.20-64.46 5.97  +0.94               K0III-IV           -0.058-0.117      +026V?
8765          BD+31 4829 217754 72949                       225746.8+311435230233.1+314650 97.38-25.63 6.57  +0.35 +0.08         F2IV               +0.005-0.016      -017        7
8766  2    AndBD+41 4665 217782 52623       16467  14396    225800.0+421312230236.3+424528102.52-15.76 5.10  +0.09 +0.11         A3Vn               +0.055-0.004 +.034+002SB    190  3.7   0.4AB   3*
8767   Pi  PsACD-3515630 2177922142751601          Pi  PsA  225757.8-351724230329.8-344458  7.99-65.72 5.11  +0.29 +0.02         F0V+F3V            +0.076+0.080 +.052-006SBO     0                 *
8768          BD+43 4378 217811 52626       16472  LN And   225810.7+433115230245.2+440332103.11-14.59 6.39   0.00 -0.58         B2V                +0.001-0.002      -008       25  3.9   7.4      *
8769          CP-69 3301 2178312553953843                   225815.8-692139230452.2-684913316.28-45.45 5.52  +0.36 +0.12         F4III              +0.040+0.063      +004SB     87
8770          BD+54 2900 217833 35092       16474  V638 Cas 225825.2+544153230243.8+551411107.79-04.41 6.50  -0.08 -0.55         B9IIIHe wk        v+0.008+0.012      -013           3.6  19.5AB   4*
8771          CD-4216177 217842231419                       225821.8-420108230359.5-412842352.86-63.75 5.79  +1.08 +0.95         K0III              +0.006+0.069      -015
8772          BD-05 5917 217877146482                       225844.7-052005230357.3-044743 69.23-56.02 6.68  +0.58 +0.06         F8V                +0.320+0.028      -014V                         *
8773  4Bet PscBD+03 4818 2178911279341602          14410    225847.2+031654230352.6+034912 78.79-49.61 4.53  -0.12 -0.49 -0.13   B6Ve              v+0.013-0.011      +000      128                 *
8774   Kap GruCP-5410197 2179022477113845                   225844.9-543004230439.6-535754331.77-56.77 5.37  +1.45 +1.75         K5III              +0.055-0.102 +.012+018V?                        *
8775 53Bet PegBD+27 4480 217906 90981 870I  16483  Bet Peg  225855.5+273225230346.5+280458 95.74-29.05 2.42  +1.67 +1.96 +1.32   M2.5II-III        e+0.189+0.137 +.022+009V          7.0 253.1AC   3*
8776          BD+05 5123 217926127937                       225857.3+060438230401.0+063700 81.40-47.41 6.41  +0.39 +0.03         F2V                +0.003+0.019      +004       68
8777          BD+59 2631 217943 20393       16481           225915.3+595424230323.7+602643109.99  0.32 6.74  -0.02 -0.63         B2V                +0.007-0.006      -017SB         2.7  33.9      *
8778          BD+57 2676 217944 35103                       225908.2+580132230321.6+583353109.22-01.40 6.43  +0.90 +0.52         G8IV               +0.067+0.017      +015
8779          BD+66 1575 218029 203983844I                  225944.2+664012230332.9+671233112.75  6.50 5.24  +1.26 +1.40         K3III              +0.026+0.007 -.002-007V    < 17
8780  3    AndBD+49 4028 218031 52649    I                  225941.5+493030230411.0+500308105.86-09.23 4.65  +1.06 +0.88 +0.57   K0IIIbFe-0.5       +0.164+0.168 +.004-035     < 17
8781 54Alp PegBD+14 4926 218045108378 871I         14417    225946.7+144002230445.7+151219 88.28-40.38 2.49  -0.04 -0.05 -0.03   B9V                +0.063-0.042 +.038-004SB    148                 *
8782 83    AqrBD-08 6018 218060146498       16497           225956.9-081401230509.8-074137 65.67-58.20 5.43  +0.30 +0.07         F2IV+F0V           +0.129+0.011 +.021-013SB    101  0.2   0.2AB   3*
8783          BD-17 6661 2180611654813846I W                225955.4-173704230512.8-170445 49.67-63.50 6.14  +1.37               K0                 -0.048-0.039      -015           5.1  56.1
8784          BD+15 4760 218101108383                       230009.8+160145230506.3+163347 89.29-39.26 6.44  +0.83 +0.55         G8IV               -0.185-0.194 +.025-027
8785          BD+00 4963 218103127960                       230010.6+004605230517.6+011825 76.72-51.78 6.39  +0.94 +0.72         G9III              +0.035-0.027      -012
8786          CP-80 1064 2181082581053847                   230015.3-800113230823.7-792851308.60-36.49 6.12  +0.14 +0.10         A3-4V              +0.084-0.038      -007
8787   The GruCD-4415149 218227231444      W                230114.8-440337230652.8-433114348.24-63.30 4.28  +0.42 +0.16 +0.18   F5mDel Del         -0.039-0.023 +.017+010V?     80  2.5   1.2AB   3*
8788          BD+17 4866 218235108400                       230119.9+175833230618.2+183103 90.87-37.72 6.13  +0.44 +0.10         F6V s              +0.232+0.057 +.016-012SB                        *
8789 86    AqrCD-2417497 218240191651    I  16511           230118.6-241700230640.9-234435 35.23-66.15 4.47  +0.90 +0.58 +0.49   G9III              +0.066-0.003 +.028+015          10.0   2.9      *
8790   Ups GruCD-3914936 218242214313      W                230119.6-392559230653.6-385332357.92-65.25 5.61  +0.01               A1V                +0.040+0.013      +016V          2.3   1.1
8791          CD-5013885 218255231445                       230123.7-500849230709.5-493624337.56-59.92 6.33  +1.45               K4III              +0.017-0.014      +022
8792          BD+19 5058 218261108402                       230134.0+192213230631.9+195439 91.79-36.54 6.30  +0.49 +0.02         F7V                +0.295+0.002      -005V?      7                 *
8793          CD-5113471 218269247739      W                230128.1-511335230714.7-504111335.89-59.25 5.83  +0.48               F6-7IV-V           -0.044-0.020      +004V?         0.8   8.5      *
8794          CP-74 2054 218288258106                       230138.2-740738230835.7-733511312.37-41.60 6.15  +1.42 +1.68         K3III              +0.021-0.011      +005                          *
8795 55    PegBD+08 4997 2183291279761603I         14428    230157.9+085209230700.3+092434 84.62-45.55 4.52  +1.57 +1.90 +1.02   M1IIIab            +0.011-0.014 +.016-005V
8796 56    PegBD+24 4716 218356 910193848I         14429    230214.3+245542230706.8+252806 95.12-31.71 4.76  +1.34 +1.16 +0.68   G8Ib               +0.004-0.031 +.006-027SB   < 17                 *
8797  1    CasBD+58 2545 218376 35147                       230223.0+585245230636.9+592511109.95-00.78 4.85  -0.03 -0.87 -0.11   B0.5IV             +0.009+0.001      -009V      50                 *
8798          BD+32 4587 218395 73010       16519A 14430    230240.7+321705230727.7+324933 98.92-25.15 6.02R +0.12 +0.11         A4Vn               -0.024+0.008D+.011-001V     165  1.5   8.4      *
8799          BD+20 5278 218396 910223850          Var      230232.6+203541230728.7+210803 92.76-35.57 5.99  +0.26 -0.04         A5V                +0.108-0.052      -012       45                 *
8800          BD+45 4147 218407 52707                       230243.3+453138230718.1+460405104.71-13.09 6.66  -0.05 -0.68         B2V                +0.005-0.001      -015SB1O  160                 *
8801          BD+52 3371 218416 35151                       230244.1+521632230710.1+524859107.42-06.89 6.11  +1.05               K0III              +0.013+0.006      +005
8802          CD-2918588 218434191674                       230256.5-292150230821.0-284924 22.62-67.22 5.60  +0.88 +0.54         G9III              -0.047-0.031      +019V
8803          BD+58 2546 218440 35152                       230256.7+591112230710.4+594339110.13-00.53 6.40  -0.01 -0.63         B2V                +0.005+0.001      -005SB2O                      *
8804  4    AndBD+45 4149 218452 527113852I  16526           230304.8+455049230739.3+462314104.90-12.82 5.33  +1.41 +1.72 +0.75   K5III              -0.009-0.028      -006V?         5.5  48.2      *
8805  5    AndBD+48 3944 218470 527131604                   230312.7+484504230745.4+491745106.09-10.16 5.70  +0.44 -0.03         F5V                +0.154+0.133 +.021-002V?      9
8806          BD+43 4399 218525 52717                       230335.2+440114230812.3+443342104.24-14.53 6.56  +0.17 +0.20         A2IV               +0.024-0.005      +002V      70
8807  5    PscBD+01 4686 2185271279933854                   230333.6+013500230840.9+020740 78.61-51.68 5.40  +0.91 +0.56         G8III-IV           +0.145+0.108      -018V?
8808          BD+62 2171 218537 20439       16530           230342.7+630532230747.7+633800111.73  3.04 6.26  -0.02 -0.60 -0.06   B3V                +0.002-0.004D+.006-019      180  0.5   0.2      *
8809          CP-67 3949 218558255420                       230348.2-672402231011.7-665127317.29-47.35 6.47  +0.95 +0.72         K0III              +0.201+0.038      -006
8810          CP-81 1024 218559258961                       230350.3-812719231212.0-805446307.59-35.30 6.41  +1.50 +1.84         K4III              +0.009-0.011      +011
8811          BD+63 1931 218560 204413853                   230353.6+634052230757.2+641321111.97  3.58 6.21  +1.10               K0                 +0.010+0.001      -028
8812 88    AqrBD-21 6368 218594191683 873I                  230406.9-214255230926.8-211021 41.76-66.03 3.66  +1.22 +1.24 +0.60   K1III              +0.055+0.031 +.010+021
8813          CD-2818099 2186191916863855I                  230420.5-283751230944.6-280519 24.56-67.49 5.87  +1.31 +1.42         K2III              +0.022+0.003      -014V
8814          CD-4315281 218630231464      W                230424.3-432409230957.3-425138348.92-64.11 5.81  +0.48 -0.02         F4IV               -0.325-0.007 +.042+012           3.4   1.0
8815 57    PegBD+07 4981 218634128001    I  16550  GZ Peg   230428.5+080807230931.5+084038 84.77-46.51 5.12  +1.47 +1.01 +1.26   M4IIIS+A2V         +0.004+0.002 +.007+014SB  =< 50  5.5  32.9      *
8816          BD-15 6360 2186391655223856                   230433.9-150309230949.6-143038 55.87-63.19 6.42  +0.01 +0.01         A0Vn               +0.031-0.009      +015V?    154
8817 89    AqrCD-2317771 218640191687    I W                230434.5-225958230954.8-222727 38.81-66.53 4.69  +0.65 +0.39 +0.36   G2IV+A2V           +0.026-0.009 +.040-005           0.8   0.4      *
8818          CD-4115163 218655231465    I         14441    230435.8-410757231009.8-403530353.60-65.18 5.83  +1.62 +1.84         M4III              +0.035-0.046      -007                          *
8819 33Pi  CepBD+74 1006 218658 10629    I  16538           230442.9+745049230753.9+752315116.42 13.84 4.41  +0.80 +0.46 +0.43   G2III              +0.012-0.030 +.006-019SB1O   22  2.2   1.0AB   3*
8820   Iot GruCD-4514947 2186702314681605                   230442.0-454718231021.6-451448344.27-62.93 3.90  +1.02 +0.86 +0.53   K1III              +0.136-0.030 +.030-004SB1O                      *
8821 58    PegBD+09 5170 218700128007                       230459.3+091648231001.5+094919 85.83-45.62 5.39  -0.08 -0.29         B9III              -0.012-0.012      +009V     175                 *
8822  2    CasBD+58 2552 218753 35186       16556  Var?     230527.3+584725230944.1+591959110.28-01.02 5.70  +0.33 +0.32         A5III              -0.003+0.009      -012V?         4.3 167.2AC   8*
8823          CD-3019460 218759191703                       230522.3-300358231046.6-293130 20.81-67.76 6.51  +0.27               F1III-IV           -0.034-0.058      +025    =< 15
8824          BD+16 4882 218792108443    I                  230544.4+170309231042.6+173540 91.45-39.04 5.71  +1.34 +1.51         K4III              +0.021-0.028      +002V?
8825  6    AndBD+42 4592 218804 527613857                   230549.8+430025231027.2+433239104.21-15.62 5.94  +0.44 -0.05         F5IV               -0.191-0.191 +.023-043       18
8826 59    PegBD+07 4991 2189181280221606                   230641.2+081037231144.2+084312 85.47-46.77 5.16  +0.13 +0.08         A5Vn               -0.006-0.006 +.029+010V     214
8827 60    PegBD+26 4580 218935 91080      W                230657.8+261826231149.2+265050 96.95-30.95 6.17  +0.94 +0.67         G8III-IV           -0.190-0.120 +.026-010           3.3 231.5AC   3*
8828          CD-5013915 219023231493      W                230733.5-500944231315.0-493708336.26-60.67 6.80  +0.92               G8-K0III           +0.002-0.018      -026           1.6   0.5
8829          CP-63 4862 219077255435                       230756.6-631357231406.6-624200320.42-51.00 6.12  +0.79 +0.35         G8V                +0.473-0.424 +.032-026
8830  7    AndBD+48 3964 219080 52787                       230757.9+485136231233.0+492423106.87-10.36 4.52  +0.29 +0.04 +0.16   F0V                +0.092+0.098 +.051+013SB     59                 *
8831          BD+28 4548 219110 91095                       230812.7+285355231304.0+292630 98.52-28.73 6.35  +0.93 +0.61         G8III              -0.031-0.033      +004
8832          BD+56 2966 219134 35236 875  W       14458    230827.8+563658231317.0+571006109.87-03.19 5.56  +1.01 +0.89 +0.53   K3V                +2.075+0.296 +.152-018V          3.8 106.6      *
8833          BD+10 4902 2191391084633858   16603           230824.5+103116231326.5+110354 87.78-45.01 5.82  +1.01 +0.91         G8III              -0.005-0.003      +016V          3.9  33.3      *
8834 90Phi AqrBD-06 6170 2192151465851607I                  230908.6-063517231419.4-060256 70.94-58.74 4.22  +1.56 +1.90 +1.08   M1.5III            +0.042-0.196 +.010-000                          *
8835          CD-4115197 2192632315123859                   230925.8-413850231458.6-410620351.53-65.76 5.77  +1.18 +1.14         K2III              +0.112-0.112      +026SB
8836          BD-11 6032 219279165578    I  16618           230927.4-111357231440.2-104119 64.11-61.92 6.12  +1.49 +1.80         K5III              -0.001-0.034      -034V?         4.5   3.5AB   4*
8837          BD+49 4071 219290 35251                       230940.1+500425231414.3+503704107.59-09.34 6.31                      A0V                +0.038-0.006      -014       56
8838          BD+28 4555 219291 91111      W                230930.1+291339231421.7+294618 98.97-28.54 6.41  +0.45 -0.01         F6IV w             -0.007-0.013      +009       86  3.5  34.9
8839          BD+23 4704 219310 91113                       230940.5+233329231436.5+240611 96.20-33.69 6.36R                     K2III              +0.114+0.007      -027
8840          BD-04 5852 2194021465933860                   231025.1-040229231534.3-032947 74.66-57.09 5.55  +0.06 +0.06         A3V                -0.007-0.003      +011SB
8841 91Psi1AqrBD-09 6156 2194491465981608I  16633A          231039.1-093757231553.5-090516 67.07-61.11 4.21  +1.11 +0.99 +0.56   K0III              +0.372-0.016 +.049-026V    < 17  4.8  49.2AB   5*
8842 61    PegBD+27 4521 219477 911303861                   231053.1+274210231546.3+281452 98.56-30.06 6.49  +0.85 +0.48         G5II-III           +0.020-0.005      +004     < 23
8843          CP-62 6412 219482255446 876                   231057.0-623246231657.7-620004320.63-51.76 5.66  +0.51               F7V                +0.173-0.026      -009
8844          BD+73 1023 219485 10664                       231103.7+734110231437.3+741352116.38 12.61 5.84R                     A0V                +0.048+0.006      -003    =< 41
8845          BD+23 4712 219487 91133                       231102.1+241332231557.9+244616 96.88-33.22 6.60  +0.40 -0.05         F5V                +0.093+0.005      +005       20
8846          CD-4514982 219507231532      W                231105.8-450204231639.7-442921344.28-64.28 5.92  +1.06 +0.91         K1III              -0.004-0.007      +025           4.8  22.5
8847          CD-4115205 219531231533                       231119.1-414426231649.8-411140350.93-66.03 6.47  +1.08               K0III              -0.024+0.012      +013
8848   Gam TucCP-58 8062 219571247814 877                   231135.7-584702231725.8-581409324.30-54.82 3.99  +0.40 -0.02 +0.23   F1III              -0.029+0.079 +.043+018       94
8849          CP-80 1067 219572258127                       231137.1-800109231908.1-792822308.06-36.72 6.33  +0.91 +0.66         K0III              +0.013+0.007      -003
8850 92Chi AqrBD-08 6076 219576146612    I         Chi Aqr  231139.9-081619231650.9-074336 69.45-60.36 5.06  +1.60 +1.60 +1.20E  M3III              -0.016-0.015 +.009-015V?                        *
8851          BD+70 1311 219586 106713862                   231146.2+702034231537.8+705317115.19  9.47 5.56  +0.24 +0.12         F0IV               +0.018-0.005      +012SB    140
8852  6Gam PscBD+02 4648 219615128085 878I                  231158.8+024409231709.9+031656 82.47-52.02 3.69  +0.92 +0.58 +0.51   G9-III:Fe-2        +0.762+0.017 +.036-014V?      7                 *
8853          BD+52 3410 219623 352853863  W                231208.6+524027231642.3+531249108.90-07.06 5.54  +0.52 +0.02         F7V                +0.115-0.233 +.036-025V?  =<  6  7.3 129.3
8854          BD+61 2413 219634 20531              V649 Cas 231208.9+612503231626.8+615747112.02  1.12 6.53  +0.23 -0.66         B0Vn               -0.026-0.002      -015SB                        *
8855          CP-68 3567 2196442554553864                   231208.8-680104231820.0-672816315.82-47.26 6.13  +1.35 +1.58         K2-3III            +0.021+0.014      +018
8856          BD-12 6461 219659165609              14483    231226.8-121533231740.0-114247 63.38-63.14 6.34  +0.05 +0.01         A1Vn               +0.039-0.003      +003V?    195
8857          BD+44 4368 219668 52865                       231234.3+443711231716.6+450951106.01-14.59 6.43  +1.07 +1.01         K0IV               +0.097-0.065      -038
8858 93Psi2AqrBD-09 6160 219688146620      D                231242.4-094342231754.2-091057 67.62-61.54 4.39  -0.15 -0.56 -0.13   B5V                +0.021-0.012      -006V?    332                 *
8859   Phi GruCD-4115211 219693231539                       231238.8-412203231809.9-404928351.44-66.43 5.53  +0.44 -0.05         F5V                +0.136-0.128      +014        0
8860  8    AndBD+48 3991 219734 52871    I  16656  14484    231306.3+482808231744.7+490055107.53-11.03 4.85  +1.67 +1.98 +1.26   M2+IIIBa0.3        +0.037+0.008 +.011-008V?         5.8 219.4AC   6*
8861          BD+44 4373 219749 52876              ET And   231314.3+445635231756.1+452920106.25-14.33 6.48  -0.03 -0.26         B9pSi             v+0.034-0.017      +003SB1O   83                 *
8862   Tau OctCP-88  204 219765258970 925                   231309.9-880153232803.7-872856303.96-29.48 5.49  +1.27 +1.43         K2III              +0.015+0.009      +031
8863   Gam SclCD-3316476 219784214444 879I                  231325.5-330436231849.4-323155 12.25-69.25 4.41  +1.13 +1.06 +0.63   K1III              +0.019-0.070 +.043+016
8864  9    AndBD+40 5043 219815 52881              AN And   231338.6+411339231823.4+414625104.91-17.82 6.02                      A7/8Vm             -0.006-0.012      -004SB1O   78                 *
8865 95Psi3AqrBD-10 6094 2198321466351609   16671  14491    231345.6-100927231857.7-093639 67.31-62.02 4.98  -0.02 -0.02         A0V                +0.047-0.002 +.012-010V     143  3.9   1.4AB   3*
8866 94    AqrBD-14 6448 219834165625       16672A          231351.1-140011231906.7-132732 60.70-64.47 5.08  +0.80 +0.41         G5IV               +0.298-0.101 +.040+010SBO =< 17  2.4   0.2 S   3*
8867          BD+74 1016 219841 106843865                   231347.3+744510231718.9+751757116.95 13.53 6.38  +0.03 +0.03         A2V s              +0.018-0.002      -008
8868 96    AqrBD-05 5966 219877146639       16676           231412.8-054015231924.0-050728 73.94-58.94 5.55  +0.39  0.00         F3IV-V             +0.202-0.020      -009SB     50: 4.9  10.6      *
8869          BD-18 6283 2198791656283868I                  231408.4-183722231924.1-180431 51.23-66.96 5.93  +1.52 +2.01        gK3                 +0.004+0.009      +005V?
8870          BD+44 4378 219891 52892                       231419.5+443525231902.4+450814106.30-14.73 6.50                      A5Vn               +0.058+0.006      +007      160
8871          CD-3415985 219912214459      W                231418.6-341515231943.2-334229  8.90-69.21 6.37  +1.30               K2III              +0.047-0.041      +023           6.1  43.8
8872 34Omi CepBD+67 1514 219916 20554    I  16666  14495    231430.9+673352231837.5+680642114.42  6.79 4.75  +0.84 +0.49 +0.45   K0III              +0.058+0.016 +.024-018V?   < 17  2.3   3.1AB   3*
8873          BD+34 4899 219927 73171                       231437.0+341446231927.4+344736102.31-24.37 6.32  -0.08 -0.39         B8III              +0.016-0.003      -001V?      5
8874 11    AndBD+47 4110 219945 52907                       231450.1+480436231929.8+483731107.66-11.50 5.44  +1.03 +0.82         K0III              +0.019+0.056 +.016+011     < 19:
8875          BD+47 4114 219962 52912      W                231459.7+474958231941.6+482251107.60-11.74 6.32  +1.12 +1.05         K1III              +0.209+0.032 -.001+023           3.3  85.8AB   3
8876 10    AndBD+41 4752 219981 529143870I                  231506.6+413150231952.4+420441105.30-17.64 5.79  +1.48 +1.90         M0III              +0.052+0.010      +003V?
8877          CD-5013948 220003247838      W                231512.6-505108232050.0-501824333.53-61.10 6.05  +0.42 +0.16 +0.22   F5mDel Del         +0.028-0.077      -018V          2.0   0.5AB   3*
8878  7    PscBD+04 4997 2200091281263871I                  231514.8+045008232020.6+052253 85.42-50.73 5.05  +1.20 +1.12         K2III              +0.082-0.062 +.002+038V    < 19:
8879          BD-06 6191 220035146652      D       14515    231531.5-062714232040.9-055429 73.36-59.73 6.17  +1.07 +0.97         K0III              -0.101-0.057      -002                          *
8880 62Tau PegBD+22 4810 220061 91186 880          Tau Peg  231541.1+231135232038.2+234425 97.51-34.61 4.60  +0.17 +0.10 +0.09   A5Vp               +0.033-0.008 +.037+016V     143                 *
8881          BD+61 2427 220074 20567    I                  231552.0+612522232014.3+615812112.44  0.97 6.45R                     K1V                +0.003-0.013      -035                          *
8882 63    PegBD+29 4908 220088 73187    I                  231555.6+295209232049.6+302454100.72-28.52 5.59  +1.50 +1.91         M0III              +0.081-0.069      -019V
8883          CD-2716284 2200961918401611                   231555.8-273204232115.5-265912 28.15-69.95 5.64  +0.82               G4V                -0.013-0.012      +013
8884          BD+43 4440 220105 52927       16685           231600.0+433410232044.0+440659106.22-15.79 6.13R +0.17?              A5Vn               -0.016-0.027      -002           4.0  13.2      *
8885 12    AndBD+37 4817 220117 731901610  W                231603.5+373811232053.3+381056103.98-21.33 5.77  +0.46 +0.02         F5V                +0.130-0.063      -009V?     12  3.4 121.1AB   3
8886          BD+61 2428 220130 20572    I  16690           231612.4+613956232034.6+621247112.56  1.19 6.39  +1.61               K2III              +0.003-0.005      -023           5.5  12.7AB   4*
8887 64    PegBD+31 4897 220222 73205       16702           231701.9+311552232154.9+314845101.57-27.32 5.32  -0.11 -0.49         B6III              +0.005-0.010 +.001+002SB    161  3.7   0.4AB   3
8888          BD+25 4924 220242 91208                       231703.2+260346232158.2+263632 99.25-32.11 6.62  +0.37 +0.01         F5V                -0.098-0.074      +010
8889          CP-60 7654 220263255474              14529    231707.5-603615232256.9-600321321.48-53.77 6.09  +1.60 +1.91         M3III              +0.090-0.009      -028V?                        *
8890 97    AqrBD-15 6406 220278165658       16708           231724.7-153517232239.2-150221 58.78-66.08 5.20  +0.20 +0.10         A3Vp               +0.112+0.019 +.022-012      141  0.7   0.4      *
8891 65    PegBD+20 5317 220318 912203873                   231741.6+201650232240.5+204943 96.53-37.45 6.29  -0.04 -0.09         B9.5V              +0.019-0.012      -014SBO    25                 *
8892 98    AqrBD-20 6587 2203211918581612I                  231743.1-203847232258.2-200602 47.35-68.59 3.97  +1.10 +0.95 +0.60   K0III              -0.120-0.096 +.036-007V
8893 66    PegBD+11 4993 220363108580    I  16715           231801.9+114556232304.6+121850 91.56-45.11 5.08  +1.31 +1.49         K3III              +0.027-0.017 -.001-004V?   < 19: 0.0   0.1      *
8894          BD+59 2710 220369 353613874I                  231804.7+593506232232.5+600801112.07-00.85 5.56  +1.68 +1.82         K3II               +0.009-0.001      -012V?
8895          CP-5410281 220392247854      W                231815.0-542124232354.3-534830328.10-58.83 6.15  +0.26 +0.13         A4III              +0.059-0.027      +007      153  0.8  26.5      *
8896          CD-4315360 220401231587                       231816.4-434025232345.4-430728345.21-66.11 6.10  +1.46               K3III              +0.015+0.010      +001
8897          BD-00 4509 220406128156    I W                231824.1-001530232331.9+001729 81.72-55.38 6.31  +1.61 +1.95         K2                 +0.068+0.010      +009           4.0  41.7
8898          CP-5212150 220440247858                       231836.8-522621232413.1-515329330.51-60.31 5.75  +1.59 +1.93         M0III              +0.005-0.041      -005
8899          BD+31 4901 220460 73223                       231852.4+315853232347.5+323153102.29-26.80 6.69  +0.45 -0.12         F4V w              +0.232+0.040 +.012+010SB2O                      *
8900          BD-19 6450 220465165672                       231851.7-191419232407.8-184115 51.13-68.24 6.19  +1.02 +0.79         G8III              +0.149+0.071      -019
8901          CP-5710268 220572247867                       231937.3-572352232519.4-565057324.29-56.56 5.59  +1.07 +0.98 +0.37E  K0III              +0.080-0.024      -019    =<  1:                *
8902          BD+40 5068 220575 52987                       231946.4+403349232435.0+410646105.80-18.85 6.72  +0.06?              B8III              +0.015-0.009      -003SB     30                 *
8903 67    PegBD+31 4904 220599 732411613                   231957.0+315008232450.8+322306102.46-27.02 5.57  -0.11 -0.26         B9III              +0.017+0.002 +.004+018V     110
8904  4    CasBD+61 2444 220652 20614 882I W       14549    232023.5+614402232450.3+621658113.05  1.08 4.98  +1.68 +2.07 +0.85E  M1III              +0.011-0.013 +.007-037           2.5  98.6AB   4*
8905 68Ups PegBD+22 4833 220657 91253 881I                  232023.2+225113232522.8+232415 98.55-35.36 4.40  +0.61 +0.14 +0.32   F8III              +0.193+0.037 +.036-011       79                 *
8906 99    AqrBD-21 6420 220704191900    I         14554    232047.6-211124232602.8-203831 46.74-69.46 4.39  +1.47 +1.81 +0.81   K5III              -0.052-0.058 +.007+016V?                        *
8907   Omi GruCP-5310461 220729247874 883                   232100.9-531630232636.6-524318328.86-59.92 5.52  +0.40               F4V                +0.038+0.123      +018SB      0
8908          CP-67 3964 220759255486                       232111.8-670751232707.2-663452315.38-48.46 6.45  +1.47 +1.77         K4III              -0.042-0.023      +012
8909          CP-59 7890 2207902478823876                   232132.9-590142232715.0-582834322.23-55.38 5.63  +0.98               G8III              +0.051+0.062      -011
8910          CD-5013976 2208022478803875                   232136.3-504228232709.1-500926332.22-61.91 6.20  -0.08               B9V                +0.032+0.006      -001
8911  8Kap PscBD+00 4998 220825128186 884  W       Kap Psc  232148.3+004229232656.0+011520 83.92-55.08 4.94  +0.03 -0.02 -0.03   A0pCrSi:Sr:       v+0.089-0.097 +.040-003V      41  7.0 163.4AB   3*
8912  9    PscBD+00 4999 220858128188      W       14568    232207.4+003424232714.8+010721 83.90-55.24 6.25  +1.02 +0.80         G7III              +0.044-0.035      -007SB                        *
8913 13    AndBD+42 4672 220885 53039                       232218.1+422141232707.4+425443106.91-17.32 5.75   0.00 -0.09         B9III              +0.091+0.015      -009SB     67                 *
8914          CD-3615895 2209292145613878                   232238.7-360542232800.7-353240  2.43-70.35 6.32  +1.20               K2III              +0.007+0.001      +013
8915 69    PegBD+24 4778 220933 91278              HV Peg   232242.1+243704232740.4+251002 99.98-33.94 5.98  -0.06 -0.17         A0pHgMn            +0.026-0.041      -016       40                 *
8916 10The PscBD+05 5173 2209541281961614I                  232253.7+054947232758.1+062244 88.81-50.88 4.28  +1.07 +1.01 +0.53   K1III              -0.121-0.045 +.016+006V?   < 17
8917          BD-12 6496 220957165708                       232252.9-115959232805.2-112659 67.58-64.92 6.37  +0.90 +0.56         G0V                +0.102-0.023      -083V?
8918          BD+69 1332 220974 10737                       232302.7+694834232716.6+702135115.92  8.64 5.60  +0.19?              A6IV               +0.119-0.006 +.023-003SB2   115
8919          CP-63 4891 2210062554973880          CG Tuc   232313.7-633940232901.0-630639317.77-51.58 5.68  -0.17               ApSi              v+0.037-0.017      +015       69                 *
8920          CD-4515043 221051231622                       232333.6-450255232900.8-442952341.10-66.03 6.43  +1.18               K2III              +0.037-0.011      -020
8921          BD-10 6120 221081146729    I                  232350.3-094859232900.6-091558 71.60-63.59 6.18  +1.44 +1.70         K0                 -0.050-0.026      +008V?
8922          BD+22 4844 221113 91295                       232406.6+222958232905.6+230252 99.35-36.02 6.35R                     G9III              -0.039-0.094      +020
8923 70    PegBD+11 5009 221115108638 885I                  232405.8+121232232909.3+124538 93.72-45.39 4.55  +0.94 +0.73 +0.45   G7+III             +0.064+0.027 +.016-015V?   < 19:
8924          BD-05 5999 2211481467363881          14588    232421.9-050439232932.1-043158 78.48-60.13 6.25  +1.09 +1.16         K3III             v+0.183-0.232 +.019-025V
8925          BD+48 4070 221246 530883882I                  232521.2+483454233007.4+490759109.51-11.61 6.17  +1.46 +1.71         K3III              +0.033+0.002      +006                          *
8926          BD+57 2748 221253 35478       16795  AR Cas   232524.6+575951233002.0+583256112.47-02.66 4.91  -0.12 -0.64 -0.14   B3IV               +0.020+0.005D+.006-013SB1O  144  2.3  75.7AC   8*
8927          BD+37 4856 221293 73308                       232546.5+380637233039.7+383943106.08-21.55 6.05  +0.99               G9III              +0.042+0.011      -009
8928          BD-07 6036 221308146748                       232551.7-065020233101.1-061718 76.78-61.72 6.39  +1.26 +1.29         K0                 -0.013-0.036      +013
8929          CD-4515055 2213232316423883                   232600.9-452341233127.0-445037339.76-66.14 6.02  +1.02               K1III              +0.048-0.025      +008
8930 14    AndBD+38 5023 221345 73311    I         14599    232622.2+384113233117.4+391411106.41-21.05 5.22  +1.02 +0.87         K0III              +0.289-0.080 +.015-059V    < 19:
8931          BD-04 5896 221356146752                       232621.5-043802233131.5-040514 79.82-60.09 6.49  +0.54 -0.01         F8V                +0.175-0.190 +.041-011V   =<  6
8932100    AqrBD-22 6141 2213571919703885                   232627.5-215517233142.1-212210 46.21-70.96 6.29  +0.33 +0.16         A9III              +0.009+0.005      -008SB
8933          BD+27 4568 221394 91321              Var      232645.4+275108233143.1+282413102.40-31.27 6.41  +0.01 +0.03         A1pSrCr:Eu:        -0.025-0.008      -006V                         *
8934 13    PscBD-01 4450 221409146756                       232649.7-013818233157.6-010509 83.41-57.73 6.38  +1.18 +1.10         K1III              +0.012+0.017      -023
8935          CP-78 1473 2214202581543887                   232652.2-775615233319.5-772307308.32-38.91 5.81  +0.68 +0.31         G2V                +0.010 0.000      +026
8936          BD+34 4948 221491 73325                       232729.4+342402233224.6+345709105.11-25.17 6.65  -0.06 -0.24         B8V                -0.007-0.006      +012      250
8937   Bet SclCD-3815527 221507214615 886          14608    232736.6-382216233258.3-374906355.04-70.36 4.37  -0.09 -0.36 -0.07   B9.5IVpHgMnEu      +0.087+0.021      +002       37                 *
8938          BD+86  344 221525  39163967                   232748.7+864521232700.8+871827121.86 24.61 5.58  +0.23 +0.13         A7IV               +0.075+0.018      -011V?     90
8939101    AqrBD-21 6437 221565191988      W                232802.6-212802233316.6-205452 47.93-71.13 4.71  +0.02  0.00 +0.03   A1n                -0.007+0.013 -.000+015      297  2.3   1.0
8940 71    PegBD+21 4952 221615 913403889I         HW Peg   232827.5+215649233328.1+222956100.25-36.92 5.32  +1.60 +1.61         M5IIIa             +0.010-0.017      +003V?                        *
8941          BD+44 4441 221661 53147                       232852.2+443019233342.7+450329108.81-15.67 6.24  +0.98               G8II               -0.029+0.010      +007
8942          BD+20 5352 221662 91349    I                  232854.4+201720233355.5+205027 99.59-38.49 6.06  +1.73 +1.88         M3III              -0.004-0.022      +005
8943 72    PegBD+30 4978 221673 73341    I  16836  14617    232859.4+304624233357.2+311931104.09-28.69 4.98  +1.38 +1.62         K4IIIb             +0.052-0.016 +.018-024SB   < 19: 0.2   0.5      *
8944 14    PscBD-02 5986 2216751467803890                   232900.5-014759233409.0-011451 84.09-58.17 5.87  +0.30 +0.16 +0.13   A2m                +0.108-0.013      -003       60                 *
8945          CP-65 4148 221740255520      W                232930.6-651433233512.7-644122315.61-50.52 7.4                       F2IV-V             +0.015+0.004                     2.4  37.7
8946          BD-16 6314 221745165780    I                  232937.1-154747233449.4-151445 62.77-68.60 5.96  +1.36 +1.57         K0                 +0.051-0.079      -037V?
8947 15    AndBD+39 5114 221756 733461616          14627    232943.8+394106233437.5+401411107.40-20.31 5.59  +0.10 +0.08 +0.06   A1Vp               -0.017-0.046 +.014+013V     109                 *
8948 73    PegBD+32 4667 221758 733453891                   232941.4+325638233438.2+332950105.06-26.69 5.63  +1.03              gK0                 +0.001+0.019      -003V?
8949   Iot PheCD-4315420 2217602316751617  W       Iot Phe  232941.8-431005233504.6-423654343.07-68.07 4.71  +0.08 +0.07 +0.07   A2VpSrCrEu         +0.046-0.001 +.005+019V      22  8.0   7.7      *
8950          BD+37 4866 221776 73351    I  16843           232951.7+372815233446.7+380126106.69-22.42 6.18  +1.58               K7III              +0.008+0.009      -012           5.2  20.2
8951          BD-08 6142 221835146795 888                   233022.5-080104233532.1-072752 76.98-63.35 6.39  +0.88 +0.57        gG5                 +0.001+0.016      +005
8952          BD+70 1327 221861 107903893I                  233038.5+710521233459.0+713832116.93  9.67 5.84  +1.80 +1.73         G9Ib               -0.004 0.000      -003V?
8953          BD+23 4769 221905 913673894I                  233055.5+240028233555.9+243340101.83-35.20 6.45  +1.73 +2.01         M1III              +0.004+0.009      -012
8954 16    PscBD+01 4744 2219501282813895  D                233117.1+013250233623.3+020608 88.23-55.63 5.68  +0.44 -0.08         F6Vb vw            -0.103+0.061      +039SB2O=< 10  0.1   0.0      *
8955          BD+32 4671 221970 73368                       233132.8+322103233630.5+325415105.25-27.38 6.35  +0.46  0.00         F6V                -0.008 0.000      -001
8956          CD-3217593 222004214659      W                233148.3-322528233705.4-315215 12.12-73.17 6.52  +1.26 +1.22         K1III              +0.048+0.002D+.003-023           3.3   5.2      *
8957          CP-77 1583 222060258166                       233211.1-772522233824.1-765212308.23-39.50 6.00  +0.90 +0.65         K0-2III            +0.082-0.032      +002
8958          BD-13 6439 222093165804       16878           233228.5-133652233739.6-130337 68.48-67.74 5.65  +1.03 +0.83         G9IIIFe-0.5        +0.037+0.024      -013           3.9  31.6
8959          CD-4614720 222095231707 889                   233228.0-460244233751.0-452933336.66-66.52 4.74  +0.08 +0.09 +0.05   A2V                +0.073-0.014 +.016+010SB    126                 *
8960 74    PegBD+16 4954 222098108729      W                233235.7+161619233739.8+164932 98.66-42.54 6.26   0.00 +0.08         A1V                +0.134 0.000      -026SBO    45  4.5  98.3AB   3*
8961 16Lam AndBD+45 4283 222107 53204 890I W       Lam And  233240.0+455459233733.9+462729109.90-14.53 3.82  +1.01 +0.69 +0.57   G8III-IV          v+0.162-0.421 +.050+007SB1O < 19: 6.5 217.6AC   4*
8962          BD+43 4508 222109 53202       16877           233238.6+435233233732.0+442545109.28-16.47 5.80  -0.06 -0.31         B8V                +0.015-0.010D+.004-011V      50  0.9   0.5AB   3*
8963 75    PegBD+17 4952 2221331087323896  W       KS Peg   233253.7+175048233756.8+182402 99.54-41.11 5.53   0.00 -0.03         A1Vn               +0.050+0.017      -016SB1   216  6.2  27.9      *
8964          BD+45 4288 222143 53210                       233302.4+453846233758.6+461159109.89-14.80 6.58  +0.66 +0.16         G5                 +0.370-0.003 +.032-001
8965 17Iot AndBD+42 4720 222173 53216 891          14646    233313.7+424252233808.2+431605109.03-17.62 4.29  -0.10 -0.29 -0.09   B8V                +0.029-0.001 +.011-001SB     84
8966   The PheCD-4714651 222287231719      W                233405.9-471134233928.0-463816334.20-65.88 6.09  +0.24 +0.11         A8V+F0V            +0.028+0.033D+.008+014           0.7   3.7      *
8967 18    AndBD+49 4180 222304 356423897                   233417.4+495504233908.3+502818111.34-10.77 5.30  -0.06 -0.15         B9V                -0.017-0.002      +009SB    178
8968102Ome1AqrBD-15 6471 222345165818                       233435.9-144629233947.1-141318 67.04-68.89 5.00  +0.24               F0IV               +0.057-0.038 +.016-002SB    102
8969 17Iot PscBD+04 5035 222368128310 892I W       14657    233448.3+050503233957.0+053735 92.46-52.96 4.13  +0.51  0.00 +0.31   F7V                +0.378-0.438 +.075+005V       6  8.7  69.9      *
8970          BD+08 5095 222377128309                       233449.2+090724233955.1+094038 95.27-49.31 5.97  +0.20 +0.13 +0.09   A2IVm              +0.099-0.011      +000       60
8971          BD+74 1032 222386 10814                       233452.7+744419233910.4+751734118.30 13.08 5.95  +0.13 +0.12         A3V                +0.012+0.011      +003SB    125
8972          BD+73 1047 222387 108173898                   233459.5+732655233921.1+740010117.93 11.83 5.98  +0.89               G8III              -0.012+0.004      +009SB
8973          BD+36 5098 222399 73422       16913           233505.4+370603234002.8+373909107.64-23.09 6.53  +0.35 +0.08         F2IV               -0.013-0.084      -016V?         4.4  14.9      *
8974 35Gam CepBD+76  928 222404 10818 893I         14656    233514.3+770427233920.8+773757118.99 15.31 3.21  +1.03 +0.94 +0.51   K1III-IV           -0.067+0.151 +.064-042V?   < 17                 *
8975   Mu  SclCD-3217621 2224332147011618          14661    233523.4-323734234038.2-320423 10.86-73.86 5.31  +0.97 +0.66         K1III              -0.089-0.055 +.027+014
8976 19Kap AndBD+43 4522 222439 532641619   16916           233528.8+434649234024.5+442002109.76-16.71 4.14  -0.08 -0.26 -0.07   B9IVn              +0.083-0.019 +.018-009V     184  7.6  46.8AB   3*
8977          BD+35 5074 222451 734283899                   233540.7+360957234040.6+364315107.46-24.02 6.23  +0.39 -0.05         F1V                +0.236+0.020      -000       30                 *
8978          CD-2417796 222485192083    I         14665    233553.8-244252234107.0-240937 39.63-73.89 6.60  +1.57 +1.92         M1III              +0.006-0.010      +018V?                        *
8979          BD-12 6535 222493165828                       233558.4-121407234108.9-114050 72.64-67.40 5.89  +1.00 +0.78 +0.35E  G9III              +0.070+0.009      -011V                         *
8980103    AqrBD-18 6357 222547165834    I                  233623.3-183447234134.5-180138 58.77-71.51 5.34  +1.57 +1.93 +0.92   K5III              -0.039-0.071      +025
8981          BD+48 4127 222570 53276                       233633.7+485730234126.9+493044111.43-11.79 6.26  +0.19?              A4V                +0.002-0.021      -006
8982104    AqrBD-18 6358 222574165836    I W       14671    233634.3-182216234145.8-174859 59.40-71.44 4.82  +0.82 +0.49 +0.41   G0Ib-II            +0.015+0.004 -.009+003SB     17  2.8 120.1AB   3*
8983          BD+06 5183 222602128335              14672    233650.9+064149234156.7+071502 94.37-51.72 5.89  +0.10 +0.08         A3Vn               -0.024-0.042      +001      210                 *
8984 18Lam PscBD+00 5037 2226031283361620                   233656.6+011347234202.8+014648 90.15-56.61 4.50  +0.20 +0.08 +0.10   A7V                -0.129-0.155 +.028+012SB     63                 *
8985          BD+56 3067 222618 35682                       233705.4+564220234154.5+571536113.61-04.35 6.24R                     G8III              +0.005-0.002      -012
8986          BD+44 4473 222641 53292    I         14674    233719.1+442615234214.8+445931110.28-16.17 6.57R                    gK5                 -0.011-0.010      -010V
8987          BD-16 6345 222643165841    I                  233717.0-160008234227.9-152652 65.53-70.18 5.28  +1.37 +1.49         K4III              +0.025-0.008 +.015+007SB   < 17                 *
8988105Ome2AqrBD-15 6476 222661165842 894   16944           233732.2-150552234243.3-143242 67.66-69.64 4.49  -0.04 -0.12 -0.05   B9.5V              +0.101-0.066 +.041+003SB    156  6.0   5.0      *
8989          BD+63 2038 222670 20791    I  16940           233737.5+635739234220.8+643056115.58  2.63 6.56  +1.88 +2.12 +1.33   M2III              +0.003-0.002      -003           4.4   2.6
8990          BD+60 2609 222682 207933900  W                233744.7+610731234231.5+614046114.85-00.11 6.40  +1.24 +1.36         K2III              +0.043-0.021      -016           6.5   8.
8991 77    PegBD+09 5268 2227641087893902I         14679    233816.9+094635234322.4+101953 96.87-49.06 5.06  +1.68 +2.04         M2III              +0.008+0.007 +.010-034V                         *
8992          BD-16 6352 222800165849    I         R Aqr    233838.8-155019234349.5-151704 66.52-70.32 6.36  +1.58 +0.32         M7IIIpe           v+0.032-0.025      -022V                         *
8993          CD-4515114 2228032317563903                   233839.8-453817234401.4-450500335.38-67.56 6.09  +0.98 +0.71         K1III-IV           +0.338-0.006 +.019-031
8994          CP-71 2771 2228052581783905                   233842.3-710249234425.4-702925310.84-45.59 6.07  +0.91 +0.62         G8-K0IV            +0.243+0.062 +.014+020
8995          CP-79 1239 2228062581793906                   233836.2-792048234440.7-784729306.98-37.81 5.75  +1.11 +1.07         K1III              +0.048+0.004      +021V
8996          CP-65 4159 2228202555573904                   233841.6-645738234412.0-642416314.43-51.20 5.72  +1.40 +1.63         K2II-III           +0.011+0.031 +.007-013
8997 78    PegBD+28 4627 222842 91457    I  16957           233857.5+284827234359.5+292142105.73-31.25 4.93  +0.95 +0.63 +0.48   K0III              +0.071-0.037 +.018-007SB   < 19: 3.1   0.9      *
8998106    AqrBD-19 6500 2228471658541621                   233900.9-184955234412.1-181637 59.17-72.17 5.24  -0.08 -0.27         B9Vn               +0.029-0.003      +014V     258
8999          CD-2616762 222872192116       16963           233916.7-264804234428.9-261447 32.50-75.03 6.17  +0.50 +0.10         F4V                -0.070-0.015      +016           3.2   8.3      *
9000          BD+55 3010 222932 35728                       233955.6+551441234448.3+554759113.61-05.86 6.51R                    gG4                 +0.020-0.005 +.008+009
9001          CD-4015239 223011231769                       234044.7-404413234601.2-401057344.81-71.20 6.31  +0.20 +0.09         A7III-IV           +0.097-0.036      +004
9002107    AqrBD-19 6506 223024165867       16979A          234049.3-191403234600.9-184041 58.82-72.75 5.29  +0.28 +0.14         F2III+F2V          +0.134+0.030 +.012-002       83  1.0   6.6      *
9003 20Psi AndBD+45 4321 223047 533551622I W                234104.5+455154234602.1+462513111.34-14.97 4.95  +1.11 +0.82 +0.38E  G5Ib+A0V           +0.011-0.005 +.001-025V    < 19: 4.0 184.0AD   4*
9004 19    PscBD+02 4709 2230751283743908I         TX Psc   234116.9+025555234623.5+032912 93.28-55.60 5.04  +2.60 +3.49 +1.35   C5II               -0.030-0.025 -.004+011V                         *
9005          BD+65 1943 223128 20838                       234150.3+661337234636.7+664656116.59  4.71 5.95  -0.04 -0.74         B2IV               +0.004+0.001      -014
9006   Sig PheCD-5014047 2231452480183910                   234157.6-504653234716.0-501336326.59-63.85 5.18  -0.19               B3V                +0.011-0.029      +011      217
9007          CP-69 3335 2231482555673911                   234152.1-685656234723.3-682339311.60-47.64 6.89  +0.46 +0.06         F2V                +0.045-0.043      -002
9008  5Tau CasBD+57 2804 223165 357633909I         14707    234209.9+580542234703.5+583907114.63-03.18 4.87  +1.11 +1.05         K1IIIa             +0.061+0.057 +.014-021V    < 17
9009          BD-12 6559 223170165880                       234206.9-122751234715.9-115439 75.12-68.59 5.73  +1.08 +0.97        gK1                 -0.049-0.086      +011
9010          BD+56 3085 223173 35761    I                  234208.4+565346234701.9+572705114.33-04.34 5.51  +1.65 +1.81         K3IIb              +0.002-0.003      -006V?
9011          BD+46 4169 223229 53374       17006  14709    234234.8+461638234733.2+464957111.71-14.64 6.07  -0.14 -0.66         B3IV               +0.013-0.003D+.002-024SB         2.0   0.9      *
9012 20    PscBD-03 5707 2232521469151623  W                234248.1-031903234756.5-024542 88.27-61.28 5.49  +0.94 +0.70 +0.33E  G8III              +0.096+0.005      -007V?  =< 11  4.4 172.7AB   3*
9013          BD+67 1562 223274 20853 895                   234307.5+671504234754.8+674825116.97  5.67 5.04  -0.01 -0.04         A1Vn               +0.014-0.001 +.015+010SB    185
9014          BD-07 6086 2233111469193912I         14715    234324.1-065609234832.5-062250 84.33-64.43 6.07  +1.45 +1.71         K4III              +0.003-0.019      -021V
9015          BD+01 4773 223346128393                       234342.2+013934234849.3+021251 93.25-57.00 6.46  +0.44 -0.02         F5III-IV           +0.007-0.042      -025SB     12
9016   Del SclCD-2818353 223352192167 896   17021A          234343.1-284100234855.6-280749 25.18-76.13 4.57  +0.01 -0.03 -0.02   A0V                +0.105-0.106 +.039+014V?         4.5  74.3AC   3*
9017          BD+64 1861 223358 20866       17020  V650 Cas 234348.3+641916234839.0+645235116.33  2.81 6.41  +0.06 +0.02         A0pSrSi:Cr:        +0.013-0.020 +.009-003           2.0   0.7AB   4*
9018  6    CasBD+61 2533 223385 20869       17022  V566 Cas 234357.7+613931234850.2+621252115.71  0.22 5.43  +0.67 -0.02         A3Ia              e-0.005-0.001D+.001-046SB?    50  5.1   1.5AB   3*
9019          BD+59 2777 223386 35794                       234359.4+592522234853.9+595844115.18-01.95 6.34  -0.01 -0.05         A0V                +0.048+0.007      -016    =< 41
9020          BD+58 2653 223421 35798                       234416.8+582427234912.0+585747114.97-02.94 6.33  +0.40 -0.01         F2IV               +0.038-0.014      +030
9021          BD-16 6373 223428165905                       234421.6-162459234931.6-155140 67.89-71.75 6.24  +1.22 +1.17         K2IIIbFe-0.5       +0.049-0.022      -064V?
9022 21    PscBD+00 5054 223438128401                       234420.2+003115234927.5+010434 92.57-58.09 5.77  +0.16 +0.12         A5Vm               -0.002-0.030      +005SB     43
9023          CP-63 4931 223444255578                       234421.9-632340234944.7-625022314.54-52.88 6.59  +1.48               K5III              +0.031-0.035      +011
9024          BD+35 5110 223460 735353914          OU And   234439.0+355214234941.0+362531109.28-24.80 5.90  +0.79               G1III             e+0.001-0.048      +001V?                        *
9025 79    PegBD+28 4649 223461 915223913                   234435.6+281709234939.4+285033106.95-32.10 5.97  +0.19 +0.13         A2m                +0.061+0.024      -004       38
9026          CD-2616796 223466192180       17029           234438.7-255312234949.6-251953 36.88-76.08 6.42  +0.12 +0.12         A3V                -0.021-0.024      +016           5.2  13.5
9027          BD-10 6177 223524165911 897                   234505.1-103157235014.7-095827 79.96-67.59 5.94  +1.13 +1.15         K0IV               +0.140+0.076      -018V
9028          BD+50 4165 223552 35823       17032           234522.4+510358235022.3+513718113.37-10.11 6.44  +0.37 -0.09         F3V                +0.123-0.015      -021           6.1  21.0AB   3
9029          BD-15 6507 223559165915                       234523.8-145725235033.3-142407 71.78-70.94 5.72  +1.51 +1.81         K4.5IIIb           +0.037-0.033      -058V?
9030 80    PegBD+08 5127 223637128421    I         HH Peg   234615.0+084532235121.2+091848 99.09-50.75 5.79  +1.66 +1.74 +0.54   M3IIIa             -0.016-0.063      -009                          *
9031108    AqrBD-19 6522 223640165918              ET Aqr   234611.5-192755235121.3-185432 60.54-73.94 5.18  -0.14 -0.42         B9pSiSrCr         v+0.021 0.000 +.023+013V      40                 *
9032   Gam1OctCP-82  905 2236472589893997                   234614.9-823429235206.4-820108305.41-34.82 5.11  +0.92 +0.60         G7III              -0.055-0.019      +015
9033 22    PscBD+02 4725 2237191284273918I                  234650.6+022228235157.9+025549 95.11-56.69 5.55  +1.53 +1.86         K4IIIaBa0.3        +0.020-0.015      +000                          *
9034          BD+76  934 223731 108743919                   234709.2+770245235157.6+773558119.66 15.11 6.55  +0.44 -0.11         F5V                +0.264-0.092 +.042+001V?
9035          BD+20 5386 223755 91548    I                  234718.7+210654235223.4+214015105.15-39.15 6.11  +1.59 +1.97         M2III              -0.049-0.015      -005V?
9036 81Phi PegBD+18 5231 223768108878 898I         14735    234723.9+183354235229.3+190713104.18-41.60 5.08  +1.60 +1.86         M2.5IIIb           -0.005-0.036 +.011-008V?
9037          BD-15 6515 223774165933                       234721.7-144827235230.0-141504 73.14-71.17 5.87  +1.28 +1.37         K3III              -0.101 0.000      +002
9038          BD+74 1047 223778 10879       17062           234731.7+745913235225.1+753241119.18 13.10 6.39  +0.98 +0.71         K3V                +0.314+0.049 +.097+001SBO        1.9 161.9AC   3*
9039 82    PegBD+10 5004 2237811088791625          HT Peg   234731.0+102328235237.1+105651100.44-49.34 5.30  +0.18 +0.09         A4Vn               -0.029+0.001 +.018-003V     156                 *
9040          BD-09 6277 223807146953                       234741.8-093309235250.5-085948 82.82-67.19 5.75  +1.17 +1.15         K0III              +0.064-0.021      -018V?
9041 24    PscBD-03 5723 223825146954      W                234747.3-034239235255.6-030920 90.15-62.24 5.93  +1.07 +0.96         G9III              +0.075-0.046      -006V          0.0   0.1      *
9042 25    PscBD+01 4792 223855128436                       234757.4+013205235304.8+020526 94.93-57.57 6.28  -0.01 -0.01         A1V                +0.022-0.012      +005SB     46
9043          CD-2417897 223884192218                       234810.6-244707235320.8-241345 42.19-76.60 6.24  +0.19               A3Shell            +0.047-0.011      -003                          *
9044          CD-2716479 223991192231       17090           234911.4-273558235421.4-270232 30.05-77.30 6.35  +0.20  0.00 +0.11   A2V+F2V            +0.020+0.025D+.006+017SB  =< 39  0.7   6.4      *
9045  7Rho CasBD+56 3111 224014 35879 899I         Rho Cas  234923.0+565635235423.0+572958115.30-04.52 4.54  +1.22 +1.12 +0.74   G20e              v-0.002-0.002 +.023-043V      29                 *
9046          CD-4015285 2240222318421626                   234924.0-405127235438.6-401800341.05-72.35 6.03  +0.57 +0.11         F8IV               +0.368+0.028 +.029-010
9047          BD-00 4585 2240621469733920I         XZ Psc   234939.5-002648235446.6+000633 94.06-59.55 5.61  +1.59 +1.51 +1.30E  M5III              -0.040-0.018      -002V?                        *
9048 26    PscBD+06 5216 224103128466                       235000.8+063054235507.8+070416 99.18-53.18 6.21  -0.07 -0.19         B9V                +0.021-0.019      +017V
9049          CD-3217723 2241132148603921          AL Scl   235006.3-322841235516.6-315518  8.14-76.89 6.10  -0.08 -0.46         B6V                +0.022-0.003      -012SB1O  199                 *
9050          CD-3217724 224112214861                       235006.4-322627235516.6-315303  8.29-76.90 6.83  -0.08 -0.34         B8V                +0.003+0.008      +001V?
9051          BD+25 5042 224128 91582                       235017.6+252355235523.0+255718107.47-35.23 6.54R                     K5                 -0.003-0.008      -015
9052          BD+56 3115 224151 35899              V373 Cas 235032.7+565120235533.7+572444115.44-04.64 6.00  +0.21 -0.72         B0.5II+B0.5II     v-0.008-0.003 +.029-026SBO                       *
9053          BD+46 4214 224165 53486                       235030.5+464758235533.6+472121113.21-14.46 6.00  +1.16 +0.94         G8Ib                0.000-0.005      -017
9054          CD-2516707 2242831922523922                   235120.8-251739235629.9-244414 40.82-77.43 6.31  +0.92               G5                 +0.003+0.010      -035V?
9055          BD+21 4999 224303 915951628I                  235135.7+220530235641.5+223853106.73-38.50 6.15  +1.60 +1.97         M2III              -0.022-0.005      +001V?
9056          BD+82  743 224309  39941650          V Cep    235145.3+823804235627.7+831128121.21 20.50 6.59  +0.05 +0.05         A3V                +0.047+0.007      -013                          *
9057          BD+41 4902 224342 535113923                   235159.1+420606235703.6+423930112.39-19.10 5.97  +0.69 +0.30         F8III              +0.008-0.001      -007V                         *
9058          CD-2716494 224350192262    I         14772    235158.7-271051235708.3-263725 32.23-77.89 6.26  +1.45               K3III              +0.071+0.017      -012V
9059          BD+54 3076 224355 35917              14773    235205.8+550858235708.5+554221115.29-06.36 5.55  +0.49 +0.03         F1/6V+F3           -0.019-0.011      +013SBO                       *
9060          CP-63 4940 2243612556063924                   235205.1-633050235719.9-625723313.13-53.09 5.97  +0.11 +0.10         A1IV               +0.082+0.018      -013                          *
9061   Gam2OctCP-82  907 224362258996                       235204.1-824333235732.7-821012305.14-34.73 5.73  +1.05 +0.92         K0III              -0.035-0.026      +027
9062   Eta TucCP-64 4391 224392255609                       235220.2-645112235735.2-641754312.34-51.84 5.00  +0.06 +0.08         A1V                +0.086-0.068 +.036+033      191
9063          BD+59 2795 224404 35922                       235230.9+592801235733.5+600125116.24-02.15 6.47  +0.01 -0.10         B9III-IV           +0.012-0.002      -008V?
9064 84Psi PegBD+24 4865 224427 916111629I S       14777    235239.7+243508235745.5+250829107.85-36.15 4.66  +1.59 +1.68 +1.34   M3III              -0.034-0.033 +.005-004V?                        *
9065  1    CetBD-16 6394 2244811659723925                   235312.5-162415235821.2-155051 72.72-73.30 6.26  +1.08 +1.03         K0II-III           +0.086-0.006      +004V?
9066          BD+50 4202 224490 35938    I  17135  R Cas    235319.6+504954235824.8+512319114.56-10.62 4.8   +1.83 +0.08         M7IIIe            v+0.080+0.009      +021V          5.4  29.5AC   3*
9067 27    PscBD-04 5996 224533147008 900I  17137  14785    235333.2-040639235840.4-033322 92.51-63.27 4.86  +0.93 +0.70         G9III              -0.051-0.072 +.029-000SB   < 19: 5.3   1.1      *
9068          BD+31 5012 224544 736503927          Var?     235343.2+314930235849.2+322254110.21-29.19 6.52  -0.11 -0.57         B6IVe              +0.003-0.007      -006                          *
9069   Pi  PheCP-5310561 224554248087 901                   235344.9-531816235855.8-524445320.30-62.57 5.13  +1.13 +1.03         K0III              +0.055+0.061 +.016-014
9070          BD+45 4381 224559 53540              LQ And   235341.2+455123235846.5+462447113.55-15.50 6.54  -0.09 -0.61         B4Ven             v+0.022-0.005      -001V                         *
9071  8Sig CasBD+54 3082 224572 35947       17140           235356.0+551154235900.5+554518115.55-06.36 4.88  -0.07 -0.82 -0.11   B1V                +0.009-0.004D+.006-013SB2   189  2.2   2.8AB   3*
9072 28Ome PscBD+06 5227 224617128513 902I         14793    235410.5+061835235918.7+065148100.69-53.74 4.01  +0.42 +0.06 +0.24   F4IV               +0.153-0.115 +.018+002SBO    38  2.2   1.0     3*
9073          CD-3019765 224630192294    I                  235419.5-300231235927.9-292906 18.25-78.33 5.62  +1.60 +2.00         K5III              -0.001-0.001      -005
9074          BD+32 4747 224635 73656       17149A          235423.5+331012235929.2+334328110.72-27.91 6.58H +0.55 -0.01         G0V                -0.060-0.080D+.051-008V?      7  0.2   1.5AB   3*
9075          BD+32 4747 224636 73656       17149B          235423.5+331012235929.2+334328110.72-27.91 6.58H                     G0V                -0.060-0.080D+.051-005V?      7  0.2   1.5AB   3*
9076   Eps TucCP-66 3819 224686255619 903                   235443.3-660800235955.0-653438311.30-50.71 4.50  -0.08 -0.28 -0.05   B9IV              v+0.048-0.024      +011      291                 *
9077          CD-4415420 224750231888      W                235509.8-445040000019.2-441726330.51-70.00 6.29  +0.76               G3IV               +0.080-0.111      +003           0.5   0.4
9078          BD+26 4727 224758 91648      W                235516.8+262146000023.9+265506109.09-34.58 6.46  +0.50 +0.01         F7-8IV-V           +0.044-0.052      +000V                         *
9079          BD+58 2685 224784 359833928                   235526.5+590013000030.9+593335116.52-02.68 6.19  +1.01 +0.80         G9III-IV           -0.077-0.024      -033                          *
9080          BD+44 4538 224801 53568              CG And   235537.1+444147000043.7+451512113.65-16.70 6.38  -0.07 -0.33         B9pSiEu           v+0.027+0.005      -003SB     49                 *
9081   Tau PheCD-4914316 224834231895                       235556.3-492200000104.5-484836323.79-66.23 5.71  +0.91               G8III              -0.022-0.009      +008
9082          CD-5113743 224865248103                       235611.7-505340000120.0-502014321.98-64.90 5.53  +1.60 +1.93         M2III              +0.002+0.013 +.008+002
9083          BD+49 4309 224870 535733929                   235612.6+492530000119.3+495854114.73-12.09 6.22  +0.97               G7II-III           +0.018-0.006      -020
9084   The OctCP-77 1596 224889258207 904                   235627.5-773704000135.7-770357306.53-39.71 4.78  +1.27 +1.41 +0.67   K3III              -0.063-0.177 +.017+024
9085          BD+60 2657 224893 21009                       235630.6+603958000136.9+611323116.97-01.07 5.55  +0.41 +0.39         F0III              -0.006+0.005      -023SB?    38
9086          BD+41 4920 224906 53580                       235636.8+414837000143.8+422202113.22-19.56 6.25  -0.03 -0.25         B9IIIpHg:Mn:       +0.005-0.003      -011V      42
9087 29    PscBD-03 5749 224926147041              14804    235641.9-033503000149.4-030139 94.59-63.14 5.10  -0.12 -0.51         B7III-IV           +0.016-0.009      +023V?     80                 *
9088 85    PegBD+26 4734 224930 91669       17175  14809    235656.7+263310000210.2+270455109.60-34.50 5.75  +0.67 +0.05 +0.43   G5VbFe-2           +0.841-0.991 +.083-036SBO     2  3.1  75.5AC   4*
9089 30    PscBD-06 6345 2249351470421630I         YY Psc   235649.8-063411000157.6-060051 91.58-65.83 4.41  +1.63 +1.83 +1.41   M3III             e+0.051-0.041 +.006-012V                         *
9090          BD-15 6531 224960166005    I         W Cet    235659.9-151357000207.3-144034 77.78-73.06 7.1   +1.80 +0.55 +3.71   S7.3e              -0.010-0.019      +013                          *
9091   Zet SclCD-3019790 224990      3932  W                235712.3-301639000219.9-294313 16.55-78.90 5.01  -0.15 -0.55 -0.14   B4III              +0.019+0.016      +000SB     47  8.0   3.       *
9092 31    PscBD+08 5164 224995128544                       235716.8+082400000224.2+085725103.01-52.01 6.32  +0.18 +0.14         A6V                +0.005-0.003      +011V      97
9093 32    PscBD+07 5121 2250031285473933                   235722.9+075548000229.7+082908102.81-52.46 5.63  +0.29 +0.03         F0V                -0.092-0.047      +010V?                        *
9094          BD+65 1987 225009 10937           1A          235728.5+653232000236.1+660556118.01  3.70 5.86  +1.09 +0.91         G8III              +0.010-0.001D+.010-018V   =< 25  1.5  15.2AB   3*
9095          BD-20 6703 225045166014                       235749.5-203618000257.6-200246 62.90-76.84 6.25  +0.53 +0.08         F6V                +0.110+0.071      -004V?
9096          CD-2417960 225069166016                       235800.2-244208000307.7-240843 45.57-78.72 6.44  +1.18               G5                 +0.039 0.000      +025
9097          BD+62 2356 225094 10942              V639 Cas 235817.4+630501000325.7+633831117.63  1.26 6.24  +0.33 -0.54 +0.23   B3Ia              e-0.008+0.057      -043V?                        *
9098  2    CetBD-18 6417 225132147059 905                   235837.0-175333000344.4-172010 72.10-75.26 4.55  -0.05 -0.12 -0.04   B9.5Vn             +0.026-0.009 +.007-005V     186
9099          BD+65 1993 225136 10948    I                  235842.4+660920000351.9+664244118.25  4.27 6.29  +1.67 +1.94         M4III              +0.021-0.008      +015
9100  9    CasBD+61 2586 225180 10954      W                235904.5+614351000413.6+621716117.47-00.08 5.88  +0.30 +0.29         A1III              -0.004+0.006      -018V          4.0 246.0
9101          BD-17 6868 225197147064                       235912.4-170503000419.7-163144 74.70-74.79 5.78  +1.10 +1.05         K2III              +0.039-0.059      -027
9102          CD-2918950 225200166031                       235913.2-294933000420.3-291607 18.53-79.41 6.40   0.00 -0.05         A0V                +0.030+0.014      +028
9103  3    CetBD-11 6194 2252121470662001I            13    235923.0-110358000430.1-103034 87.07-70.04 4.94  +1.63 +1.92         K3Ib              v-0.005-0.011 +.016-042V    < 17
9104          BD+66 1679 225216 10956                       235930.1+663632000442.0+671000118.41  4.71 5.67  +1.07 +0.93         K1III              +0.092+0.030 +.013-027
9105          BD+41 4933 225218 36037    I     30           235928.2+413210000436.7+420532113.72-19.94 6.01R                     B9III              +0.005-0.022      -008SB         3.7   5.4      *
9106          CP-73 2346 225233255629                       235926.8-732712000430.7-725352307.68-43.80 7.31  +0.44 +0.01         F2V                +0.010-0.054      +008
9107          BD+33 4828 225239 536222002                   235939.2+340602000453.8+343935112.17-27.24 6.12  +0.62 +0.09         G2V                +0.772+0.089 +.035+004V
9108          CP-72 2800 2252532556311001                   235937.0-715936000441.3-712613308.18-45.21 5.59  -0.12 -0.42         B8IV-V             +0.028-0.014      -003V
9109          BD+25 5068 225276 73731    I     42           235947.1+260533000456.0+263856110.22-35.07 6.25  +1.40 +1.59         K4IIIb             +0.110-0.012      -005           4.5  17.5
9110          BD+60 2667 225289 10962              V567 Cas 235955.9+604526000506.2+611851117.40-01.06 5.80  -0.09 -0.32         B8IVpHgMn          +0.015+0.005      +014V      50                 *`)
	stardat = bytes.Split(data, []byte("\n"))
	chnidx = make(map[string]uint16)
	for k, v := range hr2detail {
		if v[0] != "" {
			chnidx[v[0]] = k
		}
		if v[1] != "" {
			chnidx[v[0]] = k
		}
	}
	return nil
}

func fullStarData(star InnerStarData) StarData {
	star.HIP = hr2hip[star.HR]
	if info, ok := hr2detail[star.HR]; ok {
		return StarData{
			InnerStarData:    star,
			ChineseName:      info[0],
			ChineseAlias:     info[1],
			ChineseBayerName: info[2],
			CommonName:       info[3],
			ComminAliasName:  "",
			Cst:              info[5],
			CstChinese:       info[4],
		}
	}

	return StarData{InnerStarData: star}
}

func StarDataByChinese(name string) (StarData, error) {
	if len(stardat) == 0 {
		err := LoadStarData()
		if err != nil {
			return StarData{}, err
		}
	}
	if strings.HasSuffix(name, "星") {
		name = strings.TrimSuffix(name, "星")
	}
	hr, ok := chnidx[name]
	if !ok || int(hr) > len(stardat) || hr == 0 {
		return StarData{}, errors.New("no such star")
	}
	data, err := parseStarData(stardat[hr-1])
	if err != nil {
		return StarData{}, err
	}
	return fullStarData(data), nil
}

func StarDataByHR(hr int) (StarData, error) {
	if len(stardat) == 0 {
		err := LoadStarData()
		if err != nil {
			return StarData{}, err
		}
	}
	if hr > len(stardat) {
		return StarData{}, errors.New("no such star")
	}
	data, err := parseStarData(stardat[hr-1])
	if err != nil {
		return StarData{}, err
	}
	return fullStarData(data), nil
}

func (s InnerStarData) RaDecByJde(jde float64) (float64, float64) {
	//计算自行
	year := ((jde - 2451545.0) / 365.2422)
	return ZuoBiaoSuiCha(s.Ra+(year*s.PmRA/3600), s.Dec+(year*s.PmDec/3600), 2451545.0, jde)
}

func (s StarData) RaDecByDate(date time.Time) (float64, float64) {
	jde := Date2JDE(date.UTC())
	return s.RaDecByJde(jde)
}
